//Sanford-Wang Central Value, Splined HARP data for Pi+
//
//    This code is intended to generate weights for neutrinos originating from Pi+ decays
//    utilizing Sanford-Wang and Spline fits of the HARP data on p+Be->Pi+ 
//    
//    This code is adopted from the MiniBooNE flux paper and the MiniBooNE reweighting framework 
//    along with code written by Raquel Castillo, Zarko, MiniBooNE (Steve Brice and Mike S), and Athula 
//
//    Current person adding comments and functions is Joseph Zennamo (jaz8600@fnal.gov)
//        [this code was designed follow the style of PrimaryHadronFeynmanScalingWeightCalc] 
//

#include "larsim/EventWeight/Base/WeightCalcCreator.h"
#include "larsim/EventWeight/Base/WeightCalc.h"

#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Services/Optional/RandomNumberGenerator.h"
#include "CLHEP/Random/RandomEngine.h"
#include "CLHEP/Random/RandGaussQ.h"

#include "nusimdata/SimulationBase/MCFlux.h"
#include "nusimdata/SimulationBase/MCTruth.h"
#include "nusimdata/SimulationBase/GTruth.h"

#include <vector>
#include "TH1.h"
#include "TArrayD.h"
#include "TSpline.h"
#include "TMatrixD.h"
#include "TDecompChol.h"
#include "TRandom3.h"
#include "TLorentzVector.h"
#include <iostream>
#include "TFile.h"
#include "TTree.h"
#include <TROOT.h>
#include <TChain.h>

using namespace std;

namespace evwgh {
  class PrimaryHadronSWCentralSplineVariationWeightCalc : public WeightCalc
  {
  public:
    PrimaryHadronSWCentralSplineVariationWeightCalc();
    void Configure(fhicl::ParameterSet const& p);
    std::pair< bool, double > MiniBooNEWeightCalc(simb::MCFlux flux, std::vector<double> rand);
    std::pair< bool, double > MicroBooNEWeightCalc(simb::MCFlux flux,std::vector<double> rand);
    virtual std::vector<std::vector<double> > GetWeight(art::Event & e);
    std::vector< std::vector< double > > MiniBooNERandomNumbers(std::string);
    
  private:
    CLHEP::RandGaussQ *fGaussRandom;
    std::vector<double> ConvertToVector(TArrayD const* array);
    std::string fGenieModuleLabel;
    std::vector<std::string> fParameter_list;
    float fParameter_sigma;
    int fNmultisims;
    int fprimaryHad;
    std::string fWeightCalc;
    std::string ExternalDataInput;
    std::string ExternalFitInput;
    double fScaleFactor;
    double fSeed;
    bool fUseMBRands;
    
    TFile* file;
    TFile* Fitfile;
    std::vector< std::vector< double > > fWeightArray; 
    std::string fMode;

    TMatrixD* HARPCov;    
    TMatrixD* HARPLowerTriangluarCov;    
    TMatrixD* HARPXSec;    

    std::vector<double> HARPmomentumBounds;
    std::vector<double> HARPthetaBounds;
    std::vector<double> SWParam;

    bool fIsDecomposed;
    

     DECLARE_WEIGHTCALC(PrimaryHadronSWCentralSplineVariationWeightCalc)
  };
  PrimaryHadronSWCentralSplineVariationWeightCalc::PrimaryHadronSWCentralSplineVariationWeightCalc()
  {
  }

  void PrimaryHadronSWCentralSplineVariationWeightCalc::Configure(fhicl::ParameterSet const& p)
  {

    // Here we do all our fhicl file configureation
    fGenieModuleLabel= p.get< std::string > ("genie_module_label");
    fhicl::ParameterSet const &pset=p.get<fhicl::ParameterSet> (GetName());
    std::cout << pset.to_string() << std::endl;

    fParameter_list		=   pset.get<std::vector<std::string> >("parameter_list");
    fParameter_sigma		=   pset.get<float>("parameter_sigma");
    fNmultisims			=   pset.get<int>("number_of_multisims");
    fprimaryHad			=   pset.get<int>("PrimaryHadronGeantCode");
    std::string dataInput       =   pset.get< std::string >("ExternalData");
    std::string fitInput        =   pset.get< std::string >("ExternalFit");
    fWeightCalc                 =   pset.get<std::string>("weight_calculator");
    fMode                       =   pset.get<std::string>("mode");
    fScaleFactor                =   pset.get<double>("scale_factor");
    fSeed                       =   pset.get<double>("random_seed");
    fUseMBRands                 =   pset.get<bool>("use_MiniBooNE_random_numbers");
    
    ////////////////////////
    //
    //  We will configure the down stream code to run on the primary 
    //   hadron that we want to be assigning an uncertainty to 
    //
    ///////////////////////
    std::string HadronName; 
    std::string HadronAbriviation;
    if(fprimaryHad == 211){
      HadronName = "PiPlus";
      HadronAbriviation = "PP";
    }
    else if(fprimaryHad == -211){
      HadronName = "PiMinus";
      HadronAbriviation = "PM";
    }
    else{ 
	throw art::Exception(art::errors::StdException)
	  << "sanford-wang is only configured for charged pions ";
      }

    ///////////////////
    //
    // Get HARP Data:
    //   Now let's get the external data that we will need 
    //   to assess what the variations we can make around our
    //   central value cross section 
    //
    //   First: Pull in the file
    ///////////////////
    cet::search_path sp("FW_SEARCH_PATH");
    std::string ExternalDataInput = sp.find_file(dataInput);
    file = new TFile(Form("%s",ExternalDataInput.c_str()));
    //  Second: Define what we want to gather from that file which is two fold
    std::vector< std::string > pname; // these are what we will extract from the file
    pname.resize(4); // there are 4 items

    // Define what we will gather from HARP data
    pname[0] = Form("HARPData/%s/%sCrossSection",HadronName.c_str(),HadronAbriviation.c_str()); // Cross Section
    pname[1] = Form("HARPData/%s/%scovarianceMatrix",HadronName.c_str(),HadronAbriviation.c_str()); // Covariance Matrix
    pname[2] = Form("HARPData/%s/%smomentumBoundsArray",HadronName.c_str(),HadronAbriviation.c_str()); // Momentum Bounds
    pname[3] = Form("HARPData/%s/%sthetaBoundsArray",HadronName.c_str(),HadronAbriviation.c_str()); // Theta Bounds

    HARPXSec = (TMatrixD*) file->Get(pname[0].c_str());
    HARPCov  = (TMatrixD*) file->Get(pname[1].c_str());

    TArrayD* HARPmomentumBoundsArray = (TArrayD*) file->Get(pname[2].c_str());
    HARPmomentumBounds = 
      PrimaryHadronSWCentralSplineVariationWeightCalc::ConvertToVector(HARPmomentumBoundsArray);    

    TArrayD* HARPthetaBoundsArray = (TArrayD*) file->Get(pname[3].c_str());
    HARPthetaBounds = 
      PrimaryHadronSWCentralSplineVariationWeightCalc::ConvertToVector(HARPthetaBoundsArray);    
    
    /////////////////
    //
    //   Extract the Sanford-Wang Fit Parmeters
    //
    ////////////////

    
    std::string ExternalFitInput = sp.find_file(fitInput);
    Fitfile = new TFile(Form("%s",ExternalFitInput.c_str()));

    std::string fitname; // these are what we will extract from the file

    fitname = Form("SW/%s/SW%sFitVal",HadronName.c_str(),HadronName.c_str()); // Sanford-Wang Fit Parameters

    TArrayD* SWParamArray = (TArrayD*) Fitfile->Get(fitname.c_str());
    SWParam = 
      PrimaryHadronSWCentralSplineVariationWeightCalc::ConvertToVector(SWParamArray);    




    //Prepare random generator
    art::ServiceHandle<art::RandomNumberGenerator> rng;
    fGaussRandom = new CLHEP::RandGaussQ(rng->getEngine(GetName()));
   
     //
    //  This part is very important. You will need more than a single random number
    //  per event. In fact you will need per universe as many random numbers as there
    //  are rows in the covariance matrix. 
    //
    //  To properly perform Cholesky decomposition you'll use a correlated set of random
    //  numbers when creating the smeared cross section methods. Please see documentation
    //  in WeightCalc to help with this.
    //
    
    if(fUseMBRands){//fParameter_list
      fWeightArray = PrimaryHadronSWCentralSplineVariationWeightCalc::MiniBooNERandomNumbers(fParameter_list.at(0));
    }//Use MiniBooNE Randoms
    else{
      
      fWeightArray.resize(2*fNmultisims);
      
      for (unsigned int i=0;i<fWeightArray.size();i++) {
	fWeightArray[i].resize(HARPCov->GetNcols());
	if (fMode.find("multisim") != std::string::npos ){
	  for(unsigned int j = 0; j < fWeightArray[i].size(); j++){
	    fWeightArray[i][j]=fGaussRandom->shoot(&rng->getEngine(GetName()),0,1.);
	  }//Iterate over the covariance matrix size
	}
	else{
	  std::fill(fWeightArray[i].begin(), fWeightArray[i].end(), 1.);
	}
      }//iterate over the number of universes 
    }

    //////
    // Decompose the Covariance Matrix here that way we don't have to do it many times
    /////

    //perform Choleskey Decomposition
    TDecompChol dc = TDecompChol(*(HARPCov));
    if(!dc.Decompose())
      {
    	throw art::Exception(art::errors::StdException)
	  << "Cannot decompose covariance matrix to begin smearing.";
      }

    //Get upper triangular matrix. This maintains the relations in the
    //covariance matrix, but simplifies the structure.
    fIsDecomposed = true;
    HARPLowerTriangluarCov = new TMatrixD(dc.GetU());  
       
  }//End Configure

  std::vector<std::vector<double> > PrimaryHadronSWCentralSplineVariationWeightCalc::GetWeight(art::Event & e)
  {

    //Collect the event's Flux information
    //     This specifically deals with the neutrino type and parentage
    art::Handle< std::vector<simb::MCFlux> > mcFluxHandle;
    e.getByLabel(fGenieModuleLabel,mcFluxHandle);
    std::vector<simb::MCFlux> const& fluxlist = *mcFluxHandle;

    
    //Collect event's MC truth information
    //  This specifically deals with the neutrino energy and 
    //  counting how many interactions there are per event 
    //  (neutrino counting is CRITICALLY important for applying the 
    //   correct weights and not ending up with unphysical values)
    art::Handle< std::vector<simb::MCTruth> > mctruthHandle;
    e.getByLabel(fGenieModuleLabel,mctruthHandle);
    std::vector<simb::MCTruth> const& mclist = *mctruthHandle;
    
    //Create a vector of weights for each neutrino 
    std::vector< std::vector<double> > weight;
    weight.resize(mclist.size());
       
    //If there aren't any neutrinos in the event then just return an empty weight vector
    if(mclist.size() == 0 || fluxlist.size() == 0){return weight;}

    // Let's start by iterating through each of the neutrino interactions 
    for(unsigned int inu = 0; inu < fluxlist.size(); inu++){

      // First let's check that the parent of the neutrino we are looking for is 
      //  the particle we intended it to be, if not set all weights to 1
      if (fluxlist[inu].ftptype != fprimaryHad){
	weight[inu].resize(fNmultisims);
	std::fill(weight[inu].begin(), weight[inu].end(), 1);
	continue; //now move on to the next neutrino
      }// Hadronic parent check
          
      //Let's make a weights based on the calculator you have requested       
      
      if(fMode.find("multisim") != std::string::npos){       
	for (unsigned int i = 0; int(weight[inu].size()) < fNmultisims; i++) {

	  if(fWeightCalc.find("MicroBooNE") != std::string::npos){

	    //
	    //This way we only have to call the WeightCalc once
	    // 
	    std::pair<bool, double> test_weight =
	      MicroBooNEWeightCalc(fluxlist[inu], fWeightArray[i]);

	    if(test_weight.first){
	      weight[inu].push_back(test_weight.second);
	    }
	  }
	  if(fWeightCalc.find("MiniBooNE") != std::string::npos){

	    //
	    //This way we only have to call the WeightCalc once
	    // 
	    std::pair<bool, double> test_weight =
	      MiniBooNEWeightCalc(fluxlist[inu], fWeightArray[i]);

	    if(test_weight.first){
	      weight[inu].push_back(test_weight.second);
	    }
	  }
	}//Iterate through the number of universes      
      } // make sure we are multisiming
    }//Iterating through each neutrino 

    

    return weight;
  }

  //////////////////////////////
  ////////       Auxilary Functions
  //////////////////////////////

  //// 
  //   Use the MiniBooNE Implementation to determine the weight 
  ////
  std::pair< bool, double> PrimaryHadronSWCentralSplineVariationWeightCalc::MiniBooNEWeightCalc(simb::MCFlux flux, std::vector<double> rand){
    
    bool parameters_pass = true;
    
    ///////////////////////////////
    //
    // We now know that we have the hadronic parent that we want so we need to
    // prepare the kinematic information about the iteraction to go into the Sanford-Wang
    // calculation
    //
    // Computations are based on MiniBooNE code that can be found here:
    // 
    //   http://cdcvs0.fnal.gov/cgi-bin/public-cvs/cvsweb-public.cgi/~checkout~/...
    //                            miniboone/AnalysisFramework/Utilities/src/PhysicsFunctions.F
    //
    //  >>> Based on MiniBooNE function MultisimMatrix_RawMesonProd
    //
    //
    //
    //   Useful references:
    //       MiniBooNE Collaboration, PhysRevD.79.072002
    //       D.Schmitz Thesis; http://lss.fnal.gov/archive/thesis/2000/fermilab-thesis-2008-26.pdf
    //
    //////////////////////////////

    double c1 = SWParam[0];
    double c2 = SWParam[1];
    double c3 = SWParam[2];
    double c4 = SWParam[3];
    double c5 = SWParam[4];
    double c6 = SWParam[5];
    double c7 = SWParam[6];
    double c8 = SWParam[7];
    double c9 = 1.0; // This isn't in the table but it is described in the text
    
    //  Lay out the event kinimatics 
    double HadronMass;
      
      if(fabs(fprimaryHad) == 211) HadronMass = 0.13957010; //Charged Pion
      else{ 
	throw art::Exception(art::errors::StdException)
	  << "sanford-wang is only configured for charged pions ";
      }

      TLorentzVector HadronVec; 
      double HadronPx = flux.ftpx;
      double HadronPy = flux.ftpy;
      double HadronPz = flux.ftpz;
      double HadronE  = sqrt(HadronPx*HadronPx + 
			     HadronPy*HadronPy + 
			     HadronPz*HadronPz + 
			     HadronMass*HadronMass);
      HadronVec.SetPxPyPzE(HadronPx,HadronPy,HadronPz,HadronE);

      ////////
      //  
      //   Based on MiniBooNE code to evaluate the theta value within below the maximum 
      //    HARP theta coverage. This helps keep the splines well formed and constrains  
      //    the uncertainties at very low neutrino energy
      //
      ////////

      double ThetaOfInterest;
      if(HadronVec.Theta() > 0.195){
	ThetaOfInterest = 0.195;
      }
      else{
	ThetaOfInterest = HadronVec.Theta();
      }


      // Get Initial Proton Kinitmatics 
      //   CURRENTLY GSimple flux files drop information about 
      //   the initial state proton, but that this 
      TLorentzVector ProtonVec;
      double ProtonMass = 0.9382720;
      double ProtonPx = 0;
      double ProtonPy = 0;
      double ProtonPz = 8.89; //GeV
      double ProtonE  = sqrt(ProtonPx*ProtonPx +
                             ProtonPy*ProtonPy +
                             ProtonPz*ProtonPz +
                             ProtonMass*ProtonMass);
      ProtonVec.SetPxPyPzE(ProtonPx,ProtonPy,ProtonPz,ProtonE);
      
      //Sanford-Wang Parameterization 
      //  Eq 11 from PhysRevD.79.072002

      double CV = c1 * pow(HadronVec.P(), c2) * 
	(1. - HadronVec.P()/(ProtonVec.P() - c9)) *
	exp(-1. * c3 * pow(HadronVec.P(), c4) / pow(ProtonVec.P(), c5)) *
	exp(-1. * c6 * ThetaOfInterest *(HadronVec.P() - c7 * ProtonVec.P() * pow(cos(ThetaOfInterest), c8)));

      // Check taken from MiniBooNE code
      if(float(HadronVec.P()) > (float(ProtonVec.P()) - float(c9))){
	CV = 0;} 

      //////
      //
      // Now that we have our central value based on the Sanford-Wang parameterization we 
      // need to create variations around this value. MiniBooNE did this by performing 
      // spline fits to the HARP data. This is a reproduction of that code:
      // 
      ////

      double* HARPmomentumBins = HARPmomentumBounds.data(); // Convert std::vector to array
      double* HARPthetaBins = HARPthetaBounds.data();       // Convert std::vector to array

      int Ntbins = int(HARPthetaBounds.size()) - 1;
      int Npbins = int(HARPmomentumBounds.size()) - 1;
      
      //
      //  Using the HARP cross section and covariance matrices 
      //  we will now create variations around the measured HARP
      //  meson production cross sections. 
      //
      
      // Important notes of the HARP data: 
      //  The cross section matrix has 13 momentum bins and 6 theta bins 
      //  The covariance matrix contains the correlated uncertainties across
      //  all 78 cross section measurements. 
      //
      //  The covariance matrix encodes the uncertainty on a given HARP 
      //  ANALYSIS BIN instead of being a 3D matrix. 
      //
      //  A HARP analysis bin is defined as 
      //  bin = momentum[theta[]] meaning that:
      //      analysis bin 0 is the zeroth theta and zeroth momentum bin
      //  but analysis bin 27 is the 3rd theta and 5th momentum bin
      // 
      // The first thing to do is convert our cross section matrix into 
      // an std::vector for the analysis bins
      //
      std::vector< double > HARPCrossSectionAnalysisBins;
      HARPCrossSectionAnalysisBins.resize(int(Ntbins*Npbins));

      int anaBin = 0;
      for(int pbin = 0; pbin < Npbins; pbin++){
	for(int tbin = 0; tbin < Ntbins; tbin++){	
	  HARPCrossSectionAnalysisBins[anaBin] = HARPXSec[0][pbin][tbin];
	  anaBin++;
	}
      }
      
      //
      // Now using this we can vary the cross section based on the HARP covariance matrix 
      // this will allow us to reweigh each cross section measurement based on
      // the multigaussian smearing of this matrix. 
      //

      std::vector< double > smearedHARPCrossSectionAnalysisBins = 
	WeightCalc::MultiGaussianSmearing(HARPCrossSectionAnalysisBins, HARPLowerTriangluarCov, fIsDecomposed,rand); 
      HARPCrossSectionAnalysisBins.clear();

      //
      //   Check all the smeared cross sections, if any come out to be negative then 
      //   we will not pass this given parameter set.
      //

      for(int check = 0; check < int(smearedHARPCrossSectionAnalysisBins.size()); check++){
	if(smearedHARPCrossSectionAnalysisBins[check] < 0){ parameters_pass = false;}
      }

      //  
      // With a checked set of smeared cross sections we can convert our analysis bin
      // std::vector back into a TMatrixD, though this is a bit annoying since TMatrices 
      // are an annoying format. 
      //
      TMatrixD*  smearedHARPXSec = new TMatrixD(Npbins,Ntbins);
      
      anaBin = 0;
      for(int pbin = 0; pbin < Npbins; pbin++){
	  for(int tbin = 0; tbin < Ntbins; tbin++){
	  smearedHARPXSec[0][pbin][tbin] = smearedHARPCrossSectionAnalysisBins[anaBin];	  
	  anaBin++;
	}
      }

      //
      // We want to now make a vector of histograms that are 
      //  projections across the cross section matrix, 
      //  this means we want to have vectors where one histogram 
      //  is made for each bin on variable X where the histogram
      //  then contains bins for variable Y. 
      //
      //  This will allow us to do a 2D interpolation across the
      //  full parameter space using cubic spline fits 
      //

      std::vector< TH1F > MomentumBins;
      MomentumBins.resize(Ntbins);
      for(int bin = 0; bin < Ntbins; bin++){
	MomentumBins[bin] = TH1F("HARPp",";;;", Npbins, HARPmomentumBins);
	MomentumBins[bin].SetBit(kCanDelete);
      } 
      
      //Setup vectors of splines for fitting
      std::vector< TSpline3 > SplinesVsMomentum; 
      SplinesVsMomentum.resize(Ntbins);

      for(int tbin = 0; tbin < Ntbins; tbin++){
	  for(int pbin = 0; pbin < Npbins; pbin++){
	
	    //
	    // We want to create spline at fixed theta values, that way we can 
	    // study the cross section at the given meson momentum
	    MomentumBins[tbin].SetBinContent(pbin+1, smearedHARPXSec[0][pbin][tbin]);		    
	    // This will give us the cross section in discreet slices of theta (tbin)
	    // but each histogram in the vector will be the cross section in bins of
	    // momentum that can then be cublic splined  
	    //
	  }
	
	  //
	  // in a given theta slice we can now spline over the cross section entries
	  // in bins of momentum
	  //
	  // Important note about Splines, MiniBooNE used constraints on the
	  // second derivative of the first and final knot point and required 
	  // that they both equal to zero, this is not naturally the case in 
	  // in TSpline3 but it is default in DCSPLC (the Fortran CERN library spline function)
	  // this helps to control the smoothness of the spline and minimizes the variation bin to bin
	  // For TSpine3 this is controlled by:
	  //
	  //  b1 = constrain first knot 1st derivative 
	  //  b2 = constrain first knot 2nd derivative 
	  //  e1 = constrain final knot 1st derivative 
	  //  e2 = constrain final knot 2nd derivative 
	  //
	  //  the numbers that follow are the values you constrain those conditions to
	  //
	  SplinesVsMomentum[tbin] = TSpline3(&(MomentumBins[tbin]),"b2e2",0,0); 

      }
      MomentumBins.clear();
      delete smearedHARPXSec;
      //
      // Now that we have the 1D Splines over momentum we want to know
      // what the cross section is for the meson's momentum in bins of
      // theta that way we can extract the cross section at the meson's
      // theta. 
      //
      TH1F* ThetaBins = new TH1F("HARPt",";;;", Ntbins, HARPthetaBins);
      ThetaBins->SetBit(kCanDelete);
      for(int tbin = 0; tbin < Ntbins; tbin++){
	ThetaBins->SetBinContent(tbin+1, SplinesVsMomentum[tbin].Eval(HadronVec.P()));
      } 

      ////      
      // Now we can spline across the theta bins and we can extract the exact value of the 
      // cross section for the meson's theta value
      //
      //  Options described above.
      /////

      TSpline3* FinalSpline = new TSpline3(ThetaBins,"b2e2",0,0);

      double RW = FinalSpline->Eval(ThetaOfInterest);

      SplinesVsMomentum.clear();
      delete ThetaBins;
      delete FinalSpline;

      //
      // These guards are inherited from MiniBooNE code
      //
      // These were defined here: 
      // 
      //   cdcvs0.fnal.gov/cgi-bin/public-cvs/cvsweb-public.cgi/~checkout~/ ... 
      //              miniboone/AnalysisFramework/MultisimMatrix/src/MultisimMatrix.inc    
      //
      //  This forces any negative spline fit to be 1     

      ////////
      //  Possible Bug.
      ////////     
      // This seems to be a feature in the MiniBooNE code
      //  It looks like the intension is to set this to zero
      //  but it is set to 1 before it is set to zero  

      double weight = 1; 

      if(RW < 0 || CV < 0){
	weight = 1;
      }
      else if(fabs(CV) < 1.e-12){
	weight = 1;
      }
      else{
	weight *= RW/CV;
      }

      if(weight < 0) weight = 0; 
      if(weight > 30) weight = 30;
      if(!(std::isfinite(weight))) weight = 30;//From MiniBooNE; in Fortran Nan > 30
     
      std::pair<bool, double> output(parameters_pass, weight);

      return output; 

  }// Done with the MiniBooNE function


  //// 
  //   Use the MicroBooNE Implementation to determine the weight 
  ////
  std::pair<bool, double> PrimaryHadronSWCentralSplineVariationWeightCalc::MicroBooNEWeightCalc(simb::MCFlux flux, std::vector<double> rand){

    // 
    //  Largely built off the MiniBooNE code 
    //  but this is intended to expand beyond it
    //
    //  Edits from MiniBooNE code:
    //
    //  JZ (6/2017) : Remove max weight, set to max_limit of a double
    //  JZ (6/2017) : Changed guards on c9 to be double point precision 
    //   
    

    bool parameters_pass = true;
    
    double c1 = SWParam[0];
    double c2 = SWParam[1];
    double c3 = SWParam[2];
    double c4 = SWParam[3];
    double c5 = SWParam[4];
    double c6 = SWParam[5];
    double c7 = SWParam[6];
    double c8 = SWParam[7];
    double c9 = 1.0; // This isn't in the table but it is described in the text
    
    //  Lay out the event kinimatics 
    double HadronMass;
      
      if(fabs(fprimaryHad) == 211) HadronMass = 0.13957010; //Charged Pion
      else{ 
	throw art::Exception(art::errors::StdException)
	  << "sanford-wang is only configured for charged pions ";
      }

      TLorentzVector HadronVec; 
      double HadronPx = flux.ftpx;
      double HadronPy = flux.ftpy;
      double HadronPz = flux.ftpz;
      double HadronE  = sqrt(HadronPx*HadronPx + 
			     HadronPy*HadronPy + 
			     HadronPz*HadronPz + 
			     HadronMass*HadronMass);
      HadronVec.SetPxPyPzE(HadronPx,HadronPy,HadronPz,HadronE);

      ////////
      //  
      //   Based on MiniBooNE code to evaluate the theta value within below the maximum 
      //    HARP theta coverage. This helps keep the splines well formed and constrains  
      //    the uncertainties at very low neutrino energy
      //
      ////////

      double ThetaOfInterest;
      if(HadronVec.Theta() > 0.195){
	ThetaOfInterest = 0.195;
      }
      else{
	ThetaOfInterest = HadronVec.Theta();
      }


      // Get Initial Proton Kinitmatics 
      //   CURRENTLY GSimple flux files drop information about 
      //   the initial state proton, but that this 
      TLorentzVector ProtonVec;
      double ProtonMass = 0.9382720;
      double ProtonPx = 0;
      double ProtonPy = 0;
      double ProtonPz = 8.89; //GeV
      double ProtonE  = sqrt(ProtonPx*ProtonPx +
                             ProtonPy*ProtonPy +
                             ProtonPz*ProtonPz +
                             ProtonMass*ProtonMass);
      ProtonVec.SetPxPyPzE(ProtonPx,ProtonPy,ProtonPz,ProtonE);
      
      //Sanford-Wang Parameterization 
      //  Eq 11 from PhysRevD.79.072002

      double CV = c1 * pow(HadronVec.P(), c2) * 
	(1. - HadronVec.P()/(ProtonVec.P() - c9)) *
	exp(-1. * c3 * pow(HadronVec.P(), c4) / pow(ProtonVec.P(), c5)) *
	exp(-1. * c6 * ThetaOfInterest *(HadronVec.P() - c7 * ProtonVec.P() * pow(cos(ThetaOfInterest), c8)));

      // Check taken from MiniBooNE code
      if((HadronVec.P()) > ((ProtonVec.P()) - (c9))){
	CV = 0;} 

      //////
      //
      // Now that we have our central value based on the Sanford-Wang parameterization we 
      // need to create variations around this value. MiniBooNE did this by performing 
      // spline fits to the HARP data. This is a reproduction of that code:
      // 
      ////

      double* HARPmomentumBins = HARPmomentumBounds.data(); // Convert std::vector to array
      double* HARPthetaBins = HARPthetaBounds.data();       // Convert std::vector to array

      int Ntbins = int(HARPthetaBounds.size()) - 1;
      int Npbins = int(HARPmomentumBounds.size()) - 1;
      
      //
      //  Using the HARP cross section and covariance matrices 
      //  we will now create variations around the measured HARP
      //  meson production cross sections. 
      //
      
      // Important notes of the HARP data: 
      //  The cross section matrix has 13 momentum bins and 6 theta bins 
      //  The covariance matrix contains the correlated uncertainties across
      //  all 78 cross section measurements. 
      //
      //  The covariance matrix encodes the uncertainty on a given HARP 
      //  ANALYSIS BIN instead of being a 3D matrix. 
      //
      //  A HARP analysis bin is defined as 
      //  bin = momentum[theta[]] meaning that:
      //      analysis bin 0 is the zeroth theta and zeroth momentum bin
      //  but analysis bin 27 is the 3rd theta and 5th momentum bin
      // 
      // The first thing to do is convert our cross section matrix into 
      // an std::vector for the analysis bins
      //
      std::vector< double > HARPCrossSectionAnalysisBins;
      HARPCrossSectionAnalysisBins.resize(int(Ntbins*Npbins));

      int anaBin = 0;
      for(int pbin = 0; pbin < Npbins; pbin++){
	for(int tbin = 0; tbin < Ntbins; tbin++){	
	  HARPCrossSectionAnalysisBins[anaBin] = HARPXSec[0][pbin][tbin];
	  anaBin++;
	}
      }
      
      //
      // Now using this we can vary the cross section based on the HARP covariance matrix 
      // this will allow us to reweigh each cross section measurement based on
      // the multigaussian smearing of this matrix. 
      //

      std::vector< double > smearedHARPCrossSectionAnalysisBins = 
	WeightCalc::MultiGaussianSmearing(HARPCrossSectionAnalysisBins, HARPLowerTriangluarCov, fIsDecomposed,rand); 
      HARPCrossSectionAnalysisBins.clear();

      //
      //   Check all the smeared cross sections, if any come out to be negative then 
      //   we will not pass this given parameter set.
      //

      for(int check = 0; check < int(smearedHARPCrossSectionAnalysisBins.size()); check++){
	if(smearedHARPCrossSectionAnalysisBins[check] < 0){ parameters_pass = false;}
      }

      //  
      // With a checked set of smeared cross sections we can convert our analysis bin
      // std::vector back into a TMatrixD, though this is a bit annoying since TMatrices 
      // are an annoying format. 
      //
      TMatrixD*  smearedHARPXSec = new TMatrixD(Npbins,Ntbins);
      
      anaBin = 0;
      for(int pbin = 0; pbin < Npbins; pbin++){
	  for(int tbin = 0; tbin < Ntbins; tbin++){
	  smearedHARPXSec[0][pbin][tbin] = smearedHARPCrossSectionAnalysisBins[anaBin];	  
	  anaBin++;
	}
      }

      //
      // We want to now make a vector of histograms that are 
      //  projections across the cross section matrix, 
      //  this means we want to have vectors where one histogram 
      //  is made for each bin on variable X where the histogram
      //  then contains bins for variable Y. 
      //
      //  This will allow us to do a 2D interpolation across the
      //  full parameter space using cubic spline fits 
      //

      std::vector< TH1F > MomentumBins;
      MomentumBins.resize(Ntbins);
      for(int bin = 0; bin < Ntbins; bin++){
	MomentumBins[bin] = TH1F("HARPp",";;;", Npbins, HARPmomentumBins);
	MomentumBins[bin].SetBit(kCanDelete);
      } 
      
      //Setup vectors of splines for fitting
      std::vector< TSpline3 > SplinesVsMomentum; 
      SplinesVsMomentum.resize(Ntbins);

      for(int tbin = 0; tbin < Ntbins; tbin++){
	  for(int pbin = 0; pbin < Npbins; pbin++){
	
	    //
	    // We want to create spline at fixed theta values, that way we can 
	    // study the cross section at the given meson momentum
	    MomentumBins[tbin].SetBinContent(pbin+1, smearedHARPXSec[0][pbin][tbin]);		    
	    // This will give us the cross section in discreet slices of theta (tbin)
	    // but each histogram in the vector will be the cross section in bins of
	    // momentum that can then be cublic splined  
	    //
	  }
	
	  //
	  // in a given theta slice we can now spline over the cross section entries
	  // in bins of momentum
	  //
	  // Important note about Splines, MiniBooNE used constraints on the
	  // second derivative of the first and final knot point and required 
	  // that they both equal to zero, this is not naturally the case in 
	  // in TSpline3 but it is default in DCSPLC (the Fortran CERN library spline function)
	  // this helps to control the smoothness of the spline and minimizes the variation bin to bin
	  // For TSpine3 this is controlled by:
	  //
	  //  b1 = constrain first knot 1st derivative 
	  //  b2 = constrain first knot 2nd derivative 
	  //  e1 = constrain final knot 1st derivative 
	  //  e2 = constrain final knot 2nd derivative 
	  //
	  //  the numbers that follow are the values you constrain those conditions to
	  //
	  SplinesVsMomentum[tbin] = TSpline3(&(MomentumBins[tbin]),"b2e2",0,0); 

      }
      MomentumBins.clear();
      delete smearedHARPXSec;
      //
      // Now that we have the 1D Splines over momentum we want to know
      // what the cross section is for the meson's momentum in bins of
      // theta that way we can extract the cross section at the meson's
      // theta. 
      //
      TH1F* ThetaBins = new TH1F("HARPt",";;;", Ntbins, HARPthetaBins);
      ThetaBins->SetBit(kCanDelete);
      for(int tbin = 0; tbin < Ntbins; tbin++){
	ThetaBins->SetBinContent(tbin+1, SplinesVsMomentum[tbin].Eval(HadronVec.P()));
      } 

      ////      
      // Now we can spline across the theta bins and we can extract the exact value of the 
      // cross section for the meson's theta value
      //
      //  Options described above.
      /////

      TSpline3* FinalSpline = new TSpline3(ThetaBins,"b2e2",0,0);

      double RW = FinalSpline->Eval(ThetaOfInterest);

      SplinesVsMomentum.clear();
      delete ThetaBins;
      delete FinalSpline;

      //
      // These guards are inherited from MiniBooNE code
      //
      // These were defined here: 
      // 
      //   cdcvs0.fnal.gov/cgi-bin/public-cvs/cvsweb-public.cgi/~checkout~/ ... 
      //              miniboone/AnalysisFramework/MultisimMatrix/src/MultisimMatrix.inc    
      //
      //  This forces any negative spline fit to be 1     

      ////////
      //  Possible Bug.
      ////////     
      // This seems to be a feature in the MiniBooNE code
      //  It looks like the intension is to set this to zero
      //  but it is set to 1 before it is set to zero  

      double weight = 1; 

      if(RW < 0 || CV < 0){
	weight = 1;
      }
      else if(fabs(CV) < 1.e-12){
	weight = 1;
      }
      else{
	weight *= RW/CV;
      }

      if(weight < 0) weight = 0; 
      if(weight > 30) weight = 30; 
      if(!(std::isfinite(weight))){
	std::cout << "SW+Splines : Failed to get a finite weight" << std::endl; 	
	weight = 30;
      }

      std::pair<bool, double> output(parameters_pass, weight);

      return output; 

  }// Done with the MicroBooNE function


  //// 
  //  This converts TArrayD to std::vector< double > 
  ///
  std::vector<double> PrimaryHadronSWCentralSplineVariationWeightCalc::ConvertToVector(TArrayD const* array) {
    std::vector<double> v(array->GetSize());
    std::copy(array->GetArray(), array->GetArray() + array->GetSize(),
	      v.begin());
    return v;
  } // ConvertToVector()


   
   
  std::vector< std::vector< double > > PrimaryHadronSWCentralSplineVariationWeightCalc::MiniBooNERandomNumbers(std::string pdg){
    
    if(pdg.compare("piplus") == 0){	 
      
      return {{ 0.535761714,  -0.157458499,  1.27808535,  1.08446622,  -0.442281604,  -1.09398198,  0.363608241,  0.176137567,  -0.0826628581,  -1.16857898,  -2.27938056,  0.639808297,  -0.893604279,  1.01390862,  0.342893958,  -0.0875861645,  0.183766395,  0.425855458,  1.68170464,  0.239858449,  0.811432421,  -0.554878831,  -0.610771298,  0.0194131285,  -1.08742368,  1.04136932,  -0.513308585,  -1.04078472,  0.845838428,  0.244247034,  0.723694086,  0.256305188,  -0.175894216,  -0.948328376,  0.384049445,  -0.634880602,  -0.117384106,  1.1221869,  -0.7154001,  0.39915207,  2.54072976,  -0.583894491,  0.570770979,  0.224252954,  -1.4132241,  2.0970726,  1.29685724,  -1.40092695,  -0.819060385,  -0.116456158,  -0.82463485,  -0.695861399,  -0.132463381,  0.274692506,  0.234186172,  2.33748174,  -1.19481027,  2.53690195,  1.9345243,  -0.309634805,  0.127442583,  0.72363323,  0.738432825,  0.813476384,  0.439650863,  -0.432810247,  -0.293245971,  0.190662861,  -1.06455958,  -0.961356223,  0.453213215,  -1.63558269,  -2.17806959,  0.402086437,  0.334639043,  -0.264624089,  0.0799060687,  -0.292117923},{ 0.891651928,  -0.766448259,  0.111732461,  0.217581898,  1.48662949,  -0.613187611,  1.37550581,  0.98386234,  1.09035897,  0.311342746,  0.445781648,  1.09721172,  0.526746392,  1.96366978,  0.995747089,  0.756384552,  -0.94863838,  0.368287534,  0.962566197,  1.31495678,  -0.135936409,  -0.925734997,  1.62275624,  -0.751497507,  0.229210824,  -1.00176132,  0.303290188,  0.287525237,  0.710665643,  1.21735513,  -0.283227086,  1.83072388,  0.305953592,  0.936720192,  -0.52222079,  0.0680593252,  -0.794755876,  0.631135046,  1.08162951,  1.75093341,  -0.063933596,  0.591486335,  -0.0910077617,  -0.406172425,  -0.186323494,  -1.27537131,  1.85423636,  -2.02145052,  1.07352722,  -0.142996222,  -2.17887115,  0.254874915,  -0.773599267,  1.23636198,  -0.931446135,  1.65722454,  -1.02952898,  0.418248206,  -1.48584366,  -0.621750355,  0.538120747,  -0.288485944,  0.365411192,  -0.446756333,  -1.64002073,  -0.120838784,  1.19453669,  -0.856133163,  -0.575121999,  -1.36179996,  0.260309577,  -0.321321756,  0.635807872,  1.19937766,  1.31998932,  1.11344576,  -0.541000605,  1.72803843},{ -0.364503562,  0.627133012,  1.25638318,  -0.531116724,  0.370584846,  -0.133664876,  -0.972712278,  0.678416669,  -0.468806535,  0.743427694,  0.348284811,  0.154182613,  -1.32805216,  1.33863866,  0.932451904,  0.383375973,  -0.473894596,  0.378611267,  -0.140463591,  -1.20317137,  -0.944136441,  0.2150767,  1.90159428,  1.38440716,  0.605176389,  -0.800702691,  -0.994069993,  0.342898369,  0.321339309,  -1.14798105,  0.145902574,  0.279771507,  -0.773547888,  0.345466912,  1.37940717,  -0.669960201,  0.405247033,  0.888262033,  -0.430311114,  1.60872185,  0.735255659,  -0.509782851,  0.158109337,  0.984191835,  -0.22882095,  -0.485591114,  1.40371168,  0.722484052,  -0.0108465943,  0.503766656,  -0.440270752,  0.0355273001,  -0.0843760297,  1.89158773,  -0.183392823,  0.515973508,  -0.782403052,  0.972052693,  0.261292338,  0.356324166,  -0.302406698,  0.124756508,  0.584657669,  0.465722412,  0.845016956,  -1.07364023,  -0.0278151929,  0.169950515,  -0.216964006,  0.697301328,  0.604454696,  0.302213341,  -0.169361636,  -0.629430056,  -0.344363421,  -0.155766368,  1.49300396,  -0.709699214},{ -0.106461674,  0.0277657323,  0.460192233,  -0.450444639,  0.371906072,  -0.571183741,  1.33289206,  -1.24631345,  -0.716031909,  2.17232537,  -0.777489543,  0.123529591,  0.275531679,  2.00061464,  -1.59357333,  -0.676029146,  -1.3097012,  3.23431635,  -0.341174155,  0.160310522,  0.802191794,  0.0576612949,  1.14993155,  -0.122691698,  -0.824128926,  0.769506156,  1.20913208,  -2.09975433,  0.545731068,  0.111389413,  2.95126987,  -1.13163793,  0.113887131,  -0.567867637,  -0.532027781,  0.999407947,  -0.735603333,  -2.31212568,  -0.206346869,  -0.198450133,  0.194969609,  -1.18433356,  -0.00832357258,  0.512149453,  -0.171469554,  -0.0870555714,  -0.20384483,  0.889430344,  0.887699187,  0.798014998,  0.0381335653,  -0.958366454,  -0.792371035,  -0.508720875,  0.0939461663,  0.536071897,  2.06229925,  0.677179039,  -1.51035023,  -0.0965621918,  1.31589317,  0.0908534825,  1.50119174,  0.655389249,  -0.416914582,  -0.945259571,  -0.497550189,  1.00366616,  0.50148797,  0.800700009,  -1.29525471,  0.312903076,  1.05838144,  2.12632203,  -0.554131508,  0.784923971,  -0.460536242,  -0.939810991},{ 0.364397496,  -1.2389245,  -1.88126683,  -0.846731782,  -0.876525283,  -0.645682812,  0.0620014891,  -0.57924062,  -1.35864961,  -0.329583615,  0.182615533,  0.594834805,  0.27404654,  1.32341731,  0.782379925,  -0.481386423,  0.00795039441,  0.87612921,  2.44915605,  0.454194814,  -1.24466658,  0.0286886804,  -0.277502596,  0.549023569,  0.560589671,  -0.297070652,  -0.431807935,  -1.22475052,  0.349473745,  1.36365545,  0.332107812,  -0.0212409291,  -0.527721703,  -1.08328462,  0.364854187,  -1.57574654,  0.388962418,  1.23660684,  0.0583897233,  0.176456407,  0.190142512,  -0.400541157,  -0.235348612,  -0.107933976,  -0.691146314,  -0.180485025,  0.00755816232,  0.449743986,  -0.818154335,  1.83104312,  -0.16095072,  0.561244249,  -0.210672006,  -0.622161686,  0.0832657441,  -0.286477327,  1.71247137,  0.643603206,  1.40366697,  0.105953753,  1.70933592,  0.511404455,  1.10200024,  -1.49465024,  0.0853287727,  1.70647132,  -0.416631281,  -0.190220907,  1.36952615,  -0.0177737363,  0.0857332349,  0.100731738,  -0.717123032,  -1.05374825,  -0.990170419,  -0.139606014,  0.818959892,  0.605956435},{ 0.560233116,  0.865569234,  0.308522344,  -1.67638576,  0.62988025,  0.634330928,  0.754705966,  -1.32615256,  0.0571520627,  0.674316823,  1.04359949,  -0.488999724,  -0.643858552,  1.25153744,  0.580180645,  0.497384429,  -0.612829804,  0.672580004,  1.22160876,  -1.49027264,  0.750731587,  -0.887894154,  0.806154251,  -0.278012514,  -0.703384876,  0.716842413,  1.08908355,  0.0319284089,  -0.988127232,  -0.169652015,  0.918349087,  1.02134347,  1.96391463,  -2.30386996,  -1.26088011,  -1.19344389,  1.12551498,  0.0638881028,  -1.58194101,  -0.036318168,  -0.263852865,  0.793631017,  0.50735724,  0.491281927,  0.0519770086,  -1.2585963,  -0.873733163,  -2.91913104,  0.226554468,  0.969425797,  -0.386930972,  -0.630873919,  -1.6182549,  -0.617255211,  0.653215885,  0.424224317,  -1.51075232,  0.630263627,  -0.350194901,  -1.6569041,  0.974684954,  1.41843963,  1.81099927,  -1.50939274,  0.59319222,  0.0382160358,  -0.676367283,  0.85869199,  1.38549054,  -0.283996671,  -0.61376828,  0.564599931,  -0.045785401,  2.59357786,  0.804858446,  -0.450072348,  -1.72245598,  -0.0543025732},{ -0.0659210309,  -0.388336152,  0.832856536,  -0.399845392,  -0.3501921,  0.16759108,  -0.426640958,  0.271620393,  -0.382385671,  -1.00848114,  0.566415131,  0.0429255068,  0.938273311,  -0.231926739,  -1.0163852,  -1.72919261,  -1.71018994,  -0.720217645,  -1.99031091,  -0.323843092,  -0.441034079,  -0.154068395,  -1.2631371,  1.86777067,  -0.185445681,  -0.371598214,  -0.560320377,  -0.108710654,  0.0102184117,  0.147647858,  0.584237397,  1.85154498,  -0.966626167,  0.559841573,  2.00328112,  0.324259043,  0.496147454,  -1.97297025,  1.37441182,  0.095424965,  0.259707004,  0.151607007,  -0.276308686,  0.094396852,  -1.53619754,  -0.66070199,  1.61214137,  -2.35947871,  -0.0406931005,  0.746551335,  0.954723299,  -0.222426921,  -1.56861603,  -1.14753497,  0.539861023,  -0.537263632,  0.262763411,  0.140881166,  -0.383942217,  0.0964587182,  -0.538766265,  -0.0588283017,  -0.163765848,  0.0828080252,  1.09770811,  -1.75479889,  0.718584597,  0.346600085,  0.952228129,  0.125502497,  0.99266994,  0.0135325901,  -1.63532233,  0.0173781458,  -0.324508876,  -1.88863552,  0.064607434,  0.125775039},{ -1.71570837,  -0.491772354,  0.740683138,  -0.412014365,  0.789083302,  -0.0791238844,  -0.179246411,  1.94076455,  -1.06339526,  0.913390219,  0.930383086,  -0.192817435,  -0.340077639,  1.30652082,  -0.369386047,  0.0975267887,  -0.789800286,  -1.08529651,  -1.30832696,  2.08508348,  -1.35009766,  -0.255412906,  1.5038898,  0.976908267,  0.855745912,  -0.557655096,  0.0488197133,  -2.30173087,  -0.830514073,  0.0844392404,  0.503397942,  0.244664893,  0.260770679,  0.539552987,  0.838055372,  0.574979365,  -1.1413238,  -0.659674346,  -0.0338149443,  0.118508525,  -0.656103969,  -0.910755754,  -0.697844446,  -0.138749391,  -0.368782789,  -0.805838227,  -1.08695889,  -1.99396062,  -0.450058818,  0.649590552,  0.195753619,  -0.784366965,  1.54318583,  -0.810902715,  -1.42470467,  0.166676372,  -0.35431549,  0.656616271,  0.972454727,  0.050335478,  -0.520744801,  -1.24258101,  0.824167371,  -1.04506993,  0.305767715,  -1.90638351,  0.0661656708,  -0.165989101,  -0.56406343,  1.79244566,  0.0447233766,  0.133708879,  0.572856009,  0.147637323,  1.31279063,  0.105699979,  0.0698333159,  -0.863336563},{ 1.73001099,  -0.0170298945,  -0.812932432,  0.536906481,  -0.964130998,  -1.48147619,  -0.721426606,  -0.444349051,  -0.261240482,  0.13399151,  0.625198305,  -0.253557503,  0.408920407,  -0.584910989,  0.148227423,  -1.05074644,  -0.0224563759,  0.300359458,  -0.200335056,  -0.484329909,  -0.0550540984,  1.42574215,  -1.57657862,  0.188579708,  0.333119363,  1.88328731,  1.83310902,  -0.229366034,  -1.33272278,  1.17222321,  -0.0980510861,  1.06371355,  -1.46046698,  0.311731189,  2.35391593,  -0.98823446,  1.3219682,  -0.534482241,  -0.381327957,  -0.668742299,  -0.752768397,  0.766403496,  -1.41128945,  -0.961762369,  -2.42796779,  0.92888099,  -0.200111121,  -0.554198384,  0.432611883,  1.12491894,  -0.418533206,  -0.796653152,  -1.51162028,  0.453582615,  0.415656507,  -1.66321671,  -0.909478664,  0.202584863,  0.247153506,  0.0105799129,  1.22169781,  -0.134344682,  -0.827942252,  0.607912719,  0.0492705628,  0.533096015,  0.527301848,  0.782459974,  -0.302613348,  0.23312211,  -1.14854026,  1.15283012,  0.394036412,  -0.186425954,  -1.76965237,  -0.224201068,  -0.835348964,  1.1340971},{ 0.218138605,  0.665380836,  -0.411349475,  -1.04543805,  1.62008905,  0.446760923,  0.157987177,  0.177485779,  -0.213726029,  2.2744894,  -1.80624437,  -1.52704859,  0.374149173,  -1.86080992,  0.214656442,  0.176256791,  0.783421993,  -2.38349462,  2.91944408,  -0.773841858,  -0.691020012,  0.429913849,  -0.239793703,  -0.256659359,  1.15055096,  0.170808166,  -0.470384508,  1.52852857,  0.352347672,  -0.233598635,  -0.485484034,  -0.169045001,  0.867681623,  0.379527122,  -0.778461456,  -0.35933277,  0.233888522,  0.030347392,  -0.579012811,  -1.12881637,  0.504443049,  -1.22647417,  0.125240415,  1.99587703,  -0.190209985,  -0.207141861,  2.12937093,  0.937352479,  1.06991506,  0.293774486,  -0.72265178,  -1.99680018,  0.0927225947,  0.450113118,  -0.707623303,  0.525659919,  -1.46410811,  0.708974779,  0.950543344,  -0.381708354,  3.01772738,  0.561898828,  0.582397044,  1.09070587,  0.183468252,  -0.864182115,  1.0741142,  -0.591026008,  -0.278825372,  -0.908015907,  -0.110734507,  0.580177724,  0.908135831,  1.93610764,  1.29406965,  -1.63149953,  1.45799375,  1.00339174},{ 0.39254275,  -0.150595292,  -0.467517138,  -0.92119199,  -0.489234537,  0.249676898,  -0.159650147,  -1.19222128,  -0.437081218,  0.46018672,  0.9551934,  1.17128849,  -2.72347784,  -1.05027175,  0.984897077,  -1.59155917,  2.07722926,  -0.957801342,  0.31451118,  -0.937729836,  0.0621749237,  -0.349138498,  0.0318226367,  2.35814881,  0.748981178,  -1.01800394,  -2.09491467,  -0.972346544,  0.946114898,  -0.400685877,  -2.24961424,  -0.529677272,  1.28930545,  -0.317823321,  0.655088186,  1.37388277,  0.016423706,  -0.0373904482,  -0.93572861,  -1.06246138,  0.488745391,  -0.976643741,  0.307583123,  0.597384155,  0.574920774,  0.222051755,  -0.547990441,  0.897265077,  -1.56956494,  1.08098924,  0.560642779,  -1.12788951,  1.10045481,  -0.56040138,  0.321216136,  0.275359899,  -0.799779832,  0.18749094,  0.639012873,  -0.721832097,  1.91631114,  -0.0684045479,  -3.18337178,  0.216748103,  0.857075155,  -0.276083231,  0.778946579,  -2.17572546,  -2.60621786,  0.62329787,  0.0163271446,  -0.981318116,  0.738084078,  1.24998677,  0.215194941,  -2.00080371,  0.380638957,  1.38345087},{ -2.50425649,  -0.820248127,  0.493197888,  1.01494157,  -0.20887135,  1.57789445,  -0.876455009,  0.139792919,  -0.893131852,  -1.22597158,  -0.321816325,  -1.04876733,  1.32785881,  -0.599932253,  -2.64873958,  0.68078357,  -2.53240347,  -0.808233082,  -0.940032363,  0.150625691,  0.638974965,  -1.14025629,  0.0671751946,  0.284904838,  -0.400115579,  -0.944708288,  0.56589222,  -0.267354369,  0.0582141653,  1.49899113,  -1.34903562,  0.0456492975,  0.651711583,  1.2137146,  -0.711386859,  -1.93973958,  0.39669925,  -0.25797081,  0.393434495,  -0.932538986,  -0.23527047,  1.11101365,  -0.794315219,  0.54802835,  0.976576984,  -0.87332654,  -0.479160666,  0.253719032,  -0.992270648,  0.55457747,  0.530156493,  0.886591554,  -1.63218284,  -1.09595263,  -1.61730838,  -0.600421727,  1.07276368,  -1.31059778,  0.728287041,  -0.281883031,  -0.318686396,  -0.668766797,  1.55386519,  -1.16427231,  -0.579758108,  1.61257172,  0.801592052,  1.72770798,  -0.098976776,  -0.0667441934,  0.831694961,  0.743537843,  -0.0126968278,  0.340669125,  0.930092871,  -0.0648579374,  0.255214006,  0.825270057},{ -0.0755865201,  -1.47715461,  0.2367789,  0.0134198414,  0.96367836,  -1.1398623,  -2.41101933,  -0.147879198,  -0.563788176,  1.45875955,  -0.300549954,  0.577168882,  -0.944280088,  2.65156674,  -1.2247014,  1.78240693,  -1.95508885,  -0.409035116,  0.209728703,  -1.12830138,  -1.09478056,  -0.561451614,  -1.96538115,  0.0892242491,  1.28276229,  -2.30312014,  -1.07790649,  1.08980227,  -0.0429179147,  1.81519437,  1.8066324,  -0.930450737,  -1.61937284,  0.620101869,  0.169855833,  1.4181726,  2.26640463,  0.346969724,  1.79078984,  -0.751890004,  -0.0421503671,  0.120626003,  0.491882473,  -0.112236246,  -0.929167032,  -0.937751472,  1.70413768,  0.99084723,  1.37315822,  -0.0429601781,  -0.0276399087,  -0.764920592,  -0.756832719,  0.37559849,  -0.181232437,  2.45830059,  -0.337141722,  -0.684945285,  -1.13244283,  0.587028444,  0.180423126,  -1.41789556,  -0.691325724,  0.801610112,  0.274141669,  -1.43812048,  1.02085161,  -0.647396922,  0.405533224,  -0.440623403,  0.703195453,  -1.07085371,  -1.87307072,  0.640755355,  0.583123386,  0.683430195,  0.730527759,  -0.225522086},{ -0.300776333,  -0.910571039,  -0.375393778,  0.0951623097,  -1.94800544,  -0.828398705,  0.931699216,  -0.558417678,  -1.27230835,  -1.21591675,  -0.824545979,  1.04944074,  2.03610778,  -0.272137702,  -0.209918231,  -0.908924997,  -1.95396137,  -0.181084126,  0.46611774,  -0.435494035,  -0.192968816,  -0.820841491,  -1.17614925,  -0.83345592,  -0.632047772,  0.284409404,  -0.466109991,  0.87551713,  -1.36877561,  0.53758657,  0.0187873226,  -1.16620624,  -0.787746072,  0.611231387,  1.84497881,  -0.925416291,  1.50868177,  -0.560261905,  -0.282669902,  1.55398476,  0.163417965,  -0.669423401,  0.734780848,  -0.249175444,  0.11007715,  1.51091135,  0.0916898698,  -0.46267572,  -1.23530805,  -0.592999637,  0.551027834,  1.45177794,  0.758428276,  1.55871975,  0.341899395,  -3.02111626,  0.953217149,  -0.0327977166,  -0.786011815,  -1.35786343,  -0.365213633,  -0.119524732,  1.58767843,  0.334482104,  0.350734204,  -1.20478952,  0.668260574,  -0.758026779,  0.711038113,  0.659091294,  -0.750513494,  -1.08594692,  1.17849064,  -1.29641104,  1.16501224,  -1.26757157,  2.27310872,  1.4198488},{ -0.184906378,  -0.404895574,  0.920612335,  -0.40992561,  -0.636480033,  -0.283849001,  -1.28850317,  -0.87177676,  0.352926344,  -0.10339763,  -2.03893352,  -1.08204198,  -1.55744064,  -0.115374364,  -1.48947203,  0.0795655325,  -0.807084739,  -1.14585829,  -1.40294123,  -0.213781506,  -0.803611875,  0.810226321,  -0.763509095,  1.7563237,  1.18245995,  -0.652606666,  -0.177076399,  0.277903229,  0.475785404,  -0.294276357,  -1.03222191,  0.108067758,  0.44074747,  -0.334364027,  -0.0233577359,  -1.05447483,  -0.262786627,  1.49136782,  0.470233262,  -2.50355029,  -1.40263188,  -1.01037836,  -0.109875001,  1.35505509,  0.10239581,  0.0820902959,  -0.0101014199,  0.378843576,  -1.19265831,  0.66861105,  0.019454129,  -0.913159132,  2.36517906,  -0.131574094,  -1.38418341,  -0.631073058,  0.445363462,  0.949961245,  0.353116989,  0.435509443,  -0.649754584,  -1.18402517,  0.995251179,  -0.311416537,  -1.30901325,  -0.0261917822,  -2.09492517,  -0.197816536,  -0.841484845,  -0.137722343,  1.38848782,  1.61108243,  0.724696696,  -0.246387824,  0.887061298,  0.451125771,  -0.8408764,  0.967095017},{ 0.391563714,  1.33187747,  -0.311656326,  -0.802678525,  0.493950099,  0.0407976136,  -1.2573539,  0.40735063,  -0.517250776,  -1.29463851,  -0.130518362,  -0.892933905,  -0.121200472,  -0.259380251,  -0.617495894,  0.0519548878,  1.32689917,  -0.269449472,  2.46835518,  -1.34097219,  -0.0492392629,  -0.220517278,  1.99196053,  -1.42078602,  -1.41264343,  0.428341508,  -0.945307612,  -0.0213994384,  0.224501029,  0.26063475,  0.195449844,  -1.03251147,  0.360330462,  1.08049333,  -0.580641747,  -0.718504727,  -0.0110788299,  -0.534698069,  1.12763643,  -1.53776705,  0.484564573,  0.309634864,  -0.422430277,  -0.178780362,  1.63301682,  0.781423926,  -0.990369916,  -0.252958566,  -0.576198697,  -0.949099362,  -1.43416488,  0.610531926,  0.901586652,  -0.404884756,  0.220808163,  -1.3345679,  0.182133764,  0.93993938,  0.939720929,  -1.28813839,  0.601969719,  0.461714417,  -0.724280059,  0.902906001,  -0.339046597,  -1.50809443,  -0.133163005,  -1.46164274,  -2.23952699,  0.79827261,  0.522443295,  -0.540131867,  1.61807299,  0.636679471,  0.904909074,  -1.11957347,  0.608163476,  -0.190925404},{ 1.39312482,  1.41237259,  0.580132186,  -0.0144284787,  0.20562464,  -0.162978873,  0.0386922546,  0.250730038,  -1.91739047,  0.0112513481,  0.057447955,  -1.50236976,  -0.367981225,  0.704041481,  -0.558125615,  -0.693443596,  0.50142324,  -0.813873827,  0.831789017,  1.05506063,  1.15394783,  1.08900023,  -0.460309327,  0.309528768,  -0.352927178,  -0.174276188,  2.08297586,  -1.85624015,  -0.106070474,  -1.38473105,  -0.35660097,  0.444929451,  -0.0232434012,  -0.896392882,  0.325537235,  -0.838286638,  -0.333999574,  0.24483633,  -1.7897892,  -0.730540335,  -0.587412298,  1.07672513,  1.25624871,  -0.0140839992,  -1.09006238,  1.77488101,  0.186093494,  0.1333801,  -0.151164323,  -0.22033079,  0.104716159,  0.372499079,  -1.40892804,  -0.449598461,  0.329793811,  -0.952408016,  0.618243039,  -1.72844517,  -0.0867511705,  0.248587862,  0.967254877,  -0.896467209,  -1.06356788,  0.829927862,  -1.1055938,  -1.19032764,  -0.90867871,  -0.927378058,  -0.906552255,  0.591707885,  1.85295594,  -0.427879006,  -0.186272115,  -1.00139952,  -0.527923346,  -1.3225373,  0.0835105851,  1.81355476},{ 0.666259587,  -0.100829415,  -0.125669077,  1.12087047,  0.0131070809,  -1.1365546,  -0.114350647,  1.64742649,  1.67838788,  -0.122984968,  -1.62408996,  0.0779115781,  -2.17889428,  0.567383885,  -0.15766871,  -0.996020913,  0.661559939,  2.0565269,  0.595547199,  0.43529284,  -0.354043335,  0.16293034,  1.84117615,  -0.337137789,  1.90202773,  -0.520973802,  -0.191172838,  -0.752510607,  -0.0125846313,  -1.59230924,  -1.71091998,  -0.274904519,  0.873800874,  -1.55142009,  0.854856193,  -0.102422141,  -0.0703960508,  -0.811130941,  -0.692796946,  -0.994863331,  -0.291434497,  -0.400109828,  -1.79846907,  0.946106791,  -0.790291727,  -1.0038861,  1.995749,  -0.981429219,  2.1031599,  0.171865642,  1.11504245,  -0.50896281,  1.13216138,  -0.328770608,  1.29695201,  0.58762908,  -0.547137141,  -0.660578549,  -0.683359027,  1.61135721,  0.0282747969,  -0.767093837,  -1.14170003,  0.0485442951,  0.277681172,  -0.0908954665,  1.0785166,  -0.338283509,  0.236864343,  -0.180475488,  1.18874621,  0.84131825,  1.64617777,  -0.630359352,  -0.559879065,  -0.818819284,  1.14575863,  0.38624081},{ 1.09607375,  -0.0586282127,  1.10917437,  -0.602273345,  -0.378423691,  0.662041724,  0.256896853,  -1.46739447,  -2.23605061,  -0.151049852,  1.46247506,  -2.10824513,  0.21386911,  -2.17933106,  -0.124113783,  1.36987793,  -0.327673674,  0.680884778,  -0.0394980721,  0.9571504,  3.21619773,  2.46248674,  0.0299007427,  1.25115168,  -0.18023783,  1.34016573,  0.839669168,  0.668086767,  0.239028513,  -0.721206784,  -0.711902559,  0.732426584,  0.533207357,  -0.972422183,  0.731324553,  -2.35276127,  1.06047046,  -1.81471705,  -2.27547216,  0.900230408,  0.500510991,  0.362208009,  -0.840323329,  1.65572214,  0.93803972,  0.603654444,  0.304620594,  0.894361377,  -1.31816947,  0.191612065,  1.01079297,  0.951270044,  -0.535164893,  1.35138941,  -1.5426389,  -1.98585367,  -0.782065034,  0.405083895,  -1.85249114,  -0.891584277,  1.66133559,  -0.6022802,  1.11418903,  -0.0391694903,  -1.08192408,  -0.397969842,  -0.40632391,  0.485730201,  -1.03163242,  -0.121551946,  -0.162528947,  0.521636188,  0.96983999,  0.500573277,  0.0390524045,  1.80309319,  1.03574884,  0.295654804},{ 1.67360222,  -1.56737328,  0.551518857,  0.505794406,  0.0981460288,  -0.161019608,  0.961735129,  -0.205932453,  -0.386879683,  -0.888362229,  -1.49345601,  0.386382788,  1.28279936,  0.628589988,  -0.976298153,  0.792498112,  0.240200236,  -0.553658605,  1.17592382,  0.021614816,  0.427456141,  0.0284647997,  0.425062805,  -0.452690244,  0.986742079,  -0.883569658,  1.42830002,  0.789860129,  0.112402916,  -0.483973771,  0.802115083,  -0.113531195,  -0.163019061,  -0.0101078041,  -0.388379306,  0.396124423,  -0.212102696,  -0.12388514,  0.90116179,  0.228365719,  -0.554050028,  -0.472440213,  -0.10369277,  -0.202483714,  -0.0317576565,  1.11355853,  -0.477906317,  1.71472561,  -0.85195446,  0.207147822,  -0.345395833,  0.764685154,  1.7444545,  -0.615616202,  0.528943002,  -3.09377742,  0.690376043,  -0.256335467,  1.0376687,  -0.409142673,  -0.199667796,  -0.316288203,  1.26746583,  0.156711429,  1.36195791,  -0.59861362,  -0.83011049,  -1.43826115,  1.03499389,  -0.236226544,  -0.745000064,  1.29724991,  0.88009727,  0.174609244,  -0.747113407,  -0.116388582,  -0.354823589,  0.153579265},{ 0.856046319,  -0.0694629252,  1.50164139,  -1.52894163,  0.211314678,  0.528875768,  0.356936991,  0.278344423,  -0.38602829,  0.153532445,  1.4237206,  0.811006904,  1.47916579,  2.0303328,  0.738578856,  2.21686912,  0.740433574,  0.221875504,  -0.425714195,  -0.319774449,  0.700483918,  -1.87258387,  -0.225523874,  0.341677815,  -0.429287165,  0.165692642,  -0.165477768,  0.431782931,  0.390394628,  -0.196158692,  2.11333942,  -0.956852794,  1.03244174,  0.476964474,  0.185618401,  1.5871948,  0.996897221,  0.792059839,  0.654973507,  -0.941504061,  -1.13592625,  -0.299573392,  -0.321123153,  -0.229755223,  1.27877259,  -1.73869765,  0.821906507,  1.41216159,  -0.550766945,  -1.09698534,  -0.841069221,  0.942529261,  1.19920933,  -0.279528052,  -0.019590525,  -0.0439900458,  1.4333086,  0.347435057,  0.297496945,  1.24418104,  1.17438078,  0.160482928,  -1.04601395,  0.617116034,  1.97751451,  2.15283966,  1.02394605,  -0.791320503,  1.38300788,  -0.498260379,  -0.611803472,  -1.36312234,  -1.73340929,  1.84673047,  1.84943569,  -0.361070544,  2.13642406,  -2.12652707},{ 0.319810569,  0.290765703,  0.582378507,  -0.0969921499,  0.626579285,  1.64375901,  0.132238939,  -1.63893807,  -1.07420886,  -1.46224439,  1.02360594,  -0.227967501,  -1.19255805,  -0.816823483,  1.39443076,  -0.586774945,  -0.220633551,  -0.753071666,  -1.53802383,  -0.0247704759,  -1.49801576,  0.126677439,  -0.261483073,  0.549370468,  0.197202876,  0.733254254,  -0.586170435,  -1.35414553,  0.988997519,  0.246030822,  -1.18364298,  0.313901365,  -0.807284176,  -0.737749636,  -1.54514694,  -1.08768463,  0.247079298,  0.95148313,  -0.670228899,  0.630759537,  -1.47802269,  -2.06229281,  0.539440691,  0.768776894,  -0.162626758,  0.623491883,  -0.545147181,  -0.574159324,  0.789499223,  -0.822130561,  -0.361398876,  -1.00853527,  -0.708950639,  1.15397143,  -1.40711308,  0.436582148,  0.512354493,  -0.497265667,  -1.04123604,  1.84504771,  -0.425436288,  0.667889416,  -1.22204316,  -0.222348839,  -0.202879414,  0.522871494,  0.422352999,  -0.698778272,  1.2174623,  -2.15803552,  1.3432349,  -0.35880059,  0.801334798,  -0.150114745,  -1.14088631,  -0.903251827,  0.444676608,  0.0116963917},{ -0.240542993,  -1.1354785,  -0.131055266,  -1.11437678,  1.42502701,  0.168417454,  0.0893402025,  -0.481533766,  -0.181262866,  -1.61663163,  -1.38210511,  1.15393221,  0.530144572,  0.30563888,  -0.680278957,  -0.785886765,  -0.279231608,  -0.754865944,  2.33718491,  0.799705207,  -0.225114122,  1.20140851,  1.51662171,  0.223232076,  0.774474025,  -0.736871719,  1.18029058,  0.34845534,  -0.579874277,  -1.56930566,  -0.0999992788,  1.16607523,  -0.44630906,  1.75675964,  -0.22740759,  -2.22499299,  -1.20434892,  -0.772271335,  -0.936514914,  -1.19857562,  0.829941392,  0.122790128,  0.269755423,  -0.231377169,  0.503477871,  0.157922402,  2.44979119,  -1.03736365,  -0.101537131,  -0.586166143,  -0.985819459,  0.0808091238,  0.550163686,  0.333882332,  1.00155139,  0.38902843,  -0.511461139,  0.982350528,  -1.41491961,  -0.198906973,  0.493556589,  -1.93606305,  -1.49609768,  0.315502465,  1.87153876,  -0.209074333,  -1.48291349,  -0.344426662,  1.71816552,  -1.3796978,  -0.217672363,  1.07929552,  -1.30418384,  -2.71025085,  1.20959258,  -0.77567935,  -0.0216908753,  0.187448531},{ -0.219777822,  -0.099134706,  2.43217778,  1.64676535,  1.69571519,  1.31785893,  1.00324774,  -0.496485323,  -0.567364752,  1.99036002,  -0.0974948779,  0.016288314,  -0.0598250441,  1.76490045,  0.445298493,  0.604710817,  0.371785909,  -1.08554387,  -0.00965661556,  1.18423295,  1.16941082,  -0.366311461,  -1.44572198,  -0.910116374,  0.15433526,  0.587576568,  0.34716773,  0.7425071,  1.41093969,  -2.56988811,  0.515551209,  0.584409893,  -1.50711191,  0.959417641,  -0.795034826,  -0.494550556,  -0.0235776436,  0.883825123,  0.172490746,  0.391266257,  1.49770021,  0.653933406,  -0.25763616,  -0.0637207255,  1.51169538,  0.149841592,  -0.681783915,  0.762833476,  -0.709105194,  -0.0470531397,  -1.23696077,  -0.104667284,  0.12773101,  0.914848864,  0.212326348,  0.287708461,  0.619922638,  0.857928097,  1.25697374,  -0.649036407,  0.806696177,  0.476459384,  0.129802912,  0.342612535,  -1.98108566,  0.405261517,  -0.37191081,  0.966651499,  0.195393711,  0.757426083,  0.0390607938,  0.35834676,  0.70200479,  0.214730144,  -0.0805961341,  0.374692112,  -2.03672338,  1.1006341},{ -2.0451026,  0.238885969,  -0.916672111,  -0.412123024,  -1.38667834,  1.10484064,  0.21501939,  -0.617787719,  -0.119165003,  1.78590822,  -0.306788832,  -0.443555504,  0.389927179,  -0.108640425,  -1.00346005,  0.335722655,  0.459088355,  -1.58065844,  0.296557248,  0.578215718,  0.544252515,  2.02962327,  0.00971071422,  -0.320041209,  0.324500889,  0.969829381,  -0.325150162,  0.269340396,  -0.543911457,  -0.962353766,  -1.9279722,  0.387036204,  -0.814081609,  -0.788504124,  1.6269635,  1.72929132,  -0.574392617,  0.320023209,  1.06613708,  0.790235817,  0.449678123,  1.04413569,  0.939836383,  -0.175537497,  0.909180343,  1.79838562,  -0.0400580987,  -0.569221258,  1.46340859,  -1.04297447,  -0.29687345,  0.00939677283,  -0.304875255,  0.531383574,  1.16284418,  -0.0405541062,  -0.746619046,  1.37476492,  0.00488133542,  -1.09550059,  0.370578408,  -0.809173703,  0.682584703,  0.866804481,  1.69378471,  -2.14222145,  0.745199561,  -0.691226959,  1.17091954,  0.198370606,  0.755419493,  1.4522934,  -2.04461741,  -0.202241808,  -0.809531748,  0.0996883065,  -0.631271601,  -0.0892036408},{ 0.137930393,  0.284062833,  -2.5620327,  -0.229949623,  -1.0984447,  0.0729358047,  -0.149102047,  -0.321941227,  0.742407024,  0.458553672,  0.388553381,  -1.46477187,  -1.38335836,  -0.314711183,  0.195285112,  -1.31876624,  -0.0180632025,  -1.54815495,  0.224050805,  -0.505916059,  1.17499793,  -0.862238288,  -0.232523724,  -0.642995119,  0.0485862195,  -0.769169986,  -0.741548777,  0.535495639,  -1.40150416,  0.102452189,  -1.34965622,  1.32078815,  -0.98258996,  0.632693827,  -0.917513967,  0.858926475,  -0.518446445,  0.22763297,  0.227544665,  1.50230181,  0.222941995,  -0.387113214,  1.2346108,  1.60763431,  -0.0269074515,  -0.731712639,  -0.427434295,  -1.02060306,  0.261897713,  1.92959356,  0.47295326,  -0.624525905,  -0.0403812267,  0.229738116,  -0.362805724,  0.00276376004,  -0.393954247,  -1.46004415,  0.856208861,  0.528198361,  0.355430096,  -0.200606212,  -0.952475548,  -0.192382142,  -1.98913491,  -1.76966059,  -0.528112113,  1.35468793,  0.66353333,  -2.24238896,  -0.124490887,  -2.47739458,  -0.769899368,  -0.641052067,  -0.909968853,  1.03276014,  -1.69930625,  -1.46348023},{ -0.741476595,  -1.19548225,  -0.772343695,  1.73445451,  0.855581224,  0.924929142,  -1.02171731,  -1.34944582,  0.735446334,  0.553937674,  -0.160043031,  -0.0572174601,  -0.936093926,  0.286442041,  1.03814566,  1.19745409,  -0.659093082,  -1.68543458,  -1.6531471,  -0.378186196,  1.73377395,  0.55292666,  0.99470377,  0.784853876,  -0.0452584401,  1.29410195,  -1.24855578,  -1.66454816,  -0.602149487,  0.357835829,  0.176909164,  0.106106892,  -1.02334511,  1.93748677,  2.01972604,  0.212353557,  2.39850283,  -0.737704039,  0.797748566,  -1.10184073,  0.420627624,  -0.174904644,  -2.18177629,  0.796914279,  1.33363426,  1.41550291,  -1.38439095,  -1.22708952,  -0.477251619,  -0.550084352,  0.0590456277,  -0.926199675,  -0.930901289,  1.262205,  -0.836277544,  0.36412853,  0.98873198,  -1.74463463,  0.79791832,  -1.83241987,  -0.226614073,  -0.350935608,  0.761185944,  -1.09730542,  -0.634919584,  -0.281078517,  0.756629586,  0.839299142,  0.356287241,  0.80094105,  -0.371196032,  0.243192047,  0.174462631,  0.470956951,  -0.332352281,  0.987618148,  0.114209749,  -0.714746773},{ 0.446000129,  0.69013679,  0.1974006,  -0.57926333,  -0.692168653,  -0.152960002,  -0.341332346,  0.36888662,  0.10416346,  0.482041121,  -0.569711685,  0.994526029,  2.0840137,  0.775548995,  -1.11855507,  0.266831219,  -1.53216124,  -2.03781962,  0.519569218,  -0.09635856,  -0.138666242,  -0.0789761171,  0.0575225577,  0.0999168828,  0.1971284,  -1.4182353,  -1.1296463,  -0.694407284,  0.324605316,  0.164001688,  0.340827703,  0.247142464,  -0.296048284,  1.6896764,  0.976490676,  -0.582040489,  0.0911429748,  -0.0141476206,  -2.04023552,  0.743596375,  -0.109031498,  1.53133273,  0.73417002,  -0.154662594,  0.059350837,  -1.66291296,  1.63100481,  -0.972973466,  1.22823298,  1.32556009,  -0.445540369,  0.359847277,  0.69768393,  0.595766306,  1.22506011,  -0.168081239,  0.0207552146,  -0.612374008,  0.533152938,  -0.0509025902,  1.65460443,  0.465099156,  -0.655343831,  0.408652514,  -1.30439425,  -1.22609723,  -0.422801226,  -1.01632118,  -0.1799739,  0.099057883,  0.964717746,  0.556929767,  0.564091921,  0.781220973,  0.605731308,  -0.0988874733,  -0.931561351,  0.0752503425},{ 0.200337484,  -1.72793531,  -1.84940946,  1.60243881,  0.194403648,  1.41711223,  0.599126279,  -0.402692139,  -0.426807344,  -0.121990278,  1.23662484,  0.0378408395,  0.0279340371,  1.12816381,  0.610632598,  -0.0291926749,  0.839389026,  -0.719337642,  -0.967891991,  0.168164954,  -1.00178623,  -0.785260618,  -0.711084604,  1.05310118,  -0.582598865,  -1.18626022,  -2.00371242,  -1.00170624,  0.238423884,  0.578898191,  1.74882257,  1.54248571,  0.419815004,  -0.0793129057,  1.54045415,  -0.872170866,  0.46735394,  0.608653247,  -1.15245771,  -0.0280589052,  0.106019542,  1.42063546,  -0.65245527,  0.265596062,  0.350353897,  0.420880765,  -0.517700672,  -1.32985759,  2.09529686,  -0.0598556623,  1.03964877,  -0.130391389,  -2.53960419,  0.403671384,  1.18981636,  1.52981269,  0.437947929,  -1.02999485,  0.648885012,  -0.452542722,  -1.5266875,  -1.37061536,  0.339690953,  -0.317855209,  -0.303870648,  -2.41168761,  0.650831342,  -0.409301132,  -0.30326122,  -0.735027075,  0.262622952,  0.74776423,  1.00403678,  -1.15173328,  1.21344531,  -2.68265629,  -0.800121963,  -0.95974499},{ 0.71366781,  -1.18335724,  -0.119696528,  0.804971159,  -0.952998817,  0.0283365399,  -1.43794501,  0.887840569,  -0.823791564,  -0.0954139605,  0.0605490319,  1.8316983,  0.594354331,  -0.239287078,  -0.749004185,  -1.00666094,  1.31733739,  0.145527348,  1.05393291,  0.4908925,  -0.896784246,  0.54311198,  1.29246891,  -1.09343684,  0.80019778,  0.313021034,  -0.374929249,  1.53178167,  0.12902911,  2.28991818,  0.164668277,  0.971439958,  -0.995931029,  -0.731173933,  -0.367055327,  2.24996805,  -1.48128498,  -0.618848383,  1.65563297,  -0.725256801,  -0.548568726,  0.214885801,  0.0948370174,  0.959994733,  -1.18296921,  0.703549504,  -0.417533815,  -1.01338983,  0.984856546,  -0.271905929,  -1.49341965,  1.19069016,  0.236244768,  0.747533798,  0.766887248,  1.72687888,  0.787813783,  -0.457355291,  1.7984041,  1.96057713,  1.11560416,  0.672198713,  -1.13832545,  -0.247139603,  1.33143508,  -0.215089276,  0.217163712,  -0.431007802,  0.543176889,  -0.258338153,  0.736761212,  -2.1496892,  -0.0672952533,  0.241544887,  0.174228609,  0.663889766,  0.697683454,  -0.674479187},{ 0.404450625,  -0.346344203,  -0.0163437743,  -1.46192169,  -1.23875594,  1.59326112,  -0.659175754,  -0.292907953,  -0.760626435,  0.180651441,  0.04222323,  1.37220371,  -0.537710309,  -1.27501833,  0.585131764,  1.61597168,  -1.74716473,  1.42230392,  -0.746839225,  2.24177742,  0.431803793,  0.0643753782,  -0.110340446,  -0.973581791,  0.964669526,  0.421895713,  -0.161172673,  0.0651881397,  -0.116595954,  0.311314106,  0.818501472,  0.164750785,  1.00431192,  2.41099262,  1.47031474,  0.758662641,  -1.12811482,  1.28925431,  0.329482347,  0.572704375,  0.0816574171,  -1.20685339,  -0.227184266,  -0.294493288,  0.156902939,  0.917036712,  2.17553687,  0.31356743,  1.12598073,  -0.217402056,  0.249228388,  0.154328525,  1.57703984,  -0.0203402713,  -1.07026327,  0.216209248,  -1.2516433,  1.50640059,  -0.219482973,  0.78056854,  0.663710177,  0.462960035,  1.56179082,  -0.477900922,  0.985328734,  1.27009213,  -0.088707827,  -0.842158794,  0.360097259,  -0.369183451,  0.556682587,  0.963566303,  1.11507642,  -0.26374051,  0.127471566,  1.06424844,  -0.516219556,  -0.317717135},{ -0.202004805,  -0.402398139,  -1.61346006,  1.07300961,  -1.0727396,  -0.137416959,  0.57886529,  0.860391259,  -1.03002882,  0.469841152,  -0.711053193,  0.926047087,  0.972916543,  -0.720295846,  -1.29902029,  -0.390726864,  0.13533929,  0.984737992,  1.14313018,  -0.170290381,  -0.97558701,  0.672799885,  -0.34424749,  -0.26384005,  -0.885321915,  -0.0704704002,  -0.504949689,  -0.321880192,  2.16444325,  1.67058563,  0.0505420007,  0.558357894,  -0.253721446,  -1.11228764,  0.658665895,  -0.15361762,  -2.08741307,  0.253448516,  0.360378444,  0.748668194,  1.54104447,  -0.0730007365,  1.66692388,  -0.926438153,  0.0184243657,  -0.971573234,  0.0533052608,  1.45442808,  2.07616282,  -0.31812185,  -1.18689299,  0.292337984,  0.630399942,  1.07850754,  1.23329902,  -1.59877777,  -1.56425786,  -0.716314375,  -0.946207047,  -1.51529813,  0.63658911,  0.0438013189,  -0.699885011,  -0.117411166,  0.356780261,  -0.582506478,  0.118386589,  -1.79254854,  -0.436765701,  0.479664981,  2.47388577,  0.34148863,  -1.34234929,  -1.23922276,  0.356775165,  0.323520333,  2.39397693,  0.802227557},{ -0.450050235,  -0.316403836,  -0.57127142,  -1.37330699,  -1.07029533,  0.258236676,  0.74275291,  -0.457422376,  -0.0118954862,  0.454918653,  0.745512247,  1.22264397,  0.449849248,  -0.255403757,  -0.131162375,  -0.82189858,  -0.20784919,  -1.58052993,  -0.49585259,  0.609859109,  0.609113276,  0.38696003,  1.06516278,  -0.0991256908,  1.47703719,  -1.63114238,  -2.16176391,  -0.286480665,  0.534292698,  0.990186214,  -0.394129872,  -1.22547996,  -0.158417314,  -0.265139341,  0.21770972,  0.529527128,  1.04270637,  -0.161323205,  -0.548572838,  0.739609003,  0.546607494,  -0.326648593,  -0.691217661,  -1.36710811,  1.3270129,  -1.39435124,  -2.32074785,  0.166825518,  0.000800409412,  0.184291527,  -1.4348774,  -1.58635056,  0.26229313,  0.210412294,  -0.661476851,  1.33249855,  0.907317996,  -0.901034117,  -0.861084282,  -0.0144926477,  0.186284706,  0.0727038682,  1.36438644,  0.166378379,  -0.290331036,  -0.0751222298,  -0.341105312,  -0.946467876,  -0.861266851,  -0.853367805,  0.288435221,  -1.41124141,  0.436179429,  -1.20764482,  1.56293678,  -0.833205462,  -1.27144706,  1.35614753},{ 0.83624804,  -0.153416514,  0.896986842,  1.82089269,  1.34364474,  -1.39184332,  -0.00966792088,  0.112551972,  -1.04012525,  0.267872512,  1.10778224,  0.788689494,  0.853870571,  -2.13932037,  -1.04383588,  -1.57034898,  1.75981891,  2.87072086,  -0.00121466769,  -1.267344,  1.44988859,  -0.00175578264,  -0.621782482,  -0.153691456,  1.03062153,  0.915568471,  0.202568665,  0.387235552,  0.103461966,  0.24688977,  -0.498581201,  0.0424912237,  0.458032459,  0.54859364,  0.185070872,  0.148207411,  1.7368809,  1.09524989,  -0.185760275,  -0.76460731,  0.918267012,  -0.448941469,  -1.51331568,  1.40875626,  0.270887941,  0.903196573,  1.10424352,  -0.00996674318,  -0.451659232,  0.0734047294,  -1.77404332,  1.49185359,  1.32134581,  -0.825153708,  -0.0962734818,  -0.0730827078,  -0.55289799,  0.224715024,  1.13185537,  -0.490300417,  -1.57151365,  -0.022055164,  -0.564371943,  0.350180715,  0.859313846,  -0.206735089,  -0.426306307,  -0.52599895,  0.913686097,  0.303141475,  -1.00723302,  -2.23115277,  -0.452531368,  -0.665611327,  -0.0395134948,  -1.63184261,  1.40098035,  -1.3660742},{ 0.0977457762,  -0.116971292,  -1.57186401,  1.55193424,  0.0214456674,  1.46816051,  0.241162613,  -1.11986506,  0.894385874,  -1.3234297,  0.536769748,  -2.04837561,  0.426718235,  -0.410284519,  -0.743483126,  -0.729608536,  1.46133649,  0.439760864,  -0.488999933,  -0.940803766,  0.68092972,  0.430478841,  -0.660689354,  -0.301274776,  -0.292617649,  -1.24333453,  -0.0283110365,  0.0177540146,  0.123887435,  0.412719607,  -0.534025311,  -0.450545281,  1.60301232,  -0.97319442,  0.270167351,  -0.758092523,  -1.94893539,  1.15789688,  0.481427461,  1.13643587,  0.917639434,  0.234402969,  0.732592404,  -0.655488133,  -0.756605983,  -1.33439291,  0.499932468,  -0.375357836,  0.772884369,  -1.24536049,  -1.09643149,  0.0957378745,  -0.718841076,  1.0674758,  1.05456519,  -0.396609277,  0.462459981,  1.13532424,  0.00161427329,  -0.732315063,  0.520003021,  -0.853273988,  0.46149081,  0.967184722,  0.137284964,  0.311581284,  -0.642871201,  0.0547901802,  -0.23295863,  0.522630632,  0.85768944,  0.439317763,  0.0206801146,  -0.523290217,  -0.700082481,  -0.382730663,  -1.09516394,  -2.34439683},{ 0.447991192,  1.35178745,  0.148911133,  -0.564053059,  0.836357892,  -0.500752151,  -1.05054832,  0.087027207,  0.942499161,  -2.21183801,  -2.23240185,  1.96223414,  0.522132874,  0.266352981,  -0.313304365,  0.50141418,  -0.524152875,  1.18509245,  -0.910501361,  1.34592891,  -0.846624434,  1.37307274,  -0.445195645,  1.11783993,  1.70160282,  0.680690587,  -0.48041147,  -1.65962219,  -0.384435654,  0.582355738,  0.216002092,  -0.734441757,  0.534900486,  0.967216313,  1.02427185,  -1.01929963,  0.753151357,  0.75212127,  -1.38547528,  0.67916137,  0.583949029,  0.0281997323,  1.54642892,  -0.31887567,  0.49955982,  1.54955614,  -0.641318083,  0.505456924,  1.13588893,  0.924217224,  0.21977295,  -0.387144595,  -1.03217804,  -1.28627932,  0.328691036,  -0.715341032,  -0.152776256,  0.37059924,  1.47686183,  -2.11587787,  1.06022513,  0.464513689,  0.476352572,  0.548417211,  0.884868801,  -1.07762146,  0.806831837,  1.51766455,  0.355558068,  3.32132173,  -0.173510924,  -0.843897521,  0.112893552,  1.56839263,  1.77401137,  0.660739481,  0.359202623,  0.622979939},{ -0.662537754,  -0.202742055,  -0.0473945513,  1.31943285,  0.513867736,  -1.21318233,  0.517709851,  1.08325756,  -0.441925794,  -0.623666167,  0.10095565,  -0.53072381,  0.669429839,  -0.795045495,  0.0617838316,  -0.114398696,  1.65637827,  -0.277516305,  0.683286011,  1.9755193,  0.0435103662,  1.39702272,  -0.375020564,  -1.82198513,  -1.3179009,  -0.246604919,  -1.19052911,  0.948485017,  1.05524302,  -0.189516515,  0.668209732,  -0.692585707,  -0.148245513,  0.234055772,  -2.46118784,  1.00391138,  0.76246351,  0.238418713,  -0.462762505,  0.246477634,  -1.69403338,  1.32257199,  1.04346657,  2.35750961,  0.348826945,  0.0970553458,  -0.942539454,  -1.01287472,  -1.66317546,  0.140849367,  -0.349142462,  -0.858706534,  -0.0203330163,  -0.112499483,  -0.324246675,  -0.146938726,  0.0417801477,  -0.0689015463,  0.0304018967,  -0.227540314,  -2.02320457,  -2.43201447,  -1.32781875,  -1.67737007,  -1.96455145,  0.0615716614,  -0.67251426,  0.865989745,  -0.833923221,  -0.646525979,  1.05013633,  -0.0481965505,  -0.544535279,  -1.63440096,  -1.22401404,  1.02091742,  1.29640937,  -0.258711994},{ -0.0888917819,  0.661151171,  -1.05263269,  -1.82794714,  -0.542013288,  -0.203315824,  -1.14874995,  -0.0503936298,  0.604126334,  0.853955328,  0.160990685,  -0.178365722,  1.26446438,  1.02194285,  -0.257925957,  0.241539448,  -0.873701274,  -1.15259683,  0.941125453,  -0.588851929,  0.66450274,  0.288722366,  -1.8969059,  0.0198708158,  -0.556899846,  -1.288028,  0.443553984,  1.18485487,  -0.60535115,  0.728580773,  -0.152297392,  2.15605664,  0.153380588,  3.7515626,  1.82810462,  -0.123585463,  -0.873401165,  0.546432495,  0.310279012,  -0.0159769934,  -0.0726781264,  0.0931672677,  1.16299677,  1.3363421,  0.349504769,  0.909956336,  1.50246012,  2.34399438,  0.829512179,  -0.4209117,  -0.130010054,  -1.17852259,  -1.1761266,  0.29696849,  0.751010656,  0.484667957,  -2.48618317,  -0.0866193399,  0.27087301,  -1.42244041,  1.41593122,  -0.369229883,  0.386999577,  1.05780411,  -0.58117038,  0.300236642,  0.968552589,  -0.401763946,  0.748000026,  -0.106839813,  0.848013163,  -1.15314841,  0.640423298,  -2.03847814,  -0.220016062,  0.527255356,  0.525042832,  2.0216651},{ -0.0418454856,  -1.46386862,  -0.0125273112,  1.44695115,  -0.705562651,  -0.752609909,  0.582699895,  1.01634252,  0.0696598813,  0.12166594,  -2.17659259,  -1.34546149,  -0.10922417,  1.26094258,  1.02490449,  1.5531069,  -0.185164228,  0.889425516,  0.620348752,  -1.40808797,  -1.2423209,  -0.599672437,  1.8446449,  0.967682719,  -0.681349754,  -0.165960401,  -0.645735919,  0.875153482,  1.56319392,  1.40807879,  1.47385442,  0.956089735,  -0.738794625,  0.834788859,  1.59941351,  0.163683444,  1.13289344,  0.0796381608,  -0.0951677114,  0.0689561442,  0.477257699,  0.981301963,  0.737104058,  -0.93535918,  -1.21244371,  -1.50555325,  -1.90085137,  0.35024187,  -0.494849861,  0.300130814,  -0.518251121,  -1.35829008,  0.0344154872,  -0.0680886582,  -0.174194619,  -0.179979324,  -0.2994726,  -0.259023428,  0.661918223,  -0.668619335,  0.443063796,  -1.57288492,  0.91393888,  -0.366890728,  -1.16702235,  1.60702288,  -0.0830048695,  0.313204169,  -0.0536373854,  0.0899743885,  -1.29045784,  -0.13672024,  0.549028397,  0.265789062,  -0.931644678,  -1.08132064,  -0.114556856,  -0.0462110564},{ 0.506208301,  -0.0110893268,  2.01181746,  -0.0350216851,  1.93267417,  -0.215923965,  -0.125467673,  0.416981041,  -0.315648317,  0.991879702,  1.99902892,  -0.144944951,  -0.694569826,  1.7905674,  0.846854925,  -0.525131524,  0.415612012,  -1.04336047,  -0.0265984014,  -0.480095774,  -1.27180183,  0.0465221778,  -2.1161952,  1.11339056,  1.0452193,  -0.933795631,  -0.964361787,  -0.411107272,  -0.17792578,  -1.20458531,  -0.438282847,  -0.382266551,  0.439434767,  0.0297064018,  0.0570096485,  0.354800552,  0.952576458,  0.34677586,  0.0788195729,  0.966527462,  0.00530288275,  0.34797734,  0.191797271,  0.0864915922,  1.44673586,  -0.26001972,  -1.80312085,  1.44030738,  -0.360352486,  -0.0769481584,  -0.255178571,  0.218117744,  -1.20462894,  -0.0644670129,  0.631350636,  -0.090422526,  -1.26229191,  0.50290978,  -0.771724105,  0.362881541,  0.742701292,  -0.501232684,  -0.082193993,  0.00474810507,  1.89022148,  -0.630936861,  -0.788966775,  0.2776604,  -1.2115829,  0.977752507,  0.451335818,  0.00587966433,  -1.10433507,  -1.13357472,  1.83136535,  -0.533473492,  0.367282152,  0.854802847},{ 1.25271916,  1.38797987,  -0.106491923,  1.284603,  2.27560997,  -0.232443005,  -0.259727418,  1.51390123,  -0.0712759495,  0.364578515,  -0.130337864,  1.40808582,  0.240629062,  2.18856382,  0.134453177,  -0.727359831,  0.363449395,  -0.300919443,  -0.0353025086,  1.07498002,  -2.13894176,  -0.44851318,  -0.0793477371,  -0.337473661,  -0.206339523,  2.74591255,  0.142113358,  1.91913807,  2.55327606,  -1.38947082,  0.703021824,  -1.63356137,  -1.0974201,  -1.96759927,  1.9584645,  -0.457139164,  -1.79233301,  0.665339231,  -1.41037178,  -0.560216546,  0.274823487,  0.28179726,  0.205732137,  0.747825086,  0.236117303,  -2.30564928,  -0.466246843,  -0.349814326,  -1.16855323,  -0.669732094,  -0.578816593,  -0.425813079,  -1.13037086,  0.224414393,  0.666838109,  -0.697277308,  0.326314896,  0.608672261,  -1.15284002,  0.0875601098,  0.221374631,  -0.232682332,  -0.310556948,  0.290490299,  -0.294379741,  0.874159694,  -1.52061844,  -0.0804527402,  -0.856093109,  -0.219410539,  -0.334542394,  0.837881207,  -0.0125249876,  -0.202098399,  1.00338078,  -1.44726872,  -0.751516521,  0.193245873},{ 0.421821177,  1.89471209,  -0.613625109,  0.722282469,  0.549018204,  0.608883083,  0.56184411,  -0.31937027,  -0.71468991,  -0.155037418,  -1.7608192,  0.957329512,  0.0564793423,  -0.657670438,  -0.0236583184,  -0.581705153,  1.61376488,  -0.109265253,  -2.17765737,  -0.958765984,  0.625670254,  -1.63742399,  -1.30608392,  -0.769877255,  -0.472806722,  0.841686845,  0.188885286,  -1.6588887,  1.06271422,  1.13257813,  -0.973882079,  1.50894642,  0.561255157,  -0.22220926,  -0.733766019,  -0.131275326,  0.36937651,  0.604201674,  -0.320453316,  -0.258562952,  -0.129175335,  0.108247176,  0.795648098,  -0.470829964,  -2.15236473,  0.103706598,  0.357905835,  -0.56739825,  1.0374496,  0.169616967,  0.87031287,  -0.361909896,  -0.0663365871,  1.51880848,  0.359336317,  0.320515394,  -0.381274521,  2.92580271,  -0.164896667,  -0.394668728,  2.0575552,  0.541210115,  -0.321716726,  -1.03019536,  0.502825916,  -0.194512621,  -2.20109582,  0.476073563,  -0.218285382,  -0.172094405,  2.40552711,  -1.16968,  2.10743475,  -1.13618791,  -1.01436245,  2.54527116,  -0.937508941,  1.51285553},{ -1.31931889,  -1.942487,  -1.15258825,  -1.04274833,  -0.377583623,  -0.278594762,  -0.34230265,  -0.798118532,  -0.288903087,  0.349272639,  1.71470022,  1.38598275,  0.679796159,  -0.304964542,  -1.08396113,  0.00442110654,  2.46020865,  -0.372837037,  0.0846472681,  -2.11791062,  0.857217133,  -0.0714528263,  -0.0839929208,  0.888400614,  1.81163025,  2.21712589,  0.409614265,  -1.09362733,  -0.052456785,  0.608660758,  -0.643113554,  -0.742550075,  -1.16881096,  -1.08470941,  -0.158855259,  1.42868388,  0.72850889,  -0.464133292,  -0.31212309,  -0.326684505,  2.31787276,  -0.58108294,  1.6980443,  -0.798079908,  0.0478818156,  -1.52888715,  -0.190333873,  -0.968498826,  2.25779939,  1.18232679,  0.753415525,  -0.36072585,  -0.938742995,  0.703151762,  1.72333276,  0.465928465,  -1.38045657,  -0.579492331,  -1.1699245,  -0.590661347,  -0.202170387,  1.865816,  -0.572548449,  -0.702743948,  0.997904122,  0.650627971,  0.525816083,  -0.157842368,  0.120632865,  1.71730316,  1.03940976,  -0.847285926,  -0.112904109,  0.485124052,  1.2601285,  -1.5442723,  0.305884182,  0.617889524},{ -0.396018952,  0.0968716741,  2.08220506,  0.0399379954,  0.794001698,  -0.748656213,  -1.32924485,  0.939415157,  1.54024935,  -0.70748657,  -0.902465105,  -0.722184837,  -0.153029442,  0.656464696,  0.139906645,  -0.167545393,  -2.50672197,  -0.755274832,  0.602962554,  -1.34290516,  -0.411866307,  1.0833627,  -0.00442623161,  1.05356622,  -0.712279975,  0.0253355745,  -0.340236396,  -1.22181964,  0.249079168,  0.922673881,  0.178670943,  1.25249386,  -0.165826887,  -0.383422017,  0.940028369,  -0.777224362,  -0.0962566882,  0.139857188,  1.06588304,  -0.792870343,  1.28498411,  0.365249097,  0.639153659,  0.341864645,  0.317807883,  0.362373561,  0.344131291,  -0.0348955765,  0.357278228,  -0.702321947,  -1.51307654,  -0.677594066,  -0.422406793,  0.694815159,  -0.640858591,  -0.0916103721,  2.76509523,  -0.761618316,  0.838448048,  -0.452770591,  -0.606642783,  0.0883523226,  -1.79379749,  -0.667997181,  -1.69961286,  0.0413873456,  0.216968253,  -0.221594572,  -0.330899745,  -0.701091528,  0.163254037,  -0.205945954,  -1.30323935,  -0.365268469,  -0.361527026,  0.91444689,  -1.37660146,  -0.0861205161},{ 1.04635572,  -0.584086239,  0.873625994,  0.716015577,  -0.00258930377,  1.22234559,  -0.724777937,  -0.495202422,  -0.539076447,  0.893068016,  -0.497073412,  0.93371129,  0.451063544,  0.339899331,  1.2558285,  -1.22720063,  -0.232388496,  -0.576475084,  -1.67760623,  0.974249542,  -1.49444497,  0.656987607,  -0.192274883,  -0.656545877,  -0.696045876,  1.61488724,  0.396579981,  0.185596272,  1.08406401,  1.22476625,  0.720935464,  0.734314322,  1.57078791,  -1.36570835,  0.831392586,  -1.13525224,  -0.0437979065,  -0.461319387,  -0.0345579758,  0.008018733,  -1.09164107,  -0.227354124,  1.92625129,  0.25009343,  -2.70787406,  -0.443455935,  -0.859363139,  1.21067035,  0.843783379,  -0.455806494,  -1.25425649,  -0.159717962,  -0.670362651,  0.518857777,  -0.89386797,  0.0703675672,  -0.435033441,  0.0749431774,  -0.591421902,  -0.961424768,  0.149318114,  0.275574386,  0.764665365,  -2.30124044,  -0.136859909,  0.663171351,  -0.686258197,  0.108574122,  -0.312051237,  0.240892529,  0.396932751,  -0.251700342,  0.141630709,  -0.415415674,  0.475704134,  0.0822790936,  0.580667317,  1.02238882},{ -0.404261112,  -1.84058392,  -0.567831278,  2.18799734,  -0.0185510311,  0.336054891,  -0.46838817,  -0.0328433216,  0.0915913358,  1.49257457,  0.623869777,  1.94045961,  1.49776161,  1.30493331,  1.39539504,  -1.01209295,  -0.741428852,  2.62826085,  -0.984584808,  1.45026159,  -2.41953993,  0.941319287,  0.593379557,  0.458784789,  -0.280095875,  -0.511869371,  -0.83115983,  1.56028736,  1.6452688,  -0.895552695,  -0.447084308,  0.879820228,  -2.33313107,  1.34887278,  -0.762599587,  -1.59270728,  0.0948854536,  -0.278128594,  -0.606020272,  -0.403923064,  -1.37807024,  0.422632962,  2.03842592,  0.369337022,  0.812009871,  0.655439675,  1.2266773,  0.684628367,  -0.749921381,  -1.67865384,  0.339453071,  1.49702334,  -0.360912472,  0.770347476,  0.629182041,  0.047437232,  -0.56711483,  -0.436756074,  -0.72473675,  0.0588160232,  0.966127336,  -0.130635858,  -0.856358409,  -0.646533012,  -1.98045397,  0.978567004,  1.38685989,  -0.323005617,  -0.0669397861,  0.0929599628,  -0.143728733,  0.401701897,  -0.13870433,  -0.0788555145,  2.26942992,  0.325501442,  -0.0608761646,  -1.03102744},{ 0.609960318,  0.529010177,  0.454631805,  0.0867637023,  -0.827644169,  -2.58357644,  0.849632084,  2.19610643,  1.16920078,  -0.762532115,  -0.837505817,  0.443723947,  -0.914893568,  0.118874729,  -0.965428174,  -0.0756699443,  1.46606815,  2.03505063,  0.0381300487,  2.12411046,  1.48529041,  0.237747133,  1.20343518,  -0.821044326,  2.1473043,  0.255451113,  -0.279758424,  -0.298374444,  -0.560801983,  -0.286348134,  -0.72415334,  -1.41850209,  1.54186332,  0.513006389,  0.745080054,  0.89915365,  -0.98194623,  -0.423985511,  -0.783955514,  1.46265304,  -1.58113444,  -0.962470174,  0.642488658,  -1.31908762,  -2.63573813,  0.740467548,  -1.10921168,  0.0879944414,  0.0531224087,  0.106796287,  0.509953618,  -0.382676691,  -1.76925564,  -1.02795315,  1.68821657,  -0.73653847,  0.436812878,  1.94432664,  -0.730786264,  -0.412136614,  0.318155885,  0.79176414,  1.46227753,  0.460529089,  0.240350038,  -1.08418012,  0.158677757,  0.0529473238,  0.529584765,  -0.411499679,  0.38332215,  -1.08530271,  1.03670049,  -0.177554935,  -0.862318277,  -0.0815287381,  0.0575300939,  -0.0946331844},{ 0.605447829,  -0.564040124,  -0.532371104,  -0.732708573,  -0.236020416,  0.688594997,  0.555784881,  -0.702865422,  0.136790276,  -0.0178873651,  0.042056907,  0.87488699,  -0.56318748,  -1.52138984,  0.169964895,  0.848100066,  0.301781356,  0.722738326,  1.96093917,  -0.757865131,  1.10731351,  -2.1033392,  0.188908368,  0.379116833,  -0.611731529,  1.14544499,  0.937676489,  0.5207569,  0.0829979479,  1.0430882,  -0.266382635,  1.92175043,  0.962608039,  0.441296011,  -0.904027283,  -0.453967959,  -1.09387565,  -0.94843775,  0.271829814,  0.276183218,  0.0110409157,  -0.682674289,  0.902750194,  2.57518768,  0.967101038,  0.639823198,  -0.290370256,  1.40226877,  -0.661415458,  -0.513617098,  -0.642508686,  -2.98569608,  -0.383733213,  -2.25150514,  0.73658216,  -1.8379544,  0.803998888,  -0.699455857,  2.10994744,  -2.43618178,  0.364913106,  0.336670041,  -1.0485568,  -0.0775574967,  0.741900265,  1.34771264,  -0.218308687,  2.10107541,  -0.251297086,  -0.514674246,  -1.58310711,  -0.827534914,  0.598155797,  -0.823262393,  -0.200202897,  1.02144396,  -0.101068869,  0.294467688},{ -0.195568576,  -0.176956803,  0.522410452,  -2.02504969,  0.136840522,  1.92081618,  -1.35851252,  0.773250878,  -0.998423338,  1.56443059,  0.73870641,  0.685568094,  -0.0532984994,  -1.1383183,  0.559193671,  1.49865937,  0.857257962,  1.09409773,  -0.172810376,  0.314783812,  -0.943694592,  -0.254838228,  -1.18341982,  -1.19479609,  -1.65963387,  1.01979625,  0.161644205,  -0.0338629633,  -0.788535178,  1.92117667,  0.0363269374,  -0.556982458,  -0.394973338,  -1.21552742,  0.683727801,  -0.851682127,  0.0371624269,  1.53270316,  -1.31740642,  -0.651303709,  1.06432235,  1.14688087,  -0.797650039,  1.64117253,  0.549159527,  0.650052786,  -1.43344069,  0.604409873,  -1.42489636,  0.145777345,  -0.843013406,  1.10747087,  -2.36035299,  -1.2629621,  2.70938325,  0.287609607,  -0.267385721,  1.53236783,  1.57875848,  -0.0219357777,  -1.27265048,  -0.62249583,  -1.27336884,  -0.667479336,  -1.51176345,  -0.873380005,  -0.385021955,  0.913972259,  0.419649541,  0.691270053,  -2.90090942,  0.58265245,  -0.0196538232,  0.973335803,  -0.282902032,  1.01665115,  0.496323586,  0.0464100726},{ -1.11539412,  1.0570401,  -0.0337437913,  -0.607263386,  1.3792187,  0.81959796,  0.0567715205,  2.02429581,  -5.21939692E-05,  0.0760538653,  0.0488566682,  0.162309572,  -1.01577604,  0.877317548,  1.16227567,  -0.00394657208,  -0.0483472645,  0.916828513,  -0.81861347,  1.7907759,  -0.11793083,  0.654134274,  -0.093196407,  -1.06738639,  1.37139142,  -0.0174972005,  0.0811176598,  0.139517486,  -0.0460513122,  0.492975593,  2.30496788,  1.5254457,  -0.659107924,  -0.585358083,  -0.61428827,  0.788616717,  1.00712943,  -1.23560965,  0.422623396,  2.42488003,  0.309969276,  -1.27632201,  0.786978483,  0.680709898,  -1.2415458,  -0.640505672,  -1.33792961,  -0.330746442,  -0.0875166059,  -0.364892513,  0.205602288,  0.128225923,  -2.39374423,  0.346967012,  0.849083126,  -0.744369268,  -0.214743465,  1.48595881,  -0.71719712,  -1.17819571,  -0.0868854597,  -1.30404651,  0.827629626,  0.119040757,  -0.787817717,  0.419348955,  -0.463751048,  0.0809023529,  -0.115413927,  1.19896305,  -0.423525661,  0.522225022,  1.96418226,  -1.16856146,  -0.431559563,  -1.88904834,  -0.775848269,  0.821403146},{ -2.14786863,  0.0846677348,  -1.30316186,  -0.363858134,  -1.03459418,  -0.997298717,  0.941375673,  -2.20444179,  -1.19312191,  -0.520164013,  -1.30209398,  0.237268165,  -0.107848763,  1.26146829,  -0.0219267681,  1.24702466,  0.151243851,  0.277400404,  -1.51085639,  -1.75777292,  -1.17190909,  2.19050527,  -0.133085042,  -0.265172511,  -0.200162053,  -1.20989823,  1.3007741,  0.975793362,  -1.25191915,  1.30284524,  -0.484570414,  -0.923631668,  -0.133418247,  0.574405909,  -1.45594573,  -0.693556905,  -0.0376673974,  -1.74964595,  -0.50827837,  0.449453443,  -1.12933397,  0.715758085,  0.710993409,  0.179489195,  0.78312844,  0.0415584967,  -1.1356014,  -1.05338526,  1.3852725,  0.969916165,  -1.18995345,  -0.24519074,  -1.04513001,  -0.330953151,  -0.68387115,  -1.47262681,  0.163598508,  0.0300072897,  -0.250133842,  0.509964645,  1.53414226,  -1.75596154,  1.67951524,  1.18471909,  0.707781017,  0.38054955,  -1.72497869,  -0.764949858,  -0.694163501,  0.193403646,  1.63079226,  0.266699702,  0.102106534,  0.361819178,  -0.965894282,  0.493513733,  -0.248055279,  0.757559359},{ 0.0283828862,  -2.46664453,  1.39590192,  0.523362994,  -0.26403892,  -1.99958706,  0.763051152,  -1.69301856,  -0.533798814,  -0.22845678,  -0.200142413,  -1.82444048,  -1.07061994,  0.60199213,  0.123287268,  -0.619845271,  0.500320256,  -0.630702734,  1.66513479,  -1.18088484,  -1.96606851,  -0.0609080717,  -1.4301722,  -1.33674884,  -1.56064367,  -0.0175879709,  0.13477315,  -0.3288095,  0.831641793,  -0.153215125,  -0.9620049,  -0.641523838,  0.0453745201,  -0.743993819,  1.89681935,  1.14387274,  -0.0702827573,  0.0500579029,  -1.75403619,  -0.0558539107,  -0.425330907,  -0.311739653,  -0.948869705,  0.625094175,  0.649359524,  -1.09399581,  -1.20357025,  -1.63876569,  -0.05108032,  0.683789253,  1.0115149,  0.759861827,  0.0354493931,  -3.31100726,  1.37898004,  0.716377854,  0.36655435,  1.42825437,  0.409865797,  -0.381741583,  1.48390436,  0.83738476,  0.253337383,  -0.862250805,  -0.128002107,  -0.658778787,  0.586660743,  -1.9975462,  -0.935313165,  1.20374787,  -0.178387865,  -1.03728509,  0.368462026,  0.922022402,  0.554696321,  0.627891898,  1.18018019,  1.42822993},{ -0.312374949,  0.281219721,  0.726440191,  1.45495462,  -2.59277534,  0.835717082,  0.971188068,  0.765665114,  -0.365731299,  -0.711608291,  -0.259381145,  0.999519467,  -0.555109799,  -0.184666172,  0.932455838,  -0.800722122,  -1.38115096,  1.66844392,  -1.12959087,  1.88793898,  -0.6144045,  -1.28350651,  -0.953923881,  2.10580444,  1.25838339,  0.976626158,  -0.728250623,  0.481600463,  0.249591306,  -2.10611606,  -0.769404531,  0.726381838,  -0.0851083472,  -0.996941984,  -0.120603576,  -1.69119322,  0.528726876,  0.274969071,  -0.413838089,  0.312214613,  -0.816991389,  -0.3624129,  -0.987644613,  -0.16282405,  0.0641029924,  -2.06356978,  0.831002891,  1.13625085,  0.498209447,  0.78697741,  2.19533038,  -2.64824796,  -2.7935183,  -0.697880745,  0.000766736979,  0.637313664,  -0.878828764,  -0.0766608566,  -0.306777239,  1.33357918,  -1.31953847,  0.583253801,  0.386376709,  0.373728573,  -0.743092179,  0.0265143886,  -0.538902164,  0.318799824,  0.0389528722,  -0.281752795,  0.497199595,  -0.153980732,  -0.72165072,  -1.15215516,  -0.555750251,  -0.326400787,  -0.857392728,  -0.183815971},{ -0.314587086,  -1.57530844,  0.945981562,  -1.40545022,  1.34139955,  -1.17791343,  -0.785799384,  -1.69989228,  1.32984281,  1.44709527,  1.06884122,  -0.610069454,  0.269275904,  -0.0132483132,  -0.849164665,  0.883311868,  -0.984896958,  -1.45097733,  1.12734425,  -2.32854176,  0.129896328,  1.75894439,  -0.252016515,  0.754282355,  -1.43477786,  1.03314745,  -0.0482205451,  0.919359326,  -0.658148527,  -1.52085114,  1.35314882,  1.65734148,  -1.03002298,  0.92399168,  1.00743139,  -0.533767939,  1.11538529,  -0.149361148,  -0.139740154,  -1.02847207,  0.603891194,  -0.0730845779,  -0.767064989,  -0.123138599,  0.527405083,  0.153831974,  0.98120755,  1.97914541,  0.355859816,  0.235528201,  -0.313362002,  -0.997761905,  -1.7583425,  -0.456700414,  -1.56701458,  -0.684530914,  -0.527798414,  -0.178272739,  0.904156268,  1.26790142,  -1.42139351,  -0.937569439,  0.264109761,  0.289813578,  0.851730406,  -0.598656714,  1.18558598,  -0.870996535,  0.469252408,  0.555469394,  2.75670075,  0.716485143,  -0.0916570053,  0.243580669,  2.8230114,  0.611152709,  2.5095253,  -0.181047857},{ -0.00773509126,  0.332729429,  2.10996509,  -0.231888041,  0.664757907,  -1.26184964,  1.24299061,  -0.499302238,  0.990768969,  1.19755542,  -0.729558051,  0.781849086,  1.17750299,  0.283284873,  1.14217675,  -0.748787522,  0.754730821,  0.283936948,  0.0659908801,  -0.34124741,  -1.83120227,  -1.56088579,  0.873765886,  1.88001859,  -0.0124242809,  0.696094275,  -1.23403871,  0.721495032,  0.422747314,  0.967863858,  0.0455607735,  1.73447192,  0.278350353,  2.33814549,  0.854191959,  0.000690399902,  -0.612841249,  0.875814021,  -0.423379391,  -0.722332537,  0.382329702,  0.110262237,  -1.48537123,  -2.24835491,  0.142429709,  0.969160557,  -0.15247108,  -0.499251693,  -2.07952666,  -0.221542031,  0.699486971,  0.383037895,  -0.236074582,  -1.58511889,  -0.110178478,  1.21340668,  -0.138300717,  -0.44482401,  -1.19690764,  0.732839525,  -0.912855744,  0.268323928,  0.413527817,  -0.8885957,  -0.445206285,  -0.379162222,  0.837766945,  1.87103033,  -1.78171778,  0.804686248,  0.867025912,  1.79062498,  0.735552669,  -0.730369925,  -1.32608879,  1.29278588,  1.01653492,  0.180616632},{ 0.997135878,  0.738147855,  0.258565426,  -0.738312006,  -1.92781115,  0.477096945,  1.08398068,  -1.94333899,  0.433740586,  -0.0479106084,  0.731072664,  -0.89262563,  0.478775859,  -0.0509620719,  0.453007162,  -0.00389646157,  -0.445879191,  0.454726636,  0.0682100356,  1.89465737,  0.534803867,  0.133722365,  -0.886902392,  0.718579233,  -0.969846308,  -0.464839578,  1.42249835,  0.133892775,  1.64677918,  1.26899743,  0.557590365,  -0.148977578,  -0.017006686,  1.94415367,  0.623150587,  0.654070675,  -0.0747663826,  0.34289667,  0.302937657,  0.658741951,  0.287845701,  1.33974302,  -0.869269073,  -1.24757802,  -0.186614752,  0.223744273,  1.55439615,  0.579726994,  -0.974450707,  -1.4640274,  -0.76629436,  -0.488938034,  -0.0389739424,  0.43568331,  -0.462424994,  0.111188605,  -0.0597620085,  0.841175199,  -0.397598267,  0.560778141,  2.05143309,  -0.848695338,  1.85505354,  -0.68044287,  0.132483229,  -1.09288418,  -0.940755129,  0.119280711,  -1.04302347,  1.54865527,  0.856403112,  -1.74584341,  1.07471359,  -0.218346298,  -0.326205015,  0.684107125,  0.629065335,  -1.04282641},{ 0.613606155,  0.552661419,  2.17922902,  0.638635516,  0.737428188,  -0.836471736,  -0.619331062,  2.61376953,  0.065720804,  0.96286273,  -1.78812349,  0.0702832043,  -0.963558793,  -0.259360194,  -0.501597464,  -0.0745925531,  0.144050509,  -0.0714366809,  0.14903903,  -1.37022531,  -2.02068782,  0.3801952,  0.540099263,  0.871297717,  -2.14560795,  -0.214394405,  -0.223457828,  0.123412564,  1.39826,  -2.12269449,  1.21946335,  1.37515068,  -0.418315649,  -0.745204806,  -1.69904387,  -0.324872792,  0.615140557,  0.704007983,  0.331784755,  0.419353843,  -0.138414294,  -1.0521121,  0.77919209,  0.058494065,  -0.341788203,  0.903807104,  0.761764765,  0.551579535,  -0.621948361,  0.444130361,  -0.224500135,  0.00349362986,  1.0933342,  1.6878196,  -1.3418082,  -1.32728088,  -0.858041048,  0.856247962,  1.40107036,  -1.24352002,  0.75607264,  1.39628732,  -0.392066717,  -1.3350513,  -1.84935582,  -0.644873798,  0.638222039,  0.258966833,  -0.0817736462,  0.0277687721,  0.880639374,  -1.43959856,  -0.295124263,  0.00509820739,  -1.07614911,  0.395580083,  0.0482649133,  -0.807426691},{ -0.639203072,  3.06145692,  -0.341002047,  -0.729625404,  0.717458606,  0.359159619,  1.93001938,  0.891887486,  0.843016505,  -0.951676369,  -0.749614,  0.461622864,  -0.0736337379,  0.305430561,  1.55331945,  1.20393991,  1.50574005,  -0.0453134589,  -0.7169168,  1.03963053,  -0.944042027,  -0.581883073,  -0.518327951,  -0.58936882,  -0.161053434,  0.755145073,  1.24174988,  0.0483353212,  -0.312952846,  -1.0011611,  1.17486882,  1.39081681,  -0.921743929,  1.68054116,  -0.344545424,  0.0354560688,  -0.270079523,  1.09016705,  -0.829433978,  0.0226748884,  -0.745424032,  0.968190134,  0.200084433,  0.566439152,  -0.0361791141,  1.26861787,  -0.0592636988,  1.5969981,  -0.532868564,  -0.194184929,  0.194983661,  -1.69335508,  -0.27909103,  1.02320766,  -0.98031342,  -0.266772866,  -0.775065005,  1.49120021,  0.205854028,  0.217146814,  1.24362957,  -0.833354592,  0.365992427,  1.14403963,  -0.780635059,  -0.0903176069,  0.147174925,  -0.131179705,  -0.953780532,  1.72767413,  0.905153632,  -0.213343635,  -0.0227319356,  0.601292074,  -0.352221817,  1.65402472,  1.67131877,  -2.14482379},{ 0.432965755,  -0.785785258,  0.60095489,  -0.476376712,  -0.496179819,  0.297313154,  -1.81326866,  -0.582192957,  -0.236386493,  0.500183642,  -0.886066318,  -0.597730517,  0.967636764,  -0.39879176,  0.176664889,  -0.245556518,  -1.15301669,  -0.402269661,  1.09712815,  -1.17948651,  -0.228739902,  0.519642234,  -1.485901,  0.0858289078,  2.24047542,  -2.26324654,  0.279543847,  0.725180924,  -0.0381861106,  1.18452895,  -1.20125508,  -0.774908662,  -0.273871928,  -1.65311813,  0.498484552,  0.512229085,  -0.465308666,  0.646216214,  -0.25060761,  -0.866355479,  0.0602963455,  -0.807547092,  -0.617570877,  1.00208783,  0.964792311,  0.455705792,  -1.33327162,  1.03674161,  0.160380796,  -0.513529122,  0.0635905787,  -1.00946224,  -0.595212996,  0.970459521,  -0.726502657,  1.26187801,  -1.76387358,  0.401572824,  -0.547637939,  -0.422701657,  0.971839786,  0.590859413,  -0.45246169,  0.330685079,  0.875021517,  1.35508597,  -0.872063041,  0.508137584,  -1.35653996,  -0.639707625,  -1.3235054,  -0.631852031,  -0.925326586,  0.0935234278,  1.35593259,  0.637312412,  1.41964471,  1.07912278},{ -0.26268509,  0.153931737,  -1.26581287,  0.497372121,  -1.49544179,  -1.15449679,  -2.32308245,  -2.25752258,  0.0886956975,  -0.400372386,  0.48603037,  -0.422610432,  2.03381801,  1.25729096,  0.174181059,  1.68082643,  -1.15039611,  0.370091021,  1.62812126,  1.1194576,  -1.26869881,  -1.57765865,  -2.00489902,  -0.826353967,  0.739280999,  -0.933474958,  -1.48811579,  -0.937740386,  -0.862164259,  -0.757542908,  -1.66314328,  -0.429606587,  -1.15103507,  -0.614765525,  0.695994079,  -1.02721477,  0.689633727,  -0.51757437,  -1.16366792,  0.73150897,  0.243675902,  1.18153429,  -1.59104776,  0.565140843,  -2.41724014,  2.06859922,  -1.71242607,  -0.248562619,  -0.256779313,  -1.45941305,  -1.30651402,  1.2378912,  0.334403753,  0.27243185,  0.929230094,  -1.1738981,  -1.59700716,  1.77062893,  -0.994280934,  1.21309745,  1.07408226,  1.06491506,  -0.852366328,  -0.362769634,  0.486736804,  1.44200945,  0.104505986,  -0.552437484,  -0.195384353,  0.0166051351,  -1.52777374,  -0.478185862,  0.71255213,  -0.999349773,  -0.414944679,  -0.210514739,  0.597960532,  0.233542502},{ 0.919158995,  -1.52618587,  -1.02306187,  -1.32318115,  -0.699824572,  0.244132504,  0.551740766,  1.09258735,  1.10294068,  -1.19418621,  1.11682022,  0.0755874291,  -2.33468747,  1.11758828,  0.535063803,  0.0312164035,  -2.05489063,  -0.233005896,  0.537346661,  -1.03216445,  1.88419211,  0.47007826,  -0.792936206,  -2.11400294,  -2.56159496,  -0.659576476,  1.02735734,  -0.188007951,  -0.610126674,  0.766986847,  -1.06746352,  -0.835257769,  1.18718493,  0.58146441,  1.36136734,  0.834564567,  -0.72428894,  -0.835503757,  -0.0833501592,  2.10282922,  -0.316837192,  -0.860627711,  -1.19308925,  0.217005432,  -0.875667632,  0.773278475,  0.498157442,  0.405804574,  -0.274030447,  -0.667931497,  0.464626521,  -0.837787867,  -0.626214921,  -0.318840325,  0.119681627,  -0.17493248,  -1.51224697,  1.48656058,  -0.791121244,  0.187179521,  0.843476653,  -0.799478292,  -1.16251516,  1.20694613,  0.947301567,  0.139026299,  1.6193943,  -1.21137393,  -1.25283182,  0.421986073,  -0.792766988,  2.12261271,  0.911942124,  0.891366065,  0.959087491,  0.0202853363,  0.867607713,  -0.680700243},{ -1.33759272,  -0.364102542,  2.02001977,  -0.64478451,  0.277430743,  0.890338242,  0.264026016,  0.517802358,  -0.470418364,  -0.0605826564,  0.944265127,  1.51200712,  -0.627090871,  1.08896565,  -1.01088417,  0.525635183,  -0.106294498,  0.071497038,  1.52953339,  -0.409918606,  0.136374429,  -0.409652025,  0.528054953,  -0.60500747,  0.589710951,  -0.922434866,  0.314383745,  -0.0764113516,  0.713448107,  0.290054083,  -0.85299468,  0.478581369,  -2.76960254,  1.40949702,  1.20781553,  0.45065251,  -0.412140638,  0.330392599,  0.492794275,  0.591133416,  -0.0717847496,  1.51742363,  0.204056174,  1.45801389,  -0.524049759,  -1.55790317,  -0.317208529,  -0.213004172,  0.0435215198,  -1.32046711,  -1.00186205,  0.187391266,  0.12314719,  0.762407541,  0.292852998,  -2.07170367,  0.927197099,  -0.146077454,  -0.275756478,  0.640684187,  -0.350702316,  0.768773317,  -1.78638792,  0.294948697,  -1.11288834,  0.21321629,  0.0672771484,  -0.549256742,  0.0230920408,  -0.232382074,  -1.35094547,  -1.19272935,  -1.58258748,  -0.233691514,  1.09999907,  0.308289409,  0.219985947,  1.04928637},{ 0.233022362,  0.787034214,  -0.21369946,  -0.213432983,  0.664286852,  -0.444706917,  -1.3675034,  -1.16618168,  -0.346646518,  0.79149121,  2.41298985,  -2.29001331,  0.623849869,  1.06138992,  -1.22081959,  -0.6464535,  -0.55766511,  0.481246531,  1.33167434,  0.800795496,  0.638939559,  -1.73104775,  -1.52150512,  1.01723635,  2.2180016,  -0.331476003,  0.415180862,  1.22523069,  0.153052956,  0.790259063,  -0.470965683,  1.12909913,  0.26723212,  -0.216140404,  -0.827927947,  -0.319246471,  0.712711394,  0.197914779,  -0.990270138,  0.623468459,  0.961827219,  -0.233401418,  -0.585406542,  -2.13020873,  -0.802273333,  -1.66480851,  -2.13475132,  -2.64041018,  0.123316377,  1.65860295,  -0.324036449,  1.52276039,  -0.0999024734,  0.204457119,  0.822306514,  1.01633716,  -0.631124079,  -0.276404589,  -0.015049764,  0.823659897,  -0.44176957,  -0.451612711,  0.433117479,  1.07353425,  1.7092042,  0.189081341,  0.794642329,  -1.3885659,  1.21790302,  -0.610710382,  0.459698349,  -0.00249917991,  0.517468154,  -0.362816244,  -1.28552604,  1.25934863,  -0.815365672,  -2.7466054},{ -0.487973124,  0.40832448,  0.653693855,  -1.38873279,  0.338720769,  0.634308219,  -1.38406932,  -0.933761656,  -0.514989018,  1.32544684,  1.26677608,  -0.177996367,  0.98744607,  -0.247696608,  -2.88822198,  -0.394648165,  1.07230341,  0.451509714,  -1.463377,  -1.53876019,  -0.855803668,  0.159815326,  0.024676118,  0.354895413,  1.87052381,  0.790971816,  -1.5586611,  -2.3174715,  0.374519587,  -0.113944881,  0.151812583,  0.465722054,  0.24099119,  0.39774102,  0.361126661,  0.93582201,  0.130182981,  -0.966328502,  -1.55664873,  0.632802248,  -1.65068889,  0.526270568,  0.109167896,  -0.35573414,  0.101665564,  2.34403515,  -1.84287214,  -0.752836108,  0.303876877,  -0.183324367,  0.0507807359,  -1.52073944,  0.0184587017,  -0.879519582,  -0.0538196899,  -0.712145329,  1.26491809,  0.305153906,  1.09177804,  -0.550104141,  0.341067493,  0.216306999,  0.125451401,  1.41417205,  1.78670859,  -1.33413839,  0.75296855,  -1.13892078,  -1.377455,  0.972212195,  -0.968386054,  -2.00527644,  -0.208732441,  0.715275288,  -0.436897188,  0.66258508,  -0.267386734,  0.581716061},{ -2.53884959,  0.290008456,  1.09146762,  0.13703014,  -0.142849475,  0.191513374,  0.779974818,  0.162414357,  0.720491171,  -0.611456156,  0.847435415,  0.278049767,  -0.132697225,  0.847667754,  0.271967918,  -1.15892112,  0.588433981,  2.49762845,  -0.0522150546,  -0.470273823,  0.0474189557,  -0.101235539,  -0.0172570795,  1.27566874,  0.302427202,  -0.765186846,  -0.482406348,  -1.41514087,  1.1171149,  0.32155937,  -0.350556701,  0.690897465,  0.262834758,  1.16797531,  -0.47672075,  2.00902247,  0.981025994,  0.455040693,  0.423472673,  2.88009548,  -1.76733029,  0.657874107,  -1.12947285,  -0.93455869,  0.329488665,  -1.38059092,  -0.299683839,  -0.155252829,  0.52952379,  0.123925507,  -1.25651026,  -0.557086289,  0.543631792,  0.120565236,  -0.112898156,  -0.201716796,  -0.352948368,  1.09647369,  0.422075152,  -2.09237766,  -0.873184502,  -1.04616559,  1.75267315,  -0.803623438,  -1.7865814,  0.999033809,  0.656045437,  1.47286391,  -1.03158212,  0.409736484,  -0.563165188,  -0.218261093,  -0.0904315785,  -0.531542957,  0.858233511,  0.894345403,  0.722143292,  -1.27840889},{ -0.820275307,  -0.345611781,  0.77581501,  0.929164708,  -0.406594604,  -0.319687188,  0.672645748,  0.735060394,  1.86262596,  0.480248481,  1.83132195,  -1.14403749,  1.45054114,  0.128897145,  -0.124458514,  0.0484815985,  -0.478641778,  -0.249341041,  1.73440301,  0.646391392,  1.51564741,  -0.37703687,  1.18920052,  0.213946149,  -1.7408818,  1.69600499,  -0.573511422,  -1.86740446,  1.2080611,  -0.392213881,  -1.62313735,  -2.09915328,  -0.152297527,  1.23499811,  0.377011925,  0.043774128,  0.468011916,  1.96403766,  0.219241098,  -0.00326052099,  0.022997912,  1.61012518,  1.25366807,  -0.547281981,  0.454474688,  -0.703050911,  -0.692640126,  -0.655016482,  -0.41547367,  0.677269161,  0.300518483,  -1.02530515,  1.11941135,  -2.39606404,  -0.831025362,  1.34780014,  -0.443184882,  0.553039134,  0.980554461,  -0.0495949052,  -2.01968431,  -0.311447442,  -0.289916575,  0.0250695311,  0.604476988,  0.216603473,  -1.94627416,  -1.33384824,  0.577222884,  0.19595547,  0.823101759,  -0.13868995,  1.06000531,  1.43212605,  -0.0323420614,  -0.860015154,  0.358177185,  0.67858988},{ 1.21058834,  0.323750526,  1.35226774,  -0.0563035794,  -0.498598874,  -1.55496442,  -1.95570493,  -0.162721351,  0.352194458,  1.03563845,  -1.93953514,  0.470413804,  -0.643647373,  2.34005332,  -0.00369993807,  -1.79372334,  0.21297361,  1.14498043,  -0.470805019,  -0.663038194,  -0.182070524,  -0.0440491661,  -0.0530267283,  -0.48941204,  -0.522095919,  1.13100863,  -1.91760921,  0.443449825,  0.36380735,  -0.324256301,  0.706447899,  -0.892763376,  0.321015596,  1.46292841,  0.214026228,  0.428436548,  0.905511081,  -0.706969559,  1.74267638,  0.653661311,  -0.392221451,  -0.711472511,  0.39868784,  -0.831228554,  0.320373118,  0.9225685,  -1.56035757,  1.41826308,  -1.65091407,  0.723967254,  1.24149752,  1.49461138,  -0.863157809,  0.0290747341,  -0.958853245,  -0.716168165,  -0.353820831,  1.2165556,  -1.17552876,  0.854565203,  -0.234808385,  -0.390561283,  0.447872549,  0.124512263,  -0.0881026164,  1.88800204,  0.0179232303,  1.081164,  0.982870519,  -0.010561429,  -0.0756649449,  -0.870908916,  -2.80521822,  -1.02867746,  -1.11990798,  -0.868457615,  1.57458282,  0.617222905},{ 0.00636201771,  -0.154357985,  1.36925495,  -0.681807637,  0.703115702,  -0.354111344,  -0.82882762,  -2.24980187,  1.67610323,  0.089981392,  1.29960048,  0.873552918,  0.346433103,  -0.656558335,  0.696897626,  1.17776549,  -0.775859356,  0.208425522,  -0.556471467,  1.21059167,  -0.382776082,  0.854267299,  0.820314467,  1.21995664,  2.04335785,  0.7644521,  0.357433021,  0.742778838,  0.25126192,  -0.316392601,  0.242195725,  -0.377978623,  0.316738188,  -1.01514232,  -0.911245286,  0.621017396,  0.538284779,  -0.930490494,  -1.01112688,  -0.335643917,  1.43000174,  0.0412523858,  -2.04252243,  -0.590456903,  -0.866120517,  0.246221155,  -1.71518707,  -0.833862066,  0.25746879,  0.250290215,  0.774683654,  1.21983695,  0.470135629,  -0.304265827,  -0.03918726,  -0.92206192,  -1.07363009,  -0.591888249,  1.74593043,  0.812395215,  0.838615119,  0.421778798,  -0.858895004,  1.08100379,  -0.126036301,  -0.855710328,  -0.347795606,  -0.724029601,  -0.749430239,  -1.1251018,  -0.110392988,  0.86483717,  -0.227802873,  0.209343508,  1.24111176,  -1.00965524,  0.930006266,  0.826844871},{ -0.782755971,  -1.64897418,  0.112113312,  -0.690755844,  -0.294357479,  0.0932285711,  -1.16147959,  -0.578387916,  0.0377620831,  0.414331526,  0.798620343,  0.795429885,  0.210695267,  0.29064846,  0.0215766374,  -0.320117503,  0.163003936,  1.55532253,  -0.440968066,  -1.03033125,  -0.465143383,  1.12654436,  -1.24782312,  -0.249625191,  0.767624497,  -1.05287039,  1.08430433,  -0.800223887,  0.286339521,  0.734765291,  -0.401384324,  -0.479156464,  -0.47511363,  0.357492924,  -0.374089479,  -2.75366735,  -0.584771156,  -0.435967833,  -1.11557937,  0.0984900743,  -0.146758229,  -0.288873374,  1.15194273,  -0.343835831,  -1.03786743,  1.45135403,  -0.292552829,  -1.24921811,  -1.21567941,  -0.537584186,  -0.0230062585,  -1.44384909,  0.999041378,  -0.818683684,  0.0476771928,  2.5158298,  0.617062867,  -1.04217505,  -0.201462001,  0.595712066,  -1.10025322,  1.05322778,  0.495895207,  0.769725084,  -0.2560651,  0.522774398,  -1.17293859,  -0.817193627,  0.381427705,  0.796022892,  -0.887693644,  -0.542213798,  -1.71809769,  -1.16538668,  -0.155393228,  -1.32912719,  0.672573626,  2.35153389},{ 0.894083023,  -0.673641801,  -0.350572914,  0.893549263,  0.719432712,  0.0499917753,  -0.893437922,  -0.296492398,  0.782802165,  0.0476489663,  2.08001947,  -1.08886135,  1.42362225,  -0.128724381,  -0.40820989,  0.834886491,  0.277925551,  -0.881719708,  -1.44330895,  1.5528276,  0.878490746,  1.28799665,  1.10345793,  0.59692663,  -1.79870534,  -2.55317259,  0.865887105,  -0.0539624542,  0.294392973,  -1.61403108,  0.959905982,  -2.17134953,  1.27212393,  0.80040592,  0.814020097,  -0.0975802094,  0.450133353,  0.379815161,  -0.628448546,  -0.739836514,  0.0690266043,  2.3310771,  0.321062297,  -1.2733717,  -0.110490046,  0.789814353,  0.0880188346,  -0.012266851,  1.05376315,  1.70158935,  0.983800292,  0.063397862,  -0.0314870924,  0.108432531,  -0.214460522,  -0.36931926,  0.789993763,  0.480411917,  0.318517864,  -1.17755675,  -0.0130361943,  0.336053699,  -0.620462656,  1.20891011,  -0.117196798,  -0.518274963,  -0.703775644,  -0.18356885,  0.608458638,  -1.9043088,  -0.410021842,  1.60967243,  0.376007438,  2.22088075,  -1.89294195,  0.678648591,  0.155467674,  1.46862066},{ 0.505009532,  0.675697446,  -0.449098855,  -1.20087481,  -1.71674955,  -0.408651143,  -2.06123447,  -0.142299384,  -0.199990422,  0.580604374,  -0.398235947,  -1.47049224,  -0.921522915,  -1.00282478,  1.34535384,  1.31973934,  0.322156966,  -0.885927379,  -0.92144835,  0.401904106,  -1.25851095,  1.52953923,  0.00327256857,  -1.1307019,  1.5055126,  -0.524362624,  -0.153251663,  -0.407978147,  0.146678731,  -1.04368699,  -0.0584774204,  0.427486748,  -0.507312596,  -1.10114062,  0.364168465,  1.15218329,  0.806618452,  0.238785565,  1.5601095,  0.381355822,  0.527985573,  -1.07433665,  -1.81982076,  0.0758323446,  1.36498713,  1.60925102,  -1.0192343,  -0.537629843,  0.637907326,  -1.19141841,  1.96245873,  0.204113692,  0.993759692,  0.637419403,  0.270194679,  -1.32645905,  -0.341246486,  0.162303761,  0.959539652,  0.700160682,  0.242680207,  -0.722047031,  2.14604282,  -1.00310814,  0.99096036,  -1.60714257,  -0.489129633,  0.201825321,  -0.609075427,  -1.14321959,  -0.257553756,  -0.263727903,  0.848254979,  1.09917748,  0.307315588,  0.88476187,  0.175215557,  1.21567714},{ -0.872439682,  0.00638755457,  0.469442755,  0.254153907,  -0.0396723859,  1.14218318,  0.398818374,  -0.167568788,  -0.416576713,  1.29205263,  -0.344350815,  -0.403240472,  0.264154196,  -2.38593674,  -0.0949470848,  1.55607116,  -0.0414344892,  1.22932911,  1.16111612,  1.17661583,  0.868538797,  -0.0964668766,  -0.0357888937,  -0.332138389,  1.90144598,  -1.19333947,  -1.22503126,  -1.52539539,  0.519295037,  0.672548294,  1.91183412,  -0.550405681,  -1.29312217,  -1.37051499,  0.35625428,  -0.430719405,  1.21352613,  -2.09326982,  -0.210912347,  -1.13796282,  1.21227622,  -0.377664417,  2.21705103,  2.0392468,  -1.37617731,  0.246316418,  1.21554613,  0.0927825943,  -0.208996341,  -0.169863045,  0.94193089,  0.606948793,  0.404035181,  1.01552331,  -0.724865854,  -0.443379879,  0.225300178,  0.133502439,  -0.235245988,  -0.471153945,  0.891119659,  1.7448554,  0.562318444,  -0.686103463,  0.964545488,  -1.7626251,  -0.733682871,  -0.553590894,  1.68656969,  0.734351456,  -0.780424058,  0.992689788,  -0.356134385,  3.35712695,  0.775075316,  -0.0750351548,  -1.13587737,  -0.273142695},{ 2.28239107,  -1.43253398,  -0.706827581,  -0.0145041673,  -0.769438207,  -1.34538651,  1.19505048,  1.34837091,  -0.335190266,  -0.846222341,  -1.33262241,  -0.0927386805,  -1.03545797,  -1.95783877,  -0.499837846,  -1.84066129,  -0.358361423,  0.689905226,  -0.981717885,  1.79399824,  0.125390679,  -0.848578155,  0.152119875,  0.806276202,  -0.997552693,  -0.612930477,  0.379522979,  -1.24287724,  -0.302766919,  -1.68053257,  -1.1707747,  0.795958579,  0.390636295,  -0.728676915,  2.5692327,  0.130768269,  -0.36824885,  0.80904007,  -0.111359395,  0.549872339,  -1.22335553,  -0.290137231,  -1.24741423,  0.903079927,  0.526165307,  0.52011919,  -0.401852131,  1.95943785,  0.814155519,  1.62945592,  -0.874408901,  0.638017058,  -2.30096197,  0.0533293672,  -0.470707357,  1.61588502,  -0.588064253,  -0.381891698,  -1.26524556,  -0.127689302,  -0.158252954,  0.403263867,  -0.0191531014,  -1.39296389,  -0.17437245,  -1.26790011,  0.539088547,  -0.351786107,  -0.147487491,  0.936832488,  -0.576315284,  -0.692140341,  -0.063306585,  0.22806707,  1.02242804,  0.0171455014,  0.726172507,  1.37773693},{ 0.0813109875,  1.57720172,  -0.621264637,  -0.948407769,  -0.17496042,  0.900011897,  1.12593341,  2.21790671,  -0.250976205,  1.68413877,  -0.46422866,  1.44354248,  0.523741305,  0.244167313,  0.0706064999,  -1.71981192,  0.628674448,  -1.90847993,  1.19230628,  -0.729800105,  -0.720852375,  0.64483285,  -0.199570775,  1.91595674,  1.69085455,  -2.95787096,  0.00778016774,  1.16813803,  0.686154246,  -1.55347276,  0.470358461,  -0.565124094,  -0.984030604,  -0.248378024,  3.21400952,  -0.5927037,  1.42442477,  0.147310883,  1.6307652,  -0.929447174,  0.624244273,  -0.468624949,  1.70721567,  -0.45079726,  -1.06684554,  -0.678047299,  0.564630866,  -0.70131892,  0.00297166198,  -1.13009536,  -2.88981414,  1.615381,  1.64423311,  0.830620229,  1.64862108,  -0.0526465327,  0.748685837,  -0.402729183,  -0.672232032,  1.72134006,  -1.01886702,  -0.148609936,  -1.33389831,  0.0414552428,  0.838238418,  -0.0126511613,  0.531059921,  -0.134976134,  0.675619423,  -0.54034394,  0.429034919,  -0.516931653,  0.455649525,  -0.0715949312,  0.183294654,  -0.747507215,  0.794534385,  0.966074049},{ -0.189269975,  -1.65901411,  0.178028896,  -0.48530677,  0.259151548,  -1.74979639,  -1.18131256,  -0.626295149,  0.266440392,  1.27279484,  -0.606981874,  -0.042642381,  1.22215378,  -1.69048643,  -0.378802985,  1.36334956,  0.0549906604,  0.0861812532,  0.826022029,  -0.414214909,  -0.746685386,  -0.702016056,  -0.0716663226,  -0.868905187,  0.914418101,  0.416976422,  -0.716352582,  0.142409399,  -1.889341,  1.602934,  0.500049353,  -1.4863112,  0.55115062,  0.0211389922,  -0.0939902738,  -0.506095648,  0.00548935775,  0.981092155,  -0.048436068,  0.155672088,  1.50894821,  -1.03233278,  -0.229891524,  0.806193709,  3.16289544,  -0.945739508,  -1.04692447,  -1.71616483,  -1.52026236,  0.472940475,  0.825675666,  -0.766843617,  -0.469925791,  0.512186527,  -0.170969039,  -0.0813273564,  0.572068453,  -0.142534375,  -0.589407802,  -0.394221395,  1.61860323,  0.635126531,  0.627135754,  0.345584571,  0.253328592,  0.33325243,  -0.279083371,  0.552607894,  -0.868276715,  -0.0387090035,  0.444249958,  -0.127850071,  0.021944182,  -0.692069769,  0.759374142,  0.0955248624,  -0.240819007,  -0.12531504},{ -1.34949422,  -0.211832106,  -1.69314802,  0.49822855,  -0.839114726,  1.63512135,  1.06658173,  -0.289231032,  -0.666184604,  0.917238414,  0.40387091,  -0.2885243,  1.67041802,  -0.290251315,  -0.861339808,  -0.398350418,  -1.51529992,  -0.77081418,  1.43149221,  -0.230883569,  2.23101091,  -1.44185627,  0.458190769,  -0.60251838,  -0.392122567,  -0.362820119,  0.793102562,  -0.359952241,  1.84456897,  0.197877049,  -0.392803937,  0.188505203,  0.466042638,  1.9444108,  0.361824572,  -0.42617625,  0.121963538,  -1.54784977,  0.0980562195,  1.03570187,  0.649467111,  -1.33686686,  -1.93943346,  0.481581837,  0.208982453,  -0.136777148,  0.214743778,  -1.5513047,  -1.98390555,  1.61435866,  -0.649475574,  1.03061819,  -0.174014941,  -0.307463497,  -0.0468349755,  1.20285475,  0.0914930031,  -0.717142403,  -0.727569759,  -0.78542912,  0.0262232348,  1.03928995,  0.104043968,  2.15635419,  0.0721857175,  0.498071223,  -0.132938743,  -0.898379445,  0.569915354,  0.61423409,  1.46807849,  -0.904777825,  0.635608852,  -0.35367927,  -1.72977829,  -0.921210647,  -0.887496054,  -0.35515818},{ -1.15325618,  -0.195909157,  0.0486991219,  -0.889377713,  -0.472406745,  0.0116604231,  -0.625145853,  -1.56820774,  -0.720383108,  1.53747284,  -0.159012258,  -0.164070666,  1.1677134,  0.627226472,  1.5226475,  1.48460376,  0.462082595,  -1.06007338,  1.28618276,  0.66376555,  -2.91815519,  -0.389240474,  0.764143288,  0.882238865,  -2.04148841,  -2.15564561,  1.20834541,  -1.56869841,  -1.96339953,  -0.462630838,  -1.37483776,  -0.529376924,  1.81266415,  0.722433269,  -0.798618019,  2.06598639,  -0.0365589522,  -0.592717707,  -1.23637021,  -0.682772279,  0.90500164,  -0.411508411,  1.30089116,  -0.410845011,  0.0757081658,  -0.981541693,  -3.28669882,  0.065880999,  -0.461557537,  -0.474392354,  0.380741447,  1.31666076,  -0.0804171264,  1.29032183,  0.292769074,  1.03836679,  -1.11988103,  0.207543254,  0.917116761,  -1.29479825,  0.459159404,  0.862593293,  1.10965896,  -0.0979790092,  -1.39436638,  -0.566837311,  -0.320685506,  0.663621128,  1.24699306,  -0.485923737,  -0.390721917,  2.4237318,  0.189022914,  -0.192706212,  0.227396935,  -0.569599032,  -0.0593784712,  -0.945866466},{ -0.651874125,  1.03940845,  -0.0550653189,  0.334831297,  -1.35720909,  -1.58252132,  -0.095495455,  0.458347201,  -0.635181129,  -0.38040024,  -0.800773799,  0.51785332,  -1.03218436,  -0.708868325,  1.00646186,  0.400898993,  -1.25097764,  -0.324377686,  1.40943265,  -0.198890969,  -1.27635622,  0.30819723,  -0.0307089221,  -0.165611088,  -0.0461384803,  0.0324607566,  0.256889403,  0.123861566,  -0.276200116,  -0.152989909,  -0.472229004,  0.0300024543,  0.623362958,  0.355266213,  -0.556391656,  -1.19779015,  0.020309357,  -2.75775957,  0.0524106398,  1.0426122,  -1.03215623,  0.00175181008,  -0.60233146,  1.58422863,  1.44572949,  -1.5060792,  0.0500270762,  0.754185855,  0.720784366,  1.49336517,  1.09022343,  0.50502336,  -0.0216377284,  2.53432631,  0.194503963,  0.0477835014,  0.488983691,  -1.33119011,  0.658130467,  0.399625033,  -1.11490285,  -1.31149161,  -0.208463013,  -0.162757888,  2.4940145,  -0.919412792,  1.27462018,  -0.488459647,  0.282842577,  -0.243188441,  -1.17254484,  -1.73254013,  -0.311455339,  -0.412150532,  -0.367957652,  0.0884749666,  -0.456520408,  0.219725341},{ -0.537225962,  -0.29136461,  1.05370557,  -1.7947855,  -0.713335812,  -0.55745101,  0.34011814,  0.769595444,  -1.17815697,  -0.889642358,  -1.49183095,  -0.0522353798,  1.31370437,  -1.14832294,  0.687549591,  1.6370976,  0.704190552,  1.45731556,  1.04019034,  0.429410636,  0.161438033,  0.315032363,  -1.16574669,  -0.494105935,  0.731149077,  -0.683599532,  -1.04902565,  1.36970341,  -1.24479222,  -0.124813966,  -1.33307254,  -0.831472039,  -0.410392761,  -0.436778188,  -0.149780676,  0.0628173351,  -0.497856259,  1.43036234,  -0.441885799,  -0.461250901,  -1.32331765,  -0.10454762,  0.013372031,  -0.812197745,  1.4503895,  -0.716048956,  -0.632669628,  0.867430449,  1.5559572,  1.51645923,  1.65282118,  -0.60380435,  3.1528945,  0.0437273681,  0.0242540762,  1.52493477,  1.19692779,  -1.35659122,  0.78658253,  -0.237645239,  -0.288291395,  0.810746551,  0.0127923572,  -0.612153471,  0.352892876,  0.47326076,  1.54713547,  -1.49347043,  -0.1678271,  -0.714157343,  -0.983524203,  0.176748097,  0.121544555,  0.599447787,  -0.802182317,  -0.483504951,  0.124677412,  0.798984051},{ 0.848344862,  -0.628628373,  0.906153977,  0.517844796,  -1.3526479,  -0.180578217,  0.397802234,  1.19878197,  0.0763865262,  0.241223276,  1.31619763,  0.435910195,  -0.340020895,  0.523415208,  -1.09130585,  1.43065321,  -0.379408866,  0.0982441828,  -0.21702747,  -1.21752441,  1.97242212,  -0.442881703,  0.191336945,  0.874293327,  -0.414207041,  0.114502765,  -1.93919742,  -0.639342308,  -0.693614542,  2.42528105,  1.35835063,  0.18617107,  -0.656699717,  -0.777957499,  1.31285608,  1.66659617,  -0.899745107,  0.176504627,  -0.751707733,  2.15700841,  0.529480875,  -0.141866013,  1.13295662,  -0.221365377,  0.414865673,  0.14678897,  -0.924196661,  0.784999669,  -0.706757486,  0.262408376,  1.22235096,  0.191907853,  0.550436974,  1.07471669,  1.66249692,  2.06785488,  -0.599803686,  0.0468110852,  1.21792495,  0.407314271,  1.36272383,  -0.295154423,  0.170801699,  1.48726177,  -0.699477851,  -1.61689973,  -1.25622261,  -2.06504178,  1.8932215,  -1.23988926,  -1.70629406,  0.891471267,  1.65800524,  0.427295506,  -1.32822192,  0.749488354,  0.706599534,  0.361208379},{ 0.312073946,  0.975956738,  0.350076854,  1.07159841,  0.798203707,  0.706474066,  0.220186785,  -0.222583055,  2.19928408,  -1.21742976,  1.35270274,  0.939644575,  1.00499296,  -0.152795807,  -0.274087906,  -0.420154929,  0.411100626,  -2.73334241,  -0.291672528,  0.0999365598,  -0.524610221,  0.692598879,  0.7102983,  -2.40871215,  0.558648944,  -0.56947732,  -0.812327683,  0.0414099246,  0.184503287,  -0.837354064,  0.368031114,  -0.684546947,  -0.0289169569,  -1.31571341,  -2.25508308,  0.898338497,  1.26468408,  -1.95528722,  1.08840919,  0.59480232,  -1.6787008,  1.01957262,  0.646018922,  1.77602077,  -0.408589959,  -0.683737457,  0.564446092,  0.657119155,  -0.491796434,  -1.11587512,  -0.955743432,  0.0577943884,  0.109002665,  -1.40307677,  0.0806273445,  -0.0258832239,  0.628272593,  0.363879919,  0.19653523,  0.518348336,  2.6283555,  -0.963288009,  -0.201805547,  -1.75942528,  0.417564601,  0.485310733,  -0.523826241,  -0.269345224,  -1.17310953,  1.20564592,  -0.692317188,  -2.44311237,  -0.0687605962,  1.0616796,  0.524322391,  0.543002725,  -0.688780665,  -0.473455966},{ -0.343089163,  0.660361767,  -0.153523386,  0.533638537,  -0.530734479,  -0.0122584375,  1.45793164,  -0.502246201,  -0.155107722,  1.10493505,  0.51763618,  0.145517409,  -1.76329994,  0.112840459,  0.424223304,  -1.14889681,  -0.192064106,  1.40795898,  -1.46673763,  -1.73341143,  -0.522479475,  0.672130466,  0.845580816,  1.08236754,  0.0941398367,  1.23421288,  -0.960141003,  -1.26042891,  -1.28564596,  0.052632302,  -0.9229967,  1.46893787,  -0.292558849,  -0.883645177,  -0.362927973,  -1.89716661,  0.557363868,  -0.360708773,  -1.20144749,  -0.433283061,  -0.129650131,  0.465490609,  0.63022542,  0.637603939,  -1.05194044,  0.830995023,  0.0386948809,  -1.71621728,  0.0476511568,  1.24124324,  0.0187323764,  -0.391185254,  -0.0144115211,  0.485374331,  -0.130738631,  -1.75869846,  1.03969693,  1.80000579,  0.365181416,  -0.290934741,  0.450033605,  0.475217402,  0.20578973,  -1.7539767,  0.0991369784,  1.00513136,  -0.0376651883,  -0.913354576,  -0.220111176,  -0.685245037,  -0.995319307,  1.18854845,  -2.08348203,  0.885934114,  0.942391396,  0.369125098,  0.0157365408,  -0.600310922},{ -0.115613639,  0.595893979,  -0.984169722,  0.599494338,  0.805044174,  2.11230659,  -0.272086233,  1.05335951,  0.0741119906,  -0.980659604,  0.443125844,  -1.21283782,  -1.96550977,  0.0186305009,  -0.981414855,  1.29754376,  -1.57042873,  0.696820915,  0.463685602,  -1.71292365,  0.270544678,  -0.319776893,  -0.454807878,  2.27868581,  -0.31794098,  -0.473392427,  -0.125696883,  0.0234325286,  0.207839102,  0.618690848,  -0.318246037,  -0.596210837,  0.895389915,  -0.615474284,  -0.206154346,  -1.47689569,  0.727173448,  -0.589950562,  1.58067977,  -0.118143126,  -1.35033154,  1.70460391,  -0.301468015,  -0.691007316,  -0.234373644,  0.513651073,  0.103839532,  -0.021278901,  -0.716678441,  -0.356856316,  0.208160549,  -0.358053446,  0.805638134,  1.81864822,  -1.46882975,  -1.06673932,  0.121374033,  0.323921442,  0.385843724,  0.107979968,  -0.161578864,  1.75462925,  -1.12718225,  0.760256767,  0.537713408,  0.255787909,  0.322782159,  0.71023047,  -1.32157552,  1.79510272,  0.155522808,  -0.247799754,  -0.813730121,  -0.803857565,  0.393561155,  1.54473698,  -0.961741388,  0.0300093293},{ 2.69145036,  0.27828145,  -1.10712743,  0.454769224,  1.04748476,  -0.656137228,  -1.45632541,  -0.344659328,  0.129773453,  -0.194038704,  -1.65259635,  -0.643637419,  -2.50442433,  -0.745342493,  0.186743364,  -1.33812821,  1.29751694,  0.296198815,  -0.192585036,  -0.328023344,  0.465863138,  -0.774303198,  0.387049556,  0.882351637,  0.746004999,  -1.46727538,  1.55428231,  -0.171653539,  0.945907056,  -0.687761426,  1.26763403,  -0.513850927,  1.05914354,  -0.398524374,  -0.71770376,  0.111475281,  2.33625364,  -0.801662266,  0.0688628852,  1.26402986,  -0.525909424,  0.897587717,  -0.0124386037,  0.058013428,  -1.18152809,  -1.76097596,  1.7878859,  1.77083445,  1.3987236,  0.694520235,  1.39508557,  0.0842614546,  0.622422457,  0.978182614,  -0.529037833,  1.18967104,  1.01248384,  0.305818558,  0.782595992,  0.525530875,  1.51271856,  -0.543593526,  0.539310396,  -0.622273386,  0.146826342,  0.872645736,  -0.222264901,  -0.228245601,  -0.128697857,  -0.565688014,  -1.05182338,  -0.538154185,  0.585740507,  -0.282328397,  -1.77508116,  -0.857385278,  -1.75067449,  -0.270588696},{ -0.961362898,  -0.540474415,  -1.78268898,  -2.17312121,  0.282472879,  0.940340579,  2.00295281,  -1.45776689,  0.193957284,  -0.866352379,  -0.551772714,  -0.599639058,  -0.825886726,  0.856522381,  -0.954322338,  1.07365048,  -0.501955271,  0.144879818,  -1.06711745,  0.532428741,  -0.542090058,  -0.627263546,  1.06430554,  -0.575632334,  0.0310655385,  0.158278599,  0.0192853622,  1.91427588,  -1.73661947,  1.24966109,  0.871545255,  0.154738337,  -0.600958586,  0.4289864,  -1.95797348,  -1.18894935,  -0.307184249,  -0.423830956,  0.499066472,  0.849522591,  0.941467226,  -0.95037514,  0.275952786,  -1.47435939,  -0.598589838,  1.6770885,  -0.0758745372,  0.979358673,  2.0192101,  -0.82043916,  -1.16056287,  2.62650871,  0.0703596249,  -0.350306153,  1.43041861,  -0.114163421,  0.485027641,  -1.12520802,  0.438460618,  -0.928313553,  0.555952191,  -0.366309553,  0.549615443,  -0.693342745,  -3.01934171,  -0.282301635,  0.0709229335,  -0.223509625,  -0.170119211,  -0.0195819121,  -1.70165527,  0.512719691,  1.63093293,  0.669293165,  0.0114244735,  -0.532793283,  0.245295748,  1.10093749},{ -0.12540336,  0.171369925,  0.66587013,  0.269865215,  0.0395492353,  0.162021473,  -0.718903899,  -0.468853414,  -0.151117429,  -0.45343703,  1.31806076,  -0.0327063836,  -0.310899943,  0.194816962,  -1.34131455,  2.24748659,  -0.76363647,  -1.05221558,  0.100384168,  1.474123,  -2.01494694,  0.680936635,  -0.18715781,  2.47322035,  -1.08882332,  -0.773487866,  -0.350792587,  1.76475036,  -0.54946357,  1.2291925,  -0.14504455,  0.329586476,  -1.53550649,  -0.699982822,  -0.505533516,  -1.21516252,  0.139342085,  -0.190956444,  -1.21604252,  -1.66919696,  0.0766101927,  -0.529747546,  0.716522872,  -0.760100067,  -0.935541451,  0.912255466,  -0.823952377,  0.806081116,  0.0454077944,  0.993982673,  -0.217755869,  -0.652640283,  -1.56097817,  1.21221745,  0.188421458,  0.930218697,  0.834555268,  0.184993207,  0.150554597,  0.816691935,  1.381585,  0.753744602,  0.588561356,  -0.779589474,  -0.912225664,  0.217458665,  -1.38569665,  0.326794267,  -1.17811096,  0.248683244,  -1.27739882,  0.290089101,  1.4104557,  0.245603517,  -1.21678579,  -0.125614926,  1.62103367,  0.32465294},{ -0.523670554,  -1.07871521,  0.185698479,  1.04294384,  0.17304872,  0.171906039,  -2.34745002,  -0.951204658,  2.11117411,  -0.153797671,  -0.829008102,  -0.386373907,  2.55469179,  -1.74294138,  -0.706597626,  2.19841862,  0.406244248,  0.0241879486,  -1.83733308,  -0.991058528,  -0.571273923,  0.277325004,  -0.291000783,  0.0947604626,  0.857116461,  -0.551243901,  -0.126305923,  0.29524973,  0.087172322,  -1.80320382,  0.369616956,  0.605647504,  1.50628793,  -0.982284904,  -0.635394633,  -0.271804273,  0.787299752,  -1.13611948,  0.168071598,  0.487599611,  -0.0205316637,  -0.503884494,  2.13166523,  0.689531803,  0.521309018,  1.4082936,  0.117275387,  -0.723740578,  0.42847845,  -1.0384568,  0.304621279,  1.26531267,  -1.23962831,  0.286623597,  -0.529512763,  0.230908513,  0.0591210313,  -0.658076286,  -1.02183497,  -0.105057009,  -0.892524838,  0.112635784,  -1.79406309,  -0.246752784,  0.79011929,  -0.12266437,  -1.1726408,  0.537413895,  -0.00408787606,  1.17839909,  -0.466962844,  0.0793854892,  -0.637850285,  0.812219083,  -1.59787893,  -0.938137293,  2.00030971,  1.32908094},{ 0.682434201,  -0.468131214,  0.295320898,  -0.948553145,  -0.658571541,  -1.52200937,  0.0897008106,  -0.762581348,  0.162981287,  0.27346608,  0.811953485,  -1.55960298,  0.826268673,  1.18928766,  -0.516138673,  1.31051397,  0.379229486,  0.219888091,  -0.0540188923,  0.165243372,  2.23114634,  0.563898563,  1.24615622,  0.292203575,  1.13180494,  -0.228246346,  -0.438454002,  0.558756292,  0.243505001,  -0.8556633,  -0.757050872,  -0.253290266,  -1.84556246,  0.247708723,  0.909994543,  -1.07141399,  -0.78780365,  -1.58633316,  -0.0732730627,  2.30815291,  0.681724966,  -0.122114561,  0.328656256,  -0.958528042,  -0.873290181,  -1.19020486,  -0.207460508,  -0.645742357,  0.882891059,  0.282536536,  -0.566548526,  0.749228656,  1.75858915,  2.44668984,  -0.101255298,  0.239669368,  0.149961531,  0.00793569069,  0.709792554,  -0.9851197,  -2.60508871,  0.461785614,  -1.1420238,  -1.2987833,  -0.429938853,  0.364232361,  -1.28772521,  0.769910693,  -0.310669094,  -0.19252786,  -0.230761185,  -0.0603311919,  0.360579729,  0.300665081,  -1.39255667,  0.240696892,  -1.6334995,  -0.992613733},{ 1.03369701,  0.771562815,  1.1973511,  0.479390085,  0.452534437,  0.774632335,  1.256078,  0.17377989,  -0.0769312754,  -0.519218028,  0.792936504,  0.394767463,  -1.35361505,  0.220153749,  1.23359001,  0.97273469,  -0.522478819,  -0.00738280965,  -0.394037545,  1.41695654,  -0.511415482,  -0.532417655,  -2.78944397,  1.61735415,  1.19090343,  1.07620919,  1.68378067,  -0.854556203,  0.454279631,  0.421390533,  -0.272828072,  0.0668546632,  0.26668188,  -1.2145673,  1.04463482,  0.0481427126,  0.472602516,  -0.988265514,  -1.22083986,  -1.03650045,  -0.109408408,  -0.732061028,  -1.53110182,  0.117813952,  -0.949334681,  0.291385889,  -0.936646163,  -0.497056574,  -0.163250446,  -0.563358665,  0.635071874,  -0.291165471,  -0.199337795,  2.28897309,  1.54929316,  1.87768126,  0.84481287,  0.112670608,  -0.316370517,  -0.329973072,  1.14810967,  -0.668272614,  1.23164701,  -0.219603345,  -0.264650673,  1.2648021,  -0.43106398,  -1.3205961,  1.69570243,  0.416710824,  -0.218836337,  0.77325511,  -0.208756343,  -0.535301149,  -1.80266118,  -1.4729898,  0.761323631,  -1.32614088},{ 2.43107462,  -0.0360243544,  0.35235244,  -0.0340332091,  -0.896761179,  -0.0469848849,  -1.54179132,  -0.119652092,  0.19766432,  1.7549547,  -2.12236285,  1.14401603,  1.17218685,  0.822227955,  -0.0838751942,  0.302286357,  0.18918404,  1.09267426,  -0.937895417,  0.243624881,  0.433233857,  1.14819884,  0.19700554,  0.0949902311,  -0.541261733,  -0.0559371375,  1.31939232,  0.209760606,  0.336178929,  0.475656927,  1.64064395,  0.955470264,  -0.505299866,  0.399294436,  -1.89949644,  -0.301544636,  0.568318427,  -0.168969125,  1.88274121,  -1.0187639,  1.30656564,  0.352482051,  -1.04057956,  -2.13850498,  0.844617724,  -0.835068405,  0.0638675392,  0.304385841,  -1.14294696,  -0.143309101,  -0.830845535,  1.20283031,  0.845945597,  1.90710127,  1.21953714,  -0.0455657952,  -0.336800396,  -0.34972316,  0.557040632,  0.16244632,  -1.33549774,  -0.998468816,  1.39466,  0.650608003,  0.633517563,  0.475077122,  -0.132388756,  0.683764577,  -0.502827108,  -0.244199514,  0.647676408,  -0.808242381,  -1.0680455,  -0.471988469,  0.501989663,  -0.56226027,  0.587229669,  -2.16464376},{ 0.589188397,  0.3470954,  -0.324576229,  -0.213923544,  -0.950871587,  1.76216185,  0.0854868144,  -0.539516926,  0.94380337,  0.981451154,  1.08459496,  -0.0940926149,  0.0876739696,  -1.6510756,  -0.0349319056,  0.462717593,  -0.761868298,  -0.383406013,  -1.28016496,  -0.0711918548,  -0.679392576,  0.369922817,  0.0115265306,  0.277452946,  1.28044999,  -0.978367984,  0.234162703,  0.423957884,  1.72347164,  0.200022534,  0.619531512,  0.53110671,  -0.924594402,  -1.88790774,  -0.204551741,  0.327746809,  -0.0891558751,  1.37327027,  1.02451694,  -0.943666697,  -2.45239425,  -1.27312624,  -0.70465064,  0.218963549,  0.846491218,  1.15184498,  -0.110468172,  -0.962105334,  -0.712814212,  -2.11166477,  -0.425137877,  1.11190093,  -0.510758817,  0.431953281,  -0.363866329,  -0.691265821,  0.973766983,  -0.307323933,  -0.197695717,  -0.0763057545,  1.24840641,  0.752819359,  0.82157588,  0.220045596,  0.111872517,  0.687389374,  -1.89352453,  -0.64208746,  0.0172535777,  -1.90109956,  -0.576897204,  -0.0535496362,  1.11072135,  2.25453854,  -1.1678201,  -1.24663663,  1.05363774,  0.576946437},{ 0.25546211,  0.170359105,  -0.956427097,  1.41997623,  -0.217281386,  1.13829517,  1.84216154,  -0.379331887,  -0.588831723,  -0.947877884,  -1.81898403,  -0.981248379,  -0.814946234,  -1.89486039,  -0.703020036,  0.615837574,  -0.3389844,  1.99167669,  -0.0315803662,  0.345835268,  1.49376893,  0.844581246,  -1.31877553,  -0.720401168,  -0.0652067959,  -0.639699161,  -1.61232507,  -0.0630989745,  2.05681062,  1.03652346,  -0.0809737593,  -1.07328534,  0.912441552,  0.345698029,  -0.913046181,  -0.635134518,  -0.119046442,  1.14947689,  -1.21120381,  1.03240788,  1.73869503,  -0.937801957,  1.65317631,  -0.362233758,  1.03181517,  0.767639518,  -1.75030148,  -2.08537292,  0.100973472,  1.65714133,  -0.973887861,  -0.932151198,  -0.418463409,  1.255404,  -0.690651476,  1.11575973,  -0.320927173,  0.2995722,  -0.895353019,  -0.178835332,  0.253866225,  0.370503455,  2.79873371,  -0.393248022,  0.635611594,  -0.836516201,  0.280637443,  0.355249822,  0.0855774432,  -0.804961324,  0.56709975,  0.848303616,  1.49590051,  -1.06792629,  -0.856592059,  0.503334224,  -0.423071504,  -1.55771196},{ -0.773311257,  0.0116022984,  0.707918942,  -0.524874747,  -0.114086986,  0.322330892,  -0.0442553014,  0.526860356,  1.35786438,  -1.21679139,  0.00245255814,  -0.0953954086,  1.2571497,  -0.182825327,  -1.15428615,  -0.708687067,  0.23932445,  0.00158922619,  0.763223648,  0.687843382,  0.416924804,  1.75836599,  0.357183784,  -0.279035956,  2.10024619,  -0.80197823,  0.167368084,  1.90099883,  -1.31950295,  0.577414334,  -1.19743991,  -1.45170605,  -0.535036564,  0.0570385642,  0.372110993,  1.58824003,  -0.919134855,  0.630303442,  -2.190624,  -0.176286042,  -0.823027313,  -0.478460968,  0.357260138,  -0.566069841,  1.86869204,  -0.306743681,  0.611332178,  -0.534660697,  0.754043162,  -0.523002565,  -0.0600724258,  -1.35090041,  0.718509972,  -0.735519946,  -0.47025755,  0.224900231,  0.702913463,  1.40472221,  -0.0438813418,  -0.494748265,  -0.0509152152,  -0.773699522,  -0.12002442,  -1.2814877,  -0.678703308,  -1.07941639,  1.52143383,  0.0313923918,  0.745212138,  1.44184029,  1.62088752,  -0.339315712,  -0.68244642,  1.97140169,  0.525384545,  0.319512218,  -0.502431512,  -0.472706676},{ -0.615978122,  0.269610643,  -1.56998968,  0.343584329,  -0.313742012,  1.79133642,  0.964723647,  -0.440145671,  -0.635091364,  -0.871976852,  0.072112374,  0.269916624,  -0.0571452193,  -0.344160497,  0.53120929,  -0.0668598562,  0.153318778,  1.34317446,  0.845526159,  0.588867903,  -1.24390638,  -0.778680742,  -0.837612033,  -0.438426554,  0.663388968,  0.0465678088,  0.943545103,  1.97391963,  -0.641425729,  1.08610582,  0.469590366,  0.482449472,  0.382597268,  -1.42755592,  -1.14186299,  0.157692686,  0.800940096,  -1.19276404,  0.0432276428,  -0.109152839,  0.0187253952,  -0.168085679,  0.310674906,  0.612125218,  -2.30652857,  -1.15695786,  -0.156163082,  2.63674188,  0.818596184,  0.606996655,  0.774211645,  1.35555339,  2.25074816,  -0.161487103,  -0.499830306,  -1.70135558,  0.624118865,  0.149724007,  -0.772329092,  -0.176884755,  -1.06234241,  2.0785346,  1.2221837,  1.89739704,  -1.95794523,  0.0513108969,  0.00554908998,  2.01807213,  -0.0275972616,  1.3825351,  -1.50934625,  0.151024744,  -0.535483301,  -0.952575207,  1.5504421,  -0.671005666,  -0.974535644,  -1.7013365},{ 0.218962535,  -0.488299787,  -0.874751031,  0.189689115,  0.644046247,  0.512486339,  -0.785619199,  -1.63150132,  -0.820008755,  -0.225011602,  0.511345983,  -0.261515915,  -0.416150302,  -2.04959083,  -0.536863923,  -1.2081393,  0.806020916,  0.56835109,  -0.758053184,  0.357130706,  0.34585923,  1.98404408,  0.816250682,  0.620570719,  0.567483425,  2.18091345,  1.96545649,  0.603640556,  2.06728554,  0.479379982,  -0.70477283,  -2.06908631,  0.667747021,  -1.4407835,  0.95911032,  -0.558159113,  -1.52574468,  2.20473647,  0.134535968,  -0.962971807,  1.804178,  -0.0621107742,  0.890404582,  2.60180354,  0.556239545,  1.08159697,  1.27917159,  0.278348833,  -0.764937162,  -1.06477022,  -0.875643849,  -1.05853224,  -0.697177052,  0.422330379,  -0.53711164,  0.579988062,  0.118303403,  0.792900324,  0.504213035,  1.09944177,  -1.11677277,  0.144420534,  0.452227473,  -0.358674616,  -2.20165133,  -2.39940166,  -0.94897753,  1.33243978,  -0.0101742391,  -0.748070836,  -0.177169159,  0.0614691973,  -1.99911916,  0.00244346354,  1.50648069,  0.625171661,  -0.866489232,  -1.38749468},{ -0.0886176452,  2.52006793,  0.947447896,  -0.486248583,  0.0348273329,  -0.579087198,  -0.598449588,  0.0291522387,  -0.560568929,  0.798499703,  0.994935393,  0.198912397,  0.352700651,  1.20479977,  -0.363492429,  1.38490927,  -1.34630251,  2.24634171,  0.0590109751,  0.458986163,  -0.902600527,  -2.49950051,  -0.213007286,  -0.153593808,  0.00707227876,  0.210118324,  0.761613309,  -0.23473838,  0.472158045,  1.73291624,  0.0646467209,  0.606135845,  -1.0058738,  1.38284993,  -1.16609979,  -0.519132376,  -0.114393473,  1.74201369,  -1.63241363,  0.309906036,  0.0463081188,  1.27121365,  -0.632634521,  -0.828875303,  -0.152855024,  2.28751516,  -0.986120582,  0.904770255,  0.9274804,  -1.84484303,  1.54845107,  -0.733891726,  1.31145954,  0.714675367,  -0.00626411615,  -0.805803359,  0.45889926,  0.655347526,  -1.80546808,  0.108840853,  -3.46047473,  0.629994214,  -0.725844562,  -0.715420127,  0.455156326,  0.90515244,  -0.0233878642,  0.861293852,  -0.166630283,  -0.581534028,  1.59127772,  0.76031363,  -0.702930212,  -0.229974851,  -0.687911093,  0.362505883,  -0.507706463,  1.44526935},{ 0.430165946,  -1.83694744,  0.733540297,  0.261966586,  0.18158488,  0.0589546897,  -0.572314441,  0.932241619,  -1.24798656,  0.139970571,  -1.25755811,  -0.319513589,  0.0786945149,  -0.323034614,  1.00730515,  0.593697965,  -0.18219164,  0.0301807467,  -1.61876309,  1.58848798,  -0.844200015,  -0.351741582,  0.829858065,  -0.0155261373,  -0.000451640866,  1.51235533,  1.20014346,  -0.82672286,  0.0247430485,  -0.820157766,  1.47225595,  0.343247652,  0.319144994,  -0.803832412,  -0.976230145,  0.512077808,  -1.17957354,  0.755632997,  0.297764093,  -1.38725603,  0.423411518,  -1.50244999,  -0.720193624,  -0.207577601,  1.40139329,  1.04805505,  -1.04991663,  0.20283781,  -0.624874651,  0.278805077,  0.387191445,  0.0146176387,  1.40751481,  0.339642048,  -0.0685003251,  -0.202235863,  -0.460811943,  -0.296319157,  1.45067489,  1.37459195,  -0.425347835,  -0.279652238,  -0.353547633,  -0.856499314,  0.547796905,  0.479269147,  -0.219207048,  -1.17638648,  -1.32776618,  -0.946305275,  0.852841735,  -0.358011335,  1.74689627,  -0.839728892,  -0.415562421,  1.63861287,  0.543632686,  0.597487867},{ 0.141912907,  0.293505132,  -1.09394348,  -0.573165774,  -0.00477026636,  -0.982082427,  1.85911644,  -0.400378436,  1.56433046,  -0.514819741,  -0.28584677,  1.24071646,  -0.444488049,  -0.867488444,  -1.10181022,  0.248408288,  -0.0858560055,  -0.188131437,  0.521748066,  0.610408187,  -0.350336164,  -0.404311746,  0.200927287,  -1.3540082,  -1.35026181,  0.730514824,  0.490267992,  0.0160289835,  1.60847807,  -0.254698098,  -0.217138514,  0.398461878,  -0.362429947,  -0.455482185,  1.23451364,  -1.75598598,  -0.848949373,  -0.527060449,  0.713124454,  0.572347164,  0.211052328,  1.83743215,  0.282023132,  0.543356836,  0.555024087,  -1.27288997,  1.09755147,  0.80564636,  -0.575477242,  1.14992952,  -0.176579848,  0.43227452,  1.17895329,  1.41138875,  1.81022823,  1.69390011,  -0.279730499,  1.10366702,  -1.03695631,  1.60479617,  -0.428913057,  0.910354257,  -0.118322194,  -0.619609594,  0.177418634,  -0.206426442,  1.66197789,  0.221880317,  1.85508263,  -0.0781525224,  1.09789371,  -1.25820029,  -2.26466632,  -0.0923167914,  -3.6060729,  0.0673330426,  1.40649772,  -1.7837975},{ 0.587692201,  0.253596723,  0.691715658,  0.079168357,  -1.35968494,  -0.528739929,  1.43305254,  0.513170779,  0.54424274,  2.62779474,  0.0427666977,  0.677576363,  -0.0318710431,  1.92841315,  -1.06321907,  -0.178264469,  1.20313323,  0.117484733,  1.72775638,  -1.81241429,  -0.957824409,  1.20349813,  -0.259043187,  -0.0890496969,  0.683062494,  -0.693652391,  -1.28637028,  -0.691870093,  2.19873381,  -1.91032112,  0.0469675846,  -0.437354892,  -1.81263077,  2.2028389,  -1.21254206,  -0.0183985177,  -1.17880952,  0.476068318,  0.319205105,  0.0823734775,  0.355396479,  0.50130403,  0.966537654,  0.727063835,  0.384991437,  -0.919996262,  1.40575767,  0.265064538,  -1.49436462,  0.89183867,  0.183169246,  1.73931968,  -1.5495261,  2.06131887,  1.45439875,  -0.727534235,  1.29869306,  0.526528656,  -0.021645302,  -0.531754375,  -0.218253955,  2.32974911,  0.251741499,  -1.03011024,  0.979136348,  -1.24744451,  -0.279866725,  -0.661700606,  -0.45210287,  -0.413624823,  -0.0814506635,  -0.206563637,  0.396796972,  -1.45740366,  -0.341736794,  -1.27404356,  -0.197175473,  0.732667804},{ -0.0547636673,  -1.32665348,  -0.479825109,  0.857110023,  0.824061275,  -0.031461373,  -0.307929605,  -1.24196279,  -0.26628381,  -0.881965816,  -0.979322433,  -0.941930115,  -1.0412854,  -0.374701023,  1.6856755,  -1.51627302,  0.514606059,  0.163637102,  -0.802374482,  1.03400815,  -0.820967615,  -0.949317157,  -0.827581406,  1.138152,  1.12727046,  -0.313450754,  -0.661052406,  -1.82225597,  0.81786108,  -0.15933691,  -0.735443056,  1.51508057,  -1.53217375,  0.578486919,  -0.306863636,  0.672054291,  -1.1256485,  -0.524099946,  -0.879571557,  1.21887851,  2.60969806,  0.277963102,  -0.66518122,  -0.208486855,  -0.454145253,  -0.71573478,  -0.303302348,  1.45045936,  0.939979017,  -0.299751848,  0.315170735,  -0.72534138,  0.507125974,  0.0217770692,  0.301517636,  -0.011882918,  -0.462611645,  -0.0977260098,  -0.321070731,  0.454036176,  -1.18731952,  0.571902871,  1.21991754,  -1.01454985,  1.70381749,  0.732048631,  0.504033804,  -0.0140561741,  -0.230242863,  1.559304,  0.148395106,  0.530765712,  0.148547292,  -0.1107051,  0.639180303,  -2.21216512,  -0.691514313,  -2.05703712},{ -1.50433815,  0.34516114,  -0.469931066,  0.285781443,  1.54944575,  -1.17203593,  0.242002591,  0.208363995,  1.1692692,  -0.723621964,  0.928490102,  0.411004484,  -0.380350083,  -0.108063832,  0.099073492,  -0.020929059,  1.12463212,  -0.750529468,  -0.710401118,  0.393796146,  0.431112885,  -0.30773738,  0.0960757583,  -0.0579897948,  0.201715365,  0.112712942,  -1.09428954,  -1.15601575,  0.464631855,  0.290475219,  -0.664993882,  1.16575742,  -0.431834817,  0.180836365,  -0.630272806,  -0.283029199,  0.327104419,  -0.613702595,  0.568556607,  0.152966201,  0.301221639,  0.472076982,  -0.295318007,  0.260728657,  -0.300736398,  -0.556743681,  0.216065198,  -1.14563823,  -0.304042399,  -0.271095723,  0.367553502,  0.759281039,  1.31797552,  0.0559467301,  -0.0646796823,  -0.949477911,  -0.969031215,  0.308797747,  -0.183859184,  0.873410881,  0.124424875,  1.44500458,  1.29559064,  0.444413424,  1.11540091,  0.303573549,  0.632004738,  -0.711027801,  -0.919265985,  1.29763782,  0.0256867036,  0.00338118058,  0.704925358,  -0.482962698,  -0.365842611,  0.619072795,  -1.90099418,  0.854363203},{ -1.59637916,  0.165654495,  2.21974659,  -0.48244375,  0.268946767,  0.0567439646,  -1.0725925,  0.934652984,  1.44344664,  -0.797742546,  0.269499063,  0.777710915,  -0.264100701,  -2.46304297,  0.551155567,  -0.215287909,  1.45810056,  -1.12650919,  -0.581545115,  -0.0319512375,  0.800999343,  0.674560666,  -1.05275285,  1.25596142,  0.479940116,  -1.1029501,  -0.0285552591,  -0.3713696,  0.240461797,  0.210737661,  0.962412417,  1.26496315,  0.513462543,  -0.967050791,  1.29117334,  -0.75299418,  0.152066916,  2.15380573,  0.723298013,  -0.376259357,  0.6852355,  -1.1699481,  0.332593471,  -0.198358282,  0.328057319,  0.813575506,  0.203442246,  0.387077451,  -0.899241865,  -1.09077859,  -0.695109069,  0.998953283,  -1.38713241,  0.564157128,  0.75481689,  1.1674881,  1.32590711,  0.868905723,  -0.465623051,  0.495532423,  -0.373322308,  -2.30652213,  0.903462589,  1.16579831,  1.2073319,  1.77408648,  0.537614524,  -0.90020895,  -0.894805133,  0.190682009,  0.687003732,  1.40987682,  0.940324605,  -0.376225352,  -1.58746052,  0.804888725,  0.81433624,  -0.670992672},{ -0.112667099,  0.654657662,  2.11210322,  -0.941796601,  1.26975131,  2.10417485,  -0.837613583,  -1.9085108,  1.90160072,  -0.437842041,  0.980372667,  -0.479066402,  0.378664255,  -1.16904211,  -0.122807682,  -1.7569499,  0.290467411,  0.166823357,  -0.348154575,  0.133934841,  -0.622372985,  1.24702632,  -0.652054787,  -1.39305627,  0.401389152,  -0.370197654,  -0.136097252,  -0.821112812,  1.42813003,  -0.166194662,  -1.39730406,  1.46845567,  -1.38468218,  -0.540081441,  1.76403773,  0.739374638,  -1.29032481,  -0.111064464,  0.0792624578,  1.76202643,  3.36167312,  -1.3343488,  -0.248837113,  0.514840901,  -0.879604995,  0.336198866,  -0.501746178,  0.980317593,  0.668486655,  0.589484632,  -0.404923141,  1.43299508,  -0.166976258,  -0.80255568,  -0.476705492,  0.509325266,  -0.430493772,  -0.843005002,  -0.0823464543,  1.00037932,  0.0830600336,  0.113695487,  -1.17725384,  -0.149363562,  1.28646994,  -0.948432446,  -1.41247594,  -0.490110368,  -1.14787507,  0.600970268,  2.01585793,  0.71356529,  -0.789104998,  -0.933976471,  -0.192943677,  -0.213651165,  1.49542189,  1.00862885},{ 0.617886126,  0.403746516,  -1.45501053,  -0.129221216,  1.1530323,  2.1670289,  1.13926136,  1.91522765,  -1.32095504,  -0.250504345,  -0.343444675,  -0.420544624,  -1.3212359,  -0.547497392,  -0.199096859,  -2.06225967,  1.36801815,  -0.419389933,  -1.46734035,  -2.19663072,  -1.42892253,  -0.113367096,  0.276693076,  -1.30865908,  -0.760628879,  -1.66653407,  -0.403972387,  0.271300822,  1.41200149,  -0.136635214,  0.533804357,  -0.0316649042,  -0.0287434459,  1.36231828,  1.54975641,  0.722231984,  2.01668262,  -1.25655782,  -0.404143304,  -0.420030236,  0.132482126,  -0.50851655,  0.224747151,  0.368722677,  -0.68075633,  0.546076536,  0.272524893,  -0.625810981,  -0.040589273,  -1.19422245,  0.324481696,  0.237396359,  -0.351154476,  -0.608131349,  0.505452633,  -0.235840365,  -2.18828011,  1.71876967,  -1.27493143,  0.361445397,  -0.668591917,  -0.0547506139,  1.024068,  -0.954050362,  0.516022027,  0.725538015,  0.0713356808,  0.942143261,  -0.629716694,  1.53088009,  -0.531454146,  -1.19273162,  0.690154135,  0.374106586,  -0.251652837,  -0.144339949,  1.52924764,  0.814289331},{ 0.918380499,  0.23197718,  1.00142407,  0.498297453,  1.0482198,  -1.95772505,  -0.108241804,  -0.755288243,  1.03460288,  1.33210623,  0.119563922,  0.102523923,  -0.998206437,  0.07576859,  -1.830194,  2.06458092,  2.67667675,  0.519338012,  0.0680070743,  1.07477605,  -1.12334883,  -0.074886471,  1.24531555,  0.35886997,  -0.29108572,  0.289375037,  -1.19329071,  -1.56566608,  -0.887521446,  -1.39693999,  0.939285636,  0.103479505,  0.604589462,  -0.659356058,  0.144780561,  0.457434624,  0.455960631,  1.69890499,  0.043888174,  -1.23930955,  -0.166613564,  1.92665851,  -0.586936474,  0.635798931,  -0.148734719,  -0.738632083,  -1.06694722,  -0.12730369,  1.63272142,  -0.369027168,  -0.550372064,  0.46347481,  0.923673689,  0.610974908,  0.0151352966,  0.175270885,  0.73813653,  1.54633784,  -0.442920595,  -2.05375886,  -0.142835811,  2.18840671,  -0.715098441,  -0.807264984,  -0.0610028319,  0.50490135,  0.353554726,  0.431417972,  -0.39582637,  -0.0489045419,  0.810356617,  -1.78000653,  1.39927101,  0.974463344,  -0.0335873403,  1.28491616,  0.490851611,  0.185406342},{ -0.122132875,  2.3785398,  0.0805034116,  0.313673466,  0.249137223,  -0.0359546319,  0.835744441,  -0.709806085,  -0.182841212,  -1.186921,  -1.39690483,  0.153670952,  0.841417849,  -0.167912468,  -0.411452174,  -0.8205266,  -0.0697646812,  0.0819906816,  0.233293235,  0.0118851895,  -2.19716454,  0.387688845,  -0.508855224,  -1.18833852,  1.46664548,  -0.0517054237,  1.50582623,  -1.54535866,  -0.577204704,  -0.488969326,  -0.139299899,  -1.18207312,  0.0670366958,  -0.142969146,  0.526789665,  0.119644336,  1.50132704,  0.743173718,  -1.37171888,  0.925805748,  2.20348835,  0.345819771,  -0.114650637,  -0.440829098,  -0.572393239,  -0.00361702545,  -0.512855887,  -1.01161361,  2.247509,  0.487134755,  0.748189032,  -0.162994534,  -2.51856065,  -0.701772273,  -0.439555585,  0.663961947,  -0.163120821,  -1.03974593,  -0.92810905,  0.856929362,  -0.414988875,  1.43191826,  -0.89590764,  -0.326848477,  1.5354178,  0.565815389,  2.83467698,  2.02110672,  -0.0326964296,  -0.101059541,  -0.0513875447,  -0.179437876,  0.0996964648,  -2.10564518,  0.439023584,  1.19984329,  -0.395710379,  0.408280641},{ -0.713719964,  0.542104363,  0.406385839,  0.0996015519,  0.038796097,  0.00024243095,  -1.51154864,  -1.51316404,  -1.78595912,  -1.28627813,  0.342960715,  1.03388762,  0.525321484,  0.588893473,  1.56871569,  -0.549275279,  -0.28279686,  -1.16267431,  -1.19433546,  0.0616240017,  0.867013931,  -2.1155386,  0.225401089,  -0.959430873,  0.663011432,  -0.6463781,  2.39410448,  0.948384941,  -0.882661223,  -0.0114977248,  -0.654193878,  0.847963452,  0.0461356714,  0.741147697,  0.46713981,  1.43212616,  -0.838412404,  0.604844451,  -0.264656723,  -0.194176808,  -1.18265736,  -0.390678138,  0.220222712,  0.782648921,  -0.805519462,  0.856989264,  1.06908524,  -1.146649,  -0.862973392,  -0.621896207,  0.759685457,  -0.562555432,  0.738266766,  -0.474875838,  -0.707429588,  0.426475346,  0.133278131,  0.17422469,  -3.74810171,  -0.0281656347,  -0.0437297821,  -1.19784629,  -0.874273956,  1.28315961,  -0.440581113,  -0.214985907,  -0.806759238,  0.860895395,  0.428875357,  0.94295907,  0.0651796758,  -0.694855928,  -0.168061957,  0.22366409,  -0.806042969,  0.736715078,  -2.6556983,  0.345552504},{ 0.129886687,  0.313650161,  -0.00527937664,  -0.821590841,  -0.963628948,  -0.419281632,  1.8263725,  0.155189276,  -0.702248454,  0.330380291,  -0.557702839,  0.83308059,  -0.662265897,  1.00873983,  0.518506944,  -0.513579249,  -0.339648664,  -0.536450863,  -1.29515898,  0.658825099,  1.16202593,  1.2744689,  -0.891367257,  -0.260359824,  0.0590679124,  -0.385623872,  -0.264296651,  -0.100376219,  0.399262518,  -0.657655954,  0.381585032,  -0.86046958,  1.29147756,  -0.509154618,  0.811669946,  -1.17526889,  -0.594190419,  0.391091019,  1.40707195,  0.396701515,  0.707894027,  0.694354355,  0.550970256,  1.18974531,  0.338920712,  -0.389969468,  -0.581135631,  0.587232769,  1.27108705,  -1.36786246,  0.554016292,  0.398582309,  -1.77318132,  0.850635886,  -0.352325737,  -0.3717556,  -0.146768242,  0.853863657,  -0.955152929,  0.210111231,  -0.134992018,  -1.00781965,  -1.93581295,  -0.32363537,  -0.703211546,  1.58384287,  0.862070262,  0.911079407,  -1.05824661,  0.318691611,  -1.40253592,  -1.69848692,  1.07207632,  0.973984301,  2.1103034,  1.23080051,  -1.8483789,  -0.291896611},{ -1.09494805,  0.941105127,  1.07322323,  0.339100391,  0.431790978,  0.461875975,  -0.348940611,  -1.00275385,  -0.540552139,  -0.452161729,  0.849664986,  0.931355536,  0.719824851,  -0.864088118,  -0.217204362,  -0.185967252,  -1.49435639,  -0.503966033,  -1.13704002,  0.62726897,  1.82247567,  2.81483889,  2.96377754,  0.567522347,  0.697574675,  -0.593955755,  0.618525386,  -0.468752384,  1.27705705,  -1.36982274,  -0.000509047066,  -1.73416388,  0.17103596,  -1.992805,  0.566094756,  1.50046933,  0.713268697,  -1.10417259,  2.06481147,  -0.282137483,  0.590995789,  -0.482262522,  -0.0939323008,  1.21552801,  -0.35040918,  -1.10331368,  0.307967126,  -1.26761734,  0.766742349,  0.824953496,  -0.670588136,  1.96256006,  -1.97134662,  1.1450578,  0.138377622,  -0.201522633,  0.037096791,  -0.561638355,  -0.395984679,  1.1984781,  0.737656593,  2.37631893,  1.25138831,  0.632466555,  0.734317601,  -0.418183208,  -0.57924813,  -0.0999285132,  -0.302612782,  -0.560277462,  0.052122321,  0.947276652,  1.12938023,  0.459627092,  -0.509685218,  -0.288755327,  -1.61679065,  0.872650504},{ -0.0186441503,  -1.0803082,  0.281904459,  -0.518175542,  -1.66166914,  -1.60755312,  0.0168672428,  0.0771567598,  0.125940233,  -1.66439819,  -2.71171117,  -1.57911265,  0.136642337,  0.256127894,  0.690049589,  0.255325735,  -0.0780047178,  -0.363211483,  0.32269752,  2.61891437,  -1.04949629,  -0.434801459,  0.179132208,  -1.07868016,  -1.52259648,  0.74644345,  1.05604768,  0.083420001,  -1.78782225,  -0.105336003,  0.271887362,  0.765595496,  -0.935000479,  2.19625211,  0.309892565,  0.660570264,  0.0390368477,  -0.545942545,  -1.17082119,  -1.30544126,  0.820233941,  0.324836701,  -0.187033445,  0.621524572,  0.74181062,  0.14840655,  0.419438303,  1.41083539,  1.07890546,  0.423933268,  -0.456773549,  -2.34289408,  0.140310407,  0.530290961,  0.98528707,  0.352219045,  -0.860636175,  0.424275339,  -0.0592447221,  -0.367479652,  1.06139898,  1.37701976,  0.252271235,  0.517065108,  -0.000299382838,  0.17846325,  -0.0759085193,  0.32532841,  -0.626954436,  1.22218108,  0.179184005,  -0.883186936,  0.00493106293,  -0.675631225,  1.82334375,  -0.722187757,  -1.41180682,  -1.32181013},{ -0.279143512,  -0.324402064,  1.18871796,  -2.95084929,  0.82459116,  0.179604918,  -0.659240544,  -0.619070888,  1.36656296,  0.646622598,  1.18486917,  -1.11239362,  0.0651655719,  -0.887874603,  0.132168472,  -0.370119005,  1.76219761,  -0.33343932,  -0.236255929,  0.953500986,  0.36744988,  -0.0215430278,  -1.63190842,  -1.25796008,  -0.506698906,  -1.13873875,  -0.906892002,  0.507458866,  -1.41367626,  -0.352061838,  0.494793802,  0.384945184,  -1.37161136,  -0.774712741,  0.0825141668,  0.968424857,  -0.523483336,  0.844113767,  0.766278923,  1.18284333,  1.65227985,  0.300839275,  1.51987565,  1.0889293,  -0.939127505,  -0.171898738,  -0.576913834,  0.913684249,  -0.22746174,  0.88299042,  1.04931557,  0.00485167932,  0.58072865,  -0.0129342191,  -0.967020988,  -0.654250205,  -0.242580801,  -0.223486617,  -0.150409922,  0.169011444,  0.939594328,  -0.100707144,  1.05239916,  1.60599363,  -1.64390898,  -0.291639358,  -0.364570975,  -0.765240848,  -0.258436531,  0.417668343,  0.173066303,  -0.778401852,  -0.615482032,  0.560521662,  -0.0708514452,  0.0610141233,  -0.640834689,  1.37662017},{ -0.267049372,  -0.867334187,  0.247260407,  1.03183889,  -0.487969816,  -0.922609627,  -0.342065722,  0.903455555,  0.260151446,  -0.422409326,  1.75898838,  -0.544051528,  0.0219690837,  0.17388311,  0.354665011,  -0.0593082644,  -2.02172875,  0.763673007,  -0.698302388,  1.21359587,  -0.525142014,  0.770538628,  -1.27001154,  -1.2169385,  0.817929268,  0.878390491,  -0.0315670818,  0.856240988,  0.307768852,  -0.396041095,  -0.724454761,  1.16749001,  -0.334379405,  -0.945119917,  0.202282086,  0.0901930779,  -0.4405981,  -1.01800525,  -1.49091756,  -0.910381019,  1.54380476,  0.634379566,  -1.15377128,  0.18753925,  0.634649098,  0.810192764,  -0.39952597,  0.55518049,  1.32725739,  -0.76038146,  -0.594621181,  1.70760632,  -1.88829863,  0.664420784,  -0.761400878,  0.902972162,  -0.801708698,  -0.438935786,  -0.565842927,  1.05943739,  -1.50628543,  1.18913031,  -0.584606707,  0.926847219,  -0.680533826,  -0.231792122,  -1.01106453,  0.628626764,  0.0673351064,  -0.466953993,  1.0238353,  -0.46688664,  -0.426949799,  0.0933270156,  0.268458158,  -0.750044823,  0.449643433,  -1.10427737},{ 0.556089759,  0.0755637884,  0.222848877,  0.0254007988,  -0.862018585,  0.363814414,  0.85388428,  -2.48415041,  -0.283138663,  -1.99322593,  -1.3956995,  -0.095100306,  -0.793630123,  -0.172761172,  -1.07084656,  -0.170927063,  -1.04805684,  1.57045436,  -0.840376079,  1.41195261,  -0.195775837,  0.120787449,  1.06275213,  -0.489222437,  -0.104030244,  0.665415108,  -0.14207308,  2.19256473,  -0.232645229,  -0.898576558,  -0.214273795,  -2.3667562,  0.2441324,  0.958807707,  0.945305705,  -1.27250302,  -0.499633461,  -0.0615157783,  1.2459029,  1.60995626,  -1.137308,  2.6376431,  0.858981013,  -0.317736745,  -0.605600536,  0.458943248,  0.0372883454,  -0.422810167,  -0.0336569734,  1.28872836,  -0.917947471,  1.77804923,  -0.341580957,  -0.184350774,  0.183270708,  -1.82953858,  0.133531243,  0.379654557,  -1.86779189,  1.01114643,  -0.716606855,  0.825359643,  0.521632969,  0.137997419,  0.534777164,  -0.327982783,  0.290556639,  -0.321661949,  -0.135944471,  1.14415872,  0.0526309609,  -0.934468567,  -1.81679797,  0.493421137,  -0.151651576,  0.634815693,  -2.09224749,  0.541780829},{ -0.0751124695,  -0.54559797,  1.12468719,  0.240289241,  -2.9806993,  2.22947359,  0.371932417,  -0.3773368,  -0.941646695,  -0.866782665,  0.353318155,  -0.802659571,  0.403623343,  1.22495413,  -0.197968185,  -0.40292421,  -0.37979418,  -1.37525129,  -0.748654842,  0.657437801,  -0.763955534,  -0.177791491,  -0.379397303,  -0.307848185,  -1.09619713,  -0.565259397,  -1.41574275,  0.201948658,  -0.662306845,  0.027914267,  -0.50059545,  0.276092917,  -0.819976032,  -0.446769774,  -0.747463763,  -0.271443456,  0.0848722309,  -0.69408828,  1.22901654,  0.519213974,  -0.836130738,  -0.594451785,  -0.227266535,  -0.538185775,  0.313826621,  -0.413331687,  -0.200444832,  -0.240651846,  -1.39670599,  -0.54994607,  0.129446879,  0.452896208,  -0.492656469,  1.09564316,  0.0435609855,  -0.663007259,  1.85951281,  -0.433999866,  0.00616068998,  -0.921468079,  -0.628217876,  -0.165964231,  0.224579215,  -0.90928185,  -0.6013816,  -0.658106685,  0.605669856,  0.816010058,  -0.275319189,  0.0945693105,  1.10806072,  2.32570744,  -1.64134777,  -1.4208138,  -0.404691756,  -0.933497488,  0.710736811,  -0.262724429},{ -0.606033146,  1.25133383,  -1.84093022,  1.90252531,  0.617491782,  -0.516825557,  -0.772705376,  0.40515554,  1.83005834,  1.22830677,  -0.380215973,  0.91876328,  -0.494302243,  -0.226445273,  -0.431437284,  -0.315862536,  -0.206360102,  -0.549625337,  0.625670671,  -0.382384032,  3.08057022,  0.0327560455,  -0.316604346,  0.554564118,  1.30991673,  -0.719087064,  -0.891077101,  -1.20682788,  -1.60923707,  -0.741770148,  0.282426298,  -0.0333183445,  -1.33369458,  0.23789677,  0.125328794,  -0.507900834,  0.0962813497,  0.127301097,  2.07390141,  -0.572882533,  -0.405703157,  -1.75671208,  -0.0284193363,  0.56165576,  -0.471663594,  -0.0386644825,  1.24075103,  -0.180385813,  -0.891976058,  -0.187773108,  0.796919823,  -0.468826979,  0.190574661,  0.4637914,  -2.34274101,  -0.131428957,  -1.52184117,  0.0109310159,  1.16143239,  1.35728586,  0.666194379,  -0.313737303,  0.423286408,  -1.02111185,  -0.188988358,  1.42842948,  0.115845993,  0.361170381,  1.15647149,  2.90991926,  1.14574802,  -0.419120729,  -1.13489366,  -0.60369122,  0.934924185,  0.0677959397,  0.126815826,  -0.215505198},{ 1.54522824,  0.290573835,  -0.757332027,  1.67436874,  0.0161362626,  -0.0677708164,  -0.371834069,  0.297234803,  0.44693923,  -2.051893,  0.302061319,  -0.106811069,  -1.27532756,  -1.25371528,  2.43584871,  -1.48238027,  -0.257100224,  0.797432184,  0.748945951,  -0.998931944,  -1.09051752,  0.607898176,  0.311540753,  2.05920458,  -0.373423308,  -1.72856963,  -1.42827368,  1.5882051,  0.0746844634,  -0.474387586,  2.11500788,  -0.410536617,  0.283040732,  -1.79761791,  -1.38141942,  -0.484239757,  -0.726299942,  1.17420268,  -0.375837177,  -0.217214406,  -0.619429171,  0.888224661,  -0.737827659,  1.37323999,  -0.833496928,  0.433085889,  -1.26905918,  -1.09240186,  -0.104222126,  1.06178796,  1.10978842,  -0.477060378,  1.17422605,  0.1585228,  -2.20531631,  0.538275898,  0.695706904,  0.950003922,  -0.404285312,  0.635988653,  -0.00973508414,  -0.411561519,  0.400333673,  -0.00277251401,  0.694263101,  0.0670783743,  0.937644184,  -2.24090862,  0.18369554,  -0.0549660474,  -0.652793467,  -1.56191003,  1.63982201,  1.72928762,  2.14247727,  0.222837999,  -0.213959828,  0.0650046095},{ -0.318093061,  -1.4033128,  -0.0299723726,  1.31495714,  1.76542246,  1.32302439,  -0.0486465469,  -1.45888627,  -0.200317621,  0.0110721439,  0.192905366,  -0.446896255,  -0.740080118,  0.714507937,  0.611903071,  0.0219518337,  -0.109414101,  0.294194996,  1.13428891,  0.966563463,  -0.94724077,  0.0957036465,  -0.741611183,  0.0486902073,  1.027843,  -0.407749683,  -0.409319639,  0.939473093,  -0.933662772,  0.886209786,  1.13130486,  1.89284122,  0.0290279109,  -1.49921036,  0.0896696076,  -0.247428551,  -0.961723208,  1.5426718,  -0.0136285368,  2.0737493,  -0.909319043,  0.553822398,  0.866132259,  -1.60269785,  -1.00371325,  -1.26000166,  -0.450781345,  -2.98852968,  -0.967614114,  -0.572502851,  0.555214226,  -0.950731516,  1.5568645,  1.28934681,  0.572615325,  0.856258035,  -0.16375941,  0.747046411,  1.39562786,  0.423891008,  1.479864,  -1.57903516,  0.619792223,  -0.0980485082,  -0.281084716,  -0.70097667,  -0.428901851,  0.276248097,  -0.544916987,  0.059908893,  -1.56519043,  1.57707226,  -1.47124434,  0.485129714,  -2.32846475,  -0.141830981,  -0.777415037,  0.780742466},{ -0.103446506,  0.261608452,  -0.71781069,  -1.92393816,  1.49464023,  -1.6590786,  -1.72714043,  -1.92082918,  -0.935918272,  -1.29610467,  -0.673020959,  1.30911982,  0.618225873,  0.960797131,  -0.140100271,  -1.43598175,  -1.21036565,  -0.346824169,  -0.0340909101,  1.29695594,  -0.837900162,  0.352597624,  -0.754432142,  -0.280559033,  -1.19117868,  0.795817912,  0.987267256,  -0.70075053,  0.456546992,  1.46785438,  0.0644164234,  -0.151856244,  0.370153159,  0.794450939,  -2.41060114,  -0.294820875,  0.546431541,  -1.27618885,  1.64913321,  0.0580861159,  0.34393546,  -1.35655737,  0.561266243,  -0.360624641,  0.483891219,  -1.20311224,  0.768105745,  -0.03521901,  -0.639699638,  -0.870110452,  -0.860460281,  -0.384202838,  0.131945252,  1.1390301,  0.00420853822,  1.07494712,  -1.22269785,  -0.306741565,  -1.67376494,  0.0688539147,  -0.293977469,  -2.03240442,  -0.430057049,  0.0862211362,  -0.160814062,  0.166501939,  -0.542911053,  0.217712149,  0.938361645,  0.209220976,  0.012737222,  0.263253212,  -0.836847961,  -0.0822871849,  1.40387177,  -1.53640437,  -0.232584685,  1.17329025},{ 0.393358618,  0.767454684,  -0.43631953,  -0.332535923,  -0.141633183,  0.639795423,  -1.49007642,  0.290899783,  1.58555329,  1.65418935,  -0.355551511,  0.153264195,  -0.0529832318,  0.715240479,  -0.0603505559,  0.849540055,  0.763702273,  -0.303544581,  -0.797229886,  -0.107848071,  -0.388013303,  -0.40512532,  0.111167446,  -0.0861622021,  0.967700422,  0.783470213,  -2.02853537,  -0.525504291,  -0.140144944,  -0.0462301224,  1.1478734,  0.00124549854,  0.220602214,  -0.402862132,  0.130841583,  2.20126319,  -0.788446128,  0.796481013,  0.441426694,  -0.417408049,  -2.77242589,  -0.757422268,  -0.191828355,  0.104724467,  -0.0430259034,  0.618470669,  0.315214694,  -1.12490559,  0.210432664,  -1.15537012,  -0.0115351696,  0.47775048,  -0.681381881,  -1.4669143,  -1.65251839,  0.835164368,  -0.268330723,  -1.07971311,  0.587662995,  0.954086065,  -0.92491132,  -0.166726813,  0.911896646,  -0.538328707,  -0.482183099,  1.59832466,  1.25626981,  -0.377390414,  0.487339765,  0.239101961,  -0.33210814,  1.31964564,  0.281598449,  0.211591631,  0.0187230948,  -0.476038545,  1.03111839,  1.28289771},{ 0.406681418,  -0.749541223,  1.1601516,  1.1211282,  -0.618986666,  1.16345894,  1.97595334,  2.72786999,  1.13174844,  1.24825835,  0.604585469,  -0.0171299595,  -0.502468407,  0.389829636,  0.499333888,  -2.7381897,  -1.11440992,  -0.0395697728,  0.118584625,  0.552881956,  -0.315663546,  0.24449037,  -0.27626434,  0.384738356,  1.28131247,  -0.423747838,  0.92180264,  0.821648777,  -0.0373626798,  -1.42280471,  1.03291488,  0.359523773,  -0.944735944,  0.915444493,  0.290229946,  0.0420887992,  0.0823890492,  -1.05514324,  0.10214477,  -0.525076568,  -1.46513796,  -1.19143724,  2.62416077,  -0.718396187,  -0.820255101,  -0.925048828,  0.848916948,  -0.350831211,  -0.695890605,  1.11477017,  1.73759675,  0.497862548,  -0.693499148,  1.73848069,  -0.0251032971,  0.43001464,  0.405005425,  -0.51266408,  -0.950398684,  -0.541692793,  -1.13992822,  0.317788899,  0.987612605,  -0.149905145,  -0.331614912,  -0.648259342,  -0.261417508,  2.05546927,  -0.50875169,  -0.828492284,  0.279512793,  -1.26081836,  0.153503582,  -1.5982008,  1.24475265,  0.235930115,  -1.1572969,  1.68574393},{ 0.666560888,  -1.76465106,  0.670007229,  -0.120364636,  0.90459913,  1.24744534,  0.20364067,  -0.339726359,  1.53532088,  1.21946192,  -0.713275254,  -0.230333477,  1.01697683,  1.16081011,  0.513906777,  1.25712228,  2.1134665,  0.102055117,  -1.47527206,  -0.921913505,  -0.649286866,  -0.587158859,  -0.402858913,  -0.0149458302,  1.04075015,  0.407067895,  -0.158070922,  0.822749972,  -0.514202654,  -0.984887004,  -0.024885254,  0.547284603,  -0.818644583,  -1.07495677,  1.60746181,  0.0985423923,  1.08862996,  -0.213604614,  0.117077783,  -0.0884550288,  1.53557265,  1.27111363,  1.77575171,  1.07998168,  0.95411706,  1.07128072,  -1.77335823,  -2.61396432,  1.01582563,  0.410881758,  0.509665906,  -0.177335873,  1.81307006,  -0.00111841725,  1.89504623,  1.6594553,  0.538920283,  0.0279457215,  0.359370083,  0.477627814,  0.979986846,  -0.00173565676,  0.0502249487,  -1.88153231,  0.375846297,  0.224100277,  -0.164278999,  0.289076626,  -1.60561335,  -0.229415819,  -0.395184577,  2.67521048,  0.24564077,  -1.06117833,  0.580439866,  -0.313284427,  -0.00149585004,  0.484528631},{ -0.709647596,  -2.19115162,  0.618830383,  -0.543163598,  0.489192605,  -0.961391091,  0.171569794,  -0.0184649583,  0.725718975,  -1.55045474,  1.9318583,  -1.4904604,  1.13051534,  0.652368605,  -0.749602795,  -0.301260799,  -0.278854847,  -0.869273543,  -0.255298406,  -2.02233267,  1.17286825,  0.493485987,  -0.691650093,  -0.469338447,  -1.35418367,  0.63214463,  0.00252660085,  -0.533234656,  1.4272666,  -1.0561341,  1.0967387,  0.859105885,  -0.716106296,  0.301875859,  -0.681456327,  -1.38515782,  -1.60009694,  -0.472237289,  -0.987031221,  -1.46809947,  1.58208978,  -2.33693147,  0.415855438,  -0.368486762,  -0.514866292,  1.50005329,  -0.44989115,  1.19216824,  0.266504318,  0.878186166,  0.129746243,  -0.358297169,  -1.86452532,  0.892320752,  0.823767781,  0.501214564,  2.14881706,  0.621325016,  0.693484068,  2.24146295,  -0.852647007,  -0.261069864,  -0.978820086,  0.0370556749,  -0.911088765,  0.516470253,  -0.609605253,  -2.06712341,  0.685245574,  -0.848312974,  -0.556250751,  1.23332274,  -0.10555882,  -1.21038222,  -1.29155445,  -0.331498921,  0.440582752,  1.23481834},{ 0.170769408,  1.35426331,  -1.09698248,  -0.336055994,  -0.459889501,  -0.395243019,  0.1625157,  -0.348108172,  -1.09000814,  -1.02182615,  -0.199964896,  -0.471161455,  -0.0708802789,  1.1422925,  2.22342229,  -0.75152427,  1.11219907,  -0.257522851,  1.06094432,  -0.476813734,  0.180951148,  1.51986492,  -0.0354583338,  0.634923577,  -1.33762646,  -0.0550356619,  -0.584963262,  -3.12049103,  -1.20573115,  -0.947214484,  0.796617508,  -0.818455696,  2.26010251,  -0.289016843,  0.628627241,  1.49270773,  0.70697397,  1.12330663,  1.34486461,  -0.139720559,  0.738658488,  0.336696297,  0.1403023,  0.590974808,  -0.964863241,  1.50403428,  0.0549971834,  0.0900394395,  1.43634546,  1.42174709,  -1.07883215,  0.262824237,  -1.17621076,  2.72931194,  -0.870727956,  -0.762534261,  0.433162421,  0.879082203,  -0.773064196,  0.358575523,  -1.00550604,  -0.480892271,  -0.720507324,  0.509992599,  -0.141352609,  -0.747045398,  0.820978582,  -0.671055973,  0.815686941,  -1.08752608,  -0.164152771,  -0.168400571,  -2.22359753,  0.32078743,  -0.200166374,  0.469636291,  -0.554507077,  -1.9561615},{ 0.512728512,  -1.06202507,  1.00430119,  -0.692504108,  1.95803142,  -1.16777325,  -1.97768617,  0.719485939,  0.957689762,  -0.963066638,  -0.140895337,  -0.582842529,  1.4354341,  1.02548611,  1.33368194,  -1.42713046,  1.21598148,  -0.128686488,  -0.77953583,  1.07499421,  -0.991706908,  -0.33770442,  -0.283145934,  -0.757881522,  0.176073894,  -1.41899264,  -1.02018821,  -1.45844531,  1.34071732,  -0.905819774,  1.21850467,  0.842368782,  -1.08569312,  -0.00702607399,  -0.912818372,  -0.61178416,  0.727899015,  0.767482042,  -1.61758864,  0.293955773,  -0.744022489,  -1.75988317,  -0.838320673,  -0.145431712,  -0.0245031808,  -1.0970211,  -0.072326906,  -1.98586273,  -0.644632101,  0.922802627,  0.545338929,  0.402312964,  0.268528968,  0.497265071,  1.1666373,  -1.23175228,  -1.82286155,  1.51004493,  0.779334605,  -1.46955776,  -0.911110461,  -0.15471974,  -0.314167112,  1.15097618,  0.801076412,  0.755002618,  -0.391692162,  -1.07400823,  1.13467419,  -0.223481357,  -1.01096404,  0.254062116,  -0.430619061,  0.327302724,  -0.432770938,  0.00880290102,  -0.0892007798,  0.479990691},{ -0.139296964,  1.9148345,  0.6737867,  -1.26960325,  -0.655352414,  -0.927335203,  -0.0618061833,  1.5083766,  1.25449288,  -1.36031973,  1.0109539,  1.54053366,  -0.0637668371,  0.958939672,  0.737859309,  0.848458886,  -0.164545655,  -1.82398307,  -1.22360063,  -1.34304774,  0.942651629,  0.702184737,  0.393402129,  0.47401911,  -1.11262667,  0.57972765,  -1.32379842,  -0.32184279,  -0.457120568,  -0.903407097,  0.447498709,  0.0629441589,  0.786133289,  0.291835278,  1.2902621,  1.03082001,  0.238575235,  1.24670959,  -1.29047775,  1.30677938,  -0.9197855,  -0.365929067,  1.61356187,  -0.235273436,  -0.480697066,  -0.267926574,  0.708922327,  0.893189967,  0.104467638,  -0.63247335,  1.01481283,  0.438573718,  -0.787694156,  -1.45470536,  1.59137499,  -0.920634627,  -0.59554255,  1.64381158,  0.251716048,  0.708130896,  1.31437373,  -0.245525301,  0.600267947,  -0.351267636,  -0.462877184,  0.690671086,  0.0708282366,  -0.723940313,  -0.794881701,  -0.403913707,  0.691265821,  0.0751020759,  -0.423001736,  -0.77429533,  -0.209376037,  -0.118592978,  0.911110461,  1.2671864},{ 0.556273878,  0.168273315,  1.84844017,  -0.496633142,  1.00001335,  0.201357633,  -0.42385745,  -1.92186916,  1.95859921,  0.170713708,  2.46429682,  -0.160862654,  -1.52990758,  -0.667412877,  0.852644205,  1.56977868,  2.26344848,  -2.00960541,  -0.553758144,  -1.34395778,  -0.411042511,  -1.10644925,  0.826648772,  -1.16372967,  -0.789413571,  1.77241421,  -0.240677193,  1.00483251,  -0.611399412,  -0.894618809,  -0.450548053,  1.11675251,  -0.107441895,  -0.102516018,  0.784843385,  -1.14357865,  0.514670491,  2.2302928,  -0.577449024,  1.06151354,  0.494150996,  1.56556189,  -1.6889714,  0.0709868446,  -0.309241503,  0.915832281,  0.297619849,  0.379137039,  0.23488152,  -0.61224997,  -0.575787306,  0.0552940406,  0.751280487,  -0.154834583,  0.134738967,  -1.18767023,  -0.701071918,  0.550530493,  -0.207331404,  0.638791084,  0.872555912,  0.346309036,  -0.54730767,  1.60600734,  -0.837730408,  -0.0911200419,  -0.747074127,  -2.63301039,  -0.63040936,  1.20143867,  0.50716567,  0.558025658,  0.412447155,  0.306296676,  -0.885453522,  -1.01965404,  0.578366041,  -0.683563352},{ -0.557431161,  0.524125755,  0.602159739,  -1.72171915,  0.911991119,  -1.51961827,  1.25493097,  1.20763552,  -0.379077464,  -0.765067637,  -1.18237638,  -0.872361124,  1.4803251,  -0.396275133,  0.270237535,  -0.542809844,  0.782563984,  0.333114326,  0.221516386,  1.29000986,  -0.127811611,  0.270539284,  -1.0873493,  -0.468324184,  0.453118652,  0.100840814,  0.804279089,  -0.431108683,  -0.629924834,  -0.0907891765,  -0.645400405,  -0.0120817106,  -0.116593562,  -0.160714641,  0.467746258,  -0.468304664,  -1.4908185,  1.95335376,  -0.588005841,  1.00043595,  -0.121163398,  -0.761513054,  -1.87570536,  -0.263125867,  -2.00365758,  -0.111329064,  0.627053559,  0.557828486,  -0.393294275,  0.0904287696,  0.932781994,  -0.822354615,  0.721875489,  -1.12353158,  0.400070161,  0.221642703,  -1.18716502,  0.206694171,  1.85993648,  0.513375282,  -2.56284952,  0.0936945975,  -0.125000402,  0.62171483,  -1.81035793,  0.357449144,  -0.598713636,  1.16896474,  0.412085831,  -0.98741281,  -0.273161262,  -1.03584039,  -0.0884412304,  -0.159464389,  0.331212521,  -1.24232972,  -1.0266068,  -0.427179664},{ 1.23784614,  1.22652674,  -0.33576411,  1.60325539,  0.179328039,  -0.511380076,  0.913287878,  -0.347948402,  0.847281098,  -0.669938505,  -0.941933811,  -1.56024015,  0.0332655907,  -0.588821352,  -0.673725724,  -0.348255903,  -2.32688522,  -0.957490802,  0.993890643,  -0.23369886,  0.345809102,  0.169774994,  -2.33117914,  0.185154855,  -2.43995667,  0.0994489938,  -0.327840835,  -0.0228688419,  0.492589504,  0.36658597,  -1.04856586,  0.0490961149,  -0.88764441,  1.22777998,  -0.295595169,  0.873439729,  -1.25009418,  -1.68243957,  0.00648938166,  -0.798740506,  -0.0301986076,  -0.1447642,  -1.34524882,  0.192248583,  -0.663713694,  1.5800333,  0.922791004,  -0.24789685,  0.186619788,  0.0497011617,  -0.220324934,  -0.889258862,  1.20022583,  0.634964764,  0.115626864,  0.578189552,  -0.135127172,  -3.83337069,  0.899813652,  0.209388778,  -1.84741616,  -1.03680444,  -0.468762249,  -0.36881724,  -2.10671067,  -0.856283486,  0.872855961,  1.16640413,  -0.326127559,  -1.69603896,  1.03831649,  -1.14572036,  -1.43194091,  -2.53351307,  -0.18798919,  1.39276493,  -1.12573051,  1.38630879},{ -2.25577307,  1.68176699,  -0.29492864,  -0.968513012,  0.0414796919,  -1.43824899,  1.28416276,  -0.691087246,  1.95386696,  -1.07917309,  -0.528362513,  3.56221151,  1.69310653,  0.388794482,  -0.827263594,  0.447085351,  -0.373133421,  0.835978329,  1.06229079,  -0.947785199,  -0.829684675,  -0.632550359,  -0.984126806,  -0.874120891,  1.24304652,  0.435754895,  0.494165838,  0.834693611,  -1.15089953,  -1.12162197,  -0.670612633,  0.235312939,  -0.688869357,  -1.79043961,  -1.63248777,  -0.746274889,  -0.278196812,  -0.690543771,  0.508668423,  1.33470821,  -1.9317987,  -0.556247115,  -0.086888954,  -0.974561274,  -0.0846876204,  1.10368288,  1.08321321,  -0.114143237,  1.5717566,  1.28747761,  0.166138247,  -0.815671802,  1.79936445,  0.719797134,  0.090105623,  -1.51578319,  -0.410395563,  0.321633875,  0.470479041,  -1.03425825,  -0.228603929,  0.436842442,  -0.184291154,  1.15954506,  -1.00642884,  0.665187895,  -0.174076438,  -0.831668317,  0.196616337,  -0.313174278,  -0.905146062,  -0.708966613,  -1.46342659,  -1.33950341,  -0.00683075469,  -0.128108472,  0.211339965,  -1.41336393},{ 1.68418407,  -0.125146851,  0.459878474,  2.85745931,  1.64231062,  2.03617644,  1.07168126,  1.17294466,  0.494963229,  -0.000343338848,  0.192754552,  0.334865272,  -0.754245102,  -1.08858621,  -0.439278096,  0.138566419,  -0.403681397,  1.00228786,  0.903195858,  -1.53554273,  -1.40415227,  0.969293177,  -0.198891461,  0.0779243484,  0.202459738,  -0.848602235,  -0.333741039,  1.93894529,  0.567752063,  0.279907107,  0.629861832,  0.579062104,  0.862247705,  -1.06342852,  1.07306814,  -0.648546159,  -1.92575216,  1.66865754,  0.239677593,  0.107619859,  0.100683741,  -0.388659745,  -0.916948617,  -1.7908082,  0.752881646,  1.16716886,  -0.651600659,  0.0834299847,  0.457644641,  -0.219508678,  -0.949403524,  -0.54374963,  0.864589453,  0.784761548,  -0.971182108,  -0.939270079,  -1.28253078,  -0.142623037,  -1.42625141,  0.0590851903,  -1.53699565,  -1.22652102,  0.695515335,  1.6490674,  -0.436948955,  1.15086579,  0.589381039,  0.536415458,  -1.11417043,  -0.0230862927,  1.76051521,  -0.602954209,  1.23222375,  -0.721455872,  1.25058377,  0.0391401686,  -1.07345521,  -0.843967378},{ -0.446808487,  -1.6775378,  -0.455467939,  0.0194952823,  0.678754866,  0.91459018,  -0.53933847,  -0.671403825,  0.549099028,  0.8122859,  -0.665460289,  -0.221223086,  -1.0617739,  1.30290174,  0.506314754,  0.205130845,  -1.98687518,  1.19528008,  0.74690336,  1.05184531,  0.475880682,  -1.3567425,  -0.797347784,  -0.259495616,  -0.907256067,  1.92779636,  1.14839315,  -1.08971918,  1.38673759,  -0.856624722,  -0.534007668,  -0.0931050777,  -0.449009389,  0.70265764,  -0.268518239,  0.74207896,  0.679647207,  -1.1121577,  -0.443068683,  -0.666928828,  -0.674905598,  -0.29622668,  0.229958415,  0.228713945,  -0.470520616,  0.208810702,  -0.970388293,  -0.418110639,  0.166330084,  1.41271055,  -0.796266735,  -1.92563808,  -1.10419011,  0.872659147,  -0.0523249656,  -0.844534159,  1.33069623,  0.778177857,  0.0653806031,  -1.00966597,  -0.45317325,  -0.713380396,  0.170873672,  1.22039771,  -0.217896223,  -0.626078963,  -1.52254641,  0.370526135,  -0.508507729,  -1.2732172,  0.105063617,  1.10277212,  -0.458929896,  2.06104445,  -1.16151917,  1.42033613,  -0.704652011,  -0.0828118622},{ 1.06959629,  -0.800404429,  1.2664578,  -0.048624374,  0.979588985,  -0.51478225,  0.424461782,  0.321278304,  -0.851555943,  1.29678905,  0.486746877,  -1.42957497,  0.119483009,  -0.520366013,  -0.650815606,  -1.74114645,  -1.38882661,  -1.09707677,  -0.545011461,  1.15444636,  1.29203534,  -0.986825049,  -1.29372346,  -1.38536263,  2.07123685,  -1.00290179,  -0.452685535,  -1.34251821,  -1.1893723,  -0.341392815,  -0.120582879,  1.05631256,  -0.929007232,  -0.0818829313,  1.14652169,  -0.164503351,  -0.439023733,  0.391167641,  -0.177463189,  0.443455786,  0.00382911693,  -0.486213803,  -0.805381358,  1.12523842,  -0.624085605,  1.1440798,  0.324677706,  0.363957256,  -0.259613097,  0.276550144,  -0.555268109,  0.204893574,  0.152930066,  -0.243134648,  0.402940869,  -1.47277987,  -1.19656491,  -0.663655162,  0.0667252168,  1.14546752,  0.232715845,  0.322517902,  0.0411584713,  0.580828249,  -0.790016413,  -1.52350354,  -0.239657894,  0.0963357016,  -1.36769748,  0.526205242,  -1.0880512,  -0.413510472,  -0.518005073,  -1.67940784,  -0.948678732,  1.04092586,  0.210212275,  1.48061836},{ 0.572272956,  -0.154621601,  -0.619193196,  0.964573324,  -0.600992024,  -0.739705443,  -0.157687649,  -1.27458382,  -2.11655951,  -0.599822283,  -1.48748994,  1.13651419,  -0.273024917,  1.03939402,  1.03852284,  -0.178572863,  0.308603197,  0.345939606,  -1.03539658,  -1.99223483,  -0.587691784,  1.84713805,  0.727220953,  -1.11013496,  -1.13466227,  -0.563351452,  -0.367282897,  -0.21578154,  1.45195091,  -0.351407081,  -0.917571604,  0.309597582,  1.26924551,  1.63279557,  -0.903229117,  1.10672677,  1.96705616,  -0.835826159,  1.79315388,  0.103407443,  -1.20744407,  -0.372188866,  -0.257464379,  -0.384744972,  -0.63487041,  -0.998473287,  0.457509279,  -1.43264794,  0.821830869,  -1.12991393,  0.745976329,  0.576878071,  1.49396813,  -0.595685661,  0.473228514,  -0.248099983,  1.36425078,  1.6798985,  -1.4090085,  -1.03468001,  0.195980206,  0.298334926,  -0.181376114,  -1.24045289,  -0.76218611,  -0.944217265,  -0.996408105,  1.74101341,  0.964756966,  1.2219969,  -1.17007995,  0.158597156,  -0.497828841,  -0.502963483,  0.977774501,  -0.366144687,  -0.193502575,  -0.144817308},{ -0.0720490292,  0.615771115,  0.698358774,  -0.229157373,  -1.62876463,  -1.0836376,  0.350626737,  -0.107978456,  -0.179964259,  -0.147257254,  0.678243101,  -0.751170635,  0.0070074033,  -1.23842359,  -2.21781731,  1.28883743,  0.300367594,  0.275286555,  -0.200116023,  0.800206125,  0.142187998,  -0.752829552,  -1.36587071,  0.768550575,  0.50611639,  -1.05258214,  1.11900353,  -0.481751025,  0.76154387,  1.11732042,  0.666565061,  0.0598076992,  -0.630028665,  -0.437274307,  -0.869881213,  -0.901869357,  0.90149796,  0.29789117,  -1.49656987,  -1.06220436,  1.37486136,  0.423333645,  -0.784705639,  -1.86235642,  -1.10510886,  -0.458117366,  0.249465331,  0.247134611,  -0.549209833,  0.391881257,  0.0758636147,  -1.74211693,  0.0285406932,  -0.808294475,  -0.771560133,  -0.800509214,  -0.181585521,  -0.166536763,  0.735362232,  -1.20778322,  0.453922093,  0.322306365,  -0.179653481,  -0.56141448,  -1.86198795,  -2.18415689,  -0.536550999,  -0.616373599,  0.201498419,  1.05325615,  -0.14043045,  -1.32352161,  0.208645269,  -0.691674709,  -1.93561387,  0.0896409303,  2.22213626,  -1.32262385},{ 1.77459395,  -0.436463058,  1.37002933,  2.62576413,  -0.473655462,  1.16237223,  0.152813897,  0.512531579,  -0.45191887,  0.348380744,  0.0729216486,  0.0435943604,  0.338705391,  0.154872626,  0.477981895,  -1.08734429,  1.66459906,  0.459416777,  -0.732809007,  -0.817401052,  -0.384330511,  0.316769421,  0.559288859,  0.342931241,  -1.36810064,  -1.18743062,  -0.750769198,  1.0326823,  -0.394356966,  0.454332769,  -0.251177222,  -0.831962109,  1.12599134,  0.290748984,  -1.3370887,  -0.495652974,  1.70761991,  -0.55958724,  0.16919823,  1.44192648,  -0.15012832,  -0.685878277,  0.742707431,  1.22333109,  -0.60273397,  1.32456076,  1.37145078,  -0.0979746357,  0.762559235,  -1.52406347,  -0.469380438,  0.396690756,  -0.0319047906,  -0.0105836131,  0.799668849,  1.31674623,  -1.04301298,  -0.580618143,  -1.61008501,  0.565629303,  -0.0252611414,  0.556910515,  -1.58538795,  1.56925881,  0.290815413,  0.557652116,  -0.924416423,  -1.22783411,  -0.164133713,  -1.32251239,  0.222784087,  0.230854809,  0.0718155652,  1.08653295,  0.802328587,  -1.04246306,  0.658479035,  1.70585942},{ 0.870446146,  -0.863546729,  0.57062161,  -0.895972311,  1.50264859,  1.73855627,  1.99741662,  0.932764471,  -0.42915985,  0.265237629,  -0.167881295,  0.543055177,  -1.72876978,  0.646306634,  0.108945958,  -1.30493927,  0.17392309,  0.14332892,  -1.41218495,  -1.84823859,  0.0392302573,  -0.0474189408,  0.679033458,  0.829964578,  0.342058361,  -0.186535358,  0.931733608,  0.671201348,  0.0445215665,  1.26828218,  1.68498874,  -0.186147541,  -0.926238358,  -1.41509449,  0.0753725246,  -0.0279393811,  0.799280345,  -1.31834447,  0.85079211,  -0.693597913,  -0.0278034844,  1.28651166,  -0.0747399107,  1.04469156,  -0.262684435,  -0.851465106,  0.583110869,  -2.67503357,  0.77548337,  1.32830834,  -0.323445797,  -0.400776714,  2.48557305,  -1.6074847,  -0.774903595,  0.694637179,  0.591185093,  -2.10899472,  -1.55974829,  0.740419686,  0.017239308,  -1.01118958,  1.1379292,  0.959368944,  0.509557247,  0.0863239989,  -0.911168277,  -0.101883993,  0.654183626,  -1.15869188,  -0.0803519115,  -0.534196079,  -1.0875771,  -0.564876378,  1.29285467,  -0.240078866,  -0.716287732,  -0.842971146},{ -0.805259407,  0.016808847,  -1.52968884,  -0.768770754,  0.03126866,  -0.656620264,  1.48494828,  1.25768328,  -0.547134459,  -0.315172821,  -0.114341848,  -1.18761575,  0.043596562,  -0.570997596,  0.445919335,  -2.04259658,  0.737979293,  -1.47331405,  2.03936458,  -0.829973161,  0.822603106,  1.05090153,  -0.0583234876,  1.70651245,  1.02645516,  -2.03216267,  1.43931854,  -0.931726277,  -0.321961105,  0.459583998,  0.507297158,  -0.0314020775,  0.0981395394,  -1.27118325,  1.85090768,  -0.206710026,  0.379953623,  1.85292566,  -0.189975321,  -0.829756737,  0.470268607,  -0.122291811,  1.00585675,  -0.0289875381,  -0.647482693,  0.301634461,  -0.239189133,  -0.523913264,  1.37171054,  -0.468723118,  -0.570541024,  -0.828677177,  0.19565165,  0.629904091,  -0.36550349,  -0.197154403,  -1.26431417,  1.38864684,  -0.36855641,  0.202676818,  -0.3867172,  -0.670791149,  -0.918996632,  1.76891065,  0.984885812,  -1.66391861,  0.558291078,  0.205335647,  -0.623608232,  -1.08079278,  -0.337719768,  -2.22124052,  -1.57873368,  -1.56789839,  0.0689123273,  0.341606826,  0.190524697,  0.406000346},{ -3.05581427,  -0.953995466,  0.256647259,  1.04100072,  -0.571338892,  -0.235053673,  -2.25387836,  -1.18135393,  0.664669991,  -0.343923032,  0.599961162,  -0.971308827,  -0.196942374,  -0.5011186,  -0.670915604,  0.0237774011,  0.345427573,  -0.00261246832,  0.185998946,  -1.42319393,  -1.37465739,  0.152051225,  1.32548273,  -1.8504076,  0.18617706,  1.33815384,  -0.535579443,  2.83615971,  0.346287787,  1.48556936,  0.724773526,  0.829375029,  -0.921200275,  -2.25540781,  0.213939831,  -0.915437639,  -0.359944552,  -0.160875678,  1.58646047,  -2.10022283,  1.20587385,  -1.09891319,  -1.14227498,  -1.04914236,  -1.1930964,  1.06827116,  0.367606848,  -0.40961948,  0.436276734,  0.212632924,  0.570543587,  1.42355502,  -0.205937028,  -1.76550972,  1.2576524,  1.24235964,  -0.102447852,  -0.628502846,  2.39651871,  -0.244182333,  0.330284894,  0.484069228,  0.362965584,  2.07024097,  1.0083462,  -0.71800673,  -0.873307109,  0.898015082,  1.07353258,  0.111590788,  -1.0954926,  -0.702582955,  -0.168801069,  1.8721838,  -0.769197404,  0.979728162,  -0.909105837,  1.46961629},{ -0.119371399,  1.76086998,  0.230336577,  0.427416325,  1.85929048,  1.35907423,  0.997190833,  -0.11844945,  0.177577659,  1.3114773,  -0.37703228,  -0.496313691,  -0.13533096,  -0.690414965,  0.0607822053,  -0.814491987,  0.509013951,  -2.29060721,  -0.0725656599,  0.0263314992,  1.22506869,  -0.3008385,  0.239531964,  0.78969723,  -0.0316530094,  -0.208986133,  -1.45372975,  -1.85318327,  1.04597676,  0.225993216,  -1.2207166,  -0.46025449,  -0.3276788,  0.730796099,  -0.414620161,  -0.121291488,  -1.61560237,  0.477787405,  -0.421997845,  -1.26159346,  0.705610037,  0.104842559,  0.981452882,  2.71778631,  -1.54495299,  -0.330830485,  0.244693086,  2.07808805,  -1.17488241,  -0.266643345,  -1.08365333,  0.283629626,  1.18001139,  0.444523722,  -0.944941401,  1.05979335,  0.0487062335,  0.611154616,  0.169707045,  0.849217534,  0.249291748,  -0.55132103,  -0.0855937898,  -0.987446249,  0.0716814995,  -0.309681386,  0.450024545,  0.822433829,  -0.0642326102,  -1.29706156,  -1.17222786,  -1.81438851,  -1.40385509,  -1.41832089,  -1.24461782,  1.4523896,  -1.15779829,  -0.716790676},{ -1.7776897,  1.52811348,  -0.682765424,  -1.28842807,  0.41055131,  1.12194765,  0.930691063,  -0.0679107979,  0.684631228,  -1.38593507,  -0.109331183,  0.68188113,  1.30559468,  -0.532076657,  -0.746256173,  0.207139865,  0.395421624,  0.837296546,  0.216873124,  -0.441476613,  1.61625946,  -0.58509022,  0.761884749,  0.787625492,  0.455669016,  2.20256853,  0.121922314,  -0.941246152,  -0.439374864,  0.408824742,  0.407604307,  1.06259167,  -1.15973032,  -1.42252612,  1.10873401,  -0.384298086,  -1.2050662,  0.541666627,  -1.07649553,  -0.428236961,  0.126492426,  -1.49254394,  -0.0347510353,  -0.809560478,  -0.0961520746,  -0.0360587202,  -0.672139049,  0.0498553663,  -1.57865965,  -0.745677829,  -0.638784885,  -0.483116329,  1.34273922,  -1.28634107,  -0.880159438,  -0.371072173,  -0.0804077759,  0.498536706,  0.83020252,  -1.21739066,  0.350077391,  -0.557900429,  2.01660132,  -1.57244468,  0.784672678,  -0.630080462,  0.255102992,  0.445726603,  0.478946745,  0.39048335,  0.362188727,  -1.4077878,  -0.607240796,  -0.00362650934,  -0.0142963501,  -0.920331597,  -0.309003919,  -0.27462709},{ -0.123389058,  -1.49891126,  0.89259094,  -0.159336552,  0.558460057,  0.790932059,  0.357384771,  -1.19482219,  0.924299896,  -0.709946573,  -0.62467885,  0.541875184,  -0.404584527,  0.570878863,  0.773190498,  -1.75678718,  0.839414656,  0.915457845,  -0.879359663,  0.305976152,  -0.0957482383,  -1.39680123,  0.832449377,  0.355213672,  0.80815655,  0.158339888,  -0.539690793,  -1.67112255,  -0.791844487,  0.787120879,  -0.590936661,  -1.09958744,  0.955563068,  0.346906185,  -1.04556882,  1.55300915,  -1.34085584,  -1.12217712,  2.13754821,  0.155091181,  0.497122854,  -0.406535089,  1.73966014,  0.360907674,  1.08426499,  -0.546889544,  0.842826366,  -0.402572721,  0.188779056,  -0.262078226,  0.00509199873,  0.993277848,  -0.425974429,  0.887900412,  1.0361135,  -0.451727837,  0.44399339,  -1.36610723,  -1.01108158,  0.268125236,  1.44726396,  0.395945072,  2.3446691,  -0.844015956,  -0.615191221,  -1.07658041,  -0.2610704,  -0.104231186,  0.663442254,  0.462688982,  -0.456805438,  0.735861421,  0.897132874,  0.395644039,  0.62954694,  -0.658334255,  0.101505771,  0.321218073},{ -1.70556152,  -0.129768208,  -1.00878048,  -1.28031528,  -0.248969361,  -0.833500087,  0.706094086,  1.43387461,  2.05780673,  -0.0459802859,  0.642523944,  0.392168671,  -2.04701757,  1.7145524,  -0.280657798,  1.07413292,  -2.18883276,  -0.399150997,  0.872161567,  0.227883011,  0.622164488,  -0.0561461896,  0.298924685,  -1.89015186,  0.429986924,  2.01646423,  1.02490258,  -0.889580846,  0.288330942,  -0.898463488,  0.00842362829,  0.387767076,  0.0709421337,  -0.803379536,  0.0851264894,  -0.804540396,  -0.986494899,  -0.585606754,  -0.620892048,  0.609334946,  -0.548647285,  -0.483046323,  -0.582313895,  0.0127893463,  -0.0342956409,  1.63810408,  1.23810673,  0.491899133,  -0.909966528,  0.793624938,  -1.77049887,  0.112524159,  -0.118128613,  0.148812711,  0.872993231,  1.11693454,  0.540247679,  1.14287031,  0.0434527732,  0.159528568,  -1.34657228,  1.43085158,  -0.0196182542,  -1.50452757,  -0.800596952,  1.107867,  0.615416825,  -1.18130422,  1.43950164,  -1.75369859,  0.312162846,  -0.447605342,  -1.47766781,  -2.48216176,  -0.39505738,  -0.182831913,  -0.158044249,  -1.15528381},{ -1.86366975,  -0.974650741,  -0.415655196,  1.50585628,  -0.527306259,  0.413617045,  0.304127306,  -0.100848369,  1.52500641,  -0.383249193,  -0.759939849,  0.833619773,  0.722601712,  0.50938642,  -1.09019589,  -0.84858644,  0.403860331,  -0.324196845,  1.10779119,  -1.05478036,  -0.657420397,  -0.417968571,  0.172270775,  -1.74492729,  -0.747889519,  -1.34059262,  0.624715805,  0.365247548,  0.0515045822,  0.337426126,  -0.161689013,  -0.490464002,  -1.05569553,  -0.462633103,  1.3830905,  0.0433177501,  0.347794473,  -0.171282962,  -1.60634398,  0.115509592,  -0.627836764,  0.204092279,  0.449837863,  -0.427386492,  -0.125034973,  0.916690648,  -1.41960585,  -0.713043451,  -2.32870007,  1.66154897,  -0.026255494,  -1.38020051,  -1.76369858,  -0.609658718,  -0.446635693,  0.93367213,  1.35764325,  -0.245314971,  2.67755437,  -0.578757286,  -0.247744277,  0.00232228055,  1.15361667,  0.355383396,  1.08934879,  -1.41031063,  0.639842629,  1.10834444,  -0.54691726,  -0.48251912,  -0.726952851,  0.196557269,  -0.68008846,  -0.14527142,  -0.119740501,  -0.115657598,  0.36711508,  -1.30852282},{ -0.279000938,  0.376341134,  0.827523887,  -0.559298158,  0.236962676,  -0.379475594,  -0.810862899,  1.74543631,  -1.82360578,  -2.2259531,  1.82054496,  -1.30692303,  -1.38520789,  -1.42340291,  -0.481833339,  -0.0326492563,  0.625194848,  1.60047936,  -1.55790198,  1.33867216,  -0.39565742,  -0.887501717,  1.25148904,  -0.347653121,  -0.517413795,  -1.58363295,  0.955474496,  0.164643943,  0.483492255,  0.840863824,  2.39987659,  0.00628522085,  -0.49296084,  0.688849509,  -1.3319546,  -0.803866982,  -1.27488792,  -1.77051377,  -0.366218865,  -0.885724068,  0.499879569,  0.17391026,  1.44753563,  0.188102588,  0.676150024,  0.401850939,  0.346051782,  -0.575890243,  1.75979698,  2.08989954,  -0.537724912,  0.416268289,  1.54549539,  0.545750082,  -0.330772489,  1.1273644,  -0.911345243,  0.516362906,  1.42156982,  -1.4092567,  -1.56508231,  0.492534578,  -0.549406648,  -0.972801805,  -0.590667248,  0.678543508,  0.267787874,  2.01072049,  0.705133736,  -0.12554051,  -0.847874403,  1.95259595,  -0.127391398,  1.02611208,  -0.979875207,  0.57195121,  -0.96771121,  0.380062044},{ 1.21208084,  0.471071899,  -1.44664562,  0.352850735,  0.944010794,  -1.34126735,  0.136149019,  -0.0919583738,  -1.35050011,  -0.327068657,  -0.382561445,  1.88857257,  -2.20313668,  0.156512544,  0.429950565,  0.276434511,  0.521268189,  0.793910623,  0.498679549,  -1.35895681,  -0.429859251,  -0.273311943,  -0.216662005,  1.05688035,  0.922638237,  -0.855194747,  1.06701839,  0.444941014,  0.522612154,  1.45312619,  -0.345154077,  0.112782761,  0.982238591,  0.829415262,  -1.38187504,  -0.748334169,  0.38523075,  0.983836412,  -1.47270596,  -0.0983786061,  0.824152172,  -0.177821562,  -0.404420763,  -1.10626209,  0.0112443743,  -0.722935855,  -0.308840722,  -0.405567616,  -0.221804485,  -2.56059384,  1.52645576,  -0.0464448258,  -0.183200017,  0.0212350115,  -0.851782143,  -0.213257059,  1.19695735,  -0.596387148,  -1.37238312,  0.175897241,  -0.995692194,  0.0645751879,  -0.849681616,  1.96387172,  0.897345662,  -0.168670356,  -1.79706132,  -0.536413848,  -1.07338893,  0.972590446,  2.14344478,  -0.219763771,  0.371542454,  -0.471838444,  0.787056029,  -0.254472435,  -2.03108501,  -0.28619042},{ 0.464862347,  1.01612461,  -0.63591063,  0.504833996,  -0.0874104202,  -0.0206619222,  -0.0979116634,  -0.0775262564,  0.0170478113,  -1.19658387,  -1.83313107,  -1.17852259,  -0.302293688,  0.916649282,  0.178477854,  0.998162985,  0.135224685,  1.01453829,  1.31332695,  -0.043469701,  0.740334928,  0.078854464,  -0.870879114,  0.29589659,  -0.534333825,  -0.26390481,  -2.47680497,  0.452681005,  1.17572069,  0.132729188,  0.0684042498,  1.09658039,  -0.793801963,  0.0592372455,  0.456313252,  0.012093992,  -0.593665361,  0.558039427,  -0.414749205,  -0.489489824,  -1.0399791,  -0.947369158,  -0.827193439,  1.10391378,  0.557362318,  -0.408210546,  1.37438166,  0.745852232,  -0.0343147293,  0.428963006,  0.513121247,  1.80960989,  0.745111704,  -1.1309427,  -0.973162234,  1.39576757,  -0.134357363,  0.400785029,  0.51571244,  0.294627249,  0.9041664,  -1.72093093,  -0.250632226,  -0.425777763,  -0.199029654,  -0.674671471,  1.99973834,  1.98450804,  0.279838026,  0.735821068,  1.12292993,  1.13160861,  -0.688297272,  0.101771481,  -0.355949014,  0.686713219,  0.150083318,  -1.08753264},{ 1.01970589,  0.373714477,  1.79425585,  -0.563147187,  0.560333312,  -0.542019904,  1.32239962,  -1.71641409,  -0.791100621,  0.244157627,  -1.07697785,  1.77686751,  -0.802410126,  0.117023468,  -1.88609838,  0.13282086,  0.312797755,  -1.41913068,  0.164054766,  -0.0334442332,  0.228222698,  -0.223031595,  -0.215543091,  0.865408719,  0.739634037,  -0.297842115,  -1.21946406,  -2.28682494,  -0.0645198971,  1.40915966,  -0.26379323,  -1.0167712,  -0.465535641,  -0.222957596,  -1.28537154,  0.444022238,  1.57440269,  -0.277679414,  0.0301284865,  -0.558142126,  -0.218224734,  1.46494973,  -0.240486875,  0.847766757,  -0.0813934878,  -0.442091405,  0.904666603,  -0.236168429,  -0.476756424,  -0.287525684,  -1.04466844,  -1.00243938,  -1.41241634,  0.485613704,  -1.65059114,  -0.0756695494,  0.50358665,  -0.531392813,  0.64026767,  1.67668259,  0.260775059,  0.741727948,  0.264243931,  0.450247109,  0.987094998,  1.66395783,  0.356212139,  -0.59317565,  -0.276197284,  -0.38330546,  1.15460813,  1.62318242,  0.349861741,  -0.0833358765,  0.0977690965,  2.41243792,  -0.527343392,  -0.612806618},{ -0.374616444,  -0.253152341,  -0.949812829,  -0.953527451,  -0.0236916561,  0.859822214,  -0.569449186,  0.0880018175,  1.28555346,  0.698876321,  0.18325977,  0.154902712,  0.190804392,  -0.346186757,  -0.160569444,  1.78166711,  -0.0165561829,  -0.278239101,  -1.81650364,  0.864486635,  0.798152626,  0.249609739,  -1.03325701,  0.0323469453,  -0.94365871,  -0.113420039,  -0.673572481,  -0.256920487,  -0.462038219,  -1.47183943,  0.0627433658,  -0.0742451325,  -0.703399658,  -0.318594158,  -1.58223641,  0.131290406,  -1.1206063,  0.0867394432,  -1.03831947,  0.757662892,  0.892904818,  3.57800364,  0.243422538,  -0.048025202,  0.262215197,  0.157892689,  0.686175048,  -2.13338017,  0.193046123,  -1.57048464,  0.960891902,  -2.08913827,  -0.799437642,  0.34560439,  -0.429969579,  -1.03490269,  0.232641861,  -1.06149828,  0.717396319,  -0.872998059,  -0.398436517,  -1.37424755,  0.953729987,  1.09028876,  -1.57339621,  -1.08989429,  0.432709873,  1.06827462,  2.20253301,  -0.0756204873,  -0.868544519,  -1.03857243,  0.597559631,  -1.74145615,  -0.769297957,  -0.0458210036,  -1.78850281,  -0.312672079},{ 1.33699298,  -0.761840582,  -0.998373687,  -0.533065319,  -1.77208185,  -0.421668082,  0.599447668,  -0.977151513,  0.485438704,  -2.24470234,  0.790401578,  -2.05197692,  0.497688085,  2.57065892,  -0.116507284,  -0.123662136,  2.48359299,  -1.13114178,  -0.590657949,  0.368437082,  0.544744372,  -0.600765169,  -0.675157547,  0.12928316,  0.0357210748,  -0.414160848,  -0.163615361,  0.632344425,  -1.43087542,  0.956151545,  1.48105824,  0.205979556,  0.836572111,  -1.26247489,  -0.877551079,  1.41773701,  -1.28674245,  -0.0421523191,  1.82754898,  -0.779737353,  0.0102214394,  2.0006628,  -0.574588478,  0.647806108,  0.438179225,  -0.164751023,  0.968828678,  0.65383333,  -0.60715872,  -2.43547535,  0.915791571,  0.231319502,  1.56769371,  -0.268011272,  1.32694089,  0.429815561,  0.700785935,  1.07773447,  1.07713926,  0.494483858,  -1.44874465,  1.2731421,  1.95496058,  0.798897088,  -0.933775365,  -2.38216114,  -0.227638721,  -0.259466529,  0.907509744,  1.07393038,  -2.17120361,  -0.791362286,  -0.448572397,  -0.00150542939,  -1.78774619,  -0.81617099,  0.121320583,  -0.476694047},{ -0.380174875,  1.0559057,  0.461336046,  1.75991476,  -0.403009921,  0.585134089,  -0.696848989,  1.52632642,  -1.25507164,  -0.40660736,  -0.811600029,  -0.0939288363,  -3.43751025,  -0.274200082,  -0.172640651,  -0.136556849,  1.31243837,  -0.326310337,  1.95108414,  0.0696191937,  0.509194791,  0.297944576,  -0.358169675,  -0.788278401,  0.280395269,  -0.209302574,  -0.843073905,  -0.184645191,  -0.627249837,  -0.725803077,  0.620812654,  -0.704958081,  1.72682083,  -1.92863047,  0.34011215,  -0.436967969,  -0.853887916,  -1.18538117,  -0.704745591,  0.765808642,  -0.277862817,  0.440734357,  -0.525416553,  -0.0277349316,  0.378439754,  0.470952272,  0.654394329,  0.629840136,  2.17402887,  0.585000694,  -0.522841215,  0.192825854,  0.0549989678,  0.194771215,  0.65704447,  0.784517407,  0.291501999,  0.529292285,  -2.6922617,  0.907282114,  0.166180298,  1.0390892,  0.0549216755,  0.398798525,  1.54170322,  1.69979692,  1.18582106,  -0.287538141,  -1.64027941,  1.16099072,  -2.10778522,  1.33485579,  0.384784609,  0.860338569,  1.2321068,  -0.895851851,  0.640007794,  -0.804305255},{ -0.928157866,  -2.07488561,  0.631426752,  0.29872787,  1.29922879,  -0.416905493,  -0.185764939,  0.273616731,  0.645766377,  0.79973954,  0.465341657,  0.939752102,  0.18533285,  -0.562804043,  -1.0136199,  -0.363064766,  -0.0487132147,  1.03530824,  0.484134674,  -1.21351182,  -0.112996407,  0.277736336,  1.89275801,  -0.813254893,  -0.465959758,  -1.48446608,  1.5620873,  0.797368586,  0.328750551,  -1.03016853,  -1.52212381,  0.3097305,  1.0426147,  -0.849125743,  0.245901138,  0.486852467,  0.920892537,  -0.798118412,  -0.546037197,  -1.04628825,  -2.03419423,  -1.67087018,  -2.08344698,  -0.477861375,  -0.239096507,  1.76980948,  -3.05247808,  1.47617054,  2.05114794,  0.115529612,  0.314365923,  1.74109805,  0.676547647,  0.370893687,  -0.207281023,  1.09042227,  2.20603466,  0.833610117,  -0.0909091458,  1.61503613,  1.03832901,  0.199359745,  0.456465334,  0.877668142,  1.30145633,  1.29020596,  -0.152946517,  -0.426836789,  0.403291225,  0.0836466849,  0.505880594,  0.434869587,  -0.223596871,  0.584426105,  -0.285464495,  -0.386875808,  0.14326176,  0.625022769},{ -0.177615091,  -1.00088179,  -1.72592282,  0.629429638,  1.40687799,  -0.131222621,  0.947024286,  0.793373525,  -1.28980589,  -1.06476188,  -0.0307908636,  0.102154121,  0.525999188,  -0.186877146,  0.160630003,  1.67020202,  -0.329554409,  -0.146684811,  1.91306627,  0.86637485,  -0.201729313,  -1.27784574,  -0.996987641,  1.28082418,  -1.26741445,  0.687909663,  1.2752322,  1.01991737,  0.214896888,  0.518088758,  -0.746379495,  -1.14268029,  0.474937141,  0.77038008,  -0.315742463,  -1.27773511,  -0.218608201,  -0.494443655,  0.874099791,  -0.727681816,  1.09168506,  1.53622317,  0.766664386,  0.616723835,  0.836860895,  0.0956141949,  0.181904674,  0.161104783,  1.20016968,  0.257367522,  0.475968152,  0.546218872,  -0.539532185,  1.803509,  -0.0751428679,  0.493082732,  0.34357357,  1.04414344,  -0.715516627,  -1.5910666,  2.2225287,  1.27335739,  -1.017295,  -0.0650495812,  1.46082115,  1.16609538,  -0.39574033,  -1.92382228,  1.51140428,  1.54474735,  -0.980779707,  -1.59697354,  0.953228354,  -0.909591973,  -0.446637452,  -0.680252135,  0.438446432,  -1.06696248},{ -0.523389757,  1.4859376,  1.04699266,  1.58781779,  0.765243471,  1.75945044,  -0.459195256,  0.290464252,  0.724683464,  -1.29247117,  -1.57726634,  -0.912199974,  -1.0937289,  0.895829022,  1.24494112,  -0.794511497,  -0.93204838,  -0.986421585,  -0.547531724,  -1.18755174,  0.61995846,  -1.1000731,  0.0465152487,  0.0128563726,  0.349012285,  0.289475471,  1.61410153,  -0.793435991,  0.273375571,  0.384971142,  0.0112798633,  -0.54840821,  -1.63691306,  -2.3568635,  -0.401654154,  0.612276256,  0.39776504,  -0.408088595,  0.112809248,  0.104912378,  0.727226436,  -1.18526757,  1.02962434,  1.11897755,  -1.67445922,  -0.988053322,  -2.62756729,  0.851294935,  -0.565640092,  -1.0254724,  -1.10276544,  0.998375416,  1.48007512,  0.378268033,  -0.749378979,  1.13523209,  -1.2052629,  0.438908875,  1.58960974,  0.265501738,  -2.54028821,  -0.692288458,  1.06377101,  1.16156411,  -0.49315694,  0.595073521,  0.0926079303,  -0.0535859764,  -0.318955332,  -0.950598061,  0.781454444,  0.342305332,  -0.725706697,  0.484270781,  -1.09167278,  0.676405549,  -0.777595639,  -0.815775692},{ 0.504088819,  -0.122472607,  1.29162884,  0.374665231,  -1.22691667,  0.701536596,  -1.44600725,  0.21487312,  0.0130734425,  -1.42542362,  -0.232177168,  -0.0313400067,  0.642155886,  0.364909053,  -0.39452076,  -1.5751003,  0.691770613,  0.779149234,  0.159014195,  -2.06931543,  -1.24599528,  -0.548585534,  -1.13125789,  -1.02498972,  -0.785765707,  0.207727447,  -0.680798292,  -0.883600712,  1.04088259,  0.660431683,  0.2342062,  1.38986301,  0.840049684,  -2.85404444,  0.169422358,  0.106248848,  0.428423971,  0.76962924,  -0.107213326,  0.454699546,  0.793371141,  1.0140872,  -1.73655748,  -0.923684895,  -2.21172333,  -0.887238026,  -0.815156162,  0.0148065621,  -0.451497674,  -0.106481917,  -0.308723032,  0.477520287,  -1.16253614,  0.329245061,  -0.79555279,  -0.0209575947,  0.703081906,  -0.37832287,  -0.546635926,  1.92580855,  -0.365281701,  1.89462352,  -0.223008335,  0.833673179,  -1.34671175,  -0.260083944,  -1.41587555,  -1.3034482,  -0.665124357,  -0.290156901,  0.254321128,  0.821975112,  -2.47092628,  -0.796914339,  0.0786888301,  -0.488087744,  1.7053498,  1.09072196},{ -0.664935052,  0.893277764,  0.318829417,  -1.17644882,  1.6892674,  -0.563257337,  0.964108527,  3.56111741,  -1.38233674,  -0.942712128,  0.348765433,  -1.01645148,  0.929077804,  0.250650555,  -0.375780493,  0.427170157,  -0.372372895,  0.361112118,  -0.211860508,  -0.667531431,  1.03449845,  -1.68090081,  -0.742430508,  -0.551015198,  -0.32200104,  -0.16621992,  -1.02563572,  -0.457839876,  -0.733379364,  0.754230738,  -0.318867415,  0.444292665,  -1.06242537,  0.0303478986,  0.773379147,  -1.1055454,  -0.849717557,  1.0407083,  0.74434787,  -0.997492909,  -0.714009345,  0.785457253,  0.455837667,  0.832826555,  2.00371194,  -0.349979609,  -0.281065762,  -1.06871831,  0.662184775,  -0.767670155,  0.318913549,  -0.43391788,  -0.493536949,  0.361149073,  -0.38359493,  0.318046898,  -0.102326319,  1.15779519,  -0.642579079,  -1.22117913,  0.684399664,  -1.58441627,  0.892113864,  -0.887656629,  -0.392659366,  0.756093442,  0.498354495,  0.399499893,  1.62968063,  0.259691089,  1.16162884,  -0.730464578,  1.33202708,  2.86833262,  0.720015943,  -0.606881082,  -0.483281583,  0.28313157},{ -2.00400448,  -0.199190632,  -0.478388906,  -0.129535437,  0.0359177403,  0.0708184615,  0.262006134,  0.301700413,  -0.928468227,  -1.45763648,  -0.254791737,  -1.00327206,  -0.287997961,  0.349688262,  0.468065739,  -0.977319121,  -0.986369073,  1.63007486,  -0.750743806,  -0.96741724,  -0.0943525657,  -0.170395434,  -0.188714325,  -0.860846937,  -0.292213082,  -0.657148302,  0.00426134188,  -0.859891593,  -1.07095206,  0.509856641,  0.643235266,  0.758845687,  0.348495305,  -0.720908523,  -1.06375766,  2.50573945,  -1.84928584,  0.00341877481,  -1.77708614,  1.52149916,  1.19232678,  0.00891781319,  -0.133287892,  -0.681845725,  -0.250651866,  -1.09139717,  -0.503074765,  0.0656444579,  1.84348571,  -0.494889766,  -1.302297,  -0.623663485,  -0.514573932,  -0.633962452,  -0.0965217724,  0.489498734,  0.00489912229,  0.683903277,  -1.35256684,  0.330406249,  1.11406314,  1.55097198,  0.0880980119,  0.759231567,  2.47788405,  -2.41278458,  -0.991263211,  -2.3172462,  0.632684588,  0.456294775,  0.705216587,  0.663698196,  -0.804853916,  -0.406643331,  -0.213403583,  0.695468664,  -0.89111197,  2.11207819},{ -0.0183688998,  0.112900853,  0.46759823,  0.685030222,  0.318038195,  -0.29675433,  0.614579678,  -0.199079871,  -0.490188569,  0.235741824,  -0.213053405,  0.748377979,  0.37444523,  0.849149823,  0.964112699,  1.73754096,  0.239322886,  1.18231344,  0.797792971,  -0.645115614,  -0.777016282,  0.0204457343,  -0.910779417,  -0.167786643,  -2.49870682,  1.92110527,  -0.274097353,  -0.509514153,  0.325005472,  0.0431950688,  -1.05281723,  -1.36111093,  1.22025788,  0.780274153,  0.236147106,  0.978923261,  -2.83084989,  1.41435313,  -1.3064456,  -0.553990483,  -2.0313077,  -0.148812354,  0.880921185,  -1.61931384,  -2.18839455,  -0.995415807,  -0.503591001,  0.326128721,  -0.927111983,  -0.297001749,  -0.197377682,  -1.4154464,  1.14118004,  -0.197091386,  1.11857998,  0.308610171,  -0.445473611,  -0.506184042,  -1.14959204,  -1.94357562,  -1.25989461,  0.128471911,  -2.44293308,  0.925638199,  -1.23607826,  -1.21455753,  0.539747119,  -0.655098617,  1.31969738,  1.88389194,  0.198343962,  1.28740978,  1.40192652,  0.104658611,  0.581868827,  0.531682253,  -0.547002852,  -1.25187767},{ -0.502268493,  0.301908463,  1.12746906,  1.53540599,  0.0416036546,  -1.825302,  0.672463298,  1.1458832,  -0.33711049,  1.51039374,  -1.50853908,  -1.11077607,  2.24185848,  -0.119060025,  0.926100969,  -1.08137822,  0.591702223,  -1.53427184,  1.01116371,  1.3338418,  -1.71268141,  0.674205422,  0.165533528,  0.501679182,  0.21441555,  -1.34906054,  -0.860972345,  -0.81994468,  0.557354271,  -0.143251643,  0.337875754,  0.548049927,  1.71902764,  -1.12728393,  -0.589271963,  0.0488860905,  1.04330623,  -1.29200792,  -1.1719259,  -1.23723245,  -0.595858276,  1.4018029,  0.471195042,  -0.466105402,  -0.847877145,  0.197894961,  0.888367832,  -0.153197899,  -0.191569284,  -0.602912366,  0.565984547,  0.473850429,  0.41803661,  -0.0832159817,  0.788050652,  1.41883636,  0.337167919,  0.513995528,  0.322098285,  0.0570469834,  -0.737307727,  -0.269271493,  1.8211602,  -0.16033344,  -0.863876045,  -1.86435759,  1.37724841,  1.67380512,  -1.12438583,  1.16648674,  -1.06901109,  -0.625020146,  -0.233315259,  -0.572249174,  -0.947824657,  -0.175562575,  -0.59008342,  -0.819433689},{ -0.194713667,  -0.262400508,  0.215015694,  -1.05006385,  0.0330590382,  1.33945155,  0.232368097,  0.398252308,  0.89795512,  -1.7245481,  0.616201699,  -0.240186334,  -0.0506100021,  -0.254391938,  0.447301,  -0.158572495,  -0.935706615,  -1.8111645,  -2.42005897,  0.842494369,  0.269621849,  0.431730628,  -0.0846633837,  -0.425792366,  -0.454982579,  -1.13443649,  1.713256,  0.196519554,  0.968358457,  -0.319097638,  1.7746557,  -0.788239598,  0.248007745,  -1.91590488,  -1.58003116,  -0.443422079,  -0.783887148,  0.0295670461,  0.0784541443,  -0.393202543,  0.0323320739,  0.857914984,  1.37968612,  -1.27469432,  0.929212749,  0.769504488,  -0.370168805,  -1.13940728,  0.538929582,  0.295026541,  0.0826685503,  -0.180934235,  -0.428124398,  -0.210530952,  0.103805952,  -0.795371056,  1.60056973,  -0.905861378,  0.120396666,  1.10188198,  1.98809564,  -1.31795609,  1.17122829,  0.153065279,  -2.20438266,  -0.264855891,  1.16398501,  -0.0451003574,  -0.562622726,  0.45840627,  0.493686199,  -0.922268808,  -0.0999673828,  -0.111348204,  0.0538554862,  0.661684275,  1.95053661,  3.05684233},{ -1.64699495,  -0.829567611,  0.977984965,  1.494403,  -0.191997454,  1.39942706,  -0.697413802,  0.804819763,  -0.77623713,  2.27625751,  0.867768645,  0.0634158924,  0.584143281,  0.73287034,  -0.0852069855,  -1.02070701,  0.738575757,  -0.0550557338,  0.909056544,  0.132842213,  -0.208148152,  -1.52066898,  1.12569726,  0.556337297,  0.519278169,  0.78371501,  -1.18913686,  0.465439826,  1.0404855,  0.630961835,  -1.08962595,  -0.316891313,  -1.51649868,  0.0413285829,  1.07149279,  -0.104182549,  1.12197661,  0.374066651,  0.127787128,  -0.770068467,  -1.07382572,  0.00834257156,  -0.721186638,  -0.155459374,  0.755275428,  -0.488723934,  -2.00579381,  -0.771265864,  -0.876375198,  1.39189947,  1.9359206,  0.773965478,  0.0120006632,  0.139288247,  -1.13961005,  -0.481208444,  -0.865341246,  0.229719386,  -0.00971949939,  -1.30708516,  -0.0266867802,  2.0928061,  0.443841428,  -0.099902764,  0.850001395,  0.626623988,  1.70576179,  -0.593951523,  0.961485684,  -0.203135163,  2.09263349,  0.989195049,  -0.724308133,  -2.75890446,  -0.532494724,  -1.13823414,  -1.89181137,  1.4078511},{ 1.40193927,  2.53209257,  0.109312922,  -0.588777363,  -2.41675067,  0.718505025,  -0.395622462,  -0.546091199,  -0.220378384,  1.30787408,  -1.29013121,  -0.361947179,  0.0684021562,  0.411870897,  1.92784476,  -0.609969378,  -1.80999792,  -0.80343473,  -0.612647474,  0.954085469,  0.538137794,  -1.39684165,  -0.788876355,  -0.277769178,  -0.386646301,  0.877096415,  1.13374305,  1.50060308,  0.185459495,  -0.317250043,  0.560627043,  1.31203783,  0.149917051,  0.401676804,  0.592877865,  -0.300645947,  -0.382786632,  -1.42758822,  -0.260985821,  -1.50424767,  -0.931744993,  -1.46524036,  -0.128627703,  -1.52581394,  0.434338361,  -1.74822736,  -0.419846177,  0.0372121893,  -0.33098799,  1.22439301,  -0.313345402,  -1.07820857,  2.16532516,  2.77611732,  0.0380031168,  -0.976655304,  -1.2701627,  -0.841364622,  0.738583207,  0.204731569,  -1.04867423,  0.0501272194,  0.145210519,  -0.146435171,  -1.51312697,  0.00477989204,  2.53942299,  1.5859971,  2.35710597,  0.622167706,  -0.656871855,  -0.870118737,  -1.61886799,  2.37750793,  -1.36405396,  0.891513705,  -0.0670247525,  1.55424654},{ -0.101241879,  0.417330235,  -0.422774643,  0.0793827623,  -0.156378627,  0.688846231,  -0.0490367971,  -1.15115583,  1.988289,  -0.995643258,  -1.20712066,  -0.571666181,  -0.135123998,  1.12969577,  -0.244397491,  0.58891809,  -0.295556158,  1.11434019,  -0.204963356,  1.43635225,  0.00653605629,  -0.965038836,  -0.633383095,  -0.390372723,  0.878125191,  -1.08211613,  1.09279644,  0.0114551522,  0.321133316,  -0.847884715,  0.175210252,  -1.08383179,  0.836502492,  -1.18718147,  -0.54591459,  -1.55949461,  -0.570888579,  0.733836591,  -0.0386010073,  1.11323988,  -1.32908094,  0.072534062,  1.0934695,  0.761970162,  0.214783981,  0.85978663,  -0.385501295,  -0.508977115,  -0.321184367,  -0.0371381715,  -1.10329676,  -1.83907175,  -0.624601841,  -1.40190566,  -0.186327294,  -1.43178427,  0.420359462,  0.663898706,  -0.453431338,  0.00312638632,  -0.723968208,  -0.833242357,  0.854309201,  -0.267197073,  1.30538249,  1.94571817,  0.677333474,  -0.502185583,  -0.944799483,  2.22969937,  -1.15699816,  -0.411764324,  1.57454419,  -1.11168897,  -0.167562574,  0.382351816,  -1.17986536,  -1.33495522},{ -0.51785934,  -1.79422927,  0.786922216,  -1.40565181,  -0.271358311,  0.578738987,  1.34754896,  1.02230227,  -1.56957316,  0.99112165,  1.77191699,  0.486436516,  -0.973988295,  1.76251936,  -1.32749271,  -0.902194083,  0.817239165,  -0.572399139,  -0.312879324,  -0.788512468,  -0.493608296,  -0.00350352982,  -0.51657629,  -2.19388366,  -0.965125799,  -1.46378136,  -0.287097633,  -2.19863749,  -0.68233496,  -0.552410364,  -1.3879056,  -0.962713778,  -0.990841925,  -0.267331302,  -0.261160105,  0.81427747,  1.29181647,  -0.148337215,  -1.34620619,  0.802533567,  -1.68863761,  0.0632321313,  -1.37725282,  0.299783379,  -1.33990073,  0.334050417,  0.327676743,  0.404936552,  -0.675876617,  0.372879744,  -0.540856779,  -0.0113929305,  1.00982523,  1.07947898,  -0.0362051427,  1.23250508,  -0.499855906,  -1.09227049,  -0.43110615,  0.743688524,  1.85103047,  -0.37202239,  0.0992130414,  2.48794723,  0.0512531549,  0.204985574,  0.920837283,  -0.277760118,  0.301851392,  1.56316912,  -0.146588638,  -0.19078742,  0.00602978887,  -0.126395956,  -0.0980559811,  -1.73943138,  -1.02746224,  -1.62379682},{ 0.616977453,  -0.0926934183,  -0.696244001,  0.86629355,  -0.283668905,  -0.0107926102,  0.482942879,  0.471753657,  -0.0336972512,  2.08824039,  1.00887537,  0.00660745334,  -0.825515449,  -0.474163115,  2.23657489,  0.238910854,  1.61734414,  -0.0657808259,  0.862298906,  -0.0394496731,  -0.59469229,  -0.241884336,  -0.370161682,  0.0958764404,  -0.238332182,  -0.262305081,  1.00913537,  -0.300315946,  0.0923470929,  0.745248318,  0.319522321,  -0.393092334,  -0.830785275,  0.30935654,  1.17243838,  -0.820154011,  -0.952004492,  0.421427608,  1.37896287,  -1.35464168,  -1.37461567,  -0.939990342,  0.143265277,  -0.189096838,  -0.393368781,  0.292850643,  -0.96817261,  -0.903444886,  -1.48102629,  1.28306603,  -2.3440907,  0.77847302,  0.0898011476,  0.366608113,  1.60601389,  -0.683645725,  -0.132936418,  0.536262333,  0.608888149,  -1.23478377,  0.369645208,  0.930490553,  -0.781722605,  0.937451303,  0.170991004,  -0.0629586205,  0.256764084,  -1.11315894,  0.18188332,  0.648129463,  1.70686412,  -0.145746201,  0.727922916,  0.175627246,  0.197868288,  0.708758533,  0.888426244,  0.952050626},{ -0.0919478312,  0.379992574,  0.961675525,  0.474412739,  0.293783128,  1.96972251,  0.0499733463,  -0.0353769511,  -0.919408739,  1.14162791,  -0.055718407,  0.336653113,  -0.328046083,  -0.263289839,  0.0334784873,  0.0709941238,  0.567621171,  0.751453817,  -0.89147985,  0.0306546297,  2.12613726,  0.304409295,  1.18005919,  -2.44803071,  0.325861931,  -0.408885121,  -0.455469161,  0.604600668,  1.25771439,  -0.924316466,  1.14505661,  -0.963276982,  0.806478322,  -0.552698314,  0.163427636,  -1.0660274,  -1.25059652,  -1.20229459,  1.22823393,  0.782983422,  -1.40418792,  -0.810718536,  0.0925887972,  -1.22950435,  1.21854722,  -1.23884869,  0.431800723,  1.39517152,  1.07657683,  0.189427301,  -0.843070924,  -0.279185176,  0.348384202,  -0.983874023,  2.9433434,  0.921301126,  1.69449258,  1.40339839,  0.348845631,  -0.394315988,  -0.487319708,  1.11733985,  -0.980873227,  1.0006212,  -0.792192221,  1.11161971,  -1.28374553,  0.40104419,  -0.582882106,  0.805379689,  0.752219379,  0.33682546,  -2.43206406,  0.285318673,  -2.04279089,  0.45126164,  -0.757630229,  -0.402748853},{ -1.05313599,  0.243278593,  -1.90083587,  0.779531062,  0.0968312323,  0.719846964,  -0.744936347,  0.584598184,  -0.150316134,  -0.944574893,  0.683226526,  1.23880231,  -0.0215465277,  -2.05995417,  -2.0034306,  0.333503067,  0.159535959,  -1.12891042,  -1.31453085,  0.22601223,  -0.0857175291,  0.234472916,  0.553167403,  0.0605924092,  0.0710343644,  0.265439123,  0.530539989,  -1.80916107,  0.389415264,  1.45715654,  -0.591185153,  0.715724707,  0.581198812,  0.413898975,  -1.81998181,  -0.539185286,  -0.999239266,  0.283852696,  -0.33977437,  -0.66122061,  -0.76914835,  -1.4750104,  -1.47359109,  -1.29021454,  0.319335282,  1.01397872,  -0.186933801,  1.23743451,  -0.369048744,  0.22985974,  0.443221986,  0.328644067,  0.162220076,  0.372754067,  -0.835069597,  1.26437485,  2.23464108,  0.324439734,  -0.0856603831,  0.750644088,  0.891896367,  2.61731887,  2.43117499,  0.252976894,  0.620789349,  1.17865396,  -1.7707603,  0.723971367,  -1.03315496,  1.55334353,  0.168372527,  1.41375411,  0.598814607,  1.56593025,  1.63251615,  1.1436677,  0.139365777,  -1.06632447},{ -0.745153904,  -0.0730271637,  0.273344129,  -0.335421354,  -0.510826409,  -1.59538567,  0.534498572,  -0.444979757,  0.409514666,  -0.263229012,  1.5379827,  1.30262709,  -1.28165042,  -0.01756013,  1.17620695,  0.798611403,  -0.365108997,  -0.356864601,  -0.424083322,  -1.80458105,  0.698334813,  -1.38117445,  0.494329125,  -1.39896095,  -0.0531776361,  0.453172743,  -0.982006073,  -2.10702324,  -0.813035369,  0.054823827,  -1.51117122,  -0.570960879,  -1.6301589,  -3.9055028,  0.489291131,  0.979317605,  0.00204529963,  -0.59777987,  -0.790290117,  -0.135541096,  0.256257504,  -0.574447215,  1.6712203,  -0.968065143,  0.326503664,  0.522540569,  -0.224440843,  0.342053115,  0.825948536,  -1.43364179,  0.435974747,  0.352377206,  0.0970752388,  1.39107382,  -0.127227128,  -0.625338972,  0.924737751,  1.20022106,  0.387565792,  -1.38452446,  -1.00171554,  -0.887265027,  2.0190165,  0.391316831,  1.71474624,  -0.743103325,  2.00156164,  0.666826189,  0.13379015,  0.661492586,  -0.910079896,  -1.08271432,  0.67812103,  0.557464302,  -0.775034547,  0.985048294,  -1.13798559,  0.639596164},{ -1.28530431,  0.598968923,  2.46819854,  0.240816563,  0.865696251,  1.2957958,  -0.733555734,  1.38420689,  -1.17251968,  1.10116994,  -0.206887126,  -2.3346777,  -1.13470089,  -0.343576282,  0.52694273,  -0.621419907,  0.217179626,  1.57700086,  -0.531518459,  -0.617098689,  0.0537010133,  0.0786992982,  -1.90844893,  1.17248547,  -2.11788106,  -2.31264734,  -0.253971547,  -1.66921198,  0.306624621,  -0.693704188,  1.53166556,  0.330123097,  -0.0651395842,  -1.50670016,  -0.755507588,  -0.299144328,  0.974669039,  1.3116895,  -1.75156367,  0.574791014,  -0.29448095,  0.11889641,  -0.45032391,  -2.14627504,  -1.23884535,  -0.650104821,  0.583532453,  0.581284106,  -0.160696536,  -1.08630884,  -0.17055589,  -0.251372427,  2.14596224,  -0.0365478769,  -0.210511327,  -0.797741413,  0.525934219,  -1.04916191,  1.51303267,  0.664243937,  0.56973362,  1.62066817,  0.480134755,  0.783165753,  -0.41570133,  -1.26060283,  -0.650143564,  2.25511432,  1.1061281,  0.348911464,  0.305530936,  0.850274384,  -1.30930996,  1.02611125,  0.609303296,  -0.205070779,  -0.906666934,  0.0514903106},{ 0.0190584157,  1.40918267,  -0.309120506,  -1.34761751,  1.47451496,  0.68306154,  -0.172862992,  -1.30689919,  -0.831101835,  0.311937153,  -0.76737386,  0.472090483,  -2.32374978,  0.565476596,  -0.65168798,  -0.715678751,  0.0359616205,  0.97666204,  -1.09631586,  -2.30047941,  -0.185151055,  -0.194040626,  -2.55125642,  -0.465853781,  2.39770246,  0.865861833,  0.520911217,  0.600793302,  2.8007648,  0.122579426,  -0.631602049,  1.11306226,  0.24313587,  -1.08093119,  1.7393707,  -1.70513213,  1.15449095,  0.0913717002,  -0.057964202,  2.12844419,  -0.413487494,  1.00113344,  0.668703914,  -0.500294328,  1.46878588,  0.235664263,  0.0599515438,  1.08089519,  -0.513387144,  -0.390902281,  -2.59810805,  1.19122601,  -0.531733692,  -0.432231992,  -0.276048839,  0.116757572,  0.189338192,  -0.141697064,  0.451529384,  -0.867699444,  -0.0824240148,  -1.53913581,  -1.31579852,  -0.0614645518,  -0.63102901,  0.706887722,  1.16910625,  0.920510948,  -0.318488091,  1.61583078,  1.44356883,  0.46393621,  0.0145344799,  -1.01807582,  1.86859047,  0.2922346,  0.0774493292,  0.904412568},{ -0.113227546,  1.15882051,  0.441215575,  -0.850202858,  0.320516795,  1.08074129,  0.681022704,  1.63384986,  0.236930653,  -0.484879047,  -1.79270208,  -1.20675874,  -0.37551412,  -3.08372808,  0.0393295698,  0.586417675,  0.196391165,  1.27782762,  -0.789436758,  0.912607074,  0.491716385,  -0.19926025,  -0.756349027,  -0.419373512,  0.430989534,  -0.387229592,  -0.282177716,  -0.381296247,  0.0942926556,  1.38592017,  -0.669143021,  1.16629446,  -0.709143877,  0.304773062,  0.766706944,  0.967902243,  -0.0900101513,  0.602927566,  -0.171748742,  -0.866111457,  -0.114710942,  0.0923802629,  0.193946034,  0.274697274,  -0.692489624,  0.364478707,  1.61826265,  -1.01596725,  -0.675958335,  0.50572139,  -0.756169379,  -1.12043822,  -1.5235585,  0.466878057,  -1.0628792,  -0.455337167,  0.715337157,  1.5022192,  -1.55689013,  -0.0802422762,  -0.281730026,  -0.0767883062,  -0.69750917,  -0.587291539,  0.72664535,  0.201770633,  0.524515271,  -0.0513717346,  -0.275387168,  -0.617403865,  1.67746866,  -0.423606724,  1.30133367,  0.771561742,  0.593330264,  0.0551589839,  -0.680388153,  1.72859788},{ -1.46442652,  -1.11802018,  0.804584682,  1.84721828,  0.642688036,  -0.254905552,  -0.860933661,  0.183586136,  -0.229199693,  -0.46549049,  -1.57000327,  0.75685662,  -0.213579923,  -1.28120792,  -0.379123449,  -0.583697259,  0.366315067,  -1.78768647,  -0.220214278,  0.198700458,  -0.078210026,  1.25413752,  2.52979803,  1.72620773,  -0.330591768,  -1.76104271,  -0.436508,  -0.651385903,  1.00211287,  1.15383101,  1.05261743,  0.436006576,  2.45059156,  -1.2478317,  -0.178453416,  0.796683013,  -1.68166196,  -0.24213177,  -0.431253344,  0.208392709,  1.50050664,  -0.460567415,  -0.129704878,  -0.713621497,  -1.37623954,  -0.104476005,  -0.97955513,  -0.884568334,  0.515467584,  0.0146958344,  0.123060085,  -1.7751919,  -0.437541276,  1.15649712,  -0.978172362,  -0.683907509,  0.318381488,  0.57576251,  -0.111699685,  0.646800101,  -0.871332288,  0.139482677,  1.79583263,  0.267569363,  -1.06559372,  0.992903888,  2.55494165,  -0.0018158399,  -1.93547547,  -0.00892276503,  -0.84566915,  0.698403776,  0.665477812,  -2.28061199,  -1.940781,  0.921309769,  2.03514481,  -1.2740227},{ 0.987269044,  -1.0757215,  -1.33295858,  0.223460838,  -0.954626739,  -1.98642063,  -1.79740024,  0.603796959,  -0.411949426,  -1.71975064,  0.465434074,  0.753945231,  1.23970127,  -0.114814885,  -0.730629504,  -1.14201951,  0.00715085072,  1.04754078,  -0.0816872269,  -0.148970425,  -0.550390601,  -0.898399949,  0.57042557,  -1.05073559,  0.175744489,  0.993399024,  -0.110397406,  -0.401302397,  0.504583657,  -1.4232868,  0.069499284,  0.774200737,  0.909029484,  -0.217272907,  -0.00734660868,  -0.433249205,  0.59350282,  0.929234147,  0.498376399,  -0.19235903,  -0.773612261,  0.843084335,  -0.391641825,  -0.638920128,  -0.29491654,  0.171729967,  -1.69327736,  1.79585826,  0.976348996,  0.0342549682,  0.718142033,  0.275114089,  -0.199042261,  -0.551514268,  0.0783208534,  -0.643680811,  -1.39175332,  -1.57924628,  1.53599131,  0.397323966,  -0.310745955,  -0.159483939,  1.28792334,  1.06134057,  -1.05557239,  1.43961096,  -0.355178416,  -0.550181925,  0.606126785,  0.94800818,  -0.214424223,  1.01931977,  1.78037906,  -0.965309024,  -1.04950488,  2.21406126,  -1.58158708,  -0.253127158},{ -2.05348301,  -1.52452278,  -0.469749868,  -0.533159673,  0.0785490051,  0.634610176,  -0.446085006,  -0.823855937,  0.659477234,  -0.891742647,  0.629398227,  0.20695582,  0.437633961,  -1.30243182,  0.145481899,  -0.821369171,  -0.690204859,  -0.0163680892,  -0.529487431,  0.524564445,  1.44542754,  -0.514587045,  1.17844462,  -0.604842961,  0.364419669,  1.12718308,  -0.617153883,  0.358485401,  -1.29084313,  -0.101411991,  -0.106623411,  -0.928275645,  -0.66866076,  0.380419612,  0.903758287,  1.43196797,  -0.320928842,  -0.400932729,  0.500094354,  -0.670602322,  -0.464422464,  -0.0453021303,  0.687688947,  0.680321395,  -0.0445496924,  -0.486198425,  -0.333482653,  0.175629303,  0.249318555,  1.16481113,  1.43275023,  0.417523891,  0.877955616,  -0.25173071,  -0.969449759,  -0.275892526,  0.0425407849,  -0.0945535153,  -1.39119542,  1.39559543,  1.65502417,  -0.296033233,  -0.466887146,  0.608139575,  -1.73807883,  0.385655761,  -0.236275941,  -0.349266976,  0.992257833,  0.849435329,  0.230638474,  1.72321999,  0.515890062,  1.02822733,  0.273791015,  -1.31260335,  -0.11031165,  -0.537251413},{ -2.02128506,  0.673143923,  0.300916284,  0.0967313573,  -1.23159885,  0.346731484,  0.472624928,  0.739904642,  -1.2769134,  -1.20017111,  -1.02916896,  1.33463776,  -0.0173739307,  1.13007295,  -1.26655114,  0.740583479,  2.31063414,  1.28507638,  -0.52499187,  1.01526749,  0.657792807,  1.51810431,  -2.38823819,  0.395890921,  -0.173792705,  0.672156453,  0.153849423,  -0.526240766,  0.553963125,  -0.625656068,  0.681330979,  -0.169488668,  0.594237506,  -0.780249059,  0.351187646,  0.646448374,  1.98608792,  0.803717196,  0.0397180319,  -0.0643719882,  -0.146845222,  -1.44536161,  0.210625857,  0.285964042,  0.145671412,  -0.0167408846,  -0.0224239547,  0.472006947,  0.707411885,  -0.149022385,  0.397298425,  0.740738809,  -0.312696218,  1.5801276,  0.427430749,  -0.284425735,  -0.600172341,  -1.64762235,  -0.00711434707,  -1.26103103,  0.100349925,  -0.629818082,  -1.10586262,  0.31718567,  0.798254073,  0.154424503,  0.358044058,  0.990882218,  -1.35976183,  -0.48515293,  0.0132259252,  -0.481792003,  2.52561474,  1.23260641,  1.87008536,  -0.00968644489,  -0.452894449,  1.47111022},{ 0.235440046,  -0.165420845,  0.204278201,  0.7035833,  -0.588672876,  1.51021063,  0.130829588,  0.508060813,  0.247998863,  0.329805195,  -1.41104805,  -0.629289031,  -0.983450532,  0.895188093,  0.239557594,  -1.37158358,  -1.56261194,  2.32967639,  -0.786532938,  0.640771151,  -0.715828717,  -0.418845206,  1.07992327,  -1.51794446,  -0.493518978,  -0.622294843,  -0.463801891,  0.136020362,  -0.543003261,  0.682789922,  -0.683148503,  0.034641549,  -1.03453505,  1.05572701,  -1.14178085,  1.33249414,  -1.65583313,  -0.678714871,  -0.538339972,  0.260047078,  -0.200619057,  0.378455043,  -0.153526902,  -0.356652379,  0.11286068,  0.28179732,  1.02525008,  0.93993324,  -0.578980088,  -0.0762353241,  0.143297538,  1.25664663,  0.0474672653,  -1.37307441,  -1.60978019,  -0.833405137,  -1.44624448,  0.776378572,  -0.5183267,  -1.06848824,  0.391275764,  -0.105036125,  -0.656482637,  1.31562543,  1.41992581,  0.63156563,  1.48474598,  -0.662699401,  -0.0330664404,  -0.284745246,  0.413083434,  -0.898568809,  -1.16399157,  0.608474791,  0.0895143747,  1.69637454,  -1.32339168,  -1.65259373},{ 0.656492412,  0.00741535844,  -0.582275391,  2.95884776,  -0.708746076,  1.77739167,  -0.707215071,  1.40759349,  -0.872626841,  -0.170762286,  0.551518142,  0.543286622,  -0.293294311,  -0.0699339285,  0.703636229,  -0.311535478,  -1.96871769,  0.672099352,  -2.01907587,  1.62248361,  0.256806433,  -0.0882021114,  -2.18456316,  0.11044421,  0.348356724,  0.872765601,  -0.317339242,  1.80731213,  -1.29647446,  1.24948835,  -0.0559797883,  -1.23391485,  -2.08298445,  0.0951890349,  -0.549772382,  -1.91065168,  -0.904510796,  1.59160638,  -0.113979265,  -0.0799287185,  1.40927041,  -0.0460310206,  0.0693513006,  0.295588434,  0.0263452567,  -1.21120882,  0.40277037,  0.0287036933,  -1.92880177,  0.451327354,  0.501145959,  1.10345507,  1.20597863,  -0.471437842,  0.0946972817,  -0.63573277,  -0.697809041,  1.01425183,  -0.118873343,  -0.417592376,  -0.100069717,  1.24381018,  -0.919597507,  0.561189234,  -0.620768428,  2.54480338,  -0.169382215,  -0.831345081,  1.24424744,  1.59474492,  0.858654141,  1.67469966,  1.70747852,  -2.47089267,  0.745259166,  -0.857046485,  -0.496976405,  -0.239386857},{ 0.10156969,  -0.0659316033,  -1.08746886,  -1.29307568,  0.267752707,  -0.345295548,  0.768763363,  0.00283069536,  -1.76752687,  0.889491916,  -0.17098996,  1.32759893,  1.43198669,  -0.0943116695,  0.0785413384,  -0.53232789,  0.160993844,  1.001423,  1.67219043,  0.139052883,  -0.371737838,  0.293089867,  0.155642033,  -0.894735098,  -0.427044511,  0.928557515,  0.235946134,  0.0423829705,  1.08502686,  -0.95079124,  -1.14589703,  -2.49673462,  -1.33322799,  1.49873459,  -1.66125,  0.694144666,  1.34860158,  -0.326537937,  2.22041154,  0.601949155,  -1.81224954,  0.762533367,  -0.229448155,  -0.696343243,  -1.76320195,  -1.23102951,  0.424576461,  -0.78290993,  -0.0737928003,  -0.192550167,  -1.21173596,  -1.31332052,  1.09676039,  0.322738439,  -0.0452051535,  1.16829586,  -0.912442625,  -1.52081609,  1.31740701,  -1.32624352,  -1.31726742,  -0.311560065,  1.50771844,  0.749504387,  0.496337116,  -0.353174597,  0.997718692,  0.233444437,  -0.683680713,  -0.407302409,  1.09496367,  -1.31111264,  0.0468617231,  0.117926694,  -0.543197095,  1.06581438,  1.23187673,  -0.302748173},{ -0.154970571,  0.515699089,  -0.496051669,  0.770677805,  -3.32708764,  0.499539346,  0.414595753,  -0.36117819,  0.126513183,  0.308712453,  -0.934314728,  0.323496133,  1.06024611,  -0.218007043,  -0.92137599,  -2.76768041,  0.162860051,  -1.77125454,  0.362046093,  -1.75496495,  0.511684537,  -0.797223926,  -0.327641875,  -1.17802572,  -0.372242361,  -0.476133674,  1.73856723,  -0.0739608929,  -1.05967188,  -0.654157758,  1.03226161,  0.686239421,  -0.727996588,  0.452150971,  0.427469641,  2.71577716,  -2.65872312,  0.201783508,  -0.209201008,  -2.6236074,  -0.270146638,  -0.380922645,  -0.000755941262,  1.51656747,  -2.93223357,  0.419491708,  1.14194906,  0.130140305,  -1.3122319,  2.58212781,  -0.710485995,  1.98681593,  0.535470724,  -2.16904187,  -0.301253587,  -2.16991234,  0.777097225,  -0.924113154,  -0.00546625117,  -0.89017427,  1.47722113,  0.281009138,  -0.582516372,  0.226957828,  0.246049732,  -1.58589518,  2.05464196,  -0.234975532,  0.423115611,  0.662917674,  1.13161433,  -0.35177049,  -2.05187011,  -0.554054439,  0.80928725,  0.550573707,  0.244836465,  1.63217926},{ 1.05943251,  -1.66334307,  1.48935437,  -0.570238709,  1.06091392,  -0.20604521,  -1.03699493,  0.985053897,  -0.834415019,  1.34057832,  1.34796202,  -0.869135201,  -0.393484145,  1.72606409,  -1.75012231,  -1.50473487,  1.54455459,  -0.675089478,  -0.402378261,  1.47411549,  0.956523895,  0.176132083,  0.270982236,  0.520368278,  -0.862332821,  -2.30856824,  0.746223152,  0.786607563,  -0.350622714,  0.84635818,  0.61048454,  1.6674968,  0.221492141,  1.81235719,  -0.891013503,  -1.44167769,  -0.954366326,  0.781253994,  1.23173189,  0.558380544,  1.29796958,  -0.701792181,  -0.262588292,  0.708164692,  0.870347261,  0.0640154406,  2.08182287,  1.35705233,  0.35621801,  -0.838759899,  0.0909087583,  -0.571267486,  -0.0171922836,  2.41833067,  -0.699361265,  -0.914352655,  -0.070388414,  0.412351042,  0.18397665,  0.161396965,  0.973967671,  0.743605435,  1.32656765,  2.44352651,  1.36359441,  2.76227975,  0.904499412,  -0.777786613,  0.93514955,  0.411132693,  0.519570291,  -1.17474401,  0.2188209,  0.907947958,  0.444272906,  2.10942864,  -0.18817997,  -1.24892032},{ -0.679822445,  -1.28918028,  -0.0193219818,  0.921195269,  -0.940583944,  -0.949962914,  -0.594746113,  -0.859962583,  -0.0323288999,  0.413825035,  -0.969213784,  1.14434779,  -1.49433136,  0.49038142,  1.01165593,  0.280314326,  -0.263793796,  -0.00219043461,  1.00983095,  0.861710966,  -0.056030035,  -0.852506578,  -0.640699744,  -1.43918788,  -0.418693215,  -0.125280723,  -1.10406697,  -1.29250729,  2.65946603,  -0.789476991,  -0.230296686,  0.468202889,  0.390055507,  0.621314406,  -1.04716265,  0.427061617,  0.252349019,  -1.06824219,  -0.712232411,  1.36120999,  -0.175584033,  0.096258834,  0.389816701,  -0.229351059,  0.345714331,  -0.530635774,  1.11073494,  0.0975517258,  0.155572936,  0.0808505937,  1.65337288,  -0.620231569,  0.21641539,  -1.17438233,  1.37203968,  -0.890400648,  -0.150683373,  0.00889355969,  -0.220159605,  -1.75574911,  -0.0376329236,  0.0703269541,  1.50067616,  0.589549482,  0.96256119,  -0.394711941,  -0.832887173,  -0.0864278078,  -0.776347518,  -0.826712251,  -0.467797399,  -0.810889959,  -0.967879713,  -0.447627306,  0.638186753,  -1.58760095,  -0.931981146,  -0.319464117},{ 1.27263367,  -0.0407601036,  -0.114811406,  -0.449065953,  -0.425402015,  2.57726073,  -0.401809305,  0.629805803,  -0.522526085,  1.25954843,  1.2177304,  -2.13639784,  0.827492237,  -0.15488328,  -0.675486326,  -1.06377387,  0.870669246,  0.610419452,  -0.0635497496,  0.17439799,  0.543500721,  0.297281027,  0.620915413,  0.41002652,  -0.751380265,  1.65273261,  -0.703244865,  -0.752364099,  -0.840872705,  1.47798371,  0.888576925,  0.994961262,  0.572098732,  -1.22106349,  1.09487009,  0.799757779,  1.33132899,  1.08370423,  -0.58384645,  -1.8928864,  -0.0319747739,  1.76297724,  0.0678270087,  1.73757756,  -0.839158535,  0.64823097,  -0.933351159,  1.29539704,  -0.669116557,  -0.37105009,  0.426255941,  -1.84542763,  -0.204078957,  0.462672263,  1.68387353,  -0.237982318,  0.943546891,  0.993899584,  -1.29187346,  1.43147218,  1.81073213,  1.42884243,  1.37428486,  0.529732049,  -0.186309054,  0.0497221574,  -0.570251286,  -0.845595539,  -0.0253488291,  -1.37247586,  1.19938755,  -0.399579614,  1.34522998,  0.873501539,  -0.861883283,  0.074087128,  0.111912109,  -0.439718723},{ 0.0205430482,  1.63423204,  -1.9285599,  0.618066132,  0.495586991,  -1.69924247,  0.467130542,  -0.253099233,  -0.521229446,  -1.39320743,  -1.87533379,  1.22716165,  -0.146018326,  -0.465475321,  -1.69856346,  -0.0183221158,  0.0298499409,  -1.22134542,  0.0796007216,  0.0886596814,  -0.639623165,  -0.522123754,  1.69718385,  1.54131198,  0.00694469782,  -0.496925801,  0.967213213,  0.854007244,  -0.177122697,  -0.612272203,  0.294111341,  -0.813066661,  0.169134423,  -1.361745,  -0.336431235,  0.232901722,  -0.0998804793,  2.28359485,  0.337894946,  -0.736331701,  -0.0353058726,  2.36586404,  -0.0773687512,  -0.503732204,  0.45162189,  -2.09927607,  0.110761829,  -0.948290706,  -0.771597505,  1.24249709,  0.322231591,  3.46922112,  -0.63446629,  1.16679811,  0.341507554,  -3.59860587,  0.0716852918,  -0.337829828,  0.696853459,  -0.0604703464,  -1.46695518,  0.76097858,  0.60644865,  0.162655815,  0.877237678,  -1.57008135,  0.605139375,  -1.70087719,  0.62340802,  0.266111612,  1.35261261,  0.134707078,  -0.718358517,  0.913372695,  -1.01091051,  1.30409157,  -0.514475644,  0.564069808},{ 0.101135038,  -0.150084406,  0.0144490153,  -1.55657113,  -0.0209568888,  0.417679042,  0.121585511,  -1.19945884,  -2.39471197,  0.464637041,  0.876387119,  -1.03317404,  0.648870647,  0.0688607916,  1.23192692,  1.75597751,  -0.0728708878,  -0.751896262,  -0.945684433,  -1.80158317,  1.92211533,  0.165413409,  -0.475684553,  -0.0908293277,  1.65483749,  0.312710702,  -0.764314473,  0.149086669,  0.236329079,  -0.210388437,  1.9740988,  -0.887259722,  -1.68113053,  0.580274522,  -1.45411623,  -1.76016533,  -1.2441107,  0.137961268,  -0.615809321,  1.62672424,  1.0062325,  0.688219666,  1.02900517,  0.945442855,  -2.32193899,  -0.395709127,  -1.00149,  -0.346292108,  -0.596976578,  0.813505948,  -1.41479254,  -0.858460665,  -1.10733461,  -0.948612273,  -0.32636115,  -1.27874625,  1.58333087,  -1.12323213,  -1.78423071,  0.214693248,  1.39037836,  0.0077119926,  -1.32293415,  0.643049181,  -0.387023836,  0.859826624,  2.8265152,  -0.777505219,  -0.0734425932,  -2.17555952,  0.204919115,  -0.0886443332,  0.345178425,  -0.286627054,  -0.425866872,  -0.881603718,  1.07148719,  0.820662856},{ -0.17323561,  0.119300216,  -0.0712441728,  1.29724956,  -0.262845427,  -1.94521403,  -0.534728765,  -0.288564205,  -0.0521420389,  0.191240624,  -0.501358032,  0.622878194,  -0.557894826,  -0.59018892,  -1.53987896,  0.305635929,  0.290572375,  1.76945436,  2.11215949,  2.58322406,  0.225186452,  0.54854995,  1.14089966,  -0.157067537,  -0.514143169,  -1.20722961,  -0.818788826,  0.590495765,  -1.37959063,  -1.27012694,  1.05026126,  -0.878355622,  0.504482269,  1.06049454,  0.517550528,  1.03163671,  -0.534268022,  -0.559710085,  -0.421192348,  -1.28336346,  0.986236989,  -0.556881309,  -0.743649483,  0.449580461,  0.743674874,  1.70451474,  0.392295927,  -0.50195384,  -1.05632794,  1.63797617,  0.905857801,  0.715906501,  -0.498025537,  -0.951228976,  -1.95331967,  0.0811842903,  -1.44916964,  -1.32040203,  -0.0611003377,  2.86834598,  0.690682113,  0.306091875,  -0.229949832,  0.844962418,  1.17587423,  -0.432002902,  -0.237680674,  1.72085667,  1.10243213,  1.06938469,  -1.62656021,  2.43504953,  -1.03217804,  -1.22296941,  1.00204098,  -1.18587279,  1.29669392,  -1.6110338},{ -0.33605966,  -0.752688825,  0.230516136,  0.800716996,  -0.299121827,  -0.922521472,  0.708523631,  -1.72018671,  -0.328724742,  -0.87860924,  1.54444766,  -0.324934632,  -1.11393034,  -0.336453438,  -0.299751133,  1.27613652,  0.416963965,  -0.179490671,  1.16513264,  -0.298241526,  -1.0414542,  0.948961735,  1.0340364,  1.83174706,  0.522676706,  0.234551251,  -0.68247956,  -0.793344855,  0.791465282,  -1.04547834,  0.698259175,  1.42477715,  -1.29321754,  -0.480386376,  1.01907134,  0.999503791,  -1.30254686,  0.839848042,  0.783884168,  -0.514016867,  0.620313227,  -1.42218041,  -0.511667013,  0.53028363,  -0.193971664,  0.690017223,  -0.87008965,  0.0514681898,  -0.44405517,  0.950706184,  0.454672277,  -1.22289634,  0.838557422,  1.54081547,  -0.735757828,  -0.187187299,  0.313043475,  2.03742528,  -1.93627536,  0.901152909,  -0.369426101,  0.855434477,  -0.387138724,  0.80054599,  -1.01431596,  0.732665539,  -1.89957416,  -0.44740361,  -0.301770747,  -1.33716595,  0.201887429,  0.496078402,  1.58125973,  -1.03553116,  0.774558544,  -0.852806926,  1.1656425,  -0.204645276},{ 0.607681334,  -1.27567446,  0.117136583,  0.752109587,  1.04487479,  0.437891006,  -0.187026069,  -0.959504604,  0.110449493,  -0.664214909,  -1.35332775,  -0.594063222,  1.44697011,  0.224177063,  0.0775676444,  2.23862576,  -1.34911358,  0.0741917714,  -1.2719841,  1.23973298,  1.87857866,  -0.612926006,  0.996364415,  0.308386087,  -0.882997274,  1.10896909,  0.730020046,  0.244330764,  0.405755132,  -1.22600412,  1.05063033,  0.52369386,  -1.70125413,  0.395695865,  2.08587623,  0.26650542,  0.698367774,  -1.71046007,  1.17464232,  -0.301685125,  0.511158586,  -0.715911984,  -1.83190215,  -1.07280517,  -0.119429611,  0.0607104339,  -0.577356279,  -0.15317601,  -1.64615309,  0.122435488,  -1.18463385,  1.49366987,  -1.57351243,  1.62543476,  -1.28486812,  1.00071442,  -0.671296656,  0.715696335,  1.36868382,  1.97839749,  -0.0105785169,  -0.959361613,  -2.81084776,  1.29828489,  0.264591783,  0.0364286415,  -0.662530303,  -0.641897857,  0.543139815,  -0.887826979,  -0.556063771,  -0.497535616,  -0.006220154,  0.0629677325,  -0.4531551,  0.945801854,  -0.361736596,  -0.166794538},{ -0.755794764,  0.683689415,  -1.18119776,  -0.134660929,  0.688558817,  1.23766363,  0.656046748,  -0.857808232,  0.87819761,  -2.268852,  -0.532725096,  -0.0514291264,  -0.867055178,  -0.37459144,  0.150548428,  0.0994676277,  0.319953144,  -0.673567772,  0.59259665,  -1.29793799,  -2.0765903,  -0.854124606,  0.790074885,  -0.429981053,  0.231673986,  -0.961653411,  2.56038451,  -0.475422084,  0.324694276,  0.903841019,  0.35406968,  -0.54484272,  1.1423943,  0.323682904,  0.872843385,  -1.64997101,  1.66889191,  -0.324365824,  -1.15916836,  -1.23160291,  -2.02691221,  0.0159462467,  -1.56480765,  0.492914319,  -0.690864861,  0.400954485,  -2.03652191,  1.67783952,  -3.55620027,  3.47552204,  0.178405359,  -1.21172106,  0.732393563,  -1.08477187,  -1.09444332,  -0.954132557,  -1.98647881,  0.183472693,  -0.929252028,  -1.17988741,  -1.06312835,  -1.59491265,  1.11315727,  -0.20291017,  0.749454379,  -0.497004151,  -0.122410625,  0.184126019,  0.665531576,  0.955538273,  1.55383694,  -1.08321834,  -0.813685834,  -0.835241556,  0.909288108,  -0.682555318,  -0.239101052,  0.548972011},{ 2.72610092,  -0.166254252,  0.151113048,  -0.203975797,  0.995094478,  -0.674875617,  1.24591577,  0.439180493,  -1.29947901,  -0.445833206,  0.228715882,  -0.427942812,  0.457033396,  0.371922761,  0.362732917,  -0.278740197,  -1.91215909,  0.350888133,  -0.297752738,  -1.16652429,  -1.02028656,  -0.506010413,  -0.100651585,  -1.48854458,  -0.226643696,  0.442010701,  -2.41291165,  -1.70231295,  -0.941447794,  1.46108103,  0.513941705,  0.248300567,  -0.721392214,  1.87265015,  -0.167655945,  -0.582302928,  -0.136006907,  -2.11865282,  -2.89927483,  0.792950571,  0.924792886,  -0.227713406,  -0.184709281,  1.52254057,  0.00123141566,  -0.892884314,  0.970741034,  -2.63125587,  -0.535925686,  1.1010164,  1.1278857,  0.770143509,  -0.885459125,  -0.104979157,  0.560974896,  -0.619790077,  1.10403216,  0.534636915,  -0.652863443,  -0.712174296,  -0.590131998,  -1.10696971,  -0.840303779,  -1.11991763,  0.519094467,  -0.173853919,  -1.83266985,  -1.09054196,  -0.977980435,  1.8459394,  1.63384199,  -0.748214483,  -0.577829182,  -0.426738501,  1.29743099,  1.94111454,  -0.0410454646,  -1.46262932},{ 0.545895815,  -0.427119493,  0.367774934,  0.503851771,  -1.54790878,  1.5156914,  0.37663129,  0.563416243,  -0.919958591,  0.442838639,  -1.62745845,  2.07816005,  -0.314572185,  0.234069258,  0.775069833,  1.62091947,  0.417123079,  1.01494944,  1.0605365,  0.814082265,  -0.161370337,  -0.465793669,  0.0505517758,  0.513408124,  0.0395045802,  0.46905303,  -1.42157018,  -1.71787584,  -0.879789412,  0.0457089096,  -0.236504465,  -1.18472278,  -0.996732593,  -0.182910323,  -0.244708315,  0.0427906923,  0.150177076,  0.204362005,  0.0372412503,  0.0722276419,  1.10614371,  1.55278981,  0.2062051,  -0.0379092991,  -0.0964568704,  -0.559161901,  0.464964271,  1.84346581,  -0.518040478,  -1.52323401,  0.290393233,  0.431513101,  -1.29251325,  0.249461114,  -1.45835531,  -1.31398547,  0.288619936,  1.79699695,  0.0192071348,  -1.14585793,  1.67743051,  -1.51062107,  1.24969077,  0.657474041,  2.25233984,  -0.386669636,  -0.987567186,  1.16749334,  0.248265803,  0.689592898,  0.670908928,  0.306880951,  0.0309822876,  1.32723725,  1.03253007,  0.85211587,  -1.00655985,  0.0289654639},{ 0.1748368,  -1.11372542,  0.845768869,  -0.678221703,  -0.269589841,  0.468562335,  0.310554802,  -1.34918225,  1.52378702,  0.0180134773,  -0.956760108,  -1.60245335,  1.01853561,  -1.0047071,  -0.37313205,  1.02502382,  1.51586759,  0.0076246825,  0.649340868,  0.934936345,  -1.58430576,  -0.882929981,  0.590139687,  0.55660814,  0.285534143,  0.05483054,  1.33243966,  -0.320138752,  -1.46790326,  0.777246952,  -0.608766496,  -0.492704958,  0.297367841,  -0.46025002,  -0.742233753,  -0.55482018,  1.46615386,  1.02198613,  0.517247379,  1.19419861,  0.0784603506,  -0.272231668,  0.290048957,  0.66814512,  0.531264067,  -2.19158483,  0.553008318,  0.778151572,  -1.29285777,  -2.35033441,  0.0888106376,  0.465003222,  2.46896791,  -0.378260583,  0.674142838,  -0.229839563,  1.57570934,  0.0557504855,  -0.189649075,  1.04981554,  1.38001668,  -1.02083039,  -0.740468621,  0.349733144,  -1.79263628,  -0.23224625,  -2.94850206,  1.3978368,  -0.202606186,  -0.288596034,  1.00293899,  -0.615143239,  0.158234552,  -0.956601143,  -0.777039826,  -2.03306007,  0.704004943,  0.558457255},{ 0.968669772,  0.914318144,  0.568413615,  -0.151918352,  -0.27201727,  1.93192291,  0.349427819,  1.64156795,  -0.813633978,  0.95771575,  0.679525733,  -0.0753650293,  0.995943904,  -1.04341543,  1.70775867,  -1.71220601,  -1.74782503,  0.248398632,  0.657504618,  0.750234902,  -0.320204109,  -0.935042083,  0.894041777,  -0.470720112,  -0.414007217,  0.506661415,  0.753070176,  0.941563308,  0.0798834339,  1.37012446,  -1.10066986,  -1.20167387,  -0.364708245,  0.803153813,  -1.06669462,  -0.271268159,  -0.253706068,  -0.829247773,  -0.511956811,  -0.0241007283,  -1.75818551,  1.39229763,  1.75681019,  -0.0666136965,  1.66444993,  1.4521687,  -0.791253746,  -0.156556517,  2.16708779,  -0.371545136,  0.823761642,  -0.132268593,  0.480852187,  -0.821756244,  -1.0905298,  -0.0488235503,  0.656677842,  -0.386228174,  -0.788631797,  -0.814258218,  0.214094251,  -0.596241236,  0.4153364,  -1.06811643,  -1.62624526,  -0.826009572,  -0.431299597,  -0.474315315,  0.157410875,  1.21628225,  0.00982857775,  1.05130112,  0.272121042,  0.890725136,  1.64548349,  -0.555733442,  1.08267987,  0.31913203},{ -0.846868694,  1.34158206,  0.373681486,  -0.850340128,  -1.02718949,  0.127878666,  0.193701148,  -0.88101238,  1.34654772,  1.37875044,  -0.769920826,  -0.910770416,  0.0448208749,  -0.120389335,  -0.186773911,  0.60189867,  -2.02466369,  0.414392263,  1.42308223,  -0.427078903,  -1.89821827,  -1.74361837,  2.83490276,  0.778834462,  -0.193823859,  -1.07923317,  -1.07680047,  -0.990472615,  1.23264492,  -0.244526222,  0.60618341,  -0.0316507518,  0.988516927,  0.0333073512,  -0.019477753,  -0.0895880163,  -0.253840208,  0.36383009,  0.189166531,  -0.784822047,  -0.926392496,  1.09459805,  0.602639794,  0.694182217,  -0.758476853,  -1.21027672,  -0.857659161,  -0.217437536,  0.499974489,  0.793061554,  1.2939173,  1.37814391,  0.472948015,  -0.121074192,  0.235374734,  -0.0694054961,  -0.546051562,  1.88174057,  -0.33197844,  -1.00015509,  -0.805279016,  -1.21243834,  -0.0884186551,  -1.16648507,  0.502170861,  0.438031882,  -0.811418772,  -0.101985916,  -1.63931596,  1.40877962,  0.937494159,  -0.231839225,  -1.44663012,  0.362347811,  -0.566704512,  0.818355501,  -1.92556834,  1.38743055},{ 1.49874961,  0.670339286,  0.376334131,  -1.05056,  0.767835498,  -0.903054237,  1.27726269,  1.55223823,  -0.956445396,  -1.011145,  0.398828328,  0.101223238,  -0.398283899,  2.59307027,  0.980900347,  1.68442655,  -0.827603221,  -0.867235303,  -0.38011086,  1.46200836,  1.4309727,  -2.36663866,  1.11499083,  -1.22130799,  0.58016479,  -1.37322414,  -1.37537432,  -1.85756564,  -1.17564881,  0.414965153,  -0.286402196,  -1.00293326,  -0.232667357,  -0.042214483,  1.88727641,  0.142653525,  -0.283454597,  1.33832037,  -1.80724442,  0.944426417,  0.452801913,  -2.6565783,  -0.0566401258,  1.02510965,  2.41806459,  -0.793029726,  0.0738612637,  -1.39735317,  0.000761988689,  -1.23596931,  -1.02809453,  0.00408690888,  0.235376105,  1.01209629,  0.511096776,  -1.54890251,  -0.35266006,  0.399996042,  -0.441772044,  1.51202691,  1.05338943,  -0.960253119,  1.38821113,  -1.00949001,  0.0289452747,  -0.181995437,  0.125402823,  -0.591287255,  0.312848121,  0.843857706,  -0.494357139,  0.740415871,  1.10145366,  0.116509274,  -1.24437094,  -0.135173395,  0.41136986,  -0.984884262},{ 0.00995748024,  -2.13291359,  0.144480169,  -0.174237192,  -0.278344661,  0.467184842,  -2.20861411,  -0.897541046,  -0.328054637,  0.175613195,  -0.144772187,  0.0358054526,  -0.714496911,  -1.46728873,  -0.0874571055,  1.44148695,  -0.111069031,  1.18427157,  -0.738774955,  -0.511460304,  -0.39405638,  -0.398661673,  -0.644243181,  -1.68936396,  -1.5849067,  -0.527180791,  -0.267783701,  0.370910615,  1.45531917,  0.868680477,  -0.99116981,  0.619246125,  0.201200321,  1.1065644,  0.267161548,  -0.729712903,  -0.127279803,  1.6691134,  0.0506746992,  -0.704006076,  1.23119032,  0.799356341,  2.10424638,  0.284252226,  0.280655175,  -0.530652106,  -0.388844699,  0.307375193,  1.98787081,  0.397289395,  -0.637047112,  1.46354556,  1.98827016,  -2.07691717,  0.278305441,  0.513673186,  0.39383918,  -0.13095203,  -0.326768219,  1.47904611,  1.7551713,  1.64497006,  0.419078946,  0.561035633,  0.181968451,  1.21869791,  0.614901245,  1.58232903,  -0.832011342,  1.04386127,  -1.08207226,  0.565001547,  -0.814400852,  1.80219805,  0.185573295,  -2.390306,  -0.28678903,  -2.26041794},{ 1.90961301,  -0.368123263,  0.779886007,  -0.958167136,  -1.13477826,  1.43315125,  -0.0263758916,  -0.590320826,  -0.0380023979,  -0.454871416,  -0.570173204,  -0.530583262,  1.56995237,  -0.62397337,  0.444380373,  -0.211266145,  -0.230070367,  -1.4402765,  -1.34973085,  0.740152478,  0.21962215,  -0.440316439,  1.28754985,  0.577784598,  0.493738711,  1.24507737,  -1.35080433,  1.88141632,  -0.375186801,  1.46992624,  0.854871094,  -0.662241399,  0.225334018,  2.15910554,  1.69891512,  -0.793533981,  -0.508380771,  0.0783401355,  2.03309131,  0.509620547,  -0.536899447,  -0.678007424,  -0.566556215,  0.114494301,  0.0509291962,  -0.564749062,  1.34778285,  -0.42184478,  -1.12407279,  1.83342338,  0.954411864,  -0.864650071,  -0.200983524,  0.403206795,  -0.0215174854,  1.39336824,  1.71829641,  0.634199142,  -0.797295988,  0.258911908,  -0.978691459,  -0.175657615,  0.268547386,  0.138664514,  -0.325633615,  0.0844722688,  1.35508668,  0.332252741,  -0.326635301,  0.0838384777,  -2.22236443,  0.22962825,  1.62446105,  1.29269159,  -1.69615126,  -0.204051673,  0.756279409,  -1.09171224},{ 1.73777735,  -1.0757935,  0.372857869,  0.107416317,  1.659675,  0.73369801,  0.289730757,  -1.78184283,  -0.237892076,  0.327300251,  0.153253481,  0.871392965,  -0.375434637,  -0.848272264,  -0.387881607,  0.672755778,  -0.052119676,  -0.552414417,  -1.20310056,  1.93322468,  -1.302508,  -0.093200177,  -0.0431171209,  -0.366485983,  -1.61266088,  0.207190841,  0.702962697,  0.916145205,  0.100511648,  -1.53941488,  1.43610036,  -0.400587589,  -0.34024322,  -0.516596675,  -0.595999122,  0.262013614,  -1.35795856,  1.61181867,  -0.0883279517,  0.464762002,  -1.36882007,  0.966953754,  0.145576388,  -0.412423313,  1.6703788,  -0.599317372,  -0.118824035,  0.730839312,  1.6238941,  -0.735434294,  0.167244449,  0.308142424,  0.473087788,  0.518793344,  1.14538741,  -0.599678218,  -1.22682774,  -0.404002428,  -0.715866625,  -0.390523642,  0.0943277553,  0.363880694,  0.236075565,  -0.731697142,  1.12379372,  0.540955245,  -1.41840255,  0.0204535555,  0.38593477,  -1.59981859,  -1.08695865,  2.43089628,  1.04955792,  0.331286818,  -1.52924049,  -0.0952959359,  -0.396074563,  0.430059701},{ 1.00662482,  0.418925047,  -1.02900863,  0.924072504,  2.38160467,  -0.653095245,  0.7643525,  0.660666585,  1.35166883,  -0.894278407,  -0.821090162,  1.12048447,  0.269695908,  -0.700841606,  1.41691041,  -0.128973946,  -0.69952935,  1.10971749,  0.196278304,  -0.37839672,  -0.603172839,  -0.301277727,  0.116880096,  0.239497632,  -0.0487007461,  -2.30492783,  0.339720309,  -0.780013204,  -0.0158360656,  1.78740215,  2.18373418,  1.13694632,  0.122346722,  0.393065929,  0.0150795979,  -0.829606354,  -0.899113953,  0.67969501,  -0.150943905,  -0.261355966,  -0.950232089,  -0.0934385732,  0.603677392,  0.314586282,  0.717635512,  -0.0870400444,  -0.94250989,  -0.438630521,  1.36618638,  -0.483848929,  -0.806517363,  -1.22665155,  1.07492793,  1.21667385,  -1.05141628,  -1.68712664,  1.39827704,  1.64146125,  -0.0407902077,  1.04133141,  0.466339201,  0.358813405,  -2.49699235,  -0.178818405,  0.264026731,  -0.48226428,  0.640276909,  0.755856752,  -1.12325513,  -0.12460544,  -0.48751238,  0.164636329,  0.321841061,  -1.05349135,  0.337982804,  -0.82071352,  -1.23555374,  -0.0238245353},{ 0.216374367,  0.129582793,  0.150419533,  0.769227028,  -0.819028258,  0.675540864,  -0.719111979,  -0.535799921,  -2.13143015,  1.32902658,  -1.30632997,  0.98958379,  0.382470369,  -0.245603248,  -1.0124898,  -0.714770854,  -0.168476328,  0.956349134,  -0.332067311,  0.339786887,  -0.136034906,  -1.66977918,  -0.470337123,  1.28466332,  0.0683774054,  -0.405254722,  1.97373521,  0.312997818,  -1.08030534,  0.940024912,  -0.314919591,  -1.12329125,  1.49213445,  -0.760585845,  0.891389728,  2.35429692,  0.805098951,  1.10733902,  1.49310267,  0.372353554,  1.76755679,  1.17535448,  0.53031379,  0.654000521,  0.00718265213,  2.17547107,  -1.93403411,  0.446408004,  0.485952288,  -0.561495602,  0.360769123,  -0.696763992,  -0.948253334,  -0.959812999,  -0.365723073,  -0.799453557,  -0.129616752,  -0.803876221,  -0.0092079062,  -0.235143676,  -0.235355005,  1.42502034,  0.0758143514,  1.48130977,  0.0766065195,  -0.746882737,  0.596195638,  -1.45798612,  -0.331948638,  -0.14945294,  1.14721382,  0.955758154,  -1.92249715,  1.97149622,  0.660212576,  -0.0954464003,  -1.00254774,  -1.4297502},{ -1.32474029,  0.382014036,  0.511929035,  -0.221997142,  0.917985797,  1.12804973,  3.13675928,  0.956349015,  -0.0846827403,  -0.553748488,  1.2953372,  0.130408078,  1.31462824,  0.620068729,  -0.10547097,  -0.871870935,  0.00663053105,  -1.07530141,  0.239519253,  -0.399035662,  -1.22227001,  -0.144618496,  0.289090186,  -0.566160738,  0.515525699,  -1.55643272,  -0.85520643,  -0.797765076,  1.06537116,  -1.02236557,  -2.46455789,  -0.301056623,  -1.38497579,  -0.209702283,  0.68524766,  -2.54552031,  1.18125927,  -2.02723336,  1.16681159,  -0.15071927,  -0.062595278,  -1.09370363,  0.262553006,  -1.85237563,  0.159855962,  0.545863211,  0.246944159,  1.14575934,  -1.07188892,  1.62323236,  0.166197807,  -2.14594364,  -0.015311175,  -0.243186995,  -1.02301562,  2.02788091,  0.325661689,  1.85573816,  0.345369905,  -1.48771858,  1.07687521,  0.840865195,  1.44946742,  0.660957694,  -0.205573156,  0.29090789,  -0.25234744,  0.784932673,  1.43566883,  -0.530181229,  -0.00286464463,  0.559795201,  -1.33328164,  -1.47866869,  -0.411356986,  -0.385840535,  1.87045264,  -0.808231831},{ -0.390745699,  0.652863443,  0.387996078,  -0.685514152,  0.369765162,  -0.61887759,  0.587760031,  -0.0342822894,  0.180321425,  -0.166850418,  0.825477839,  0.500837207,  -1.00362587,  -1.42814338,  1.54147422,  -0.630165219,  -1.29654205,  -0.114244692,  -0.142156944,  0.0975532308,  -0.568394244,  -0.950915277,  1.19661045,  -0.128245234,  1.7476027,  0.879864335,  0.0997410044,  0.500216722,  0.335914224,  -1.12236893,  0.515177548,  0.754104495,  0.594101191,  -1.22245777,  -0.463845044,  0.115498327,  -1.86657846,  0.781420529,  0.508628428,  1.004143,  -0.538882315,  1.69045317,  0.357774377,  0.0347186364,  -1.40364718,  0.311351627,  -0.829164207,  0.425086766,  2.0843749,  0.114834577,  0.0227296129,  0.20331645,  2.35440636,  0.0822918117,  -0.582449317,  -0.862936199,  -0.502203465,  1.88272095,  -0.460072577,  -0.974627435,  0.372594237,  1.85203981,  -1.04481626,  0.025444543,  1.56458545,  -1.00117159,  -0.257343173,  0.407681823,  1.70565927,  0.317433089,  -0.979500592,  -1.20207512,  0.373384982,  0.20073548,  -0.0750509277,  -1.04849267,  0.332286119,  1.1627562},{ -1.57437456,  0.406950802,  -0.332772344,  -1.60078692,  -0.206874818,  -1.0351001,  0.786334634,  0.086998634,  0.712534487,  1.75756609,  -1.16985917,  0.594474912,  -0.0850950032,  -0.317645401,  1.48073661,  1.05846322,  -2.07452106,  -1.21754324,  0.0878837332,  -0.865563869,  -0.315511853,  0.623617291,  0.66252774,  -0.756751537,  -1.66503966,  -0.612876356,  -0.229214251,  -0.220679536,  -0.50020504,  -0.243965074,  0.351516604,  -1.73516297,  0.779524624,  -1.03819537,  0.300501436,  0.183936805,  -0.0173277333,  -0.751489937,  0.0467600152,  0.183745787,  -2.72353745,  -0.0376852788,  -1.04174447,  -1.45315754,  -0.498996019,  -0.0925665945,  -0.921551704,  1.19879973,  1.33381236,  0.311909497,  -0.274187118,  0.219357923,  -0.321619272,  1.50933063,  -0.922157049,  1.08168769,  -1.98860967,  -0.218540415,  1.28668356,  0.174707651,  0.211355507,  -1.01731968,  -0.223098397,  0.0669670478,  0.167521149,  0.625322402,  -0.678213775,  0.123540752,  0.123870485,  -2.14148068,  -0.270945132,  2.47119713,  1.09020782,  -0.570415497,  -0.191010296,  -0.856436729,  -0.463269055,  -0.640293717},{ -0.421670794,  0.783313632,  -0.612106204,  -0.269683301,  -1.95094562,  -1.58936882,  -1.57329285,  -0.13029094,  -0.459382832,  -0.222573608,  1.09907448,  -0.121751189,  -1.04116774,  0.949010313,  1.65942824,  1.11793959,  0.980621576,  -0.239862591,  2.29480624,  -1.25049806,  -0.326845616,  -0.478915244,  -1.54447591,  0.706318796,  1.22161007,  0.143110022,  1.55127287,  -0.265253037,  -0.00810705777,  2.29773736,  -0.0548355132,  2.07346654,  1.56559026,  0.902644992,  0.660146177,  -0.569389284,  0.852135599,  0.397069395,  2.98342562,  -0.371962607,  -1.21458387,  1.06471622,  -0.0435541235,  -0.887346983,  0.900806129,  1.08367002,  -1.4224875,  0.110030346,  -0.305898905,  1.40047121,  -0.595960259,  0.208921313,  -0.741965294,  -1.5273577,  0.593819201,  0.745797396,  1.37754774,  0.222718671,  -0.131562769,  0.446350276,  -0.672148108,  1.21931779,  1.12714899,  0.51017189,  1.79335916,  -0.664854527,  -0.044711411,  0.772094727,  0.805260062,  0.0721727461,  -2.35105801,  -0.191082016,  0.066390954,  -0.846680164,  -1.44975054,  -0.22177431,  0.960017681,  -0.740131736},{ -0.331873387,  0.410212725,  -0.491240025,  -0.509946346,  1.14706278,  -0.953805268,  0.873776615,  -0.179139063,  0.707426071,  0.51650393,  -0.19732675,  -0.753266275,  0.220985472,  0.251599789,  -1.99724817,  -0.938159227,  -0.185260057,  1.48817468,  0.775325418,  1.02064109,  0.750605345,  -0.0511995293,  -0.579473197,  0.465290666,  -0.238728046,  1.50632751,  -1.52394056,  0.889267802,  0.255817711,  0.916742861,  1.36174798,  0.398659617,  -0.200676188,  -2.13425541,  -0.282816976,  0.364817113,  -0.748174906,  0.830244601,  1.55438495,  -0.521359622,  -1.32504749,  -1.11363018,  0.371961951,  -0.214721009,  1.14962125,  -0.43296206,  -0.600227177,  -1.1569823,  1.31681502,  -1.89650953,  -0.508818865,  -0.774412751,  -0.439344049,  -0.0564111769,  -1.38685071,  -0.73314321,  0.456265539,  0.578643918,  0.0752309561,  0.302103221,  0.405896604,  -1.53159881,  0.357406259,  1.06911683,  -1.16073239,  0.65736264,  -0.378305167,  -0.0633759871,  1.95500755,  -0.855779946,  -0.562122047,  1.54741859,  0.841791391,  -1.04547954,  0.123544954,  -1.14181435,  -1.67491543,  -0.406632304},{ -1.09924257,  -1.21560645,  1.66318226,  0.385647416,  1.61174238,  0.50110513,  -1.23773789,  0.888299465,  -1.74240434,  -1.39101541,  1.59167588,  -0.471515924,  0.956840456,  -0.357080609,  -1.29668176,  -0.432974696,  -0.0919144452,  0.97949934,  -3.81005645,  -1.46508932,  -0.706514001,  -0.672970235,  0.746034145,  -0.372933626,  -1.65411711,  -0.223545238,  -0.0985644683,  0.525470138,  0.739283919,  -1.21850097,  -1.69181752,  -0.932299256,  -1.40861738,  -0.345755786,  0.584894121,  -0.803072572,  -0.771967053,  1.54197729,  0.462165475,  0.644754052,  0.728396595,  0.293052435,  0.607573628,  2.47994804,  -0.952053726,  -0.25113073,  0.589244306,  -0.209149033,  2.42611909,  -0.357887983,  1.10498405,  0.554935575,  0.819693625,  -0.481499761,  -0.153024599,  0.725063324,  0.256144732,  -0.392220497,  2.33834839,  -0.145012826,  -0.779298902,  1.07627559,  0.326011539,  -0.650134504,  -1.07060373,  0.0991685838,  -0.8668679,  1.15742671,  0.619597495,  -2.07603884,  -1.46363389,  0.2000615,  -1.28802955,  0.141473472,  -0.975466609,  -0.0224018078,  0.0879152343,  1.5685941},{ 1.25298464,  -0.824058473,  -1.3296684,  0.0794044733,  1.78069639,  0.371966481,  -0.989254415,  -1.17058933,  0.257360429,  0.229629204,  -0.737543106,  1.14238513,  -1.32154799,  -1.03639722,  0.317064196,  1.28313816,  -1.53210485,  0.420391947,  -0.426999778,  -0.971427023,  -0.505100906,  0.153180927,  0.500645995,  0.695146322,  0.399295926,  0.718301773,  -0.289519668,  -0.486860603,  1.19721246,  -0.975535393,  -2.22996044,  -1.14764643,  -0.493303895,  -2.85358095,  -0.117357835,  0.294132054,  1.16284204,  0.830269635,  0.461544275,  0.704457045,  0.0492615141,  0.0835690349,  2.43174005,  0.737427294,  -0.881587446,  0.888570011,  -0.919156611,  -0.314506739,  0.788091302,  0.051783815,  1.83214784,  -0.461151332,  2.65109253,  0.380042493,  0.395268768,  -0.291017115,  -0.421674103,  0.194988817,  1.35438728,  -0.482748717,  0.205815062,  -0.0481401458,  0.120763905,  -0.201986596,  0.300866485,  0.313042611,  1.26327717,  -0.684631765,  -2.08234596,  0.426012009,  -1.78202641,  0.781528234,  -1.16456103,  0.668530107,  0.531423867,  -1.74832022,  0.123197891,  -0.535578072},{ 0.966448724,  -1.09897077,  -1.42420006,  0.108135231,  -1.81845856,  0.208947197,  0.687415719,  -0.0382225886,  0.377430022,  -0.292182416,  0.127397597,  -1.88470662,  1.48857701,  -0.226527989,  0.353398293,  0.281537384,  1.48385108,  -0.596973181,  -1.40635622,  -1.14882338,  -0.431557745,  0.88779819,  -0.592497945,  0.763891578,  0.387675583,  0.76556474,  -0.451580256,  -0.336862147,  -0.178540692,  -0.450624228,  -0.130382627,  -0.351970911,  -0.0844441205,  -0.37538597,  -0.95083648,  0.770406127,  1.11982119,  -0.203678071,  2.07287216,  -0.819070101,  -0.156903014,  -0.655808985,  0.994705677,  0.336206734,  0.640170634,  0.0482921042,  0.676645935,  -1.97716808,  0.139493093,  1.34559584,  -0.358261973,  -1.25785506,  0.864009976,  0.577154875,  1.13486397,  -0.721403182,  1.16596127,  -1.32474566,  0.704351008,  -0.507159412,  -0.605653644,  -0.438244849,  0.611836374,  -0.549393058,  0.903960943,  -1.31103897,  0.597444654,  0.0729297772,  -1.74348438,  1.16252947,  0.512996674,  0.904077768,  0.935304224,  0.907044411,  0.233990952,  -0.937497795,  -0.652076006,  -1.63724351},{ -0.0454979278,  -1.84275138,  -0.889397979,  0.314265788,  1.27761734,  1.23785675,  -0.457736135,  0.420210332,  1.6950202,  -0.0503014997,  0.913976014,  -0.81754905,  1.2392174,  0.030144427,  -0.54884994,  -1.60821211,  0.0355579183,  0.745503664,  2.69148612,  -0.159215361,  1.21181834,  -0.0208603684,  -1.59141159,  -0.173312172,  -0.467608422,  -1.22703493,  -1.52362359,  -1.59721458,  -0.843653023,  1.42530262,  0.734288037,  -2.0083046,  -1.34896231,  1.4947176,  -1.70419502,  0.898172677,  -0.25583759,  -0.620732546,  2.08923364,  0.652952015,  1.83872259,  -0.282480538,  0.402618259,  -0.566480339,  -0.351154119,  -1.28968489,  1.37258244,  0.30751586,  1.22822523,  -0.375379503,  0.204190627,  0.710068285,  -0.434272259,  1.5093857,  -0.817230344,  -1.43547463,  -0.705591321,  0.568252683,  0.0806759447,  1.05985188,  -2.70153165,  -0.924486458,  1.12907457,  -0.0247012116,  -0.584114254,  -0.909987688,  -1.31913948,  0.360856026,  0.984911859,  -0.912425518,  1.35200524,  0.538660169,  -2.23361492,  0.792565823,  0.436796635,  0.0676063299,  1.12556696,  -0.181065336},{ -0.495723218,  0.812069356,  -1.89041829,  0.102882095,  0.342475116,  1.35225379,  -0.435848981,  -0.693155766,  -0.606947124,  0.419381857,  -0.196507156,  1.37409735,  0.167901486,  1.71364069,  -0.0479431748,  2.39094257,  0.791386306,  -0.107544512,  -0.509267628,  -1.26568544,  1.00683761,  0.494301289,  0.783660233,  -0.134337455,  -2.03620768,  -0.889223576,  1.2232523,  -0.28578037,  1.21886444,  -0.337941408,  0.26833111,  -0.123783424,  -0.954225779,  1.45674872,  0.238971293,  0.0141907753,  1.17924297,  1.36734283,  -1.96486211,  0.93155551,  0.115335241,  -0.577587306,  0.00920050591,  0.405109227,  -0.324777335,  -0.757154047,  0.063075304,  -0.122732602,  0.704596758,  -0.327262104,  0.630623639,  2.28080988,  1.71122313,  0.938622594,  0.301871866,  0.506200552,  -0.377293527,  0.779275715,  0.266343951,  -0.679808795,  -0.410531938,  -0.156060219,  0.911615193,  0.879510343,  -1.65469086,  -1.07979572,  -0.935963631,  0.591168761,  -1.32751369,  -0.590894282,  0.897604346,  -0.120265312,  -0.498850077,  -0.232549727,  0.877951562,  -1.89664721,  -0.308469176,  1.47020829},{ -0.528653085,  0.304933697,  -0.12316528,  0.147689149,  2.49572921,  1.17489207,  -0.360114157,  -2.08323455,  2.18152571,  -0.469321162,  1.33042455,  1.01357138,  1.9602952,  -1.35207307,  1.41836917,  0.604754806,  -0.519741476,  -0.249662876,  -0.500343621,  1.05074787,  1.05432343,  1.32819712,  -0.140169576,  -0.495260179,  1.35190892,  0.205632523,  -0.0235279128,  0.546615303,  -0.261744887,  0.387775928,  0.304639041,  0.91023916,  0.176200509,  -0.686776578,  -1.28639567,  -0.749980867,  -0.436945558,  -0.21774143,  0.519266069,  1.00718284,  0.31760478,  0.698851526,  -0.922156811,  -2.18164182,  0.92207706,  0.877395451,  1.01412904,  1.43157589,  -2.16473269,  -0.590010643,  -0.204265341,  -0.128423288,  1.32977676,  0.149589434,  0.501579881,  0.495094031,  -1.9355669,  -1.29682183,  -1.05330956,  0.192819998,  -0.730945051,  -3.00884914,  0.975692213,  -1.02558398,  1.17079759,  -0.623767436,  -0.365939587,  0.354096144,  1.74758959,  0.154996216,  -1.90203643,  -1.67734861,  0.22107631,  -0.892224729,  0.380766958,  0.497615159,  -0.539540291,  0.580898285},{ 0.751352787,  0.015490138,  -0.519825697,  -2.13544345,  -1.81388462,  -1.4042722,  0.209303111,  1.88807333,  -1.01976001,  -1.07648027,  -0.240568116,  -0.427877814,  0.419367224,  -0.534225404,  0.851427555,  1.4538343,  -0.061397545,  0.481696337,  -0.607949793,  -0.345456094,  0.845399916,  0.910179675,  -0.122331575,  -0.138179228,  -0.0757010579,  -1.12416482,  1.19326127,  -0.915030599,  -0.681055605,  0.2173713,  0.88118279,  0.59690702,  -0.311861068,  1.40595651,  -1.37179434,  0.735961556,  1.69836414,  0.351862758,  0.468308866,  -1.03769112,  0.238703966,  1.19441688,  0.877450109,  1.30018902,  -2.08699799,  0.23958008,  0.167784572,  1.43713272,  1.12343299,  -1.1237421,  -0.385598958,  -0.850302577,  0.0169601217,  -0.0749163702,  0.0913399532,  0.311827898,  -1.24333894,  0.777312875,  1.24188697,  -0.984349608,  -0.392014682,  2.0729754,  1.04424334,  0.446651369,  1.24969685,  -0.352350563,  0.0781168342,  -1.10307443,  0.559682429,  0.641907215,  0.891970575,  0.937496543,  1.57320809,  1.00374866,  0.505054474,  -0.463472337,  -1.02848136,  1.06517243},{ 0.697450519,  -0.698834419,  -1.19620955,  -1.01407421,  0.63768065,  -0.990515769,  -1.07792819,  -1.11231947,  -0.86442858,  0.0852089971,  -0.70011729,  -1.09361959,  -1.21020591,  0.961218297,  -0.314765304,  -3.31315327,  -0.662763059,  -0.0089354869,  0.753168464,  -0.389367789,  0.527748704,  -1.25474405,  1.35122633,  -0.667567611,  0.265172988,  0.365584731,  -0.473422617,  -1.16962647,  -0.36086902,  -0.968918145,  1.93990779,  -0.537868321,  1.42024326,  0.111940645,  1.14395523,  1.17330694,  0.606801331,  1.60664988,  -0.629500806,  1.04909444,  -0.893498182,  -0.0663715303,  1.05207705,  -0.71940726,  -1.45134711,  0.216906354,  -1.16952038,  -0.0545166098,  -0.978731275,  0.318836957,  -0.794306636,  -1.78529453,  0.093974404,  -1.41276836,  0.119787954,  -1.3913976,  0.741533756,  0.716086447,  1.04502833,  0.133681729,  -0.229880527,  1.31235492,  -1.60152471,  -1.1894691,  -2.78950024,  -1.84132481,  1.73135257,  -0.430942833,  0.134220168,  0.131498531,  -1.22307646,  -1.27105796,  0.832383752,  -1.45545638,  -0.80994755,  0.411165833,  -0.674308181,  -1.55896664},{ 0.332578063,  0.266513467,  -0.575897336,  -1.3483243,  1.42830861,  1.53177154,  0.579577863,  0.148079023,  1.91456795,  1.36023319,  -0.815886796,  -0.757303953,  -0.173940748,  1.81852233,  -0.541307867,  0.457225472,  -1.34301257,  -0.906455934,  0.186335385,  -0.712929428,  0.525889933,  1.74166727,  0.518340647,  0.174772799,  -1.15155327,  -0.122089952,  -0.97974503,  1.56813109,  -1.2957896,  0.198732495,  1.01387644,  -0.0494755469,  0.428038418,  0.00588206295,  1.19548297,  -2.76416183,  -0.126519844,  0.122035131,  1.46838653,  1.65494943,  1.35789132,  0.673733294,  -3.32154417,  -1.86288333,  0.774093747,  -0.0952306837,  -0.0905865282,  0.421664953,  -0.780282438,  0.975894153,  1.98854268,  0.246270165,  -1.54852593,  -0.991714895,  0.355221897,  0.748069763,  -1.49520159,  0.721304655,  -1.79504514,  -0.194112048,  -0.0963268727,  -1.11332321,  1.40847325,  -0.352097958,  -0.554142177,  1.58281744,  -0.875739276,  0.107480012,  0.299503267,  1.37294865,  1.27244568,  0.529295206,  -0.130701721,  1.48302996,  0.516665459,  0.322000712,  0.88318789,  0.132379696},{ -0.0889499262,  -0.818787754,  -0.871608973,  0.56309396,  0.207282409,  0.862512887,  0.359999835,  0.239289284,  -0.590918183,  0.89443022,  -0.200872809,  0.170080304,  0.604797363,  -1.29652166,  -1.21246815,  0.00527282432,  0.423421085,  0.494263142,  1.52543139,  -1.01582384,  0.427415997,  1.57532716,  0.152758911,  -0.713316083,  -0.926405966,  0.365972787,  0.420881599,  0.303625673,  1.30566382,  1.49925065,  0.17429693,  -0.51110065,  0.947257817,  0.96882093,  0.788692832,  0.294224173,  -0.55371666,  0.570490539,  1.04458535,  -2.37508464,  -0.504038215,  0.328956664,  0.0191449244,  1.00518596,  0.346826673,  1.44423413,  -1.08273709,  -0.103343166,  -0.743369222,  1.73283339,  -0.0138575351,  -0.366330177,  -0.286841512,  0.686451018,  -0.349931449,  -0.0743063837,  0.422039211,  0.12628898,  -0.761930525,  -1.14215267,  -0.759300232,  -1.10164511,  1.27947628,  1.06008732,  -1.88220036,  -0.943677068,  0.501427531,  -2.51669884,  1.34856653,  -0.827479303,  -1.12753165,  1.7938633,  1.95639729,  1.28284919,  0.732874691,  -1.03300416,  1.01318419,  1.32379937},{ 0.413656145,  0.0123449462,  1.95011926,  -0.26604107,  0.473739803,  1.05300701,  0.155656368,  -0.184472129,  -0.655009925,  0.95376116,  0.455039561,  -0.86086905,  -0.801759958,  1.99629891,  1.33161092,  1.06809914,  1.09113073,  -1.76776981,  -0.325183839,  0.900568008,  0.63121444,  0.308470547,  0.674310744,  0.328080684,  1.57945848,  1.46931791,  1.204198,  2.81916857,  0.223439619,  -0.342643499,  -0.279816657,  0.692788005,  0.664642036,  1.23701143,  1.07235563,  -1.16309261,  0.0574208386,  -1.10431087,  -1.61821139,  1.64987218,  -0.414267302,  -1.31419694,  -0.987579107,  1.33642614,  0.787503242,  0.403082699,  0.352641553,  0.199935839,  1.25165343,  -0.247457445,  0.37886253,  0.222551927,  0.504808486,  -1.12155938,  -0.72393024,  0.147751108,  0.106680326,  -0.361826509,  -0.218029886,  -0.508182287,  -1.24903214,  0.651451468,  0.983920991,  0.319641024,  -0.114806309,  0.202016562,  -0.394977778,  -0.405920208,  1.23743415,  0.507853746,  -1.25564432,  0.534944594,  -0.117182665,  -0.450095803,  0.531347454,  -0.101593614,  -0.291232467,  -0.766894758},{ 1.30377066,  0.548416853,  -2.26769519,  -0.522082925,  -0.340485394,  1.82086921,  -0.470897555,  -0.788161397,  -1.00954902,  -0.196032375,  0.381372422,  -0.292778164,  0.111924052,  1.07664382,  -0.438989729,  1.91248715,  -0.598852336,  -0.431202263,  -1.22659099,  0.790125847,  -0.419588655,  -0.493180692,  0.717679739,  -0.228349373,  -0.730306804,  0.0965346098,  0.456170082,  -0.972333729,  2.6300168,  -0.43606624,  -0.0509141833,  -1.8987416,  0.692530572,  -0.507366657,  -0.346966326,  0.0824653283,  -0.453066975,  1.367908,  -0.407517076,  -0.751223862,  -0.0978349373,  1.70439601,  1.07671344,  -1.0866369,  0.106131695,  -0.894592166,  -0.741330147,  -0.852433741,  -1.2104001,  -0.286291629,  -0.105068333,  -0.394250482,  0.691450894,  0.787389815,  0.015571692,  -1.36189556,  -1.49685013,  0.318017364,  0.356551647,  0.900711238,  1.05879021,  -0.424693823,  0.472000897,  -0.698692322,  1.27162111,  0.300254524,  -0.0457937494,  0.193139419,  0.82047236,  0.943836153,  2.05428672,  -0.312733531,  -2.99949074,  0.16181311,  2.52450109,  -1.73529232,  1.86798048,  0.73225975},{ 1.89338601,  -0.067509003,  -0.632793784,  -1.01543987,  0.161391482,  0.274713904,  -0.176306173,  -0.0930191875,  -0.18483144,  -0.979673564,  -0.588217616,  0.435008317,  1.08335185,  0.331310064,  -0.0876774788,  -0.774700165,  0.510397613,  0.826295435,  1.37451386,  1.96137023,  -0.145054445,  -0.0590732247,  0.357056051,  -1.36126554,  0.727810025,  0.270955205,  -1.31084573,  1.12826908,  -1.43906593,  1.54958463,  -0.307900876,  0.146454737,  -0.106949508,  -1.25042009,  0.760050297,  0.227953687,  0.629027307,  0.862966359,  0.934842408,  0.254190236,  0.973625839,  -0.503479421,  -1.28350472,  -0.288638502,  0.90050137,  0.737329364,  -0.45799768,  1.39421475,  0.981586397,  0.351305544,  -0.00878090132,  -0.861014366,  -0.562934458,  -0.67163974,  -0.112826318,  -0.647113979,  0.17217952,  -0.277591944,  -0.145114958,  3.23028898,  0.686164737,  -0.48036778,  -0.71053493,  1.16513181,  0.483816832,  -1.32397556,  -1.36754835,  -0.0369253531,  0.976659954,  0.409156829,  -0.627203465,  -0.84771508,  -1.85016263,  1.29410493,  2.10798955,  -0.227643952,  0.777023673,  -1.14580739},{ 0.75755769,  -0.698079288,  0.0338442475,  0.13137041,  -0.245371938,  -0.491655201,  0.146261841,  0.022886999,  -0.144275546,  -0.702573061,  -0.956854105,  1.64297903,  -0.817112803,  -0.489222676,  0.852737069,  -0.0902197137,  0.650008261,  0.0563426241,  -0.796461821,  -0.0584873743,  0.0361829586,  1.47859073,  -1.07277286,  1.29123497,  -0.40190345,  0.0850654319,  0.45402196,  -1.06906891,  0.0971498564,  -0.412369579,  1.80699396,  -0.607741773,  -0.0464786105,  0.833342493,  -0.701212645,  -1.72795069,  -2.69482923,  1.47344768,  1.27265406,  -0.149467975,  0.974840522,  0.96380043,  0.225805774,  -0.724749386,  -0.0579262003,  -0.498708248,  -1.99954557,  0.834459543,  0.00722153811,  -1.13438666,  -0.36512062,  -0.452211171,  0.898138762,  -0.0521721393,  0.15442422,  -0.215872779,  -2.09205675,  -0.577281117,  0.366442353,  -0.187586278,  -0.204081059,  -0.320672303,  -0.543527007,  1.1755352,  -0.745620847,  -0.56378001,  -1.48393047,  0.785435915,  0.718923926,  1.39449441,  0.718689144,  -0.139833912,  -0.481382281,  -0.250421286,  0.935136616,  0.214000687,  0.510778248,  -0.147451356},{ 1.94675803,  0.903415561,  0.899320066,  -0.848689795,  0.85778892,  0.502200484,  -1.77114666,  0.568855107,  2.45178413,  -0.317874312,  0.40301165,  0.0525562204,  0.626997352,  -1.38416886,  0.307293802,  1.01950943,  -1.78819573,  -0.00792557374,  -0.0298178289,  -0.770448625,  -0.870618641,  0.713620007,  0.307409436,  0.584581017,  -1.01377714,  -0.585599899,  -1.35101497,  0.942090452,  -0.36023128,  1.43546021,  0.821724415,  1.57358801,  0.245417953,  1.58552158,  0.654484749,  0.850549221,  -0.059884239,  0.609339356,  -0.422727883,  -1.57706892,  0.877551138,  0.523216665,  -0.284379572,  -0.407348812,  0.540018082,  0.979869962,  0.28746736,  0.0139143066,  0.493421376,  1.06923532,  -0.554747939,  1.4454453,  -0.730900586,  1.40976775,  0.439299375,  -1.35594797,  0.467082262,  0.0788861439,  -1.33164084,  -0.0222687982,  -1.03032124,  1.39292169,  2.13356733,  0.737556219,  0.607304752,  -0.91450721,  -0.274146378,  -0.165266186,  -0.804400802,  -2.37263775,  1.21235895,  -2.15597796,  0.966358662,  -0.955449343,  1.48789513,  -0.746922195,  0.11453031,  -0.70800215},{ -0.53753829,  1.47359669,  -0.13458389,  0.394057333,  -1.50446165,  -0.345180899,  2.48575807,  -0.638946831,  0.014580437,  -0.155341461,  -1.01084924,  0.652431667,  -0.78833729,  -1.46612346,  -1.23763096,  1.13780618,  -0.124156691,  -1.03585923,  -1.30561841,  -0.190837026,  -0.188507274,  -0.825601161,  0.240127936,  -1.25933695,  -0.0849338621,  -1.739429,  1.87202394,  1.67136526,  0.282239735,  -1.02811992,  1.2432431,  0.482992858,  -1.14018536,  -0.110728018,  -0.699019909,  -0.0573975481,  -0.818582535,  0.846602976,  -1.48185384,  -1.07129061,  -1.13375306,  -0.137067437,  1.49893069,  -0.754446745,  -0.57682842,  0.756329894,  1.14207268,  1.21247721,  -0.123918936,  1.51027799,  -0.0546205454,  -0.640401483,  2.1252656,  0.266235054,  -1.39803827,  0.494947165,  1.08043861,  1.3152355,  -0.739432395,  0.777239323,  -0.0982151777,  1.22893989,  -0.116099522,  -0.484638363,  -1.05408883,  0.520829976,  1.73572958,  1.15721381,  0.0310347583,  -0.0897617787,  0.853501618,  0.740006328,  0.84476459,  1.2114799,  0.106914036,  -1.07000041,  -0.607851565,  1.0721432},{ 1.25818312,  -0.549459875,  0.0890886411,  0.753789723,  1.02178907,  -0.771645308,  0.689314246,  1.73756433,  -0.502142429,  -0.773573816,  -0.598315239,  -0.320816696,  0.985359192,  -0.864401519,  0.304969788,  -0.826854229,  -1.06174648,  0.233283922,  1.02404094,  -0.99384129,  0.688817501,  -0.416990042,  -1.37592137,  -1.5421716,  0.102390774,  0.304403007,  -0.287238359,  -1.04179513,  -0.88051039,  -0.311689228,  -0.633430123,  0.500590086,  0.0978246555,  1.09520972,  -0.3342053,  -0.443980485,  -0.71256125,  -1.36579752,  1.05597186,  -0.6452443,  -0.0136261471,  -0.75979811,  0.190707728,  -0.205517545,  0.630702972,  1.78081262,  0.399026543,  -2.46422219,  -1.85763991,  -1.27148855,  1.38959515,  0.275913656,  1.9086864,  1.4847616,  -0.0552326962,  0.469633013,  0.839196742,  0.469732195,  1.2817359,  0.593297601,  1.55889904,  0.670904338,  1.43532825,  0.116875432,  -0.177631125,  -1.10578632,  0.0266526006,  1.95226109,  -0.362447977,  0.0892532095,  1.50202739,  -0.0242939275,  1.33527327,  -1.470837,  0.822711051,  2.4453938,  -1.10494697,  0.100341216},{ -0.745627582,  0.174136445,  -0.365520179,  0.242157161,  0.814955235,  0.184005931,  1.13197792,  -0.94870919,  1.46688271,  1.81031501,  -0.772858083,  0.384426981,  -0.527935088,  -1.24212313,  -0.428991497,  2.41933513,  -2.39704275,  -1.34764516,  0.288212538,  0.305309415,  1.06576574,  -0.680109441,  -0.780046821,  -0.372862786,  1.21676624,  -0.26557678,  -0.89671737,  -1.44536364,  -0.0471171774,  -1.27747405,  0.679606259,  0.784075677,  0.318597496,  2.31330967,  -0.272772193,  0.430708915,  -1.2244314,  0.608335555,  0.406439722,  -0.338781774,  1.5211556,  0.550073564,  -0.418588907,  2.05249166,  -0.653340816,  -1.49601007,  -0.206637338,  -0.623045862,  -0.767674744,  0.23638314,  -1.57199836,  -0.472754657,  -0.718575537,  0.319082737,  -0.42045626,  0.0966256484,  -1.37157774,  -0.551256478,  -0.510206521,  0.570307136,  -0.763909042,  -0.210046709,  -1.42259705,  -0.0164611563,  -0.945857704,  -0.0366789363,  -2.15137506,  -1.09029043,  0.447393596,  -0.141653523,  -0.126330778,  -1.0431689,  0.0959873423,  -1.01094556,  -0.267394423,  -0.369187891,  0.368369013,  0.16719313},{ -0.97070688,  0.862585306,  0.961373448,  -1.16585529,  0.431205124,  -0.0863690302,  -0.799777806,  -0.276163906,  -1.04621756,  -0.819437206,  -2.23162985,  1.07445204,  -0.1321363,  -0.12026228,  -0.203825474,  0.547536969,  -1.64864659,  1.86068332,  -0.226345479,  -0.326230228,  -1.00416303,  0.432063103,  0.0377224162,  -2.06809235,  -0.848058701,  2.45136142,  -0.264757127,  0.441660166,  0.108399525,  -0.281084836,  -0.999158621,  1.88151586,  0.301813483,  -0.296860874,  -0.247324064,  -0.897008955,  0.00332947168,  -0.56510514,  -0.297197193,  -0.291293055,  0.592378676,  0.213991359,  1.18280625,  -0.455242842,  0.67917943,  -0.775008261,  -0.2704705,  -0.644721389,  -0.52510798,  0.890754044,  0.577165544,  0.243367255,  -0.692234576,  -0.53719151,  -0.23659046,  -1.06890845,  0.908711731,  -1.32888627,  0.552653611,  -0.579813182,  0.588371456,  1.54661715,  0.131136984,  -1.37779403,  2.45661211,  -1.37975514,  1.48195267,  -0.511616945,  -1.05923176,  -0.366974354,  -0.671022236,  1.16556692,  -0.376529664,  0.901467681,  0.16047211,  -0.419974416,  -1.98533273,  -0.0184711702},{ 0.601390839,  -1.07030404,  0.464072049,  1.66195393,  0.883929312,  -0.896737278,  -0.77914536,  0.368295938,  0.203915119,  -0.288817793,  0.718132019,  -1.2833221,  -0.216846764,  -0.804534972,  -0.685765743,  -0.639698625,  -0.821107566,  0.244220167,  -1.86749697,  -0.795372605,  0.0150806252,  -0.952372372,  0.5407601,  0.0194081515,  0.0936091766,  -1.0687598,  -0.164830297,  1.00344932,  -1.36475766,  0.0922754109,  -0.864895821,  1.62064314,  -0.8162058,  0.266485602,  0.311089665,  0.569638431,  0.280657589,  -1.70308781,  -0.559496284,  -0.224057049,  -0.0139068067,  -0.939466417,  1.91054249,  0.020241268,  -0.390728772,  -0.618566096,  0.593245149,  -0.797284186,  0.281700581,  0.769824147,  -0.547740579,  -1.89198041,  0.607945144,  -1.17276192,  -0.0771279186,  -0.0343877375,  -0.682001412,  -0.549513102,  -0.646283746,  0.935911298,  1.63534105,  1.28817046,  0.704446077,  0.453243524,  1.01023972,  0.357191056,  -0.576414526,  -1.26080263,  -0.637964189,  0.818878829,  -0.747537851,  0.236773491,  -1.13677371,  -1.2933557,  -1.09431112,  -1.5182873,  -0.637745976,  1.72794402},{ -0.449397564,  -0.951136529,  0.01369204,  -0.397158027,  -0.197535232,  0.383399397,  0.0492393374,  0.0653727874,  -0.765176237,  0.350357443,  -0.594422817,  1.75105047,  -0.0670923889,  -0.449917823,  -0.144949079,  0.0772781745,  -1.62087059,  -0.483115554,  0.380386114,  -0.410002828,  0.45721963,  0.367040902,  1.75655031,  -0.971453905,  -1.08011353,  1.08636737,  0.14369534,  0.0142761683,  -0.298586577,  0.947372139,  -0.201643139,  -0.7059654,  1.43765581,  -2.45241642,  -0.290260553,  -0.581521034,  -0.280155391,  0.949276328,  0.337642819,  -0.238755241,  0.0648461357,  -0.659337759,  0.806072891,  -0.0634485036,  -0.924824655,  0.489872396,  0.293634802,  0.311503112,  0.415532351,  0.537751436,  1.76382577,  -0.720526338,  1.31889904,  -1.15547752,  0.720594823,  0.146594301,  1.11145329,  -0.0512391403,  -0.174909413,  0.472861141,  -0.390674859,  -0.0493421406,  -1.79501927,  2.58386707,  0.529013932,  -0.451462299,  -1.11559105,  0.281756043,  1.00962734,  -1.56604075,  0.100143969,  -0.642324448,  -0.854679525,  -0.190671861,  -1.96900511,  -0.827026725,  -0.780014813,  -0.186002076},{ 1.51964355,  0.562444806,  0.572633028,  -1.51004934,  0.262406707,  -0.0611522868,  -0.739073038,  -0.37246716,  0.146783754,  -1.0615927,  -1.40538466,  -0.246472076,  0.858068764,  1.94038475,  -1.06705201,  -1.00858438,  -1.15285385,  2.14976835,  -0.401358962,  1.29952264,  -1.68556976,  -0.25405243,  0.89412123,  -0.839494228,  -0.48304671,  -0.0347075686,  0.125281543,  0.348474979,  -2.11817837,  0.107838899,  0.59678483,  0.768587589,  -0.80339402,  1.59880853,  -1.49505782,  0.326628119,  -0.431973904,  -1.10688865,  -0.269553483,  -0.548889041,  -1.80113804,  0.293860227,  -0.0339378864,  0.858650148,  1.19487166,  0.79212141,  -1.01417565,  1.03498137,  0.207200646,  -0.199809581,  0.185785636,  -0.852642059,  -0.106555849,  -0.447035611,  -0.687400997,  0.504063547,  0.441267073,  0.302760541,  -1.08082175,  2.54048586,  0.688472331,  -1.56430304,  -0.747931838,  -1.37454414,  -1.05883276,  -0.510312676,  1.43954837,  1.1918962,  0.907317698,  1.01949489,  0.803018212,  0.497550428,  0.0567461811,  1.40396452,  -1.54004359,  -0.680317879,  0.26714927,  0.657323718},{ -1.15420997,  -2.07024455,  0.559096932,  -2.11134553,  -0.650625825,  -0.854714155,  -0.877672493,  -0.111401498,  -0.316542774,  -2.0739255,  1.36479485,  -0.00510424236,  -0.364721596,  0.568853855,  -0.416448325,  -1.01663768,  -0.359223783,  -0.363945931,  -1.26926899,  1.22539866,  1.23121464,  1.76906502,  0.996391594,  -0.749069571,  -1.03280544,  1.21647131,  0.818235636,  0.0149329174,  0.877210438,  0.612016559,  0.636744559,  0.944100857,  -0.526018143,  -1.06980646,  -1.56694949,  0.741181076,  -0.509688377,  0.0775134414,  1.90440679,  -1.41295528,  1.35274374,  0.886415184,  -0.839767039,  0.661912382,  -1.72659612,  1.17789733,  1.6179142,  0.0867906809,  1.67616558,  -0.548636675,  0.496886402,  -0.759878218,  0.86860615,  -0.627927303,  -1.66341901,  0.664095581,  -2.02509117,  -0.18432644,  -1.33661604,  1.27964807,  0.0568307154,  2.31027317,  -0.609494746,  -0.889847696,  -1.75199056,  0.209543496,  0.450734705,  -1.85841334,  -0.794060886,  0.705753207,  0.489998043,  0.245365217,  -0.672068357,  -1.59891868,  0.554900467,  -0.708255112,  -1.1694777,  -1.03374779},{ 0.362340957,  -0.682374835,  0.169300526,  0.471600324,  -1.50706029,  1.26012015,  0.430890352,  -1.65568137,  -1.58496857,  1.04564106,  0.418404788,  0.0310492702,  1.54396605,  0.627941012,  1.85425377,  2.72517157,  -0.142557994,  -1.46995664,  1.63706887,  -0.657906711,  -1.12624002,  -0.935337782,  -0.165814683,  1.22737873,  0.50882411,  0.790255249,  0.648291469,  1.51170504,  -1.20903492,  0.436840385,  -0.479566187,  0.955926597,  0.198272809,  1.63010216,  -0.874446571,  -0.0452101566,  0.392490327,  -1.49078465,  1.537099,  -0.0842678547,  -0.661397696,  -0.127806246,  0.395092994,  0.467500955,  -0.122821212,  -1.11289501,  -1.42690361,  1.86217666,  1.48169971,  0.865366161,  1.70394337,  2.4020946,  -0.366644382,  -1.38916445,  -0.0406130105,  -0.999502361,  -0.0749754682,  0.966787398,  0.512630641,  -0.23372893,  0.366884679,  -1.18722177,  0.625184298,  0.718119979,  -1.86128664,  -0.864453673,  0.137666583,  -1.44907463,  -1.41159046,  -0.223336414,  -1.6755619,  -0.138235554,  -0.0368046425,  0.571763337,  -0.554067969,  1.11969817,  -0.0270378571,  1.75517499},{ -0.15407005,  1.12767351,  -0.932377458,  -0.919444025,  0.661145031,  0.429650575,  -0.704092741,  -0.188049018,  0.128795624,  -2.01348901,  -0.533899605,  -0.471253514,  -0.146935582,  -0.761687517,  0.64254868,  0.780390024,  0.731163323,  -0.115352675,  -1.2234385,  0.293035805,  1.05861616,  0.182544157,  0.238638952,  -1.20886981,  0.427346349,  0.482718915,  0.898789406,  0.617894292,  0.418441206,  -0.348391175,  1.09382975,  -1.44981468,  1.33289635,  -0.931963146,  -0.55779624,  0.425675541,  -1.25024772,  0.796503961,  -0.719893873,  0.697707891,  -0.139312759,  -0.0177431423,  1.46254635,  1.09577405,  0.0480091162,  -0.524246395,  -0.924328923,  1.04960191,  -0.857660413,  -0.625714898,  0.413700104,  1.61493719,  -0.339116424,  0.52108103,  1.22080362,  -0.697602093,  -0.937473774,  -2.17704439,  0.862195551,  1.468279,  -0.591456175,  -0.814785957,  -0.0645403117,  -0.188826323,  0.065376088,  -0.527823925,  1.16178405,  -0.127203763,  -0.17320019,  0.621796787,  -0.202867344,  -0.537065268,  0.163873285,  2.71373844,  -1.05100787,  -0.133599699,  0.555124342,  1.85979915},{ -0.748928726,  0.790579319,  -0.154445261,  1.35934603,  0.185409069,  -2.86276054,  0.575507879,  1.06638312,  -0.071021229,  -1.04859865,  -0.21402064,  -1.03262067,  0.986397505,  -0.682135582,  0.799522102,  -0.56452775,  0.230384409,  0.61162281,  -0.248129115,  0.0562248938,  0.0295219682,  0.278400719,  0.167556629,  -0.948687553,  -1.54637563,  1.28738773,  -0.222787678,  0.126719281,  -1.69556046,  0.124897286,  0.732122362,  0.0375774354,  -1.04201818,  -0.831364334,  1.10913861,  -0.351786315,  0.919552803,  -0.545623422,  -0.433998555,  -1.16265726,  -1.06496382,  1.39632821,  0.979723811,  -0.694326282,  1.53797948,  -0.904813945,  1.52424407,  -1.16893518,  0.703545153,  -2.6027844,  -2.1740129,  -0.0035710712,  -0.587877452,  -0.753768802,  1.04734898,  0.828280151,  0.788099706,  -1.3615464,  -1.05652475,  0.462709665,  1.42371976,  -1.00762045,  -0.123902194,  2.30973506,  -0.235247225,  -0.941355348,  -0.827794969,  -0.221857116,  1.2527355,  -1.58437753,  -0.329542965,  -0.847998321,  -0.511178851,  0.173202351,  -2.19439244,  -1.25620806,  0.803777158,  -0.696148217},{ 1.96399605,  0.812968254,  -0.354558557,  -0.858981192,  -0.654013872,  0.603729308,  0.706637502,  0.827254236,  -0.362624019,  -0.400243163,  1.00345469,  1.05087137,  -0.72199434,  0.0551868342,  0.330718011,  0.872687221,  0.189093158,  -1.33208382,  -0.760155022,  -0.652244329,  0.430356652,  1.06254828,  1.71857762,  0.517793059,  -0.693403482,  -2.23199129,  0.420670003,  1.07070065,  0.284955233,  2.06336641,  -0.0212631635,  -0.826402724,  0.50448519,  -0.915400922,  1.59510696,  -0.199641585,  0.81068486,  1.84319472,  -0.660331607,  0.39194423,  -0.704619408,  -0.732409894,  0.37757045,  1.29096997,  -2.18175173,  0.387850612,  -0.693128884,  0.421694368,  -0.416240484,  0.0847959593,  -0.484228134,  0.204835474,  -0.565466523,  0.78123194,  -1.88921452,  -1.66493189,  0.451180696,  0.333690047,  0.301348984,  0.961704493,  1.51136434,  -1.43553042,  0.539542496,  -0.205724731,  -1.28198171,  -0.985827029,  1.44126093,  0.418472201,  -0.350999087,  0.603862166,  -2.52415919,  0.797416449,  0.0031171157,  0.690938056,  0.541349471,  -0.147103921,  -1.02271807,  0.000436509959},{ 0.651560664,  0.055275958,  1.14112544,  1.1237123,  1.68868458,  -0.396505415,  -0.170704931,  1.83719659,  1.44801402,  0.117664687,  -1.43880832,  -0.0167357139,  -1.48089111,  -0.09287595,  0.513278842,  -0.692398965,  0.245605394,  0.978507996,  -1.68964744,  0.132547513,  0.336514533,  0.0815675855,  0.951539159,  0.372867525,  0.608144045,  0.647488892,  0.929051757,  0.0342473648,  0.2757622,  0.585506797,  -1.51512456,  -1.84552073,  -1.36766028,  0.231921703,  0.487358838,  -1.18943608,  0.875013232,  -0.339909852,  -0.795517325,  -0.379137784,  -0.213643521,  -0.0517079048,  1.54206789,  0.800971627,  0.092976287,  0.119596265,  0.950100958,  -0.241020918,  -0.371508837,  0.872181058,  -0.390747786,  1.78180861,  1.76336789,  0.369850218,  -0.00925700087,  -0.147088349,  -0.118140675,  -0.328331649,  -0.174571842,  0.0115135508,  -1.73776102,  -0.0151989702,  -0.0542535782,  -2.51010108,  -2.04291892,  0.431105256,  0.725866437,  -1.81674385,  1.32434857,  -0.830912709,  0.79278028,  0.365794569,  3.26220226,  -0.0874896571,  -0.161931187,  0.947589159,  0.607577801,  -0.262395263},{ 0.933877409,  -0.982802987,  -0.763755858,  -1.87195456,  -0.0612887293,  0.315499306,  -0.21411702,  1.12681723,  1.24449968,  1.85363841,  -1.12088132,  0.808908224,  -0.785001457,  -0.206322849,  -0.249102429,  0.102444388,  -1.10431445,  -1.70958745,  0.576469243,  -0.770240784,  -0.174415976,  1.10875475,  1.8045218,  0.422906518,  -1.63719928,  0.0708186328,  0.55629307,  0.432833761,  -0.90443033,  -0.289084107,  0.396247149,  0.835250556,  0.28731218,  -0.633865237,  -1.09782267,  0.716192365,  -1.07179892,  0.284366846,  2.13205671,  1.11421621,  0.0466389358,  -0.533667564,  -0.956111372,  0.113204688,  1.67005849,  -1.10638678,  0.906328321,  1.07839942,  0.278772801,  1.99567664,  0.267706841,  1.3192035,  -0.524903119,  -0.318920702,  0.404060274,  1.12246537,  0.185154423,  0.329589963,  0.184283167,  0.5952968,  -0.838844001,  0.859189034,  -0.970261037,  -1.03969157,  -1.69168067,  -1.69043052,  1.8321991,  -1.33487415,  1.76809275,  -1.28696489,  0.946753144,  -0.527524471,  1.26684165,  -0.743838131,  0.649492323,  -0.0279845223,  0.575927317,  0.0317160636},{ 1.4477036,  -0.666917861,  0.814370692,  -0.126111448,  0.248365656,  0.904632509,  0.954066396,  1.00420153,  -0.930978537,  0.840224147,  0.0385560989,  -0.745379746,  -1.72582245,  0.25987336,  0.735011578,  -1.11350155,  -0.116073951,  -0.968008041,  0.177191183,  -0.00504729757,  0.865588427,  -1.23075545,  -0.255861402,  -1.11626768,  1.68637967,  0.434857965,  0.16358386,  0.195087194,  1.63825572,  -0.365346283,  0.588163137,  -1.21565378,  0.384456128,  -1.78409195,  -0.278357476,  0.281585544,  0.0436102003,  -0.6754089,  -1.48917437,  0.356052548,  -0.383768827,  -0.650332093,  0.806891978,  0.650711119,  0.574069738,  0.518398225,  -0.584366918,  1.25162089,  1.57161808,  -1.29408431,  -0.370034933,  1.76455283,  1.57475877,  -0.154588103,  1.37253451,  -0.347330898,  0.660181642,  -0.460041493,  0.607521355,  0.33988446,  -1.15232015,  -0.121181421,  0.455686301,  0.552222371,  -0.46728763,  0.866711557,  -0.371891439,  1.25201094,  0.915178478,  0.691582382,  -0.420377374,  0.716179192,  0.613483489,  0.312654614,  -1.57150507,  1.42099452,  -0.0556154624,  -1.08812547},{ 0.0577394031,  -0.302553117,  -0.85780704,  -0.403477818,  -0.0833189934,  -0.550584435,  -1.38666856,  -1.20525002,  0.800271749,  -0.100251786,  -0.961518764,  0.458438337,  -0.0302443858,  -0.420478463,  -0.360656053,  0.742233634,  0.00944703259,  -0.487439364,  0.577498496,  0.078774713,  0.196611464,  0.0389760472,  -2.06599259,  -1.77635217,  0.372183114,  -0.885543764,  -0.277976662,  0.500844121,  1.21388614,  0.90074873,  -0.00180395099,  -1.19508171,  0.675043225,  -1.06691027,  1.62411726,  0.0757920742,  -1.10854447,  0.200710654,  0.202116191,  0.149344787,  0.162424713,  1.87290347,  0.520151675,  0.403153718,  -0.679986238,  0.963848531,  -0.710775435,  0.362191439,  0.644040227,  -1.8163954,  -0.125939697,  -0.524406612,  0.226758584,  0.20036909,  -0.706048906,  -0.549180567,  0.544381082,  1.23438168,  -0.273690253,  -0.658561468,  0.913354039,  -1.87307453,  1.49139428,  0.665357471,  0.499267608,  0.209511757,  -0.0370083526,  1.62150264,  1.47135556,  0.403555453,  -1.50414562,  -0.376711726,  1.35947406,  -1.09695792,  -0.370692879,  1.24365377,  0.846358657,  0.140588045},{ -0.26319465,  -0.284503043,  0.403074622,  -0.278523564,  0.218784034,  -0.808889091,  -1.14272213,  -0.206032708,  2.19310999,  -1.43124783,  -0.319206953,  -0.336764336,  -0.119370632,  0.904172063,  0.0250381306,  -2.39756322,  -0.0168969538,  1.03857136,  1.75026548,  1.73164535,  -1.10315967,  -0.736548185,  0.589781821,  -2.30182123,  2.18833995,  1.91411281,  0.2240659,  -0.114566237,  1.18324661,  1.55061126,  0.642448187,  0.952269793,  0.526278138,  0.177669391,  2.07516289,  -0.368945271,  -0.287451357,  -0.267903388,  -0.820418715,  0.182869986,  -0.251583576,  1.73201203,  -2.14779067,  0.852066517,  -0.320461303,  -0.214951158,  1.81548405,  0.739005685,  1.60699475,  -0.366275698,  0.324719369,  0.295382112,  0.307063341,  0.0206347946,  -1.02855504,  0.650476754,  -0.168138787,  -1.33792186,  -0.812748253,  1.03123009,  0.690351009,  0.674045384,  0.516822994,  0.890884519,  0.842493892,  -0.520319939,  0.95966959,  1.45627248,  -0.358052909,  -2.49893594,  -0.599570036,  -1.31176674,  0.654282272,  -0.961617112,  -0.167312369,  0.880265594,  0.0632353574,  -0.20006907},{ 0.811111152,  -0.969391108,  0.291278332,  0.56919229,  -1.66113293,  -0.61231488,  0.181324184,  0.528229773,  -1.11088789,  0.395249724,  1.50163472,  0.564353943,  -0.692232728,  0.696686506,  0.757144034,  0.394079059,  0.259218514,  0.0899444297,  2.14212823,  -0.312097669,  0.459689111,  -1.28408158,  0.887815535,  -1.08722067,  1.04947865,  1.64792502,  -0.18317984,  -2.48359656,  1.69876504,  0.815813184,  -0.503258467,  -0.140722513,  2.31108761,  -0.228618875,  1.19053388,  0.417139709,  0.617884278,  -0.841867626,  0.825841427,  -2.31385732,  0.114712916,  1.91586411,  -1.83625281,  0.217468485,  -0.148998976,  0.985093951,  -1.53145075,  -0.284001023,  -1.42094755,  2.12074423,  -0.789402544,  -0.599906266,  0.795397937,  1.21537483,  -0.473650157,  0.963925421,  0.0105748707,  1.45730519,  -0.942901552,  0.647963524,  0.317356974,  1.58803618,  -1.19088244,  0.135159239,  1.56414843,  -1.24972212,  1.66006601,  -0.385211766,  0.327539355,  0.0423544943,  1.02389193,  -0.33286494,  -0.278142542,  1.12309289,  -0.336528271,  -0.700612664,  0.299661875,  -1.19244635},{ 0.945112467,  -0.252482504,  -0.980415821,  0.0158562902,  -0.466310114,  -1.05908227,  0.282085508,  -0.0585291535,  -0.508040309,  -0.314694434,  -0.629788041,  0.22535105,  1.30421221,  1.20230627,  -0.322484404,  -0.700391829,  -0.775483072,  -0.692992985,  0.654881179,  0.513824582,  -0.980199277,  0.68911165,  -0.630796254,  0.268932849,  -1.06158245,  -0.865587234,  -0.601191401,  0.936876833,  0.667896807,  -0.400876731,  1.72964454,  1.25143635,  -0.933414042,  0.181760922,  -1.30561376,  0.566213608,  -0.658163249,  0.345856249,  -1.29443514,  -1.2222693,  -1.98957396,  0.84841615,  -0.0588410608,  0.805865943,  -1.43866563,  1.69202518,  -0.962679803,  -0.84417218,  -0.778245032,  1.01696813,  0.0505065508,  -1.18196952,  1.5172776,  -1.72187769,  -0.287525773,  -0.525514603,  -2.47028923,  -1.43390977,  0.173486874,  0.297576964,  1.72858429,  -1.01316988,  0.76214999,  -0.931807876,  0.959774375,  0.887454033,  1.31645691,  -0.397658229,  -0.0116654029,  0.652303755,  0.4996126,  0.930229068,  0.980845273,  0.88634634,  -0.343054056,  1.28668785,  -0.596978724,  0.801971912},{ -1.28847599,  1.19348216,  -0.644410253,  -0.338219583,  0.00340896449,  0.453258306,  -0.289565533,  0.636333048,  -0.654827416,  0.817054868,  0.596338272,  -0.137602687,  -0.732068896,  2.6321702,  0.353317142,  0.75754267,  0.549481332,  -0.781013489,  1.26099181,  1.13178897,  0.377458543,  -1.16224611,  -1.48391771,  -1.82430542,  -1.77896762,  0.144752845,  -0.913058937,  0.709869385,  -1.71936452,  -0.671148896,  0.382965952,  0.0519487187,  -0.893000305,  -0.427088767,  1.61857438,  1.82303619,  1.19382298,  -1.07852566,  1.20925748,  1.30309153,  -0.442515582,  -0.189045429,  -0.418818593,  0.690651417,  -0.723050475,  0.0867477357,  1.2959429,  -0.820397556,  1.66112077,  -1.21931136,  -1.99393964,  1.01064503,  0.614351213,  -0.83251518,  0.110046744,  -0.314419866,  0.14047341,  0.603630722,  1.13132501,  -0.520378649,  -0.851992011,  0.795530379,  -0.0563201718,  0.304034442,  -0.169502273,  -0.281169385,  -0.36526981,  -1.51643062,  -0.513642371,  -0.395927787,  -2.1616087,  -0.384401411,  0.869371891,  -0.789982975,  0.654247403,  1.49579024,  0.0188148338,  -0.0804179385},{ -0.436213762,  1.71458566,  0.377110273,  1.33634245,  0.444905132,  0.0379712842,  -0.590632737,  -0.196517199,  0.188201174,  -0.580277026,  -1.2180531,  0.410485595,  1.00403643,  -0.1605497,  0.0794225037,  0.838881969,  1.07754397,  -0.771821558,  -0.385957748,  0.272624463,  -0.27191323,  2.06836915,  0.58815378,  -0.270021558,  -0.0351525955,  -2.09215117,  2.3551507,  1.30140936,  -0.250214726,  -1.97426212,  -1.35052621,  -0.145039901,  1.34750271,  -0.868592858,  0.413530707,  -0.884430408,  -1.32206678,  -0.408600807,  1.56687999,  -0.367945254,  0.180194095,  0.510641336,  -0.181962088,  0.467971772,  -1.4083333,  -1.21755672,  -0.0836538002,  0.921662986,  0.559975088,  0.933844864,  0.568389773,  -0.916754007,  0.37089631,  0.0297852196,  -0.245909929,  0.266913056,  0.59969449,  0.808415234,  -0.6471892,  0.0638142303,  0.198874027,  -0.548957765,  -0.988812566,  0.687372029,  0.423319817,  0.775019228,  -0.723209381,  0.651514053,  1.07834828,  0.799892128,  0.241472453,  1.12007105,  -0.15204744,  -0.263212591,  -1.30932713,  -0.601277649,  -0.392984837,  -0.137371793},{ 0.407318264,  -0.166215941,  -0.263957083,  0.52100271,  -0.176784277,  0.757791877,  1.21303988,  0.00201607123,  -0.415268004,  0.898413002,  1.50053501,  -0.074831754,  -0.61409235,  1.46552479,  -0.568364382,  0.408802539,  -1.21260381,  -0.474339187,  0.108336993,  0.553785861,  0.643706679,  1.26344192,  0.710139573,  0.67296344,  -0.335711628,  -1.0937314,  -1.63409209,  1.08830011,  -0.31641227,  -1.88450825,  2.00125432,  -1.42153478,  1.18684292,  1.75588596,  2.54612947,  0.301201969,  0.455678463,  -0.602792084,  -2.30764723,  -1.0987308,  0.27306062,  0.265323192,  -1.1645211,  0.109919831,  1.09664941,  0.772293627,  0.0546567515,  -0.0923513696,  0.0739705712,  -0.992842197,  -1.20984578,  1.36310101,  -1.38969934,  -0.358505845,  -1.53255332,  -0.159043953,  0.545714319,  -1.11783648,  -1.5489192,  1.68109179,  -0.329469532,  1.31602216,  -1.70504439,  0.695874751,  -0.593081474,  -0.442786783,  1.39055741,  -1.39248002,  -1.431759,  1.44505095,  0.853227317,  -0.36922431,  -0.519476831,  -0.101795018,  2.30442309,  0.143083215,  -0.958398342,  -0.354868531},{ -1.18661714,  -1.9344039,  -1.39964449,  0.562096298,  -0.652187943,  0.625783324,  0.719668984,  -1.63757241,  -0.836158812,  -1.86797488,  0.265979975,  0.206495076,  -1.06165051,  -0.10236226,  -0.730281532,  -0.0993789956,  0.739735186,  0.966834009,  -2.35180163,  0.307408154,  0.161933795,  -0.756437063,  0.216277093,  -0.546295047,  -0.172394022,  0.711582601,  2.02425718,  -0.483462989,  1.02016008,  -0.318164796,  -1.80509341,  -0.0625056699,  -0.521449864,  -1.01579762,  0.405920327,  -0.216194272,  0.786357343,  -0.13204734,  -0.0345521942,  -0.763518155,  2.12947559,  0.189258248,  -0.364796162,  0.232409999,  -0.428402603,  -0.319673091,  1.09463954,  1.11021566,  1.32276964,  -0.412748039,  0.133295104,  1.52131045,  1.34897006,  2.35347652,  1.14080966,  -1.20833278,  0.555919886,  0.0928576291,  1.18961263,  1.90383685,  -0.903159857,  0.500271261,  -0.725442588,  0.538135648,  0.568242669,  -1.29685199,  -1.03728449,  0.209805697,  0.552349925,  2.49490952,  -0.817356229,  -1.29223335,  -1.70593262,  -2.26603723,  1.73310399,  0.495995075,  -1.61398959,  0.556943893},{ -0.981486022,  0.808193326,  -1.40161264,  0.76167959,  -0.604992151,  0.484781533,  -1.39539039,  0.571856439,  1.81162524,  -1.56363916,  -0.8452124,  1.66314471,  0.0806309804,  1.33905077,  -0.564993858,  -0.433990568,  -0.849736929,  -0.835972428,  -0.411268562,  -0.436257422,  0.220552132,  0.687201977,  0.231223449,  0.92294848,  0.258932054,  -0.554103553,  1.53570223,  -1.20770073,  0.728749096,  -0.0683417022,  0.239471719,  -1.17455304,  0.0547385998,  0.296962917,  1.82571828,  -0.911677599,  -0.791864932,  -1.07854903,  -0.424548447,  0.621715903,  0.0947837085,  2.13742208,  -0.600471735,  -1.56648624,  -0.445003927,  1.0577141,  0.691149712,  1.41658616,  0.229450807,  0.0810621455,  -1.75390315,  1.08689117,  0.446105361,  0.143712282,  0.00244128099,  0.257968724,  -1.21989667,  1.7491852,  0.586940646,  -0.191146597,  -1.26158524,  1.67658567,  0.955818176,  -1.27396739,  2.29317045,  1.27984571,  0.626257181,  0.0755013674,  -2.54954123,  1.07022393,  0.893577337,  0.485514522,  1.51256096,  1.26701164,  0.537266076,  -0.706194401,  0.772312105,  0.905902088},{ 0.232356906,  2.31156039,  1.37862706,  1.75739992,  -0.00740180025,  -0.280682296,  -0.557050169,  -0.925307214,  0.694670141,  -2.69803214,  2.14430761,  0.443484336,  -1.50386,  0.725966275,  -1.37101603,  0.82438308,  -0.671676993,  0.15458487,  -0.369892448,  1.84527898,  1.10667586,  0.589758158,  -0.158106863,  -1.74868572,  -0.18819578,  1.48474491,  0.844197094,  1.29596734,  0.356594533,  1.1161077,  0.971240163,  -1.52535796,  -0.764590621,  -0.198363259,  -0.912367105,  0.69544971,  1.14120841,  0.580176055,  -0.860538185,  0.363939226,  -0.469580293,  -1.23937297,  0.950919569,  0.227164373,  0.209234178,  0.153737217,  0.305033714,  0.521878541,  0.36372748,  1.81143117,  -0.124018319,  -0.160772249,  -1.61908424,  0.740824223,  0.224667415,  -1.62952352,  0.080533728,  -1.22468138,  -0.825347722,  1.45587575,  -0.217063114,  -0.435970664,  -0.0315772258,  1.25140572,  -0.272895128,  1.9915961,  -0.0695223659,  -2.25635624,  -0.170052975,  1.27369821,  0.0256718807,  0.355735481,  -0.124007031,  -0.945607662,  0.48421365,  0.0975023285,  1.29658854,  -1.58318174},{ 0.355364591,  -1.02642715,  -1.36552727,  -1.68542302,  0.583353519,  0.0179426037,  -2.1612618,  -0.231670544,  0.0366220623,  -0.297190785,  -0.096795693,  -0.471862763,  0.422391444,  -0.184726432,  -1.25521779,  -1.14452791,  1.46451223,  -1.17100811,  0.702449203,  -0.330766439,  1.23332822,  0.957551539,  -1.14776409,  -1.01196241,  -0.700838983,  -0.359137952,  -1.42742383,  -0.903330743,  0.394908756,  1.31136394,  0.0266129449,  -1.14535713,  -0.216444969,  -0.862352967,  1.3434937,  -2.6999681,  -0.677528977,  0.570585132,  0.0289945472,  -0.079618156,  1.22004688,  2.15729403,  1.28010225,  1.39459002,  -1.19338584,  0.560388684,  -1.0525887,  0.150007442,  -0.302810639,  -0.481259465,  -0.989332676,  0.177942559,  -1.90719295,  0.314222455,  0.18488428,  1.95432758,  -0.383517385,  0.727487087,  -1.04641974,  0.605955184,  -1.41628397,  0.160584971,  0.351925075,  0.684399426,  0.813650012,  1.49078822,  0.547075033,  3.07186127,  0.305845886,  0.87023288,  2.73963618,  -1.04133856,  -0.620545924,  2.50117874,  -1.90092814,  1.2618283,  -0.774528623,  -0.382285118},{ -0.181525931,  -0.458490849,  -1.20490432,  -0.505297065,  0.592795908,  -1.15257645,  -0.485194147,  0.34767437,  1.0669018,  0.476308167,  -1.84221137,  0.528404713,  0.587958753,  0.964917541,  0.352633923,  -0.631211281,  -0.175953493,  0.878256977,  0.804823458,  -0.331787705,  0.116294399,  -0.227914929,  -0.216771245,  -2.8022325,  1.33226418,  0.156076565,  1.31246877,  1.46059239,  -0.443167716,  -0.307792693,  -1.17628479,  -2.96293712,  -0.487638175,  -0.151230678,  -0.313276112,  1.12453294,  -0.278325737,  -0.522289693,  0.324763983,  -2.24468064,  1.73373413,  0.321142852,  0.923640847,  0.284514636,  0.727588415,  0.807100296,  1.60271668,  0.421235055,  0.688777387,  -1.16099906,  1.67376173,  -0.164214015,  0.541861296,  -0.12750186,  -1.22060716,  -1.26346886,  2.19088793,  -0.384904504,  -0.699382186,  0.434487998,  -0.509521008,  0.971040666,  -0.12682271,  -1.09161079,  -0.407745808,  -0.212121382,  -0.831034184,  1.623739,  -0.0558050312,  0.505521119,  -1.00291169,  -0.248395652,  -1.34915841,  -0.946351886,  0.593776941,  -0.285541236,  -1.16728592,  -0.931363583},{ 0.780878425,  1.23969555,  -0.174633294,  0.273333907,  -1.0135771,  -0.425648749,  0.528699875,  0.836863637,  -0.173343822,  1.05545807,  -1.27701581,  -0.122326195,  -0.789396226,  -0.462847531,  0.737335265,  -1.99078083,  -0.116088532,  1.8138926,  -0.915661573,  -0.193871647,  1.43662035,  1.53288949,  -0.0400927514,  -0.222778425,  1.69082415,  0.638117015,  1.8743906,  -2.05546761,  -0.300434619,  0.49182716,  0.882161856,  -0.143425912,  1.43475878,  -0.261328429,  -0.545522869,  1.04713321,  0.580473781,  0.638132393,  0.512199223,  0.813975573,  -0.33221674,  -0.950462699,  0.779789269,  -0.381982476,  -0.722943127,  -0.891966701,  -0.163905695,  1.96484625,  1.5675118,  1.48123062,  0.3925744,  -1.11150444,  -0.582638323,  -0.414418817,  -0.92101723,  0.399893492,  0.395077437,  -0.473513633,  0.116670839,  0.219054505,  2.15470576,  -1.40438437,  0.181744859,  -0.985226214,  -0.0521381311,  0.561168253,  -0.215768412,  -0.055150263,  -0.701415956,  0.229323596,  -1.24732959,  1.31705678,  -0.459685087,  -0.977362812,  0.0149829816,  -1.79841208,  -0.0469631888,  -1.3097595},{ 0.380685151,  -1.27911949,  -0.674483299,  0.310629547,  1.34720635,  0.205515429,  0.00460617756,  -0.828765869,  0.506020486,  0.812947333,  1.44934034,  0.307410628,  -0.700176656,  -0.674932182,  0.572100222,  -1.98170841,  1.21474242,  0.322004169,  -1.25460255,  -0.58456111,  -0.510462821,  -0.858590186,  0.240118742,  0.228590161,  -0.0701830834,  -0.989718199,  -0.0171918515,  0.47810173,  -1.48682237,  -0.394843191,  -1.1129415,  -0.554808795,  -0.318850487,  -1.0023675,  -1.50356305,  -1.71888244,  -0.356250763,  -0.433326721,  -1.25472832,  -0.743835092,  -0.510058582,  -2.53778744,  0.412531048,  -1.32923031,  1.68236172,  -1.51928568,  0.33245936,  2.16576195,  -1.01029515,  0.20958373,  0.0497297384,  0.373874187,  -0.798120022,  0.938401759,  -2.58737564,  -0.474752814,  1.13256311,  1.8476305,  1.65039968,  0.618847847,  -0.155014068,  -0.234334484,  -0.848010004,  -1.89200568,  0.487330943,  -1.26464903,  -1.49312687,  0.658663511,  1.57055581,  0.979295373,  0.607798576,  0.345956862,  -1.04053223,  0.740644693,  1.52448273,  -1.46852779,  -0.230143115,  0.0884165838},{ 0.0631366447,  0.121853344,  1.71284127,  -1.33336401,  -0.618203163,  0.26349631,  0.452721953,  0.255775064,  -0.570901811,  -0.768580675,  -0.136384696,  0.626146793,  -0.172320157,  0.286213607,  0.812826335,  -1.46684539,  -0.651672482,  -2.18031287,  -0.776340723,  -1.15794289,  0.777704239,  0.605417967,  -0.578506589,  1.113011,  0.593950927,  1.45324576,  0.744969487,  0.136068135,  -0.470781982,  -1.41830027,  -0.580495596,  -0.148463368,  1.1474663,  1.99078882,  -0.14537023,  1.77521622,  -0.581046224,  -0.714266717,  -0.39405185,  -0.404343963,  -0.951127112,  -0.961423635,  0.558807671,  0.51042062,  1.43176746,  1.33357716,  0.0993726104,  0.00455336319,  -1.40104949,  1.63752961,  -0.00587127637,  -1.27786112,  0.398811162,  1.78418338,  -0.13521567,  -0.18326965,  -0.774217844,  -0.584148705,  -0.749564767,  0.216128886,  -0.91913116,  1.30528557,  -0.436773092,  -1.65268385,  0.279722184,  -0.185249969,  -0.037269555,  1.07656562,  -0.979510605,  -0.990325153,  -0.686530769,  -0.756594837,  -1.62496281,  1.73970687,  0.0161103532,  1.08656323,  -1.26433659,  0.806969702},{ -0.146876991,  0.199923187,  0.433465123,  -1.3571955,  0.923323214,  0.436516404,  1.39540136,  0.52078265,  0.0262210257,  -0.297996998,  1.20592153,  1.6545769,  -0.159758806,  -0.517001152,  -1.50111055,  0.559883058,  0.695950389,  -0.0620978326,  0.0644839481,  -1.29712117,  -0.804553151,  0.075967364,  -0.344890893,  -0.294645548,  0.745932341,  0.86631012,  -1.11539674,  0.583011508,  -0.771611452,  -0.77899009,  1.18949544,  0.0941452011,  -1.08174479,  -0.27040574,  -1.0392971,  1.73183131,  -0.109628335,  -0.246613279,  1.56632459,  0.0846370533,  -1.14808428,  0.679561496,  -0.114439726,  -0.80229044,  0.112244368,  -0.964531422,  0.327733606,  -0.717921555,  0.790845096,  -0.731019855,  -1.42992175,  0.463746369,  0.479808182,  -1.11123145,  0.71165055,  -1.23155367,  0.0369922258,  -0.0413783565,  0.602000356,  0.643373787,  0.818145871,  0.110387743,  1.06464493,  0.357377499,  -0.0650669038,  0.433664143,  1.84026957,  2.00371313,  -1.35147631,  -0.827623665,  -0.136248663,  -0.783622622,  -1.79726839,  -1.56080759,  0.630636394,  0.198537469,  -0.53500104,  0.452187002},{ -1.24648249,  0.332973421,  -1.25894797,  -1.32250273,  -0.320140988,  -1.76084375,  -1.12628508,  0.911324918,  0.766613185,  0.242209539,  1.05367076,  0.459694624,  -0.564671278,  1.39357102,  0.898253381,  -1.25787687,  -0.282197744,  -0.199862733,  0.0519676656,  1.28559172,  -0.775925577,  0.536095381,  0.281426847,  1.96733296,  -1.12546349,  0.208555475,  0.459470689,  0.77409637,  -0.787885189,  0.343603402,  1.58849907,  0.114150204,  0.210178152,  -0.650663257,  -0.740603805,  0.769442737,  -1.02254593,  1.24484432,  -0.673091471,  0.498188615,  -0.236407921,  0.0916217044,  1.01156771,  0.769824684,  -0.959664285,  1.00979757,  0.856754601,  1.51617324,  1.74188852,  1.07417989,  -0.915421665,  0.45334965,  -1.07394016,  0.271957546,  -0.861063957,  0.0086804647,  -0.157311752,  1.03962874,  0.798311353,  0.851937294,  -1.37641943,  0.988206446,  -0.135720924,  0.215196073,  1.12057865,  -1.08279252,  0.176232383,  -0.438209325,  0.885075212,  0.722900748,  0.62413162,  2.53317761,  0.950177073,  -0.796075344,  1.64655852,  0.104883477,  -1.44553137,  -0.421815604},{ -1.11159146,  -0.0751781315,  -0.078456454,  -1.53364038,  -0.310541093,  -0.71099031,  1.17570949,  0.917813301,  -0.247889772,  -0.492232919,  -1.61050594,  -0.0229747985,  -1.75129306,  0.0758536607,  -0.263136297,  -0.801103055,  0.111534335,  1.02371454,  -0.289759874,  -0.0413579233,  -0.117601968,  -2.85351038,  0.271049261,  -1.10073817,  1.04629207,  0.769562483,  -1.17563367,  0.00565943448,  0.584300339,  -0.632485807,  0.490671396,  1.00002754,  -0.407001913,  1.40742707,  1.12691474,  -0.633009374,  0.442396998,  0.172291547,  -0.361403555,  -0.295834601,  1.22164214,  1.06411171,  1.49182296,  0.350289971,  -0.407445282,  -0.373680145,  -0.804811656,  1.3853631,  -2.10620332,  0.167236954,  1.37966084,  0.252918243,  -0.82599622,  -1.07597232,  1.82715821,  -0.204897076,  -0.888593972,  -0.979262114,  0.707332611,  0.912031412,  -1.61677301,  1.00242889,  0.29676199,  0.922472894,  -0.603345275,  -0.3449049,  -0.988113165,  1.04977572,  0.548819959,  -1.199278,  0.764112651,  1.14912021,  0.923149765,  -0.537744403,  -0.870222807,  1.1257093,  0.400080681,  0.636642516},{ 0.00674373796,  -0.053703595,  0.179822028,  0.347705573,  1.09325564,  -1.089697,  -0.684160113,  -0.673774719,  -0.0149911977,  2.09678364,  0.0932483897,  1.93044603,  -0.095658578,  1.12828076,  1.51341474,  -0.459275573,  0.214079276,  -2.17287731,  -0.619623721,  0.444330692,  -0.790077388,  0.236051291,  1.71808112,  -0.0540902726,  0.824415207,  -1.19766247,  -1.05346704,  1.27977538,  -1.52739787,  1.52685475,  -0.650137782,  0.559811294,  -1.57615566,  0.0450395532,  -0.350181192,  0.168762311,  0.00406573527,  -1.0154891,  -0.240114018,  0.707161486,  0.369478971,  0.441992283,  1.57807183,  0.393562257,  1.74398327,  -0.080015637,  0.696901917,  0.925812244,  -1.76896548,  1.72872937,  0.717190325,  -0.117855951,  -0.833516359,  0.688531339,  0.572400391,  1.22007132,  0.0179283451,  -0.482226104,  -0.956268132,  -0.364236236,  0.0232061222,  -1.33497405,  -0.650993705,  -0.730681896,  1.01029873,  -1.34777999,  -0.431324154,  -0.445726126,  -0.178921759,  1.78287971,  -0.0174814928,  -1.00505292,  1.02873361,  0.459296376,  0.375758141,  1.20904875,  1.15004468,  -0.927427769},{ -1.4795624,  -0.0509649478,  0.799440503,  -0.0236032046,  1.83946633,  -1.59093761,  0.132523119,  -0.497080296,  -0.110158809,  -0.380690098,  -0.0704879314,  0.153441399,  -1.27783,  -0.107392974,  -0.422793239,  -0.633701682,  -0.147719547,  1.0302037,  -0.283811152,  0.111711465,  -1.50291872,  3.60559654,  -0.368693322,  0.00519224955,  0.48039192,  0.810642064,  0.572149336,  -1.20393622,  0.774580538,  0.450150847,  0.0487327911,  -0.132840097,  -1.19102395,  1.61230648,  -1.05028427,  -0.688920736,  -2.99228024,  -0.402177781,  -1.98952448,  -0.475807548,  0.399288565,  -0.262988687,  0.163821653,  0.189418182,  0.910717785,  -1.00861955,  -0.0360622182,  -0.273834288,  1.26055276,  0.985079765,  -0.0840392709,  0.173148155,  -0.87313658,  0.0767885372,  1.45305741,  -0.126401275,  -0.534703016,  0.0497665815,  0.852374136,  -0.285600752,  -0.252751917,  1.80386722,  -0.596691668,  -0.659641445,  -0.992184103,  0.125776902,  0.857913792,  -1.18962395,  -0.25366649,  -0.245090246,  -1.17545271,  -1.05811369,  -0.0272250436,  -1.08458924,  0.957003534,  -1.45986855,  0.810407519,  0.75898242},{ -0.399323434,  -1.85428262,  -0.418617725,  1.68380666,  1.75660598,  0.423206657,  -1.33447528,  -2.14618945,  1.3384639,  1.77309644,  0.89457345,  0.140198246,  -2.02833939,  0.358384818,  -1.31297553,  -0.942319334,  0.877584577,  -0.858852863,  -0.883321464,  -1.26175439,  1.56718588,  1.02592969,  0.820412457,  -0.785010576,  0.855634391,  0.202506766,  1.45577264,  -1.66571474,  -0.604370058,  0.270100206,  -1.06405354,  -0.199024528,  -0.468309015,  1.83156168,  0.768801451,  0.642399788,  0.321206301,  -1.02133334,  -1.2975049,  0.81579864,  0.68785876,  -0.253048152,  -0.215610012,  1.36930048,  -0.572391152,  1.87481165,  0.623108089,  -0.6557464,  -1.95455015,  -0.305587471,  -0.920666516,  -0.401932925,  -0.512915075,  -0.697508276,  -2.16514683,  1.56449854,  2.76534462,  -0.757644832,  -0.496263832,  2.59143996,  -0.00641090749,  -1.10801828,  0.547573388,  1.44595873,  0.219112039,  0.834939003,  -0.258298367,  -1.59398305,  1.05241668,  -0.90237987,  -1.14632642,  0.601447701,  1.26722646,  0.852612555,  0.525465071,  -1.15776753,  -0.0623296276,  -1.58471739},{ -1.24544942,  0.372238755,  1.3619144,  -0.768968761,  0.359688431,  0.426594913,  -0.598846376,  -1.12309611,  1.49244583,  -0.122563906,  -0.422768235,  -0.137277484,  1.00332916,  1.3861407,  -0.512115061,  0.170152768,  -0.275996685,  -1.41359413,  -1.38146722,  -0.423501015,  -1.03341663,  0.706814528,  0.874138355,  -0.92678535,  -0.120323651,  0.437495708,  -0.252908915,  -0.946783781,  -0.296084762,  0.0899464041,  -0.667248189,  -1.42055058,  1.20924556,  0.962830067,  0.555386782,  0.168078855,  -0.0501361452,  -1.26321828,  0.219396085,  1.02211893,  0.0591747276,  0.233758807,  -0.328914404,  0.064720422,  -0.0232973732,  0.0252278559,  0.827362955,  -1.08949828,  0.425002545,  -0.152409673,  0.52878809,  0.891686499,  0.446299881,  0.84076333,  0.751241744,  1.46673894,  -0.104683869,  1.10339594,  -1.07639062,  0.741433561,  0.994011998,  -1.56201923,  -1.33732569,  0.244307995,  -0.854648888,  -1.44768453,  0.530781865,  0.67499274,  -0.055474136,  0.40461871,  0.446408331,  -1.51949644,  0.851818621,  0.111508057,  -0.740701914,  -0.387460917,  1.02426231,  0.888865173},{ -0.180099413,  -0.00302266679,  0.645764709,  -0.391654253,  0.0277739652,  1.89002383,  -0.975400209,  0.946278095,  -1.20796299,  -0.852303684,  1.01217496,  -0.703493476,  0.148360193,  1.12221408,  -0.552777767,  0.178474084,  -0.21680747,  0.412601888,  -1.94169128,  0.629704475,  0.712403595,  -0.732501626,  -1.41913354,  -0.389245689,  0.104769632,  0.956233144,  0.0825295374,  0.644096076,  0.635102212,  -1.52754629,  -0.602162898,  0.533947349,  0.371792018,  0.133810505,  -0.283312142,  0.758470595,  1.20703268,  0.285027683,  -0.607771039,  0.499649376,  0.692115068,  1.03194785,  2.25058055,  -0.180436656,  -0.680754304,  -0.368535697,  -1.48283553,  1.37502348,  0.220636785,  -0.337863326,  -1.68874109,  -0.686118901,  -1.6927464,  1.4020108,  -0.689384162,  -0.603294373,  -0.311691284,  -0.423063844,  0.903159261,  0.293302745,  -1.32346237,  -1.95815301,  -0.249791905,  0.459610701,  -0.258480936,  -1.5350951,  -0.340179503,  -1.16308308,  -0.572680533,  -1.48188603,  -1.95445657,  -1.83271229,  -0.565840304,  -0.684605062,  -0.734133482,  -0.583349109,  -1.3247782,  -1.94751549},{ 0.85507077,  0.409120888,  -0.623887062,  0.275965184,  0.20933421,  0.84964478,  -1.1211406,  0.863963485,  -0.485196382,  0.94364959,  1.51900816,  -0.0398230068,  0.648400486,  0.197944432,  0.215575278,  0.482113659,  0.634296358,  1.04788375,  -0.198090911,  0.432603151,  0.947444916,  0.92203176,  0.618997931,  1.16382325,  0.539326489,  -0.631935,  -1.94198287,  1.81064808,  0.119116411,  0.77088511,  -1.68411982,  -0.0601394661,  1.02964604,  0.555438161,  1.31030154,  0.658961952,  -0.700114429,  -0.236249581,  -0.149447337,  -0.575740695,  0.169712931,  1.73507965,  0.321769327,  0.882549405,  2.79828668,  -3.32323575,  0.903837562,  -0.537084937,  0.631261289,  -1.26838994,  -1.14102805,  0.571006596,  -1.18015039,  0.438822538,  0.306627423,  -0.473908961,  -0.525456607,  0.905849695,  -0.819727421,  0.479266733,  -2.42558217,  1.10465467,  -3.07234764,  1.92640722,  1.48655248,  -2.38134527,  -0.392662525,  -0.659943581,  -2.36374235,  0.838410854,  0.536097884,  1.83140993,  0.435645938,  1.06939077,  -0.103479281,  1.007429,  1.06958032,  -0.217248648},{ -0.518108487,  0.263869345,  1.27672768,  0.166729882,  0.0189116821,  -0.155237645,  -0.381747335,  -2.80394292,  -0.639692664,  -1.85994267,  -0.260390848,  -1.04180503,  0.922919571,  0.26437813,  -2.02089143,  -1.12800348,  0.388938069,  1.39893508,  0.0610368513,  1.38321972,  0.530829906,  0.440954924,  -0.308811367,  0.400185823,  0.955492198,  1.03451371,  0.121085137,  -1.16845739,  0.781966686,  -0.314657658,  -1.18860292,  -0.442248583,  -1.44182646,  -0.440282643,  -1.0178498,  -1.90855646,  1.45208168,  0.312171102,  -1.65719867,  -0.428293079,  2.06534338,  -0.645184278,  0.0503468998,  2.61319208,  -0.419255376,  -0.42600885,  1.09809339,  0.254470378,  -0.70189178,  0.49434337,  0.807592452,  1.21568227,  0.673900545,  -0.258043706,  -0.484060347,  0.426707059,  -1.1006248,  -2.3493855,  1.29914653,  1.00116551,  -1.44540477,  1.8136996,  -0.124256998,  -0.816365123,  1.40844834,  -1.07491994,  -0.948263109,  -0.136336729,  -0.446967274,  1.13349748,  -0.254865199,  -0.94913274,  2.47005939,  -1.66208887,  1.31450379,  -0.204997301,  0.570074618,  0.352288991},{ -0.654827416,  -0.500217497,  -0.804673135,  0.195282578,  -1.22128463,  0.660285294,  -0.444431007,  2.07433939,  0.627177179,  -0.354618788,  0.580548882,  0.362305611,  -0.508852422,  -1.36710334,  1.25037158,  -0.378145546,  -0.0805945322,  -1.62639773,  1.34702361,  -0.576904058,  -0.42660746,  0.342189342,  -0.558219731,  1.21264482,  1.48388767,  -1.02029669,  2.59216094,  -0.417016715,  -0.906750381,  0.758729935,  -0.315398723,  -0.921212733,  -0.258269548,  0.944859624,  0.732978106,  -0.259350628,  -0.791966379,  0.400902808,  -0.309645504,  0.600862563,  0.416444778,  -0.0712541863,  -1.6412555,  -0.134217724,  -0.188477844,  0.709343731,  1.04446626,  -0.374302626,  0.167180806,  0.474521786,  2.54369593,  -0.774388552,  -0.162815198,  -0.0731456652,  0.725760043,  -0.297758579,  -0.338254213,  2.00221801,  -0.387585849,  -1.20703983,  1.10155976,  -0.282557905,  -1.57502532,  1.75054491,  -0.420880675,  0.37939024,  -0.617548823,  -0.401650846,  -1.0919981,  -0.363757342,  -0.41714403,  -0.878041208,  -1.19751143,  -0.501473188,  -0.170023441,  -0.813081205,  -0.791739881,  -0.809930205},{ 0.444727659,  -1.53780568,  -1.83873689,  -0.321003467,  0.283462793,  -1.3530091,  -0.719821811,  0.387671828,  0.593332827,  -0.995245218,  -0.586037397,  0.00775916502,  -0.132342964,  -0.664209783,  1.46291566,  1.72434354,  -2.37408638,  1.462479,  -0.389440775,  -1.82508278,  0.806233108,  -0.624211013,  0.890269756,  0.842632115,  0.285503149,  -0.358817428,  -2.00608492,  -1.02719522,  -0.565976858,  -0.403833151,  -0.730203032,  0.206467733,  0.319768637,  1.93845439,  -0.101983339,  -1.41816568,  -0.27216959,  0.934923649,  -0.819413722,  -0.852995217,  -1.08418715,  -0.219694138,  -0.728008032,  1.17129433,  0.45323199,  -0.25715661,  1.18811619,  0.841660082,  -1.09685147,  1.19333637,  0.152385116,  -0.788326621,  1.29465997,  0.384065151,  1.17235732,  0.718859255,  -0.123567075,  -0.367038578,  0.687597036,  1.62711287,  -0.799706638,  0.162238777,  -2.17119646,  -1.63242984,  -0.555051148,  -2.24216032,  0.00724551408,  -0.942930043,  0.863585293,  0.525668144,  1.30157936,  0.52149868,  -1.20358407,  -0.978240252,  -0.601622224,  1.21525407,  -1.02524495,  0.671699345},{ 1.49875879,  -1.58022428,  -0.702025533,  -1.45972514,  -2.07859087,  -0.926833987,  1.0353303,  -0.0934959352,  -0.495910943,  0.656203449,  -2.61033797,  0.0592026189,  -0.507167816,  1.98924673,  0.562370181,  0.340387076,  -1.24136126,  0.655604601,  -0.0143404147,  0.215273634,  1.13667691,  -0.67391187,  2.36425757,  -1.27999902,  1.11078346,  0.0445202999,  -0.143607855,  1.2021482,  -0.61437577,  -0.602401793,  1.43396866,  -0.061982695,  -0.144707531,  -0.502991796,  1.30848932,  -0.36392203,  0.552202761,  -1.67495072,  0.378856689,  -1.74524462,  1.15625894,  1.81567216,  -0.532705128,  -0.901351452,  -0.136561781,  0.0786438063,  1.35091841,  0.320317745,  0.546546817,  0.260633469,  0.376229703,  0.365705609,  -0.779470325,  0.826793313,  -0.621022582,  1.65099382,  -0.154219002,  -0.861154377,  -0.919744372,  -1.94090295,  -1.30889904,  -1.08181155,  -1.20811009,  0.868101537,  -1.07995844,  1.03360593,  0.0602307245,  0.897992432,  2.26947618,  -1.84928751,  1.3666507,  1.57602525,  -1.18034041,  -0.482350379,  1.04248941,  -1.24574125,  1.09216666,  0.145141691},{ -0.741670728,  -0.990918279,  -0.777579427,  0.81061846,  -0.489000708,  0.977023363,  2.01490164,  0.0113071678,  0.13449885,  -0.32334739,  0.181121498,  -1.02916145,  0.153818071,  0.0184154343,  -0.228087455,  0.620657325,  0.591842234,  -0.449873835,  -0.0356221534,  0.474638849,  2.58667183,  -0.235256612,  -0.997478247,  0.71443522,  1.01627648,  -0.416865259,  -1.42491198,  -1.09237921,  -0.425692856,  -0.598988414,  1.3143487,  0.103548892,  -0.199038073,  0.370027423,  -0.195887119,  -0.186349198,  -0.192421824,  0.295364141,  0.532486916,  0.47625342,  -0.0975153297,  0.445728779,  -0.342612982,  0.0268669035,  0.122148409,  -0.982950747,  0.2673738,  -2.16011286,  -0.34779337,  1.61847317,  -0.348839313,  -1.52642322,  -0.342525393,  0.67555964,  -0.682689667,  -1.05112183,  -0.690663517,  0.422792137,  0.661800385,  1.42162824,  -0.156345367,  0.180329561,  -1.07996774,  -0.241688594,  1.41578341,  -1.03915656,  -1.27867723,  -0.447139889,  0.428393483,  -0.124935299,  -0.0439973883,  -0.290871024,  1.01247585,  0.747382581,  -0.226474077,  -0.448736012,  -0.546276093,  -0.881767035},{ -0.45136565,  1.76556551,  -1.15863061,  0.155512437,  -0.165168688,  1.69180667,  0.238205403,  -2.11061954,  0.535360098,  -0.378568083,  -0.8715415,  -1.51053834,  -0.479617536,  -0.102205619,  -0.850295007,  -0.0376419127,  -0.869069397,  0.781565964,  -0.240643665,  -2.29708743,  0.410073429,  0.949151695,  -1.50897026,  -0.258249164,  0.51123786,  -0.635619104,  0.0490516201,  -0.0727445185,  0.245486274,  1.04333508,  -0.763050199,  -1.63220882,  0.287014216,  -1.66243482,  0.0921418071,  -0.902420938,  -0.537135541,  0.679921865,  1.69890058,  -0.739803731,  -0.216013268,  -0.161138043,  0.899393022,  0.397440434,  -0.0877982453,  -0.631613135,  0.710492969,  1.62777448,  2.25195885,  0.199828014,  1.11676037,  1.18838,  -1.65164292,  0.416882068,  -1.35702991,  -1.05394197,  -0.581485212,  -3.15952969,  -0.430796534,  -1.44574761,  1.69830406,  -0.0333643593,  1.31774449,  -0.665554583,  -0.810996294,  0.129067793,  0.687340856,  -0.61498189,  -0.29523918,  0.758005321,  -1.69784534,  0.458988726,  -1.41150486,  -0.970533371,  0.393509358,  0.788976312,  0.597351432,  0.235390991},{ -1.97482038,  0.180409521,  -1.08319879,  -0.175804257,  1.37158096,  -0.923954546,  0.482436717,  1.46065915,  0.681992531,  -0.76053983,  -0.448665321,  -0.223132059,  -1.1596117,  -0.0530442297,  -0.279436052,  -0.293032497,  1.89075994,  1.11707413,  -1.19120789,  -0.882138968,  -1.12328827,  0.0427988134,  -0.0582627878,  0.145280316,  -2.36234474,  0.456173569,  -0.969969153,  0.527463198,  0.699607253,  -0.0579666756,  0.677746356,  0.907583296,  1.27035248,  0.520365655,  -0.970800877,  -0.237704262,  -0.767316878,  0.1448645,  -0.786491334,  -1.53221774,  -0.894142091,  -0.508345306,  0.129643038,  1.56206691,  -0.479542255,  0.110686444,  -1.52957451,  0.182040438,  0.922656,  1.25896132,  1.40792418,  -0.379013956,  -0.173834577,  -0.549569786,  0.732945561,  0.174693391,  0.0313593671,  -0.499851644,  -0.268298775,  -0.984899223,  -0.673076451,  1.81387067,  1.34862578,  -0.40521422,  -0.334653527,  -0.163985744,  -0.306982785,  1.60286641,  -0.0677033439,  0.574125528,  -0.639546216,  -1.44054985,  0.367816776,  -0.0746572167,  -0.168355539,  0.575473189,  -1.59284139,  -0.593589962},{ -1.27593541,  -2.74248934,  -0.87960279,  -0.254463553,  0.018842252,  1.08899498,  0.0638194829,  1.85961688,  -0.342428327,  0.09699554,  0.460456878,  -0.23788467,  -1.28849018,  0.166086972,  0.422489911,  -0.530548692,  0.327708483,  -1.06573987,  0.043817535,  0.27993688,  1.17457068,  0.233197406,  -1.07567298,  -0.407869875,  1.0512954,  0.716637611,  -1.63025403,  1.58440495,  -1.41810882,  0.7108863,  0.0314729922,  0.963209331,  0.992590606,  -0.75829339,  -0.96283865,  0.163707972,  0.835427225,  -1.08980024,  0.400228679,  1.12085843,  2.13416815,  -0.906169891,  0.61820364,  -1.80635202,  -0.118409686,  0.342368782,  0.789019942,  0.108753897,  0.0526745282,  -0.395375282,  -0.173475787,  -1.71369624,  0.589729548,  -0.154594317,  -0.0101949284,  0.201407149,  0.0761713386,  1.38328075,  0.053800039,  0.16702944,  0.338843942,  0.208144158,  1.14198542,  -0.142727807,  -0.170007065,  0.266713142,  -2.41754293,  0.712584019,  -2.43746686,  -0.0693600774,  -0.43392846,  -1.16949713,  -0.27923879,  -0.339372933,  0.667919576,  0.708826542,  0.417041153,  0.552938163},{ 1.43326056,  -0.42338866,  -0.208816782,  0.486075044,  1.27109587,  0.0309484713,  0.234750986,  0.143276751,  -0.475456804,  -0.294451833,  0.976461291,  0.0362962969,  -0.660230458,  0.0569469184,  1.48767114,  -0.484809369,  -1.70493019,  0.478966057,  -0.0388953947,  -1.93943429,  1.63959706,  1.5580982,  0.571749151,  1.74891901,  -0.178309202,  -0.389939845,  -1.17576146,  -0.465078473,  1.90052688,  0.170899153,  0.984169841,  0.063969776,  0.419451147,  -0.81160754,  -0.658690393,  1.38302445,  1.10388887,  0.452404767,  2.11367059,  -1.09118402,  0.649260402,  0.792236745,  -1.2142787,  -2.11334634,  -0.470561475,  -0.558225274,  -0.527732909,  0.365602523,  1.38219345,  -0.8627702,  0.469113231,  0.0313177742,  -0.656086504,  -0.917294443,  0.020990232,  -1.20092869,  -1.26339197,  0.0221481957,  1.92588282,  -1.29191375,  1.11819875,  0.204616427,  0.349233717,  -0.442549556,  -0.444312245,  -0.758236706,  0.412676722,  0.103222236,  -0.884999096,  0.430897415,  -0.756995916,  -0.198664993,  -0.188226208,  0.210786268,  -1.32797718,  -0.463959992,  0.689401746,  0.895523787},{ -0.516289592,  1.11475861,  0.353444993,  0.693806767,  -0.133638054,  -0.703914881,  1.30771983,  0.239840269,  0.454042017,  -0.272445261,  -1.1778264,  0.371894658,  1.77037549,  2.37785363,  -0.81984365,  1.36878526,  -1.01226568,  0.151207,  -1.85023522,  -0.370304585,  0.306281239,  -0.370274514,  1.14071095,  0.435962081,  0.943411708,  -0.257013381,  -0.878331125,  0.421427578,  -0.719237506,  0.894585907,  -2.92012382,  0.964520454,  0.184756085,  -1.61049891,  0.777725935,  1.13251996,  -0.155660108,  -0.366722196,  0.0806478485,  0.891640961,  0.304899126,  -2.07995486,  1.66633821,  -0.581239462,  0.381816149,  0.191800073,  0.475711614,  -0.907224655,  0.299375683,  0.841228604,  -0.658079565,  0.598800898,  -0.469057798,  -0.154070139,  -0.053290315,  -0.689196467,  -0.258718908,  0.182475612,  0.0617921799,  -0.0808211043,  0.521918476,  0.350754797,  1.89683652,  -0.443635911,  0.998604,  -0.592705369,  -0.338605404,  1.25961685,  0.298333228,  -1.52505136,  -1.27632856,  2.27402806,  0.26835385,  0.248648971,  0.60443598,  0.728076458,  -1.02817619,  0.475365728},{ 0.487352818,  -1.04605615,  0.633784175,  -1.2198844,  0.122558631,  2.54504752,  -0.216466695,  0.239016831,  -0.642974019,  1.27209234,  -0.19163464,  -0.316339254,  0.577646673,  0.504004776,  -0.0718178749,  -0.55643785,  1.68688774,  0.89251107,  0.83717525,  -0.203118056,  -0.130955592,  -1.09626496,  0.121720687,  -0.905926049,  -0.780306935,  -0.184303895,  -0.247514188,  0.900388122,  0.709131181,  2.60908556,  -0.97075814,  0.755143762,  -0.54454422,  -0.650847793,  1.61441612,  1.60984731,  -0.147162735,  0.417045176,  -0.637776136,  1.50057769,  0.0808343217,  2.30050039,  0.316707045,  1.52948856,  -0.51913029,  0.904743671,  1.67859972,  -0.74412781,  -0.0487488359,  -2.49722695,  -0.117125243,  0.831216872,  -0.120462283,  -1.4109354,  -0.246285409,  -1.05684972,  -0.9858706,  -0.646075606,  0.607521296,  -0.203651443,  -0.152109921,  1.3072201,  -0.287533343,  0.388984114,  0.911682069,  -1.16893303,  0.078338407,  -0.420586228,  -1.56359458,  0.59282124,  -0.722896397,  -1.46691942,  0.142330319,  -0.377288461,  1.44879699,  0.452077538,  -0.477903008,  1.08499932},{ 0.195437834,  1.59128296,  -0.321733832,  -0.825781822,  0.744106472,  1.77124071,  0.595157146,  1.52546155,  -0.50584811,  -0.0458931178,  0.240249708,  0.107202418,  -0.446040899,  -0.0117892595,  0.490637183,  1.15441155,  0.98425287,  -1.70905447,  1.43400812,  -0.665614069,  -0.211490303,  0.887611151,  -0.348956227,  -0.588701963,  1.3549906,  0.0854540616,  1.79944468,  -1.57678092,  0.677538991,  0.836172998,  0.713188052,  0.976038098,  -0.47498402,  -0.48632735,  -0.00152646052,  -0.222428486,  1.76456594,  0.529386878,  0.360628814,  0.256535709,  0.31025213,  -1.19142938,  0.0501490012,  -1.3672471,  -1.90864551,  -0.815833032,  1.53483045,  0.917373419,  -1.11994958,  0.098884806,  -0.317370385,  -0.2762492,  1.33524466,  0.503585398,  1.04978597,  0.607072294,  -0.140771195,  -1.37876451,  0.999553561,  -0.821238458,  0.461228132,  -2.0581224,  0.308330923,  -2.25763607,  -1.57970762,  0.0543014891,  -0.664608419,  2.28890657,  0.596385598,  -0.94393748,  0.0558343157,  -1.01649046,  1.78648055,  -0.0881008282,  0.577281952,  1.0584451,  0.943930924,  -0.572851479},{ 0.145881221,  -0.908144176,  1.00880742,  0.611125946,  0.495067894,  0.373412162,  0.566208661,  -1.43798041,  -0.067470789,  0.559842169,  -1.79047775,  0.382741451,  -0.381142467,  -1.13038707,  0.157898858,  0.0567739531,  0.824317813,  -1.52134335,  0.134101376,  -2.06874752,  -0.214953423,  0.490239352,  -0.525895238,  -1.08894944,  0.858021319,  -0.137623861,  2.06479669,  1.04500008,  1.18398869,  0.018001698,  0.745263278,  0.487232119,  1.29992604,  0.332485557,  -0.0426453017,  1.32819617,  0.0218923632,  1.54827619,  -0.538846731,  0.12574102,  -0.806639135,  -3.45089984,  0.831447005,  -0.718493938,  -0.30862245,  0.476053566,  0.0655586794,  -1.55481446,  0.570359826,  0.257997036,  -0.442470133,  1.14876401,  0.424252778,  -1.74679971,  -0.452475071,  1.98237336,  -0.461237669,  0.156179711,  -1.04178238,  -1.06625712,  1.02909243,  -0.4158687,  0.804664314,  -1.15262592,  2.16898656,  -0.0557408035,  0.431634933,  0.173281506,  0.181528166,  0.0904240459,  -0.479393631,  -0.276214004,  0.139354855,  0.277064741,  -1.2145263,  -0.695894837,  -0.796034873,  -0.991945207},{ 0.266068369,  -1.07591212,  -0.0707742348,  1.14008486,  0.560275972,  -1.86886036,  -1.33090961,  0.880931735,  0.236672655,  -0.268626928,  0.0393039472,  -0.211597174,  0.376020491,  0.778372347,  -1.09601879,  -1.37851524,  -0.130511343,  0.59430021,  0.627911687,  -0.592178941,  0.870920241,  -0.585384071,  0.801171422,  -0.738191009,  -1.40614343,  -0.770232618,  1.25854146,  0.684588969,  0.797763884,  -0.292198241,  -0.656593621,  -0.618582726,  1.25667512,  -0.0948427916,  0.0159381162,  -0.927103579,  1.06287146,  0.592601299,  0.607081413,  -1.14260554,  -0.468189061,  0.357581347,  1.72617269,  0.127462879,  -1.96097624,  0.896185815,  1.63531363,  0.806173563,  0.174187467,  -1.32559872,  0.347806633,  1.85314929,  1.26387691,  2.07642508,  0.355800867,  0.265289634,  1.66196978,  -0.0363896303,  1.27694726,  1.14155471,  1.12703943,  -0.0867978185,  -0.522316217,  -1.10405362,  -1.05402648,  0.903034449,  -0.893931329,  0.21825695,  0.44874844,  0.519149125,  0.276326001,  -0.260899216,  0.587173879,  0.260251939,  -1.05010116,  2.19970536,  -0.366508067,  -1.45052075},{ -0.39167583,  -0.150794491,  0.318550348,  0.122308061,  0.788795352,  -0.32357797,  -0.126937121,  -1.26246893,  -1.64348793,  0.499310851,  -0.456230551,  -0.765752316,  1.70204449,  -0.206115216,  -0.592759669,  -0.473758847,  1.38389647,  -0.854361296,  -1.29098213,  0.694291234,  -0.218876794,  -0.38306877,  1.79049194,  0.282650441,  2.25566697,  -1.16396999,  1.19902658,  -1.2416352,  -1.85313499,  0.60052532,  1.09829533,  -1.26262152,  -1.1938864,  1.36281526,  0.80114764,  0.611785412,  -0.860145986,  -1.03115249,  -0.455111772,  1.57890832,  0.0820204914,  0.403142303,  0.428829044,  -1.12784314,  -0.425643533,  -2.86826801,  -0.084519729,  -1.5625931,  0.0804925635,  0.215379775,  -0.11255578,  -0.516229033,  -1.13690579,  0.86996454,  -0.0149435457,  0.616432905,  0.502584636,  1.69878864,  -0.186233446,  -0.279110044,  -0.928081274,  -1.13619637,  -0.251507998,  -1.03761244,  -1.41782224,  0.162662029,  0.395019323,  0.0256153308,  1.52182019,  -0.850396812,  0.724867105,  -0.195874497,  0.566311538,  -1.3759954,  -0.298790187,  -0.978860259,  0.815566838,  0.647624195},{ 0.375362605,  -0.453663856,  -0.0463968888,  0.41427654,  0.795352161,  0.381666511,  0.562644005,  0.121475331,  0.203508481,  -0.138779074,  0.297589391,  -0.173678964,  1.46399784,  -1.33181739,  1.0451107,  0.748543441,  -2.51220679,  -1.08403111,  -0.187212244,  -0.119869813,  0.543387949,  -1.08561623,  -0.994351983,  -0.761485577,  -1.05903518,  -1.39392936,  0.54474777,  1.20157218,  -0.791822612,  -1.37859988,  0.243209437,  0.52143532,  1.44640505,  -0.0275802221,  2.31783533,  0.603673458,  0.435259283,  0.425128222,  0.304737657,  -0.51933378,  0.196988165,  1.50682425,  0.406158864,  0.744779885,  -0.433653593,  0.144428506,  -1.28332198,  -0.791809678,  0.541670203,  1.01118433,  1.5192883,  -0.747408748,  0.188432395,  0.521861732,  0.785333812,  -1.29393053,  -0.136445194,  -0.530426681,  0.553708911,  2.09801555,  0.492828637,  -1.16668546,  0.373147756,  -0.901447892,  0.823406994,  0.508483529,  0.511276722,  0.363963246,  0.824217498,  1.1698575,  -2.4456532,  1.54596329,  0.311310887,  0.708821952,  0.376730859,  0.284989476,  -0.651767254,  2.11952972},{ -1.20176399,  0.14120546,  0.424655825,  0.84569478,  -0.432297409,  -0.921202064,  -0.666203856,  -0.0430448167,  0.902332246,  -1.23330677,  -1.24268544,  0.711704433,  -2.21604753,  -0.569519222,  0.243766204,  -1.9208504,  -1.8886348,  -1.01025987,  -1.09534121,  0.751898527,  0.219404027,  -0.394418776,  0.273273617,  2.61180592,  -1.61160266,  0.659222662,  -1.87229323,  0.941434324,  0.187704101,  -1.09423912,  0.242671907,  -0.238752022,  -0.223751351,  -1.95044482,  2.11130571,  -2.4541564,  -0.775157094,  0.289045423,  -1.69750404,  1.00857747,  -1.71920502,  -0.325629592,  -0.0134863853,  -0.150723055,  -0.934808314,  -0.978941321,  0.202700317,  0.487715691,  0.363194376,  0.0756427646,  1.80465782,  0.912510395,  -0.326231629,  1.02835274,  -0.543454885,  1.22622478,  0.816055655,  -0.517154217,  0.413436264,  0.650963306,  -0.20184809,  -1.70551443,  0.426461667,  -0.427275956,  -0.0169036221,  -1.03367627,  0.885546327,  -0.243470088,  3.09401059,  -2.08966017,  0.645609975,  -0.195393279,  -0.740236044,  -0.0881107077,  0.526621282,  0.0502199866,  -1.86525476,  0.541589975},{ 0.352951735,  1.73341119,  1.30936289,  0.435311168,  -0.696552455,  -0.536671877,  1.33074272,  0.11490164,  -1.40871286,  0.763961554,  -0.0542804599,  -1.44957912,  0.132479936,  0.407207519,  -0.95635736,  -0.971537352,  0.444794029,  -1.33956516,  -0.825220466,  -1.65816355,  1.45095479,  -0.00288899266,  1.32110441,  -0.0372361615,  -1.12166929,  0.162498727,  0.389990777,  -1.91671443,  -1.27583146,  0.851566494,  0.0224119313,  0.690956593,  0.747241855,  1.57231116,  0.509070098,  0.607610524,  2.57924414,  0.339482248,  -0.355853677,  -1.85592878,  -0.676422477,  -0.288337469,  -0.880256593,  1.50114727,  1.2471931,  -0.0386395007,  -0.368701011,  0.314271688,  -0.421946824,  0.0542933531,  0.961935043,  0.925780237,  0.730360687,  1.48507321,  -0.488265008,  0.858387411,  -1.05801821,  -0.611554503,  1.15138364,  -0.412816137,  0.796688318,  -0.30193603,  -0.774794221,  -1.25601804,  0.795277834,  0.287452728,  1.20982194,  1.04915094,  -1.02354801,  -2.81356049,  0.467927784,  0.0777580887,  -1.4970541,  -0.0205283966,  -0.550459325,  -0.538844705,  0.383201331,  -0.997265935},{ 0.765115678,  0.933733404,  -0.0940397903,  -1.43031931,  -0.144619331,  -0.52957958,  -1.49524903,  -1.86828125,  -1.13330412,  -1.52027261,  -0.456263393,  0.984124243,  -0.566988349,  0.422684073,  0.912750006,  -1.57556784,  0.31634751,  -0.700688541,  -0.256753027,  1.58642304,  -2.34442234,  -1.72731149,  0.402043164,  1.41930425,  0.0232569519,  1.36247289,  0.23618497,  0.616165221,  -0.0362202898,  2.28660965,  1.41422987,  0.702705503,  0.737180829,  -1.5391736,  0.481743097,  -1.59392416,  1.09368932,  -1.131796,  0.899800062,  -0.0503857434,  -1.4592129,  0.403024167,  1.97841942,  -1.2208786,  0.646601081,  0.0764061511,  -1.64527893,  0.212155044,  1.07353818,  -0.166503146,  0.0520972162,  0.392578602,  -0.296704531,  -1.06718636,  0.647327304,  1.7092241,  1.02757311,  -0.528223515,  0.696473181,  2.57709694,  -1.66963375,  -0.000464514102,  1.28946698,  0.071145691,  -0.592172503,  -1.07593131,  -0.684220791,  0.540248156,  0.371279567,  0.928311825,  -0.153726935,  0.305248797,  1.12239468,  0.201552734,  -1.15517116,  -0.397969693,  0.252362639,  1.24865437},{ -0.0636109784,  0.56308639,  -1.13972318,  -0.606667817,  -0.909241557,  -1.48272502,  0.898446441,  1.02785671,  1.95204437,  0.246277079,  1.30301189,  0.0324501134,  -0.221961871,  -0.519026518,  -0.486707449,  1.27135694,  -0.0993171185,  -0.0737972111,  -0.198365226,  0.90570873,  -1.497648,  -0.177806288,  1.68475676,  1.64966488,  0.82151866,  0.227594107,  1.60777795,  -0.611321628,  0.273987025,  0.17562069,  -0.143281475,  0.105603345,  -1.18877947,  -0.377149343,  0.931602895,  0.277468055,  0.741041899,  1.81977415,  -1.42029178,  0.12735942,  0.0424069799,  0.429221392,  -0.115210429,  0.790056229,  0.220260784,  -0.885773182,  -2.33018661,  0.17798315,  0.107147977,  -1.08073807,  0.37474677,  -0.84488225,  -0.454847604,  -0.125681072,  1.75888634,  0.903591037,  -0.178702518,  -0.617837429,  -0.85751605,  0.860449195,  -1.13154626,  1.63024521,  -0.729204774,  0.340208262,  0.902139366,  -1.83731997,  0.665252268,  1.23048818,  -1.09692287,  -0.518486977,  -0.0334425382,  -0.526668608,  0.148532972,  -0.548880458,  -0.934855878,  -1.14142311,  0.474913836,  -1.72122145},{ 1.01389003,  -1.60580754,  -0.141672432,  0.694642246,  -0.374617636,  0.192127883,  -0.60336858,  0.433063209,  -0.640026212,  -0.889756322,  1.49056721,  1.98822069,  1.28291225,  1.23291433,  0.118650362,  0.0788573623,  -1.42114222,  0.833516359,  0.469655186,  0.335672677,  0.545737922,  0.566865206,  0.781255126,  0.404969841,  0.207229912,  -0.795286417,  -0.809222281,  1.14456975,  0.528030932,  -1.68223643,  -1.06074548,  1.53009593,  -0.938085198,  -2.4650104,  0.170385346,  1.85168684,  0.145144105,  0.272819221,  -0.120874032,  -0.805954337,  1.76203072,  1.15049243,  -0.412173837,  -0.102890581,  1.54486287,  -0.829494059,  -0.277807862,  -0.953809261,  0.259812653,  -0.546628058,  2.379179,  1.34466505,  -0.589321494,  0.322806299,  -1.08555949,  -0.116324812,  -0.0444368236,  -1.0063287,  1.34146166,  -0.785835505,  0.359553069,  0.533583224,  -1.22177029,  -0.327848971,  0.860119462,  1.79705262,  0.798035204,  -0.374532282,  2.95530367,  -1.21692526,  1.63036859,  1.3246038,  -0.899908602,  1.07523239,  -0.83837229,  2.03988004,  0.0424888097,  0.515606582},{ -0.990075946,  -0.483640164,  0.176019877,  0.128267795,  0.55980289,  -1.18801713,  -0.242433339,  2.02488327,  0.61776495,  -1.8764981,  -1.46338785,  1.32631981,  0.939748585,  1.30999172,  -1.19461465,  0.202080041,  0.975092888,  -0.494124532,  0.201585993,  0.758355737,  0.450153291,  -0.00925531331,  1.08508003,  0.573779106,  -0.42605257,  0.178405836,  -1.21926355,  -1.47005498,  1.66390586,  1.19555616,  -0.0049913004,  0.0482461825,  -0.00146901887,  -0.267962754,  0.400477976,  0.444095522,  -0.418059975,  -0.292001277,  -0.11494945,  -0.900833309,  -0.185051799,  -0.0701036975,  -1.14036512,  -0.566544056,  -1.01862311,  -1.03089523,  -0.50562185,  0.502421319,  -0.459222376,  -0.28427878,  -0.371118695,  0.176019058,  -0.618952274,  -0.168860942,  -0.0870517641,  0.380293906,  -0.941990495,  0.86126411,  -1.14499831,  -1.14575005,  -0.867771804,  0.0543018132,  1.1807245,  -0.679916441,  0.534748495,  -1.34392536,  -0.0847896561,  0.788432837,  0.314106822,  -0.944821537,  0.92983079,  -0.679275095,  0.546060026,  -0.743208468,  -1.04390144,  2.81925225,  -0.271812677,  0.118783899},{ 0.489328235,  -0.517605543,  -0.172428608,  1.0774976,  0.252765089,  -0.25215894,  -0.754386008,  -1.56025434,  -0.005019865,  -0.00214011013,  0.0925100148,  -0.661399126,  -1.63064277,  0.69130975,  -0.827151835,  0.62294817,  0.354328394,  0.765625834,  1.79495752,  0.6709795,  0.803350687,  0.803182244,  -0.552134633,  -0.0250658933,  -0.055483263,  0.876202106,  0.292391717,  0.39739719,  1.61165106,  0.409831256,  -0.901437104,  0.306555599,  -0.0327952467,  -0.156467736,  1.26079404,  0.163288027,  0.651549995,  -1.72456646,  2.20854354,  0.282262713,  -1.06782913,  -0.767460346,  -0.371729106,  -1.39318907,  -0.0930289179,  0.253370881,  0.357795179,  -0.05138826,  -1.59181404,  0.272180796,  0.855996311,  -0.694241107,  0.424458116,  0.379716337,  -0.902266562,  -0.713491201,  -1.50257742,  1.04966533,  -0.0943843648,  -0.0716089681,  0.466029227,  -0.704963565,  -1.25317752,  -0.0324703865,  -0.909789145,  -0.669764161,  -0.23829937,  -0.159048289,  2.25319552,  0.215807915,  0.49959442,  -0.92593348,  1.43898308,  0.826875985,  1.61663389,  0.984042585,  -0.259456903,  1.12223876},{ -0.149458513,  -1.60247362,  1.10431063,  -0.582090974,  0.909215689,  1.61035526,  -0.332473993,  0.0183144715,  -1.0322094,  -0.946938217,  1.40998995,  0.913543046,  0.00486580562,  0.259876102,  -0.760179937,  -2.02958465,  0.540902138,  2.08756566,  -1.21754539,  0.828627229,  0.482056111,  -1.10919321,  -2.54664373,  -2.47784591,  0.460577786,  -0.421906143,  -1.32545066,  0.279722244,  -1.32083821,  -0.186828345,  -0.041759342,  0.760203719,  0.476613164,  0.880571485,  -0.628250957,  0.635315001,  -0.327572763,  1.74280381,  -0.0264595915,  1.1425581,  -1.0523684,  -0.360388726,  0.872828901,  0.808657825,  -1.10149968,  -0.278861552,  -1.31331563,  1.20505297,  0.297078937,  -0.543441474,  0.363068372,  -1.4073683,  -0.61794734,  1.79881346,  -0.5154863,  -0.791551948,  -0.985600531,  1.25227582,  0.0916722864,  -1.14272046,  0.863834202,  -1.90096128,  0.537697911,  -0.372733712,  0.217533007,  1.04547358,  0.00384287792,  0.0793000385,  0.870159805,  1.28703701,  0.570226073,  0.834430218,  -1.03191769,  0.727936625,  1.46014571,  -1.65208006,  1.60842991,  -0.374597698},{ -0.309149712,  -0.959847093,  0.204593822,  0.547234952,  0.0548769906,  0.97578299,  0.0842490941,  -0.69413507,  -0.0648301542,  -0.0169764441,  0.255802661,  1.34794736,  -0.402154654,  0.178227633,  0.975242198,  0.163281813,  -1.15329695,  0.690160394,  -0.0620056055,  -0.873972118,  2.65342474,  -0.820535302,  -1.05934703,  -0.0564172827,  -0.0799306706,  -0.969440997,  0.394115686,  0.0920200795,  0.0912176743,  1.20831811,  1.54645216,  -0.615463674,  0.542524517,  0.602476954,  -0.606109142,  0.993029058,  -0.0218489636,  -1.27191031,  -0.534803092,  0.827997148,  -0.684711754,  -0.184879288,  -1.56488645,  1.54725385,  1.1670146,  1.2184546,  0.430275619,  0.635361075,  0.499834239,  -0.099824138,  1.36278558,  -1.90238154,  -0.326451927,  -0.322580248,  1.72676659,  -0.643867552,  -0.127683491,  -0.839049101,  0.248138681,  1.45774651,  -0.22220403,  -0.292552382,  -1.17909396,  -0.960841656,  0.871214211,  -0.244283527,  1.56758058,  -0.616896689,  1.33594537,  -1.215276,  0.343960732,  0.3365134,  1.22786307,  0.258683771,  0.571129262,  -1.05203652,  -0.452866226,  -1.25168717},{ 0.534655809,  0.255425423,  0.648568034,  0.52367121,  0.950886071,  0.547576129,  -0.980471134,  0.532986522,  -1.02859187,  2.12228584,  -1.38926816,  -0.802259982,  0.229748875,  0.134195387,  0.177494004,  1.20043135,  -0.80723983,  -0.680804133,  0.0129854921,  0.25275892,  0.448373079,  0.242336839,  1.63100338,  -0.966783583,  -1.16603065,  -0.728228271,  -0.477839828,  -1.44595611,  -0.407925963,  -0.243168399,  -1.4190973,  -0.784977853,  0.605914891,  1.71181488,  -0.973876238,  1.09215915,  2.05712891,  0.0708877519,  0.497746497,  0.0748293474,  -2.56853056,  -0.124731541,  1.52878022,  -0.482537866,  -1.06364453,  -0.930201232,  -0.254916042,  -0.235773161,  0.249641612,  -1.78922653,  -1.40623319,  0.325586289,  1.04205155,  0.964744568,  -1.04075074,  -0.33785665,  1.59982324,  1.20853341,  -1.75755966,  0.340552032,  -0.307996988,  0.77851522,  1.51864016,  0.698102951,  -0.735488296,  -1.59639871,  1.27174652,  0.111688532,  -0.102631427,  -0.408858955,  -0.298591584,  0.733019769,  -0.470266879,  -1.39813244,  1.67683244,  0.601445258,  1.29839611,  0.193763226},{ -1.83882546,  1.54730403,  0.288238198,  0.802767038,  1.05305898,  0.00366672873,  -0.641733825,  1.28501737,  -0.0719206706,  -0.76069802,  -0.233325765,  -1.11194432,  0.89794749,  0.521576166,  1.02596748,  -0.368415505,  0.173048973,  1.46786702,  -0.260641575,  -1.7388792,  -0.539700091,  0.313618213,  -0.69629252,  2.61279202,  -0.467261255,  -0.614954889,  0.660006523,  1.00607514,  -0.544636548,  -1.5504874,  -1.66584671,  -0.619842768,  0.224040478,  0.462967485,  1.63527393,  -0.163786888,  -1.79451489,  1.18879712,  -0.816289485,  -0.0806710944,  0.325733453,  -0.0263889097,  0.18698059,  0.72606349,  -0.288646519,  1.86883605,  -1.18482375,  1.1954385,  -1.39356983,  0.648472428,  -0.78904599,  0.272005409,  2.07062721,  -1.07458448,  -0.342057288,  0.995776057,  0.080298312,  1.00002718,  0.24005188,  -0.0738416314,  2.0051167,  -1.35101473,  -0.115885831,  -0.631455481,  -0.185211584,  0.787338614,  0.750697434,  0.820696354,  -0.396244943,  0.095725581,  -2.06351948,  0.0918642133,  -0.218405515,  3.00769091,  0.970411122,  1.51049244,  0.394511729,  -0.894440591},{ 0.642055869,  0.65944016,  -1.08018744,  0.148872599,  0.473065138,  -2.35823822,  -0.0881000683,  0.0478315987,  -1.19618821,  0.376855344,  1.61191249,  0.809354663,  -0.594740629,  -0.988034606,  -1.15016627,  1.18935311,  0.525542796,  1.09335411,  -0.483213216,  -0.0348660387,  -0.0997396559,  -0.120942488,  0.464755148,  0.0981641784,  1.76950848,  -0.582076907,  -0.123351626,  -3.07696819,  1.68241596,  -0.681681156,  -0.812569559,  0.265217543,  -1.09214377,  0.654083848,  -0.182349622,  0.0218745545,  0.941947281,  -1.08780766,  -1.07546556,  -0.464268684,  -1.08846259,  0.0199873019,  0.434165776,  4.43947268,  -1.09523511,  0.318358839,  -0.123498179,  -0.159652665,  -1.28957021,  -0.506135583,  -0.523141325,  -0.647678196,  0.47005865,  -0.00547826337,  -0.559780121,  -1.4013027,  1.6404072,  -0.732381642,  -0.558414519,  -0.301175922,  -1.27232182,  -0.568143845,  0.769214988,  0.390583605,  -2.11193967,  -0.16711843,  -0.102801062,  0.645032048,  0.502507567,  -0.0894578919,  1.19922125,  -0.863159657,  -0.872096479,  0.248025924,  -0.19413358,  -1.31606746,  -0.734637797,  1.3892206},{ 0.07855241,  1.77755487,  0.389254123,  -0.77355051,  -2.50713301,  -0.547294021,  0.994771242,  -0.295781732,  0.697436988,  0.826111197,  -0.751217127,  0.448195875,  0.951031268,  -1.55361843,  0.632528365,  -0.140572444,  -0.092560336,  -0.142497569,  -0.645913541,  0.213839799,  0.784323871,  -0.958148181,  -0.418575197,  1.95596111,  0.373449802,  -0.25399968,  0.827203929,  0.0948156789,  1.92163777,  -1.31870532,  0.45834437,  -0.318117797,  -1.1019851,  1.1635834,  -0.928254187,  0.291064084,  -0.0348306224,  0.280898213,  -0.0993754566,  -0.694476247,  -0.620065987,  -1.00775111,  0.636048377,  1.52398062,  -0.93386811,  -1.30516517,  -1.66787779,  -0.0512386933,  -2.03777599,  -0.841448784,  0.184662193,  -0.643122673,  2.21833968,  0.0900593251,  0.407559037,  -0.888937712,  0.167631254,  0.646113694,  -0.379618049,  1.14118624,  1.22294962,  -1.70190847,  0.605895877,  -0.310109556,  -0.928211153,  -2.06900954,  -1.60935402,  0.356738359,  0.819979548,  -0.238466024,  2.74370956,  -0.165863067,  0.383346558,  0.0314766765,  -0.827938914,  -0.699557841,  0.147223115,  2.19352865},{ -1.1229465,  -0.28663975,  -0.397124767,  -0.312563747,  -2.15661383,  1.15474606,  -0.129238188,  -0.0401130319,  0.562789559,  -0.800810397,  -0.632927537,  -1.18652415,  1.09561157,  1.13583946,  0.094653964,  1.14912689,  -0.215748399,  -0.530925572,  -1.60626423,  -0.43410188,  1.37175751,  -0.110352457,  0.820613444,  0.154842302,  0.822971106,  0.431084037,  -1.90494585,  -0.488834828,  0.767562032,  -0.867989123,  -2.14115047,  -0.532348692,  -0.0274774842,  -0.345771879,  -0.850504041,  0.0645510554,  -0.0101432251,  -0.857722878,  1.21016955,  1.51839018,  0.37705636,  -1.33025587,  1.49847543,  2.006109,  -0.702706277,  1.43407273,  2.13229561,  0.102972381,  -0.718306422,  -1.3207531,  0.0863474756,  0.32449168,  0.7544204,  0.117088795,  0.123439804,  1.02077687,  -2.06001115,  -0.701196074,  0.0545432046,  0.631124854,  -3.0461905,  0.717507124,  -1.971771,  0.213117316,  0.75514251,  -0.728984296,  -0.8491714,  -1.22546136,  0.325104296,  -0.580927014,  -0.777646601,  0.810825944,  -0.0915782899,  -1.17490768,  -0.253604025,  -0.0636107028,  -0.0537466742,  0.349226803},{ 1.54792118,  0.679130077,  0.547370076,  -0.372537374,  -2.07401037,  0.126738608,  0.398649573,  -1.89225614,  -0.450281441,  -1.1783011,  0.162303597,  0.8886289,  0.798494339,  -1.63766348,  2.71459889,  -0.234675243,  -0.34829101,  -0.868570864,  0.932713747,  -0.330592811,  -0.0391636789,  1.26243138,  -0.939010143,  -0.14792268,  -1.0784874,  0.230082929,  -1.59159541,  1.58587563,  0.301906437,  1.529302,  -0.041994106,  -0.523748696,  0.16914393,  -1.1487422,  -1.00212193,  -0.460031241,  1.89065218,  0.951953948,  -0.248210505,  1.64989746,  1.14318669,  -0.936088622,  -0.453363657,  -0.105299026,  1.19295168,  -0.607932031,  -0.635082424,  -1.39227045,  -1.27412236,  -0.508014441,  -0.397650361,  0.534829319,  0.0215459354,  -1.82178855,  0.248251513,  0.403541774,  0.235953897,  0.640181959,  0.0787435919,  -1.30382538,  0.655365944,  -0.963461697,  -2.56441879,  1.67101157,  -0.673171699,  -0.486838728,  -1.20023263,  -1.2713393,  0.56176728,  -0.594011545,  0.514261007,  0.0660946444,  -1.39059007,  0.00670421962,  0.899481058,  -1.3058145,  -0.585293114,  -0.277730107},{ 0.560349166,  -0.51723659,  -0.200984195,  -0.440300137,  1.65768242,  -0.85373944,  -0.135577098,  0.287595779,  0.0710716546,  -0.695374787,  0.432985276,  -0.417236239,  -0.260755926,  -0.580621362,  1.19464302,  -0.152568266,  0.864382982,  0.427585572,  -1.32412124,  0.517685175,  -0.317079335,  0.367142618,  1.4596765,  1.67355192,  -0.896465003,  -2.50323105,  -0.186565891,  -0.194723397,  1.81948256,  1.35666919,  -0.531680763,  -0.00317191146,  0.741233468,  -0.0808907673,  -0.454936147,  2.15304542,  0.512444913,  -0.349450916,  1.18345201,  -0.718518257,  0.329381078,  0.00475933263,  -0.537516952,  -0.915356696,  0.554558992,  0.438304782,  -0.0476632677,  0.732420504,  -0.710869372,  -0.778278708,  -0.457200766,  1.41585505,  0.129397318,  0.0259216148,  0.730885625,  1.2394098,  0.458507091,  -0.277595997,  -0.767495871,  1.16466463,  -0.20206745,  0.0692981854,  1.58822417,  0.685313761,  0.0387000293,  0.626346409,  -0.556694865,  -0.848489106,  -0.327870518,  0.818035424,  0.983044505,  2.93212843,  -1.3504771,  -0.482328176,  0.185456857,  0.624495983,  1.49833262,  0.129579321},{ 0.260763884,  0.113151692,  -0.527499974,  -0.0231169183,  0.604237974,  -0.118219092,  -0.201164916,  1.4600445,  0.239389837,  -0.0485059135,  -0.507256448,  -1.12731576,  -0.502826214,  0.781330585,  0.346818745,  -1.5511632,  1.62359416,  -0.300312251,  -0.853892207,  -0.442413598,  -0.887698591,  0.223799661,  1.48081052,  -0.754139841,  -0.971835971,  0.431063741,  -0.733347714,  0.107788235,  -0.0667027384,  -2.63880754,  -0.729250431,  1.4193728,  -2.13152575,  -0.179359809,  0.845450997,  -0.804972649,  -0.753669322,  -1.05708921,  0.0511064604,  0.00692829676,  -0.0828955621,  -0.197270781,  -0.360491246,  1.58473742,  0.773234189,  -0.486492485,  0.515847385,  1.11481357,  -0.419794381,  -0.685025811,  1.2589395,  1.16440642,  -0.207530156,  0.0948954672,  -0.541641951,  0.290157825,  -0.591125727,  -1.43165982,  0.284229636,  0.693133712,  -1.19028139,  0.493614614,  0.122554906,  0.639949679,  -1.33688509,  0.801422834,  0.357407123,  -1.14417064,  -0.916720986,  -0.197412148,  -0.555982709,  -0.113991708,  1.86388159,  -0.399385899,  0.743745923,  0.791387856,  1.15407431,  -0.823353052},{ 0.346905321,  2.64629388,  -0.0728372112,  0.800025403,  1.05054903,  -2.00782561,  -0.141014889,  0.103989594,  -1.43979204,  -0.0934327841,  -0.850402236,  0.471393943,  -0.905811071,  0.631637454,  -0.0501201749,  -0.840185821,  0.531713724,  -0.131231844,  0.908643067,  0.991288364,  0.866545022,  -0.0160107221,  -1.36945796,  1.15798378,  0.487474173,  0.606909275,  -0.664353967,  -0.657007456,  0.400291085,  -0.197271764,  0.452107936,  0.706805646,  -1.14185011,  0.366716892,  1.59262073,  -0.0224406291,  -2.29535937,  0.413828969,  0.231911913,  0.101745784,  0.959604323,  0.369129896,  -0.183029011,  -0.600975573,  2.49743676,  0.175808668,  0.134740651,  -0.0410457663,  -0.388230979,  0.227143213,  -0.886297941,  -0.349286973,  -0.943848968,  -0.893115699,  1.32588649,  -0.834353209,  1.18853509,  -0.386401653,  -0.0147117497,  -0.932016432,  0.107357159,  2.09094167,  0.722232044,  0.550476193,  -1.41773725,  1.31065249,  -2.0834465,  -1.24771833,  -0.451665074,  -1.01561022,  -0.873278916,  0.843965292,  0.900247455,  -0.245391667,  0.418276012,  0.62076962,  -0.460099727,  1.67120051},{ -0.744485438,  -0.157505184,  -0.211799636,  0.0329375826,  2.35099053,  -1.12125587,  -1.61641479,  -1.25209391,  -0.307254344,  0.417497665,  0.531605244,  1.60314906,  0.920049429,  -0.732895911,  -0.185893834,  0.960813403,  -0.147430047,  0.843280673,  -0.110197231,  0.416308522,  0.312599629,  2.0143168,  -0.378415048,  0.0284214485,  -0.919381201,  -0.848900259,  -0.204166755,  -0.281877816,  -0.656708241,  2.40070796,  -0.832435548,  -1.78467822,  0.920043886,  1.84479201,  -0.44576478,  -1.16006255,  -1.0916549,  0.338210583,  0.423915714,  0.0965171829,  -0.572976887,  -1.56218469,  -1.63859653,  0.864825726,  0.542269468,  -1.20668435,  1.32308364,  -0.264652371,  -0.533156455,  -0.180899054,  -0.234448209,  -0.820995808,  -0.469100952,  -0.934225857,  0.331133515,  1.41781545,  -0.139858365,  -1.18025124,  -0.148720965,  -0.316788226,  -0.0757742897,  0.600607038,  1.68592381,  -1.75511527,  -0.371552855,  -0.195674211,  0.584813118,  0.0499770492,  -0.00197386718,  0.648640752,  -0.248278722,  -0.953370035,  0.287554353,  0.61063391,  -0.986874759,  -0.168196127,  -1.58593583,  1.3102175},{ 1.01761365,  -1.32087529,  0.793420911,  0.239717439,  0.281086862,  0.105087921,  0.843667388,  1.30303502,  2.2721622,  -0.0639089718,  -1.29262698,  0.566172183,  -1.63167655,  0.600063086,  -0.515615702,  -0.142803118,  0.515518308,  0.999775708,  -0.170164958,  1.14873302,  -1.18818855,  -0.883263171,  -0.593082845,  -0.0412679538,  1.32672942,  -3.351897,  0.190198749,  -0.634271801,  -0.284179032,  1.1095866,  -1.03974283,  1.80044103,  0.651976049,  -1.68174386,  -0.623069704,  0.592345297,  -1.50837326,  0.620002508,  0.72693181,  -0.517730176,  0.540883422,  1.41145957,  0.384136438,  0.150302604,  -0.315745413,  -0.38866502,  0.298321962,  -0.105671868,  1.03066432,  -1.38294995,  0.600989699,  -0.380900115,  -1.29664433,  0.185585484,  -0.525042415,  1.63626611,  -1.52934802,  -0.987750232,  -0.413109452,  0.777070642,  -0.159435987,  -0.388854742,  1.40134358,  0.0799737945,  0.663126647,  -1.7943325,  0.346604466,  1.02183235,  -2.33035564,  -0.864866734,  0.387887567,  0.173275098,  -0.580909073,  -0.707176685,  0.537153065,  -0.891899049,  0.14125897,  -0.383154929},{ 1.26390493,  -1.45877266,  -0.489212811,  -0.705946386,  -0.750499249,  -0.279467136,  0.154453129,  0.452263176,  2.26698399,  -0.389435172,  -0.298844397,  -1.23443675,  0.260893852,  -1.6534121,  0.478895426,  0.757160187,  0.797584832,  0.445779741,  0.0311761312,  -1.57782662,  -1.22736716,  -0.978194892,  0.117770612,  -2.72667837,  0.330770433,  0.249318823,  0.221721128,  0.803006053,  1.11172545,  -2.30923843,  -0.205592439,  0.0779521167,  -0.0848589316,  -1.42662466,  -1.45226884,  -0.390602648,  -1.52507496,  0.644203722,  -0.923915982,  0.365048945,  -0.305951297,  -0.40879485,  0.384728402,  -0.763868511,  -0.38314113,  0.530097425,  -0.763139546,  0.180150717,  -0.912668824,  1.98056245,  -1.8127737,  0.434802532,  0.257016033,  -0.971460879,  1.63457751,  -0.642035246,  0.50180757,  0.298408359,  -0.605145574,  -0.00187779183,  -1.260656,  -1.08512831,  0.126098707,  0.14823845,  -0.377563953,  -1.80399227,  0.169623971,  -0.034935575,  -0.242898658,  -1.04049504,  2.83268929,  0.100987539,  0.16812776,  -0.0827112123,  -0.419396371,  -0.481373399,  0.047058627,  0.287277699},{ -2.22523165,  -1.58727896,  -0.0584599897,  -0.320885003,  -0.108161516,  -1.59835184,  1.06088626,  -0.618593156,  -1.31090367,  0.0983634219,  0.0633716583,  0.914106607,  1.20291448,  0.0320937373,  -0.13374877,  1.06412888,  0.550063074,  -1.03135991,  -1.37921858,  0.875213146,  1.35424769,  0.557644129,  1.42340982,  0.956018388,  -0.0696031451,  2.1284368,  -0.708600223,  0.0827992558,  -0.385085732,  1.8227222,  -0.0736403465,  0.286682665,  1.41620195,  -0.549907327,  0.756581962,  -1.14225256,  -0.650771797,  0.134744063,  -1.76475012,  -0.354665816,  1.14146769,  -1.36117411,  0.391950965,  -1.38251734,  1.77397633,  -1.78085041,  2.2036953,  -0.553851485,  1.30339658,  -0.406737149,  0.0910583809,  -0.945372641,  0.311660826,  -2.36946392,  -1.90369368,  -0.684237719,  1.76507413,  -0.997072101,  -2.26630187,  0.572660089,  0.610731542,  0.438365102,  -0.183260471,  0.487474322,  -2.02885127,  -0.0955101326,  1.74787784,  2.13456249,  -0.0428928621,  -0.980763793,  -0.0561151467,  2.38228655,  0.8429299,  0.959651351,  -0.963331282,  -1.18942821,  -1.44107866,  2.47349739},{ 0.865590751,  0.841540694,  0.415786922,  1.03142381,  -1.27787089,  1.09956896,  -0.698101461,  -0.101662308,  -0.9259727,  0.248906121,  0.346803963,  1.59564281,  -0.938556373,  -0.367290586,  -2.57748675,  2.09792519,  -1.18014336,  -0.930014372,  -0.676779032,  -0.277619511,  -1.67191708,  -2.13786292,  -1.95114958,  -0.663275063,  0.183391809,  -1.2556144,  -0.362516582,  1.34482419,  0.320164114,  -0.855923235,  -1.81099808,  -1.30674839,  -0.16800952,  0.326698154,  0.237488747,  -0.308749825,  1.42380583,  -2.1818552,  -0.0919763148,  0.783293128,  -1.10178053,  -0.231299043,  0.199009806,  -1.98905277,  1.20608687,  0.561272681,  -1.13965487,  2.12759113,  0.894216955,  -1.49294114,  -0.607540965,  1.41961265,  0.567483723,  1.90321827,  0.197020575,  1.29487336,  -0.179023713,  -0.331936538,  0.407520831,  -1.0354979,  -0.393590301,  -0.873970032,  -0.0937126651,  0.178158775,  0.41814965,  0.54294467,  0.478555411,  -1.7517482,  -2.37461686,  1.13857138,  -0.702867746,  1.66287136,  -0.397496879,  -0.581138432,  -1.18997633,  -1.22757053,  0.720383823,  0.641906917},{ -0.601699889,  1.19654965,  1.08033359,  2.53749084,  2.29999137,  0.215683714,  -0.378901094,  -0.137189329,  2.69823408,  -0.210487321,  -0.372246534,  0.719944537,  0.299284816,  0.964079976,  0.055740796,  -0.338767141,  -0.959323227,  0.174024165,  -0.0861563757,  -0.758964241,  -1.52418256,  -1.89360487,  1.27115977,  -1.66657698,  -0.687561631,  -0.893372416,  -0.808377326,  0.191930279,  0.720542967,  1.56428885,  -0.756666243,  -0.505599976,  -0.766489029,  -0.192098603,  -0.268481255,  0.0421459638,  0.597209573,  1.51510608,  -0.0341618173,  0.470730633,  -0.484324336,  0.835582256,  -1.45605624,  -0.926803708,  -0.752165139,  -0.932890952,  0.901209712,  -0.255431324,  -0.652855515,  -0.105531067,  1.73117566,  0.190979064,  -0.824124753,  -0.691688657,  0.420635909,  -0.530103862,  -0.548654437,  0.12218494,  0.645457864,  -0.953895628,  -0.352726698,  -0.223776519,  0.498340398,  -1.5592351,  -0.618560731,  0.886894643,  -0.854308784,  0.341416389,  -0.257430822,  -0.0543339774,  0.128382698,  -0.463587314,  -0.376758069,  0.158789128,  0.618381739,  -0.194809482,  -0.886651099,  -1.74578202},{ -1.11344826,  -0.220851824,  -0.329575211,  -1.09723091,  0.0625503212,  -0.386909157,  2.10431027,  0.398717195,  -0.16270037,  1.32561243,  0.154345557,  0.296570212,  -0.985001266,  0.794504285,  1.58602738,  0.159303769,  0.549225807,  0.717216372,  0.0685589537,  -0.497044683,  2.96710253,  -0.507375658,  -1.27098012,  0.0218016561,  -0.50009644,  -0.879736125,  -0.996607244,  -0.516036868,  -0.947378635,  -0.48887974,  0.410130143,  0.018675996,  1.47462511,  -0.537085235,  1.92258728,  0.97029388,  -1.36029983,  -0.848520577,  0.0174400564,  0.0967637822,  0.117334552,  1.13242412,  -0.625445426,  0.263275236,  -0.851482749,  -1.03791773,  -0.807908356,  -1.62102985,  -0.45976004,  -1.27807307,  -0.243103057,  0.495564699,  -0.510176778,  -1.22367084,  -0.510899901,  1.20720017,  0.555888832,  0.42441383,  2.32484317,  -1.7610867,  0.0136152171,  -0.510199487,  1.4492358,  -0.630662441,  -1.2896589,  -0.323020339,  0.865461349,  0.798810422,  1.13192856,  1.58281481,  -0.244870216,  -0.821133792,  1.40328205,  0.277448863,  -1.51024318,  -0.582028449,  1.9828279,  1.43062389},{ -0.840566158,  0.134020284,  0.68339771,  -1.0103333,  0.621519983,  -1.28442311,  1.15003693,  -0.327522874,  -0.44685638,  -0.349155426,  2.58287263,  -0.799203277,  -1.95006609,  -0.935688078,  -2.50105643,  -0.353323817,  1.50344265,  -0.924150527,  -1.14846325,  -0.0638187826,  1.35898244,  -0.450339645,  -0.384218246,  0.195576385,  -0.799641848,  1.02871215,  0.858367443,  0.352362335,  0.0728647336,  1.52971196,  -0.405435771,  -0.978050053,  -0.238800868,  -0.795779526,  0.0914496109,  0.137558371,  -0.348377585,  0.281494498,  -0.351618946,  -1.68454623,  0.154170647,  0.708247721,  -0.0555451177,  0.894593358,  -0.405456007,  -0.0399993584,  -0.866701782,  0.280416578,  -0.59559375,  0.542732775,  1.15236378,  -2.67145967,  0.65825963,  1.18746316,  0.738799274,  0.112981722,  -0.0191425532,  0.286513686,  -0.301532567,  1.224949,  0.0353198647,  -0.71010077,  -0.90353018,  1.47300243,  0.084546037,  -0.476776928,  0.085607335,  0.766780317,  -0.596779287,  0.148986995,  0.0349899419,  0.71904552,  0.19487448,  0.362803549,  0.214204952,  0.743075728,  0.178839028,  -0.0126226852},{ -0.118579254,  -0.350036949,  -0.668716967,  -2.30055475,  -0.263336092,  -0.772228062,  -0.0271273013,  1.81853211,  2.66720414,  2.25106454,  -0.042961292,  1.58654916,  -0.0439996198,  0.317473918,  0.0769585446,  0.754086316,  -0.155273318,  0.293592721,  0.0748308897,  0.198671564,  -0.744738042,  1.07312238,  -1.87775135,  0.135796592,  -0.494443417,  1.3692615,  -0.792153239,  0.411899865,  1.26884079,  1.75816524,  -0.558316708,  0.134536222,  -0.893332303,  0.113165922,  0.106804065,  0.63529098,  -0.171025813,  0.819399595,  1.73176074,  -0.254211187,  -0.58413291,  0.622268558,  0.998575807,  -1.47605515,  2.17164922,  -0.89029485,  1.02161956,  -0.265463322,  -0.422213614,  0.818115711,  -1.06773961,  0.685318768,  0.148276672,  -0.816597044,  -0.308041722,  -0.283984959,  1.19020963,  0.779860437,  0.180975839,  0.447892696,  -0.761213362,  0.0106376195,  0.875533581,  0.200109571,  -0.869711399,  -0.320870906,  -0.0362877473,  1.23674631,  0.894159853,  1.36189663,  1.2256254,  0.242805183,  1.01947927,  0.956095874,  1.27940345,  -0.16547811,  -0.594207108,  -0.628209054},{ 0.281748086,  1.3731519,  1.2368809,  0.557857513,  0.567358255,  -0.494458109,  0.8681615,  -0.871067822,  -1.56349134,  -0.884612501,  0.788076997,  0.344476134,  -0.0316686444,  1.65829289,  0.384337038,  0.726964593,  0.761083424,  -0.91462934,  1.24296331,  -1.45449781,  -1.30164921,  1.03806055,  0.494925022,  -2.21333241,  -0.508505285,  -0.978065491,  -0.211255759,  0.0928217247,  -0.685863078,  0.159909651,  1.4995575,  1.51214767,  0.085879676,  -0.65710026,  1.46187532,  -0.323053867,  0.278547972,  -1.59534013,  1.36134481,  -1.47616661,  -0.0690099671,  0.745079875,  0.81564039,  -0.300566077,  1.8733108,  -0.249737307,  -0.886746168,  0.289003581,  -0.778526783,  1.20552921,  1.23800325,  -1.4725132,  -0.680625498,  0.824299335,  1.01319945,  -0.0318268761,  -0.109263793,  -1.54067707,  -1.44018483,  -0.190380454,  0.406681806,  -0.065401718,  0.0898136571,  0.239815682,  0.238831684,  -0.0213372465,  -0.261604309,  -1.16893637,  -2.44813681,  0.295745969,  -0.698329806,  0.668089032,  0.195143253,  -0.0911335051,  1.9608115,  1.15059674,  0.62942481,  0.785086989},{ 1.17616248,  0.552769899,  -0.410077095,  0.900067568,  1.23252511,  -0.294187248,  1.34632373,  2.748142,  0.646495283,  -0.864021719,  -0.237706423,  -1.25910378,  0.367037028,  -0.580999672,  -0.457431585,  -0.706413507,  -0.967372775,  0.176194325,  1.85775995,  -1.09336591,  -0.581225634,  0.213829607,  0.0609564632,  0.0245061107,  -0.0494079068,  0.316293448,  0.364352643,  -0.900107801,  -0.609842479,  0.573752522,  0.865315914,  -0.0803521574,  -0.0113752317,  -0.457350612,  1.14977026,  0.370701253,  -0.273234218,  -1.54875124,  -1.27920818,  0.156795099,  0.197208852,  -0.581729531,  -0.731334627,  0.450507313,  -1.72102416,  0.0738910213,  1.74798858,  -1.07826269,  0.0372850783,  -0.0804173946,  -0.898255348,  -0.22424379,  2.06281066,  -0.71065408,  -1.2877847,  1.03797555,  0.0526287295,  -0.399811149,  2.84576893,  1.39700162,  0.880162895,  -0.593849659,  1.44667614,  -0.27807191,  0.636077344,  -0.957407773,  -0.146560773,  0.850279093,  0.115539283,  0.682265937,  -0.706961095,  -0.919956148,  -1.2671659,  -1.00977421,  -0.111713454,  -0.793397188,  0.843899965,  0.669024885},{ -0.422475249,  -0.0848889574,  0.287795514,  -2.42208767,  1.44952846,  -1.13686264,  -0.225330189,  0.299469709,  -0.869874597,  0.635056198,  0.880405843,  0.590546489,  -1.67552376,  0.813384712,  0.407770634,  -1.26054204,  -0.140461445,  -0.577347875,  -0.44743678,  -2.81365204,  1.32379627,  1.0764879,  0.777897,  0.552844226,  0.563025832,  -0.578704476,  1.16154003,  0.985628366,  -0.618035614,  0.170303032,  -0.821682572,  -0.828121185,  0.669184268,  1.34119928,  -0.00512401992,  0.790290952,  1.21535861,  -1.47531629,  0.171512082,  -2.33842015,  -0.27406171,  0.975159168,  -0.121448904,  1.93203545,  0.67120558,  -0.502455473,  1.01418984,  -0.874399483,  3.03243542,  -0.233563304,  0.514181435,  -0.800389349,  1.32571423,  -0.695631683,  2.01191425,  2.09360337,  1.13824415,  0.0439426377,  0.503860176,  1.33300924,  -0.536694944,  0.34330672,  0.168527558,  1.31069124,  0.562721968,  -2.064677,  -1.20262814,  -0.103488907,  0.500605345,  0.451228976,  0.642401397,  -0.349458039,  -0.87300992,  -0.971494973,  0.254045516,  -0.153729498,  0.495455295,  0.747767806},{ 2.35419822,  1.43353415,  -0.224210858,  0.99219501,  -0.0975697115,  1.90313423,  -1.73806798,  -1.69931769,  -0.460579157,  2.19696569,  0.0563227534,  -0.740103662,  0.47436133,  1.18658507,  0.187876493,  -0.371441841,  1.84247136,  -0.0757195204,  0.360961437,  -0.0495556965,  0.928676724,  2.4892621,  0.490924776,  1.30062473,  -0.760862768,  0.223543823,  -1.17915583,  -1.05938005,  -1.46691334,  0.196129397,  -0.487456709,  -0.222509816,  0.489610255,  -1.00633371,  -0.862782776,  -0.302217305,  -1.24383688,  0.338101208,  1.44538653,  -0.165088326,  0.223942429,  0.227500543,  -1.17940295,  -1.49628901,  -0.675948262,  -0.192847267,  -0.992254794,  -0.0208491217,  1.97839904,  -0.832144201,  0.283819765,  0.267781675,  -0.607920885,  0.35232681,  -0.611829162,  -0.356982499,  -0.15588221,  -1.80589151,  -1.46861291,  0.492427886,  -0.482195705,  1.01304138,  0.342308193,  -1.6419394,  0.27330485,  -0.6594311,  -1.18360972,  -0.526216567,  0.29298529,  1.50312364,  -1.86394179,  0.0591461137,  -0.613069952,  1.03283608,  -0.374946862,  0.95639509,  0.166963145,  0.368511736},{ 0.37357682,  1.08838177,  0.966770947,  -0.493147939,  0.0964527279,  -1.81254494,  -0.130870521,  0.845964611,  -0.290504277,  0.449900419,  0.813761413,  -0.667067289,  0.283138335,  -0.887606859,  -1.01311123,  -0.491258323,  2.05456781,  2.21007752,  0.287776023,  1.13695025,  2.35289645,  -0.541058481,  1.39817202,  -0.850077808,  -0.885124087,  -1.11700356,  -0.995594144,  -1.95915103,  -1.4405247,  -0.382331103,  -1.46231222,  -1.47942901,  0.434888721,  0.244228393,  -0.2994937,  0.235654637,  0.685486019,  -0.222016037,  0.0657031983,  -0.186531723,  -0.345800638,  -1.00321734,  -1.04264343,  0.449485511,  -0.28638643,  0.617932737,  0.906106591,  -0.779524922,  0.231145442,  -0.739238799,  -0.312944233,  1.10367966,  -0.0218417291,  0.373868674,  0.115672693,  0.100215122,  -1.17361808,  0.593352914,  -1.89951873,  0.0314318947,  -1.56457448,  -0.555570185,  -0.313581079,  -1.17424607,  1.49314952,  -1.54254031,  -0.667327464,  -0.504153728,  0.887800813,  0.929388046,  0.082782872,  -0.00856549386,  -0.496215075,  -0.229135245,  -0.0185484998,  1.65283608,  -1.20712817,  1.33958483},{ -0.243252531,  0.0888992026,  -0.0811806247,  1.15847635,  0.304320127,  -0.854964793,  0.685426056,  1.53999543,  0.837373614,  -0.880806267,  0.397106588,  -0.324812025,  0.236096978,  1.96880436,  -1.92002702,  1.12704873,  0.45696041,  1.21801448,  1.46958911,  0.145023316,  -0.295865655,  -1.53821623,  1.51656699,  0.895072639,  0.840975106,  -0.135928705,  0.527202249,  -1.04556108,  0.87145555,  -1.43213964,  -0.544240117,  0.81501174,  -0.202470377,  -0.896503508,  -1.00670636,  0.938716233,  0.579993904,  -1.08741534,  -0.707864463,  -1.28403318,  -1.06634152,  -0.982746005,  -0.465818435,  -0.678398371,  -0.798751116,  -0.526585042,  -0.450015962,  -0.0115327546,  -0.253409058,  0.575597525,  1.0718441,  0.665747762,  0.572984517,  0.22718136,  -0.102249183,  0.534571171,  0.840888619,  -1.01238835,  -0.790367305,  0.680241764,  -0.540826201,  -1.11541867,  -0.865195215,  -0.544542611,  1.1692574,  0.0108185485,  -0.75302124,  -0.408419847,  -0.379654497,  -0.602235377,  -2.09626317,  -0.339220405,  0.711054325,  -1.89898825,  -1.62492371,  0.0510458201,  -1.78656912,  -0.860343993},{ -0.982215524,  0.166006371,  1.56207371,  1.47256613,  -0.329094976,  0.650855184,  0.675709486,  -0.291842908,  0.705726862,  0.741201639,  -0.780584157,  -0.502967119,  0.888435602,  -0.1182191,  0.446259946,  -0.58393687,  -0.484996647,  -0.177290887,  1.68618512,  0.425961524,  0.293941408,  0.237859473,  0.185036838,  0.86880374,  -0.560460508,  0.40723303,  1.16879082,  0.0489917099,  -0.23942925,  -1.70657325,  -0.0187325813,  -1.32680953,  -1.95180845,  2.06211638,  0.0106988316,  -0.00317480648,  -0.459059089,  -1.23557317,  0.150856435,  -0.877012134,  0.0411137901,  0.229725018,  -1.77682865,  1.00803101,  -0.977038264,  -0.480721623,  0.203501761,  0.890512168,  0.0535630174,  0.602539599,  -0.353136629,  -1.90562153,  -0.0712959394,  0.982438862,  0.72141695,  -0.111634798,  -0.690870166,  -2.19521761,  2.13398528,  0.00841086172,  -0.0145433797,  0.280228615,  -0.322333127,  -0.24138464,  -1.81155169,  1.22625077,  -3.24784493,  1.25471497,  0.894656301,  -1.1520257,  0.926441312,  -0.021093851,  1.06014693,  0.339143157,  0.785172284,  0.612035453,  -0.597228169,  -1.31993771},{ 0.200313017,  -1.01201129,  -0.865582705,  0.700384557,  1.20358741,  0.626207411,  -0.264590859,  -0.178024873,  0.213174611,  0.911845207,  -1.59253633,  0.802254915,  0.232704088,  0.562273681,  -0.906005323,  0.205719009,  0.0771320835,  1.40817249,  0.430711001,  -0.111771315,  1.21625638,  3.23338938,  -0.277470231,  0.879202187,  0.767143011,  -1.36553752,  1.57565653,  -0.46764496,  -0.414362639,  -0.247673869,  -3.34363341,  -0.0486248694,  0.236291885,  -1.02804828,  -0.161603898,  -1.23667753,  -0.923906803,  -0.821605563,  0.401999861,  -1.69345748,  -0.329285026,  1.02862155,  0.447392792,  0.25274682,  -0.450284749,  1.04110014,  -0.492567837,  0.219976217,  -1.26972532,  -1.83172429,  0.856470883,  0.230208054,  0.180786625,  1.3604883,  1.26784754,  -0.0533563346,  -1.94336486,  -0.501751304,  -0.929361224,  0.00529217673,  1.27072179,  0.37800321,  -0.408154011,  0.199262619,  0.4280563,  -0.32455191,  -0.598198235,  0.708339334,  -0.416529119,  -1.07847667,  -1.38031197,  0.524095953,  0.267094165,  -0.352518022,  -1.17663896,  0.674622536,  0.491364598,  0.722988248},{ -0.15525648,  -0.85491091,  0.0584923662,  -0.922603369,  0.672922134,  -0.2487005,  -0.0365420952,  0.397401929,  1.60309279,  0.432276666,  0.190615997,  -1.04422522,  -0.673753679,  -0.0174373183,  0.835257828,  -1.54985094,  2.63138938,  -0.920850039,  -0.717205107,  1.9149344,  1.27480674,  0.316401839,  1.8518374,  -0.0277027376,  0.181233317,  -0.784847558,  1.60006237,  -0.346720517,  0.410574704,  -1.0803591,  -0.138101265,  1.35368669,  -0.103359886,  0.392157763,  -0.7926476,  0.17589964,  0.120272599,  1.31107771,  -0.225207448,  0.773965657,  -0.384483933,  0.0130200237,  -1.31213284,  -0.133295819,  0.0390063338,  0.210689083,  -1.46435666,  -0.939432621,  -0.587034762,  2.64522362,  -0.645380199,  0.770141065,  0.599368274,  0.980059505,  -1.42438412,  -0.0390501395,  -2.24235916,  0.589884937,  -1.07293093,  0.485591501,  2.22806406,  -1.69296408,  2.09169531,  0.181863844,  -0.499898523,  -1.74436319,  -0.238410532,  0.729292274,  -0.266351074,  0.624515176,  -1.53467369,  -0.27281931,  1.92999327,  -0.425395727,  -1.33475101,  1.3955617,  -0.99417007,  0.945685089},{ 0.150315374,  -0.365695417,  -2.04564166,  0.803872406,  1.6297375,  0.563588262,  0.680289388,  -0.801749289,  -0.34419626,  -1.61057949,  -1.47751772,  0.606250286,  1.71410596,  -0.0181144737,  -0.756162763,  0.484190255,  0.118620701,  0.131853014,  0.174969375,  0.436543673,  -1.04534841,  -1.85974193,  -0.139965311,  -1.99787009,  2.42673516,  0.0275255702,  1.04583025,  0.568735123,  0.550462008,  0.0239073522,  -0.798579872,  -0.814451694,  0.822585762,  -0.822730005,  -1.25594974,  -0.307652265,  -1.12363017,  -1.71219003,  1.12365735,  -1.61689329,  0.989872336,  1.52746952,  0.87149024,  0.393295348,  0.587471902,  -0.865743637,  -0.957772434,  -0.529866815,  0.365999579,  0.0322491936,  -0.602444828,  -0.570738316,  -0.459875256,  0.976708472,  -0.988762856,  0.901637137,  -0.0469843149,  0.61182332,  1.68058729,  0.147397086,  -0.938713014,  -0.843071997,  1.69900894,  0.0310245994,  1.53282607,  -2.12687111,  0.00545427855,  -2.26995301,  -0.943162918,  -0.207333878,  0.407142431,  -0.70650357,  2.19962358,  -0.0899440795,  0.170643821,  -0.424162805,  -0.470688701,  1.72372937},{ 0.336328983,  0.497917026,  -0.00667149154,  -1.4751575,  -0.0438326709,  1.10379362,  0.532051861,  1.41685331,  -0.345736951,  -3.47521877,  1.0240134,  0.663583934,  1.0723033,  -1.7986778,  -1.04759669,  1.39949942,  -0.0262668375,  -0.783503175,  -1.01009572,  1.3771162,  0.719946742,  1.58786273,  0.231488153,  -1.24388731,  -2.29635215,  0.496562272,  0.0733216852,  1.64527047,  -1.38424015,  1.10315466,  -2.85180664,  -0.8030774,  -1.08120596,  -0.905347705,  -0.792980731,  -0.544721484,  0.430065095,  0.197147369,  -0.573451817,  1.56830859,  -0.0183095783,  0.0885852203,  -0.886264086,  -0.805838823,  -1.03790545,  1.02586043,  -1.21098363,  -1.29188597,  -1.01147735,  -0.250739843,  0.246939719,  -0.872040033,  1.05005014,  0.222657889,  1.478966,  0.72277534,  -0.551999867,  -1.81804729,  0.684314132,  -0.299450904,  -0.561511695,  0.133169159,  0.215703353,  0.0875691324,  -0.539158046,  0.185589492,  -0.494892448,  1.35732675,  -1.41926205,  2.07650018,  1.38919902,  -1.33019793,  -0.135811001,  -0.475177318,  0.525277793,  0.62913245,  -0.55534023,  -0.919143498},{ -0.707978427,  0.00463786116,  -0.661363423,  -1.1231209,  0.32886374,  0.223427564,  -1.53148401,  0.89504844,  2.41456485,  0.629301786,  0.132203206,  -0.154003054,  -0.277134657,  0.0356628336,  -1.41133535,  -0.224853188,  -0.269144714,  -0.905243635,  0.828176618,  1.09144914,  -0.729072273,  0.130663067,  -0.467931509,  0.553943396,  -0.758393407,  -0.239585012,  -0.839277446,  -0.000847638235,  0.581402361,  -1.62400603,  0.285181344,  0.640460908,  -0.637268305,  -1.76899254,  -2.22021508,  -0.612105191,  1.0391984,  -1.32335949,  -0.295758396,  0.278919011,  -1.06767273,  -1.46706581,  -1.06197023,  -0.560640454,  1.90032136,  -0.495463312,  0.176900521,  -0.822331727,  1.34938967,  -0.0264603123,  0.590340257,  0.291247904,  1.58316541,  0.729022145,  -1.9246763,  -0.257149756,  0.571050823,  -1.24147058,  -0.285673499,  -0.0715864375,  -1.43885827,  -0.360653251,  -0.228367761,  -0.631940484,  0.83869797,  1.66394103,  -1.21901381,  -0.554492474,  1.68881166,  0.588292122,  0.308888108,  0.213496402,  0.0443982668,  0.557125211,  0.459489018,  0.894242585,  -0.4483926,  0.432775468},{ -0.023991799,  -0.359647721,  0.393494129,  -0.809213042,  0.848816276,  0.952092946,  -0.472857088,  0.163320735,  0.0340804234,  -0.845507383,  0.00354745239,  0.590884149,  0.328396201,  0.0681099445,  -1.88865638,  0.87615782,  1.34944212,  -0.708868861,  0.340755343,  -0.980036676,  2.50207758,  0.877025187,  -1.0763402,  1.16815913,  0.719857037,  -0.878950536,  1.6082685,  -0.236220315,  -1.39834392,  0.07200142,  0.642195582,  0.145763889,  0.64236635,  -0.112734877,  0.750142217,  -0.636557996,  0.255190074,  -0.944539487,  -0.601483107,  0.913699389,  0.947352469,  -0.263601601,  0.516247511,  -0.269176364,  1.90213823,  0.368500143,  -0.505230546,  -0.16262272,  -1.08678889,  0.685435653,  -2.54291749,  -2.0826993,  -0.613135099,  1.30543447,  -0.300391346,  -0.910679162,  -1.71159971,  0.334626228,  1.20909846,  -0.902076483,  -0.309967369,  0.0910392478,  2.16044927,  0.127400771,  -1.26668143,  -2.00949645,  1.97575533,  -0.347188711,  0.478494287,  1.31623197,  -1.72211146,  -0.601939201,  -0.536744237,  -0.284432769,  0.723720491,  0.202113956,  -0.156514436,  -0.398304015},{ -2.13980198,  1.06029952,  -2.47359133,  -0.683898389,  -1.43436825,  0.340669304,  1.01033771,  0.244617909,  -0.932210326,  1.14447916,  -1.66445303,  -0.0528369397,  1.19897389,  -0.0143582262,  -0.864675581,  1.98739707,  0.25962469,  -0.065535292,  -0.577461243,  -0.579998851,  1.67544019,  -0.512807727,  0.148338899,  -0.036744643,  -0.833238244,  -1.02724564,  0.256355673,  -1.65067399,  -0.00624619704,  0.915187538,  0.536215484,  0.294122487,  -0.218594015,  -0.104454793,  -0.704943597,  -0.264678329,  -1.92386413,  0.775597453,  -0.675267994,  -2.15446401,  -0.594799042,  0.0052084988,  -1.05706668,  -1.35223138,  0.127843827,  0.428907722,  0.535020232,  0.0606144704,  0.35778138,  -0.528358638,  1.01962948,  0.728243709,  -0.429338306,  -1.35204947,  1.83549654,  1.41292167,  1.061203,  0.405721217,  0.920565248,  -2.04777765,  -1.46420693,  0.157720208,  1.42252195,  1.58864605,  -1.55507851,  -1.25217736,  0.485217482,  0.636673927,  -0.63459146,  -0.573658288,  2.04671168,  -0.287665337,  0.754588246,  0.321091771,  0.625658214,  1.63746309,  -0.0673784465,  -0.739975989},{ -0.626062393,  -0.354599029,  -0.498133272,  -0.455753177,  -1.73229384,  -1.02273285,  1.02555108,  1.24132872,  -1.14137828,  -0.866500378,  0.19589211,  0.85903883,  0.14230758,  -0.0656138435,  -0.267457068,  0.338993758,  1.7183398,  0.41033569,  1.12140834,  1.51946139,  0.258393526,  0.65986836,  0.850918472,  0.571944356,  2.02571392,  0.308353692,  0.896719038,  0.434263945,  2.54607415,  0.761807919,  -0.778466105,  0.250064313,  -0.0910255015,  -0.324556351,  -1.60388589,  -1.71113575,  -0.201271161,  -0.18883203,  1.50789976,  -0.755436361,  -0.751267433,  0.0349709466,  0.440385252,  0.426389337,  1.00985789,  0.324256599,  0.714908361,  -0.0776987523,  -1.03117013,  0.865884662,  -1.50512302,  1.15899003,  -0.454144239,  0.0307773408,  -1.04634345,  0.603246927,  0.186279371,  -0.930350363,  -0.135008439,  -0.712095857,  1.27494645,  -0.505281568,  1.1364162,  1.22834957,  -0.718741834,  0.545753896,  -0.393924415,  0.79543221,  -0.938603461,  1.13380587,  -0.702488542,  0.913047969,  0.533312857,  0.0134565737,  1.97302723,  1.54524136,  -0.7077021,  0.814881921},{ -0.754301846,  -0.443723589,  0.367478341,  1.05614698,  -1.06138408,  1.08269179,  -2.0918653,  0.680974185,  -1.32159936,  0.406004727,  1.59177077,  1.26557016,  1.03663707,  0.311153263,  -1.50215864,  -0.382418573,  -0.00348539813,  -1.25174069,  -0.0183808375,  0.183417261,  -0.780439198,  -0.338578135,  1.46801579,  -0.418729991,  1.42009151,  -0.10286843,  1.04037762,  -2.44091988,  0.493199289,  -1.52158415,  0.269538522,  1.02717745,  0.233238339,  0.175621092,  1.79439616,  -1.17051041,  0.58545965,  1.44500065,  2.10711861,  -0.0681133419,  -2.39876366,  -0.653163314,  0.163638845,  0.721036732,  -1.95870209,  -0.553995967,  1.31396663,  0.693861187,  0.406227052,  -1.62024462,  -0.565667033,  0.500980973,  -1.57524335,  1.20633745,  -2.38658309,  -1.56192613,  -1.7266115,  0.459973931,  0.627456248,  1.55719686,  -1.38236594,  -0.337552845,  -0.475180715,  -0.434615225,  0.960937738,  -0.321994901,  -0.163509771,  -1.78954327,  0.573500633,  -1.46935451,  -0.66084373,  -0.399580508,  -0.549070358,  -0.181539983,  -0.531392157,  -2.19018531,  -0.459707737,  -0.588837624},{ -0.738348067,  1.55854392,  -1.63145936,  -1.51960182,  -0.398194432,  0.711859405,  -0.440411359,  1.13282561,  -0.242023617,  0.185260668,  -0.0313655771,  -0.00593753019,  0.723706603,  -0.471615195,  -0.656325698,  0.611200273,  -0.327046186,  0.237451732,  -2.30459213,  0.528703928,  -0.6113469,  -0.149449542,  -0.216957122,  0.775166988,  -0.75009799,  0.251743674,  -0.300958186,  -0.825634778,  -0.603671551,  -0.281961113,  -1.49350548,  1.41903007,  -0.791254938,  -1.50898874,  1.21616518,  0.371475369,  -1.31628609,  0.016110206,  1.685341,  -0.0480195321,  -0.0421972983,  1.67590332,  0.253429323,  -0.5784477,  -1.37849998,  -0.851032734,  0.370933175,  1.19454658,  -1.56405365,  -0.401726335,  -1.48136628,  0.0648106933,  -0.786491573,  1.25348115,  0.268138111,  -1.41240001,  -1.92462194,  -0.438622564,  0.331092566,  -0.102520868,  1.63793588,  -0.984419763,  -0.285511553,  -0.0665462539,  -0.765357196,  1.13674033,  0.00401658891,  0.545990229,  0.343377829,  -1.7325561,  2.06302786,  -1.1681937,  -0.0708834082,  -0.444272876,  -2.18206596,  0.241349891,  -0.0103561282,  -1.04214036},{ 0.237187743,  0.819487095,  -0.375068545,  -0.848888576,  0.902788699,  0.753686428,  -0.885419548,  1.11250782,  0.925310254,  0.558893859,  -0.426157564,  -0.586286902,  -1.061643,  0.418862849,  -1.13401628,  2.85053039,  0.011876802,  -0.979229927,  2.52870083,  0.405360162,  0.459957987,  0.571746409,  -0.321011513,  0.305458874,  -3.04571199,  2.48838472,  0.460968733,  0.774509132,  0.0593595803,  0.95661062,  1.54616475,  -0.21043773,  -1.54569447,  2.26249218,  0.45806095,  -0.663583517,  -0.164965719,  1.20554423,  -1.39034271,  1.17205799,  -0.414813071,  -0.131776735,  -0.226044431,  -0.426288694,  0.460781157,  0.894786417,  -0.719090462,  1.03801918,  -1.22082222,  -0.491257638,  1.42253566,  -0.865842998,  1.23404753,  0.783101022,  -0.125467658,  -1.01205182,  0.214691579,  -0.159534246,  -0.630839765,  0.591412604,  0.365208596,  -0.772068322,  0.303629994,  0.107787132,  1.35777891,  -0.176062733,  1.74678409,  0.385116398,  1.38385475,  -0.13415809,  -0.422031343,  0.48839736,  0.223848894,  -0.84436965,  -0.74356699,  -0.0522623584,  -0.652850568,  -0.341362298},{ -0.49413076,  -1.54818428,  -0.251939714,  1.48712528,  0.782657385,  -0.182558581,  -0.907442272,  0.0175710041,  0.43874833,  1.26223075,  -1.45204306,  -0.928872526,  0.65385592,  -1.12948608,  1.56196594,  -0.724630177,  -0.535440445,  -0.153484955,  1.49716651,  0.0552842505,  0.226175413,  -1.92004156,  -0.0570004322,  1.17750037,  0.903606474,  0.366404355,  -0.997125506,  0.796547115,  0.0952593684,  0.691550136,  -0.483774513,  -0.695923984,  0.400877476,  0.349032521,  -0.396701783,  -0.830258131,  -1.95258439,  -0.116532288,  -0.275138587,  -1.32905388,  0.0588861965,  -0.612147212,  0.73420012,  1.5418551,  -0.885660708,  -0.110424548,  0.189087272,  -1.08888412,  -1.17698359,  -0.100921191,  -0.352572948,  1.55323005,  0.760975778,  -0.524960518,  0.69591856,  0.270481974,  0.0473534986,  -1.49469268,  0.257250249,  0.849209607,  -0.126033679,  0.639235795,  1.43740106,  1.27098012,  0.815438092,  0.272995621,  1.01990449,  -1.25582123,  0.0769665614,  1.92508698,  1.15345657,  -0.634312153,  -0.884930611,  0.637750447,  0.415047199,  -0.600739717,  0.0417745747,  -0.994374514},{ 0.230360478,  0.318317622,  -0.791649044,  0.297766238,  1.5745331,  0.38670665,  0.0171585009,  0.658315837,  -1.02937436,  -0.048744332,  0.083084859,  0.345924467,  1.62039053,  2.05432868,  -1.36543083,  0.670271575,  -0.412962765,  -0.391782314,  0.511107504,  0.564980924,  -0.133191273,  0.489221424,  0.488028795,  -0.0985000581,  -1.70227337,  -0.466819942,  0.240563467,  0.157580823,  0.171360105,  -0.718119383,  1.50061953,  -0.074171111,  0.28246218,  -0.907435238,  0.208356753,  0.883364499,  -0.0966537371,  2.49575281,  1.21292698,  0.264807701,  -0.230771646,  0.991787493,  -1.40931046,  -0.801039875,  -1.25742006,  0.500410914,  -0.883147895,  -0.00262502534,  1.42786884,  0.683569252,  2.53840375,  0.544058561,  0.270656675,  1.02747011,  0.56139195,  -0.233600527,  -0.974084795,  0.931325138,  0.00328988442,  1.16400015,  -0.104517974,  -0.059212096,  -1.06011963,  -0.302575827,  -2.01941466,  -1.45713234,  0.84811002,  -1.37438726,  -0.109660625,  -0.197465837,  -0.387276262,  -0.895160019,  -0.884628832,  0.2160611,  -0.85243088,  -0.135923088,  -1.15014052,  -0.216868341},{ 0.663357198,  0.0535182655,  -0.793177068,  -0.00719227735,  -2.54733133,  0.084372744,  -0.428729922,  -0.536316574,  -0.939555705,  0.340285629,  1.8392235,  1.58603334,  -0.0694992617,  0.550699472,  -1.01367736,  -0.746576786,  0.220640093,  -0.541134596,  -1.21564114,  0.00362583925,  0.320696592,  -0.57476908,  0.63234973,  1.20713651,  0.0586745553,  3.11022949,  1.08318651,  -0.264172316,  1.01658881,  2.10361552,  0.0292233359,  -0.463698536,  0.0136312693,  -1.8822118,  -0.906384826,  0.0796225816,  0.942373574,  0.734690309,  -1.47403693,  -0.0352295525,  -0.895388603,  1.23480868,  -1.01343942,  -0.20601368,  1.19632983,  -1.17735755,  0.137115836,  0.76698488,  0.772705197,  -0.644656956,  0.565897286,  -1.01874578,  0.0967424288,  0.232103944,  -1.65598595,  1.3986032,  0.490082264,  1.89561069,  -0.862895668,  1.41597569,  0.502029419,  1.7651639,  1.01071453,  1.07034814,  0.505712271,  -0.207229555,  1.38407278,  0.830540359,  1.27310824,  -0.472106636,  1.08506012,  1.08605134,  -0.143700302,  2.62953997,  -0.864993334,  -2.1527071,  -1.26933682,  -0.510470927},{ 1.22517276,  1.65233076,  -0.491268754,  -0.091767557,  -0.821894646,  2.83747911,  -1.98120034,  0.676846743,  0.717002094,  -0.0126360906,  -0.943922758,  -0.0351872444,  -0.932710528,  -1.57199407,  0.36555481,  0.896205962,  -0.00331335468,  -0.668866038,  -1.58244526,  0.700527489,  0.949447572,  0.647720456,  -0.257964492,  0.113946669,  -0.433566689,  1.15104103,  -1.54858208,  -1.13786948,  0.88439554,  0.278819352,  -0.795048535,  1.12310529,  0.838264167,  -1.54364836,  0.388029069,  -0.456344187,  -0.0454050265,  0.294959635,  -0.221020058,  -0.556191564,  1.35529006,  -0.446544081,  -0.979268491,  -1.23139,  0.552692354,  0.915652275,  0.596896052,  -0.362267643,  0.630642533,  0.113455489,  1.21087599,  0.550078869,  0.51499629,  1.08239222,  -2.48751116,  -0.481673449,  1.28066075,  0.329670459,  -0.323679477,  -0.316178769,  -1.37969458,  2.18885899,  0.450231642,  -0.145408273,  2.07491994,  -0.51471746,  -0.61086905,  -1.06099379,  0.274107605,  0.414117128,  -1.0536598,  -1.31649721,  0.308625937,  -0.0861025304,  -0.243509635,  0.954967797,  0.226816162,  0.250198156},{ 0.431671917,  1.24104035,  0.445370346,  0.195661008,  -0.917796195,  0.698369801,  -0.677065253,  0.0978196114,  -1.20297134,  1.08785987,  -0.728826761,  -3.41841602,  1.2288959,  -2.18126202,  0.666426063,  -0.179909199,  -1.62539136,  -0.0660120845,  0.583329856,  0.18116425,  -0.728353918,  0.673165202,  0.924572706,  -0.863544762,  -1.47022116,  -0.999860525,  -1.86354816,  -1.24993658,  0.672682703,  -0.081538476,  -0.606621861,  -0.0429525338,  0.581376433,  1.56887817,  0.671832442,  0.0460672341,  -0.324754268,  -1.39080834,  0.124003395,  -0.613667488,  0.807594717,  0.026340792,  -0.608188391,  1.99877524,  1.1202935,  0.682274461,  -0.392244726,  -0.678732753,  -1.89798963,  0.12946111,  0.738853753,  -0.364285171,  0.521335721,  0.67629838,  -0.805398881,  -0.297585845,  0.275660068,  -2.26985335,  1.79282236,  0.255599499,  -1.04113543,  -1.34030032,  0.462027133,  -0.384154201,  0.344197184,  0.718147337,  -0.603024185,  -0.379805565,  -0.370380223,  0.889012337,  0.83445549,  1.62597656,  0.246346489,  -0.884942532,  -1.82709742,  0.00142500014,  -0.530798674,  0.184154108},{ 0.33438313,  -0.428543866,  -0.941086113,  -0.343346417,  -0.942333519,  -0.756152928,  -0.0399636328,  -1.2585634,  0.0750247985,  -0.0716587678,  0.92253083,  -0.0601911321,  -0.783264697,  0.673463643,  -0.774873853,  -0.38795796,  -0.257160366,  1.48117483,  0.17090258,  -0.00620104885,  2.90934539,  -1.7784909,  -0.887717843,  -1.76099265,  0.641960204,  -0.49853909,  0.152707651,  1.24543571,  -1.73728395,  -1.70814502,  0.359805822,  -0.0254354868,  2.14021921,  -0.355218768,  0.117240906,  0.583862245,  -1.45219719,  -1.6631974,  0.765752733,  -0.104315042,  -1.40220118,  1.69150627,  -1.0496757,  -0.872374177,  -2.18239069,  -0.164537743,  0.786048353,  -1.54263985,  -0.819406033,  -0.265259355,  -1.56946158,  2.18400025,  -0.692064226,  0.0410069861,  -0.706457734,  0.114182137,  -0.0935531631,  -1.51157737,  2.88950801,  -0.310010463,  -1.26961148,  -0.69420892,  0.255860925,  0.795975149,  0.718105555,  -0.672877073,  0.133021265,  1.00231898,  0.68549639,  -0.204471514,  1.49212658,  0.152206883,  -0.0254943781,  0.0693891943,  1.34620833,  0.555239916,  1.30208504,  -0.178863868},{ 0.962255478,  -0.59279573,  -1.33740103,  -0.624284625,  -0.328300774,  -0.910453439,  -0.982439816,  1.58752608,  -0.309464425,  1.05984747,  -0.677527905,  0.293905199,  0.862046599,  0.483782679,  -2.10352683,  0.0584688373,  0.241940811,  1.09530842,  1.16980159,  -0.549760818,  -1.18996608,  0.718133926,  -0.482866764,  1.45883787,  0.510253131,  -0.1185681,  -0.367882043,  -1.21977341,  -0.304229051,  -0.643118441,  0.766429842,  1.31340313,  -0.633180141,  -0.676793993,  -0.455370247,  0.129625201,  -0.761512637,  0.130690739,  2.05182457,  1.44341111,  -0.112756088,  -0.462439477,  -1.38028276,  0.283330113,  -0.409260958,  -1.49291158,  -0.098687768,  0.442993402,  0.895696163,  -0.815340996,  -0.488773942,  -1.00168145,  0.723911941,  1.05152214,  -0.315632463,  -2.03555679,  0.179051846,  -2.49218154,  -0.736021757,  2.01685357,  -0.905200839,  0.358686984,  -0.198025256,  0.448747069,  0.0644487664,  -0.855439961,  -0.42133525,  -2.0518837,  -1.00116098,  -0.201195002,  0.907197475,  1.44925451,  -0.65645498,  0.059211392,  -0.609203875,  0.955193818,  1.3395853,  0.172502726},{ -0.965203464,  -0.430227906,  1.85388541,  -0.61771965,  -1.00366819,  -1.35688555,  -0.635183215,  -1.88164914,  1.58571088,  0.801041961,  -1.06637478,  0.698626339,  0.712537646,  -1.29261661,  0.171859518,  1.97702765,  -1.53764677,  0.716869354,  -0.341980845,  -0.0954608768,  -0.772717535,  -1.08687353,  -0.608025968,  -1.10450709,  0.326316059,  0.2603136,  -0.921961784,  0.139357761,  0.477119535,  1.19399333,  1.19679427,  0.841320217,  -0.67283231,  -1.05519903,  0.669691682,  -2.02452469,  -0.120396338,  0.372149915,  -0.9503842,  0.577848554,  -0.948138297,  0.865686357,  1.41747046,  1.29807079,  -1.4040941,  -0.582515299,  -0.0281210765,  -0.160087839,  1.257177,  0.458427131,  -0.741682172,  -1.1147244,  -0.123626299,  -0.822668731,  2.06748796,  -0.15151675,  -0.234837204,  0.5928123,  -1.40515792,  0.164287299,  -0.74005723,  -0.652145684,  0.760529399,  0.936102271,  -0.0618369915,  0.0579808205,  -0.0202524792,  -0.547498405,  1.24343967,  0.985081077,  0.304803461,  -0.560720563,  1.14487791,  -0.191903502,  0.926903546,  -0.229876116,  -0.513472438,  0.443798363},{ 1.35062528,  2.79252887,  -0.246628463,  0.00249017263,  0.640466988,  0.00606437447,  -0.934752524,  0.493471771,  -0.312776804,  -0.222627461,  1.97307301,  -0.468299419,  -0.0437472835,  -0.0882898942,  0.727195501,  -0.490862399,  -0.700006187,  -0.735156775,  0.309815109,  -0.308242172,  0.458057493,  -1.88763118,  -2.64434528,  -1.11516058,  0.302158028,  -0.249527887,  -1.76860058,  -0.644290209,  -1.39546645,  1.14559567,  -1.26936424,  1.82141531,  0.635561824,  -0.582265794,  -0.225565284,  1.67709875,  -0.705964565,  0.29921183,  -0.50951165,  2.39633536,  -0.0120628774,  -0.779837072,  0.445686191,  1.08232975,  -0.203223839,  0.0805742741,  -0.562955558,  -0.344121873,  -0.973782837,  0.634267569,  -1.33067715,  -0.868551552,  -0.99952656,  1.17832398,  -1.65084267,  0.0152264191,  -0.760496318,  -0.663477004,  -2.05000806,  -1.01872766,  0.706327617,  0.015931014,  0.761475086,  -1.30042851,  0.170976743,  0.903992116,  0.900391459,  0.0256391503,  1.63630748,  0.677399337,  1.64013815,  -1.8436197,  0.252772689,  -1.45736158,  -1.6131196,  1.51849151,  0.502871871,  -0.425852895},{ -0.00209564157,  -0.0784850568,  0.474030137,  0.301192164,  -1.00609386,  -0.959968328,  -0.57285136,  -0.572105229,  -0.980910659,  -1.87178493,  0.106290102,  0.0628917292,  0.402312577,  0.058121983,  -1.24855947,  1.92334974,  1.50387609,  0.340287924,  -1.00565839,  0.742392004,  -0.826105654,  -1.24393892,  -1.64550257,  -1.54710102,  -1.57925606,  0.738489628,  0.752374411,  1.47214603,  -0.902651966,  -0.177719325,  -0.960158169,  -0.149981141,  -1.25575662,  0.938571274,  -1.19630015,  -0.595794141,  -0.924020708,  0.504539728,  -1.07466149,  1.11299145,  1.43733466,  -0.252793372,  0.247769922,  -0.966726542,  0.285776973,  0.799618721,  1.31479049,  0.562738121,  1.71009338,  1.07506096,  -0.989847064,  -0.0644618645,  0.0705245063,  0.388489783,  0.0993896276,  -0.19983384,  0.42948404,  -1.3670541,  1.51110291,  0.058833614,  0.570815146,  1.26447129,  -0.601281464,  -0.448516041,  0.178884223,  -0.456879228,  0.772976816,  1.22129893,  0.0421484709,  0.328657389,  -0.0218877085,  0.179768905,  0.369571626,  0.327310205,  -1.63209724,  1.09798026,  1.52072465,  -1.39622355},{ 1.83526003,  0.197284013,  -1.03931367,  -0.451169908,  0.696157396,  -0.787856877,  -0.141847134,  0.924659193,  0.90619427,  0.0986345112,  1.08011186,  -0.955247521,  0.329768389,  -0.0748121813,  -0.39532274,  -0.309518784,  -0.75509882,  -0.237719208,  -0.653708398,  -0.535223722,  0.405040443,  -1.54709184,  -0.557958305,  -0.437632442,  -0.595918,  0.7972368,  0.322687209,  0.651640654,  -0.00407245383,  0.721969783,  -1.45249403,  0.534342587,  -0.792409301,  -0.126639381,  1.22429609,  -0.0668145791,  -0.611635566,  -1.89360654,  0.461783677,  -1.73905611,  0.800439477,  -0.269121855,  0.066756323,  0.277613968,  0.824269474,  0.723973572,  -0.082336165,  -0.285643995,  0.513289213,  -1.03377962,  -0.149352744,  2.12579775,  -1.17724895,  -0.296604544,  -0.282086521,  1.07933092,  -0.608546317,  -0.48278603,  -0.0311731398,  -0.520032763,  0.352717221,  0.0710212812,  0.281329811,  -0.046709884,  0.690056503,  0.463096648,  -0.96957773,  -0.520269156,  -0.406460822,  -1.51967669,  -0.996509492,  0.895315111,  0.356680572,  0.495024472,  0.124076806,  2.44842219,  -0.64533937,  0.128190756},{ -0.450784534,  0.490501106,  -0.275383234,  -1.90962338,  -1.03040445,  0.416344732,  -0.22616446,  -0.209683016,  -0.35048154,  0.992725372,  -0.238940269,  1.83351719,  -0.52817142,  -0.289158195,  0.0673519447,  0.845697045,  -1.95670927,  -0.0696171969,  -0.435413361,  0.519044578,  -1.30069661,  -1.53797579,  -0.339257717,  0.281148732,  1.03949034,  -0.467364728,  -1.34139907,  -0.723665416,  -1.80662382,  0.582911611,  0.256630749,  0.870684385,  0.455760956,  1.93536878,  -0.993548453,  0.716564238,  0.126279131,  0.0920048803,  -0.30130744,  -1.34557557,  0.235000357,  -0.715317369,  1.77988017,  0.00803812128,  -0.427798063,  -0.111652538,  0.606938064,  2.2661972,  0.346603662,  0.951414406,  0.224790663,  -0.33298859,  -0.0818877965,  -0.873938322,  0.259940565,  0.705786467,  -0.578194797,  -0.211416602,  0.339791775,  0.284867704,  0.491255164,  -0.11376974,  -0.435874522,  -0.863461137,  -0.769487441,  -0.565883815,  2.60885739,  -0.747364998,  1.01059937,  -0.185486645,  0.476019055,  -0.758030415,  -0.671733499,  1.10912693,  -0.855075181,  1.053002,  -1.70373929,  0.715768874},{ 1.2627008,  0.902254343,  0.155340061,  0.317078203,  0.111353777,  -0.405906647,  -2.03957462,  0.171939865,  -1.77702117,  0.885529041,  0.590382278,  -0.283425152,  0.520880699,  -1.01147401,  0.85574317,  -0.743618309,  0.465901822,  -0.732782781,  1.0349853,  -1.52482104,  0.511936188,  -0.93101114,  -0.470058888,  -0.563149989,  -0.692926407,  -0.771910906,  0.487631142,  2.06133485,  0.495624125,  0.158744365,  0.538872421,  0.619192541,  -0.0750450194,  0.78182888,  0.613675594,  0.36578387,  -0.191406533,  0.738214433,  -0.816100657,  -0.284583718,  0.774229586,  -2.16953897,  -0.86275965,  -0.211090773,  0.259722859,  0.437764585,  -0.0045346776,  -0.604607582,  -0.519979835,  -1.218876,  0.671847463,  0.0379066132,  -0.0888212025,  -0.619659185,  -1.39494979,  -1.21596014,  0.651068032,  -0.795326471,  0.487861574,  0.898924828,  -1.88858557,  -1.31152844,  -1.49431372,  1.78858936,  2.23070121,  -0.62918967,  -0.688928664,  -0.665537059,  0.0891994908,  1.0453608,  0.813965082,  -0.790353715,  -0.443764269,  1.08534348,  -1.85661662,  0.1761861,  0.902256727,  -0.990144193},{ -0.428863138,  -0.514729142,  -0.908554792,  -0.364184946,  0.942594349,  -2.0587368,  0.485772997,  1.0235163,  -0.935738683,  0.527165949,  -1.54899108,  1.34645617,  -0.255436987,  -0.56132102,  0.704863667,  -1.43932498,  -0.598558128,  -0.57207489,  1.29727447,  1.49529052,  0.567800581,  -0.75958097,  -1.06591213,  0.00677320268,  -0.131711647,  -0.985391259,  -1.03376198,  -0.78126955,  0.747069538,  0.849209666,  -0.859006464,  -1.25731242,  1.14551532,  -0.231821567,  -0.087471135,  -1.28117263,  -0.233857989,  -0.597082555,  -2.60262346,  -0.273512691,  0.426489085,  -0.120057665,  0.66118145,  0.687132299,  -1.52972496,  1.05810583,  -0.438984126,  0.575910807,  0.321389198,  -1.20011878,  0.179558426,  0.512109876,  -0.0430691279,  -0.473188132,  0.705270231,  -0.547744274,  0.856048167,  -0.527910411,  -0.0717737824,  -0.66674608,  1.55382156,  -0.269171238,  0.299735606,  0.833950102,  -0.545926213,  0.808407545,  0.980346501,  1.10550416,  -2.3987093,  0.453757912,  0.15700382,  -0.672407627,  1.60239708,  -0.426695198,  0.555359781,  -0.675686955,  0.466786951,  -0.155476451},{ 0.112973221,  -0.971421123,  -0.789417207,  0.0884764791,  0.924708366,  -1.47145951,  -1.70173645,  0.569287121,  1.13915539,  0.0567221083,  -1.12847173,  0.938472331,  0.805786967,  0.0178520251,  1.16507912,  -1.44052935,  0.937683523,  0.280869067,  0.577792346,  -0.016770415,  0.60391432,  0.0884250104,  -1.63638914,  0.984844565,  0.800976455,  0.213261768,  -0.57910502,  0.396774203,  2.05266738,  1.39433539,  -0.97900939,  0.464374483,  -0.750860333,  1.00062597,  0.334107459,  -1.88305438,  0.590887725,  -0.575610697,  1.04256415,  0.192348212,  -0.649907887,  0.618416011,  -1.40382147,  -1.88407731,  -0.419062018,  0.712960422,  0.71253258,  0.964058459,  0.721821785,  -0.414510995,  -0.664814651,  -0.617720544,  0.050598219,  0.695006013,  -0.718549252,  -0.659018636,  1.28876078,  -0.308649242,  -0.959080458,  0.144798517,  -0.426737875,  -0.485462159,  -0.244825006,  -0.237706602,  -0.579907179,  -1.25152969,  -1.32684171,  -0.210962549,  0.638352334,  -1.33967721,  -0.00583153917,  -0.549561024,  -0.605149269,  0.560588121,  -0.771756053,  -0.115985103,  -0.403402805,  -1.26145482},{ -0.631163776,  1.20282555,  0.693385184,  0.824804723,  -0.128632978,  -1.05365157,  -0.671876967,  0.0416341797,  -1.30909956,  -0.720838547,  -1.2216779,  -1.0026859,  -0.0220069792,  1.9855994,  -0.507838011,  0.292126536,  -0.225312248,  1.45415556,  1.74629545,  1.55065536,  -0.115024351,  -0.755357385,  0.135387868,  0.173767224,  1.58619916,  -0.431530565,  0.214061275,  0.162033141,  0.0180215798,  -0.417647868,  -0.850998759,  -0.122478075,  -1.45513356,  0.924571812,  1.35607255,  -0.319449484,  -0.0714103207,  1.22582459,  0.684287846,  -1.16185558,  1.44093478,  -0.571110964,  0.564355731,  -0.770020783,  -0.350508779,  -1.02439177,  0.627940655,  0.676674843,  -0.799017847,  -1.34751642,  0.337049574,  0.801460207,  1.14887011,  1.28907728,  -0.883829474,  -0.146816403,  0.841970325,  -1.01997054,  -0.538734913,  -0.452653348,  0.564148068,  0.281440973,  1.16985595,  2.19194531,  -0.21122089,  0.0419083349,  -1.52844167,  -1.20037138,  0.632954597,  -0.431220204,  0.629670799,  -0.926940143,  -0.943936825,  0.109121077,  0.347419083,  0.601774871,  0.321171969,  0.208367929},{ -0.957424223,  0.692847967,  1.16588569,  0.930411518,  -0.513380229,  -0.98735708,  -0.532515526,  1.19881248,  -1.03684568,  -0.309282303,  2.06866479,  -0.284603506,  -1.46228158,  -0.415394813,  1.49571252,  0.616175175,  -0.776106596,  0.513103366,  1.46273112,  0.712609589,  0.398388684,  -2.30283356,  -1.7587738,  -1.26032567,  1.15636849,  0.25014621,  -0.87737,  -0.91403234,  0.467604011,  0.498928696,  -0.266720295,  0.563235462,  -1.58033884,  0.240177557,  0.697971523,  -0.229143694,  -1.00726616,  1.34407699,  0.317854702,  -1.06729865,  -1.01785409,  -1.34253156,  -1.89774191,  -0.411643624,  1.05638647,  -1.47269666,  1.8824091,  -2.45440388,  2.13800597,  0.159101829,  0.24602063,  -0.831621826,  -0.216684923,  -0.57419914,  -1.9772476,  1.535478,  0.403721809,  1.45961571,  0.104460008,  -0.721086442,  1.07921195,  0.452444255,  -1.76617801,  1.12106299,  0.957727849,  -2.21987009,  -0.272221088,  -0.622193217,  0.500236213,  1.55058324,  -1.09916937,  -0.509489298,  0.079123117,  -2.17646456,  0.267448574,  0.332137018,  1.57072067,  -0.467337281},{ 0.910230279,  -0.479223013,  -1.12145138,  -0.403156668,  0.294713974,  0.556372583,  -1.77152681,  -0.210876226,  -1.01791179,  -1.235937,  0.200325608,  1.01667953,  -0.212905273,  1.57900655,  -1.03357244,  3.50300002,  -1.4737711,  -1.8916682,  0.393273085,  -1.63654363,  -1.09735179,  -0.0164771378,  -0.38065818,  -1.0556047,  0.0614329465,  -0.00147251214,  -0.0597744845,  0.934171915,  0.908080161,  -1.82533395,  2.39624548,  -1.57275116,  -1.2270596,  -1.60731518,  -0.388534665,  0.322801203,  -0.304462641,  -1.1944809,  0.314325601,  1.7040664,  2.46700621,  0.280490935,  1.16352177,  0.494576663,  0.776135445,  0.11100626,  -0.678612649,  -0.0354289636,  1.19333196,  -0.00093202322,  0.327471942,  0.579898357,  -0.819102407,  -0.251489967,  -0.72375536,  -0.0242115036,  0.0742813423,  1.34714949,  0.665067971,  -0.156211525,  0.817610264,  -1.06447911,  1.15858352,  0.509870589,  -0.827664912,  -0.132500261,  0.777423739,  0.244055822,  -0.749119103,  0.38109076,  0.79317081,  -0.472311258,  -0.259160668,  0.914826751,  0.858031988,  1.46042168,  0.0591851287,  1.87189102},{ 0.199706003,  1.82505488,  2.51658726,  0.22608377,  -0.521236062,  -0.290338755,  -0.873387933,  0.112986103,  0.421233237,  -0.982547939,  -0.470126361,  0.839958549,  0.196529537,  0.797779441,  0.254115075,  0.198470831,  -1.00110996,  3.06193137,  1.67155647,  -1.41367614,  2.34020638,  -0.76971072,  -0.285198689,  -1.36846423,  -1.03076708,  0.118174918,  -1.67449868,  -0.783528447,  0.603497863,  0.717208922,  -0.724589348,  0.465484083,  -1.31091905,  -0.187030166,  0.827868521,  0.318950653,  -0.910976946,  0.337750852,  -0.313358545,  -2.38369799,  -0.0157590676,  0.595570147,  3.0713954,  0.751013577,  -0.789812207,  0.290097117,  0.0802230015,  -2.40503287,  0.0509440601,  1.11644435,  0.441162825,  1.26709723,  0.325614244,  0.102701873,  0.393494189,  -0.589671195,  0.0835156366,  0.983005702,  0.607768595,  1.80112374,  -0.381540567,  0.736669779,  -2.42449737,  0.216778994,  0.817833722,  -0.458126992,  0.384681046,  0.647176147,  -0.630631626,  0.243076295,  -0.212498873,  -1.19310093,  -0.62863332,  -1.13892877,  -0.93914324,  0.927704036,  -0.257158905,  0.643118322},{ -0.755337358,  -0.92503792,  -0.177517191,  0.972988904,  -0.76834631,  1.29735875,  0.70430845,  0.404449731,  0.198594123,  1.46576679,  0.462341517,  -0.0056161317,  -0.508246303,  0.232152849,  0.928156018,  3.17901778,  2.28475952,  -0.125297323,  2.80634594,  -0.168997437,  0.163324222,  -1.40803063,  2.45139313,  -0.756214201,  -1.04750276,  -0.154492006,  1.2140404,  -1.2779032,  -0.769604623,  0.479194134,  0.635242462,  -1.93457341,  1.26351786,  -0.350375295,  -1.07842982,  0.576279461,  1.54481697,  -1.29817712,  -1.48956943,  -1.32108378,  -0.255337059,  0.00151888945,  -1.24403167,  -0.395712495,  0.433723897,  1.78242457,  0.254006654,  -0.554112196,  -0.0971988216,  -0.196712628,  -0.318235576,  -0.090128608,  -0.755825877,  -0.421313405,  0.503144324,  1.11830413,  -0.933512747,  1.60981095,  2.76642489,  0.631769955,  -0.726078451,  0.0539316125,  2.75899565E-05,  1.76657152,  0.109099254,  -0.754045129,  -0.963968754,  -1.03732872,  -0.487071753,  -0.777748883,  -0.560765564,  -0.25329873,  -3.0931282,  0.26620993,  0.105985299,  0.728391051,  1.19480288,  -1.52501154},{ -1.78123093,  -1.13815677,  -1.34126973,  -1.31009901,  1.61715198,  1.15862072,  -1.40489793,  -0.384721547,  0.743534446,  2.02050757,  -0.669214129,  0.941161752,  -0.342873544,  -0.00569790229,  -0.666231573,  1.70589256,  -0.708186865,  0.983457088,  -0.455778867,  -1.10397017,  1.20031393,  -0.504032612,  1.17025876,  -1.01982641,  -1.23923707,  0.921934128,  0.445589572,  0.338788718,  -0.71608007,  -0.713874996,  -0.83963567,  -0.74728328,  -1.23152173,  -0.680799603,  -0.0403267518,  -0.0964459553,  0.0645952895,  0.528815448,  0.194212615,  0.831586123,  -1.13988101,  -0.972221375,  1.11608708,  0.579190731,  0.698921204,  0.539734423,  0.72942853,  -0.995307267,  1.09561074,  1.46236503,  -1.09936011,  -0.0446934141,  -0.0977980122,  -1.07198954,  2.35454059,  0.768459082,  -0.814428985,  0.0642077997,  0.628845513,  -0.545319736,  -1.27779114,  -0.696885288,  -0.136190504,  0.756391466,  0.688707173,  -0.555039585,  -1.56776059,  0.0615520105,  -1.41896951,  -1.59465146,  3.58047438,  0.0493610874,  -2.30097294,  -0.140971214,  0.323256284,  0.744714797,  -1.51604438,  -1.27099514},{ 1.12927496,  0.600687265,  0.619168222,  0.0311356485,  0.199281946,  0.377790987,  -0.638676822,  0.70487386,  -0.707084835,  2.26411057,  0.905058503,  -0.582908034,  0.17233105,  -1.60462976,  0.0106394058,  0.119050615,  -1.07261932,  0.285529971,  0.660648346,  0.0224527866,  0.854269207,  -0.758346558,  -1.21418321,  0.0598758683,  0.00129401381,  0.924301207,  -1.22470498,  1.42384934,  1.26106429,  -0.0567260683,  0.290175587,  -0.366587371,  0.527547419,  0.857413113,  -1.3062309,  0.479935288,  0.32946381,  0.747205198,  -0.0921732485,  -0.605597019,  0.39473471,  -1.08844006,  -0.47499916,  0.277647644,  0.961648047,  1.24911582,  -2.36368871,  -0.551907361,  -0.26230666,  1.48746455,  -0.0633998737,  1.99528599,  -1.36166918,  0.671357214,  -0.717040658,  1.95787084,  1.0481385,  -0.627223969,  0.31439954,  0.0560173914,  0.223955229,  0.0212740209,  -0.698864222,  0.342877984,  1.26198685,  0.749443471,  0.716954052,  -1.03893054,  1.67094624,  -0.362180829,  -1.42526817,  0.108914837,  0.303352326,  0.574094772,  -0.635942101,  1.12437308,  1.58060467,  -1.08847225},{ 0.0373628996,  0.0450168252,  1.42730808,  -0.345341474,  -1.21052361,  -0.564196646,  -1.02239919,  -0.605939984,  -0.658928573,  0.049817156,  -1.12803733,  0.893680096,  0.603185117,  1.03769255,  -0.345135093,  1.2252866,  -0.252072632,  -0.561423063,  -1.82540083,  -2.14347363,  0.364018291,  -0.604999781,  0.300017178,  0.375708729,  -1.5471698,  -0.113994189,  -0.0903221369,  -0.704012632,  2.49609327,  0.607392192,  -1.21662724,  -0.53780055,  0.411268055,  0.684131384,  -0.276460201,  -0.101673737,  0.519028306,  -0.203968346,  0.780243456,  0.212549672,  2.01763058,  0.328566432,  0.0441675335,  -0.213005736,  0.240528658,  -0.45606941,  0.899667442,  1.39593291,  1.50113368,  1.39013672,  -1.04763889,  -1.63303041,  0.0760154799,  1.03672612,  -0.334738761,  -0.62765795,  0.360576093,  -0.448123693,  1.0710032,  -0.166025281,  -0.859042346,  -0.28251794,  -0.486634105,  -0.290061682,  -1.34188068,  1.53807199,  2.68844771,  -0.431133866,  0.677354157,  -0.349935502,  0.388634175,  0.47892186,  -0.390927404,  0.263192236,  1.53748786,  -0.784594774,  -0.705958426,  -0.145640939},{ 1.12112474,  -0.553636193,  0.200577259,  -2.54364347,  0.138130993,  -0.847728014,  1.02473581,  1.00195897,  0.150267974,  -0.0291121136,  -0.889009058,  1.10650253,  1.24833119,  -1.61448336,  -1.53977299,  -0.155604124,  -0.927683532,  -1.48484361,  -0.56906718,  0.135094523,  1.37292135,  0.697658658,  -0.121278897,  0.573600113,  2.52663207,  1.02750647,  -0.0016800611,  1.04971004,  -0.634588182,  2.04314566,  1.18221092,  0.247186735,  1.84505928,  1.26769698,  0.13648653,  -0.39503029,  1.81622064,  0.433343947,  0.256917864,  1.09014845,  0.992283642,  0.606657505,  0.925513148,  -0.52461344,  -0.368449509,  -1.95387614,  1.54604292,  -1.20856071,  -2.0603559,  -1.31261432,  1.89086938,  -0.165100932,  0.984054387,  0.645525932,  0.661133885,  -0.361822665,  1.67262506,  -0.2025709,  0.347978324,  0.546960473,  -0.915305018,  0.325758398,  -0.317981511,  0.795741439,  1.92208707,  0.400607824,  -0.683739483,  -1.45300448,  1.39682341,  0.529356301,  -0.607891738,  -1.7234062,  -0.260070384,  -0.013535942,  0.98265475,  1.80172539,  0.641262054,  -2.24986696},{ -1.69329047,  -1.43821669,  0.00537714176,  -0.209927052,  0.710826993,  -1.23141825,  -0.586366236,  1.08309424,  1.07949078,  -0.429191589,  0.288495988,  -0.129267886,  -0.48861897,  -0.406911671,  0.205953404,  2.07909203,  0.400138438,  1.13392091,  0.69106847,  -0.652716041,  0.206596121,  -0.384299725,  -0.536960065,  -0.980776906,  -0.0446811318,  -0.474911809,  -1.33615899,  0.906038582,  0.0151466625,  -0.107954264,  -0.796809673,  -1.54358482,  0.0370461494,  0.24526988,  -0.728497803,  -0.132055223,  -0.770183444,  -1.12103498,  -0.313224435,  0.765426457,  0.934641182,  0.279820442,  0.111286558,  1.44619739,  -0.445230335,  -0.546570301,  2.06122112,  0.809742749,  -0.851315737,  -0.542014241,  -1.13474035,  -1.43652987,  0.599213064,  0.214292511,  0.789393485,  -0.444301873,  -0.825149953,  -0.571679175,  -1.14268541,  0.0216841474,  -0.371707469,  0.91397202,  1.51519561,  -0.323837817,  -1.58896124,  0.627544463,  0.885268033,  2.07946706,  0.0832445174,  0.460807562,  -1.21067345,  -0.448356241,  -0.570316553,  0.740032136,  -1.18402398,  0.742353857,  0.0467812195,  -1.27436149},{ 0.752290189,  0.424614966,  -0.928435683,  0.0771973953,  0.671165705,  0.533501565,  -1.13711405,  0.55945456,  -0.380434602,  0.540607989,  -0.128207669,  1.13962996,  -1.54655647,  -0.987720013,  -0.396768153,  1.52034605,  -0.592991531,  -0.537362218,  -0.4978531,  0.176789671,  0.3339459,  0.74310571,  -0.313899785,  -0.394495785,  -1.35565639,  -0.78188467,  1.73815525,  0.432118386,  0.404975116,  -1.49917972,  0.537075698,  -1.06751072,  0.196551189,  -0.466731668,  1.13330829,  0.830624163,  1.9695586,  -0.00711300224,  0.990498066,  -0.683769107,  -0.56323421,  0.088777639,  -0.632382631,  -0.0261694547,  3.05762124,  0.326895297,  0.631557584,  -0.167736322,  -1.05585825,  0.740154088,  0.35207659,  0.571572006,  -0.132745743,  -0.462408096,  -1.11425662,  -1.33653164,  1.22517681,  -0.167114794,  -2.88437796,  0.349362582,  -1.72043192,  -0.0352026485,  -0.106149398,  0.744266689,  -0.701129079,  0.0579624064,  -0.357266307,  -0.303732604,  0.294871181,  -0.317202985,  -0.725409806,  -0.837697506,  -3.50347471,  -1.42810059,  -1.45878887,  0.26542154,  1.17496121,  1.09558856},{ 0.756954312,  0.774956763,  0.811466575,  -0.487855256,  1.47559643,  -0.383623958,  2.05817103,  -0.746162355,  -0.906225026,  1.03554308,  -0.598186612,  0.446359158,  1.22412336,  0.432685822,  -0.94858253,  -1.19896233,  0.655710816,  -1.94870663,  0.247673675,  1.68957603,  -0.22233592,  -1.61621511,  1.06898451,  -0.760790646,  0.638664901,  0.506641865,  0.533715367,  2.11639023,  0.361456186,  -1.15801728,  -0.323611379,  0.680816531,  -0.82557559,  -0.526292384,  0.289327085,  -1.10873008,  -0.161665097,  0.678442061,  -0.397804976,  -1.143224,  0.568974793,  1.23845184,  0.0763234869,  0.317816168,  -0.835539341,  0.016815044,  0.403278112,  -1.02723968,  -0.548792422,  0.897035837,  -0.954817474,  1.36149645,  0.672164738,  -0.25461489,  0.927627742,  0.311904252,  0.272733986,  0.786964834,  0.197523698,  0.278790593,  2.55529261,  0.531115651,  0.769411743,  -0.337468684,  0.309297055,  0.18869108,  -0.363666028,  -0.78873688,  -0.0558713675,  0.0883418769,  -0.72616291,  1.12806916,  1.17712569,  -0.900261998,  -1.0060935,  0.845231056,  -1.31950748,  -0.543634117},{ -0.637445033,  0.566662252,  1.29733562,  0.921527147,  -0.88351047,  0.496519357,  1.22216892,  -1.02526999,  -0.174120188,  -1.53404951,  0.348024607,  -0.810083508,  -0.0139477616,  -0.573268175,  0.956492186,  2.12077737,  -0.561838806,  -0.869039893,  0.571566164,  0.049980212,  -0.11170277,  0.888031721,  -0.689123452,  -0.593460917,  0.305856675,  -1.53792286,  0.0498123579,  -1.39899433,  -0.701298058,  0.635250032,  0.228573531,  -0.0821858868,  -1.06439555,  0.59251678,  2.62893176,  0.470154285,  -1.26410401,  -1.11152089,  -0.239421278,  0.171676755,  -1.31359124,  -2.77732134,  1.29294562,  -1.13304698,  -0.269109726,  2.20220542,  0.137725413,  -0.260117382,  0.676628709,  0.469119251,  -0.413475156,  0.441752464,  1.46032357,  2.1874609,  0.287758857,  0.949232042,  -1.07459807,  0.482552022,  0.683134139,  1.30899239,  1.19537354,  1.49431372,  -0.122336999,  -0.315131724,  -0.614278495,  0.290220141,  0.438523442,  0.313797385,  -2.48786592,  -0.0222693179,  0.0778030604,  1.10646892,  0.488824427,  -0.109306693,  0.354256064,  0.416831702,  -1.04035747,  0.434914231},{ -0.0721070021,  1.36939967,  -0.448679328,  -0.402201414,  1.15455234,  -0.185266137,  -0.488573015,  -0.881058931,  -1.09768128,  -0.209351316,  0.42879796,  0.603144825,  -0.0855604336,  -0.130821392,  -0.832429826,  -0.128151715,  0.5372684,  0.638058662,  0.107099295,  -1.86942506,  -0.811180532,  0.324619949,  -1.13048911,  -1.7279278,  0.474968702,  -0.707812011,  1.36522222,  -0.474683911,  -1.53036535,  0.363606691,  0.478300303,  -0.268237233,  -1.30539882,  0.112181149,  0.444123924,  -0.0252454802,  1.82668722,  0.785456955,  -0.665687799,  0.249464825,  -1.22442627,  0.802120447,  0.847201347,  -1.34253192,  -0.919775665,  -0.917396426,  -0.00216700346,  0.570611954,  1.37795222,  0.649035752,  -0.268515199,  -0.22620827,  1.15389335,  -1.4767741,  0.451216578,  -0.135487825,  1.01024294,  -1.37907577,  0.0966773778,  -0.179255098,  -0.63360256,  -2.50680375,  0.109910876,  -0.15374127,  1.45724964,  1.54999745,  1.35387814,  -0.108056769,  -0.792167962,  -0.446576715,  -0.191241398,  1.95057774,  -1.1579268,  0.235507607,  -1.36697614,  0.888719976,  0.147760525,  -0.964480579},{ -1.50184858,  -0.180759162,  -2.16888142,  0.851185381,  -0.73116976,  0.463703722,  -0.156863227,  0.221347243,  -0.343964398,  -0.619651675,  -2.35947919,  0.809037507,  -1.55869091,  0.131125808,  -1.44354761,  -0.0101902662,  0.996128142,  0.308112532,  -0.839283645,  1.35274029,  0.813437164,  -0.481430739,  -2.10102844,  -2.10631418,  -0.949331284,  0.0996587053,  -0.288533747,  -0.096050106,  0.0955431461,  -2.38289022,  -1.44222796,  -1.05891407,  0.277232945,  0.80703938,  0.935843229,  1.28303707,  0.517367899,  0.0449386537,  -1.04784405,  -1.74886382,  0.493191928,  0.967539191,  1.17532289,  -1.56449091,  -1.14700627,  -0.266628891,  0.78128159,  -0.438703179,  1.51666331,  0.994553089,  0.720168531,  -0.756623328,  -0.0856377333,  -0.0393224321,  -0.814306796,  1.98494542,  -0.555463016,  -0.12278232,  0.179081202,  -0.391265899,  0.245504484,  -0.209108874,  -1.79027653,  -1.0301187,  0.0508451425,  -0.388781428,  0.139252529,  -1.10641706,  0.534967363,  0.0466255955,  0.914193332,  -0.00102811132,  0.320011139,  0.443458736,  -1.85851741,  -0.140578762,  -0.281993747,  0.522404909},{ 0.735686719,  0.109732434,  1.13308167,  -0.854477823,  -0.157991171,  0.133658364,  1.5386343,  -0.134793267,  -1.88338518,  -0.950419366,  -1.61501288,  0.723355055,  -0.88078922,  -0.337182641,  -0.606918693,  -0.285836637,  -0.312761396,  -0.376219004,  0.596487224,  2.78697848,  0.92459774,  0.941899896,  -0.96521008,  0.116596498,  -0.672731936,  0.51976341,  0.716382623,  0.363294333,  -1.09157741,  1.24315667,  0.678239107,  -0.331890166,  2.31660891,  -1.12857163,  -1.18489122,  -2.12414122,  -0.00585300801,  -2.05452442,  -0.237045273,  0.0871749669,  -2.1218791,  0.434871048,  -1.38320231,  1.15278065,  1.02424729,  -0.16082485,  1.70669413,  1.2226119,  0.151461199,  -0.124280997,  0.291752875,  -0.0883895606,  -0.599751711,  1.09008062,  0.598261058,  -0.73800081,  -0.324462861,  3.73438859,  -0.0123000341,  0.44305566,  1.32459664,  2.43576241,  -0.641609788,  1.00447071,  -1.65762925,  -1.07512498,  -0.394954473,  1.24926984,  -1.20745659,  0.364722222,  0.48707673,  -0.920468628,  1.26490951,  0.636617839,  -0.83346647,  1.0869658,  0.123132333,  1.37913847},{ 0.0866045132,  0.215504467,  -0.0126922512,  1.95189953,  0.237528026,  0.629731536,  -0.962680876,  0.267271459,  -0.793154955,  0.41747424,  0.294797182,  0.0810813531,  -0.0537903942,  -1.95929897,  0.0212907679,  -0.715488791,  0.995269895,  1.80776072,  -0.156207666,  0.510630965,  0.202610537,  0.380770534,  2.2579143,  -1.74948847,  -1.49866617,  -0.53230989,  -0.535930991,  0.242196381,  0.586205721,  -1.80775106,  -1.00247812,  1.25188613,  1.24630642,  0.686263859,  -0.939465404,  1.1727618,  1.46410513,  0.811466753,  -0.351651549,  1.83669794,  2.24475861,  0.621438146,  0.553496361,  -0.121817529,  0.686970353,  1.16345441,  0.73500663,  -0.566438138,  0.850941002,  1.17673242,  0.706170499,  2.52388787,  0.723150551,  0.792094827,  -0.356654495,  -0.901376963,  1.8575356,  0.327676207,  0.832608342,  1.02292633,  -0.607647717,  -1.21364319,  -0.684760034,  1.67529356,  2.18207049,  -0.0938013345,  0.265312403,  0.721302032,  0.694354415,  -2.99524736,  -0.0787240639,  1.01666188,  0.738982856,  3.11656213,  -1.51922596,  -0.180401951,  -1.91496503,  0.746075034},{ 1.20681,  -0.845812619,  -1.70553112,  -0.827620089,  0.619614482,  -0.318352133,  -1.76665604,  -0.550560057,  -0.315028757,  -0.042416025,  1.03770077,  0.585331321,  1.13468242,  -0.0598979294,  1.44559205,  0.78503412,  -0.171148911,  0.117385507,  -0.133674636,  1.31833029,  -0.900622308,  -0.200745493,  -0.724452972,  0.722261548,  -0.504643381,  0.968323231,  -0.276496649,  1.19651389,  -0.60100764,  0.375653118,  0.016920533,  1.19996512,  -3.1671896,  -0.134438857,  -0.337833494,  1.82900655,  -0.188151121,  0.234663412,  -3.69108677,  1.68394685,  0.0891546831,  0.283593744,  -0.136899561,  -0.542374074,  -0.540248871,  -1.11366165,  -0.440544426,  0.0606457815,  0.360498756,  0.0229628514,  0.164190263,  0.0248338934,  -0.947630346,  -0.428647518,  0.62460649,  0.515223801,  0.530523777,  0.685525954,  -0.284375519,  -1.10473073,  0.562415957,  1.95531678,  2.11421633,  1.73307323,  0.0858955681,  0.86434859,  -1.49724913,  1.90739679,  0.817170739,  0.194987476,  -0.0865750387,  -0.692516863,  0.484203875,  2.06847167,  0.465902835,  -0.790473461,  1.05236781,  -1.58439529},{ -0.167208493,  -0.314201444,  -1.09404874,  2.03599572,  0.551370144,  0.599196851,  0.068175301,  -1.1490128,  -0.804983974,  0.0795583054,  -1.16578841,  0.552256048,  0.119658686,  0.097359851,  -2.21170831,  0.889581501,  -0.425204098,  -2.19422412,  -0.668842852,  2.42976093,  0.786001921,  1.5613414,  -0.691061854,  2.04717183,  1.08765745,  1.75390244,  0.10654939,  -0.84846729,  -1.0276432,  -0.39537999,  0.827068388,  -0.517792106,  -0.218861133,  0.801023245,  -0.10351,  1.06780183,  -0.559791327,  0.573215902,  1.54432154,  0.535347641,  0.445030063,  0.299287856,  -0.886299372,  -1.30098283,  -0.582686663,  -0.20582369,  0.616317451,  -0.553897798,  -1.05045855,  -1.20348597,  1.38406003,  0.229563102,  0.321220249,  -1.26541269,  1.30211854,  0.522649705,  2.86961436,  0.536392093,  -0.291389376,  0.906956196,  -1.29978132,  -0.521336854,  -1.2115972,  0.997096419,  -0.14396736,  0.25146386,  -0.738612473,  0.901580453,  2.04530931,  0.664395273,  -0.501352847,  0.464417905,  2.07090735,  -0.760175824,  -0.479108572,  -0.252937704,  0.889016926,  -0.0163210221},{ 0.0222034305,  2.20693564,  -2.73793769,  -2.10768628,  -0.232937127,  0.203085601,  -0.186818868,  -0.752647758,  -0.375240982,  0.950914443,  -0.62698096,  0.771760345,  0.0775946677,  -0.715969801,  -1.62626743,  2.13503051,  1.15529799,  0.0275432542,  0.724363148,  1.04676116,  1.25581741,  1.11666727,  0.425492406,  0.472745746,  0.618171453,  -0.0314254351,  0.495568395,  -1.58595681,  0.393792629,  -0.722621918,  0.176689863,  0.0491967797,  1.57561862,  0.371519655,  2.42613459,  0.580319583,  -0.360263705,  1.17083561,  0.424641371,  -0.48147282,  -1.16303015,  0.193393886,  0.656748772,  -1.18661904,  -0.85140264,  0.503296494,  -1.04951882,  0.585406363,  -0.0642182603,  -0.0241260454,  0.622652411,  0.42587328,  0.43257454,  1.64349413,  -0.106471561,  -0.441771328,  0.108897857,  0.504867733,  -0.739635289,  -0.800238252,  -1.37976849,  -0.286517888,  -1.3039099,  -0.0351786017,  0.477768898,  0.688572407,  1.08739448,  1.87196076,  -0.0639335364,  -0.173426762,  -1.39410746,  1.26401889,  -0.647971272,  0.169541061,  0.186924562,  -0.676724136,  0.404465735,  0.982933104},{ 0.974804044,  0.955511689,  0.649770081,  -0.950631022,  0.480746031,  0.965204954,  1.27489972,  -0.0222901404,  -0.748120725,  -0.0504590981,  0.293815196,  -0.421368152,  0.86578244,  -0.616103053,  0.393912256,  -1.39115655,  -0.406356841,  -0.84316498,  1.44019067,  0.421090335,  -0.575349927,  0.388702303,  0.289642125,  -0.748869956,  -0.933417678,  -0.118760653,  -0.76274091,  -0.803913772,  0.550575495,  -1.1874274,  -0.581948578,  -0.973820925,  0.157477558,  -0.0909930766,  -1.09666002,  0.697583258,  -0.931275308,  -1.3068583,  3.02901793,  1.41818452,  -1.02388537,  -2.72965884,  1.39525282,  -1.25764894,  1.14796221,  -1.83640325,  0.374836922,  0.437330544,  1.31011963,  -1.41864252,  -0.275904745,  -0.927691221,  1.0545373,  -0.230069667,  0.121488407,  -0.491280377,  0.841501415,  0.859545648,  -1.10059488,  0.501796424,  -0.350497454,  -2.64205885,  1.53802037,  1.10952878,  1.13346589,  -1.40791214,  0.936006904,  0.23057799,  -1.15404129,  -1.34989429,  0.665483177,  0.128121421,  -1.22505653,  -1.00312531,  0.100120381,  -0.0161644015,  -0.579634309,  0.529995382},{ -1.71120965,  0.0124856923,  0.560080051,  0.142519072,  -2.40013933,  -1.33178616,  -1.21132636,  -0.672664046,  0.424752265,  -1.24439228,  0.0926307216,  1.65060973,  -0.974387288,  -0.633259773,  2.68680453,  -0.68816036,  -0.831653953,  1.46870315,  -0.66302067,  0.480344772,  -0.909168303,  0.209807605,  -0.215497643,  0.315825373,  -1.54984879,  -0.0800677985,  -1.57498229,  0.107842557,  0.283306569,  -0.510676801,  -0.885662377,  -1.05370569,  0.0468476899,  1.2043041,  -0.690199792,  -1.24329388,  -1.1739949,  -1.01541698,  1.02393699,  0.505861044,  0.487800717,  -0.417147517,  -0.81100446,  1.34501028,  0.462505668,  -0.297524244,  -0.640069067,  0.823763847,  -1.6223824,  0.633643329,  -0.173313946,  -1.59770834,  0.625857472,  -0.185790077,  -1.05191636,  4.05635357,  -0.333527058,  1.31781399,  0.149289697,  0.397901267,  0.81302166,  -0.556434751,  -0.360395312,  -1.15581024,  0.744405568,  2.07706833,  0.801406682,  1.1673063,  0.0347835831,  0.0703279823,  -1.5497402,  -0.535332024,  0.842522264,  0.756303549,  0.109220639,  0.571436524,  -0.175628364,  0.42694515},{ 0.0484882556,  0.233173668,  -0.135154292,  -1.38761437,  -0.167795166,  0.302436918,  -1.90105116,  0.246156171,  0.517239988,  0.840891778,  0.411393136,  0.190437675,  0.148155421,  0.849830568,  0.339203596,  -1.39076221,  1.24096632,  -0.354758322,  0.664421797,  -1.65685797,  -0.307385892,  1.14772451,  -0.464639306,  -1.37352443,  -1.14217186,  0.595478415,  0.258716047,  -0.156728208,  -0.642261624,  0.460349441,  -2.06072831,  -1.76305711,  0.0732378811,  -0.999864995,  -0.153381079,  2.07981682,  -1.44417787,  1.56552541,  -0.848051548,  1.08878839,  -0.227184728,  -1.74762654,  0.598935962,  -1.47697842,  -0.879350245,  0.560097754,  1.05039668,  -0.450928718,  0.284962654,  0.221129745,  -0.779157341,  -0.264391899,  0.593627751,  1.98390472,  0.733497739,  -0.990294993,  0.993980765,  -0.27294749,  1.83355975,  0.435543865,  0.556698799,  -1.34844816,  0.357453316,  0.701215684,  -3.07076073,  -1.63255,  -0.45582214,  -0.575696111,  -0.190671682,  -0.563817739,  0.705343187,  -0.98747772,  1.8777256,  -1.28996229,  -0.755150616,  0.503482461,  -0.99125725,  -0.727553427},{ -1.78501511,  -0.481876224,  -0.334966093,  -0.301038891,  -0.778790474,  -0.511988699,  -0.55125767,  -0.438097388,  -0.649524629,  1.5809077,  0.213605165,  -1.72453618,  0.162584484,  -0.802662075,  -0.168516114,  0.0242983662,  2.10799575,  -0.384461641,  -0.536039472,  -1.50068831,  0.130418435,  1.54555821,  -1.12927735,  0.421210229,  -0.535854757,  0.723623693,  -1.69500256,  0.238612488,  0.918204486,  -0.199197397,  -1.69592714,  -1.28063321,  0.824105561,  0.655508697,  -0.991134942,  -1.25785577,  -1.38657856,  0.353817701,  0.358129263,  1.03038216,  0.542224109,  -0.981230736,  -1.33655632,  0.68569082,  -0.873659909,  0.191625476,  0.144339368,  -0.312666804,  0.644146144,  0.0566538423,  0.0593643598,  0.531776905,  0.723526955,  0.435807437,  0.120743386,  1.99120045,  -1.18112314,  0.56716311,  1.35307097,  -0.690822244,  0.749943018,  0.227204531,  1.71019125,  -0.642654061,  -1.22248077,  -0.473616391,  -1.0289501,  -0.242985606,  1.14464295,  -0.0880468413,  1.04487395,  0.264153928,  0.0701769739,  1.57717717,  0.089864634,  -0.402947009,  0.115363844,  0.927625597},{ -1.00103176,  -0.565585613,  0.968299329,  0.0775773525,  0.520443201,  0.319388032,  0.140637606,  0.0359477177,  -0.455759287,  0.262961447,  -0.861256957,  -0.80453229,  1.7244724,  0.196247667,  1.00768256,  1.01128316,  1.20013082,  -1.72596169,  -0.935117185,  1.10044193,  1.69202065,  0.997611344,  -1.15140831,  1.37570655,  -0.809452116,  2.10736752,  1.2674818,  0.61954248,  -0.500645816,  -1.13937426,  -1.48598993,  0.741181493,  1.91622949,  -0.271837026,  0.0508546233,  1.41025972,  -1.29664624,  -0.240771204,  -0.674349725,  -0.785752416,  -1.1151638,  0.943535924,  -1.27549362,  0.297070861,  -0.370460242,  0.952760279,  0.556423247,  0.382772654,  -2.98495889,  1.63918781,  -0.873699784,  -0.410064667,  1.09424996,  -2.33111358,  -0.75044322,  0.520416558,  -0.924737275,  0.40208894,  -1.87266493,  0.0767465383,  -0.345479041,  1.13938189,  0.0364913046,  0.209995121,  -0.92111975,  -0.821356177,  0.653054535,  -0.912928045,  -0.181083262,  0.080924727,  1.37845862,  0.473492712,  -2.08611274,  -0.701821506,  0.323267788,  0.483281642,  -0.187258571,  0.565861642},{ 0.971278489,  0.630972087,  1.11418366,  -0.580015659,  -0.455204576,  0.0202927832,  0.412960559,  -0.143705115,  -0.648807406,  -0.916328788,  0.321864843,  -0.631707072,  1.07544506,  -0.678924382,  -0.268686146,  -2.22030687,  -0.577695787,  0.743807554,  -0.137775004,  0.168977544,  -0.824252605,  -1.06176317,  -0.308213234,  0.722223699,  0.465382457,  1.91614926,  0.779098213,  0.265821218,  1.48610914,  -0.0739524215,  -0.686333477,  -1.24708974,  0.537312806,  -1.75096655,  0.769916654,  0.0244575143,  1.17809451,  -0.305669844,  -1.22060978,  -0.0790552944,  -0.595421076,  1.38755155,  0.216922611,  1.36429012,  0.219775081,  1.10330439,  -0.62860316,  0.742401302,  1.53838181,  -0.312761784,  0.0221991818,  0.700600564,  1.40455484,  -0.689340174,  -1.2722497,  0.00965301134,  1.29517436,  -1.87998188,  -0.148097456,  -1.95935392,  -0.197046399,  0.725657463,  -1.14985538,  1.70176113,  -0.414559186,  1.03564787,  -1.07706606,  -1.69558167,  -1.5529331,  0.118743792,  0.480012923,  -0.341642976,  -0.441782504,  1.1394161,  0.354628921,  -0.709265053,  3.18864441,  -0.404545009},{ -1.18353236,  0.809961557,  0.365015984,  1.3249898,  -0.284762084,  -0.874289513,  -0.471504748,  0.183326289,  -1.56473172,  0.202655911,  1.32670593,  -1.77788615,  1.34551561,  -0.223528579,  -0.195733413,  -0.154302403,  -0.847312987,  1.01702857,  0.629480839,  -0.271780044,  -0.342680991,  -2.14833808,  0.332097828,  -0.676942647,  0.813370049,  -0.0925978348,  -1.07829773,  0.11290554,  -0.0575704984,  -0.93982321,  -2.03327942,  -1.97006655,  1.32971323,  1.62554634,  -1.46526635,  -0.132038549,  -1.2437042,  -1.06418574,  1.43352246,  -3.40943599,  -0.461419165,  -2.05540705,  -1.32888591,  0.0503558703,  -0.717345774,  1.36164379,  1.08182621,  -0.364764363,  -0.422867298,  -0.134826675,  0.460267097,  0.0594132394,  -0.675659835,  2.06921029,  -0.088618122,  -1.45736885,  -0.15082252,  -0.16111204,  1.84723628,  0.554284692,  -1.23617363,  -2.57466221,  -0.85523653,  0.540090621,  1.04278064,  -1.10555947,  0.432709664,  0.0121733928,  -1.00394905,  -1.78357255,  0.425510705,  0.389623135,  2.48710728,  -1.13233495,  -1.16347313,  -0.820273221,  -2.26652765,  0.881520152},{ 0.399018675,  0.248860151,  -0.67576158,  -0.66240406,  0.0221446026,  -0.0823773593,  0.887004077,  -0.704179764,  1.07457864,  0.268936396,  0.8303684,  0.759286165,  -0.545520723,  -1.4214586,  -0.995948672,  -0.51329124,  1.1992265,  -0.726698995,  0.678836346,  1.7593962,  -0.132007986,  0.460717022,  1.08289993,  0.10515973,  0.163388953,  0.629498303,  -1.20959973,  1.53588009,  -0.978911102,  -0.808931053,  -0.384358197,  1.27215016,  -1.19647205,  1.2647239,  0.239643514,  -2.03677034,  -1.68236959,  -0.588164032,  -0.2968463,  1.07761562,  1.14465845,  -0.881114781,  -3.47079182,  1.03696895,  0.484854162,  0.296904176,  1.49907756,  2.344836,  -0.0972350687,  -0.948413372,  0.965279341,  -1.03139281,  -0.245343655,  -0.252716631,  -0.685833216,  0.836074948,  -0.21012637,  -0.42195937,  0.500487506,  0.814615309,  -0.229486004,  0.0743162483,  -0.923203111,  -0.25371924,  -0.260519683,  -1.18902981,  -1.43251169,  -0.260207176,  -1.38923752,  -0.614618897,  -0.241302937,  -0.176066592,  0.297039002,  -0.332362622,  0.368431956,  1.1868788,  -0.395523995,  -0.853152215},{ -0.851997733,  0.681494415,  0.26209572,  0.884343266,  0.954344392,  0.948481679,  1.18506789,  0.599660099,  -0.560827076,  -0.6980744,  -0.848648131,  -0.0296835285,  1.10112119,  0.0405894481,  1.04736042,  0.892052889,  -0.0278464817,  0.692642152,  0.0480950996,  -0.406075925,  -0.121136397,  -1.42726696,  0.15738295,  -0.542769909,  -0.173416883,  0.425753683,  1.08139992,  -2.86855936,  -0.383206069,  1.5543946,  -0.0464869142,  0.473161787,  1.0303899,  -0.205360845,  -1.86110294,  0.184203699,  -1.40214956,  0.13521789,  -0.607351661,  0.582937241,  -1.70534837,  2.04427075,  2.38081384,  -0.443157017,  -0.0982805789,  -1.67167985,  -0.715546548,  -0.212415203,  0.142155752,  -0.414859265,  1.47385061,  -0.31713903,  -1.85620236,  2.1898818,  0.000904873887,  -0.745645583,  -1.21362269,  0.00638773292,  -1.17427635,  0.0189435165,  0.276523173,  -1.66902828,  -0.723165393,  -1.83779418,  -1.16295505,  -1.67325807,  -0.619557619,  -0.826512158,  -0.215296492,  -0.0154948495,  -0.65328902,  -0.978463471,  -0.048844181,  -0.744713902,  -0.360090584,  1.54458964,  -1.8025099,  -1.70629025},{ 0.902091861,  -1.32702577,  -0.059654329,  -0.706082225,  -0.484334141,  1.21800995,  0.728611708,  1.03226006,  -0.411862105,  -2.0220449,  0.810279369,  0.419354916,  2.51240468,  -0.308097214,  1.6344589,  -1.78516197,  -0.272428483,  1.31198704,  -0.534125328,  -0.635756731,  -1.29838884,  -0.126919746,  1.26113796,  -0.0180028379,  -0.273967236,  -1.00664103,  2.61844516,  -0.154600129,  0.176431879,  0.433745831,  0.526795924,  -0.260638118,  0.48105824,  0.468322814,  -1.66747928,  0.24863106,  1.17801952,  -0.472401887,  -0.0817267895,  -0.330341548,  -0.760852993,  -2.56125522,  -0.820880651,  -0.190703183,  1.16890645,  0.220247746,  1.45267355,  -2.90345001,  -1.25749207,  -1.93651664,  -0.511760116,  0.294789881,  0.674924314,  1.47336793,  2.27941179,  -0.0166297164,  -0.910949707,  0.685697436,  -1.33978093,  1.60149944,  -0.916383505,  -0.603819847,  -0.280375779,  -0.929781497,  -1.41735041,  -0.830696762,  1.25803447,  -1.80656803,  0.408645123,  0.508906484,  0.719156682,  0.136326343,  -0.702090979,  0.43178761,  0.572084606,  0.305695027,  -0.268747777,  0.965683103},{ -1.85234988,  -1.0195632,  -0.323521525,  -1.0417031,  1.01598155,  -0.269859105,  -0.0312869996,  0.245410919,  0.229806811,  0.77036792,  -2.2096436,  -0.513680935,  -0.421834648,  1.16999304,  -0.553882599,  1.81771886,  -0.055053059,  0.596372128,  -0.177915424,  -0.435991585,  2.10626745,  -1.56921697,  2.05450249,  -0.906213522,  -0.970798671,  0.295068353,  -0.893877327,  2.23529315,  1.22056377,  -0.624189913,  -1.64233351,  0.625568151,  -2.53357339,  -0.345230818,  -0.534522414,  1.93654644,  0.0881058201,  1.0947262,  -2.42297435,  0.523450077,  1.23368716,  -1.00210214,  -0.541741788,  -0.0746607259,  -1.66302717,  -0.57750994,  0.122964077,  -0.145724088,  -1.45064723,  -0.342175066,  0.357851475,  0.991942048,  -1.06884801,  0.548132062,  -0.75549221,  -1.05977702,  0.441511542,  -1.35434318,  0.694436848,  0.232654661,  -1.63158453,  0.208952889,  0.585580409,  -0.663520575,  -0.652579486,  -1.63309371,  2.17580795,  0.0279001351,  -0.938592672,  -1.35459375,  1.86844778,  -0.204989478,  -1.93348765,  -0.22373727,  -0.743626356,  1.40619361,  0.791397691,  -0.0277199429},{ 1.17619431,  -0.418894857,  -0.554019213,  -0.348525882,  0.148593768,  -0.479196429,  -0.290161699,  0.253749609,  0.524495006,  1.18848121,  -0.342499942,  -1.04631829,  1.39813364,  -0.174118832,  1.15586007,  -0.664194703,  1.48159552,  2.95993781,  2.26184392,  -1.44926941,  -0.150319263,  -0.602641702,  1.28356445,  -0.437644452,  -0.6301682,  -2.17256141,  -0.302946031,  -0.337532133,  1.9167639,  0.2485632,  -1.12589908,  0.358761013,  0.448943675,  0.0171329305,  -0.0637422055,  0.419167072,  2.87838054,  -0.179593772,  1.33632505,  1.40356851,  -1.65263903,  -0.710692286,  -2.16242409,  0.843790174,  -0.543716311,  -0.561935663,  0.414753854,  1.28651369,  1.40081,  -0.871751368,  1.08164573,  -0.276654899,  2.0759418,  1.24939227,  -0.537265062,  0.253889382,  2.09882021,  -0.621352851,  -1.09512734,  0.0314442627,  -0.22189419,  0.934662223,  0.48936224,  -1.07706022,  1.27649605,  1.55345535,  0.708442688,  2.04383898,  0.122454368,  -0.324520648,  0.703221321,  1.11096251,  -0.545431852,  0.643237472,  -0.284824461,  0.712700784,  0.832517087,  -0.604526699},{ -1.50157404,  2.58120298,  0.0418440774,  0.312835187,  -2.07478046,  0.395238787,  2.3700285,  -0.271478862,  -0.131429568,  0.238454551,  -0.464883089,  -0.275223821,  -0.167371571,  -1.32694232,  -1.31348014,  0.287406087,  -0.0567723662,  -1.08268487,  -0.69034332,  -0.347354352,  -1.3806622,  -2.09584308,  0.513828993,  -0.337505132,  0.613879442,  -1.50944436,  -2.72462153,  -1.11900795,  0.246287271,  1.18370402,  0.800214291,  0.716412961,  0.785563707,  -0.863913476,  0.649805069,  -0.782811344,  -0.387712479,  -0.38532865,  0.00193778658,  -1.7085557,  -0.201844066,  -1.34136462,  1.01191401,  -1.48523784,  1.74094999,  -1.04621661,  -0.605115831,  0.00868575554,  -0.962555647,  -1.51918721,  -0.97477746,  -0.839447439,  1.72439492,  0.0858186483,  -0.985067785,  0.238057852,  -0.892722011,  0.250568688,  -0.162007645,  -0.967393517,  -1.73741329,  1.35094726,  1.02974546,  0.704686582,  2.17539835,  0.314700425,  0.110552765,  -0.569869697,  -0.877071857,  -3.27924919,  0.0678403154,  0.519319534,  0.224808112,  0.396597475,  -1.17579842,  0.672432005,  0.959905684,  -0.227839589},{ 0.627733231,  -0.839273036,  0.190666884,  -0.57460624,  0.109432377,  0.900450587,  -1.42454255,  0.433328182,  1.07370126,  -1.39057338,  0.54103893,  -0.586227775,  0.406013548,  -0.270844966,  0.410034746,  0.285094857,  0.299219877,  -0.872787833,  -0.0354761072,  0.157737732,  0.0997122377,  0.786721349,  -0.314668268,  0.616261721,  1.46413887,  -1.11564231,  2.14325285,  1.23936355,  1.28181386,  -0.327120453,  -0.43079257,  -0.915178597,  -0.503275275,  -1.04298472,  2.97801924,  -0.548764944,  -1.16009068,  -1.06100237,  0.679274201,  -0.95584178,  1.70461631,  0.562607348,  -0.25992462,  -0.50440222,  -1.07630992,  0.21793139,  -0.0425668731,  2.20178056,  0.294623673,  1.24426436,  1.15918195,  -3.54454565,  -0.387403518,  0.409827381,  0.357690573,  0.96584475,  -1.31231856,  -1.33864117,  -0.440019369,  1.15243554,  0.591242433,  1.2854526,  1.6320889,  0.594290674,  -2.16564298,  -1.13276017,  0.138616174,  0.919054747,  1.00931239,  -0.431873292,  -1.11123681,  1.44554293,  0.329532593,  -1.29716766,  -0.86227262,  0.546937406,  -0.69436115,  -1.23352087},{ -0.743713915,  -0.97645849,  0.750460863,  0.490943551,  -0.111643314,  -0.561788857,  -0.7924968,  -0.361899287,  -0.255856723,  -1.09236825,  -1.33958793,  2.40668011,  0.45433557,  1.39715564,  0.756296337,  -0.372493356,  -0.808379233,  0.270423174,  -0.0213844888,  0.110663854,  2.25479102,  -0.984833479,  -0.54516834,  -0.0994128063,  0.25793907,  0.241624728,  -0.186187893,  0.0250299275,  0.429145604,  0.106367081,  -0.443312019,  -1.25013387,  0.553750992,  0.493995965,  -1.04479539,  -1.59918177,  -1.62977362,  2.09380221,  -1.33769059,  -0.431473494,  -2.04050589,  0.164771482,  1.02418077,  -0.0228249021,  -0.283652514,  -0.144767553,  -1.59763122,  0.878671348,  -2.10976028,  -0.456845969,  -0.122233003,  0.114997827,  0.0227452666,  -0.793937325,  -1.09876513,  -0.197260037,  0.429379195,  -2.16696095,  -1.64305782,  -0.168503463,  -0.0793258771,  0.957184255,  2.58531594,  -0.359389991,  0.106911182,  -0.736869156,  1.04109406,  -0.235111088,  -1.66654146,  1.7417345,  -0.063776046,  0.939711213,  0.00705845887,  -0.365721583,  -0.496835709,  -0.822184503,  0.725468516,  -0.409052849},{ 0.76599443,  0.446097225,  1.03117073,  0.175470963,  0.888700128,  0.981998622,  0.266307086,  0.412114382,  2.65342426,  -0.0845790356,  1.32822311,  0.108499445,  -0.274309486,  -0.695370257,  -0.404580325,  -1.22358418,  -0.477402151,  0.990155935,  -0.0837658197,  -2.17957783,  0.297157794,  -0.518747866,  0.839954436,  0.898723662,  0.131051809,  -0.675318122,  -0.0288065374,  2.01244974,  1.51714349,  -0.592770517,  -0.835464299,  0.232456058,  1.16041231,  0.87550205,  -1.6033088,  1.45808136,  0.53882134,  -0.819392264,  -0.381930023,  -1.32465053,  -2.01281667,  -1.68754506,  -0.202070192,  0.424099863,  -0.152405351,  0.523425102,  -1.71555793,  -0.187057421,  1.65188217,  0.685805678,  0.931608856,  1.36258972,  -0.938984394,  0.397373796,  -0.118340172,  -0.385762066,  -0.217431456,  1.98299015,  0.154977664,  0.366633117,  -0.190353334,  -0.610597849,  1.17906761,  -0.761361718,  -0.551060438,  2.84773517,  -0.662196457,  -1.11783612,  0.188744098,  0.0965500176,  0.578730822,  -0.46398896,  -0.396638334,  0.19750683,  0.433603466,  -0.832334816,  -1.02989423,  2.35398531},{ 1.13826799,  1.87067401,  -1.34675908,  0.203335643,  1.26778543,  -0.649035931,  -0.703294456,  1.37511003,  0.724008381,  0.493139595,  -2.10072112,  1.78891253,  0.228917852,  0.282191426,  -0.78152293,  -0.240277842,  -0.32320714,  -2.04559445,  -0.280713826,  -1.19816995,  -0.789587796,  1.12208295,  -0.403157771,  -0.173291415,  1.23512471,  1.17947865,  0.446306467,  0.852694571,  1.40216041,  -0.138198048,  -1.06462801,  -0.0406190492,  -0.0650195926,  -1.71107781,  1.52364779,  -1.2252115,  -0.0536056571,  1.10454547,  -0.755832314,  1.57717121,  1.09942305,  -1.8221848,  0.594754279,  1.681916,  0.0997235551,  1.2343936,  0.919800043,  0.639028251,  -0.519957542,  0.322826236,  0.902124166,  0.574611485,  -1.86433434,  0.108055182,  0.796504319,  -1.0775969,  0.262303621,  -0.218252584,  0.181704491,  -1.70453668,  0.299387723,  0.723220766,  0.136659995,  -0.293546826,  1.24940038,  0.869962215,  0.565215945,  -0.230031282,  -0.578579724,  -1.17569602,  -0.10842555,  -0.547435343,  -0.181336969,  -0.907436609,  0.228611186,  -0.102889463,  -0.3283315,  -0.0433877334},{ -0.166387156,  -0.484322399,  1.25836837,  -0.299601555,  0.547281861,  0.314818114,  0.676122665,  -0.472569674,  -0.933317184,  -0.0375508144,  -0.480506212,  -1.4954921,  -2.05054188,  0.74201864,  0.287248731,  0.047657989,  0.367203921,  0.115070485,  -0.152584195,  0.496354043,  0.65849793,  -1.02775621,  -0.636795878,  -0.852078199,  0.187009037,  -0.0883052722,  0.593928218,  0.543132603,  -0.272545636,  2.16698217,  -1.35415578,  0.601887167,  -2.04399395,  -0.0692859739,  -0.199820623,  0.194720581,  2.04021049,  -1.14368808,  0.809062541,  1.1053164,  -0.137425542,  -1.52543163,  -1.75484073,  -1.04105973,  -0.974029422,  -1.55972564,  0.00429499196,  1.84727979,  -0.041877348,  -1.0998584,  0.469908774,  -0.665814936,  0.36864084,  0.617861152,  0.707481265,  -0.275397629,  -0.815997303,  0.0767889097,  0.682636678,  0.81992662,  -1.29137373,  -0.53480792,  -1.0936532,  -1.01917934,  -0.560300112,  -2.03473997,  0.0511718243,  0.0663059428,  -2.79766965,  0.845466912,  -0.272555113,  -0.186195537,  -0.159890205,  0.651010215,  0.247721672,  -2.97096276,  0.0997258946,  -0.807640016},{ 0.108698189,  -1.20639729,  -0.693595231,  -0.754043639,  -0.106985979,  -0.0356025733,  0.12281324,  -1.03980851,  -1.53268754,  1.69980013,  -0.801786482,  0.115165174,  -0.684895694,  -0.230020419,  -1.23111808,  1.68339097,  0.0540822595,  0.268573195,  -0.117764182,  0.374909312,  1.22207534,  1.15209508,  -1.68210328,  0.341339409,  0.737544596,  2.34724927,  0.00199357187,  -1.02420735,  -0.836358786,  -0.500613689,  -0.559201479,  -1.23875749,  0.847531021,  -0.227860659,  1.05558348,  -1.54237711,  -0.0994083732,  -1.51983106,  1.37672246,  2.40455866,  1.02307713,  -0.183841273,  -0.186419189,  0.124732584,  0.379210174,  -0.666303337,  -1.5599134,  -0.875691473,  1.75884831,  -0.15410544,  1.38157964,  0.520145655,  0.307774931,  -0.247058764,  0.271832824,  0.454335362,  -0.398658991,  0.593723178,  0.192747504,  -0.279488325,  -0.119391419,  -0.645854533,  -0.840846777,  0.487112164,  0.979119182,  -0.866952717,  -1.07990777,  -0.272659868,  0.426369697,  -1.26695096,  1.42962313,  0.296236843,  -0.270305365,  -0.237872332,  -1.27455986,  -0.0965339169,  0.785253465,  -1.77683771},{ 1.28247654,  -0.0291920006,  0.342422515,  1.21806931,  -0.30226469,  0.243461296,  -0.321669728,  0.275154054,  -2.03791404,  -1.69424105,  -0.662669897,  0.135874912,  -0.139492765,  0.779789746,  -1.5168407,  0.398377389,  -0.0583981723,  -0.959124267,  1.23274827,  2.26722956,  0.183982834,  -1.80684781,  -1.44947898,  0.413030148,  -1.53279257,  0.314779371,  -1.10843325,  -0.397528023,  -0.0401397608,  -0.463805437,  0.340747029,  -0.763521194,  0.195269018,  1.76375329,  1.2371372,  0.260612547,  -0.180712923,  0.576307476,  0.188459814,  -0.242724463,  -0.467307895,  -1.49287474,  0.898266792,  -0.650714338,  1.72560692,  -0.22047846,  -1.06361949,  -0.985107958,  -1.36098552,  0.770446539,  0.0795606151,  -0.705708921,  -0.799371541,  -0.199238151,  0.633844197,  -0.776834488,  -0.188972682,  0.300547302,  -0.780934811,  0.427949727,  -0.538354933,  2.12459993,  0.678854167,  1.24382675,  -0.97389847,  -1.19367492,  -0.569803894,  0.199846476,  0.983342469,  -0.456624836,  3.01572037,  1.92983782,  -0.484649271,  -0.815876842,  0.463976055,  1.26738477,  1.59162915,  -0.203589022},{ -0.0410037339,  -1.31628299,  -0.0610885918,  1.58621681,  0.605025768,  -0.746121645,  -1.06659377,  -1.08880901,  -0.307633162,  -0.549368083,  -2.20333385,  -1.09470427,  0.567311347,  -0.54266727,  -0.532333493,  -2.35161519,  -1.43494344,  -0.069373928,  -0.551505566,  -0.229406685,  -0.527599514,  0.404561818,  -0.616997063,  1.04359376,  1.62320566,  1.32913709,  1.05201805,  0.94387871,  -1.62667954,  -0.361083359,  -1.50959051,  1.97240317,  2.22975063,  0.47022289,  0.0685374811,  -0.252487659,  0.522691607,  -0.13931708,  -0.992254853,  -0.136137083,  -0.445442528,  -2.57897425,  -0.850055099,  0.862742841,  -1.20071137,  -0.408956409,  1.05358505,  -0.169097558,  -0.828751445,  1.48108351,  0.747594953,  -1.09621119,  1.32033134,  2.05429077,  -0.534623086,  -1.20150721,  -0.761823058,  -0.869347513,  -0.69045639,  -2.01948023,  1.11108577,  -0.0510963798,  -0.80295682,  0.751850128,  -0.222982645,  1.07140696,  -1.00077057,  1.53429317,  -0.211427271,  0.920434296,  -0.0184489228,  1.39440954,  0.723366499,  0.719955623,  -0.945188701,  0.421348602,  2.30335927,  0.127133042},{ 0.828573525,  1.1392144,  0.650940716,  -1.90914345,  -0.589282215,  -1.2094506,  -1.58479059,  1.25224733,  -1.53440475,  0.60217458,  0.585585117,  0.138335302,  -1.34938943,  -1.87716138,  -0.737916887,  1.83953226,  -1.03910673,  -0.654174387,  -0.574979067,  2.86210227,  -1.6606127,  -0.942018569,  1.34350359,  0.127584949,  1.36069524,  0.368849814,  -1.1836189,  0.0198190156,  -0.214585677,  0.845806599,  -0.12989752,  0.750564575,  0.266139716,  1.48497295,  -1.4794482,  1.1430968,  0.247371435,  -0.609050274,  -1.295102,  0.685114861,  -0.289413661,  -0.674112141,  -0.85428673,  -0.296213508,  -0.759848297,  -0.281441242,  -1.09226298,  -0.11592748,  0.71361798,  1.73487949,  0.0109232133,  -0.609685957,  -0.839226007,  2.32272291,  -0.719140112,  0.419242442,  -0.282592148,  -0.797955573,  1.45482707,  0.896676779,  -0.440760285,  0.519079745,  -0.417592824,  1.61358988,  -0.337646484,  -0.277002394,  -0.943136871,  -0.635513544,  2.43226719,  -0.73874706,  -1.05341613,  -0.107682802,  0.47758016,  1.88173914,  0.106016614,  1.7843703,  -1.58927,  0.680681825},{ -0.427419573,  -0.571071804,  -2.73626852,  -0.751284778,  0.641338527,  1.04512215,  1.23771417,  0.117539592,  -1.53607357,  -1.25061738,  0.0379589014,  1.14527774,  1.59963667,  -0.708104849,  -0.567526996,  -1.53360128,  -0.7655074,  -0.794292271,  -1.33092523,  -0.610868454,  0.310239851,  0.100588992,  0.0108910259,  -0.41728124,  0.392527044,  -0.59733355,  0.744972944,  -0.0373975858,  0.709989369,  -0.138361707,  -0.999916315,  0.346847057,  1.18664539,  -0.077836737,  1.27616858,  0.773440421,  -0.315704018,  -0.853359759,  1.00382686,  -1.43876421,  0.363818556,  0.16512385,  0.570578456,  0.999992847,  1.94515443,  -0.886280477,  0.982389569,  -0.151601538,  0.270417064,  -0.264893323,  1.66861451,  -0.241220996,  1.55081272,  0.634112,  -0.794912517,  0.838809192,  0.39330259,  0.284936577,  0.0536192693,  -0.287739158,  -1.14303493,  -0.957252443,  -0.367899269,  -0.602117658,  -1.13019907,  0.738258541,  2.24270892,  0.221660137,  0.3151007,  0.370416254,  1.04786301,  -0.852603674,  0.314376056,  0.454945475,  -0.829904318,  -0.627282858,  0.653941751,  -0.695665061},{ -0.808926642,  0.3783755,  1.12217188,  0.455799878,  0.6893332,  -0.498422086,  0.278943568,  0.702687323,  -0.592382669,  -1.03156817,  -1.0545969,  -0.510867834,  -1.22673094,  1.24604106,  0.703419149,  0.352705061,  -0.189731747,  -0.927917302,  0.0947230831,  -1.25653982,  1.11743367,  -0.8924793,  1.69005525,  0.179689199,  0.483724207,  -0.213989288,  -0.65166527,  -1.22552526,  -0.656885207,  -0.727047503,  1.07421958,  -0.925824285,  -0.342636406,  0.0296253692,  -1.36379015,  0.795258999,  -0.0505871773,  0.350725442,  -0.636249781,  0.383653641,  2.24468589,  0.712901831,  1.37553823,  -0.466068596,  1.89761615,  0.178206682,  2.18219686,  0.708366752,  -0.356996894,  0.195835844,  -0.476233989,  0.977014124,  1.71462381,  -1.2744931,  0.20001328,  1.13813448,  -0.0641584471,  0.134325966,  -0.383304417,  -0.396230698,  0.73536098,  1.14277112,  0.150030479,  0.903406024,  0.104005262,  -0.855325699,  -1.34493041,  -1.48196852,  -0.153532252,  -0.245945886,  1.22993314,  -0.817649424,  -0.64057529,  0.20416455,  1.98479009,  1.37840927,  -0.57362622,  -0.431921035},{ -0.198642612,  -1.02111828,  -0.673903525,  1.12556601,  0.722333133,  -0.300739229,  -0.243915424,  -1.71803749,  0.357688099,  0.108750492,  0.0764488056,  2.51977754,  -0.095442012,  -0.694118083,  -0.44051072,  -1.57017064,  -0.376172692,  -0.00291219167,  -0.423707843,  1.62181854,  -0.0284070279,  0.763979495,  0.0898275748,  -0.0670426786,  -0.388195664,  0.794784188,  -1.51147187,  0.836709261,  1.32209754,  0.614892721,  -1.37583256,  0.997579873,  -1.49980175,  0.332057774,  0.853945315,  -0.173684821,  0.32670626,  -0.584929883,  -1.62495792,  1.7166121,  0.114084423,  1.0474211,  -2.14123511,  -2.69102287,  1.70169306,  -0.010298416,  0.395605981,  1.21116352,  0.804108202,  -0.400039822,  -0.806638479,  -0.997689128,  0.621995986,  -1.19445062,  1.11971414,  2.00794077,  -0.590611815,  0.332424462,  0.907789528,  -0.376934648,  2.72838068,  -2.2567029,  -0.186256751,  -0.381354332,  -1.32529473,  0.344469041,  -0.800802529,  0.899940073,  -0.414285392,  -1.58787227,  -1.40838194,  0.593181551,  0.943194807,  1.74119985,  1.04217756,  -0.0496773496,  -0.695601285,  0.587969363},{ 0.96825546,  -0.295143068,  0.806061447,  -0.654729962,  -0.975786507,  0.878579319,  0.0114254216,  1.25943756,  0.159868345,  0.696128547,  -0.628825247,  0.610575557,  1.85289228,  -2.86324263,  -0.496168077,  2.03700447,  1.01793361,  -0.186323628,  1.55617917,  0.851562381,  -0.266412824,  0.324506432,  -1.0322696,  0.508811235,  -1.71692836,  -0.210638911,  -0.48905459,  -0.287213355,  0.231344447,  -1.77814054,  -1.23462927,  0.236111119,  -0.873338878,  -0.725528777,  0.355635077,  0.994491041,  0.69993943,  -0.227132797,  0.396849751,  -2.38529396,  -0.919131696,  0.438265622,  0.600159407,  0.245808437,  -0.468588978,  -0.717086792,  0.6443941,  0.0392735861,  -0.751488328,  -0.779875278,  0.749220669,  0.658471107,  -0.775973678,  -0.0432091542,  -0.493487597,  1.78463817,  -1.02992928,  -0.430837989,  1.27320886,  -0.508607686,  -2.18400502,  1.24743223,  -0.697095752,  -2.6885469,  -0.230758041,  1.56036246,  0.28171885,  -0.957010746,  0.286584556,  1.43525231,  -0.508099914,  1.80503035,  -0.867491007,  0.443151653,  0.914694071,  0.771360338,  0.265179068,  -0.314323634},{ 0.285300612,  -0.0755875632,  0.777665317,  -1.16081989,  0.0296952911,  -1.23766279,  0.37089169,  -1.06367791,  -1.97277141,  -0.294719487,  -0.616075933,  2.20276093,  -0.964182258,  1.69603741,  -0.677973151,  -0.24027136,  1.13039899,  0.0204368886,  -0.318492591,  0.0556054376,  0.0588494428,  1.912817,  0.932531416,  -0.775682211,  0.160359919,  0.748212397,  -0.644357979,  -1.04579687,  -0.257519633,  -2.80502033,  -0.348142475,  -0.388001472,  0.210648045,  0.695002854,  0.567728341,  -1.17014182,  -0.627871037,  0.83862704,  -2.06677723,  0.868577302,  0.702198327,  0.150040075,  0.00461051287,  -0.983329952,  -0.49726817,  0.559156537,  0.168876499,  -0.713638604,  1.34926534,  0.979787886,  -0.515236139,  1.27582133,  -1.16414952,  -0.433970839,  0.0494835563,  -1.66753817,  0.998424709,  -0.532258332,  0.29833439,  -0.0729327947,  1.19819331,  -0.154034495,  -0.0918526277,  -1.86561346,  -1.37238836,  1.71120799,  0.114949219,  0.326485366,  -0.862596691,  0.675918639,  0.546368599,  1.54557168,  -1.54969001,  0.605173528,  -0.663822293,  -0.344424993,  0.155239195,  -0.134623721},{ -0.602018237,  1.11591566,  2.24867773,  -1.23369324,  0.116359361,  -1.47169769,  3.27330279,  -0.639207661,  0.764329255,  -0.912523091,  -0.101028815,  -1.0409497,  0.18753165,  0.43292588,  0.0172069706,  0.304992795,  0.82310307,  -0.637147784,  0.18001844,  0.275950491,  1.02599657,  -2.26540828,  -0.349886805,  -0.280085087,  -0.946832836,  1.18211043,  -0.60988003,  0.740013778,  0.665615559,  -0.919941008,  -0.674001753,  -1.92220068,  1.62331772,  0.619398773,  0.0641571879,  0.00727075059,  -2.05590558,  -1.90467811,  -1.35262585,  -0.90694797,  -0.487251997,  1.04510677,  0.311927885,  -1.09187567,  1.91502178,  0.0545923561,  -2.08209705,  0.0557028055,  -1.35781336,  -0.509320021,  -0.178336248,  -1.45978999,  -1.28613102,  1.40809238,  0.396342158,  0.717707813,  0.868878603,  0.287650108,  1.23965216,  0.403527319,  0.619131386,  -2.50619721,  1.71318197,  -0.143084139,  1.01479065,  1.31871378,  1.52596462,  0.44385159,  1.28378713,  0.781262577,  -0.0351728946,  0.193840414,  0.168498039,  0.449227095,  -0.540027738,  0.432583213,  -0.928773701,  -1.51880085},{ -0.766434014,  -0.367021054,  0.896966696,  -0.106606327,  -0.661498368,  0.918260038,  0.148627862,  0.0728586018,  -0.85874778,  0.386457324,  -0.236369818,  0.318387598,  -0.398956239,  0.314112633,  -0.422515363,  -0.783429086,  0.637229145,  1.00316107,  -1.42217124,  0.233624369,  -0.0829457119,  -0.0393484384,  0.46939531,  0.123032726,  -0.41981864,  0.583862782,  -1.67424917,  0.164618939,  1.00718319,  -0.673593938,  0.782046556,  -1.84524834,  -0.439642847,  2.75544143,  -1.74736023,  -0.173890635,  -2.54071832,  -0.977209747,  0.751648068,  -1.15538287,  -1.18576396,  1.53702915,  0.956148803,  0.0982610956,  -1.66792953,  -0.694984972,  0.526222467,  0.0953656733,  -0.0398597829,  0.522224545,  0.147925019,  1.12344015,  -0.38719964,  0.484151006,  -1.21363533,  -0.615060687,  -0.251839697,  -0.423249602,  -0.782801986,  1.53343034,  -1.34998393,  0.553575337,  -0.350208282,  -1.89755797,  -0.119859517,  -0.339714646,  -0.11271286,  0.399033159,  0.632895708,  0.455314964,  -0.266690463,  -0.291713744,  0.0503069609,  -0.412561834,  -0.631043017,  1.06825769,  0.327337474,  0.138074383},{ 1.78469491,  -0.13875702,  0.505161881,  -0.343252093,  1.41678262,  -1.31355512,  -0.618242085,  0.655539572,  -1.07370317,  -0.841737449,  -1.54982686,  1.95313048,  0.333301038,  -0.209334701,  -1.07457852,  -1.59705651,  0.74344939,  0.218244001,  0.0896278843,  0.816110611,  -0.599215209,  -0.235909358,  2.49082637,  0.339949369,  0.0343860202,  1.23775887,  -1.08496737,  -1.00371897,  1.30395532,  0.234219238,  -0.768191755,  -0.633752823,  0.958591044,  -0.361476928,  -0.615578771,  1.47836626,  0.479192019,  0.238765478,  -1.95871663,  -1.35469639,  1.17458057,  -0.647504985,  0.167202309,  1.00028002,  -1.0323602,  0.30847314,  0.238850698,  0.0160720181,  0.0266840253,  1.20162773,  0.718816161,  0.333154887,  1.51472652,  1.0314163,  1.05467379,  -0.0915924981,  -0.333243042,  0.710057855,  -1.17139304,  0.0949379653,  1.01257527,  -0.760701299,  0.825357735,  1.34742713,  0.0951441377,  1.21774352,  -0.281696707,  1.24847996,  -1.3156637,  1.04283798,  -1.5939666,  0.579062819,  0.8603701,  0.745759726,  -0.0452179573,  -0.839491665,  0.0270799734,  -0.978839576},{ -1.02351511,  -0.179316759,  -1.10692847,  0.528157771,  0.677558661,  -0.0115154656,  -1.63622165,  -0.267422497,  -0.712110519,  -1.57689261,  0.134695828,  -0.258364439,  -0.56968677,  0.409066945,  -0.648239255,  1.01855326,  0.528597414,  -0.288996279,  -0.973639131,  1.38739824,  0.389479548,  -0.183367103,  1.03197443,  1.39939249,  -1.17941535,  -0.864747047,  0.681039214,  0.176893935,  -0.676991045,  -0.911346555,  -0.129156157,  0.127959445,  -1.48579228,  0.253093481,  -0.193909794,  0.62586242,  1.29415202,  0.177961111,  -0.745330215,  0.733419418,  -1.89904797,  -0.692187548,  -0.177717879,  0.624760449,  1.16623592,  -0.309052169,  -0.00295732496,  -1.34557664,  -0.794961631,  -0.0440336652,  -0.527805865,  1.06228018,  0.0860455781,  0.702399731,  0.594543278,  1.12819195,  1.10112023,  -0.387625664,  -0.534371734,  -0.180110261,  -0.403193325,  0.641669571,  -0.439986229,  1.00249302,  -1.57388997,  -0.704184413,  -0.916734755,  -0.140980989,  -0.525707901,  0.510498643,  0.172043785,  -0.434385091,  -0.527241111,  -0.283182591,  -0.342295766,  1.1963557,  -0.545579553,  -1.94823396},{ -2.4413383,  -0.0207765531,  -1.20388925,  -0.0485720411,  0.329822659,  0.402552396,  1.03840816,  0.18557854,  1.36372006,  1.12301052,  0.873340249,  -0.292261451,  0.520386279,  0.597816527,  -0.914804995,  -0.00198167819,  0.738274932,  -0.328434438,  1.55109179,  -0.599412024,  -1.96673143,  0.218689084,  0.554392338,  -0.65595293,  -0.895690024,  -1.29113674,  -0.412083268,  0.753666043,  -1.02042484,  -1.49873555,  1.72016299,  -1.41938782,  -0.589304328,  0.328834921,  1.52608728,  0.382904559,  -0.160635859,  -0.058806289,  1.06788194,  0.342319906,  -0.445227504,  0.0957534611,  -0.377304018,  0.722186029,  0.503214359,  2.03641462,  -0.563213229,  -0.179896235,  1.71299756,  -0.386648923,  0.198939204,  -0.328954816,  0.327835381,  0.975622118,  -0.65918982,  0.559167504,  0.0693720803,  0.186695576,  0.512413323,  -0.388480932,  0.0893382207,  -2.21783829,  -0.48485902,  0.44012019,  0.560020566,  -0.658393562,  -0.634547949,  -0.431896299,  0.101478562,  0.457216531,  -0.690949202,  0.196399525,  0.471109241,  -0.732325077,  2.56238055,  -0.110602997,  -0.582307816,  -0.251354396},{ 0.381631613,  -0.437175572,  1.84878278,  -0.645651519,  -1.26101077,  0.608837068,  -0.172705188,  -0.534390211,  0.748374045,  0.647783816,  0.600925207,  -0.925857902,  0.717141628,  0.704627156,  0.487360775,  0.583225369,  0.0973860845,  2.04719782,  0.407328248,  -1.74724293,  0.480851322,  -0.31954211,  -0.2869412,  0.721541703,  -1.66519046,  -0.145510212,  0.583811998,  1.08926785,  1.30999041,  -0.266686976,  -1.20557797,  0.771458864,  0.0292641558,  0.169427142,  1.15043008,  -0.457570314,  0.507906079,  -0.858477235,  -1.30326283,  -0.361594051,  -1.14249539,  -0.378065199,  0.355899692,  -2.72532916,  1.58527911,  0.0537912771,  -0.718142748,  -1.41128719,  0.550361872,  0.0969718248,  0.666508853,  -1.21214318,  1.02259123,  -0.308491826,  -0.18699263,  1.21153259,  -1.0602268,  -0.403603762,  0.0341298282,  0.173151344,  -0.370273322,  -0.623045623,  -0.695920169,  -0.635146141,  -2.09651732,  0.167138442,  1.09576225,  -0.737099767,  -0.24198778,  -0.125363663,  -1.22873414,  -0.029831728,  0.698010147,  0.387611955,  -0.233851895,  -0.0198983457,  -0.894838393,  -2.62520409},{ -1.10020363,  -0.620870113,  0.810903311,  -0.0382805765,  0.0239186473,  0.225247309,  0.977342367,  0.214152813,  -0.479839236,  1.38463581,  0.812853217,  1.27499008,  1.42490399,  1.44493008,  -3.57167649,  -0.478311211,  -0.426953197,  0.221318603,  -0.124143779,  -0.999659538,  -1.30482423,  -1.20141661,  0.0877976343,  0.824237823,  -0.53231281,  -0.294536144,  0.0712177977,  -0.862486422,  -0.704809427,  -1.11599851,  -0.286964118,  -0.58704704,  0.383497506,  -0.140845492,  0.181093171,  -0.043869257,  0.91216284,  -1.52024961,  -0.621698499,  -0.133229852,  -0.0273991674,  0.255003363,  -0.781426907,  -0.00296278787,  0.59776634,  1.73741853,  -1.28873813,  -1.34781551,  -0.616382718,  -0.350537479,  0.889492512,  -0.522518218,  -0.284625828,  0.337786257,  -0.179749548,  -0.322271466,  -1.1172328,  -0.0977537185,  -0.763317406,  -0.496655285,  -1.89116681,  0.122188203,  -0.0431571715,  1.44773543,  1.71701097,  0.0600391142,  -2.7061801,  0.492173821,  0.999275088,  0.0494775362,  0.969047308,  -0.732722759,  -0.372493953,  0.112309515,  0.316958725,  -1.25416672,  -0.290389478,  1.60924733},{ -1.30870771,  -1.20681393,  0.0566579811,  0.695219278,  -0.592396736,  1.28977776,  -0.116289645,  -0.544711709,  -2.11565757,  -0.331732184,  1.00296199,  -0.387628645,  -0.0327560976,  -0.102875166,  2.30582952,  -0.0989410877,  0.104599252,  -0.425271273,  -0.222353294,  0.318851143,  0.933471859,  0.460000575,  -2.28643847,  -0.556180358,  0.0107920971,  0.845168114,  -1.29945505,  1.49450803,  -0.639179528,  0.761090219,  -0.309877545,  -0.0765852481,  -1.09892404,  -0.166924804,  0.522717535,  -0.844985068,  -0.89757663,  0.427738369,  -0.728524089,  1.23757851,  0.682283938,  0.00502381753,  0.198776782,  -1.29974449,  -0.229459241,  -0.443246603,  1.54978395,  0.528913617,  -0.773959219,  -0.021013787,  0.444654763,  1.58943164,  1.00761139,  1.01350462,  -1.45427752,  -0.443954766,  -1.41727877,  -0.832652509,  0.00174617581,  0.905897796,  1.67110741,  -1.76567638,  -1.08915591,  0.159897923,  -0.616058052,  -0.796637535,  0.891191721,  -1.97007585,  -1.51348341,  -0.564533472,  -0.637812078,  0.56210959,  -1.00652754,  -0.14882037,  -0.371219248,  1.61729538,  0.370769769,  0.59793359},{ 0.8191728,  -0.596303403,  -0.241081223,  0.808822393,  -1.88907886,  -0.133595362,  -0.466254264,  -0.378398657,  0.405482024,  1.9055177,  -0.0984812975,  0.708615541,  -0.872516036,  1.14329243,  0.659195364,  1.26503181,  -1.15259218,  -1.59891415,  0.9110623,  0.650302649,  1.18476498,  1.48637426,  -1.07986712,  0.15352872,  2.81732583,  -0.504825473,  0.32089591,  -0.832776904,  0.52607435,  0.0552226938,  -0.997019768,  0.216883883,  0.587825418,  -1.06421065,  -0.885932088,  0.18565923,  1.04569101,  1.02523971,  -0.103455618,  -0.32181254,  1.61974633,  0.517160654,  -1.00806022,  -0.25088051,  0.113925047,  -0.688384533,  -2.10113096,  -1.69355011,  0.178346097,  -1.91857874,  0.193517491,  -1.33229554,  -0.832826853,  1.1368953,  -0.824590147,  0.176590055,  -2.25278282,  -1.12723124,  -0.102780737,  -0.190783918,  -0.807060301,  1.61937237,  -1.300892,  -0.875194013,  -0.116550654,  -0.364842564,  0.464065462,  1.87016535,  -0.354386806,  -0.894685268,  -0.596710742,  -0.317695558,  -0.377148181,  -0.606669784,  1.27477407,  0.48983708,  -0.540371239,  0.124435887},{ 2.46355462,  0.193870619,  0.477621585,  1.43502212,  -0.728263199,  0.239268437,  0.883243501,  0.723484516,  -1.20646393,  -0.501146436,  1.91406131,  -0.512914777,  -1.36454594,  -1.33151448,  -0.180663541,  0.924960673,  -1.08812928,  0.435780138,  -0.966318488,  0.289529115,  1.15341365,  0.566768646,  0.68065542,  -0.116794616,  0.645625174,  1.06748283,  -0.0551312789,  0.71875453,  1.42535007,  0.185114235,  1.38729739,  1.45368636,  -1.34710801,  -2.00043631,  2.68541312,  1.5458498,  0.0812292993,  0.099679485,  -0.705310702,  1.16604304,  -0.933258712,  -0.706316531,  -0.896461785,  0.00969774369,  0.219808996,  -0.126270592,  1.09769785,  -2.38880229,  -0.484953016,  1.03276813,  -0.832562387,  0.578721344,  0.918565035,  -1.93225086,  0.435108542,  -0.276532143,  1.05872679,  -0.787144244,  0.345009834,  -0.60350275,  -0.731972694,  1.30553102,  1.265733,  0.344822615,  2.5505054,  1.19781685,  0.577617943,  -0.56522572,  0.787660122,  -0.372422546,  -1.97403443,  -0.64086175,  0.188107014,  -0.353204489,  1.86299062,  -1.40088034,  -0.204635292,  -0.972490907},{ 0.78511703,  0.634821892,  -0.538599074,  -3.25735927,  0.563142478,  -0.138341472,  1.43494749,  -0.718808949,  0.897726357,  -1.25794196,  0.75011766,  -0.331302881,  1.79922736,  -0.923451841,  -1.53370166,  -0.85639286,  -0.086297445,  -0.696373403,  -1.110394,  0.2535851,  -1.16432011,  -0.218042552,  0.341942489,  -1.30479825,  -0.298269421,  2.37576151,  0.299379349,  -0.731994092,  0.164013177,  0.316771001,  -0.64420104,  1.71462035,  1.31582618,  1.92957258,  1.06733,  0.806180358,  -1.41537642,  1.29951978,  -1.16843331,  1.55914664,  1.25746918,  0.594112158,  -0.8407529,  1.24577916,  0.240422323,  1.18817997,  -0.430004716,  -0.140687197,  0.400993407,  0.135044843,  0.0440047234,  -1.52450979,  -1.45953357,  -0.214985475,  -1.35259402,  1.28619194,  2.37653971,  0.611302853,  -0.196714163,  0.612877309,  0.575092256,  0.34219861,  1.2022419,  -0.29441309,  -0.679758668,  -0.824347794,  -0.614796162,  1.22794676,  -1.50945807,  0.0435998142,  -0.348194063,  0.179374665,  1.96761346,  2.0078187,  -1.43527174,  0.228462934,  0.691063225,  -0.439222008},{ 0.149309412,  0.799620211,  0.701607287,  -0.0692739189,  -0.0205694884,  -0.4466106,  0.972475469,  -0.575813234,  -0.418881565,  0.129094854,  0.55010128,  -1.21413386,  -2.05706263,  -0.333977908,  0.355663538,  -1.00567079,  0.870255291,  1.43549418,  -1.75823462,  -0.159391537,  -0.935297847,  -0.888521314,  -0.159242973,  0.181219995,  0.352174222,  0.0638804585,  -1.34040785,  0.387201488,  1.08097875,  -0.701375782,  -1.41007531,  -0.344826818,  -2.00932622,  1.07886326,  1.46789575,  0.459887803,  -0.210952491,  0.0557802878,  -1.17532218,  -1.74724138,  2.10873652,  -0.227226317,  0.256424099,  -0.948626935,  -0.665822387,  1.4095273,  -0.146700457,  -1.01802027,  -0.613923371,  2.24448729,  1.63474607,  0.193809003,  -0.477733731,  2.18916845,  1.0890404,  -0.875246465,  -0.477208316,  1.53853345,  -0.776049852,  1.3469106,  0.865356088,  0.428006917,  0.655003071,  0.0149354544,  2.13826084,  -0.0473025143,  1.32905972,  -0.521814525,  -0.476171613,  -0.0421835259,  -0.818157315,  -0.520582259,  0.156182379,  1.28049314,  -0.499596715,  -1.72775757,  1.36770797,  0.13686046},{ 0.979855597,  -0.366355121,  1.21701431,  -1.42390156,  -0.621100366,  -0.291368991,  0.850931168,  1.88248646,  -0.918957531,  -0.529059649,  1.68223584,  1.22819352,  -0.271367192,  0.784018695,  0.892455935,  -0.501612782,  0.399502486,  0.0743271187,  -1.03519726,  0.754998863,  2.30927801,  -0.871337175,  0.0343797803,  1.40998793,  -0.776371062,  0.206754461,  0.577600002,  0.361216784,  -0.403931528,  -0.122292735,  0.418298572,  -0.775977492,  2.28020978,  -0.286219627,  0.329169542,  2.01647377,  0.222036749,  1.15110636,  -0.230547622,  -0.578372836,  -1.93898821,  0.447869509,  -0.397424966,  0.534158647,  -1.98060358,  0.569950402,  0.675031781,  0.933268964,  1.60009038,  0.263316959,  0.441447616,  0.266100168,  0.342033446,  0.753510177,  0.0826772153,  1.24599099,  0.478907049,  -1.33645022,  0.878312111,  0.668021977,  -0.338854462,  -0.560974419,  0.490632862,  -0.762239695,  1.36545479,  -0.973890305,  -0.288361222,  -0.28418991,  -0.728538215,  0.920149326,  -0.488839477,  0.338265181,  -0.886294723,  -0.46759212,  0.383084357,  0.883470178,  -0.0726338997,  0.559154332},{ -1.54832733,  -0.44800058,  -0.301526666,  -0.040739689,  1.50518548,  -1.44250834,  1.13191521,  -0.544825912,  -0.175761864,  2.20295286,  -0.113411471,  -1.82111466,  -1.51990879,  -1.27642131,  -0.247146875,  1.56442618,  -0.343642414,  1.95559454,  1.96458304,  -0.73157984,  1.91416276,  -0.885110915,  0.204878882,  0.0860310867,  -0.223073661,  -0.881190121,  1.50441313,  -0.789019704,  0.804439962,  0.104951873,  -0.0521860272,  1.6873939,  0.788997352,  1.71919692,  -2.03108454,  2.66163898,  -2.05159354,  1.48436224,  -0.0668943673,  -1.95595765,  0.4292202,  0.388762563,  -0.390000463,  1.57089067,  -0.720598042,  0.8066293,  0.272122324,  -1.88871801,  -0.15821059,  0.646558285,  0.434668481,  0.567824602,  1.20133436,  0.4823111,  0.159897447,  -1.69390333,  -0.826122582,  0.102755718,  0.344659805,  -0.439934909,  0.186151892,  0.145817891,  -0.656378865,  1.14507973,  0.865343034,  -0.859710276,  -0.961068571,  -2.1449585,  0.41101563,  0.233799681,  1.1866591,  0.0536014549,  0.401012599,  0.385133952,  -0.988642514,  0.22254698,  1.34666812,  1.34121704},{ 1.19513738,  0.0825101659,  -2.12697959,  1.00497448,  -0.537573755,  0.874498725,  0.522540033,  1.05108666,  -0.127460375,  -0.818681777,  0.727884769,  -0.125272185,  0.638540387,  1.95479548,  -1.03639078,  -0.451861829,  -1.25535095,  -0.167123139,  0.502818048,  -0.00546671217,  1.31369781,  0.604489028,  -0.319946915,  2.37948632,  0.87113148,  0.135252357,  0.289636046,  -0.0458933339,  2.10462499,  0.661414742,  1.29216921,  0.0341235697,  0.0277634878,  -2.48479819,  -1.26224005,  0.268278271,  1.36543131,  0.221959516,  -0.0150894346,  -0.617139041,  -1.1523906,  -0.732008934,  -0.544539928,  0.192266434,  0.379496336,  -0.0835330188,  -1.48651052,  -1.69109559,  -0.298426002,  -0.0115954354,  0.970029831,  2.40945888,  1.77340329,  0.489755034,  -1.00600827,  -0.482393414,  1.03458428,  -0.753327668,  -0.000441257725,  -0.780939519,  0.435082346,  0.490874499,  0.758787036,  -0.185109928,  -0.264348418,  0.518971682,  0.130903065,  -0.592003226,  -1.8047576,  -0.0568554811,  1.69131899,  1.43195462,  0.966453612,  -1.85349822,  -1.20237339,  0.461484641,  0.779032767,  -0.768900871},{ -0.369554162,  -0.703611195,  -1.22086346,  -0.743624389,  -0.65705806,  1.63247669,  -0.22852543,  -0.0888336748,  1.19829202,  -0.942446768,  0.157946795,  -0.966719449,  -0.627203286,  0.0538551956,  -1.20067406,  0.808036506,  -1.12169516,  -0.822021723,  -0.217146784,  0.98013401,  -0.938177824,  0.26353392,  -0.416282713,  -0.730620325,  1.00621235,  -0.457228363,  1.4154588,  -1.74306762,  -0.0857459009,  0.803651035,  0.0753561482,  -0.773318768,  -0.200518474,  -0.773357213,  -0.00973131508,  -0.160381794,  -0.599912822,  -0.333509058,  -0.889515281,  -0.40898627,  1.447173,  -1.3548466,  1.22605395,  1.06398833,  -2.25227499,  0.47654435,  -1.06234121,  0.466612637,  0.290980965,  1.25781178,  1.5419594,  -0.162339061,  1.11673725,  -1.64605916,  -1.15035892,  -0.117896393,  -2.93428087,  0.12946026,  -1.01426268,  -0.636625051,  -0.458981872,  0.935911298,  0.703434527,  0.202469915,  0.630502641,  -0.551687479,  0.042920161,  2.57781649,  -1.05905545,  -1.06294191,  0.456901968,  0.232592627,  -1.72570229,  0.256956071,  0.882369161,  2.32110524,  2.34369874,  -0.988548398},{ -0.664929271,  1.80828536,  -0.739332736,  -0.140150651,  -1.53558493,  1.48666668,  -1.46301663,  0.60222286,  -1.16680992,  -1.16181672,  -1.33136845,  1.58696818,  -0.756603479,  -0.246330842,  0.142208129,  1.0277257,  -0.571028948,  -0.376479566,  1.13109112,  0.855987191,  -0.79805249,  0.719542265,  -0.249621555,  -0.990773916,  1.98855913,  -0.430611253,  -1.29179251,  -1.4504627,  1.4379369,  0.569562793,  -0.321996808,  -1.64513874,  -1.32410717,  -0.890365839,  0.186122805,  -0.46683228,  -0.38177532,  -0.690091491,  0.854512274,  -0.118225522,  0.230815426,  -1.2289629,  -2.3272891,  -0.434477121,  0.3102144,  -0.499701142,  0.446530163,  0.932547331,  0.751017869,  -0.410384417,  0.760711432,  -0.19256337,  0.236019164,  -1.17650449,  1.37795115,  1.66051888,  -0.51940006,  0.639556885,  -1.50067246,  0.023803357,  1.61003339,  1.56118953,  -0.320814401,  0.895213842,  0.538499773,  1.14132452,  0.258877456,  -0.0194823351,  0.286121696,  0.425648212,  -0.921723127,  1.36388922,  -0.238717675,  -0.103027232,  -0.451491177,  -0.973755121,  1.45796573,  -1.60963809},{ 1.02877605,  -1.56833446,  0.125227243,  -0.324519277,  1.1256206,  -0.445313394,  1.36587775,  0.128898039,  -0.766352713,  -0.0730299801,  -1.88061464,  1.30166113,  -0.046566423,  -1.05943692,  -0.649733722,  0.889172673,  0.326224327,  1.29898584,  -0.490987182,  -0.786927521,  0.374040484,  -0.472439796,  -0.42549789,  0.976019382,  1.44294,  0.160995081,  -1.28065062,  -0.600108266,  0.418918759,  0.745783448,  -0.672999561,  -0.00706765242,  0.790838003,  0.324941128,  1.04697561,  2.1269238,  0.185125053,  0.271119684,  -2.35049462,  0.238290429,  -2.09278846,  -1.27848852,  2.73682261,  0.66280067,  -0.555107772,  1.50189352,  -0.859528601,  1.15911627,  0.609813511,  -2.26622796,  0.58408314,  -0.315020114,  -0.853717327,  -1.2990973,  1.32314193,  -0.121968947,  0.250338405,  -1.51575923,  1.98327982,  0.713108718,  -0.685842752,  -0.162397996,  0.257135957,  -0.847181082,  0.35991025,  -0.144656464,  -1.34672153,  0.59185636,  -1.57763588,  -0.0994590074,  1.65664351,  1.38647437,  -0.48699829,  -0.97797215,  1.07767665,  0.308555782,  1.14122415,  0.291344851},{ -1.44914079,  2.06655049,  0.0386869386,  0.648189723,  0.130548745,  -0.595626235,  -0.336409599,  0.418422043,  -0.93041563,  0.0592014343,  1.21944475,  1.45588398,  -1.70746934,  1.22928059,  1.73943424,  -2.16052794,  0.158250272,  -0.206851318,  1.11899638,  -1.62604392,  -0.678973258,  -0.762103796,  -0.0257074498,  -1.23630047,  0.607573092,  -0.2150978,  -0.837062478,  -0.626100838,  -0.535355091,  1.26930583,  -0.54494518,  -0.596213996,  -2.34675741,  -1.65758646,  -0.314529091,  -0.143989906,  0.398886114,  0.294369459,  -1.4568156,  1.52329278,  0.958386362,  1.24978709,  0.591466486,  0.499113888,  -0.83069396,  1.07072532,  -1.64094758,  0.747577608,  -1.55844927,  0.742351413,  -1.55581224,  -0.876804352,  1.01682484,  -0.352576941,  0.454879433,  1.69412911,  -0.299610257,  1.18686616,  0.253729105,  -1.51476157,  0.902828097,  0.388901651,  1.18379617,  0.507521033,  0.61873734,  0.370104402,  0.327438921,  -0.707805812,  -0.391574711,  -0.513708413,  0.120086379,  0.626176655,  -1.45355129,  -0.175172016,  0.466709524,  1.26174808,  -0.500906348,  -0.587088883},{ -1.34329426,  0.50562638,  -0.110923171,  0.251920134,  1.0243175,  0.0536082685,  1.47625411,  0.15237911,  1.40244794,  1.48532879,  0.0237973463,  -1.04820943,  -0.811477363,  0.42287308,  -0.36102742,  0.375456512,  -0.673014522,  -1.19439483,  -0.150993034,  0.329920858,  -0.00127986562,  -1.07722843,  0.736176014,  -0.357880324,  -0.901513278,  -1.28017521,  -0.846559405,  -2.95138168,  0.489869833,  0.896733522,  -0.982975662,  -0.219239816,  -1.17678845,  -1.15998077,  -2.14854383,  0.757753253,  -0.537880123,  1.06354594,  -0.903589547,  0.386760294,  -1.05574691,  0.897325993,  1.09269536,  0.402888298,  -0.752770424,  0.627958357,  -1.24070704,  -1.76413965,  -1.59597087,  0.867767692,  1.8106215,  -0.464279056,  -0.375077426,  -0.527763128,  -0.510554552,  -1.55062413,  1.02431417,  0.861473262,  -0.957273364,  -0.0367389657,  -0.023646351,  1.04592121,  0.905353367,  -0.103490904,  -0.690893948,  0.68006748,  -0.491322607,  -0.324118555,  0.283337891,  0.793502271,  -0.651611984,  -0.208617181,  -0.319650024,  -0.291063815,  -1.83558047,  -0.14371115,  1.07215643,  0.486434221},{ -0.539503753,  0.174505293,  -0.812044084,  -0.235092968,  0.975646973,  0.947822809,  -1.61589348,  -0.392064631,  -0.538075387,  2.01961565,  -0.987564504,  0.898736894,  -1.1315254,  -0.115867861,  0.703437567,  0.971192241,  0.510603487,  0.314606577,  -1.70302558,  1.44950473,  0.758994102,  1.05369437,  -0.386064231,  0.872687519,  0.359710276,  1.02675903,  -0.502992988,  0.462850958,  -2.62742233,  -1.21017563,  -1.65117204,  -0.0478654727,  2.68720651,  0.242702365,  1.90657055,  -0.0494327545,  -0.634153485,  -0.82555455,  0.380951613,  -1.25775683,  -1.19265139,  -0.492906183,  1.23020828,  -1.90717661,  -0.316617399,  0.0651896894,  1.57335889,  -0.331135809,  -1.58847535,  1.05235994,  -0.1301976,  0.720351934,  -0.934826076,  -0.475031406,  -0.888690174,  -0.622627735,  -0.629942417,  1.01438892,  0.538426816,  0.878300309,  0.577233434,  0.678639054,  -0.864585102,  1.21128511,  -0.177803665,  1.11986446,  -1.00394118,  0.0818974301,  0.805388868,  0.568951309,  -1.18418407,  -0.624064267,  -0.319738358,  -1.00743794,  -1.05737865,  -0.961386323,  -1.07817078,  0.908021688},{ 0.315430015,  -1.02095449,  1.69334197,  1.79623425,  -0.491455287,  0.994573832,  0.208412156,  -1.33307981,  0.0166261774,  0.0805537105,  2.67289996,  1.69511425,  -1.14265203,  1.50496244,  0.673976362,  0.224254131,  0.294862002,  1.96821749,  0.588064849,  -1.87628436,  1.22778761,  2.90300059,  -1.00785112,  0.312178224,  -1.92278326,  -0.647945106,  -0.329075873,  0.836243272,  -0.547798097,  0.683288693,  -1.85418093,  -0.0732005686,  1.27893746,  -0.906627178,  -0.550983191,  0.262358606,  -0.485913217,  0.851886392,  2.09172082,  -0.308662564,  -0.666876972,  -1.17197001,  -0.916301847,  0.553752303,  0.789486945,  0.0181333777,  0.00319192163,  -0.0216562022,  -0.658482671,  -0.43675977,  0.481241763,  -0.0331416465,  -0.349482238,  0.11098133,  1.35456955,  1.4373666,  0.070613496,  -1.19049931,  -1.14828742,  -1.01260066,  -0.180594295,  -0.198897347,  1.07758129,  -0.0627003461,  -0.0232297741,  0.665278912,  0.201676726,  -0.447420686,  -0.552876174,  -0.740151227,  0.121496953,  0.279863,  0.213559419,  -0.815230072,  0.708974719,  -0.266928196,  0.776704311,  1.08867955},{ 0.435014546,  0.546794891,  0.502977669,  0.581857443,  1.67283928,  -0.369537085,  1.28462029,  -1.383075,  -0.838634014,  -0.385574222,  -0.165955171,  0.475914747,  1.40661955,  -1.43455517,  0.406995386,  1.130041,  -1.70657289,  0.642988026,  1.10196912,  1.1755631,  -1.53451276,  0.291191012,  0.117397323,  -0.535282135,  -0.318390757,  0.41386199,  -0.603943229,  1.04194021,  0.895620108,  0.225660026,  2.00126743,  -0.850324631,  2.3606267,  -0.855075777,  -0.371537417,  0.318955392,  -0.433877379,  0.890231609,  -0.0648980662,  -0.417197645,  -0.696815968,  1.68154919,  1.61656153,  -1.34228671,  0.27969864,  -1.36751997,  0.79870069,  -0.572208762,  -1.20481145,  -0.897641897,  1.01494372,  0.344406515,  0.418016493,  -0.403336376,  -0.337810278,  0.199020803,  0.305401504,  -0.34499836,  0.958846867,  -0.195730165,  1.66183305,  -0.574236691,  1.23408699,  0.864712954,  -1.02094841,  -0.543153524,  0.578086436,  -0.869600356,  -0.552058816,  1.99250329,  0.0569062233,  -0.545084476,  -0.529342771,  -0.64189887,  -0.460379034,  0.270171106,  -0.0764783248,  0.822841346},{ 0.304353356,  1.02620411,  0.592583001,  1.19224477,  -0.936319113,  0.0899346918,  -0.701902807,  -0.590894699,  0.0707753152,  0.81610328,  -1.68888557,  -0.0152956266,  0.78135711,  1.6213994,  1.17134559,  0.355397016,  0.884656131,  1.28020775,  1.31885934,  0.571050823,  -0.00808137283,  1.11195493,  0.15945521,  0.0667230785,  -0.549876273,  0.210641265,  -0.454044282,  -0.308769524,  0.99232769,  -0.109950654,  0.896727443,  0.519022524,  -1.04392982,  -1.23904824,  0.0829350129,  -0.254879773,  -0.386099696,  -1.75282466,  -0.622786522,  -0.863559663,  -0.623281062,  -0.946810544,  -0.702691436,  -0.197871998,  -0.0561453849,  -1.45672774,  1.00021482,  0.482274801,  0.0504109114,  -1.13461852,  -0.526057184,  0.370555222,  1.86640692,  1.47261655,  0.590833843,  0.652500391,  0.982089102,  -0.968246102,  0.44538942,  0.0239041522,  1.92216218,  -0.170890704,  1.30655909,  0.904149354,  -0.299247503,  -0.899507225,  1.78509426,  1.08674335,  0.430130512,  -0.225247964,  0.890942216,  0.997023702,  0.556180239,  0.828669846,  -0.972464085,  0.59107095,  1.82794213,  -1.17125022},{ 0.338913441,  -0.828950167,  -0.138433456,  0.246434629,  0.302106827,  -0.0183568485,  -0.549841285,  0.741423965,  1.09530008,  -0.223448366,  0.597523451,  -0.163442507,  0.0798330978,  0.0724379793,  0.369981945,  0.579261124,  -1.56475163,  1.69652164,  -2.20693779,  0.0535888337,  0.764416397,  -1.15714693,  0.00388549594,  0.847941935,  -0.357860267,  0.556674421,  -1.76166689,  1.75549245,  1.04100394,  0.645736337,  1.12124825,  0.0691882074,  1.18125606,  -1.16972327,  0.743328094,  0.858692169,  1.53933942,  -0.904961526,  0.13403672,  -1.49509299,  -1.20678723,  0.614198029,  -1.08634007,  0.304441303,  0.632744312,  -0.901438951,  0.459554911,  0.118490301,  -0.0399584733,  0.579814374,  0.510508776,  -1.04758036,  -0.409171581,  1.32964826,  -0.040962249,  -0.288790524,  0.196801126,  1.31680977,  -0.788094699,  -0.685775042,  -0.547379076,  0.215209737,  0.68633157,  -0.110412762,  -1.7646358,  -0.543782115,  0.0328121893,  -0.635835469,  -0.824243128,  -0.760159612,  0.794859648,  -1.25831199,  -1.0208075,  1.43653202,  -1.31496906,  1.26726437,  -0.598316848,  0.656555891},{ 0.705734193,  -0.378309071,  1.98591268,  1.78057563,  -0.0971388519,  0.425804287,  -0.891916811,  -0.118028529,  0.130002633,  0.89350152,  -1.3347199,  0.23168537,  0.0492949709,  -3.58524156,  -1.29885912,  -0.458916456,  -0.411847502,  1.54115534,  -2.00557733,  -0.44871825,  0.578237653,  0.713456273,  1.04268026,  -0.333377153,  1.33155656,  -1.05435979,  0.873278081,  -0.920280099,  0.657582641,  0.0318147019,  -0.497308403,  0.775181532,  0.543605268,  0.688572943,  0.478224486,  0.445948839,  0.424490273,  0.266561121,  0.440012097,  -0.942997336,  0.350280315,  1.64332795,  0.499162465,  -0.764084578,  -1.0561856,  0.662360549,  0.637043715,  -0.839146495,  0.467202514,  0.947798848,  0.424842417,  -0.345579326,  -0.798361421,  0.266258061,  1.09926164,  0.927694142,  0.796254635,  0.0753190815,  0.808238924,  -0.226241633,  -0.517767966,  0.818199515,  2.20097065,  -1.14272845,  -0.644602239,  -1.72939396,  -0.660369158,  -0.274909198,  0.750330448,  1.39260411,  0.393221349,  0.849016547,  0.917869508,  -1.39376247,  2.03413296,  0.0726235509,  -0.349073052,  -0.552549243},{ -1.70805132,  -0.0927290916,  -0.457241744,  -0.892858028,  0.798448205,  -1.13516355,  -1.03942454,  -0.0477180406,  -0.460592836,  0.0583771728,  -0.219162017,  -0.588322103,  -1.1278367,  -0.452710658,  0.775570273,  -1.13002431,  1.20816267,  -0.790956676,  -1.37961257,  -0.686466992,  0.398160458,  1.0972985,  1.43346274,  0.311777502,  1.80471241,  0.0896706134,  -0.92691195,  0.735947847,  -1.18048775,  1.41204691,  -0.218184099,  -0.376592427,  -0.275357634,  1.17484057,  0.113966934,  0.769305587,  -0.737511039,  0.640280366,  0.013784172,  -0.202770874,  0.504595935,  0.203243107,  -0.0293737147,  0.800873339,  0.330656707,  -0.341854095,  -0.582280755,  0.955877423,  0.155598849,  0.537382185,  0.185420126,  0.281707495,  0.308021903,  -0.121435195,  0.465554893,  0.533160567,  0.0102722663,  -2.01042008,  -0.749166131,  -1.69752586,  0.478776723,  0.596162021,  1.44137895,  -0.83337605,  -0.920648396,  0.686095417,  0.632826149,  -2.07890987,  -0.188937798,  -0.463150918,  -0.925373137,  0.55473417,  -0.691167355,  0.939221144,  1.29220569,  -0.402537107,  2.01900125,  0.0890631452},{ 0.178815186,  1.4374342,  0.436482131,  0.882050991,  0.602944374,  -0.167655826,  -1.81486809,  -0.944820583,  0.559482574,  -0.754903913,  0.496350676,  0.346018553,  2.36380148,  0.76327616,  -1.19108176,  0.386642277,  -0.0504747592,  -0.956688821,  0.493395507,  0.0132791558,  -0.163842276,  -0.751730561,  0.726136148,  0.905183911,  -0.564565361,  0.252611965,  -1.8210398,  0.531058848,  0.61191386,  -0.546911359,  0.745330572,  0.638653457,  1.40827835,  1.47269642,  -0.589797258,  -1.94333136,  0.026155185,  0.630499244,  1.87100768,  -0.92607975,  0.160337955,  0.112079166,  -0.728948772,  -0.907094777,  -0.552389085,  0.358068526,  -0.181426048,  0.194713384,  -0.165609837,  0.821649075,  -1.08215189,  0.743647099,  0.231703818,  -0.879560649,  -1.94391191,  0.265522659,  -0.246214271,  -1.83644903,  -0.42016691,  -0.538729072,  -1.89711034,  -0.328925908,  -0.173820704,  -0.607324302,  0.0659805462,  -1.57207394,  0.0869431123,  0.778415978,  0.0887361169,  -0.572729766,  1.62900949,  1.4633925,  -2.00331593,  -0.221688524,  -0.510670722,  0.693004251,  0.00463698059,  -1.50880575},{ -0.625304282,  -0.381848633,  0.497171462,  1.65222406,  -0.0761635676,  1.58210886,  -0.883582175,  -1.00511122,  -1.48517203,  -1.60027003,  0.388803035,  0.0710903108,  0.210006505,  1.93571055,  0.0976668894,  0.233706489,  0.0801918209,  -1.9694103,  0.0867422372,  -0.210160673,  1.83031237,  -1.70424223,  -0.621505976,  0.114631549,  0.0452435166,  -0.646939158,  -0.209194317,  -0.425832242,  -0.00625057565,  0.935029805,  0.835603476,  2.58896708,  2.6788404,  1.03962946,  0.590785027,  0.499200553,  -1.55617619,  -0.578588367,  0.948115408,  0.415745974,  -1.19169629,  -0.0193043314,  -0.345986813,  -0.587357104,  -1.10020494,  0.025235435,  -0.167873293,  1.87412226,  -1.23295248,  -0.414937854,  0.649880171,  0.0727631748,  0.0470940284,  0.091516763,  -0.492165059,  -0.225859821,  1.03727305,  -0.716557443,  0.863342881,  0.0797806829,  0.0252562407,  -0.895912826,  0.17279312,  0.698217213,  -1.2857995,  -0.367486149,  -2.50432396,  -1.00409961,  -0.348198414,  -0.648041427,  0.169601753,  -0.847332776,  0.0555500165,  0.822941244,  -0.985003829,  0.139117807,  -0.245674953,  -0.580893636},{ -0.262069523,  0.592400253,  -0.863253534,  -0.902432919,  -0.616795063,  1.92638493,  1.29495311,  -1.81325066,  -1.20530331,  -1.23201478,  -1.02541745,  1.03781307,  -1.17812622,  -0.33885017,  -1.41767299,  -0.816891611,  0.0747112408,  -1.13947594,  0.56530422,  0.0421129949,  -1.38539672,  -0.945905864,  -0.944623411,  0.337144226,  0.602542639,  1.22898471,  0.80462271,  0.309471905,  0.728704214,  -1.36908865,  -0.323405564,  1.57273543,  0.375387698,  0.855271459,  0.152172148,  -1.40054739,  -0.517954767,  -2.13404489,  -0.529158473,  -0.0405644514,  0.314661056,  0.443265855,  3.50259495,  -0.474120021,  -0.619525313,  1.55894744,  0.190163508,  0.0255717728,  -0.0670619458,  0.597271502,  -0.0466661304,  0.438519716,  1.13198495,  0.134168625,  1.42332613,  -1.90844834,  0.599600971,  -0.279964298,  -0.673486471,  -1.77924228,  1.22379923,  -1.78711212,  -1.03388619,  0.135363743,  -0.841219544,  0.728077471,  -0.450725108,  0.0488776974,  0.316476434,  0.363725603,  0.154562056,  -0.972616851,  -0.153081283,  -0.127986372,  0.436402231,  -1.11866236,  1.19995618,  -0.172327787},{ 0.115815371,  1.24551916,  -0.71221298,  0.68740207,  -0.966325164,  -0.728415012,  -0.660488009,  -0.396475047,  1.26082706,  0.277045667,  0.623932362,  -0.298807323,  0.683960319,  -0.880689919,  2.68241978,  1.76776516,  -1.22739959,  -0.630211413,  0.984945178,  0.424321294,  -0.791055679,  -0.650763988,  1.45251203,  0.0640474409,  1.43067706,  -0.35838604,  -0.807851493,  0.0368749239,  1.75247216,  -0.984451354,  0.666715443,  -2.17394257,  0.937821865,  -0.0549409166,  0.235525489,  -0.432417989,  -0.286103934,  -1.49472141,  0.167635009,  -0.0552094802,  -1.23698938,  -0.641651332,  1.74559069,  -0.259117812,  0.370558977,  0.0450494178,  -0.877549171,  0.31968987,  -0.258201748,  -0.174101964,  -0.435703218,  -1.5170877,  1.34760594,  -1.31264651,  -1.87508464,  -0.768375754,  0.1854662,  -0.220997468,  -1.5683918,  -0.774242997,  0.0368322916,  1.82733321,  -1.04472864,  -0.448067963,  -1.00271034,  0.861882567,  -1.51707602,  1.15508676,  -0.608805239,  -1.20854771,  1.18845868,  1.52489364,  -0.52038312,  -0.489158362,  1.18094778,  -0.0731001496,  -0.28310591,  -0.184044406},{ 1.07375634,  1.27168906,  -1.13738847,  -0.583473861,  -0.984135151,  0.408144921,  -1.12908733,  -0.29266873,  -1.67962909,  0.0601884834,  0.915429592,  -0.513400495,  -1.37772608,  0.0246705245,  -0.0489202924,  -0.0365435295,  -0.35722205,  -0.354550004,  0.897248685,  1.07612681,  0.416345417,  0.105921529,  1.09304976,  0.737181187,  0.910991311,  2.15517998,  -0.204913557,  0.975709498,  -0.261181772,  1.51365328,  0.880444407,  0.092523858,  0.576664329,  0.719377697,  1.10136092,  -0.545529723,  -0.0503144227,  -0.636545777,  2.3814981,  -0.544311166,  -0.362122446,  0.814752102,  -2.16532826,  -1.66975188,  -2.69727373,  -0.295588762,  -0.524450302,  -3.16676378,  0.406462759,  -1.11859083,  1.62156296,  -0.827147484,  -0.507029057,  0.972183704,  1.43436658,  -0.0686228797,  1.18725562,  -1.54708922,  -0.0122032342,  -0.0488001145,  0.677275002,  -0.213278383,  0.0775128901,  -0.881542861,  0.00809550006,  -1.26090753,  0.54377538,  1.24903846,  1.36050594,  1.06240165,  0.947108626,  0.217133611,  -0.0367090441,  -0.665038228,  0.4558689,  0.451072276,  0.110547721,  1.12955403},{ -0.245787099,  -0.475580871,  0.165988922,  0.288164884,  -0.221086845,  -1.58022666,  0.187284648,  1.31185865,  0.609156907,  -1.52009261,  0.481100023,  -1.97054493,  -1.35647702,  -1.83492851,  1.8461858,  -0.610415697,  -0.454691827,  1.0041008,  0.390649199,  0.078755185,  -0.800656557,  -0.905985773,  -0.387126386,  0.608258426,  -0.43090263,  -0.193650767,  0.44401446,  0.373171926,  1.3719734,  1.63215232,  0.309275776,  -0.561161578,  -1.15796268,  -0.16464287,  -0.907299638,  -1.10557926,  0.502302527,  -0.462093145,  -0.162227601,  -0.926266909,  -0.287204087,  -0.256022215,  0.175445944,  0.472790271,  -2.26566911,  0.258837789,  0.808744848,  0.0776055083,  -0.446245044,  -0.159737721,  -1.7851963,  -0.247939497,  1.33157325,  -0.326886684,  0.40771246,  -0.221273437,  -0.509902596,  1.7820704,  -0.698230565,  -0.224125087,  0.845283985,  -1.96273208,  0.523141325,  -0.526005268,  -0.109691344,  -0.178500354,  -0.0473733619,  -0.63128835,  -0.786845565,  0.870779097,  0.510676742,  1.36228168,  0.417367607,  -2.31878161,  -0.737889051,  1.24729121,  0.463031411,  0.266978025},{ -1.17487371,  1.91361141,  0.902176857,  0.527403057,  -0.614892125,  -0.463680416,  0.318452328,  0.409406483,  0.0663225427,  -0.244267911,  -0.262162745,  0.0729065463,  1.65141535,  0.00232437719,  0.305090308,  -0.317145199,  0.377601773,  0.180101514,  -0.373826265,  -0.693873346,  -0.251842886,  -0.263339579,  -0.704475701,  -0.198293135,  0.126453146,  1.59057105,  -1.43570364,  1.57630146,  1.19586051,  0.758987129,  0.191752359,  -0.156170413,  0.203645289,  1.19426715,  -1.34606636,  -0.327771187,  -1.13451231,  -1.58009696,  -0.134382188,  0.0195097551,  0.499549031,  -1.92713761,  -1.06015265,  -1.36619973,  -0.307505488,  -0.0910436586,  0.35922578,  1.11261988,  -0.931716084,  -0.96372962,  0.956732571,  0.741437018,  -1.49091458,  -0.578765333,  0.383013785,  -1.49778819,  -2.04591727,  -0.38305974,  -0.276367158,  -1.31692326,  -0.683927298,  -0.63742888,  -0.75102371,  1.11857283,  -1.2278775,  -0.108259805,  -2.50365472,  0.954048336,  1.44069552,  -2.6084125,  -1.8926003,  0.0266979933,  0.665015996,  -0.970387876,  -0.372693777,  -0.0596136823,  0.373162001,  -0.933929205},{ -1.37265372,  0.752474546,  -0.712500513,  -1.02863467,  -1.21876788,  -0.433183998,  0.220365077,  0.303078294,  0.771047115,  0.0256124511,  0.440910429,  -1.22680378,  -0.953052342,  1.1195128,  0.64586252,  3.24489331,  1.09211123,  0.829320073,  -0.92526412,  0.622567832,  -0.148971871,  -0.145191237,  1.01616132,  -1.37659764,  -0.112879179,  0.592902362,  0.281414002,  -0.184903011,  -1.72089744,  0.645907521,  0.944757581,  0.0219154283,  0.307817042,  0.522081196,  -0.171489388,  -0.842971623,  -0.631000042,  -1.17417204,  1.25329077,  0.471378833,  -0.708116651,  -1.51307154,  -0.580087185,  2.60517931,  -1.44669509,  0.233326703,  0.336527705,  1.75110936,  -0.748951733,  1.95141113,  1.10544467,  -1.72659731,  -0.115270421,  0.981966674,  1.42422271,  -0.025722798,  -0.710350811,  0.0773681328,  0.715964735,  0.00194774114,  0.196621761,  1.23685026,  1.70115244,  0.350313038,  1.04370141,  -2.08260536,  -2.22092509,  0.012339551,  0.608608305,  1.15646458,  0.420782387,  2.81222725,  -1.14405644,  -1.38500369,  -0.304673433,  0.160211861,  1.1122402,  -1.35505056},{ -1.77681482,  0.47641775,  0.581120431,  -0.843935013,  0.911637664,  0.843702376,  0.252373248,  -0.541709542,  -0.00218714285,  1.71872282,  -0.863325238,  -1.1561656,  0.759893179,  0.671322107,  -0.423736215,  -0.722528875,  -0.552681625,  -0.275598228,  -0.972773492,  -0.36336562,  -1.55262709,  0.644746304,  1.76913154,  0.185332522,  1.16833842,  1.56490517,  -0.519739866,  0.441647768,  -0.195298389,  0.829768896,  0.127204806,  0.166824237,  -0.77807349,  0.305682927,  -0.430302322,  -0.898191631,  0.673888028,  -0.626504779,  1.5326134,  0.879401088,  -0.86756593,  -1.42701745,  0.454934895,  1.48555529,  -0.488322139,  1.53741395,  -0.359567881,  2.19936967,  0.221809581,  0.180474386,  -0.680699646,  -0.482068121,  0.351113945,  -0.524621546,  -0.229913354,  0.33797878,  0.0578596182,  2.86696219,  1.5426693,  -1.30096793,  -1.72390664,  -0.898797512,  -0.4672077,  0.158898667,  -0.866697371,  0.339740485,  -0.845629632,  0.527049601,  -0.381999254,  -0.77394408,  0.320452273,  0.627001226,  0.234636247,  1.43222678,  0.558578312,  -0.704604685,  -1.24592042,  0.643953323},{ 0.387649328,  -0.684832454,  1.03196192,  1.30852926,  -0.59921962,  -1.7229315,  -0.383087456,  1.07598603,  -0.34682411,  0.0541508794,  1.64880168,  1.50548494,  -0.46227634,  0.871089458,  -0.526113927,  1.871099,  0.899636924,  -1.20120227,  0.0307579096,  0.249535978,  1.38860357,  -0.544122219,  -0.260119677,  -0.247364312,  0.838902473,  -0.628752768,  -0.697866023,  -0.699769378,  -1.66666615,  -0.824116766,  0.265579194,  0.821182489,  -0.267983109,  0.179859325,  0.215266168,  0.936133265,  0.893644273,  -0.343590647,  -0.834363818,  0.167385116,  -0.410207897,  -1.13818824,  1.11805654,  1.23068464,  -0.620706975,  -0.705549419,  -1.24273002,  0.0370145664,  -1.75505376,  0.0852206424,  -0.753280997,  1.33202469,  0.180294663,  0.15460448,  -1.28272176,  -1.46294117,  -0.00770425191,  1.41869628,  -0.346207559,  -0.860704303,  0.13076517,  0.216028199,  0.143573985,  0.746324062,  0.946423292,  1.09782267,  1.18309879,  -0.354539573,  -0.110627152,  0.92435503,  -2.06306148,  -0.111696109,  -0.0899000242,  0.243939459,  1.17230165,  1.59927642,  0.26701057,  -1.12637675},{ -1.99624884,  -0.137925431,  -0.874854088,  -0.993397713,  -0.678896189,  0.927267075,  -0.0067310473,  2.66746569,  0.859331667,  1.16518819,  -0.445496142,  1.60162902,  0.759677351,  1.13035035,  -0.182395294,  -0.0416783355,  -1.06144345,  0.151262179,  -2.06753683,  0.534785807,  0.114202537,  -1.18123078,  -0.356274486,  0.184084177,  0.639132977,  -1.5162046,  0.960509956,  -0.512450159,  -0.380032837,  0.876623869,  0.0634700283,  0.144744068,  0.529874563,  1.31873202,  -1.10626388,  -1.53192985,  0.0308090486,  -0.13934797,  0.469704747,  0.81744504,  -1.04883099,  -0.164325505,  -0.71651119,  -0.287837774,  0.0706285164,  -0.54210788,  -0.982739091,  -0.263191491,  -1.57674229,  -0.880665779,  -2.50530195,  0.353022009,  0.776486874,  -0.697170019,  -1.04641843,  1.75043845,  1.97916865,  -1.38273609,  0.532154202,  -0.26158002,  0.182099342,  -0.626488864,  0.685176015,  1.6751833,  1.31453264,  0.496580988,  1.22665024,  -0.140614986,  -1.26951873,  0.0467537604,  0.477038354,  -1.22422934,  0.467468321,  -0.743315458,  0.332018733,  -0.985850096,  0.212513328,  1.80859935},{ 0.688687146,  -1.42492735,  -0.950494289,  -0.111055702,  0.201336786,  -0.229057968,  -0.853847444,  -0.29075852,  -0.331957191,  0.712193668,  -0.45178166,  -0.728046477,  0.228532434,  -0.916069269,  -1.04437017,  -0.994905412,  1.76599324,  -1.12805271,  -0.188341856,  1.67401421,  0.852783203,  0.440255344,  1.86904323,  -0.0289061833,  0.807527006,  1.39090931,  -0.709112763,  -0.281519145,  0.927383304,  0.897507906,  -1.00231898,  1.22981918,  -0.96701926,  -0.161139727,  -2.57478213,  -0.910073459,  1.39765751,  1.99548328,  -0.184724241,  0.0708975196,  -0.191413984,  -3.03240347,  -1.43323791,  0.732499003,  0.925867498,  -1.31148088,  0.703003824,  0.292285651,  -0.143476173,  -1.41345906,  0.0448833145,  0.471895635,  -1.15815544,  -0.581329942,  -0.829826832,  -1.57803905,  0.802989781,  0.676763356,  -1.0422169,  -0.546520412,  -1.11385047,  0.58508569,  -0.666745484,  1.20106542,  -0.849179864,  -0.737943709,  -0.253116935,  -0.14413622,  1.32176244,  0.325410813,  1.74753547,  1.09168482,  -0.620352566,  2.71369982,  1.15332425,  -0.573836923,  -0.240139499,  -2.04742384},{ -0.0493886173,  -1.2435348,  0.545490682,  -1.38201022,  0.727540791,  -1.43651628,  1.59514403,  -0.386429578,  0.0309973862,  0.811468899,  -0.327369452,  -1.67340684,  -1.15475762,  0.103661329,  -2.08330464,  0.126369238,  -1.80029249,  1.39672565,  2.01950431,  0.530846298,  1.28824937,  -1.47177434,  -1.68859994,  1.49058712,  -1.34000611,  0.654094934,  0.948682427,  0.288644373,  -0.0179136302,  0.862931371,  -1.59757757,  0.896298826,  -0.992132902,  -0.522501588,  0.864782751,  1.55078864,  0.295433193,  1.27538741,  -1.57155263,  -1.05136228,  -2.30328512,  -0.653655708,  0.0243257284,  0.125177398,  -1.56047571,  1.21796083,  -0.543281674,  0.119524226,  -0.88164103,  -1.65222752,  1.5351342,  -1.1278578,  1.06002748,  0.507999897,  -0.394978791,  1.15549731,  -1.60428178,  -0.0462292023,  -1.30479729,  -0.481370658,  0.228531733,  0.300377488,  1.94370425,  0.183370233,  -1.1056447,  1.69601071,  -0.144017622,  -0.376861274,  -1.34695041,  1.97091734,  1.71737778,  2.02547455,  -0.189867213,  0.0326598622,  -1.10203195,  0.51484406,  0.304706305,  -0.454437166},{ -2.43513036,  -1.027004,  0.376107365,  0.586106777,  1.77391422,  -0.867503643,  -0.776178479,  -0.942188501,  -1.01891375,  1.35013735,  -0.806524158,  0.3130292,  0.381652385,  -0.346240371,  1.28717804,  -1.14929605,  -0.89612627,  0.813979208,  1.57149148,  -0.185613573,  0.602256298,  0.137483269,  1.07370961,  -0.250799596,  -0.549767852,  0.398781031,  1.30118811,  0.224575251,  0.0139756128,  -0.451966465,  1.61428809,  1.35396075,  0.334486932,  0.972089171,  0.617318809,  -0.965488374,  -0.305560857,  0.38463062,  -1.42425179,  0.641593874,  0.0236730166,  -0.206900313,  0.71818763,  0.390675843,  0.836992085,  0.423198223,  0.539255083,  -0.204767779,  -0.155680373,  -1.41132951,  1.08086729,  -0.896937013,  0.46775341,  -0.507532239,  0.741439342,  0.65039897,  -0.161534771,  -0.790436983,  -1.14901459,  -1.193066,  -0.7902264,  0.991746724,  -0.326846957,  0.562840283,  -0.935921729,  -0.338076621,  -1.40700483,  -0.474592,  -0.0151769631,  1.893525,  1.52977777,  1.48708677,  -0.265733242,  0.725894809,  -0.613777339,  -0.372815728,  0.0641483292,  0.297460705},{ -1.22753012,  1.01045847,  -0.0205327626,  -0.734967053,  0.745353043,  -0.363352507,  0.199744418,  0.734555125,  -1.54245603,  0.329044312,  -0.0853345767,  -1.98390055,  1.0656358,  -2.57172585,  0.262174606,  -0.76863575,  -0.483552992,  -0.517960966,  0.733221471,  0.3810229,  -2.15852213,  1.3189503,  0.48377797,  -1.98877323,  1.87174642,  0.562877834,  -0.210956216,  0.708359838,  -0.982223153,  1.10252404,  -0.533095002,  -0.630065918,  2.31337261,  1.01774406,  -1.59886885,  0.211426213,  -0.334345192,  0.558447003,  1.62024403,  -0.709696233,  -1.32311523,  -1.16173577,  -0.942056715,  0.908419073,  0.36516884,  -0.450066835,  -0.43288675,  1.10656154,  -0.592945933,  -0.841016054,  1.5760026,  1.9584496,  -1.02598202,  1.16046333,  -0.671179712,  1.09799635,  -0.335160494,  0.863864899,  0.1153998,  -0.897777975,  -1.96118331,  -2.39031982,  0.237205148,  0.806923866,  -0.440022349,  -0.0371954851,  -0.118350677,  -1.31002247,  1.00724292,  0.587814093,  0.924722791,  0.935297966,  -1.16060889,  -1.17960072,  -2.8638413,  0.851238906,  -0.521748424,  -0.387010574},{ -0.954321682,  -0.2980524,  -0.378014356,  0.120484121,  -0.0815013871,  0.517200589,  0.76321876,  0.418531269,  -1.71883285,  0.056610927,  1.47362983,  -0.805378616,  1.50318873,  1.03472936,  -0.226525515,  0.545505464,  2.52744865,  -1.19515872,  -0.491338342,  -1.16875732,  0.252602816,  -0.557875752,  -0.939099729,  1.77945542,  1.06099558,  0.378608882,  -0.417975843,  -0.512043476,  0.737348258,  0.625894487,  2.31843996,  0.773901463,  2.09192777,  -0.285810232,  2.33407855,  -1.79056394,  0.384589165,  -1.79027331,  1.53911984,  0.203629866,  1.6240139,  -1.31895757,  -0.350243121,  1.66727495,  0.0428717658,  1.62446678,  1.73674107,  -1.56022668,  1.49341643,  -2.74919701,  0.2673527,  0.0274836663,  -0.146711484,  0.301113427,  0.815265119,  -0.244965777,  2.12730312,  0.808463156,  -0.6049124,  -0.303501785,  1.3560468,  0.544734597,  0.204717338,  -0.162137434,  0.819703639,  -0.0577171557,  0.374006569,  0.621761799,  -2.71741796,  -1.14986587,  0.588733375,  -0.1570362,  -1.38078797,  -0.485081434,  1.69286621,  1.59698582,  -1.41326368,  -1.06197894},{ 0.765156031,  0.766249955,  0.198095664,  1.49811566,  0.00559193734,  -0.175348178,  0.0794469714,  0.905005634,  -1.8532207,  0.778273165,  0.368835002,  -0.691100895,  1.76565266,  -0.869748175,  -0.787221372,  0.565842807,  -0.897093356,  -0.235229522,  1.14476037,  0.303952664,  0.806711435,  0.913446426,  2.15313745,  1.2775588,  -2.06717753,  0.843324184,  1.12815595,  -0.34468779,  0.503211498,  -2.29994226,  0.513238072,  1.75266206,  -0.349943638,  -0.640078545,  -0.576925278,  0.359810114,  0.836021304,  0.262452811,  1.30109239,  0.2552405,  -0.834268034,  1.50104022,  -0.833264768,  3.22410107,  -0.753417432,  -0.190447539,  0.0413901173,  0.545283139,  0.571353793,  0.745379627,  -0.367583781,  -1.24036956,  0.586497903,  -0.487750411,  -0.858664036,  -0.933988869,  0.344324648,  0.250339955,  -0.614691436,  0.664627075,  -0.585850418,  1.45886433,  -2.74478054,  -0.724999905,  -0.447438508,  0.295972258,  0.675502717,  -1.48042309,  -1.4442606,  -1.00694311,  0.810110211,  -0.898424864,  -0.0449728481,  -0.467338979,  0.633980095,  -0.195893362,  0.239732668,  -0.147526681},{ -0.514463842,  -0.711577117,  0.694009542,  -0.416030824,  -0.584732473,  -1.0228219,  1.2733016,  -0.240596786,  -0.747080624,  0.532334626,  0.243975326,  0.585531116,  0.546955466,  -2.23034382,  -0.208040148,  -0.0366045013,  -0.525524378,  0.633376598,  -0.22341688,  -0.634913206,  0.41836217,  0.635283709,  0.162216887,  -0.240032673,  0.669210255,  -1.46574795,  0.339666367,  -0.524448752,  -1.37704682,  0.976795495,  0.626422286,  0.487153679,  -0.204252034,  0.769635618,  -0.834938526,  0.563493133,  0.319650263,  0.248509169,  0.116019867,  1.33960187,  -0.830467641,  -0.787434816,  0.195377931,  -0.210973129,  0.394736409,  -0.0463061109,  -1.74168503,  1.31149006,  -0.0334407315,  -0.484186262,  -1.20384324,  -0.689604342,  0.209819123,  -1.2420485,  0.0332624428,  -1.04943585,  -0.337245673,  -1.7878027,  1.59269249,  0.78063643,  0.711370945,  -1.54094756,  -0.790022433,  1.05330586,  -1.59100366,  -1.67637396,  -0.947406292,  0.519896805,  0.557635546,  0.594455063,  -0.418148726,  0.423631787,  0.162371248,  -1.10343301,  1.27035129,  0.69009304,  1.17955625,  -0.420844734},{ 1.63239837,  1.26458216,  -0.62324059,  -0.855391145,  -1.39160132,  -1.10158086,  0.172469482,  -0.40163517,  0.244245216,  0.686452091,  3.20139313,  0.192608148,  -0.000708903244,  0.493175685,  0.806615293,  0.478572518,  0.155646205,  -0.0138134724,  -0.239069387,  0.0186895058,  -0.840565085,  -1.40339625,  -0.481682301,  -0.525628686,  0.97081089,  0.0835468993,  -0.958089232,  0.0718746036,  1.16623259,  -2.27347183,  -0.551842511,  0.885131836,  0.288142473,  -1.65565133,  0.175872728,  1.70894396,  1.47953999,  -0.513977706,  0.52307266,  -1.90932906,  -0.907145381,  -0.546270072,  -0.407729208,  0.0106847323,  -0.0378451347,  -0.34957701,  0.54167062,  1.55092657,  -2.04390502,  1.15433598,  0.598863959,  -0.148186624,  -2.42886186,  0.55371213,  -1.352422,  -0.373178154,  -2.23268151,  1.13855362,  -0.22767809,  0.311971992,  -0.531009793,  -0.0718073174,  0.368876487,  -0.500499487,  0.242923439,  0.370077789,  -0.728953481,  0.360966206,  0.508392394,  0.860117674,  -1.02723861,  -1.40721929,  -1.09885228,  0.450579345,  -0.92803365,  1.18719518,  -1.02583599,  1.06137836},{ 0.459466279,  -1.88332045,  -1.47848868,  -1.3597976,  0.624360621,  1.13942242,  -0.918003321,  -0.304056048,  0.0650679842,  1.46616149,  0.0874180496,  0.0222671833,  -0.369256228,  0.123033106,  0.738902748,  -1.15751767,  -0.916910529,  -0.973771989,  -0.251722276,  -1.76803195,  -0.104644917,  0.34245047,  0.492263705,  0.628228128,  -1.05329776,  0.339701802,  0.89725703,  1.20171332,  -2.23610735,  1.31172192,  1.00741827,  -0.298567653,  -0.617189467,  1.31494927,  -1.23712254,  -1.5029521,  0.0927134752,  -0.477597028,  -0.6479792,  0.245154172,  -0.312810481,  0.922477126,  0.455093712,  1.650877,  0.588979304,  -0.486172855,  0.482313126,  -0.673797786,  -1.07477307,  -1.00038552,  -0.963609695,  -0.504848897,  0.623660386,  -0.497275233,  -0.550572574,  -0.0254976247,  0.308142453,  -0.266912669,  -1.65861106,  -0.174832284,  0.334795743,  -0.30678156,  -1.35075021,  0.307013005,  -0.986281335,  -1.92791104,  -0.427712262,  -3.00161648,  2.87567759,  1.09610295,  -0.0907010883,  1.06476343,  0.527973771,  -0.477559924,  -0.941929579,  -1.02216029,  0.26799652,  0.806391001},{ 0.807914793,  1.62225914,  -0.0965180174,  0.674263716,  -0.0535730273,  -0.210604295,  -1.16123796,  0.755880535,  -0.510392427,  -0.285350621,  -0.777478278,  -0.180671334,  -0.154759824,  -1.82489097,  -1.89124537,  -1.30122292,  -0.422346801,  0.0994576514,  0.348601103,  0.00641510822,  0.6921525,  0.163922176,  -0.636875987,  0.438292623,  0.516522527,  -0.876039505,  -1.84808755,  -1.74689305,  -0.340661973,  0.360895783,  1.21994817,  0.455952495,  -1.99617815,  0.952135861,  -0.820800245,  -0.250283062,  -0.0351551697,  0.302223325,  -0.707144141,  1.44752192,  -0.933865905,  1.0251379,  -0.459507972,  0.823268533,  -1.98053145,  -1.9702481,  -0.0740543455,  -0.992562652,  0.614982367,  -1.21685052,  2.43043637,  -1.39834011,  -0.555197954,  -0.341582566,  0.956830144,  1.76824248,  0.259503096,  1.30660474,  -0.732403517,  0.414176494,  0.669675231,  0.914048433,  -0.904287875,  -1.39138615,  -1.40887797,  1.53696883,  2.13737321,  -1.29668701,  -0.425925195,  -0.572709382,  0.501656473,  -1.02769375,  0.198759705,  -0.73714453,  -0.323792905,  -0.984091282,  1.65728462,  -0.508334577},{ -2.62676167,  1.33812869,  1.11254644,  -1.403494,  -1.50410402,  -0.541017473,  -1.52834189,  1.03723741,  -1.18159163,  -0.467971027,  0.350840718,  -0.11615885,  0.0606410839,  0.435699344,  -0.568988025,  -0.131621659,  0.522769988,  -0.296505302,  -1.02734435,  1.14373338,  0.42880708,  1.75522995,  0.396993071,  0.533772826,  -0.0209464785,  0.116740681,  -0.0626005009,  2.77217174,  0.540952802,  -1.00854027,  1.53330874,  1.29355776,  -0.292082995,  -0.602286458,  0.982898891,  0.960646927,  2.02309012,  -1.48060811,  0.38566646,  -0.624323189,  -0.467296332,  1.10779691,  0.656757474,  -0.95944041,  1.11936629,  -1.62387431,  0.132819623,  0.403363436,  0.1102034,  0.468603939,  0.548115492,  -0.275114924,  -2.22667742,  0.891359806,  0.610781372,  0.363847971,  2.36100221,  -0.376455933,  0.692346156,  -0.422829479,  0.337351143,  1.29006422,  -0.362480283,  -0.426113099,  0.510635138,  1.71923804,  -1.08778942,  1.77434433,  0.659629881,  -1.0486939,  -0.691329896,  2.45223522,  0.0874594077,  -0.550285995,  -0.549551547,  -2.02869105,  0.528017461,  -0.0945730731},{ 0.0915413052,  0.472120255,  -0.153974459,  0.591235816,  0.374673784,  0.731610298,  -0.297183603,  -0.408250242,  -1.26942134,  -0.340554088,  0.273112535,  0.693293929,  -0.30274123,  0.656086326,  0.223025307,  -1.88031292,  0.388534278,  -0.824014723,  -0.274121344,  -0.542285681,  0.73364234,  0.384864956,  -0.44850713,  0.722089946,  0.456895918,  -0.589825809,  -0.804592729,  0.657942355,  1.36652887,  -1.87951148,  0.250993222,  0.242676467,  -0.683318615,  1.89140689,  -1.35718024,  0.0639890805,  0.777116239,  -0.402181119,  -0.599231362,  -0.136375472,  -1.66999459,  -1.01893663,  0.798093259,  -0.739504933,  2.1296134,  -0.813131213,  -0.24029991,  1.20697975,  -0.137468621,  -1.34285414,  0.774671078,  0.585336745,  1.19188774,  0.176702812,  0.140802756,  0.969983578,  0.643540084,  0.502221286,  -1.55610466,  -0.888670385,  1.55462253,  0.293964058,  -0.44700712,  -0.168067664,  -1.84395039,  -0.331216127,  -0.967126906,  0.2649526,  -1.24985516,  -1.10362518,  -0.140369266,  0.894224703,  0.222852856,  -3.61137176,  -1.59463072,  -0.104291752,  -0.0502301864,  1.06376863},{ 2.36546493,  -1.05587447,  -0.0681112856,  -0.710755885,  -0.73704797,  1.10418296,  -0.365730256,  0.34453097,  -0.257249862,  1.62471282,  -0.0352198146,  -0.266439408,  -0.9167611,  1.57211447,  -0.44579348,  0.287351787,  0.868548453,  0.718294799,  1.0594058,  -1.10399985,  0.953235567,  -1.26329529,  -0.880716562,  0.17255047,  0.155154437,  0.384215623,  0.0318105668,  0.293063551,  -2.04427052,  0.395600379,  0.397829026,  -0.301146418,  -1.37065887,  -0.805527747,  1.15889585,  -0.773911655,  -0.336944342,  0.16112864,  -1.17816567,  -0.155934349,  -0.800348163,  -0.213558421,  -1.40876651,  -0.0743482485,  -0.847114921,  1.16828227,  1.04716337,  0.920983613,  0.216529369,  -0.846463919,  -0.0576987453,  0.645455778,  -0.944839299,  0.800434351,  0.664404273,  0.237531632,  -0.650569439,  -0.849605739,  0.700247407,  -1.45301044,  -0.036148604,  -1.50049603,  -1.72172368,  -0.125894696,  -0.785711527,  -0.390904665,  -1.09750617,  0.780672014,  1.12093961,  -0.290824324,  0.041226849,  0.71729517,  1.61040223,  0.285696357,  1.46710503,  1.05605948,  -0.0177580006,  -1.95115328},{ -0.176118508,  1.83171189,  1.11726439,  0.278683096,  -2.30910277,  1.52554452,  1.91420007,  1.21841145,  -1.33265269,  -1.06342196,  0.410327375,  -0.591935456,  0.0200090185,  -0.44358626,  0.207140386,  0.937836587,  -0.580668688,  1.21957552,  0.346632749,  -1.33851981,  -0.78373152,  1.66097784,  -1.28719449,  0.187279552,  0.927788556,  -0.729950488,  -1.14759457,  0.341122359,  -0.51136893,  -0.631762028,  0.971170545,  -1.29533601,  -0.380161017,  1.3510164,  -0.149975926,  0.677344441,  -0.494714022,  1.5109539,  -0.0904960558,  -0.193519503,  1.28003371,  -0.599422276,  0.350003988,  0.0174892452,  -0.48508206,  -3.56116843,  -1.39913523,  1.77297771,  -0.456395358,  -0.255769938,  0.190525755,  1.08931136,  -0.799288452,  -0.77805829,  0.921128869,  0.6407969,  -2.75570607,  -0.474374324,  -0.319433779,  -0.0529633053,  1.0020591,  0.535873652,  -0.950488627,  0.753215432,  0.879012942,  -0.297558486,  -1.90447807,  0.269840121,  1.28755057,  1.10111177,  -0.522972465,  1.46666455,  -0.0133879343,  0.822888672,  -1.493922,  1.41046572,  -1.21206486,  -1.04540169},{ -0.0982905775,  -0.252536446,  -0.550754428,  0.60216713,  0.0528517328,  -0.533755898,  0.323545516,  -2.90426421,  -0.143317446,  0.175371766,  0.596422076,  0.155999348,  2.52193999,  -1.170663,  0.119171157,  -0.439613909,  -0.798370242,  -1.04894102,  1.73472726,  0.110031761,  -0.461798012,  1.38601196,  1.23869467,  0.960214913,  1.56592143,  -0.429383337,  1.73451245,  0.565512955,  1.18817532,  1.02900708,  1.08256853,  -0.00485613896,  0.0506360866,  -0.430248678,  0.155757606,  -0.00094126194,  1.6035732,  -0.717627764,  0.687132716,  -2.25489688,  0.185000941,  0.0299293194,  0.204101846,  2.40670466,  1.09197271,  -0.493863344,  0.79724139,  1.77751851,  1.12356937,  -0.325160563,  0.0875987187,  -0.0893856511,  -1.43356979,  -0.58065021,  0.161665455,  -0.530046701,  -0.830907524,  -0.0436459817,  -1.22535169,  -0.00800941233,  1.09956086,  -0.690909743,  -0.804229081,  0.533778012,  0.333700031,  0.569995582,  0.0674005672,  0.783252478,  -0.641941726,  0.765114725,  -0.156287059,  -1.09707677,  -1.26020241,  -1.07809377,  0.605468929,  -0.758092284,  -0.0347007513,  -1.76770592},{ -0.819486916,  0.924123526,  -0.0768079758,  -1.82436299,  1.1825285,  1.9499613,  -0.721326113,  -1.09404099,  0.993718624,  -1.19259167,  -0.257751644,  -0.251741707,  -0.479683369,  -1.23667288,  -0.0476214476,  -0.750915706,  0.786203086,  0.575866461,  0.608017266,  0.941140473,  2.62367535,  -0.348032981,  -0.0368251614,  0.32662639,  -0.437256962,  1.63310862,  -0.0293251686,  -0.477133542,  -0.260676682,  -0.977898955,  -0.733091891,  2.38276792,  0.928631186,  0.772017598,  -1.15132165,  0.358700991,  -0.0445220061,  1.35621309,  0.291755885,  1.11004949,  -0.161961585,  0.603960991,  -0.461739391,  0.326793432,  0.450168282,  0.0474271849,  1.17891467,  -0.38191846,  -0.300157487,  -0.0817525834,  -0.901748478,  0.148936704,  -0.383475751,  0.49650979,  0.551643193,  -0.120545164,  0.573577583,  0.47499603,  1.09458232,  0.111195035,  -1.40341783,  0.684687734,  0.587040782,  -1.20781636,  -0.565922916,  -0.656057179,  1.22979629,  1.27614772,  -0.956784248,  -2.0799396,  1.59335017,  -1.50091338,  -1.19059563,  0.185088038,  -0.69712472,  1.2886802,  0.958461881,  1.89517331},{ 0.242348507,  1.08437991,  0.373675257,  0.656450868,  0.615296423,  -0.398435175,  -0.871201873,  -1.0695852,  0.478779107,  -1.6808008,  -1.10269821,  -0.681622148,  1.1171788,  -2.00576568,  0.60848552,  1.17228806,  0.747293949,  -1.51995945,  0.887905657,  0.00801868737,  1.39373231,  1.57616425,  0.609567344,  0.240498379,  1.09831619,  1.98312569,  0.250954747,  -1.65352297,  0.957442701,  -0.195637405,  0.919862211,  0.514705896,  -0.825012267,  -2.44159627,  -0.706046522,  -1.21899676,  -1.73992383,  -0.903207481,  0.294559181,  0.169831961,  0.824066699,  -0.124288119,  -0.0713199303,  -0.67415452,  0.720091343,  -1.71301293,  -0.212602377,  0.53188014,  -0.636401415,  1.75156486,  -0.84731096,  0.759912491,  -1.34043777,  0.764606655,  -0.226778999,  -0.380332619,  0.65773958,  0.107054688,  0.154393047,  -0.0752346218,  0.336026937,  0.973250508,  0.152579248,  -0.0830464661,  -1.95461953,  0.255985826,  -1.91545439,  -0.568331301,  -0.503716528,  0.114102721,  -0.805514276,  -0.372643381,  -0.335229754,  -0.540602624,  0.790907919,  1.17040205,  0.655990541,  -1.96471691},{ 1.56550276,  2.03137636,  0.440130353,  -0.267319411,  1.18280029,  0.370837688,  0.100604318,  -1.00107372,  1.39287138,  -2.59049678,  -0.471223593,  -1.31969678,  -1.86225057,  0.508789659,  -0.287001312,  -0.840331078,  0.644083679,  1.50520408,  0.407816321,  -0.654315591,  -0.695689678,  -0.473874807,  0.818189204,  0.670341015,  -0.557472944,  -1.05296683,  -0.900425434,  -0.146586433,  -0.363766104,  0.363818049,  0.213257849,  1.97016084,  0.311256826,  -0.382870525,  -0.132618397,  -1.71181679,  -1.13609624,  -1.62506831,  0.0435110815,  1.27993464,  0.943023801,  1.40564585,  -2.16682053,  0.354737014,  -0.040129628,  0.0987510979,  0.0643662512,  -1.6810838,  -0.69101578,  0.611574948,  1.12886536,  -0.0628467649,  0.783698559,  -0.627759576,  -0.571325421,  -1.08144379,  1.17259955,  -0.0167314131,  -0.345971584,  -1.33723485,  0.892537057,  -1.7162236,  -0.477116436,  0.812891066,  -0.682185352,  0.27035436,  0.218623325,  -0.613071322,  -0.43099153,  0.728691161,  -1.65586007,  0.732806861,  0.245800436,  -1.23021853,  0.787782609,  0.414550722,  -0.310590804,  -2.65151596},{ -0.465940624,  0.178232267,  -0.0450487323,  -0.473850787,  0.452526093,  -1.01503289,  0.787858248,  0.509975016,  1.94314253,  0.203966767,  -0.472515881,  -0.711216688,  1.034217,  -1.15547347,  -0.314089268,  1.65742958,  -0.41029796,  0.345607251,  0.357979774,  1.40687835,  -1.70451713,  1.49211717,  -1.55615199,  0.70568949,  1.80242193,  0.790092468,  1.35018063,  1.16401398,  -0.40537104,  0.729548275,  -0.557316184,  0.922011971,  -0.645714402,  -0.184021339,  -0.159286395,  1.3617444,  -1.42206156,  -0.692071736,  -1.10919857,  0.0722669512,  1.11027038,  0.196913481,  0.859014571,  -0.911660194,  -1.01248872,  0.768256605,  -0.175476953,  0.796361685,  0.533173084,  -1.18474913,  -0.3240017,  0.463906199,  0.748928547,  -0.213770673,  0.737420857,  -0.910836041,  -0.627233386,  -1.55827975,  0.0886685476,  1.55962682,  -0.262271643,  -0.454041749,  0.687279284,  -0.363038301,  0.320670336,  -0.883213103,  0.285295337,  0.55260998,  0.496602178,  0.487462401,  0.939822257,  0.491093576,  -0.546345353,  0.660179257,  -0.483719677,  -0.0711919516,  0.736545205,  1.03376245},{ 0.202153802,  -1.53121912,  -0.121414632,  -0.449986935,  1.10445082,  1.8045963,  0.144824848,  -0.431415766,  -0.211538747,  1.67091191,  0.705667853,  -0.181763396,  1.17062366,  -0.269020766,  0.0998878852,  -0.495535165,  -0.595057964,  1.16119301,  0.245810643,  0.846715391,  0.971111178,  1.18847358,  0.663836122,  -0.260970712,  0.343817085,  0.610794425,  -1.70395947,  -1.1286875,  -1.68676829,  1.49409747,  0.976725101,  0.946150005,  0.0954672918,  -1.06913042,  1.96944201,  1.48758388,  0.108564936,  0.181799546,  0.611665785,  -1.16671562,  0.960772693,  0.796436369,  -0.715919316,  0.744248927,  -0.594300628,  -1.39774168,  1.36454749,  -0.0238670353,  1.18400991,  -0.509814322,  1.22374213,  0.243971482,  -0.555915892,  -2.95321321,  1.06293833,  1.64140725,  0.187396273,  -0.582992852,  -1.84216654,  0.503272235,  0.363220632,  0.551869094,  -0.862635493,  0.178983822,  -0.299868166,  -0.805646241,  0.141571209,  -0.183933362,  1.68824363,  -0.0374878012,  -0.234962821,  2.9879992,  -0.0530032143,  -0.707941175,  -1.77172911,  -0.646703601,  0.800289571,  -2.25047541},{ -1.48651934,  1.07599807,  -0.597484171,  0.450673133,  0.0521224365,  2.52649808,  1.18256795,  -0.202779904,  -1.13001275,  -0.241386518,  0.243372887,  -2.41364908,  1.50924933,  1.46293068,  1.91881466,  -0.899174631,  0.107723117,  -0.394376963,  -0.766835153,  2.04485846,  1.3204242,  0.15709579,  -0.462172717,  2.14590168,  -1.39937079,  -0.717935622,  0.385130882,  -0.0638723969,  -0.64192605,  -1.93604648,  0.502510488,  -1.47940087,  -1.38004434,  0.185735181,  0.525681376,  1.45836711,  1.52675712,  -1.17353261,  -0.998014152,  0.226516649,  0.0248619113,  -0.0976783335,  0.305409253,  -1.51114798,  -1.48863494,  -0.952934802,  -1.31334507,  0.769057333,  0.796815932,  1.5923878,  -0.181465417,  0.126414657,  -0.450089663,  -0.0463011377,  -1.37629151,  0.270810097,  1.97947156,  -1.23043001,  1.07445657,  0.995869815,  -0.848643959,  -0.75736022,  0.312485456,  -1.659091,  1.07089412,  -0.580991864,  -0.330265224,  0.93798995,  1.37858665,  0.416043997,  -0.331155568,  -0.558515072,  0.0267249979,  1.17898798,  0.836663187,  -1.38857615,  -1.20248806,  -0.489747882},{ -0.137953505,  1.76943231,  -1.11534846,  -0.973583877,  -0.246867463,  -0.250115305,  -1.52360547,  -0.727012634,  -0.079433471,  0.782865882,  0.16272904,  -0.574645519,  -0.2447512,  -0.225718945,  -1.1522181,  -0.602908254,  -0.0781628639,  -0.109404027,  -0.625601768,  -0.506818533,  1.57896984,  1.13466382,  -0.0904255137,  0.239964023,  0.668497443,  -2.49299788,  0.191367283,  0.0384734906,  -1.02982986,  -0.302021474,  -3.36459827,  0.513942897,  0.270907164,  1.14021206,  1.13973498,  -0.758528829,  -1.67404199,  1.6688267,  0.790466845,  0.900880098,  0.915914416,  1.04616606,  -1.51297593,  1.16137874,  -0.360583991,  -2.06910133,  -0.342446208,  -0.0261522662,  0.141504586,  -2.18453264,  0.177634463,  -1.91264606,  -0.214314029,  0.080182068,  0.762227237,  -2.34462023,  1.16245711,  1.34425211,  -0.62228626,  -0.0608711019,  -0.0699302033,  1.82450569,  0.337621123,  0.458423436,  -1.41347301,  0.499485165,  -0.84907949,  -1.20294321,  1.33914196,  -0.48788178,  1.75113559,  -0.249059454,  0.0287154596,  -1.26010227,  0.660502911,  -0.53610301,  -0.40598163,  1.22475052},{ 1.85122871,  1.32582164,  -0.00329394755,  -0.164250165,  0.478569001,  -1.53593278,  0.145497933,  -0.672629416,  -2.19875526,  -1.77366102,  -1.29045784,  -0.980116963,  -0.0724722147,  -0.53645885,  -0.61974895,  -0.871784091,  0.396877468,  1.24065864,  1.084005,  -0.29812938,  -2.8948667,  -0.69285512,  0.327238977,  -0.681868911,  0.14673622,  -0.453365147,  1.08332443,  -0.151005462,  -0.931729198,  1.34280658,  -1.528615,  -0.606729507,  -0.425676018,  -1.70043778,  -0.0586111583,  -0.461343288,  0.402899265,  0.152877554,  1.3039012,  -1.79144847,  -0.337496132,  0.817273736,  -0.335574478,  0.668631077,  -1.38598609,  -0.308602691,  -1.38873708,  -1.35223341,  0.144936427,  0.0893802941,  0.43049863,  0.684742033,  -0.93456316,  1.6748004,  -0.106469288,  -0.282603532,  -0.883986056,  -0.331729174,  0.710900128,  0.471808255,  0.499328256,  -0.412086666,  -0.609238863,  0.387194216,  0.18895781,  -0.599778354,  -0.477135211,  -1.51271021,  1.31273949,  1.00912702,  -0.837761641,  0.514456987,  -0.0517553017,  -0.562057674,  -0.512803078,  -2.17777443,  -0.105505109,  -1.52813792},{ 0.318573534,  -2.87452006,  1.12315106,  -1.26431143,  -0.20047307,  -0.762231648,  -0.286565512,  -0.616678476,  1.421278,  0.357071847,  0.704622805,  1.06230068,  1.46649694,  -0.728433192,  1.08394325,  0.217513084,  0.374529123,  0.882206798,  0.200908408,  0.165248096,  -1.68309319,  0.131926835,  -0.776859581,  0.733980596,  0.844423711,  -0.00965582579,  0.293188244,  0.220014811,  -1.02825236,  0.0398467742,  -1.42731559,  -1.19803071,  1.25906205,  3.30642271,  1.06688726,  0.629873335,  -1.72338307,  0.562175393,  1.03300261,  -1.41931689,  -0.940628886,  -1.78763008,  -0.786788464,  0.667278647,  0.549947143,  -0.221426576,  -0.851330519,  1.73967814,  -1.02463412,  -0.901324272,  1.02595794,  -0.412173122,  -0.387369543,  -0.242083862,  2.11909008,  -0.814023852,  0.171057999,  -0.548696697,  -0.819972217,  -0.300098777,  0.507441401,  -0.283003777,  -1.03894413,  -0.18679814,  1.2164917,  1.32058132,  0.278514832,  -0.434034258,  -1.86670971,  -0.453845561,  -1.81840134,  -0.774233341,  0.112514891,  -0.0338471979,  -1.07413316,  -0.0385382064,  0.344725251,  -1.57252097},{ 2.45467448,  1.06522524,  -1.48533952,  0.132099167,  -0.977576852,  2.20335793,  -1.51691198,  -1.07371211,  3.27482605,  -0.45921874,  -0.581330657,  -1.6754148,  0.734900177,  2.09807062,  0.442869931,  -1.57431507,  0.692034304,  0.842961907,  -0.637233794,  1.15154755,  0.158369586,  -0.923409939,  -1.4486891,  -1.38740301,  0.979548991,  -0.0156967342,  0.474443465,  -0.691740334,  1.02900565,  0.140566871,  -1.40570986,  0.421314746,  -0.0104695093,  0.463564217,  -2.53412032,  0.0325277783,  -0.528145492,  -0.196774334,  -0.501920223,  0.58730793,  0.245693833,  -0.374537796,  -1.93193352,  -0.439692229,  -0.144440889,  1.61431134,  -1.51774931,  0.194827065,  0.9983899,  -0.607658029,  0.479918808,  0.229663357,  0.341897964,  0.00766551448,  -0.313089788,  1.27013052,  1.4262296,  0.286542207,  -0.134941831,  0.472173989,  0.544718623,  -0.721921861,  0.281465083,  -3.30681133,  -0.139178321,  0.00459014205,  -0.159480989,  0.817465007,  -1.49042916,  0.371024489,  -1.87139225,  -0.7212286,  -0.425192714,  0.877031922,  0.52243191,  -1.23561144,  -2.11238575,  -0.435179919},{ -1.89408195,  -1.38970804,  -0.0942054689,  0.666115224,  0.224229202,  0.00155659358,  1.62159884,  -0.172711998,  2.07518411,  1.62766349,  -1.07915008,  -2.20424247,  1.12647784,  -0.776296616,  1.52890611,  1.11940038,  0.789028585,  0.814393461,  -1.41411507,  -1.46293926,  -0.612942338,  1.2830838,  -1.90985346,  -0.657404423,  0.155291155,  0.450504988,  0.682807267,  0.214436844,  1.50550675,  -0.90630877,  0.947955906,  0.136486709,  0.074830763,  -1.31812143,  0.358831137,  0.244561896,  0.867933333,  1.14488459,  0.105153799,  -2.09493923,  0.54641664,  -1.08550513,  1.58279443,  -0.0674575791,  1.19228423,  1.46779943,  -0.0982013643,  1.11640275,  -0.0857974812,  1.77829957,  -1.47219694,  0.0255789068,  1.02722847,  1.07066584,  -0.96621722,  0.865666926,  1.34719598,  -0.376665145,  -1.39465964,  -1.59683216,  1.34389925,  -0.878021777,  0.897407055,  -1.2130146,  0.459660292,  0.233324274,  0.879905522,  1.60276926,  -0.796072483,  0.345616102,  0.0486891493,  2.39374375,  0.839286625,  1.39330089,  -2.81503749,  -1.58755195,  1.23888779,  -1.18587971},{ 1.06479681,  0.0972583294,  -0.864431322,  0.151158661,  0.300034314,  1.33139896,  -0.802206278,  -0.564500809,  1.43066847,  -0.532804906,  0.195006311,  -2.64052463,  0.433885664,  0.550722897,  -1.5679276,  1.19471312,  1.04294097,  0.544895887,  -0.395702392,  -1.5403055,  -1.2967608,  -1.39632487,  0.718167305,  0.276134014,  0.344966739,  -0.895633578,  -0.423784494,  0.210413411,  1.00893545,  0.158240512,  2.20714164,  -0.789086759,  -0.248284623,  -0.528174341,  -0.385713488,  1.15136468,  0.136798486,  -0.118244559,  -2.97302413,  -1.6696887,  -0.611881256,  0.549655855,  -0.386679858,  0.945635259,  -1.04800653,  -0.233298808,  -0.74092114,  1.767609,  0.582947314,  -1.76464176,  0.566584408,  -0.508689284,  0.498852938,  -0.523907483,  -0.130647108,  -1.49897826,  0.818360507,  2.31394172,  -0.694211602,  -0.773398161,  0.126129031,  -1.15476084,  -0.0921143219,  0.708313763,  -0.681713402,  0.103763372,  1.02312076,  -0.5561046,  -0.228118584,  0.816957772,  1.26559091,  1.2091738,  -0.790932119,  0.0679462329,  -1.06563783,  1.71004033,  0.214471877,  0.435972601},{ 1.3976289,  0.893442333,  0.418908983,  -1.2797929,  0.0833007172,  -0.65348351,  1.7365855,  1.39142144,  0.926515996,  0.2332986,  1.28171539,  -0.664609075,  -1.4551785,  0.157159135,  0.504578054,  -1.06909347,  2.80324483,  0.839947939,  -0.321947902,  0.456454903,  1.30598617,  -0.604550362,  -0.816376865,  1.62397897,  0.48619923,  -0.0360128991,  -0.144123495,  -0.236819416,  0.389897108,  -0.435768813,  -1.58730316,  0.253842533,  -0.917937994,  0.0482380278,  1.31716931,  -0.194235176,  -0.0102596581,  0.168341577,  0.125345781,  1.05014384,  1.88920224,  0.197714776,  0.177071974,  -1.46047282,  -0.535674036,  -0.507811129,  0.249285102,  0.961778939,  0.600730777,  0.716791868,  1.22071314,  0.359707683,  -1.65814447,  1.39235342,  -1.32153773,  -0.413654029,  1.08381927,  0.0260841977,  0.0993115231,  1.69683504,  0.354662031,  0.825579107,  0.627126217,  1.19446957,  1.23635125,  0.0283635147,  0.408551961,  -0.408574402,  1.82693803,  -1.65786827,  0.826211333,  0.0485402793,  -0.531650066,  1.05590749,  1.15901828,  -1.32932782,  0.0571417548,  1.49863029},{ 0.932450414,  -2.93994451,  -0.509790003,  -2.24029779,  -0.706549764,  -0.404839039,  -1.81197917,  0.391450346,  0.145933002,  -0.989848912,  0.481576651,  -0.508137941,  0.268805265,  0.455486476,  0.904579341,  -1.07567906,  1.2950331,  0.758635223,  0.456200123,  3.22609186,  1.84713197,  0.225472823,  -1.72775793,  0.822327554,  0.365249783,  0.681438446,  -0.567471743,  -1.38193977,  -1.03241897,  0.469475865,  1.09587204,  0.0471275225,  -0.018925678,  0.37918371,  -0.957433462,  0.733361244,  -0.225525975,  0.457434744,  0.789677441,  -0.44891566,  0.567685068,  -0.200311556,  0.759134531,  -1.80671477,  -0.328383654,  -0.0596480332,  -1.37040067,  1.20347941,  -2.33167672,  0.0237661246,  -0.587747931,  -1.33374608,  -0.265092999,  -0.671165645,  -0.137763441,  1.7299695,  1.66993463,  0.697092116,  -0.339046031,  -1.7593174,  -1.17587745,  0.858836591,  0.379160851,  0.249923855,  0.720731616,  -0.325735271,  -0.128902406,  -1.0146364,  -1.60979104,  0.186922669,  0.143703461,  -0.273528665,  0.311697543,  0.235764548,  0.122764289,  0.759891152,  0.598592639,  1.98172307},{ -0.269251108,  1.99303782,  -0.262983561,  -0.943871677,  -0.542154014,  -0.211130649,  0.781864047,  -0.260623842,  0.0342008509,  -0.644652724,  0.916533411,  0.349819839,  -0.564106405,  -0.972635508,  -0.785525978,  -0.311991155,  1.2393018,  -1.16791737,  1.64796758,  1.48325038,  -2.76319313,  -0.651395857,  1.08285964,  0.101079643,  -0.823813677,  -1.54806471,  -0.400152236,  -1.95359385,  -2.07399201,  0.355319828,  -1.76230633,  -0.24745205,  -0.381045371,  -0.192358166,  -0.447769254,  -0.0184024312,  -1.0912714,  0.724304557,  0.279563218,  -1.14366865,  -0.211144686,  -0.388690144,  0.245940864,  0.360146314,  0.436099648,  1.04225266,  0.547647655,  0.484116495,  -0.44495821,  0.730512083,  0.948891699,  -0.320544332,  -1.40645111,  -0.784484148,  1.11715317,  -0.159477755,  0.432911307,  0.707851231,  -0.731474698,  -1.75215638,  1.85585511,  -0.756158292,  -0.0654040799,  2.15539742,  -0.436209828,  0.873603344,  2.19290471,  -0.555701494,  -0.537068963,  -0.482137889,  0.265348762,  -0.72989434,  0.502270162,  -1.467116,  0.641049922,  -0.141992554,  0.589004278,  1.07557714},{ -0.756028831,  -0.610777617,  -0.952641904,  0.00699499762,  0.146089315,  -0.124866173,  -0.144702062,  0.875735939,  1.15545475,  -0.197631955,  2.3693943,  -0.234217212,  1.05410576,  -1.56178617,  -0.395159394,  -0.922982395,  -1.22362828,  -0.945235729,  -0.180701494,  0.199663967,  -0.764373124,  -0.955590725,  -1.58962607,  0.214972928,  0.82943666,  0.389345169,  -0.215642214,  -1.21716487,  -1.61801386,  -0.355709195,  -1.55351114,  -1.34349132,  -2.09760571,  1.45140433,  -1.67675936,  1.71921802,  0.882668495,  1.34487498,  0.872625351,  -0.095816493,  0.194126859,  0.441349268,  -2.67949963,  1.00178528,  -1.42069638,  0.668697596,  -0.521590054,  1.56147838,  0.0889995173,  -0.725781798,  -0.0464424603,  0.585021257,  -0.2737858,  -0.289650232,  -0.597540438,  -1.46367729,  -2.12427044,  0.160486311,  0.516131639,  -0.530338049,  -1.62216771,  0.630007446,  -0.981700301,  1.20388222,  -2.58162951,  -0.632341743,  -0.684666574,  0.737752736,  0.99393487,  0.783030212,  0.0946253762,  0.849951446,  -0.619622827,  2.50751996,  1.84822226,  -0.343383193,  -0.643552005,  0.615420699},{ 0.526552796,  -0.0858887434,  -1.42091954,  -0.837799191,  2.32944322,  0.396983713,  -0.578880072,  -0.203394488,  0.870671511,  -1.04844606,  3.20962119,  -0.180482432,  -0.347030997,  0.0263186079,  -1.54831529,  -1.87170768,  -1.72680402,  0.941586554,  -2.50603271,  -0.714976907,  0.787110031,  -0.0627841949,  -0.807027221,  0.0611686967,  0.232339501,  -1.53530812,  -0.929643571,  -0.187502131,  0.176146403,  0.32560569,  0.0312165059,  0.578014493,  -0.473384559,  -0.318349272,  0.517407656,  -0.544350445,  -0.552994132,  1.24848545,  -0.0896819532,  -1.3056612,  -0.861873448,  0.629784703,  -0.392785758,  -0.39342925,  -1.30860066,  -0.687020898,  0.272255331,  0.0491055958,  -0.858033776,  -0.244519398,  0.244399443,  0.118059188,  -0.468742341,  -0.12856406,  -1.92757058,  0.393969119,  -0.52529335,  1.04460931,  0.0146556618,  -0.0843033344,  -0.771518886,  -0.97621733,  0.634488583,  0.666115642,  0.388186008,  -0.939672112,  1.13301408,  -2.69017696,  0.398351967,  -2.29811716,  -1.34693837,  -0.381268173,  1.01784647,  -1.17444968,  -0.00916643348,  -0.077535823,  -0.804960907,  0.330743372},{ -0.559567034,  -0.36284098,  -1.64889932,  -0.294948667,  -0.282256007,  0.670350254,  -0.306052387,  -0.256203085,  -1.72362947,  1.14456534,  1.26079869,  1.83225596,  1.33662212,  -0.927546144,  0.903227448,  -0.447890669,  0.198460564,  0.605872393,  -0.578567743,  -1.55013144,  -2.1896944,  -0.164810002,  -0.169021606,  0.643491626,  0.46588847,  -1.21303618,  -1.59254658,  1.58528686,  1.27275026,  1.59157002,  1.23439777,  1.35498333,  0.51275456,  -0.240108833,  -1.62774765,  -0.932716608,  -0.101115532,  0.325878203,  -0.840733051,  -1.55973816,  0.950790226,  -0.520478487,  0.235246778,  1.20601773,  0.570283651,  -1.13920975,  0.376982033,  -0.254120559,  -1.64647305,  -1.010023,  0.0862346888,  2.14124966,  -0.556958914,  -0.862131894,  -0.293764055,  -0.474742055,  0.173629463,  -1.59135759,  1.87367451,  0.10098397,  -1.08704031,  1.06177664,  0.71971041,  0.460493654,  1.02098024,  0.12957871,  -1.28093803,  -1.43573713,  -1.39558268,  -0.223446399,  -0.312072724,  -0.906685829,  0.028388653,  0.597112536,  -0.132930487,  0.649111629,  0.0978929996,  -0.142798871},{ 0.168762192,  -0.137956247,  1.15518808,  -0.466450274,  1.46195722,  -0.700716138,  0.82513845,  0.293661565,  0.0881614313,  0.503994048,  0.345625311,  -0.190122336,  -2.17945766,  1.83217597,  0.714667141,  -0.221452177,  -1.70449936,  0.298548758,  -0.323099375,  -0.207186878,  0.0741565973,  -2.3063426,  -0.486236125,  -0.7832582,  0.900723755,  -0.452359319,  -0.869186282,  -0.528831124,  0.901466668,  1.45818448,  -0.262624294,  0.775785446,  -1.49736369,  1.16316819,  0.219987735,  -0.778043032,  -0.463296503,  -0.0827805847,  0.799923241,  0.360242665,  -0.430452913,  0.222667843,  0.933390498,  -0.891582608,  -1.14253116,  -0.170354769,  -0.672268569,  0.842805386,  -0.0557446927,  0.606552482,  -0.0526929274,  1.28626311,  -0.379949093,  0.860714138,  0.456954211,  -0.68877542,  1.56939447,  2.47065091,  -1.2096287,  0.00683698524,  -1.12082982,  0.311224312,  -1.22634149,  0.764209032,  -0.831006527,  0.0816265121,  -0.0792657956,  0.100960866,  0.0770017877,  0.330684125,  -2.32245207,  -0.586023927,  0.986083746,  -0.985996604,  0.224132657,  -2.27077007,  -2.02707386,  -0.32800132},{ -0.795827746,  0.0591231249,  -0.533291101,  0.0491992794,  -0.229323134,  1.69481111,  -0.724753082,  -0.252362996,  -0.664738655,  -0.311455101,  -0.55926764,  -0.0600095578,  -1.7021184,  1.52850735,  1.38066304,  -0.621717155,  0.757326722,  -1.20682681,  -1.17689586,  0.55006969,  1.13586724,  0.228392407,  -2.04993224,  0.217965648,  0.887344182,  1.34834254,  -0.293062001,  -0.227462307,  -1.30596578,  -0.268769115,  -2.59438276,  0.167682171,  0.430670112,  0.435981035,  0.723495781,  0.984821498,  -0.382507622,  1.56572449,  -0.805012703,  1.12068748,  -0.0283352826,  0.529132843,  0.217772618,  -0.582743347,  2.57570815,  1.76836491,  -0.0341375247,  -1.76653194,  0.388320148,  -0.608325005,  -0.443809241,  0.236354962,  -0.0679336637,  1.4084748,  1.06426907,  -0.237889275,  0.109735906,  0.676662207,  0.715777278,  -0.553856254,  0.932061374,  0.629588723,  0.714962721,  0.994214058,  -1.06017804,  0.661103606,  1.80902779,  0.507911265,  0.258442104,  -1.15412402,  0.637990057,  0.689700127,  -0.693192482,  0.910150945,  -0.828934908,  0.966222823,  0.044866167,  -1.03820574},{ 0.465457886,  -1.05868638,  1.45750797,  -1.8645277,  0.250288278,  0.770969927,  0.553346515,  0.966012955,  0.430004239,  -1.06515133,  -0.908412933,  -2.01301932,  -0.587446511,  -0.123156615,  -0.554087818,  1.03270817,  0.391421109,  1.38248491,  0.942387819,  0.214260161,  0.783293188,  0.242317557,  -0.426027119,  -0.786863387,  -1.12833118,  0.217879042,  1.08560061,  0.0873054117,  2.03443599,  1.85261822,  -0.366290271,  0.801531672,  0.0282158814,  0.047937341,  -1.86514139,  2.6611774,  -0.844334006,  0.315447658,  -0.586900353,  0.011144503,  -0.364834249,  1.25318635,  -0.265251398,  0.818566918,  1.90424216,  -0.235090494,  -0.648936927,  -0.665800273,  -0.519202828,  0.976682007,  -0.320851058,  0.631519675,  -1.14168358,  -2.02628255,  1.9421804,  1.7945497,  0.466218174,  0.0549705438,  -0.701395273,  -0.729094684,  -0.339185804,  -2.04117608,  0.509524882,  -3.06512213,  0.349977136,  0.39697656,  0.0237360783,  -0.869641781,  -1.11385548,  0.387804866,  1.48984134,  -0.711785614,  1.59305418,  1.25852442,  -0.247388929,  0.621697128,  0.606491327,  -2.91244698},{ -0.0562701486,  -0.0392585173,  -0.835181296,  -0.748252332,  1.64557266,  -0.552790821,  -2.90931392,  1.01763272,  -0.314765215,  0.249303117,  -0.984175265,  0.0876885131,  1.02558029,  0.99091202,  0.101996489,  0.160879076,  -0.895287514,  -0.468026787,  -1.16905797,  0.116739273,  1.18676245,  0.296045661,  2.0723865,  0.41098243,  -0.943495095,  0.743003607,  -1.06298506,  -0.227474153,  0.0608019531,  0.635225177,  0.814959347,  0.25323391,  -1.58175576,  -0.658745646,  -0.262922525,  -1.74397862,  -0.438023418,  0.628563404,  -0.663388669,  -1.66023242,  -0.647940099,  -0.482505351,  0.625224292,  0.162064508,  0.0253877211,  -0.0984101817,  1.16118503,  0.631789446,  0.173792094,  -0.71014905,  -0.995186388,  -0.778330743,  0.805598497,  0.884613454,  -1.6650728,  -0.734731019,  -1.25703907,  -0.881009281,  0.339717358,  0.138820559,  -0.682276964,  0.38070786,  -0.0273414813,  0.94712472,  0.722814322,  -0.0640561581,  -0.224654093,  -0.900165379,  0.0692378059,  0.320292473,  0.782931745,  0.0892890766,  -0.505283356,  0.859330058,  -2.22982383,  0.494758427,  0.651640415,  1.09965336},{ -0.081139639,  -0.584493876,  0.477034599,  -0.839253664,  -1.23684788,  2.11670423,  -0.330088973,  -0.932183325,  -0.634370327,  0.605384529,  -0.726532102,  0.803257704,  -0.389186591,  -0.303798109,  0.0542392321,  0.638380587,  -0.359868169,  -1.11653125,  -0.908911645,  0.629238844,  0.0852300972,  -0.209585503,  0.57964164,  -0.556777537,  -0.268017173,  1.32922876,  -0.249661848,  0.069632858,  -0.697938502,  0.640246987,  0.359216481,  -0.358761609,  -0.0652933568,  -0.0113313962,  1.42835236,  0.567821503,  0.102675229,  -0.812490225,  0.227326378,  -0.140172973,  -2.55981421,  -0.0789843872,  -1.24090314,  2.14733815,  1.01964438,  0.529969633,  1.39704525,  1.35942662,  -1.26602018,  -0.536536872,  -0.717908919,  0.616387665,  0.994905233,  -0.402002394,  -1.81279552,  -0.112907611,  -0.600062191,  -0.0474577732,  0.734559178,  -0.170297667,  -0.238953382,  -1.00913835,  2.35454059,  0.0937670544,  1.25576198,  -1.455585,  -0.424062401,  0.392291933,  0.716690361,  -0.13769719,  1.40699601,  -0.60631144,  -0.195311949,  -1.2431736,  0.279240578,  -0.87592721,  -0.501251221,  -1.56018138},{ 1.0616492,  -0.270694524,  -0.822009087,  -0.162269264,  -0.548247218,  -0.0729062557,  -0.226397336,  1.5931257,  0.843176067,  0.6093961,  1.06600463,  -1.17117417,  -0.677266538,  -0.212631881,  -0.713342845,  1.31152606,  -0.49312824,  0.595656276,  0.339274377,  -0.0018903598,  -0.10614419,  -1.85320461,  0.902250111,  -0.499327064,  0.699090183,  0.691846788,  -0.627741277,  0.171082601,  0.0954092965,  -0.0233115498,  0.13696447,  -1.0608784,  1.24218988,  -0.0136368964,  1.71062708,  0.46940273,  0.0921165645,  1.16730142,  -0.248295709,  0.980447769,  0.93152827,  -0.348413557,  0.159187779,  0.644962966,  -1.19731867,  -0.889542818,  -0.404246986,  1.36969113,  -1.53028548,  0.138280794,  0.14684923,  0.286988974,  0.226494983,  0.914110243,  -0.0953174904,  0.897301972,  -0.503318906,  0.585062683,  0.256315708,  1.24125874,  0.192518666,  2.01744533,  0.442197978,  -2.14632535,  0.577542782,  -0.798478067,  0.55919683,  -0.919329464,  -1.01059473,  0.493485719,  0.329858571,  -0.943437815,  -2.18038511,  1.84260738,  0.708134353,  -1.23071694,  -0.974789441,  -1.80896699},{ -2.53192186,  1.22216237,  -0.135701478,  1.10489213,  0.132161811,  -0.260441899,  -0.399936467,  0.0485886037,  -0.604838133,  -0.391983092,  1.84842563,  0.0600312948,  0.0913834274,  0.193682611,  -0.54575491,  -0.127877846,  0.219332501,  0.833687901,  -0.591753185,  0.556450903,  -1.23471868,  -0.355292588,  2.38302183,  0.0542832799,  0.010090041,  0.678221524,  -0.378069401,  1.83424199,  0.841046631,  -0.493515313,  -2.45568967,  -0.497869521,  -0.590247989,  0.89411211,  -0.755778611,  0.753501117,  0.0062328889,  -1.79491305,  -1.02964675,  0.294612586,  -0.798327982,  1.31125033,  0.167364985,  -1.22703016,  -0.803225279,  -0.798522294,  -0.0505162477,  0.676895916,  0.142476931,  1.5669812,  -0.720595717,  -0.893482864,  -1.17740321,  -0.105317906,  -0.628523231,  0.348910242,  0.571062744,  1.94793439,  -0.00407066988,  0.0908435136,  -0.166426226,  -0.357417881,  1.5057416,  1.40156543,  1.73469138,  1.27452099,  -0.0368164927,  2.62281656,  -0.780950665,  1.65030026,  -0.47438097,  0.390813261,  0.0764678866,  0.0779065937,  -2.69032097,  -0.421816319,  0.516756415,  1.02101243},{ -0.849610329,  -1.537305,  -0.851787329,  0.811556399,  0.25867039,  -0.808685124,  0.830702066,  -0.172668949,  -0.439670682,  0.334768355,  -0.0030613686,  -0.870406508,  2.49856973,  1.51580179,  -0.532285273,  0.131707206,  1.1856389,  -0.172627196,  -0.132262215,  1.12411308,  -0.55974108,  -1.10109401,  -0.844870746,  0.585211992,  -2.0632062,  1.17961025,  -1.01987326,  0.533040226,  -0.372441173,  0.572342515,  0.688197076,  -1.11307955,  1.06908429,  -1.33942831,  -0.682471395,  0.555967689,  0.272462368,  -0.653389037,  -0.920831263,  0.747514546,  -0.33993116,  0.397175252,  0.1713029,  -2.0270648,  -0.017787315,  -0.0182014164,  -0.494070083,  -0.0462970808,  -0.926851809,  -0.138373464,  -1.37228739,  -0.969062686,  -0.618351638,  0.47900781,  0.548801064,  0.393077224,  1.719841,  -0.357085973,  -0.165011019,  0.0470302701,  -0.132819533,  -0.386218399,  -0.216976345,  -0.388614714,  -0.302159131,  -0.109641582,  0.60893786,  0.528348863,  0.934142053,  0.532911062,  -0.566159546,  0.296589017,  -1.46058416,  -0.231187537,  -0.418417215,  -0.373804778,  -1.99216175,  1.37319779},{ 1.04446459,  0.383829117,  -0.214442328,  -0.465379745,  -0.510284245,  0.0279696807,  2.65518713,  0.703547776,  1.02484703,  0.738969088,  0.733191729,  1.47479892,  1.9998852,  0.139808938,  0.288722247,  -0.644986808,  -1.54362261,  0.0872187987,  0.789655507,  -0.533373117,  0.971170247,  -1.09719718,  -0.484300405,  -0.123330325,  -0.912401319,  -0.995000958,  -1.20424294,  -0.775110006,  -0.918684006,  1.04732347,  0.402938575,  0.92502141,  -0.564092875,  1.88756573,  -1.15644455,  -0.119287774,  -0.36109516,  1.82621109,  -0.315984994,  0.561420083,  -0.524419248,  0.809552372,  -0.661552668,  -0.282816321,  0.290974349,  1.72638154,  -0.122225083,  0.663674235,  2.21717739,  0.373507887,  -0.0874607787,  0.228923678,  0.422268242,  -0.793893099,  -0.45393455,  0.00790660176,  -1.50864947,  -1.51652026,  1.6035322,  -0.201964512,  0.890537262,  0.716399431,  0.0144381709,  -0.622754455,  -0.367874652,  -0.0212802924,  0.279023051,  -0.517016649,  -1.21917248,  -0.0382040702,  -1.54319775,  0.200860396,  1.21067095,  -0.173358798,  0.00782638881,  0.841796815,  -0.804312706,  1.32146394},{ -1.41060293,  1.20480764,  0.114974342,  -0.0268989373,  1.07690132,  1.27509749,  0.948858619,  -0.160260886,  2.26652575,  0.331152171,  0.265013993,  -0.815537035,  1.00259089,  -1.55490208,  -0.485486239,  -0.879060984,  1.8448863,  1.84606886,  -1.83834112,  0.464354515,  -0.0792144537,  1.11132765,  1.17858434,  -0.797468483,  -0.905296028,  -0.0337393768,  -1.2079376,  -0.631404936,  0.781758964,  -1.1677053,  1.29239166,  0.388562709,  -0.84247005,  -0.697396517,  -1.91721356,  -0.0829902217,  -0.727760434,  1.04863012,  1.07298481,  0.831595361,  -0.291962624,  -0.901159883,  1.47662795,  -0.658338785,  0.669031084,  -1.33822906,  -1.43074524,  0.893759429,  -0.70072782,  1.85834229,  1.07969177,  0.699513257,  0.27580747,  0.901063025,  1.11314785,  1.27043355,  0.564112961,  -0.337320179,  -0.85949856,  1.54871368,  0.361157268,  1.36845398,  0.245960951,  0.827914596,  0.418816626,  -1.35983944,  -0.431833118,  0.581667483,  2.25945497,  -1.69173551,  -1.79253745,  -0.597121537,  -1.64961457,  0.866693139,  -1.20975733,  -2.01339126,  0.672258675,  1.18397057},{ -0.0185559168,  1.18158734,  0.271218032,  0.786609888,  0.012384057,  1.34689939,  0.509687603,  -0.623060346,  0.756931543,  0.370738804,  -0.912458003,  -0.772838831,  0.783879042,  1.18686032,  -1.13075054,  0.651055813,  0.632963955,  1.9746393,  0.387019545,  -0.0810484588,  1.29220426,  -1.59613752,  -0.821258903,  2.0877254,  -1.84904027,  -0.887066066,  -1.48561025,  0.668895423,  0.149097532,  -1.0535022,  -0.829915524,  -0.809742332,  0.29990688,  1.00194204,  -1.85723865,  -2.13905978,  0.0900650024,  -1.12186205,  -1.5435611,  0.220979705,  0.368338794,  -0.85631299,  -0.527132154,  -1.02433729,  -1.96593094,  1.05992007,  1.51429129,  0.0697470829,  0.986703455,  1.32350981,  0.244690672,  0.873226583,  0.580145478,  -1.48821974,  1.31893718,  0.0315837786,  0.778134048,  -0.111923061,  -0.0241832621,  -1.35666883,  0.374394476,  0.422808677,  0.342282712,  -1.16420233,  -0.0490740277,  0.146686986,  -1.8372184,  -0.267167002,  0.594467223,  -0.269161612,  -1.74432266,  0.397219032,  1.52725303,  1.11757028,  -1.07278979,  -0.492031366,  0.914938152,  0.678846538},{ -0.667304337,  0.491121948,  0.584603071,  2.18449497,  -0.271834046,  -0.826527178,  0.145900518,  -0.189266428,  0.964100242,  0.0592055246,  -1.31246364,  0.781278372,  -0.285723627,  -0.507575274,  0.0364605784,  0.437004626,  -1.75342166,  0.31344682,  0.892971456,  -1.06392407,  0.321774572,  -0.678032458,  1.48250902,  0.832117558,  -0.517825723,  -1.98192036,  -0.427716404,  -0.576703668,  0.217955649,  0.784469187,  1.96604097,  -1.93075871,  0.159013063,  -0.56837976,  0.915691435,  -0.43378824,  -1.43864667,  -0.701875925,  -0.771394312,  1.33280122,  -0.56116569,  -0.958145797,  -0.747861445,  -0.282199085,  -0.0718430057,  -0.549635589,  -0.121198207,  -0.693685234,  1.20572197,  -0.271496892,  0.759997845,  -0.494537771,  0.657227874,  -0.42640236,  -0.964716136,  -0.631728947,  -1.69842589,  0.013657121,  1.58782423,  1.15907717,  -0.887350023,  -0.645540774,  -0.857018113,  -0.414709091,  -0.10529203,  0.023019515,  -0.54612267,  -0.360185236,  0.709852993,  -0.428656787,  -1.13698268,  -0.241166398,  -0.0545455478,  -0.193275318,  -0.223942354,  0.465069979,  -0.463767171,  0.0950175524},{ -0.19849059,  -0.802639067,  0.417468965,  0.660695493,  -0.735877872,  -0.655113697,  -0.0947928801,  1.56696224,  0.477726102,  -0.778428733,  -0.388298094,  -1.47883427,  0.725438297,  -1.37629354,  0.282650828,  -0.44606629,  0.20869951,  0.172193035,  0.224766865,  -0.123440661,  0.081206046,  -0.710490525,  -1.60472906,  0.267344981,  0.906691849,  0.263046712,  0.982232094,  0.00462249201,  0.947730243,  -0.128071815,  0.42311722,  0.345903307,  0.159251809,  -0.0543919317,  1.08991718,  0.849631786,  1.1354413,  1.86924958,  -1.0810163,  -0.0613854937,  -0.934108078,  -0.262299508,  -0.894426227,  0.0290575251,  0.212712109,  0.503433943,  0.344211489,  0.373442352,  -0.41450876,  -0.404454708,  0.124033928,  -0.121114925,  0.960992157,  -0.369177967,  -0.309494168,  1.95953107,  -0.466143012,  0.305189461,  1.00941932,  -0.911764085,  0.386921257,  0.182929561,  -1.11220217,  -0.247633144,  -0.46075666,  0.0985850245,  0.142395839,  -0.417007864,  -0.120701559,  -1.04852772,  -1.90699947,  2.22650766,  -2.66949201,  -0.415716916,  0.903050125,  0.281796724,  1.33394623,  1.30579948},{ 0.0970871076,  -1.49116182,  -0.430756271,  0.693192959,  -2.5146625,  0.532905579,  -0.354974598,  -1.98438048,  0.0483013019,  -0.648795605,  0.105701245,  0.0188354254,  0.209433407,  -0.913112521,  -0.473778814,  1.18412435,  -1.72611797,  0.0700969547,  1.4469173,  -0.543657422,  -0.349412411,  -1.35336173,  -0.0147185046,  0.758933485,  0.276379943,  1.272295,  0.906829715,  0.652209282,  -1.04978037,  -0.5352543,  0.471648663,  -0.968650401,  0.494082063,  -2.39949679,  0.269157469,  -0.912478089,  1.87092555,  -0.0607215203,  -1.74563313,  1.41131246,  0.590878546,  0.210610494,  0.21893613,  1.56218064,  0.507135332,  0.414447367,  -0.106010631,  -1.76429963,  0.176217735,  -0.417159379,  -0.274008393,  1.19631445,  0.909230113,  -0.251689464,  -0.0355110019,  0.633029878,  0.777810454,  -1.61392963,  1.18174434,  -0.51850903,  0.0490452051,  -1.00066435,  -1.80131137,  0.664246738,  1.13437128,  -0.173975125,  2.68733811,  2.35300493,  0.902057409,  -0.148134425,  0.0123721194,  0.197925732,  0.940307319,  1.13704348,  -0.203980803,  0.478144944,  -0.284988761,  1.81586576},{ -0.948130071,  -0.0183537081,  0.767914414,  2.3161869,  -0.836436987,  1.47827911,  1.46846747,  -0.515905797,  0.00875892118,  0.950108409,  0.191969231,  0.974085569,  -1.20032358,  -0.097338669,  0.397450566,  -0.241325051,  0.0472463928,  0.679075778,  0.210922778,  -0.396408856,  0.762636244,  0.709711015,  -1.43616974,  1.36049449,  -0.0957891643,  1.14927697,  0.588504076,  -0.433767855,  0.553160191,  -0.064744778,  -1.23926401,  -0.302607536,  -0.629701197,  -0.113445975,  0.191766009,  0.397068858,  0.180919692,  -1.55279005,  -0.597158313,  -0.867672861,  2.29994154,  -0.192292467,  0.232203811,  0.989098728,  0.129142061,  -0.266967237,  1.0600282,  0.0244946834,  2.23177123,  -0.4279356,  -0.862347901,  -0.163388342,  -0.148779362,  1.04859829,  -0.274026215,  -0.661083043,  -0.978713393,  0.627115726,  -0.847233772,  -0.393082798,  1.59501266,  0.981541872,  1.97312331,  -1.90574515,  -0.822545767,  -1.22733557,  -0.502620697,  -0.735387564,  0.0369875431,  1.17336941,  0.0721975639,  0.455652177,  0.771603286,  1.1191566,  1.09690213,  0.26110664,  0.906601369,  -0.981013358},{ -1.18166447,  -0.511668026,  0.90081656,  1.9631778,  0.579680622,  0.791822553,  0.657842517,  -0.102475159,  1.6321665,  -0.0754398853,  -0.0222660806,  -0.714459419,  -1.08203173,  -3.37851238,  1.21875656,  -0.388608098,  -0.0240015946,  -0.473266512,  -3.63263798,  -0.0543935075,  -0.339161336,  -0.0894379243,  -0.334030271,  -2.07999611,  -1.08417606,  -1.23634207,  0.125876456,  -0.48535043,  0.712839842,  1.50415206,  -0.174571097,  -1.35811591,  0.0228119008,  -0.369924277,  1.0905503,  -2.12083244,  0.617312431,  -0.338442832,  2.47357583,  -0.84290415,  0.0193024669,  0.0866344869,  0.386749685,  0.257995874,  -0.00891798083,  -0.72275573,  1.32526243,  -3.47263384,  -0.0810612664,  -0.562864244,  -1.61995196,  -0.454183757,  0.331731558,  -0.401132405,  -0.653451145,  1.25713527,  1.23312616,  -1.05309021,  0.191724703,  -1.56635511,  1.66732502,  -0.410928041,  1.05192995,  0.409778804,  0.542388439,  0.684218287,  0.0630435944,  -1.83295155,  2.21214771,  0.235012591,  -0.791825116,  -1.02782834,  0.214211375,  -1.08477366,  -0.850133896,  0.219040543,  0.372733146,  0.902663827},{ -1.52912843,  1.03042436,  1.57549191,  -0.45035544,  -0.65531534,  -0.188338816,  -0.22190702,  -1.94012117,  -0.0420219041,  1.28350604,  -1.26639795,  -0.25661388,  -1.01657188,  -2.70498347,  1.4120692,  -1.01595104,  -1.24398255,  0.647756457,  -0.143531635,  1.01085079,  0.910957098,  0.532902062,  -0.73062855,  -1.23962295,  -0.675750017,  -1.57997,  -0.235453025,  0.706771553,  -0.338447154,  -0.334944516,  -0.78459394,  0.869415939,  -0.118449591,  -1.09163809,  0.478423566,  0.101819381,  -0.466566116,  -0.405339837,  0.555674374,  -0.0897396207,  -0.74401623,  1.17250037,  0.244230315,  -1.01675081,  -0.172727346,  -1.32929826,  -0.542838931,  1.57374132,  -0.673969865,  1.57830024,  0.237548172,  1.45573795,  -0.460947901,  0.354733944,  -1.48253155,  -0.382240176,  -0.337165534,  -2.24093485,  -1.34776139,  -0.75870949,  1.11736202,  0.441640824,  0.154174,  1.66041028,  -0.457509518,  -1.0169524,  -0.929504156,  0.694078147,  -0.953802347,  -0.962352872,  0.411531538,  0.485526413,  -1.10504806,  0.1691304,  1.37559915,  -0.536222041,  -0.898373663,  1.56752682},{ -0.656698883,  -0.294530332,  0.590453625,  -0.323488951,  -0.364595056,  -0.741952896,  -0.669623911,  -0.557592034,  2.10779858,  0.169064328,  -0.368934482,  0.561320543,  -1.20721185,  -1.06315029,  1.34646976,  0.819410086,  -0.568909824,  0.0913448855,  0.722038567,  0.107950032,  -0.102520473,  -0.347109973,  -0.88933301,  -0.901557565,  -0.322386295,  0.0517075695,  0.667421401,  -1.17939615,  -0.762704909,  1.52714384,  -0.831851423,  -1.5796988,  0.332854986,  0.585260808,  -0.63575381,  -0.201681331,  -0.39374283,  0.71077466,  0.804120719,  0.0333924517,  1.49591196,  1.25485742,  0.608958662,  1.18369889,  -0.602078617,  -0.132164836,  0.579204559,  0.206902355,  -0.428135663,  0.843629241,  -0.795610726,  0.379049659,  -1.49588895,  0.639184833,  -1.06880832,  -0.227805093,  -0.823414683,  -2.37187219,  0.742703378,  -0.0672147721,  1.94233823,  -0.90364486,  -0.595369339,  -0.941504598,  -1.3243351,  -0.542208672,  0.837261498,  0.517704308,  0.528806329,  -1.70770204,  -0.786551058,  -0.0907933861,  -0.747994602,  -0.390438497,  1.20913947,  -1.18658197,  -1.59418225,  -1.39642692},{ 1.38646936,  -0.987397611,  1.750772,  -0.0772726536,  -0.493799537,  1.88562965,  -0.612517118,  1.50227141,  1.07265973,  0.412612379,  0.0751066804,  1.59914243,  0.871224225,  -0.0994509012,  -1.35434151,  -0.96620208,  -0.501887321,  0.678216636,  1.74887061,  -1.17376602,  -0.536834002,  1.1363225,  -0.132736504,  -0.957483888,  0.416365981,  -0.0667560622,  -0.365226865,  0.924651623,  -1.24207282,  0.0825331956,  0.576894462,  0.662894607,  -2.07672572,  -0.545032859,  -2.5286057,  -1.03382468,  1.28749645,  0.890644193,  -0.172315896,  0.98012346,  -0.280893773,  -0.466895282,  -1.20421195,  -0.514251113,  0.0897180587,  -0.929911256,  1.48967135,  -0.900755107,  -0.150888935,  -0.987501144,  0.403095961,  -0.889797509,  1.31242168,  0.79708302,  0.383785546,  0.642349303,  0.133438557,  -0.7166785,  1.92650533,  -0.443547755,  -1.94612515,  -0.465083957,  0.655844092,  -0.344319433,  -1.00089574,  -0.601138532,  -0.632338047,  1.47580624,  0.745514452,  -1.37245286,  -0.238182768,  -0.415386468,  -0.441685438,  0.136767536,  0.537357688,  0.602267921,  -0.176058307,  -1.04060709},{ -0.15091975,  -0.373274297,  1.26973581,  -0.214231044,  -0.22542353,  0.488892972,  -0.830991268,  0.28039971,  -1.46288967,  0.626510799,  1.16489899,  -0.302871495,  -0.855007231,  0.601685703,  -0.127684221,  0.664889812,  -2.32086229,  -1.38026404,  -1.29597545,  0.786856115,  -0.540872276,  -0.349802434,  -1.43914533,  2.19369674,  1.07775617,  -0.214394763,  -0.26523456,  2.17120957,  -0.709255874,  -1.1664412,  -0.83921349,  -0.150863111,  -0.0561823286,  0.0471425653,  -0.478098065,  0.119925007,  -1.11760592,  -0.908781886,  1.03380394,  0.423088014,  0.618112385,  0.146045953,  1.04471278,  -0.618033826,  1.24828362,  -0.392451912,  -0.682108223,  1.29065835,  0.435926855,  -0.288859785,  1.23000157,  -2.23460793,  -0.190125257,  -0.857513189,  -1.27422452,  -0.891157985,  0.365914792,  -1.05641317,  -0.565454364,  1.23543429,  -1.04449558,  2.6616075,  0.598394215,  -0.548289299,  1.48669803,  -2.15930963,  0.351240426,  -0.241326749,  -1.86352026,  -0.329931885,  -0.436268359,  0.598951817,  -0.433948904,  0.632290125,  -0.393817872,  0.684491098,  0.166564867,  0.00182660879},{ -0.374067903,  -0.170424014,  1.16634715,  0.0917077661,  0.206549436,  -0.222365335,  -1.22906244,  0.928039789,  0.453226537,  0.783334136,  0.783175826,  1.13378,  -0.0777918324,  -0.885064781,  -0.959527731,  1.05246937,  -0.429048389,  1.16143978,  0.93212831,  1.36760604,  1.27856982,  -0.717570245,  0.201883033,  -0.603937685,  -0.546854258,  -1.6963253,  -0.247843787,  -1.70056164,  -1.57649124,  -0.12898919,  -0.156517163,  -0.70188731,  -0.123024091,  -0.901437998,  -0.315283775,  -3.08351946,  1.55906272,  -0.474321991,  0.660179734,  0.565177619,  0.779686093,  -0.530488193,  0.908060193,  -0.332423687,  0.966247797,  0.140976399,  -0.0718736798,  -0.401605576,  -0.872308373,  0.987001359,  -0.513294578,  -0.0636626482,  2.03687549,  0.657524645,  2.69410563,  1.19931209,  -0.0470616929,  0.935730875,  1.06097305,  0.325364351,  -0.811462462,  -0.246258646,  -1.27218425,  0.455585331,  -0.933709264,  1.25491166,  -0.656617701,  1.70829606,  0.814351439,  1.4018892,  -1.7038995,  0.639966369,  0.791595817,  1.07118356,  1.49376976,  1.5442934,  0.946974874,  0.304928124},{ 0.634684443,  1.97733557,  -0.896639705,  -0.0760482326,  1.01299775,  0.167178318,  0.175426558,  0.280161798,  -0.6258986,  -0.0659590214,  -0.542301476,  -1.13386846,  0.324786514,  1.54968357,  0.811576664,  -0.147379696,  -1.13741028,  0.487365633,  -0.344014406,  1.13463175,  -0.757704079,  0.492857516,  0.57410574,  1.11649382,  0.246444434,  0.234676212,  -0.567375302,  -1.34728682,  -0.468175739,  -0.467592686,  1.4409734,  -0.283200115,  -0.0157186799,  -0.0855051428,  0.878718257,  0.486521691,  -2.23253131,  -1.16632247,  0.582371593,  -1.92415118,  -0.411982566,  1.65111113,  1.16227579,  0.391482711,  -0.476880759,  -1.07850349,  0.851984203,  0.362718046,  1.17279637,  0.185107976,  0.652142406,  0.236042485,  1.31626129,  0.876845479,  1.57899892,  -0.865765572,  -1.17414343,  -0.312782198,  -0.526418388,  -0.110827081,  -0.45466733,  -0.46296519,  -0.435046732,  -1.18122554,  0.293469518,  0.0689487308,  0.706027329,  1.2654748,  0.652082026,  -0.60890466,  0.421961248,  0.355163634,  -0.0716452748,  -0.352708757,  0.440062553,  0.917746425,  -0.944058657,  0.82795167},{ -1.15984058,  0.804825962,  -0.263648212,  1.30576289,  -0.865552902,  -1.41461003,  0.237585485,  0.108309478,  0.7594468,  0.749049664,  0.426757872,  0.215228572,  0.626816988,  0.330533653,  0.771838069,  -0.030999558,  0.133829191,  0.341953307,  -0.655903161,  0.594429314,  0.0926519707,  -1.8628732,  -0.0979480147,  1.87863517,  -0.826712787,  -0.226140484,  -0.461042851,  0.801101923,  -0.472792864,  -0.573711276,  -0.365778476,  0.82629782,  -0.0343191363,  1.08991683,  -0.889815092,  2.04278278,  -2.25889063,  0.293293983,  0.874553859,  -1.32370627,  1.48788261,  -0.307163268,  1.87985563,  -0.623802423,  -1.38910389,  1.02413809,  -2.363204,  -0.526136696,  1.68078601,  -0.528641403,  -1.10666299,  -0.99319005,  -1.48495877,  -1.21038032,  0.381308556,  -0.229580671,  0.427860528,  0.511527479,  0.844883502,  -0.706734061,  -0.762284517,  1.73225784,  -0.389646828,  1.04500782,  1.7094655,  -0.573265851,  0.678943634,  1.35721672,  0.236101329,  -0.648240685,  0.303187072,  1.6034708,  -0.0732739717,  1.30676532,  -2.03290033,  -0.486633003,  -0.509271801,  -0.113260634},{ 0.0422701538,  0.9407323,  0.369676769,  1.20652461,  -0.234577462,  0.0669444799,  0.59051913,  1.90154195,  -0.658784568,  0.50499773,  -1.81451261,  -0.00909288228,  -0.0961073861,  -1.48402262,  -0.916357994,  -0.52789253,  1.69219363,  0.169523701,  -0.997720718,  -0.00107852288,  1.67226541,  -0.468714058,  1.09132886,  -0.344429046,  1.14980364,  -0.915473342,  0.240029588,  0.757692754,  1.16475654,  -1.32275414,  0.699047923,  -1.18231344,  -1.41969013,  -2.01349688,  -1.45513868,  -0.292262644,  -0.621664941,  -0.706874788,  -0.421876967,  1.07617843,  -0.640163779,  -1.08776498,  0.291567624,  -0.333364844,  0.978340447,  -0.213221923,  1.66475809,  -1.32784474,  0.44472751,  -1.00533712,  -0.9558599,  1.62054145,  -0.0934108794,  2.64497733,  0.559004724,  0.085812211,  1.04419088,  -0.399535716,  -1.56294513,  -1.05485868,  -0.883354604,  1.36544633,  0.489801943,  1.45158672,  0.214189515,  -0.64397198,  -1.0956651,  0.113906324,  0.183677673,  0.0102463048,  0.717114508,  -1.61882019,  -0.526792586,  0.165134519,  -0.397168279,  -1.13178098,  -0.772823811,  0.411102027},{ 0.53726089,  -0.158027828,  0.863050461,  -0.0605604872,  -1.03620827,  0.539733231,  1.16791439,  0.379982799,  0.185586929,  -0.307710379,  0.0982178077,  -0.978388667,  0.568067312,  -0.941812277,  0.664487779,  -1.72202122,  -0.00382684148,  0.213788435,  -0.522988737,  0.921342373,  0.227599412,  -0.0635528415,  0.0172175467,  -0.8754372,  0.680596769,  1.35844874,  0.380836993,  -2.35772038,  -0.311420143,  -0.879501581,  -0.67033422,  0.134401381,  -0.638260603,  -0.436169088,  0.498675823,  -0.669399977,  0.907252729,  1.01993251,  -1.1852088,  -0.606616318,  1.15655625,  0.422844142,  -1.51350462,  0.430588484,  -0.0847346932,  0.781300306,  2.3732276,  2.01355577,  0.931549191,  -1.72429121,  -0.0917266011,  -1.16398692,  -1.60954058,  -0.744345844,  1.34874773,  -0.0250370782,  -0.809662521,  -0.548368216,  -0.0243300553,  -0.562335372,  -0.0848554298,  0.201478109,  0.0397246592,  1.03731728,  -1.11401331,  0.682154,  1.49912763,  0.0588373132,  0.685639262,  -0.131091505,  -0.563517213,  0.680057943,  -1.33521497,  1.13590586,  -2.60073447,  -0.0247179735,  -1.26739311,  1.30068862},{ 0.11425668,  0.97588259,  0.3082982,  -0.6645872,  0.0183360223,  1.1381731,  0.744313598,  -0.129698545,  -1.5978235,  -1.36879325,  1.22554255,  0.302059323,  0.383497477,  1.21189582,  -0.557081699,  0.517949879,  -0.487045556,  0.849237084,  0.129794106,  -1.72619843,  2.24761653,  0.263955563,  -0.806051373,  0.731945634,  -0.643442571,  -2.97169685,  1.07598519,  1.34316361,  0.426585406,  -0.410679728,  -1.14439964,  -0.319362938,  -0.208871678,  -0.0483204983,  -1.83225083,  0.846987069,  -0.900990069,  0.797626674,  1.14814472,  0.732190847,  -0.290050149,  -0.634686112,  -0.942476153,  0.0963759273,  1.72341585,  -0.555753946,  1.18928671,  -0.253341556,  0.401183158,  1.651034,  -0.491676569,  3.44398472E-06,  1.72253132,  2.15404034,  -1.22838986,  -0.0722012669,  -0.668895125,  0.089165099,  1.69337392,  0.596743882,  -0.0623054914,  -1.24289238,  0.991354167,  -1.63029265,  -1.25032079,  -0.998228908,  0.0567047559,  -0.190075934,  0.200875342,  -0.313512832,  0.429333031,  0.723359585,  -0.701708972,  -0.011704484,  0.11476779,  -2.24693155,  1.29199147,  1.23105502},{ 0.52678287,  -2.60411453,  -0.495972484,  0.933743715,  -0.0277850367,  0.955905795,  -0.134947121,  0.0574171878,  0.711479783,  0.760005713,  0.226943061,  1.91255665,  -0.75388658,  -1.03506541,  0.344084024,  0.152808651,  0.369234115,  -0.34798035,  1.6638751,  0.488140136,  -1.13625324,  0.0566904545,  0.358573645,  0.415799677,  -0.0804686695,  -0.513231814,  -0.310397387,  1.30769706,  1.24249148,  0.784502864,  -0.311824471,  0.512889743,  -1.57749438,  0.587893128,  0.513874114,  0.830402434,  -0.00384112005,  0.477215916,  1.18632305,  -0.939685404,  2.02237535,  0.0523832366,  -0.40494287,  0.191775799,  -0.451462895,  1.51176786,  -1.04941118,  -0.679166913,  -0.96432209,  1.03911293,  -0.814377606,  -0.683068573,  1.66781175,  0.492965162,  -2.61761308,  -0.119729891,  -0.293863744,  1.43308985,  -1.00906932,  -1.13011467,  0.353083491,  -0.295461893,  0.269866616,  -0.1844998,  -0.112715639,  0.0937864855,  1.80988514,  -1.82841897,  -0.452014476,  0.258398175,  1.31106782,  1.07847333,  -0.0751041844,  0.0721781403,  0.984828174,  -0.614229679,  -0.974441946,  0.00114672724},{ -0.729698896,  -1.01622915,  0.739808559,  0.867390573,  0.45439887,  -0.114111744,  -0.0850970745,  -0.674171805,  0.0477069952,  1.27288079,  -0.410219371,  0.356753498,  -0.404861599,  0.448792875,  1.13790739,  0.562437356,  1.02475011,  -1.78121448,  -0.228293538,  1.83699501,  1.29138327,  0.364344984,  0.73158282,  0.488000035,  0.40013814,  -0.517346621,  0.397018075,  -0.179086402,  -0.347774893,  0.183149874,  0.756888747,  -0.12576507,  0.103354618,  0.85677892,  0.731984735,  1.07348979,  -0.250272095,  0.152561903,  0.555453777,  -1.07865107,  -0.238605082,  -0.129520953,  0.861480415,  0.106633402,  -0.660233736,  1.68658662,  0.191598743,  -0.555600882,  -1.71680391,  -0.00192882819,  2.07646036,  -1.42648792,  -0.0638515279,  -1.52810109,  0.268934637,  -1.73199785,  0.409296453,  0.546370745,  -0.499245942,  0.109766282,  0.0841592997,  0.565769374,  -0.0671146736,  -0.404082328,  -0.870077193,  0.632523417,  0.624879122,  -0.761250854,  0.334335089,  -0.539270163,  -0.909935236,  -0.839753151,  1.94253242,  -0.121325307,  0.388635546,  1.02183592,  -0.617892623,  0.610542417},{ 0.786802709,  -1.02169347,  0.111502707,  0.619919062,  0.324284405,  1.28810573,  0.274850339,  1.31348956,  0.294316202,  0.174122229,  0.471300811,  0.0802333131,  1.55357862,  0.393679768,  0.103834264,  -0.0711632371,  1.24727619,  -0.0212557279,  0.582709134,  -1.36723411,  0.277319849,  -0.820207477,  -0.951980174,  -0.146412939,  -0.432768703,  1.29392576,  -0.579932868,  0.0969680399,  0.851342082,  -0.22508277,  -0.476402521,  0.0901347846,  -1.42911601,  -0.686112583,  -0.344507337,  0.67003423,  -0.113500297,  -0.490627736,  1.96979535,  -0.629157782,  -0.738848448,  -2.21317315,  1.0220114,  -0.0122320373,  -0.192519575,  -1.22442913,  0.93497932,  -0.591044962,  -1.03086662,  0.0666783527,  2.50926042,  2.43228388,  1.35623384,  0.394107699,  0.646997571,  0.495161086,  -0.0809979811,  0.855567515,  0.193663001,  -0.049460832,  0.335650563,  -0.140217975,  0.784435451,  -3.00247693,  -1.01268077,  1.44351578,  -0.334033579,  -0.163018152,  -1.1110903,  -1.26017714,  -1.18732488,  1.05961788,  -0.339928776,  0.815911055,  -1.71917033,  0.815488875,  -1.60781717,  0.213298947},{ 2.33299375,  -1.53136241,  -0.961715579,  -0.787388325,  1.14785612,  -1.52610826,  -0.109436758,  0.0956007764,  -0.0103587992,  1.04430401,  0.58374238,  -1.29917467,  0.543923259,  0.192027733,  0.450813383,  0.677440584,  -0.799385071,  -1.1843909,  -0.153798133,  1.07509243,  -0.204456478,  0.845064521,  -0.410102338,  -0.292604268,  1.27106428,  0.0175025314,  1.56887686,  2.29502892,  -0.130336031,  0.380504847,  -1.42044806,  0.477165252,  0.705340147,  -1.16266608,  0.899972558,  -0.293802649,  0.277377218,  0.700980008,  0.709914565,  2.2308743,  0.0262412634,  0.189850703,  -2.29541874,  0.912041545,  -1.74152756,  1.99990535,  -0.0268000234,  -1.44823158,  0.132606626,  0.253312171,  0.77883482,  -0.404147238,  -2.2041986,  0.791142464,  0.210405201,  -0.651893914,  0.333229512,  0.443926752,  -0.175821796,  -1.44105387,  0.956999242,  -0.226484805,  -0.625778615,  0.114379115,  -0.974825621,  -0.988933742,  -0.47352156,  0.525889993,  0.695643067,  1.53678727,  -1.07511091,  1.10167706,  -0.253626913,  1.38465798,  1.42652667,  -0.295441359,  -0.307048559,  -0.657109559},{ 0.28421551,  1.46588624,  -0.927394509,  0.674833775,  -1.75546885,  0.665201724,  -0.481687456,  0.875766635,  0.444527864,  -0.446416944,  -1.74747801,  -2.04611826,  0.574541748,  -0.304395378,  0.109274209,  -0.236533806,  -1.38308609,  0.763692617,  0.468707204,  -1.78514683,  -0.0917676166,  1.25550926,  -0.330514818,  0.390867591,  -0.189737722,  -0.648120224,  -1.75552678,  -0.687196374,  -0.519253373,  -0.320650697,  -0.325847894,  -0.719573438,  -0.962644577,  -0.122867674,  -0.0247938987,  -0.444044858,  1.28841197,  -0.695631504,  0.105739281,  0.314276308,  0.39938134,  -0.0611405186,  0.650740087,  -0.73854816,  -0.31987378,  0.381262124,  -0.438434601,  0.918419838,  -0.652538061,  0.189271063,  0.871303618,  -1.46173358,  0.41769439,  -0.350251317,  0.943181157,  0.824294567,  -0.247892186,  0.820334852,  0.324778646,  0.962482274,  0.337422252,  0.376573652,  -1.08385742,  1.46655035,  -0.795548618,  0.226869553,  -0.934375763,  0.524060726,  -0.421896636,  0.757456958,  -0.355588734,  -0.274634808,  1.61253631,  0.770610213,  0.100743555,  -0.654512823,  0.115923144,  0.245954752},{ -0.870932996,  0.842850506,  0.263522983,  -0.381201923,  -0.885541975,  0.267942399,  0.308609724,  -0.273243159,  0.272898704,  -0.49097386,  0.436618328,  1.08061516,  -1.06861305,  0.388375968,  -0.0421343036,  0.441470504,  -1.1815151,  0.650128901,  1.8080008,  -0.37682125,  -0.782903492,  -2.33379555,  0.844133615,  1.25891984,  -0.0310568064,  0.971364677,  -0.36021179,  -1.40161812,  0.608014464,  -2.14179492,  2.88599992,  0.950030684,  -1.6314503,  0.559015453,  -0.679113805,  -0.749720275,  -1.76829207,  -0.504730582,  0.305678308,  -1.67350841,  -0.337447464,  0.685634017,  -0.744191468,  -0.853419721,  -0.111969024,  0.584293783,  -0.597182631,  0.583241105,  -0.358327329,  1.08068812,  -0.94634676,  0.905031562,  0.210171685,  -1.3203342,  -1.54758954,  0.224740773,  0.774029732,  0.041384887,  -0.343299031,  -0.27804181,  1.01652443,  -1.59502542,  -0.119672969,  -0.016435517,  3.41287065,  -0.713282943,  1.28788602,  0.131579801,  -0.0149909668,  -1.5208838,  -2.66394091,  -1.27438581,  0.296183318,  -1.3765955,  -1.26414084,  0.109738879,  0.352537274,  0.831889331},{ -1.13195825,  -0.40775609,  0.461240321,  -0.954839349,  -1.49492311,  -1.08651841,  0.520434082,  -0.0599742085,  -0.548234642,  0.827060759,  0.909589231,  0.0902573913,  -1.68380082,  -1.85173476,  -0.21488601,  -0.654173791,  -1.07892406,  -0.236364752,  -0.429978162,  0.807748318,  -0.103093043,  -0.177763447,  0.221904904,  -1.30236447,  -1.34050906,  -0.929976702,  0.000748411345,  0.228863627,  -1.06622899,  0.840085089,  -0.931709647,  0.62466234,  -0.892747641,  0.652932405,  -0.0565694794,  -2.1199615,  -0.824342608,  0.928510308,  0.407722354,  -1.64867926,  0.996118665,  1.0004425,  0.651875198,  -2.00409245,  -0.567330301,  -0.603299558,  0.178371444,  0.83553952,  -1.17718065,  -0.431837589,  0.124841705,  -0.0982636288,  -1.36505651,  -0.595716059,  -1.26406944,  -0.665346682,  1.63033521,  0.497967988,  0.588970542,  -1.17702556,  -0.0726090744,  -2.26791167,  -0.893121779,  0.43950212,  0.065796867,  1.2687633,  2.141747,  0.637025356,  0.110316522,  -1.64733326,  2.16527319,  -0.153586015,  0.384511173,  -0.760378957,  0.667281091,  -0.475316316,  0.231628448,  0.690015554},{ 2.11377096,  0.734473109,  -0.130000189,  1.27295363,  1.59129238,  1.42237914,  1.76798713,  -0.698895097,  0.441848755,  0.0829893723,  0.872972965,  -0.107620567,  -0.718527615,  2.36786461,  0.70509088,  0.665139616,  0.140150011,  -0.824447513,  0.996302247,  1.46127665,  0.682396054,  1.16261125,  0.797057331,  -0.80171454,  0.139226943,  0.169716433,  -0.097581163,  -0.0814738423,  -1.18595338,  -1.91073859,  -0.0177724306,  0.433796585,  -0.406325519,  0.722628355,  -0.445076197,  -0.728668034,  -0.102381721,  1.82659972,  0.546566546,  -0.481775224,  0.453953147,  -0.796054602,  -2.21083999,  -0.566528082,  1.66151929,  -1.28828239,  1.64860344,  0.922675729,  0.375836492,  -1.83339667,  1.37413359,  0.311428308,  0.3040995,  1.86366355,  -0.195866704,  0.172997013,  0.210587859,  -0.641661882,  0.553540409,  -0.186585888,  1.16083455,  -0.0670649111,  0.263198048,  0.932486713,  -0.0394025445,  1.4032588,  -0.117956996,  1.29108417,  -0.778132975,  1.3795861,  -0.476687759,  0.0186505802,  -0.0887412205,  1.09793282,  -1.84034455,  -0.238269866,  0.757604897,  -0.905441344},{ -0.308941424,  0.954833567,  -0.145454094,  0.788852632,  -1.18417335,  -1.11743486,  0.602849305,  0.138184071,  -1.60813498,  -0.398747504,  2.18194914,  -0.470270634,  -1.2260251,  0.354751408,  -0.0433810987,  -0.386585146,  1.35321009,  -0.479003757,  0.566129446,  -0.479611397,  -0.434825242,  -0.138856947,  1.3900491,  -0.480997175,  -0.80551511,  -0.668424666,  -1.45178616,  -0.560063541,  -1.08621144,  0.159967676,  1.31186795,  1.51420832,  0.497204542,  -0.0237390921,  0.135148674,  0.0517467111,  0.212965533,  0.00669283792,  -0.980138779,  2.08060932,  0.559905767,  -0.441568553,  0.691993892,  -1.01386285,  0.37969622,  0.617435873,  0.868009388,  -0.712129116,  -0.435690314,  0.431128651,  0.572204828,  0.0783507749,  1.09981489,  -1.31525135,  0.45886597,  -1.10836124,  0.77704066,  0.282607973,  0.397618234,  -1.67712355,  0.263127655,  0.0264838897,  -1.68367088,  0.287706733,  -0.114895366,  0.459133059,  2.1646893,  1.02847421,  1.5448215,  1.41238499,  0.44715476,  0.660731912,  0.484270096,  -1.04810715,  1.30010653,  -1.59412336,  -1.642851,  -0.673381388},{ -0.0963850915,  0.176083252,  -0.00561596546,  0.160758674,  0.948952734,  0.652328789,  -1.30271518,  0.943484604,  -0.801558733,  -0.00781793706,  1.12478244,  -0.032921344,  1.6329788,  -0.150177732,  -0.894637704,  0.121339343,  0.750150204,  -1.20540261,  -0.230175227,  0.69689554,  -0.486714542,  -0.502152741,  0.265343189,  1.45709848,  -0.410702109,  1.44092846,  0.904420316,  1.32738197,  0.0180819836,  -1.28631318,  1.21723485,  -0.676282465,  -0.515982985,  0.878109753,  -1.88374531,  -0.699352443,  0.709319174,  0.225881934,  -0.398709863,  0.676182747,  -0.756783783,  0.272143424,  0.500418782,  0.661440194,  0.304474324,  -0.485046864,  0.165718943,  0.347511113,  -0.0770697817,  1.15239501,  -0.976110339,  0.0302561633,  0.113921881,  0.219330251,  1.06522548,  1.7153163,  0.252283871,  1.11997938,  -0.724149108,  -0.770779788,  -2.40508723,  0.355863184,  -0.278775036,  -1.95274949,  1.36996627,  0.71482724,  -0.361204982,  0.200277671,  0.111359328,  -1.28705573,  1.05350995,  0.497597605,  1.64754951,  -1.33323705,  0.290884197,  0.669274211,  2.21481633,  -0.221483842},{ -0.319528371,  -0.283693492,  1.08819127,  0.736612618,  0.0565943755,  0.738191426,  -1.50785279,  -1.51352823,  -0.709648907,  0.183142483,  -0.232645854,  -0.269962251,  0.955335319,  0.145316169,  -1.68216074,  1.41178215,  -1.64145398,  -0.524946272,  0.86112231,  0.690483212,  0.721476495,  -1.78624284,  -0.326842517,  -0.0462207682,  0.0640416369,  0.214384049,  0.0741178989,  -0.110063888,  -0.526001334,  0.849195242,  0.149698377,  -0.426241606,  1.11374998,  1.27478182,  0.856624722,  1.01350415,  0.790674984,  0.578414619,  -0.119429849,  -0.61924547,  -0.959312975,  -0.347292393,  -0.311253846,  0.903270125,  -0.198271424,  1.2247287,  1.32627511,  0.720102549,  0.838761508,  0.330416769,  1.10074973,  -0.347396821,  -1.8750155,  -0.0470285751,  0.279112905,  0.851130366,  -0.773671567,  0.132287323,  1.02058399,  -1.38061476,  -1.1072799,  1.36717367,  1.77803993,  -0.867955804,  2.41302776,  -1.74215674,  -1.84259105,  -0.155329019,  -0.742825627,  0.313955277,  -0.674922228,  0.228050843,  -2.42465925,  -1.75966752,  0.5076527,  0.828862727,  -2.49587011,  -0.677046895},{ -1.21891451,  -2.07382345,  0.31626907,  1.27476358,  -1.07352078,  -0.965702355,  1.69928026,  -0.279756039,  -1.11132753,  -0.978165507,  -0.595101893,  1.35886669,  -1.32709348,  -0.778169632,  0.951258004,  -0.228481159,  -0.477481157,  0.763249755,  1.39757955,  0.303774714,  -0.334735274,  -0.849175572,  0.340746909,  0.423859537,  -0.708302021,  0.106291637,  -1.63439548,  -1.62766314,  -1.76674879,  -0.492621779,  -1.27682316,  -1.39487231,  0.200314954,  1.42076612,  -0.0987268463,  0.806245208,  -1.45862544,  0.898787022,  0.419899583,  0.134646788,  0.315910429,  -0.152295619,  -1.50688243,  0.9075948,  0.331891477,  0.53333056,  0.677639484,  0.197223663,  0.0374615826,  0.996192634,  -1.08623338,  0.380105764,  0.435131699,  0.839805126,  1.47166991,  1.07773304,  0.243145198,  -0.517128646,  0.830042839,  1.06681824,  0.0554330721,  0.279618829,  -1.21870196,  -1.83456182,  -0.399965495,  2.94444537,  0.301756203,  -1.80649292,  -0.186848581,  1.86092114,  -0.574965477,  0.533718526,  -0.029861141,  0.31703341,  -1.12270594,  1.78479004,  0.407589704,  0.235390678},{ 0.762190044,  1.25890648,  -0.032948371,  -0.611019194,  -1.14408445,  -1.63933265,  -0.549803197,  -0.709477663,  0.936755717,  1.77578676,  -0.613876224,  0.263511419,  1.13447809,  -0.891508579,  0.146451294,  -0.0931810439,  1.51019907,  1.45331454,  -0.289139301,  -2.89327884,  0.280760288,  0.200317457,  -0.113725685,  -0.375101894,  1.23875916,  -2.09504986,  0.128783062,  -0.861638784,  -1.43184102,  0.359595567,  -0.701214254,  0.769273758,  -1.64217341,  0.440971941,  -0.840638578,  -0.308534712,  0.875965357,  -0.816204488,  0.0830523521,  -0.0381319709,  1.11339283,  1.18110859,  0.647772849,  0.139190391,  1.03324497,  0.644818366,  0.590655148,  0.482566029,  -0.286417425,  -1.92000222,  -0.398553789,  0.544130087,  1.25660586,  -1.2827903,  0.765462935,  -2.12608743,  -1.96643972,  -0.0687976778,  -0.132202744,  1.52572393,  1.797539,  -2.01309347,  0.461386442,  2.62321877,  -0.398631781,  -0.13735424,  1.5344348,  -0.773022354,  0.399099648,  0.150887281,  1.19972634,  -3.15596104,  1.05899167,  -0.0228403099,  -0.97411108,  0.356559008,  0.36371848,  -0.71217221},{ 0.99713856,  -0.458703846,  -0.769915938,  1.00198197,  -0.12409436,  0.905740798,  -2.13389206,  -1.54662013,  0.0928451717,  -0.348950744,  0.264502257,  1.08487523,  -0.829256415,  2.04528117,  -1.10530543,  0.479388595,  0.277204216,  -0.371383011,  -2.4813242,  0.612206638,  -0.0715019032,  -0.200037614,  -2.89435291,  -1.13154364,  -1.26095974,  -0.259441465,  -0.445701808,  0.439694852,  -1.60648286,  -0.422641158,  -0.832937002,  0.329378188,  -0.0159720369,  -0.375780225,  -0.781942666,  -0.1650047,  -1.33628511,  0.0852635577,  -1.81772804,  -0.406983942,  1.28996933,  -1.01561606,  0.752163887,  0.686042249,  -0.414274782,  0.0715015382,  0.248106137,  2.02532578,  -0.139437139,  -0.510483503,  0.204735294,  1.70861351,  -1.08950603,  -1.06741548,  0.08309149,  0.241788805,  -0.55133611,  -1.73153305,  -0.776057184,  1.16951609,  0.57244885,  -1.82965171,  -0.558595896,  -0.891693234,  0.0990534648,  -0.579380333,  -1.65590346,  0.141839787,  -0.627621531,  -0.811416745,  -1.38761914,  1.07206488,  0.494980097,  0.524119258,  0.427149683,  1.5259217,  -0.766691923,  -0.796817899},{ -0.759486079,  -0.0247663949,  1.07740855,  1.10920441,  -1.92047405,  -0.586636007,  -0.965265036,  -0.0759183541,  -0.234554276,  -0.55194366,  -0.813355029,  -0.299069852,  0.176403329,  1.23115289,  0.0107065849,  0.763100505,  0.776173949,  -0.453365684,  -1.00284123,  0.546225011,  -0.777439177,  -0.118949026,  -2.70997739,  2.04006743,  -0.281385779,  0.13743934,  -1.79664266,  0.477555633,  -0.0266691372,  0.736256659,  1.3526808,  0.263004273,  0.430488259,  -0.786477268,  -0.801687896,  -1.28414047,  -1.60034835,  -1.17414117,  0.606600404,  0.665340722,  -2.51471376,  -0.522574008,  -0.696535051,  0.908947885,  -0.0189156365,  0.206599325,  -0.0590903573,  -0.459669262,  -0.495834619,  1.81502664,  -0.214713573,  -0.133700147,  0.0572013445,  -0.286492944,  -1.05654073,  0.666870236,  1.50790608,  0.325002909,  2.1123271,  -0.236091748,  0.108422875,  0.205223382,  0.0243803989,  -1.04048789,  2.00776744,  -1.13277686,  -1.12629807,  1.37580168,  -0.646527469,  -1.58303928,  -0.523670495,  -0.385600924,  -1.62246287,  0.633498907,  0.686046362,  0.0586826168,  0.775301218,  -0.28611666},{ -1.70398283,  0.889712036,  -0.537956357,  -1.94968677,  0.121471867,  0.45175916,  -0.141208529,  0.32479918,  1.22689497,  -1.60480881,  -0.0543922223,  -1.0767318,  -0.494868964,  1.4546777,  0.70209235,  1.22452629,  -1.71448457,  -2.11436963,  -0.353981853,  -0.757984161,  -0.399272442,  0.372284532,  -0.0150025822,  0.267327458,  -0.325240105,  1.03093028,  -1.71484768,  -0.112755045,  0.209570557,  -1.05722749,  0.0746519566,  -0.413808823,  -0.315048754,  -2.10589671,  -0.322767466,  -0.0915465355,  -1.79937673,  -1.5642494,  1.46020019,  0.190057382,  -1.46190989,  -1.28672671,  -0.198934764,  1.06030548,  0.358788878,  1.22466671,  -1.43032038,  -1.52689421,  -1.60670316,  1.59862494,  -2.51290083,  0.352252513,  0.754156291,  0.467444628,  -0.0371478721,  -0.674921215,  0.205850855,  -1.7957412,  0.466207772,  -2.16105485,  1.53717887,  0.0137265259,  -0.00970971212,  0.615606487,  -0.209286183,  -2.09378052,  0.208904162,  -0.266167074,  -1.1018945,  -1.84055078,  0.765367508,  -0.945770502,  1.0784713,  -0.681769431,  -0.144309416,  -1.18910277,  -0.0779759288,  -1.05524743},{ 0.0112432567,  1.97560287,  -1.51933181,  -0.80589658,  0.412399799,  -1.49277687,  0.361423612,  -0.983467221,  -0.65925014,  0.0370188691,  -0.105638288,  -0.621752501,  -0.750278056,  -1.35271823,  -1.37983084,  0.971856296,  0.377346903,  0.419831365,  -1.65574753,  1.77980709,  -1.08914077,  -0.20086877,  -0.204711109,  0.246358529,  1.18550885,  0.774119496,  -0.613664448,  0.395417273,  2.16574359,  -0.921176374,  -0.155759901,  -0.583198726,  -0.262477309,  -0.155190289,  -0.528764486,  1.0638876,  0.935182512,  0.654985785,  0.235290334,  -1.65743446,  -0.989554882,  0.160089657,  2.13566303,  -0.149941534,  -1.27531886,  0.978530705,  1.41694272,  0.459670186,  -0.508753777,  1.63280082,  -2.7982831,  1.40291905,  -1.07359278,  1.32548499,  0.854194582,  -1.45378733,  -0.807171047,  0.0875632837,  -1.63219059,  0.633768976,  1.4672178,  -0.423352003,  1.98291349,  -0.476857275,  -1.37040424,  1.16255414,  -0.172213092,  1.4492234,  0.448966235,  0.667891085,  0.22968854,  -1.33692539,  -1.10408354,  0.681905568,  0.399754286,  2.69002891,  0.50950104,  0.424698979},{ 0.743088186,  1.33261096,  1.85850835,  -0.786539555,  1.20614445,  0.694797993,  -1.07209682,  -0.31424731,  0.903587162,  1.91978979,  0.615729213,  -1.34269929,  -1.03753483,  -0.358990312,  -0.951548278,  0.312124848,  0.71530211,  0.44996345,  -0.519664884,  -1.2357446,  -0.634777546,  -0.957265854,  0.141516671,  0.124938078,  0.497269064,  -0.363959849,  -0.998894811,  -0.616320848,  -0.133713275,  0.632054031,  -0.821495771,  -0.396753132,  -0.642668068,  0.996487796,  -0.941922128,  -0.503331065,  0.209099069,  0.484350622,  0.426143914,  1.6459893,  -1.2214793,  -0.00385685498,  -0.129136905,  0.243042186,  -0.570318282,  -0.969702721,  -0.774793804,  -0.207958221,  0.81936878,  -0.254801929,  -1.46136236,  -0.00550371502,  2.0293448,  1.09323692,  -0.200593516,  0.719241858,  0.559602976,  -0.5080387,  0.768605292,  -0.907242477,  -0.953726709,  0.0766419992,  1.62817478,  -1.91007507,  0.550051391,  -0.595641077,  0.196914181,  1.282516,  0.725599229,  0.485597134,  -0.320396155,  1.58155966,  1.04559064,  0.495889455,  -1.63216245,  0.80506134,  0.456471384,  0.833916187},{ -1.1526202,  0.071309261,  -0.0427622721,  1.10912752,  -0.278786987,  2.03885722,  -0.635150075,  0.780539572,  -0.230355471,  -0.733893096,  -0.337066889,  -0.932185411,  -0.20322454,  0.781353772,  -2.5468595,  -1.50532103,  -0.58588773,  -1.28597319,  -1.77463257,  -1.33491397,  1.52022266,  0.68038553,  -0.389320761,  0.392404377,  -0.201643616,  -0.477741927,  0.365566075,  -1.4850595,  -0.00349733024,  1.60476851,  0.400007993,  0.915238857,  -1.27551758,  1.41841233,  1.61776292,  0.550821424,  -0.381547362,  0.125142977,  0.22376664,  0.666108131,  2.57689881,  -0.321323901,  -0.273568779,  -1.80322194,  1.34013176,  1.08912706,  -1.20756364,  -0.784237921,  0.670638204,  0.835907459,  -0.624539495,  -0.567427993,  0.033946801,  1.15875483,  1.08534729,  -0.960796714,  -0.0334065966,  0.5886181,  -0.399930209,  -0.676980376,  0.822286844,  -0.399423599,  -0.52203393,  -0.119964249,  1.02597833,  -0.0635866225,  -0.233848497,  0.284131885,  -0.348229051,  0.417835683,  0.844424725,  -1.11583745,  0.913853586,  0.929932773,  -0.0110265184,  -0.856701314,  0.332811952,  0.322685152},{ 0.438448131,  0.466522902,  -0.472127587,  1.9916054,  -1.16203487,  1.15438461,  -1.21503437,  0.487478614,  -0.335580677,  0.410479158,  1.48321879,  -0.565073669,  -1.53731108,  0.663993835,  -1.38441658,  -0.16431351,  0.377405286,  -0.692315161,  -0.0165350195,  0.508224487,  1.28169346,  -0.867820859,  -0.0877133608,  -0.287434071,  -1.21512997,  -0.862352788,  -0.184544921,  -0.813002586,  2.23554277,  0.486973941,  -0.251919806,  -0.15589048,  -0.157717943,  0.526285231,  1.65245259,  2.37210917,  -0.854097188,  1.28836274,  -1.19001341,  0.466002733,  0.349800706,  0.159451753,  -0.88613987,  0.837694407,  0.356506616,  -0.803379536,  0.42619288,  -0.481838644,  0.330108523,  0.892176867,  0.709511042,  -1.12505901,  -2.86165857,  0.114231542,  1.37805319,  0.162965998,  0.949832439,  -0.61534214,  0.976508319,  1.53803325,  -1.09296405,  0.187394485,  -1.641819,  0.031380143,  -0.158490732,  1.00499761,  -0.240904421,  0.382848889,  -0.530180454,  -0.741107523,  2.42474484,  1.07357013,  0.183706,  0.435940504,  -0.414925396,  0.306054503,  -0.482545346,  2.2461164},{ 0.647942364,  0.00237075263,  0.885311186,  -1.55316389,  0.99672842,  -0.825869799,  0.259569764,  0.306612819,  -0.895531952,  -1.61448264,  -0.495499164,  -1.09054935,  0.78345412,  -0.417718917,  -1.05961013,  -0.149539977,  -0.515838802,  -2.08740878,  0.54953289,  -0.828828096,  -1.32283974,  -0.558378458,  -2.17765927,  0.879703641,  -0.149005368,  2.51726866,  -0.404851109,  -0.424765855,  0.37696448,  1.41394424,  -2.33326292,  -0.306077302,  0.849154651,  -0.137589499,  2.33791304,  1.83014822,  0.447537363,  1.24012017,  -0.228806317,  -1.28158712,  -1.02795076,  0.455751449,  1.73641765,  -0.889311314,  0.0641316846,  0.643666983,  -0.485981494,  -1.94715893,  0.467555225,  0.913301468,  -0.0531155318,  -0.103616334,  -1.46878755,  -0.0293869358,  2.09223413,  0.292100966,  0.474051744,  -0.123140626,  2.64641809,  0.283768147,  -0.357873261,  0.210851848,  0.744129717,  0.391454488,  -0.0127935559,  -1.45371175,  0.291635752,  1.67521894,  -1.31267273,  1.06243789,  1.75093305,  -1.15680766,  0.605341077,  -1.39461029,  -0.874856651,  -0.121899284,  1.14197969,  -0.00294035161},{ 0.436285317,  -0.0674489737,  -0.386247694,  1.33465803,  1.26402271,  -0.139454469,  1.03720534,  1.36705661,  0.8897928,  -0.0406180173,  -1.95954299,  0.169057146,  1.06884897,  -0.279853642,  -0.448470592,  -1.16988111,  -1.80930126,  1.17673826,  2.0297997,  1.81215537,  0.376427591,  1.83287442,  -0.485803306,  -1.7557236,  0.834605753,  1.39985025,  -0.546514988,  -0.917360067,  -0.397555858,  -0.430373639,  1.50430429,  -0.766236305,  0.842201233,  -1.318645,  -0.126801908,  -0.590297818,  -1.002859,  -0.739386082,  -1.14832544,  -0.0139184641,  0.39322111,  0.607236922,  -1.81561482,  1.19735503,  0.148133472,  -0.544726372,  0.292342782,  0.763754189,  0.103780702,  -0.736237168,  -0.0248881448,  -0.111352816,  1.17736781,  -0.440705806,  0.220724627,  0.57640481,  0.534399748,  -2.07738328,  -0.744916677,  -0.461067468,  -0.111682676,  -0.345754683,  0.0734690949,  -1.3108077,  -0.367093116,  -1.21344829,  0.502372444,  1.29735851,  -0.0267859027,  -0.290137082,  -0.767203808,  -0.366697937,  0.883446753,  0.329401702,  2.10681129,  1.38105607,  1.01632154,  1.24682653},{ 0.350125194,  -0.675412118,  1.18710911,  -0.381134957,  -0.269947439,  0.478502393,  -0.389174283,  -0.368145734,  -0.469370991,  -1.43339396,  -0.892728508,  0.298538327,  -0.161996022,  -0.777900934,  0.474981636,  -1.27624404,  -1.60302711,  1.28433657,  -0.160635829,  -1.02557576,  -0.111522309,  1.10954964,  -1.14532089,  0.253095597,  -1.78093326,  -0.316993684,  0.0790923983,  -1.47075641,  -2.08737683,  0.714463294,  -0.012963308,  0.649449706,  -0.358008623,  0.369857013,  0.19383952,  -0.962410808,  -0.29501763,  0.225355983,  -0.390465766,  2.19418311,  1.90114033,  -0.936689794,  -0.264161378,  -0.705463529,  -1.09289479,  0.741945386,  0.0937538594,  1.35400724,  -1.16105914,  0.724012971,  1.41374362,  -0.769526899,  1.62862062,  1.05194247,  1.30456889,  -0.180113122,  -1.38691616,  0.224650249,  1.7137593,  0.267671853,  -0.903342485,  0.273795843,  -0.804598033,  0.0549604967,  0.96081543,  -1.13616705,  -0.106423736,  -1.46953595,  0.0233820919,  0.355362415,  0.171864033,  0.596202016,  0.867260039,  -1.53256512,  1.55533755,  1.97572458,  2.01070642,  0.0202239808},{ 2.35798836,  0.577801704,  0.8672086,  1.470963,  -0.230924636,  1.20701158,  -0.568035424,  1.50273681,  0.0575195774,  -1.34007561,  0.650201738,  -0.828623354,  -1.08423376,  1.5547967,  0.548885465,  0.015787825,  0.898107946,  -0.606003523,  1.59279716,  1.51780486,  0.876033962,  0.0898859724,  -0.548785567,  -0.71411705,  0.705846131,  0.572048485,  -0.357575655,  -0.491372228,  0.7716676,  -0.718524635,  -1.53749442,  0.492915273,  -2.13426256,  0.284684628,  0.425583392,  1.55113482,  0.675712168,  0.694188774,  -1.69405746,  0.477001816,  0.168308467,  0.380178601,  -0.129597232,  0.957263708,  -0.435488731,  -1.12366784,  1.02822828,  0.276736826,  -0.188381165,  1.43875921,  -0.290585488,  0.590836108,  0.368768156,  1.04850721,  -2.43131995,  -0.16323176,  0.290905893,  -0.367515236,  1.20392537,  -0.966565907,  0.996908188,  -2.12118196,  -0.57167083,  0.407171041,  -0.625631094,  -0.399713963,  -0.184709921,  1.14696038,  0.410938621,  0.385324776,  2.2347846,  -0.840364516,  0.636586368,  0.239606306,  0.996210098,  1.0968461,  -0.480015039,  -1.42366934},{ 0.846881926,  -1.07498479,  -0.673468351,  -0.430464894,  1.62644863,  0.261302292,  -0.115600854,  0.974151373,  -1.47683132,  1.41943669,  -0.485701919,  -2.14933372,  0.898765445,  -1.26219428,  -0.0927504525,  2.23073983,  -0.743923545,  -0.860190749,  0.371262938,  0.936763465,  -2.10175276,  1.02209687,  0.319585383,  -0.819091737,  -0.738014638,  -0.812026799,  1.01308143,  -0.579288781,  1.93051147,  0.741598725,  -2.69164467,  0.732673168,  -0.359879434,  -1.26686704,  -0.193669319,  0.328700006,  0.970268607,  -0.293688565,  -0.0972791612,  -1.20338428,  0.971574485,  -1.52158594,  0.628956735,  -1.12326825,  0.792226434,  1.15938103,  -1.04725301,  0.0880970806,  -1.54980218,  -0.238869041,  -0.732423604,  0.130439028,  -2.10287142,  0.756214261,  0.286860853,  -0.314983875,  -2.7720654,  0.171868116,  1.03413701,  -0.810119331,  -0.234044507,  0.253792137,  0.81615144,  -0.865598023,  0.283758253,  2.12823176,  1.8983016,  0.112267427,  -0.187633589,  -1.70917261,  0.533205211,  -0.0297386944,  -1.44138896,  0.30076912,  0.166716248,  -0.299962789,  -1.18987215,  0.0665336102},{ -0.3646954,  0.468633413,  -0.0309683774,  -2.58759952,  0.517103851,  -0.732922137,  0.289280087,  0.138315767,  -0.152025729,  -0.127659738,  1.09659994,  0.212639958,  -0.969078302,  0.335176915,  -0.787568152,  0.601110637,  -0.633881152,  0.704543293,  1.71590483,  -1.82889795,  -0.539138496,  2.06409979,  -0.483440518,  -0.380210608,  -1.60794067,  1.21142316,  -0.556990147,  -1.0522033,  -0.41285038,  2.02834725,  0.286804438,  -1.6500355,  -0.729779899,  -0.123232745,  0.453175515,  -1.12109149,  0.22124657,  -0.910039306,  -0.530035734,  -0.203524098,  0.396070033,  0.784088135,  -0.909953475,  0.29710415,  -0.518726408,  0.252322435,  1.3745352,  -0.485089093,  -0.00925094262,  -1.0085634,  0.0461851656,  -0.0228496417,  -0.426586062,  0.15098539,  0.848466754,  -1.61141825,  0.222060665,  -0.827090323,  -0.802600026,  -0.285914898,  0.375350147,  1.1810292,  1.30193877,  1.00545025,  0.106449232,  0.0163086765,  1.42604554,  -0.839748025,  0.376314759,  -1.10889959,  -1.34464288,  -0.454774171,  -1.10330689,  0.644163549,  -0.536775172,  0.0919435769,  1.53319347,  -0.759756327},{ 1.53926611,  -2.06418896,  -1.2337358,  1.77833378,  1.84278476,  -0.750762165,  0.649456382,  0.329264849,  -0.748753905,  0.103570901,  -1.63514972,  -0.885523558,  -0.505018592,  2.18575358,  0.0282314997,  0.188728288,  -1.12684262,  -1.25828123,  0.324562132,  0.670929849,  -0.715271294,  0.0539479516,  -0.0248195007,  -0.951850474,  -1.19293797,  0.434784085,  -0.0415346399,  1.2378974,  0.560384154,  -0.334258527,  0.503351986,  0.826692343,  -1.95751071,  0.598226368,  -0.908803821,  0.441793442,  -0.827902615,  -0.934279382,  -1.02160931,  0.546856403,  0.324717045,  -0.2420353,  0.480884075,  0.138674513,  0.261173785,  -1.61381614,  0.0477324575,  -1.05972123,  0.301934481,  -0.636043906,  0.149367183,  -1.25827253,  0.950415254,  -1.48802376,  0.987484455,  -0.818217337,  -0.709796429,  -0.0933037847,  0.238153294,  -1.90189481,  1.50130939,  0.20345448,  -0.553723037,  -0.666354239,  -1.82168519,  -0.03653007,  0.0618414469,  -0.641914368,  0.599798143,  -0.0690326914,  -1.75405633,  -0.332714438,  0.236835733,  1.17709827,  -0.31032601,  0.0371646583,  -0.506063879,  -1.21601546},{ 0.535291314,  -0.947864771,  0.112719059,  -0.356733948,  0.0450132936,  1.21952879,  -1.71408546,  2.32217908,  1.14140522,  1.17976534,  -0.270026952,  0.185402751,  -0.244449928,  0.90236938,  1.31643724,  0.1440323,  -0.119688496,  0.105711766,  -0.0740474984,  0.42635864,  -0.211530551,  0.882281959,  -0.963077664,  -0.575523138,  -1.42546642,  0.00990807917,  -3.17500734,  0.373460919,  0.363819569,  0.999038696,  0.432101518,  0.00809397921,  0.399404317,  -0.486014694,  -1.17154181,  -1.06387603,  -0.312932879,  -0.797683001,  -0.667936206,  1.26231909,  -0.143744141,  -1.26848054,  -0.664400697,  -0.895263612,  -1.14519501,  1.02753687,  -0.906023026,  1.77022886,  1.50634611,  3.19883704,  1.72817338,  -0.905291557,  0.948805869,  -1.92030513,  -1.17759454,  -0.366486251,  -1.06463611,  -0.309154361,  -0.137616619,  -0.146972626,  -1.30442154,  0.456388175,  -0.656467974,  1.34046924,  0.0304620173,  -0.266466737,  -2.07084107,  -0.288530558,  0.0286496338,  -0.50938946,  -1.1885519,  -0.843173683,  -1.04260433,  2.01040721,  -1.14381456,  -1.6004945,  -0.344016641,  -0.0407637656},{ -0.155189157,  0.050682921,  -1.22199881,  -1.05419588,  -0.305974185,  0.615124285,  0.515465677,  0.336128712,  -1.18712902,  -0.5677163,  -0.174019128,  -0.0902558342,  0.920312941,  -0.83010149,  -1.69821846,  0.918527126,  0.814347386,  -0.402222931,  -0.208713964,  -1.21666384,  -0.943169296,  -0.379856884,  -0.127085283,  0.661470234,  -0.714673221,  0.265705556,  0.839416623,  0.0570354015,  1.74265075,  -1.88548541,  0.00757341646,  -0.0157050285,  -1.12057626,  0.736495435,  -0.499351382,  -0.919245183,  -1.4551667,  0.768082857,  0.441110343,  1.89505053,  0.992020488,  0.668141305,  1.3730942,  0.344246864,  1.44032562,  0.463212818,  -0.331780612,  0.804393232,  -1.16695583,  -0.218140185,  -1.80312812,  -1.28497708,  0.144147769,  -0.597952604,  1.25051606,  -1.40794313,  0.449771315,  0.462094933,  -0.451081067,  -0.673929453,  -1.94040465,  -0.574045479,  -1.19307995,  -0.6861251,  0.610648394,  -0.381012768,  -0.32183221,  -0.848178267,  -2.43794012,  -0.745040834,  -0.782125354,  0.157665148,  -0.583191335,  -0.667782962,  0.529179752,  -1.40359366,  1.55831659,  0.456185758},{ -0.698095798,  -0.0440362059,  1.62250555,  -0.669053853,  0.523049295,  0.156598985,  1.07529056,  0.400527418,  -0.0235550683,  -0.590557396,  0.408444464,  1.15248382,  -1.66744649,  -0.913969457,  0.996572793,  0.39487341,  -0.656643689,  1.60888731,  -0.553708971,  -0.901331723,  0.0619852543,  -0.221841663,  0.129273131,  1.26376784,  -0.16679284,  0.395382732,  -0.0642501786,  0.694179058,  -0.225693256,  -1.83406401,  1.10105729,  1.00421298,  1.16517019,  -1.02690911,  -0.497036487,  0.446522713,  -2.3930254,  1.24514198,  -0.301279098,  0.00216751383,  -0.5457201,  -1.0938015,  -0.0892174914,  -0.96275574,  -0.524752915,  1.93258786,  -0.326965868,  -0.666697979,  -0.166585565,  -1.04979682,  -0.674860001,  -0.216076717,  -1.03746426,  -0.940603077,  0.61775589,  1.62191296,  -0.502878308,  0.346429795,  2.9975481,  -0.561557055,  -0.837447643,  0.606793821,  0.844667137,  0.317915112,  0.3595604,  1.56703544,  -0.56695354,  0.561470032,  0.72761637,  -1.63723862,  -0.399498612,  -1.5419904,  -0.535531342,  1.35190582,  -0.901198685,  0.962444723,  0.559754968,  -0.790756106},{ 0.8847574,  -0.00915702991,  -0.343874723,  0.380961716,  1.10643852,  1.22855771,  -0.466408551,  1.27023435,  1.24705327,  0.253321975,  0.198983833,  -0.24390173,  -0.882297277,  -0.26475963,  0.506078184,  1.60547304,  1.13844037,  0.0519405752,  -1.12547624,  -0.22641018,  -0.0540397242,  -1.47728503,  1.09708035,  0.170675784,  1.48622131,  1.07641149,  0.801538348,  -0.438847721,  0.497785687,  -0.22023651,  -0.0251072962,  -0.93530643,  -0.63345176,  0.0707790107,  0.172956601,  0.843284011,  0.116313569,  1.25772262,  -0.905861795,  2.11255026,  -0.758638859,  -0.36270985,  -2.06712699,  -0.243066549,  1.20500374,  0.170634314,  -0.711754799,  -1.6737113,  0.643846214,  0.2558617,  1.5129652,  0.403250962,  0.817701399,  0.0274699349,  1.15456629,  0.856879115,  0.202133417,  0.910782874,  -0.015922416,  -0.0461680628,  0.0480075739,  0.66379261,  0.73396337,  -0.350088656,  0.31123814,  0.858445704,  1.42174983,  -0.973083198,  0.673497915,  0.074756749,  0.164829418,  -0.743064702,  1.46458471,  -1.00211203,  0.386496812,  0.730257273,  -0.631099164,  0.119280525},{ -1.0649538,  -0.854539335,  0.189299405,  0.968105316,  1.30976903,  0.728627861,  -0.860794842,  0.597893298,  -0.974658012,  0.694508374,  0.664215326,  0.681738973,  0.1694168,  1.63148856,  0.837957263,  -1.20228863,  0.293925822,  0.130495697,  -1.81587887,  0.478047431,  -1.025962,  -0.890408933,  -0.383467942,  -0.115319073,  -0.639814854,  -0.285112709,  -0.708677649,  1.62683761,  -0.392094672,  -0.810043633,  1.24006927,  -0.256420285,  -0.475868762,  0.336814553,  -0.894586861,  -0.432131141,  -1.14531767,  1.18515491,  -0.205353126,  -1.12100732,  -0.903343558,  -0.809369922,  0.704778075,  0.698229611,  -1.18197286,  -0.121667527,  0.0318390056,  1.49373639,  2.0614841,  0.118814372,  0.314185649,  -1.43239284,  0.901020229,  0.475871623,  -0.218167678,  2.42699647,  1.10748124,  -0.56733346,  1.64031911,  -0.110374697,  -0.195253894,  0.0929827169,  1.14124155,  -0.0652537122,  -0.24732919,  0.706421733,  -0.224391609,  1.32038152,  -0.275378138,  -1.09489763,  -0.0205418505,  -1.36871243,  0.0338253006,  -0.436583757,  0.44801867,  0.190388054,  -1.12207007,  1.32164061},{ -1.59707034,  1.23399127,  -1.05852699,  -0.0358496308,  -1.47180867,  0.911435306,  1.22321439,  0.754818916,  -0.0193883628,  -0.867277086,  -0.0759315118,  -1.4214505,  -2.05072856,  -0.0313571803,  0.440226287,  0.626502812,  -0.929880083,  -2.50644112,  1.89277554,  1.78646433,  -0.112150848,  -1.42928219,  3.20253038,  -1.09194648,  -0.857266128,  1.7955569,  -0.637444437,  1.33582139,  -0.297081113,  -0.891370058,  0.80200088,  -1.28149748,  -1.00559664,  0.337411582,  1.83482087,  -0.896496773,  0.00986656919,  -0.107658595,  -2.17615938,  -0.993531883,  -0.499576658,  1.05072308,  0.240813687,  -0.117988512,  -0.472990125,  -0.202497572,  1.33718634,  -1.4582504,  1.03985834,  -0.953177273,  -0.310679257,  -0.165905282,  0.769515872,  0.0346807875,  -1.02266037,  0.606696784,  0.446489185,  0.801996648,  0.705356717,  0.130067334,  0.0526449457,  1.70597601,  -1.60532069,  0.319100589,  -1.74147546,  0.651709437,  1.24873245,  0.0884029195,  1.62151885,  -0.115818582,  0.979727507,  0.45153895,  -0.122714736,  0.64180243,  0.0179341007,  1.49516976,  -0.26215595,  0.576209664},{ 0.48477152,  -0.947952449,  1.65058792,  0.97178787,  1.05374694,  -0.348353446,  -0.619718075,  1.33642721,  1.05632031,  0.207654253,  0.705281079,  -0.844865382,  -1.11190987,  1.56450152,  0.0298998915,  -0.573461235,  -0.665137589,  0.12188381,  0.20504196,  1.03564334,  1.07924342,  -0.916556299,  -0.210496381,  -0.459937811,  0.712942839,  -0.450749248,  -0.717667639,  0.943257868,  -0.695853829,  -1.5086292,  -0.224252671,  0.0822436661,  0.52561003,  -2.11817765,  -0.198057115,  -1.18253791,  1.0431509,  -0.206129283,  1.41078663,  -0.105392672,  0.0483306311,  -0.220394552,  -0.334571332,  1.32308459,  0.270167559,  0.648825943,  -1.42168009,  -1.16772366,  -0.21541594,  0.557416975,  0.0611622185,  0.811580658,  0.894499898,  -1.48044705,  -0.81173116,  -1.10553205,  -0.28158918,  2.51081777,  2.32018018,  0.819264889,  0.0630785972,  0.170528084,  -0.301271766,  0.560378909,  0.380908191,  1.01249957,  -0.820995152,  -1.13080394,  -0.451376826,  0.18999882,  -0.134943366,  0.514416277,  0.497207552,  1.07607019,  1.96582103,  0.0890811384,  0.726605952,  -0.375514448},{ 0.00902303495,  0.99086076,  1.0858916,  0.453903049,  -0.412995458,  0.771077275,  -2.02218699,  -0.7321347,  -1.35199249,  0.0726625919,  1.78916693,  0.273153514,  0.684518933,  -0.627911508,  1.14536583,  1.78104174,  1.1034261,  -1.83509457,  -0.620095372,  -0.106914908,  0.915011644,  0.50790745,  -0.282672524,  -1.31705511,  1.51885462,  1.95096231,  -2.01054358,  1.28281486,  -1.03162885,  -0.798070848,  -0.14818947,  0.217618734,  -0.855841875,  -0.0605586581,  -1.04688048,  -0.274780929,  -0.256457001,  1.11888158,  -0.965950072,  0.614448845,  1.94103706,  0.439968854,  0.103452869,  0.654978156,  0.66377002,  0.284379005,  0.595731735,  0.254901856,  0.549201488,  1.83690739,  0.850016773,  -0.268327802,  -0.9517079,  0.863536298,  -1.28275204,  -0.789545774,  -1.5167222,  1.06986153,  -1.16954577,  -0.512655556,  0.410792202,  2.47911835,  -1.86497521,  1.40348566,  -0.724559426,  -0.596030235,  0.737253726,  -0.365597576,  0.165224582,  0.931063771,  0.585804641,  0.103215411,  -1.41103411,  -1.25659311,  -0.613112628,  -0.566738248,  -0.536795497,  2.38889074},{ 1.79260373,  -0.917461634,  0.377433896,  1.06129634,  -0.0428218544,  0.353072822,  0.981118262,  0.065445371,  -0.388064086,  -1.85195374,  0.156980187,  1.15240943,  -0.469396681,  -1.35706437,  1.11179936,  0.866327345,  0.583738387,  0.429461122,  -0.811155975,  -0.239069909,  -2.26787877,  1.75734878,  0.500518382,  1.03260303,  -1.66635883,  -0.447226971,  1.56552529,  0.771269441,  0.719151139,  0.512250185,  -0.195786744,  0.130676299,  1.25102866,  0.494972914,  -1.02621961,  0.342184335,  -1.78404975,  0.908093035,  1.46298349,  0.238319367,  0.46387893,  -1.19768691,  0.786836028,  -2.35141301,  0.904300511,  -0.677348316,  0.0852861106,  0.127904743,  0.459627837,  -0.165715933,  0.807978451,  -0.668707192,  0.14990364,  -1.77877641,  0.0081153987,  -2.43819022,  -1.01653636,  1.01276398,  0.979445875,  -1.41265678,  -1.86669338,  0.964371085,  -1.90035057,  0.620436728,  0.269790679,  0.953802705,  -0.631907284,  0.940024555,  -0.100321397,  -1.06440079,  1.38465106,  -1.0082134,  0.115872912,  -0.270164281,  -1.06180823,  0.43090412,  -1.52380919,  1.69974387},{ -1.42610168,  0.00109907519,  -1.93318832,  -1.56698811,  -0.577132821,  -1.66112924,  -0.257518172,  0.876395583,  -0.234023884,  0.0894275978,  0.221739665,  -0.223196253,  -0.280493289,  0.409510911,  0.399232298,  -1.00531006,  -0.980418921,  0.824866354,  0.909529507,  1.04743075,  -1.06054139,  0.197266713,  1.13076496,  -1.19696009,  1.21088064,  -1.58541155,  -1.1996901,  -0.177563921,  1.31802857,  0.649898052,  1.81205261,  0.478815556,  0.582173467,  -0.950002313,  0.740448534,  -0.92905426,  -1.77237952,  -1.07008111,  0.0631710887,  0.284720302,  -0.0944208503,  0.68280679,  -1.61592698,  0.0627115741,  -0.688890219,  1.16028476,  0.194401383,  0.625535667,  -0.592437208,  0.717197895,  -1.63043749,  0.501155555,  0.862587929,  -0.511070192,  -1.60537064,  -0.0493597463,  1.31044483,  0.323962897,  -1.29979706,  -0.897002637,  -2.4177084,  -0.989409149,  1.9040997,  1.01349962,  1.80905724,  -1.62963402,  1.16037226,  -0.247149169,  -0.412335783,  0.710793078,  -0.0826197341,  1.74734032,  -0.192490444,  1.06707251,  -1.74470973,  -0.900876522,  0.406163007,  -0.00330349035},{ 0.958729982,  -0.414681405,  -0.59517324,  -1.07059312,  -0.572688878,  -0.235643327,  1.35630047,  0.453960657,  -0.274083048,  -1.378757,  0.119561881,  0.00499084499,  -1.77335906,  0.641087949,  -0.376115203,  -0.0532094687,  -1.24796844,  -0.856066585,  1.11503005,  1.68704069,  -0.944068909,  2.42775154,  -0.48669678,  -0.266275257,  2.13546634,  -0.975525081,  -1.43574679,  0.346407354,  -0.572906911,  0.551500618,  -1.02128053,  -0.320889473,  -1.94087243,  -0.696902037,  -0.242923468,  -1.64438677,  -1.38055682,  -0.632739723,  -0.574851811,  -0.376042426,  -1.63506341,  -0.14788644,  0.22050117,  1.51531029,  0.809034705,  1.08508158,  -0.471216291,  -0.212108374,  0.909476161,  0.533922374,  -0.128767848,  0.617263317,  0.160788298,  -1.03063273,  2.06390667,  0.584299982,  1.26848698,  0.121884286,  0.877549946,  0.177517638,  -2.16453624,  1.52363086,  0.319910407,  1.12748361,  -1.46628332,  0.0880003795,  -0.768921733,  -0.133536786,  -0.682227552,  0.341006428,  1.00541091,  1.59846187,  -0.338149518,  -0.0855719671,  0.208177805,  0.0844217539,  1.80324602,  -1.95863819},{ 1.53616214,  0.83804065,  -0.816568434,  -0.295260817,  1.36499059,  -0.183722556,  -1.54477262,  0.413463205,  1.47796702,  0.050021898,  1.83915734,  0.751717329,  -0.334848225,  -0.0389072075,  0.122343734,  -0.532117665,  -0.53321141,  -0.150890633,  -0.55630964,  -1.52077496,  1.0667038,  -0.249130681,  -0.772704899,  -2.05522728,  -0.954589069,  0.204183862,  -0.731905699,  -0.751296759,  1.20389748,  -1.10407472,  1.34664547,  -0.310514182,  0.487501413,  0.838955939,  -2.37330866,  0.56377393,  1.08332419,  0.184394807,  -0.345056236,  -0.51292938,  1.6047188,  1.4700042,  1.87105858,  0.507584095,  1.66770661,  -0.497830242,  1.57931781,  -0.735396981,  1.91269183,  -0.278721362,  0.172934338,  2.90597296,  -1.01420987,  -0.547106564,  -0.411076903,  0.68759805,  1.08947849,  0.812137485,  0.811086416,  -0.421929717,  -0.492302537,  1.17775643,  0.345903546,  -0.0417430364,  -0.0319993123,  0.413989276,  0.856154442,  -0.0255170278,  0.988735974,  0.897447407,  -0.137002856,  -0.47562027,  0.853668571,  -0.116808802,  0.224011391,  -1.06418395,  -0.630356669,  -1.23997295},{ 0.260575622,  -0.568381786,  -0.81790638,  -0.730229437,  -1.9404999,  -1.35686445,  -0.584164858,  -0.757071078,  -0.427450716,  -1.04749382,  -0.275427163,  -0.408729345,  1.08425272,  0.759246171,  -1.52861905,  -0.669156969,  -2.43618369,  -0.432610065,  0.236809045,  -0.718331635,  1.77727544,  0.350758076,  -0.163269609,  -1.41867185,  -1.13465595,  -0.959142268,  0.26472798,  0.383037776,  0.71938318,  -0.617670894,  0.627947211,  0.0579690784,  0.856283128,  -0.148329109,  0.692553222,  0.758441806,  -0.0650098473,  0.325874239,  -1.60889959,  -0.233722419,  0.484589994,  -1.95214176,  1.04256713,  0.302856416,  -0.763056219,  0.923994839,  0.424805462,  0.385558426,  1.15101802,  -0.49249509,  2.41184855,  -1.09732878,  0.189321682,  0.406043619,  -1.11401892,  -1.41521013,  0.880862534,  0.718575597,  0.922743618,  1.58406413,  0.498427749,  -1.95433545,  1.0194639,  0.329499096,  0.291806042,  1.55982149,  0.416106969,  -0.818886161,  1.06239402,  -0.290025175,  -0.498441756,  0.430431604,  -0.723389864,  -1.58317709,  0.332830966,  -0.0139817586,  -1.76663995,  1.69316089},{ -0.216756791,  0.4332605,  -1.71079504,  1.25362015,  0.12054231,  -0.198713928,  0.754036009,  -0.58141911,  0.668624759,  -0.424305528,  -0.964911103,  0.78436321,  -0.144172728,  -0.327302337,  0.0697320327,  0.156075329,  -0.378946394,  0.727634609,  -2.738343,  -0.774813771,  0.0183180533,  -1.01391125,  1.96060097,  -0.510601699,  0.948709011,  -0.79318434,  -0.541941226,  0.107157715,  -0.220455647,  0.201535344,  -1.76962161,  -1.10094512,  0.542805076,  0.0218764264,  -0.988066494,  1.01631522,  -0.0282651726,  1.68706906,  -1.77878141,  0.957718968,  0.866122186,  -0.113727622,  0.43375653,  0.0144843441,  0.588695526,  1.09906101,  0.270310998,  0.784622788,  0.620689392,  -0.0729110688,  1.26201463,  -0.591982961,  -0.775402308,  0.547465205,  0.0762678981,  -0.493318111,  0.127452746,  0.733731866,  2.49969554,  2.09131002,  -0.150837913,  -0.0729722828,  -0.387040883,  -0.098380968,  -0.208105534,  1.48522365,  -1.12417066,  0.117055751,  -1.20962954,  0.124345198,  0.0193798561,  -0.224178091,  -0.906865478,  0.781407714,  1.74568141,  -0.320666462,  -2.28199387,  0.938890278},{ 0.52401942,  -1.04850841,  -0.195509106,  -0.707465112,  0.920398533,  0.301552802,  -0.416498542,  1.11424816,  0.786546826,  -0.802210927,  -0.185766384,  -1.15847528,  -0.426854849,  0.699984729,  -0.455413759,  0.359407455,  -1.15257859,  -1.13346851,  0.832951486,  -0.296959341,  0.314380974,  1.95972562,  0.76466918,  1.21236515,  -0.187978223,  -0.67333132,  0.224551171,  -0.377017468,  1.61187017,  -0.414999306,  2.36288142,  -1.32577622,  0.251016676,  0.545141578,  -0.329553962,  1.05086136,  -0.229076013,  -0.37349087,  0.416892081,  1.04835212,  -0.350305885,  0.27226609,  0.814823508,  0.178305998,  1.21604407,  1.14733458,  0.0707282722,  -0.406330615,  0.0522746034,  0.0960420296,  0.0685992613,  1.20336008,  0.0992086008,  0.245764941,  -0.0448641255,  -0.692403734,  0.68797487,  0.396615386,  -0.440284461,  2.30801344,  0.0557801165,  -0.246885598,  -0.349429786,  -1.12225389,  0.7860232,  -1.05707145,  -1.29675937,  0.018481046,  -0.312779576,  0.746862173,  -0.817871094,  -0.290941447,  -0.184579134,  -0.489695072,  -0.587168217,  0.833614111,  -0.844843805,  1.09008038},{ -1.03580749,  2.53476739,  1.22413576,  -0.966209888,  0.108714066,  -0.0597241633,  1.22105825,  0.846814871,  0.324800104,  1.09219801,  0.378983736,  -0.0971843973,  -0.167498693,  -0.632399738,  -0.778869033,  -0.735664785,  1.22532868,  0.288059473,  0.0935486928,  0.128151521,  0.831232786,  -0.0320661217,  -1.38906181,  -0.0426234752,  -1.29010105,  0.241414785,  -0.283049792,  1.05074358,  0.553402901,  -0.748980939,  0.188904241,  0.312357277,  -1.299981,  -1.15722322,  -0.190794781,  -1.14727426,  0.138072744,  -0.512613833,  0.361211628,  0.442363769,  -0.203616366,  0.730999231,  1.66864967,  -0.0629169792,  -0.10269583,  0.259819955,  -0.0386679508,  -0.927277446,  1.77132845,  -0.962387562,  0.714222968,  1.94802356,  0.476263106,  1.78939557,  1.22958946,  0.239649504,  0.977419853,  -1.48894048,  0.125585288,  -0.0607171245,  0.574058533,  0.900292099,  1.22003019,  -0.518722534,  -0.432697177,  0.197553396,  -0.454270661,  -0.138394788,  -1.00580263,  0.150132298,  0.87331444,  1.34509206,  -0.558557987,  -0.151272863,  -2.28078961,  -0.0687887669,  0.186733246,  -0.0216202512},{ -0.993043125,  -1.34140551,  0.278647393,  -1.40934479,  -2.23963857,  1.33041334,  -0.259636909,  -0.442915052,  0.444993824,  0.0160497092,  0.265321285,  1.40971482,  -0.264926195,  0.437213033,  -0.614174783,  1.19160831,  0.304386854,  -0.290779591,  0.876729786,  -0.72346431,  -1.26208091,  -0.68642813,  -2.23716259,  -0.762537718,  0.962898493,  0.658916593,  -0.192614645,  0.18882212,  -0.0584821254,  -0.125980064,  -0.762150645,  1.46987569,  -0.788410962,  -1.05932868,  -0.679459572,  0.250154883,  1.08611763,  1.08327138,  0.0090766279,  0.0169659778,  -0.782366395,  0.339392543,  0.366136551,  0.780736446,  -0.66740942,  0.313317001,  1.09234893,  0.48253119,  1.57303119,  -0.633906543,  0.179918453,  -0.699657083,  -0.225812823,  -0.294923991,  0.410281122,  -1.62263501,  2.05109715,  0.937308073,  -0.952702641,  0.151212737,  0.68593502,  -0.861086011,  -0.375919849,  -0.105409741,  -0.917897761,  0.932041168,  0.397350729,  -1.37397516,  0.295490384,  2.42201543,  -0.306470782,  1.42013693,  0.753891945,  -0.506854117,  -0.239555568,  -0.237418368,  0.433377594,  -1.79997456},{ -1.35994971,  -0.210560143,  1.88479459,  -0.395239621,  1.69784427,  0.0936479568,  1.32217944,  -0.18511945,  0.972129047,  1.11550128,  1.07374942,  0.129141673,  -0.444403082,  0.91565305,  1.28994727,  -0.362323612,  -1.56973374,  0.293566823,  -1.20863938,  0.073125951,  -0.420362622,  -0.762011051,  0.412887305,  -0.353678793,  0.319355398,  0.0430328548,  1.90336704,  1.8434875,  0.529773116,  -0.553472877,  -1.19507635,  -0.675485909,  -0.809249163,  0.443849713,  1.09736061,  0.411257744,  0.964951456,  1.8726474,  0.8541345,  1.09141243,  -0.789543271,  0.812786996,  -1.27214622,  -0.37164402,  0.4210127,  1.40523899,  -0.228990123,  0.409006476,  0.054453142,  1.33905387,  1.1348933,  -2.22714639,  1.27133358,  -0.0770294145,  0.398691148,  1.59543109,  -0.135288477,  0.971004248,  0.817574084,  0.752827466,  0.686843812,  -0.0840230212,  1.60910058,  -0.453234106,  0.440223664,  1.29662001,  0.346704453,  0.261273086,  -0.416274071,  0.697881877,  -0.0914965272,  0.104227558,  1.70543289,  -0.39122954,  0.736284494,  -0.0630520806,  -0.986969948,  0.578244448},{ -1.7854749,  -0.904060245,  0.684661925,  0.0728349537,  -0.676623762,  1.16704035,  -0.104657859,  -0.00350489723,  -0.356868267,  0.234089166,  0.463038385,  1.00386071,  -1.3897078,  0.469276786,  -0.497862995,  -0.838517249,  0.298623711,  -0.628976583,  1.09134114,  0.653889,  0.634799361,  -0.161808908,  0.791005969,  1.45234072,  0.404550642,  -1.76561832,  1.0241797,  -0.0336948037,  -0.668794692,  0.399580598,  -1.40102649,  -0.387663811,  -0.0032218867,  1.85790026,  -1.42508256,  -1.53048503,  -1.15907526,  -1.05474126,  -0.0492372178,  0.106153823,  -1.98003483,  -0.388545126,  -1.1858269,  -0.3817828,  1.04139459,  -0.312662989,  1.24603713,  -0.0914862752,  -0.401380479,  -1.73543036,  1.05812109,  -2.4015677,  2.0682404,  0.202780187,  0.340937674,  0.559049904,  -2.17173457,  0.00726461364,  0.923031986,  0.582820475,  -2.18582106,  0.386337727,  -0.170779079,  -0.232561618,  1.41448951,  0.705903292,  -0.0566292442,  -0.0529227778,  -0.428400218,  -0.49973321,  -0.416839421,  -1.45859361,  0.050402388,  -1.39586687,  -1.10515559,  0.695418537,  0.629987776,  -2.21683121},{ 0.13768059,  -1.56870234,  -0.116648622,  0.419859737,  0.288656861,  -1.31681299,  0.0496926941,  1.70073283,  -0.37545231,  0.816801012,  -1.9113735,  0.739403963,  -1.59810841,  0.506300628,  0.461380035,  1.01317668,  2.04440856,  1.13173497,  2.00414205,  -0.584006608,  -0.445131868,  -1.57851863,  -0.282021075,  0.0232087504,  -0.855483115,  -0.0496424511,  0.872404575,  -0.270168662,  2.25656199,  -0.0239661466,  -0.182105169,  0.295475304,  -0.639494956,  -0.0501375645,  -1.55221963,  -0.161844164,  -1.1500814,  0.191655174,  -1.3175894,  1.6514461,  0.024078114,  -0.0451266356,  -2.44268608,  0.65874964,  -0.32196337,  0.0696296021,  -1.57989502,  0.647679269,  0.0676752552,  0.64688462,  -0.00962408911,  -1.16952026,  0.480208099,  0.416950166,  0.981601477,  0.439793289,  -0.196167246,  0.736573637,  -0.59951055,  0.793986082,  0.732499659,  0.676541805,  -0.334141552,  -0.421652764,  0.872027576,  1.71663988,  0.311098218,  2.29732776,  -0.177064955,  0.58999753,  -1.17616522,  0.561621726,  -0.0302534383,  -0.569474697,  0.0471637398,  -0.249012649,  -1.65347874,  -0.413045257},{ 0.306770742,  -1.06539834,  -1.17676353,  1.63684428,  0.102227025,  -0.790951669,  0.199798703,  -0.226114452,  -1.68494344,  0.391037375,  -0.40883863,  0.786732912,  0.800244451,  0.563851714,  0.717100799,  -0.930406034,  0.215395272,  1.13959205,  0.178963318,  0.0611671507,  -1.82719731,  -0.735759377,  -0.0102567254,  -0.818260431,  -0.538142204,  0.293136358,  -0.941043735,  -0.0952731296,  0.069323428,  -0.159225151,  -0.667489231,  -0.810888886,  0.40429312,  -1.71925962,  -0.173522726,  0.29098621,  -0.973967612,  0.298924088,  -0.847940624,  -1.65771556,  0.822361767,  0.577839732,  -1.38744652,  -0.0172064509,  1.29369044,  1.41050506,  0.0332734324,  1.02324116,  1.9028033,  2.0033381,  -2.73532891,  1.0551914,  0.449127048,  -0.459006339,  -1.31957626,  0.391671538,  -1.84083128,  0.283311576,  -0.0178069994,  0.83338666,  0.668416023,  0.065578118,  0.928513587,  -0.638949633,  -1.44090152,  1.67558563,  0.641499758,  -0.164460734,  -0.967507184,  0.726167679,  0.145309493,  0.951734602,  0.148205027,  0.434023201,  -2.34389758,  -0.137550786,  -1.04334521,  2.07349205},{ 0.752167404,  -1.38010287,  -0.379098326,  1.0296036,  1.02657187,  1.17324924,  -0.593370795,  0.90057379,  -0.373327225,  -0.345319003,  -0.957986653,  -2.58197045,  1.85719383,  -1.42553616,  -0.722578585,  -0.986851871,  -1.36414242,  2.82889199,  0.437032998,  -0.764958203,  -1.61963463,  -1.21808577,  -2.22628593,  -1.21532953,  0.602434456,  -0.145025477,  -0.112700649,  1.00835299,  -0.852819741,  0.973197043,  2.18803453,  0.433984727,  -0.113724403,  1.15808976,  -0.468184024,  -0.177750528,  -0.605168045,  1.8302772,  -0.162054092,  -0.140175655,  2.09018135,  0.315743357,  -1.0263108,  -0.354450345,  0.405192226,  0.199260592,  0.38848111,  -0.911961257,  -0.650868595,  0.721541286,  1.04732108,  0.109177075,  1.12107706,  -0.781759322,  0.268103153,  -0.920292974,  -0.425359666,  -0.221700847,  1.68859875,  0.387550741,  -0.316382855,  0.782616913,  -0.0318927132,  -1.00965846,  1.1777488,  -0.805938125,  0.398430198,  0.257966846,  -0.540973723,  1.95558393,  0.552047729,  0.43785885,  -1.05751669,  -0.424972177,  -0.92480582,  -0.112439387,  0.480647713,  1.35578918},{ 0.243976891,  -0.716809869,  0.0588694736,  0.249452963,  0.774145544,  1.56089258,  -1.17797565,  0.908640742,  -1.24528503,  0.389545828,  0.100531615,  0.0104110269,  -0.282108486,  -0.961755395,  -1.43827343,  -0.812109351,  -0.920089185,  0.894451857,  0.411333382,  1.44845951,  0.132104859,  -2.03840828,  0.409884244,  0.554561377,  -0.170803949,  -1.5386095,  0.568154395,  1.96284807,  -0.0457147509,  0.784075618,  -0.0289529283,  0.350059539,  -1.00334597,  0.306980759,  1.435449,  0.342255175,  -0.527335882,  0.976939976,  -0.262121409,  1.1377418,  0.0160705019,  -0.370725095,  0.377112806,  -0.00349429972,  -0.261117369,  0.231855929,  -1.47246909,  -1.40409875,  0.719336092,  -1.05631411,  -1.08243358,  0.0171525981,  -0.740301013,  -0.44348374,  -1.50753534,  0.155246213,  -0.613893688,  -0.0830456987,  -0.823281944,  -0.960662901,  -0.0324887745,  -3.07222414,  2.79181623,  0.444313496,  0.966125131,  0.199247435,  1.02392173,  -1.41688061,  -1.68628013,  1.31202567,  -0.542035103,  0.822903395,  -0.123972476,  -0.602801979,  1.24562824,  0.373876989,  -1.43544877,  -1.24954545},{ -0.559467137,  1.20052338,  1.91131639,  -0.443908721,  1.49479783,  0.489275783,  -1.21443009,  1.30651975,  -0.139618069,  1.50217497,  0.716246665,  0.705689847,  0.797740757,  0.113347344,  1.26601887,  0.278226286,  2.31719613,  -0.943130374,  -0.353256553,  -1.63397241,  -0.359251797,  -1.58375645,  -0.432282686,  -1.32947409,  0.208079219,  0.142359555,  -0.569242179,  -0.876358986,  -0.0379478298,  0.0991358012,  1.17950559,  -1.37106252,  2.01349711,  2.69901085,  1.12839234,  -1.3631109,  0.651386738,  0.50298363,  -0.342791408,  -0.758979976,  0.697157919,  1.06456411,  1.96130455,  0.777190983,  -0.855705559,  1.16481173,  0.264151096,  -0.155621693,  0.749810934,  0.482377172,  0.0612991899,  0.548327923,  -1.42390335,  -0.0163918976,  1.89134765,  0.861110568,  0.648101211,  -1.10240459,  0.768245995,  -0.611647666,  1.86299348,  -0.498923779,  0.778328001,  0.533259034,  -2.02803206,  0.256637126,  -0.00789844245,  0.0127267307,  0.509069443,  0.797599256,  -0.580922484,  1.13438451,  -0.24165532,  -0.928803742,  2.07432413,  -0.128505796,  0.570103109,  -0.851238191},{ 1.41815805,  -0.0663344488,  1.70016098,  -0.460007578,  -0.0225010347,  -0.014069899,  -0.579032302,  -0.540301025,  -1.14918923,  -0.752910376,  0.0781754404,  0.694747567,  -1.56309044,  -0.0946866795,  2.63786578,  -0.177649468,  0.601968586,  0.190696791,  0.761720121,  -0.243646145,  0.166058049,  1.71841562,  0.661340237,  -0.313680261,  1.61117649,  0.460013986,  0.323142946,  -0.524612963,  -1.52822888,  -1.1548636,  -0.422253758,  -0.778142869,  1.93733704,  0.334555626,  0.268314838,  1.03685737,  1.10891485,  0.798034787,  0.243454814,  0.947666943,  2.88495374,  -2.19245648,  0.472850412,  -0.995478272,  0.396347255,  1.30830407,  0.267567486,  1.30573189,  0.240043014,  0.501977742,  1.18362594,  -0.481816322,  -0.2627469,  -2.16751933,  0.379309952,  -1.42304301,  -1.47833097,  -1.0512048,  0.111602202,  1.11306489,  0.678231239,  1.10079992,  0.470013112,  0.544283509,  -1.15738952,  0.372042805,  -1.67130637,  0.505069315,  1.49003732,  1.19830978,  0.0770942122,  1.265769,  0.0882428065,  2.03557158,  -1.59156811,  -0.830648899,  0.62426573,  0.0559961982},{ 0.657857001,  0.50023514,  0.377188891,  -1.407107,  -1.40512729,  0.413770109,  0.0168295391,  -1.05003238,  -0.446227729,  -0.972913802,  -0.554504216,  -1.01281679,  0.575865269,  -0.835029483,  -1.51524866,  0.0207898598,  -0.0560739264,  -0.275072306,  -1.08597934,  1.14763439,  -1.05821705,  1.3608886,  0.418207318,  -0.890434146,  0.372249365,  0.763959944,  -0.0854348391,  0.452045023,  -0.838582456,  -0.676117539,  -0.514651299,  0.0263394974,  -1.93088436,  0.755418658,  -0.291854203,  -0.566813767,  -0.36635536,  1.50536394,  0.595337033,  0.466250122,  2.27883697,  -1.22520518,  0.836874485,  -1.72171187,  -1.19863367,  0.209616452,  0.434375644,  0.0504356213,  -0.12501514,  -1.61219025,  1.09999144,  -0.165022686,  -0.0267591923,  -0.138775378,  1.60329521,  1.6346662,  1.82039535,  1.15451932,  0.209336102,  1.71034217,  -0.832052529,  0.214166239,  2.30493712,  -0.811873198,  0.614130676,  0.108172826,  -1.19470119,  -2.38557148,  0.796527207,  0.812693417,  0.244719118,  1.43198955,  0.0486858636,  1.80954301,  0.227686256,  -0.793970883,  -1.65665507,  1.21784115},{ 0.316530675,  1.19482827,  0.178411439,  0.274118692,  0.632396817,  -1.27393591,  -0.821805775,  -1.54333401,  2.30602455,  2.30505466,  -1.76320708,  -0.280260712,  -1.17202604,  1.52212846,  1.36700225,  0.544440448,  0.823938191,  0.572035968,  -0.181569085,  -0.352965385,  -0.647721589,  -2.69172239,  0.3717089,  -0.388912648,  0.00425935118,  -0.229353219,  -0.768977523,  -0.511444092,  -1.16516829,  0.468113899,  -0.589905262,  0.789351225,  0.427372009,  0.539601147,  -1.17188179,  1.15102267,  -0.554190695,  -0.184847996,  2.87618184,  -0.910670459,  -1.31894553,  0.644992292,  -1.24814427,  1.25554681,  2.61175418,  -0.547743261,  -0.138590813,  0.596844375,  -1.13308346,  0.0984031633,  1.02537596,  1.01835561,  1.66762269,  1.3765955,  -0.272916794,  -0.577828526,  -0.184922382,  -0.174449772,  0.579759479,  0.879640758,  -0.888223708,  -0.486881435,  1.08954489,  -1.56236482,  0.530352116,  -1.60539734,  -0.878622591,  -1.28372288,  1.24921465,  0.37022832,  2.30057693,  2.2883234,  1.00398695,  -1.43056703,  -0.188280597,  0.804045498,  1.05173588,  0.537522912},{ 0.114540875,  -0.404122919,  0.358635336,  0.31421116,  1.22322416,  1.32479715,  1.34205103,  0.123793393,  0.277493566,  -0.12625742,  -0.350317329,  1.27216208,  -1.05705094,  -0.250688136,  1.75452924,  1.33451951,  1.26120102,  -0.98750037,  0.122192271,  -0.441034049,  -0.180138797,  -0.434111953,  -1.30192041,  1.48353493,  -1.19140947,  1.01810646,  1.25124395,  1.26603568,  0.0651506335,  0.738829434,  0.834913671,  0.397682011,  -1.62327278,  -0.728304029,  0.024387747,  0.809144616,  -2.03318095,  1.04764581,  1.95923913,  -1.63810205,  1.38108063,  1.69212258,  0.0185928531,  -1.42922699,  1.4994632,  -1.49457788,  0.797244847,  -0.553530514,  1.12950742,  -0.457637757,  -1.24449849,  2.67899346,  -1.44920564,  -1.07388568,  2.17338467,  0.216904595,  1.31180501,  -0.878700614,  -0.354774117,  0.139695808,  1.76683176,  -0.478940696,  1.22210133,  0.656995893,  0.224500701,  1.891451,  -1.50864708,  1.44020867,  1.16948771,  1.29710567,  0.98637414,  -2.1894722,  2.76758432,  -0.637936354,  1.03247511,  -0.247827813,  -0.716425121,  0.0865506828},{ -0.172537059,  -1.7558372,  -1.19530845,  0.775262296,  1.16154909,  -0.418784499,  0.218787596,  -0.423816502,  0.134959862,  -0.62560463,  0.205151066,  -1.25230742,  0.971946657,  1.28807449,  -0.502010942,  -0.944679677,  1.33944607,  1.23894072,  -0.12225274,  -1.90054512,  -0.735827804,  0.27658692,  1.45132184,  0.567256987,  1.74289787,  -0.0754896775,  0.195328146,  0.477389157,  -1.34435058,  0.897495627,  0.268755406,  0.298577338,  0.0210397523,  -0.0316697173,  -0.722935021,  -0.40448463,  -0.817878425,  -0.528079748,  0.655179262,  1.78277683,  -0.574509323,  0.213123083,  0.989978373,  0.337451041,  0.0182377044,  0.43271026,  -0.990087032,  -0.289148927,  0.797526121,  0.856163979,  -0.228541836,  0.937667787,  1.34765935,  0.924224555,  -0.920607626,  0.826142073,  -1.00443459,  0.00961856171,  -1.11284065,  0.447129369,  -0.873060226,  0.110272296,  0.938595533,  1.07171416,  -0.529738963,  -0.70631671,  0.480966777,  -0.327872008,  1.30174649,  0.339835972,  0.353714168,  1.10857296,  0.281480044,  -0.294692129,  0.767204642,  0.0757328942,  0.835688531,  -0.816979468},{ 0.315714359,  -0.887282133,  0.892929316,  0.0849845484,  1.34950399,  1.29539049,  -0.898557544,  -1.43953216,  -1.16813767,  -1.58504605,  0.252806395,  1.06746805,  -0.227058426,  -0.851051092,  1.1305362,  0.190947041,  0.103994548,  1.0143193,  0.0114060147,  0.956598401,  0.747571766,  0.326953471,  0.161178797,  -2.05783081,  0.171275407,  1.18631041,  0.27939108,  -0.483697712,  -1.52075493,  -0.482622415,  1.6321106,  -0.337517291,  1.54255641,  0.512566924,  1.13259888,  0.145153061,  0.904586434,  -0.699888647,  0.470060378,  -0.511954129,  1.11391771,  0.528113663,  -1.32210588,  -2.46534419,  0.945509315,  -0.618430555,  -1.58417583,  0.297268718,  -1.3840369,  0.192730978,  -0.883351326,  -0.41323638,  1.6965456,  -0.840757847,  0.0459898673,  0.212334991,  0.977557957,  -0.636581481,  0.910892487,  -0.204923719,  -1.24977839,  0.704949021,  -0.629683733,  -1.46947837,  -0.303733468,  1.38528013,  1.54557443,  -0.188137263,  -1.05790746,  -0.731430113,  -0.362639695,  -0.225961804,  -0.389821112,  1.01035237,  -0.544261754,  -0.742770791,  0.591099441,  -0.767991364},{ -0.954145253,  -0.956091046,  0.907630742,  0.448533833,  0.505403221,  -0.198378608,  2.4596014,  0.128548309,  -0.238821313,  0.145338774,  -0.541179359,  -0.400155783,  0.192239717,  -0.796803713,  -0.606682777,  -1.45444977,  -0.568867505,  0.300855696,  -0.181219131,  -0.00387040083,  1.25757551,  -0.837613106,  -1.31937289,  -0.494071782,  -0.192326799,  -0.927351654,  1.971632,  -0.463875711,  -0.898853302,  0.881501555,  -0.946401119,  0.511945605,  -0.356906265,  0.414750874,  0.497359961,  1.15799224,  -0.306030959,  2.50189328,  0.0276592318,  0.0244456585,  -0.831904888,  0.40718627,  -1.07792449,  0.503088295,  -0.0156218512,  -0.255718678,  -0.506715417,  -0.0677657127,  0.205313176,  0.066207163,  1.32747066,  0.958637357,  -0.212959543,  0.536589146,  -0.0678667277,  -0.669811666,  1.40222037,  -0.719609559,  0.860057473,  1.17288959,  -1.16628599,  1.01514709,  1.51550496,  -0.0705987513,  0.462400645,  -0.837306142,  0.443468243,  0.391545981,  1.16950202,  -0.938102543,  -1.87338603,  -0.472302198,  0.806603432,  1.55883014,  1.95138884,  1.69247317,  1.64613366,  -1.63764238},{ -1.12306833,  -0.0155819105,  -0.463667721,  -1.84934974,  -1.01384091,  -0.282578498,  -0.639246047,  -0.545099854,  0.480776817,  0.182766065,  -1.51552224,  -1.14400184,  -0.956342697,  -0.0104756504,  -0.912488639,  -1.1833775,  0.509185731,  -1.39197218,  1.15044343,  0.200354248,  1.24511254,  0.22885558,  -0.0325714797,  -0.131921172,  0.769408643,  0.533483028,  0.202496022,  -0.752773166,  2.96176004,  -0.285270602,  -0.201692745,  -0.519580126,  0.365354061,  -0.203444481,  -0.0751819015,  -1.18222547,  -1.39614105,  1.08109915,  -0.658910215,  -0.812940359,  -2.13743997,  1.22298884,  -0.430979013,  -0.880495608,  -0.398001969,  -1.130054,  -0.121052392,  0.915426254,  0.767112076,  -0.30289939,  2.75188541,  -0.921028852,  -1.35888577,  -0.201530591,  0.67701596,  0.472949535,  -0.694837928,  -1.07022166,  0.188539714,  -1.22452641,  -0.884004056,  0.30147931,  -0.569865406,  -1.08608544,  0.128799558,  -0.947146714,  1.10911751,  1.41537774,  0.237381637,  -0.111306787,  -0.814812779,  -1.10734534,  -2.27050948,  0.0310270358,  1.44214225,  -1.78064072,  -0.362138569,  -0.0927606821},{ -0.997933507,  0.260240585,  -0.381589323,  0.096548371,  0.375815034,  -0.717768312,  0.582843721,  0.528730214,  0.143657833,  -0.611226201,  -0.542311013,  0.244886145,  -0.554498076,  -1.01845431,  -0.34738028,  0.131126538,  -0.747130454,  -0.648658991,  1.06621981,  1.10279393,  -0.730184555,  0.919129908,  1.48406541,  -1.23004413,  0.00756922923,  -0.960144937,  -2.41268349,  -0.258823395,  -0.321008235,  -0.101112314,  1.11521685,  -0.612682939,  -0.0304695591,  0.30342117,  -0.262604177,  -1.71021187,  0.0574128926,  0.145909652,  -0.116549343,  -1.07857823,  -0.184264123,  -0.997899354,  -1.74155354,  -0.495657176,  0.149983153,  -1.32555413,  -0.350020826,  -1.84123325,  -0.909948885,  -0.144063875,  -1.64757872,  0.732580304,  0.954548419,  -0.163142592,  0.334806859,  0.386134565,  0.629587114,  -1.88964117,  0.736281872,  -0.0969005078,  -1.00777543,  -0.247723415,  -0.890832663,  -0.448695332,  -0.673986673,  0.145094559,  -0.420124769,  -0.5293594,  0.721808136,  1.07369423,  0.019965468,  1.20108211,  -1.10465431,  -1.49452007,  -0.844121516,  0.216611102,  0.165637523,  -1.38731086},{ -0.0330641642,  0.0535434298,  0.844244957,  -1.55002785,  0.38511312,  -1.65163636,  -1.54668999,  2.11936307,  0.143228352,  -1.6670506,  0.961635709,  -0.672797024,  -2.23320913,  0.668461502,  -0.332898408,  -1.0618422,  0.0371786468,  -1.2448827,  0.742954671,  0.633604825,  0.414280862,  -1.15766871,  -0.485689074,  -0.358287215,  -0.567639172,  -0.392275661,  -0.0168152507,  1.28977478,  0.843749404,  1.29003215,  -1.99543619,  -0.26466611,  -1.8875823,  -0.802753031,  1.1478256,  0.574790001,  1.87784398,  -0.49955368,  -0.654163897,  0.466398895,  -0.208692655,  -0.05400737,  0.0968980864,  -0.516852617,  0.329500675,  -2.00668669,  0.846460402,  -0.484959304,  -0.063068755,  0.288724363,  -1.6671896,  -0.00293596974,  -0.300189197,  0.118076697,  -1.02746129,  -0.861063123,  0.31568113,  0.774914384,  -0.683934867,  -0.601044595,  1.02473688,  -0.428620845,  -1.29034102,  1.85843408,  -0.311304033,  -0.252838612,  -2.60667467,  -0.757186294,  0.875947356,  -2.04167104,  2.68966794,  -0.167710274,  0.398733735,  1.18043041,  -0.0601227321,  -0.156749815,  -1.46064401,  -0.304437816},{ 0.364070535,  -1.47254765,  -0.131091565,  0.0123464223,  -0.157708302,  -1.12621546,  -0.33809945,  -0.599094391,  0.438704044,  -0.608100712,  -0.0611103401,  0.303506672,  1.06552923,  -0.979270995,  0.767499208,  1.21029687,  -0.314103693,  -0.134526849,  -1.64231825,  -0.245683983,  0.130238697,  0.652036428,  0.0694535151,  -0.147783428,  -1.29060018,  0.503139555,  -0.10881564,  -1.00247836,  -0.704903424,  0.870493174,  0.411174834,  0.538788199,  -0.423388332,  -0.295543551,  0.449445426,  0.816251755,  -0.201368988,  0.820244789,  -0.214176551,  -0.604949355,  -0.278421998,  -0.261629432,  -1.11495483,  -1.20192349,  -0.825436532,  -0.181355298,  0.331140399,  0.0428022109,  -0.311316639,  -0.402914375,  1.92447233,  -0.302536219,  -0.889144301,  -0.598828554,  -0.611661375,  -0.275133431,  2.08331823,  -0.984626532,  -1.48542166,  0.244164154,  -0.921913385,  0.0337699763,  0.0258584376,  1.17920589,  -0.355747879,  -1.2626878,  -0.823174238,  -2.33527327,  -1.05097115,  0.804842949,  -0.0734231248,  -0.650110185,  -0.022974981,  0.182398379,  0.218860865,  0.570122361,  -1.27623856,  -0.755574822},{ 0.190152958,  1.17206776,  1.63352919,  0.00747539243,  -0.281530619,  -1.36961806,  0.0216521155,  1.30126119,  1.59905493,  -0.650360882,  -0.117330119,  -0.523351789,  0.334842175,  0.392660469,  0.440826088,  -1.23869562,  0.778013229,  1.10127294,  -1.62019384,  0.538165867,  1.23196828,  -0.338053852,  -1.14826941,  0.321779013,  0.0225615408,  -1.81004941,  -0.940970004,  0.296130121,  0.1164084,  0.360356212,  1.64871502,  -0.717021763,  0.193114802,  -0.164909318,  0.414966702,  0.0885800794,  -0.122383088,  0.178188771,  -0.728666425,  -1.40810215,  -0.103887044,  0.744960308,  0.114140131,  -0.311261743,  0.657643497,  -0.286286563,  0.708550751,  -0.0291424114,  1.10462856,  -2.45592785,  1.13629389,  1.77876139,  -1.23679519,  -0.363549441,  0.298227727,  0.726653576,  1.61287272,  0.0967876911,  -0.55100143,  -0.517918169,  0.182228565,  -2.05917358,  -1.53129518,  0.0687085763,  -0.0105830329,  -0.332175851,  0.475237429,  -0.119649991,  2.08334446,  2.16088057,  -1.85175586,  -0.54142189,  0.933703661,  -0.133424312,  0.199300244,  0.696883678,  0.445323676,  0.417549521},{ 0.963602901,  -1.43193555,  -0.181032002,  -0.638759017,  0.38936916,  1.18479848,  -0.448304266,  -1.04444242,  2.01254797,  0.476557434,  0.119934052,  0.216502696,  0.0241742358,  1.42597854,  -0.672995985,  -0.811907351,  1.33241022,  0.668004036,  1.17652309,  1.38025606,  -0.29433319,  -2.23488045,  -0.8736341,  -2.54452825,  0.223581895,  -1.50366342,  0.665939391,  0.400694549,  0.438834876,  1.23745263,  2.48950934,  -0.404232949,  -0.631942213,  0.316086531,  -2.51291227,  -0.0244926848,  0.303797036,  0.763271451,  0.513344944,  1.38783503,  -0.702931523,  -0.499702811,  0.892642975,  1.1318332,  -1.58167529,  -0.722186863,  1.03342295,  -0.689046502,  1.38639712,  -0.973052561,  -1.31014359,  -0.460899174,  -0.330756187,  -0.21228233,  1.24580443,  1.06971502,  0.727129281,  0.588590324,  1.13878834,  -0.630862772,  0.232296929,  -0.767307341,  -1.5583849,  -0.594605327,  -1.63730049,  2.23311138,  0.6607548,  1.24561107,  -0.439366609,  -0.820364296,  -0.853901505,  0.326099545,  -0.880357325,  -2.15863585,  0.344155133,  -1.13172698,  -0.447707444,  0.741299152},{ -0.521553159,  -0.977205873,  1.43041575,  1.69862008,  -1.34191871,  -1.08352578,  0.593324065,  0.559137106,  -1.07659602,  -0.82298094,  -1.47870636,  -1.07733703,  1.24300003,  -0.578003585,  -0.375400037,  1.20157123,  -1.99535704,  -0.179070204,  0.406751782,  0.197675169,  -0.435311288,  0.324564248,  -1.34876287,  0.0685338229,  -1.49538863,  -0.292126745,  -0.0930319503,  0.130825177,  0.782615781,  -0.324497312,  0.744937181,  0.222107247,  -1.17570245,  -0.723929524,  0.401637554,  -0.0831471831,  0.668190539,  0.332995325,  0.0312782973,  -0.648114085,  -0.988292217,  -0.589144051,  0.566616058,  0.555767834,  1.18518889,  -0.251828611,  -0.445323676,  0.215678632,  0.0956734195,  1.65986884,  -0.617172182,  1.20252419,  -0.914471507,  -1.1471535,  0.559313655,  -0.408366799,  1.14011347,  -0.500581264,  -0.758159339,  0.104790509,  -1.60984218,  -0.574621439,  1.53723323,  0.24376072,  1.71128714,  -0.659782708,  -0.244780198,  0.154822201,  1.05777836,  -0.423309118,  0.786027014,  -0.416523218,  0.675640821,  -0.874193728,  2.12170196,  -0.31780085,  -0.183178276,  -1.15208149},{ -1.54365838,  1.62677538,  -1.19702744,  1.24296892,  -0.510490358,  1.10425556,  -1.00393057,  -0.975710809,  1.62943268,  0.629844308,  -0.431684762,  0.812709451,  0.0122871241,  -1.33856785,  -1.68490958,  1.09372032,  1.05448651,  -0.545651019,  -0.709642947,  0.593731821,  -1.60689509,  2.21551991,  -0.753350973,  -0.498263299,  -0.17004478,  -0.754521906,  -0.506967545,  0.28465423,  1.40478337,  2.64889693,  0.206895381,  -1.78020346,  0.139306128,  1.39452815,  -0.313708365,  1.11991882,  0.517990112,  0.664286017,  1.12260342,  -1.20532417,  -0.825538218,  1.07539916,  0.343834966,  0.810391128,  2.63209081,  0.498621523,  0.648831427,  1.29505134,  0.913021863,  -0.104938254,  0.982830465,  -0.552596807,  -0.780005991,  -1.1570977,  -0.18717517,  -1.20419562,  2.21832299,  -0.11282479,  -1.97582221,  -0.619670689,  -1.61320353,  -1.64651549,  0.407716244,  0.459272236,  -2.33109236,  0.986987054,  2.54248166,  0.8602373,  0.579056263,  -1.43167388,  -1.79933536,  -0.822512329,  1.61200249,  -1.10767102,  1.34131896,  -0.141932055,  -0.210723311,  0.384372085},{ -1.26113999,  -1.01883173,  0.216860846,  0.542126238,  1.33616412,  1.13476288,  -0.798098922,  0.1404268,  0.873766124,  0.262702107,  -0.256837934,  -1.29151952,  -0.178348377,  1.24781585,  -0.459470153,  1.77261662,  1.21282423,  0.0297486149,  -1.30592859,  0.378459066,  0.841441453,  -0.380218089,  -0.00437988807,  0.927507579,  -0.347135305,  0.374966472,  -1.57916689,  1.33548403,  0.376688123,  -1.26472807,  0.202455714,  -0.79935962,  0.4508425,  -0.0305401832,  0.754272878,  1.02381098,  2.12092924,  -0.0560985096,  0.727027237,  -0.889054596,  0.268655866,  1.35403919,  -0.448730856,  -1.07400882,  0.95994246,  1.48515987,  0.0817440376,  -1.53929329,  0.315898061,  -0.567409992,  -0.556453109,  2.038095,  0.555553019,  0.629996359,  0.704229712,  0.903404713,  -1.57763159,  -1.45012081,  1.38524449,  -0.173908785,  -0.171166986,  0.157047465,  0.467961848,  0.543418109,  1.93908131,  0.226086676,  -0.395085752,  -0.238775283,  -0.366629034,  -1.22071791,  -0.489894152,  -0.644841015,  2.57119346,  -0.868390918,  -0.711602092,  0.590842128,  -0.141669944,  0.45200941},{ -0.578681827,  -0.330022186,  -0.413789958,  1.02953088,  -0.609831214,  -2.49794507,  2.19961119,  -0.827846706,  -1.48027825,  -1.27257574,  0.340013504,  -1.43100023,  -1.30100417,  -0.0695291162,  -0.445894986,  1.26009607,  -1.28394485,  1.24419618,  0.59469384,  0.163998529,  0.0954450443,  -0.787397683,  1.41536295,  0.929818034,  -0.324362725,  0.523391843,  -0.11487253,  0.18540509,  -1.59728777,  -0.00765063521,  -0.606624961,  0.880567193,  0.297194541,  0.89391762,  -0.15602769,  0.25889048,  0.219606727,  -0.667734206,  0.648023129,  0.00771255232,  -0.729457378,  -0.359372139,  0.801768959,  -1.79858875,  -0.777597368,  1.41153288,  -2.23723245,  0.928486943,  -0.81816113,  0.687013865,  0.105555616,  -1.05111194,  -0.501720726,  -1.06358981,  -1.01642752,  1.79474568,  2.06155252,  -0.168350995,  0.885344207,  -1.33087075,  0.770130992,  -0.790671408,  0.66626066,  -1.88443255,  -0.122368209,  -0.479514301,  0.476613313,  -0.170405954,  0.188558146,  -0.542308927,  1.32897449,  -2.06868076,  1.02362823,  -0.204553574,  -0.461819589,  0.141027138,  1.48645353,  0.0889879689},{ 0.674712121,  0.793443441,  -1.33606553,  0.718822658,  -0.579476714,  0.0189179219,  -0.368794084,  -1.73722446,  -1.41397405,  -0.839553654,  -0.933912754,  -0.317119271,  1.42265248,  0.768330634,  0.19385314,  0.374673218,  0.540073752,  -0.30620411,  0.436368614,  -0.236199021,  0.947819889,  -1.45939136,  0.607074738,  -1.10048091,  0.200083748,  0.145350173,  -0.0678542703,  -0.705978572,  0.294535756,  1.05443287,  1.90644252,  -0.0287573189,  -1.13250494,  0.637184143,  1.45406532,  -1.1218549,  -0.368746877,  0.985787213,  -1.45114577,  0.072202906,  0.0195722859,  -1.37419319,  -0.0363615714,  -1.31967914,  -0.478445292,  0.132090881,  0.0603377037,  0.405225992,  -0.773123026,  0.36607641,  -1.40970004,  -1.04459763,  -1.39780021,  0.0786975324,  0.439635277,  0.640112162,  -0.166472703,  0.747550189,  -1.19406092,  -3.12702751,  1.30798781,  -0.525947511,  -1.86400974,  -1.71294665,  0.193356723,  -0.454659522,  -0.0298979096,  -0.859925151,  0.329489321,  -0.3590523,  0.614675701,  -1.99284363,  -0.951623559,  -2.59726691,  -0.817635775,  -0.973495722,  -0.286477596,  1.23280203},{ -0.592418373,  0.269837528,  1.37959731,  0.147543386,  -0.78249079,  0.373952121,  2.28452206,  0.470003992,  -0.428023189,  -1.10093892,  -1.27105057,  -1.16474879,  -1.0225116,  -1.43685925,  0.427427411,  -1.18170762,  -0.802694678,  -0.60900414,  -0.360248953,  -0.235595688,  -0.917047143,  -0.50601089,  -1.05181289,  -0.744715869,  1.36417639,  -0.824817955,  0.0899678916,  0.737525046,  0.343479067,  2.64183331,  1.50127161,  -0.325368673,  -0.799919784,  0.929395676,  -0.542858422,  2.718009,  1.71730125,  -2.20644522,  0.70342505,  0.505268276,  1.24269295,  -0.281912208,  -0.339667469,  -0.431665063,  0.384844482,  1.58391368,  -0.900094509,  0.0972540826,  -0.792755425,  0.126508087,  1.90365314,  0.265106916,  -0.22525984,  -1.32151437,  -0.657948494,  -0.756536067,  0.588095486,  0.0923474431,  -0.460902303,  -1.42902637,  0.352107495,  -1.88895285,  0.668460667,  -1.46417952,  1.08331168,  -1.65814841,  -0.673538089,  -1.09895015,  1.001194,  -0.302347481,  0.416837484,  -1.68915045,  1.80795658,  0.83336699,  0.204420805,  0.846779823,  -0.100460224,  -1.3797437},{ 0.840927541,  -0.996186197,  1.22123599,  0.937837899,  0.157365888,  0.219862297,  0.345427632,  -0.317297667,  0.560111105,  1.84155273,  1.02643549,  0.59666729,  -0.0600290261,  0.25062263,  -1.52289259,  -1.55389571,  2.47979617,  -2.05028582,  -0.0644121319,  -0.840934396,  1.18166971,  0.0065519968,  1.61873245,  0.22177051,  -0.518191934,  0.987971783,  0.275580764,  -0.755915999,  -0.991874576,  0.174699903,  1.15810561,  0.167691484,  -1.04345036,  -0.339534402,  -0.0702281222,  0.872382998,  -0.287111461,  -0.718510687,  1.14196622,  -0.948949277,  -1.92792118,  -0.771124601,  -1.49144638,  1.10817015,  -0.119119495,  1.10320115,  0.730427206,  -0.40523228,  -0.616592169,  -0.0979070067,  -0.570759773,  -0.599993169,  0.673624635,  -0.978098035,  1.01795018,  0.759765327,  -1.18070745,  1.3333993,  -0.36993587,  0.55048275,  -0.410298795,  1.40495813,  0.0628976971,  1.12631476,  -0.938354433,  -0.0905556828,  -1.01142192,  -2.70963693,  0.167422488,  0.812080503,  0.617825687,  -1.28609478,  -1.83487415,  -0.588407636,  1.66986358,  -1.24516487,  -1.85072756,  -0.28771323},{ -1.0984199,  0.916659296,  0.734363139,  -0.364390433,  -0.131692544,  1.1647085,  -1.43405414,  -1.6261797,  -0.216396019,  0.181715637,  -0.490454525,  1.64662921,  -2.23850393,  0.594896257,  -1.744367,  0.258482039,  0.0261987858,  -0.759668708,  -1.33926249,  -0.0540427603,  -2.00308013,  1.50984955,  0.128817454,  0.0127463434,  0.140836567,  0.00402971869,  0.946008265,  -0.0597574227,  -0.30822444,  -0.0611059554,  -1.45799959,  2.1683569,  0.445826918,  -1.1465764,  1.29576433,  -0.965419412,  -2.09945393,  1.55069065,  -1.34784043,  0.12004707,  0.262915134,  -0.237383798,  -1.56115758,  1.34680784,  -1.15945292,  -1.61401677,  -0.362911612,  0.905091524,  -0.346949369,  -0.426937312,  -0.871124685,  0.913710535,  -0.0940982178,  1.1127491,  -1.32339334,  0.778318584,  -0.355352402,  0.992540002,  1.23666084,  0.310335338,  0.475611776,  0.141551137,  -0.116687648,  -0.438489735,  -0.462981761,  -0.411281973,  0.792031169,  1.09413481,  0.0763957724,  -1.08988202,  2.72000909,  1.99257791,  -0.409027696,  -0.611507416,  0.158808619,  1.08066392,  0.908036947,  1.2292496},{ -0.506437123,  -1.11144733,  -0.330111742,  0.450803548,  0.963506281,  -0.324523866,  -0.211112276,  -0.40779084,  0.18689923,  0.0655513629,  1.61148477,  0.132816628,  -1.97929287,  -0.575223744,  0.214347139,  0.0492455624,  0.669645011,  -0.897404134,  -0.0474452712,  -0.418978453,  -0.839170873,  0.252689064,  1.08149612,  0.492714226,  -0.71666497,  0.695976913,  1.81349516,  -0.0775901005,  -1.31650424,  -0.68866694,  -0.150243834,  -0.636103332,  1.9640435,  -0.2538836,  -1.28531456,  1.47927582,  -0.89490062,  -0.0819490626,  0.18760477,  0.480849087,  0.992415428,  -0.0297863707,  -0.278267145,  0.474578738,  -0.639789879,  0.727221787,  1.23869777,  1.69026029,  1.74910784,  -1.81682622,  0.344585687,  -1.9411397,  -1.45993233,  0.0154602798,  0.039692346,  -1.45586157,  -0.16933693,  2.11906123,  -1.48542178,  -0.00655321684,  1.00334167,  -0.973780036,  -0.319507152,  0.0806597099,  0.846679807,  -0.61081773,  0.520620346,  0.675059378,  -0.339226723,  1.3356173,  -1.3665731,  -0.838208199,  -0.633488953,  0.0850010961,  -0.470456481,  -0.798230886,  -0.575710893,  0.444216073},{ -1.95070899,  -1.04184079,  0.225463197,  0.932463586,  1.91050196,  0.923946977,  -0.92264539,  0.67335552,  -1.01891196,  -0.0931402221,  -0.730981886,  -1.65115643,  0.258224696,  0.658363819,  -0.14505139,  -0.611829102,  -0.524676085,  0.742702901,  0.198828295,  1.35646021,  -1.40622342,  0.113472603,  -0.0866526589,  -0.479845792,  2.17334819,  -0.27239722,  2.46375561,  0.0476385392,  0.269845694,  0.491531879,  -0.93584615,  0.778604507,  -0.416542381,  -2.29510164,  0.602133453,  -1.04191649,  0.146254197,  1.15205538,  -0.794314206,  0.603267372,  1.85780144,  -0.0392008536,  -0.46865949,  1.39208639,  1.15597093,  -0.604796946,  0.0571164489,  -0.951183677,  -0.103072524,  -0.766002893,  -0.343885243,  0.791908562,  2.60281205,  -0.0738843679,  -1.99152982,  -0.770968795,  -1.38867319,  -0.803745687,  -0.106031805,  -1.76523745,  -1.24433708,  0.476572305,  -0.893987417,  0.719627678,  -2.70721817,  -0.927634537,  -0.713110685,  -0.335111856,  1.8980329,  1.66822755,  0.0633271188,  -0.586802423,  -1.26266265,  0.845407665,  0.243378967,  0.316635489,  0.0164988525,  0.814662039},{ -0.562123537,  -0.635534585,  -0.463034153,  0.0947434232,  -1.20322955,  -0.746026337,  0.653277993,  0.533626795,  0.636952341,  -1.19716692,  0.234879717,  1.2183826,  1.71468318,  -0.993932843,  0.382563084,  -0.188051075,  -0.748399377,  0.369847596,  -0.0185217001,  0.0573434457,  0.713493168,  1.54170513,  -0.0639766008,  -0.938160241,  -1.73795259,  0.0684924498,  0.21784988,  1.26855421,  -1.40579879,  0.107221231,  0.778890908,  -1.33486724,  2.10441947,  -0.566263556,  -0.370974272,  1.23915327,  -0.45234257,  -0.5614025,  -0.389850885,  -0.933897316,  -0.337593257,  -1.24336493,  -1.91525173,  0.818439126,  -0.420426369,  0.569578528,  1.11258852,  -0.335900009,  -0.979387939,  -1.51131713,  1.72763443,  0.136947706,  0.0418651849,  -0.299508125,  -0.213479996,  0.0375572778,  0.678365052,  -1.90921438,  0.524625242,  -0.340746939,  2.11553216,  -0.482396036,  -0.767526329,  1.63565731,  0.354457796,  1.98015022,  -0.438633233,  0.367248863,  0.96484077,  -0.58419466,  -1.37860966,  -1.23084199,  -0.27541855,  0.073354125,  -2.2193892,  -1.68439531,  1.00013447,  0.980107486},{ 0.669207394,  0.268654525,  1.55363178,  -0.970217168,  0.549007952,  -1.46611023,  -0.1535016,  1.44431794,  0.748643637,  -0.430912524,  -0.0440568849,  1.07874417,  -0.666012406,  0.736781299,  0.681013107,  0.656384408,  -0.826300502,  -0.591186583,  0.970959187,  -0.0183291472,  -1.86814106,  -1.62147534,  0.281853408,  -0.163411483,  -0.956541002,  1.42144489,  -0.849981248,  0.600888729,  0.378171504,  -0.436299503,  -1.50507891,  0.589768589,  -1.99068069,  -1.73355997,  0.75371325,  1.02500582,  -0.405243546,  -0.938273072,  -0.964980602,  -0.0848744139,  -0.475738227,  -0.465569556,  -1.28951025,  -1.4082197,  0.469001353,  -0.367621332,  0.176954731,  0.848333895,  -0.374187469,  -0.11121887,  0.698811114,  -0.985764861,  -1.77504396,  0.0716772527,  -0.563375711,  0.815781116,  0.136148751,  -1.39309072,  -0.389994889,  0.355817467,  0.989535749,  0.592105269,  0.0692403167,  -0.400377095,  -0.645844221,  -0.285534143,  0.465067863,  -0.297249258,  0.527570844,  0.410432696,  -1.0021168,  -0.723706961,  -2.1452713,  -0.0725960061,  -0.705663025,  0.290502131,  1.29072142,  0.438949019},{ 1.76630449,  -0.0315154083,  -0.757159293,  -1.0124135,  0.589464128,  0.54215157,  -1.58892918,  0.158290148,  0.468990654,  0.729208529,  0.148166761,  -0.803928554,  -0.110967956,  -0.0891803801,  -0.449259937,  -0.32220301,  -1.12287664,  0.00486014178,  0.440371901,  2.42941952,  -0.584120631,  -0.633563995,  0.788195431,  -0.350442946,  1.32576132,  -0.105241463,  -0.910393834,  -0.430284649,  -1.04346383,  -0.744956672,  0.808858633,  0.337150425,  0.53558135,  -1.12523723,  -1.0365696,  0.261092782,  1.14898062,  -1.66174686,  0.410617441,  0.0498190112,  -0.188560724,  -1.64259279,  -0.141311169,  -0.198259234,  2.05090237,  -0.526318908,  -0.250853688,  1.20967257,  -0.5843274,  0.278343052,  1.2225703,  -0.610085011,  -1.19294572,  1.19183612,  -0.338715911,  0.668089688,  -1.13550591,  -0.770189166,  -0.625230908,  -0.870587826,  0.105822653,  1.44983876,  0.598358393,  1.08298421,  1.11255789,  -0.442379236,  -0.0792526528,  0.929372847,  -0.230378121,  0.267227739,  -0.340089828,  0.0546684153,  -0.64630276,  1.00003839,  -1.98249233,  -0.194688007,  -0.589291811,  -0.194516852},{ -1.24482596,  -0.643559396,  -0.0277179535,  1.01405644,  -0.498286247,  2.01909494,  0.183729589,  -0.849964261,  -0.490355253,  0.000859952881,  -1.63675046,  -0.433186293,  -0.864343762,  -0.174302459,  0.311391503,  2.29141688,  0.252761334,  0.00613452215,  1.33706605,  0.286448628,  -0.673946261,  -0.0159253702,  0.787539303,  -1.68125355,  -0.114872396,  0.795449853,  -1.40646267,  -0.885067403,  -1.20124078,  -0.0564913824,  -0.972834885,  -1.78869629,  -0.458474785,  -0.403383225,  0.122546531,  0.0256712213,  1.7305398,  0.518645287,  -0.778533399,  0.0941699222,  -0.969569981,  1.22780657,  -0.220146433,  -0.678769648,  -0.513035774,  0.125983596,  0.761947155,  -0.0434851088,  1.45529222,  -0.593918383,  0.548372209,  -0.391343355,  2.95864725,  0.772554517,  0.823669434,  0.0886403769,  -0.313241959,  -0.485918611,  0.426290512,  1.27425563,  -2.23797846,  -1.58733666,  1.94499624,  0.387216598,  0.722188234,  0.50860858,  0.557857871,  -1.4930402,  -0.422759235,  0.200309083,  0.595187724,  0.683266938,  0.638363183,  0.0106685339,  1.33546925,  -0.876998067,  -0.964894712,  -1.3315568},{ -1.85338545,  -1.07833719,  -0.254305512,  -0.846572161,  0.490639418,  0.915142477,  -0.910978854,  0.0101301596,  -0.539183855,  1.3773185,  -0.413882881,  -1.48509967,  -0.315473288,  0.892364562,  1.17044342,  -0.153001279,  -0.812244117,  0.922956944,  0.88225168,  -0.289537579,  0.207297534,  0.503234684,  -1.63810813,  -0.599624693,  0.849994004,  0.938019514,  -0.427801132,  1.65117764,  0.289983898,  0.318541199,  0.373098046,  -1.42388725,  1.24320912,  0.695092082,  0.712198079,  0.14763239,  1.81126392,  -0.281159371,  0.849030435,  -1.2590102,  0.0637477711,  -1.00533986,  -0.287787378,  0.303930879,  -2.24218798,  1.35114443,  -0.632731616,  -0.433956295,  -0.345753819,  0.142318904,  0.294075131,  0.601050138,  -0.846952021,  -0.130340502,  -0.0895878896,  -1.54940271,  -0.237358987,  0.656498671,  2.72590446,  0.0787890032,  -1.33915031,  -0.363437414,  -1.55835366,  -0.839311719,  -1.22870636,  0.593680501,  0.859531879,  -1.19425797,  -0.160495237,  -0.101401098,  -1.47837639,  0.423151731,  -0.59591341,  -1.67660201,  -0.823613822,  0.94028604,  -1.05159485,  0.293047756},{ -0.353190154,  1.38507915,  -0.524877369,  -0.0249064285,  -0.21818541,  -0.968526185,  -0.500713646,  0.461991996,  0.39154321,  1.68090105,  -0.410051972,  -0.488658309,  0.849917352,  -0.290008545,  0.813916445,  0.700785398,  -0.218669295,  0.471427411,  -0.831702173,  -0.590823114,  0.207498625,  2.21049595,  0.0179141965,  -0.877346098,  -2.07320619,  0.83914125,  0.183066607,  0.207032382,  1.26205397,  -0.647667587,  -0.382883817,  -1.1960572,  -0.292864233,  -1.89357448,  -0.89576304,  0.681507468,  0.117124982,  0.318812281,  -1.21936643,  -0.589311063,  0.494299412,  1.86008584,  2.49549317,  -0.510007501,  1.29682481,  0.633889139,  -0.349888593,  -0.524156034,  1.15365517,  0.14401634,  -1.54340875,  -0.484365165,  -1.0841819,  -0.0167276431,  0.400610834,  0.104931243,  -0.357020587,  0.489135295,  -0.932825565,  -0.710719824,  -0.182553202,  1.85735095,  0.444471478,  -1.51735294,  -1.2745074,  -0.91167897,  -1.03157926,  -1.40636551,  0.440562487,  -0.872830033,  0.834297419,  0.368738383,  -0.0516489074,  1.0093224,  -0.47136423,  -0.411767334,  -2.14318609,  -0.223766178},{ -0.304970443,  -0.257580698,  -0.22354947,  -0.547437847,  1.5077517,  0.172971785,  0.201787576,  -1.65125942,  -0.533577621,  -0.585364044,  -0.554065585,  -0.0956291109,  0.949867427,  -0.33715269,  0.128933817,  -0.656049728,  -0.111050859,  -0.528864145,  0.93931973,  0.503357649,  1.97598612,  -1.24832487,  -0.884325266,  -0.70232749,  0.568981409,  1.17324865,  -1.38961995,  -0.172332257,  -0.0110316901,  -0.909392715,  -1.21462107,  -0.0800497308,  1.99727106,  -0.533151686,  0.0454384163,  0.723043323,  0.61819011,  -0.200489745,  -1.27001071,  0.00537839858,  -0.751722097,  1.34931636,  -1.15010655,  0.395188481,  1.46667027,  -0.230664626,  0.888339758,  -0.586766303,  -0.183452681,  -0.639247179,  0.939659655,  1.73994136,  -0.958405375,  1.32682049,  1.34577763,  0.98467195,  0.217119515,  0.273213118,  -0.820800543,  -0.861731529,  -0.224496484,  1.71537232,  -0.425186604,  -1.87080991,  0.101448156,  -0.83255285,  1.43564355,  0.302024841,  -1.08726943,  -0.618966162,  -0.389898568,  -0.538222969,  -1.11356115,  0.408783346,  0.0581661016,  0.764305651,  1.21584797,  0.0272462424},{ 0.202084824,  -0.613965809,  1.54578137,  0.208999678,  -0.902681768,  1.06378722,  -1.51595187,  -0.164459243,  0.900833488,  -2.01878977,  -0.969691575,  0.541745067,  1.16302907,  0.04416354,  -0.221596971,  0.748961151,  0.787261724,  1.42147553,  0.90792954,  -0.343075186,  -0.982818127,  -0.670981288,  -0.257767141,  -3.20807703E-05,  -0.828595638,  -1.79078567,  -0.321077377,  0.464639813,  1.5758791,  0.580041409,  0.906344235,  0.438133478,  1.3902396,  0.0290373564,  1.46973717,  0.920211017,  -0.587981343,  -3.16963601,  0.566817582,  -0.295635581,  -0.469351232,  -1.70839608,  -1.25124311,  0.921329796,  0.69258666,  0.176697329,  -2.78019452,  -0.333439589,  -0.433369488,  -1.75926507,  -1.34474373,  -0.551675737,  -1.01704741,  0.470688552,  -0.218083262,  0.433168262,  1.89436543,  0.254746258,  -3.03770065,  0.430748343,  0.613395035,  0.160535172,  -0.452896655,  1.25853705,  -1.359887,  -0.0954970643,  0.312244236,  -0.374844193,  -0.346425235,  0.115689144,  -0.138503045,  -0.550212383,  0.0960645676,  0.407017201,  1.49643219,  0.0416798703,  0.456661075,  1.58038676},{ -1.97120726,  2.91944003,  2.36120129,  -0.679241121,  0.183349535,  1.25242531,  -0.595517814,  0.496174365,  0.392564088,  -0.326694876,  -0.0604627058,  0.239534467,  -0.0874744803,  0.184044883,  0.916956663,  1.11242676,  0.0628187507,  0.414498061,  0.45122993,  1.39705634,  -1.1618135,  -0.115277044,  -0.745138168,  0.221411243,  -1.92955065,  -0.0274030957,  -1.13383961,  0.751419365,  -0.452752352,  -0.233841375,  0.358823746,  -0.555662632,  0.0323691033,  1.7110604,  0.485781103,  0.426601768,  1.15700638,  -0.293274581,  -0.707468927,  -0.192492455,  0.793126285,  0.470098943,  -0.794893265,  -1.5413667,  -0.278369129,  0.183737516,  0.892369211,  -0.646549821,  -0.246822745,  -0.194455147,  0.927798629,  -0.556046903,  -0.320782959,  0.238063917,  -1.22208261,  -2.22207999,  0.672650933,  0.307128698,  -0.0307558347,  -0.171119153,  -0.293726325,  -0.0996836126,  -0.643356681,  -0.2750507,  -0.382558078,  0.480523378,  -0.39762795,  0.759653389,  -0.821201801,  0.463065237,  0.208534613,  0.27782774,  -0.552162468,  1.11389971,  0.777756274,  -0.834110975,  -0.362823814,  0.0562742949},{ 1.57340646,  1.93166995,  -0.977068126,  -1.28301954,  0.57121551,  0.0668923259,  -0.590870023,  0.552483916,  0.694144249,  -0.0971201435,  0.601158738,  0.0909056887,  0.991578877,  -0.304596722,  1.39288795,  2.0529778,  1.58744383,  0.0521718375,  -0.714214921,  -0.468449771,  0.300032973,  1.13514495,  -0.557591915,  0.271670848,  -0.0584138893,  1.23211658,  -0.672396421,  -0.112793215,  -0.321785778,  -0.183775231,  0.149251387,  0.267366827,  0.741971195,  -1.49313676,  0.566429973,  0.714997411,  -0.560803771,  2.56390119,  -1.04515636,  -1.57994211,  0.639421046,  0.374681354,  -1.75416958,  -0.98951,  0.393779248,  -0.759918094,  -0.0290286392,  1.12412763,  -0.0439260304,  -0.0417072438,  -0.900467932,  2.27164316,  0.179415256,  -0.713574529,  0.540612578,  -0.942644238,  -1.45396936,  -0.942477167,  1.44832337,  2.02457237,  -0.553771973,  -1.83548272,  -0.780941486,  -1.38237786,  0.45263344,  0.641573548,  -0.700606704,  0.74024868,  1.13546824,  0.789646327,  -1.56031668,  1.41433716,  -0.950694203,  -1.00813866,  0.335974276,  2.15746212,  0.9336918,  -0.189072624},{ 0.865432382,  1.36199141,  0.249082983,  -0.320976436,  0.0175322071,  0.825790465,  0.569665134,  0.998491108,  0.155553833,  -2.46042013,  1.41123092,  -1.16743946,  -0.791918874,  -1.44614756,  0.390280783,  -1.42614841,  -0.448122889,  0.106636271,  -1.70956862,  0.0446842946,  0.571534097,  -0.521270454,  -0.352220953,  0.00693483138,  -0.015214406,  0.697821021,  -0.226691306,  -2.69233322,  1.15056992,  -1.53514421,  -0.195034131,  -1.30360866,  1.73708069,  0.620681167,  0.33166796,  0.388563365,  0.870476305,  0.401216596,  -0.102933101,  1.08015156,  -1.0505724,  1.38459933,  -0.528594553,  -0.197015509,  0.240298852,  0.683880627,  -0.844962418,  -0.802299559,  -1.17275155,  1.69420505,  0.370784044,  -0.986905277,  -1.06200254,  0.343849242,  -0.805234432,  2.62035632,  0.48224619,  -0.874489844,  -0.391070575,  -0.772755682,  -0.161182299,  -0.267285556,  -1.60259736,  -0.797725081,  0.398477435,  -0.560539842,  -1.05240536,  -0.143363729,  1.12541902,  1.57426286,  -0.0508971587,  -0.998524666,  -1.53433537,  0.0668851361,  0.444545984,  0.555164933,  0.202457294,  -1.74522376},{ 0.450575262,  -0.265812218,  -1.30280602,  1.36523008,  -1.42838347,  -0.836192131,  -1.35226774,  -1.64679301,  0.921598196,  1.88437843,  -0.245774746,  -0.294625759,  0.4157103,  0.435912669,  0.580532312,  -0.999090254,  -1.1557107,  0.455909938,  0.54413408,  -1.21589291,  0.224853039,  0.242698729,  -0.182709724,  -0.313379169,  0.113402747,  0.600477457,  -0.310965359,  0.9296242,  -0.657255173,  1.34238434,  2.06778121,  0.660898566,  1.60212934,  -0.960723758,  0.19641377,  -0.776842594,  -0.991768539,  0.335823417,  -0.243853003,  -0.27758646,  0.663621306,  -0.68457371,  -0.270711273,  0.573275805,  0.619436383,  -0.107900411,  0.327042282,  -0.159884393,  -1.12681413,  -0.996222496,  0.322748065,  -0.855922222,  -0.217701495,  1.59259176,  0.74859041,  -0.13831228,  1.2460165,  -2.04138589,  0.60586822,  -0.291808188,  -1.99200034,  -1.21930623,  -1.07909322,  -0.651768982,  -1.17530215,  -0.369561851,  0.689698696,  -0.817013443,  0.0579255223,  -0.428586662,  1.47098255,  0.0148912631,  -0.130556524,  0.488886505,  -0.187300712,  1.32952404,  1.41735208,  -0.315312356},{ 0.128567308,  0.0346443206,  1.03277588,  -0.363896519,  -1.21971965,  0.998949409,  0.638273418,  0.148498982,  -0.545573592,  -0.381658763,  -2.19937778,  0.0756232068,  1.354743,  -0.264179736,  0.319016069,  -0.867592812,  -0.486420959,  -0.21403563,  1.46157932,  -0.179030299,  -0.432193518,  0.444762826,  -0.0520919152,  -0.0518838279,  1.44650662,  0.949382544,  0.0232684482,  0.161045656,  1.34172916,  0.567725718,  1.22757912,  -0.850905061,  0.098257117,  1.2826581,  -0.699967146,  -0.0177728925,  1.46782696,  0.27444759,  -0.455148101,  0.522398293,  0.391549498,  -0.452878803,  0.000386978441,  0.250942439,  0.276296496,  -2.19629955,  -0.784829617,  0.591081262,  1.19893336,  0.964959979,  -0.476964414,  0.0386206657,  1.10762334,  0.595172286,  -0.0291444827,  1.07339931,  -0.864666641,  0.922081888,  0.262113154,  -0.141397998,  -0.992800355,  1.09948754,  0.42550385,  1.52903771,  0.940409958,  -1.01126337,  -0.374796808,  -1.9714371,  1.81232631,  -0.0171556231,  0.364310443,  -0.301233917,  -0.190109313,  0.711822271,  -0.354490846,  -0.511434615,  1.62253857,  -0.0355067737},{ -0.107341662,  -2.14622045,  1.18288898,  1.06022143,  -0.708836257,  -0.453277588,  0.34522751,  -0.378292441,  1.56610763,  2.12088561,  0.538944662,  0.0817563534,  0.74730587,  -1.41240501,  -1.3454783,  -0.614285827,  -0.676469922,  -2.22798944,  -0.0901073292,  0.869579136,  -0.0488768257,  0.817358732,  -0.206652567,  0.853162289,  1.78733265,  2.58461213,  -0.163257867,  1.6376096,  -1.331532,  1.30195785,  -1.01224148,  -0.145494744,  0.290305048,  -0.655888617,  -0.38662523,  -1.60618818,  0.70464772,  -0.533928096,  -1.2952379,  -0.529728591,  -0.041385524,  1.27659774,  0.245502621,  0.420765966,  -1.96900737,  0.00846847333,  1.11899698,  0.401274979,  -1.7157011,  -0.894185424,  -0.366591543,  -0.666741848,  0.84470892,  0.0969289988,  0.625676692,  1.13242924,  0.590052187,  -0.089623943,  -0.168354452,  0.467726022,  1.09330511,  1.61672342,  -1.31455958,  1.16052377,  0.228734881,  0.85587281,  -0.474249721,  -0.175946221,  -0.199892893,  -0.455372304,  -0.865533948,  0.231185004,  0.635797024,  1.3830905,  1.68571746,  -1.4849577,  0.962752283,  1.22707367},{ 1.9048084,  0.858384967,  -0.324553847,  0.111555599,  0.863354623,  0.659527004,  -0.210010916,  -1.82528961,  1.11661184,  1.26039183,  -0.359398931,  -0.311494678,  0.15760386,  -0.233398303,  0.736575663,  2.06020188,  -1.43824971,  -0.947946787,  -0.0340082385,  -0.285257638,  0.111559533,  -1.16006875,  -0.342551947,  -1.43537676,  -1.09613621,  0.349545598,  -0.18333663,  1.07897043,  0.323093593,  -0.835175037,  -0.794286549,  -1.06399024,  -0.791885138,  -1.16991055,  0.441269964,  0.351542294,  -0.872956634,  0.549278557,  -0.264127016,  -1.08238125,  0.210554913,  -0.238798067,  1.43695533,  0.0601742454,  1.67979228,  0.0197825264,  0.234171316,  -1.00766933,  0.263576925,  -0.64769268,  2.37663579,  0.438130349,  0.160979226,  -1.58384299,  0.898116529,  -0.135917917,  1.59352303,  -0.339997619,  -0.253263235,  -0.507695436,  0.280747682,  0.895540297,  -0.484630734,  -1.85041344,  -0.578825951,  0.105066925,  0.04880495,  0.387732118,  -0.106969833,  1.42611599,  0.499515831,  -2.03099442,  1.33738887,  0.132619113,  -1.20092344,  2.92666554,  0.879698038,  0.550309658},{ 0.194075122,  -2.23893666,  0.360769808,  0.0987490341,  0.585705578,  0.768685222,  1.33083272,  2.0646708,  1.32837725,  0.261562437,  -0.799720645,  0.175255492,  0.224567473,  -0.0791529343,  -0.495305091,  0.415787309,  -0.329887539,  -1.52721536,  -0.927764356,  -0.449796706,  -1.1881547,  1.4951489,  -0.326916695,  2.27057052,  -0.452082306,  -0.862919986,  -2.0750587,  1.36718118,  2.23981071,  -0.123547964,  -0.194897413,  -1.8455143,  0.020291226,  -0.162884846,  1.24808991,  -1.24355721,  1.08679318,  0.334277868,  0.453312904,  -0.140355244,  0.945890367,  -0.704991162,  0.702549994,  0.445903897,  0.652967691,  0.0796974376,  -2.00436378,  -0.0985920057,  0.37496528,  -0.892869473,  1.03099167,  -2.12330914,  0.35326916,  -0.779839873,  0.132001728,  -1.04956663,  1.58096647,  1.80422091,  -0.492305309,  0.74666518,  0.475640923,  0.378575116,  0.514054596,  -0.492077202,  -1.19223857,  2.54865789,  -0.749396086,  -0.180472359,  0.320427895,  -0.585415125,  -1.05297828,  1.34527731,  0.867098808,  0.412463576,  -0.937329054,  -2.04687119,  -1.08134854,  0.857293427},{ -0.603523135,  -0.834717989,  -0.892144263,  -0.733322442,  -0.286907285,  1.30472648,  -0.237382501,  -0.149192557,  0.58141762,  0.918905795,  0.613042653,  0.645643175,  -0.0547542088,  0.582394779,  1.25014329,  0.444913119,  0.821580887,  -1.07687736,  0.579897523,  -1.0713259,  -0.311550111,  -0.233587027,  -1.06544912,  0.439977169,  1.92010295,  0.0108371396,  0.652225733,  -0.460555851,  1.89463389,  0.973885119,  -0.188072741,  0.834343672,  0.010145071,  0.124721855,  -0.872182012,  1.67524922,  -1.17438054,  0.318069935,  0.899316847,  -1.22554076,  -0.05771989,  1.03196609,  -0.0300053321,  -1.15026534,  1.23356164,  1.5495472,  0.344327688,  -0.971982598,  -1.02273989,  -1.13178539,  0.598109126,  1.47834766,  0.496348321,  -1.92787504,  0.962085128,  -0.610829473,  -0.798675418,  -0.299518198,  -0.525552154,  0.177204862,  -1.21328533,  -0.462466925,  -0.510841668,  0.443080634,  0.388036788,  -0.430305988,  -1.19665575,  -1.91741621,  0.938323677,  0.256753236,  -0.454380363,  -0.874071836,  0.624744534,  0.231056705,  0.708744586,  -0.209287837,  0.180958375,  1.47780061},{ -0.315945774,  0.0520009845,  -0.708321393,  1.76427376,  1.23513484,  1.28604639,  -1.19806814,  -0.691548586,  0.134493083,  -0.0129394429,  -0.391804785,  -2.24928737,  0.660343945,  -1.3225764,  -0.144299224,  1.52170324,  -0.849655151,  1.46178281,  -0.881158829,  1.0725764,  -1.0381577,  0.0597563237,  0.612447262,  0.907823741,  -2.5284307,  0.295018315,  -0.835016251,  0.199757725,  1.1272862,  0.227303445,  0.37882188,  0.0523488,  -0.109258033,  -0.351484805,  0.140138149,  0.832439423,  1.68637419,  0.392768979,  0.729661226,  -0.128245801,  -0.155424967,  -0.19427225,  -0.993979216,  0.873600543,  -0.992284894,  -1.26666808,  -1.48544097,  -2.33449697,  -0.679628491,  -0.611711383,  -2.14142108,  -0.00106978149,  -0.248372972,  0.569255233,  -0.79397428,  0.725802183,  1.99977875,  1.28407061,  0.888371587,  1.23103082,  0.138507277,  0.470434815,  -1.60337865,  1.03767836,  -1.05585408,  -0.347107887,  0.225592673,  0.109427437,  0.184873194,  -0.850703001,  -1.15737712,  -0.392672271,  -1.6641134,  1.18015587,  0.385628641,  0.15606761,  -0.354837984,  0.965257645},{ -0.758568645,  0.864623785,  0.437453389,  1.21732426,  -0.285660952,  0.811255813,  -0.851290345,  -1.15645397,  -0.939874351,  -0.881096661,  -1.46383536,  0.507221758,  -0.577833593,  -0.806660175,  -0.715542436,  -1.32926798,  0.3539702,  0.645572484,  -0.149613649,  0.0332667604,  -0.83788228,  2.10851717,  -0.891908705,  -0.570730686,  0.0356797837,  0.24529843,  -2.24444866,  1.05045462,  0.768330634,  -1.13062251,  0.259453118,  -0.278995097,  -0.267420471,  -0.38926059,  0.0285106823,  0.183313265,  0.598172486,  -1.52134871,  1.49558651,  0.353423208,  1.71032846,  -0.175439194,  0.450496346,  -0.442947149,  1.10078597,  -1.06659198,  0.0754665807,  0.845661342,  0.215568528,  -0.987287879,  -0.216421083,  1.64254701,  -0.282045126,  0.692829967,  0.155965805,  -0.818261862,  1.00024581,  0.402273685,  0.15767853,  1.64557111,  -0.554890454,  0.0630530939,  1.44240677,  -0.316983283,  0.072940439,  -1.28253138,  -1.79032469,  1.66837347,  -0.503693044,  -0.86352855,  -0.0637138933,  -0.221395642,  0.786759257,  -0.37082848,  0.765990555,  -0.0332652181,  0.681499302,  1.09053326},{ -0.256260961,  2.18349433,  0.566308677,  -0.304829717,  -0.369119883,  -0.176605538,  1.37198973,  0.431380063,  0.199291036,  0.539955616,  1.85195112,  -0.105755851,  0.738736033,  -0.366585642,  -0.687835276,  -1.45952094,  0.22252503,  -1.60257649,  1.40370083,  2.07562256,  3.06406784,  2.09326696,  0.0959249213,  0.790169477,  1.33213305,  -0.638791323,  0.121928029,  0.121235453,  0.142171293,  -1.95168531,  -0.8023206,  -0.118097506,  1.02151358,  0.287990808,  -0.314059913,  -0.0921623334,  -0.255478233,  1.89058506,  -0.134320736,  -0.27923733,  0.447945327,  -0.957177937,  0.38912496,  0.921718836,  -0.489752233,  -0.467357606,  -2.46141887,  -0.323502392,  -1.57429993,  -0.267208546,  -0.118966326,  -0.740540981,  -0.773241699,  1.41098905,  -0.302360296,  1.79557693,  2.4170897,  -0.553520977,  -2.01059318,  -1.53480434,  2.18238616,  1.16586268,  -1.03394735,  0.0789247081,  -0.7737602,  1.16092908,  -0.516851664,  0.0965311825,  0.781332314,  -0.0980175808,  -0.809453666,  -0.357719243,  1.22123444,  -1.00679243,  2.23980904,  -0.185636371,  -0.885480642,  0.776180565},{ -2.42089653,  -0.990595818,  -1.52440655,  2.00534463,  -0.639071047,  -0.068076469,  2.35957575,  -0.465949476,  -0.1199205,  -1.13157105,  -0.138828933,  0.563786149,  1.94544435,  0.324868828,  2.23743939,  0.932136953,  -0.907082915,  -1.04057598,  -0.177180365,  -1.32794416,  0.579210341,  -0.325069219,  1.09527242,  0.802707851,  1.58748913,  1.10923874,  0.0171763226,  -0.901765466,  0.489120275,  -0.39470309,  -0.233200371,  -1.33705974,  -1.48627353,  1.81749463,  0.674126029,  0.0804139227,  0.618192554,  -1.25161898,  -0.0114245871,  0.275512338,  -0.739543796,  2.09222817,  0.694420457,  0.627061427,  -2.44854426,  -1.17705083,  0.100745708,  1.77013493,  -1.5743444,  -0.518833518,  -0.887458801,  -0.471276402,  0.196126312,  1.42869425,  -1.29340422,  -0.346703678,  -0.617402494,  0.13509132,  -0.203560084,  -0.91332674,  -1.23311949,  -0.360627979,  -0.276432693,  -0.208722621,  0.873648763,  0.976589859,  -2.29414177,  -1.62028098,  0.598818779,  -0.959030211,  0.158680946,  0.244784281,  -0.0942120552,  -0.323519796,  -0.769822001,  -0.163967982,  -0.918415904,  -0.320647985},{ -0.64708972,  -0.255283833,  -0.815057039,  0.121702857,  -0.340655506,  1.24638009,  -1.57482469,  1.69561064,  0.281127691,  -1.56957209,  1.42674184,  -0.127491474,  -1.12360084,  -0.528599679,  -0.665708423,  1.85554492,  -0.100620247,  0.463498503,  0.585884273,  -0.373798788,  -0.822275937,  -1.01192856,  0.536641181,  -1.03695261,  0.0989439636,  0.148308873,  -0.949974835,  1.41775525,  -1.20519567,  0.241697341,  1.10340667,  -1.38924611,  0.848240912,  -1.50373101,  -0.607527792,  -0.694436431,  -0.472000003,  0.159123138,  0.598443925,  -0.694652259,  -0.29298225,  -1.32333291,  -1.7012738,  -1.93035066,  -1.95989609,  1.11957145,  -1.41359985,  0.27951768,  -1.87369049,  -1.05564356,  0.636599302,  -0.361185968,  0.100648977,  0.429417551,  -0.569484413,  0.744276822,  -0.671364188,  1.78859472,  -0.393150896,  0.276740581,  -0.943086863,  -0.0133450013,  -0.00599088892,  0.78320688,  -1.79740357,  0.285863221,  1.47051847,  1.20969236,  0.64825511,  -0.157653004,  0.123303168,  -0.411982208,  1.92802715,  -0.288497299,  -0.0663029775,  -0.55741632,  -1.11173987,  -0.540537775},{ -1.54358363,  -0.382133633,  0.184381589,  -0.369750768,  -1.31364107,  2.46618319,  -0.937788785,  0.349868745,  0.375496775,  -0.364227146,  -0.519673347,  -0.794443667,  0.0435607731,  0.700289905,  2.05796385,  0.919006228,  -0.922487199,  -0.11224629,  -1.26384187,  0.156170532,  0.523333788,  -0.319270998,  -0.475784749,  1.82625687,  1.37220454,  -0.750394881,  -0.153064355,  0.389599621,  -0.307730466,  0.165762827,  -0.039497517,  -0.702209771,  2.55331087,  1.48032701,  0.680998147,  -0.773003697,  -0.568135023,  0.121175714,  -1.14118779,  -0.0373409018,  1.10474062,  0.890356779,  -1.96875882,  -0.763085306,  0.848650634,  2.31998396,  -1.92999458,  -0.82017225,  0.747840285,  0.0363870598,  -0.460508436,  0.313696265,  0.270833433,  -0.587825179,  -0.0187169332,  0.265220076,  0.59626472,  0.474416882,  0.792723119,  0.41263181,  -1.49689269,  2.8436203,  -0.417352021,  -0.648895741,  0.615634441,  -1.58312321,  -0.101211905,  1.00686336,  1.72861028,  0.960570037,  0.506116807,  -0.183223099,  1.41540885,  0.729917228,  0.946779072,  -0.503621459,  0.0581563264,  0.78666085},{ -0.40008086,  -2.70659399,  0.466189861,  -1.19311404,  0.780309737,  0.723558366,  0.452390134,  0.0800936148,  -0.870488048,  -0.689416051,  -0.360071808,  0.89637506,  -1.17629766,  0.656710505,  -0.330862522,  -0.471168697,  -1.36748362,  0.433955908,  -0.413022757,  1.67781091,  0.820014715,  -1.78949547,  -0.646138012,  0.239500806,  2.23781037,  0.743309438,  -1.2826041,  -0.667937756,  1.57145202,  -0.207066238,  0.189689979,  0.634754181,  0.204599783,  -0.114696555,  0.32397747,  0.28283,  -2.04795885,  -0.780131936,  1.53580987,  0.812201858,  -0.0958793089,  -0.136547357,  -2.13116789,  -0.153194398,  -0.349857628,  -0.984906733,  0.74808228,  -0.951513052,  -1.28085351,  -0.369781226,  -0.174649462,  -0.941897631,  0.563071012,  0.746949732,  -1.66402721,  0.907119036,  -0.499258935,  -0.0135351419,  -0.62786752,  0.0210270658,  1.48058546,  -0.912673771,  -0.955394804,  0.193377122,  -0.244878501,  -2.42187047,  -1.04455447,  0.708885849,  -0.112479486,  0.0524522476,  0.243023634,  -1.21932113,  -0.562181711,  0.60885191,  1.78222942,  -0.207579345,  0.295887381,  -1.18408656},{ 0.18390283,  -1.27286792,  -0.488721102,  -0.784094572,  -1.35474324,  0.882885754,  -0.397226363,  -0.246715754,  -0.210798606,  -1.1403358,  -0.710711479,  0.601634264,  -0.372212052,  -1.34914148,  1.35544217,  0.0512619205,  0.662543416,  -0.416256011,  -1.11141336,  -0.164906204,  -1.36605644,  -2.37187243,  -0.17545408,  0.602957726,  -0.399302721,  -1.14713657,  -0.239243507,  2.2642417,  -0.654290378,  1.34139979,  0.0678276047,  -0.432281435,  1.16290891,  -0.193851173,  0.502684414,  -0.364461869,  -0.455084652,  0.858428717,  0.353639215,  -1.61012864,  -2.2840302,  0.341622233,  0.607278943,  -2.25530553,  0.46150735,  0.333890587,  -0.0552870184,  -0.033044409,  -0.272367865,  -0.718988955,  -0.166375235,  1.37032187,  -0.892540336,  0.26248607,  0.689710736,  1.11365879,  -1.82305956,  -0.987588942,  -0.147365257,  -0.16458936,  0.902907968,  0.236230969,  -0.875345767,  0.744942069,  0.226601571,  -0.189037517,  0.0870998502,  -2.59939504,  0.195867285,  0.629171968,  0.314730465,  -0.104626693,  -0.465790451,  -0.206721783,  1.53907871,  0.609051406,  -0.0841266215,  -0.281366944},{ 0.294412345,  2.04917789,  -0.35300836,  -1.06616449,  0.110334538,  -1.69426942,  2.24648595,  -0.166117311,  0.542726696,  -0.83697921,  -1.33157742,  -0.405365318,  1.08737242,  0.806744099,  -1.17353857,  -0.331092745,  -1.62822664,  0.707692564,  -0.757082939,  0.587705672,  0.579786718,  2.49523449,  -1.33959234,  0.176628724,  2.03253984,  0.321397781,  0.0981415957,  -1.34859037,  -0.305055857,  -1.70687044,  1.28770876,  -0.499999166,  -1.15341604,  -0.49449119,  -0.871248186,  -1.01635492,  -0.777821362,  0.730961144,  0.362868607,  0.569848418,  -2.1276803,  0.782260358,  0.50293225,  -1.77382004,  1.7187742,  1.37347329,  1.74431062,  -0.141276658,  0.403335363,  -1.54380345,  1.43712473,  -0.495630264,  -0.693233252,  -0.761156678,  -0.232261688,  -0.383777887,  -1.26238537,  -0.503499448,  1.56523168,  1.05336797,  0.533300042,  0.257884026,  -0.742046654,  0.871294141,  -0.821755946,  -0.377589226,  0.260620803,  -0.0761542469,  1.33649468,  0.582991064,  -0.552692294,  -0.47800988,  1.40612054,  -0.255376369,  -0.603727341,  -0.23354584,  -0.265829593,  1.77861762},{ -0.19561474,  0.432290882,  -0.1416789,  -1.75065708,  0.621874094,  0.420465678,  0.300308257,  -0.960030854,  0.268858343,  -0.566739619,  1.67302966,  0.42003274,  1.59464014,  -0.0607370995,  -0.475160658,  1.59880793,  1.36500132,  -1.22914839,  -0.84579432,  0.522344112,  -1.30257118,  1.17594492,  0.439934045,  -0.877981067,  -0.867610693,  -0.176338792,  0.296371371,  -1.60151529,  -0.895720601,  1.31374133,  -0.280966133,  0.902427793,  -0.540408671,  -1.16219747,  -0.868077159,  -1.96485186,  -0.393252701,  -0.0602156334,  -1.50398719,  -0.28652367,  -0.997830927,  -0.161821619,  0.079263933,  0.152730852,  0.302043349,  1.51330578,  1.30826259,  1.5506928,  0.619053841,  0.30473572,  -0.175080195,  0.175456673,  0.445175558,  0.55032289,  -1.15124166,  0.42296192,  1.32485878,  -0.0193072241,  -0.373175591,  -0.933741331,  -0.956213057,  -0.322288543,  1.06624269,  -2.05912328,  0.165800229,  -2.75325751,  -0.656476319,  -0.00520336069,  1.67927051,  0.101376362,  0.0391985849,  -0.154870778,  0.697867155,  -0.690200806,  0.969898462,  -0.469871938,  -2.29587483,  -0.052375298},{ -0.730670631,  0.918729186,  0.663360894,  -1.30343044,  -1.3854059,  -1.2796582,  -0.561877608,  -0.511470973,  -0.933739662,  -2.38577533,  0.780606568,  -0.0161868427,  0.852335691,  0.249673486,  1.33170557,  -2.34815526,  0.706186473,  0.965496898,  0.553764164,  0.848224759,  -1.41964376,  -0.418572128,  -0.132978424,  0.215652063,  -1.13058829,  2.08374596,  0.0794427842,  1.81562531,  -0.554883838,  -0.0184011515,  0.143498331,  2.15601921,  -0.0766555518,  0.656501651,  1.1833142,  -0.169977665,  0.481660843,  0.865161836,  -1.5611918,  -1.97326803,  -0.587045133,  0.71210748,  -0.790168583,  -0.287430674,  -1.3144263,  2.36121297,  1.58086991,  0.454501092,  0.543295264,  -1.46647692,  1.07415926,  -0.12437918,  -0.00558445882,  -0.599539936,  1.87338793,  -1.19362426,  1.21860945,  1.12848639,  1.0903039,  1.54669857,  -0.736374259,  0.819644392,  -0.0922154263,  0.540708661,  1.18812418,  1.35247743,  -0.929831564,  -1.06752193,  0.703666329,  0.737398624,  -1.25165105,  0.219733059,  0.634308815,  -2.03986096,  -0.737617373,  -0.927308738,  -0.125564098,  0.33133319},{ -0.216617107,  1.62983453,  -0.476526707,  -0.520829558,  -0.0904386789,  -0.616368294,  -1.46280301,  -0.490982056,  0.0593989603,  -2.19007158,  1.56358838,  0.772295654,  -0.564131558,  1.65827215,  0.78425014,  1.10044324,  1.3159833,  2.34461403,  1.0509479,  -0.220405415,  0.927401185,  -0.679793358,  -0.906966925,  2.25846624,  1.54791319,  0.743867695,  -3.07767081,  0.465860903,  0.334512383,  -0.559958637,  0.920521259,  -0.37874046,  -1.09445381,  -1.55097437,  0.374872833,  -2.74672771,  -0.329796612,  1.32974172,  -1.17554474,  -0.598209381,  -1.42876554,  0.618382454,  -0.936148226,  1.12756097,  0.45014739,  0.619669735,  -0.297113895,  -1.38710833,  0.596744776,  0.667229414,  -1.07348418,  -0.89813447,  -0.421829462,  1.45739734,  0.814765513,  -0.40853107,  0.250811636,  1.13821113,  -1.06333876,  -0.110906795,  0.303307265,  0.337096304,  1.20570505,  1.39268017,  -1.18871379,  -0.486928135,  -0.335805744,  -0.485766888,  2.43409562,  -1.30799234,  -0.393458962,  -0.0378929004,  0.557627916,  -0.000684050552,  0.698209822,  -1.13447607,  -0.118210636,  -0.738351822},{ 0.327318341,  1.12028408,  0.481650323,  -2.09673357,  0.754851639,  0.75819397,  -0.182108641,  0.871700346,  -0.699979246,  -1.07151401,  0.048802793,  -0.805090129,  0.121523403,  2.09378934,  -1.39461076,  1.2447592,  0.154002815,  0.58018297,  0.523578048,  0.0816285908,  0.694248676,  -0.207419097,  -0.93839103,  1.23607945,  -2.24437165,  -0.984523714,  -0.965930939,  -0.226816446,  3.14747047,  0.39691788,  0.292156905,  -0.805122852,  0.918408394,  -1.56145084,  -0.575275481,  -1.75710297,  -0.245086566,  0.518922567,  -0.138197854,  0.1636329,  -0.548026979,  -0.17382659,  1.4703685,  -0.756063104,  -0.362774998,  0.330006391,  -1.37167037,  0.73334533,  1.23800564,  -1.39594042,  -0.271800637,  -1.14395177,  -1.66240811,  0.443705052,  0.281502396,  0.219055563,  -1.25095212,  -0.375781089,  0.377527267,  1.29317284,  0.102574095,  -1.19209146,  -2.12433147,  -0.982487798,  0.385906041,  -0.612370491,  0.625506699,  0.479202121,  0.855683684,  1.5952884,  1.63461137,  -0.783076167,  0.956826746,  0.552380681,  0.0297622364,  1.65972567,  0.427150965,  -0.61399132},{ -0.202173814,  1.79985511,  -0.4162862,  0.631982923,  1.03479576,  -0.0259554274,  0.437598586,  -0.346178502,  0.0914832279,  -0.454469889,  0.00148705265,  -0.588553846,  -1.96213555,  -1.33954108,  0.336928606,  -0.819161236,  0.3481884,  -0.982761562,  -0.278594553,  -1.09089029,  -0.395911247,  -1.13447928,  1.08856285,  0.0237788055,  -1.43166327,  0.0408746861,  -0.764026403,  1.01280475,  0.100321077,  0.127386585,  -0.144655377,  -1.69449306,  -0.491448343,  -0.541356742,  0.226348743,  -1.19024348,  -1.73201275,  -0.659248829,  -0.704237401,  -0.896169603,  1.43329406,  0.42917031,  -1.23179102,  -0.426810414,  -1.0711236,  -0.123360962,  -0.719599366,  -0.100784384,  0.982019663,  -0.656183183,  0.201917246,  -1.46798289,  -0.35145691,  0.114401162,  -0.284856707,  1.60370398,  0.899131894,  -0.560823739,  -1.67891312,  0.945951819,  0.38679719,  -0.536928892,  1.01926208,  1.84856403,  0.695224404,  1.14589405,  -1.23802793,  -0.357058614,  0.124461345,  0.291244745,  0.0264949892,  0.766029656,  -1.29096198,  0.866628528,  1.42472231,  0.25164789,  0.34820044,  0.576352477},{ -0.043639373,  -0.66240561,  0.67127794,  -0.599799097,  0.514153302,  0.0383192264,  1.78392935,  -1.80714393,  1.673581,  0.684475303,  0.394239813,  -1.64592803,  0.19570516,  1.70433152,  1.36736989,  0.00200583274,  0.600488186,  1.32863736,  -0.0856239498,  -0.229577363,  -0.529592991,  0.812475979,  -0.42481339,  0.547694623,  1.24334788,  0.893529356,  0.581705749,  0.122445934,  0.605693817,  0.778390586,  -0.540457368,  -0.5732373,  1.8741132,  0.0468052588,  0.858888566,  -0.294189423,  -0.361212045,  0.0731288046,  0.381391317,  1.11046529,  -1.32434499,  0.549316823,  -0.323048562,  -0.191632137,  0.483798951,  1.2291013,  -0.592350125,  0.217734873,  -0.981278002,  0.927118182,  1.63304341,  0.557913244,  0.289505482,  -0.578234732,  1.72857404,  1.66566777,  -0.211175874,  0.525907695,  0.109344371,  -0.601742387,  1.12412834,  -0.673250139,  0.486842513,  -0.14607726,  0.0907838494,  0.573344469,  0.513086677,  -2.00265074,  -0.123234421,  1.30008709,  0.821774721,  -0.079422079,  -0.0411569662,  0.342195302,  -1.536538,  -0.245673284,  -0.180370107,  -0.730497539},{ 0.667216241,  -0.581557214,  1.28552043,  -1.49383581,  0.728839397,  -0.714022875,  -0.429982543,  0.020730231,  0.540750027,  -1.80229902,  -0.386852115,  -0.230090797,  0.198407352,  -0.0191860888,  0.704098821,  -0.54022783,  1.36455214,  -1.76573503,  0.417536795,  -0.47138381,  1.74658179,  -0.922821105,  0.0538984947,  -1.12949193,  0.219937459,  -1.4972744,  0.310352176,  -0.780517161,  -1.29294097,  1.4414041,  -0.14031446,  1.48420858,  0.285826325,  -0.294622481,  1.21612775,  1.60274136,  0.795149267,  0.0546365157,  0.26477167,  -0.440282643,  -1.68995416,  -0.610187292,  -0.161549866,  1.30297518,  -0.244551659,  0.158779189,  -0.500466943,  0.244159237,  0.72135669,  -1.41812003,  -1.11552989,  0.686599851,  -0.261629999,  0.170117572,  -0.848554373,  0.33625412,  0.25368613,  1.74282718,  0.95386672,  -0.809575498,  0.538492262,  -0.621022284,  -2.98545933,  -0.949810982,  -0.980253816,  0.227600664,  0.525265634,  -1.00447226,  -2.52308702,  1.09502995,  -0.207787201,  1.04645646,  0.592922628,  0.428947747,  0.896855712,  -0.0315897204,  1.03754497,  0.37124151},{ -0.983040452,  0.264985532,  -0.904114962,  1.16207016,  -1.21512091,  0.378416181,  -0.960667133,  -0.621965468,  0.610297263,  1.82224715,  -0.00693181297,  0.00567854708,  -1.52849662,  -1.61792266,  1.02571988,  1.66661382,  2.02725077,  -0.991830409,  -0.491596729,  -0.612244129,  0.278549105,  0.736713648,  -1.99683332,  1.33411551,  0.14166981,  1.57168937,  1.70711315,  0.0968336165,  0.073950313,  1.11761808,  0.0276220739,  -1.30079496,  0.469038963,  1.07207334,  0.443776608,  -0.183118224,  -0.336320728,  -2.26356339,  -1.31446707,  1.46595216,  0.807583869,  -0.199285075,  0.85456109,  0.786923409,  0.286106318,  0.305441439,  -0.185992196,  1.43588984,  1.25083435,  1.18699801,  1.24063349,  0.437136292,  0.806983709,  -0.718048632,  -0.66559279,  -0.0477113798,  -0.431545913,  0.0421349518,  -0.71203953,  -1.02232337,  -0.177599296,  0.0146768298,  0.299561948,  1.47818315,  1.37900209,  -0.305498511,  -0.412742645,  -1.23842669,  -0.05658371,  1.33958602,  1.70831263,  0.521009505,  1.79174304,  -0.183032468,  -0.309771597,  -0.288574249,  -0.363632083,  0.627143562},{ 0.516385913,  -1.12960315,  1.01536918,  0.198383451,  2.02367878,  0.157240465,  -1.62229562,  -0.00849841163,  0.343854994,  -0.240049914,  0.482192367,  -0.935484529,  -1.23478484,  -0.224446848,  -0.406315327,  0.558662593,  0.93853575,  0.657719672,  0.994922101,  -0.950354159,  0.957878649,  -0.861592412,  0.489666551,  -1.27505183,  0.715983927,  -1.25707698,  -0.547861636,  -0.100424662,  0.203060344,  1.17224789,  2.20097208,  -1.46068907,  0.855971634,  0.828747869,  0.772202432,  -1.92920816,  -0.0145906406,  0.224554867,  0.580426872,  0.240111575,  0.842700601,  0.444908053,  0.523846328,  -0.547211587,  -1.11924231,  -0.287320554,  0.80608958,  2.33163428,  -1.15220404,  0.0211411044,  0.714033544,  0.388181537,  1.30625284,  0.15794529,  0.394674152,  0.96753633,  0.905496716,  0.616663992,  -0.546535373,  0.931945086,  0.494900942,  1.05245006,  0.0775493532,  1.9955442,  0.0373112038,  -1.02639532,  0.519326746,  -0.0315555073,  -1.65613186,  -1.90768099,  0.901360035,  -1.70867956,  -0.765305459,  0.149867117,  -0.858297467,  1.386518,  -2.36932492,  1.86884809},{ 0.509516358,  0.329325497,  1.01844764,  0.135375604,  0.5972386,  0.269689411,  0.255365014,  0.350087255,  -1.55507815,  0.144870803,  -0.623112679,  -2.40799737,  0.30540359,  1.98879802,  0.480239093,  -1.6234535,  0.828066289,  0.644545972,  -0.431701362,  -1.39960063,  1.37600505,  0.768147647,  2.68453312,  0.251907408,  0.162852496,  -0.141554981,  -0.896650553,  -0.652787089,  0.119553208,  0.111652024,  0.895113647,  0.91215694,  1.39646542,  1.40566623,  0.0481204242,  -1.50818849,  -1.67728651,  -1.0184052,  0.856502533,  -1.15779459,  -1.03148019,  0.697187245,  1.1803534,  2.24734092,  -0.450577378,  -0.367189407,  -0.872871399,  -0.748894274,  -1.12977898,  -0.745821893,  -1.40895307,  -0.746864438,  0.0644359291,  -0.421385407,  -0.321274161,  1.07705486,  1.19232225,  0.201839581,  -1.36499619,  -0.632259309,  0.796396911,  -1.32269812,  1.02373827,  -0.343153566,  0.18867518,  1.12285638,  0.100099042,  -0.0220114719,  1.40945435,  0.173186213,  -0.85169524,  -1.43827987,  -0.758890808,  -1.1443485,  2.1272316,  0.715428352,  -1.54692435,  -0.185623065},{ 1.34319437,  0.640642643,  -0.846635103,  -0.854612947,  -0.381271958,  1.32734919,  -1.12415159,  -1.72202837,  -1.88816202,  -0.581123173,  -0.44951871,  -1.33060431,  -0.105121218,  0.162973329,  -0.0776609555,  -1.54157126,  1.44541323,  0.242436737,  0.133325785,  1.04972851,  -1.93041408,  0.710647404,  0.785470128,  0.360650271,  -0.320302844,  0.684868515,  -0.812058508,  0.454269141,  -0.624089956,  -0.00553519093,  1.14559531,  -0.982929766,  0.88087219,  -0.86426121,  0.29530859,  -0.0208592936,  -0.536972761,  -0.206580117,  -0.413636953,  -0.264897168,  -0.456075847,  1.82908201,  -0.944384158,  -1.44862878,  -0.835519075,  -0.779184818,  0.793237269,  -1.08512354,  0.326450944,  -0.176171005,  -0.486366808,  -1.48068523,  0.200501114,  1.23233962,  -0.134892777,  -0.492221117,  -1.8471247,  1.64296055,  -1.17495716,  -1.55743277,  0.191715837,  -0.0743122771,  0.268878728,  1.94872546,  -0.0690907314,  0.559106469,  -0.0594714209,  0.644643128,  -2.13732052,  -0.378402412,  1.16120291,  -0.171150938,  1.06042314,  -0.31106776,  -0.853921354,  -0.25040108,  1.09799469,  -0.20711343},{ 1.40222132,  1.51293707,  0.848694503,  -1.04469645,  0.429981947,  -0.54355514,  0.389240831,  0.473548949,  -0.727933526,  1.06259012,  0.845808983,  0.235436931,  -1.1967907,  1.7826587,  -0.155567557,  0.379576832,  -1.00226569,  1.61625743,  -0.267114699,  -0.486256838,  0.433039963,  -0.551607966,  0.0435209759,  -0.287927061,  -0.0606382638,  -1.21629155,  -0.711185217,  1.10010374,  1.6118474,  0.207860798,  0.0934279636,  -0.64791584,  0.0737299323,  -0.00465331133,  -0.821025014,  1.20978439,  0.162379593,  -2.05317283,  -0.644270182,  -1.11459744,  -0.228427574,  0.321747422,  0.577595115,  -1.71700883,  0.362609893,  0.103019468,  0.149330586,  1.30450428,  -1.19291079,  0.135586739,  1.29152131,  0.662704647,  -0.558699787,  0.159117222,  -0.771924973,  -0.475824445,  0.396363378,  0.28701973,  -0.199850172,  -1.32376838,  -0.193951324,  -0.0069877645,  1.72769761,  0.773824573,  0.0255789142,  1.45530581,  -0.548373878,  -0.951766014,  -0.834976315,  0.283704221,  -0.363753438,  -0.45623523,  0.340806693,  -1.00722384,  1.06803465,  0.872288227,  0.626158535,  0.909743011},{ 0.315489292,  1.1711421,  1.1969943,  -1.14455605,  0.566866696,  0.200299203,  0.489547491,  -1.19980085,  1.39870727,  0.634393394,  -0.213365912,  1.96874797,  0.0443962179,  0.629425168,  0.401739717,  -0.253956646,  0.657856882,  0.555995345,  -1.05733478,  0.824111462,  -0.368680954,  0.0148021821,  -2.21485734,  0.243312389,  1.40436447,  -1.91357207,  0.291370183,  -2.27687931,  0.229239091,  -0.446188629,  1.03949177,  0.216815293,  1.84551287,  -0.152518734,  1.72903192,  0.273563683,  1.55365622,  -2.03365469,  -0.00298152771,  -0.458536804,  0.0980092287,  0.563362241,  -0.173987448,  0.900248647,  -1.10123062,  -0.824876904,  -0.518003166,  0.209559396,  -0.74829489,  -0.769913554,  -1.01652348,  0.114409104,  -0.515570223,  0.00587109989,  -0.144424558,  -0.129773304,  -0.227953181,  -0.711920381,  -0.901823044,  -0.413459271,  -1.12831402,  -0.0598908253,  0.557121813,  1.35437334,  1.70566809,  -0.252491534,  -0.904010415,  0.524394453,  -0.719420671,  -0.447440863,  -1.91025734,  1.48315334,  1.10833478,  0.252885938,  -0.269814819,  0.910680234,  -0.680807352,  -0.36802268},{ -2.16048026,  0.310340405,  0.677368581,  1.11278653,  -0.528183222,  -0.848548889,  -0.965617359,  -0.52496177,  0.458025038,  -0.607241809,  -0.086071454,  -1.39076364,  -0.0390117876,  1.11563158,  0.964559436,  -2.29801726,  1.19854927,  0.381680518,  -1.07312715,  -0.348235667,  -0.694196939,  -1.7772615,  0.0579294302,  -0.718459606,  0.908737719,  0.511308551,  -0.693098903,  -2.32098317,  -1.22954249,  -1.18993127,  0.394236058,  0.265905052,  0.730274737,  -1.60801482,  0.356762588,  0.213975713,  0.815217376,  -0.473366499,  -0.212234825,  -0.0907237083,  -0.422880083,  -0.230883464,  -1.4455539,  0.992742002,  -0.574924409,  -0.466150492,  -1.0701741,  2.80546093,  0.706811905,  -0.784946561,  -1.09000313,  -0.0210863259,  -1.11731541,  0.23122853,  0.79202795,  1.05486786,  0.661935389,  0.684356928,  -0.299858719,  -0.734012246,  -0.434711784,  -0.336162448,  -0.800627351,  1.82919824,  -0.0824519619,  1.47215366,  0.194233596,  -1.2710669,  0.836356521,  0.949724555,  0.245081872,  -1.14882553,  0.104713373,  -0.243744493,  0.2624304,  -0.833615184,  2.21405125,  -2.07553172},{ 0.00600910001,  -0.229068354,  -0.510428548,  0.828450561,  0.973095953,  1.62741339,  -0.50114584,  0.983504057,  0.125422612,  0.326750159,  -0.928919435,  -0.0597695522,  1.08931208,  -0.884192765,  -0.50467366,  1.13047838,  -1.10416996,  -0.581992328,  -0.942408264,  -0.109658137,  -0.687999189,  -1.44874823,  1.17335808,  -2.43989706,  -1.24884725,  -0.153467387,  -0.84819597,  -0.785893559,  -0.935861468,  -2.12548876,  2.01697898,  0.975895703,  -0.32135576,  -0.265689045,  0.962726951,  -0.0673360303,  0.60589993,  1.30434346,  0.804212451,  -0.597813547,  -0.535859048,  2.05531478,  0.644525111,  0.296953708,  -0.062212199,  1.84879804,  0.104857884,  0.548242807,  -1.15812409,  -1.82866013,  1.46427572,  -0.390324563,  0.281542033,  -0.346993268,  -0.80541116,  -0.686026275,  -0.412442833,  -1.76805973,  1.16147566,  0.407855749,  0.459796578,  0.773324907,  -0.316227168,  0.0710024834,  0.299060166,  -0.484942883,  -1.268978,  0.99972862,  -0.540058672,  0.392461032,  -0.59181869,  -0.952900946,  0.440173388,  0.172832146,  -0.103186943,  0.03394695,  -2.1710732,  -0.998176694},{ 0.754419863,  0.120772153,  1.32014418,  -0.922532439,  0.188274652,  -1.02215266,  0.875610888,  -0.815294087,  -1.3275038,  0.358420521,  -0.869132996,  -0.293563366,  0.474994957,  -0.29813394,  -0.276870728,  0.691152871,  -0.31800285,  -0.717878103,  -1.08933151,  -0.6964522,  -0.135163605,  -0.272876054,  -1.11218274,  1.00850987,  0.811958909,  -1.85347342,  -2.21073747,  -0.101022311,  0.312133342,  0.785767555,  -0.494538963,  -1.64526904,  0.742883682,  -0.693336964,  0.779653788,  -1.03207135,  0.340778142,  0.229866371,  -0.0252742171,  -0.354060739,  -0.229803786,  -1.86682081,  -0.27478236,  -1.32270491,  -0.765511036,  -0.251744777,  -0.762056053,  -0.267760992,  -1.06656516,  0.800971985,  0.353883803,  -2.85453391,  -0.502302527,  0.20504421,  1.72497058,  -0.52386564,  0.395672053,  0.40610978,  -0.245128155,  -2.43985391,  -1.64429474,  0.567793906,  -0.238894895,  -0.430746675,  0.875710189,  -1.0072757,  0.128262848,  -0.311191678,  -1.6570996,  0.848509669,  0.425244123,  -0.885271966,  0.168259129,  -0.807299674,  -0.872829258,  0.459067047,  -0.133308128,  0.144819051},{ -0.815214694,  -1.31044483,  -0.301840454,  1.69393349,  -1.01960051,  -0.263664901,  1.33969986,  0.413483351,  0.841190219,  -0.134787306,  -0.847987235,  0.519283116,  0.357851386,  -0.939786494,  0.619649827,  2.4005897,  0.324426949,  -1.56994081,  0.657586217,  1.87094498,  0.501269996,  -0.510728478,  -0.113958076,  1.4085741,  -2.34730053,  0.192133769,  -0.360179693,  0.160833836,  -1.32588685,  1.78706336,  0.588397801,  0.9945485,  1.18724477,  0.86732465,  -0.31524989,  0.537898183,  -0.0654807985,  -0.468440026,  0.585295916,  0.16813682,  -0.786435843,  -2.63146853,  0.828914464,  0.286141902,  -1.33484972,  0.0907304138,  -0.63581413,  -0.411203206,  0.526307106,  -1.70886767,  -0.17374213,  -1.01541388,  -0.132965967,  0.306419134,  -0.301928759,  1.37320542,  0.435063511,  0.913833559,  0.0635221899,  -1.57641745,  -0.512370408,  0.839589953,  -0.114921927,  0.350237846,  -0.541451037,  -0.00639933953,  0.0318637379,  1.24021745,  1.18806243,  -0.531757712,  0.721067071,  -1.10529852,  1.23993993,  1.61199343,  0.167509481,  -1.1033318,  -1.56662202,  1.20114315},{ 0.781466961,  0.562354326,  -0.664638281,  0.787364483,  1.71784759,  -0.902477503,  -0.560051918,  -1.511271,  1.25305605,  -0.630436778,  0.266250908,  -0.947546303,  1.11064923,  0.0805977955,  0.41518116,  -1.63478637,  0.545049727,  1.93988371,  -0.195649937,  -1.46778548,  0.814618945,  -0.120810203,  -0.718967795,  0.506834507,  -0.314813942,  0.401091367,  -0.657118797,  -1.87001455,  -1.37063563,  -0.293241024,  0.98929894,  0.173895791,  1.75011015,  -1.18470955,  1.47092581,  1.29061878,  -0.676721811,  0.208170414,  0.688555896,  -0.0476483479,  -1.13880384,  -0.308606714,  0.347017497,  -0.805014372,  -0.166164026,  -0.723809838,  -1.38495231,  0.213242844,  -0.130885854,  1.21836329,  -1.22422898,  0.834628224,  -0.0455901287,  1.14880681,  -1.30579281,  1.86492789,  -1.32217979,  -1.93811977,  0.0698899552,  -0.127017379,  -1.10127127,  0.0190114267,  -0.0432045385,  -0.406912565,  0.775252998,  0.00203320733,  0.215009481,  2.27126408,  -0.653899431,  1.09838176,  0.426654786,  -1.38356376,  -0.334104717,  0.306929886,  0.503647327,  0.148286507,  -0.0210279096,  0.315139681},{ -0.0624669679,  -0.686881006,  0.335790813,  0.471930981,  0.363486528,  -0.904271483,  -0.393744737,  -1.54184544,  0.670256078,  -1.05724525,  -0.406327903,  1.08583927,  -0.235951662,  -0.24724856,  1.01654184,  0.742123365,  0.463908017,  -0.0633594841,  -0.525048256,  0.391662121,  -0.600393951,  -0.715672433,  1.12462509,  -0.467415214,  -1.17574036,  -1.43576193,  0.121384151,  1.03932643,  -1.46269572,  0.0698079467,  -1.36271346,  -0.246991798,  0.969162703,  -0.470023125,  -0.0288272873,  0.459563613,  1.35549617,  0.788079202,  -2.1562531,  0.34205243,  -0.126112074,  0.537235856,  1.18321586,  -0.368305713,  -0.352838278,  0.295916378,  0.366658479,  -0.755398571,  -1.5607028,  0.507840812,  0.772269607,  -0.7187711,  1.04552817,  1.2428509,  0.26979506,  2.15319419,  0.00728895282,  0.852957845,  -0.504312456,  -0.00526049221,  -0.331352144,  0.224789172,  2.11391735,  0.00356625021,  0.144888163,  -0.228135988,  -0.401043326,  0.23809424,  0.882097483,  -0.281298369,  -1.50300944,  1.16042733,  -1.85191774,  -0.315482497,  1.3473233,  -1.66571188,  1.30838549,  -0.642509103},{ 0.0261276644,  -0.582703054,  0.18292518,  1.20901,  -0.383003473,  0.545004427,  -1.23003078,  -1.29420412,  2.1190176,  0.27397573,  0.616614044,  -0.613566279,  2.09770274,  -1.0254755,  1.13224876,  0.00773664517,  -1.04839623,  -0.944074869,  0.22388716,  -0.139654562,  1.45080197,  -0.204273686,  1.02045035,  -0.876819491,  0.578035533,  0.204164043,  -0.21060133,  -0.708185911,  1.29504657,  -0.237878963,  -1.20063984,  -0.0131347347,  1.46980798,  0.400987446,  0.799140632,  0.890315175,  0.032720577,  -1.49165499,  -0.244602963,  -2.67788982,  -1.46565855,  0.306870311,  -0.681739688,  0.869986653,  -0.847319901,  0.910473943,  0.70691371,  -1.49476683,  0.425245345,  0.112155654,  0.949747562,  -1.28706694,  0.651792347,  0.981829941,  1.69560194,  1.15642536,  1.21602452,  0.774026632,  -0.307923436,  0.586808145,  -0.757998765,  0.360534906,  -0.142918333,  0.48387599,  1.03317738,  0.36855346,  0.917539954,  1.66223884,  -0.605806291,  -0.808442771,  -0.610678554,  -0.775225461,  -0.936472535,  1.09042895,  0.161693975,  0.42941612,  -0.027216278,  0.913333476},{ -0.996714592,  1.28853858,  -2.37993598,  1.18488693,  -0.0619356818,  -0.47066322,  -0.635842502,  -0.622806191,  0.442064166,  -1.69593275,  -0.993799269,  -1.28451848,  -1.00049281,  -0.797153234,  -0.273751378,  -0.763302863,  0.377631724,  -2.38511753,  -0.191246435,  0.108544171,  -2.14748168,  -0.0688902587,  -0.0888501927,  -1.29368436,  1.12173891,  1.90226567,  -0.527590632,  -0.458052784,  -0.822797239,  1.85659623,  -0.361287743,  -1.84338772,  -0.498947293,  -0.646394908,  -0.875529945,  0.449762136,  -0.427102119,  -1.14092994,  1.3759501,  -0.588366568,  0.566486955,  0.783776402,  0.730902791,  0.489241511,  0.716451228,  0.776278257,  -0.211856842,  -0.329030722,  0.990493536,  -0.379386067,  1.07322538,  0.298224777,  2.62839246,  -0.193205103,  -0.0346628278,  0.297400594,  0.579130173,  1.45665598,  1.04550183,  -1.49572814,  -1.32946694,  -0.590999067,  -1.47457123,  -0.476528943,  -0.356485158,  -0.402452171,  0.637739658,  0.419499129,  0.416722894,  0.0294802617,  0.574567914,  0.506258547,  0.0368696935,  -0.432994306,  -1.0055505,  -0.0119915502,  -0.68806982,  2.10397983},{ 2.64164901,  -1.1383785,  0.768852055,  0.642280161,  -0.654791057,  1.55580473,  0.816225588,  -0.476305008,  0.249599859,  -0.818600535,  1.10295498,  0.0974769145,  0.293315411,  -0.340259463,  -1.07893991,  1.4545368,  0.929481089,  -0.790670753,  0.297099352,  1.24808812,  0.87409085,  -0.138048157,  -0.49299112,  -0.959671617,  -0.319421083,  -0.757167697,  1.09904027,  0.540358126,  -0.381920248,  -1.37935615,  2.02828121,  0.0593626536,  -0.046239879,  -1.14066088,  0.819143236,  0.497254014,  -0.900867522,  2.03314328,  -0.0748135522,  0.275600731,  1.331815,  0.139292806,  1.00301039,  -0.101464354,  -0.979170203,  -0.629263699,  -0.898209751,  -1.26447272,  -0.544206858,  0.348386526,  -1.9813782,  1.36021864,  0.0782499611,  -0.773241997,  -1.07962477,  -0.757660449,  1.36519492,  2.20909858,  2.26452017,  -0.773977518,  0.0412243344,  -2.24876356,  -0.316113591,  -1.23969471,  0.783198953,  -0.950745583,  1.1007781,  0.774888039,  0.965175331,  0.485469103,  -0.943247259,  -1.21394181,  0.137667686,  -0.0272222217,  0.126453906,  1.96209288,  -0.380178541,  0.177147925},{ -0.0584458672,  -0.0696254671,  0.125916585,  -0.165409088,  -0.453596532,  -0.953282654,  -2.29491186,  -1.50532961,  -0.251091361,  1.1414392,  -0.224690408,  -1.29208136,  1.10036802,  0.00934845768,  0.491005838,  -0.770968914,  -0.707248807,  -1.97730565,  0.799587965,  -0.0737234652,  -0.119479015,  -1.67086625,  -1.64242208,  -0.00276617706,  0.269034475,  1.58134246,  -0.436606675,  -0.39953357,  0.312100023,  1.66725338,  0.414056152,  0.683679998,  -0.272175461,  0.0945507064,  0.072097078,  -0.581711531,  0.738346517,  -0.719404697,  -0.699698746,  -2.24281144,  -0.204144701,  -0.677059114,  -1.63034058,  0.787077785,  -0.388370663,  0.623164713,  0.246542573,  0.846901,  1.57449853,  -0.265651435,  1.72640872,  0.107565299,  0.586872041,  0.45134002,  -0.817615032,  -2.19011641,  -1.04716337,  0.76459235,  -0.811534166,  -0.671490788,  -0.0993690118,  0.122629724,  -0.171425432,  0.72508055,  -0.786123633,  0.410433859,  -0.690090656,  0.0736645609,  0.19275102,  -0.0710126832,  0.00389837474,  1.72202384,  0.320072383,  0.345958591,  -0.115696236,  -0.975664854,  1.15142524,  -0.0477053411},{ -0.880616188,  -2.15265369,  0.0878264606,  -1.29086995,  -0.259180188,  1.06707811,  -2.73338675,  0.696699619,  -2.33335662,  1.63382959,  0.578602016,  -0.284049273,  0.984574437,  -2.16882515,  -0.867914259,  -0.553852797,  1.72002625,  1.91778624,  -0.546150506,  -0.424391299,  -0.490755707,  -0.518293917,  -0.0534003265,  -1.62528896,  -1.14674723,  0.352559417,  1.54191172,  1.78898692,  -0.335344255,  -1.51057887,  -0.211415201,  0.407001376,  0.825962245,  0.907001793,  -1.11737192,  -0.35931161,  0.343369931,  0.429319888,  0.259377986,  0.169539377,  0.184890211,  0.0348287672,  -0.881202817,  1.11414194,  -0.721735299,  0.0724454969,  -0.326612592,  -0.303334594,  -0.810453057,  -0.171480164,  -0.636162996,  -0.02618552,  0.754706681,  -1.07786059,  3.71374416,  1.57123232,  -0.514585555,  0.0240272451,  0.937678933,  -1.67141318,  0.669506133,  -1.04880428,  -0.422230721,  -0.459597617,  0.3006275,  2.4696424,  -0.293088764,  -0.426687449,  2.91447687,  -0.011229137,  0.683339059,  0.039223671,  -0.721449852,  0.829774618,  1.18645561,  -0.833091736,  -0.633306861,  0.557548761},{ -0.814237058,  -0.405191809,  0.575914145,  -0.0958012938,  1.26367629,  1.63671231,  -0.844740927,  0.550344586,  -0.463755965,  0.929295719,  1.2021997,  -0.0562213399,  -0.353946865,  -0.326749861,  2.24659443,  -0.699413121,  -1.07276416,  -0.554571092,  -1.22800255,  1.90718853,  0.692269981,  -0.809999645,  0.749806464,  0.458471417,  1.15214384,  0.538380206,  -0.247342214,  -0.692935526,  -0.741400898,  -0.766082466,  0.415356427,  0.253016263,  -0.364705354,  1.3133657,  1.28309321,  1.41492021,  0.251199603,  -0.590963542,  -1.35997581,  -0.230414912,  0.697577417,  -0.424921483,  0.364783138,  0.639304578,  0.0332452208,  -0.985246003,  -1.40910172,  -0.544535995,  -0.656247914,  -0.272746414,  1.61069107,  -0.558571815,  -0.951191545,  0.750313997,  1.27633953,  0.85227567,  0.46404618,  -1.73486149,  1.2258904,  -1.31212032,  1.19328153,  0.359987557,  -1.39875925,  -0.258014321,  0.471825898,  -0.710758269,  0.541174769,  -0.190665081,  -0.00650773896,  -0.24953045,  -2.22826409,  -1.76587379,  -1.59819067,  0.605551839,  -0.152337506,  0.332042515,  -0.461547822,  -0.077713646},{ -0.513887525,  -0.148456857,  -0.203958794,  0.0521590486,  0.772798002,  -0.369850099,  -0.188580438,  -0.459430456,  -0.229713932,  -0.084678255,  0.559787214,  3.02714324,  0.212600425,  -0.510709047,  -0.177573055,  1.34119594,  -2.44032621,  -0.232200131,  0.661083043,  0.488106519,  -2.51858735,  0.683654904,  0.019292172,  -1.42725599,  0.0328818858,  -1.08183551,  -0.751704037,  -1.02462244,  -0.386760265,  0.971334696,  0.926876485,  1.40323913,  -1.78980207,  0.0206684433,  -0.6022228,  0.482795477,  0.497243226,  -0.52199167,  -1.03399706,  0.653608859,  -0.626707971,  0.781858802,  0.0467878953,  0.801697135,  -0.973959148,  0.462469131,  0.672878444,  0.553226173,  -1.77179408,  -2.30010962,  -0.0524404272,  -0.311129183,  0.393267155,  0.112794295,  -1.63730919,  0.317148,  2.53355122,  -2.15490532,  -0.33748427,  0.0845761672,  1.43327248,  -1.0159874,  0.380403399,  1.68678474,  0.99636066,  0.723646045,  1.55368567,  -0.559154093,  0.133876115,  -0.2593669,  -0.0341115966,  0.266710073,  0.381693423,  -0.795546472,  -1.18448257,  -0.413357466,  0.32236746,  -0.594508827},{ 1.25296247,  -1.02050877,  -0.785461307,  -1.31389678,  0.711855292,  -0.619724512,  0.965578914,  -0.589183152,  -1.00232542,  -1.38822293,  0.529331088,  0.631613195,  -0.459355175,  1.02522361,  1.48919487,  0.999003947,  2.00022197,  -2.03373981,  0.191704124,  0.980749249,  0.45643571,  -0.214923874,  -0.318042189,  -0.664422035,  -1.24561667,  0.877299249,  0.093243368,  -0.29143703,  0.718104959,  0.643797338,  -0.388315529,  -0.366222411,  -1.39124918,  0.611495972,  -1.23996449,  1.98121274,  0.113292642,  0.524899483,  -1.18978441,  0.670358598,  1.62715578,  -1.14300609,  0.880129158,  0.0579880662,  -0.901004314,  -1.15088212,  -0.53342998,  0.262435824,  1.1598134,  0.839289546,  0.289615899,  0.352719456,  0.989067852,  -0.908542454,  0.0894654617,  -0.532925963,  0.195896998,  -0.53846395,  0.487318486,  -0.210501507,  0.55788362,  0.530697644,  -0.62473464,  -0.18117626,  0.639334857,  1.52030623,  1.21717155,  0.701363742,  0.430576891,  -1.61055017,  -0.264788568,  -0.204319134,  0.830602586,  -0.352761835,  1.76816046,  0.781524658,  -0.20776996,  1.00042927},{ -0.191925809,  -0.119214058,  -0.600447118,  -0.496949255,  0.864545882,  -0.415094078,  -0.664734602,  -0.730563819,  -0.550556302,  -0.317804396,  -0.040711768,  -0.216003492,  -0.401795447,  0.218960509,  0.0445880145,  -1.93266487,  1.32721686,  -0.968583822,  -0.800627232,  0.540231287,  -1.43699062,  -1.94865882,  1.04678392,  -0.729290664,  -0.400625527,  -1.67846882,  0.144417614,  0.818539679,  0.287572235,  0.271577835,  0.0174442735,  -0.79374671,  -0.0255427416,  1.99186218,  0.209138319,  0.144925877,  -0.866920054,  -0.0405929759,  0.493528187,  0.943257272,  -0.409476697,  -0.0962539166,  1.41295314,  0.484764904,  -1.07026935,  -1.2786715,  -0.929394364,  -1.39355361,  -1.65191901,  -0.241441444,  1.59059072,  0.790610433,  0.165586889,  -0.200207487,  1.3201288,  -0.590586483,  -1.03257811,  0.174927607,  -1.65535557,  1.3247627,  2.07680488,  0.60244441,  0.0802938268,  0.096578002,  0.32133767,  0.417660564,  2.91911483,  -1.37792385,  1.44763446,  0.531448483,  1.02923179,  0.146496326,  1.48219156,  -0.98795557,  0.237924114,  0.608152568,  -1.27162087,  -1.75533807},{ 0.0481711216,  0.546902537,  1.14736092,  0.262564749,  1.06576216,  0.13964799,  -1.31593704,  0.0493312739,  1.13753414,  0.170778334,  -0.906605601,  0.396136194,  0.274691403,  -1.68039095,  -0.410497308,  0.827867508,  -0.859034061,  -1.64989257,  0.500868618,  1.44917285,  0.249326557,  0.296135724,  0.405516952,  0.0974405333,  1.01211226,  0.927198589,  2.17482853,  -0.937297523,  0.0382105075,  0.175772533,  1.53444564,  0.338821471,  -0.093397148,  -0.787116945,  0.30612728,  -1.75700414,  -0.796866119,  0.037163265,  0.603185058,  -0.879913211,  -0.0353108197,  1.72440159,  1.19224715,  0.498672038,  0.818672657,  -0.564627647,  0.629565477,  -2.0996182,  0.945938945,  -0.291900456,  -1.13525665,  -1.01959467,  0.905915618,  0.356707633,  0.308344811,  -0.722542405,  0.58787322,  -0.279621989,  0.491467953,  -0.414715886,  -0.466467649,  0.0062204306,  -0.913641095,  0.649934888,  -0.502789676,  -0.33179301,  -0.872819304,  0.660157561,  -1.88583493,  -0.969584465,  -1.0151819,  -0.299441785,  -1.51942551,  -0.658491552,  1.25536716,  -0.56693846,  0.711872578,  -1.45068109},{ -0.329875141,  -1.23822999,  1.98933434,  0.385637462,  0.528933644,  0.078444086,  -0.892530441,  -0.76286447,  0.968122125,  1.1019727,  -1.11948514,  0.313686013,  -1.58938372,  -0.97509253,  0.937158883,  1.29241037,  0.150292784,  0.580857813,  0.217661455,  1.07119465,  0.178551853,  -1.84762096,  -0.976558864,  -0.318875611,  -0.329090536,  1.58586156,  -0.817734957,  -1.03325629,  1.38265359,  0.14122355,  -0.133739114,  0.426079303,  -0.186001137,  1.85064852,  -1.96464252,  -1.89330518,  0.360146672,  0.199593559,  -1.06843269,  0.157092094,  0.652447462,  -0.362122655,  -0.0357011892,  1.32647967,  -0.222209543,  0.370852083,  -0.148993582,  0.228338644,  0.17563872,  1.00944293,  -0.510566294,  0.112421639,  1.44297218,  0.41266191,  -1.73066115,  -0.331942737,  -0.252860099,  0.886624873,  0.678556859,  -2.21653008,  0.979854524,  0.416770279,  0.148019344,  0.359330148,  0.535130382,  0.669402003,  0.972147763,  1.1437335,  -1.45510519,  -2.13905168,  -2.13680816,  -0.0218008198,  -0.909754813,  0.147933155,  0.402362585,  -1.30313253,  0.301256388,  -0.154582232},{ 0.647591233,  1.14735842,  -0.964945138,  0.382047623,  0.276841402,  -0.739782691,  0.492939979,  -0.881532729,  -0.111320578,  -0.8805933,  0.442652315,  -1.15235925,  -0.734831452,  -0.154861867,  -0.945270181,  -0.153137416,  1.56049013,  -0.730315089,  1.11820233,  -1.17556596,  -0.291280091,  1.54454064,  -1.55546653,  -0.385930061,  -0.750591099,  -0.0179628879,  -0.368720412,  0.269258529,  -0.753222287,  2.21103573,  -0.202002063,  -0.9740659,  2.02362514,  -1.11422396,  -0.030401146,  -1.11168861,  0.185066074,  -0.523694396,  -0.769387066,  -0.928744078,  0.893379927,  0.474335462,  0.689707816,  -0.691679955,  0.0608792379,  0.0630323514,  -1.2047919,  -0.433023542,  -1.08239508,  -1.06858182,  -0.0471835174,  -0.193616569,  -0.516944289,  0.798211336,  -1.24222589,  0.586709917,  1.47388041,  0.354126066,  0.348226696,  -0.779909015,  -1.11268711,  -0.842099607,  0.68191278,  -0.07900846,  0.155468866,  0.899845183,  1.09738493,  -2.58474207,  -0.752969146,  -0.157289147,  0.297022641,  -1.14229262,  -0.0976707712,  -1.46522593,  1.20826638,  -0.598944604,  0.624163985,  0.0383757688},{ 0.270745426,  -0.196796328,  0.731677115,  -2.09763432,  0.43863824,  0.5063802,  0.511337459,  0.103851736,  -0.87638849,  0.313144386,  -0.364440918,  -1.38384748,  1.34773767,  -0.787863016,  -0.193447068,  0.446042866,  -1.12065995,  0.49139142,  0.34357205,  0.993587911,  -0.931167185,  -0.712316453,  -0.434865475,  0.110129386,  0.25022316,  1.1316396,  -0.371465445,  -0.0266106948,  0.347473532,  0.173976168,  -0.394602984,  -1.13005173,  0.86274308,  1.33634305,  3.63441563,  -1.24913096,  -1.62390995,  -0.867665112,  0.685911,  -0.498011827,  0.330091029,  0.687853098,  0.3761006,  -0.535478175,  2.39026618,  1.46235466,  0.321438521,  -1.8295337,  -1.41981637,  -2.07265234,  0.657555819,  1.24726629,  -2.52450728,  -0.906919003,  0.738288403,  -0.3188878,  -0.457603931,  0.881084979,  1.09416211,  1.31840825,  0.843377352,  1.04455948,  0.194347546,  -1.15783417,  -0.600574732,  0.519439459,  0.368192524,  -0.782271981,  0.386478066,  2.21391177,  1.77552247,  -0.839608073,  -1.62066698,  1.63670576,  0.401859969,  0.969220877,  3.05270267,  1.1622566},{ -1.35293245,  0.39465335,  -1.75450218,  -0.488948405,  1.5371238,  -1.36653256,  -1.84343684,  0.301952571,  0.266901612,  -0.856032312,  1.19624913,  1.74627638,  -0.370116442,  -2.01957989,  -0.393585116,  0.240095988,  -0.260190427,  -0.887631714,  0.940228343,  -0.223241866,  0.4896456,  -0.387735575,  -1.12552238,  -0.793962061,  0.0206863675,  0.239672557,  -0.0452246368,  -1.23603511,  0.971082151,  0.459306508,  1.00906408,  -1.36199856,  0.066241093,  0.564267635,  2.5577755,  3.12669921,  0.866036952,  -0.751369298,  -0.518962622,  0.0566652454,  -0.564060807,  0.276211917,  0.0557542518,  -0.767898321,  -0.180052295,  1.08116424,  -0.244536877,  -0.723178566,  1.01340163,  0.414984375,  0.539522886,  1.06965435,  -1.65387928,  -0.251064599,  0.187769935,  -0.202496797,  -0.592356026,  -1.32302225,  -0.928346515,  0.881831169,  -0.955987751,  -0.576561809,  -1.23428953,  0.519576728,  0.441421241,  -1.22028816,  1.92733753,  -0.791231811,  -2.42519569,  -0.0854493976,  -0.563035548,  -4.01807308,  0.802595198,  0.0915735364,  -0.318855852,  -1.33414972,  -0.424354434,  1.24185383},{ 0.521094978,  0.415237933,  1.18553901,  0.490156204,  1.04378843,  1.08566856,  0.153840497,  -1.89574623,  -0.879281759,  -1.30485499,  -1.17708492,  -2.04052091,  0.358964562,  0.1965601,  -0.381751567,  -0.591329575,  1.20000458,  1.5483681,  -0.28633669,  0.0855986699,  -1.23810923,  -2.26791048,  1.46246457,  -0.200591534,  1.53231704,  -0.825639844,  -0.156213045,  3.23823905,  0.0412786677,  0.944130301,  0.619642437,  -0.105803609,  0.456279457,  -0.582625687,  -0.799538493,  -1.08385289,  -0.172588125,  -0.770601511,  0.821389616,  0.514042914,  -0.0311332252,  1.07635522,  -1.45165765,  -0.635772049,  -1.03467047,  -0.986018062,  1.12684667,  0.520304799,  -3.05996156,  -0.905766606,  0.348654598,  1.48362601,  -0.0850688666,  0.666802049,  -0.0260335449,  -2.33146954,  -0.076648809,  0.0509659834,  -0.0989145786,  0.701309979,  -1.60245872,  -1.34704506,  2.09648943,  -0.175169304,  -0.326081574,  -0.861062884,  1.16828108,  -0.569266438,  -1.85919547,  -0.344614923,  -0.786608458,  1.11651242,  -0.374188781,  2.45185542,  -0.547982275,  -1.06891346,  0.454247028,  1.22251463},{ -0.339084178,  -1.53206491,  1.31824636,  0.855228066,  0.312609762,  -0.399669051,  1.54801488,  -0.715867937,  0.905983746,  0.28175965,  2.97270513,  -0.924407661,  -0.188190341,  -0.904454529,  0.429518014,  0.354098469,  -0.825844049,  -0.217843369,  -0.569899023,  1.25496459,  0.0375903361,  -0.18046771,  -0.397164881,  -0.983710289,  -0.0549099892,  -0.615961373,  0.779583693,  -0.863168955,  -2.23162103,  -0.188529164,  -0.729182601,  1.38443446,  0.256386667,  -1.08748341,  -0.547763169,  -0.419750929,  -0.942689002,  0.464025289,  1.04548931,  -0.531633914,  -0.971922159,  -0.880498052,  0.704880714,  -0.0668555722,  2.10289264,  1.30128121,  -1.01876736,  1.15967703,  0.305698425,  -0.30181098,  0.621825278,  -0.643108189,  -1.34303761,  -1.47050536,  0.525901735,  -1.30831397,  0.429649115,  0.904242933,  -0.952802181,  1.12110043,  0.572006524,  -1.0956881,  -0.62973249,  -1.44278514,  -0.0652216822,  -0.0303470064,  -0.383767128,  0.31592375,  -0.324794948,  -0.203784108,  0.429369867,  1.06461012,  -0.310198337,  0.731244564,  -0.411025047,  0.723269641,  -1.11528695,  0.718548775},{ 0.0285826642,  1.62728119,  -1.19670594,  -1.21863687,  -1.2057693,  -0.859270632,  -0.866646111,  0.73682642,  1.32683635,  -0.264908552,  0.653819442,  0.0633776411,  0.838104784,  -1.18537295,  0.122283667,  -0.66317004,  -0.156455949,  -0.455090255,  -0.412131369,  -0.950848818,  0.261267483,  -0.351766616,  1.56811118,  1.26583099,  -0.147347763,  0.702153206,  -0.784350693,  0.182762697,  0.333467573,  -0.0682067126,  -0.444825411,  0.183173507,  -1.15993845,  1.23605037,  -0.121717557,  -1.27835798,  -0.472567648,  1.37117326,  0.720746934,  1.50399065,  0.607322693,  -2.19837523,  -0.614450514,  -0.121757373,  -0.693689048,  0.784791112,  0.0449337699,  1.96185911,  0.365845144,  1.51900125,  0.200239435,  0.856328309,  2.07512808,  0.30768013,  1.03009284,  1.52245188,  -0.216119424,  1.37848532,  1.57551098,  -0.929511547,  0.903882623,  -0.361690789,  -1.45169532,  1.07832599,  -0.848325431,  -1.73341322,  1.86337304,  0.674424767,  0.317772985,  2.46762371,  -0.802777946,  -0.370848298,  0.78129518,  1.43808389,  -0.427601755,  -0.0820068792,  -0.703531504,  0.565639198},{ -0.334085882,  1.26389658,  0.466197193,  -0.230221227,  -2.4353931,  1.16941941,  2.0402894,  -0.492297977,  -0.116295248,  -0.741086781,  0.689795852,  -0.598251045,  1.90928376,  1.85466874,  0.528619587,  -0.499921799,  0.393679321,  -0.877522409,  1.03046596,  1.27343082,  1.74708474,  0.0651632696,  0.13165845,  1.52290606,  -0.547764063,  1.39449775,  -1.00896144,  0.360034674,  0.51659584,  0.955504537,  1.28760958,  -0.38490656,  0.279368967,  -1.86166298,  -0.380832464,  -1.34891343,  -1.3106432,  0.205459252,  -0.0787214711,  -0.714409232,  0.0129547007,  -0.855469584,  -0.868889451,  -0.273515999,  -0.200528219,  1.69713604,  -0.68702209,  0.189779848,  -0.73192364,  -1.2693913,  0.00453669066,  0.562349677,  0.236064136,  1.12620902,  0.142956287,  -1.33545494,  1.45140123,  -1.03665066,  -0.709244788,  -1.45206475,  0.855117142,  1.61376452,  0.88841778,  -0.363122255,  0.190138519,  -0.899577081,  0.283783674,  -0.554822624,  0.994805217,  0.993355691,  -0.540793598,  1.41325438,  0.649189353,  -0.122056901,  1.6601516,  0.569814205,  -2.11824179,  0.0204514489},{ -0.782450497,  1.27209258,  -0.960687101,  2.03474164,  0.197560266,  -1.71084535,  0.962010682,  -0.59641403,  1.12223268,  -0.16670996,  0.0226573031,  0.0335415527,  -0.77659291,  0.592770219,  -0.928620756,  -0.278118491,  -0.359966874,  -1.11150396,  -0.514457285,  -0.547761202,  -0.636810243,  1.69913268,  1.13915014,  1.82004023,  0.0717756003,  -0.269464165,  0.715774536,  -0.486645281,  -0.925515056,  -1.89674938,  0.252914548,  -0.229609981,  -1.09399295,  0.257995963,  0.205226436,  -0.0671927035,  -0.266205281,  -0.431181967,  1.17290413,  1.14190412,  -0.716248035,  -2.19820619,  -0.630222917,  -0.0869831145,  -2.09357905,  0.569546819,  -0.314487278,  0.993962109,  -0.00375476666,  0.549372971,  -1.59499002,  1.41842377,  0.524812698,  1.9248631,  -0.489432961,  1.09128332,  0.875332892,  0.00958271232,  -1.40034759,  -0.0257203933,  0.695721209,  0.766703665,  -0.20129092,  0.662349045,  -0.744667053,  -0.917941391,  -0.781510234,  -0.043357905,  0.931256473,  1.35740662,  -0.442765713,  0.0704225227,  -1.13257349,  -0.535110176,  1.3655684,  0.911265194,  -0.791047633,  -0.80987227},{ 0.142558739,  0.42274642,  -0.69161725,  0.311937571,  1.33758485,  -0.954073966,  0.774183631,  -0.335110039,  0.687926829,  1.03606701,  -1.33794081,  0.563826025,  0.711197853,  -0.530024946,  -0.758574665,  1.06762731,  -0.0800525919,  -1.10597312,  0.902776897,  -1.26037192,  -1.33677161,  -0.625190854,  -1.17980421,  1.55813742,  0.995912969,  -0.166727513,  -1.29465663,  -0.986777425,  -1.24976408,  -0.826801956,  0.729448795,  -0.501397371,  0.746314049,  1.2846874,  -0.405385107,  -0.430695415,  1.23190594,  -2.02712989,  1.34899759,  -1.93601835,  0.794128537,  -1.93922055,  -0.717517734,  0.181279406,  -0.377010524,  -0.439227015,  0.0949670151,  0.367284983,  0.327835768,  -2.06775236,  0.532094061,  -0.520527482,  2.13229871,  -1.44439149,  -0.625195086,  -0.27064833,  0.153825879,  1.63913357,  -1.016343,  0.773440242,  -0.260733873,  -0.037100587,  1.15312827,  0.936436057,  -0.796065211,  1.23302341,  0.460490823,  -0.821457624,  -1.99962485,  1.03099859,  -0.177074373,  -0.995691478,  1.20493865,  0.298359185,  -0.129004732,  -0.456084609,  -0.303664923,  -0.459053606},{ 0.41193229,  -1.75593376,  0.483855933,  1.2578969,  1.76344657,  0.584180593,  1.03718126,  -0.316168487,  1.19236887,  0.386052817,  -3.21832085,  -0.863095284,  0.872429132,  -0.526954293,  -0.855750382,  -0.0403376706,  0.344545633,  0.0346476883,  -0.801623106,  0.978537977,  1.0731256,  -1.308658,  1.14706719,  -1.13183272,  -0.105532542,  -1.18588829,  0.0147786541,  -1.56288719,  0.947666883,  -1.72009444,  -0.196332768,  0.382178843,  -1.59228492,  -0.422694534,  -0.883709371,  0.364468217,  0.767850101,  0.924961865,  0.992715716,  1.61705828,  -0.0966941491,  0.693481863,  0.795579314,  -0.160791829,  0.364307523,  -3.24517655,  0.712405384,  1.95608318,  1.20705676,  -1.07814813,  0.76248014,  -0.376326859,  0.506359577,  0.534467876,  2.02965808,  -0.0924251229,  1.14296019,  -0.105858333,  -0.0216273144,  1.05010056,  1.91076994,  -1.41079903,  -1.40780985,  -2.52510786,  -0.292612284,  1.21067715,  -0.479289502,  0.747937381,  0.62317723,  1.24730551,  2.32321548,  -0.587836862,  -0.0864168853,  1.21359169,  -1.74397349,  0.266707748,  0.555301547,  -1.68986547},{ -0.187685207,  1.72180736,  0.0820387676,  0.144263729,  0.102347568,  1.02994359,  -0.453108579,  0.35187313,  0.0654729232,  0.948485255,  0.718960345,  -0.891509295,  -1.04708004,  -0.92119211,  0.692745924,  0.00238798442,  1.16731334,  -0.318901837,  0.829107583,  -0.0797112957,  -2.25937009,  -0.354065686,  -0.286857039,  -1.03951895,  1.21162713,  -0.253611475,  0.279085904,  -1.58297396,  -0.479997307,  0.0773608536,  0.364437371,  0.229761317,  -0.499260634,  -1.509372,  0.0566241741,  0.323907018,  -1.781703,  1.44684434,  0.540022075,  -0.135828793,  0.200421035,  -0.786551774,  0.31570819,  -0.00567430165,  0.131535888,  1.373564,  0.245697543,  1.43537176,  -0.50485158,  1.09440076,  0.226395413,  0.289912522,  0.178094864,  0.802314878,  0.949010551,  1.67594874,  -0.817413628,  -0.638159692,  -0.24876222,  0.733660221,  1.26105881,  -0.19886826,  0.212175354,  0.92212081,  1.39933562,  -0.382863671,  -2.00316191,  0.468913078,  -0.729994893,  0.624602258,  -0.357653886,  1.07162118,  0.596395314,  1.07255626,  -0.0394997336,  -1.2930311,  0.634587646,  -0.446595758},{ -1.41464972,  1.30573952,  1.32014894,  -1.74862647,  0.908843398,  -1.44968295,  -0.82567662,  0.741828263,  1.73138118,  1.24007487,  0.163675413,  -1.49912882,  0.348482698,  -0.522408068,  -0.448136836,  0.268181235,  -0.236027077,  1.46458817,  1.00388455,  1.51455998,  0.389669806,  0.838123441,  -1.94357288,  -0.263710797,  -0.729589343,  0.00648918888,  -0.586373746,  -0.295248985,  0.757792056,  -0.382864445,  0.778572381,  0.708226144,  0.669114172,  -0.862634182,  -1.06718111,  0.424292028,  1.2316016,  1.97128797,  1.4334408,  0.767747819,  -0.396431357,  0.357569069,  0.915224969,  1.48187184,  0.712157667,  0.164448231,  -0.925770402,  -1.63953435,  -0.159547508,  -1.25889742,  2.30871987,  0.0540730245,  -0.970124543,  0.258653045,  1.96367562,  -1.0040561,  0.584091842,  0.406981826,  -0.933582366,  -0.638908446,  -0.322095424,  1.03030622,  -0.387833863,  -0.860228539,  1.44425225,  -0.0179971084,  1.6769923,  0.91552639,  -0.342308879,  -0.755096376,  -0.0061529153,  -0.774713874,  -1.1114409,  0.467809826,  -0.706834674,  0.156819046,  -1.92916501,  0.753497362},{ 0.932923138,  0.248753235,  -0.142150879,  0.0198266935,  1.95562434,  0.0654879436,  0.17870827,  0.575152814,  -0.0738300905,  -1.37613225,  -0.651626766,  0.0975313783,  1.2916379,  1.04451537,  0.160467163,  1.13796282,  1.71614337,  0.17261456,  -2.53068924,  1.14608216,  0.60267204,  -0.340357304,  -0.406336933,  -1.78854561,  2.00706911,  -0.956160069,  -0.807081759,  -1.16091466,  -0.159721345,  -0.950027406,  0.96605289,  -0.493288249,  -1.87014043,  -1.11505938,  -1.23994482,  0.698144913,  0.481315017,  -0.298437208,  -0.0106364302,  -2.013026,  -0.502896667,  -1.01632011,  1.52663052,  0.117497288,  -2.82212615,  0.194836646,  -0.39103657,  0.638356745,  -2.58022809,  1.30081928,  0.320456296,  -0.855122209,  0.689284861,  -0.133983761,  0.548120439,  -1.15159893,  -1.54705882,  1.20394588,  1.50457382,  -1.09536922,  0.432375342,  -0.618145704,  1.2674098,  0.0699547529,  -0.953730822,  -1.30345309,  0.301470846,  -0.0615818091,  -0.883608043,  0.0942133814,  0.72808373,  -1.71566904,  -0.956607819,  0.0537516847,  -0.657654226,  -1.10001993,  0.440539837,  1.25358117},{ 2.12208271,  -1.02973807,  -1.30702496,  -0.0785154849,  0.325962573,  -0.583604991,  -0.805958033,  1.23289478,  -0.629002452,  0.357037187,  -1.41002738,  0.359291971,  0.345632941,  1.46538544,  0.0840368271,  0.0303770024,  -0.723167539,  -0.600328624,  -0.441973567,  -0.554167151,  -0.107095949,  0.128107205,  0.712223411,  -1.42266476,  0.155103236,  -0.394320875,  -0.0625300631,  -0.965355933,  -0.101362824,  0.815626442,  -1.06149685,  -0.463806301,  0.0126549006,  0.772283196,  -0.229821578,  0.416011989,  -1.532938,  0.240715086,  -0.141008675,  -0.322039098,  -0.372177273,  0.968718827,  -0.51287818,  1.04054379,  -0.740045369,  -0.21946536,  1.99109721,  0.710780799,  0.0252991952,  -0.899672389,  -0.527387261,  0.320594788,  0.282438338,  -0.548436821,  -0.790363014,  0.227070853,  0.358577669,  -0.504260957,  -0.710848331,  0.340730995,  1.13849604,  -0.0547631904,  0.5239079,  0.78349632,  -0.0571850948,  -0.0409264714,  -0.333233267,  -0.180840567,  -1.44265056,  -1.77238202,  1.08630621,  -0.82363838,  0.323386997,  -0.489871621,  -0.00856486708,  1.38569653,  0.828667939,  0.995349109},{ -0.187749416,  0.641698718,  -1.07108223,  1.21844578,  0.675679684,  0.332065552,  -2.10819578,  0.899788976,  0.40527609,  1.27203321,  0.277822614,  -0.779134452,  2.2281692,  0.820331097,  0.766874433,  -0.0992828906,  0.12014614,  0.616680562,  2.60328341,  -0.289532393,  -0.469759196,  0.0921761841,  0.263648182,  -0.512542725,  1.04396641,  -0.639633,  -0.00588890724,  0.147031218,  1.54575884,  1.94483674,  1.70564842,  0.205763429,  -0.112151958,  -0.671631694,  0.489569187,  -0.413318783,  1.31352592,  -0.254191816,  -0.95494312,  0.788061321,  -0.251907289,  0.674099565,  -1.20362413,  -0.318766326,  0.959931433,  2.06398392,  0.300963312,  -0.224630505,  -1.63257349,  -0.631009817,  -0.0529500358,  0.880371153,  -1.34224355,  0.710611701,  0.725579739,  -0.0817855,  -1.1103195,  0.422097504,  -0.366830677,  0.207914963,  -1.42795348,  -0.510520875,  0.511404276,  2.21749353,  -0.0339347199,  -1.62988496,  0.576007903,  -0.468842417,  1.26175082,  -0.70340389,  -0.733055115,  1.07603824,  1.77993572,  -1.35462272,  -0.0243293252,  0.142227218,  0.144706696,  0.331313431},{ -0.0120561514,  0.585528791,  1.07865477,  -0.818631828,  -1.01181281,  0.848103881,  -0.573993504,  1.66559315,  -1.10225093,  -1.16038299,  -1.33905935,  -0.943683863,  -0.167803973,  -0.0279544536,  -1.24012923,  -1.33836734,  -1.08958542,  -2.24809241,  0.254150152,  -0.439390451,  -0.91626966,  0.296965182,  -0.44626376,  0.298010737,  -2.28937888,  1.72210252,  2.25430012,  -1.83884072,  -1.95747375,  0.209254459,  -0.829762876,  0.906960785,  -1.48957503,  -0.0658361614,  -0.0262990687,  -1.48000658,  0.32703191,  0.501312673,  0.232933447,  0.586786628,  0.422687441,  0.6863994,  0.612101257,  -0.0546555333,  -0.932857335,  1.72491896,  0.421715915,  -1.9873172,  -1.65554368,  -1.1542033,  -0.796804368,  0.0822789595,  0.817687094,  0.605233073,  -0.0353717767,  0.132026479,  -0.636215627,  0.0592893101,  0.874647558,  0.843501806,  1.17200899,  -2.63710904,  -0.547355831,  -0.754377842,  1.26852381,  1.05872917,  0.320865721,  -0.234486163,  -0.762040675,  0.179453537,  -1.58394492,  1.05296874,  -0.230753988,  0.301027089,  1.11548245,  0.466191173,  -0.608282208,  0.353738606},{ 0.283283025,  -0.815009713,  1.62890875,  -1.83530152,  -1.51260376,  -0.594268203,  -0.410268337,  -0.802645147,  -0.453439474,  3.32795501,  0.285782635,  -0.0417596363,  -0.751551509,  -0.997698784,  0.705701828,  0.580189943,  1.6122371,  -0.611774504,  0.218653247,  -0.208196178,  1.32772386,  -1.2132045,  1.02543843,  0.259016424,  -0.130269855,  0.274146318,  0.217147619,  -0.0276364144,  0.0245015919,  -0.0651509985,  0.340854526,  -0.202571243,  -0.852374136,  1.19879973,  0.332046539,  2.07462192,  -0.00705990382,  1.08383286,  -0.462021321,  0.779765546,  0.28403309,  -0.41902107,  -1.48742151,  -0.115847468,  0.0294590238,  0.916854799,  0.507748365,  0.108881265,  -0.212453902,  1.0284996,  -0.859906852,  -0.796808004,  -0.742863059,  -0.952113271,  1.05881381,  0.899514496,  -1.06494522,  0.443278372,  1.32865298,  0.210921481,  -1.81353354,  0.939857841,  0.164908782,  1.49359727,  1.02303696,  -0.0513381436,  -0.0639889389,  -0.744698107,  0.830328703,  -1.01493633,  1.46990848,  -1.30399704,  -0.974909902,  1.28148603,  1.23099887,  -2.72030711,  -0.393395662,  -2.23402786},{ -0.904365182,  0.31931594,  0.974548638,  0.12845017,  -0.164368659,  2.52155542,  0.531122923,  1.03585768,  0.330419183,  -0.0480740108,  1.22740996,  -0.905056357,  1.16603065,  0.591085315,  0.136379063,  -0.0621052459,  0.847005486,  -0.0168758519,  1.14960587,  -0.338563204,  -0.00972590782,  0.217978448,  -0.631226122,  0.318955451,  1.03062618,  -2.45972395,  -0.531285584,  -1.00810683,  0.0705249161,  -1.88408756,  0.648976505,  -0.142765775,  0.716925859,  0.108852126,  0.0523171499,  0.56804502,  0.0634495094,  -2.48498893,  -0.0851914138,  0.258897036,  -0.56998229,  -1.58110535,  0.908739209,  -0.602004588,  0.45314768,  0.0853877366,  0.166398346,  2.61248422,  0.60310775,  2.15794468,  0.231506765,  0.977353156,  0.00352152786,  0.224226832,  -0.0296955314,  0.951941013,  -0.541158557,  -1.51301062,  1.7893194,  -0.775062084,  -1.07902133,  0.922327697,  0.847800016,  1.33656108,  0.894498467,  -0.847993076,  0.95113337,  1.0445708,  1.55538321,  -1.84658885,  1.15089464,  -1.83236277,  0.376820624,  -1.30125284,  -1.2730093,  0.602582276,  0.378086299,  0.815177798},{ 1.40431952,  -2.23680758,  -0.50515002,  0.520328581,  -0.0609975159,  -0.247823849,  0.223004878,  -2.85590029,  -1.05515373,  2.94161081,  -1.00333643,  0.451743424,  1.71888745,  -0.18935746,  -0.464070976,  -1.52228367,  0.72867471,  3.17730212,  -1.89998746,  -0.468050063,  -0.0270421375,  0.585342467,  1.45824802,  -0.0481213666,  -1.11459112,  -0.551148236,  -0.284787178,  0.0747661218,  1.5730387,  -0.621557891,  0.852870643,  -0.82575959,  0.537249207,  -0.512743235,  -0.570702612,  1.98211563,  -0.222313136,  1.00935054,  -1.47708476,  -1.59902954,  0.129808798,  0.11898277,  -1.08336806,  2.02535868,  -0.469926029,  -0.0438514128,  -0.115990371,  0.117524713,  -0.936786056,  1.00365627,  -0.605569363,  -0.491705149,  0.653187037,  1.43166804,  1.31882811,  -1.28246248,  1.37527764,  -0.306216002,  0.355905503,  0.402291238,  0.29201445,  1.29895842,  -0.818303704,  -0.181113064,  0.131738633,  0.440992713,  0.514635205,  0.994200587,  -0.174237952,  -1.13901234,  0.870832741,  -0.742799163,  0.465609521,  -0.63940537,  -0.430874944,  0.163442686,  -0.892907381,  -1.16519964},{ -0.416022152,  1.22478557,  -2.538234,  -0.139315173,  -2.17492247,  -0.770791829,  0.385065228,  0.204569906,  0.535455585,  -1.16350031,  1.868101,  -0.619628489,  -0.250798851,  -0.333069563,  0.974325895,  1.5310353,  -0.566610634,  1.10729551,  -0.308857381,  0.227453396,  0.30485934,  2.03138685,  -0.901291907,  1.15423059,  1.40690684,  0.5073964,  -0.166240528,  -2.49456763,  0.925262094,  0.510705173,  -0.558047831,  1.65031421,  0.61058718,  -0.603401303,  -0.539914787,  -0.470809072,  0.489922166,  -0.421638876,  0.0383280292,  -1.52077854,  -0.214361295,  -0.577413023,  0.765249252,  -0.417215377,  -0.4702079,  -1.2877475,  0.72400099,  -0.968874753,  0.519685924,  -1.28993165,  -0.0402131304,  1.42727578,  1.58309269,  -0.390861988,  -1.18068409,  -0.259737879,  -0.621651232,  0.122864969,  1.60813129,  -0.0688762218,  -0.107284881,  -0.841943264,  0.822632492,  0.136746868,  -1.42061663,  0.596194685,  0.497380674,  -1.08458948,  1.82658744,  2.58536482,  -0.891439438,  -0.907919943,  -0.827689409,  -1.80429614,  -0.773385763,  -0.959326267,  0.370417655,  0.768465936},{ 0.0911488086,  0.387008905,  0.434763521,  0.373482049,  0.728702605,  -0.498588383,  0.11226169,  0.861857176,  0.747418225,  -0.586345196,  -0.153157562,  1.7831831,  1.0716958,  0.677629709,  -0.539649427,  -1.03585494,  0.750195622,  0.401906431,  -1.15841544,  -1.20890796,  0.13836205,  0.277218372,  0.805172503,  1.02643347,  0.275742412,  -0.684408605,  -1.04142344,  -0.87303412,  0.0568035506,  -0.946372569,  1.21344447,  -1.29108441,  -1.35845435,  -1.79649603,  -0.433107316,  0.758493543,  -1.26838243,  -0.594404578,  1.47781789,  -0.196651444,  0.529649734,  -0.186763182,  0.179920435,  -0.133985698,  -1.37366545,  -1.46319294,  -0.882349193,  0.271617144,  -0.355974615,  -0.212933064,  -1.03865659,  -0.648494959,  -1.85602963,  0.759097576,  -0.108202621,  -1.81710708,  -1.75552583,  -0.262757748,  -2.82412529,  1.42415357,  -1.43363094,  0.331344962,  1.20351672,  -0.913439989,  1.05830228,  -0.184789434,  -1.38753963,  -0.230421826,  1.40796816,  0.279289752,  0.362542361,  -1.03683019,  0.419560671,  -0.190085754,  -0.0658769757,  0.909177423,  -0.629272521,  -0.592111945},{ -0.140265942,  0.872562647,  -0.248449817,  1.07481718,  0.0515727066,  -0.577462614,  0.598244607,  0.258206934,  1.59203792,  -0.377328634,  -0.539822519,  1.21304274,  0.139556482,  -0.442853093,  -0.721091986,  0.377139628,  -0.24091506,  0.398064882,  -1.51319015,  -0.45464769,  -0.169281512,  -2.90099883,  1.32736278,  -0.0387172177,  -0.105716631,  -1.74389434,  -1.05391574,  0.0391900577,  2.43364143,  0.0142458575,  -0.200374499,  -1.21906102,  -1.34787977,  1.08779371,  -0.687235892,  0.875871539,  1.48292637,  1.02997065,  0.551374018,  0.444458961,  -0.279915631,  -0.182280406,  -1.20176709,  -0.530940831,  0.713033736,  0.756799757,  0.410123169,  0.372752339,  0.328602999,  -0.623597562,  -1.60315073,  -0.227282137,  1.14221597,  -0.541040003,  -0.100320436,  0.119316258,  -0.552065611,  -0.00991733745,  -0.296727926,  -1.90398431,  0.134466738,  0.139426693,  0.916489184,  -0.130635008,  -0.781148374,  -2.97614527,  1.56811357,  1.16741991,  -0.552092016,  0.876779318,  -0.377612561,  -1.90837765,  -1.97601199,  -0.904225767,  0.153343067,  1.53995466,  1.26326811,  0.714612484},{ 0.618314385,  -0.0651876852,  -0.391711295,  -0.297101825,  0.842141867,  1.85738993,  0.955297947,  -0.906758964,  1.04663789,  -0.664323509,  -0.801887512,  -0.558013201,  -0.36707443,  -0.290156722,  0.636918008,  1.37150371,  0.501040876,  -0.510570765,  0.489778072,  1.17275882,  1.86767197,  1.24840856,  1.29892707,  -0.894118428,  -0.83074826,  0.38228476,  0.84673965,  -1.05429339,  0.435911864,  -0.529645383,  0.21355395,  -0.813649476,  -0.193730637,  1.2361654,  0.939259648,  -0.892119169,  -0.0529538468,  0.668197811,  -0.19702591,  1.05030775,  1.30253923,  -1.97248435,  0.423478335,  0.459297508,  0.580537498,  -0.223255157,  2.48514485,  -1.18289208,  -0.727581143,  -0.307295233,  -1.82808375,  -0.625981212,  1.19083428,  1.43869448,  -0.339983732,  -0.619678974,  -0.45805338,  -0.0170887653,  -0.391423225,  0.148303181,  -0.497244924,  0.399649918,  -0.872939646,  0.917104185,  -2.3263905,  -0.885120988,  -0.688231766,  -0.0261561088,  -1.19725192,  0.502294302,  -1.30790079,  -2.47560883,  1.11660242,  -0.0496174134,  -0.297008663,  0.653134227,  0.250116587,  0.0887112692},{ -0.0121951075,  -0.607615054,  -0.36319527,  0.433918595,  0.304676414,  1.79160726,  0.964774489,  1.00292337,  0.642006278,  0.00711235078,  0.5572927,  -0.252380222,  0.038514778,  1.08807516,  2.42279363,  1.30030596,  1.53576994,  -0.429525375,  0.240234569,  -1.81757653,  -0.928991556,  1.38095403,  0.381574392,  -0.949742019,  1.58673608,  -0.478025347,  1.31559384,  1.34943461,  0.717761576,  0.706826687,  0.705640912,  0.489252806,  0.463799536,  -0.180647463,  -0.4356426,  0.494342059,  -0.989550769,  2.34709978,  -0.813446283,  -2.04579306,  -0.632451296,  -1.27392435,  -1.41976404,  -0.970716059,  -2.00581169,  -0.289119065,  0.258857042,  -1.19707942,  1.3972789,  -0.906333208,  0.922267556,  -0.155210406,  0.166905046,  0.622965574,  -0.375277817,  -0.337171912,  -0.246965989,  2.11098504,  -0.228741497,  -0.627900004,  1.82234108,  -1.11201382,  -1.05035675,  -0.243442327,  0.39590016,  -0.0129729472,  0.853193045,  -1.65106177,  -0.774189234,  0.905284882,  1.57417345,  -0.286232471,  -1.46830738,  2.19698215,  0.613700628,  -1.42542219,  -2.13731813,  -1.61046052},{ -0.758945525,  0.216401964,  0.480849981,  0.0785065144,  -0.284663737,  1.31085908,  -0.625413358,  1.52247834,  -1.68892026,  -0.694304526,  -1.38289642,  -0.301401168,  -0.0436951704,  0.333441198,  -0.566991329,  -0.129814014,  -0.700311661,  0.252270162,  1.20045078,  -0.217369273,  0.414875656,  1.03970551,  -0.0993288904,  0.347517699,  1.28389549,  1.29645312,  0.545651615,  -0.947985709,  -0.930832803,  -1.13865566,  1.07232594,  -0.593102098,  1.33888173,  0.485492796,  -0.403581262,  -0.174749851,  1.48079634,  0.98203826,  0.450647086,  -0.722932279,  0.133622587,  -0.525282919,  -0.687509537,  -1.1033392,  0.416972071,  0.692341626,  -1.84736979,  -0.458236217,  0.198758915,  2.56772971,  -0.147300526,  -0.799956024,  0.938243926,  0.564202011,  -0.366559684,  -0.347050607,  -0.0580481291,  -0.742298543,  1.21105373,  2.83969641,  1.75663936,  0.72108686,  -0.598254919,  0.0574829951,  1.18311226,  -0.26280123,  0.536627114,  1.27111495,  -0.765245259,  -0.705670238,  0.811800063,  -0.22073257,  0.298232228,  -1.56422436,  -0.180275992,  0.022465786,  0.812066436,  -1.90963292},{ -1.58856332,  -1.4393183,  0.887096226,  1.32378685,  -0.428382248,  -1.07442617,  0.0533760004,  0.162531391,  0.494656771,  -0.0827830657,  0.23298189,  -0.690510929,  2.32881165,  0.589998782,  -0.899057567,  0.446880132,  0.951458216,  0.120179854,  0.0327906311,  0.313651651,  0.0913648233,  -0.39994508,  -0.652009606,  -1.34038532,  -1.08787549,  1.22896409,  1.27011919,  -0.465948194,  -1.10423493,  1.23391569,  1.01627207,  0.0105140992,  0.258428246,  0.170875549,  -0.634309471,  -1.64415145,  1.60396719,  0.142580152,  1.99884439,  -0.961974382,  -0.223531008,  1.09944355,  0.271274298,  0.0630734041,  0.497218877,  1.04165733,  -0.129944056,  -0.133938223,  1.44792414,  -1.67519987,  0.388066113,  -0.728093028,  0.0141259963,  -0.773117423,  -0.567145228,  -1.87824965,  -0.073219046,  0.788553119,  0.212044194,  -0.113575548,  1.5520221,  0.100114338,  -0.19826135,  -1.21190655,  0.055133,  -0.0814124346,  1.77850723,  0.471656054,  -0.0293439906,  0.439854652,  -0.0639328957,  -0.638249278,  0.908963561,  -0.622435987,  0.557259917,  -1.43209052,  0.370005429,  2.34322691},{ 0.311246663,  1.61216605,  -0.161738724,  1.02872443,  0.741705,  0.592881322,  1.76643646,  0.153885022,  -0.638388038,  0.0539438426,  -0.249148399,  -0.00171914056,  0.109592482,  0.39009735,  -1.66401815,  0.0291625503,  -0.960977376,  -1.49652123,  0.959987044,  -0.0920457393,  -0.841834009,  -0.717518568,  -0.998918235,  0.52215755,  -0.32199508,  1.14507997,  -1.80240643,  -1.30870342,  -0.395906001,  0.155463979,  -0.97043246,  -0.913367152,  -0.974632144,  -1.27422237,  0.785904467,  0.136157438,  -0.630794048,  -1.8005321,  0.0484454185,  0.0842982531,  -1.14058995,  0.868577361,  1.43604755,  -0.368926018,  0.536268651,  2.37237191,  0.633022904,  -1.1631844,  -1.7591337,  -0.989932716,  -1.35448205,  -1.96180677,  0.439345628,  0.930600703,  -0.031794697,  -0.79395026,  -0.973223567,  -0.0636074245,  -0.368448108,  1.63101399,  -0.829953074,  1.37262046,  0.607474744,  0.738669753,  1.17593777,  0.347098351,  0.848226845,  0.278657675,  0.0875575468,  -1.34527361,  1.23166668,  0.299956292,  -0.272964954,  0.807041228,  -0.192280918,  -1.36707544,  0.452076077,  0.929308772},{ -0.425615788,  -1.39855242,  -1.65356553,  1.42045355,  -1.45838749,  -0.56115973,  -0.130892098,  -0.190724343,  0.458680302,  -1.24227428,  1.34800553,  -0.735050261,  1.32492912,  1.57973576,  0.962502062,  -1.56658947,  -0.501783013,  1.22076738,  -0.558280826,  -1.06729138,  -0.000254336424,  1.08027697,  1.198645,  0.526550412,  2.50087476,  2.70752597,  0.329647273,  0.305314243,  -0.754554451,  1.69637477,  -0.14908132,  2.4225626,  0.377636492,  0.876989961,  -1.73931682,  -0.392973155,  -0.263639539,  2.40600729,  -1.72144556,  0.972123325,  -0.826372802,  0.0805411637,  -1.18833983,  -1.82278395,  -0.0255081374,  1.00598848,  -0.346017003,  1.50268304,  -1.40444911,  -1.25418103,  1.52929974,  -0.942513168,  0.0613511093,  1.69426858,  1.41093624,  -0.119648442,  -0.656852663,  0.409065664,  1.29461145,  0.783730268,  0.323078603,  1.39931858,  2.30912137,  0.217555016,  -0.505031288,  0.953999519,  -0.596254468,  1.1382786,  -2.23545313,  1.68709087,  -1.06547356,  1.33760107,  1.3652854,  1.20158243,  -0.109011799,  0.492717385,  -0.736284733,  -0.0353600271},{ -0.0264349803,  0.844703436,  0.558827341,  -0.550700426,  -0.0868142024,  1.90386307,  0.555111408,  -1.93432188,  1.8214711,  -1.07312226,  0.466322362,  0.142430604,  -2.11769891,  1.1044867,  0.0423374884,  -1.2971946,  -0.210638732,  0.929016531,  2.36310267,  -0.514381289,  0.111141235,  0.39953956,  0.0938214287,  0.00484666042,  0.624104083,  1.11569071,  -1.02230084,  -0.591668129,  -1.08118379,  0.283370733,  -2.0939014,  0.531791329,  0.141668409,  -1.09878564,  -0.380624592,  1.67544591,  0.903295338,  -0.385695577,  2.48316717,  -0.0157186277,  -0.599945128,  -0.354355633,  0.602644384,  -0.947650313,  0.275685728,  0.355803341,  -0.731922805,  0.775343657,  -1.53061569,  0.567700565,  1.59818578,  -0.428846061,  0.953621984,  0.0808251128,  -0.871127546,  -0.420830131,  -0.52685374,  0.449793965,  0.0800610706,  -0.801663935,  -0.85007894,  -0.779256165,  0.53607738,  0.272886217,  -0.829247057,  -1.74868572,  0.345161617,  -0.395640224,  0.878032267,  0.694567442,  -0.244472966,  0.00835596025,  0.910383105,  -0.890251994,  -0.41851759,  -0.870803833,  -0.875679016,  0.293602526},{ -1.17925167,  1.09997058,  1.44024897,  -0.18352437,  0.202252418,  -1.30101919,  1.14761984,  1.17091691,  -0.309559941,  -0.960019112,  -0.0754082352,  0.174754411,  -0.134904176,  0.161711946,  1.60733235,  -0.0631603599,  0.71502465,  -0.340206563,  0.320465475,  -1.99303365,  -0.48510924,  -1.02747726,  -0.62565428,  -0.952805877,  -0.268567711,  1.14007974,  2.31208658,  0.439091355,  -2.05139375,  -1.11173952,  -0.453851461,  0.0526913665,  0.203475967,  -1.37321365,  -1.0528121,  1.85670257,  -0.402463257,  -1.34298992,  -0.763557076,  0.595563889,  1.27517056,  -1.74900901,  -0.0695194751,  -0.764319301,  0.787429094,  1.38518357,  -0.157629862,  -0.977666259,  -0.76880306,  -2.18012547,  -0.842368126,  0.619255304,  -0.830178559,  0.752047837,  0.284210235,  0.759711742,  0.35986346,  0.666485846,  -0.430190295,  -0.615339339,  1.41263247,  -0.0971722156,  -0.368787259,  0.608324647,  1.55711365,  -0.174304768,  -1.10478377,  0.586932719,  0.53602159,  -0.0028875859,  0.831589282,  0.418128312,  0.640052378,  1.28525686,  1.21098197,  -0.818946242,  -0.328176171,  -0.134888306},{ 1.81994474,  0.533532262,  -0.749768853,  1.35802329,  -0.283097416,  0.0179243498,  1.87918675,  3.01207495,  0.300386071,  -0.155557603,  -0.760628581,  0.482683063,  -0.748423159,  0.17893225,  -0.810287833,  -0.484915286,  0.103263862,  -0.20010443,  0.637394965,  -0.680500686,  0.598547161,  0.200310409,  1.22454393,  0.0346153043,  -0.28453958,  1.09631157,  0.187114432,  0.733974814,  0.0134687619,  0.399350792,  0.307174832,  0.0350675099,  0.488644689,  -1.3738569,  -0.339236975,  2.29078579,  1.92933249,  -0.668137074,  -0.218665242,  1.38525176,  0.690301538,  -0.0947471485,  -0.479248136,  0.253441513,  -1.06140983,  0.229439557,  -0.83009541,  -0.0969848335,  -0.930762827,  0.876874864,  -0.330923676,  0.41986087,  -1.0450604,  0.255892873,  -0.439223081,  -1.02251828,  -1.85441458,  0.486637175,  -1.29352009,  0.273697227,  0.7119385,  0.473807424,  -0.39334622,  -0.645291448,  0.169095457,  2.38224697,  -0.198360577,  0.0843240246,  1.49306822,  -1.84001029,  0.769391954,  -0.528069735,  0.816125453,  -0.922148108,  -0.406182855,  0.754220188,  1.55099392,  -3.03157306},{ -1.20231771,  0.514977574,  0.0435490645,  -1.35779989,  -0.81168735,  -0.606589496,  -0.00726137031,  0.099744685,  -1.28543162,  -1.11331153,  0.70899421,  0.548916876,  0.321932793,  -0.258569092,  0.610849798,  -0.398997128,  -0.369222283,  -0.542006731,  -0.713160157,  -0.2317072,  -0.321651816,  -0.743796766,  0.793036282,  -1.83527386,  0.254439861,  -0.705596626,  -1.03732729,  -0.144472644,  0.691304803,  -0.400258154,  -0.36717546,  -0.710357308,  0.0398407578,  -0.824873924,  2.50242186,  -0.960597694,  0.445999146,  -0.0441550836,  -0.361752093,  -0.792627335,  -1.56902385,  1.12759912,  -0.817855418,  0.67903018,  -0.101269372,  1.0173285,  1.04446459,  0.841564596,  -0.656529963,  -0.104501441,  -1.82430375,  2.44064164,  1.4585712,  0.545531571,  1.30138195,  -0.130816251,  0.122562401,  -1.37393606,  -1.11471879,  -0.93799746,  1.29235041,  0.321805507,  0.820735216,  0.1109569,  1.05503416,  0.794652522,  -0.130291238,  0.557969213,  0.0333923511,  0.0548300408,  -0.410948366,  0.77465868,  0.148572579,  -1.06919611,  -0.491772592,  0.0654459298,  0.0969100222,  0.260064721},{ -0.900517046,  -1.04808795,  -0.769569099,  -0.754526913,  -0.257331014,  1.01440477,  0.428874969,  -0.490616143,  -0.792696714,  0.197708189,  0.306395501,  0.884807587,  -0.805681646,  -0.0947224274,  1.1586777,  -0.148346588,  0.773964286,  0.268771678,  0.875367522,  0.765540242,  1.13539112,  -1.20678878,  0.187991664,  0.260431707,  -0.700170875,  -0.66912359,  1.64640856,  -0.449502915,  0.801523805,  0.435052902,  0.795055449,  0.761352003,  0.12908949,  -0.0507194176,  -0.508786082,  -0.782522559,  0.353871703,  0.357990146,  0.771176577,  0.165292427,  -1.48632216,  0.540670276,  -1.78251565,  1.16073632,  -1.1835686,  0.490628779,  -0.666648567,  -0.715662658,  1.00056398,  -0.887487233,  0.146486595,  -0.156302571,  -0.155359566,  0.0857839584,  -0.713448703,  -1.31208742,  -0.214861497,  0.493569255,  -0.91368103,  -1.03748,  -1.5254482,  0.575814247,  1.30969644,  0.905106425,  -1.19708622,  1.35632229,  0.662205338,  1.93556488,  -1.623559,  -0.379457265,  0.582084119,  -0.33807236,  0.562426507,  1.23746312,  -0.750775039,  2.9361608,  -0.0328701548,  -0.478459746},{ 0.0378353894,  -0.00800900627,  0.255916923,  0.352184683,  0.858881056,  0.956448913,  0.0702953786,  -0.380344182,  0.0280328169,  1.25464535,  0.900643229,  -0.851470768,  0.422274351,  -1.45666981,  0.732397556,  0.08188954,  1.60366225,  -0.735126674,  2.46790123,  -0.385485291,  1.03066599,  -1.20074844,  0.0437852256,  -0.267825127,  -0.310047656,  0.222554132,  0.128083855,  0.439775854,  1.10640466,  -0.236752689,  1.1502341,  0.238692373,  -0.262323529,  -0.17324695,  -2.64620686,  -0.653024673,  0.883771598,  -0.709674358,  0.286227912,  1.64462209,  0.334719151,  -0.27873376,  -0.95156765,  0.0262576789,  -0.903881609,  -0.737159312,  0.142808229,  -1.0646373,  0.14808853,  -0.284506828,  -0.148024917,  -0.84893322,  -0.137914628,  1.16988373,  0.503705621,  0.0186367836,  -0.331106901,  0.536099613,  -0.169067696,  0.624956012,  -0.716037273,  -0.0694736019,  0.65281409,  -0.628394306,  -0.223336205,  0.853656709,  -1.77059615,  -1.84682369,  -0.0453506932,  -0.0806033909,  0.361447603,  0.734635055,  1.5814898,  1.07709718,  -0.754235625,  -0.430236459,  -0.997550189,  -0.980964065},{ -0.769909263,  -1.04051411,  0.594140291,  -0.750156939,  -0.37382862,  -1.31201017,  -1.80449259,  -1.02532673,  1.22913623,  -1.81874907,  2.78537631,  -0.877571464,  -0.925203025,  2.15151811,  0.46085906,  1.55403733,  -1.19448137,  -1.01044953,  0.314654976,  -0.959833622,  -0.337843299,  -0.153647736,  -0.252469033,  0.45680812,  -0.536687613,  -0.0372306034,  -0.328144789,  -0.92262423,  -1.16751266,  -0.60268873,  0.596667945,  -0.302518696,  -2.47063303,  1.31885123,  0.584909379,  1.09814703,  1.36811376,  0.286934286,  0.337914705,  -0.875829518,  0.596434891,  -0.92051512,  -0.078268908,  -0.938566089,  -1.07932389,  1.96015239,  0.304529607,  -1.20734489,  1.78909814,  0.566485167,  1.61611581,  0.580280721,  0.339223593,  1.19253302,  1.29967821,  0.297768712,  1.28923154,  0.426336348,  -0.821862519,  0.824106634,  -0.947749913,  -0.272677928,  1.74543226,  1.3484633,  -0.743095517,  -0.999912918,  0.622919261,  1.41089308,  0.122448958,  -0.506882906,  -1.11472666,  -0.813681126,  0.0921196789,  -0.363220364,  2.12224126,  -1.1313349,  0.673362136,  -0.365830094},{ 0.917177916,  1.89761877,  -1.26433921,  1.7557106,  1.04223812,  -1.06469035,  -0.71987915,  -0.575397909,  1.96586919,  -1.20448709,  0.950934231,  0.588297904,  -0.088288106,  -0.412270188,  -1.48761487,  -1.37352347,  0.0942486078,  0.840367258,  -0.272381067,  -0.209847152,  -0.207396179,  0.586858928,  -0.236000687,  -1.27405083,  0.598677635,  1.28463912,  0.543532252,  -0.421652913,  0.363455266,  0.0566857196,  -0.406594336,  -0.972511947,  -0.0763605759,  -1.07310045,  -1.28918791,  -0.426086992,  1.99370503,  -0.166391864,  2.43927956,  -1.05174518,  -0.629476488,  -0.334957242,  1.43812084,  -0.654906094,  0.261820257,  -1.16211426,  -1.01078129,  1.87157154,  0.312307805,  -0.252985954,  0.0926295668,  0.832362115,  0.0513137169,  0.998039305,  0.611328542,  2.52039433,  2.02145696,  -0.702754736,  0.0456254482,  -1.33464479,  -0.656101286,  1.10773325,  0.298351437,  -1.18304431,  1.24217474,  -0.706770003,  0.102379575,  -1.44028175,  -2.14401937,  -2.15506315,  0.314421207,  0.0970957726,  -1.43102336,  0.160348341,  1.03037381,  -0.400171399,  0.304942697,  1.48006189},{ 0.149155885,  1.03900635,  -1.00935936,  -1.28811347,  -1.10356319,  -1.90354085,  -0.85308969,  0.120874643,  -1.45199549,  0.128040463,  -0.543519735,  -0.156869665,  -0.286114752,  -0.0243043527,  1.02618551,  -1.30340505,  -0.00427222624,  0.680150926,  0.0292483307,  0.448485076,  0.297735453,  -1.7184521,  0.746497333,  -1.54515588,  2.20847845,  -0.253752917,  1.48645473,  -0.367912084,  0.23076722,  1.02259219,  0.885083795,  0.900937021,  1.04735041,  0.803854764,  -0.58812809,  -1.17486238,  1.84504795,  -0.287377089,  -0.28548789,  0.670427322,  0.247159779,  -0.692531765,  -0.45116505,  -0.524379373,  -3.14559269,  -0.444856405,  -1.18333709,  0.090506047,  0.0829127431,  -0.575579762,  1.09733295,  1.34914398,  -0.0853637382,  -1.46209466,  1.63463295,  -0.153179199,  -1.12671208,  -0.0606705882,  0.0347482748,  0.388948232,  0.416120857,  -1.35460699,  -0.226556614,  2.33345747,  0.244503841,  -0.29438737,  -0.0812515691,  -0.628486931,  -1.53835714,  -0.233025834,  -1.90188992,  -0.102461547,  -0.178984523,  0.0146710798,  -1.51044273,  0.110386476,  0.988179386,  0.0243676398},{ -0.786067486,  -0.754265368,  1.80031323,  0.238139838,  -0.824421108,  0.314892709,  0.450308353,  0.125025317,  -0.25135085,  -0.434237331,  -1.29886878,  1.20497096,  -0.705101669,  0.682918251,  -1.3975904,  0.0789242685,  -0.559855282,  -0.877040803,  -0.606957078,  1.45425737,  -0.580551744,  0.571818233,  -0.799391806,  0.298873246,  -1.48663116,  -1.0195781,  -1.78714538,  -0.770260692,  -0.796571136,  0.228384852,  -0.126548022,  -1.87533128,  0.0135727078,  -0.0380341746,  1.68043637,  -1.79145515,  -0.213726223,  -1.29708827,  -1.73566282,  1.41205275,  0.238247007,  -1.36355293,  -0.632762969,  -1.69277132,  -1.22693551,  -0.268298626,  -1.44505215,  0.57681644,  -0.617499471,  1.32262731,  1.005162,  -1.28363442,  -1.10000384,  0.24633801,  1.93035245,  -0.589631081,  -0.13628082,  0.163877577,  -0.317059755,  -0.478171766,  -0.241559282,  -0.272643983,  0.433995485,  -1.6055162,  0.342815936,  1.09459412,  -1.05561364,  0.943314612,  -0.744229317,  0.832008123,  -0.774084032,  -0.777433455,  1.51758742,  -0.097270444,  0.823081911,  2.18016815,  1.46102154,  0.0224705227},{ -0.406736195,  -1.12905157,  0.800323069,  0.703589022,  -1.32675314,  1.13742507,  0.479910463,  0.256089687,  1.57911789,  -0.829370916,  0.479727298,  -0.454274863,  0.18842335,  0.394984424,  -0.628512561,  1.74133277,  -1.03856277,  -0.154394314,  0.509488225,  0.778096914,  -0.58346343,  -2.17893696,  -2.02802467,  -0.0765179619,  -0.159963831,  1.31148112,  1.3711499,  -0.761086941,  -1.80675197,  -0.899997234,  0.44278124,  -0.153466135,  -0.0415298119,  -1.20657289,  -0.107804112,  -1.72228348,  0.991555452,  1.06681097,  -2.13443851,  0.160273641,  -0.779392481,  0.150981963,  0.660602093,  0.988400042,  0.496848017,  1.89768946,  -0.436155438,  0.355946064,  -0.438845754,  -1.62410033,  -1.08485758,  -1.10585594,  0.671652198,  -0.25244987,  0.0633765683,  0.250016332,  -0.436941475,  -0.0384497643,  -1.5308218,  -0.24834381,  -1.8835535,  0.559122264,  0.294835895,  1.94895375,  1.9345535,  0.659242868,  0.636374354,  0.656533837,  0.214926466,  1.30286038,  0.148367912,  -1.12357581,  1.20376432,  -1.12681031,  -0.936185539,  0.737723112,  -0.0042375722,  -0.0333200283},{ 1.371382,  0.710002422,  1.20088029,  -1.73158062,  -1.70092595,  0.519030631,  -0.168970481,  -0.16180715,  0.483056694,  0.30017373,  0.872475982,  0.334839672,  -2.4437983,  0.639411867,  0.0757433027,  -1.28624475,  -1.07764435,  0.148133367,  -0.946095586,  0.298522949,  0.48367238,  0.154075533,  0.718316257,  -0.710473895,  0.861718893,  -0.479299724,  0.839564145,  1.11514258,  0.913182139,  -2.52486873,  0.183170065,  1.67266405,  -1.32161272,  -0.203223646,  -0.0717962533,  -0.906177402,  0.580827415,  0.35378173,  1.04934549,  -0.385239333,  0.6360991,  0.739616156,  -0.307097286,  -1.43436503,  1.03377056,  -0.0912895426,  -1.44367445,  0.270123512,  -0.18351528,  -0.18270117,  -1.57961571,  0.804603159,  0.233393803,  1.34992385,  0.51164639,  0.441944242,  -1.43771768,  0.963201165,  0.118517108,  -0.945872366,  0.154728815,  1.72846138,  -0.763258517,  2.37415028,  -1.48226595,  -0.553461134,  -1.77913308,  1.3404876,  1.13545322,  0.648049712,  -0.857750237,  1.46674275,  -0.405852467,  -1.16666353,  -0.632439196,  0.768081486,  -2.14829159,  -0.540331244},{ -2.23438501,  -0.466671735,  0.0388889275,  0.291137367,  0.568288088,  0.527397633,  -0.194473475,  1.01639938,  -2.5397234,  -1.42290533,  1.07012546,  0.264575273,  -2.47523499,  1.81063473,  -0.124002412,  -0.0689094365,  -0.906807661,  -0.533763349,  -0.831607819,  -2.16521692,  -0.732788324,  -0.724294722,  -2.71155238,  -0.426068515,  -0.913605511,  0.190986916,  0.309697896,  0.832060337,  -0.13873893,  0.650451779,  -1.23017049,  -0.139362141,  0.316078603,  -2.32116532,  -0.98791039,  1.75339127,  0.301746428,  -0.308362335,  0.54448247,  -1.08389854,  -0.368395627,  -0.708083212,  1.16091967,  1.71163273,  -0.80701375,  1.78244197,  0.574660182,  -0.1987333,  -0.634623408,  0.622900724,  0.491398394,  0.837263107,  -0.153634042,  -2.78145766,  0.148438647,  1.38534725,  1.61157119,  -0.746041954,  -0.250090301,  -1.8593533,  -0.899310172,  0.183020681,  0.38799122,  -0.477467716,  0.556090117,  -0.306975484,  -1.89484465,  -0.624199152,  -0.650945127,  0.459185869,  0.452219427,  -1.88964629,  1.10149527,  1.17601573,  -1.07949805,  -0.846476495,  1.64478612,  -1.0516994},{ 0.140383556,  0.269707322,  -1.12748325,  -1.54961979,  -0.27804023,  -0.295661181,  -1.31190288,  0.877426147,  0.333093792,  0.557249784,  -0.844821036,  0.676619709,  -0.626590014,  0.511222243,  -0.0433010794,  -1.08534098,  1.65112066,  0.545144498,  -0.401128203,  -1.30627346,  0.256770372,  -0.458494544,  -0.959102094,  -1.17568958,  -0.408291847,  0.160154894,  -2.12491488,  -0.066485025,  -0.0811096579,  -0.640054226,  0.178539604,  1.63206077,  -0.50807482,  1.376423,  -0.0293690842,  -0.0621251576,  1.1478641,  -0.526633978,  1.39792264,  -0.116242737,  -0.111876123,  2.23162413,  0.81052953,  0.867368817,  -1.254794,  -0.779736221,  0.514793694,  -0.314278483,  -1.91233778,  0.0613990836,  0.133371904,  1.15019298,  0.956729114,  0.693087578,  -0.269290686,  -1.03162837,  -0.0509901047,  -0.219421238,  -2.5584054,  -0.35992521,  -1.35207403,  -0.928617537,  -1.35562181,  -0.696673751,  0.772225022,  -0.664793432,  -0.130303487,  1.43942261,  0.591646612,  0.186454758,  1.19293141,  -0.222878128,  -1.24934101,  -0.196909919,  -0.0570301078,  0.169886172,  0.0590310134,  -0.111870483},{ 0.842128456,  -0.204667553,  -0.873831093,  0.326358259,  0.936200678,  0.0694565102,  1.34366488,  -1.91161251,  -1.82437491,  -0.434769541,  -1.8365562,  -0.351823658,  0.807611585,  -1.80563378,  -0.681071281,  -1.26321793,  0.853059351,  1.88278043,  0.862431169,  0.00901525188,  1.12660134,  1.52684605,  1.10494006,  0.650662005,  -0.57247436,  -1.11236584,  -0.804900169,  -0.563954175,  1.57447159,  -0.42700541,  1.06887484,  0.207378268,  0.845390856,  -0.946854115,  -0.691649914,  0.779070318,  0.947055042,  -0.693337083,  -1.21520424,  -0.392307103,  -1.85702443,  -1.25608575,  0.0486505069,  -1.76941884,  -0.329022199,  -1.18323636,  1.48704541,  -1.73279762,  1.06204104,  0.688203037,  0.411585242,  1.28542709,  -0.185775414,  -1.88340199,  -1.00011623,  -1.24012458,  -0.250755668,  0.0995438471,  0.267970949,  0.816788614,  -0.250943691,  -0.86314261,  -0.305243492,  -0.918651104,  -1.09207022,  -2.64155626,  2.28061295,  -0.394309074,  -0.842645645,  -0.216599077,  -2.13063192,  0.963097215,  -0.357803732,  0.193018705,  0.374157697,  -0.56248796,  -0.514980495,  -0.272202969},{ -0.202883914,  -1.18311918,  -0.530535579,  1.43346977,  0.127829298,  0.101297848,  -1.69524038,  0.768189728,  -0.827920496,  -2.0562427,  -1.4191972,  -0.62395978,  -0.148718849,  0.229942188,  -1.32686865,  -1.08961225,  0.613849819,  -1.12902796,  0.270919085,  -0.772284806,  1.25217652,  -1.18206668,  0.589997292,  -1.7480818,  0.576191664,  -1.21261334,  0.325492859,  2.0296185,  0.760739148,  0.556475878,  0.497132629,  -0.903669238,  0.421041697,  -0.933074653,  0.674354911,  0.0434733182,  -1.1994406,  1.44632924,  1.66650498,  1.33702064,  -0.294831485,  0.11652936,  2.51945138,  -1.85240722,  0.128938481,  -1.27203918,  0.34339878,  -0.930796921,  2.05933666,  -0.872259021,  0.480344057,  -0.780678153,  -2.16926265,  -0.39729318,  0.0895169526,  -0.275907665,  -2.00713062,  0.158265486,  0.117031284,  -0.681370854,  0.745553792,  -1.68716788,  -1.41743171,  2.52212524,  0.379434288,  1.6882143,  -0.430123806,  -0.985555172,  0.25968802,  -1.23624325,  0.911963224,  0.0893827006,  -1.36555874,  1.36615324,  -0.387332201,  0.622661769,  0.858930349,  -0.508706331},{ 0.815694213,  -0.199248597,  0.896089375,  0.721589565,  1.55394626,  -0.0489606783,  1.68383265,  -1.85409105,  -0.305990666,  0.133616716,  -0.487681627,  -0.95106709,  0.616099477,  -1.67655075,  -0.941836417,  -0.515483975,  0.192020342,  -0.359071106,  1.43392444,  1.88835919,  0.202643901,  -0.618859529,  0.564586639,  1.38942003,  -0.342148483,  -0.520079494,  0.943479717,  -0.232964873,  -2.11850715,  0.44912526,  -0.299989313,  -0.143693954,  -1.40481734,  1.10553074,  0.131769478,  0.236958832,  0.963588357,  1.74829292,  -1.81995058,  -1.73814535,  -0.294798553,  0.916817784,  1.8198328,  0.902540803,  0.823986053,  0.207080856,  0.35716176,  0.688812494,  -1.7296387,  1.38804173,  -0.308512717,  0.462695897,  -0.428682953,  -1.67000866,  -0.54665947,  1.05671883,  0.607482612,  0.686555743,  -0.123627216,  -1.01135159,  0.132542878,  -0.352983028,  -0.606417477,  -1.03146112,  0.0384027138,  -1.18286133,  1.7298789,  1.67263103,  -0.623983204,  0.93374747,  1.50445747,  -0.206887245,  -0.287323207,  -0.00223992486,  -0.723954201,  0.380419999,  1.19469523,  -0.0559167266},{ 0.450419694,  1.12539816,  0.117169358,  1.10080969,  0.563390493,  0.0617145933,  0.664081693,  -1.23754215,  1.50439072,  -0.630849302,  -0.204259485,  -2.08890557,  1.98877692,  1.35043001,  2.97685695,  -0.196990475,  -2.07621932,  0.823658466,  -1.55641961,  0.928919017,  1.22751212,  1.45335579,  -1.54657221,  -1.48631215,  -1.06754041,  0.786266744,  0.0174482223,  0.320545405,  -0.468423456,  -0.0653659552,  1.1055522,  1.27438319,  0.0848313272,  -1.8250438,  0.960138083,  -1.26501572,  -0.459973305,  1.09552085,  -0.450509042,  0.526867092,  -0.41250053,  0.895432234,  0.0194075443,  0.969685256,  -1.01676762,  0.724721253,  -0.0147399716,  -0.268222362,  -0.70483458,  -1.26111042,  1.85541403,  -1.51261938,  0.240687966,  -1.14281261,  0.371465623,  -0.691844106,  2.22992921,  0.0106260646,  0.517251372,  1.23253667,  0.102828734,  1.09629178,  -2.93906927,  1.53777242,  -0.23570329,  -0.339402735,  -1.3922075,  1.49970257,  -2.64625072,  2.93539453,  -0.0322008505,  -0.988342643,  0.326212257,  0.658112764,  1.44879138,  0.813731968,  0.221252084,  -2.01522231},{ -0.0710134432,  -0.142531604,  0.589170575,  1.19912851,  -1.0931145,  1.53401804,  0.333430499,  0.0436400957,  -0.284527838,  0.82260412,  2.15480542,  -0.446089029,  2.81379056,  -1.3241986,  -0.0364707969,  -0.676522136,  -0.0833092257,  -0.90673691,  0.522635937,  -0.55024296,  -0.414894223,  0.821662307,  0.117203645,  -0.0948460624,  -0.42518574,  0.823803425,  1.21656728,  0.879565418,  -0.687038183,  0.201409221,  2.77006507,  -0.430630326,  -0.174610853,  1.30690861,  -0.596946061,  0.139262125,  2.6808157,  1.44191229,  -0.930828154,  -0.00441112043,  -0.291792244,  0.773090124,  -0.722643495,  -0.851355195,  0.455570936,  1.36500823,  -2.1379776,  0.61454314,  -1.21726072,  0.888842106,  -0.199747592,  -0.828233242,  -0.440397412,  -1.20162308,  0.856143594,  -0.655092239,  -0.81480515,  -0.434418052,  -0.719060361,  -0.705655754,  -0.0729734823,  -0.615515471,  -1.00844359,  0.53667444,  -0.274301708,  -1.57068527,  0.786177337,  -0.267533481,  -0.471791089,  -1.18730545,  -1.9973743,  0.996260583,  -0.868951499,  -0.858977914,  -1.27425754,  -0.111856334,  0.186837241,  0.0225216039},{ -0.572787404,  1.48234832,  -0.00436226092,  -1.68491399,  0.813827872,  1.52020776,  -0.351277918,  0.417560816,  1.60962272,  0.485851884,  0.808239341,  -0.759813845,  -0.121952303,  -0.2248611,  0.814960182,  0.233686298,  -0.439056188,  1.86377299,  1.05701864,  -0.891761482,  0.910239756,  0.425880641,  -0.50235188,  1.3888154,  -0.821969688,  -0.153676122,  -0.0445227362,  0.524660408,  1.86643565,  0.0229395628,  -0.782638431,  0.774688065,  -0.34784025,  -0.067289494,  -0.285917342,  -0.0616143234,  0.926629722,  0.606693923,  -0.925215662,  0.210654497,  -1.41806364,  1.5963608,  2.35513973,  0.545487404,  -0.0882331058,  0.992354393,  0.554998338,  -0.599356234,  0.0294330604,  1.60711968,  0.676287711,  0.575919211,  -0.931653798,  0.0697796196,  0.586561441,  0.944954574,  0.675827622,  1.75845087,  0.671279788,  -1.45687222,  0.891010225,  -0.732213736,  0.192660734,  -0.415011287,  0.721486866,  -0.575696051,  0.854454994,  -1.14476919,  -2.88115025,  0.153942898,  -1.28377712,  -2.28500724,  0.31201458,  -0.579696596,  -2.55647731,  0.225930244,  0.03761461,  -0.595983863},{ -1.14362144,  -0.528728366,  -0.526518106,  -0.159315273,  0.100777835,  -0.445407331,  0.573795676,  -0.353183717,  0.269286901,  2.42425966,  -0.765185118,  -0.613470733,  -0.756631613,  -0.239762843,  -1.5283972,  0.256076068,  -0.813624978,  -1.26858401,  -0.0501278341,  1.19151282,  -0.756550848,  -1.2574687,  -1.57882702,  -0.173084989,  -0.714399517,  -0.637894571,  -0.0668096021,  1.96117747,  -0.269393951,  0.654041588,  -1.05846059,  -0.580445468,  -0.565363169,  -1.82916343,  0.410301358,  -0.235904381,  0.81193924,  0.91466099,  -1.39652801,  -0.19301872,  -0.247275561,  0.0117435073,  0.46066761,  -0.0982453227,  -0.534626305,  0.383305758,  -1.3078773,  -0.324785978,  -1.10872662,  -1.28285587,  0.751179397,  0.203565359,  1.81365633,  -1.11859953,  1.45822382,  -0.787556589,  -0.364232659,  -2.34822273,  1.8307476,  -0.536918938,  0.10315989,  -1.39555299,  0.666271925,  0.663256168,  2.12354779,  0.25006032,  1.42862129,  0.198520407,  0.977363467,  0.314070463,  -0.891568124,  -0.678404033,  -1.48528302,  -0.919318855,  -0.729594529,  -0.170980275,  -1.22253799,  0.516975701},{ -0.35597828,  0.465388566,  0.738016248,  0.668191433,  -0.814713895,  0.985319853,  -0.0628024042,  2.37811637,  -0.132377476,  -0.164037198,  0.563025892,  -0.415795803,  -0.476329654,  -2.18714476,  1.10198641,  -2.74841118,  -0.265609324,  -0.404028237,  -1.47886741,  -0.752876878,  1.15535951,  0.97857815,  2.17340589,  -1.24166155,  -0.26875329,  1.65686285,  -2.24978209,  0.161681339,  0.823254228,  -1.37278962,  0.0671723261,  0.595410645,  1.29110777,  -2.2153964,  0.0998273864,  -0.293778092,  0.0310517382,  1.39681482,  1.99746549,  2.73459435,  0.0636379868,  1.900424,  0.683089137,  0.491205096,  -0.647410095,  0.726458848,  -1.19555044,  -0.502713203,  1.0720259,  -1.13907444,  -0.0166675113,  -0.503434658,  0.922727644,  -0.177198112,  -0.33336553,  -0.450730801,  -0.0520730615,  -1.10691881,  1.02786398,  0.912923813,  0.19343026,  0.237909406,  1.7101357,  0.716855645,  -0.921169817,  -0.423891991,  -0.179608047,  0.964811265,  0.118254706,  1.22871256,  0.202807322,  -0.565973997,  -0.319992453,  1.01581967,  1.90250552,  0.061622601,  -0.740231454,  1.55138755},{ 0.685438275,  0.192872465,  0.619153976,  -0.367738098,  -1.08500695,  1.29344559,  -1.06329,  -1.39368057,  -0.381057233,  0.29422915,  0.286101937,  -0.542282939,  0.456768483,  0.480906755,  -0.0762891322,  -1.21097946,  -0.107556447,  -0.652866006,  1.8820678,  -0.764554024,  0.122448586,  -1.67472863,  0.835447729,  0.441956937,  0.36071524,  -0.406616837,  -0.494482189,  -1.29042482,  -0.0871191248,  0.785968959,  -0.219446734,  -0.554318249,  0.709147573,  0.859537005,  0.703116536,  -0.886559129,  -0.72734791,  -1.96047068,  0.954409659,  0.13725391,  -0.597685993,  1.7174052,  -0.701223195,  0.934932053,  -0.422040999,  1.74967206,  -0.180126399,  -1.28760612,  -1.20298612,  0.818734765,  1.68387878,  1.7582655,  0.79593569,  0.122388788,  -0.266311377,  0.524664402,  1.24176013,  0.997067273,  0.331910133,  0.774911642,  0.729911804,  1.32242811,  1.24826968,  -0.0716222376,  1.70558631,  -0.466873527,  0.461176395,  1.30852509,  -0.0314008333,  0.182106838,  -0.261314332,  1.07108533,  -0.445342839,  0.787271678,  -1.0567348,  0.70289129,  0.807487965,  -0.149923056},{ 0.726906121,  1.26964629,  0.289327204,  -1.43297052,  1.2603339,  0.164330184,  -1.47548091,  -0.147842705,  -0.18214722,  1.43144417,  1.88071382,  0.255559236,  -0.783494353,  0.966097951,  0.234052509,  0.63872993,  -1.82109153,  -0.972666502,  -0.468289673,  -0.578738809,  -1.50105107,  0.140574977,  0.664999008,  1.24352384,  -2.18013573,  0.201497465,  1.1827184,  0.0819233954,  0.59397012,  -1.07593942,  -1.44355547,  0.122050665,  -0.645818651,  0.0175718702,  1.01529014,  -0.430801958,  -0.465501726,  0.333126754,  1.30113554,  1.64264631,  0.967599213,  0.0813505873,  0.52888906,  -0.365445852,  0.990042329,  0.300209373,  -0.0727811232,  0.256852567,  -0.531122804,  -1.17358887,  -1.72638118,  1.56056392,  -0.269708693,  0.537983775,  0.501704931,  -1.44505394,  0.379757911,  0.508199036,  0.747828841,  -0.452987522,  -1.75491607,  -1.35775685,  -0.097209245,  2.07426572,  -1.27989161,  0.468203664,  -0.0218640883,  0.75484103,  1.1519351,  1.11336136,  0.384616822,  -0.224353135,  0.717362702,  -0.969817221,  0.724568307,  0.548369527,  0.892886341,  2.43554115},{ 0.767259419,  -0.738638103,  0.274213225,  -0.162301436,  -0.476233631,  -1.27393973,  0.130487517,  1.05897927,  0.567970872,  1.26683581,  1.32206178,  -0.00823141728,  0.682645082,  -1.51783192,  -1.60199606,  -0.599601924,  0.27788347,  1.26026535,  1.53884363,  -0.510862529,  1.73380268,  -1.2729938,  2.62006164,  1.59710884,  0.211638689,  0.976659119,  1.40255356,  -0.339770168,  1.2371639,  -0.169406161,  -0.600924015,  1.22397125,  -1.25901592,  -1.36325884,  1.05517292,  -0.450348914,  0.252673954,  1.05142891,  0.335737944,  0.427072555,  0.0330617093,  -1.06835127,  -1.00956404,  -0.791587234,  -1.08928084,  1.01135373,  0.213000298,  0.14781636,  -0.598868728,  -1.95030236,  1.57381094,  -0.736477017,  0.372737586,  0.218763217,  -0.0314121358,  0.583106399,  -0.72047919,  0.0246986747,  0.535475612,  0.474641502,  0.00380484178,  0.018188525,  0.48389402,  0.669775903,  1.00099635,  -0.166827306,  1.07494271,  0.391425997,  1.00886881,  -0.120975353,  0.155669585,  -0.0474846512,  -0.595519185,  -1.44371438,  -3.17230225,  -0.158698052,  0.822599888,  -1.54172051},{ -2.03901935,  -0.717834175,  -1.00405157,  -0.58212924,  1.16933179,  -0.117638543,  0.746562958,  0.00707573164,  0.439704865,  0.206330463,  -1.02976978,  1.93176568,  -0.26788938,  0.54812777,  0.00864933617,  1.69690681,  0.686374962,  -1.30043459,  -1.68281734,  -0.421739489,  -0.0753872171,  -0.386977106,  -0.709423125,  0.118724637,  -0.0985648707,  0.677794278,  0.528594911,  -0.291670471,  1.07937741,  -2.02392864,  0.529656887,  -1.29349542,  -1.41305673,  -0.390116483,  -1.7769717,  0.719134271,  1.93630624,  0.979846835,  0.0280407444,  -0.602847219,  -0.867816746,  1.14076233,  0.0561208799,  -0.387805372,  -0.966772199,  -0.766193807,  -0.925843358,  0.0886099637,  0.526213527,  -0.245730281,  -1.30510902,  0.595363021,  1.33060586,  0.0696003884,  0.348031551,  -0.848258436,  -0.327773303,  0.970719278,  0.523624241,  -0.998764575,  -0.00253372523,  1.22964549,  0.783700705,  -1.53965616,  -1.15499187,  0.224688262,  1.56701267,  -0.919327497,  0.660219073,  -0.889084756,  0.0557723306,  1.88708103,  -0.604302585,  -0.592204452,  0.523889303,  0.101424679,  0.986684144,  -1.07030463},{ 0.719060302,  -0.323705435,  -0.274759352,  0.153546184,  0.404634386,  -0.771155179,  0.73499912,  1.35166597,  -1.02001679,  1.26540744,  0.421620697,  0.276052535,  1.48447657,  0.829429328,  -0.246859863,  1.49133968,  1.57350743,  -1.55620527,  1.37310112,  -1.22051954,  -0.591504395,  0.271731347,  0.541859746,  0.53755039,  -1.43002439,  -0.990249097,  1.04549742,  0.0585593544,  -1.76819086,  0.212863997,  -0.552272499,  0.564405799,  1.04052687,  -0.0984348804,  -0.557289064,  1.38776791,  -1.16086781,  -0.315493345,  2.16628051,  0.302484006,  -0.345630437,  -2.59742141,  -0.681069553,  1.18525052,  0.566937745,  -2.05566692,  -0.248167396,  0.997879446,  1.15254188,  0.535601139,  0.401988596,  1.53839946,  -1.21472752,  0.522697747,  0.373707175,  -1.37425196,  -0.342406541,  -1.53346217,  -0.132555977,  1.13921118,  -1.907601,  -0.659374356,  -1.02223563,  0.70012176,  1.54641032,  -1.02865648,  -1.37064564,  -0.175115004,  -0.15265885,  -0.55909586,  -0.271900684,  -2.55541921,  0.906398535,  0.778661668,  -1.2700026,  0.391086519,  0.257564157,  0.43143639},{ -0.278033257,  1.06476867,  -1.12157619,  1.63768876,  0.890277982,  1.17679393,  1.93046677,  1.3208307,  0.453234106,  0.214251652,  -1.15720785,  -1.2777698,  2.06523442,  -0.00724269357,  -1.00903058,  -0.0911699831,  -1.32652521,  0.158575103,  -0.0185613688,  1.29978788,  -1.12002265,  0.155291334,  0.150572419,  0.955078661,  1.50579977,  -1.04484141,  0.619253933,  0.0369915105,  0.977463365,  -0.88866967,  1.68859696,  0.61640799,  -0.795742989,  -0.163282424,  0.486019731,  -0.0299130399,  0.105513804,  -0.180112645,  -1.32773161,  -0.225393772,  -0.477214605,  -1.04681373,  0.676512241,  -1.298298,  1.1394707,  0.396856844,  0.684694409,  -1.14495909,  -1.47587895,  -0.0793713853,  0.445329368,  0.295415401,  -1.98962963,  1.31220078,  1.38377297,  0.536562681,  0.427812457,  -0.188153073,  -1.30629575,  -0.320087761,  -0.280784905,  1.41122794,  2.26055431,  0.376725435,  1.62533367,  0.145020992,  0.152848199,  2.40056562,  -0.438244253,  -1.85321569,  -0.691198766,  0.135190517,  1.12463427,  0.0885675773,  0.975801349,  -0.807468891,  -0.12777622,  -0.584793389},{ 0.989837527,  0.613747776,  -0.12077795,  0.375522763,  -0.0910109803,  -0.775612891,  0.163383454,  0.58778131,  -0.368380845,  -0.713793099,  0.082134746,  0.539397597,  0.411988646,  -0.437238932,  1.55980754,  1.49810815,  1.23952627,  -0.112013943,  -1.09546924,  -0.823907256,  -0.245243654,  -0.517412484,  -0.235128254,  1.64354873,  -0.772125125,  1.88107502,  -0.37439087,  0.799870491,  -0.817833304,  -2.44790053,  -0.36450392,  1.48310745,  0.234845787,  1.43591607,  -1.43926418,  0.167384356,  0.206410378,  -0.330572397,  0.47500366,  0.250671208,  1.20768976,  -0.733565271,  0.452302456,  -2.15448022,  -1.2573365,  -0.505892634,  -0.178576142,  0.305825979,  -0.65865314,  -0.551568687,  -0.385573208,  1.57051349,  -0.896398187,  0.0794268399,  0.123708606,  -0.122879148,  0.889671326,  -0.0653000325,  0.0899456665,  -1.26866186,  -0.73352766,  -0.1834203,  -0.492398739,  -0.936181247,  -0.556416214,  0.0463823341,  -1.75136161,  1.40156937,  -0.52704668,  -0.459568769,  -0.383752376,  1.08770061,  -1.11696064,  0.27311793,  1.38383949,  -0.0504893623,  1.79642618,  -1.37151337},{ -2.14656496,  2.25735879,  0.21357058,  -3.84789443,  -1.57989764,  1.36570859,  -0.912440419,  -0.948696911,  -1.05885983,  0.211919501,  -0.44754526,  1.11723065,  1.62323642,  -0.713747561,  0.240009353,  -1.26191175,  -1.80347669,  0.152901754,  0.769495547,  -0.792208672,  -1.32874978,  -0.0159094948,  -0.605335891,  -0.111786626,  1.77903509,  1.33189201,  1.45648706,  -0.788619936,  0.165816098,  -0.0512901619,  0.863683105,  0.313510358,  0.513674736,  -1.39438176,  -0.114618868,  0.604569077,  0.793005109,  -0.547262013,  0.386103243,  -0.125133902,  -0.103526138,  1.77388465,  0.052928824,  0.789825857,  -1.22452366,  -0.0397755764,  -0.379387915,  1.95721543,  0.0596624911,  0.00857821107,  2.16499996,  -1.06720877,  -0.756807566,  0.169761777,  -0.293943197,  -0.910778046,  0.413653702,  2.37420797,  0.405667812,  0.254579365,  -0.0338859297,  0.252968043,  -0.246700689,  1.2590152,  0.585379004,  -0.920395911,  0.0531264916,  -1.13355124,  -1.02185869,  0.953359187,  0.339320272,  1.80159533,  1.32341456,  0.5433622,  0.899250329,  -2.11153078,  -0.23356469,  -1.36072171},{ -0.173799917,  0.320770562,  0.661561728,  -0.533405781,  0.0456389673,  -1.89932334,  -1.34610581,  -0.19901669,  -1.91916645,  0.920370877,  1.28571272,  0.691550016,  -1.00975347,  -0.656874061,  -0.674046457,  -0.119049735,  0.708911121,  0.195841983,  2.04552388,  1.1922096,  -0.356049627,  -0.0653596446,  -0.2685664,  -0.0783281699,  -0.114614584,  2.3881669,  -0.62769419,  -0.749853134,  0.0664303377,  -0.24708505,  -0.10691411,  0.318454534,  -0.899586558,  0.195636287,  1.19235992,  -1.26858175,  -0.971282125,  1.22671282,  -0.592124343,  1.36865151,  0.329565793,  0.987036526,  -0.658864558,  0.552805483,  0.424714595,  1.06786251,  -2.26122832,  0.620775938,  -0.606027901,  0.54753387,  -1.11511111,  0.0194655117,  1.41129792,  -0.382808447,  -0.365069836,  0.575376213,  0.326883137,  -1.03433847,  -0.394868821,  -1.06866777,  -1.46386468,  -0.448049754,  0.334351331,  0.17772083,  0.141889244,  0.43370679,  -1.1477617,  -0.849360406,  0.645175695,  0.511926234,  1.04635274,  0.640632451,  -0.217986315,  0.663119256,  0.366733938,  0.377130479,  -0.343662024,  0.254294187},{ -1.26314569,  1.64813864,  -1.29954302,  0.0706237257,  0.538758278,  0.0841836631,  -1.47805989,  0.407152414,  -0.80856365,  0.00293915626,  -2.08532429,  2.06008482,  -1.76897466,  0.30514136,  0.860521913,  -0.0676460341,  -0.207128882,  -0.852244437,  -1.74388444,  -0.163195878,  1.55869222,  -0.155779704,  -0.14847742,  -0.606160522,  -1.25701571,  -0.856644809,  -0.778207183,  -0.926999748,  0.661423802,  0.105907783,  -0.905090094,  -0.0697092637,  0.40177241,  -1.98496532,  0.41542092,  0.80062896,  -1.12901199,  -0.744535923,  0.330585182,  -0.799544215,  0.623283863,  -1.32893205,  -0.713736117,  0.652191401,  -0.243633047,  -0.295045674,  -0.695045352,  1.34083819,  2.45476937,  1.67235661,  0.0544665419,  -1.20421553,  1.52535093,  -1.98286676,  -2.1121068,  1.05640304,  1.72153342,  -0.884894788,  1.18716884,  0.0136618223,  -0.429241151,  0.815241456,  0.213860229,  0.737743855,  0.585768282,  -0.104154386,  -0.235829249,  -0.303850681,  -0.510344267,  -0.413253665,  -0.203578413,  0.249195293,  -2.42527437,  0.426401913,  -0.402662754,  -2.23392677,  2.06419325,  1.07774854},{ -0.0803780109,  -1.39931631,  0.514926732,  -0.529493332,  0.446646541,  -1.71261752,  0.726548493,  -0.119838178,  0.467624217,  -0.666082263,  -0.936147749,  1.91206062,  -1.95774925,  3.21040845,  -1.46306527,  -1.20314872,  0.756133854,  -2.88925529,  -0.787740588,  -0.526017606,  -0.963329375,  -0.455634713,  -1.99361682,  1.39690101,  0.147220343,  1.05692935,  0.281274676,  0.448693842,  0.433653414,  -0.564345539,  -0.556148171,  0.0804706961,  -0.782621801,  0.172073483,  0.281009316,  0.0719204098,  -0.595508337,  1.28883493,  1.43343413,  -0.191472054,  -0.645142078,  -0.481148183,  1.42400312,  -0.191287115,  -1.58250189,  1.26423621,  0.119177707,  -1.90823805,  -0.55077064,  -0.165243313,  -0.594389796,  0.907889724,  -0.0517287478,  -0.214786872,  -1.18121994,  0.986141264,  -0.550494194,  1.19262075,  0.929350674,  -0.878514886,  -0.721659005,  -1.28790319,  1.70665884,  1.5301652,  0.652804434,  0.791910946,  0.430041701,  1.63019621,  0.224941358,  1.84459138,  -0.600815117,  0.0426841676,  0.769189596,  -1.31632185,  -1.60555494,  -1.19383192,  1.01777339,  1.20068419},{ -2.23123407,  -0.52111423,  1.73280215,  0.549885511,  -0.0933639854,  -2.1662209,  1.29906285,  -1.10155213,  -0.448461771,  0.660358548,  -1.21375358,  0.00740592089,  -0.563480139,  0.292504847,  1.84151864,  -0.366041541,  -0.00541624939,  0.0298752524,  1.36038852,  0.92237401,  1.54166806,  -0.00385662168,  0.261811167,  0.293958813,  0.334657371,  -0.439712256,  -0.342967838,  0.286328346,  -1.17086482,  -0.923486531,  -0.126784399,  -0.356031954,  -2.50582981,  -1.20460081,  0.305384457,  -9.58154778E-05,  0.232782587,  -0.0648516417,  1.171489,  -1.47973013,  1.46377993,  2.80692673,  0.576045513,  -0.0207385011,  0.565735877,  -0.088844344,  -0.425943345,  0.188439548,  -1.31505883,  -0.326666445,  -2.19156194,  -0.407213122,  0.441535652,  0.20227778,  -1.46895397,  -1.20164061,  1.32771063,  -1.40921652,  -1.65202284,  -0.748981178,  1.63986278,  0.0670944825,  0.292551547,  1.07421613,  0.719378769,  0.261508435,  0.273150235,  1.7861495,  0.0189314745,  -1.0449605,  0.765518308,  2.04801369,  -0.0457500666,  0.447355419,  -0.445097774,  0.252513587,  0.170866117,  -0.14344947},{ -0.24940075,  -0.245801568,  1.00372505,  0.778951228,  -0.337187529,  1.15896022,  -1.37578285,  -1.49286819,  -0.496422827,  2.24966526,  1.12681925,  -0.0242182314,  0.294915825,  -0.0165670309,  1.42002869,  0.461517125,  -0.855672181,  1.69438457,  -0.0469109081,  -0.781255543,  0.586560905,  0.0858525783,  -0.811671376,  -0.45191285,  -0.813242137,  -0.69657129,  0.267768323,  0.487013936,  0.671376944,  0.390312612,  0.653925598,  -2.02082205,  -1.39410484,  -1.72971356,  1.82059741,  -0.508083642,  1.17984605,  -0.47429508,  0.0288942661,  0.606925309,  -0.73170799,  1.15282881,  1.10405385,  -0.567135572,  0.899101019,  0.216856837,  -0.0638580918,  1.11698234,  -0.671595037,  -0.588531256,  -1.51118541,  1.41827166,  0.147004455,  -0.935939431,  -0.552598596,  -0.267109096,  0.972547352,  2.08824086,  0.135040939,  0.19340533,  -0.0368118472,  -0.581312597,  0.296048462,  -0.426303327,  -1.86230946,  -0.175003931,  1.02074289,  0.846055627,  0.184001356,  -1.26609957,  1.35671687,  -0.554433048,  -0.362989873,  -0.171914741,  -0.676181614,  0.714990735,  -0.917137384,  0.488061011},{ 1.10256755,  -0.13722074,  0.284755856,  0.781763077,  0.511239469,  -0.139253244,  0.106689274,  -0.773300886,  -1.23202562,  0.753629506,  -0.766866922,  1.14331579,  -0.186783567,  -0.247750521,  -0.0203914642,  -0.915956616,  0.096565187,  0.649521589,  -2.05035758,  0.358741015,  -0.566288054,  -0.180717155,  0.0327087678,  0.571307302,  0.225259855,  -0.188193187,  1.61321998,  -0.925672412,  0.4488253,  -0.950417459,  -0.199481651,  -0.736164272,  0.0758640468,  1.66106486,  0.167649999,  -2.00395918,  -0.321644485,  -0.0117881503,  -0.391427279,  -0.958545446,  1.16223943,  -1.4895103,  0.557308137,  0.0606987625,  -0.401318878,  -0.920850933,  1.35996699,  0.414706975,  -0.830326974,  -0.0599946678,  -0.0927445292,  -0.743622005,  0.935564399,  -0.109254122,  1.6989361,  -0.401907176,  0.163067326,  0.202372476,  0.189911813,  -0.549150229,  -0.681180954,  0.00218673726,  -1.32035756,  1.35676372,  1.58259904,  -0.474923372,  -1.45556867,  -0.125854298,  0.596534789,  0.656286836,  -0.0597367436,  0.782413363,  -0.552353978,  0.658377111,  0.924275279,  -0.285409421,  1.25485373,  -0.593771636},{ 0.419420749,  0.739039958,  -1.70538759,  -0.597255051,  -0.754241288,  -0.500936627,  -0.435477257,  -0.055798158,  -0.43138501,  0.498646528,  -0.821787894,  -0.548586607,  2.28648043,  0.303833961,  -1.39761889,  -0.959271193,  -0.931811631,  -0.505986094,  -0.504888773,  -0.411117285,  0.65168345,  -0.596589804,  0.348908663,  0.356853038,  0.689169943,  -0.723923385,  0.114325441,  -1.30293357,  -0.0298387595,  2.05745173,  0.263451785,  1.81287813,  -0.855966747,  -0.584047556,  0.233455315,  -0.781263769,  -0.244983226,  -0.135779053,  -0.0210561715,  -0.891432047,  -0.205367938,  0.803866923,  -0.301963419,  -0.400734037,  -0.826273263,  -0.458776712,  -0.967478991,  -0.0762193874,  0.0807711631,  -0.658310175,  0.603815675,  -0.430694044,  0.34962818,  -2.01578164,  -1.07063496,  -0.5807212,  0.129371256,  -0.356610596,  1.26817787,  1.38243937,  1.61874378,  -0.143361345,  -1.10798371,  -0.750458717,  -0.362800121,  -0.360021442,  -0.367079407,  -0.987060845,  1.4795537,  0.139692754,  0.264095664,  -0.463859469,  0.543174565,  0.993004322,  0.410214722,  -1.40133715,  -0.520777583,  -0.226104051},{ 0.476661414,  1.56907547,  -0.239681765,  -2.3057127,  -1.08467162,  0.30901587,  2.43226433,  -2.55565643,  -0.579298079,  -1.32199371,  -0.350226045,  1.07160485,  0.799602926,  1.22256172,  1.1508435,  -0.239290804,  -0.759242177,  1.46370625,  0.887022734,  0.451258183,  -0.352586061,  0.298855215,  -0.771543682,  0.736697555,  -0.441738158,  -0.371344209,  -1.06575942,  -0.225599915,  0.461627275,  0.171746165,  0.953370214,  1.15435898,  -0.895957768,  -1.19679594,  0.0575662479,  -1.57297528,  -1.19424462,  -1.14409661,  0.661054552,  -1.89119589,  -0.581127584,  -2.13168287,  0.684221745,  2.39056349,  0.661991358,  -0.0879341066,  0.298664629,  -0.532987773,  -1.85484147,  0.394746304,  -0.215650573,  1.08094883,  -0.764846206,  2.25470424,  1.16980553,  -0.850339949,  0.602313638,  0.368102908,  1.34546864,  0.044457525,  0.192901894,  0.923447132,  0.295841724,  0.326538354,  -0.501440942,  -0.717380345,  0.767017066,  0.452031702,  1.70437503,  -0.908160329,  1.25798535,  1.97977376,  -1.52002096,  -0.443595976,  1.32657588,  -1.42643821,  1.58190417,  -0.494568139},{ -1.30914426,  -0.0130758034,  -0.313366085,  0.929386258,  0.946127892,  -2.54334807,  -1.42754459,  0.3951855,  -0.0143274078,  -0.621580243,  -0.193969041,  1.2334069,  -0.191613764,  0.29891777,  -1.41942179,  1.15729439,  -0.273787856,  1.54248428,  0.55318296,  0.833781302,  -1.27308047,  0.177951992,  2.04392052,  1.54523575,  -1.44280171,  -0.620032609,  0.364307463,  -0.413506806,  0.0262948032,  -1.37934136,  -0.0149682676,  1.17443681,  -0.982057214,  -0.244223863,  0.227047756,  1.52455413,  0.218354598,  -0.0985170007,  0.167782247,  0.294454604,  -1.0236789,  0.149423778,  -0.0062538241,  -1.7626884,  1.13241374,  -2.00075507,  2.07050586,  1.6006999,  -0.568872213,  1.16197038,  -0.544257641,  0.525622785,  -0.0903795362,  0.852217257,  -0.975198984,  1.1948818,  -0.979896665,  -1.2813555,  -1.09673238,  -0.276849747,  0.0534877591,  -0.604239643,  -0.687984943,  -0.46042034,  -0.741381228,  0.148733363,  1.52433717,  0.100848034,  0.118165977,  0.412446529,  0.174732894,  -0.764501274,  1.99748135,  0.270090371,  -0.668807149,  1.62912381,  -0.145791575,  -1.38767862},{ 0.719840229,  0.771956861,  2.25897193,  0.111495741,  0.251715511,  0.813824534,  3.68570733,  -0.571604788,  -1.30536747,  -0.450784922,  -1.26566887,  -0.313832581,  0.779706359,  -0.0234316271,  -0.135815084,  -0.590895712,  -1.39883459,  1.12047327,  1.08037317,  -0.173848704,  0.153253838,  0.206970111,  -1.05059981,  0.906887472,  -0.312785506,  1.80573118,  -1.95950675,  -0.373929799,  1.43610191,  -0.608172417,  -2.1802485,  -0.754786372,  -0.276665211,  0.601831555,  -1.74345744,  1.52012563,  -1.04185259,  0.391660273,  -0.199012399,  -2.06043053,  0.00295738992,  -0.0585604571,  -1.69894922,  1.50033116,  -1.52189338,  0.174600169,  -0.95587641,  -0.302369803,  -0.0691168457,  0.386515558,  0.710013926,  2.09433699,  1.58917856,  -0.294406176,  0.222138658,  -0.227270856,  0.0394276343,  0.124296837,  -1.35905027,  -0.297117561,  0.432616502,  0.261272848,  0.995054126,  -3.03883076,  0.431880504,  1.90919149,  -1.14521325,  -2.95395637,  1.98749566,  -0.0974546745,  -1.01509356,  0.00941700302,  0.297726244,  -1.43084252,  1.26369631,  -2.19070387,  0.249274105,  1.4204917},{ -1.21237993,  -0.515258372,  0.807232857,  1.25268984,  1.18820608,  -0.663545787,  -0.56902194,  0.049607344,  1.57911849,  -1.5362848,  0.958607137,  -1.02266657,  -0.575232208,  0.21520184,  0.660554886,  0.321596146,  -0.143578827,  0.337819934,  -0.793876588,  0.386826456,  -1.23946345,  1.42218161,  1.75706828,  0.573714018,  0.935267925,  -1.64496064,  0.25332436,  1.67980683,  0.520130157,  1.06869411,  0.410673767,  0.719947755,  1.29072213,  1.2644372,  -0.440239996,  0.0415968858,  0.969884515,  0.784400702,  0.947425425,  -0.241120204,  1.95185471,  1.20584667,  0.0191062074,  -0.353734672,  0.713854671,  0.752111852,  -0.0526464656,  -1.64954126,  -0.804256022,  1.20641565,  -0.964543283,  -0.349454314,  0.506527185,  1.98845637,  0.868705451,  -0.839823782,  -0.937938154,  -0.173549935,  0.467870593,  0.163363054,  0.164924085,  0.0140080908,  -0.915541649,  1.81829119,  -1.42177141,  -1.30541348,  1.55078423,  -0.0904415771,  -1.26358235,  2.44192982,  0.560060322,  0.0160917528,  0.403811425,  0.50759685,  -0.0338965468,  1.06270409,  -0.845756471,  0.688742757},{ 1.23744106,  -1.5544529,  -1.20568681,  0.210383818,  -0.488890618,  -0.152368724,  -0.395589828,  -1.4693327,  -0.404723167,  -0.541205645,  1.10112178,  -0.803150892,  -0.347274125,  0.55379796,  0.785397708,  0.152864054,  1.08275986,  -0.211723492,  1.48905873,  1.01268995,  -0.119250178,  -0.580903351,  -2.11924982,  0.237472847,  -0.829185426,  0.32998687,  0.199074075,  0.82148087,  1.5406692,  0.805363238,  -0.152957782,  -0.378297836,  0.39037317,  0.66383642,  -0.178370073,  1.13243043,  -0.12438754,  0.687096834,  -0.783699334,  -0.42501229,  0.314221203,  1.86980879,  0.999751806,  1.67264211,  0.0430809483,  -0.341269255,  -0.0980598927,  -1.19759727,  -0.524127662,  -0.17208226,  1.04675055,  -0.385302991,  -0.277509928,  0.552494824,  1.34667158,  -0.405496359,  -1.40161216,  1.1792866,  0.63033247,  1.21294415,  0.129717737,  -0.0809185877,  0.204851747,  -1.04584146,  -1.2315948,  -1.3426764,  -0.856105566,  -0.202000976,  -2.03579021,  -0.958776236,  0.348185688,  0.407781243,  -0.00873612333,  -1.63732457,  0.0811099112,  1.07246888,  -0.699207127,  -0.308594406},{ 0.843367815,  -0.108548507,  -1.20858777,  0.0317158028,  -0.27910316,  -0.252070457,  -0.24035871,  -1.56796753,  0.25154689,  0.467129499,  -1.68020761,  -0.953338683,  -0.619282424,  -0.435306907,  0.0359556973,  -1.52329993,  -0.976008058,  2.33648682,  0.244963184,  -3.09390616,  0.245730847,  -0.0347429849,  -0.433815777,  -1.73212707,  0.0827042088,  -0.307909906,  1.4070313,  1.90405226,  0.497038186,  -0.643134534,  0.159639463,  -0.850002706,  0.434774399,  0.791490495,  -0.198642582,  0.749722838,  -0.201797664,  1.13743865,  0.622741997,  -0.473086476,  0.552753925,  -0.597587168,  -1.83293641,  -1.92581904,  -1.45760405,  -0.70155859,  -0.256196678,  -0.104721718,  0.250352412,  0.900859058,  0.0239048172,  -1.22629118,  -0.612621188,  -0.751053154,  1.03717911,  0.903822601,  -0.531128109,  0.0310861412,  0.00390769122,  -0.0272082109,  1.30164158,  0.58711499,  0.708334863,  -1.20333385,  -0.722272277,  -0.281047732,  0.508790791,  -0.934221864,  -0.916352391,  0.0571682826,  0.10693524,  0.317993253,  0.386577338,  -0.0244393256,  0.660390317,  -0.368335545,  -1.08539772,  1.041067},{ 0.560367703,  -0.571934879,  -0.890189052,  0.213153809,  0.61560744,  -1.19247031,  0.94335115,  -0.831572294,  -0.385434628,  -0.0206926279,  1.71777451,  -0.282120198,  -0.451950401,  0.267961472,  -0.672206104,  -1.20395803,  -0.256248981,  2.53283501,  1.51188803,  0.538026512,  -0.911103249,  -0.17900762,  0.263543308,  0.543758035,  0.986459076,  -0.465193212,  0.986044228,  -1.14473963,  -0.202066824,  -0.16214788,  0.94570148,  0.642714858,  -0.68821013,  -0.603474975,  1.83116794,  -1.56843996,  -0.173375309,  -0.039846506,  -1.816221,  -1.25691426,  1.49175858,  -0.254124969,  -0.694220006,  -0.354325503,  0.515773594,  2.09457374,  -0.422677189,  0.880894542,  0.222360075,  0.302788138,  -0.127664223,  -1.00216699,  0.240755141,  -0.567700267,  0.739322066,  1.30393338,  -0.644927621,  1.42077076,  0.414438903,  0.239209861,  -0.421308428,  0.12685515,  -0.141618252,  -0.378692567,  1.12987089,  -0.134899929,  -0.568035543,  -0.174324945,  1.53760433,  -1.72803485,  -0.00384258619,  0.343025029,  -0.937462389,  0.120976962,  1.1811161,  -0.203224704,  -0.261911958,  -0.513530433},{ 1.77727866,  -0.0791103244,  1.3432132,  -0.69594866,  -0.933661401,  1.46919167,  -1.47242439,  1.35067618,  -0.0589688532,  -0.146232858,  -0.455586702,  -1.01352775,  0.507979512,  -2.3016603,  1.13547647,  1.00303102,  -0.0511322916,  1.40545249,  0.892570138,  0.138168812,  0.334795922,  0.620209515,  -0.836345673,  1.59698951,  1.47017968,  0.203799486,  1.51862907,  -1.67574024,  0.279250234,  -0.0832224339,  1.62953341,  -0.644029856,  0.237102106,  -1.55251288,  2.14636421,  -0.384315014,  0.111729071,  -0.889158845,  0.732073605,  0.688146651,  -0.603175044,  -1.31570828,  1.01560736,  0.595837176,  0.163922027,  -0.247135699,  -0.370778024,  -1.3200618,  -0.733081937,  -0.0755418986,  1.33806431,  1.01162767,  -0.283291042,  -1.33143795,  0.400599957,  0.923318923,  -1.32629311,  1.41367066,  0.476775527,  -0.808345973,  0.275375664,  1.6674614,  -2.15154934,  -0.50342685,  -0.697573185,  -1.30604637,  -0.943495929,  0.2991063,  -0.216086641,  0.284011304,  1.29958308,  1.74530745,  0.22572796,  0.622740686,  -1.85418534,  0.465880811,  0.0331845246,  -1.46406722},{ -0.178707361,  -1.31662881,  0.195548341,  -0.548736811,  -2.33567595,  -0.803415596,  -1.10187411,  -1.19767272,  0.447373569,  -0.066138491,  0.363372982,  0.396422297,  0.175425962,  -0.242216602,  -0.398559511,  -0.948976398,  -0.181013942,  1.83796525,  -0.49799934,  -0.979424715,  -0.511478424,  -2.44938684,  -1.23704994,  0.937158406,  -1.33686817,  -0.294752657,  0.375492424,  1.44798005,  -1.10487366,  0.296272457,  -0.373282164,  1.59431279,  0.91201961,  0.0779193565,  0.368106782,  -1.04175341,  -1.85883391,  -0.702729583,  -1.24436927,  0.477395296,  0.330289513,  1.96274817,  -1.0448693,  -0.474864453,  0.45360148,  -0.115835533,  -1.88430834,  -0.319636732,  0.331378132,  0.678522706,  -0.664422572,  1.29715633,  0.194176272,  1.01719689,  -0.251229107,  0.00266491505,  -1.67935205,  -2.48329425,  0.21177949,  -1.48081303,  2.2800796,  -1.61585116,  -0.469843835,  -0.689073741,  1.00441647,  -0.354839563,  -1.18242943,  0.735039532,  -0.456491381,  0.190624043,  -0.154570684,  0.994383097,  0.306995273,  1.02317011,  -1.32540536,  1.14501798,  -0.670903683,  -1.09532678},{ 0.44019416,  0.323966742,  -0.52777338,  0.107163623,  0.579782546,  1.37829959,  0.400776684,  0.762564898,  -2.17165089,  0.678157806,  -0.381338596,  1.35591006,  -1.23817122,  -0.660283923,  0.953356445,  -0.94737041,  0.282548487,  -2.18552208,  0.119711593,  0.351975709,  0.732855201,  0.742010236,  1.25303185,  0.946636975,  -0.772076786,  1.00728273,  0.367736161,  0.637267351,  -0.565382898,  -0.0263196696,  -0.358378053,  0.491564333,  -0.0454510562,  -0.770471334,  1.59198344,  -1.63796163,  2.25150037,  1.04554784,  0.648200989,  2.19756174,  -1.04741752,  -0.532357335,  0.305592358,  2.02670503,  -0.11565809,  -0.311608255,  -2.57456493,  -0.102873683,  -0.14206247,  -0.584569812,  -0.596676469,  -1.26574469,  0.0351699516,  -0.567961633,  -0.637619734,  -0.355835825,  -0.950118363,  -0.306617618,  0.281680077,  -1.96095574,  -0.0602105595,  0.733955204,  -0.652948797,  -1.91769755,  0.308915973,  -1.32192695,  -0.462735206,  0.221173257,  1.17108893,  -0.266994238,  -0.291034132,  1.17734361,  -0.255321205,  -0.343742162,  0.686916351,  -1.78335512,  -0.338213831,  -0.12869449},{ -0.761110604,  -0.927473307,  -0.123962268,  1.13244212,  0.409990221,  -0.890527546,  0.876982987,  0.18651244,  0.684357584,  -0.501309514,  1.29156327,  -2.63337088,  -0.543852031,  -0.958607137,  -0.842855394,  -0.67501533,  0.602068305,  -1.76991808,  -0.598733604,  0.512050152,  -0.803597927,  1.47455704,  0.471391052,  -0.283065587,  0.930107176,  -0.165360808,  0.6155774,  0.355604291,  1.15322387,  0.104429029,  -0.0787111595,  1.87108302,  -1.28370094,  -1.2318604,  -0.568978488,  -0.483069628,  0.435632527,  -0.787944973,  0.674111426,  -1.22909808,  -0.010697308,  -1.49241579,  0.261126429,  2.20243216,  0.420403302,  0.136540413,  1.16132128,  -0.0289646853,  0.937097073,  0.449804068,  1.14930332,  -0.0324179158,  0.589028835,  -0.429758906,  0.134721756,  -0.890622318,  -1.88438129,  0.919723272,  -1.17909765,  -0.0228745677,  0.449384302,  0.313011825,  0.855533123,  0.069447048,  0.352680653,  -0.457064956,  0.805747032,  -0.250688702,  0.992292404,  0.168120861,  -0.163555339,  0.487093747,  -1.28654182,  0.945723057,  0.783520997,  -0.308587253,  0.212460876,  0.226233557},{ -0.27880913,  0.987634599,  1.33972895,  2.9987855,  0.135137424,  -1.08570266,  -1.41802132,  0.762007236,  1.26313126,  0.529785097,  -0.29277572,  0.713996232,  -1.14401007,  0.356752813,  -0.202320248,  0.340968102,  -1.17846119,  0.716637969,  -0.386683047,  -0.223613545,  1.01972854,  0.250745088,  -0.0182171557,  1.41439712,  -0.000221687573,  1.23140144,  -1.18049991,  1.00720215,  1.50819099,  0.396448314,  0.0564473085,  0.230681777,  -0.206582338,  -0.330157965,  0.74908334,  0.675451338,  -0.300665766,  1.04385245,  -0.321519494,  -0.807491243,  1.97011936,  -0.433103681,  0.0663520917,  -1.86825156,  0.874061882,  -2.05372,  0.391418487,  -1.77247429,  0.491988331,  -0.437655032,  0.615053594,  1.50908399,  0.21224229,  1.49601603,  0.0546782054,  -0.684931159,  -1.40932095,  -0.807675242,  0.240027189,  -1.8653357,  0.93457222,  0.494826525,  0.868983567,  -0.202811062,  0.433303267,  0.687137246,  0.888627946,  0.284246951,  -0.80938369,  -1.0918535,  -0.220313117,  -0.441259712,  0.0593774654,  0.980179727,  -0.61023587,  1.11194801,  0.67942822,  -0.711078107},{ -0.162080973,  -0.0899004489,  -1.19155514,  0.869769692,  -0.723156989,  -1.69137526,  -0.904484868,  -1.2941581,  -0.67599529,  0.463167876,  -1.28013158,  -1.724105,  0.257438093,  1.46207964,  0.561418176,  -0.426039666,  -0.506780684,  0.577732265,  1.44644654,  2.24449754,  -0.385741144,  -1.28105247,  0.765740693,  -0.56617105,  0.810678482,  -1.02851999,  -0.0227512699,  1.33465731,  1.50091219,  -0.0505057946,  1.61913383,  -0.284437537,  -0.784566164,  -0.322054744,  -0.722295523,  -0.42469433,  1.2476716,  -1.09496939,  -0.743299961,  0.92921114,  -0.588773847,  0.355858386,  -0.226434872,  -1.33065081,  -0.783911705,  -0.622561753,  -0.652828336,  -1.74307573,  1.93344128,  0.673301101,  2.59987521,  1.24026465,  0.119180635,  -1.21837556,  -0.718179584,  0.820501208,  -0.541645944,  1.44028962,  -0.686040521,  0.738899529,  2.34207392,  0.168443158,  0.096962668,  0.714354098,  -1.60927463,  -0.111444004,  -0.89962697,  -0.507215559,  0.457578003,  -0.3816517,  -0.411982745,  0.691113234,  1.0874294,  0.471495479,  0.484865278,  0.00951174181,  -1.24681866,  -0.163140431},{ -0.0927102193,  -0.517381251,  -0.0559464246,  -0.399016172,  -1.23621333,  0.252871692,  0.172414109,  0.477482617,  -1.30086839,  0.0175953824,  -0.468327194,  -0.404862106,  -0.348469138,  0.375949711,  0.305992395,  0.399767011,  -1.22593868,  1.32416952,  -0.11649771,  2.17258191,  1.49172497,  -0.572635055,  -0.608203173,  -0.092550531,  -0.976751745,  -0.392352015,  -1.35485399,  -1.7136023,  1.88030303,  0.269719243,  0.812274694,  0.963304102,  0.536990941,  -1.99130547,  0.562485099,  0.0260115415,  0.32037273,  0.000577464234,  -0.288777441,  1.88290858,  1.07012308,  -1.37934685,  -0.492043108,  0.242296815,  0.462253839,  -1.36363983,  -0.817991257,  -2.27598119,  -1.29378808,  0.996262491,  -0.0729947537,  -0.220489189,  1.74784148,  0.357317507,  -1.4905659,  0.422178358,  -0.94489485,  1.38294327,  -1.562011,  -0.833366811,  2.35514259,  -1.02539694,  0.283272922,  -0.392025828,  0.932089388,  1.10637283,  -1.58740985,  0.126424,  -0.752470434,  1.14059126,  0.738472402,  0.527285099,  -0.947678328,  0.47896567,  -0.533882678,  0.527344882,  -0.308151215,  0.00255763484},{ -0.10489013,  -0.568435967,  0.0617522933,  -0.825128615,  1.49654388,  0.697076201,  -0.497978568,  0.330105007,  -0.35437116,  -0.160504505,  0.244985208,  1.05400062,  0.0303429104,  -2.17431903,  3.6917119,  0.147164434,  -0.307097375,  0.656050742,  -2.11755872,  -0.878633738,  0.162648767,  -0.0182351619,  0.721741498,  0.972661495,  -1.73946822,  1.15214479,  -1.03602445,  -0.612953782,  0.415836126,  -0.477431327,  -0.999601543,  -1.25198793,  -0.537720919,  -1.49121606,  1.45162857,  0.148899853,  0.812806606,  1.09327173,  -0.485421807,  -0.248217881,  1.37042797,  -0.223364756,  0.295556694,  2.07422161,  0.369691759,  1.01463854,  -0.581961334,  0.206973612,  -0.0965265259,  0.410507143,  1.88253069,  1.30909908,  -0.605820835,  -0.472525209,  -0.491239548,  0.883287668,  0.67543447,  0.3025406,  0.684624076,  -0.514506638,  1.45329285,  0.497941434,  -0.641194642,  -0.35699755,  -0.610588372,  0.0888927057,  -0.361946523,  1.5402807,  -1.15008175,  -0.302643567,  -0.104060292,  0.467897832,  -0.837738395,  -0.318326533,  0.350191265,  0.609104753,  -0.107152276,  1.65684927},{ 1.31666768,  -0.893316507,  1.48296559,  0.643694282,  1.85999012,  -0.303800076,  -1.6479063,  -1.69893944,  -0.932097912,  0.207194939,  0.744016826,  1.68639588,  -0.658335447,  -0.215411752,  -0.279589891,  0.914813459,  1.30746615,  -0.859274387,  0.43053633,  -0.0953401327,  -0.38650763,  -0.855350971,  -0.371520072,  -0.492171437,  -1.07906663,  1.69609785,  -0.307253927,  -0.555655539,  0.366059721,  0.458772719,  -1.528584,  0.769999564,  -0.825005829,  -0.914057076,  1.22365797,  -0.323933512,  -0.330563813,  -1.55586457,  -0.999159575,  0.683800101,  -1.93693852,  0.973434865,  1.02057755,  0.484870583,  -0.372401208,  -2.29468417,  -0.681966305,  -0.064409405,  0.415818274,  -1.52816606,  0.962095261,  0.32124123,  0.843454242,  0.115286462,  -0.0313678198,  0.0346770063,  -0.094165355,  2.26650238,  -1.40580785,  -1.08489418,  -0.547376692,  0.350304097,  1.62755644,  -0.774908364,  0.12832661,  -1.01339626,  -0.619722068,  0.4487755,  1.16496599,  0.243282229,  -0.70653367,  -1.05070341,  -0.411973834,  -0.864296973,  0.913793325,  -0.667737126,  -0.670142293,  -1.12090945},{ 0.197530463,  0.780297279,  -1.4049114,  0.186759278,  1.24371243,  0.247029066,  1.72757113,  -0.133717358,  0.577536523,  1.82518125,  -1.72619176,  -0.70420146,  1.09779799,  0.472682685,  0.503052354,  0.580427766,  -0.0189389046,  -1.38100159,  -0.287273049,  -1.49449492,  0.293450147,  -1.46439803,  0.374649972,  -0.441331416,  -0.474897444,  0.838434815,  1.03281903,  0.0706917793,  0.00255108252,  -0.126082912,  1.00202882,  -1.26875103,  -1.20779407,  -0.508200288,  -1.0587281,  1.68609977,  -0.0348710157,  -0.415818542,  -0.876448095,  -0.76374054,  1.85404754,  1.00394523,  0.151455685,  -0.480278552,  -1.36784124,  -0.476130754,  -1.65906155,  -0.0898329541,  -0.21353282,  -0.0705773756,  -0.606220722,  1.77710307,  2.28026557,  1.16086996,  1.29433882,  -0.131457642,  0.0803292766,  -0.600255191,  -0.0660375208,  1.01039171,  0.768993735,  -0.925800204,  -0.119076349,  -0.158749267,  -1.96425414,  -0.363225073,  0.299370915,  1.70995557,  0.503258049,  -0.862468898,  0.857800484,  -0.105609745,  1.4852612,  0.320682526,  -0.543643832,  0.416105717,  1.24880874,  -0.218709886},{ 0.483201176,  -0.586320698,  0.9320243,  0.309406936,  0.198436856,  -0.958264172,  -0.377240181,  -0.381565988,  -0.333548546,  0.245282054,  0.0467962697,  0.588793218,  -0.579629064,  -0.513948321,  -0.234358266,  1.10006261,  1.37241471,  -0.177161202,  -1.61957383,  -0.770638704,  -0.30062139,  0.318289638,  0.879450381,  0.00822748803,  1.56008315,  -0.483524382,  0.149518922,  -0.737188816,  2.10629582,  0.0736241788,  0.328582346,  -0.0649648458,  -0.910883248,  1.17827761,  1.04093754,  -0.0944652706,  -0.682789326,  -1.95210946,  -0.407265961,  -1.31286228,  -0.316194445,  1.61128736,  -0.495374709,  0.248368934,  -0.394090056,  -0.468603969,  -1.05544508,  0.1124648,  -0.923318267,  0.656260729,  -0.0538439415,  -0.724876702,  -0.675700188,  -0.315197498,  -0.0739231557,  0.131988153,  0.974565744,  0.343944818,  -0.745895863,  0.239105299,  0.714599848,  0.156575128,  0.392471224,  0.568983912,  1.11418712,  -0.287538826,  0.0143307419,  -0.481467009,  -0.0434434451,  0.0103060966,  0.285785586,  0.626165211,  -0.566401243,  -0.74445039,  0.76672399,  -0.616400659,  -1.39774799,  -1.19716072},{ -0.753191471,  -1.26794589,  2.20917821,  -0.131341681,  0.00942719262,  0.92063874,  0.323582321,  1.2021811,  1.1319567,  0.549127102,  -0.182700247,  -0.59453094,  -1.22384441,  0.714978516,  1.61925995,  -0.383339703,  0.930152476,  -0.288288772,  1.77535009,  -0.0150156505,  1.44352031,  0.499795228,  -0.814042389,  0.8090446,  -0.64382875,  -0.893443048,  0.860206902,  0.16809319,  0.241846517,  -0.0719519407,  0.937424302,  1.51837933,  0.664373994,  -0.775944114,  -2.31583023,  -1.11596179,  -0.980119348,  2.75694251,  -0.797674954,  0.338640958,  -0.611053646,  -0.104142666,  -0.629801452,  -0.885733962,  0.53725177,  -1.25137818,  -0.369553775,  -1.4874233,  -0.566367626,  2.6911006,  -0.921545804,  -1.33601558,  0.316563278,  0.404586464,  0.0440366901,  0.094346948,  0.36592111,  -0.0353588723,  1.33949649,  0.388890952,  -0.00517261168,  -0.746121526,  -1.21807551,  -0.128410339,  -0.62024051,  0.933808625,  -2.01726985,  0.422849029,  1.21946633,  -1.09027541,  -1.94974732,  0.0292302966,  1.39064705,  0.577973306,  -1.17388701,  0.236519352,  -2.3300879,  -2.86275744},{ -0.545525312,  -0.628615677,  0.401945233,  -0.0917637348,  -0.398200154,  0.0601306632,  -1.57971382,  -0.723337531,  -0.406406879,  0.767568052,  0.964381754,  0.722438157,  -0.77602458,  -0.594905794,  0.441768199,  0.784825504,  0.466103852,  1.25096631,  2.12419677,  -1.733518,  0.885367095,  -0.682454944,  -0.323121756,  -0.749518335,  0.617217362,  -0.56698662,  0.336560726,  -0.976149797,  -0.581838429,  -1.10059941,  -0.0128359869,  0.641452432,  1.50635791,  0.879279912,  -2.18813014,  0.696957946,  2.22753358,  -0.274542719,  -1.98267627,  2.117064,  0.150599152,  0.142271072,  -0.0624996908,  2.29350615,  0.557771027,  -1.38784444,  -0.519210279,  0.172676176,  0.851524234,  -0.902051091,  -0.821000636,  -0.671690762,  -0.946002364,  1.0927757,  -0.0866070092,  0.743947506,  0.364375621,  0.325391829,  -0.361346781,  1.04100525,  -1.52823877,  0.655032158,  0.50898838,  0.311719418,  0.966429114,  -0.427902907,  1.99094176,  0.846534073,  0.75974822,  0.841095746,  1.55794585,  2.11117625,  0.497650623,  1.85997307,  -0.16816546,  -0.766299248,  1.1220932,  -0.82480073},{ -1.94213426,  -0.725873828,  -0.936961114,  -0.906940818,  0.110358767,  -1.2804817,  -0.913967729,  0.496011496,  1.79579628,  0.251058519,  -0.163735628,  1.35773909,  -0.144876704,  -0.283264041,  -0.964940488,  0.653747678,  1.11785781,  -0.246293202,  1.68833494,  0.114915825,  -1.44278502,  0.811711431,  1.56509054,  0.747770607,  0.143061951,  0.117392622,  0.463663071,  -1.6050384,  -0.896824896,  -0.373968452,  -1.15782785,  -0.590955734,  2.0571475,  -0.491480231,  0.394816041,  1.44904494,  0.728491485,  0.732508004,  0.230564341,  1.64713228,  0.0904967189,  -0.178153172,  -1.99498177,  0.457417518,  0.258637637,  0.0350198559,  1.07039797,  -0.0414617397,  0.0297808927,  -0.395426303,  0.527840614,  -0.538635314,  -1.34153759,  -0.602846265,  -0.00250637205,  1.04959762,  2.28315783,  1.62202191,  -1.15444839,  0.411453724,  -1.11341619,  -0.270400494,  0.781570196,  -0.88315767,  -2.43033981,  -1.1277715,  -0.568008542,  -0.144965053,  0.757879674,  1.31429052,  1.50241971,  -1.05623496,  0.290641606,  0.906535149,  1.42341566,  -1.20193803,  0.680642009,  0.743712008},{ -0.525821209,  -0.415056288,  0.721214294,  -0.214942798,  0.608159006,  -0.335019648,  -0.36118567,  -0.323763967,  -0.955527365,  -0.329896361,  0.129331678,  1.82992589,  -0.774069846,  2.14370394,  2.51465821,  -0.799275279,  0.85786581,  0.305920005,  0.66355437,  -0.129991353,  0.38747865,  0.855254352,  0.456468701,  -0.480497599,  -0.583250523,  0.0539351404,  -0.728133082,  0.101458542,  1.00730908,  1.38833117,  0.865915537,  1.62149906,  0.293885022,  0.281700999,  0.885711372,  0.0570568219,  0.45882988,  -1.51619446,  0.356144756,  1.41787112,  -0.991310418,  0.045749642,  0.36692059,  0.947801292,  -0.0205363128,  0.37470305,  -1.39117646,  -1.04807961,  -3.4142611,  -0.529805839,  -0.675393939,  -0.121681832,  -1.21822917,  -0.295621485,  1.26841521,  0.177293107,  -1.2597717,  1.27948272,  1.32185245,  0.0649206415,  -1.04804218,  -1.51828659,  0.418426096,  1.35304582,  -0.553542256,  0.414950848,  -0.755925536,  1.5411799,  2.75093818,  -0.0854481086,  0.319610536,  -0.076973021,  -0.710433304,  1.0268203,  -0.863481998,  1.17575014,  -0.189331979,  -1.0463506},{ 0.805217028,  0.249822751,  0.385016352,  0.446746707,  2.03879905,  -0.597429931,  -0.172413141,  -1.12959075,  -0.658421993,  -1.14561236,  -0.536197066,  1.48797143,  -0.395294219,  -1.38719916,  0.273924351,  0.414030761,  -0.445326388,  -0.362125754,  -0.442171037,  0.00505525433,  -1.775769,  -0.930800021,  0.523066998,  -0.0195372757,  -2.18122888,  -0.713648319,  0.311111391,  -0.954728246,  1.42895675,  0.911643028,  0.35413909,  -0.123032451,  -0.500350773,  0.676731884,  -0.302970231,  -0.387387425,  0.386593103,  1.63684916,  -0.54346323,  -0.864238501,  -0.593946218,  -1.01520216,  0.691593349,  0.182946578,  -1.63563478,  0.690000713,  1.01258588,  0.745087922,  2.42981458,  0.479158431,  0.826039672,  -1.28251934,  -1.72549212,  0.523588598,  -1.80425787,  0.539943576,  1.20344234,  0.0776912868,  -1.21776116,  0.813403249,  -0.816264451,  -0.731360912,  -1.01898694,  -0.856835306,  -1.42056727,  0.365108371,  0.326403618,  -1.37211192,  -0.621177077,  -0.405086666,  0.678127825,  -0.199115396,  -0.154782712,  0.326485425,  1.10905337,  0.130828217,  0.797572076,  -1.2943114},{ 2.30200863,  2.04186535,  -1.74344051,  -0.720928907,  -2.00779963,  0.595855951,  -1.09138453,  1.24787617,  -0.0973075479,  0.64858222,  -1.8339256,  -0.46931085,  -1.35750401,  0.201226562,  0.160629496,  -0.768550634,  1.12112761,  1.07317579,  -0.707828403,  -1.67765832,  -0.315693855,  1.68246543,  -2.26225805,  1.32523203,  -0.785356283,  -0.519339442,  0.0194901116,  -1.64427865,  -1.13051736,  -0.386268049,  -0.0768574476,  -0.575583339,  1.08997345,  -0.218567893,  0.00517615862,  -0.445092261,  2.53431487,  0.536042452,  0.29011932,  1.06478965,  0.714827776,  -0.505717635,  -0.15142332,  1.36684144,  1.04105723,  -1.99320531,  0.0106397066,  0.0727833882,  1.73798895,  0.932591438,  1.78929186,  -0.934355855,  0.552935243,  -0.357375622,  -2.46271086,  -0.345363855,  -0.917030156,  0.77044189,  -0.475827396,  -0.209067076,  1.39663196,  2.04021478,  0.494062573,  0.920479834,  0.61039561,  -0.387694836,  2.46592546,  -1.90897954,  0.305701196,  0.0594216585,  1.98882151,  -1.59933937,  1.61794746,  0.724486291,  -2.12440157,  -0.380581409,  -0.101278685,  -0.509043276},{ 0.739551485,  -0.469686568,  1.1256969,  0.529337525,  0.669604957,  1.06968606,  1.22014928,  -0.997574568,  0.0574205033,  0.804132521,  -1.40271592,  0.955789983,  -1.40733612,  1.01208878,  0.0130874477,  0.139250487,  1.65781271,  -1.34778893,  0.655259192,  -0.582461059,  -0.021995347,  2.20110917,  1.30867803,  -0.905698836,  -0.837821543,  -0.828107417,  -0.143611774,  -0.399906874,  -1.34652376,  0.607533455,  -0.722865582,  1.12546313,  0.772924423,  -0.809887052,  1.70127678,  -0.459029227,  -0.627676964,  0.711623847,  0.132835254,  1.52435791,  -1.98682034,  0.425630718,  -1.07940435,  -0.0279806741,  0.124512203,  -1.69389844,  -0.494304508,  1.00731373,  1.69463444,  -0.114279941,  -0.568364859,  0.171696231,  0.910200059,  2.7981782,  1.39366806,  -0.0594177246,  1.98976779,  0.520980954,  -1.02842903,  -0.748806,  -0.088551335,  -1.48953795,  0.458580941,  -0.320472479,  1.47079706,  0.754633784,  0.739482224,  2.10380983,  -0.122691579,  0.545247436,  -0.644281805,  -1.08281076,  0.159369901,  -1.53568602,  1.18841898,  -0.651372433,  0.379234493,  -0.981796443},{ -1.08818662,  -0.0858112052,  -0.341365695,  -0.206193477,  -0.355401874,  -0.0540179983,  -0.646504045,  0.213731617,  -2.78894448,  0.743077397,  -1.0108614,  1.40836406,  0.516706765,  -0.148955747,  -0.367730379,  0.583703637,  0.940045357,  0.820084393,  -1.58434486,  -1.40617526,  0.714830279,  -0.609523892,  1.33318067,  -1.22508681,  -0.141513035,  0.0141548114,  -0.0264461823,  -2.31173277,  0.418380857,  -0.610259712,  1.14736199,  0.632038832,  -0.37677753,  -0.674955606,  0.65047121,  0.0221698824,  -0.370430619,  1.68930101,  0.773708344,  0.019940732,  2.22016716,  0.80395931,  0.918845415,  1.02910101,  -0.302724689,  0.36908555,  0.804633737,  0.911525011,  -0.621761322,  0.588319957,  0.581384003,  -0.9540748,  0.0547096133,  -1.88442099,  -0.554767013,  0.267354041,  0.177599981,  0.0879499838,  0.328784347,  0.290830135,  0.821131289,  0.594268739,  -1.07795644,  0.374438554,  -1.11152327,  0.464534581,  -0.0443126261,  2.14677501,  0.92397207,  -0.819946229,  -1.21360552,  -1.85301733,  -0.91071111,  -1.35681474,  -1.25756979,  -0.105737023,  -1.58080614,  0.606884539},{ -0.0669001266,  -0.107259676,  0.606506765,  -0.290716588,  -0.91081506,  -0.0856149197,  0.397631466,  0.577718914,  0.353276014,  0.896081626,  0.12308532,  -0.694073915,  0.259434015,  0.261870354,  -0.417311817,  0.557764053,  -1.26315677,  1.19241691,  -0.353552669,  -0.577269137,  0.738829494,  -0.566872478,  1.84079432,  0.644729257,  1.13721967,  0.205897391,  -0.099678725,  0.449560076,  0.139965221,  -1.17211807,  -1.97735405,  1.2031095,  0.425581604,  -0.526643991,  0.124371208,  2.13226938,  1.23234022,  -1.34286368,  0.593575954,  0.767959833,  0.732264578,  0.634747982,  0.318031162,  1.1346513,  -0.800211132,  -0.32029596,  -0.0813739002,  -0.130912989,  0.53155607,  -1.26080191,  -1.56430876,  0.0073165833,  0.769165516,  0.747547805,  -0.93225807,  -0.491488874,  2.03320074,  -0.841446042,  -0.521005094,  0.727247775,  0.0043119546,  0.245079622,  0.606626868,  -0.365778357,  1.73970783,  -0.120657139,  -0.71831888,  -0.638770103,  1.51278543,  -1.81850004,  -0.778145373,  1.11371434,  -2.12488604,  0.173600748,  -0.396773636,  -0.206223831,  0.320624202,  0.310028583},{ 0.0162259433,  0.428949505,  0.454186231,  -1.34885728,  -1.53346694,  -0.549715638,  -0.395079792,  -1.00754106,  -0.829315722,  -2.04666376,  0.998056948,  -0.685318887,  -0.846381545,  -1.78085136,  1.08922946,  -0.605409801,  2.21084499,  -0.482257247,  -1.13011432,  -0.330332965,  -0.122006387,  -0.443443805,  0.267650068,  1.58240151,  -2.35696459,  -1.03905082,  1.72290397,  -0.811817646,  1.23234534,  -0.0912718475,  -0.241819993,  0.047310207,  1.40120149,  -0.372223645,  0.211012959,  0.790504813,  -0.295116097,  0.390708447,  -0.264161944,  -0.776821256,  0.424942076,  -0.944650233,  -1.00633788,  -0.420549363,  1.12374961,  0.955818832,  0.219322726,  1.37217355,  -0.192940637,  1.30928278,  -2.23836446,  -1.08281648,  1.02815485,  1.26113248,  1.07410431,  -0.422649205,  0.682316065,  1.95219254,  1.000875,  -0.629884958,  0.0836914405,  1.38698745,  1.30895293,  -0.57074064,  0.356817186,  0.684748113,  0.713962436,  0.0384293869,  -0.00935575552,  -0.272414833,  -1.81090176,  0.284892589,  1.43441629,  -0.0187107231,  -0.127360836,  -0.860459805,  0.711752176,  -0.25663662},{ -0.101294622,  -1.02463984,  -0.260435224,  0.672762573,  1.68044794,  1.1248765,  0.752319634,  -1.48487377,  1.14005065,  -0.647922695,  0.660003603,  -1.04753661,  -0.00734934676,  0.0773243085,  -1.37225485,  0.104752459,  0.942910969,  0.571175575,  1.14175439,  -2.13809752,  0.117121637,  0.200287536,  -0.955691993,  -1.17750716,  -0.149525911,  -1.13041031,  0.519091904,  1.26784682,  0.475331992,  -1.87057149,  0.248658448,  1.16177523,  -0.283360839,  -0.0433964282,  0.745495796,  -2.05113649,  0.0684325099,  0.989808202,  0.750303447,  0.030216001,  -1.92222619,  -1.33430016,  -0.584339499,  -1.53908765,  -0.548180401,  -0.275955379,  0.97292918,  -1.04093468,  -0.657145381,  -0.291266412,  0.90047431,  -0.34068355,  -1.62477326,  -0.998491049,  -0.47098735,  0.33864513,  -1.77193677,  1.09344161,  0.691086054,  -0.358820558,  -1.3623507,  0.427276462,  0.889689624,  -0.717386723,  -0.490869641,  0.725634813,  -0.75025928,  0.0558339953,  -0.239337191,  -0.137331918,  0.820971847,  -2.03640866,  1.36280632,  -0.344762385,  1.21090674,  0.139004856,  0.647977412,  0.000515847409},{ -0.998517513,  0.599027932,  -0.979510546,  1.81424499,  -1.47257316,  0.155538067,  -1.36566675,  -1.49119687,  -1.37933147,  1.30836546,  -0.339245677,  0.681088865,  0.93733418,  -0.856548667,  1.60245383,  -0.292155027,  1.07986104,  1.21517622,  0.799538672,  -0.629230261,  -1.31154501,  0.794744074,  -0.752602696,  -1.56841898,  0.495143056,  -0.572362185,  -1.2394284,  0.0112970984,  0.300289452,  -0.0997000933,  0.144449472,  0.511853278,  -1.02732003,  -0.903650701,  -0.62330085,  -0.664105535,  0.0917736888,  -0.465129137,  0.38408941,  -0.584238768,  1.32596314,  -0.945163488,  -1.22987235,  -0.301605135,  -0.282960385,  -0.581650496,  -1.11769438,  -0.091708906,  1.15691841,  -0.912612915,  -0.311193883,  1.07986939,  -1.07671368,  1.20590472,  1.60949528,  0.384500474,  1.03130507,  0.32259649,  -0.277133226,  0.378230274,  -1.97555518,  0.808925569,  -0.990345418,  0.712934136,  0.0314029604,  0.995866418,  -0.886173725,  -1.50246406,  0.720558882,  -0.161643341,  0.466063678,  -1.36563504,  0.548217714,  2.02027917,  0.315325826,  -0.310742944,  0.477964371,  2.13938093},{ 0.361140966,  -0.204085812,  -0.267049849,  1.65268028,  0.661569297,  0.583538592,  -0.654841483,  0.365090877,  -1.80782247,  1.07766533,  0.0953159258,  0.845515847,  0.673571765,  -0.186545849,  -0.861748755,  -1.9852289,  1.03895938,  -1.28138363,  -0.545324802,  0.752026856,  -0.520359039,  0.495501995,  2.45213723,  0.373540938,  0.818917215,  -0.907927811,  -1.77288878,  0.310381055,  2.07843065,  -0.114837378,  0.506864369,  0.926356554,  0.436000049,  1.15437257,  0.726707578,  1.01779652,  -0.952795208,  -0.496093392,  -0.578057408,  0.880360425,  -0.834368885,  2.07034039,  -0.481285483,  0.0977698267,  -0.576418698,  -1.90621138,  1.06771481,  -0.0642596632,  0.630672216,  -0.149154484,  0.0649365261,  1.97459638,  -0.10536164,  0.490583748,  1.02405417,  -1.15325439,  1.01024961,  -0.240881398,  1.03773546,  -0.855827034,  0.29788512,  -1.79681051,  0.388542265,  1.51246154,  0.22450833,  -0.15416941,  0.617774844,  1.60406899,  1.1016736,  0.485117584,  0.576071978,  1.06655562,  0.42553398,  -0.389718175,  -1.10838079,  0.360723764,  0.693071961,  0.117201239},{ 0.176984757,  -1.19767153,  -0.796020865,  0.733118474,  -0.209279433,  1.64336169,  1.69023883,  1.52705753,  1.85880125,  0.31144312,  -0.0434955209,  0.922467232,  0.782709002,  1.57955658,  1.01810789,  -0.547020257,  -0.00145755697,  -0.756350935,  -0.540504992,  -1.14304161,  -0.293622822,  -0.571370721,  1.33970773,  0.00404504919,  1.15533566,  -0.341761023,  0.781187654,  0.886647522,  2.06727052,  -0.117700167,  0.581640542,  -1.28676629,  -1.35496604,  2.03896761,  -0.450934023,  0.646862388,  -0.340841115,  2.30085373,  0.183356225,  0.442373753,  0.586342514,  -0.213487267,  0.0823890716,  0.502307713,  1.65074992,  -0.1118351,  -0.439990431,  0.441135764,  -0.891224682,  1.23933423,  0.602839828,  0.286713362,  0.28540352,  1.0324893,  1.08000588,  -1.1735301,  1.73476648,  -0.115351319,  1.10323286,  -0.275918931,  -0.465141028,  -0.953920901,  0.486734211,  -0.262966543,  0.564561725,  2.76008463,  0.342483163,  1.32194638,  -0.304495424,  0.78432411,  0.319481164,  0.269693524,  0.262810498,  0.316975057,  0.796295345,  -0.0977852419,  -1.44545519,  -1.54366004},{ -0.941831648,  0.560002804,  -0.072642602,  -0.00757614383,  0.921657503,  -1.59135747,  0.322881043,  -0.133656055,  -1.25835884,  1.91678298,  -1.27746356,  4.48120975,  -0.634328187,  -0.885961533,  -0.505360305,  -0.686874807,  -0.522948325,  1.29960334,  -1.24736071,  1.27905023,  -2.15399647,  -1.16581893,  2.42808247,  -0.00443723612,  0.1000451,  0.556916833,  1.22857118,  0.988278389,  0.343390733,  1.02141523,  -0.389196634,  -1.73953378,  -0.116840288,  -1.22916543,  0.52201426,  -0.68675983,  0.32348597,  -0.442909479,  2.03335667,  -0.999660969,  1.09983802,  1.08658338,  0.137085482,  -1.67830455,  0.153569356,  0.627691686,  -0.888320446,  1.02358627,  0.430019051,  -0.15615809,  -0.368356436,  -0.245418862,  -0.711627722,  0.214164764,  0.380086899,  -2.02425814,  -0.329244643,  0.873981237,  -0.899162471,  -1.32683396,  -2.07542515,  -0.0803143233,  -0.659375608,  0.243949488,  -0.388253897,  0.50202477,  -1.6089499,  2.93218231,  -0.597409546,  -0.6019575,  0.447649747,  1.55953276,  -0.504897475,  -1.36573589,  1.60212564,  1.3721348,  0.644536018,  1.24728525},{ 0.835769355,  -1.84278393,  1.91148055,  -2.53571701,  -0.422308892,  0.849550843,  -0.0467943959,  1.17318785,  -1.35543704,  -0.733518779,  1.16612244,  -0.502649724,  0.222720236,  0.197761044,  -0.376895845,  0.772384405,  -0.209676564,  1.82498252,  -0.75706017,  1.22744048,  -0.368007869,  1.45226705,  -0.245900109,  -1.43001759,  0.518062174,  -0.195396602,  0.562505186,  1.08643913,  -0.0896754563,  0.082343325,  -0.873640001,  0.118718505,  -2.12798667,  0.959481716,  -0.129731908,  -0.421589136,  -1.1079545,  0.59864074,  0.115961641,  0.777428389,  -0.781036794,  0.245991185,  0.502062142,  0.570976794,  -0.205554947,  0.0124462536,  0.725953698,  -0.61557591,  -0.0866195783,  -1.46355355,  1.30372715,  -0.205780968,  -1.3340826,  -0.832597554,  -0.0697393417,  2.05867577,  -1.73345399,  0.87414223,  0.871956468,  -0.865823984,  -0.264600635,  1.8477931,  -0.10019774,  0.096129857,  1.19529366,  -0.935593903,  0.792288303,  0.564159632,  -0.923836708,  -0.441225916,  -1.88902986,  1.27207172,  -0.732991636,  -0.978996515,  -0.392352879,  -0.233681351,  -0.608836532,  -0.720274568},{ -0.642468572,  -0.721163809,  1.77912164,  -0.921288252,  -1.35605931,  -0.333087951,  -0.805787086,  -0.202039704,  0.0579356775,  -0.673726737,  0.558620334,  1.79031563,  0.846997023,  -0.648366272,  -0.451977372,  -2.09967399,  -1.73856258,  1.6444428,  -0.448979706,  -0.0392631255,  -0.0867069215,  -1.05284333,  0.382136017,  -0.506757498,  1.05387592,  -0.822485805,  0.324308008,  -1.15351796,  0.156319782,  1.74494815,  0.15292348,  0.527923942,  -0.466197193,  -0.809930444,  1.57714927,  -1.58087349,  -1.98337317,  1.971174,  0.657913089,  1.66566086,  -0.454036683,  1.47033203,  -1.03678715,  0.5941028,  0.559985399,  1.34220326,  1.29303133,  1.3728044,  -0.355748028,  0.0780892968,  -0.220393613,  0.187833667,  0.553236842,  -0.186120465,  -0.743128657,  -1.32727456,  -0.297162682,  -0.46555087,  -0.891586065,  -0.269217879,  -0.759390831,  0.929091036,  0.200061008,  -0.141630664,  1.1635493,  -2.30609632,  1.28160179,  -0.138614029,  -0.378274769,  -1.14354146,  0.506090283,  0.893459022,  1.61451828,  -0.460617602,  0.123499244,  -0.470799685,  0.0917802751,  -2.05055714},{ -0.584197998,  0.672518015,  -1.08396876,  0.349322379,  -0.293138921,  1.00636375,  -0.139122963,  -1.3206706,  0.132480338,  0.823758304,  0.389360994,  0.869154632,  -1.76919591,  1.33956075,  0.539153934,  1.31265926,  0.0213170648,  -0.195695251,  0.459151179,  1.38140202,  1.14489019,  0.160675302,  0.34119764,  1.59777629,  0.136340246,  -1.86381948,  0.762216866,  -0.150100917,  1.31341541,  -0.272827804,  -0.438044041,  -1.45247245,  1.86619985,  -1.24752188,  1.70481646,  -0.530983925,  -1.46142018,  0.888539433,  1.53141892,  0.278285652,  0.342489034,  1.12219059,  1.46329069,  -0.0605830587,  -0.0265105888,  0.218856797,  -2.22503376,  0.664598167,  -0.0779043436,  -0.656316876,  0.641347349,  2.24964714,  0.233606815,  0.718938828,  0.0372598879,  0.333858699,  0.320631653,  0.0422408432,  -0.901428401,  1.79563713,  0.280673683,  -1.87024355,  -1.34283745,  0.806347847,  -0.123611294,  -0.148843765,  -1.89611375,  -1.7748729,  -0.471440345,  -0.515394509,  -0.952200115,  0.569507062,  -0.0771169513,  1.45394003,  -0.0201701559,  -0.986219347,  0.200086251,  2.07607102},{ 0.649904013,  -0.578366578,  -1.17643785,  -0.714329123,  -1.34646988,  -0.664227247,  0.461338967,  -0.594662726,  0.388448,  0.752497375,  0.349752218,  0.42667979,  -0.116472594,  -0.0441782437,  -0.166592926,  0.39889425,  0.764159262,  -0.751012266,  1.11520219,  0.268649548,  0.329148233,  -0.239720151,  -0.123859696,  2.79215598,  0.069293566,  -1.62295377,  -0.198672622,  0.0048888661,  1.00630891,  0.455521405,  -1.10091782,  0.892737925,  0.0281252973,  0.865740955,  -0.977611184,  0.145779282,  0.504624069,  -0.334214091,  -0.292301118,  0.329297692,  -1.32400095,  1.8342272,  -1.18562102,  1.13524246,  0.745883167,  1.37254357,  0.497112989,  0.0254533328,  -0.323060781,  -0.6115641,  -1.13240349,  1.21826446,  0.22871545,  0.25182125,  0.526987016,  0.663609147,  0.376871496,  1.11830783,  1.89533007,  -0.251348525,  0.866647243,  -1.30325866,  -1.67015922,  -0.400333434,  -0.510526061,  0.77513665,  1.02813065,  -0.407268196,  -0.959961057,  0.455339223,  -0.410148025,  1.22214031,  -0.632665157,  -2.11720204,  -0.805322647,  -1.30421567,  0.0475583971,  -0.0249404367},{ 0.20727849,  -0.0896398947,  -0.959931493,  -1.49259782,  0.519652009,  -0.469507009,  -1.19456112,  0.495191544,  -0.231237575,  -1.83640373,  -0.497748762,  -0.572791696,  -0.734519422,  -0.931150615,  -0.663001835,  0.313693047,  -0.175716609,  -0.299394846,  -0.671059251,  -0.0741730183,  -0.263352215,  0.404464662,  -1.27501953,  0.790674388,  -2.67420363,  0.570673227,  -2.19723034,  0.0853790268,  -2.00152588,  0.179649368,  0.222246721,  -0.860865474,  -0.447786599,  0.0955400243,  0.759629965,  -0.787680924,  0.253556103,  0.187141374,  1.02357912,  0.186104864,  -0.292158335,  0.311545193,  -0.640421093,  -1.1041528,  0.16492939,  0.883787334,  -0.70977515,  0.505998075,  1.22537601,  -0.412648678,  0.0519032739,  -0.871070981,  -0.186589211,  -0.681906879,  -0.385928571,  0.173707262,  -0.166702077,  -0.576193571,  -0.109068982,  1.29115212,  1.15209913,  0.227194235,  0.657365203,  0.346740097,  0.0574618354,  0.721514165,  0.37028712,  -0.916248977,  1.85692525,  -0.311088949,  0.506138086,  0.441740453,  1.20943725,  0.666511536,  -0.0915447399,  0.212645262,  0.924427986,  -0.807466447},{ 0.672224581,  0.665691197,  0.888746679,  0.588074684,  0.290324956,  0.351579517,  0.131140351,  -0.613772571,  -0.675782681,  -1.04238045,  0.261751026,  0.0478394851,  0.523494065,  -1.12795818,  -0.0975074023,  0.324932873,  -0.556902707,  0.00621486455,  -1.13320899,  1.81326842,  -0.342080802,  0.389048845,  -1.5378828,  0.260051012,  0.168866143,  0.846772015,  1.17171597,  -0.421783328,  -0.877356708,  1.01950741,  -0.84577179,  -0.587874413,  -2.27130127,  -1.8444829,  0.686817408,  0.347132713,  -0.680950403,  -1.44803691,  0.143984288,  0.0880657509,  -0.53389138,  -1.24926949,  -0.409024715,  0.47416985,  -1.31869674,  -0.00475420197,  -0.132711753,  0.260433018,  1.04192162,  -0.357575297,  -0.391849071,  -2.72032523,  -1.0114826,  1.31053793,  0.635336161,  0.381745517,  -1.8043673,  -1.02563703,  -0.817168772,  0.486045778,  -0.0422633812,  -0.269488126,  0.419248909,  -0.593538642,  1.19387591,  0.901544333,  -0.716094077,  0.839067638,  -0.0565106608,  0.0564437807,  0.0952686965,  -0.969885647,  2.78112411,  1.33840191,  -0.179196134,  1.02193964,  -0.426107138,  0.500440836},{ -0.422950149,  -0.951745033,  0.929627419,  0.83583951,  0.0360366218,  0.372206479,  -0.0901619941,  -0.136263594,  0.0264606383,  -0.95140028,  -1.39774263,  -1.09901786,  1.52251446,  -0.513100564,  -0.541931927,  0.25936532,  -0.254751176,  -0.343314976,  0.752073765,  0.400574684,  0.0018363375,  -1.44232202,  0.459526598,  -0.663617969,  -1.14176333,  -1.29796338,  -2.44672894,  2.34814548,  -0.544756234,  0.99133575,  1.30093133,  -0.591079712,  0.303231537,  -0.159233108,  -0.675252855,  -1.93278861,  -0.375253677,  -0.716335714,  0.384678096,  1.95290256,  -2.08736324,  -1.10505939,  0.207863703,  -0.367322832,  -2.14684939,  0.33434993,  -0.697157085,  -0.781612396,  -1.2234298,  -0.544913888,  0.955607116,  -1.37863445,  -0.468765616,  1.15912223,  1.48853862,  -0.484695345,  -0.710271955,  -0.566938341,  -1.29867661,  0.346359193,  -0.773522556,  -1.23350334,  -0.24605009,  -0.96554786,  -1.48851573,  -0.343322337,  1.40549028,  -1.24036145,  -0.522062778,  -0.925463736,  -0.455324382,  -1.40273941,  -1.45378482,  -0.021441685,  -0.466728628,  -0.25378114,  -0.634111524,  0.894914806},{ 0.134082019,  0.0352609195,  -0.863797784,  0.45749861,  -1.00199246,  -0.917956054,  1.50735188,  0.0945924297,  -0.571812809,  -0.471857667,  1.57200599,  0.858097851,  1.09939051,  1.04554069,  -2.25380802,  0.0504817963,  -2.68899417,  -0.434952497,  -0.608159363,  0.771143734,  -0.568177581,  -0.24480845,  1.13504708,  1.11869979,  -0.575546682,  1.20611286,  -0.652707398,  1.13472056,  1.774611,  -0.742593288,  -1.00353527,  0.674799681,  -0.797948241,  -1.29068851,  -1.25573075,  -1.77159572,  0.405712754,  -0.270784587,  1.43762696,  1.67236674,  -0.782781541,  -0.625504196,  1.09044862,  1.45438635,  0.736813962,  0.14023228,  -0.121929608,  -0.478783607,  -0.957750797,  -0.922259212,  -0.90729183,  -0.758687198,  0.7441082,  -0.299104184,  -1.64134371,  -1.04538536,  1.17664325,  0.380367845,  -2.18808532,  -1.60984015,  -0.658755302,  0.914670408,  1.70294106,  -1.42758644,  0.189555764,  -0.193569258,  0.967380702,  -0.449374974,  -0.573258758,  -0.110929199,  2.14331603,  -0.44003585,  -0.209234565,  1.07831633,  0.483998805,  1.85918105,  -0.356760591,  1.30652213},{ 0.325189918,  0.0485778637,  1.0641557,  0.708163142,  0.296239197,  -2.05049205,  1.74400008,  0.557648897,  -0.482432604,  1.60337794,  0.33380568,  1.0034523,  -0.025887521,  0.588284731,  -0.571980476,  0.621417403,  0.0417318903,  1.09014142,  -0.763542295,  0.613375485,  -0.311575651,  -0.0918627232,  -0.279315948,  -0.782086134,  0.578583896,  -0.883128047,  0.686948717,  0.41153875,  1.66003668,  -1.45106113,  1.94040763,  0.306981981,  0.396449566,  1.19639707,  -0.805627942,  1.36146176,  -0.142119631,  0.176382095,  -0.191685304,  -1.16857958,  0.766983688,  -1.88901949,  -2.96784425,  0.333317488,  -0.881792068,  0.406827688,  1.00384128,  -0.600129902,  0.946510792,  -0.959650874,  0.439178169,  -0.42681253,  -1.93239057,  1.4635272,  0.237610221,  0.905590892,  1.45704532,  1.23558593,  -1.13253307,  0.600228608,  -0.387262821,  -0.42649731,  -1.42869794,  0.51975137,  0.0972767696,  0.164716184,  0.718900442,  1.27652133,  1.78566039,  1.23284435,  0.334904343,  -1.49979222,  -0.171235591,  1.30846131,  0.577671289,  -0.567882121,  -0.11147029,  0.945832491},{ -0.443827868,  -0.376238018,  -1.91922593,  0.379823834,  0.382213801,  0.898179471,  -0.0823916495,  0.392873764,  1.01011622,  0.0580310933,  1.66760921,  0.643923998,  -1.36095953,  -0.674982488,  0.43685022,  1.02052546,  0.386508465,  -0.779589355,  -0.135836482,  -0.221383736,  -0.0447935499,  0.201428652,  -1.28151095,  0.0447264835,  -2.04764366,  0.177132264,  1.2223227,  0.259957016,  0.776808262,  0.824102819,  -0.544374764,  -1.61569452,  -2.4219923,  0.0476978831,  -0.553499937,  1.01736307,  -0.0514883399,  1.43329573,  1.22480655,  -0.687558055,  -2.98115897,  -0.699461997,  -1.26625371,  -1.34018815,  1.22692251,  0.202012137,  -1.01587629,  0.288333863,  1.30596769,  -0.397028804,  -0.297091067,  -0.260852784,  -0.283557773,  0.918341815,  -1.0494436,  -0.498235375,  0.0986568034,  0.892049909,  0.0623328947,  -0.778398693,  -0.710238874,  2.22168875,  -0.0522571169,  -2.20824718,  -2.48916984,  -0.994564652,  -0.992734969,  0.815213382,  0.792402744,  -0.994714081,  -0.365077585,  -0.431090176,  -1.7008872,  1.58644831,  -1.05362427,  -0.0220120046,  -0.685065985,  0.0774108246},{ -0.191000164,  -1.17314804,  0.637281179,  2.42050052,  1.22455359,  1.41069853,  0.245531619,  0.508383691,  -0.195069462,  0.482899576,  0.574452996,  -0.287912548,  -0.799625218,  -0.665262341,  -0.483418703,  0.889943004,  -0.677891195,  -1.31888473,  1.28718591,  0.682327867,  -0.38443464,  -2.18286228,  0.691016734,  -1.49386775,  0.344353676,  -0.411646515,  -1.47030103,  1.75122976,  0.571642518,  -0.881606758,  0.0835005641,  1.37420404,  0.635218024,  -0.652891994,  -0.871690094,  -0.047253713,  -0.946860671,  1.99873304,  0.362921566,  0.636421382,  0.500746787,  0.593915105,  0.562006593,  -1.92712617,  -0.712358952,  0.44824937,  -0.485878676,  -0.0713769943,  -0.226592734,  -1.38288283,  -0.11072959,  0.490560383,  -1.68372953,  -0.705627799,  -0.97023958,  -0.184802696,  0.609916329,  -0.400575459,  0.435692728,  0.932899952,  1.76348019,  1.29678881,  0.074882865,  -0.166457444,  0.4368065,  -0.569232583,  -1.24761558,  0.784911394,  -0.0495892987,  0.293637574,  -1.23988295,  0.981208384,  0.141748443,  -0.0310810916,  -0.715592682,  -0.203693181,  0.0268722046,  0.59446454},{ 0.987497509,  -1.51806593,  -0.853430331,  -0.746488571,  -0.738608658,  -0.317934304,  0.798566043,  -0.928491175,  0.407346189,  0.952741504,  0.258803934,  -1.38618171,  -0.757721007,  -0.743051112,  -1.31998026,  -0.988164723,  0.885177195,  0.809252501,  0.237727329,  -1.4478966,  0.349393219,  0.520707667,  1.8346144,  -1.24566054,  0.389490217,  0.164950967,  0.657823861,  -0.966446221,  -0.918728113,  0.754510224,  2.08748484,  -0.135842085,  1.09819221,  0.330898494,  -2.06862926,  -1.15495479,  -0.0206439961,  -0.457060337,  -0.636533976,  -0.181751877,  -0.212439135,  -0.503237307,  -1.88429606,  -0.778634489,  0.529980659,  -0.905992568,  0.869160354,  0.678166687,  1.11979795,  -0.00625912007,  -1.32351542,  0.73780632,  0.79921025,  -1.49090493,  1.38535786,  0.145324558,  -0.612861156,  0.695564687,  -0.588417053,  0.475461751,  -0.31670028,  0.525157988,  1.7331953,  0.939509392,  1.46968544,  0.614986241,  -0.777009308,  1.09757018,  1.89873779,  -0.0791133121,  -0.564155519,  0.222140431,  0.214810729,  0.704796433,  -0.637115061,  0.690568745,  -0.134225622,  0.890318036},{ 0.66032511,  0.243087858,  -1.57655966,  -0.159883082,  0.643647194,  0.75307554,  -0.525687754,  -0.806192756,  -0.8429721,  -1.10942769,  -1.47311389,  0.696181238,  1.5226655,  0.0507216565,  0.769141555,  1.20260322,  1.81421876,  -1.62466097,  -0.768801093,  0.936762035,  0.492176682,  0.0501489453,  1.80701411,  -1.17436409,  -0.715110838,  0.101733662,  0.0276078749,  -1.24732304,  -1.47504485,  -1.60625851,  0.027584346,  1.33270383,  -0.60308826,  0.666102886,  1.30424178,  -1.23556066,  -1.02996242,  -0.138560578,  -0.739144504,  -0.314452916,  1.72749734,  -0.356506139,  -0.584025323,  -0.920742393,  0.999412775,  1.08466399,  -0.136698112,  -0.189486504,  -1.09695005,  -1.15574598,  -0.286484808,  -0.351137817,  -0.127826378,  0.497146696,  -0.358039439,  0.549468637,  -1.44526041,  -0.998873293,  -2.28748989,  -2.16960001,  0.0164069124,  0.156555787,  1.97233987,  0.209533483,  0.0858906433,  0.0578186475,  -0.19794412,  0.70360148,  0.36926806,  0.258814961,  -1.35081196,  -0.414390415,  0.541141987,  0.519029737,  0.890008092,  -0.770711422,  1.59170854,  -0.0125082973},{ -0.744853497,  0.246354774,  0.0108049661,  0.136275768,  0.791631341,  -0.741792858,  -0.0759384036,  -0.00739891734,  0.317430049,  -0.58747077,  0.0384554267,  -0.70003736,  0.570208013,  -1.12646317,  -0.423345149,  0.131565437,  -0.740145624,  0.112758547,  -0.0629279092,  -0.156094253,  -0.378344834,  -0.633583486,  -0.589645743,  -0.27067107,  0.459560812,  -0.181326956,  -0.010076086,  0.360363752,  -0.133497864,  -0.437426507,  0.0959511623,  1.60089004,  0.145426735,  -0.113471948,  0.72623837,  1.51773727,  1.09922111,  -1.21011615,  -0.736962259,  -1.09658265,  -0.135552451,  -0.77262944,  -1.33888006,  -1.88992357,  -0.961058021,  -0.0149887176,  0.55393362,  -0.0787756294,  -0.951395452,  -1.32393539,  -0.244322553,  0.0106461104,  0.0668923333,  1.44854581,  -0.619797409,  -0.201720506,  1.44590318,  0.35952431,  1.06279922,  -0.437435359,  0.190992758,  1.04783058,  -1.0908072,  1.08634984,  -1.8374846,  0.755884826,  -0.766436994,  -0.875150979,  -0.134429023,  -0.552314997,  -1.40036821,  -0.459617972,  2.06757784,  -0.919430077,  1.3997649,  -0.0103504928,  0.271608293,  0.330499053},{ 0.716057181,  -1.34721613,  1.13214684,  -1.78161049,  -0.184361577,  0.971493661,  0.590634823,  2.28461552,  -1.10504901,  2.13734651,  0.803421557,  0.0217406694,  -0.51613462,  0.268498003,  -0.267261714,  -0.617037952,  0.618914127,  0.10812071,  0.14689815,  2.88268709,  1.1639576,  0.237349272,  -0.488266706,  0.76135999,  -1.5551846,  -0.316623658,  2.03346968,  0.504564047,  -0.566822648,  0.121917315,  0.545348108,  0.0735944137,  -1.18879116,  -1.48436701,  0.100501396,  0.835765898,  -1.02183092,  -0.052442804,  0.55138278,  0.00830154493,  0.13826248,  -2.36068058,  0.5887779,  0.318988264,  0.189396784,  0.947066128,  -0.223772794,  -0.856397271,  1.71589482,  -1.58987272,  1.65401113,  0.357901186,  2.54901457,  0.501184165,  -0.259273499,  -1.24921274,  1.05272007,  -1.30762851,  -0.258071601,  1.63002467,  1.03742051,  -0.297174424,  0.616029143,  1.15459967,  -0.458324283,  0.28262493,  0.581697583,  -0.771792352,  2.56852007,  -0.331251293,  -1.86804032,  0.823862553,  -1.65159738,  0.470632374,  1.9821192,  -0.504127026,  -0.44532299,  -0.690775454},{ -0.326597601,  -0.00687396573,  0.126438975,  0.0460412167,  1.01698077,  1.28133106,  -0.214495122,  -0.372798175,  -1.47858119,  -0.794326425,  -2.19953823,  -0.943689525,  -0.388139725,  0.191461533,  1.44481492,  0.94330585,  1.98837066,  -0.609080374,  1.77547944,  0.0951999873,  -1.82432103,  1.67967427,  1.07572997,  0.499009967,  -0.999064803,  -0.521628797,  0.317546666,  1.20544291,  -1.86776233,  -1.69345713,  0.597948432,  -0.77369386,  0.1536991,  0.389747709,  0.202614993,  -0.979553461,  -0.0806935504,  0.450116843,  -1.14689505,  0.683695912,  0.099814713,  0.701034188,  -0.184404373,  -1.50697672,  0.797425747,  -0.601692855,  1.37077141,  -0.967574656,  -1.83771861,  0.0928587243,  -0.232232749,  -0.149414182,  -0.251035154,  -0.370668471,  -0.438569486,  -2.11236691,  0.822803199,  -0.842329562,  -0.664750516,  -0.767086864,  0.480324537,  -0.119031847,  0.517869234,  0.859496772,  0.291008174,  0.794801474,  -1.62258303,  0.571964204,  0.815377414,  -0.582360864,  -0.555631876,  1.70345163,  -0.868979692,  0.806589007,  0.86512351,  -1.09117329,  -0.104971498,  0.0892682597},{ 0.245768726,  -0.289562255,  -1.44156122,  1.49491596,  -0.39275974,  -0.533101797,  0.289949358,  0.259948879,  -0.514576197,  -0.921372175,  -0.702329099,  0.0938563198,  -1.5354712,  0.322710097,  -0.317845374,  -0.964311123,  0.430085838,  -0.294499487,  -1.66285813,  1.06804514,  2.5436604,  -0.472206742,  -0.510427058,  -0.582532406,  -1.36095309,  -0.817003191,  -0.418182611,  -0.241310149,  1.83662212,  -0.494906962,  -0.532657802,  -1.004215,  1.03281903,  -0.646123469,  -2.28369403,  0.0153775383,  0.528276265,  -1.2934866,  0.0420611799,  0.646179795,  0.272337079,  -0.734886765,  -0.0956869721,  -0.422786176,  0.577232599,  -0.342138678,  0.443218499,  0.321987331,  -0.32336548,  -1.91714919,  -1.23814785,  0.448622525,  1.85509932,  0.674001396,  -0.0301061384,  -1.89608288,  -1.91635609,  0.342135847,  -1.87803125,  -0.252780795,  -0.130791977,  -1.12859464,  0.26249662,  -1.81106782,  -0.0786689371,  0.43893078,  -0.443745494,  -1.37040973,  0.0966777802,  -0.730085433,  -0.606571615,  0.40899241,  0.807499349,  -0.387130827,  0.19184041,  -0.724052668,  0.892219841,  0.866103828},{ 1.80282879,  -0.759962261,  -1.88134921,  1.00606024,  -0.210969329,  -1.17392218,  0.334718764,  -0.117992952,  -1.21135938,  0.736210227,  0.800495028,  0.672809243,  1.61272812,  1.17450094,  -0.743499637,  -0.100057974,  0.191837817,  -0.638727427,  -1.87641299,  1.04678226,  0.778754413,  -0.530355453,  1.22711742,  2.46765327,  -0.620415986,  0.568679214,  -0.322538733,  0.148833722,  -0.932715654,  0.393875599,  -0.474198431,  -1.25612581,  -1.12402916,  1.72976136,  -1.83061314,  -0.212584659,  -0.0977740735,  -0.273606569,  -0.0123170391,  -0.800852358,  0.548253417,  -2.71265054,  -1.41358852,  0.772771478,  0.650865436,  -0.0817723796,  -0.86037755,  -0.78241092,  -1.91509926,  0.395554364,  -0.369648963,  0.458254516,  -0.212732837,  1.40181303,  0.84151715,  -0.720141113,  0.369519562,  1.17630005,  0.636659622,  0.704644501,  1.30917978,  -1.0500294,  0.0923331976,  -0.95380652,  0.951326668,  -0.352085799,  1.00372899,  0.207954347,  -0.283684939,  1.22746229,  1.19915295,  -0.841124952,  -0.928232789,  0.958738863,  0.143146038,  -0.950219333,  -0.792039335,  -0.665565729},{ 1.62899601,  0.656506717,  0.593328178,  -0.091050446,  0.124125913,  -0.409851193,  -0.708592951,  0.200674623,  0.504133105,  -1.40576661,  1.61690903,  1.19664454,  0.821344137,  -0.615061104,  1.17324972,  0.92372328,  -0.359433234,  -1.92048609,  -0.0110606607,  -0.174610481,  2.34238386,  -0.287937105,  1.14541101,  0.455244899,  1.02435875,  -0.48955062,  -0.0534009226,  -0.556793272,  0.747334003,  1.66107178,  -0.444230855,  -0.0652588457,  -1.2677058,  -0.270752579,  -0.862472773,  -0.405727118,  1.84432948,  0.308906972,  0.420007169,  1.15021551,  0.561805546,  -0.686282277,  0.767958283,  -0.461126834,  -0.558435202,  -1.38325536,  -0.775248885,  -1.26570094,  -0.680994749,  0.373157352,  0.846211851,  2.50651884,  0.692520201,  1.44000018,  0.205875486,  -0.580315173,  -0.08860147,  1.41202509,  -0.221097931,  0.951053619,  0.347856551,  -0.18424648,  -0.821928382,  -0.505372882,  -0.392678976,  0.19812806,  -0.277117461,  0.0918704942,  0.642561555,  -0.997115791,  -0.182974011,  0.577444434,  0.348322988,  1.6330694,  -0.44475019,  -0.114152245,  1.64578187,  -0.72680819},{ -0.525954545,  0.6343413,  0.783015311,  0.40393579,  -0.693398833,  -1.75717402,  -0.110384375,  -0.489225,  0.972820044,  1.02627313,  -1.34204447,  -1.14884102,  -1.39413989,  1.23068833,  0.104732223,  -1.57728589,  0.486186057,  -0.972020984,  -0.585294068,  -0.99673599,  0.847687006,  0.16290088,  -0.974484026,  0.234567568,  -1.31257236,  0.884268165,  0.115209006,  0.901030242,  -0.224849731,  0.970317662,  -0.0935866311,  0.369124442,  1.62103188,  1.86934173,  -1.43662632,  -1.79007244,  1.32849872,  -0.574714959,  -2.18913841,  0.187928692,  3.69198179,  1.59938157,  1.04174936,  -0.145394772,  -1.40817249,  0.342577666,  0.830872416,  -0.23208718,  0.195297346,  -1.19939756,  0.231426716,  1.30627251,  0.897538185,  -1.95912862,  -0.555218279,  0.626812637,  0.938445151,  -1.0666244,  -0.458388269,  -1.27598596,  1.4121902,  -0.482085884,  1.24134886,  -0.780357063,  -0.316359013,  0.518883348,  0.283577472,  0.0540091991,  0.517026126,  -0.609480023,  -0.197057784,  -0.125833631,  -1.44843733,  -0.164946944,  0.630237579,  0.359177738,  -0.965032935,  0.910241663},{ -0.564708173,  0.0668073222,  0.332070321,  -0.340732574,  -1.38409984,  -0.773483694,  0.343791395,  0.0900452286,  -0.663473189,  -0.530818641,  -0.489442021,  -1.64553905,  0.651503861,  1.00462735,  0.222128868,  0.993898988,  0.529893875,  -0.642867267,  0.503740489,  -0.980702758,  0.83415395,  0.891104043,  -0.558408797,  0.275616199,  -1.79768538,  0.468185276,  -0.44861421,  0.582081854,  -0.813553274,  -1.28244495,  -1.47183931,  -0.340667158,  1.0241344,  -1.22733986,  -0.56301266,  -0.385634184,  0.965275645,  -0.654403567,  -1.38411534,  -0.630598783,  -1.70223594,  0.509629548,  -1.32070363,  -0.0108783944,  -1.09079325,  -1.67483807,  1.47825348,  0.629577398,  0.246313646,  -0.411882252,  0.0668010041,  -1.60443699,  0.52767396,  1.27379,  1.47963381,  2.01543307,  -1.15095866,  -0.432227015,  1.17988122,  -0.992312074,  -1.10837317,  0.721722126,  -0.99282521,  0.290941179,  -2.59548378,  -1.31351197,  0.734389484,  0.126084417,  -2.60961795,  1.00838387,  0.439223289,  0.0399913862,  -0.192409024,  1.0074203,  -0.0926629007,  -0.277255505,  0.813875556,  0.196057916},{ 0.686448276,  0.856794596,  -0.135245427,  1.033728,  -0.231436059,  0.566846609,  -1.32801807,  1.57619965,  1.25094247,  -1.45451176,  -2.38978243,  -0.304638565,  0.684703708,  -1.94696724,  1.16660309,  0.435216099,  -0.00874791387,  1.83446789,  0.0324018225,  -1.15750957,  -0.0176194739,  -0.630850434,  -0.154129088,  -0.0333223119,  -2.87635517,  1.13812017,  1.28966665,  0.966484845,  -0.491194874,  0.299415648,  0.719588995,  -0.938191295,  -1.09034598,  -0.557386816,  0.792299628,  -0.209304497,  0.387600809,  -1.31143749,  0.698339701,  -0.983751416,  -1.06711161,  0.802827358,  0.0871672332,  0.207028106,  -0.50379622,  -0.3782821,  0.0554126799,  -1.00746894,  1.44000244,  -0.170828655,  0.629174829,  -0.0777341276,  -0.643242598,  -0.845182955,  -0.0889686793,  1.0623523,  -0.334992558,  0.295045823,  2.37566185,  -0.174932137,  -0.462829709,  0.384363592,  0.465248078,  1.70007718,  0.733226955,  -0.181923911,  1.69809568,  1.83124804,  0.97462374,  0.918096721,  0.635593355,  -0.307793796,  -0.00636599725,  -1.36897206,  0.980121315,  2.03730178,  -1.769382,  -0.962892592},{ -1.00917709,  0.522525311,  -0.609161615,  1.08657467,  1.10589385,  -1.05209589,  1.24016881,  -0.706146896,  -0.526771545,  2.57559061,  2.16988778,  1.81407356,  -0.0831536427,  -0.540096819,  0.870197117,  0.736348391,  -1.29503405,  -1.04432297,  -0.927211463,  -0.647643507,  -0.873357177,  -0.716519296,  0.831725776,  -1.25439525,  -0.66049093,  -0.446721584,  -0.901920438,  -0.553515375,  1.79507351,  -0.623262465,  0.182890415,  -1.56599677,  1.42558467,  -1.9248805,  -1.23846245,  -0.380069226,  0.552219987,  1.97685802,  -0.819010973,  0.246159658,  1.83384907,  -0.159348249,  0.476818651,  -0.467143714,  0.289996117,  1.54312193,  -1.68902838,  -0.23632364,  0.46019125,  -1.24518502,  0.389166802,  -2.03840303,  0.628129959,  1.66863298,  -0.296061546,  -0.47331199,  -1.03754234,  -0.156084403,  1.08471274,  2.18957281,  0.908363938,  -0.16272524,  -0.646971285,  -1.44061565,  -0.106948353,  0.42035836,  0.332891226,  -0.240485162,  0.0294229016,  1.27827835,  -0.629424632,  -0.0665694401,  -0.97930491,  -0.358844131,  1.54057288,  -0.537630677,  -0.798395276,  -1.17140782},{ 0.570965707,  -0.495251209,  0.639929652,  0.781356275,  -1.35244453,  0.056856662,  -0.0395545177,  -1.23559833,  -0.738664508,  -0.45683521,  -0.569513321,  -1.04327571,  0.836663306,  3.38724756,  0.263070256,  0.468885601,  -0.133521691,  -0.0664417371,  -0.283534318,  0.799853742,  0.383235186,  -0.591190159,  -0.0133963786,  1.24885786,  1.73193419,  0.880237699,  -0.451381654,  -1.06425726,  0.835388422,  -0.88223654,  0.380433738,  -0.444185495,  -0.40021795,  0.229847386,  -0.331956923,  -0.131239235,  -1.31008804,  -0.931476057,  1.0878588,  -0.102794625,  -0.586177826,  0.627577007,  -1.11161864,  1.00498414,  -0.402612895,  0.378520966,  -0.326647818,  0.511729121,  1.03697801,  0.519847214,  -0.0470551178,  2.06656861,  -2.29337502,  0.568850577,  -0.896382034,  -1.86736465,  1.08816373,  -0.150409564,  -0.613065064,  0.53318584,  0.18704313,  -1.0961926,  0.20679909,  0.599536538,  1.44364238,  1.30107868,  -1.27647352,  -1.92889082,  -0.204616204,  0.586758196,  -0.103739567,  -0.50105381,  -0.135585502,  2.12719083,  -2.14737344,  0.579987109,  0.334519744,  0.381606251},{ 0.680075407,  1.02221143,  0.655127108,  1.58407187,  0.582350373,  0.776476622,  0.0242285654,  -0.690348625,  0.654476285,  -0.578459144,  0.722633302,  -0.841914177,  0.487151384,  -2.92248726,  0.289894879,  -0.730916739,  0.0441098884,  -0.505199254,  -0.30271402,  -0.00360814412,  2.36164927,  -2.48505545,  -1.24748719,  0.950769007,  0.531726003,  0.99297905,  0.99510771,  -0.511474729,  0.774465144,  -1.07500231,  -2.0960443,  0.927040458,  0.925958157,  0.805214465,  1.04732382,  -0.0410054103,  -0.283845186,  0.379403889,  -0.599181831,  0.292520046,  -0.105051324,  -0.413534909,  -0.957908273,  0.9359411,  -0.0387104861,  0.111710884,  0.228649497,  1.16110277,  0.86100167,  0.853469491,  0.376141042,  0.108967409,  0.356667459,  -0.61137569,  -0.373337477,  -0.236155927,  -1.11238921,  1.11044347,  -0.789599955,  0.872082055,  -0.171258956,  2.11083317,  1.34235299,  2.61881804,  -1.04987073,  -1.73916852,  0.725095689,  0.779257059,  0.686576247,  -1.05861795,  -0.90893656,  -0.303348541,  0.0680699274,  -0.421370298,  1.25467336,  -0.992065609,  -0.676237404,  1.18598557},{ 1.08944464,  2.41272902,  2.22980356,  -0.744734466,  0.871053994,  -0.451435953,  0.795625269,  0.871441782,  1.61186647,  -1.56969404,  0.450670689,  -1.48678243,  -0.854369998,  2.13207912,  -1.30703247,  -0.2485542,  -0.889489055,  -1.51107717,  -0.299752623,  -0.0629307628,  -0.0728722587,  -2.01942205,  0.652788639,  -0.534647763,  0.0815038458,  -0.315676004,  -1.89763224,  -0.79220295,  1.71960104,  0.773415148,  0.696436405,  -0.512013257,  1.63029516,  1.13215911,  0.849102974,  -1.36382389,  0.272852898,  0.27601853,  -1.55566883,  1.15590894,  2.38523102,  -0.863865972,  2.08518577,  -1.73082006,  1.00362372,  0.81987381,  0.120418444,  1.53862453,  -0.152512431,  1.46794188,  0.816247702,  1.55996406,  -1.06540978,  -0.222496361,  -0.832745254,  -0.286129713,  -0.903490245,  -0.203361154,  0.707187772,  -0.0951875895,  -0.722454011,  0.1375743,  -1.537521,  0.49970305,  -1.28582406,  0.158855617,  -1.45076549,  0.709457934,  -0.550619781,  -0.156934261,  -0.10252656,  -0.576999068,  -1.33224857,  0.427900583,  0.704237282,  0.142405197,  -0.234870598,  -0.541513205},{ -0.0885541216,  -2.01042008,  2.28962278,  1.06415009,  -0.0107940631,  2.47751236,  -0.362980276,  0.562809169,  0.243716076,  -2.07719231,  -0.192053765,  -0.680407643,  1.38522959,  -0.625998378,  -0.471867263,  0.63304913,  0.0433508381,  -0.59447211,  1.98639107,  -0.798237443,  0.50946337,  -0.58203423,  -1.82857811,  1.53540611,  1.35446358,  -1.52999556,  -0.243363559,  -0.437789023,  0.570598423,  -0.60942179,  -0.271600962,  -1.06135118,  0.178270906,  1.25792897,  0.0446085371,  -0.555415928,  -0.976742029,  0.633091152,  0.873708665,  -0.385737926,  0.309759527,  1.13775039,  -0.642215669,  1.02861965,  0.707298636,  -0.141571999,  0.328446865,  0.837484956,  -1.09312701,  0.721445978,  0.208718166,  0.983029962,  -0.386385232,  -2.45996952,  -0.247107595,  -0.420372427,  -0.687173486,  -0.207584873,  -1.86902261,  0.349865168,  -0.779977679,  -2.25081873,  -1.49840021,  0.847998798,  0.788349032,  0.77505511,  -0.83808893,  1.82016313,  -0.771316648,  0.729602635,  -1.25559163,  -0.496477574,  0.926735103,  -2.05838943,  2.08303356,  -1.18150663,  0.972744286,  0.703709006},{ 0.470848262,  0.829845905,  1.56512618,  -0.927296281,  0.711609662,  -0.906362593,  -0.999404907,  -0.175402537,  -0.0368471295,  -0.22113435,  1.74878347,  -1.08155799,  1.39469349,  2.03824472,  0.546537697,  0.254118532,  -1.33389056,  0.469481647,  -1.05290937,  0.634328783,  -1.74317253,  0.691090167,  0.350552797,  -0.472849727,  -0.778238893,  1.09327495,  -1.50755095,  0.0654262677,  0.750695229,  0.98531872,  -0.861454546,  0.248151734,  1.55305469,  -0.772485256,  0.0152074834,  -0.699029505,  0.198530778,  1.0503521,  0.217049554,  2.03128386,  0.574630439,  -1.91162205,  0.73148489,  1.42067707,  0.324598312,  -0.297105551,  -0.187490374,  0.983501077,  -0.466117293,  1.39222503,  1.8095305,  -0.39798516,  -1.17774475,  0.768235266,  -2.53433681,  1.50265849,  -0.195989653,  -0.1651375,  -2.3294282,  1.07996035,  1.54887009,  0.260947108,  -1.1379205,  -0.00190348702,  1.29384625,  -1.37633359,  -1.79418492,  0.527279496,  -0.572117567,  -0.414998889,  0.398544729,  2.40038157,  1.1119535,  -0.990689814,  1.62594318,  -0.285353392,  -0.18574971,  1.32257962},{ 1.06184959,  2.34224534,  -1.64880455,  0.795653403,  1.21096456,  -1.19917142,  0.209498882,  0.730115652,  0.380348295,  -1.05485022,  -0.424491704,  -1.9045006,  0.902385414,  0.674857855,  0.0913003236,  1.05351746,  -0.670709133,  0.768518507,  0.96112895,  1.17277408,  -2.16932368,  0.34640792,  -0.689443767,  0.767494261,  1.69688869,  0.16229628,  0.347060174,  -0.812371969,  -1.19168365,  0.301907182,  0.335982084,  1.17738736,  0.932344317,  0.908926189,  0.546640337,  0.869058907,  -0.857612669,  0.639665961,  0.581635535,  -0.887919128,  0.53365016,  -0.52657342,  -1.05942249,  0.534479558,  1.26650846,  -0.688491046,  1.31103408,  -0.0310177132,  0.177169025,  0.163563713,  -0.153582186,  -0.553190351,  -1.65649486,  0.422987491,  0.309427679,  0.786316991,  -0.549056888,  0.299469799,  -0.526759505,  -1.09533346,  0.92012012,  0.341488481,  -0.196429223,  1.57071555,  -2.5656848,  -0.329587787,  1.11593103,  -1.47618198,  -1.02107048,  0.0132498443,  0.730312943,  0.46714142,  1.60792911,  -1.84762156,  0.635735154,  -0.244271114,  -1.66658545,  1.08279335},{ 0.548279881,  1.65779245,  1.11805415,  0.170308322,  1.43457699,  -0.202276245,  2.25415778,  0.0543271415,  -1.32196391,  0.0369377024,  -0.713011503,  1.08747077,  0.819833875,  -0.594011068,  1.56498516,  0.074142687,  -1.3269676,  -0.581929564,  -0.661728024,  -0.735221744,  0.392015874,  0.984583974,  -0.982169151,  -1.92418134,  -0.0715704858,  -0.135131851,  1.04121387,  1.55052257,  -0.505147874,  0.985897005,  -1.04134107,  -0.149064064,  0.282545835,  -0.843100548,  0.363272429,  -0.245318845,  0.131906912,  2.30769968,  -2.17228889,  0.268326849,  -1.02154529,  -0.0648703501,  -0.593412399,  0.710676253,  -0.0220364779,  0.09329772,  -1.63245249,  0.911213875,  -1.07531035,  1.7692709,  0.751937211,  -1.49981928,  0.765387058,  1.49862671,  -0.347074032,  0.857236087,  0.403300464,  -1.49198163,  -0.3838875,  0.974485934,  -0.223220721,  -1.08187997,  -1.54323161,  0.197789773,  -0.244640857,  0.346601158,  1.27687323,  1.15687168,  1.9726634,  -0.771208107,  0.763064742,  -1.28497493,  -1.40552557,  -1.80772281,  -0.121470444,  -0.446909994,  0.854859293,  -1.97268665},{ -0.775136232,  0.749821544,  -1.17824662,  0.897847354,  0.94483906,  0.320503533,  -0.612947643,  -1.43461978,  -1.64569783,  -0.447537005,  0.966823995,  -1.48568618,  -1.95987225,  -0.777380526,  -0.328679502,  0.530590832,  -1.02018285,  -0.515850008,  -1.13335669,  -2.08074474,  -0.416083574,  -0.295712411,  -0.228969112,  -0.720336735,  -0.338641107,  0.710990965,  0.223993152,  0.666400313,  -0.241136938,  0.333101362,  -0.416543812,  1.83995461,  1.74649489,  0.904500902,  -1.90194547,  -0.439148456,  -0.857816279,  -0.998427033,  -0.718968272,  1.33285558,  -0.55388236,  0.169038966,  1.2348094,  0.710750759,  -0.356561929,  0.571259081,  -0.127092078,  -0.117681935,  -1.45582068,  1.10082793,  -0.192271143,  -0.444600999,  0.271291077,  -1.87981486,  0.978002012,  -0.256512016,  -0.181025699,  1.46384299,  0.0456959084,  -0.343524396,  0.281092107,  -0.629378915,  0.316226184,  0.89543891,  -1.2049247,  0.0430091508,  -0.587134123,  0.379937202,  0.374519318,  -0.596746564,  -1.65144634,  0.0839601234,  -1.37269258,  -0.691446841,  -0.633172572,  -1.52070582,  -0.53975296,  0.489273161},{ 0.0548084378,  0.850256801,  0.532704473,  -0.0600798428,  0.817227125,  -0.309598446,  2.22021222,  -0.105128914,  -1.70931089,  0.859552026,  -0.752445102,  -0.343423992,  1.2376368,  1.2690748,  0.0969509929,  -1.81859648,  -0.723196983,  -0.708465636,  -0.141161323,  -1.47010672,  0.467770487,  -0.696870148,  -0.565044165,  -0.336115599,  -0.133399487,  -1.14526641,  -1.01878405,  -0.742669821,  0.150212079,  0.87053442,  -0.92595315,  -0.618835032,  1.38455081,  -1.89074421,  0.294776291,  1.04886591,  0.235124439,  -0.634056211,  -1.81632686,  -1.83339572,  -0.923560381,  -1.9407444,  -0.610130668,  -0.696238279,  -0.772510827,  0.817254603,  -1.27094626,  0.963917673,  1.08729792,  -1.35965693,  -0.363081753,  -0.429244071,  0.598515153,  0.249016955,  -0.811885655,  -0.477357566,  -0.105291396,  1.6373049,  0.937785566,  0.612445056,  2.35860443,  -1.97927439,  -1.65373695,  0.659671247,  0.826194048,  -2.01558495,  -0.0687647611,  0.0163634531,  0.00486847432,  -1.62656534,  1.18770993,  0.657761574,  1.15111268,  -1.06217337,  0.0684704185,  -1.98280358,  0.741533816,  0.577730298},{ 1.32544672,  -0.795992494,  -0.312926263,  -1.18289363,  0.207552403,  -1.27197826,  0.406516075,  0.606114686,  1.57659793,  -1.25720763,  -0.207376644,  -0.273620099,  -0.972975194,  -0.511163533,  1.53609121,  0.635001481,  -1.97262025,  -0.682629347,  0.0979781374,  -0.899097502,  0.328851491,  0.341411024,  -0.994098604,  1.07636237,  0.148199245,  0.539389789,  0.985500157,  -0.0785258412,  -0.817826927,  0.30932501,  -0.471743107,  1.136253,  -1.03136706,  -1.0847342,  -0.0367222317,  -1.534006,  0.404405117,  -2.02962303,  0.390594423,  0.500864685,  -1.77396405,  0.021411607,  -0.599398196,  1.56080234,  0.560432553,  -2.04552531,  -0.685569584,  0.62073189,  0.763722599,  -1.50953948,  0.895058572,  -0.00113287102,  -0.962228775,  0.25372985,  -2.13789201,  1.19457436,  -0.585436106,  1.65085351,  2.96869874,  -1.30319285,  -1.10867167,  -0.593028843,  0.894256353,  0.0875224322,  0.184636697,  0.51797384,  -0.0706879199,  0.171983704,  0.261901349,  0.472679973,  -0.533448994,  -0.0368298702,  -1.43841898,  -0.528790057,  1.56184185,  -0.0987524167,  0.25242129,  -1.73035324},{ 0.0530103855,  1.54417717,  -0.632810056,  -0.555547535,  1.3223877,  1.61550272,  -0.911331773,  -0.663288593,  1.26609671,  0.262503535,  1.38558316,  -1.12045562,  -0.37933585,  -0.207982093,  0.323762089,  -3.10298777,  0.486366183,  0.764155746,  2.01629519,  1.00339031,  -0.042078048,  -0.516311407,  1.84334493,  0.377692133,  0.374120593,  0.534403861,  -0.670349479,  -0.345325023,  -2.30576944,  0.298883617,  0.515388906,  0.398843914,  -0.165410995,  -2.30020857,  0.101709746,  -0.877127171,  -0.835369825,  -1.44448376,  -1.08979547,  1.45139933,  -1.40260959,  1.02432764,  1.88541031,  -0.139472887,  -0.669454932,  0.603909075,  0.452713996,  0.515902102,  -1.0453161,  0.833057582,  0.135530248,  0.811898232,  0.240865618,  -1.18702304,  -2.51139879,  -0.422105163,  0.80744803,  -2.87041855,  0.809555709,  0.56614995,  -2.04939675,  2.37465835,  -0.366685271,  -0.564698458,  1.18350518,  -0.0920389816,  1.79281914,  -0.332693934,  -1.12364018,  -0.910851061,  -0.540750325,  1.17577231,  -0.279016674,  0.625186563,  -0.351049066,  0.632020593,  1.22666776,  0.483201355},{ 0.508525074,  -0.484334707,  1.70689082,  0.314604729,  -1.32553375,  0.673367202,  -1.44639933,  -0.659417808,  1.43474746,  -1.80646586,  1.61271262,  1.5777905,  -0.195879519,  1.43854058,  0.567321181,  -0.400645077,  -2.49610257,  -1.07831371,  -0.228750229,  -0.189663798,  1.39458919,  1.02118933,  0.177456737,  0.558374703,  0.618628323,  0.41504696,  1.70204413,  0.65468812,  -0.161265835,  0.701528668,  -1.01872504,  -1.29914987,  0.0110972822,  -1.16207457,  1.47866857,  -0.676929116,  -0.827319384,  -0.608979046,  -0.0593199618,  1.17251432,  1.29374266,  -1.00243163,  -1.08751404,  2.41883039,  1.99785829,  -0.800167084,  -0.990570962,  0.856781662,  -0.242462516,  -0.180715874,  1.98008192,  0.240298375,  -1.39662051,  0.597306132,  0.230650499,  1.09745574,  -1.98223984,  0.794170201,  -0.148761988,  -1.14692748,  0.0849436522,  1.15109849,  0.372616142,  0.82437861,  0.138751999,  0.491703153,  0.263832361,  -1.44605553,  0.362985581,  2.27544594,  0.148329079,  0.538708389,  2.73676729,  0.731139302,  -0.865092695,  -0.127801165,  -0.812002718,  -0.0995478705},{ -0.855333626,  -0.921408296,  0.520324469,  0.873543441,  1.29181409,  0.949801624,  -0.0474947803,  -0.435529232,  1.24021137,  -0.129673868,  1.04225326,  2.44369292,  1.19862425,  -1.11580789,  -0.40272218,  0.155944869,  0.88628155,  0.33987239,  0.421818525,  -1.26885903,  -0.0697684735,  0.868406594,  -0.318905056,  -0.7119205,  -0.0227432679,  0.0967463404,  0.0455449261,  -0.201947078,  0.742001593,  0.256323189,  -0.599654734,  3.0251379,  -2.86882925,  -0.711341262,  -0.240341991,  -0.962521434,  -0.732473552,  0.324866802,  0.251178533,  0.36711815,  -1.09898841,  -0.168323591,  0.0300143547,  0.767553449,  -0.827622771,  -2.57158804,  1.29244816,  0.207036749,  0.343134642,  -0.947472632,  1.60606849,  1.69323587,  0.272547454,  0.440511882,  0.0857214704,  -0.418821275,  -1.11135137,  -0.19785215,  0.840485156,  0.466143042,  -0.915403962,  -0.668010354,  -0.400882751,  2.1269896,  -0.149708793,  -0.890696049,  0.873947561,  0.161411047,  0.777314007,  2.30309248,  0.741866112,  2.44065404,  -1.82437766,  0.650201797,  -0.348777086,  -1.01528001,  -0.206950501,  0.0643700734},{ -0.779080451,  -0.679921925,  1.31309783,  0.535005569,  0.385798454,  -0.907465696,  0.589624941,  0.393331081,  0.836543739,  -0.965997875,  0.709096849,  -0.285617888,  0.614117742,  1.47843468,  0.0664531216,  0.789562821,  0.183158487,  -0.430157691,  -0.485133231,  -1.07920694,  0.542406142,  -0.174486712,  -0.348810196,  -1.0291214,  0.657744527,  0.56243515,  -0.426545739,  2.97639561,  -0.976031482,  -0.491135597,  0.0847644508,  0.274037182,  0.113407552,  -0.354947627,  -0.739117861,  -0.652694643,  -0.148994744,  -0.669933915,  0.792960942,  -0.586428404,  0.897993445,  -0.793037653,  -0.61807245,  -0.111669593,  -0.183757707,  -1.72397411,  -0.421759844,  1.85711241,  -0.406346083,  0.520635068,  0.0957552418,  -0.900591731,  0.640608072,  -1.8984524,  0.880341649,  1.71904564,  0.0484147333,  0.566003919,  -0.802158535,  -0.28513813,  -1.30055213,  0.0063717314,  -0.382856131,  -0.620936871,  -0.860878527,  -0.845637083,  -1.02170861,  0.317119151,  0.194722146,  -0.610504985,  1.35744393,  -1.67559481,  -0.0122377072,  -0.669351995,  1.78214729,  0.743747354,  -0.957298994,  0.276887983},{ -0.480617046,  1.27235126,  -0.39891988,  0.531332612,  -1.80341601,  1.66740906,  0.419049203,  0.363286018,  0.0190697983,  -0.0982138067,  1.62582088,  -0.897445858,  -0.358005047,  1.6509372,  0.0696512386,  -1.55332744,  -0.716619194,  0.506558478,  0.886671066,  -1.02887666,  -1.42782736,  -0.81495738,  -1.32994807,  0.391360253,  0.413899958,  -0.143478066,  0.929943323,  0.15619044,  -0.0243623182,  -0.72936976,  -1.12055695,  1.9745096,  0.990323603,  -1.50870144,  0.6176157,  -0.122856669,  -0.441898823,  1.20865762,  0.306460589,  -0.15598698,  0.59231317,  -0.373719722,  -0.0638070628,  -1.55932021,  -0.279237092,  -1.08503985,  -0.781206131,  1.44916379,  -0.386020005,  0.705245137,  1.98475897,  -0.332815677,  -0.265437096,  -1.05731964,  1.74946773,  -1.33769286,  0.0129794469,  -0.0919075981,  -0.6415869,  1.6475637,  0.73997134,  -0.927716017,  0.428432137,  -0.151510134,  0.534303308,  1.78016293,  -0.794971406,  -0.251222193,  -0.13964431,  1.03173232,  0.375210643,  -0.515190303,  -0.758123934,  -1.26511097,  -0.191927582,  -2.1424346,  -0.321143985,  -0.0969464555},{ 0.0415687673,  1.52631366,  -0.338801056,  -0.280480444,  -1.31896865,  1.895877,  0.199041471,  0.748996079,  -0.3297773,  -0.67204088,  0.46918276,  1.25902069,  2.42370701,  0.298381001,  -0.352174163,  1.18756819,  1.33859611,  1.50567901,  0.304114878,  0.347459257,  0.569217086,  0.756493628,  -0.350711018,  -0.648370445,  -0.801742494,  -0.346254528,  -1.48507583,  0.283722758,  -2.08239794,  0.164767534,  0.0260319952,  -0.0496423841,  0.457616448,  1.26779413,  -1.39678776,  0.250530481,  -0.789757669,  0.104635023,  -0.00309287687,  0.0811867118,  -0.49324429,  -1.9734838,  -0.809651315,  -0.208692312,  0.148478091,  0.254262537,  0.732701719,  -0.414832294,  0.0721477494,  0.676267564,  -0.745508909,  0.477559507,  0.521780908,  -0.654137313,  1.19406164,  -2.27150655,  -1.39069593,  -1.59477139,  -1.72776198,  -0.466559231,  1.73630929,  -2.00698137,  0.315739661,  0.410085231,  1.02670658,  2.13380384,  0.451720566,  0.237978593,  -0.351877064,  -1.47161472,  1.73646343,  0.328964382,  -0.434720039,  -0.120576225,  -0.38391462,  0.325616628,  0.726168215,  -1.48256421},{ -1.94698203,  -2.96277833,  -1.27488041,  0.496324629,  0.50396347,  -0.663172781,  -0.0934229568,  -0.0843490437,  -0.138858527,  -2.23330379,  0.0482876599,  -1.96697605,  -0.0627606139,  -0.173216701,  -1.88841045,  0.268548608,  2.77987075,  -0.295079976,  -2.76349044,  -1.26855135,  0.960856795,  -0.499239475,  -1.18805516,  -0.447725117,  -1.16200447,  -1.66534686,  0.437441707,  -1.71708286,  -0.0578521118,  0.629919469,  -0.733327746,  -1.10514736,  -0.00186792261,  0.448672354,  1.23263526,  0.624478102,  -1.8713721,  -1.39194369,  -0.0289851688,  0.491815925,  -0.575564384,  2.14538646,  1.04120159,  0.155134216,  -0.541510761,  0.776522517,  2.30195379,  -1.52650452,  0.26209116,  0.296901286,  -0.865180731,  -0.50567925,  -0.35320124,  0.371967107,  0.904159427,  0.483042032,  1.73076189,  -1.03648365,  0.106119916,  -0.264651805,  -1.22201204,  1.57195795,  0.103423543,  0.916488707,  -0.304210097,  -0.94051075,  0.0537078381,  -0.302324831,  1.02480412,  0.970425427,  -1.07163203,  0.369497389,  1.07125747,  -0.119485781,  0.565752923,  -0.225758716,  0.37379235,  -0.131119072},{ 2.49229956,  -0.562537968,  -0.718332469,  2.1561017,  -1.53758717,  -0.851981819,  0.671634018,  1.97331834,  -0.364593148,  -0.58111763,  1.90590322,  0.0868576095,  1.34273863,  -0.00476601068,  0.196701005,  0.454908192,  1.35462534,  -0.254604995,  0.67728442,  0.346756667,  0.337319821,  0.356123656,  0.0267058965,  -1.46630621,  0.776387811,  -0.758274615,  0.739805639,  -1.27695632,  -0.750513494,  -1.50534642,  -0.296820462,  -0.827207208,  1.23306763,  -0.157757416,  1.27621794,  0.389881432,  0.487719685,  0.925112009,  0.620942116,  1.26568127,  0.400293052,  -0.259438723,  -0.543002903,  -1.13026381,  0.323353201,  -0.932161391,  0.297038913,  0.142173752,  0.350243837,  -0.291589558,  0.852385223,  -0.646067798,  -1.48250771,  -0.641692996,  -0.716341555,  0.135125652,  -0.43210718,  -0.282655418,  -0.390382171,  -0.454808146,  -0.432863176,  1.49210572,  0.525216579,  -0.0593141839,  -0.0306813587,  -0.381336331,  -2.15383267,  2.88087153,  -0.422155678,  0.525600314,  -1.51780903,  -0.146271646,  -0.591634512,  -0.608353019,  0.821513176,  -0.30789426,  0.773496449,  -0.155456766},{ -0.833270192,  -0.66509825,  0.309401244,  -0.650382459,  0.101338945,  0.0595783032,  -0.190819561,  -0.0467826836,  0.326180875,  0.440554082,  0.292025089,  0.410698533,  -0.890681744,  -0.0452602841,  0.550433397,  -1.34671104,  -1.29019439,  -2.54326272,  0.72743094,  -0.113732487,  -0.650143623,  -0.18149741,  0.0859205946,  0.434693098,  1.59990323,  -0.962325752,  -0.62614131,  -0.0609967858,  0.250411808,  -0.0640292838,  0.58251065,  0.567027569,  2.39451313,  -0.68939805,  1.48023152,  0.253554195,  0.317197084,  -0.930435777,  -2.20366096,  0.434846073,  -1.75023592,  -0.493702143,  -0.308271855,  -0.232781306,  1.91160083,  -0.784456551,  0.697099209,  -0.462421209,  -2.21377969,  2.50185513,  -0.920152128,  0.364493638,  0.687606335,  -0.723422408,  1.69299841,  0.71577698,  1.06583762,  -0.248444572,  -0.371306419,  -0.0984633863,  0.742938221,  0.518976271,  0.095593892,  0.408659875,  1.16525912,  -0.755221069,  -0.46257475,  -0.219932467,  0.710706413,  1.12450802,  0.0446484424,  0.437800348,  -0.114328921,  -1.36410785,  -0.37361753,  0.346119434,  -0.181114495,  0.159532562},{ -0.1137539,  1.17723823,  0.0768014118,  0.66543746,  -0.776126087,  1.29880846,  -0.814751804,  0.30015716,  -1.5612843,  -0.553411305,  0.421797693,  -0.260976344,  0.385049999,  0.453840047,  0.315327913,  -1.61166263,  1.66111755,  0.924256861,  -1.22187614,  0.043277394,  -0.0883019343,  1.0422616,  0.0246861912,  -0.546073377,  0.403027564,  -0.476432681,  0.461097509,  -1.11746955,  1.38452995,  -0.884913266,  1.35417366,  -0.300963342,  -0.432842582,  0.237987503,  -0.311132431,  -1.17914224,  -0.348646075,  -0.918494761,  0.726632774,  1.19717848,  -0.940772474,  -0.93333447,  -0.608730078,  -1.54329276,  1.59617233,  -0.0504125878,  -1.35628521,  0.204428956,  -0.465804249,  0.613457382,  -0.754448712,  -0.924822092,  0.736767828,  -0.599208415,  -1.29818654,  0.800975561,  0.242671207,  0.917762578,  0.361391097,  -1.44777155,  -0.147966191,  0.694627881,  -1.03178477,  -1.04894328,  -1.868348,  1.23129439,  1.11286426,  1.58822763,  0.424347907,  0.267539978,  1.01325941,  0.437566072,  -0.519466877,  -0.574434817,  0.350346744,  -0.0492568836,  0.775173485,  0.115329541},{ 1.22742963,  -0.526121199,  0.161045298,  0.536321819,  -0.33908543,  -1.31303561,  -0.398124307,  -1.83945787,  -0.719111323,  0.253114998,  -1.15497422,  -0.458052099,  1.10048103,  -0.491476327,  -0.826147854,  -0.28608489,  -1.01720834,  -1.85902667,  -0.12371102,  0.924095273,  -2.08606911,  1.38171673,  -0.382194936,  -0.521396518,  -0.532811284,  -0.00112791546,  0.334673434,  -1.36827147,  -0.28564921,  1.61294937,  -0.609805167,  0.038685862,  -0.186964318,  -0.65105474,  0.741814137,  -0.0250302367,  0.338112444,  1.80255198,  -0.617277324,  -0.0184087884,  -0.736002445,  -2.6348989,  0.0572532006,  -0.221128508,  -0.143637985,  -1.32819963,  -0.402750164,  0.720531523,  -0.067783013,  -1.01402867,  0.504520237,  -0.738710225,  -1.53778648,  -0.370888531,  0.1848692,  -0.982121527,  0.929681122,  -0.334101319,  0.958793819,  -2.02569056,  -1.32505906,  0.740900218,  -1.00047028,  -0.56668222,  0.104876257,  -0.235853925,  0.284552872,  -0.300382644,  1.47387671,  0.667056024,  -0.56090492,  1.12421179,  -0.510297894,  0.808798134,  1.86293709,  1.14987624,  0.656648874,  -1.0707109},{ -1.12552536,  0.66946286,  -0.890812039,  -0.507418752,  -0.508038521,  1.44457531,  -0.812179804,  1.13475025,  3.20648026,  -0.512801349,  -0.341275126,  2.17954946,  -2.24928999,  -0.120710582,  1.23063493,  -0.3739416,  0.0347329415,  1.93155849,  2.69496083,  -0.426415592,  0.218649089,  -1.9439975,  -0.127411693,  0.135863081,  1.11866093,  1.44618154,  1.21852517,  -1.02636993,  0.0746553764,  -0.69940573,  -0.0519585274,  -0.328282684,  0.366559505,  -1.58329678,  0.49439168,  -0.25458312,  1.267555,  0.137750208,  1.57008862,  -0.524061501,  0.138534829,  1.1325438,  1.60131884,  -0.52939558,  -0.431026429,  -2.45119691,  -0.325984746,  2.16481876,  0.228641033,  1.21365047,  0.124896623,  1.11313343,  -0.517139494,  -1.20872951,  0.548333347,  2.04101801,  1.19085646,  1.54084218,  1.12099385,  -2.74764156,  0.0623262897,  0.649650455,  1.3093996,  0.994927585,  1.35737288,  0.921576142,  0.720204294,  0.351279676,  0.804698646,  -1.08348536,  -0.532074153,  0.3744587,  0.554560542,  0.840278625,  0.511423945,  -0.509200275,  -0.938914359,  0.730594814},{ 1.34126794,  -0.230002448,  0.475499243,  -0.733779132,  0.431553304,  -0.749852717,  -0.55007118,  -0.455952346,  -1.74118936,  -1.11214066,  -0.249973267,  1.57182288,  -0.323363841,  -1.44271791,  -0.0496560112,  -1.44500756,  0.440142691,  0.290582299,  -0.387052476,  -0.148852482,  -2.82337046,  0.205580384,  0.435529649,  0.227508634,  1.74537563,  0.0760412142,  -2.66116142,  1.48419023,  -1.12382579,  0.25025624,  -1.17086208,  -1.14480329,  1.09303963,  -0.0919506699,  1.22701144,  -0.492325127,  1.00046039,  1.47438741,  -1.04023528,  -0.874854386,  0.719824255,  -2.27559996,  0.210492998,  -0.662634552,  0.442235768,  0.264782369,  -1.71806037,  -1.48948061,  0.752689004,  -0.621813536,  -0.475317031,  -0.554002821,  -2.13374662,  1.34792089,  0.317508161,  1.54980648,  -0.937568367,  -1.0585798,  0.265899241,  -1.77412128,  1.12684202,  0.378634572,  0.698402047,  -0.65809691,  -0.811985612,  0.825552881,  0.778601825,  -1.22596145,  1.15972757,  -0.273117423,  -0.088905111,  -2.0749588,  2.41212535,  -0.190860718,  -0.196334004,  -1.14885056,  -0.675409973,  2.07877207},{ 1.32137167,  -0.118937336,  -0.0779108703,  0.894954503,  0.972248197,  0.0402876623,  -1.1686697,  0.0665335134,  -0.66088748,  -0.5454759,  -0.0459023528,  1.25580275,  1.59092867,  -1.78919899,  -1.42394006,  0.356290638,  -0.507882118,  0.11114043,  -3.01090074,  -1.77617013,  0.128878519,  -1.40179539,  -0.122356683,  -0.959403396,  -0.224467352,  -0.193419844,  -0.0228285529,  0.752969801,  0.670663059,  -0.512981355,  -1.06563187,  1.75580513,  -0.49188754,  -0.591618657,  -0.143381983,  0.329425126,  -0.504936814,  0.543591142,  -0.035399463,  -1.4021374,  0.0779981166,  -0.222348601,  1.22682643,  -0.412555009,  -1.17376256,  1.93064737,  1.19673002,  0.0602287315,  0.939367235,  0.0808043107,  -0.979889929,  0.745740473,  -0.192987978,  -1.56694293,  0.911314964,  1.16872084,  -1.24700975,  -1.6297133,  -1.12265646,  0.353602886,  -1.07723737,  -0.153085068,  -2.20513058,  -0.0163990539,  1.03729606,  0.640263796,  -0.673997164,  0.537652016,  -1.75743675,  1.07583022,  -0.245640934,  -0.433272302,  0.687290251,  0.237793699,  1.01083338,  -0.419204623,  -0.597464204,  0.333362609},{ -1.35823131,  1.13925171,  1.76301253,  0.2096937,  0.684166908,  1.50391877,  0.937412739,  -1.42866623,  0.661922455,  0.620975018,  1.64415836,  1.75805104,  -0.52352798,  0.0376678556,  1.40047765,  0.480938286,  2.13049388,  -0.541474223,  0.0311978254,  -1.74541676,  0.838479221,  -0.572072506,  0.410650402,  1.39523053,  -0.264884144,  -1.30999362,  -0.206013843,  -0.572363615,  -0.678670406,  -1.63704526,  -0.660943627,  1.61138439,  1.48759317,  0.4210473,  0.841139019,  -0.436743498,  0.592048466,  -0.678067565,  -0.933648944,  -0.784176588,  -0.890906274,  0.3530595,  1.32422531,  -2.16301346,  -0.351239413,  -0.158235639,  0.0219737161,  -1.51870382,  2.16425037,  -0.194217071,  -0.241940036,  -1.04815948,  -2.32491779,  -0.413582981,  0.332071781,  -2.12683916,  -1.25537515,  -0.350212872,  0.575157404,  1.96055996,  1.38405406,  -0.011504842,  -1.58513772,  -1.1429553,  -0.396857172,  0.907284379,  0.0185780544,  0.273632348,  -0.206495211,  -1.19424546,  -0.138907567,  0.017211996,  -0.20200789,  -1.02585638,  0.0734397918,  -0.420577914,  1.20731592,  0.276372433},{ 0.532132387,  0.629355788,  -1.08274388,  -0.623546422,  0.748275161,  0.248305127,  0.597362697,  -0.0274387468,  0.109855406,  -0.231058389,  -1.40418947,  0.404133379,  0.600539565,  0.157106787,  1.4036355,  1.18577135,  -1.34563076,  -0.629477203,  -0.0486283116,  1.03065038,  0.582632244,  1.45733786,  -0.559331536,  0.724788547,  -0.367128104,  -1.05205381,  -1.8001864,  0.404750615,  1.00167179,  -0.896986306,  -0.248094723,  -1.03826487,  -0.913470626,  -0.210003197,  0.298273027,  0.565839946,  -0.552081525,  0.0200119354,  1.0171535,  0.800528228,  0.775859118,  -0.657263637,  -0.464960545,  -1.07766128,  -1.37478709,  0.0679795817,  0.386731476,  3.08729243,  1.44431543,  1.20838583,  -2.41007376,  -0.0802634954,  0.44475913,  -0.854742944,  0.308559239,  -0.2506046,  -1.03824615,  2.18954539,  -0.389518529,  -0.0574640743,  0.843134761,  -1.00409985,  -1.67699063,  1.27354109,  0.378364921,  -1.59106588,  -1.02155447,  -0.217058524,  -1.83698142,  -0.832670927,  -1.00021207,  -1.03990293,  -1.07727551,  -0.58106041,  0.284612477,  0.592248619,  -0.707184374,  0.912566304},{ 1.2775141,  -1.33088374,  0.38197571,  -0.205388874,  0.30628562,  -0.102650352,  0.0715259388,  -0.720073044,  -0.749460995,  0.25906679,  0.852595687,  0.471729547,  0.0543772466,  0.329679579,  -0.350968331,  -0.504089117,  0.183467016,  0.561558068,  -0.55640614,  -0.829708338,  0.320481956,  1.49297774,  0.988381922,  0.0734629706,  0.48547405,  -0.815422595,  -0.185974851,  -0.782621443,  -0.786649466,  -0.711447001,  -0.855466247,  -2.20935869,  -0.0094391536,  1.25083828,  -0.588391781,  -0.396504581,  -0.922663271,  1.50898492,  0.694998801,  1.73422861,  0.125823826,  1.02599025,  -0.543566883,  0.00310196588,  -1.37631214,  1.48006988,  -0.26590082,  0.769824147,  1.45009112,  0.43558082,  -0.600034893,  -0.737454176,  0.695079088,  1.26288497,  -0.788321495,  0.122599013,  -2.76334953,  -0.682472229,  -0.668978155,  1.61395383,  -2.14746976,  1.17459738,  -2.26400256,  1.10562003,  -0.810218453,  1.23255205,  0.343166322,  0.103065118,  -1.43234563,  -0.949755967,  0.846377373,  0.132452324,  -0.656081319,  2.99652958,  -2.14492464,  -1.54460251,  0.509582341,  -0.478485048},{ -0.473173529,  -0.462141842,  -1.08603704,  1.00839567,  0.00265934458,  1.44082534,  0.188382655,  -0.342669308,  0.959834039,  -2.60646057,  0.0234620608,  -1.22722602,  -0.327557355,  -1.81290185,  -1.62604856,  0.924083054,  -1.5395937,  -0.326364577,  0.695359409,  -0.87751925,  0.153085709,  0.68894738,  3.00099015,  -0.0186823774,  -0.399775237,  -0.193577096,  -0.07297346,  0.28643328,  -0.491223127,  1.76436651,  0.875982285,  -1.51025212,  -0.141803354,  0.0405240059,  0.204766348,  -0.433865994,  0.686877608,  0.18131943,  1.43382347,  2.05193806,  -0.0243015476,  -0.323565543,  -0.294739008,  -0.558981597,  0.404701442,  -0.194131792,  -1.66564047,  -0.0244455263,  -0.77242732,  -1.04005361,  -0.59589082,  0.00752735883,  0.641966283,  1.71279645,  1.71977794,  1.18840897,  -1.16066194,  1.18810248,  0.46733278,  -0.122624375,  -0.734852612,  0.70303601,  0.882740974,  0.744384527,  0.12158253,  -0.274878532,  -1.36119819,  -0.536203861,  2.17903328,  0.662428975,  0.326009691,  -0.842413723,  -1.16915941,  0.842016101,  0.0589140095,  0.281636357,  1.16425669,  -1.13076293},{ 0.533793867,  -0.481506795,  0.621625125,  0.844448924,  -1.73296607,  -1.26492834,  0.436445087,  -0.308789313,  0.403434247,  -2.28401852,  0.693123758,  -0.0606373399,  -1.31145811,  0.98811698,  -0.45717141,  -1.11781955,  -0.708176851,  -2.27961135,  -0.273284018,  -0.814421415,  -0.393696666,  0.0470945574,  -0.483814538,  -1.83029974,  0.227538407,  1.1696583,  1.09174407,  -1.29017603,  0.706785381,  -1.39918399,  -2.14638114,  -0.100388616,  0.342689127,  -0.164924294,  1.04119515,  1.40342367,  1.89200127,  1.11978233,  0.0484208837,  -0.474851072,  -1.39992106,  0.561151445,  1.44848609,  -0.0992082357,  -1.16883111,  0.780885458,  -0.81217736,  1.33868313,  0.590302765,  -0.785710335,  -0.806557655,  -1.74416256,  0.864166915,  -0.21017012,  -0.127632216,  -0.127703935,  -0.169760242,  1.04970205,  1.91037011,  1.75921893,  -1.49675274,  -0.951355875,  -0.066585511,  -0.0221919697,  -0.504368722,  -0.758028507,  -0.355392277,  -1.20100749,  0.882009208,  -1.56424797,  0.176863879,  -0.766629696,  1.02198565,  0.403116018,  -0.880791306,  0.0707056224,  1.76651669,  -1.6620543},{ 0.880700707,  -0.187701374,  0.277504355,  -1.80523813,  -0.921736538,  -1.40310645,  0.386826515,  1.13892114,  0.501173258,  -0.61308676,  0.152689472,  1.51106918,  0.139149487,  -1.18895829,  0.498969048,  0.787534058,  -0.644618809,  -2.32358479,  -0.847523868,  0.219737336,  0.204325169,  0.529053271,  0.968453526,  -1.71636558,  0.101236053,  1.34359455,  0.256974369,  0.410054028,  -0.535537183,  2.81199002,  1.15975904,  0.933750033,  -0.914159596,  0.335995734,  1.00833511,  -0.267557919,  -1.08844018,  0.252797425,  -1.53011906,  0.0258726813,  -1.01253974,  -0.747513294,  -1.12351739,  -0.0501696728,  -0.773795843,  -0.117717169,  -1.36923516,  0.997725368,  -0.767043889,  0.0530648902,  -0.471550286,  -0.10254205,  -1.02880359,  -0.114760391,  0.258746654,  -1.27942085,  -0.010651662,  2.50503707,  -1.78041947,  -0.134168789,  -0.985806465,  1.85229445,  0.122829013,  0.343921453,  1.32370758,  -1.80012262,  -2.11757183,  -1.02637076,  0.429628819,  -1.0278554,  -0.764710486,  0.635561466,  -0.566182673,  0.531460226,  -0.350167304,  -0.735269785,  -0.121535249,  -1.34870279},{ 0.846349478,  -0.0076038409,  -0.262119085,  -1.0616914,  2.25479412,  -1.35431457,  1.41894519,  -0.595362961,  0.238217026,  -0.320515215,  1.62152159,  -0.902194202,  -0.376806647,  0.890205503,  -0.946727872,  -1.73179603,  1.21316206,  -0.130143434,  0.240821525,  -0.368241727,  0.976316512,  1.35077822,  -1.24060392,  1.74696386,  -0.706707954,  0.176481888,  -0.578831792,  -0.564803362,  -1.45124924,  0.00580410659,  1.08306873,  -1.38857698,  0.361845821,  0.309913367,  -1.8500669,  -0.937206745,  -0.2942065,  -0.604237318,  -0.0310843196,  0.151961356,  -0.805168509,  -0.443058103,  0.243305013,  -0.371195734,  0.227634043,  -0.704528511,  0.429249316,  1.77958882,  0.209587991,  -1.89754641,  -1.45816708,  -0.212358519,  -0.592350006,  0.882097065,  -1.57324839,  0.00998659246,  -1.96090305,  -0.553204834,  -1.54036307,  1.56171691,  0.632192314,  -0.396536797,  0.533228695,  0.810845673,  -0.81581068,  0.590777516,  1.05521059,  0.132376388,  -0.673328936,  0.1063869,  0.725226581,  -0.215632111,  0.770879269,  0.362113237,  0.495771646,  0.203990877,  0.218287244,  0.652846098},{ -0.469037175,  -0.569045186,  -2.3231976,  -1.38098681,  0.342981607,  1.30301225,  0.658906996,  -1.49987853,  -0.0684252381,  -1.34213233,  1.73977923,  0.209944427,  0.174126327,  2.8407042,  0.68475008,  2.30141854,  -0.297847748,  -0.380810201,  -2.21386313,  1.5054121,  0.308240622,  -0.0634661093,  -0.954534292,  -1.61937284,  -0.851143062,  -0.557129622,  0.104708157,  0.806090355,  -0.976705134,  0.224462479,  -0.498407215,  -0.347328544,  -1.20949638,  -0.379042894,  0.624279618,  -1.94347501,  0.737989187,  -0.313186616,  -0.0649848133,  1.14038563,  0.305046231,  -0.950372398,  -1.37262726,  0.66981107,  -0.521851242,  0.163735688,  1.11672926,  -0.803926587,  -0.922004282,  -1.30251908,  0.628379464,  -1.79077792,  0.157203004,  0.345090985,  0.827795804,  -1.10777533,  -0.515112817,  0.558523655,  -0.470685601,  -1.70623243,  -0.398827374,  0.1288151,  1.56199121,  -0.471654445,  0.634959638,  0.29274419,  -0.205231965,  2.35495114,  0.29031679,  1.35925746,  -0.227709323,  0.235399514,  0.970368862,  0.803069115,  0.397234112,  -0.471975356,  -1.01929951,  0.382237256},{ 0.0435721129,  0.118868887,  1.77912366,  -0.836417735,  0.467317343,  1.4632678,  0.959158123,  0.351177424,  0.57085067,  -0.00537955575,  0.109835051,  -0.753771544,  -2.0436461,  -0.244946018,  0.86900723,  0.608260155,  -2.29795122,  -1.1862371,  0.457424909,  -0.689232945,  0.205559403,  -1.59877574,  -3.34143162,  1.09253573,  -0.161429584,  1.64084852,  -2.05395865,  -0.813645601,  0.236555517,  0.680011928,  0.158104196,  -0.0186133254,  0.408986807,  0.18893145,  -0.624761224,  -0.478207767,  -0.473313421,  0.839614272,  0.508751869,  1.38326406,  -2.47508335,  0.0488047525,  0.370947599,  0.868889034,  -0.208388627,  1.19528186,  -0.696043968,  0.796571434,  0.178717449,  0.42276147,  -0.0946305767,  0.742495894,  1.73136199,  -1.25039911,  0.402931929,  0.154571265,  -2.25865364,  -0.592650175,  -0.523233235,  -0.222480372,  1.05987489,  -0.136359245,  -1.23175025,  1.52695405,  -1.06723976,  0.277455002,  -2.41834998,  1.10000753,  -0.347871602,  -0.347735912,  1.99932277,  0.489465743,  -2.03811908,  0.501333773,  -1.13971055,  -0.429101795,  0.113627173,  0.885152102},{ 0.246026024,  -0.978807509,  1.46137488,  0.337225914,  1.01412272,  0.203715131,  0.521448195,  -1.67631781,  0.238161072,  -1.0842545,  -0.393500298,  -0.837493658,  -0.827058375,  1.20508921,  0.874221563,  -1.17743754,  -0.597425044,  -0.66493994,  0.883559108,  -0.593768716,  0.0719769746,  0.921403646,  1.57902956,  -0.330413759,  0.449801087,  -0.120672926,  -0.208552375,  0.408809304,  0.0200785007,  1.61678493,  0.877322018,  0.901102543,  2.06553841,  -1.69388914,  -0.811910629,  -1.85413969,  -0.0366336331,  -0.358682245,  -1.13334858,  -0.442528754,  -1.08186138,  0.16082491,  1.16457844,  0.0487447828,  0.645483077,  1.90278232,  0.475529224,  -0.545078576,  0.305548191,  0.364995152,  -0.85552454,  2.96195579,  -1.51073802,  0.668658912,  1.59586871,  2.72620559,  -0.322185844,  -0.49993512,  0.156974465,  -2.50337362,  0.443711609,  0.0339421965,  -0.798216939,  1.52067828,  -1.9415127,  0.545526326,  0.310926646,  0.951233029,  -1.95715904,  -0.70110774,  -0.873859167,  -1.43324065,  -0.964404523,  1.77535713,  0.28420648,  -0.368388236,  0.266059548,  -0.0246721022},{ 1.02356291,  0.161913097,  -0.286500722,  0.555640936,  -1.26295257,  0.717922449,  -0.28175202,  0.780228496,  -0.362428725,  -0.0138608301,  0.0266139824,  1.05889738,  -0.321754038,  -1.76973176,  1.85823452,  0.372617275,  0.698760152,  2.34503198,  1.85227215,  -0.839690149,  0.500061989,  1.18083394,  -2.16769314,  -0.846380353,  -0.617194951,  0.039316155,  -0.723510981,  -1.73955381,  -0.443542063,  0.480167359,  0.202337235,  0.661567926,  0.342790753,  0.167430565,  0.462887764,  -0.664096773,  0.448408246,  0.14567396,  1.75570154,  -1.00361741,  0.625639915,  -0.656461477,  -0.334777445,  0.373253107,  -0.0609848313,  0.110406391,  0.527547002,  1.60271871,  -0.189647719,  -0.656806707,  -0.46014604,  -1.71290767,  -1.38117802,  1.62970114,  -2.09843612,  0.544575214,  0.174861163,  0.683651686,  -0.365528852,  0.173683941,  0.00180706766,  -0.302780449,  1.15495622,  0.369722962,  0.200231075,  -0.814870656,  0.196532816,  -0.645154297,  -1.5358392,  0.230821237,  -1.2776742,  0.476755589,  -0.702248454,  -2.47747016,  0.889797866,  1.40794146,  0.0789514408,  -0.579108059},{ -1.27670324,  -0.365810335,  1.24760413,  1.25533581,  0.713150501,  -0.337612182,  -0.671738684,  1.41576815,  0.719475985,  2.11362696,  -0.568538189,  0.441946208,  -0.694380939,  -0.43182072,  0.638604522,  -1.56415355,  -1.99589109,  -0.786341369,  -0.397442997,  -0.432145387,  -1.80605483,  0.930276752,  0.131268278,  -0.263266593,  0.135168895,  -0.619551897,  0.750692368,  1.97285724,  -0.574931264,  0.363739938,  -0.0867565498,  -0.249897882,  0.0482269675,  -0.192917556,  -1.29648662,  -0.149548441,  0.781082809,  0.519401252,  1.45766532,  -0.463739097,  -0.407669455,  -1.64611626,  0.470273584,  1.13477683,  -2.94146132,  1.37665021,  0.401238292,  1.43420935,  0.0217220746,  -0.201913998,  -2.45967054,  -2.11058927,  1.63296413,  -1.22582996,  -0.889319897,  -0.0707114562,  -0.727323472,  -1.34717715,  -0.55699718,  0.27011919,  -0.910888612,  -0.0942678377,  1.43065774,  0.00372930965,  1.79845107,  0.919238031,  -1.34644568,  0.573382914,  0.552316964,  0.575405955,  -0.131774172,  0.316207021,  -0.854536712,  2.02933311,  1.08413506,  0.351138145,  1.06050658,  0.96375972},{ -0.272652,  -0.825649679,  0.891374826,  -2.49608064,  1.00864255,  -0.982464731,  0.645361245,  0.436931282,  -0.143898681,  -0.700695872,  1.18782675,  -0.227435082,  -1.8571887,  -0.137355879,  0.291081697,  -0.0863847286,  1.58691466,  0.87816608,  0.847872972,  -0.165978312,  -0.147794783,  1.14532745,  -1.65383542,  0.123614796,  0.666647792,  0.302536398,  0.871622622,  0.893239915,  -0.180622235,  -0.135597318,  0.455999553,  -0.420410216,  1.60190821,  -0.18718791,  1.06668341,  0.411571532,  1.00944281,  -1.31374431,  -2.13430023,  -1.02639711,  -2.14579082,  1.17971325,  -1.53400779,  0.173867702,  -2.46800423,  -1.24445522,  0.525079906,  0.95034337,  0.0883499607,  -0.814552486,  0.00760761043,  0.613842547,  0.772193849,  0.0290363282,  -1.49374759,  0.29037261,  1.41821682,  0.0151563995,  -0.179506689,  0.19182089,  -0.388038188,  -0.515700698,  0.586281717,  -1.64252329,  -0.16526933,  0.22662279,  0.627189696,  0.462056488,  -0.548559487,  -0.907976508,  0.0868147537,  -0.090999186,  -0.143002853,  0.299635172,  0.4908683,  -0.0208830703,  -1.12242413,  -0.0941896588},{ 2.18334103,  0.380724818,  -0.369121373,  -0.132124737,  -1.56322718,  -0.573301136,  1.40946925,  -0.0775422156,  1.54726374,  0.459035456,  -1.10972035,  0.62951225,  0.610404968,  1.12309265,  1.23735774,  0.0648740903,  0.685558379,  0.433240622,  -1.2153635,  -2.75142384,  1.2644341,  0.422197402,  -0.818382919,  0.617363095,  1.12795758,  -0.202350602,  0.477960557,  0.433640361,  -2.28951955,  -0.0667181611,  -2.2785759,  0.261132687,  -0.120590247,  -2.35633683,  0.21639064,  0.760247946,  -0.11020945,  0.879541218,  1.35727429,  0.229724735,  1.44738948,  0.701347411,  1.6945802,  -0.639834166,  0.284624398,  -0.226110414,  0.890089035,  -0.396638036,  1.86939573,  0.517829776,  0.973918855,  0.615216672,  -0.159940943,  -0.691414535,  0.61934787,  -0.306476831,  -0.333581746,  2.98524165,  2.10051942,  0.00196487457,  -1.01278472,  -3.06781387,  -0.360874891,  -0.066799663,  1.44126475,  -0.209293857,  -1.93176126,  0.76300925,  -1.42198598,  -0.586164713,  -0.511972725,  0.597867906,  1.8635546,  1.28710437,  -0.880575716,  1.31826651,  0.221871704,  -1.05720222},{ 0.149842903,  0.407816619,  -1.01202321,  0.766333759,  -2.00218606,  -0.988315821,  0.622900546,  -1.23098934,  0.925286293,  2.25081944,  0.831498742,  -0.185584456,  -1.70908415,  -2.50093579,  1.05303371,  0.929598987,  0.620036662,  1.04666078,  0.62907517,  0.0666603819,  0.946887374,  -1.52514172,  1.81141198,  -1.12181866,  0.999942124,  -0.99302429,  0.0169653073,  -0.191095129,  1.6967628,  -1.3782475,  -0.739022076,  1.93937075,  1.97689855,  0.332214504,  0.717958152,  0.581077158,  0.874350607,  0.0670566037,  1.44121361,  2.73425126,  0.239136726,  -0.301043391,  1.31349409,  1.22285199,  -0.650918126,  0.318744004,  0.657154143,  1.68536091,  0.256356388,  1.82553911,  -0.183164164,  0.578534842,  0.425185651,  2.63243771,  -0.637220263,  -0.741838038,  1.88941848,  1.52683473,  -1.04188251,  -0.430822134,  0.93094939,  -0.928423226,  -1.90258718,  -0.643720746,  1.30915523,  2.46687579,  1.01158154,  -0.74614203,  0.182134509,  2.13271904,  0.473322153,  2.25466037,  -1.5448482,  -0.304983228,  -0.891224325,  0.514176071,  0.631936073,  1.02933967},{ -1.49544585,  -0.0281782951,  -2.08302546,  -0.650465727,  -0.788696051,  -1.38094401,  -0.0196081046,  0.336165696,  0.402591258,  -0.494646579,  -0.899499834,  -1.61063588,  -0.0764508843,  1.07069325,  1.278754,  0.607912481,  -1.23079228,  -1.1195749,  -1.25131798,  0.235515624,  -0.292398244,  0.264177978,  -1.10372829,  0.972916901,  -0.391794562,  0.0602723695,  -0.169138566,  -1.33818054,  1.11343718,  -1.62816036,  0.6493541,  -1.35303354,  0.341734022,  0.301639557,  -1.65382922,  -1.33881843,  0.578443527,  -2.21624923,  -0.282773107,  -1.26263571,  -0.688378334,  0.100108758,  -0.569371283,  -0.39160338,  0.729431868,  -0.267152905,  1.21143603,  2.21111298,  -1.34902835,  -1.48384547,  -1.34928775,  -0.084735848,  0.797853351,  0.269943833,  0.0835163221,  0.752857924,  -0.650957167,  -0.916266441,  0.101793632,  1.07710409,  0.36357224,  0.685190737,  -0.356583148,  -1.37049079,  -0.771097064,  0.264193684,  0.52278322,  -1.64684105,  -0.165931806,  0.348201692,  -0.108669937,  -0.509875119,  -0.714003682,  0.598181665,  -1.38467741,  -0.149421111,  -0.286830246,  -0.0857580155},{ -0.192112312,  -0.0701397434,  2.30134034,  0.413887233,  -1.77792251,  0.821342885,  0.667289317,  -0.0312069096,  0.170524493,  1.11678183,  0.143011928,  -0.385682881,  1.00505972,  0.636654496,  0.317680568,  -0.309031337,  0.0482212827,  -0.695420325,  0.133270964,  1.54357171,  0.592275202,  0.972652733,  -0.494820774,  0.644128203,  -0.550126851,  -1.91047788,  0.334640861,  1.31526291,  -0.962676704,  -0.665168583,  0.700576842,  -0.111206807,  -0.328775972,  1.96091712,  0.336313516,  -1.20486426,  1.57172251,  1.75114918,  2.12369084,  -0.211843595,  2.12318254,  -1.14725363,  -1.47339487,  0.735019028,  -1.48485434,  0.419511318,  -0.0874952674,  -0.764922738,  -2.24764442,  1.76176727,  -0.637414217,  0.0842622295,  -0.586525738,  -1.15869951,  -0.203705832,  -1.10771728,  0.412284076,  0.340471327,  0.335224897,  1.62494826,  1.40196955,  1.31381023,  -1.04414666,  -0.566410661,  -1.73359919,  0.581656694,  -1.60831487,  0.844367027,  0.268042147,  -1.00862682,  0.090799205,  0.284046531,  -0.210632712,  -1.29945827,  1.35119081,  1.15060461,  0.203749776,  -2.31941605},{ -0.735795081,  -0.744559884,  -0.226377726,  -0.28596881,  1.58712375,  -0.598643899,  0.327609241,  0.0773837045,  -0.253326952,  1.85005713,  -1.07865739,  -1.25113821,  -1.59365845,  0.127325207,  -0.88866204,  -0.295716405,  -0.0814766213,  1.49433076,  0.789823174,  0.286906421,  0.978605807,  1.09032774,  0.0859126151,  1.69818747,  -0.640972912,  -1.48879123,  -0.245178148,  -1.4313246,  0.785042107,  -0.952481687,  -0.279205173,  -0.797201633,  -0.916277409,  0.871226311,  1.6945889,  0.257793784,  0.0684022307,  -0.131787375,  1.02155828,  0.434565067,  -0.398773998,  0.00228725467,  0.0345214047,  0.455214262,  -1.76847315,  -0.955582142,  0.120565571,  0.36252898,  -1.50248957,  -1.22918701,  -2.29059553,  1.43895435,  0.883299649,  0.429072052,  0.906330228,  0.484577566,  2.10877824,  -0.411742836,  2.1105566,  1.09702408,  0.977794051,  -1.31478894,  -2.18826127,  -2.0865202,  -0.252411246,  0.0829288438,  0.0452109389,  1.11791372,  0.455382705,  -1.40641928,  -0.867687404,  -0.0575280264,  1.91184473,  -0.306596249,  -0.247206554,  0.798459351,  -1.15086329,  -0.526723266},{ -1.03075457,  1.65990555,  0.872129798,  0.681244254,  1.16254127,  0.856766522,  -0.884294152,  -1.37112284,  -0.354360938,  1.85670578,  0.570716619,  -0.954055309,  0.305133224,  1.27644205,  1.00894499,  1.29627538,  -0.505379915,  0.435800165,  0.0441146158,  1.22590017,  -1.13911998,  -0.381636947,  -0.621000886,  -0.526834309,  -0.121855401,  0.869377673,  -0.817282021,  -0.059611544,  -1.40399897,  -0.657173514,  0.861319304,  2.05780578,  0.694191992,  0.0368108898,  1.04374659,  -0.885705829,  -0.646100581,  -0.193216071,  -1.24294639,  1.497774,  0.317476481,  0.0077191852,  0.683041275,  0.783448219,  0.42753759,  0.438487887,  -0.474282891,  0.955519617,  0.801919937,  0.0334943347,  -0.887933135,  -1.33942842,  -0.00612034742,  0.0599121116,  -0.163362369,  -0.994759858,  0.511171639,  0.0500961393,  0.81834954,  -1.19746304,  0.730373681,  -0.158994973,  0.544534922,  0.0665305108,  0.607191622,  0.208668828,  0.327018738,  2.21313167,  0.0139493123,  0.737064838,  1.47593796,  0.546213448,  -1.08433533,  -1.46470296,  -0.733368754,  1.2766217,  -1.2096324,  -1.66554022},{ 1.15884829,  -0.495968819,  1.1747489,  -0.702186346,  1.16928649,  -0.0169279985,  -2.15867949,  -0.389864802,  -0.537460506,  -0.661726594,  -1.42773914,  0.73645705,  -1.6099571,  0.524039268,  0.901796818,  0.518586218,  -1.30112731,  1.10403883,  1.1187644,  1.10336065,  0.647227108,  -1.03127944,  0.110466488,  -0.209693551,  -1.96173346,  -0.0276674386,  1.24857903,  -0.77483052,  0.542114556,  -0.617753088,  0.0861682296,  -0.72425431,  -1.04250646,  0.264214039,  0.218923166,  -0.625025213,  0.241929978,  1.09176397,  -0.229296386,  -0.351285815,  1.1717099,  -1.15790606,  -2.0895822,  -1.85771525,  0.525519729,  2.3629775,  0.166561365,  0.917537868,  0.870738983,  0.010373001,  0.30823046,  3.02015138,  -1.04531848,  1.4471364,  -0.348621756,  0.585705101,  -0.829432964,  -1.81108487,  0.24107857,  1.55016577,  0.983632207,  -0.0210638978,  0.741496921,  -0.0628192648,  1.08322346,  0.506473005,  -1.01532352,  0.389613181,  0.644307733,  3.03638864,  -0.344955236,  0.782672524,  -0.0332731865,  0.080739744,  0.621548831,  0.591149628,  0.228531867,  1.56747723},{ -0.0875044093,  2.06603169,  -0.850580156,  -1.65945089,  -1.77293408,  -0.307590008,  0.329339296,  0.232777134,  0.612945378,  0.255035579,  0.0909034684,  -0.61265713,  0.77639842,  -1.31480336,  0.0684549212,  0.607007742,  -1.00064516,  0.295872718,  -0.686304986,  -0.217433259,  0.110158287,  0.0806726515,  1.98620892,  0.630958974,  -0.762976766,  1.37108874,  1.07726777,  1.23381233,  -0.576899886,  0.66441375,  -0.573706746,  -0.586390972,  1.62611461,  -0.553341329,  0.907627463,  0.350616425,  -0.896067917,  -0.0929280296,  -0.289580733,  0.73396188,  -0.928423405,  -0.445439756,  0.595365584,  -1.09253848,  0.674895048,  -0.435491383,  -2.0681293,  0.213965684,  1.09831965,  -1.08057308,  -0.857951462,  -0.123438783,  -0.234562233,  -1.0451988,  1.41902649,  -0.12431328,  -1.50853848,  0.653226197,  -0.577975333,  0.19826898,  -0.956523538,  -0.453808457,  0.135914177,  0.26860854,  0.761269629,  -1.18381143,  -0.270426631,  -0.582271278,  1.51865733,  -1.0331738,  -0.0760750771,  1.86251938,  -0.251411438,  -0.504730463,  -0.396567583,  0.802933097,  0.401700377,  0.437108845},{ -1.43553567,  0.482317626,  1.81843948,  0.0513462648,  -1.09183908,  -0.275656044,  -0.147641793,  -0.146627977,  -1.35728371,  -0.464183241,  0.440996677,  0.592842698,  0.0150975557,  -0.16189456,  -0.703584969,  -1.81208801,  -1.63864553,  -0.49615407,  -0.692953706,  -0.196099997,  -0.60027045,  1.51498234,  0.209068,  0.718848228,  0.42489776,  0.166256115,  -1.08454156,  -0.6527915,  0.592112601,  -1.97491026,  -0.161786437,  -1.07436752,  0.789330781,  1.64168298,  0.0761914924,  -0.626140356,  -1.31075239,  1.62404144,  1.44184268,  -0.532707036,  -1.21159065,  1.98275602,  -0.920327902,  -0.0784313977,  -0.758870304,  0.721380532,  0.549813509,  0.418766707,  0.212585196,  -0.302750707,  -0.548875213,  1.18041575,  0.201893389,  -0.191024646,  0.190013781,  -0.481898189,  1.30980229,  -0.2471122,  0.337118626,  -1.5506109,  0.281678557,  -1.25206029,  -1.38236296,  1.40593135,  -1.56364822,  -1.1033951,  -1.09829998,  -1.93630695,  -0.638877511,  1.90837419,  -1.20058072,  -0.362099946,  -0.59820956,  0.111915387,  -0.501670241,  -1.21383607,  -0.295692801,  0.724871635},{ 0.676099062,  0.903740108,  0.0872133225,  0.0340682343,  0.944518447,  0.812326372,  -0.400166392,  -1.1760031,  -0.51067239,  -1.60579669,  0.855540037,  0.268855453,  -1.47663879,  -0.164746657,  -0.361345857,  -0.0688500553,  -0.829298139,  1.19572484,  -0.206260517,  -0.961134255,  0.43041721,  0.743881047,  -0.0430974588,  -0.589781463,  1.20888662,  -0.333261013,  2.14247799,  -1.94975793,  1.02040589,  0.202546969,  0.609579861,  1.13777006,  -1.94838333,  -0.530529678,  -1.22753465,  -2.65735674,  0.498476326,  0.948600709,  1.17098391,  -0.27588582,  1.91450667,  0.467566818,  0.324152172,  0.658194244,  1.42195797,  -0.0384304561,  -0.531031609,  -0.461234152,  0.668275177,  0.572359622,  -1.78019345,  2.00370622,  -0.552548945,  0.116776451,  -0.318052262,  -1.38275957,  0.256478786,  0.297809809,  0.376487583,  1.53253222,  0.707917392,  -0.304997742,  -0.341454566,  1.26972139,  -0.681036174,  0.959991992,  0.556457996,  -0.982419908,  -0.370241523,  1.7846477,  0.296896189,  -0.644740939,  0.0935576335,  0.421576113,  0.323312193,  2.63312626,  -1.91007984,  -1.09098959},{ -2.50334311,  -1.5496459,  -1.50189281,  -0.552947521,  1.53589022,  0.324725419,  0.144184172,  -1.14492536,  -2.61618829,  -0.0542776734,  -0.41122359,  0.713849068,  0.434294045,  -0.729647696,  -0.21737045,  -0.329121113,  1.30283713,  0.594654322,  -2.45796919,  -0.360415995,  -1.41208684,  0.0387842283,  -0.0797356144,  0.00527288718,  -0.964128971,  1.30734611,  -0.419787407,  -0.699884057,  -1.61199963,  1.34570014,  0.628275692,  -0.178126678,  0.596749842,  0.575615466,  0.334952712,  -1.11970758,  -0.318593442,  -0.881809294,  1.45327938,  -0.885932446,  -0.51019311,  0.6700477,  0.901227355,  -0.296875328,  1.24025154,  -0.451052368,  0.210941032,  0.327692419,  -0.498384207,  1.643453,  0.113324985,  -0.927484274,  -0.196005151,  -0.804003358,  0.00852163602,  -1.54009831,  -0.0867332444,  0.697301388,  -1.34627056,  -0.354845524,  -0.926755905,  1.50275588,  0.800472975,  0.477951497,  -0.737295449,  -0.0735843256,  -0.852835,  0.381389409,  -1.30673563,  -0.614033163,  -0.980739057,  0.335932881,  -1.8329432,  -1.17491317,  -0.616670072,  0.84567523,  0.231274009,  0.667780757},{ 0.28488186,  -0.10492193,  -0.0156763196,  -0.713593066,  0.240339518,  0.790531695,  0.632508457,  1.40457666,  1.00032985,  -1.26053679,  0.0688436925,  0.539700687,  -0.679474473,  1.14390564,  -0.532378793,  1.85792136,  -0.790206611,  -1.00964749,  -1.181427,  0.0811208934,  -1.66581523,  0.326017439,  -0.0211848263,  -1.76677036,  0.927291632,  0.113416657,  -0.306970626,  -0.417233527,  2.10943151,  -0.742956519,  -0.203315914,  0.0670026541,  -1.39257026,  -0.685704172,  -0.068060033,  -0.322505802,  0.223386958,  1.02109754,  -0.38748312,  -1.00206339,  -1.87997425,  -1.60576916,  -0.257466465,  -0.635904193,  0.465808272,  0.419002771,  0.274051458,  0.0622947551,  0.266938299,  -2.62558866,  -0.462004244,  -0.616643131,  -2.52784276,  0.69838208,  0.259309053,  0.414499998,  -0.17076844,  0.980553925,  0.00375194242,  0.0455724411,  0.0746288896,  -0.0885177925,  -0.723729372,  0.494090617,  -0.895734668,  -1.09985113,  -0.408875912,  -1.52286124,  -0.778867602,  0.438131601,  1.01172876,  0.597660244,  -0.0337947831,  -0.850726724,  2.48199534,  -0.454139501,  -0.587418556,  -0.923475146},{ -0.583715081,  0.619754672,  0.733203769,  -0.249685928,  -1.17380917,  -0.285545439,  -0.449469656,  -0.418369889,  -0.776103675,  -0.144085467,  0.808301449,  1.15274429,  1.01596665,  0.170333877,  1.83202553,  0.046510715,  0.401879936,  -3.36302066,  0.769422829,  0.586412132,  -0.257926702,  0.0873705521,  0.869521558,  -0.191174492,  0.36995247,  -0.333038062,  -0.182996333,  -2.33377099,  -0.458715141,  -2.31872821,  0.10469541,  -1.32815266,  1.59802949,  -0.848764956,  -0.969036639,  2.0237875,  -1.08282948,  -1.31514192,  2.5573895,  -2.09559274,  -0.616753161,  0.77492106,  1.11519349,  -0.775374532,  2.27837563,  -1.11741161,  0.436507046,  -0.616898596,  -0.185969368,  0.476829797,  -0.955231726,  -0.197005391,  1.53539443,  0.611886859,  -0.750891149,  0.0353146344,  1.3244592,  -0.823115587,  -0.259512454,  -0.246007442,  -0.159068614,  -0.304124504,  -0.119196773,  0.232327536,  0.672395349,  -0.580535471,  0.00909120776,  -0.0377016813,  0.342546791,  0.254251331,  -1.42047799,  -1.4312396,  -0.37239489,  1.53098965,  -0.64118427,  -0.708883762,  -0.595338464,  -0.66865766},{ -0.69224292,  0.763866186,  2.07495117,  0.118079558,  0.759849191,  0.491583705,  1.00192368,  -0.255568862,  0.409187555,  -2.91402268,  -0.199814737,  -0.551493227,  -1.59688091,  -0.150581419,  -0.439454585,  -0.499708384,  -0.35243696,  -1.20840991,  3.0140965,  -0.62023598,  1.13618863,  -0.599242926,  0.410388649,  0.618773878,  0.16136241,  -1.07139039,  0.863184512,  0.163005024,  1.36885309,  -0.50759393,  1.61191189,  0.114954375,  -1.65065253,  0.783860326,  0.538165212,  -0.150479764,  1.51482415,  0.175027668,  0.466948837,  1.98134232,  -0.894469619,  -1.91210854,  -0.699070334,  0.387091279,  -0.966995478,  -1.45108187,  0.333389074,  0.452917695,  0.211379737,  -1.06359506,  0.909045577,  0.172157437,  -0.652340293,  -0.891548097,  -0.881075025,  0.575061917,  -2.00213337,  0.17780827,  0.0965931639,  0.288387418,  0.124763981,  0.978629291,  -1.52157521,  0.154498637,  0.262733817,  0.601324618,  -0.649526954,  -1.16689789,  1.13993895,  1.34666419,  -1.08593309,  -0.805331469,  -0.638209522,  0.311598659,  -0.292339236,  0.341430962,  1.36627913,  0.126867756},{ 0.245767221,  -0.451190561,  -0.453100741,  1.63516331,  0.31789282,  -0.312095463,  -0.707066,  -1.11279726,  0.417780012,  -0.320867002,  0.0721468702,  0.686850548,  0.327662289,  1.00359929,  0.443442941,  2.02393651,  -0.314450145,  -1.16971421,  0.303465158,  1.88462317,  -0.926859796,  -0.539322376,  -0.0533652529,  1.14428651,  1.98681664,  -0.364410669,  -0.245083392,  -1.50822246,  -2.01423383,  2.22206497,  -0.204360515,  1.8588388,  0.733075619,  -1.86156511,  1.13697255,  -0.463250458,  0.0544103645,  -0.153928369,  -0.86181587,  -0.528831363,  -0.416254491,  -1.12642813,  -1.08042645,  -0.533228815,  -0.709089696,  -1.24395323,  0.355086863,  0.243748933,  -0.573134542,  -0.740453601,  0.224510342,  -0.0516867489,  0.765642405,  1.05479598,  -0.599380016,  0.298852265,  -1.06157768,  -1.38229394,  -1.14873898,  0.595636249,  0.593475997,  -0.318573266,  -1.26124322,  -1.65398121,  0.744457781,  0.0555707142,  0.0880454704,  -1.16610467,  0.196689263,  1.45031321,  0.276855469,  -0.656193435,  2.47102141,  0.717216909,  0.816154957,  -0.176161885,  0.459053427,  -0.991180241},{ 0.338575155,  -0.011569364,  -2.0919919,  -0.60693711,  -0.89991945,  0.72754705,  -0.477616072,  -0.22049211,  0.0315268114,  -2.18467069,  0.0141818402,  0.390575439,  1.19871473,  0.782553256,  0.311204523,  -0.862707436,  -1.06619966,  2.06535506,  1.79390121,  -0.899448514,  -0.00631145528,  -0.693447769,  -1.03134954,  -0.435435802,  -0.367031723,  -0.772896886,  -0.233783334,  0.252017528,  -1.30758941,  -1.68917823,  -0.203962103,  -0.66967994,  0.915878773,  0.897548437,  -0.543175519,  0.849197149,  1.20312953,  1.2473352,  0.781242788,  0.277944982,  -1.4299562,  0.59836787,  -0.786041617,  -0.891277313,  0.934169531,  -0.0107667586,  -0.0562274307,  -0.143078551,  -0.229015186,  -0.0700916052,  0.192413643,  -1.20031881,  0.768040359,  -0.213978842,  0.699398041,  0.32947576,  -1.68157089,  -0.352041334,  -0.889312267,  -0.575233519,  -0.985361636,  0.0966550708,  -2.150805,  0.328342915,  1.57946646,  1.77649224,  -2.18347859,  -0.648413062,  -2.02477908,  -0.724407434,  -1.18569577,  0.93040967,  -0.0955308527,  -0.406500906,  0.684656382,  1.89374363,  0.853767335,  -0.383260995},{ -0.0341007262,  -1.53768456,  -0.781221032,  0.463149369,  -0.751007795,  -1.06845975,  0.863781393,  -1.08372808,  0.163693681,  0.680056632,  -1.01407087,  0.671700835,  0.92148447,  -0.914796531,  0.310710728,  -0.313459545,  -0.424031198,  -1.55738783,  -0.25267154,  -0.196917221,  -0.105520882,  0.847670197,  0.280147761,  0.570439398,  -0.659322441,  0.470458299,  -1.52573311,  0.33689931,  1.25166118,  0.700763047,  0.0321798809,  0.186557457,  -1.0862757,  1.8402456,  0.224376187,  0.389728695,  2.4250772,  1.38526464,  -0.034095712,  0.013913665,  0.0335214622,  -1.22026324,  -0.345034033,  -0.396615356,  0.851554811,  -0.84428668,  -0.95893389,  1.17704475,  0.4382523,  1.49098599,  0.1705589,  0.894101322,  0.10648229,  0.50139147,  -1.51300466,  -1.15628636,  0.508010626,  2.27404666,  0.359832734,  -0.977744758,  -2.19898629,  0.799266338,  0.411780983,  1.84857845,  -0.712046266,  -1.6146034,  1.29871273,  1.00148869,  -1.45985925,  -0.319305182,  -0.187293157,  -0.537580431,  -0.256297797,  1.46860683,  -1.04743063,  -0.646852612,  -0.533110678,  0.419799119},{ -0.780517161,  1.6339612,  -0.86084342,  0.26193431,  1.99819791,  -1.4522531,  0.57233578,  -1.95449972,  1.74043024,  -2.00823665,  -1.9773227,  -0.766794145,  -1.83597481,  0.611122549,  2.80176997,  1.83854842,  0.764840841,  -0.312095553,  -0.382912278,  -0.574703634,  0.770935416,  -0.85607779,  0.414833248,  0.894527018,  -1.23651552,  -1.59883237,  -2.94999456,  -1.54888964,  0.945572376,  0.016746439,  -1.02426791,  3.11753225,  0.497568011,  -0.950854123,  -1.06756234,  0.602929354,  -0.453703374,  0.215746462,  -0.392260462,  -0.0727055296,  -0.214992762,  -2.1115458,  -0.461028457,  -0.343810529,  0.251790613,  -2.19620252,  -1.08329129,  -0.816277266,  -0.661106467,  0.667809367,  0.177029639,  -0.158309519,  -0.107871212,  0.915128589,  -0.174694404,  -1.71859384,  -0.626288414,  -1.13581479,  0.69779247,  -0.901009202,  -1.12399125,  -0.172575802,  1.39626777,  -0.288748592,  -0.742675424,  1.31898785,  1.05078387,  0.254616261,  0.175905645,  -0.796143651,  -0.378103852,  0.49374488,  -0.293217897,  0.00715027051,  -0.800528347,  -1.53938448,  1.37389743,  0.436094761},{ -0.191232264,  -0.830936134,  -0.584095359,  0.0793002322,  -0.371461958,  -0.666697025,  0.746064901,  -1.08178639,  -0.723296106,  -0.977509081,  0.278331786,  -0.192580253,  0.135503978,  0.256231964,  1.87124789,  1.48056006,  -1.25238073,  0.122665584,  0.387681425,  0.742749333,  0.930089653,  0.36305663,  0.778367817,  0.0494190045,  -0.468082011,  -1.66731536,  -0.0136492737,  -0.0747095942,  -1.1516372,  -1.00868034,  0.0946829394,  -0.090927504,  -0.293278426,  -0.00493126502,  -0.131989405,  1.02972245,  0.544351995,  1.71033895,  -0.281959683,  -0.654203951,  2.20657825,  -0.369993299,  0.495182723,  -0.155087411,  -0.207408488,  -0.58259201,  -0.234055489,  1.12311232,  -0.346173793,  -0.352816343,  -1.52932227,  -0.550291002,  0.680368781,  -1.60525382,  2.02524304,  -0.0741241276,  -0.996234477,  1.1967901,  1.07017279,  -2.47797561,  0.146516889,  0.065137051,  -1.84771836,  -0.0344641656,  1.12432528,  1.03295505,  -0.896262467,  -0.0571219884,  0.147776693,  1.48705339,  0.342512697,  0.0529069789,  0.687843442,  -0.927952826,  -0.211624101,  0.0700575337,  0.18284069,  0.352655679},{ 1.4750011,  -0.311011016,  0.721797705,  -2.3319068,  -0.182301313,  -0.388673902,  0.455860764,  -0.336482644,  1.01016402,  1.00625777,  -0.486994147,  0.936825812,  -0.586029708,  0.826784372,  0.706899345,  -0.600211382,  0.727411687,  -0.752329051,  0.792283773,  0.10969764,  0.140717089,  -1.52923596,  -0.118801497,  0.105719678,  -0.447052747,  1.5667758,  0.111088842,  -0.340197086,  -0.230006009,  2.02288818,  -0.679783881,  -0.636039674,  0.169369519,  0.219267085,  0.537574589,  -1.76678193,  -0.350418955,  0.9912889,  0.974420428,  0.58469063,  -1.12059391,  -1.46195018,  0.33885029,  0.632758915,  0.485740483,  1.05879092,  -0.908617496,  0.477412015,  0.877060413,  0.581649005,  -2.41042972,  -0.609985113,  -0.335225821,  0.328557372,  -0.214390397,  1.06007493,  -0.0663409233,  -1.19317174,  -0.290884107,  0.628170788,  0.915463984,  -0.198538229,  1.80905282,  1.35073411,  -0.373331338,  -0.416698277,  -0.0122433389,  1.1382457,  0.147336349,  -0.901005685,  1.9891659,  0.602826715,  -1.18931007,  1.67750239,  -0.32617569,  0.236592725,  -0.357862443,  -2.17685986},{ 0.171665549,  1.41515112,  0.0456027649,  -0.305495501,  1.50184536,  -0.0347975977,  0.248993278,  2.57316685,  -0.58550483,  -0.813557625,  1.17257905,  -0.0747640729,  -0.0794095621,  1.36791551,  -0.218155071,  0.817098618,  0.33829093,  0.0966832861,  -0.667776525,  -0.472750485,  0.0543824062,  0.0137312328,  1.27926397,  -0.846252978,  -0.0750181824,  -0.913083375,  -0.34379968,  -1.33435524,  -0.397747546,  0.588772833,  -0.313562572,  -1.46356571,  -1.24093604,  1.47122765,  0.43095836,  -0.369798958,  0.800901711,  -0.129694566,  -0.967826128,  1.39261818,  -2.26500297,  1.28346574,  1.21887708,  -1.44543135,  0.939279974,  -0.423529297,  0.668384254,  -1.67204607,  1.66701269,  -0.750322938,  0.329243422,  -1.29703498,  0.627905965,  -2.27037621,  -0.520270348,  1.2262938,  -1.94737959,  2.35949302,  -2.29061151,  -3.11584735,  0.463623077,  -1.41086459,  0.370759338,  0.64725101,  -0.894259036,  -1.09059346,  0.975713551,  -1.06046605,  0.882073164,  -1.15143025,  1.55536771,  -0.395540029,  0.0184551757,  0.422872603,  0.795003235,  -0.311967134,  0.573722363,  -0.0670572221},{ -0.922852516,  -0.376650035,  2.21839738,  0.190743893,  2.31643081,  0.474573731,  1.71036375,  0.0166757908,  1.08974755,  -0.900909305,  0.286643296,  1.2015487,  -0.369045258,  -1.01844239,  2.12682104,  -1.62353075,  1.12007856,  0.0798032805,  -1.35768747,  -1.82433081,  0.881786764,  0.221282423,  0.305653095,  -1.07302845,  -0.854152322,  0.186536267,  2.26501679,  2.73035502,  2.1755724,  -0.917727947,  -2.6770246,  1.52946544,  -0.0779567063,  -1.05955088,  0.465076715,  -1.10271728,  -1.68700385,  -0.999661446,  -1.88892734,  -1.84620297,  0.492797315,  -0.982900441,  -1.01313639,  0.264460862,  -2.12037325,  -0.966906548,  0.580301821,  -0.257145464,  -1.63759351,  -2.36339092,  -0.328999192,  -1.054896,  0.26458317,  0.100631475,  1.09436011,  0.112882711,  -1.5733012,  -0.0839034989,  0.279234678,  1.15176427,  0.31492576,  2.37159586,  0.306553066,  1.28247476,  2.11436749,  2.18632388,  0.786964834,  -0.506247222,  -0.255323559,  -0.255457342,  -1.08176827,  0.926654279,  0.725532532,  -0.44196102,  -1.32823622,  0.418877423,  -0.459858984,  0.970785677},{ -1.14543021,  1.46104753,  -0.379552364,  0.431536078,  0.230005622,  -0.811664045,  1.85752809,  0.814850926,  0.0742784366,  1.52685499,  -0.365781367,  -0.189164922,  1.65592968,  -1.31694603,  -0.0354944952,  -1.78529084,  0.0782954097,  -1.45122623,  2.34568071,  -0.362233609,  0.256051391,  -0.428169131,  0.824190497,  0.0285964794,  1.35370708,  0.306951165,  -2.38702798,  1.53175771,  -0.427091986,  -2.5031445,  -2.49589682,  0.320079297,  -0.624144018,  -0.960808933,  2.11171055,  0.356883228,  -0.459971935,  1.28533161,  1.2173841,  -0.586243272,  -2.01991916,  0.763657451,  0.633908749,  -1.29820788,  0.941970348,  0.674177527,  0.18806909,  0.226664945,  0.667281091,  -0.666425228,  1.12271249,  -0.389000177,  0.670687854,  0.202902719,  -0.148545131,  0.81336242,  -2.22800803,  -1.14033091,  -1.60401702,  0.258318841,  1.5926497,  1.6782887,  -1.20449257,  0.193648145,  0.432489187,  1.44176543,  2.1722877,  1.18977976,  1.80918968,  0.740416408,  -1.11664617,  0.903226495,  0.378375888,  0.783139646,  0.441486061,  0.197011918,  1.32230949,  -1.57616949},{ 1.09421206,  0.443057477,  -0.356077462,  -0.112167038,  -0.150489092,  -0.651875615,  -1.01213288,  -0.308222502,  0.952544153,  1.48836339,  0.408430219,  -0.561135828,  2.3593545,  0.0843015388,  0.653654754,  -0.706618726,  -2.08351493,  0.600542665,  -0.0977739543,  -0.165451676,  -0.0185041465,  -0.50156647,  0.195957229,  0.817891538,  1.40889478,  -1.10991597,  -1.2552675,  2.18368244,  1.15832007,  0.825274885,  -0.214296401,  -0.499115467,  -0.542496562,  -0.639766157,  -0.610087037,  0.732364595,  -0.625454545,  0.351843625,  0.138797969,  0.830604911,  -1.61951804,  -1.0754149,  0.184951112,  0.377620071,  0.898421764,  -0.092384696,  0.892895579,  1.14760184,  1.11025941,  -0.969081461,  0.309164882,  1.25242317,  -0.433840811,  -0.244265243,  1.13579261,  -0.801922977,  -0.722073913,  0.737879634,  0.176226914,  -0.381977081,  -0.0474410988,  -1.21606505,  0.818935871,  -0.503871799,  -0.216114283,  -1.57245553,  -0.854378879,  -0.213856593,  0.654238999,  -1.14523602,  -0.121304713,  -0.70881325,  -0.76710248,  0.145892307,  -1.46631014,  0.899988174,  0.744467318,  -1.19918787},{ 2.04553795,  -0.558771908,  0.417106599,  1.53943467,  0.814817309,  2.1184144,  0.0243173502,  -2.38689303,  -1.38767016,  -0.119568646,  -0.201273143,  -0.0240212791,  -0.480942994,  -1.15226936,  -1.40925908,  1.21816802,  -2.49962378,  -0.313548148,  -0.662263334,  -0.426035911,  1.45688593,  0.687091351,  0.439067096,  -0.249326825,  1.52792096,  -0.641983449,  0.0636295527,  1.20105517,  0.600608349,  -1.73800004,  1.26340353,  0.44670862,  -0.368320882,  -0.446036607,  1.75006306,  0.547258317,  -1.0474304,  -0.552643836,  -0.0181759633,  1.4684006,  0.191262305,  -0.148662567,  2.1276629,  0.192527711,  0.92992413,  2.05071759,  -0.449538738,  0.691713393,  -1.28059638,  1.07602119,  2.07005191,  0.300834,  0.811566412,  -0.369547188,  -0.351068497,  -2.60658216,  0.24248758,  1.40845883,  -0.431530476,  -0.337942243,  -0.132920012,  -0.171691671,  1.75587237,  -1.30897236,  -0.997143865,  -0.795819759,  -1.2880981,  0.98308897,  -1.98359215,  -0.178836599,  0.66597122,  0.839261055,  0.150175586,  -0.574427366,  -0.423261821,  -1.27467787,  1.6439358,  0.092052415},{ -0.360887796,  0.331396669,  0.332424045,  -0.593422472,  2.48082495,  1.71349967,  1.42742908,  -0.596716702,  -0.524255157,  -0.712137461,  -1.22142792,  -0.688485384,  -0.333619297,  0.049516879,  -0.0995926782,  0.706458688,  2.04070592,  -1.40383458,  0.965632975,  -1.83744776,  -1.97793353,  -1.37268198,  -1.92926705,  -0.551818669,  -1.19778156,  -2.06035304,  -0.35024491,  -0.406380296,  0.576641381,  0.414695621,  -1.11754096,  -1.1693995,  -1.05818892,  -1.52869129,  1.39196503,  1.61880469,  -1.37470388,  3.17686081,  0.381097555,  -0.0612078048,  -0.262262881,  0.418141127,  1.09555233,  1.17198896,  -0.876844466,  0.108137771,  -0.974782825,  0.843309581,  0.249588579,  0.169998854,  -0.20404768,  0.602494121,  -0.410679191,  0.689886153,  -0.178069949,  0.011464674,  1.69354475,  -0.638186753,  -0.0391220786,  0.459838301,  -2.73865604,  -0.280382842,  -2.7608583,  -1.92179894,  0.391043931,  -2.92267179,  1.08688247,  -0.575481057,  0.0669530407,  -0.0910687894,  -0.438238263,  0.45084092,  -0.476156473,  -0.567354381,  1.39031398,  -0.923128545,  0.639180243,  -0.242039502},{ 0.270634025,  0.588445246,  -0.278312385,  1.31017065,  -0.473655373,  -1.55734253,  -1.24684191,  -1.00095034,  -0.0422433689,  -0.156072959,  -0.158543646,  -0.128706306,  -0.223335087,  -0.132587001,  -0.784819722,  0.220737964,  -0.56320405,  -1.11601663,  0.912155271,  -0.932698488,  1.19842589,  -0.0593156591,  -0.474872947,  -0.466342896,  0.145499483,  -0.260871291,  -0.422371536,  0.762786031,  0.546103776,  -0.835936844,  -2.24537396,  -1.20964432,  0.52898562,  1.13347006,  -1.05874622,  -0.603111207,  1.64197874,  0.76764226,  -0.358936876,  0.845966756,  -0.660794377,  -0.183962345,  0.0711629391,  0.40935111,  1.03721833,  -0.831973255,  -0.799783647,  -0.820689976,  1.48627782,  1.06781781,  -1.06819451,  0.000351530296,  0.207539499,  0.706861317,  1.02266443,  1.71699107,  -1.80459154,  0.4214091,  -0.382480472,  -0.726901472,  -0.727429986,  0.454728633,  -0.269606024,  -0.745199263,  1.23071778,  -0.310915947,  -0.241104171,  0.141766131,  -1.07924366,  -2.1644845,  -0.790913224,  1.02729654,  -1.28379619,  0.166957483,  -0.139030814,  -0.0179568511,  -0.771395802,  0.708363533},{ -0.50464654,  1.15173137,  -0.320323735,  -1.25010514,  0.147102788,  0.236637533,  -0.397508174,  0.359150887,  0.803128362,  -0.377029806,  -0.945849776,  0.814990878,  0.962222397,  2.37933564,  -1.7300421,  -1.33283103,  1.50936449,  -0.623763621,  -2.44621611,  0.369983613,  1.00736022,  0.0728559718,  0.603344083,  -0.925013244,  0.882626116,  -0.106460586,  1.85024357,  0.0630188733,  0.97428745,  -0.923751712,  1.16435945,  0.3897807,  -0.151668787,  0.225169212,  -0.527986407,  -1.80878699,  -0.526648521,  3.0985086,  1.12159204,  -0.827862561,  -0.107459903,  0.183190167,  -0.677570522,  -0.190576702,  -2.3314755,  -0.354953825,  -0.944825232,  -1.56546557,  0.636667907,  0.557837427,  0.409523726,  0.13842459,  1.08959603,  1.1364826,  0.317329079,  1.51341414,  0.508060157,  0.951257944,  -0.256098896,  0.430628508,  -1.33860362,  -0.160148621,  1.27061677,  -0.442233413,  -0.552591681,  0.962630868,  1.91415048,  1.40321076,  0.0943842381,  0.245134026,  -1.55290186,  0.25988704,  -0.179614574,  0.770433128,  -0.585672677,  0.432428092,  -0.934666634,  1.9949106},{ -0.210495502,  -0.513008714,  0.836049557,  -0.320546269,  0.173454776,  -0.52545625,  -1.58301687,  -1.15171623,  0.0372239277,  -2.10672975,  -2.07771492,  -0.710215092,  0.0648862049,  0.780700326,  0.53431493,  1.87170923,  -0.554728627,  0.599607944,  -0.765263557,  1.24891555,  -0.57936424,  1.23084724,  -0.632680476,  0.0272450987,  0.173622519,  -0.377287447,  0.407452255,  -0.927931011,  1.39269316,  -0.127018705,  1.21546292,  -0.578882515,  -0.696226656,  -1.38671649,  1.19344795,  0.123341732,  0.295639783,  -0.623785853,  1.29486227,  0.180126354,  -0.101125121,  0.42905882,  0.933164239,  -0.079806231,  -0.985730886,  -0.454845458,  0.0838495046,  2.02387214,  -0.0409981646,  1.13995409,  -0.00725433882,  1.81574118,  0.0147824632,  -1.06902683,  -0.467914134,  2.80484223,  0.914994001,  0.428356618,  -1.50692475,  1.58437872,  -0.303675145,  -0.751051247,  0.72192955,  -1.11252737,  0.433773696,  -0.363636076,  0.639324248,  -0.211702019,  -0.700844407,  -0.81399101,  1.14125252,  -0.404364139,  0.116359711,  -0.0248403121,  0.831975698,  -1.73731267,  -0.619758785,  -1.19067132},{ -0.107396886,  -0.290145338,  -1.81527722,  -0.699428737,  -0.865280509,  -1.38778281,  -0.383976966,  -0.36018607,  0.0964278504,  2.03458428,  2.47461057,  -0.859143317,  1.0711329,  -1.21383047,  0.679270387,  0.276758194,  0.265786409,  1.7395463,  0.445654958,  0.53941685,  -0.163087577,  1.76033092,  2.56780553,  -1.02690601,  0.815562427,  -0.486221462,  0.365668446,  1.65979731,  0.962564766,  0.123747796,  -1.30148244,  1.33895183,  1.16483319,  -0.688942492,  -0.234667495,  0.188009173,  -1.7543509,  -0.40949446,  0.989767909,  1.24137926,  1.94420457,  2.56121612,  0.525180638,  -1.51037765,  -1.11417925,  -0.570367098,  0.844649434,  0.602760375,  0.170352504,  0.652835727,  1.1097492,  -2.22959161,  -3.25046945,  0.582295179,  -0.493696481,  0.883885622,  0.0975225419,  1.21581841,  -1.07973039,  -1.09285152,  -0.269829124,  -0.424649328,  0.709605277,  -1.16068518,  1.28741336,  -0.435972691,  1.38367081,  0.211116433,  -1.31091154,  0.81995815,  -1.59006095,  -0.113054492,  0.358137816,  1.90636337,  1.25084674,  -0.426512867,  0.817014575,  1.27959037},{ -0.617233157,  0.703968108,  -1.54414368,  -1.27641535,  0.0856740102,  -0.232324183,  0.164270058,  0.033707995,  1.05951679,  -0.136217028,  -0.325478762,  -0.0744217411,  -0.656418681,  0.43350184,  -1.18896973,  -0.585687399,  0.302530587,  -0.686621428,  -1.21557581,  -1.39467669,  -0.275575459,  0.289799362,  -0.519128859,  -0.544964612,  -1.22402358,  1.30304229,  -0.755782664,  0.666607499,  -0.185061261,  -0.83826822,  0.435404539,  -0.373724848,  -0.420006603,  2.10164714,  -0.127799302,  0.749030948,  1.0842787,  -0.227102846,  2.36498523,  0.186029926,  1.15666616,  0.343074322,  -0.993895948,  0.248550341,  -0.588452756,  0.745967805,  0.668214142,  0.111359417,  0.0946504027,  -0.926333547,  -0.481837213,  0.434752792,  -1.43843031,  0.0382943563,  0.890833676,  -0.739837348,  -2.50418854,  -2.36557341,  0.592577457,  -3.15445781,  0.0672570094,  1.69106674,  -0.145311415,  0.723034441,  -0.17390804,  -0.602412164,  1.66245997,  0.108180396,  -1.50048947,  -0.272761315,  -0.468708873,  -0.723952532,  -2.28395605,  -1.22577786,  -1.24462438,  0.635903776,  0.286998272,  -1.13436735},{ -2.05223894,  0.729622483,  0.38692224,  -0.31144619,  0.544176817,  -0.616468787,  -1.05276072,  0.846224785,  0.610469997,  0.894728422,  -0.481385916,  -0.264215767,  0.0215086881,  0.314460844,  2.10429144,  -1.6809032,  1.46579075,  2.29366159,  -0.952295184,  -1.6452533,  1.18446195,  -0.770565927,  1.23327219,  0.114401855,  1.01188505,  -0.16876626,  0.349001318,  -0.693683147,  0.772639155,  -0.915340066,  0.664262712,  0.59283787,  -0.216712788,  -1.5130471,  0.409277976,  -0.483094662,  0.623264015,  0.216453195,  -0.49406901,  -1.0049603,  -0.776879668,  2.24377966,  -0.19988592,  0.578221679,  -0.232763678,  -2.38597965,  -0.652462304,  -1.55751145,  0.449176788,  0.154128566,  1.29199219,  1.34237969,  -0.496546239,  0.679413617,  -0.0609395467,  0.0227170121,  0.209704593,  -0.41461125,  -0.21712032,  -0.746005952,  0.807083905,  0.42686525,  1.14354312,  0.012551018,  -0.0274480172,  -0.675707996,  1.03205466,  -0.69901967,  -0.869406462,  0.585324228,  0.5101735,  0.235203594,  0.0260546692,  0.824464738,  0.204467103,  0.730555117,  0.864598513,  -1.01463211},{ 0.95287925,  -0.196887806,  -0.275812656,  -0.0478189327,  0.814448953,  1.16263437,  0.0688896552,  -0.338964641,  1.43928456,  1.28863072,  -0.451107055,  1.9951309,  0.938875377,  1.3656112,  -0.877020001,  0.393474162,  -0.423057079,  -1.26153541,  0.741239607,  -0.436000258,  0.593310893,  -0.418324441,  -0.717334747,  -1.15614927,  0.0530824326,  0.111359857,  -1.10283244,  -0.21693705,  -0.426742524,  0.479728431,  1.26140666,  -0.268537015,  -0.226680204,  -1.26765156,  0.385670304,  -0.622529149,  -0.270203561,  -0.649861693,  1.16508567,  0.83951211,  0.525612772,  0.57696569,  -0.0497312211,  1.48353493,  -0.104033582,  -0.845588923,  -1.45961392,  1.93150616,  0.285215616,  0.570483148,  0.0238550119,  -0.953300655,  0.480270177,  1.00933349,  1.17490339,  -0.438680321,  -1.69649982,  2.01094174,  0.481714338,  1.74992776,  -2.11557794,  -1.3225348,  -0.111080334,  0.959740639,  -0.0397689976,  1.74587107,  -0.0300526656,  1.25540638,  -0.426245481,  -0.921652973,  -1.40085316,  -0.425785393,  -0.344154358,  0.715996683,  0.282458395,  0.436767608,  0.642238259,  -1.66627455},{ -1.57718444,  -0.542405188,  1.24666011,  0.947373211,  0.668067455,  0.355781227,  -0.300101459,  0.516013324,  0.387638777,  -0.027360009,  -1.62439442,  1.77686262,  0.609723568,  -0.470629036,  -0.747144759,  0.860297561,  0.166811526,  -1.50598025,  -2.6643188,  0.896892011,  -0.484715223,  -0.0295821913,  2.41082454,  0.22923772,  -1.16730285,  1.07785296,  -1.72211361,  -1.30004907,  -1.81823683,  -0.0840772316,  1.82590687,  -0.282255918,  -0.280607849,  -0.0522682257,  -0.253281474,  -0.605399668,  0.72463572,  -1.11900294,  0.542943716,  1.87502205,  -0.857192338,  0.64613384,  -1.06547129,  -0.336009711,  0.822963476,  2.19104028,  -0.0571294911,  1.65009272,  -0.992152095,  -0.0974782705,  -1.31894934,  1.01003778,  0.0861313194,  0.616235793,  -0.748867035,  1.20279908,  0.676914155,  -1.52102149,  2.26856923,  -1.23925996,  0.979280531,  0.939722717,  0.393437415,  0.793545425,  -0.568342209,  -0.04592181,  0.763856232,  -1.75712073,  -0.304701865,  0.16816026,  0.270243585,  1.7306273,  -1.35702825,  1.27818382,  -1.2665025,  -1.90922821,  0.623411179,  0.69475776},{ -0.351859778,  -0.146398082,  -1.46055365,  0.697422385,  0.773795664,  0.514309645,  -1.93760419,  0.458040774,  0.774484038,  0.851429343,  0.776760638,  -0.0410936773,  -0.0607019514,  0.761988163,  0.0506155714,  -0.852030277,  -1.72282422,  -2.08590698,  -0.529482245,  0.374591142,  0.0481629223,  -1.33598971,  0.355159909,  -0.969614506,  0.520230651,  0.862767756,  0.273909956,  0.822014153,  -0.411087275,  -1.66689003,  0.137977928,  -0.911336362,  -0.950550139,  -0.940441132,  1.20482981,  0.0494840592,  0.837749004,  -0.821263611,  0.969532967,  -0.0741635859,  -0.240891874,  1.67829299,  -0.0876235291,  0.114488706,  -0.550613523,  0.918625653,  0.0960320085,  0.806783974,  -0.644588709,  -1.36029446,  -0.897032201,  -1.7759968,  0.645214379,  -0.944734454,  -0.753121018,  0.171372503,  0.845402241,  0.768596292,  1.78194106,  0.305777669,  0.306657255,  -1.56957805,  -0.0571605153,  -0.513538361,  0.0580255613,  -1.10064638,  1.48114502,  1.79221225,  -2.55113292,  -0.557491958,  1.49224687,  1.36640286,  -0.964656293,  -0.0694982633,  -1.98657703,  -1.3346926,  1.12143147,  0.66322422},{ -0.343916655,  -1.02384508,  -0.581193864,  0.482295662,  0.457183152,  0.185805827,  1.05878782,  0.681391478,  -0.0594505407,  2.21950173,  1.14383852,  0.580738366,  -0.34333849,  1.71741939,  0.00306581729,  -1.19075155,  0.0705782548,  0.212766632,  0.305129141,  -0.428319544,  -0.171404094,  0.647898555,  0.737378478,  0.264911532,  1.56471562,  0.175878182,  0.352885067,  -0.571619391,  0.0987671092,  -0.445479274,  -1.47121763,  -1.7100637,  -0.131997183,  0.779017746,  0.0454107635,  -0.970433891,  -0.346309215,  -1.10791707,  0.310026944,  -0.0587753467,  1.50724149,  0.152632996,  0.322770059,  0.681716502,  -0.97229898,  0.0223947279,  -1.49729872,  0.407117844,  0.035425704,  -0.686851919,  -0.551840544,  -1.21313226,  1.05066025,  0.15909934,  -1.34138572,  -0.298757225,  -1.8665024,  -0.0715324879,  -1.15812194,  -0.386078,  0.822557569,  1.36078,  0.453652322,  -0.588418782,  -0.736720622,  -0.742781818,  -1.32061303,  0.619868279,  0.965755522,  -0.45902285,  0.60914284,  0.0290146973,  0.901622951,  0.824660838,  0.712243676,  0.526909709,  0.0103716152,  -0.685800552},{ -0.727269769,  -1.31836033,  1.24436796,  1.33042538,  0.500212908,  -0.0271636434,  1.49248576,  1.1379627,  1.8857311,  -0.109964088,  0.757359684,  -0.402111411,  -0.434114724,  1.69977033,  0.34542796,  -0.295784384,  -0.289424121,  -0.0432785712,  -1.46740699,  1.15508795,  -0.806672871,  0.311867654,  0.938751042,  -0.564067602,  0.0612557493,  0.859577656,  0.934704542,  0.844329238,  -0.73179698,  0.927927077,  0.483546019,  -0.362232208,  2.24953675,  1.5023334,  -0.704682708,  1.18159533,  1.47255421,  -0.988462985,  0.39431572,  -1.78010404,  -0.304981291,  1.3409369,  -1.2599504,  0.991513908,  -0.6027174,  -0.180491418,  0.190026775,  -0.540592849,  -1.56202829,  -0.94799006,  -0.339531869,  1.60050023,  2.22977495,  0.788006544,  0.0450434908,  0.709357679,  0.741923451,  0.369350761,  0.717773199,  1.17835045,  -1.05098069,  -0.272716433,  -0.291735977,  -1.07902598,  -0.759354055,  1.21335089,  0.641050637,  -0.410287768,  -0.296332866,  -0.855227947,  1.01770484,  -1.10734391,  -0.278280884,  0.216463447,  -0.270810902,  -1.23376834,  0.83112669,  -1.2746495},{ 0.385474622,  -0.644316018,  -0.524175167,  0.754379988,  -0.178043708,  -3.12800813,  -0.270991713,  1.1666348,  -0.594962358,  -0.598904431,  0.297525018,  -0.467768431,  -1.30895293,  0.318497449,  -0.712455034,  -0.0386947319,  -0.726177692,  0.179543227,  -2.19659591,  -0.950237393,  -2.04376817,  -0.3188591,  0.410716116,  0.623337448,  1.29128277,  -0.036533691,  0.48673442,  0.501082838,  1.0855509,  0.148304299,  -2.07432747,  1.08115983,  -1.16916072,  0.721678793,  1.50810778,  -0.391014487,  0.585434854,  -0.103508718,  -0.559652269,  -1.02111399,  -0.38384378,  0.287913024,  0.310165524,  -0.40050602,  0.566063225,  -0.488415897,  -0.186500832,  1.7681284,  0.586830139,  -0.732877016,  0.785178483,  -1.40252364,  1.77302909,  0.37590763,  -0.0230455473,  -1.22686708,  0.863373458,  -1.22272015,  0.255311638,  -0.746338189,  0.866009235,  -1.13952994,  0.402239949,  -1.24658012,  -0.300178796,  1.3092171,  0.111213334,  1.41000092,  0.434769988,  -0.146467596,  0.711324215,  -1.3632611,  0.934183717,  -1.25307274,  -0.287822545,  0.277413964,  0.668529034,  -0.284030378},{ -1.20341623,  0.294192046,  -0.549242854,  0.623603523,  0.0753799379,  0.25148055,  -0.582968831,  2.00078726,  0.348627269,  -0.312542349,  -0.465657264,  -1.11739242,  1.49643195,  -0.864132524,  1.54661083,  1.16485524,  1.93439984,  -0.769819796,  -0.308155686,  -0.715875804,  -0.40661028,  0.177912861,  -0.591157675,  0.267504603,  -0.0693228021,  -0.926558137,  3.48811102,  -0.689091682,  -1.42798769,  -0.0890952647,  1.4610039,  -1.23177302,  0.472797245,  0.609536707,  1.83189487,  -0.00598728703,  -3.42465305,  0.653743863,  -0.81344384,  -0.474466443,  -0.77607882,  -0.305788696,  -0.594794095,  -0.613683581,  0.143328488,  0.55205065,  1.20606852,  0.324074626,  -1.08611619,  0.591893077,  -0.978813887,  -0.775553524,  1.1995784,  -0.207111165,  0.430032581,  0.0704755113,  1.38539493,  0.625164509,  1.23864877,  0.65492785,  0.0824838951,  0.41849795,  -0.948556542,  -0.863448441,  1.17707598,  -0.95618248,  0.718952358,  1.28208947,  -0.209947795,  1.12167096,  -0.721178055,  0.423712999,  0.680207491,  1.21327209,  -0.70862627,  -0.728096724,  -0.0685364902,  0.665957808},{ 0.410647869,  0.107442074,  0.720949233,  0.599947333,  0.426523,  -1.43762076,  0.273640782,  0.111964107,  1.65590942,  -0.457806438,  -0.0400792956,  -1.9813484,  -1.08912253,  0.748026609,  0.891370356,  -0.305573106,  -0.964228451,  -0.881602585,  -1.05339885,  1.29385221,  0.555104196,  0.00774857309,  1.55667889,  -1.1986506,  1.50687122,  0.039576374,  -1.16739774,  -0.753227234,  0.286306262,  -0.74354738,  1.35680211,  -0.224186733,  0.84669888,  -0.224487841,  -1.48191357,  1.34034264,  0.840806484,  0.435610741,  -0.0938018188,  0.833766043,  -0.505761623,  0.848852456,  1.08654952,  0.875521123,  1.43898332,  -0.0312672146,  1.70840979,  0.513779283,  -0.505658448,  0.172678322,  0.330007315,  -0.6205073,  0.253413767,  0.473916829,  0.720963776,  -1.16371,  -0.264616072,  -0.208794996,  1.86016774,  1.52556133,  -0.470647663,  -1.58461165,  -0.0453213267,  1.01085591,  0.16135478,  -0.791491866,  0.838214874,  -0.0887488723,  -0.0994833931,  0.404344201,  0.934731424,  0.113387987,  -1.00476587,  0.978544772,  -0.544240057,  -0.331682801,  0.0969245657,  -0.514751613},{ 0.470514059,  1.46460783,  0.0152744511,  0.140011787,  0.287116379,  -0.861833036,  1.20361829,  0.0251414105,  0.863739729,  -0.762045205,  -0.254250795,  0.251489133,  0.0221509822,  -0.0783945173,  0.0318024717,  1.17200911,  0.570951283,  -0.645499885,  0.0281831753,  -0.203776538,  0.0456715524,  0.0911610946,  -0.634285212,  1.58775961,  0.0908916891,  0.292703837,  -2.03311992,  1.43995082,  0.92708689,  0.323518604,  0.243470997,  1.37713957,  -0.915027916,  1.00957978,  1.57228243,  0.0126803387,  1.63846588,  -1.44397449,  -0.256788194,  2.26365781,  -1.02154958,  0.949775338,  0.338629603,  -0.513069689,  -0.261568516,  0.304093689,  -1.65829825,  1.44527912,  1.49702203,  1.24149919,  0.341924429,  -0.0967082158,  -0.50707233,  -0.138165236,  1.43834889,  1.16830957,  0.791507065,  1.6832968,  1.22941494,  0.589261651,  -0.799119353,  -0.622431397,  -0.249333143,  -1.55072498,  0.600682616,  0.0195383169,  -0.60921073,  -0.481897324,  1.5005368,  0.722716391,  -0.538594365,  -0.664880931,  -1.18335032,  -0.559902489,  0.941009879,  0.104166463,  -1.72115815,  0.309375644},{ -1.51205254,  1.44786882,  -1.85382152,  2.02611637,  0.587831259,  -0.259808362,  0.106989555,  0.275927812,  -1.38923013,  0.719503224,  0.41277501,  -0.636182785,  0.201822728,  -0.632313848,  -0.31613487,  -1.15950358,  -0.600940704,  0.0654078126,  1.24725926,  0.562568724,  -0.0915831998,  -0.97814399,  -1.28565609,  0.771777213,  -0.787417114,  0.962307632,  -0.557434738,  1.54212892,  0.482987642,  -1.5227828,  0.147816733,  -1.76229596,  -0.298832387,  0.409300238,  -1.13762712,  -0.121416263,  -0.25707674,  -0.34619537,  -0.488468736,  0.781218052,  0.759268045,  0.30042699,  0.849779963,  -0.227224275,  -0.905505359,  0.870923877,  0.58338809,  0.324456036,  -0.237795547,  -1.4929409,  -1.28054023,  0.312687993,  -0.930806518,  0.854747951,  0.785432577,  0.0472604893,  -0.647941351,  0.217120469,  -0.00445163576,  -1.21342015,  0.21405527,  0.473944485,  1.11953092,  1.32512259,  -1.62411189,  0.075982362,  1.05404389,  -2.10940385,  0.49056986,  -0.318833858,  1.39201832,  1.10742116,  1.16206169,  0.899662137,  -0.359638453,  0.0921455994,  0.791034579,  0.47129342},{ 0.842622042,  -0.753562033,  -0.836179972,  -1.74347138,  -0.399192899,  -2.48183656,  -0.332910568,  0.451592922,  0.497596174,  -0.287365586,  -0.999036789,  0.723818541,  -0.0652038008,  -1.30011344,  1.96874344,  1.75788331,  -1.26420462,  -0.477442443,  0.294365525,  2.51249719,  -0.357915133,  -0.594197094,  -0.789954901,  1.97869706,  0.823374212,  -1.21206784,  1.56060863,  1.11622036,  -0.735235512,  -0.376809597,  -1.3295368,  0.879256427,  0.649680793,  -1.26625538,  -0.758760571,  -2.95133567,  -0.0603926554,  -0.515214324,  -0.147087887,  -0.282315433,  0.195346668,  1.86696744,  -0.405099958,  1.33609498,  0.243103892,  -1.4114213,  -0.230682075,  1.45245361,  -2.28550553,  -0.906515598,  -0.523204982,  -1.94392645,  -1.79181838,  -0.506139517,  -1.24972153,  0.126914591,  -0.719049335,  0.860142708,  0.266230792,  0.633673966,  0.747546494,  -1.40161884,  0.398179859,  0.00483784545,  -2.03844142,  0.356497377,  -0.578871489,  1.02749205,  -0.519364297,  2.2320776,  0.538963258,  0.844909966,  -0.714435458,  0.418502837,  1.04271162,  0.469594628,  -1.03033733,  0.925896704},{ 1.54304361,  1.33893383,  2.26336002,  0.316426665,  0.271671891,  1.69311392,  -1.10069919,  0.609948695,  -0.0407755114,  0.0873925611,  0.103173301,  -0.419142246,  1.71123469,  -1.2796694,  0.65029031,  2.23447561,  -1.12095642,  1.35681844,  0.239492506,  1.0447073,  -0.424204707,  0.341477484,  -0.321992725,  1.10474062,  1.36654353,  -0.0893853158,  -0.662693143,  -0.622009873,  1.22621953,  -1.03373373,  -0.269321829,  -0.593918681,  0.304096937,  0.486572772,  -0.483656973,  -0.233930722,  -0.973177195,  -0.428553253,  1.67030025,  -0.874390841,  0.130743638,  -0.430925965,  -0.619300663,  -1.14073265,  -1.42968166,  1.43869758,  -0.173185304,  -0.255004257,  0.526700675,  -1.07865334,  1.52840447,  -0.124999575,  0.495550066,  -0.712698281,  -0.199658647,  0.736605823,  -1.57028127,  -0.885236323,  -0.163364962,  0.771905184,  0.699066639,  -0.915763855,  1.04086304,  -0.455804646,  -1.02609861,  -0.095290944,  -0.182683781,  1.55884886,  -0.740182221,  0.507171631,  -0.612871945,  -1.70210838,  1.4030869,  0.488252997,  -0.0223361868,  -1.63369775,  -0.841370523,  0.292106181},{ 0.421323031,  0.530737817,  0.0205376949,  2.75576401,  -0.943016171,  -1.31214869,  0.480687261,  0.885106504,  -1.16972899,  1.08733165,  0.601836503,  0.236078799,  -0.572838366,  -0.117460676,  0.560289919,  -0.888006032,  0.511530519,  0.586625099,  1.2899586,  -1.05308187,  1.5391134,  -0.428346127,  -1.14443088,  -1.76987553,  -1.93903875,  0.188750669,  -0.634672463,  -0.780006111,  -1.17878687,  -1.16608834,  -0.605897188,  -0.413116068,  -1.11477447,  -1.25283897,  -0.425285071,  0.0668417364,  0.261818349,  1.25942028,  -0.628281236,  -0.348127872,  -0.162233487,  0.678590298,  -1.80263662,  0.364199966,  -0.935302973,  0.860430479,  -0.29209891,  0.337535769,  0.105611682,  0.676238716,  -1.67091012,  0.969826758,  -0.526090622,  1.68201447,  -1.53106809,  0.114040293,  -2.10056901,  -1.66221559,  0.883812428,  0.112687357,  0.104406357,  -0.76332289,  2.11762905,  -1.00425541,  0.81022495,  0.982162952,  -0.660682142,  -0.392304808,  1.20993257,  0.331787795,  -0.221290305,  0.424178362,  1.77492404,  1.21265018,  -0.828991532,  0.494503081,  -1.93716168,  1.9534632},{ -0.467175454,  0.548759937,  -0.289984882,  0.219573826,  1.46305215,  0.501211166,  -1.22212756,  -1.16589236,  0.179869249,  -0.355817229,  0.821848094,  -0.472492546,  -0.0965206772,  1.89841998,  2.09406042,  0.610723555,  -0.196061015,  -0.557070971,  -0.565486729,  0.816565275,  0.297913998,  0.731033862,  -0.659603953,  -1.37321579,  -0.582704842,  0.135711879,  1.63144374,  1.14505398,  -0.790325284,  0.976962149,  1.92854273,  0.809669375,  -1.08059037,  -1.61663127,  -1.29085982,  0.613467097,  -0.379754752,  -0.796342731,  -0.48328796,  -0.457861692,  0.790931284,  0.321549654,  1.11854565,  -0.26353386,  1.08444023,  1.91447389,  1.16393209,  1.82358694,  -0.123603612,  1.37224376,  -1.56864524,  0.134479612,  -1.1166873,  -0.270001948,  -0.371773392,  0.639413059,  -0.321302474,  -1.3065666,  -0.437161207,  1.45754647,  0.172205448,  -0.00919259898,  0.359285206,  -0.822651684,  0.293732494,  -0.95267874,  -0.000546338269,  1.72364223,  0.803500116,  -1.36319613,  -1.71157491,  -0.516654193,  0.112973332,  -0.470761329,  -0.244171679,  0.0299734119,  -0.343364924,  -0.560423374},{ 0.962208211,  -2.02798295,  -0.13630791,  0.812978446,  -1.28736269,  -0.802929878,  1.15795743,  -0.953169942,  -1.05077958,  -0.542814076,  1.67684746,  0.835322976,  0.467122674,  0.693323374,  -2.03015542,  -1.0507952,  -0.139268026,  -0.650657177,  -0.495076299,  -1.2313323,  0.272533,  1.43126297,  -0.879961431,  1.30620706,  -0.80528599,  -0.330548882,  0.460200727,  -0.546624362,  1.36716926,  0.0208149925,  0.95507741,  -1.04187775,  0.478931159,  0.395190388,  0.700068653,  0.00577310286,  -0.60543561,  -0.848164439,  0.130790308,  -1.23299325,  0.154607803,  0.513863623,  -1.22004843,  3.41231036,  -1.49254847,  1.7036835,  -0.889908969,  -0.011128068,  0.59062624,  0.91727984,  -0.52605021,  -3.4444437,  1.00269771,  -2.99762082,  -1.16553938,  -1.35296726,  -1.79716837,  -0.782742858,  1.15026963,  1.00728452,  0.20926258,  0.709668815,  0.138021514,  2.59912229,  0.131805867,  0.584594846,  0.286004573,  -0.246206269,  -0.121662632,  0.485516399,  1.34188592,  -0.26445514,  0.370926201,  1.28835201,  -0.552965939,  -0.474716187,  0.933858991,  1.46272719},{ 0.539366126,  0.631481349,  0.248875514,  1.05966508,  0.896790087,  0.518314481,  -1.0449785,  0.0104630021,  -0.60125041,  0.644095361,  -0.597206831,  0.0533067733,  -0.659049213,  -1.05935299,  -0.314227015,  -1.44432342,  -0.416707218,  1.28918183,  -0.119531743,  1.00985146,  1.19486022,  1.80861247,  1.20284808,  -1.05818188,  0.861696064,  -0.233592585,  -0.398326695,  0.260888696,  2.08474827,  0.0739489198,  2.41378713,  0.736515582,  -1.33515966,  -0.46958214,  -0.574847102,  1.55360568,  -0.246809274,  0.689810038,  -0.549426019,  -1.21563447,  -1.28258288,  -0.219092593,  -0.608414888,  0.902654886,  0.339357436,  -2.56907892,  -0.020323379,  -0.792904794,  -0.694398224,  0.506643116,  -0.547081053,  0.68093586,  0.23611562,  -0.161391407,  -0.200530782,  0.324536353,  0.0661209375,  -0.194344446,  -2.08883739,  1.25650311,  0.147831485,  -0.797845662,  0.886703491,  0.47853756,  0.622757256,  1.42957127,  -0.0882555693,  0.329301447,  0.498025417,  -0.723982811,  0.310664326,  -1.12234581,  -1.41018951,  0.0387888923,  -0.180013612,  1.09103668,  1.03277111,  -2.25650716},{ 0.165338874,  0.157871649,  0.368360311,  1.60834885,  -0.618247867,  -0.644791067,  -1.08216584,  1.56101191,  -0.466283739,  -0.97715646,  -2.43064737,  2.31667924,  0.662835121,  1.62992859,  -0.753575265,  -1.5165627,  -0.287286073,  -0.330083281,  0.717845857,  -0.575222313,  -0.389326364,  -1.04375362,  0.207746133,  0.0611711182,  0.561745942,  -2.13663769,  1.01756477,  0.103355199,  -0.535524487,  -0.558695197,  -1.59286773,  -1.68267548,  -1.99117494,  -0.947578371,  -0.119756058,  -1.27972364,  0.0702629983,  1.67105794,  0.269431144,  -0.403743088,  -0.43575564,  -1.34623706,  0.51365751,  2.20125175,  1.16593194,  -0.194085419,  -1.11606872,  -0.92120105,  0.71979332,  -0.846074104,  0.224436209,  -0.360288918,  0.126681641,  1.09850502,  -0.706680059,  0.487055212,  -1.22494459,  0.00698017189,  -1.24304092,  -0.155930653,  -0.554042399,  0.979211807,  0.380636215,  -0.444492579,  -1.03853416,  -2.93251181,  0.392471701,  -0.0129826665,  -0.213667363,  -0.266471565,  0.601387858,  0.266801953,  0.535317898,  -0.0907699987,  -0.519175828,  -0.188663721,  0.675343752,  -0.18618463},{ 1.63644922,  0.702428937,  0.116814591,  0.172654942,  0.760389984,  0.417168379,  -2.33142471,  -1.52321768,  0.379700989,  -0.123110987,  0.566987336,  -0.300427139,  0.409910023,  1.67317724,  -0.969725192,  -0.298798472,  -1.03167522,  1.08610785,  -1.25034094,  -1.1341964,  -1.37614119,  2.62475729,  -1.50869775,  0.851573706,  -1.44100082,  -1.57926774,  0.269258142,  0.415254772,  -0.220634848,  -1.47804189,  -0.362955958,  -0.773404777,  1.38039422,  0.293577373,  0.701429307,  0.320950627,  -0.0145552317,  2.05445099,  2.36731601,  -1.1001544,  0.155740604,  -2.0903151,  1.18762863,  -1.32131577,  0.842831016,  -0.808507979,  0.352811009,  1.09157455,  -1.32788837,  -0.348488301,  1.77964306,  -0.147708043,  0.552366376,  0.691932082,  -0.242817402,  -1.00608814,  1.68355548,  1.20963824,  -0.283279687,  0.198546618,  -0.984266043,  0.452692121,  -1.65988004,  1.22937417,  1.84392643,  1.52477443,  -1.40300167,  1.92869735,  0.589301407,  -0.673267365,  0.966183722,  1.37324893,  0.749219716,  0.485616148,  0.192356527,  2.47313333,  -0.419238955,  -0.655540347},{ 0.258060694,  -1.10071814,  -0.160350561,  -1.10918355,  0.286971152,  -1.07010603,  0.0682490021,  -1.26570261,  1.37662399,  1.88890922,  0.745022893,  0.69853133,  0.574642479,  0.385378897,  0.325872183,  0.842033088,  -0.318146855,  0.0926309228,  -2.30012012,  -2.05936718,  0.507505298,  0.613816619,  0.523458838,  -0.162172049,  1.41939497,  1.22177839,  1.0439781,  0.474022239,  -1.03708661,  -0.118739657,  -0.00827554427,  0.793809235,  -0.355147332,  2.03529048,  2.12332487,  -0.497409344,  1.77444744,  -1.49925041,  -0.653764665,  -1.0371387,  -0.531524718,  -0.83952862,  -0.860916138,  0.550373733,  -1.81398344,  -1.34582984,  0.648437023,  -0.62282145,  0.435223222,  0.917320073,  -0.148646727,  0.753739357,  -1.09576154,  1.17859495,  -1.64815891,  -0.600988567,  1.17931342,  -1.54120755,  0.737096727,  -0.230416939,  0.421049327,  0.232780308,  0.515794635,  -0.642171502,  -1.35417151,  0.37374723,  -0.415866613,  0.721628368,  -0.462384284,  1.31590593,  1.09031188,  -0.326706082,  0.170611814,  1.86450469,  0.743554771,  0.178853586,  -1.54248667,  1.89830422},{ -0.24445051,  -0.316495627,  -0.6741243,  -0.188394889,  0.635618627,  0.49593237,  -0.04194884,  -1.72915483,  -1.29629505,  2.91135764,  0.988572478,  -1.25173509,  -1.05337334,  -0.0583166629,  1.13253784,  0.829632282,  -0.791597247,  0.0162757989,  0.31173858,  1.32587326,  1.76711261,  -0.856653452,  -1.63126791,  2.6126132,  0.188323766,  -0.125993088,  0.730697632,  -1.02052081,  0.393669575,  -0.818465292,  0.709594309,  -0.747053444,  1.25161588,  -0.504579782,  0.165908545,  -0.108548909,  0.457058877,  -0.392206848,  -0.14320603,  0.131012827,  -0.621988952,  -0.160426512,  1.42135227,  0.433211267,  -1.08142221,  -1.86043167,  -0.653638661,  -1.59477031,  -0.0370446965,  -2.39941454,  -0.41160965,  1.53545558,  -0.397922456,  1.16132402,  2.07383776,  -0.0234762765,  -0.461978078,  1.11324441,  -0.223683968,  -0.479262769,  0.89905256,  -1.11866057,  -0.471815616,  -1.16715932,  -0.0192756485,  0.268904239,  -0.617267549,  -0.465581477,  -0.0874059275,  0.56334126,  1.94736421,  1.13721108,  0.481481791,  0.489225298,  -0.244093433,  -2.15327168,  -0.422623426,  -0.0594773032},{ -1.65197182,  -0.676246345,  0.791176736,  0.687880158,  -1.0553019,  -0.50813657,  -1.86752319,  0.326403469,  1.08620405,  -0.230041787,  0.254198164,  -0.438589007,  -0.386780202,  -0.502305865,  -0.966148019,  -0.627460778,  -0.59259057,  1.90516448,  -0.679653227,  -0.380932599,  0.921314538,  0.0537876785,  -1.25755525,  -1.65113997,  -1.49240291,  0.803906918,  1.00189936,  -1.06416154,  -1.02453578,  -1.45193684,  0.424987286,  1.94461954,  0.0261343922,  -0.189894557,  1.75524747,  0.895160794,  2.01784563,  -1.22005379,  0.503416479,  1.39071441,  1.70730317,  -0.457925528,  -0.094319202,  -0.799626648,  0.159647763,  -2.06416225,  0.34738785,  0.892062724,  -0.98277241,  1.49107075,  2.40361452,  0.108622983,  0.621013999,  1.80064857,  -0.595745206,  -0.474950194,  1.94976294,  0.875515223,  -1.07124865,  0.736655116,  0.857794106,  -1.06432009,  0.379933447,  1.38647687,  -0.066103071,  -1.48921287,  -0.112195447,  0.965963423,  -0.00710030319,  -0.66733098,  0.613400877,  0.141241997,  0.0274910182,  -0.662324727,  0.525428295,  0.507936656,  -1.02300441,  -0.419014186},{ 2.63615823,  -0.942123711,  -0.353221923,  0.500244915,  0.862668514,  -1.4558965,  -0.0925326049,  1.18029487,  1.25694621,  -1.21147227,  1.82736254,  -1.8914144,  1.35820615,  0.531480253,  0.00468801241,  0.595243573,  0.331397593,  0.824164987,  0.553797066,  -0.259666115,  -0.00306754303,  -1.478356,  0.0113599868,  0.739262462,  -0.631640077,  0.833392262,  -0.0574204288,  1.62244618,  -0.520761073,  -1.6827141,  -1.21758616,  1.15233207,  0.402049005,  -0.619651914,  -0.177938238,  -1.00867927,  -0.829239368,  -1.9182359,  0.354186386,  -0.548457801,  -0.947926223,  0.615207553,  -0.386532634,  0.899852693,  1.49212515,  0.0506993905,  1.90210104,  1.61996412,  -0.115931839,  0.0093109943,  -1.22215939,  -0.552613914,  0.955105007,  1.75623405,  -2.60093141,  -0.43407321,  1.09691346,  -1.28018057,  -0.449724793,  0.553063869,  0.260367393,  1.64471734,  -0.872824073,  -1.97063732,  0.324122459,  -0.338199645,  0.00719746435,  -1.24816847,  -0.429629147,  0.203678995,  -0.871111214,  -1.09020257,  1.65336597,  -0.00354210124,  -1.53287864,  1.09166133,  -1.76297665,  0.0277676117},{ -1.39489532,  0.182458758,  1.20651138,  0.698073566,  -0.100357272,  -0.437798083,  -0.331661969,  -1.05341136,  -0.354796559,  -0.182344064,  -0.182510734,  0.132592395,  0.705676198,  -1.17380214,  -0.698914886,  0.208577648,  -0.681017578,  -1.85733294,  -1.07723737,  0.375990003,  -0.287037253,  -0.165055662,  -0.212594017,  1.32157624,  -0.905061364,  0.560409069,  -0.361176968,  -0.0721569881,  -0.721225083,  -0.527686775,  0.18921116,  -2.35378194,  -1.47361398,  0.726478457,  0.798212707,  0.137888789,  -0.0588015728,  0.0258816052,  -0.133278698,  0.358602166,  -1.252249,  -0.803894579,  -0.469522953,  -1.82430482,  1.79875374,  0.252395034,  -0.165582195,  -1.49308336,  -0.371321082,  -1.38433039,  -1.98619425,  0.7243734,  0.627983749,  0.0409637429,  -1.12208235,  -0.0766619071,  -0.66521132,  -0.695057988,  -0.145228699,  -0.787921786,  -0.546798468,  0.0304627735,  -0.601780057,  0.558580279,  0.506443381,  1.36563563,  0.175973549,  -0.214549825,  1.53993452,  0.791899264,  0.028889643,  0.143961102,  1.15814888,  -0.101847477,  1.14131796,  -0.451298922,  -1.07443058,  -0.196289673},{ -0.0608855039,  -1.7484318,  -0.459192932,  -0.569134653,  0.0165351648,  0.641246259,  -1.21595573,  -1.30862963,  0.155840024,  0.164578006,  0.875711739,  -0.43030265,  0.0398274213,  0.60747999,  0.449844569,  0.313475817,  1.7072196,  1.25298238,  -0.869128525,  -0.807732761,  -0.579262674,  -0.899987996,  -0.695458055,  0.22617352,  -1.15428412,  -2.27780366,  -0.98628211,  0.256289154,  0.363815308,  -0.775982738,  -0.816531062,  1.17844427,  -1.58449948,  -1.80333412,  -1.0395236,  -0.203153774,  -1.62995744,  1.20147264,  -0.682647049,  -0.259129822,  -0.58272332,  1.15798867,  -0.292405814,  -0.390370756,  -0.394118905,  1.34756219,  0.650215805,  0.0450237729,  0.381813496,  0.660453677,  -1.52044547,  -0.894864678,  0.241813555,  -2.13287711,  -0.0392063893,  1.11672962,  1.24498916,  0.931982577,  -1.28317249,  1.21939027,  0.679781973,  -0.504433453,  -0.556207955,  -0.364230931,  -0.849019587,  -1.81607354,  -0.375258058,  0.877201736,  0.760768056,  -2.31483841,  -0.357077211,  -1.71783531,  1.25585675,  0.318245798,  -0.815165639,  0.631236911,  0.580094278,  -1.94365072},{ 0.350230902,  0.304565638,  0.150548041,  2.05891657,  -0.656623125,  0.865807354,  0.102787971,  -0.129281089,  0.26333186,  -0.642634213,  -0.766923785,  -0.0626482219,  0.0869521722,  -2.08121657,  1.2665956,  -0.19913803,  -0.744524539,  -0.0717898011,  0.643215001,  0.245605618,  -1.33154404,  -0.691629887,  0.572182655,  0.648340583,  -0.651382923,  0.896540403,  -2.43421817,  -1.44380224,  0.381632835,  -0.685640454,  -0.620345473,  -0.271203727,  -1.49036455,  -1.3716985,  -0.492595643,  -1.05707335,  -0.54304558,  0.427082688,  0.880536437,  1.62034202,  -1.52782702,  -0.360688418,  1.26837647,  -0.612252772,  1.9668771,  -0.844113648,  -2.27804613,  1.08889043,  0.614931524,  -0.353246599,  0.361247003,  -1.14703572,  2.0464251,  0.688204885,  -0.092134729,  -1.26129758,  -0.00384585373,  -0.486284733,  0.0771777108,  -1.86204779,  0.89971143,  -0.608688176,  -1.16440701,  1.60616994,  -0.648771048,  -0.41817072,  -0.374988019,  0.255901933,  -0.694583714,  0.67147702,  0.529107094,  -0.500321209,  -0.759457409,  -0.268294871,  0.470987052,  -2.11069345,  0.0448087864,  0.714170635},{ 2.83916903,  -0.23595804,  -0.883070409,  -1.36506236,  -0.626613081,  -0.203560472,  -0.982610166,  1.43881047,  0.0612911358,  1.01954997,  1.1805228,  -1.1917361,  -0.877931595,  -0.103685014,  0.862884164,  -1.02854121,  -0.52447468,  -0.355596215,  0.709857643,  -0.838384926,  0.874966621,  -1.16118348,  -0.363467485,  -0.0124202278,  1.44917417,  -0.0847528726,  -2.11456108,  -0.57321018,  0.864667237,  -1.29341912,  -0.164653122,  1.58072793,  1.85046649,  -1.94637334,  -0.324511319,  0.539500415,  -0.478297472,  0.0329513997,  -0.363110811,  0.686744273,  -1.45342958,  1.29431069,  -0.357788712,  -0.12111973,  -0.281878233,  -1.26008987,  2.06474352,  -1.08489788,  0.118740298,  -0.556212604,  -0.263252318,  0.27795428,  -0.147957638,  0.455718994,  2.2057097,  0.135379091,  0.0676900074,  -0.463055909,  0.68389374,  -0.821463406,  -1.09190786,  0.699442744,  -1.29857826,  1.28111029,  -1.34571517,  -1.33023667,  0.778717697,  1.39779019,  -0.872433484,  -0.213731602,  -0.665280044,  -1.63370216,  2.41641259,  0.426849812,  0.291689306,  0.885504186,  -0.356930405,  -0.526182294},{ -0.223621994,  0.0742806941,  0.882314801,  1.70414019,  -0.917038321,  -0.768128514,  0.372245044,  -0.639494658,  1.01635373,  -0.20280318,  0.825853348,  0.110314071,  -0.618480384,  0.141691789,  0.00720753381,  -1.5860517,  1.80638003,  0.958635092,  0.274812102,  -1.62433267,  -0.504468203,  0.543728828,  -1.53620803,  -0.293176144,  -1.44314826,  0.60811621,  -0.47575146,  -0.958534896,  0.608400822,  0.0345564,  -0.457614422,  0.915805638,  0.445962161,  0.244990885,  -1.75473416,  0.163422048,  0.8058213,  -0.548408449,  0.870497525,  1.25425792,  0.753762126,  0.789594531,  -1.35307753,  0.870429456,  -1.54717433,  0.737840116,  0.879365444,  -0.163635999,  1.63193154,  -0.618744552,  1.00575507,  1.92687941,  -0.184654728,  2.18867993,  0.400078267,  0.497112155,  -0.544799685,  1.47472262,  0.120615281,  -0.00268722302,  1.27630305,  0.172831893,  0.183036804,  -0.355906755,  -0.737553358,  -0.557223201,  -0.24296248,  0.618014395,  1.17551219,  -0.589121401,  1.21130323,  -0.844772696,  -0.548092961,  -1.22363973,  -0.803537726,  -0.671374083,  0.663724482,  0.0958623737},{ 0.178778112,  -1.4272846,  -2.51410675,  -0.750357568,  0.736775756,  1.0999248,  -0.98356688,  2.43256211,  -1.55103695,  1.08797848,  2.03579068,  -1.54696298,  0.855114341,  -0.922655165,  -0.0969194397,  -1.60589755,  -1.39749265,  -1.05954254,  -0.414907992,  -0.17693755,  0.0952380747,  0.920462608,  -1.53312778,  0.455913395,  -0.996898651,  0.0106330523,  -1.17513061,  -0.949451268,  -0.704511285,  0.501259685,  0.720788062,  0.115469106,  0.277353227,  -0.555298805,  -1.2025677,  0.615335166,  0.00910419971,  0.403078109,  -0.578739524,  1.49983048,  0.62035358,  0.136529297,  0.270940304,  1.12409878,  0.523175776,  -0.877521157,  0.175588429,  -0.0929416418,  3.23583984,  0.372347325,  0.102649547,  1.23586214,  -1.22421086,  0.0473971255,  1.59592474,  0.688396156,  -0.769221723,  -0.691854358,  1.79897237,  1.0735321,  0.761980236,  0.053878013,  -0.0526390374,  -0.276314348,  0.217508063,  1.00376487,  -1.94295752,  -1.26649845,  2.52575588,  -0.204730988,  -0.341925234,  0.0988636836,  -2.06303191,  -0.229968593,  -0.401564658,  0.907312572,  -0.895234525,  0.645779729},{ 1.19540846,  -2.20460343,  0.0711972564,  -0.992975414,  1.09600329,  0.150784165,  0.626250505,  -1.67345953,  0.19865109,  0.677708447,  0.00333844079,  -1.45934296,  0.227601841,  1.4410584,  0.884744644,  0.213645354,  1.31816328,  0.344438136,  1.9916296,  0.105456077,  -0.531997919,  -0.821473181,  -2.55419636,  0.112920433,  -0.330658823,  1.60634577,  0.548732579,  -0.222526968,  0.13529186,  1.10912871,  0.0378407091,  -0.450854421,  0.905260503,  1.99528182,  -0.260941476,  -1.03392756,  0.196097821,  -0.499894798,  -0.670683503,  -0.876722336,  -1.49565971,  -0.537909985,  1.12298048,  0.253224909,  -0.269921243,  -0.93130976,  0.375819355,  -0.277647585,  -0.797624648,  -0.143690035,  1.21175945,  -1.79038608,  0.972500205,  -1.15292978,  0.216831997,  0.654362798,  0.458717704,  -1.20235348,  0.121543169,  -0.388112932,  0.482023299,  -1.01984787,  1.20983803,  0.457328647,  -0.405325353,  1.12294877,  -0.353841066,  0.58140111,  -0.890283883,  1.08309758,  -0.848421752,  -1.13131189,  0.345120996,  0.115346052,  1.10901809,  1.65200472,  -1.39122272,  -0.512385488},{ 0.245285332,  0.590925157,  0.955259085,  1.04093897,  0.38876465,  -0.990810931,  0.922262967,  -0.664082408,  -0.323513836,  0.0335314572,  -0.346066862,  1.14897823,  0.685892224,  0.725241661,  0.217968479,  2.42737007,  -0.944901526,  -0.35634473,  -0.470223308,  -0.259739369,  -0.166660637,  1.01571202,  0.0610995032,  0.58930707,  -1.2192198,  0.902052164,  -0.774374783,  -0.688127697,  0.72756815,  1.52992499,  -0.497582823,  1.30768013,  2.15873384,  1.69930303,  0.656836152,  1.02677166,  0.682025135,  1.36633301,  -1.4130578,  -0.0442126654,  -0.852241576,  -1.22256768,  0.504946887,  -0.0833336115,  -1.55976176,  0.218573675,  -0.542677164,  -0.843598843,  -0.293384969,  -0.235136494,  -1.68082798,  0.0957426056,  0.467617691,  -0.672490895,  0.79394722,  -0.206960782,  1.14033222,  0.565053582,  0.740169346,  -0.510464132,  0.289345503,  -1.22900665,  -1.46773636,  0.401628166,  0.212516204,  -0.323615044,  -1.90738928,  -0.424723059,  -1.17207158,  -0.654285252,  1.38856363,  1.85345018,  1.68410313,  0.58253628,  -1.00460923,  -0.628937066,  -1.02202463,  1.14261079},{ -0.797834814,  0.431840956,  -0.217297673,  0.2081763,  2.39883757,  -0.340613306,  0.671524107,  -0.791666031,  -1.02916753,  -0.746993303,  -0.458687097,  -1.34777057,  1.34787095,  -0.646464527,  -0.901937842,  1.60060191,  0.427343071,  0.614967108,  -0.499213755,  0.401676357,  1.59385693,  -0.262245476,  -1.48139524,  1.45184231,  1.22580647,  0.0763058141,  -0.888839006,  0.315618485,  -0.591623962,  0.111687459,  -0.945413351,  -0.94691205,  -0.152435511,  2.30586743,  -0.772903442,  0.159553885,  -0.412375242,  1.97802043,  -0.462198675,  -1.14232409,  -0.633361042,  -3.12978482,  0.247361764,  0.442551672,  0.379207134,  -0.482348531,  -0.910312235,  1.77476203,  0.144035965,  0.712351859,  -1.04898453,  0.492588609,  -0.496142924,  1.20855618,  0.491082877,  0.245747447,  1.12542927,  1.56783438,  -0.388165474,  -0.00744798593,  0.511749744,  -0.832115829,  -0.530122459,  0.322448313,  0.633447766,  -0.421935111,  0.361963689,  -0.185989738,  -0.124709845,  0.458266705,  1.06822526,  -1.59638071,  -0.45340246,  0.973577559,  0.273712009,  0.799355388,  1.75653028,  0.331360161},{ -0.378598422,  0.224157095,  0.959392846,  0.728613555,  1.06040263,  -0.236822754,  0.354753792,  -1.48773384,  -1.07827532,  1.19848835,  0.818331301,  1.50920808,  -1.1470381,  -0.0899568051,  -1.69901264,  -0.0371603593,  -0.326070875,  0.0981159657,  0.0383265242,  -1.80354774,  -0.383303851,  -0.848745465,  1.69722974,  -0.580636263,  -0.0262776166,  -1.04461622,  1.75864518,  -1.85883176,  0.62681663,  1.01371944,  -0.104618296,  0.953504145,  0.0438228771,  -0.135397568,  -0.173135608,  1.0054996,  -0.125400096,  2.13798952,  2.31225276,  -0.425235569,  -0.0529957861,  -0.224752188,  -1.51854742,  -0.67970103,  0.922872782,  0.933715105,  -0.306949764,  0.873340189,  1.26061881,  -0.409233123,  1.13765776,  -1.08168375,  -1.57287383,  0.455488324,  -0.182135627,  -0.254564643,  0.0277217533,  -0.408971548,  0.566106737,  -0.259359956,  -0.0939799398,  -0.704516768,  -1.1453737,  -0.234328985,  0.458918005,  1.03242147,  -0.93527925,  -0.39585042,  0.186566696,  0.782357275,  -0.880786717,  -0.183383465,  0.955966234,  -2.40328074,  0.591419458,  0.345934987,  0.873749673,  1.80559683},{ 1.02311099,  1.4466213,  0.428101093,  -1.78856719,  -0.743925273,  -1.83491015,  -0.669687033,  0.71986258,  0.0184850581,  0.228617132,  -0.329528362,  0.0176052488,  0.216226891,  -1.23957646,  1.79453909,  0.613335729,  1.33363581,  0.554888785,  -0.684309304,  0.770916939,  -0.250507146,  0.203105256,  -0.736640632,  -1.41802597,  -0.565400362,  1.17089117,  0.939626813,  0.56940794,  -0.419989079,  -1.16871142,  0.211898401,  -1.99686992,  0.660985708,  0.744596422,  0.721262336,  0.489441991,  0.968757689,  -0.175655723,  -2.38938594,  -0.80158478,  -0.0783786029,  1.4651047,  -0.256450772,  -0.383690149,  1.19480908,  -0.603173494,  0.00518903276,  -1.56011248,  -1.6057148,  -0.344575971,  0.423297077,  -0.591471255,  -0.282406867,  0.589867949,  -0.038994085,  -0.0703698397,  -0.803529203,  -0.344983041,  0.962006867,  -0.714232862,  -0.683871627,  1.33843553,  0.222318977,  -0.536338985,  -0.575311303,  0.311878502,  -1.29607928,  0.600286245,  0.983885825,  1.18962801,  -2.69087076,  0.28408125,  1.74417782,  0.899977207,  -0.512469888,  -0.0364443995,  1.36189413,  0.541172147},{ 0.347173452,  -1.74891424,  0.366702229,  0.043744158,  -0.321813256,  -0.00228603114,  -0.143512011,  -1.86551332,  0.222158656,  0.883811414,  0.281476229,  -0.418397307,  -1.32114875,  -0.887076199,  -2.77669978,  -0.744277596,  -0.118882939,  0.565206349,  -0.80740279,  1.10242677,  1.17360425,  0.0102082947,  0.647871435,  -1.60829413,  0.751343787,  -0.721891582,  1.3595717,  -1.39802086,  -0.268669277,  0.0218692161,  -0.173170477,  -0.154443324,  0.413496941,  -0.562773407,  1.23267877,  -0.193776995,  0.0101915635,  -0.284492761,  0.0285143144,  -0.679771721,  1.47214282,  0.633935571,  -0.456543237,  1.56439674,  -0.576029599,  1.48720312,  -0.688699782,  1.32021224,  -0.345056474,  -0.227175623,  0.901568413,  -0.845171988,  0.0892178714,  -1.72065496,  -0.278360069,  0.514657676,  -0.372748822,  0.0767561719,  0.325594723,  -0.309252292,  -0.171244815,  0.766674101,  -1.68385124,  0.162569061,  -0.221349701,  -1.48745978,  0.304045498,  0.459751427,  -0.495019257,  -0.0536879301,  1.2497344,  -0.243432507,  -0.859161317,  -0.571158528,  -0.90667671,  -0.234482467,  -0.826322556,  1.04394019},{ 0.181102902,  -0.202869937,  0.0789674893,  -0.633423924,  1.51094496,  0.712230623,  0.384669632,  -0.239572823,  -1.52304304,  -0.744342923,  0.340420872,  -0.0991923511,  1.97271204,  -0.774825394,  0.473645717,  -0.762135029,  0.56382668,  0.84551549,  -0.619588912,  -0.129819691,  -0.321145415,  -2.5753634,  0.162374035,  -0.75539875,  0.066070132,  0.171943903,  -0.298591435,  -1.49083853,  0.0172984209,  1.2973845,  0.951695561,  0.398310363,  -0.0206452794,  -0.861395061,  -0.38510114,  0.377732426,  0.175145969,  -0.0917456523,  0.377484232,  1.58353615,  -0.0138168829,  0.0620972328,  -0.0523704924,  1.10778785,  -1.6102252,  -1.21565342,  0.30038479,  -0.552628815,  0.0979516581,  0.36455211,  -1.0487684,  -1.28846049,  0.21684359,  -0.583860755,  -1.20139933,  -0.736878216,  0.10375791,  0.740737081,  1.71125662,  -0.982966006,  0.883996844,  -0.464170992,  1.0352999,  -0.147935554,  1.08119869,  1.52148163,  -0.782742739,  1.12656748,  -0.147005811,  0.348608434,  0.469178826,  -0.632259607,  0.382808745,  0.591413379,  2.22712278,  0.277336925,  0.195988193,  -0.687951207},{ 1.13996994,  0.368008852,  -0.834641218,  0.984290242,  0.100938626,  -0.268579483,  -0.51901263,  1.58254421,  0.63491565,  -0.385111481,  1.75335979,  0.00711235777,  2.11041951,  -1.39284837,  -1.17120492,  -0.723992646,  1.55825126,  -0.819906771,  0.598170757,  0.265901417,  -0.837775111,  0.676699519,  2.12165976,  0.79579258,  -1.99085557,  -1.97560096,  0.322570503,  0.390988708,  1.92397237,  1.50574481,  2.02142596,  -0.741650939,  -1.57612872,  1.46938539,  1.89989555,  -0.980727494,  1.00620711,  -0.684069216,  0.516904473,  0.264084458,  -0.647775233,  -0.108682945,  0.147439256,  0.140520275,  1.81391621,  -1.06403649,  -0.127030373,  -1.23469138,  -1.04249954,  -0.533070803,  0.283405989,  1.13228726,  0.925188243,  0.288026631,  1.03012741,  0.81508714,  -1.12651229,  -0.653009474,  -0.413412064,  -0.695948958,  -0.48601979,  0.228178784,  -0.120371237,  -0.837195992,  -2.02423644,  -1.23491383,  0.952037334,  0.364004135,  -1.50220275,  -0.431798667,  -0.416049451,  -2.29439902,  -1.35063994,  0.175411254,  -2.33283997,  1.14844525,  -0.130718663,  0.690130532},{ 1.81520796,  1.24119699,  0.145066321,  0.426136523,  0.262087792,  1.73642576,  0.820864618,  0.195837364,  0.157384783,  0.601363361,  0.527958453,  0.136430442,  0.572729945,  -1.14042616,  1.87826943,  -1.49056387,  0.959665358,  2.55740309,  0.484383613,  -1.03492284,  -1.26077771,  1.25783682,  0.535982907,  0.0372994803,  0.00534357876,  -0.712746143,  -0.110869989,  0.664290249,  -0.193609297,  -0.000323321845,  0.259956688,  -0.378617972,  1.80013037,  -0.382882297,  0.0827528313,  2.79580116,  -1.23883975,  -0.735780299,  -1.27069676,  1.36899686,  -1.70113635,  0.187737808,  0.636444032,  1.18219936,  -0.661262691,  -0.871612906,  0.145399153,  1.60193312,  -0.350118935,  0.737468004,  0.174758717,  0.89481777,  -0.242038146,  0.169784546,  1.12956548,  0.614541531,  -0.578970373,  -0.577630758,  -0.0270806793,  0.269532859,  -1.60232615,  0.900448859,  1.21353316,  0.800885379,  1.50398922,  -0.518088281,  0.806830645,  -1.27691948,  -1.37000811,  0.96789211,  0.508455455,  0.717388213,  1.80034268,  0.17897962,  1.65874898,  0.29663831,  0.0957228839,  -0.648346663},{ -0.464627951,  -0.0555719621,  1.58445358,  -0.869904935,  -1.13188195,  0.238292709,  0.245100126,  -0.38560009,  0.168997556,  -2.30668855,  0.0124196764,  0.452000171,  -0.728978515,  -1.25251961,  0.569241762,  -0.422663361,  -0.155135885,  -0.827724338,  0.828814089,  -1.11962855,  -1.80193555,  -1.59572053,  0.991768539,  0.954014301,  0.521872878,  0.245746613,  1.37353694,  0.0631252155,  -0.367484033,  -2.10856247,  2.50857663,  0.945907176,  1.45865941,  0.478572041,  0.565595925,  0.3867127,  0.827490926,  -0.8112275,  -0.110289328,  0.0148825832,  -0.0702940226,  1.40693521,  0.266808361,  0.100326665,  0.0188256092,  0.240347922,  0.307824731,  -0.573128402,  -0.449369878,  1.39254951,  -0.166630521,  2.68456221,  -0.768801868,  0.839969575,  0.000610213552,  -1.48458028,  -1.78028607,  0.530349016,  0.0454328358,  -0.279227346,  0.0852381811,  0.747829974,  1.3377924,  0.319690049,  -1.21836114,  1.88644791,  0.173340276,  0.00883063022,  -0.899255216,  2.17845488,  1.538903,  0.122318022,  3.07402921,  0.371988297,  -0.0479844362,  -1.60257387,  -0.311710179,  -0.75771606},{ -1.02357805,  -0.869705141,  0.801079988,  -0.498784095,  2.0249815,  1.11740434,  -0.352713466,  -0.752840221,  0.187873021,  -1.03989363,  0.115209721,  -1.18811214,  1.69785309,  0.936704636,  1.57637048,  -0.804322064,  -0.34387064,  0.520322382,  0.154650688,  -0.697302461,  -0.844485581,  -0.649681926,  0.708642423,  -2.13135982,  0.7274822,  1.2400831,  -0.271252066,  0.105295382,  1.54572928,  0.577706277,  0.0609889515,  -1.30964231,  -0.977691352,  0.784200311,  0.948118329,  1.08482027,  0.341724873,  0.239913955,  -1.06339979,  0.405056655,  0.523926318,  -1.45248723,  0.546080351,  -0.111430563,  1.55778837,  -0.350087196,  0.683758318,  1.35971379,  -1.18295944,  0.926925004,  -0.563162923,  -1.05573678,  0.30088681,  -0.509161174,  0.671228051,  1.92442739,  0.76850301,  -0.597191215,  -1.10190558,  2.39966989,  1.27353728,  0.108759627,  0.297023922,  0.540191531,  0.0683723986,  -0.79205966,  0.842878997,  0.745233297,  -0.263194263,  -2.1884706,  2.62528038,  -0.289496928,  -0.253942907,  2.30863786,  -0.995199561,  -0.858014286,  -1.82570279,  0.13715452},{ -0.750900924,  -0.238704041,  -0.1583758,  0.915276289,  0.198502079,  -0.468774766,  2.22912073,  0.858035088,  1.3472302,  0.857101262,  -0.297754288,  -0.831955731,  -0.299063325,  0.554600298,  -0.389843255,  0.515927196,  0.908483028,  1.60296738,  -0.38043353,  -1.42731023,  -0.923776448,  0.7069152,  -1.0117383,  -0.286700249,  -0.262388378,  -0.249588102,  0.471808255,  0.772756934,  -0.193459213,  0.174002111,  0.635428965,  0.620803595,  -0.906694055,  0.816195071,  -0.51228565,  -0.27636373,  -1.40728736,  1.55103981,  0.912165463,  1.30323815,  0.233875915,  0.430952221,  0.51404196,  1.12032199,  -0.655372024,  -0.716492593,  0.205904946,  0.26145786,  -1.75820208,  0.231201574,  -0.649236321,  -0.759763956,  0.606964052,  0.296659559,  -0.892682672,  -0.368808508,  -0.268896282,  -0.482456535,  0.699486613,  0.466497362,  2.30418468,  0.992487073,  -0.588880599,  3.122123,  1.06302464,  -1.12087893,  -0.135969117,  0.656699777,  0.208583087,  0.936568797,  0.0670448691,  0.70957756,  0.0688100085,  -1.25709999,  1.98757684,  -0.623405993,  -0.485530436,  -1.45471239},{ 0.502277017,  -1.16887903,  -0.808414221,  0.974964559,  0.1615206,  1.04460704,  -0.646253467,  -1.14570355,  0.272874475,  1.63621044,  -0.00340629695,  0.31379661,  0.650484562,  1.48234642,  2.15105247,  1.00502121,  -0.986742079,  -0.0120576676,  0.531021893,  0.477388918,  -0.0174665246,  -1.09992921,  -1.03187835,  -2.29346442,  0.712526798,  -0.456504017,  0.233115718,  0.599297762,  2.65390205,  -0.79396075,  -0.766688585,  0.851780057,  -0.405086696,  -0.630408645,  0.43203035,  -0.967635393,  0.488395005,  -1.95390868,  0.777989566,  -0.827027559,  0.0903827548,  -0.38738519,  0.739269555,  0.956495345,  0.295974046,  0.76911056,  0.373251498,  -0.771082819,  0.778064013,  -0.353885651,  0.361722231,  -0.204554036,  0.656674504,  0.343428195,  0.434849262,  -0.144244581,  0.467907339,  0.626359761,  0.656010747,  -0.67691946,  0.627230406,  0.564799428,  0.10675706,  1.04834139,  2.13332891,  -1.11729205,  -0.47652781,  -2.05705357,  -1.08570433,  0.829328179,  -0.620039999,  -1.00516045,  -0.470608115,  0.617266297,  -0.00414274912,  -0.263129652,  -0.729679704,  -0.0480154566},{ -0.388493568,  -0.161338672,  0.204359457,  -0.36635232,  -0.687554061,  -0.851249456,  -1.14069712,  -1.66933417,  -0.413900465,  2.17418766,  0.258319288,  -0.243203834,  -0.711237431,  1.16942751,  2.05831242,  -0.659732878,  0.690195739,  -1.10149813,  1.35683048,  -0.588723481,  -0.618140757,  -0.174943879,  0.567101955,  0.0550475456,  0.944746196,  0.0248210244,  -1.80579448,  0.0928826258,  -0.0734496787,  -1.11315489,  -1.81146789,  -0.903081596,  0.392288119,  -0.0695268065,  -0.859587312,  0.840267777,  1.91852832,  -0.185918733,  1.41175354,  -0.613355458,  0.944547176,  -1.60800052,  -1.07623434,  -1.35764396,  0.0725500509,  -0.451427728,  0.393893123,  -1.4154917,  -0.185118616,  -0.620178998,  -1.49876904,  1.02394891,  -0.579996347,  0.878790021,  1.47418272,  0.726539671,  0.0315120704,  0.115993284,  -0.427243114,  1.00035048,  -0.157064572,  0.293221951,  -1.48432446,  -2.70819211,  0.227214798,  0.443645209,  2.02816916,  0.430989116,  0.11617326,  -0.86946857,  -0.0507749543,  -0.710601032,  -1.18359303,  -1.38135839,  0.819041252,  0.543058395,  1.57312,  -1.14745522},{ -0.254392147,  0.487298518,  0.755078197,  -0.23919268,  -0.261506617,  1.01900613,  2.25837135,  2.09416962,  -0.370847732,  0.840930104,  0.771347284,  2.0294199,  1.52001286,  0.311933875,  0.139366224,  0.150369808,  0.731788218,  -1.22810125,  -0.577995598,  0.686775804,  1.10988736,  0.487465233,  -0.13383472,  -0.552725554,  0.978457272,  0.392871022,  -1.45314443,  -1.75623798,  0.61766398,  -0.173990041,  -0.486152053,  1.90997684,  1.19114721,  -0.495028615,  -0.264450431,  0.252003103,  0.0160084572,  0.50424099,  0.405757099,  -0.104412846,  0.0735481381,  -0.486641645,  1.3946842,  -0.726124704,  0.504760444,  -1.05127954,  0.580732524,  0.655286908,  1.38869071,  -1.04094791,  -0.777354419,  -1.57453477,  -0.24703519,  0.410467833,  -2.16399074,  0.308966815,  0.534788668,  -0.337748528,  -0.0751993135,  -1.38208735,  -0.80586499,  -1.08041334,  -0.886363626,  0.547650874,  0.455044121,  0.174492478,  -0.192407355,  2.04227567,  0.373751491,  -0.606944621,  0.405787796,  -0.63935113,  0.604063153,  0.153914973,  -0.731645942,  0.868901134,  -0.396984339,  -0.733005166},{ -1.48537385,  -1.44686675,  0.228411466,  0.428359061,  0.932112455,  -1.30808628,  -1.89314866,  -0.172126234,  1.65582848,  -0.422541142,  -0.920574665,  1.37237775,  -0.613468468,  0.736189544,  -0.326297432,  -0.0809111223,  -1.02953374,  -0.427701175,  -0.438212812,  1.02896178,  -1.48518741,  0.600819111,  -0.668501914,  1.40900469,  0.958101749,  -0.0707717612,  0.820299566,  1.59274542,  0.642937303,  2.05462813,  0.012377793,  -0.986474693,  -1.48239589,  -0.0289448574,  0.0731174201,  1.63712716,  0.979759336,  -1.27499163,  -1.21010828,  -0.699690819,  0.961522102,  -0.264915764,  -1.52604806,  0.711751282,  0.411277711,  0.0335460342,  2.04426765,  -0.511456251,  1.25145912,  0.249296054,  -0.863754451,  0.151842192,  2.33486629,  0.207897544,  0.557955921,  0.547949255,  1.07597125,  1.32014942,  0.754431486,  1.20679033,  0.844486117,  0.031853985,  0.513772488,  -1.51138699,  1.18287611,  0.457577884,  1.2382381,  -0.649281561,  0.425532639,  1.4883641,  0.826883674,  0.231322214,  -0.484811485,  -1.28911233,  0.430050045,  -0.0680304468,  -0.0923286304,  -0.155730918},{ 1.76209116,  0.583667397,  -1.40781999,  -1.55588186,  -0.527271509,  -0.287430614,  -2.06755519,  -1.6558615,  1.1027801,  2.1057713,  0.343144327,  0.892431617,  -0.00447618961,  1.69845045,  0.100985318,  0.0567285754,  -0.64105159,  0.635766625,  -1.38577294,  0.457753241,  -0.373738736,  -0.362437457,  -0.558385789,  -2.45192599,  -0.19436641,  -0.720120192,  1.17126143,  0.810354769,  -0.241914034,  -0.122957617,  1.13000727,  0.619161248,  -1.2602278,  -0.578180254,  -0.556849182,  1.5897094,  -0.0612804517,  1.73892725,  -0.599967241,  0.947644234,  1.14521408,  0.132508904,  -0.656782806,  1.25478792,  0.594006181,  1.02112842,  0.0596990325,  -0.140472114,  -0.00299571478,  0.268338561,  -1.45787156,  0.45158264,  0.0157385413,  -0.427061319,  -0.219346493,  -0.849915147,  -1.02051067,  0.183948636,  0.269983739,  0.113268688,  -0.288767397,  -0.648460865,  0.640979469,  -0.721682668,  0.40080291,  -1.08797503,  -0.860487819,  0.558877945,  -2.58904171,  -0.112655282,  0.561131179,  0.0299646016,  0.184275314,  1.02529347,  -1.09837294,  -0.115396343,  -0.187358856,  -0.676758587},{ -0.659874976,  -0.193645641,  0.501477063,  -0.640127599,  0.811887205,  -1.03118885,  1.70331705,  -0.996145725,  1.5289104,  2.20301151,  -0.865247548,  -0.958413899,  -0.0510512032,  0.544006884,  0.244756386,  -0.312670976,  -1.31110895,  0.904349864,  0.270907909,  -2.475986,  0.529673517,  1.81898403,  -1.09959126,  0.254360437,  0.782720327,  -0.461294532,  -0.512795806,  -0.436322838,  -2.25448132,  0.266169965,  1.55452895,  -0.0456301719,  1.8847512,  0.473284215,  0.673038363,  -0.230308384,  -1.92442441,  -1.6122824,  -1.84420311,  -0.404226184,  0.570746541,  2.89163995,  -1.60886872,  1.21018052,  -0.511421084,  -0.455056697,  0.772043169,  0.600478709,  0.459877253,  0.670143843,  -0.0647923872,  -0.0525228195,  -0.343925029,  1.85948014,  -0.980910897,  -1.17645025,  -0.989273846,  -0.570458055,  0.828918993,  0.16420123,  0.981610119,  0.482301593,  1.33476043,  -1.39359474,  -1.28942382,  -0.411894113,  1.11945748,  -0.0140538905,  0.954766631,  -1.78892589,  0.726262808,  -0.283753961,  0.335053414,  -0.218024597,  0.982361555,  -0.723297119,  -1.30218625,  -0.223041013},{ 0.122372203,  -0.693852842,  -2.35516429,  0.491661191,  -1.21193337,  -0.35354051,  -0.314118147,  1.21855128,  -0.389190435,  0.105122559,  2.00127792,  -0.957972169,  -1.58718014,  0.570397854,  0.370074272,  -1.90673244,  -1.42142665,  -2.85014987,  0.00917473342,  0.994773686,  0.168272585,  0.912791252,  0.280229479,  0.637789845,  -0.696744502,  -0.564732373,  -0.166999415,  -0.165804833,  -0.578486919,  0.0102847172,  1.16828275,  -0.645759463,  -0.684751093,  0.852817833,  0.280522764,  1.20453954,  1.10053372,  0.00270305644,  -0.868231833,  -0.372827828,  0.236552253,  -1.51944959,  1.72323453,  -0.563026667,  -0.85583061,  0.641864538,  -0.520208657,  0.87699759,  -0.682088733,  -0.561790884,  0.487173289,  0.460157305,  0.636620522,  -0.940274537,  -0.289366394,  -0.874560893,  -1.51953518,  0.439913899,  -1.72691917,  1.76054537,  -0.814358532,  -0.546870649,  -1.12791407,  0.147351354,  -0.0226775464,  -0.926015437,  0.53886342,  -0.015450988,  1.35766399,  0.520052373,  -1.22055674,  -0.331441849,  -1.04677057,  0.586592615,  -1.43647361,  1.16990125,  0.387864143,  -1.80392671},{ -1.62328374,  -0.386574388,  -0.646770477,  -0.318193942,  0.723467708,  -0.815693974,  -2.65427923,  -0.395912379,  -0.633423448,  -1.4674027,  0.749246061,  -0.14439562,  0.67034328,  0.2216665,  -0.866336286,  1.26505041,  -2.83034086,  -1.12734795,  0.387408644,  -0.315872967,  -1.43182361,  0.0727831423,  0.612828732,  0.904641986,  -1.22512972,  0.188165635,  -2.38250709,  0.87357372,  -0.972798347,  0.212619171,  0.364409536,  -1.66522002,  -0.975723147,  0.122397073,  -0.404941797,  -1.04630709,  -0.0273192935,  0.902555943,  0.120377183,  -0.259115487,  -0.389238238,  -0.0675279275,  0.941084087,  -0.802999258,  -1.39785671,  1.14739549,  -2.4047277,  0.497387946,  0.223669991,  -0.002647229,  0.045004461,  -0.466289312,  -0.637524843,  -0.946270466,  0.23027809,  -0.684384763,  0.246411741,  0.315472305,  -0.204231903,  0.667822361,  0.103793226,  -0.360041052,  -0.220071793,  0.535461187,  0.173628747,  -0.442838758,  1.48171282,  0.121096238,  -0.229062796,  -2.91981292,  -1.58206546,  -0.604884267,  -0.0720571727,  1.8928473,  1.06804717,  0.372261316,  0.27608189,  1.02833581},{ 0.660541534,  -0.893075168,  0.0149184717,  -0.00013501654,  0.675245464,  0.387054831,  -0.930985689,  -1.31484616,  0.959901154,  1.10380888,  0.709728241,  0.31751436,  0.308542758,  0.694040716,  -0.65429467,  0.700098395,  -0.060598027,  -0.940971851,  -0.211585075,  -0.245073438,  1.0458039,  1.36907721,  0.422866762,  -1.04023898,  -1.04826784,  -0.0952010006,  0.190503746,  1.23259807,  0.466754705,  1.11989367,  0.452510208,  -0.327738136,  -0.864859223,  -0.720032394,  -0.63325721,  -0.857827663,  -0.878724277,  0.000195758126,  -0.73548311,  -1.03410208,  -0.182071745,  0.0686197504,  -0.465374291,  -1.03137434,  1.0260042,  0.705163002,  1.99595642,  0.361789346,  -0.869532347,  2.20303035,  0.820777893,  -1.92282248,  -0.827221751,  -0.624292672,  -1.13103235,  1.00726628,  1.57428873,  0.112530142,  -0.102257013,  0.195095181,  -0.547394812,  0.715999424,  -0.845964849,  -1.77654564,  -1.11130869,  1.8566407,  -1.17792177,  -1.25814831,  0.753775299,  0.393375874,  1.26811182,  0.466642976,  0.0179791208,  -0.584061861,  -0.29339993,  -1.50386655,  -0.631991863,  -0.940599501},{ 0.477985561,  -0.395426691,  2.85275984,  -0.579726338,  -1.10596406,  -0.170681506,  -0.794202626,  1.16778827,  -0.355634749,  -1.1995194,  1.17859125,  0.641650856,  -1.8480227,  1.06046116,  -0.0220544729,  -2.28808761,  0.0721096918,  2.2760129,  -1.29361117,  0.504222572,  0.13236922,  0.373714596,  -0.361124575,  -0.360876769,  1.16763854,  -0.656924903,  -1.80823338,  1.06122768,  -0.983037651,  -0.251888067,  -1.0043422,  1.1554234,  -2.45532417,  1.20107996,  -1.16049254,  -0.559607387,  -0.050100483,  0.83946228,  0.254526943,  0.397150606,  -0.472655624,  -1.11088252,  -1.20348406,  1.02700543,  -0.550905406,  -0.103149965,  1.26730943,  -0.269020647,  -0.149870515,  -0.567312479,  0.221500129,  -0.185968295,  -0.530602276,  0.502691031,  1.71783149,  -0.265514076,  1.4136436,  -0.467674971,  -1.04673338,  -1.35179639,  -0.594029009,  -0.651289344,  1.81876266,  -1.29210651,  -1.12532532,  0.354686677,  0.677827001,  -0.476495445,  1.29604125,  -1.89778399,  -0.438526571,  -0.0558406785,  0.0974018648,  -1.12057698,  -1.01200426,  0.43843478,  0.248308614,  0.914812565},{ 0.0155732725,  0.64809233,  -1.87318003,  -0.914448619,  -0.0675154328,  1.53323281,  0.826092958,  0.345845729,  0.414710909,  -0.147430286,  1.09060991,  0.785953939,  -0.867877483,  -0.615032196,  0.147626847,  2.52259779,  -0.862550497,  -0.603022039,  0.839589119,  0.878642738,  0.0547768027,  1.16650438,  -0.840872347,  -0.7950629,  0.992348909,  0.482542455,  -2.01113057,  -1.21980131,  -0.473612696,  0.0409552939,  -2.55220151,  0.656865001,  0.178649187,  0.801589668,  -1.40954447,  0.763967335,  0.535748839,  0.0682409555,  -0.876756847,  -1.74946094,  -1.27679265,  0.469830364,  -1.30734873,  -0.804549396,  0.759746909,  1.03052616,  -1.52664292,  0.820743263,  0.480963916,  -2.20472813,  -0.293865055,  -0.702797353,  -1.47310734,  -0.806390524,  -0.934790671,  0.133690983,  -1.20210302,  1.21765339,  -1.90905368,  0.737356901,  0.0612358972,  -0.746842325,  -0.0734770223,  2.25580335,  0.921659529,  -1.40017796,  -0.381316781,  1.65184367,  0.903109908,  -1.75262034,  1.5973804,  -1.48814034,  0.879938841,  0.467340559,  -0.787811637,  0.212272704,  -0.219771847,  -0.759566426},{ 0.879277468,  -1.13903868,  0.929105997,  -0.191931024,  -0.75058496,  1.06431007,  0.25316602,  0.383600593,  -0.284776747,  -1.76300323,  -0.286965668,  -0.0301706214,  0.587868333,  -0.276857853,  1.27633595,  -0.263654321,  0.531511426,  0.0172099508,  -0.274639159,  -0.535286784,  0.132940456,  0.803073168,  -0.255036056,  3.58427715,  -1.53768897,  -0.123962358,  -1.01629961,  0.190415516,  -2.22499418,  -0.0523695312,  -0.873470485,  0.610572755,  0.489878029,  -0.249139905,  -1.05787039,  0.825118244,  -2.70574093,  -0.340524673,  -0.710166335,  0.497664332,  -1.16980016,  0.752918243,  -1.3388139,  -1.59781861,  -1.46103919,  0.227026671,  1.22371578,  0.288739473,  -0.188070565,  0.448010713,  0.419738233,  0.602716506,  -1.38841629,  -0.125267789,  2.23951006,  -0.329873204,  1.11494017,  -1.59694731,  -0.119905353,  0.0621328279,  -0.25283587,  -0.877099931,  0.2779724,  1.24125326,  -1.80219865,  1.35988379,  2.56791377,  -0.278840184,  -1.62970078,  -0.403428406,  1.04864478,  -0.0299906023,  1.26705897,  0.846987486,  -0.853881836,  0.0386129655,  1.30043244,  -1.63888502},{ -2.04775143,  1.47928441,  0.175683916,  0.00794931129,  -1.14336419,  -0.186937481,  1.42131841,  0.173516318,  1.24977481,  -1.69959331,  0.852330625,  -0.146737382,  -0.575423837,  0.799259901,  1.1051774,  -0.0950313881,  0.373349339,  1.12434852,  -0.620198727,  -0.0239921026,  -0.636959672,  -0.266449302,  0.107249349,  -0.231965601,  0.877707124,  0.762345076,  0.308110833,  0.056878075,  -0.41611281,  -1.30450976,  0.963628232,  0.994061172,  -1.17674363,  -0.841254771,  1.59758246,  -1.0972445,  -1.12679517,  0.194448113,  -0.243437558,  1.28751385,  -0.423702121,  -0.655164421,  0.639734983,  -0.243828192,  1.30903435,  0.364160985,  1.89109814,  0.314520001,  -1.43541729,  0.169690922,  -0.280525893,  -0.599541068,  -0.527542174,  0.125186488,  -1.57087898,  -0.866981387,  0.400271863,  0.481957197,  -0.616233468,  0.340958834,  0.0170403179,  -0.921376705,  -0.0936298147,  -0.850937486,  2.06401896,  -0.157847255,  0.217755347,  -0.542724013,  -0.0686936826,  -1.24061441,  -0.557992578,  0.227184266,  1.04820693,  1.24171638,  1.10847294,  1.20430934,  0.486352295,  -0.344288945},{ -0.701842546,  -0.38173604,  0.59495312,  0.0665591732,  -0.354062229,  0.329086393,  0.809019983,  -0.274111092,  0.902215898,  0.510243535,  -0.740684986,  -0.448448986,  -0.587339699,  -2.09287763,  1.47255826,  3.13934445,  -2.93473053,  0.770852625,  -0.366176188,  -0.98267442,  0.45018664,  -2.0242362,  0.0617898107,  1.53793907,  -1.91717029,  0.726618469,  0.0798196048,  -0.917014658,  -1.60558701,  0.978222787,  -0.381945431,  -1.35249364,  -0.591553628,  0.750374317,  -1.39738214,  -0.804855406,  -0.520345807,  -1.63923466,  -1.59471416,  -0.969072759,  0.59353286,  -0.900062978,  0.396176875,  1.23038745,  -2.01452613,  0.797584057,  -0.410416007,  0.530026257,  -0.936773419,  0.865828753,  -1.94105136,  2.04757786,  2.20536494,  0.104366258,  0.626306295,  -1.98835397,  0.397245198,  0.245058849,  0.929902554,  -0.130687013,  -2.07642817,  0.627686322,  1.30638552,  0.840042889,  -0.631221294,  1.91681659,  1.06508505,  0.644745171,  0.538979292,  0.463222504,  -1.99816024,  -0.0806654692,  -1.28024185,  -0.356672913,  -0.557498455,  -1.33695078,  -0.0623505451,  -1.88525677},{ 0.318521738,  0.946231246,  1.23872936,  0.743638515,  -1.67802835,  -0.626036584,  -1.13336873,  -0.303815782,  -1.10972917,  -0.114081912,  0.390989363,  0.19696638,  1.96962011,  0.399477243,  0.496161312,  -0.812756002,  0.804311812,  1.51657438,  -0.153002128,  -1.384956,  -2.10298228,  0.398301572,  -0.583385825,  -0.437140077,  -2.2739892,  0.343400389,  0.218669713,  0.80993098,  -1.8622092,  0.0201265663,  -0.167485684,  1.90121102,  0.744057178,  -0.144153729,  1.04295361,  0.576046586,  1.93029702,  1.59794581,  1.48266983,  -0.374846548,  1.33851743,  1.21592057,  0.476427674,  -0.576898694,  1.06690967,  0.682775259,  -1.51964808,  0.366741449,  -1.27147305,  0.156654626,  2.63780236,  0.602025688,  1.50710928,  -0.260625005,  0.50054729,  -0.331573606,  0.0267161652,  1.41404676,  0.936968982,  -1.0757103,  -0.491004616,  -0.614421129,  1.73030424,  -1.3992188,  -0.860980749,  0.563149631,  0.923972726,  0.339322865,  0.585434079,  -0.82989943,  0.967080414,  -1.7281518,  -0.815010667,  1.28204978,  -1.52150178,  0.657244086,  -1.95315468,  -1.64478993},{ -0.732614815,  -0.120594665,  1.05683303,  0.642684639,  1.8185761,  0.677604914,  0.75958544,  -0.294125736,  -1.43540752,  2.52644873,  -0.471009314,  0.961546779,  1.38564003,  -1.08326972,  0.659410775,  0.166956544,  0.106742889,  -1.11875093,  0.889155805,  1.22207701,  -0.908978641,  0.208308473,  -0.546975553,  0.770478606,  0.156889409,  1.5547837,  2.37560177,  -0.104963921,  0.507250309,  0.848489583,  0.501131594,  0.840436876,  1.40010643,  -0.0469449945,  -1.07234132,  -0.229079276,  0.570185721,  0.0473258756,  -0.677788854,  0.670336843,  1.61980093,  -0.301553011,  0.640319407,  1.24877834,  -0.0665191114,  -0.738384426,  1.24438357,  -0.997119367,  0.761504233,  0.598918557,  0.94731015,  0.135840043,  0.41770643,  1.05219936,  -0.0510244705,  -0.556574345,  1.68334806,  0.00515681459,  0.76228267,  -0.593818665,  1.58269393,  -0.986462533,  0.931160092,  0.970581412,  -0.461553872,  0.294866025,  -0.0255837049,  -1.5910908,  0.24165839,  -0.484805942,  0.165898502,  2.52733159,  -0.536168039,  1.16864967,  0.96267271,  -1.53582132,  -2.31585312,  0.996820331},{ -0.347342938,  -0.522075355,  0.295829684,  -2.3615191,  0.46114248,  -0.083067663,  0.131692246,  1.8427577,  -0.545159638,  -0.694093525,  1.03746426,  -1.35549533,  1.24924672,  -0.127793878,  -1.54985797,  -1.08184159,  0.716110706,  0.289821863,  -2.02484775,  1.63871217,  0.204972461,  -0.206646323,  -0.349161506,  -1.51087832,  -0.211086512,  -1.27624881,  2.68038487,  -1.22662628,  -0.00876456406,  -1.49598241,  -0.458313823,  -0.275287867,  0.733769417,  0.0937522873,  1.55850804,  0.691657007,  -1.35691953,  0.696129382,  -0.793475509,  -1.23796964,  0.977305233,  -0.276822418,  0.18343012,  -0.0577878617,  -0.130417556,  0.561244249,  -1.73876774,  -1.47480392,  0.00353999506,  -0.950138152,  -0.735292256,  1.79622769,  0.839534879,  1.19167888,  0.136996478,  1.12348509,  -0.0540245883,  -0.965811908,  0.382327974,  1.78073847,  0.0131086707,  0.508014441,  -0.625963032,  1.19580352,  0.0677238181,  0.723392308,  1.9966588,  0.276616007,  -1.43047285,  0.324241072,  -2.38153625,  0.00680389907,  1.79657841,  -1.30540657,  -1.79719555,  0.625886261,  -0.445035249,  0.235927537},{ -1.79064846,  1.11189783,  1.01890767,  1.26441348,  0.0653790161,  0.784044325,  0.225663707,  -0.891246796,  1.03961158,  -0.691240311,  -1.15023994,  -0.102942407,  0.607397795,  0.0666366443,  0.269645482,  -0.912293553,  -2.20856285,  -0.0616851375,  1.8343612,  -0.00960699655,  -1.82962191,  -1.33155203,  -2.04433918,  0.76705569,  1.53327072,  -1.31180203,  -0.845880508,  0.0324151292,  0.931101084,  -0.939609528,  1.15255868,  -0.248655155,  0.060393665,  0.773965597,  -0.834997773,  -0.0276924111,  0.303756744,  1.50310147,  -0.708495975,  -0.23048532,  -0.999821842,  0.160550222,  0.0689697638,  0.893992782,  0.537067711,  2.07373929,  1.49180901,  1.62452078,  -0.635810733,  0.353964299,  -0.747251213,  -2.16864109,  -0.864893436,  0.158814937,  -0.085105747,  -0.409946978,  -0.789668024,  -1.25393987,  0.331764609,  -1.31113291,  -2.13833714,  2.17994785,  0.396213949,  1.74812031,  -1.27106428,  0.605811238,  -0.541511238,  0.0569588095,  -1.37465143,  -0.553365827,  -1.0465734,  1.59881437,  -0.42501685,  2.26505542,  -1.50228345,  -0.509855688,  -0.93742919,  0.569293559},{ 1.05247355,  -0.640237033,  -0.656244516,  0.360503525,  -0.644368947,  1.35981679,  -0.319460124,  -0.691098154,  -1.23091066,  -0.618305147,  0.845157743,  -1.25746667,  -1.70502353,  -1.44410086,  -0.263400912,  1.11424136,  -1.73714709,  -0.561633885,  -0.674710095,  -0.360696733,  -0.436728477,  0.992871284,  0.255033195,  -0.0154752964,  -1.30384159,  -0.302913547,  1.752756,  -0.423598975,  -0.660835564,  -1.53246403,  0.2667059,  0.521296084,  -0.944588184,  -0.133155853,  -1.18507409,  0.277839005,  0.810427725,  0.265381455,  -1.24417102,  1.25383914,  1.12260032,  -1.49234486,  0.240517229,  -0.372279972,  -1.1227026,  0.179010838,  -0.90553236,  0.80293256,  0.753751934,  -0.833072245,  1.20626986,  0.304496735,  -0.427709043,  -0.591320992,  -0.913764238,  -2.11905551,  -0.777162313,  -2.67698956,  0.950304031,  -0.409709245,  0.629028618,  -0.175590038,  0.196335733,  -0.184325546,  1.07679403,  -1.34140205,  1.30123627,  0.305674434,  -0.913060844,  0.328413039,  -0.858306527,  0.00377201615,  0.565184891,  1.57883132,  -0.199568063,  0.909744978,  2.05749583,  -0.247898817},{ -0.730132341,  0.879010499,  0.341029018,  -0.885059774,  0.461556882,  1.14024425,  0.061354503,  0.180786818,  -0.417617232,  1.70253873,  -0.253149837,  1.00577414,  -2.27757525,  -0.34164089,  0.361660689,  -0.158560365,  -0.0941562355,  -0.757855237,  -0.230639666,  0.960397005,  -1.22000039,  0.719944477,  0.356503367,  0.09512683,  -0.925055981,  -0.574460626,  1.31074119,  -0.143363506,  -0.546747208,  0.432965517,  -0.515728891,  0.0203541275,  0.596569777,  0.00848455075,  0.61804235,  -0.919668198,  -0.178449482,  0.248530239,  0.20692645,  -0.753430724,  -1.47812557,  -1.00189054,  0.438665956,  0.670219541,  1.38253129,  0.749869108,  0.096359618,  -1.51798236,  1.39266169,  -0.79503864,  1.86417329,  0.390050858,  -1.44715917,  -0.221641824,  0.0736329406,  0.144943416,  1.12904167,  -1.9238528,  -0.630662501,  0.405877054,  -0.219932273,  -1.36469662,  1.290609,  -1.2331295,  0.354848742,  -0.301425934,  -0.0836741105,  0.268099099,  -0.65749532,  0.199999675,  0.517836869,  -0.529645264,  1.40623474,  -0.0267271344,  1.47040701,  0.374481708,  -0.962454021,  -1.34000266},{ 0.943452716,  2.01122713,  -0.427659363,  -0.368530899,  0.0357638784,  0.691524804,  0.126523569,  -0.674880564,  -1.13415325,  -0.572996914,  -0.292980015,  0.496645987,  -0.129349589,  0.550535798,  -1.14070976,  0.0520982668,  1.29000652,  1.77847791,  -3.60079312,  -0.345276952,  0.497413427,  0.577111065,  -0.428567261,  -0.0685039014,  0.845267653,  1.44658434,  0.302570909,  -0.208403885,  -0.0463018715,  0.904027104,  -1.27785718,  0.276849955,  -0.30003798,  -0.796830118,  0.154159516,  -0.73346144,  -0.57689929,  0.371019393,  -0.895038307,  -0.936049223,  0.422683626,  -0.164269269,  -1.16397536,  -0.133312523,  -0.980068386,  0.627264857,  1.22390127,  0.00150875328,  -0.476484179,  -1.92144096,  -0.885851622,  0.224445671,  0.432812452,  0.142257005,  -3.59988117,  -0.397660345,  0.398929685,  0.335345209,  1.76352155,  -0.900815904,  0.485174984,  -0.679037571,  -0.255128384,  -1.32707524,  0.33471486,  0.568926215,  0.757252038,  -0.00530546531,  1.59661615,  -1.83794105,  1.30333912,  -0.456449419,  -0.884699881,  0.190134764,  0.251030505,  1.0553,  1.02185762,  0.164255008},{ -0.254549742,  0.208838344,  -0.123051532,  -1.92825198,  -0.480338275,  0.507217884,  0.833737969,  -0.047094658,  0.622449756,  -0.549366951,  -0.575886428,  -1.39891899,  -0.987013936,  -1.00885046,  -0.424592555,  1.33255112,  1.4042542,  -0.513473272,  0.966008663,  -1.24040115,  -0.715088725,  1.6153692,  0.166369423,  0.864378631,  -0.581904829,  -1.83900762,  -0.230458751,  -0.860886455,  -0.219483122,  -0.157956302,  0.640134513,  -1.7943809,  0.00107972184,  -0.575045466,  -0.374667138,  -1.09962332,  -0.547676682,  -0.597615957,  -0.0157115515,  -0.165417254,  -1.38782144,  0.801762402,  0.525700212,  1.11184609,  0.623274565,  0.112981439,  -0.108804837,  0.638130844,  -0.869583726,  0.944346726,  0.55722481,  -1.49725497,  0.880751371,  -0.614584029,  -0.779195428,  1.5706352,  -0.400348097,  0.568061173,  0.848134041,  -0.450954676,  1.03145468,  -1.11586821,  -0.0967435092,  -0.104377642,  -1.60171974,  0.116635926,  -0.0260744467,  -1.45334983,  0.126984671,  0.57017833,  0.340570658,  0.74646455,  1.80700672,  -1.36605775,  0.66854322,  -1.73825622,  0.166967288,  -1.22100139},{ -0.255540669,  -1.40244985,  -0.802676499,  0.228592798,  -0.130317464,  -1.50456786,  1.30330205,  0.424131066,  1.11702967,  0.552966118,  -0.886894345,  2.23241019,  -0.29116407,  -1.4438566,  0.0862762555,  1.85317552,  0.821467042,  0.249905393,  0.338988036,  -0.808653891,  -1.13749099,  0.71002084,  2.07889104,  -0.494558752,  -0.185141459,  0.25457567,  -0.236724645,  0.667830229,  0.904882848,  1.83340561,  -0.498373181,  0.61978066,  -0.327777207,  0.87762773,  0.6816324,  -0.670329213,  1.97570646,  0.97337985,  1.33268547,  0.0408294015,  0.744025052,  -1.78152716,  -0.0502308793,  1.14274991,  -0.358858615,  -0.0474149585,  -0.51493746,  -0.131103486,  -0.310381979,  -1.42335045,  -0.805017114,  -1.05124426,  -0.393259436,  0.627946138,  1.13555264,  0.412371427,  -0.250307739,  -0.867099643,  0.00595435686,  0.864761114,  -3.7122519,  2.18460155,  0.0020678821,  -0.393805891,  2.15428281,  -1.94642806,  1.71249413,  2.87181926,  -1.55785894,  1.82324576,  0.458107471,  0.0453092083,  -0.225011021,  -0.0187670588,  -1.59356606,  -0.393492728,  -0.180882201,  1.0402},{ -0.268453866,  -0.247344211,  -0.192725241,  -0.0263265315,  0.812919319,  -1.16085899,  -1.88758326,  -0.399820447,  0.206909105,  -1.33926213,  0.0312915407,  0.0895054638,  -0.217414156,  0.541203439,  -0.346707433,  -0.541618049,  0.642947674,  1.46807969,  1.38379383,  -0.300252199,  -0.157975405,  0.827150643,  -0.420885146,  -0.632449806,  1.10076094,  1.71232629,  0.955568433,  -0.880492628,  -0.843084753,  0.762135923,  -0.529235184,  0.489977419,  1.17221999,  -1.69669628,  -0.396082789,  -0.178596511,  0.0310484152,  -1.19113374,  0.146695331,  -0.333060145,  -1.49857473,  0.885800362,  -0.399614543,  -2.10039377,  -0.472202063,  -0.61608386,  -0.124404803,  -0.0517448932,  -1.55502892,  1.52317619,  -1.34986842,  3.08834434,  0.191825822,  2.11984038,  -0.307340205,  0.806410789,  0.240147635,  -0.051517304,  -0.355322093,  1.31374466,  1.28718841,  -0.407676548,  0.271171927,  -0.55182451,  1.29393792,  0.0423938222,  -1.3238225,  -1.26181161,  1.21686566,  0.107811145,  0.749483347,  0.642208278,  -0.773215175,  -1.49339747,  1.32154977,  1.85627604,  0.143837988,  0.299821019},{ 0.91253227,  -1.34365523,  -1.24238122,  0.91127193,  -0.403915435,  -0.226235941,  -0.426944911,  -2.02574396,  -0.261675775,  -0.776737332,  -0.167029351,  0.126173034,  -0.0712657496,  0.322410852,  0.263200998,  0.572352886,  -0.847991884,  -1.17699492,  0.123083726,  -0.754121125,  -1.61282218,  -0.40525794,  -0.220402256,  -1.62622344,  -0.345094264,  1.93009675,  0.452574044,  0.208201796,  -1.9716804,  -1.10859513,  -0.622570157,  -2.08593082,  -0.834772348,  -0.608917534,  0.449862957,  1.07837319,  -0.0203037243,  1.12461519,  1.47545958,  2.24879742,  1.03681719,  0.166710317,  -1.51336074,  -0.108856283,  0.65631181,  0.48101747,  1.1139394,  -0.414898962,  -0.333279788,  -0.32013914,  -0.641149402,  -0.391049504,  0.355312258,  -0.163729936,  -0.505403638,  0.439750105,  -1.07366502,  -1.35260773,  1.75921679,  -1.92058241,  -0.572050035,  0.977981746,  -0.922702909,  0.358656853,  -0.192847297,  1.01465774,  -0.494946092,  -1.03263962,  -0.945022941,  0.110773161,  -2.34386992,  -1.03181946,  -1.10707772,  -0.726920843,  -1.14014781,  0.296651542,  0.614881754,  -0.123593293},{ 1.12062836,  -0.63932699,  -0.988462925,  0.139226675,  0.547267616,  0.169120684,  -1.36948991,  -2.20525861,  0.300770968,  -0.772970974,  -1.71156955,  -0.323732644,  -2.30734181,  1.09279144,  -0.452816129,  0.637464046,  1.47500646,  0.393501639,  0.302913785,  -0.428724259,  -1.25199747,  -0.667351484,  -1.31651044,  0.0206282176,  0.198283419,  -0.10952577,  1.06429887,  -1.40978181,  -0.720144153,  0.656486154,  -0.226394832,  -2.02349782,  0.6371122,  1.79636919,  -1.75264668,  0.203845382,  0.595350325,  -1.61455786,  -1.2321173,  0.533051372,  0.595827997,  0.610392511,  0.0400099009,  2.6724515,  0.14544788,  1.11330879,  0.0992270038,  0.791943729,  -0.696919858,  -0.033327058,  -0.127898321,  -1.30786955,  -1.13264239,  -0.372282565,  -1.00320995,  0.617723644,  -0.80815196,  0.249641746,  1.11995244,  -0.259605765,  0.894878089,  -0.954843044,  0.849416554,  1.24579859,  0.763549447,  -2.17722273,  0.479587793,  -0.985989988,  -0.532697856,  -0.40348196,  -0.581414759,  0.108382545,  0.985958934,  0.322905689,  1.45337367,  0.314485759,  -1.11309683,  -0.568248034},{ -0.480192989,  -1.57284331,  -0.36013788,  -0.0204962455,  -0.87874639,  0.517492712,  1.24533641,  1.06813931,  2.14776778,  0.0507432483,  1.21846461,  0.02515091,  0.218267485,  -1.17349827,  0.921468675,  -1.56054127,  0.391799092,  0.368335873,  -0.949706554,  -0.519625485,  -0.00805145036,  -1.29445565,  0.647443175,  -0.896582127,  -0.873380423,  -1.36921465,  2.5874083,  -0.610225499,  0.736557305,  1.41497278,  0.390106052,  -0.25822255,  -0.424764961,  -1.07291615,  -0.799584091,  -1.77728212,  0.750085056,  -1.43220115,  -0.574650288,  -1.08269393,  0.904627562,  1.07037127,  -0.264750063,  0.584542036,  -0.0186196435,  -0.791667402,  -0.089873068,  -0.309484661,  0.227230892,  1.34992874,  0.866904378,  0.101991497,  0.66264683,  -0.593422651,  -0.0789680704,  -0.229418397,  0.830559731,  -0.442483753,  2.33362174,  -1.0344069,  0.903627753,  -1.20046413,  -1.80006528,  1.19578874,  0.362244159,  -0.743994117,  -0.729615808,  -1.04757905,  -0.558828652,  -0.585814178,  1.03389144,  -0.068007566,  0.481898338,  -1.79368317,  0.346650541,  -0.194455504,  0.371498436,  0.826128781},{ 1.34043932,  0.875893235,  0.323174864,  0.946338713,  -0.676671803,  0.208799616,  -0.269280702,  -1.09384882,  -0.157901332,  -1.47851658,  -1.77406168,  -0.828944266,  1.28753793,  -0.419342816,  -0.247706518,  0.180655316,  -0.745170414,  -0.563944042,  -0.0711242259,  -0.528553069,  -0.465925604,  -0.13317208,  -1.15512431,  1.17945266,  -0.404529244,  0.420819014,  0.154866248,  0.835460782,  -1.67623436,  0.0405512266,  0.719360232,  -1.23278701,  -0.72298795,  -1.06067526,  0.524091482,  -0.811562121,  -1.86568141,  -0.762737572,  -0.296878755,  1.67843199,  0.152819037,  0.0756260529,  -0.308925122,  0.683450222,  -1.93146694,  0.559117675,  0.0440917313,  0.0342481621,  -0.374919564,  0.55869323,  1.06727636,  -0.709522188,  -1.24803042,  0.618790507,  0.593565881,  0.157200307,  -0.598891199,  1.01083434,  1.84059346,  0.56761086,  0.343438894,  -0.0710742623,  2.01315093,  0.30893746,  -1.80047667,  0.458468765,  -1.53454804,  0.420118153,  0.533635616,  0.389954209,  -0.0741387457,  0.498636007,  -2.08113027,  1.81426001,  -0.00602426054,  -1.35767233,  0.447291285,  2.66005206},{ 0.485706359,  2.07357025,  -1.34048975,  -0.0366946384,  -0.521411598,  -1.79972529,  1.21321619,  0.838392019,  -0.377461851,  -0.566125035,  -1.14192343,  -0.569380939,  -1.74676657,  0.944004357,  -0.445722669,  -0.71214956,  -0.950111985,  -0.445138097,  1.24598908,  0.795954466,  -2.04560328,  0.871708751,  2.06894445,  -0.189589188,  0.92358005,  0.0240828171,  -1.27313101,  -1.02436352,  -0.354760557,  -0.0776842833,  0.464057952,  0.0671249777,  0.253875285,  0.523924172,  2.00602102,  0.349090487,  0.145286992,  0.367284477,  -0.781888604,  0.461554229,  -0.799384773,  -1.32039952,  -0.830918074,  0.104180172,  -0.14020519,  0.731987596,  -2.24481392,  -0.8529917,  1.85734761,  -2.06431103,  -0.115110308,  -2.5974195,  0.723121405,  0.0511817075,  0.197552159,  -0.288001001,  -1.30016363,  0.16531311,  -0.590376198,  -0.391207606,  -1.3258909,  0.0576174408,  0.858259201,  -0.866922796,  -0.695152581,  -1.85233831,  0.944417596,  0.43561542,  0.628408253,  1.77927363,  -0.882825851,  0.81621021,  0.859004796,  1.17323959,  -0.567255318,  -0.149319857,  0.386375755,  -0.527847707},{ 0.257231802,  0.485633343,  -0.45139572,  -0.0829174593,  0.077241376,  0.598152876,  -1.40676403,  -0.652956843,  -0.928967714,  0.508046389,  0.328889638,  0.100559607,  -0.0906056538,  0.161246672,  -0.840846598,  -0.877448559,  0.184261635,  0.181992784,  -0.933238208,  -1.89028955,  1.24778342,  -0.482095748,  -0.289341569,  0.957121193,  -0.666733682,  1.28527868,  -0.32479322,  0.86836499,  -0.205113739,  -0.126563326,  0.605114222,  1.73418772,  -0.726627827,  0.565209031,  0.414295733,  -1.00237143,  1.63751245,  -0.515308201,  0.621614397,  -0.285377085,  1.25931907,  -0.54738915,  -1.20635736,  -1.7818507,  0.493885159,  0.144041762,  -0.3880063,  -0.402479112,  -0.357273281,  -0.438710481,  -0.26393199,  -0.965069175,  0.710879445,  -0.277341843,  0.881864548,  -0.0519729592,  0.602505624,  0.101145655,  0.617592096,  0.418568164,  0.944411397,  -0.589476585,  1.74688971,  0.512047708,  1.08293867,  -0.503352344,  -1.25528491,  -0.099090077,  0.485127151,  0.237041011,  -1.18757975,  -1.08556819,  0.181364402,  -0.25783968,  0.4217529,  0.875560462,  -2.17614365,  0.186217204},{ -0.0180083625,  -0.59250772,  0.428726703,  -0.459392011,  -0.630415082,  1.20707488,  0.0792746246,  -0.440978408,  -0.300724983,  -0.493665814,  -0.466091365,  1.52017736,  -2.07632494,  0.576891661,  -1.07367182,  -1.13903153,  -1.90095782,  -1.69397616,  0.194669992,  0.941996813,  0.436749429,  -0.799205363,  0.257067412,  0.844367206,  -0.187426209,  0.689139187,  1.03050864,  -0.123130433,  2.35951376,  -0.25500077,  -0.310166806,  -0.350348413,  0.0356667973,  0.346253276,  0.41245684,  -1.75893724,  0.230412439,  -0.656624079,  0.393267751,  -0.045960594,  0.118950166,  0.880066335,  0.632787764,  1.59252226,  0.82209909,  0.26439485,  0.574845016,  0.207340539,  0.157850191,  1.19679189,  -0.650835156,  -0.0301534291,  0.56858784,  -0.670010328,  -0.784463108,  -0.182796359,  1.94191945,  -0.155772403,  0.028700294,  0.271737754,  -0.14178279,  0.00228656433,  -0.525802851,  0.05158972,  1.63404596,  0.191296905,  -0.462385267,  1.18011963,  0.15166162,  0.0169746112,  -0.547820389,  -0.476902992,  -0.85721153,  0.11542964,  -0.0449075773,  1.367625,  -0.840921938,  -0.798766077},{ 1.71306705,  -0.0689865276,  -1.86206567,  -0.669908464,  0.462688386,  -0.456602931,  0.539117813,  0.0891657919,  -0.34701851,  1.91519082,  -1.44638467,  0.0709191337,  -0.430619657,  0.0313831531,  -0.767257035,  0.240492195,  0.950022757,  0.966435134,  0.0908009484,  1.1230396,  -1.33114302,  -0.0803038627,  -0.134985015,  0.202283829,  0.246040002,  1.50277853,  0.0284703728,  1.89049017,  -0.149022415,  1.66470885,  -0.774098217,  0.513104975,  0.410835743,  -0.490844131,  -1.14856172,  -1.09611118,  -1.32069576,  1.18555975,  0.883974433,  -0.0496281758,  0.807906568,  1.49302053,  -1.16419101,  1.03927314,  0.755760014,  1.03394485,  -0.692092896,  -1.1398201,  0.371726602,  0.0527141392,  0.164068788,  -1.63673389,  -0.577124476,  0.762735069,  0.943445504,  -0.81258744,  1.25611138,  0.652695239,  0.925735116,  -0.317254722,  0.502450228,  0.362205565,  0.310012847,  -0.152994037,  -0.122037433,  -0.0521611571,  -1.1822418,  1.22992873,  -2.10275269,  -0.151145414,  1.37883031,  1.1521095,  -0.619019628,  0.538644552,  -1.65652585,  -1.14386809,  -0.795265138,  -0.370844543},{ 0.296570748,  0.0866894722,  -1.87660384,  -0.148831412,  -1.35812593,  -0.904021442,  -1.01976597,  1.11089933,  -0.484931469,  -0.0308093131,  -0.354032308,  -0.0916375518,  0.773642361,  1.62265015,  -0.291045845,  -0.50009656,  -2.35107374,  -0.193577468,  0.957080603,  -0.202456161,  -2.31485653,  0.0103764869,  -0.619154155,  -0.657156646,  1.58113205,  -1.10657525,  -0.312148213,  0.689917326,  1.53151035,  1.44561672,  -0.985036314,  1.0555079,  -2.06212807,  -1.02012682,  -0.515645921,  -1.32666361,  -0.14603366,  -0.72808677,  1.23669541,  -0.33791253,  0.703218102,  -0.660617054,  -0.810106874,  -0.0214801431,  -0.291531116,  -0.236862913,  -0.562480628,  -0.688210309,  0.0641079172,  -1.59749961,  0.135789663,  0.859777987,  -0.936480582,  0.427493423,  0.739971578,  -0.457084924,  0.238137826,  -0.32759285,  -0.492332458,  0.845527947,  0.177237242,  0.325776786,  -0.0784922764,  2.05757046,  -0.256036162,  -0.728480279,  -1.5284605,  -0.178971335,  0.350151777,  0.683327019,  -1.26752615,  0.667092144,  0.0349429771,  -0.102727599,  0.00196767389,  1.08255899,  -0.579918623,  0.06597583},{ 1.09831977,  -0.379914552,  -1.33937907,  0.956732213,  -0.336211771,  1.1138258,  -0.649910033,  -1.73051178,  -1.1814003,  2.1160233,  1.22898817,  0.281392843,  -0.306184381,  0.298279762,  -0.493059218,  0.324041396,  -0.164696619,  0.800693452,  -0.939845681,  -0.780024469,  0.0985226482,  0.629606783,  -0.295545012,  -0.963263929,  -1.25526106,  0.849610627,  0.298016965,  1.58690476,  0.487163097,  1.72138369,  0.0684718341,  0.992886484,  0.668553293,  2.43936563,  0.556232154,  1.07203853,  -0.302908033,  -0.127470896,  -0.953142166,  1.16852868,  -1.62977242,  1.10396707,  1.3315326,  -1.26507747,  0.781772971,  -0.401324868,  0.0792580619,  0.29986918,  0.284889013,  0.660325527,  0.809827745,  -1.59525537,  0.0561894663,  -0.501857877,  0.750485182,  1.10764623,  0.23793675,  -0.0882303268,  -1.81407332,  -0.0953177661,  1.37632489,  0.885123372,  -0.13394253,  0.629331827,  -0.801965177,  0.208680779,  -0.967449605,  -0.668618858,  -1.84897113,  0.400506884,  -0.7605654,  -1.43616068,  0.0801499709,  1.66364598,  0.545398712,  -0.808311522,  1.16963828,  -0.279495776},{ 0.97334379,  1.20924389,  0.65607655,  0.633340836,  -0.0700820833,  0.0538717732,  -1.20415103,  -0.875042677,  -2.23214674,  -0.524993598,  -1.31041443,  -0.316125453,  -0.161012158,  1.2167666,  -0.667317331,  1.38998449,  1.10961199,  -1.51092279,  0.105808988,  0.86571604,  0.199643895,  0.355265021,  0.47270599,  -0.956670761,  0.486004055,  -0.332322061,  0.749660134,  -1.49868786,  1.4441421,  -0.395200014,  0.468387693,  -0.30745852,  -0.0560856499,  -0.471965343,  1.08735716,  -0.850804687,  1.26325166,  0.181994036,  0.952839375,  0.344224066,  -0.779507458,  -0.705544829,  0.0245231315,  -0.0869099647,  -0.5648458,  0.479643017,  0.684696615,  0.227813348,  1.73892868,  -2.50768352,  -0.746913791,  -0.166121751,  -0.345559567,  0.42945233,  0.183175996,  0.265568882,  -0.205250904,  0.760390282,  -1.60888827,  1.32839119,  0.504275799,  0.668405533,  -1.34965742,  2.73142576,  -1.72336411,  -1.30021036,  -1.5104897,  1.20716405,  1.0640341,  -0.782560885,  0.326514095,  0.0482753403,  -0.45607087,  -0.828314424,  -1.62222338,  -1.39296734,  0.513795137,  2.107795},{ -0.996560812,  -0.33871147,  -0.494136006,  0.997813702,  0.457934648,  0.454299748,  0.989390671,  -0.487353593,  -1.74232554,  -0.639889956,  -0.792819738,  0.841357052,  -0.304875731,  0.686404228,  -0.50161314,  1.29753697,  -0.444435656,  0.30353722,  -0.508677483,  -0.635517716,  -1.97907019,  0.0745126233,  0.973303795,  -0.889403939,  0.360520601,  -2.87068439,  -0.12626268,  -1.05955422,  0.309576064,  2.06540799,  -1.08812177,  1.19089806,  -0.373921782,  -1.05557907,  1.78316247,  0.955019474,  1.21083486,  -1.58649349,  0.425989836,  0.327012837,  0.315637201,  -1.13625515,  -2.13807416,  1.34625661,  -0.990679026,  -1.17916703,  0.284260809,  1.35877168,  0.073777549,  -0.273243755,  0.319460452,  -0.678007126,  0.902567625,  -0.578562617,  -0.595090926,  0.210425511,  1.40588558,  0.202232704,  -0.137354106,  -0.273857564,  1.55754411,  -0.276227474,  -0.468498111,  -1.68849277,  0.65143472,  -0.417916894,  -0.569796741,  -0.675680935,  -0.708846569,  1.57296062,  0.142777532,  -0.181671485,  1.04549587,  0.173075557,  -0.313290566,  -1.06666338,  1.66053939,  2.23354149},{ 0.122134104,  1.26911056,  0.478824764,  0.379460216,  -0.117152356,  0.638082683,  -0.871143043,  2.08580112,  1.05415344,  0.419108033,  2.13624501,  -0.510910153,  0.673512161,  1.66715956,  -0.241028622,  -0.0989296138,  -0.686837852,  -2.25727057,  0.837143123,  -0.946129799,  0.125629768,  -0.384246737,  0.870127439,  -1.10237157,  -0.0377894565,  -0.0588349253,  -0.346729577,  0.606143534,  0.710527658,  1.88994372,  -0.94861418,  -0.360648662,  -1.65165102,  -0.372788996,  -0.158529714,  -0.195463642,  0.264499933,  -0.239215225,  0.0332194455,  0.99896735,  0.427398235,  -0.315348029,  -1.14971352,  -0.813737988,  0.514551342,  -1.46776497,  -0.330750287,  0.545745075,  -1.29594111,  -0.278837532,  -1.59820843,  0.44022432,  1.71381581,  -0.305691719,  -0.281887263,  1.28303301,  0.114653952,  -1.04137027,  1.39066708,  -0.756010354,  -0.300174147,  -0.141704753,  -0.66602087,  -0.746388912,  -0.56590879,  -2.03797936,  1.25346577,  -1.29071403,  1.07541275,  -0.76190865,  -0.0860336572,  -0.0270771701,  0.293527752,  -1.2267611,  1.09802544,  -0.282515645,  0.856117845,  -0.521146536},{ -3.00381637,  0.0759068802,  -0.178318366,  2.09718776,  0.221320003,  1.23995304,  -0.0239563584,  -1.45370483,  0.926199973,  -0.459267825,  -0.59452343,  0.827594578,  -0.972550511,  -0.784278989,  -1.07337844,  -0.289532185,  0.486114264,  -0.498796999,  -0.705149889,  -1.05149364,  0.153248385,  0.0502449125,  1.60107553,  -0.104113504,  0.17144227,  -0.329212755,  -0.77494067,  2.05771375,  0.673723757,  0.225836694,  1.25331855,  -0.560717881,  0.241548404,  -1.60639083,  -0.689721644,  0.734539211,  1.25416553,  -0.5039801,  1.65763295,  -0.465399027,  1.14582217,  -0.0322598554,  -0.535904527,  -2.27518463,  0.0451034866,  1.06719649,  1.2794286,  0.225446597,  -0.113167651,  -0.972642303,  -1.0979805,  0.507945359,  1.5245769,  0.146681055,  0.0357733257,  -2.39348054,  0.311830878,  -0.140789405,  1.02836597,  -0.951948881,  -0.798152626,  0.403790176,  1.4632746,  -1.37895107,  -1.00193059,  0.576889455,  1.27324426,  -0.340742975,  -0.0768620893,  -0.151100531,  1.75383365,  -0.581053078,  0.485420197,  0.633664429,  1.26923311,  0.512068272,  -0.483706117,  -1.55893171},{ -0.709986448,  0.012691604,  -3.01510191,  -0.00631295238,  1.93287313,  0.453682065,  1.34864855,  0.480627477,  -1.49640357,  0.00544313481,  0.997682333,  1.27790129,  0.0598958991,  0.30328989,  -0.469455242,  -0.113942355,  -0.711241603,  -1.98252785,  -0.222859949,  -0.881628931,  -1.17588007,  0.623879969,  0.117644824,  -2.24237919,  -1.43435955,  1.25219572,  0.993587196,  -1.70503032,  -1.4385668,  1.4981674,  1.4872719,  -0.649372756,  -1.29297578,  -0.45044294,  1.38405812,  1.08362651,  -1.39719725,  0.100665301,  -0.149506226,  0.0692086741,  -2.19771814,  -1.50347424,  -1.13040733,  1.01786411,  0.0461657122,  1.22989321,  -0.012356204,  3.01412058,  -1.49673486,  0.0934302434,  1.63152504,  -0.417618066,  0.828543842,  0.615988374,  1.47091591,  -0.702295423,  -0.503499746,  1.11784172,  -1.49336112,  0.645766854,  -0.00520379283,  0.398550391,  -1.31918895,  -0.476702094,  -1.14106774,  0.324923396,  -0.164397433,  -0.511885047,  -1.35905623,  0.732453763,  -0.709078968,  -0.137780324,  0.450686485,  -0.550514519,  1.00536072,  0.460986108,  0.401788831,  -0.137027144},{ 1.18002605,  1.13958478,  -0.690491021,  0.332756728,  1.48808849,  1.15162563,  0.862587512,  -0.651966453,  -0.455000758,  0.306145668,  0.568182349,  0.34542191,  -0.413742512,  0.389702678,  -0.611078382,  1.45722127,  0.343490422,  0.703507185,  0.421765476,  0.32377246,  -0.19650504,  1.14048719,  0.0146630835,  -0.435233593,  1.07757235,  -0.598547339,  0.711616874,  -0.370831698,  0.667734563,  0.379258454,  0.876268685,  -0.851777077,  0.819941878,  0.257852674,  -0.275279462,  -2.11041856,  -1.09654534,  0.0541408584,  1.77017438,  -1.16807628,  1.08509278,  0.0199501403,  -0.136332139,  0.42621541,  -0.572645009,  -0.441249222,  0.762173831,  0.219319478,  0.239734322,  -0.346468151,  -1.94412613,  2.19726729,  1.49723566,  0.405920535,  -2.29675579,  -1.25430739,  -1.08959484,  0.224020168,  1.07568157,  -0.577871621,  1.02244496,  -0.702358186,  -3.12549853,  -0.880710304,  1.12579632,  1.64165616,  -0.373273581,  0.638944209,  -0.452573597,  0.496356219,  -0.287331849,  0.48076576,  2.70380068,  0.155801758,  -0.912406266,  0.748128533,  -0.0788433701,  1.82228851},{ -0.676106155,  0.659172177,  -0.901813567,  -0.105491832,  -0.3483603,  1.04515934,  -0.184665799,  0.151866063,  2.75665975,  -1.73019814,  -1.62325859,  -0.35458672,  -0.470594853,  1.63877046,  0.460158408,  -1.18970561,  1.30506599,  0.0897240266,  -1.21199679,  0.877196372,  -0.216111541,  -0.541049302,  -1.57156694,  0.179212987,  -0.766616702,  0.985565603,  -1.29091167,  0.232701138,  0.306772321,  -2.00600696,  -0.292552739,  0.0186788626,  -1.00167763,  -0.813524067,  -0.209345967,  -0.858727992,  -1.79433584,  -1.67746592,  -1.62155402,  0.790300429,  -0.0444127508,  -0.103043459,  0.239980608,  -0.624062121,  -0.554141164,  -0.321615815,  0.461912423,  0.563637674,  1.09744787,  0.85031563,  -1.22812045,  1.63998485,  -0.377640605,  -1.68997979,  -0.16710408,  1.17358589,  0.462207347,  -0.265002996,  0.274054527,  -0.15394251,  1.43777442,  -0.562864304,  0.257875264,  -1.18802476,  -0.238201022,  1.28890216,  -1.27702904,  -0.847083867,  -0.0515674315,  1.87335682,  0.297019124,  -0.46525991,  0.0241002943,  0.207376286,  -0.0393518731,  0.190447286,  -0.0107659353,  -1.48372674},{ 0.601159096,  -1.66247737,  -0.298949957,  2.36073542,  0.683294058,  0.812805295,  0.122635208,  -0.0541189201,  1.22049391,  -0.887621045,  -1.26663697,  -1.24852574,  0.583034933,  -0.285932481,  0.278817356,  0.368290782,  1.14120424,  -1.10335445,  1.39056587,  -1.0193789,  0.0635101274,  -0.450985968,  -0.3572644,  0.318222672,  -0.17759192,  -0.197238639,  -0.289220929,  -1.45103621,  0.637967229,  0.133928224,  -0.677799821,  -0.868284106,  0.0441386029,  -0.297754854,  1.95281219,  -1.08987033,  1.55312252,  1.15585494,  0.745171845,  0.931098282,  -0.455633342,  -0.611493707,  -1.61198306,  0.2572335,  0.62877512,  0.660218418,  -0.93838197,  -1.61805046,  0.191144779,  -0.667860568,  1.16092312,  0.123295821,  1.89973962,  -1.18067145,  -0.0287874527,  -0.123496108,  -1.42507482,  0.763757467,  0.345090777,  -0.20368959,  -0.40959534,  0.00929221604,  1.27553558,  1.48999393,  -0.270314693,  0.184808671,  -1.15839958,  0.337382674,  0.267371505,  -1.23152208,  -0.55179143,  1.61519623,  -1.50233197,  -0.505748153,  -1.17816794,  0.145925269,  0.737413585,  0.260013819},{ 0.0126896165,  -0.882300973,  -1.71290064,  -0.34359917,  -0.498432755,  0.236011922,  -1.45692682,  1.83020508,  1.41569996,  -0.285402179,  -1.30090237,  -0.132787302,  -1.96063995,  0.884525061,  0.211128458,  -0.333531737,  0.346591592,  1.26576018,  -1.85085976,  -0.413105994,  -0.0820807889,  -0.232095912,  1.18315494,  -0.216009229,  -0.677107036,  1.87244916,  0.562662959,  1.62519455,  0.846159458,  0.673662424,  1.66857934,  1.25574589,  -0.348130256,  -1.82055473,  1.61437249,  0.708008468,  0.523962021,  1.11396623,  -0.331392676,  0.581099689,  1.95036209,  -1.25900209,  0.487309009,  -0.341949522,  0.33219707,  -0.271326393,  0.104429886,  -1.05821645,  -0.593567133,  -0.470211476,  0.537706614,  1.98297882,  -2.13990903,  0.591268063,  -0.37054944,  -0.0994123816,  -1.34365916,  -0.474707723,  1.06617832,  -0.497320473,  -1.58758712,  -0.380705208,  0.880586505,  -0.89176476,  -1.23582399,  0.21217899,  -2.37922168,  0.800197184,  -0.0892231986,  0.424253792,  -1.05855548,  -1.818663,  -1.83998895,  0.329569757,  1.21728694,  -0.258099318,  -0.789571583,  0.141416222},{ -0.43342939,  0.32332328,  0.230851024,  -0.0509046614,  0.812548935,  0.487812072,  -1.27984476,  -0.773102641,  -2.62711811,  -0.049967818,  1.38855803,  0.455599248,  0.138619244,  0.948936164,  -0.90434587,  1.04010189,  0.364926964,  -0.0103465216,  0.38551566,  1.11456931,  -1.5095644,  0.593184412,  -0.663730919,  0.750237346,  -0.281235039,  -0.606641054,  -0.714553654,  1.96366107,  -0.927766979,  -2.48980832,  -1.02524209,  -0.732125521,  -0.828157008,  -1.17481947,  -0.0310841482,  -0.103991784,  -2.33965445,  0.0454460941,  -1.53802431,  -0.422490627,  -0.399436831,  -0.0884429216,  -0.964972615,  -0.571149707,  -1.05575061,  -1.54152393,  2.26058459,  -0.222140923,  -1.34676015,  -2.32349944,  -1.34840226,  0.827433825,  1.60927427,  -1.01332784,  -1.06072605,  0.178575352,  0.402399153,  0.380255103,  0.952090561,  0.950009644,  -1.09238327,  -2.45953774,  -0.26485014,  0.278734207,  -0.194792807,  0.742930293,  -1.20327711,  -1.72403514,  -1.50209534,  -1.0299387,  -0.0650170073,  -0.720935881,  0.525938272,  -0.210085034,  0.25969246,  0.0991887227,  -1.05228412,  -0.510827184},{ -1.28175139,  0.679611385,  -3.20701408,  0.407169551,  1.37433648,  0.873681784,  0.503637493,  -2.19229031,  1.95624971,  -0.430283755,  0.217993796,  0.0396013409,  0.671979606,  -0.5172773,  -1.06661379,  -0.625865877,  0.0700461939,  0.864363492,  0.293005735,  0.220624045,  1.14325273,  -0.35336861,  1.17618573,  -0.309608132,  0.261279434,  1.31625926,  0.807767689,  -0.802087545,  -1.30517769,  -0.437542796,  0.280012935,  0.132005319,  -0.88682574,  -2.11703038,  0.939254105,  0.787875772,  1.20527685,  1.33919156,  -0.604127824,  -0.331811637,  0.589153528,  1.48438799,  -0.339636505,  -0.625790894,  -0.380312711,  0.997810364,  1.65289116,  -0.0210988577,  -0.637781441,  -2.01681852,  -0.495317906,  0.0127637377,  -1.75932086,  -0.0272401534,  -0.569122493,  0.560168505,  -1.67329907,  1.15551198,  0.0156276897,  -0.309034288,  -1.04448307,  0.493979722,  0.811320126,  0.699037611,  0.0937949866,  -0.668160439,  -0.378882527,  -1.36164415,  -0.378306657,  0.379661828,  0.988949895,  1.03732669,  0.674448013,  -1.08046901,  -1.07506645,  -0.235490635,  0.0121743893,  0.891392589},{ -0.0268549547,  0.622401059,  -1.6765976,  0.322853833,  -0.885469854,  -0.0206127055,  0.407843858,  -0.979918063,  0.309672713,  -2.20411086,  -1.5521636,  0.962140024,  -0.884940147,  0.317913949,  1.58163679,  1.30218637,  0.726764977,  0.164139748,  1.30418682,  -1.58325207,  2.75066924,  -1.11126423,  -0.712875962,  -0.269558609,  -0.70703882,  -0.0418042243,  0.502969921,  1.93072116,  2.59994864,  -0.400242716,  1.3928802,  0.636197448,  0.303653717,  0.166446224,  -0.986081958,  1.18194604,  0.962233603,  0.0761346817,  0.302543521,  -1.7248559,  -1.33441222,  0.0841129571,  -0.627935529,  0.45494929,  1.58818209,  0.624138176,  1.07873428,  1.11278069,  -1.2865144,  0.33096981,  0.722543478,  0.354216546,  0.274572045,  -0.346918106,  -0.0211496893,  1.2405864,  -0.0344836451,  1.36562777,  0.127080426,  -0.329188645,  -0.361211091,  0.963053048,  -0.52243185,  0.392373621,  0.579368234,  0.417785555,  -0.392101288,  -0.76784122,  -1.98859978,  -0.475539297,  0.141998604,  0.392991483,  -0.917427599,  -0.988331139,  -0.507837057,  0.918826461,  0.560597718,  -0.386482},{ 2.40337968,  0.883951485,  1.54506016,  0.181413695,  -0.531826675,  -1.43500531,  1.12925041,  0.125150457,  -1.14878905,  0.657434642,  -0.903428853,  -0.204140946,  -0.275360018,  -0.57885325,  -0.819779992,  2.50804257,  0.493784994,  -0.584205806,  -0.896301806,  1.24673057,  -0.449475884,  -0.0230629351,  -0.860247552,  -1.40577269,  0.591300189,  0.158009067,  -0.394436449,  -2.30967569,  -1.59669065,  -0.402651399,  0.125730649,  1.24684799,  -0.546662688,  -0.437215775,  -0.721652806,  0.163172007,  -0.655585885,  -1.91640854,  -0.613865077,  1.89892578,  0.125362098,  -1.31408286,  2.07507396,  -0.558400869,  -0.1829101,  -0.626291335,  0.05719411,  1.3189044,  -0.129579023,  -1.13645816,  -1.24181962,  1.35877395,  1.33522856,  1.48823774,  0.651662111,  -1.25881612,  -1.56226027,  0.79711163,  0.177054778,  -0.39219135,  -1.46495986,  -0.259218961,  0.170074999,  -1.35707581,  -0.260284007,  -1.06698585,  -1.40282869,  -0.269871175,  -1.24877381,  -1.30783713,  -0.116751581,  -0.879603624,  -1.16504467,  0.120427497,  0.468908757,  0.154312685,  0.596375942,  -1.05698073},{ 0.575520694,  1.01093113,  -1.1822089,  0.494463474,  -1.24940288,  0.647358716,  -0.100948602,  1.3152051,  1.08883512,  -0.433330834,  -0.347391218,  -1.72656727,  -0.355013728,  -0.0896055996,  2.12169647,  -0.611525059,  1.03919125,  -0.184155241,  0.896811962,  1.34924829,  0.322230011,  -0.490421981,  -0.325413048,  1.00758827,  -1.0054903,  -1.46126473,  -1.04383004,  1.63522375,  0.401732713,  0.720571756,  -0.873278439,  0.63752377,  -3.01869798,  0.61399591,  -0.0691934451,  0.249709979,  -0.66777426,  0.984053433,  -0.838374436,  -0.0160672031,  1.38395596,  -0.838100731,  -0.129191875,  0.0371508673,  -0.319377691,  -2.40446639,  0.59843874,  1.03330851,  1.31066251,  0.803388655,  -0.334398568,  -0.246136069,  -0.38910532,  0.284810036,  0.0852366239,  -0.636941314,  0.73216033,  0.326459438,  0.345637113,  -0.936726987,  -0.3936508,  -0.67058146,  0.295056373,  -1.37019563,  -0.41151607,  -0.432126909,  -0.173180073,  0.433346719,  -0.474928975,  1.483513,  0.617600799,  1.16499579,  0.741108596,  0.304521233,  -0.349791378,  -0.829065561,  1.34468007,  0.863354385},{ 0.144885898,  -1.60960782,  0.821802437,  1.48119044,  0.304190069,  -0.125763431,  -2.60975194,  0.928354502,  1.06903434,  -1.0157342,  -1.07508063,  0.00823836308,  -0.348158062,  0.840009868,  -0.321556151,  -1.30210364,  0.0283690598,  -1.16515255,  -0.73232305,  -0.763369679,  -1.45278132,  0.373171896,  -1.1009649,  -1.90024531,  0.0583473891,  0.863999605,  0.881287456,  0.890396833,  -0.0263449438,  -0.531059861,  0.0940426216,  0.275816321,  -1.63100493,  -0.355968773,  1.11569893,  0.602968574,  -0.120628312,  0.328561723,  0.387300879,  0.778932989,  1.38356113,  -2.26819348,  0.503413677,  0.651063323,  -0.502566516,  1.04370809,  -2.51276135,  0.514020562,  0.458470494,  -0.0405342802,  2.08639359,  -1.23291838,  1.19078386,  1.0812794,  -0.802832603,  0.100357637,  -0.722729146,  -1.87971222,  2.4880712,  2.42729783,  1.48153639,  -0.116307452,  -0.108390503,  0.893257141,  -1.67663765,  0.183523774,  0.230929866,  -1.00594723,  0.107203737,  -0.553321302,  0.777975917,  -0.674525023,  -0.613160908,  -1.41425788,  0.0532454252,  -0.711360633,  -0.906082213,  -0.262883276},{ -1.48340774,  -1.14174497,  -0.925966859,  0.612928808,  -1.45133746,  -1.7839762,  0.828798711,  -0.428633213,  0.101698883,  -1.52572608,  1.81088924,  1.53971326,  -0.539931774,  1.26875949,  0.743456006,  0.74669534,  1.41562212,  1.34982669,  0.892532885,  -0.0289987214,  0.390555769,  -2.44102836,  -0.658971786,  -0.263895988,  -0.609115958,  1.71107948,  0.948633611,  0.860756874,  0.340216279,  -0.664880633,  0.0400314592,  -0.595064819,  1.52844739,  0.163498476,  -1.88683522,  -0.0625526607,  0.35533759,  0.230907857,  0.19373624,  1.169801,  -0.00054423057,  -0.271869242,  0.835081875,  -0.275978059,  -0.964529157,  0.266927749,  0.457532555,  0.148247421,  -0.309505463,  0.688488245,  -0.933819413,  0.109764859,  0.0218400471,  1.00508237,  0.715065718,  0.993093431,  -0.372909933,  0.162362918,  -1.60969102,  -1.50099885,  1.9884975,  -0.91060549,  -1.10541654,  0.367438436,  0.0586928725,  -0.0253959615,  0.335110337,  1.0465169,  -2.32043004,  -1.50412714,  -0.363389254,  -0.932996988,  -1.03834069,  0.151415855,  -0.962059975,  -0.881930172,  -0.547699392,  -1.58277905},{ 0.373188674,  0.926416636,  -0.912458241,  -0.547807217,  0.868351817,  -0.181280211,  -0.885223687,  0.211199403,  0.47868374,  1.10179996,  0.396765858,  0.209545776,  0.164171651,  0.786192656,  -1.17268121,  1.14439738,  -0.498770028,  2.48717117,  0.206577629,  -0.466765851,  1.04823565,  0.486599565,  -0.28267473,  0.38382712,  1.93236876,  -0.837933004,  -0.493532568,  1.49356174,  -1.1307888,  -0.527626514,  1.2150681,  -0.919309199,  -0.852694273,  2.08201218,  1.70444989,  0.129969418,  0.225983575,  0.478366196,  -0.490647256,  -0.0561526343,  0.135298431,  -0.234731361,  -0.164597511,  -0.724248469,  -0.360882968,  2.39778209,  -0.343590975,  -0.529818952,  -0.0115738055,  -0.0553138591,  -0.581736386,  -1.5860765,  1.03832388,  1.8626771,  -1.09486604,  0.735031784,  -1.4612509,  0.531418741,  -2.74273968,  -1.27399182,  0.451873332,  0.233865529,  0.889163613,  0.844701529,  -0.787224174,  0.784968376,  0.835188389,  -0.0684304833,  0.900522768,  -1.29615664,  0.63289088,  0.00940397196,  2.01342678,  -0.155723676,  -0.500236869,  0.805342197,  -0.950497508,  -0.438608348},{ -1.44341528,  1.28292787,  -1.47122788,  -1.23303246,  1.66856217,  -1.68491971,  0.923164308,  -0.0800826773,  0.61825794,  0.847097993,  -0.71103549,  -0.0490169525,  -1.13556409,  0.342759073,  0.87668401,  0.732923448,  0.296520323,  0.991604328,  1.70605004,  -1.21116209,  -1.63767314,  -1.46775842,  0.195530742,  0.00870318618,  -1.89830101,  -1.95857227,  0.101186186,  -0.342009872,  1.59149897,  -1.13760722,  1.09861434,  -1.12065375,  -0.815428436,  0.573986053,  0.32547307,  0.0616181418,  1.01779938,  1.74617708,  -0.0768114552,  2.15960836,  0.355323374,  1.95057964,  1.70606267,  -0.0145043414,  -1.13627338,  0.252311438,  -0.356458247,  -1.91463184,  1.16896415,  1.54180956,  -0.249470994,  -0.400179118,  0.20763202,  -0.470381737,  0.830549836,  0.915289938,  -0.0177392773,  0.203092307,  -0.662116349,  -0.516103208,  1.21583247,  -1.15088618,  1.53554082,  0.874539375,  -0.239395082,  -1.298558,  -1.92130029,  0.0413091443,  0.304602146,  -0.880991876,  0.683481157,  0.389832079,  -0.945016563,  1.0417012,  -0.25667116,  0.278866082,  2.09158468,  -1.43653727},{ 1.04265296,  0.337148368,  1.00226438,  0.863148332,  -1.0791744,  -1.45905268,  -1.66467643,  -0.807998121,  -0.994926274,  0.30338639,  0.271951765,  1.84496415,  -0.502966225,  -1.1249193,  0.107679814,  0.0579781942,  0.7305094,  -0.268748462,  0.296255589,  -0.617143214,  -1.96667492,  -0.520488858,  1.07006681,  0.161841527,  1.19509554,  -1.1602962,  1.36555922,  -1.55273664,  0.372225463,  0.310971081,  0.287263751,  0.37943688,  -0.213666931,  0.0845619366,  -0.762055814,  -1.38119578,  1.6837405,  1.21988893,  -3.3870635,  -1.39012313,  -0.399261892,  1.04479277,  -1.02618849,  -0.748657644,  -0.593717456,  -0.833698869,  -0.601381719,  0.24385573,  1.60213447,  1.38252997,  0.962673664,  -0.332205325,  0.228183195,  -0.055479303,  1.13130629,  -1.14571404,  1.15481198,  -1.48473537,  1.88089085,  -0.718182743,  -0.484296173,  -0.710317194,  -0.445049852,  -2.12955737,  -0.74115473,  1.19022298,  0.133178115,  0.615931451,  -0.700697482,  -1.45787203,  -0.791480243,  -0.940497577,  1.89778602,  1.14802027,  0.184350818,  -1.83958077,  1.75817049,  0.40980041},{ -0.610807061,  -0.446032315,  -1.17242193,  -1.26402736,  -0.0604654625,  -0.318665326,  -0.516601443,  -0.118438892,  -0.902911246,  0.14606525,  -0.450315267,  0.476752371,  0.264136642,  -1.34885466,  2.33086252,  -1.41652763,  1.99436545,  0.790462732,  -0.0674079284,  0.602983117,  1.45309663,  -1.08899379,  0.607562423,  -0.498275101,  0.213038251,  0.485998631,  2.38565588,  -1.22286177,  -0.509432673,  0.169811189,  -0.650627017,  -0.965782464,  -0.525895894,  -1.41070855,  -0.264420211,  -1.94918668,  -1.35713685,  0.0474898815,  -0.850348532,  -0.634723186,  1.99685252,  3.02571321,  -1.49889195,  -0.685856819,  0.068237707,  -0.490864128,  -0.434934229,  1.68746173,  2.42610908,  0.486195982,  -0.15718469,  0.650803566,  0.34752807,  -0.183279157,  -0.534334183,  -0.184626564,  -1.11807406,  0.171765164,  -0.128754795,  -0.581925869,  -0.822888315,  -0.01002602,  0.952714086,  -1.03522027,  0.750255764,  -0.770798743,  0.132816821,  0.572987974,  0.108009152,  -0.169107378,  0.345324606,  1.20330489,  -0.0412221663,  0.491580039,  0.253150016,  -0.758190155,  -0.588800073,  -0.894921899},{ 0.408340335,  -1.05017495,  -0.564569831,  -0.280963093,  1.40778649,  1.96818399,  0.551277101,  1.02870417,  -0.185949251,  -1.13312948,  1.12840271,  0.073575981,  -0.359429717,  -0.845790744,  -1.14311969,  -0.134556457,  0.351076007,  0.826398194,  -0.873534679,  0.681091964,  -1.08768964,  -0.31446144,  -1.85171664,  0.154432356,  0.193127871,  0.301391423,  0.884061337,  -0.485478282,  0.949528039,  0.764935076,  1.52647018,  -2.17157197,  0.882769585,  0.279978156,  -0.469728291,  0.0133563271,  1.54985523,  0.182097763,  -0.272094905,  -0.06734249,  -0.679220021,  1.53384626,  0.959577858,  0.399099618,  0.734908223,  -1.07361662,  0.06377884,  -2.27231216,  -0.490326166,  -0.509233117,  1.04099298,  1.41155326,  -0.632449627,  -0.404487669,  -1.30359924,  0.00597921107,  0.548409462,  -0.126661882,  0.567837656,  1.45330739,  -0.952537298,  0.682318568,  0.284002572,  1.10994947,  -0.0571749881,  1.24604928,  0.190566018,  0.671742916,  -0.139226392,  0.63547349,  0.700139701,  -0.433762461,  1.00893152,  -0.396295637,  0.201524645,  1.08547449,  0.485708624,  0.389033735},{ -0.651857376,  1.50465214,  1.36100817,  -0.343510211,  1.50066948,  0.825615287,  0.438308209,  -0.810815394,  0.1993687,  1.04688048,  0.88300401,  -1.03281808,  0.182255968,  0.0218660906,  1.88655448,  -2.05506516,  0.714774013,  1.433851,  0.960368812,  -0.175060213,  -1.23295212,  1.70208538,  -0.332161278,  -1.02821791,  0.533502698,  -1.53800392,  2.1509378,  -0.287512183,  -0.13210921,  0.737208188,  -0.00413848786,  1.32358432,  0.28938511,  -0.691565871,  -0.312498927,  -1.92672956,  -0.0598433055,  -0.546197832,  -0.606632054,  0.439321518,  0.340632021,  0.396456152,  -1.61177838,  -0.683616996,  -0.304588795,  3.28583336,  -0.923876166,  0.656050801,  -0.243610561,  1.23694003,  0.839010179,  0.923150778,  -0.400332928,  -0.254501194,  -0.534006476,  0.34321931,  -1.16227221,  -0.0357351042,  -0.617034853,  0.51537478,  -0.845057011,  -1.04558611,  0.754756391,  -0.122992054,  0.433151692,  0.602001071,  -1.95074952,  0.140884459,  -0.0422236137,  0.943837225,  0.757987976,  -0.0865297467,  0.0115818046,  -1.51493192,  1.5157131,  -0.343557715,  0.790452063,  -0.0913420022},{ -0.305028379,  -1.16235232,  0.279084921,  0.536039948,  2.2867074,  -0.708375335,  0.449955434,  -0.345932186,  -2.43653083,  1.3428365,  2.65417027,  1.62750363,  1.26835048,  1.17479014,  0.631766856,  -0.0802708566,  0.205078751,  1.04969037,  0.0837715119,  0.405884504,  -1.17938089,  -0.325380445,  -0.0912490115,  1.2540586,  -1.30309284,  -3.24013329,  0.540832162,  -0.164793745,  -1.72943544,  -1.89124358,  0.884176612,  1.06497443,  -0.653417945,  -0.238201365,  -1.10820436,  0.320738971,  0.798584461,  -1.66065192,  1.27428472,  -0.0370862111,  -2.21161771,  -1.75006771,  0.0697429031,  0.481073499,  2.13136196,  -1.35063601,  -2.94354844,  -0.0450505875,  1.08043075,  0.1826033,  0.697235107,  -0.647332966,  -0.505642831,  -0.327434272,  0.0584852062,  -1.02913034,  0.244246662,  0.864241898,  0.45840767,  -1.67170656,  2.56599212,  1.50884962,  0.113382041,  -1.20240247,  -0.458675921,  -0.189089462,  1.17263365,  -0.955907047,  0.766375065,  0.515756965,  0.648914695,  -1.60634124,  -0.530592203,  -1.83072305,  1.57057714,  0.188986361,  0.847645462,  -0.483707964},{ -0.707338333,  0.0421651267,  0.298931271,  -0.0723367706,  1.13851094,  -0.00614679884,  0.428555548,  -1.66596746,  0.60730207,  0.288592309,  -2.12419271,  1.20055568,  -0.47007823,  1.0269295,  0.144663915,  -0.305082232,  1.19939351,  -0.0639457926,  -1.51725805,  -0.827146709,  -0.823120356,  0.651898682,  0.0378514305,  -0.777045786,  0.7901057,  -0.921160638,  -0.588540375,  -1.17637682,  -0.900888562,  -0.120472021,  0.881813943,  0.165471271,  0.697183788,  -0.964607716,  1.03092015,  0.0720853508,  -0.44047603,  -0.633824646,  1.06823468,  -0.568718195,  0.869630516,  -0.525419235,  0.679028332,  0.0975081623,  -1.1796875,  1.61968327,  -0.0737016574,  -0.435097098,  -1.43740249,  -0.362062305,  -0.645175338,  1.35701406,  0.651046216,  0.0999001339,  0.0112743536,  -1.50527585,  0.417662829,  -1.09456909,  -0.223080412,  -0.343824029,  1.18017006,  0.401549518,  -0.121774077,  2.71777558,  0.760507762,  1.02930427,  -0.0372795835,  0.479802042,  -1.5991087,  -0.296242476,  1.60872221,  0.826895237,  1.03043091,  0.869594634,  -0.626734018,  -0.834432781,  -0.712373316,  -0.206456035},{ -0.361870706,  1.05219042,  -0.0323375501,  -1.87915635,  -1.45346856,  -0.417717785,  -0.209106848,  1.3718642,  0.130482465,  1.01163566,  -1.50472939,  -1.57600129,  0.43429631,  0.75748837,  0.166685298,  0.114326626,  0.343621969,  0.481747329,  -1.61447859,  -0.60433501,  0.823190987,  0.892660201,  1.64751458,  0.0558281094,  -1.09162915,  2.59263897,  1.57810462,  0.091270186,  2.02119875,  -1.29051054,  0.144387662,  -0.229006648,  -0.777499557,  0.746645033,  -0.924928188,  -0.176574528,  -0.473239511,  1.07175756,  2.44771528,  0.438032299,  -1.17094457,  0.520574272,  0.432533622,  0.799180388,  1.87199485,  0.0956790522,  1.23299885,  0.888195872,  0.662593424,  -0.833372414,  0.351329297,  -0.168695077,  -0.937509239,  -0.411705375,  -0.356296837,  0.37594828,  0.41108337,  2.35570049,  -0.273550689,  -0.321519494,  0.359101027,  0.466737926,  1.66270506,  -0.487400144,  -0.834178448,  -0.118841946,  -1.86498523,  0.424343795,  -0.705530286,  0.646033406,  -0.387373179,  1.00515294,  0.270156533,  2.60822606,  -0.0786460787,  -2.19465637,  1.04648733,  0.688568175},{ -1.41089094,  -0.110662155,  -2.1574831,  1.32121515,  -0.0154136736,  -0.398532063,  0.406519264,  0.178822383,  0.362460136,  1.33872342,  0.662776232,  -0.24607645,  1.77630293,  -0.569212198,  0.152048916,  -0.335900962,  0.825717926,  -1.23279047,  0.66477555,  0.143652454,  1.01358986,  1.81946301,  2.34355521,  -0.371620774,  0.0861200094,  0.662683308,  -1.08316374,  0.218804255,  -0.934090137,  -0.220857888,  1.10322857,  0.0702049136,  0.257750124,  -0.893674791,  -0.431914955,  0.0131138777,  0.570374131,  -1.10733342,  0.621762335,  -1.2602216,  1.25587702,  -0.503965616,  0.0450822636,  0.542783141,  -1.35182369,  0.295896292,  0.415699512,  0.187128097,  -0.286357254,  -0.619912207,  0.049833782,  0.967262924,  0.297399074,  0.595664442,  1.18574095,  -0.573960483,  0.723750055,  -0.511037588,  -0.416415691,  -0.607388735,  -0.0897945091,  -0.499166936,  -1.57269681,  0.482901722,  0.993674994,  1.53084302,  0.44204089,  0.674357116,  1.98332262,  0.535010874,  -0.786568463,  0.572580934,  -1.78312755,  -1.39321518,  -0.522669315,  -1.54447985,  0.366809487,  -2.03742743},{ 0.211938575,  -0.754615545,  1.04942513,  1.29592335,  -0.988146484,  -0.547493279,  -1.33238482,  -0.755594909,  -0.627446413,  0.182715684,  -0.12177489,  -2.58786416,  -1.16409743,  0.852973402,  1.32312071,  -0.289507002,  -0.22568436,  -0.0820161477,  -0.849525869,  -0.330641299,  -0.844982624,  0.539614081,  0.698505342,  1.5243504,  0.576748312,  -0.583323121,  0.481905729,  0.507429123,  0.485960245,  0.165646568,  -0.599555373,  -0.575121522,  0.274138957,  -0.445042193,  -0.114001557,  0.721344709,  -0.0331747644,  -0.570335686,  0.655459821,  -0.554753661,  -1.26928544,  0.108771436,  1.01131225,  1.46641314,  -0.0672960207,  0.350178838,  1.7417289,  1.86463523,  -0.0394325033,  0.569705665,  -1.24701905,  0.458176076,  -0.49245882,  -1.28177953,  -0.566054463,  -1.52478933,  -0.571282446,  -0.769945204,  0.0633110851,  -0.196275339,  1.29756033,  0.0370062292,  0.341246992,  -1.04148364,  0.934341431,  0.466844171,  0.839927316,  0.538515925,  -1.10644877,  0.972541869,  0.243978366,  -0.930671573,  0.427541256,  -1.00592792,  -0.488901317,  -0.113348186,  -1.10936427,  -0.866619825},{ 0.180491403,  0.326636136,  -1.03299785,  -1.12600005,  1.76177764,  1.63898671,  2.15415812,  0.684088409,  0.516636968,  1.47462189,  0.391863525,  0.152103096,  -1.90595889,  -0.848365664,  1.09568191,  -0.805646777,  2.13180089,  -0.491344839,  -2.35636878,  0.279154718,  -1.44737983,  0.561320305,  -0.434667438,  -0.632290959,  -0.696320176,  -0.011828077,  -2.43869472,  3.06204486,  -0.618482053,  0.935564458,  -0.756044805,  0.799260914,  -1.64915907,  0.43413046,  0.592771471,  0.624145269,  -0.751922011,  -1.31274748,  -0.306628555,  3.05192089,  -0.144376561,  0.334381282,  -0.274627,  1.00980878,  0.392885208,  -0.284811348,  -0.837008297,  0.649471641,  0.70436877,  -0.408177644,  -0.388324589,  -1.2682724,  0.0286585782,  -0.510382235,  1.61727321,  -0.0801136494,  0.468706995,  0.562971413,  0.868061006,  0.661160469,  -0.441832691,  0.919665158,  0.415132999,  1.64402139,  -1.47949719,  -0.663566351,  -0.37968117,  0.727309108,  -1.05743825,  0.962432623,  -0.351721764,  0.415241748,  -1.42669976,  0.309835911,  -0.3659091,  -0.565128803,  0.1033246,  1.5946641},{ -0.0489177369,  0.810902894,  0.539974868,  -1.55885839,  0.228113085,  0.36683473,  1.05681705,  1.02424622,  -0.602900922,  -0.709777176,  0.882401049,  -0.521240473,  -0.716468036,  0.855303645,  -0.0315343849,  0.326024562,  0.220329657,  -0.104302526,  -2.07656479,  -0.928720355,  -0.384545535,  0.0635452047,  -0.270234376,  1.61406314,  0.990610003,  -0.883903146,  -1.19357145,  -1.09554243,  0.917621374,  0.342536509,  1.58773983,  -0.330910325,  0.349727958,  -0.980984986,  0.281940937,  0.121627875,  0.720253825,  0.0473907776,  0.94103241,  0.755799234,  -1.00750804,  0.328084767,  0.754863679,  -0.266148537,  -0.883264542,  1.18268609,  -1.18850386,  1.12005067,  -1.10064638,  -0.731427073,  -0.722449422,  -1.76720381,  -0.111940362,  -0.382531345,  -1.86561465,  -1.0281285,  0.411683619,  -1.7124244,  0.881311417,  -0.205894619,  0.870765865,  0.145485923,  0.17635949,  1.37417614,  0.123972617,  -0.113485061,  1.47995782,  -1.5044775,  -0.667275071,  -0.0648039728,  0.335444778,  0.0869946852,  1.68663955,  0.854787469,  -0.116354071,  -0.715749264,  -1.41075957,  0.18396011},{ 0.21268706,  0.947843015,  -1.0704422,  0.054640051,  1.09015191,  0.165284827,  -0.103058748,  0.374299526,  0.492686093,  -0.860737681,  1.18615127,  0.19240281,  0.757298768,  1.55966246,  0.396100819,  -0.10283234,  -0.90762037,  0.289693296,  0.0399731584,  1.61641717,  0.925756395,  -1.05383992,  1.76682448,  -1.08906913,  -0.639128149,  -0.895258009,  -0.440594494,  1.46309614,  -2.31318808,  -0.279056102,  0.266600698,  1.02716923,  0.147762775,  -0.271193981,  -0.465429366,  -1.26542962,  -0.849645197,  -0.587909758,  0.102919966,  0.189639673,  0.261684656,  0.841361642,  0.14581655,  1.77541649,  0.742218196,  0.723320067,  -0.734978259,  0.151074111,  1.28396618,  0.0309294369,  -1.00017166,  1.65375578,  -0.280762464,  1.63324869,  -0.554210603,  1.37326145,  0.209154204,  -2.2118988,  -1.54658365,  -0.337590337,  0.401030213,  -0.745573997,  -0.312192291,  -0.291876227,  -1.11472559,  -0.53693682,  0.89944911,  0.554713249,  1.07605255,  0.477863431,  -0.419176489,  -0.0116765406,  1.13912165,  -1.12750185,  0.672928572,  -2.46518278,  -0.368570238,  -1.00685441},{ 0.217920065,  0.233113751,  -0.918396711,  -0.345625967,  -0.438609511,  -0.500473201,  1.33446932,  -1.40363908,  -0.610779524,  0.699589431,  -0.202955589,  0.323703706,  0.478565991,  -0.175514624,  -0.254405797,  0.529800653,  0.28906247,  -0.376899272,  0.556990623,  -0.268465281,  1.29584312,  -1.03291333,  0.591295362,  2.28368115,  0.433754891,  0.789726675,  -0.358953118,  1.29399633,  -0.177455634,  -0.580707073,  -1.42158973,  0.495536655,  1.36414945,  -0.929174006,  -0.848590493,  -0.842417717,  0.692284107,  0.230345637,  1.26625514,  -0.126737714,  0.555599213,  0.754667282,  0.750105858,  -1.24559045,  -0.198287398,  1.26379967,  -1.62730122,  -1.3785373,  -0.648957729,  0.613826334,  0.925354719,  -0.41703096,  0.131045714,  0.676774323,  -0.380297661,  -1.95354259,  -1.17046082,  0.364518225,  -0.824669182,  0.546769857,  -0.0155675588,  -0.00210118573,  0.948820055,  -0.600848675,  1.03318918,  -0.620708227,  -0.710998356,  -1.01772988,  3.28338718,  -1.2914046,  -0.979129672,  -1.23277307,  -0.859713435,  -0.77633357,  0.310773671,  -1.38458347,  -0.713676453,  -0.266401738},{ -0.440785587,  -0.511491895,  -1.47363663,  -0.431907922,  0.930282891,  -0.77283746,  0.542821348,  -0.226437539,  0.801557183,  0.412005752,  1.2173785,  0.572013736,  0.66656816,  0.635098755,  0.245686725,  -0.00885669515,  -1.74513507,  -1.33667016,  0.340731204,  0.909804165,  0.482287109,  -0.116654985,  -0.541564643,  0.751253068,  1.42012548,  -0.924627304,  -1.09433341,  0.415216833,  1.33113563,  0.736812413,  -1.01526773,  -0.431588292,  0.611779332,  0.104189716,  0.221606225,  2.36555815,  -0.568989277,  1.30936766,  -0.0854174942,  -0.0800345689,  -0.482964933,  0.758136868,  0.723779559,  -1.11900842,  0.0204990935,  -0.651304305,  0.549910486,  1.65369892,  2.32975674,  0.435621768,  0.913411438,  -0.624519706,  0.0934489444,  -0.37488699,  0.245855823,  -1.31216061,  -1.95408309,  -0.365810454,  -1.43211687,  0.558071196,  -1.58434582,  -0.297228366,  2.29273748,  -0.618653417,  -0.560082376,  -0.371684313,  0.363522142,  2.32083154,  -0.047034055,  0.398608327,  0.760819614,  -0.774804533,  0.162152603,  -0.212871984,  -1.37918091,  0.117415093,  0.379328638,  0.456879467},{ 0.223322839,  -2.06437278,  0.292057037,  0.163457498,  1.07098103,  -0.93996495,  0.335267365,  -0.470205635,  -0.959730268,  1.56632257,  0.17990981,  1.96289599,  1.26471829,  -2.87857556,  0.526405096,  -0.213037595,  -0.445912331,  0.0110973613,  -0.586702526,  0.83324182,  0.118317798,  -2.3665781,  1.45256734,  -0.342975885,  1.46717501,  -0.735827684,  0.0952720419,  2.16640878,  -0.062885873,  -0.708635569,  -3.31717086,  -1.86822188,  -0.41239059,  -1.08940053,  -1.23448098,  0.921986043,  0.383249104,  -0.127863005,  1.52162457,  -0.362199694,  0.352423161,  0.73239255,  1.17028093,  0.339118987,  0.570599139,  0.890917122,  -0.449190855,  -1.09956479,  0.654165626,  1.45059693,  -0.670957983,  -0.336571932,  0.774647355,  -0.404314637,  -0.157696888,  -0.670894802,  -0.831936479,  0.206176803,  1.16298604,  1.31680286,  0.100964069,  -0.99576354,  2.41033721,  -1.97913146,  1.12241447,  1.20540547,  0.338799983,  -0.597856641,  0.265541136,  0.209678322,  0.577485979,  0.435602635,  0.113568686,  -0.395394892,  0.287004679,  -1.64355624,  -1.40062237,  0.453300238},{ -1.18491626,  0.574377179,  -0.822503507,  -0.71638602,  -1.53446138,  0.654504359,  -0.89590615,  -0.133867145,  1.40782166,  0.413870156,  1.15252054,  -1.22032893,  0.0339287594,  0.522313654,  1.03817797,  -0.816935182,  0.018070614,  0.0323570482,  -1.49994373,  0.983093619,  0.120768234,  0.234965339,  -1.61661625,  0.190029219,  -0.763798535,  -0.515474439,  1.24648213,  -1.58312714,  -1.17466807,  -0.603243053,  -0.845721662,  2.10348105,  -0.372109145,  -0.654754937,  0.272216797,  -1.00433969,  -0.0852678195,  0.75935936,  1.4225713,  0.941623747,  0.723662734,  0.622870445,  -0.598542631,  -0.586243093,  -0.226190642,  1.00305378,  0.497406393,  -1.67177439,  -0.0187975857,  -0.0289830472,  -1.24634099,  -0.444474697,  -0.480476618,  -0.98142153,  -0.101064287,  -1.07930934,  -0.132327422,  -0.580811143,  0.99759835,  -1.1070056,  -0.631632328,  0.764165878,  -0.434482694,  0.112283692,  -3.1906693,  0.17386958,  0.470905334,  1.26163876,  0.0374930725,  0.34595114,  1.76764548,  -0.692686617,  -0.604290664,  -1.23973048,  -2.28275609,  1.99346793,  0.0364305191,  0.473053068},{ 0.54532522,  -0.949012339,  0.254015476,  0.625328958,  0.0663027689,  1.03653038,  -0.898310661,  0.291237056,  -0.570991218,  -0.650961161,  -0.830199301,  1.59450805,  1.20957768,  0.222274333,  0.481005371,  1.28756809,  0.901090205,  0.87639147,  -1.22143519,  0.498020738,  2.2573328,  -0.618893564,  -0.251817226,  0.466220379,  1.89431119,  0.200768098,  0.3722983,  -0.602394104,  -0.284256548,  -1.27663267,  1.70867991,  0.358282566,  1.15052497,  -0.255922556,  0.183020204,  -0.941722155,  -2.97913194,  1.73424757,  -0.527631402,  -1.1969527,  -0.00888322201,  0.0515996814,  -0.696730912,  -0.418029279,  1.46970963,  0.0885739326,  -1.46529579,  -1.43407202,  1.70559633,  0.217421949,  -0.800483048,  -0.721789002,  2.01367807,  -0.000325726287,  1.20480657,  -0.870117962,  -1.1452837,  0.14551127,  1.30267537,  -0.121599443,  -0.270606935,  0.843223095,  -0.753478944,  -1.23522949,  -0.291472673,  -0.20750545,  1.02632844,  0.000334336364,  -1.02914762,  0.334017456,  -2.3125062,  -0.479980558,  -2.40219879,  0.761950076,  -0.498892963,  -0.400376976,  -1.38375163,  -0.225962356},{ 1.64560986,  0.271875471,  0.492425799,  0.517558277,  2.54733658,  -0.419585913,  -0.513979912,  -1.05490613,  -0.954609036,  1.06797862,  -1.29353845,  1.16245008,  0.890907645,  -0.147624388,  -0.804711282,  -0.0379519053,  -0.1976289,  -1.17436361,  -0.536707461,  0.443633407,  0.75263989,  -0.304278404,  -0.651004672,  -0.42968291,  0.105708174,  -1.5560509,  -1.08440495,  -0.22525911,  -0.709919453,  -0.06181629,  0.93939358,  1.94321072,  -0.815008163,  -0.0512968004,  1.15288186,  0.711074889,  0.6652022,  -0.148707122,  -0.839075446,  -0.69431442,  0.577631772,  1.24895108,  -0.288898766,  3.18485355,  -2.48930192,  -0.327573478,  0.118941329,  0.435378551,  -0.553954959,  1.14108503,  2.59315801,  0.186018914,  1.08754003,  -0.507937074,  -1.22263432,  -0.114915736,  0.644414127,  -2.3078835,  0.619707525,  -0.301092565,  -0.895270228,  1.33340979,  -0.135821104,  0.165950805,  -0.678825915,  -2.20748305,  -0.842276156,  0.364290774,  -0.875356317,  0.853909016,  1.70831108,  0.542916954,  -1.08437479,  1.11744583,  -0.237656951,  -1.33558142,  -0.598516524,  0.877213776},{ 1.05673552,  -2.46988916,  0.794660389,  1.73760033,  0.131797954,  2.17206764,  1.06722224,  0.829338968,  -0.526849151,  0.813709617,  1.41960168,  -0.253960758,  -0.533367872,  1.08861291,  0.0432302728,  1.46276712,  -0.660525918,  0.499712706,  -2.72474504,  0.794475675,  1.19411552,  1.32700515,  -0.369992733,  -0.0846848935,  -0.0886851847,  1.85568559,  -1.90634882,  1.47398496,  0.971841156,  -0.895056784,  -0.265269786,  -0.694393098,  -0.291374922,  0.0856109336,  -0.801132143,  0.816819906,  -0.749765575,  -0.2736727,  1.79149377,  -0.627904415,  -0.594750941,  -0.776219785,  1.19891226,  -0.602007091,  0.538989007,  0.0345004685,  -0.159306929,  0.406551868,  0.186698124,  -0.749376774,  -0.951343298,  1.04831636,  0.464782864,  -0.34679547,  -1.55762601,  -0.515343845,  -1.94359398,  0.323590249,  -1.37545347,  1.34185052,  -0.555244267,  1.24211502,  1.08956826,  -0.221996292,  1.08514822,  1.03710067,  -1.92985213,  -0.0862276182,  -0.436779827,  0.446410656,  -0.132259905,  0.89497453,  -0.512038469,  -1.2646507,  0.837051511,  0.267021537,  -0.13917473,  0.0780028179},{ -0.514987171,  0.866164029,  -0.572993875,  1.58827066,  1.30636346,  1.5785588,  -0.401534975,  -0.468294114,  0.529460907,  -0.0851768926,  -0.569076478,  -0.381115258,  0.831600547,  1.39585483,  -0.157357767,  1.39909184,  -1.49048316,  0.70563978,  -0.316022068,  1.49658,  -1.22712123,  -0.621585906,  -0.272769362,  -0.373986274,  -1.26040995,  0.0586443022,  0.95216924,  0.530378222,  2.2115097,  0.947565556,  0.77392143,  0.503858328,  0.931249321,  -0.593733549,  -0.951482356,  -0.962876022,  1.31667972,  0.0475032665,  -0.666610301,  0.742475569,  -1.03490102,  0.82420814,  -1.36209786,  -0.821112037,  -0.914655626,  0.350447714,  0.391927987,  -1.07723463,  0.143367052,  0.783472776,  0.487050414,  -0.684542596,  -0.314687312,  -1.04120183,  -0.480989754,  0.98289001,  -0.645422101,  -2.30754924,  0.86584878,  -0.0351581089,  0.809933484,  0.800665379,  -0.378126085,  -0.44890213,  0.0852407217,  -0.527346969,  -0.513932467,  0.37411201,  -0.462745965,  -1.80850399,  -1.20057499,  0.0707484856,  0.935993433,  -2.06049323,  0.411102742,  1.15744185,  -0.974041283,  0.83223325},{ 0.267000347,  -0.470069706,  2.13502026,  0.475280046,  0.797427356,  -0.0668724924,  0.750540733,  -1.08273709,  1.61486006,  0.21988067,  1.51246953,  -1.01466894,  0.715748429,  0.318234801,  -0.268309653,  0.120577522,  0.506694019,  0.860106587,  -0.0403276794,  0.52746129,  -0.352931708,  -0.203199342,  -0.242239386,  0.0466944613,  0.391496867,  0.105397671,  1.59809971,  0.389291376,  0.464886874,  -0.554270387,  0.947310507,  0.425827652,  2.33457685,  0.083343178,  -1.15891504,  -0.455719471,  -0.240154162,  1.17669654,  -0.937420845,  -1.97379494,  -0.395085901,  -1.32885134,  1.47687805,  0.0973713547,  -0.152268335,  0.281081408,  -0.907529593,  0.192371041,  -1.10462236,  1.87608051,  -0.591919959,  -0.61751169,  -0.990586996,  -1.66675699,  0.590582132,  0.680349112,  -0.32734409,  -1.22554886,  -0.93758595,  0.967978001,  1.19782221,  0.404717505,  -0.24997288,  -1.13568187,  0.78387171,  0.114708632,  0.446535259,  1.63742447,  0.340725601,  1.17900372,  1.85809374,  1.18052626,  -0.273942709,  -0.336329877,  -0.171397269,  1.1037246,  1.71786857,  0.378342897},{ -0.893716395,  -0.767500341,  -0.545393765,  0.75784725,  0.202599227,  0.987791657,  -1.21251178,  2.18587589,  1.4602747,  0.245344967,  -0.244318262,  0.568636537,  0.0124018369,  1.21595013,  -0.804459274,  -0.375544131,  1.20125878,  0.10918805,  -0.366142631,  0.0607317053,  0.234762177,  0.0943773985,  -1.83415294,  -1.40613055,  -0.968697786,  -0.634644508,  -0.165876508,  -0.918127537,  -0.783062816,  -0.828863144,  -0.170756832,  -1.32326162,  -0.486445457,  -1.59670866,  -0.895343959,  -0.154315263,  -0.654392898,  0.610596538,  0.579392612,  -0.499760628,  -0.490427524,  0.531750321,  -0.376797765,  -0.457650185,  -0.391330749,  -0.0213920902,  -1.53254354,  0.264098167,  -0.163844779,  -1.34021187,  2.77979565,  0.56430918,  0.24461098,  -0.782823145,  0.261132628,  1.04347646,  0.0344580598,  0.0208721664,  -2.27478194,  2.16044879,  -0.46598193,  0.682226896,  -0.868932188,  -0.360657603,  -1.31250334,  0.895520866,  0.0555358157,  0.0663790181,  -0.192599043,  -1.51886904,  0.0284654647,  0.733836591,  2.84289479,  -0.01830847,  1.02549827,  0.397960663,  1.65998697,  0.0712645501},{ -0.513074696,  0.439426184,  -0.753380656,  -1.05250084,  1.53366005,  -0.538122356,  -0.834859729,  1.10679257,  -0.282124311,  0.575021029,  0.798898339,  -1.0164938,  1.28316987,  -0.0818552151,  0.153880194,  1.55882645,  0.411619395,  0.645816803,  0.734218597,  1.4821682,  1.12858272,  -1.93143404,  1.46583986,  0.373687297,  -0.580691993,  0.196549326,  -0.0311327428,  -0.6544649,  -1.52842331,  -1.30798948,  -2.22228217,  -1.81930399,  1.01520431,  -0.152275413,  1.33032656,  -0.698742688,  0.27656436,  0.712992668,  2.03329349,  2.52977777,  -0.275528312,  1.28557193,  -1.59233856,  -1.05502927,  1.2156142,  -0.711535633,  0.502695978,  -1.57731259,  -0.952902377,  -0.231730983,  1.04021859,  -0.894328594,  -0.138470531,  -1.40004432,  0.0295554567,  -0.101521082,  -0.457534224,  -0.422917128,  -0.389762163,  2.63775349,  -0.266250014,  0.260747433,  -0.631899476,  -0.45923844,  -0.778510869,  0.412760586,  2.04523325,  -0.200619534,  -3.23186946,  0.521226048,  -1.33952975,  0.235345975,  -0.994487822,  -0.0912615359,  0.527749658,  1.50889564,  0.221187726,  1.22294796},{ -0.289014757,  -0.473547012,  -1.34955895,  -0.56319505,  1.23231256,  2.16307878,  -0.628515065,  -0.661655247,  0.275238335,  -0.0851637945,  -0.578857183,  0.444416434,  0.123373792,  2.38764358,  -0.745161474,  0.357959598,  0.958709359,  -0.336368352,  0.89549607,  0.729377449,  -0.292422324,  -0.437022418,  -0.944821477,  -0.0294156857,  0.965735376,  -1.36011124,  0.861436307,  2.29264879,  -0.773405135,  -0.386830926,  -1.38210499,  -0.982836425,  0.248589247,  -0.323710382,  0.252217203,  0.795912862,  -0.683939755,  -0.0733661577,  -0.371566355,  0.152845725,  0.810344636,  -1.74763358,  0.686177909,  -0.314823568,  1.18696165,  -1.30756831,  1.78782439,  0.755380988,  -1.13432407,  1.28675008,  0.710890412,  0.30113548,  1.17730951,  -0.30843854,  -0.197268173,  0.310969383,  -0.498948693,  0.269036889,  0.0810182542,  -0.363751471,  1.91988647,  -2.10714841,  -0.291603386,  -0.453134328,  -0.713453233,  -1.33118904,  -0.266055644,  -0.599922419,  1.20924222,  1.13214195,  -0.651958287,  -0.43890059,  -0.29630366,  0.205852121,  0.280910879,  -1.49791396,  -1.6033802,  0.348341852},{ 0.644812882,  1.62003016,  1.27198207,  -1.01975965,  -0.312285453,  -1.89436877,  0.281857789,  -0.205287978,  -1.78272557,  -0.365815818,  1.68382609,  -0.0486221276,  0.388822407,  0.700685978,  -1.5016917,  -2.27054358,  -0.258740485,  0.891411901,  1.76747251,  -0.296202481,  -0.559564829,  -0.92562753,  0.0434808806,  -1.97786951,  0.8165434,  1.63601732,  -0.0861041322,  0.150719017,  -0.937364399,  1.15074301,  -0.902611315,  0.0560086146,  -1.95673454,  0.174073413,  0.85981077,  -0.642025352,  1.32853866,  -0.393966407,  0.0704948381,  -1.61860871,  -0.157554999,  -0.884837806,  0.379366338,  -0.240131572,  0.623096704,  0.651421607,  0.711038589,  -0.259182006,  -0.677800357,  0.344521224,  0.376547545,  -0.150565654,  0.836978614,  -1.3366276,  -0.60156548,  -0.342909724,  0.144969866,  0.518918514,  0.240482926,  -1.31955051,  1.49446511,  -2.48538899,  0.311628938,  0.530383468,  -0.69902575,  1.23304009,  0.424592435,  -1.32740319,  0.230164394,  -0.338843167,  0.740814984,  -0.103675693,  -0.288544208,  -1.62530208,  0.428812265,  -1.29829526,  0.108283535,  0.0101969792},{ -1.77572179,  0.931418777,  -0.315585166,  0.985618412,  1.15516174,  -0.0678517893,  0.142845184,  -0.455404967,  1.11843348,  -1.58710289,  0.465012878,  -1.22248173,  -0.390986919,  2.30028486,  -0.494345188,  0.103594117,  0.40692386,  -0.59169358,  -0.606800973,  -0.00620404864,  -0.339747816,  0.0175399575,  -0.770126998,  0.474664062,  -0.742870927,  0.743626356,  0.429023534,  0.680056155,  1.00677848,  -1.46403086,  -1.32911217,  -2.46030188,  -0.470285833,  0.582165539,  0.0804497376,  0.0253306553,  -0.725943506,  -1.84386015,  1.45709956,  -0.136773169,  0.512812734,  -0.243361548,  -0.595392466,  0.553831518,  0.194594443,  2.13056517,  -0.496644676,  -0.43063587,  1.56559026,  1.19020581,  -0.203302905,  -0.859456539,  0.743777215,  1.53901172,  1.93810868,  0.929387689,  0.0271491464,  -0.0814206079,  -0.672671676,  -0.395474195,  -0.293313503,  0.538698077,  2.91980386,  -0.484278262,  1.58494866,  -0.100406773,  1.70956409,  0.282095999,  -0.136187717,  -1.74109614,  -1.41766155,  -0.714865506,  -1.12288773,  -0.526345313,  -1.3285594,  -1.24295688,  0.608228743,  1.41695678},{ 0.324920207,  -0.0974253863,  0.777230918,  0.63086611,  1.0142864,  0.499026418,  0.823466003,  -1.04397297,  0.374904394,  -0.0437298864,  1.63930357,  1.19241822,  -0.622566164,  1.92051196,  -0.434650034,  -0.862735391,  0.179171503,  0.289880663,  -1.85191607,  0.475491494,  -0.507453084,  0.379295886,  0.227939263,  0.812780797,  -0.459239066,  0.694645047,  -1.23714995,  -1.86870503,  1.81172991,  2.38161087,  0.472417027,  0.116071291,  -1.66286564,  -0.822547019,  0.485060751,  -0.639070749,  -1.27110744,  2.03157687,  1.41310382,  -0.610831797,  -0.45713973,  -0.427550048,  -0.225797579,  0.514612675,  -0.921595097,  -0.153771624,  0.812127233,  0.302843302,  1.32237613,  -0.117747165,  -2.49562955,  -1.96784067,  1.05364203,  0.130196914,  0.638577878,  0.404917121,  1.17903244,  0.0181703009,  -0.758667588,  0.208545566,  -0.729059935,  0.139779285,  2.29185081,  -0.441365629,  -0.0805861652,  1.10772717,  1.10625589,  -0.127023041,  1.84262156,  -0.0747806951,  1.69335461,  -0.290727019,  0.0140058463,  -0.122074924,  -0.605544269,  -1.02925205,  0.453065693,  -0.215047106},{ -0.217564002,  -0.481149107,  1.69084287,  -0.237248063,  0.0522426106,  0.437017441,  0.260497898,  -1.15009046,  0.20255585,  0.630402923,  0.308378249,  -0.552458763,  -1.26861417,  -0.0532988198,  0.780996263,  0.289794296,  -0.445071459,  0.945754826,  -0.961763263,  -0.777564287,  -1.3520031,  -2.74893403,  -0.400011778,  0.0988327637,  1.91815877,  -0.196207777,  0.121376805,  0.147608072,  0.811987817,  0.93796128,  0.848977923,  -0.8080163,  1.17662406,  -0.0800221413,  0.615603387,  1.23837388,  -1.12179315,  -0.74088639,  -0.718237281,  -1.09763813,  -1.01762474,  -0.612913013,  2.66939664,  0.514692783,  0.00569065008,  0.594730258,  0.96851933,  -0.304067373,  0.994575262,  0.116085134,  0.727624595,  1.35630059,  -1.41474593,  0.0592713691,  -1.06523049,  1.33894742,  -1.10728729,  1.58219635,  1.29692888,  -0.790275335,  1.88810539,  0.480754077,  0.683363259,  0.444133371,  -0.868103921,  1.22274458,  -0.658338964,  1.06568539,  -0.338156015,  0.141256392,  1.72874629,  -0.18557471,  -0.717206955,  0.648276031,  1.34100842,  1.36641836,  0.00203370769,  -0.505705714},{ -0.898775637,  0.492508531,  -0.19207117,  1.23337781,  0.120734051,  0.156838477,  0.616771281,  -1.12672603,  -2.49151373,  -1.16134501,  -0.746942759,  0.827566743,  -0.0997477099,  0.842357755,  -1.12589025,  0.0157350358,  0.189820156,  1.30271447,  1.16277945,  2.62743664,  0.492804676,  0.731569171,  -1.11130476,  0.45392102,  -0.485640615,  1.35051608,  -0.426127553,  -0.436642498,  0.92241317,  -0.317961633,  -0.191942528,  0.0427876152,  0.774706066,  -0.424662411,  -0.119095176,  1.52696371,  1.67430627,  -0.0735013783,  0.694372714,  -0.138601124,  0.00956730358,  0.387388855,  1.43458843,  -0.76927942,  -1.82912886,  -0.683458328,  0.142074823,  -1.37109375,  -1.04456449,  -1.26282763,  0.986825764,  -0.828612626,  1.20151305,  0.424019128,  -0.746556461,  1.9758029,  -0.244039908,  0.516224861,  -0.513763905,  1.33582509,  0.871150613,  0.56300813,  -0.617562771,  -0.526343882,  -0.380982578,  0.367941022,  -1.64238155,  2.33112788,  -1.63432384,  0.390931189,  -1.42852688,  -0.308636874,  -0.33034116,  0.690128267,  -0.554684579,  0.470478088,  -0.91244024,  -0.414041817},{ 0.631668448,  -0.119953074,  -1.82632029,  0.637552857,  -0.30665186,  -1.01518154,  0.335902601,  -0.615952253,  1.52293074,  1.6300745,  0.301443338,  1.03102005,  0.561285913,  0.6362589,  -0.163795307,  1.45384359,  -0.00487680035,  -0.476045996,  -0.857856214,  1.50500762,  -0.457040548,  0.121475905,  -0.380696565,  -0.936712384,  -1.44130409,  -0.806873739,  -0.739179194,  1.69524801,  0.474086791,  -1.47191381,  -1.26245713,  1.2908262,  0.348109066,  0.0910486355,  -0.107996717,  1.3658694,  -0.740834415,  -0.117528938,  -1.17161763,  0.21209617,  -1.28941417,  0.420657128,  -0.417845845,  0.758681595,  -0.572910786,  1.14735508,  -1.44428289,  0.00414150441,  1.44792771,  1.38209391,  -0.827660859,  0.173239768,  0.687119365,  1.0767132,  1.35792494,  0.0467094742,  -0.434516698,  -0.341530174,  -0.82855171,  0.0997837335,  1.41413236,  -1.26765716,  -0.514845192,  0.0317622162,  0.693572998,  2.44034958,  0.158284336,  0.837862134,  1.02699542,  1.1613946,  -1.14203238,  -2.03307772,  0.336838722,  0.185309723,  -1.27656043,  0.233109385,  0.588699758,  0.923965812},{ -0.0865907818,  -0.254098237,  -1.08799517,  0.474289894,  -0.683847249,  0.724711299,  -0.306582689,  -1.93380225,  -0.354003429,  -0.831415057,  -1.6972158,  0.190670222,  -1.05148768,  -0.377230316,  -0.0767280757,  -0.497145176,  -0.0450751074,  0.380667895,  -2.03789783,  -0.271487206,  -0.513534427,  1.79304051,  1.14441931,  -0.00372538762,  0.234343663,  0.230811208,  -1.77594817,  -1.60719764,  -0.453607202,  0.263263524,  -0.739308238,  0.478347749,  -0.31003359,  0.129627258,  -0.0256075449,  0.9352054,  -0.0696666092,  -1.95759964,  -1.29196453,  1.41398811,  1.46059644,  -1.52246726,  -1.1036278,  0.656089365,  0.329196244,  2.82473731,  -0.824538708,  -0.243077934,  -1.34452152,  1.3174628,  -1.57657945,  -0.797877669,  -1.84811485,  -0.322117627,  0.152238131,  0.331281841,  3.15964317,  1.44822359,  -0.0806168765,  -0.5560866,  -1.29949164,  -0.361690491,  1.6450814,  -1.30572724,  -1.00532496,  0.642049193,  -1.35213184,  -1.669209,  -0.60256958,  -0.0893878341,  0.471109718,  -0.90345794,  0.717353821,  -0.30512327,  1.72299421,  0.133092448,  -0.801410854,  -0.0924482867},{ -0.502381444,  2.06747198,  0.265765905,  -0.987728477,  0.381306589,  0.162579343,  -0.96708113,  -0.496212959,  0.656352699,  0.629692197,  0.302659452,  -1.20660222,  -0.580736995,  0.217766479,  -1.61346614,  -0.419391632,  -1.95198417,  -0.429519415,  0.935705245,  -1.55609798,  -1.13308156,  -0.400093704,  2.10465097,  -2.09914756,  1.91946495,  1.20978832,  0.2622886,  2.2980659,  -2.20348477,  -1.24898636,  -0.634120047,  0.77208221,  0.850542843,  -0.221044898,  1.67032349,  -0.285670817,  -0.965300679,  1.40315199,  0.871751189,  -1.71331,  -1.19650376,  -0.194588006,  0.319647789,  1.11731231,  0.70318538,  0.131219104,  -1.12397778,  1.28534913,  1.88408685,  -0.53802371,  0.257151037,  -1.64829397,  0.327399522,  0.0386555418,  0.945620894,  -0.986385167,  1.96574318,  -0.419362545,  0.786399186,  0.725222528,  1.03588998,  2.09154177,  1.15854394,  -1.11538053,  0.0115015814,  1.99003005,  -0.883215189,  0.482140213,  0.688746333,  -0.787992299,  1.11309683,  -2.52109051,  0.291129678,  0.212037638,  0.218425289,  0.58617264,  -0.502652049,  -1.1598959},{ 0.208206177,  0.816243351,  1.93466318,  0.302827686,  1.03699517,  -0.45673874,  -1.23131049,  0.985680997,  -0.986881614,  -0.0764826611,  -0.688735425,  -2.63054442,  0.466070116,  0.595121443,  -0.482098639,  -0.0616935976,  0.763458133,  1.43026102,  -0.240140781,  -1.34079993,  0.781521797,  0.00135742,  -0.643898427,  -0.774692118,  1.29913437,  1.31098771,  0.775447309,  -0.577385485,  0.278277397,  -0.785817266,  -1.27666032,  1.4166193,  -0.671126604,  0.444026917,  -2.00662875,  -1.09604371,  1.02892506,  0.897184432,  0.25021261,  -0.158603847,  1.27131939,  0.761783481,  -0.457395315,  1.0945884,  0.775835633,  0.00690659322,  -1.08228648,  -0.736699045,  0.898407221,  -1.1516993,  0.0677348077,  -2.17105532,  0.680154085,  0.664077759,  -0.308133513,  0.429477066,  -0.0377837718,  -0.856029868,  1.41604567,  0.663826823,  0.946662366,  1.20389605,  0.00311509124,  0.134497359,  1.05141819,  2.3428669,  -0.739957571,  -0.210961834,  0.649582386,  -0.0523130074,  1.52493596,  -0.115116157,  -0.552530885,  -1.07923758,  0.617700338,  0.543343306,  -0.300039142,  -3.1277492},{ -0.609503269,  0.116690747,  -1.58201826,  -1.11203206,  0.388116449,  -0.339288861,  0.207862139,  0.634512246,  -0.0958783254,  0.564449847,  1.25762773,  1.09252632,  0.191289172,  0.673594296,  -0.794731498,  -0.284645259,  -0.09574911,  -0.230374664,  0.395590425,  -0.901730657,  0.0392322578,  -0.158518299,  2.14479303,  -0.193076015,  0.512894869,  0.879482985,  0.262939125,  -0.997395277,  0.585381746,  0.607508659,  0.246915638,  -2.55740309,  -0.221192241,  1.76824558,  2.87317824,  0.516636133,  -0.89470464,  0.925351083,  0.726601064,  -0.912899613,  -0.249713734,  -0.0508482307,  1.43098044,  -0.309365362,  2.55818319,  -0.533415079,  2.55076885,  -2.04179549,  -0.273111552,  -0.662483215,  -0.0772732049,  1.02120948,  2.30689001,  -0.518436849,  -2.00943112,  0.19444266,  -0.109390639,  1.17158389,  0.251392424,  -2.35774279,  -0.513192773,  2.20653629,  -0.67149806,  -0.404310018,  0.589084804,  -0.482386053,  0.113880225,  0.246088609,  -0.427357644,  1.95687413,  0.830744982,  2.77689624,  0.892091572,  -0.159861267,  2.32529545,  0.374392539,  0.609417856,  -0.096427761},{ 0.599529803,  -0.0138371205,  0.379547685,  -1.40166068,  -1.34201825,  -0.975666285,  0.435574532,  -0.100136444,  2.272156,  1.43412077,  -0.0488823317,  -0.404174209,  -0.0226186365,  -1.1803509,  -0.490888298,  -0.0366862752,  0.934970617,  0.167563066,  -0.078563951,  -1.06989467,  -0.990019023,  -1.64177406,  -1.18500543,  0.693501055,  1.36479998,  0.285743296,  1.01068509,  -0.705662489,  -0.54417789,  0.637935698,  0.81847465,  1.31055915,  1.53107989,  0.926683009,  0.647331178,  0.309248596,  -0.015071108,  0.18252863,  -0.183254704,  1.26163113,  -0.725498617,  -1.20452988,  0.852702677,  1.37996602,  -1.88212204,  -0.36230132,  -0.0920301899,  0.683952808,  -0.56967932,  1.26481891,  -0.772068024,  -0.139113754,  1.03891718,  2.32463908,  1.42769682,  0.114345849,  1.69451106,  0.559227049,  0.192725614,  0.212441012,  -0.0228878018,  2.0556047,  -0.500750303,  0.267019659,  0.493054032,  0.731657982,  0.0842913687,  0.760162055,  -1.33836544,  2.31877685,  -0.709160268,  -0.972238541,  -2.12969971,  0.252485394,  1.02926099,  0.420389265,  -0.909405231,  -0.560368836},{ -0.856890976,  -0.041945681,  0.542518437,  -0.90855819,  -0.58019805,  0.371310502,  -0.723633051,  -0.297228664,  -2.11325264,  -0.107668757,  0.868155718,  1.60591984,  -0.170320675,  2.00747371,  -1.0089879,  1.30097151,  -0.455607504,  -0.0153651973,  -0.613296688,  -1.05498743,  0.449630857,  0.414204568,  0.0270238854,  -2.33387208,  -0.0647957623,  0.781253934,  0.540449798,  0.407402545,  0.371695548,  0.509176731,  1.94188082,  -0.656273186,  0.523056149,  -1.09389067,  1.00194347,  1.31623077,  -0.150828272,  0.923439801,  -0.700044572,  -0.50337708,  -0.791771233,  -0.23221603,  0.653491378,  1.22055566,  -0.24672921,  2.61235094,  0.192384705,  -0.840274572,  0.048594743,  0.052773159,  2.6529839,  0.726895452,  -2.0977993,  -0.553167462,  -1.36294639,  -0.955452979,  -0.170904413,  0.811451197,  -0.529631436,  1.58591866,  2.03880429,  -0.826932907,  -0.796643019,  -0.291165859,  -0.37879312,  1.47049952,  1.62453413,  -1.4715637,  0.105615743,  0.714533567,  2.20952773,  -0.124104157,  1.00134599,  -0.329287499,  0.723386526,  1.08485913,  0.0655016452,  -1.06453514},{ -0.245122656,  1.19181097,  0.912016034,  0.108611517,  1.52340722,  0.306236744,  -1.1057055,  0.292923301,  0.717701912,  -0.734826803,  1.75830173,  0.0779099241,  0.0625343844,  0.298174232,  2.27403879,  0.539813876,  0.495042771,  0.396844238,  -0.890693665,  -1.3300221,  -1.62160552,  -1.18131316,  0.18683818,  0.137603238,  -1.42013681,  1.89184403,  -2.28560448,  -1.94459403,  2.04791832,  -0.553284049,  0.553655326,  -0.666335046,  -0.488284498,  -1.01059043,  0.993658841,  -0.694801211,  -1.79752874,  1.37952232,  0.214095667,  -0.167910948,  -0.22564733,  -1.30891848,  -1.17941022,  0.496848017,  0.0139662623,  -0.10481818,  0.351936072,  0.425078064,  -1.33000457,  0.0890991837,  -0.538565993,  -2.21374679,  0.603912115,  0.150619403,  0.0402967669,  0.131702483,  -1.60549951,  0.146877676,  -0.250052601,  -0.922409892,  -0.0959247798,  -0.884033084,  2.55134439,  -1.41909254,  0.391806573,  -1.70576966,  -0.324804991,  1.36107159,  -0.905027807,  0.776619315,  0.843924999,  -0.303915441,  0.0428457521,  -0.0357463844,  -0.35759145,  -0.106793791,  -0.939590633,  0.227095649},{ 0.605212152,  -0.763481855,  0.219211817,  0.858492196,  0.0537097752,  -2.32074213,  -0.180605143,  1.47203112,  -0.0294349212,  1.14538169,  -0.268491715,  -1.09511137,  -0.758412778,  0.649094641,  0.026888283,  0.932621539,  0.416407883,  -0.756599605,  -1.48788309,  1.98654401,  -0.788565516,  -2.19381166,  -0.893732011,  -0.133063108,  -0.945906758,  -0.108446866,  0.679142475,  1.24801469,  -0.525838137,  1.60395408,  0.773982167,  -0.75533092,  -0.855810285,  -2.28816915,  0.974403858,  -0.0652529523,  1.02001941,  0.961818933,  -4.559165,  0.00382854883,  2.05849195,  -0.894488871,  -0.135615423,  -0.440386534,  1.46871376,  0.34306848,  -0.257168829,  1.23015988,  0.2459151,  0.438519537,  0.778661132,  0.129760727,  1.37399757,  -0.0710977986,  0.183781058,  -1.05063832,  1.68761957,  0.332724243,  -0.884513676,  -0.999933839,  -1.04114342,  0.836317301,  -0.0803755671,  -0.175802574,  -0.91879648,  1.81480992,  0.272654325,  -0.666262627,  1.1396836,  1.00681996,  -1.2972753,  -0.369185358,  -0.384569734,  -0.430288047,  -1.02879977,  -0.171589971,  0.259409755,  -1.38627708},{ -2.65586162,  -0.493869185,  1.31278181,  -1.42969275,  0.832350433,  -0.885786712,  1.8594172,  0.635657609,  0.295378327,  0.594086349,  0.565194607,  -0.112724617,  -1.09616268,  0.449011475,  0.116937935,  0.057700891,  -0.626285136,  -0.243662938,  -0.60862267,  1.23814571,  0.933900714,  -1.12735438,  1.12958837,  -0.294026732,  0.339488268,  -1.83486986,  -1.4735148,  -0.18868953,  -1.41554248,  2.50686193,  0.805048168,  1.31337237,  -1.05066764,  -0.847796082,  1.26084447,  -1.57713723,  0.0544186011,  1.14563322,  0.620067179,  1.55160809,  -1.59130406,  1.05505729,  0.356544465,  -0.659534454,  0.80247587,  -0.832270861,  1.21936858,  2.09568262,  0.539860964,  -0.225305647,  1.95304477,  0.395683259,  -1.25892758,  0.618572474,  -0.189575925,  -0.928828478,  0.638665676,  -0.165271282,  0.0492772423,  -1.04287612,  -2.70742345,  -0.123765692,  -1.08147228,  0.104078293,  1.40292561,  1.01426458,  -0.791353583,  -0.0755528957,  -0.749806166,  -0.395251572,  -0.652514458,  1.54019976,  0.0659275129,  0.268328816,  0.902375698,  1.66285169,  0.827017248,  -1.25164115},{ -0.320787191,  0.501436174,  0.967350662,  0.693097472,  0.283987522,  0.744496763,  0.413345248,  -0.0949029401,  0.445278347,  -1.74366724,  -0.621125162,  -0.0462116301,  -0.318306983,  -1.03076541,  -0.774285138,  0.33223787,  0.199911267,  1.6431551,  0.451741368,  -0.000320685824,  -0.436071157,  0.236525506,  -0.460603446,  0.709069073,  -0.154919118,  -0.00991784967,  -0.0471273437,  -1.97553754,  0.27343002,  0.994609416,  0.991084874,  1.66537809,  -1.91597068,  -0.985891342,  2.2057004,  1.02552891,  0.581639171,  1.67424154,  1.64840508,  -0.898318529,  -0.511636376,  1.26763439,  1.35277438,  -0.369857967,  -0.329036117,  1.59133947,  -0.947752774,  1.23703587,  -0.288502008,  -0.287042171,  -0.637073934,  0.960067332,  0.620819151,  1.54804456,  0.19572562,  0.822069168,  0.525752962,  0.464108169,  0.957292855,  0.133022502,  -2.42626047,  1.38281119,  -1.48549485,  -0.491115481,  -1.16716087,  0.717944682,  1.17847419,  0.381350458,  1.22207534,  -0.153519258,  -0.653417766,  0.189036727,  -0.144444913,  0.623601377,  -0.914096892,  -0.554142535,  -1.08421183,  -1.062639},{ -1.5353297,  0.468890756,  -1.59254694,  0.862145841,  0.824097633,  -0.934996009,  -0.0803711116,  -0.715790153,  -0.156702921,  1.11023927,  1.98545325,  0.0743785277,  -0.337456018,  -0.490897149,  -0.225712255,  0.995400846,  -0.545504749,  0.729533017,  0.772145569,  0.561768353,  -2.28989029,  -0.729517937,  1.16388607,  -1.51237583,  0.849722862,  -1.5517,  1.2633065,  -0.405235201,  0.168420315,  -0.642201781,  -1.55757117,  0.538507342,  -0.981680632,  0.393101335,  -1.09554553,  -0.80158782,  0.612504244,  -1.0472151,  -0.875647724,  0.0694942474,  -1.20850933,  1.24260545,  -0.723309577,  0.23559688,  -2.01448154,  0.5602507,  0.371238708,  1.9128511,  -0.816904187,  0.393952668,  -0.675947309,  -0.302269101,  -1.30906153,  -0.506870329,  0.262364328,  0.157818496,  -1.24721718,  -0.399502397,  -0.0278145876,  0.72555691,  -0.772041917,  -1.41580343,  -0.824029922,  1.02511156,  -2.69361186,  0.433630675,  0.577690721,  0.493159473,  0.731461823,  -0.903451145,  -0.477989197,  0.405129224,  -1.0297358,  -0.0691578388,  -1.27888489,  -1.34490848,  0.299125284,  -0.0268992428},{ -0.422009528,  0.264723241,  -0.782831967,  -0.628614902,  -0.286134928,  -0.577269852,  -1.4357518,  -0.826922059,  0.0581972227,  -1.24621415,  0.158203766,  0.342764556,  1.27856362,  2.1794796,  -0.328548968,  0.400947541,  -0.00122392119,  -0.837151289,  0.775655687,  -1.75067031,  -0.789655566,  -0.395730674,  0.255359352,  -0.776610136,  -1.6206919,  -2.91287065,  -1.97563398,  0.809207797,  0.589970529,  1.24729931,  0.127649859,  0.880320847,  0.973808348,  1.19408607,  0.46623674,  0.965002418,  -0.322359473,  -1.07363725,  0.299894929,  0.261962861,  0.97395432,  -0.361744255,  0.235561281,  1.00217998,  -0.655517399,  -0.488901168,  0.813061297,  -1.00977457,  -0.145555496,  -0.176770255,  1.30580759,  -0.187568873,  -1.01547086,  1.02430129,  1.02676165,  1.29995728,  -1.12423849,  1.77028656,  -0.046738077,  1.56280553,  -0.333873063,  -0.189271286,  -1.01167047,  -1.70240247,  -0.882657826,  -0.35702306,  -0.746428609,  -0.450195074,  1.6534903,  -1.09902096,  1.50318027,  1.071998,  -1.00699437,  0.538143098,  1.11416256,  -0.131298825,  -1.09877622,  -1.7931447},{ 1.19763339,  -1.19411349,  -1.47608173,  0.813149154,  0.751681864,  -0.949995875,  -1.34880471,  0.540412903,  -0.705842376,  0.919516087,  -0.714058876,  -0.342732638,  0.178349361,  -0.422290683,  -1.16420996,  -2.03836441,  0.227221668,  -0.382745802,  1.63239717,  1.51459038,  -0.936727822,  0.954245448,  1.51856101,  -1.07109773,  -0.385077775,  -0.264555186,  -0.443349421,  1.85230577,  0.706252337,  -1.57349181,  -1.43644834,  -0.951792836,  -0.429307967,  -0.412053466,  -1.17846227,  -0.316097051,  0.518310249,  0.957656503,  -0.412883312,  0.423301399,  0.20642601,  0.855903685,  -0.359981716,  1.04735911,  0.45307973,  -0.283425272,  -0.166314647,  -1.05195546,  1.01070166,  -0.616856515,  0.575735211,  0.434539437,  -0.12383569,  -0.930325508,  0.286889076,  -0.0348009132,  -0.471856773,  -2.3293004,  -1.56076241,  -1.04261518,  -1.12115204,  -1.73517561,  0.282102048,  1.95217836,  0.286506116,  -0.0812010393,  0.840848207,  0.00716030831,  1.06358922,  -1.42406368,  -0.606043816,  -0.766219139,  1.3240844,  -1.30429983,  1.09412205,  2.07634521,  0.542831838,  0.0961218774},{ 2.04018521,  -2.17607713,  -0.481542736,  0.292676389,  0.273715615,  0.500542402,  -1.70701945,  -0.336599082,  0.369484454,  -0.689751208,  -0.23976028,  0.477470845,  2.48711491,  -0.0148767885,  0.257567465,  0.439433366,  -0.941505253,  -0.238064006,  -1.19440007,  -1.75737393,  0.36808601,  1.16258371,  -2.17625928,  1.52698112,  -0.362778276,  -0.839603364,  0.598376155,  1.19075215,  0.726372778,  -0.183822691,  -0.349610358,  -0.197109267,  1.17158294,  1.53273153,  -0.0410830118,  0.0554809198,  1.03756487,  1.05479109,  0.411254346,  -1.01358426,  -1.35353029,  1.14974308,  0.953085899,  -3.61243248,  1.86187363,  -0.762335122,  -0.248610169,  -1.22794282,  0.0374393016,  0.0457330085,  0.1335143,  -1.25260592,  2.00391388,  0.223774657,  0.118005529,  0.990288913,  -1.44069684,  0.693977177,  0.715759933,  0.423824131,  0.371994823,  -0.939026773,  0.147232398,  -2.5031538,  0.202764168,  0.570678949,  0.272859901,  0.809482634,  -0.401009083,  0.237792134,  0.681744456,  2.51667309,  -2.06546426,  -0.270660996,  -0.214171782,  0.762542367,  1.11254454,  1.21242332},{ 0.10752771,  0.392590195,  0.929857254,  -0.569441855,  -0.0165591445,  -0.433894992,  1.82636964,  1.68545318,  -0.782893598,  0.809042454,  -0.676206946,  1.11932468,  -0.196562082,  0.898289621,  -0.328012735,  1.31778073,  0.182236701,  0.33008036,  1.25005245,  1.63117015,  -1.68121898,  -1.95374036,  -0.340848297,  1.57519865,  -1.07663441,  -2.40227079,  0.615501761,  -0.33686921,  0.00834691711,  1.22370541,  0.789356768,  0.283448994,  0.614278018,  0.483439177,  0.0404925272,  -0.642532289,  0.249034032,  1.24999893,  0.630849063,  0.552630007,  -0.469270319,  -0.933624089,  -0.0368175171,  -0.394400507,  -0.708090603,  1.6992805,  -0.558934867,  -1.23751581,  1.06437051,  -0.255636483,  -1.57876575,  -1.15949285,  0.27750957,  -0.0452228896,  -1.82971025,  1.32545984,  1.57173657,  0.0535206869,  0.493149161,  0.26510945,  -0.875397742,  2.8465097,  2.43895125,  1.43441236,  -0.27488929,  0.195313856,  0.712773979,  -0.152282119,  -0.266819537,  0.408527553,  0.0737488791,  -0.283236474,  -1.24594951,  0.159302488,  0.159069911,  0.792153537,  -0.928062797,  0.888477802},{ -0.216558039,  -0.719584882,  0.00175218284,  -1.1222024,  -0.401341915,  2.63144708,  -0.467800856,  -0.442474931,  0.577391803,  0.493443787,  1.03764701,  -0.0362891071,  0.777691722,  0.620302141,  -0.473232716,  -1.49314547,  0.975580037,  -0.667852104,  -0.373847872,  -0.301292181,  0.513348579,  0.816491604,  1.0856812,  -2.22707701,  0.677612007,  1.30245626,  1.11659169,  1.01789796,  -0.180035472,  -0.66873312,  0.382883012,  -0.118306763,  -0.507474065,  3.95991683,  -0.892273784,  -0.864342451,  -1.2692405,  1.75773549,  0.0860361159,  0.232395113,  1.11312973,  -1.84638774,  -0.639327765,  0.631653965,  -1.97075427,  1.32770824,  0.811297238,  0.154951245,  -1.47886753,  0.680761158,  -0.324508935,  -2.93613029,  0.603364289,  0.0736070424,  1.36733592,  0.772589028,  -0.534090996,  1.4047066,  0.786611497,  0.29463172,  -0.3605977,  1.35949945,  -1.66795552,  0.457689583,  -1.16790247,  0.794634998,  -0.723783374,  -0.55780679,  -1.33862901,  0.305807859,  0.374985427,  0.661262453,  -1.70815849,  0.13347888,  -1.18674612,  1.28360415,  -0.810048401,  -0.660313547},{ -0.44022271,  -1.15669143,  1.42806029,  0.338632554,  0.307883888,  -1.23096454,  2.27071548,  -0.523615539,  0.0792576522,  0.28784886,  0.88931489,  0.464665294,  0.0603064224,  0.92303139,  -0.512294948,  0.565962136,  0.726595581,  -1.16025972,  0.312712997,  1.53377688,  -0.414238662,  -0.843425512,  -0.0365764238,  0.707706034,  -0.670722187,  0.614531994,  0.434777677,  -0.101573415,  -0.908884764,  1.19389856,  -0.766972721,  -0.762218773,  -0.340983838,  0.0569685549,  1.35179305,  0.516813338,  0.00148796965,  0.964200974,  1.14615405,  1.59355104,  0.335366726,  -2.58475876,  1.90990603,  -1.2082541,  0.166991696,  -0.305656672,  -1.23943055,  -0.757081926,  0.658427715,  0.209469557,  0.576305985,  -0.971586466,  1.28840554,  1.35545933,  -0.818684697,  0.785224259,  0.493342429,  0.214133307,  -0.875620902,  1.08668923,  -0.80325371,  -0.579801798,  0.660168886,  -0.419022918,  -0.591536045,  -0.0716197938,  -0.567202628,  1.96877682,  -0.629758596,  -1.52692127,  -0.401953667,  0.152730376,  1.41545391,  0.982141018,  0.963016391,  -0.513482869,  -1.07387996,  1.91736293},{ 1.64093077,  0.39898175,  -0.246633261,  1.47233617,  -1.49213696,  -0.213181645,  -0.616209626,  -0.344767451,  -1.50554299,  -1.83386803,  -0.400011271,  0.271693736,  0.805026889,  0.863702893,  -0.029499514,  -1.26240969,  -0.727022648,  -0.277176172,  2.18504286,  -1.66169429,  -0.75407666,  -1.50519562,  -1.10122573,  0.113855086,  -1.10694957,  -0.902298212,  -1.24330103,  -0.558010399,  0.198865846,  1.34527028,  0.7856673,  -0.0600988716,  -0.369573265,  0.320864975,  1.12801552,  -0.605705917,  0.0104793534,  -0.906884551,  -0.41773805,  1.11209607,  -1.58748245,  0.509101629,  -0.374094635,  -0.707631469,  0.053770937,  -0.573059678,  -1.20505965,  0.0650567785,  -0.911260784,  -0.300040483,  -1.29394734,  0.846780241,  -0.647387028,  -1.30217564,  1.46647668,  -1.29719818,  -0.228703782,  0.592348695,  0.0926069915,  0.657196581,  -0.265577048,  0.776423812,  1.01036096,  1.05634844,  1.70468295,  -0.0957589224,  -0.272185534,  0.177676424,  0.547167838,  -0.783365726,  0.780617535,  0.748758554,  -0.616571844,  0.352569759,  1.85334516,  0.754668951,  -1.23056006,  0.469845086},{ -0.132962316,  1.66880107,  -0.660250783,  0.367993385,  1.21861303,  0.648057103,  0.0785315782,  -1.0816735,  -0.94211638,  0.137785882,  -2.16273093,  -1.16897023,  0.117213629,  0.236559466,  -1.4153403,  -0.431530237,  -1.15167391,  0.0974676237,  1.7679249,  -0.339495867,  1.00465941,  1.31193888,  0.349476725,  -1.11246157,  -1.41704762,  -0.918026447,  -0.63685137,  -1.3191216,  1.58979464,  -0.181265026,  -0.0160874464,  1.13689518,  0.735087216,  0.518268466,  0.596454382,  0.623333216,  0.706895888,  0.579484463,  0.0426844582,  0.388194621,  0.166389674,  0.219896302,  -1.17538857,  0.415702075,  0.0392517857,  0.633281708,  0.00515229534,  -0.168584928,  0.0324175097,  0.542161524,  1.40892041,  0.424478978,  0.0797884092,  0.788467348,  -1.06879473,  -0.0355701931,  -0.379755497,  0.0975214317,  0.965069234,  -0.179610863,  0.6060642,  0.492151856,  -0.425534546,  0.812857449,  -0.627383709,  -0.0657406449,  -0.924117386,  -0.514603317,  1.1205653,  1.61318099,  -1.06659555,  -0.331891388,  -2.71541429,  0.0906978846,  -0.45911482,  0.834837794,  -1.53718221,  -1.50083804},{ -0.0548548326,  0.313070148,  -2.32221079,  0.113476068,  -0.351283938,  1.91259992,  1.18545318,  0.207750306,  0.13472499,  -0.523395896,  -0.742257655,  0.13989915,  0.232595831,  0.369596511,  0.27699694,  0.373714775,  0.710033,  1.11785853,  -1.15808511,  2.04643655,  1.17982376,  -0.10275352,  0.425293624,  -0.342466086,  -0.393794924,  1.33315015,  0.524485409,  0.376343399,  -0.695392132,  -0.23655127,  1.15801418,  -0.46405071,  -0.778924823,  -1.27748656,  1.74179125,  -0.13973704,  -0.384813786,  0.252516478,  -1.61438918,  -1.400352,  -1.49919653,  -1.68031311,  -0.319943339,  -3.59006023,  -1.01524496,  1.78424644,  -0.00518820947,  0.452503562,  1.06642795,  -0.0789046511,  0.381632447,  1.18136764,  1.39704859,  1.02422225,  -0.600984395,  -1.44545627,  -0.70976299,  -0.246245295,  -0.782335401,  0.438214898,  -1.58086729,  0.295606464,  0.0763295293,  -1.51288724,  0.843237758,  0.171452299,  -0.121338986,  -1.98213494,  -0.467435837,  -0.926253974,  -0.992609024,  -0.829140306,  -2.05953026,  -1.61469758,  -2.03994393,  0.00706021767,  -0.185309425,  -0.951238334},{ -0.0295088291,  -0.364888877,  -0.710404694,  0.608636379,  0.745793998,  -0.520059705,  0.183545172,  2.48314333,  0.199647024,  1.62095559,  0.237959623,  -0.505668283,  1.61848521,  -0.609077215,  -1.44909763,  0.687877178,  1.27658677,  -0.0306343231,  0.680055618,  0.463653594,  1.63636124,  0.64526397,  0.181711823,  0.44740966,  0.168963626,  0.132896349,  1.53511727,  -1.77663934,  1.18279052,  -1.92243552,  0.989170313,  -1.49334145,  -2.5197134,  -0.503623903,  0.73107022,  -0.94891715,  -0.548790455,  0.504657447,  -0.976836801,  -0.51444298,  -2.68451047,  -0.859591484,  1.54074633,  2.52315021,  -0.0719335154,  0.308556646,  -0.587270796,  -0.197281644,  0.303539813,  -0.776124835,  0.226483509,  1.25164008,  -0.8249318,  -0.537840307,  -0.086717017,  -0.734460115,  -0.0498607419,  1.4218334,  -1.85889173,  0.0652298108,  -0.341986924,  -0.106927119,  -0.775347769,  -0.818952084,  0.238537163,  -1.75763035,  1.4256593,  0.508334816,  0.333095342,  -1.59563923,  -1.52795494,  1.56000042,  0.00494906586,  -2.10445118,  -1.3114326,  0.258828014,  -0.643892348,  1.33385015},{ -1.15895748,  -0.273890257,  -0.14137274,  -0.226489946,  0.0798513815,  0.025935309,  -1.78536093,  -1.0533911,  -0.592577279,  0.318931609,  -0.653509676,  -0.34558475,  0.158062354,  -0.739676535,  0.0913956687,  -0.0387706794,  0.226556227,  2.20244527,  1.46658921,  -1.0408355,  -0.493682474,  1.75051522,  -0.31099546,  1.80072355,  1.40298057,  -0.808056056,  -0.651989341,  1.869326,  1.08051574,  -0.495552778,  0.41378215,  0.779322743,  -0.54766506,  0.0636842996,  -0.730651319,  -0.338844419,  0.852236807,  -1.15164006,  -0.732344031,  0.188609555,  -1.88395989,  -0.635592699,  0.629070222,  -0.671327233,  1.15536189,  1.96716642,  1.81937861,  0.10329359,  -0.0646202713,  -2.57573652,  -0.0670394078,  0.281244606,  -0.283479661,  -0.350418448,  0.185870752,  0.44912231,  0.47849974,  1.12088358,  0.996280432,  0.506468594,  0.638507307,  2.09924221,  0.591395497,  -0.943653345,  -0.114674702,  -0.491475016,  0.480440259,  0.458229184,  0.509521306,  -0.727802336,  -0.12607044,  -0.934175551,  -0.432290703,  0.0385767147,  0.722694278,  -1.66221392,  -2.1708312,  0.337898672},{ -1.38960803,  0.744349837,  0.683646858,  1.67024577,  -1.45008528,  -1.75380933,  -0.133961052,  0.272687137,  -0.174288779,  1.26893485,  -0.0312735476,  1.86968684,  0.605627716,  1.15709472,  0.831990778,  -0.916540265,  0.243602544,  -1.20636964,  1.5622983,  -1.02253306,  0.80739218,  0.82754606,  0.023723973,  -1.11972106,  -1.11730111,  -0.261489153,  1.91468596,  0.773715019,  2.91893816,  0.313321799,  -1.64328718,  0.129349694,  0.0916537866,  -0.940621972,  1.69023514,  0.923467815,  -0.546222925,  -1.78811514,  0.271883458,  0.0442771055,  0.337631613,  -1.0862056,  0.693207204,  -0.851528764,  0.815713048,  2.31906819,  -0.549182296,  -0.6700418,  0.663639069,  -0.872457564,  0.227395579,  1.11005175,  0.354114652,  0.792444944,  0.199813485,  -0.667071819,  -0.03443712,  2.67865896,  0.408571035,  0.137474254,  0.191284642,  0.388526738,  -1.46324837,  0.559787035,  -0.817441344,  0.478048474,  1.36018956,  0.50536871,  -0.386552185,  -0.654699445,  -0.983135819,  0.925026715,  0.544222355,  0.183419719,  -0.253339559,  0.685700059,  -0.277241021,  -0.925274372},{ 0.654262722,  0.656212032,  0.415753335,  -0.614291012,  -1.79224861,  -0.517472088,  -1.5091821,  -0.772706628,  0.980031013,  0.632535279,  -2.59157014,  1.00220501,  1.72891784,  0.668881595,  1.32982028,  0.245816752,  0.41515404,  -0.170377344,  1.03106189,  0.636313796,  0.137303025,  1.54749012,  2.56969905,  -0.484591395,  -0.758108974,  1.46461499,  -0.290880412,  0.53459841,  -0.351178408,  0.243790641,  2.02896571,  -1.75683951,  -1.21590471,  0.673652053,  -1.87959993,  -1.07335663,  -0.620764196,  -0.474193662,  -0.38767755,  -0.714006245,  0.248679698,  -0.906499565,  0.594225943,  0.029037809,  -0.555438817,  -0.528977513,  0.473033786,  -1.68757319,  0.296018124,  -0.245091811,  0.372091621,  0.738846123,  1.17504966,  0.59705925,  -1.15387845,  -0.545602977,  1.38575196,  -0.501168907,  -0.381379157,  0.819773555,  -1.13643312,  -0.398376524,  1.78667462,  -1.29493487,  -0.249926507,  0.412246615,  0.177050665,  0.112147972,  1.54558396,  0.913904727,  -1.77081764,  -0.495568305,  -1.13932216,  -2.82997775,  0.307829112,  -0.454516798,  -0.425176203,  0.949736774},{ -0.326794416,  -1.29869521,  0.968604863,  -0.649941802,  -1.54637516,  0.446030319,  -0.040795967,  -0.229747221,  1.32194567,  0.833482981,  -1.46674478,  0.0423088856,  -1.48489392,  1.42771745,  -0.487946332,  -1.01555884,  -0.617037117,  0.941062629,  0.136982009,  -0.099596329,  -0.855005682,  0.537978709,  0.644359827,  0.160789713,  -0.127293974,  2.59544563,  -2.80361724,  0.889859676,  0.906624317,  0.63771683,  0.661056459,  1.74959755,  1.07944787,  0.770145297,  0.0717219189,  1.22389972,  -0.859860957,  0.0556445196,  -0.781985044,  -0.967278838,  0.327788472,  -0.83690238,  -0.213399276,  -0.875504553,  -0.359030485,  1.63583064,  1.6661371,  0.629537642,  -0.683592319,  0.716290295,  0.548765719,  -0.214068383,  1.0352844,  -0.553507745,  -0.181015447,  0.143502817,  1.86980522,  -0.513828754,  0.362050474,  -1.03251708,  0.489306927,  -0.44056493,  -1.54579425,  -1.26297271,  1.52096689,  -0.136026561,  -1.44778204,  -2.24522209,  1.03901267,  -1.24740493,  0.839652956,  -0.185901776,  1.15174043,  0.634126663,  -0.967655122,  1.0658313,  0.154331252,  1.34742212},{ 0.584518015,  -0.171622068,  -0.00645994907,  -0.828246415,  1.08868706,  -0.27581501,  -0.158904582,  -0.266517729,  -0.22265473,  -1.75000584,  -1.11992657,  2.16446137,  1.52991438,  -0.148984239,  -0.400476694,  -0.578863859,  0.712706029,  -1.38250315,  1.77627254,  -2.86313725,  0.104451194,  -0.0512180515,  0.153664157,  1.62664676,  -1.09332979,  0.614488184,  1.84818673,  -0.176173136,  0.671446145,  0.362475961,  1.72854829,  1.31049728,  0.152236462,  1.58183968,  -0.841278791,  0.759945691,  -0.324436218,  -0.218719333,  -0.25438574,  0.664191008,  0.906366885,  2.0867002,  0.625059783,  -0.280134588,  2.99889541,  -0.700158656,  -0.824747801,  0.247509509,  0.113419272,  0.0456952676,  -0.723084927,  -0.0235838257,  0.0836739764,  -1.08446133,  1.05170119,  -1.503546,  -0.931396604,  -0.41314432,  -1.24694169,  -0.704712391,  0.552140296,  0.518358886,  -0.447436392,  0.0512358062,  1.30824673,  1.24889863,  0.715898573,  0.320052356,  -0.123715803,  -0.843135297,  -0.0382237807,  -0.584617734,  1.03416944,  1.86614799,  2.06281829,  1.66185737,  1.12128329,  -0.893794715},{ -0.852469325,  2.19923139,  -0.561107099,  0.129270911,  0.115808226,  1.24514973,  -0.0373549201,  0.38698405,  -0.161552012,  1.40858746,  0.906055212,  0.257784784,  0.0486913919,  -1.70985425,  1.77375984,  0.741878211,  -1.73293567,  0.134564295,  1.16450977,  -1.10009837,  0.0535123125,  -0.614842713,  -0.056607645,  -0.0858058259,  -0.503966749,  0.368988574,  -0.899286091,  -0.819376469,  0.238192484,  -1.08481646,  0.443669826,  1.02683437,  -0.361792058,  0.120283253,  0.0247029513,  0.606668532,  0.201926872,  -1.98505068,  0.14464201,  -0.343204349,  0.265695781,  0.320550591,  -0.279668152,  -0.148998007,  0.764033437,  -0.858987629,  -1.69993186,  0.517787099,  -0.488772303,  0.458550423,  1.10667145,  0.832800984,  0.43567878,  0.434544861,  -1.32985675,  -0.349936604,  1.1472646,  -0.932747126,  -1.88894415,  0.576015055,  -0.651540875,  -2.8146348,  0.501889408,  -0.381439656,  0.53171581,  -0.501791179,  0.414982319,  -0.744956315,  1.18283272,  0.406760961,  0.457130134,  0.139628455,  -0.892623723,  1.04903865,  2.09009099,  1.30898941,  0.608577967,  0.828231454},{ -0.388952851,  0.719192326,  0.935465693,  -0.859239101,  -0.769133866,  0.976571441,  -0.377993673,  -1.30979609,  -0.0328754634,  0.0281113274,  0.991171479,  0.0484411642,  0.197088212,  -1.61208785,  -0.0585891828,  1.5711261,  -3.26405454,  1.04797506,  1.91030955,  0.165028259,  0.0389874466,  -1.71120107,  0.222400054,  0.146899596,  -1.47207808,  0.51836586,  -2.40509033,  -0.398988456,  -0.629222035,  0.140151605,  1.19563711,  -0.402500093,  -0.0453221835,  -1.31609154,  -0.512107432,  -0.0762269497,  -0.761439979,  0.221445158,  1.72485459,  1.13929665,  1.07046878,  1.36039996,  1.27078772,  1.10717523,  1.43486381,  -2.19253182,  0.910776556,  0.0124119371,  -0.369760066,  -0.38828966,  0.262782872,  -0.119688354,  1.24555397,  -0.193272352,  -0.905767441,  -0.22002095,  1.36632609,  0.141646549,  1.37431991,  1.60251272,  -0.610894263,  1.09423566,  1.31839406,  0.576691628,  -0.129417792,  -0.979357302,  -0.0844842046,  0.900130451,  0.86634469,  0.429360539,  0.964438319,  -1.32106054,  -0.415718436,  0.96058929,  0.261350274,  -1.78024113,  0.993937194,  -0.927473068},{ -0.524185658,  -0.210413367,  -2.4975071,  0.502897561,  0.493301928,  -1.97814143,  0.274938405,  -0.422227263,  -1.72002316,  -0.875961304,  0.660990953,  -1.73512137,  0.679612339,  -0.368356168,  -0.0808227584,  -0.454941511,  -0.331464708,  -0.991559327,  1.49114823,  0.624241769,  -1.09657955,  0.707360327,  -1.34784615,  1.30864036,  0.285066247,  -1.23766541,  -1.06217635,  1.2198863,  0.419421107,  0.405711561,  0.16136077,  1.33696663,  0.290610492,  0.17392157,  0.0503087603,  0.4665277,  -0.857953429,  0.828902245,  -1.62955558,  -0.525268793,  -0.319290757,  1.03561819,  0.407102972,  -1.43781698,  1.64788437,  1.46423662,  0.971749008,  0.655767322,  -1.5325793,  1.82371807,  -0.115813144,  0.0617526844,  0.162004605,  -0.597264647,  -0.393842131,  1.279585,  0.301347971,  -0.430882096,  -0.406669945,  2.12906957,  0.3419815,  -0.727452457,  -0.782553136,  -0.155693874,  -0.297844112,  0.196932301,  -0.0516046733,  -2.17016387,  0.079067044,  0.486964613,  2.65519166,  -1.07872033,  1.59799659,  0.0496193729,  0.860768676,  0.793081999,  -0.0274806488,  -0.354674101},{ -0.534884036,  0.929838836,  1.68486571,  2.05412459,  -0.223506436,  -0.12499173,  -0.626384735,  -0.398963124,  1.89530265,  -0.432863683,  1.09277737,  1.2120322,  0.41382584,  0.610459268,  -0.862023115,  0.917305827,  -0.343649596,  -0.271405667,  0.485914588,  -0.0926930532,  -1.59130812,  1.17776072,  0.03669516,  -0.465785176,  0.459163576,  0.959686041,  -0.641702473,  -0.290326089,  1.50802469,  -0.70386976,  -1.88087761,  -1.66208923,  0.0453920327,  0.5268448,  -0.0189283602,  -1.25024986,  0.903037429,  0.686246276,  -0.211504698,  1.18996918,  0.396201402,  1.41560912,  -1.22747707,  0.832455575,  0.843113244,  0.317435354,  0.845241845,  -0.73865968,  -0.346498728,  -2.94329381,  0.956189096,  -0.605675042,  -0.560322762,  2.55151987,  1.45193541,  -0.356857002,  0.590815723,  -1.36086917,  2.42973566,  0.151629373,  -0.168280855,  -0.996648669,  -0.269453943,  0.191945344,  0.31707862,  -0.627008557,  0.0681681708,  0.165985167,  1.90117145,  -2.88780236,  -1.14726889,  -0.37980631,  -0.220129654,  1.03497434,  0.684013486,  0.839943171,  0.426746786,  -0.108734816},{ -0.945587873,  0.238385499,  0.145610094,  -0.797363818,  0.186163157,  0.0850764662,  -0.810743034,  0.489829689,  -0.910664737,  -0.358468652,  -0.989764929,  0.823212802,  0.94405061,  -0.0659208372,  0.317848742,  0.0142402891,  0.101617649,  2.00381374,  -0.304291993,  -0.918850482,  -1.13252068,  -1.32754993,  1.09630704,  -0.010164815,  -0.638122499,  1.55208254,  -1.22718287,  1.64543462,  0.213077307,  -0.532416761,  -0.335145712,  0.103902318,  2.30786419,  -0.202127397,  0.125964761,  -2.11108184,  1.81342745,  -0.903623521,  -0.556413293,  1.28062868,  1.72095585,  0.162357301,  0.417596906,  -2.09491873,  -1.00005543,  0.954770803,  -0.200577259,  0.527796149,  2.52710009,  -0.934441805,  -1.0941323,  1.50384772,  -1.08620703,  0.347497135,  -0.132132977,  -1.48497283,  0.834591925,  0.215648323,  0.620253742,  -1.25550377,  2.48472524,  0.175901726,  0.360071391,  0.708149195,  -0.0179592781,  0.565898538,  -0.0887509063,  0.546394885,  0.493095189,  -0.173251644,  1.41856289,  1.83876657,  0.149613127,  0.720034361,  -1.70789683,  -0.0218204074,  0.420190573,  0.332835436},{ -2.23636866,  1.33253908,  0.540298522,  -1.35236371,  -0.403144181,  -1.05095184,  0.0615208149,  -0.914195478,  -0.343878716,  -0.90368861,  1.66071808,  1.87040138,  0.607701302,  -1.35091734,  1.0432179,  -0.176299691,  -0.821519554,  -0.868935704,  -0.348992288,  -0.978494108,  -0.884715915,  0.796780825,  -1.1667949,  -0.46676901,  -0.0322799161,  -0.211787075,  -0.597939968,  -2.34254456,  1.15374756,  -0.887561917,  -0.558882177,  0.973326385,  -1.06071377,  2.12643766,  -1.49515808,  1.78837025,  -1.14183152,  0.328400522,  0.314571142,  -0.00396542158,  -1.36157811,  0.535292029,  0.599726975,  -0.00288687251,  -1.69010532,  1.50961757,  1.17270339,  0.513671219,  -0.572821498,  -2.32967496,  1.05784345,  0.500572741,  -0.0500500835,  1.01624584,  0.150581852,  -0.0768207237,  -3.15402126,  0.255416125,  1.63827705,  -0.0630751848,  -0.136744156,  -0.504576266,  -1.06370533,  -1.03209496,  0.328242242,  -1.82495427,  -0.187636673,  0.982247889,  1.58459878,  -0.972673893,  2.71099782,  -0.299423993,  -0.189046383,  -0.551534176,  -0.5561409,  1.38887751,  -0.424491853,  -0.483825296},{ -0.0494512692,  2.04735565,  0.75893569,  -0.502772808,  -1.7346952,  -1.28710926,  -0.652163327,  -0.256474584,  1.0578804,  0.647142053,  0.116136573,  1.14348137,  0.817262709,  -1.37419355,  1.33119655,  1.27105403,  2.3714447,  1.05732286,  1.71934319,  0.0697026774,  -0.953063071,  -1.05288231,  1.33689404,  1.10375917,  0.663501024,  -1.15332508,  -0.779007614,  -1.50195777,  0.220398903,  -1.42013562,  -1.49160719,  1.53713274,  -0.130933538,  -1.17602432,  0.119022913,  -0.076657936,  -1.91951406,  0.698531032,  1.11531639,  0.482582569,  1.20767236,  0.0540809073,  -0.585435569,  -0.941429138,  -0.214978918,  -0.336702108,  0.858062565,  -0.176844195,  -1.24320757,  -1.64480662,  -1.08692563,  -0.968667984,  -0.0311424863,  0.646890938,  -1.3532573,  1.01523697,  -0.812839866,  1.35888767,  1.38099968,  1.87712586,  0.655514419,  0.134029567,  0.153621688,  -0.915515184,  -0.694715738,  -0.959502518,  -0.10560932,  -0.15675582,  0.988045454,  -0.228148535,  0.746274829,  2.09724331,  -2.32374883,  0.721559048,  -0.805401087,  -0.789850712,  -1.64988434,  -0.927032828},{ -0.0364180468,  -0.245682091,  -0.93935132,  0.766602993,  2.49094582,  0.057185214,  0.332692623,  -0.344720304,  -0.906774461,  0.100585915,  1.58191788,  -1.17481363,  0.684106886,  -1.05414033,  -1.38815498,  0.904609263,  -0.610865772,  0.0119768893,  1.12286949,  -1.29465055,  1.14112508,  1.97699845,  -1.07105458,  -0.319048405,  0.542114556,  -0.676636517,  1.57981467,  0.70790118,  -0.175551474,  0.908970535,  -1.56823885,  0.055721011,  0.217867598,  0.305134654,  -0.0324356332,  -0.0165557023,  -0.903284371,  1.23667097,  0.946277142,  -0.911871076,  -0.570512474,  -1.06095278,  -0.520762205,  0.505430281,  -0.888457239,  -1.02735806,  -0.638271332,  -1.81971622,  0.405957699,  -0.463242829,  -1.56568241,  -0.667637587,  0.598759651,  0.341873914,  -1.88971519,  0.773072124,  0.17879492,  0.591337979,  0.508985102,  0.450807512,  -1.2003932,  -2.24063277,  0.0795150995,  -0.430309206,  -1.05122554,  -0.362961888,  0.56253612,  -0.736502528,  -0.355682701,  2.20740771,  -0.354738444,  -0.420894057,  -1.59644401,  -0.31123203,  0.539980471,  -1.1268754,  -0.613834381,  -0.532868803},{ -0.126864702,  -0.785527647,  -0.370012671,  -0.284624904,  -2.62024736,  0.178426042,  0.0381531008,  -0.0930515751,  -0.806511223,  1.35066521,  -2.48120165,  -0.875079036,  -1.0123229,  0.615822494,  -0.405877829,  0.668887198,  -0.493759096,  0.950576365,  -0.0892530978,  1.93519127,  -0.501402676,  -1.19487,  1.41609561,  0.982971072,  -2.44392419,  0.574145436,  -0.881756365,  -0.815319061,  0.26963076,  -0.335148036,  -1.36954057,  -1.73677027,  -0.0110943429,  0.760878503,  1.25500596,  1.29325509,  -1.07156515,  -0.152938038,  0.898136795,  1.97738469,  1.68542731,  1.10016561,  1.07798696,  -1.3081435,  -2.15766716,  0.411372721,  0.0914341658,  0.962620854,  0.556617439,  -0.226814955,  0.150713235,  1.13249004,  1.37277365,  0.125285983,  -2.37695765,  -1.66065311,  0.164174721,  1.13027251,  0.823946297,  0.248819128,  -0.806663573,  -0.953870356,  0.228858992,  -0.342028946,  2.69850302,  -0.184397489,  1.45373547,  -0.278997749,  0.722100794,  -0.812846124,  0.133252338,  0.731847763,  -0.620489776,  -0.995427251,  1.10079718,  0.886360943,  0.482310534,  0.333601147},{ 1.06137002,  -1.21638966,  -1.15029466,  0.741228104,  -1.70288432,  0.673319399,  1.06060755,  -0.973319769,  0.163128421,  0.711779594,  -0.160505444,  1.37226903,  1.69326651,  0.242923975,  0.586742282,  -1.74260271,  -0.0877261311,  0.2599805,  0.601584077,  -0.825027525,  -1.0560782,  -1.02342391,  1.3383652,  0.048952926,  -0.228354454,  1.54618013,  0.85365355,  -1.02323854,  1.0418123,  -0.809446871,  -0.463119507,  -0.585072458,  -0.614224434,  0.247678593,  1.9763068,  1.59494328,  1.51629889,  0.263914675,  0.0545916669,  -1.74929416,  -1.38720322,  2.06854177,  -0.705839276,  0.0236143637,  -0.746685207,  -0.663262367,  1.03365695,  -0.879748464,  -1.95432663,  1.39620042,  0.924338996,  0.154811919,  -0.180717677,  -0.70906198,  0.197617158,  2.13269234,  -1.85011673,  -0.01784035,  1.66706431,  -1.26550734,  0.210046172,  -0.437578648,  1.11667037,  -0.069909133,  1.48684549,  1.27029026,  -1.52271652,  -0.532103658,  -0.524304152,  0.66779989,  -1.2307775,  -0.509146392,  0.41721791,  0.0699279457,  0.589181662,  -0.185719147,  -0.728477359,  1.15590143},{ -0.437979013,  -0.314264029,  0.125753194,  -0.583149791,  0.420858473,  -0.334901541,  0.515801966,  -0.777126908,  -0.753491223,  -1.1179173,  -1.20303953,  1.46273553,  0.458629459,  0.742142797,  -0.111857302,  -0.760140538,  -0.771381378,  -0.0886500329,  -0.0208435804,  -0.754251242,  1.40809083,  -0.9730106,  0.180179074,  -0.15963614,  -0.0885275453,  -0.810965061,  0.764754236,  -0.944258571,  -0.453768939,  0.53457582,  0.011777252,  -0.169319421,  -0.255734593,  -0.394703597,  -0.164054841,  -0.59740752,  -0.37874347,  -0.615437925,  0.761281669,  -0.496352166,  -0.83418715,  1.11390984,  0.484648883,  1.9478581,  0.586532176,  2.98300743,  -0.359353542,  -1.61820805,  -1.65096068,  -1.73951113,  -0.0324381478,  -0.115297347,  0.695480108,  0.163723305,  -0.300914109,  -2.52456141,  0.278196186,  0.226695314,  -0.597650647,  0.580918372,  -1.08646834,  0.525488555,  -0.770916343,  -0.711855829,  -1.14750063,  -0.390288442,  -0.677146494,  -0.0251341425,  -0.689010561,  0.707271636,  0.00751131726,  -0.238844618,  -1.14339101,  0.738744557,  -1.34930933,  0.0660707951,  0.989737391,  -1.1147815},{ -0.436546564,  -0.258224249,  0.827557445,  -0.27883625,  -0.966243505,  0.450580478,  0.125978038,  0.757127106,  0.956716657,  2.47903776,  0.835115016,  -0.969409883,  0.71685499,  0.607948601,  0.741530657,  0.857694745,  0.574320793,  -0.475036025,  -0.803502142,  0.578660667,  0.837411582,  -2.16700721,  0.124368422,  -0.0889302418,  -2.50151324,  -0.332269788,  -0.642710328,  1.1633451,  1.35017014,  -0.375701278,  -1.86354744,  -0.104320295,  -0.287535042,  0.760421515,  0.558132291,  1.42352057,  -2.85698748,  0.945176661,  0.494943649,  0.681014419,  -0.32202217,  -0.556624115,  0.490508944,  -0.158873022,  -1.9407413,  -0.981397748,  0.3469643,  -0.672213614,  -0.623772204,  -0.930494249,  -0.160122842,  1.38187003,  -0.559534013,  0.579706609,  -1.58384717,  -1.03882587,  0.0280356985,  -0.84391135,  0.66934365,  0.200804591,  0.309321553,  0.811317861,  -0.291515559,  0.549139082,  0.609821856,  1.80796659,  -1.48663628,  -0.497506678,  2.58490372,  0.273267418,  -0.332385838,  0.936273098,  -1.39677513,  1.28947771,  0.290949464,  -0.254904807,  1.83079076,  -1.4053899},{ -0.243324563,  0.808128655,  -2.20457339,  -1.28334439,  1.36251473,  -2.06294227,  0.04882266,  -0.251396596,  -0.956647635,  -0.287144423,  1.64935958,  -0.264524043,  -0.236652344,  0.261564672,  0.438458532,  -1.78988457,  1.1944356,  -0.491419017,  -0.298695236,  -1.19537318,  1.72317946,  -1.30131805,  -0.343197554,  0.222736955,  -1.01775527,  0.0977063999,  -1.19223046,  -2.17466331,  0.20563738,  -0.646800876,  0.126018822,  -0.00701214466,  0.650716066,  0.8820104,  -1.8832655,  1.34351432,  2.10114288,  -1.16119158,  0.203114629,  0.384728849,  -0.812872767,  0.793031633,  0.213275701,  0.442329705,  0.245193377,  -0.754766047,  -0.475287199,  1.7873143,  0.665234923,  0.420156389,  -1.3500663,  -0.724251509,  -0.33192268,  0.361095786,  -0.618735671,  -1.00116527,  -0.749340832,  0.793226957,  0.699509442,  0.802448809,  2.0488112,  1.34566677,  0.330658644,  -2.44219542,  -0.666597664,  1.25123024,  0.980976284,  -1.29317904,  0.973732293,  -0.729598761,  -1.53229177,  -0.863179445,  0.284534693,  1.2625978,  0.452563137,  2.02205229,  -0.658720315,  0.854019582},{ 0.830459833,  1.49621224,  1.19491887,  1.22032833,  0.583054364,  0.253783643,  0.329975992,  1.0293926,  0.0752508342,  0.16955708,  -0.121942051,  -0.61861819,  0.400965929,  0.690102756,  0.725840211,  0.215792358,  -0.173235044,  -1.34323692,  0.238817677,  1.11616218,  -1.00885832,  1.65745962,  -0.134706557,  -0.0732130408,  -1.0270015,  1.26941621,  0.0447367169,  0.569188714,  0.0180950873,  0.0351940542,  -0.18939364,  -0.482236087,  0.448752135,  -0.391384631,  -0.0232289694,  0.299350798,  0.817306161,  -0.526168108,  0.87537092,  -0.0194622893,  -0.706227183,  -0.682452381,  0.0297054648,  -0.0692187846,  -0.479268134,  1.32932091,  0.528623581,  0.369031549,  1.49869919,  0.493686289,  0.893182874,  0.0528059714,  0.454797566,  0.743054688,  -1.86312306,  -1.7143451,  0.491014659,  -1.10232627,  0.739448845,  -0.38621518,  -0.446226686,  0.408188641,  -1.34035885,  -0.317981511,  0.0017136113,  0.774759054,  2.07125664,  -0.96688503,  0.183858216,  0.581557631,  0.0317177512,  -0.0922096595,  -1.4872874,  -0.2040627,  0.446029782,  -0.918789923,  1.54357934,  0.375071138},{ 2.00047994,  -1.5415045,  -0.678283572,  -0.331353366,  0.417386264,  -1.1180954,  1.73351634,  -1.03258824,  0.134581819,  0.518926203,  -0.31887722,  -0.925581455,  -0.868314743,  -0.0712840632,  -1.03780186,  2.35229564,  0.463750213,  0.18415001,  -0.384175777,  -1.27136695,  0.567648351,  2.37437797,  0.592398226,  -1.32524264,  0.993740082,  -0.206332549,  1.05293512,  0.646526873,  0.347149789,  -0.814580023,  -1.71009815,  0.599973381,  2.15087867,  -1.39682651,  -0.193848014,  -0.121721528,  -0.850113153,  0.818656683,  -0.793101549,  1.1896224,  -0.241409346,  -0.480708718,  1.25190067,  0.478490353,  -1.19048178,  0.430065542,  0.923196673,  0.02660303,  -0.669011354,  -0.841114223,  -0.708292603,  0.398273051,  0.522544622,  0.273953378,  -0.730422616,  0.0206836965,  1.03975797,  1.76418614,  -0.355072588,  -0.020397447,  -0.144367129,  -0.107010506,  1.1728828,  0.839131773,  -0.550528347,  -0.653248429,  -2.00059342,  -0.0552375391,  0.836967349,  -0.0164973065,  0.293239117,  -0.593637347,  1.01070225,  -0.762524009,  0.227232441,  1.65449321,  -0.403277755,  -0.666491508},{ 1.06787765,  0.564941347,  -0.083442688,  1.59268403,  -0.407887161,  0.109800227,  -1.25793564,  -0.356369317,  1.18860352,  0.953789115,  0.574324489,  1.72562361,  -0.921692312,  0.900458038,  2.89423251,  -0.349181741,  2.03567123,  0.504583061,  -1.07183552,  -1.35641599,  1.13912284,  0.678619802,  2.65201402,  -1.36773252,  0.723097146,  0.29558605,  0.484286577,  -1.34638739,  0.120704666,  -0.195277587,  -0.361586004,  -1.3701762,  -0.863981843,  0.749488473,  0.154260024,  -0.388439596,  0.915353537,  0.157414556,  -0.547378242,  1.61911988,  0.0123107452,  -0.758736372,  0.865285993,  -0.246978417,  0.143343866,  -0.507841766,  -0.585940063,  -0.29320392,  1.4304651,  0.28288877,  2.79442787,  -0.763963282,  -0.754096806,  -0.0645778775,  -0.938440263,  0.527305841,  -0.77250576,  1.56457222,  -0.467631072,  0.33200258,  0.872259736,  -0.111582749,  -0.863626719,  -1.76006198,  0.0781434104,  0.366097689,  -1.00843179,  -1.69608355,  0.349759847,  0.420242667,  0.051299151,  0.094337292,  0.786628485,  0.886766553,  -0.527761817,  -1.01928413,  0.456843853,  -0.784409463},{ -0.34046638,  -0.836937726,  0.536144197,  -0.492776364,  1.98876607,  1.46172607,  -1.29750323,  1.41865134,  0.689876378,  0.0894715562,  -2.84788251,  -0.639633596,  0.250617892,  -1.20025742,  -0.906975985,  0.367111981,  1.48299837,  0.67948848,  0.0767990798,  -1.31906807,  -0.387101114,  -1.89331663,  0.334915847,  0.514989614,  -0.360510677,  0.200665236,  -0.0459782816,  -1.80649054,  0.521666408,  0.368102431,  -1.72573674,  2.15885639,  -0.448705226,  -0.0751636103,  1.28073668,  -0.605044961,  0.254957139,  0.417185098,  1.46142018,  1.18375504,  -0.911851406,  -1.27589333,  1.02345073,  1.60972393,  0.916448772,  -0.0773975104,  0.879337013,  1.17396092,  -1.69470656,  -0.222558334,  0.744549274,  0.169622213,  1.40734553,  2.43526888,  0.415168017,  -1.36263454,  0.345765591,  1.2070514,  1.18274283,  -0.242445618,  -0.284268826,  1.56742954,  -0.631347716,  -0.343269438,  0.0484534763,  -1.60533297,  1.07400835,  0.608272731,  -0.208954096,  0.192080304,  0.58992815,  -0.654264748,  0.0670915693,  -0.373879671,  -1.41625381,  1.26842916,  0.615316212,  -2.49158263},{ 1.61285615,  -1.73470545,  0.593150377,  -0.100533612,  -2.40587497,  0.868631124,  0.0242310725,  -0.677205503,  -0.319274575,  -0.583417416,  2.25002837,  -0.891150951,  -0.986056328,  0.0686337054,  1.83858728,  1.50851309,  -1.26329494,  -0.604619265,  1.03104234,  1.00710297,  0.513913393,  0.577744901,  -1.34995842,  -0.305281073,  -0.311001778,  0.554476798,  -0.895053506,  0.829554856,  -1.31961715,  -0.428352535,  -0.532778442,  -0.535662234,  -1.58390403,  0.0489403605,  -1.1830951,  -0.616149902,  0.72378999,  -0.954502165,  0.865884185,  1.01426458,  0.681251705,  2.38594604,  -1.13466585,  -0.775362551,  -0.334513187,  -1.13868988,  -0.734664321,  -0.189611673,  -0.668005764,  -2.27693319,  0.56749475,  -0.39467746,  -1.63929689,  -2.25046706,  -1.31168282,  -0.592813492,  1.53042722,  0.10758388,  -1.31379104,  -0.311661988,  -0.985955179,  -1.25132239,  -0.714856863,  3.15170741,  -0.20818004,  0.193142653,  -1.47169387,  0.482676059,  3.01658034,  -0.708938301,  -1.17350638,  -0.662907362,  -0.322967172,  0.0870887637,  -1.78330064,  -1.40094948,  0.03906307,  -0.308525473},{ -2.6060667,  1.58447242,  0.268954575,  0.778524935,  -0.60797143,  -0.37315008,  -1.7514832,  -1.03647482,  -0.429914296,  0.425388336,  -1.27932942,  -0.268139809,  0.623736262,  -0.959343255,  -0.959649801,  -0.618334591,  0.947960854,  -2.09181213,  -0.990932643,  1.69519663,  -1.200652,  -0.876511455,  1.79307997,  0.108610511,  0.24073863,  0.278638989,  -0.121388204,  0.858658314,  -1.63355041,  0.361016184,  -0.891522586,  -1.3210597,  1.54031229,  -0.36586827,  -0.776764274,  1.94274378,  0.60995692,  0.68876344,  -0.0261341445,  -0.0583597049,  -0.856775522,  0.142769545,  -1.43294144,  0.391952991,  -1.49867451,  -0.923132777,  0.127222776,  -0.308737546,  -2.51794076,  0.677883625,  -0.301299989,  -0.438005269,  0.186879277,  -1.668504,  -2.10775518,  0.725131452,  -0.854735196,  1.7567836,  0.406037509,  -1.07750535,  1.26648033,  0.0370415896,  -1.84003103,  -0.748438716,  -0.043645639,  0.677708149,  1.49668849,  0.747530222,  1.00830972,  1.55061984,  -0.616042614,  0.756315589,  -0.61126405,  -0.284118474,  -0.948315024,  -0.752479196,  -0.0258926675,  -1.50602651},{ 1.38260424,  -1.01223207,  -0.455723971,  -0.104088739,  0.306737036,  -0.950399339,  0.555732965,  1.71206057,  -0.643064916,  1.45174074,  0.169044763,  -1.20989764,  -1.62589622,  0.605387866,  0.124775447,  1.19469476,  -0.407447189,  0.419565171,  0.928580344,  0.563749611,  -1.06194234,  -0.105123803,  0.434972227,  -0.1934589,  -0.171445623,  -0.245265886,  0.792697191,  0.0459647365,  0.000625082117,  -1.84615052,  0.680620372,  0.056238208,  0.992689073,  -1.14559102,  -0.959267497,  0.663030505,  0.639488399,  -0.694773018,  -0.916181862,  -0.624272525,  -0.321857482,  -0.545595646,  -0.239541665,  -0.636342347,  0.0195453763,  -0.392741323,  0.976306736,  -0.889252245,  0.410663962,  0.489509344,  0.902384102,  0.304519713,  1.33975732,  0.348609656,  0.133293927,  -0.0188347194,  -1.05293643,  -1.05865645,  0.650236487,  0.785550833,  -0.0737809092,  1.33786345,  -1.41325605,  0.473780304,  -0.210851893,  0.0152008776,  0.240042612,  0.792950749,  1.86575747,  -0.229212716,  -0.651643336,  -1.29534221,  0.843494534,  -1.15911365,  -1.80233216,  -1.74258196,  0.185840264,  0.977323472},{ 0.934396505,  0.620985925,  -0.652015448,  1.1052736,  -0.977578998,  0.573090076,  -0.399919629,  -3.11447334,  -2.13020587,  -0.961430907,  0.839441895,  -1.3223865,  -0.997971296,  -1.57182217,  1.26032686,  -1.53712332,  0.819910467,  0.171983093,  0.964366615,  -1.31646013,  -0.17807132,  0.848127544,  1.87842202,  -0.22079879,  -1.01403403,  -0.675900817,  -0.0662861615,  -1.80712283,  0.368807882,  1.70124149,  0.51690501,  0.0971487761,  -2.62315679,  -1.49103999,  0.270193547,  -1.549353,  0.177466273,  1.28513384,  0.569063008,  -0.654127002,  1.37132239,  -0.0947853252,  0.777065098,  -0.844581246,  -0.118275531,  -1.03201377,  -0.991569877,  0.484392554,  -0.246396199,  -0.258772671,  -0.662648678,  -0.115396097,  -1.02421021,  -1.28020918,  0.55295831,  -0.0848043114,  -0.756003797,  -1.03848076,  0.661136031,  1.78021109,  -0.633172929,  0.282997906,  0.969450593,  -1.25333726,  -0.966385007,  0.299116999,  -0.716156185,  -2.07194638,  -0.711051524,  -0.487157375,  0.721654475,  0.324866563,  0.839609623,  -0.829303801,  1.33650517,  0.0381657481,  0.819277644,  -0.109400027},{ 0.598222733,  -1.99372435,  0.567370832,  0.207819656,  -1.35434449,  0.217783034,  -0.619045436,  0.582903028,  0.361487299,  1.01516306,  0.755759597,  -0.77763015,  -0.749259949,  0.11729034,  0.700001419,  0.988245428,  -1.21816123,  0.568933129,  0.470297873,  -0.0202986784,  0.200196639,  -0.0487995893,  -0.0335145667,  -0.334694088,  -0.30821234,  1.86659563,  0.591626346,  1.24448824,  0.835408449,  0.398120522,  2.28895617,  -2.56665325,  0.391070664,  -0.133393154,  -0.694687366,  -2.05661035,  -1.41251874,  -1.39759922,  -0.239889681,  3.25393677,  -2.08558846,  0.971017301,  1.91384161,  -0.204197794,  0.695285439,  0.349660009,  -1.93311453,  -1.66540861,  0.956278503,  0.329025567,  1.31370795,  -0.812021375,  -1.58030152,  -1.47932649,  0.92001313,  -0.249242485,  0.58480835,  0.623541474,  1.24360621,  1.57830453,  1.66280997,  0.174738467,  1.38837051,  -0.0524428487,  1.27293277,  1.34751928,  -1.24506235,  -0.555824995,  -1.1511457,  0.979467094,  1.23699391,  -1.53088748,  -0.745224953,  1.34233022,  -1.16998506,  0.15134643,  0.727878034,  -0.227879778},{ -1.57676637,  -0.511040568,  0.23940222,  0.229617745,  -0.238810346,  1.12724316,  0.561444819,  1.09106004,  -0.699084818,  0.569781244,  0.634051025,  -0.781753957,  1.21712089,  -0.637569845,  -0.175660491,  -1.35810125,  2.14964581,  -0.220881134,  0.204910994,  0.495047361,  -2.0295372,  -1.03990614,  -2.27414441,  -1.26230121,  0.198163733,  2.28507423,  0.883837461,  -0.636996686,  -0.94703877,  0.852829039,  -0.623609483,  0.868581414,  1.3916235,  -2.04709387,  -1.26808608,  0.523306787,  0.333586961,  0.267487794,  -0.129329443,  -0.859036565,  -0.243102849,  1.18525565,  0.723519623,  1.07072306,  -0.280130982,  0.496676087,  0.565954745,  1.16537893,  -1.31241405,  0.188602611,  0.108197026,  0.710749447,  -0.897044361,  0.196177423,  -1.34767175,  0.630607307,  -0.168935478,  0.62422961,  -0.438005269,  0.257740945,  0.673648357,  -0.513506293,  -0.392971575,  1.75909197,  0.798276842,  -0.472918391,  0.0470485799,  -0.25140053,  0.144998148,  -0.601722181,  -1.62994468,  -1.0775342,  -0.238092333,  -0.175181448,  -1.14186096,  1.20551848,  -0.998543382,  -0.562364697},{ 0.654835641,  2.07343602,  0.976114035,  1.09576881,  -0.530575514,  0.0660227835,  -0.639954925,  1.60304892,  0.539523661,  0.694987357,  1.95395648,  -0.000434429676,  -0.417752802,  -1.67763638,  1.09834719,  0.226894453,  -0.815769374,  0.423627168,  1.11265504,  0.344281167,  0.225869238,  -1.35172546,  -0.50293529,  -1.29569745,  -0.0771215707,  -1.1360364,  -0.578437209,  -1.51680684,  -1.18037367,  -0.269363105,  -1.04047918,  0.894131184,  -0.799688935,  -0.786661208,  0.28659758,  0.305818915,  0.500210881,  0.152353942,  1.04779816,  -0.0949394628,  -1.3929013,  1.41533601,  2.06203294,  -0.00661424315,  -0.886258066,  0.546357751,  -0.00577874621,  -0.253026754,  -0.968321204,  -0.590315342,  3.38325143,  -1.46310806,  -2.39658356,  -1.1792661,  -1.10040915,  -0.833893955,  -0.698122144,  -0.573566258,  0.459067971,  -0.460862905,  -1.93895853,  0.997114539,  -1.48801517,  -0.10932532,  -0.702312946,  0.103160031,  0.00827371702,  -1.95480454,  2.09338212,  -1.1304003,  0.266870826,  -1.35847759,  0.596805871,  -0.745284438,  -0.303998917,  0.0544265248,  -1.65423632,  0.389331043},{ 0.668792427,  1.55417752,  -2.43757987,  -0.251363337,  1.02434576,  -1.09147942,  0.913730145,  0.884493053,  -2.15335035,  0.531220555,  0.47155714,  -0.0207414292,  -0.932933748,  0.316258043,  0.984119833,  0.0706339851,  0.268886626,  -0.478027403,  -1.14332759,  -0.0221851747,  1.40954912,  -0.891530097,  -0.636289835,  0.519707441,  -0.221323818,  -0.716131568,  0.404102415,  0.858841181,  0.554470241,  1.0959059,  -0.0104503203,  -0.244275063,  0.574839175,  -1.80856049,  0.376690358,  1.76453412,  -0.0151320901,  -1.4713949,  -1.51080585,  0.0699438155,  -0.863280118,  2.31978083,  0.246469647,  0.285019368,  -1.03544533,  -1.65027702,  -0.292804807,  -0.00986678619,  0.425511152,  -1.25287306,  1.36700201,  -2.41788769,  0.699128032,  -0.528612673,  0.535774767,  1.11224186,  -1.10426545,  1.83963776,  -0.605895877,  1.44396758,  -0.31914115,  1.10612524,  -0.594911098,  1.96312892,  2.71290755,  2.4052372,  -0.346563637,  -0.995543897,  -0.0703176931,  0.507433355,  -1.09029806,  0.938396752,  -1.39975989,  -0.676060498,  1.62962949,  1.24935508,  -1.09617484,  -0.365365386},{ 1.08684623,  -0.506270707,  -2.37863183,  0.0185406785,  0.218454301,  -0.0983218402,  -0.512100458,  -1.44027209,  1.23751688,  -0.362469673,  -1.62907553,  -0.184822381,  1.14046514,  -0.126155183,  0.906122267,  -0.489532411,  0.082731612,  -1.13881111,  0.561829925,  -0.772869587,  0.597461045,  -0.600502014,  0.245809913,  -0.463288754,  -0.642285168,  -0.259757191,  0.170349553,  -0.417222649,  -0.256162226,  -1.73585904,  0.810133278,  -0.42669037,  0.0300302934,  2.79821682,  0.987208724,  0.98740232,  -0.338688254,  -0.584171474,  -0.621163785,  -1.18530607,  0.445412338,  0.360425681,  0.704349399,  -0.570185363,  1.0248841,  1.88420165,  0.377851665,  0.526239097,  0.700885773,  -0.833818078,  0.651970565,  0.86875838,  0.963408649,  -0.830238938,  -1.36094451,  -1.3038255,  0.446711063,  -1.96100724,  -0.0526413433,  -1.00213885,  -0.992943764,  -0.348304033,  0.831531882,  0.521118999,  1.65531218,  0.0238144435,  -0.999002814,  -1.46485233,  0.695101976,  2.07163024,  0.0421709269,  0.679016471,  0.412856281,  -0.759162962,  2.04145193,  0.639495432,  -0.636220396,  -0.875640392},{ -0.194557533,  0.581079423,  0.845853686,  -2.03954887,  -0.674997568,  0.713438869,  1.16392362,  0.671395063,  -0.688991904,  -0.319886059,  0.04124856,  0.393641889,  -1.14806759,  -0.840468109,  2.71613479,  1.09802306,  2.58911586,  0.977380812,  -0.498217434,  -0.56616807,  -0.970984221,  0.126639932,  0.707210422,  -2.02583122,  -0.807858706,  0.523876429,  -1.36935925,  -0.249974802,  -0.0772441179,  -1.05333507,  2.36846185,  -0.197288558,  0.604632616,  0.70374918,  0.54667753,  -0.380521715,  0.324489057,  -0.690266728,  0.429948062,  1.35898995,  0.433835477,  -1.33636034,  -0.50661695,  0.732405901,  0.329908878,  -0.221641138,  0.251994699,  -0.36732614,  0.353563756,  1.37106669,  0.765450895,  -1.63886845,  1.38841546,  0.434919566,  -0.089141205,  -0.926556766,  0.541247666,  2.04841256,  0.937710583,  -0.245258868,  1.02088499,  -1.17146349,  2.67291665,  0.333450526,  -0.198872849,  0.728449881,  -1.52869558,  0.82910949,  -1.26680553,  0.0912491605,  1.60216153,  -0.782169878,  -0.431506276,  1.71084666,  -0.632729411,  0.143241078,  -0.257113665,  1.46304309},{ 0.249630168,  1.08572423,  0.263868511,  1.9939115,  1.72637773,  0.853831708,  0.464566559,  0.35490033,  -0.881111741,  -0.848357737,  -0.526381135,  -2.75442386,  0.273057401,  -1.36590564,  -0.214092478,  -0.754306018,  -0.0916255563,  0.129657134,  1.65746439,  -1.53765643,  -2.97989249,  -0.198021516,  -1.28887987,  0.496387035,  -1.82321632,  -0.066051811,  1.20782733,  -1.30895352,  0.144064829,  0.309378475,  0.644721031,  -0.00101992942,  -0.385105401,  0.489537805,  0.658086538,  0.165410981,  -0.73350054,  0.471732825,  1.42888963,  -0.582469523,  -0.911405861,  0.0586062036,  -0.772533894,  0.149634227,  -0.184785426,  0.343162447,  -0.537693799,  -0.732431114,  0.122833788,  -1.89991713,  0.128059775,  -0.633784056,  2.50673461,  -0.165367246,  0.681994736,  -0.257121086,  1.82043386,  1.09234869,  -1.59471762,  -0.957373679,  2.40841818,  -0.252168,  -2.18979359,  -1.52677119,  1.71944153,  1.5984261,  0.230719045,  -0.145765096,  -0.511824369,  -0.195707411,  -0.45143941,  -0.0936188102,  -1.20745504,  0.65830338,  -1.13477373,  -0.835511386,  -0.308787853,  -1.66368198},{ 0.177565783,  1.43191493,  -0.370754123,  -1.08542013,  0.239893094,  -0.312372059,  -0.122502461,  0.290130019,  0.777069509,  -0.150106013,  0.249532133,  0.22122547,  -0.330560803,  -3.58133507,  -0.423342943,  0.0085670175,  0.0506944805,  0.440060109,  0.419328988,  0.305503696,  -0.974973619,  0.415738404,  0.135411933,  -0.487340122,  1.09473717,  -0.379502624,  2.28304911,  -0.352566093,  0.766272247,  -1.45536745,  1.12735331,  1.04895031,  0.821920872,  -1.60882759,  1.22930884,  1.16903687,  -0.415136367,  1.02807081,  -0.204466939,  0.346497625,  0.0504754111,  -0.201701164,  -0.621213973,  -2.67483139,  -1.26648772,  -0.448781669,  -1.46550989,  0.36256507,  -1.01790118,  -1.08543372,  1.06340921,  -0.914121807,  0.279745936,  1.24768901,  0.977624178,  0.349423826,  0.936197698,  0.437433004,  -1.01100147,  -3.51801181,  1.20039082,  0.0458774269,  -0.898586869,  2.15497804,  -1.01471376,  -0.562010109,  -0.139866889,  0.200620145,  -0.640529513,  -0.0364941023,  -0.204670385,  0.967245281,  -0.142592892,  -0.565531433,  1.34656835,  0.0630026534,  0.310972005,  0.475880384},{ 1.16212332,  1.58907008,  1.66611552,  -0.838397086,  0.324455559,  -0.0481884107,  -1.73403764,  0.189047098,  -0.467125982,  -0.796054661,  -0.610909641,  0.991812706,  -0.574610054,  1.17483151,  -0.591289997,  0.228506625,  0.765110254,  0.705299377,  0.943069696,  0.362547219,  2.09897351,  0.459832311,  1.92140496,  -0.112316996,  0.0918068513,  -2.57026625,  -0.021410659,  -1.42520821,  -0.0949915498,  0.747178495,  0.30923301,  0.232648596,  0.341362178,  -2.75611567,  -2.35822415,  -0.172989696,  1.35743654,  -0.763497531,  -1.17762673,  0.47089538,  -0.786767125,  -0.875887513,  1.34058404,  -0.717772186,  0.380688965,  1.52588642,  -1.95162451,  0.0436375216,  -1.65550697,  1.16953135,  -0.461825669,  -1.75759757,  0.667029262,  -0.309438258,  0.686770976,  -0.656646907,  1.29490566,  0.709688246,  -0.847572744,  0.618795812,  -1.06326616,  1.14138627,  0.0349052586,  2.77708197,  0.655493796,  0.356658578,  -1.51062977,  0.261877507,  0.326818258,  -1.97073507,  -0.600272834,  2.76897955,  2.18964028,  0.0585101433,  0.507175863,  0.596090257,  0.172038287,  0.842042446},{ -0.713306546,  0.297725201,  0.0863648728,  -1.54552686,  0.805585682,  -0.0355625339,  -2.07775068,  0.576247215,  -0.118247725,  -0.87663579,  0.902548671,  -0.0582150295,  0.152890131,  -0.988018394,  1.01524198,  0.970637023,  1.0770961,  0.646679938,  0.879916251,  -0.704090357,  0.262989312,  -0.344000161,  -1.24412525,  -0.181611702,  0.25113216,  -0.00367236952,  -0.604761422,  -0.27388525,  -0.185024098,  -0.274695069,  0.234203324,  1.47608507,  0.339837968,  1.51653552,  -0.213198796,  -0.378175408,  -0.556416929,  -1.24538684,  -0.457108438,  0.0807863921,  0.871987343,  -1.03052628,  0.246626541,  -0.0960871875,  -0.689990163,  -0.841994047,  0.787311256,  1.12868989,  1.56517386,  -0.520161331,  0.467219263,  -0.204218104,  -0.855289757,  2.57900286,  -0.202739209,  1.94217145,  0.516806543,  -1.04026616,  -0.51781553,  -1.30076289,  0.290557027,  0.950182319,  0.107854605,  1.00129366,  0.122845918,  0.443232447,  -0.270023525,  1.31982565,  -1.17891586,  -0.7200073,  0.926939547,  -1.95474267,  -1.04076195,  -0.308314174,  -0.808075786,  -0.876742303,  -0.370871127,  1.6915319},{ 0.588709772,  1.05323386,  1.51476038,  1.37944686,  -0.511622608,  -0.0723648295,  -0.280969828,  0.44105351,  -0.67625761,  0.267751276,  -1.94410145,  -0.811520517,  1.40647709,  0.352657437,  -0.362781763,  -0.864637911,  0.606364489,  0.151540473,  -0.89122653,  1.01621997,  -0.00658281986,  0.836831987,  0.0621936806,  -0.450602531,  -0.642466784,  -0.118478358,  -0.614426494,  -0.430076987,  -0.418107212,  -0.504673362,  0.050548017,  -0.00952529814,  -0.55758363,  -0.343499482,  0.939045131,  0.228010118,  0.775178611,  -1.68337655,  2.03072691,  0.626098812,  1.25790596,  1.42544389,  -0.48469156,  1.01425707,  -0.641176045,  0.668009341,  0.742212951,  -0.0256615821,  -0.420977592,  0.501814485,  1.48419309,  -0.924016416,  -0.202161655,  -0.348517954,  -0.329648465,  -0.59877193,  -1.15615547,  1.32792675,  -1.2320261,  -0.312176615,  -2.40275908,  -0.748806834,  -0.108564153,  -1.81087816,  1.00804412,  0.790377736,  1.66107917,  0.583102167,  1.31253624,  -1.17925608,  -1.51213717,  0.0528154969,  0.015154996,  -0.151696742,  -0.155139834,  0.445402652,  0.888733804,  1.57194769},{ -2.52913833,  -0.0933112577,  0.191966757,  1.29420471,  -0.912885725,  0.320791543,  0.360533506,  0.43059665,  0.646087646,  1.23220086,  0.945358157,  0.238379925,  -1.05531263,  0.260760903,  -0.398640186,  -1.11828351,  1.33979189,  -0.391263843,  -1.68056011,  0.763467133,  0.708656609,  0.628917098,  -0.657715559,  0.403468341,  -1.32193434,  -0.75467813,  -1.73912632,  -0.652280629,  1.27799678,  0.851376832,  0.284372598,  -1.91121113,  0.414724588,  0.454458892,  2.27495909,  -0.661396503,  1.31421864,  -0.841159284,  0.071577765,  2.07093167,  -0.753075421,  -0.678978443,  -1.3568604,  2.04488373,  0.753961861,  -0.229653373,  -0.824034035,  0.650799155,  -1.5553304,  0.874141812,  0.292459041,  -1.02835834,  0.171467245,  0.18016842,  1.37401795,  0.974165678,  0.0817670748,  0.83425498,  -0.0118270703,  -2.17488456,  -0.959374726,  -0.121510535,  -0.85768497,  0.922441304,  -0.373236656,  0.383741885,  0.111414395,  1.90962434,  1.01911211,  -1.35048354,  0.168512478,  -0.282966495,  -0.251409799,  -1.98154533,  -2.43633127,  0.549450338,  1.69566631,  -0.964418232},{ 0.291855663,  -1.79163146,  1.70012856,  -1.07594419,  1.33894813,  0.739743888,  0.843282223,  0.015610625,  1.19568622,  -0.21218805,  1.79236197,  -2.66503501,  0.679959118,  0.419228107,  -0.388942122,  -0.55076617,  1.98087215,  0.0350581817,  -0.503126085,  -0.25723359,  -0.220077336,  -0.791095793,  -0.127335727,  0.0926951393,  -0.3487885,  -0.515390873,  0.136015654,  0.921860814,  0.243949354,  -0.641312301,  1.6926676,  -0.593653619,  0.0282731466,  0.405350506,  0.453618437,  0.0578004643,  0.436179221,  -0.100820243,  0.030599853,  -0.487386227,  -1.56686366,  0.0340570658,  -0.971787393,  -1.20875907,  -0.360110998,  0.208046749,  0.393134475,  0.854386926,  -0.321897626,  -1.42092466,  0.529138982,  -0.96585387,  -0.266398907,  1.18932283,  -0.3648431,  -0.259690017,  -0.215119869,  -0.353025913,  1.02462125,  0.658425629,  -0.656401396,  1.44545567,  0.445041955,  -0.0463921353,  0.184075043,  -0.539522648,  -0.469921589,  1.32168531,  0.886990368,  -1.61791086,  -1.08539259,  -2.18291116,  0.838915467,  -1.2229892,  0.122526884,  0.999500275,  1.19402874,  0.106520206},{ 0.808507681,  1.32152462,  -0.0446853302,  1.61631739,  0.132068604,  -0.375985503,  1.56698155,  -0.857674479,  0.561454713,  1.22194338,  0.508604765,  1.0208894,  -1.77944589,  0.170305952,  0.88380903,  2.32701945,  -0.821613908,  -0.426537395,  -1.2726959,  -1.01375544,  -1.3206321,  0.331438571,  2.142699,  -0.595550835,  0.246845499,  -0.433149248,  -1.09353924,  -1.08882511,  0.480311066,  -0.174736589,  0.00471009919,  -0.259683281,  1.04822683,  0.415423632,  -0.322163433,  -0.191985607,  -1.61271179,  1.08550608,  1.10685992,  -0.814597785,  1.12621403,  -1.12847161,  0.929967046,  0.351520747,  -0.281101167,  -0.00817776658,  0.0530900881,  -0.511389852,  -0.0206761118,  -1.32690024,  -0.433007181,  0.782383919,  -1.01059043,  0.0208115447,  -0.497633904,  -1.83473086,  -0.0277824104,  1.06693375,  -1.2251035,  -0.793484271,  0.957823098,  0.09195555,  0.481130928,  0.42041415,  2.09932065,  -0.144580737,  -0.176212907,  1.06358647,  -2.26511645,  1.29381025,  -1.65369427,  -0.518515468,  0.34479019,  0.0739267915,  -0.152255759,  1.43543851,  0.395081282,  1.50604558},{ -0.4450652,  -0.113093644,  -0.65233475,  -1.66062629,  1.49182093,  0.345677316,  -1.31938744,  -0.699952722,  -2.21710896,  1.71758962,  -0.98224318,  0.516763031,  -0.668961287,  -0.796603143,  1.03447318,  -2.11973929,  1.35693049,  -0.0905646086,  -0.0293590762,  0.406309366,  -1.08856463,  -0.438597262,  -1.80074692,  -1.41901815,  -1.65292656,  0.558759332,  -2.0458262,  -1.65375614,  -1.28333259,  -0.927719533,  -1.40004086,  0.443199784,  1.64276445,  -0.815020859,  -0.408623785,  2.04748106,  0.514267683,  -1.07096076,  1.52244532,  -1.31231642,  -0.579699457,  -0.12165945,  -2.92679048,  -1.41764688,  -1.55544436,  -0.884169698,  2.49703026,  -1.28299201,  -0.113450736,  -1.28273678,  -0.0713141859,  -2.00806069,  -1.19020557,  0.966483116,  1.8021313,  0.0774142295,  0.657235146,  1.24835348,  -0.200132117,  0.34545356,  0.954555869,  0.135269463,  -0.0553358123,  0.866287172,  -0.78552115,  -1.11891091,  -0.351718456,  -0.160151362,  0.140927821,  -0.251869857,  -1.56089973,  0.0474381559,  0.0618738532,  -3.04442048,  -1.79279637,  1.00738788,  -0.0236089993,  -0.934572101},{ -1.01017797,  1.03087008,  0.677275479,  0.700190067,  -1.23342741,  -0.712067842,  -0.891924679,  -0.259998381,  0.45682168,  1.47098625,  0.457808197,  0.869163871,  0.188382566,  -0.481115669,  0.0751721039,  -0.642317712,  0.478277743,  0.864299536,  2.60252213,  0.0700062662,  -0.427226812,  0.439987779,  -0.380511701,  1.06560099,  0.880225599,  -1.39396155,  0.234167844,  -0.232290253,  -1.89418614,  1.01910365,  -0.443980813,  -1.76556981,  -0.0160797089,  0.368391454,  0.86127758,  0.670287788,  0.253308058,  -1.44845176,  1.37946439,  1.09238279,  -0.397583157,  -0.145422697,  -0.426848441,  1.11212528,  -0.31427002,  0.240552559,  -1.28880036,  1.20743048,  1.71264362,  0.127431422,  0.744173646,  0.429357409,  -1.61192453,  -0.0649536327,  2.5436492,  0.80979836,  -1.32013857,  -0.175499007,  -0.403927535,  0.416479051,  -0.582277417,  -0.497308791,  -0.806565404,  -0.468869358,  0.827084541,  -0.316131085,  1.27319694,  -1.98203266,  0.896756649,  0.0956688821,  -0.0867098719,  0.433858573,  0.0115770837,  1.3438139,  -0.483750671,  -0.81079042,  0.36586827,  -1.39093733},{ -0.8910622,  1.00463808,  -0.894326687,  -0.754352868,  1.28686726,  0.960152626,  1.90493274,  -0.396691203,  -1.25021851,  -0.254438639,  -0.150587738,  -0.771743119,  1.73753619,  -1.34618747,  1.60996974,  0.478081971,  0.425934762,  -0.198582858,  -0.400794655,  0.696079135,  -1.1327163,  0.062005274,  1.55940497,  1.21748304,  0.339286506,  -1.11644351,  -1.40467107,  -0.216380209,  -1.32611537,  -0.270348012,  0.15478164,  0.624486923,  1.18394482,  1.93175066,  0.189294562,  -1.39825654,  0.431255519,  -0.3337937,  0.242830455,  1.10220039,  1.09452951,  1.68096066,  0.400127947,  -0.503875971,  -0.135657907,  -1.04844058,  -0.864001811,  1.01447427,  -1.13411427,  1.6672647,  -0.947684467,  -1.68012702,  1.33323634,  0.619986773,  0.358685225,  1.04888511,  0.260919631,  0.138350502,  -0.101437032,  -1.87149632,  -0.926927686,  1.19150519,  1.08900118,  -1.07754374,  -0.32180807,  -1.17404187,  -0.524828911,  -0.873416603,  -0.26706937,  -0.80748111,  0.827754378,  -0.49997288,  -0.218022034,  -0.112801209,  0.314301223,  -0.2162866,  0.462894797,  -0.273632407},{ -2.09734321,  0.597699523,  -1.53390384,  0.211183101,  -1.01275265,  -0.414119273,  0.27135998,  -0.692645669,  -0.561595559,  -2.19196677,  0.0739986151,  0.734135032,  -0.710390806,  1.39513183,  2.27727079,  -1.3932544,  -1.46741366,  -0.43417123,  -0.447047532,  0.748956978,  0.163373396,  -1.9160744,  0.177776441,  0.428875208,  -0.864703834,  -1.80018675,  0.189631209,  0.146456838,  -0.718220592,  -1.08655882,  0.557795525,  -0.588872015,  0.478542596,  -0.629448235,  -0.0965211838,  -0.438548326,  -1.42004263,  -0.213931635,  -3.01691413,  0.79450047,  0.327819049,  0.487847388,  -1.64853597,  0.483977407,  -0.6852507,  0.879883945,  0.361610353,  0.0818405822,  -2.01160407,  0.199921757,  1.2571876,  -0.458147883,  0.224764019,  -0.0919618458,  0.280041724,  1.3853991,  -0.170403376,  0.376297265,  -1.48433769,  -0.198045209,  0.366565555,  1.40738618,  -0.726306915,  0.973247349,  -0.26966545,  -1.31801438,  -0.392824024,  0.704900324,  1.16183043,  0.38263908,  -0.517595232,  -1.25594461,  0.817322791,  -0.83213985,  0.00147995225,  0.0403145887,  -0.268534005,  0.264637798},{ 1.59170318,  -1.05831552,  1.59287822,  0.902594209,  -0.0181594435,  0.267898738,  0.434071481,  -0.754221261,  0.236999407,  -0.166377947,  -1.0016886,  0.907578468,  0.749898195,  -1.66414225,  0.467526078,  -0.432893038,  0.404431075,  1.31436265,  -1.20416319,  1.29651284,  0.582556069,  0.886183739,  0.558436513,  0.0913150087,  -0.500965297,  0.842174709,  0.770155966,  0.947803617,  1.42741346,  0.358262092,  -0.25897187,  1.25564158,  -0.329096109,  -0.767126739,  0.592662752,  -0.6787467,  -0.259398341,  -1.18286824,  1.01557958,  -0.227959037,  -0.342099428,  -0.542515218,  0.454965621,  -0.313458502,  0.698529959,  -0.491044968,  1.17388141,  -1.01879418,  0.855136216,  -0.326415002,  0.294130206,  0.343377769,  0.829544127,  -0.411588818,  0.227220297,  1.86180246,  -0.97375226,  0.28893581,  1.475402,  0.142474487,  0.885017514,  0.856689572,  1.33725393,  -0.35973686,  -0.53043896,  1.57428133,  0.555647552,  0.909577787,  0.683977604,  0.735006094,  1.26107371,  1.49403381,  1.57222903,  -0.485720128,  -0.954774499,  -0.684467793,  0.681517005,  1.8984195},{ -2.06650996,  0.197414309,  -0.0583260804,  -0.631464005,  -0.114924245,  0.0133997248,  -0.359692335,  0.459453106,  -0.531150699,  -0.5803684,  1.20156515,  -0.401536822,  -0.541332185,  0.273606747,  -0.439261317,  0.852068424,  0.177095771,  -0.284399718,  -0.21875675,  -1.64425075,  -2.21518064,  -0.832945645,  0.749035835,  0.0829415768,  1.13376725,  1.09580624,  0.388504148,  0.541476786,  -0.118475042,  -0.397556752,  0.379574955,  -2.23405981,  -0.394699812,  -1.08131623,  -1.19237483,  -1.43226933,  -1.02898252,  1.00376356,  -0.133138686,  -0.909184635,  0.755518436,  0.263869077,  -0.465774029,  0.857052147,  2.20005465,  -1.1424849,  0.60775739,  1.43528795,  0.115700111,  0.998920023,  -0.0151591236,  -0.718909979,  -1.14747834,  -0.181144655,  0.930006087,  -0.576242745,  -0.53162986,  0.297225565,  0.903015316,  -0.652927577,  -0.306178063,  1.03861141,  0.654889941,  -0.772051394,  0.449685335,  -1.64273381,  1.22974908,  -0.514230609,  0.566342294,  -2.50340605,  -2.83549333,  0.715812087,  -0.191687092,  -1.35878611,  0.246489391,  0.598598778,  0.211638689,  -0.959147692},{ -0.374736398,  -0.317373335,  2.31809163,  -1.63918686,  -1.25649798,  2.42299533,  -0.0382829569,  -2.38330865,  0.677399635,  -0.415322989,  -0.633883059,  -0.376129061,  1.90937006,  0.3106682,  -0.747754693,  0.746465445,  -0.0371348411,  0.210818723,  0.625941753,  -0.189820305,  0.164493531,  -0.195653051,  -1.44034016,  0.345158935,  1.33664536,  2.09830952,  0.968765616,  -2.13411832,  -0.0427977815,  1.36638892,  1.01181781,  0.31134674,  -0.560862184,  0.950874805,  2.35213947,  0.464553207,  -0.117364138,  -1.56649983,  -1.13391244,  0.920241177,  0.7210024,  0.537174582,  0.264712989,  0.630102098,  0.767591774,  -2.27662301,  -0.989240885,  1.46730828,  -1.33712196,  0.767186344,  -0.367664397,  1.00929856,  -0.58169359,  1.50109828,  -0.755440652,  0.872041047,  -1.39186454,  2.6083827,  -0.913198292,  -1.31821585,  0.252083272,  1.34127176,  -0.494309306,  1.59156013,  1.09380758,  -0.116962194,  0.443896621,  -0.804523647,  0.403062642,  0.713315606,  0.529276192,  -0.167083025,  -0.455331981,  1.39437151,  -0.596677363,  -3.22774758E-05,  1.05984712,  0.128113776},{ 1.40187538,  -1.49045467,  -0.0526377112,  -0.53707993,  0.649561286,  0.136879772,  0.262565583,  -0.996783018,  0.785640538,  -0.00973932259,  -0.270004928,  -0.144601554,  -0.437869072,  -1.20338953,  1.09102225,  -0.94870168,  -0.794451714,  -1.20462346,  0.21800673,  -0.303490847,  0.288386345,  0.761860907,  -0.0992640331,  0.361243665,  0.843957841,  0.0164125133,  -0.11457333,  0.0652871653,  -0.528342724,  -0.421064883,  -0.444007516,  -0.245490611,  -0.0704702884,  -0.279242605,  0.410449117,  0.460595131,  1.54744291,  -1.73288393,  0.89517802,  1.75696254,  0.470339358,  1.69052446,  -0.659295857,  -0.0792860538,  -0.0498309806,  -0.00797032192,  0.520271957,  -1.99603403,  0.955719173,  1.32122719,  0.765840054,  0.717632294,  -1.35329831,  -0.154401869,  0.303362727,  0.310245216,  0.00826288387,  -0.0549854524,  -1.81385088,  -0.773482919,  1.56377912,  -0.392511636,  1.16278315,  -0.991341412,  -1.11881256,  -0.582622409,  -0.120270766,  -1.756266,  -1.03276515,  2.56113505,  -1.27529836,  0.175009593,  -0.876518846,  -0.335647106,  0.485044837,  0.414944738,  -1.03818941,  0.0855407566},{ 0.269874483,  0.491047144,  0.197737709,  -1.01822567,  -0.529431939,  -0.915659726,  1.12531579,  -2.08734059,  0.451128155,  -0.793604434,  1.76092386,  1.05302095,  -0.743233621,  0.413268417,  1.54592061,  0.307707876,  -0.771581769,  -0.749600887,  -0.801425695,  -0.622385919,  -0.244549409,  -0.902638435,  2.1288178,  0.232437551,  -0.566948533,  1.83612764,  0.825452209,  0.983165622,  0.717809618,  -0.360061646,  -1.67917681,  1.28339159,  0.513558686,  -1.03497148,  -0.0383906141,  -2.82051706,  -0.615725756,  1.09909296,  -0.106577486,  -0.96277827,  1.39127851,  1.07815433,  0.530740619,  1.36022198,  -0.145084143,  1.31318438,  1.40477169,  0.233553931,  -0.705907762,  1.01173246,  1.11031425,  -0.114712641,  -1.44092667,  -0.669894814,  -0.576517642,  0.754050851,  1.18790722,  -1.41066015,  0.214607328,  -0.291682839,  0.448524207,  -0.153291121,  0.288737625,  -0.203393996,  0.518775463,  -0.177532464,  0.843701184,  -0.450278103,  -0.387750745,  0.818881869,  0.224994436,  0.859676003,  0.18393223,  0.100945137,  -1.66769958,  1.88865221,  -0.147953093,  0.963701665},{ 0.42834869,  0.304488152,  2.02232528,  1.58680415,  -1.01681364,  -0.659344137,  -0.956555963,  -0.705530286,  0.163200021,  -0.353100806,  0.379585743,  0.662608445,  0.829509139,  2.39772701,  1.413432,  -2.10680318,  -0.632862628,  2.268435,  -0.267931044,  0.649981856,  -0.68442136,  -0.520976305,  -0.031717997,  -0.307002634,  -0.0184437819,  -0.268444479,  1.16294658,  0.825234652,  0.922599196,  1.26968694,  0.182982296,  0.832069874,  1.60198236,  -2.06173682,  0.155087411,  -1.01797044,  -0.960340023,  1.47125351,  1.14081168,  0.913420916,  -1.18769062,  -1.74138963,  -0.377649307,  -0.40491572,  -0.156791419,  0.752430856,  -0.199577585,  2.06458807,  -1.75325799,  0.445348322,  -0.750129461,  -0.0945485085,  0.213896915,  0.828976274,  0.620002508,  1.07533348,  -1.47824335,  -0.76294142,  -0.579221904,  1.95083749,  -0.347682804,  -0.911250234,  -0.0456906334,  -0.306378871,  -0.412049174,  -0.0089088995,  0.123312935,  1.31013453,  -0.520830333,  0.726086438,  1.09956419,  -1.64185274,  -1.278561,  0.925611615,  0.954709888,  0.547498047,  -1.25128663,  0.341158599},{ -1.03427565,  -0.178031772,  1.23112798,  0.142274559,  -0.694655418,  -2.40031838,  0.891744077,  0.367718905,  -0.486655325,  1.19137466,  -0.459208012,  -0.0478930362,  -0.00246660621,  -0.466217399,  1.1142379,  -2.07774591,  0.310338169,  -1.76241577,  -1.67879152,  0.170715943,  0.193340078,  -0.0317246094,  0.63613683,  0.687976658,  -0.307815999,  0.950298488,  1.44758272,  -1.88079989,  0.568424702,  0.433802694,  -0.670590758,  0.0863980353,  -0.962138712,  0.0891482979,  -2.32053947,  -0.864821196,  1.09591067,  -0.645679116,  0.561197996,  1.92624378,  1.60910511,  -0.324035525,  -1.46890533,  0.487790257,  0.0542732403,  -0.213821188,  -0.476023495,  -1.254475,  -0.088206239,  -0.388293058,  -0.0275541935,  0.996124625,  -0.20156388,  0.502090633,  1.27262592,  -0.0559207126,  0.997104228,  -0.945665359,  -0.884533226,  -0.649855971,  2.19454408,  1.74610305,  0.959347963,  -0.979178667,  -0.357263952,  0.148165822,  -0.380075574,  -1.81483114,  -1.32671928,  0.651568294,  1.40117407,  -0.923994899,  -0.653052926,  -0.734667718,  0.411081612,  -1.91541719,  -0.74907434,  -0.438712835},{ 0.931714594,  -0.0110529521,  0.256869853,  1.13572407,  -1.23662925,  -1.28618526,  1.09824336,  -0.425236315,  1.61696088,  -0.56921947,  -0.745051801,  -1.31211579,  0.00478657382,  2.03475475,  -0.688831091,  0.5548141,  0.949953973,  0.19977814,  1.51306224,  0.776019812,  1.04194832,  -2.59514141,  0.714738011,  -1.4910692,  0.741364598,  1.29216194,  0.43232131,  -0.796378851,  -1.62752545,  0.500774145,  -1.24353683,  1.45582318,  0.285947353,  0.688717604,  2.84127855,  1.5095681,  0.80288142,  -1.33733571,  -0.195225552,  0.577212393,  1.05970967,  0.860346913,  -0.138366804,  0.485022396,  -1.14157462,  -0.508009732,  0.0241309442,  0.752158999,  -0.13781102,  -0.929475605,  0.000786086253,  -3.07504225,  0.503343403,  -0.920523942,  0.348421872,  0.551601171,  0.412482947,  -0.218025506,  -0.0182539225,  0.682818532,  0.512718499,  0.0953228623,  -0.181743234,  0.854031801,  0.69030869,  2.78291225,  0.43526426,  2.3949995,  0.184156567,  1.62244213,  -0.645103097,  1.75056314,  0.40531674,  1.11838007,  2.04697609,  -0.00944111217,  -1.64527607,  -0.907369852},{ -2.15006828,  -1.2402482,  -0.00963430852,  0.918158948,  -0.464402616,  0.263346642,  1.09049869,  0.582850933,  0.86181587,  -2.4261663,  -0.106621921,  0.934289336,  -1.25181973,  0.773212671,  0.716392159,  0.466957301,  0.0743049607,  -0.346964598,  -0.509956717,  -1.09398043,  0.0445085093,  -1.40650129,  -1.01101983,  0.952178299,  -0.777633667,  0.0453825109,  0.222665012,  1.19692206,  -1.61573505,  0.554109216,  0.271376044,  0.251034468,  -2.01293516,  0.812996328,  0.776946008,  -2.09283924,  0.282067269,  -0.967433155,  1.0662514,  0.257250965,  2.38218403,  -0.260350376,  1.07320786,  0.817641795,  0.307110131,  1.075055,  0.265196264,  -2.17209172,  -0.590374053,  0.183052853,  -0.0152088432,  0.134464338,  1.12240303,  -0.994508564,  0.740341306,  0.105483882,  -0.437152773,  0.142579719,  0.898022592,  -0.719106078,  0.645960212,  -0.338344038,  -0.512169302,  0.867565393,  0.392485738,  -1.58256352,  0.962171733,  0.260462582,  1.34368074,  -1.05945957,  0.546671569,  -0.977416873,  0.918211937,  0.184318691,  -0.10335122,  1.41013539,  2.25164008,  -0.406621307},{ -0.103211828,  0.285570651,  -1.21139657,  0.659090102,  -0.308022618,  1.23430622,  -1.50483155,  0.0927658528,  1.07096875,  -0.037417829,  0.0788675398,  0.535800219,  -1.71499741,  0.596226811,  -1.49619412,  -0.372290075,  0.783589602,  2.20427561,  -0.773030281,  -0.916521907,  -1.14155877,  -0.946250439,  0.155978829,  1.43248606,  0.898162961,  -0.134641394,  -0.134268388,  0.352977961,  0.566372037,  0.813608408,  1.12830853,  0.665313542,  -0.0368258618,  -1.95799029,  -0.194992989,  -1.6507802,  0.0232982654,  0.582521975,  -0.890361786,  -0.814969659,  0.00430587586,  2.4974761,  1.6475141,  -1.30346489,  0.21073018,  0.762748897,  1.14208114,  0.338489443,  1.2422837,  -0.415499061,  -1.12017465,  -0.176116899,  -0.123440653,  -0.687761903,  -1.23968482,  -0.0565033294,  1.05581474,  0.605292141,  -1.00862622,  1.5223316,  0.898579478,  0.936554909,  -1.22872806,  0.530081332,  0.422443211,  -0.757384896,  1.29210722,  -0.772007108,  -1.23170269,  0.295945227,  0.966903985,  -0.570069492,  0.664811373,  1.02156901,  0.69689399,  0.777795196,  1.02957428,  1.1259563},{ -0.842215061,  1.26169658,  0.242037103,  -1.02479196,  -0.0301170945,  0.572951019,  1.61291003,  -0.132069051,  0.700980484,  1.02241027,  -1.72123659,  -0.292096138,  0.720755041,  0.894650102,  2.63291454,  -0.240070194,  -0.455327153,  0.76572758,  -0.220445678,  -0.54537648,  -0.399778903,  1.16708267,  -0.116026349,  -0.97429812,  1.32973742,  -1.97142577,  0.153939143,  -0.6500507,  -1.3958354,  0.528927326,  -0.902490199,  -1.00121665,  1.49924505,  0.375867009,  0.455828339,  1.25149059,  -0.218076572,  0.638702035,  0.860474288,  0.928574324,  2.11575866,  -0.768128037,  -1.49778593,  0.059698388,  -0.576410532,  -0.759852409,  -2.02059221,  0.790794671,  1.61597228,  -1.58798337,  0.125977695,  1.10492325,  -0.461795479,  -0.566994429,  -1.55176437,  0.884646535,  -0.08200977,  -0.22181268,  0.408666342,  -2.15006614,  0.521996617,  0.430213273,  0.0667181015,  -0.786164284,  -0.627562046,  -1.06587172,  0.626162052,  1.22411644,  1.12733769,  0.339714527,  -1.2784375,  -2.37330866,  -0.00434578303,  -0.700611949,  -1.17946458,  0.726234376,  -0.253076404,  2.08661175},{ -0.219092309,  0.599671602,  -0.696557939,  0.121588729,  -0.864750624,  -1.48303998,  2.17099047,  -0.111756362,  -0.986883998,  0.0184263587,  -0.609499335,  -0.86730504,  -1.63581407,  0.238702118,  0.0569320247,  1.95633376,  -1.33301103,  1.36715651,  0.641623914,  1.07145905,  -1.84448409,  -1.27584851,  -1.57326424,  -1.45530045,  -0.207555965,  -1.37126529,  -0.751337349,  0.112418719,  -0.427960932,  1.24048138,  -1.00519562,  -0.966224015,  0.810480416,  -0.420867205,  -1.23807728,  0.38175413,  0.785907447,  -0.127669856,  -0.190667182,  -1.52314579,  0.387445688,  1.30132949,  -0.561725914,  -0.0856339559,  2.66617489,  -0.621272266,  -0.96592766,  -0.716792881,  -0.637461603,  1.10372627,  0.750409186,  0.135323778,  0.00990078039,  0.490844578,  -0.501040459,  0.244927421,  0.390161842,  -0.876469374,  -0.863027751,  1.27170849,  0.662162483,  0.0344201811,  0.925837874,  -0.851230621,  0.391547501,  -0.453220248,  -0.151122332,  1.37995446,  -1.17150247,  -1.30049622,  -1.20425332,  -0.775039196,  0.451959103,  1.5634619,  -0.739629686,  1.11283863,  -2.02391839,  -0.108679637},{ 0.872706175,  -1.10044193,  1.01597786,  -0.57525897,  1.32884324,  -0.0924972668,  0.518877566,  0.0676696748,  -0.300898492,  -0.0361836478,  -0.12914744,  -0.622134149,  0.344671756,  0.140799552,  -0.666320682,  1.18050742,  0.0332207084,  1.17728865,  -0.314088941,  -0.24872449,  -0.776319027,  0.834913492,  -1.66438496,  0.165306017,  -1.3109622,  0.894339859,  -0.658506095,  0.177987069,  -0.228582487,  1.03353298,  -2.0451026,  0.00747672748,  -1.09644246,  0.582518399,  -0.0380050577,  0.398597449,  -0.801237702,  -0.409988582,  -0.262194008,  0.501968622,  -0.851506174,  0.881310046,  2.54340482,  -0.204915226,  -0.833323061,  -0.357723802,  2.22800684,  0.557862461,  -0.92253691,  2.32171011,  -0.00501747802,  0.350101143,  -0.342042029,  0.992993653,  -0.778654277,  -0.0103519121,  0.823693812,  -1.03077042,  -0.448207468,  1.06269574,  -0.0135097569,  0.285200119,  2.12907362,  -0.619674921,  1.21903133,  0.638403118,  0.402527004,  0.731676817,  -0.607053101,  2.41291833,  -0.430173099,  0.879046082,  0.23188971,  -0.359711587,  -1.3484323,  0.145970836,  0.918884277,  1.18980646},{ -1.32503569,  -1.92727458,  2.02454209,  -0.378960341,  1.24201059,  -0.253906786,  0.0224429537,  0.340105742,  0.386521488,  1.74503386,  -0.360655516,  -0.675308704,  0.843363047,  0.432231456,  0.462242633,  2.60452604,  -0.348202765,  1.33487344,  -0.944129348,  -0.496634364,  0.372306973,  -0.259726852,  -0.306713283,  0.60812521,  -0.417590648,  0.81647855,  -1.0772264,  -0.802911639,  -0.74031961,  1.20491004,  0.845754504,  0.751108885,  -0.645009458,  -0.33680889,  -0.4168365,  1.57798707,  -0.553125083,  -0.132045716,  1.1767,  -0.875116289,  -0.587529898,  1.14705014,  -1.37241864,  -0.231068641,  1.04148436,  0.102011815,  -0.263223201,  -0.831468165,  -0.0810266882,  -0.0805546939,  0.0176809933,  -0.329672754,  -0.136577815,  0.299439967,  -1.74992025,  -0.134677023,  -1.49340332,  -0.147009745,  0.0347184092,  0.739691317,  -0.535647273,  -0.175629586,  -0.95829761,  1.09658551,  -1.61654103,  1.47344375,  1.33083594,  -1.72204792,  -0.153748378,  0.087554805,  0.686825395,  -0.10770373,  1.35976863,  1.29964352,  -0.752507806,  0.122810483,  -0.753986597,  1.95040643},{ -0.575973153,  -2.33511066,  -2.41421556,  0.558526635,  -2.74422479,  2.14319015,  -0.640510619,  -0.953096092,  0.0520329513,  -0.670329213,  -1.40480506,  -0.79422456,  0.0455858484,  -1.70793521,  0.859754443,  -0.255381942,  -1.29014897,  0.657774031,  0.989196897,  -0.526404381,  -0.802666783,  -0.642120183,  -0.45258534,  -3.05531597,  0.999198854,  0.468576163,  -1.7186532,  0.249620721,  -0.683113217,  -1.09152496,  -0.733650625,  -0.291099787,  -1.06204367,  2.3097446,  -0.452835113,  -0.192916319,  -1.50333011,  -1.50071108,  1.54595351,  0.289946556,  0.241465002,  1.62676764,  -1.84836864,  -1.28509605,  1.38822734,  0.145809844,  -1.10165071,  0.465012491,  -1.07774103,  -0.0418318212,  0.0227100234,  -0.223663956,  1.57193649,  0.214066297,  1.0269388,  2.4823482,  -0.672973692,  0.651714742,  0.866189539,  1.83167267,  1.81071246,  0.349583834,  -0.385469735,  1.08872581,  -0.534406662,  0.916558862,  -1.01999211,  -0.285696387,  1.19553936,  -0.671478033,  -1.2000289,  -0.372176439,  -0.751642108,  0.483113229,  1.56057596,  1.28886652,  -0.415997803,  0.0348385498},{ 0.00537811918,  -1.17037106,  -0.221963316,  0.0737300888,  0.665898621,  1.21944034,  -0.0099523114,  1.11480129,  0.383687705,  -1.04303551,  -1.63383818,  -0.0707230717,  -0.603416145,  0.149434894,  0.960435808,  -0.409830421,  -0.159621313,  0.556933224,  -0.471917659,  0.0426802896,  -0.406493932,  -1.14756238,  -1.1513375,  0.0385756455,  2.01890135,  -1.42214727,  -0.812799037,  -0.214572787,  -0.795283735,  0.243938401,  -0.152051762,  -1.71977317,  0.991737783,  0.973743141,  0.376579314,  1.71564734,  1.15486145,  0.166281492,  1.57085729,  -1.2252506,  -0.451838672,  -1.74286067,  -0.475910813,  -1.78110909,  0.850754142,  0.67840147,  0.324825585,  0.0183682609,  2.4973917,  1.3120209,  0.468442738,  -0.268836319,  1.10501027,  0.883647382,  -2.27495313,  0.64679873,  0.149416536,  0.765533745,  2.22715449,  -0.564208984,  0.423534483,  0.107046865,  -1.6822778,  1.65897965,  0.717691481,  1.96765542,  0.858066499,  1.08265901,  -1.22694373,  -0.505579889,  -0.441155434,  -0.771870732,  -1.99204898,  -1.11457503,  -1.15089023,  1.58436358,  0.351472855,  0.667138994} };
    }
    if(pdg.compare("piminus") == 0){	 
     
      return {{0.083232, -0.202927, -0.681112, -0.458239, -1.283392, -0.722760, -0.926030, 0.591387, -0.121371, -0.156712, 0.600272, -0.277082, -2.522305, 0.249658, -0.929785, 0.419728, -1.227760, -0.115393, 0.951869, 1.722068, 2.025256, 1.306820, -0.714791, 2.327456, 0.464784, 0.421549, 0.132262, 0.488832, 0.207246, 0.292936, -0.987169, -0.676416, -0.443983, 0.305685, -0.027425, -0.578191, 1.082757, 0.971419, 1.246165, -0.311415, -0.630093, 1.218414, 0.656285, 1.669269, 0.510760, -0.254937, 0.323633, -0.115464, 0.838653, -0.428518, -0.707735, 0.502917, -0.126893, 0.206069, 0.073150, -0.933027, 1.792524, -0.142755, 0.660562, 0.819838, -0.111530, 0.304162, -1.010832, 0.925499, 0.917253, 0.414495, -0.806521, 0.026703, -0.549195, -2.404996, -0.364031, 0.553399, 0.399646, 0.482028, -1.396676, 0.527166, -0.158212, -0.084561},{1.322838, -1.417246, 0.419192, 0.779808, -0.669362, 1.180503, -1.382267, 1.601406, 0.982284, -0.181260, -2.436139, 1.015097, 1.143302, 0.900160, -1.017206, -0.299648, -2.066511, 0.354848, -0.099721, -0.962870, -0.735129, 0.189101, -0.416898, -0.697327, 0.660725, 0.223324, -0.151489, -0.563822, 0.449815, 0.913822, 0.621692, -1.594786, -1.230709, 0.248389, -1.117641, -0.289593, 0.942588, 1.395868, 0.369336, 0.897640, -1.099780, -1.005494, -0.341839, -0.220963, 1.494107, -0.630792, -2.149534, 1.186833, 0.692887, -0.199878, -0.363356, 0.299849, -0.562692, 1.480883, 0.250387, 0.079097, 0.743127, 2.090343, -2.059072, -0.139620, -0.651694, -0.173147, -0.443710, -0.899107, 1.218865, -0.162318, 0.761036, 0.631709, 0.376121, -1.471601, -1.161258, -0.985225, -0.451108, 0.378978, -0.180255, -0.906312, -2.584536, 0.153158},{0.326008, 1.420673, -0.823234, 3.406882, -1.045303, -0.257886, -0.047115, -1.352031, -0.080032, -0.270888, 1.602675, -0.801780, 0.255145, 0.290806, 2.699321, -0.957882, 0.980177, 0.803550, -0.936948, 1.344895, -0.207325, -1.051448, 0.885394, 0.068980, -0.554363, 0.655502, 0.527401, 1.247480, -0.230996, -0.313010, 0.629167, -0.101125, -0.836280, -0.410359, 0.040678, -0.870611, -0.841152, -0.488597, -0.000170, 0.150870, 1.603016, -0.090185, 0.346855, -2.412726, 0.378837, 0.088994, -0.801101, 0.276394, -0.052677, 0.671338, 0.220588, 0.184111, 1.645371, 0.226843, -0.676653, -2.101396, 0.085536, 0.403079, 0.601824, -0.960297, -0.204845, 1.314213, -0.456238, -0.834941, 0.674789, -0.998374, -0.899977, -0.335727, -0.509271, 0.822223, -0.371017, -0.017752, 0.007078, 1.078886, 0.235206, 1.031222, 1.196820, -0.264573},{-0.455372, -0.585180, -0.229294, -0.445036, 1.526110, 0.488786, 0.481942, -1.022623, 0.704554, -0.778493, -1.360432, -0.949574, -0.716427, -0.985347, 0.207167, 1.016018, 0.285000, -1.680140, -0.901595, -0.766188, 0.194742, 0.665672, 2.198694, 0.404658, 0.061028, -1.383181, -0.608420, -1.481052, -0.072291, 0.165420, 0.833713, -0.035052, 1.509440, 0.340731, 0.827500, 0.115936, -1.120967, -1.315316, -0.770302, 1.026940, 0.426517, 0.138902, -0.306176, 0.227304, -1.712261, -0.023844, -0.407217, 1.515815, -0.153040, -0.882670, -0.355947, -0.944263, -0.744212, 0.785989, -0.117380, 0.407235, -0.720276, 0.371171, -0.623068, 1.206310, -1.338784, 1.443330, 1.340838, -0.213050, 0.796096, 0.328113, -0.747830, 0.256842, -0.828112, -1.221463, 0.676686, -1.522390, 0.245303, 0.438434, 0.514249, -1.786024, 1.073617, -1.038162},{1.775077, 0.651287, 0.974368, 0.606641, -0.466290, 0.114246, -0.332827, -1.021569, -0.043717, -0.101319, -0.116937, 1.249611, 0.104493, 0.844199, 1.471071, 0.388373, 0.179434, 0.264444, 0.488846, 0.644253, 2.112877, 1.289219, 0.930315, 0.225303, 1.178474, -1.255945, 2.132611, -0.562222, -0.745760, -1.039434, 1.132246, 1.288242, 0.268321, -0.380376, 0.421680, -0.299617, 0.023107, -0.231465, -0.448059, -1.195045, -0.797713, 0.167128, 1.175578, 2.706177, -1.150390, -0.818225, 1.525455, 0.838114, 1.061504, -0.968003, -0.749866, 0.145022, -0.346577, 0.821200, 0.330570, -0.771490, 0.337901, -0.319308, -0.173908, 0.208853, -0.070963, 0.484289, -2.726153, -0.292540, 0.725519, -0.756032, -0.492070, -0.761790, -0.838025, 0.152246, -0.374422, 2.388584, 0.576344, -0.807954, -0.106207, 0.736606, 1.687739, 0.471733},{-0.792894, 1.023351, -0.448698, -0.016707, -0.702252, -0.435832, -1.845813, 2.845200, -0.336373, 0.181526, -0.597551, 0.041656, -0.220787, -1.568677, -0.021439, -0.151570, -0.557249, 0.074485, 0.951204, 0.141677, 0.422018, 0.859301, -1.004342, 0.724411, 0.828818, 0.543748, -0.204527, 0.017737, -1.016162, 1.403525, -0.350875, -0.092979, -0.525400, 1.183770, 1.667934, -0.171099, -0.002741, -0.537486, -2.206536, -1.526594, 0.198126, 0.711390, -0.154998, 0.221855, 1.631782, -0.104201, 0.030754, 0.518989, 0.258313, -1.321771, 1.272448, 0.500110, 1.712255, -0.254940, 1.071617, -0.868081, 1.430470, -0.226816, -0.652972, 1.087093, -0.758425, 1.009695, -0.935824, -0.160108, -0.541669, -0.221633, -0.452392, -0.629967, -2.097083, -0.029227, -0.765084, 0.465546, -0.664473, 1.274884, 0.409896, 2.383475, -0.466509, 1.068749},{-1.646046, 1.317816, 0.412631, 1.113755, 0.246556, 1.337627, -0.463491, 0.605977, 0.430492, 0.663160, -0.264050, -0.274094, -1.704924, -2.022718, -0.017969, 0.757046, 1.392832, -1.743178, 0.766556, 1.286776, -0.054048, 0.622373, 0.771567, -0.178045, -1.032630, -0.090337, 1.395375, 0.698227, -0.140253, -1.156868, -0.132454, -0.432872, 0.808620, -0.598189, 0.937617, -1.399596, 1.503984, -0.183382, -1.707858, 1.867415, -0.133135, 0.698105, 1.102974, -0.687514, 0.875643, 0.021580, -0.107505, 0.803610, -0.877752, 0.494905, -1.034390, -0.156946, 0.921208, -0.231123, -0.419527, 0.414914, -0.721187, 1.423755, 0.318776, -1.001851, -0.425851, -0.439492, -1.006327, -0.057629, 0.756132, -0.177127, -0.376536, -1.313164, 1.616255, 3.000985, 0.227752, 0.332170, 0.311467, -0.793225, -0.924394, -0.553599, -1.087525, -0.130626},{-1.027069, 0.538524, 0.235025, 1.039207, 0.889366, -1.217739, 0.716855, -1.060813, -2.124826, -1.034543, 0.634814, 0.989517, 1.473858, -0.378247, -1.987297, 0.486220, -0.525897, 0.078202, -1.480713, -0.637300, -0.717408, -0.745224, -0.333478, -1.456184, 0.595413, 0.956483, 1.092470, -0.939548, 3.343417, 0.981971, 0.420650, -0.575903, -0.407460, 0.315218, 1.093204, 0.836579, 1.762569, -1.321318, -0.075229, 1.203564, -0.355203, 0.614249, -0.108896, -1.010171, -2.173265, 1.945832, 0.442838, -0.062215, 0.452940, -0.368409, -0.936607, 1.183823, -0.152340, -0.334888, 0.849432, 0.204066, -0.765493, 0.375241, -0.336937, 0.099662, 0.759222, 0.756874, 0.546185, 1.324826, 1.032442, 0.405801, -2.304355, -0.265798, 0.554561, 2.361348, 1.487905, 0.242986, -0.538387, 0.161511, -2.083176, -0.015921, -1.851610, -1.420821},{1.466899, 0.281957, -1.416587, 0.613920, -1.267973, -1.401933, 1.628380, 1.352900, 0.592530, -0.833809, -1.725670, 0.383111, -0.916717, 1.242949, -0.544691, 2.261414, 0.743821, 1.006866, 0.856124, 2.335398, -0.353459, -0.373061, -1.078719, 0.006126, 1.354105, 0.580689, 1.957207, -0.460069, 0.713970, -0.567433, -0.572790, -0.099242, -0.432254, 2.270069, -2.498437, 0.225028, 1.790850, -1.899954, 2.321056, 0.164020, -0.827832, -0.486258, -0.267190, 0.032056, -0.444725, -0.564111, 0.151993, -0.821917, 0.426446, -0.247765, -0.754998, 0.377518, 1.130764, 0.662034, -1.422636, -0.109412, -0.239099, -0.619270, -1.247027, -0.482140, -0.484262, 0.327871, 0.436897, 0.178531, 0.303926, -0.960965, 1.307768, -0.090309, 2.162086, 0.565036, 0.115842, -0.120922, 0.535835, -0.847005, 1.641761, -1.290951, 1.439569, 1.370887},{0.702948, -0.543828, -0.352661, 1.043841, -0.179553, -1.604517, -0.736929, 0.548200, -1.458765, 0.973261, 0.432209, -2.135425, -1.205417, 0.206081, 0.420307, 1.272084, -1.897452, 0.846825, -0.391586, 1.597171, 0.287146, -1.035525, 0.844467, 0.985412, -0.662100, 1.559360, 0.938446, -1.066331, -0.360398, -1.625808, 0.261183, -1.071061, 2.771181, -0.658083, 0.772175, 0.173153, -0.583925, 0.870498, 0.991453, 1.525377, -0.662653, 0.213861, 2.659992, -0.122773, 0.473922, 0.351177, -1.192336, 1.151667, 1.731662, -2.555165, 0.479229, -0.663500, -0.069063, 2.348911, -0.770476, 0.018125, 0.260699, 0.120188, 0.853652, 1.466297, 0.287600, 2.294826, -0.123163, 1.681296, 1.666340, -1.330348, -0.690066, 0.299677, 1.250413, -0.419373, 0.170141, 0.634822, 0.592895, 1.662051, -0.108144, 1.113527, -0.878288, -1.528652},{-0.717096, 0.215876, 0.151139, 0.368163, -0.720976, 1.322657, 0.290357, -1.273913, 1.625581, 1.737641, 0.769024, -0.107931, 1.034714, -1.070178, -0.543815, 0.217388, -0.613779, -1.780306, 0.368005, 1.025486, -3.142086, 0.280111, -0.301361, 1.140926, 0.075558, 1.552108, -0.365106, 0.413529, -0.090404, -0.213557, -1.362146, -0.029372, -2.269926, -0.784876, 1.269171, 1.243824, -0.679513, 0.649573, -0.426478, -0.078447, -1.356338, 1.467983, 0.617578, 0.889802, 1.065375, 0.201596, 0.493835, 1.231681, 0.469942, 0.472914, 0.632859, -0.055454, 0.564671, -0.517869, 0.669380, 1.243315, -0.970022, 0.622006, -1.038933, 1.459785, -0.455445, -0.861537, 0.731784, -0.055685, -0.219704, -0.361630, 0.852358, 1.800644, 0.155356, -0.051666, 0.778403, 0.295993, -0.992754, 0.310540, 0.542797, -0.007342, 0.621824, -1.294535},{0.513571, 1.645984, -1.365433, 0.472376, -1.378670, 0.282435, 1.042629, -0.021490, 2.253745, 0.103265, 0.803470, 0.206518, 0.268478, 0.500236, 2.110694, -1.214724, -0.389094, -0.938833, 0.370097, 0.392367, -0.433845, -0.834537, -1.063187, -1.003692, 0.014216, 0.158002, 0.625359, -2.821743, 0.792729, 0.539363, -1.462203, 0.147458, -1.136582, -1.438605, -0.881244, 0.569960, 0.919196, 0.361123, -1.897012, -1.004194, -0.388467, -0.081046, 1.099893, 1.707172, 0.514024, -0.101003, -0.144339, 0.388698, 0.093129, 0.101015, -0.106812, 0.142815, -1.264328, -1.436255, 1.405578, 1.563194, -1.172186, 1.532986, -0.286838, 0.726776, 0.471532, 1.333512, -1.199547, -0.510278, 0.345308, 0.478782, 1.383355, -0.573313, -1.196202, 0.984273, 1.518957, 1.294207, -0.169432, 0.654852, 0.641951, 0.508018, -0.151237, -0.333143},{1.064778, 0.264700, 1.229589, -1.644717, -0.672340, 0.960051, -0.120951, 0.313402, 0.743835, 1.416887, -0.199973, -0.388815, -1.013072, 0.534305, -1.738025, -0.247132, 1.169136, -0.518987, 0.943097, 2.008090, 0.783298, -0.643691, -1.418397, 1.337313, 1.283936, -0.710218, -0.164191, 0.885485, 0.771148, -1.452056, -1.315714, -1.990277, 0.224447, -0.426895, -2.596544, 0.563207, 0.059696, -0.004749, 0.712261, 0.779209, 0.421979, -1.016323, -0.251481, 0.159409, 0.431616, 0.303597, 0.261252, 2.064508, 1.439646, 1.289984, 0.045169, 0.580753, -0.939076, -0.894713, 1.499964, -1.073460, 0.487579, -1.465797, 1.224503, 0.151022, 0.078231, -0.548020, -1.301921, 0.844102, -0.386525, -0.413584, -1.550788, -1.051688, 0.432777, 0.069258, -0.576905, -0.278492, 0.947125, 1.259478, 0.453441, 1.971358, -0.712609, 0.383624},{0.060450, -0.772047, -0.336093, 1.078970, 0.608218, 0.634033, 1.965793, 0.007905, -0.771655, -2.193378, 0.226457, -1.061482, 0.944744, 1.710281, 0.960216, -1.343124, -1.836213, 1.140043, 0.377679, 0.924938, -0.998821, -0.472848, -0.026891, -1.372228, 0.269665, -1.051613, 0.333176, -2.525720, 0.164062, -1.282775, -1.288784, -0.023507, 1.155517, 0.469569, 0.730151, -0.271341, 0.813022, -0.522192, 0.541977, 2.376024, 2.110697, -1.293112, -0.471811, 1.798808, 0.929424, 1.463759, -1.205947, -1.177687, 1.208201, -2.150759, 0.284967, -1.051534, -0.243071, 1.190163, 0.077728, -0.993340, 1.599853, -0.116967, -0.286453, -0.611525, 1.234354, -0.782866, -0.531996, -0.116976, 0.657106, 1.906215, 0.367644, -0.654925, 2.344529, -1.000197, -0.346632, -0.881328, -1.021948, 1.783980, -0.245449, 2.275931, -0.158445, 2.001600},{-0.480960, 0.372185, -0.112994, 0.184659, -0.500776, 1.454379, 0.441085, 0.567293, 0.576538, -0.246280, 0.094090, 1.768794, -0.664341, -1.598945, -0.633015, -1.238251, -0.176919, 1.857426, 0.177650, -0.110741, -0.112598, 0.302039, 0.287186, 0.694064, -1.458107, -0.306982, -0.470189, 0.054238, 0.139386, -1.000046, 1.985804, 0.367222, -0.595095, -0.846103, -0.401192, -0.582544, -0.314967, 0.149581, -0.519713, 0.598411, 0.344972, -0.671816, 0.508366, -0.902535, 0.826364, 1.810517, 0.334250, -0.103693, 0.492774, -0.177255, -0.469445, 1.073267, -0.398572, 1.223138, -0.417194, -1.480931, 0.901055, -0.017060, -0.564291, 0.241030, -0.752517, -1.457620, 0.387403, -0.953861, 0.556775, 0.259979, 1.346009, -0.105394, 1.049025, -0.655879, 0.218940, -0.576391, 0.919748, -0.727523, 0.753040, 2.203084, -1.122194, -0.677374},{0.592937, 0.739282, -0.177864, -1.350273, 0.199130, 2.051534, 0.147061, 0.016308, -0.553742, -0.609356, -1.181925, 0.162433, -0.832114, 0.830442, 0.438676, 0.232352, -0.841204, -0.168560, 1.048374, -1.069982, 0.565336, 0.295699, -0.438052, -1.033114, 0.680175, -2.064323, 0.014829, -0.555104, -1.080195, 1.507897, -0.218189, -0.141066, -0.954474, 0.485431, -2.684067, -1.017245, 1.760678, 1.674958, 0.159886, 0.137965, 0.815438, 0.632699, -0.189390, 0.548017, 0.394728, 0.740805, -1.000563, -1.464975, 0.859392, 0.883396, -0.730414, -2.236786, -1.044757, 0.814866, 0.987760, -1.232237, -1.160344, -0.903080, 0.494254, -0.243725, -0.483585, 2.109071, -2.084899, -0.681020, 0.340835, 0.349391, -1.113822, -0.833067, 0.358166, -1.622322, -1.372771, 0.379480, -0.033222, -0.980862, -1.716237, 1.337445, 0.811510, -2.719708},{-0.149863, 0.035024, -1.324794, -0.251021, 0.945798, 0.784555, 0.300606, -0.159708, 0.347710, 0.682394, 0.012012, 0.340894, 1.913240, 1.097523, 0.772497, -0.636352, -0.120140, -0.904577, -0.135555, -0.933189, -0.864236, 1.266397, 0.253141, -0.272175, -0.371193, 1.879133, 2.424397, -1.083357, 0.286866, -0.949526, 3.378528, -2.539450, 1.831341, 0.324683, -0.816902, -0.393306, -1.092621, 0.643662, -1.682272, 1.305001, -1.414537, -0.466238, -1.325939, 0.427755, -1.152085, 0.473565, -2.066170, 1.630649, -0.526172, -1.207120, 0.437964, -0.197619, -0.491229, 0.182410, -0.202279, 1.134782, 0.086788, 0.479923, -0.847881, -1.166259, -0.404267, -0.162982, 2.128286, -1.227885, 1.228866, 0.228487, -0.089342, -0.278272, 0.220701, -0.146805, 1.260344, 0.115731, 0.259611, 1.084543, 1.802930, 1.601518, -0.625412, 1.856193},{-0.839921, 0.995145, 0.409952, -0.660769, -0.896613, 0.868117, -0.509389, 1.587618, 0.394360, 0.571655, -1.283896, -0.376519, 0.167793, 0.059340, -1.572699, -1.184608, 0.748138, -0.041027, 0.119445, -1.559065, -1.030573, -2.067932, 0.828166, 0.282097, -0.508022, 0.840617, 0.942115, 1.826047, -0.958934, -0.766488, 0.249559, 0.521131, 1.388359, 0.470866, -0.341440, -0.549247, -1.460814, -0.287117, -0.142441, 0.775800, -0.575386, -0.522312, 0.603854, -0.196871, -0.746856, -0.445611, 1.026595, -0.085377, 1.002631, -0.569039, -0.582421, 0.382757, -0.235768, -0.128634, 0.256252, -0.153504, 0.130182, 0.389459, -1.025900, 0.196520, -0.412675, 0.021652, 1.373603, -1.472630, 0.661081, 0.163775, 0.238446, -2.193904, 1.188115, -2.182588, 2.045311, 1.437885, 0.774275, 1.111271, -0.528317, -0.677568, 1.266563, -1.242893},{-0.253180, 0.986552, -0.587714, -0.169073, 0.771261, -0.075059, -0.220744, 0.120314, -1.800024, 0.843749, 0.905284, -1.790474, 0.115891, 0.223764, -1.654627, -2.145726, 0.033127, -0.683007, -0.481337, -0.922946, 1.099640, -0.046608, 1.156762, -0.709631, -0.883420, 0.845377, -1.431002, 0.621783, 0.680582, 2.346527, 0.667060, -1.240417, -0.242343, -0.376666, 0.903133, -0.565577, -1.368981, 0.368679, 0.313608, -0.754434, 0.623414, 0.820944, 0.651081, -0.880722, -0.313667, -1.699944, 0.098646, 1.244657, 1.691629, -1.164623, 1.539404, -1.642790, -0.108142, -0.311081, 0.451121, 1.006998, 1.073178, -1.050813, 0.445757, -0.176755, 0.665745, 0.147427, -0.184208, 0.927787, -1.771424, 0.373262, -1.993350, 0.974309, -0.492180, -0.011999, -0.661859, 1.605902, -1.111524, 0.705791, -1.137177, 0.045349, 0.006342, -0.610654},{1.199205, -0.385390, -0.324417, -0.927565, 1.804500, -0.847617, 0.273619, 0.889010, 0.819371, 0.290840, -0.750742, 1.632984, -0.085224, 1.728784, 1.071247, -0.244577, -0.278253, -0.878969, -0.704243, -0.121148, 0.285537, -0.976426, -0.613354, 0.915547, -0.470021, 0.878206, 1.307064, 1.264454, 1.962944, 0.544340, -0.004136, 0.369391, -1.322515, 0.811125, 1.454268, 1.017848, 0.713607, 0.797433, -1.272384, 1.118570, 0.964106, -0.941062, 2.026883, 0.982440, 0.280374, -0.333902, 0.297465, 0.328559, 0.190051, -0.175004, -0.576957, 0.056804, -0.336745, 1.108220, 1.037670, -1.484519, -1.089573, 1.800795, -0.885519, 0.384636, 0.772271, 0.654557, -0.946713, -0.801353, -0.403052, -0.279589, 1.543961, -0.374280, 0.554516, 0.666391, 0.727034, -0.541745, -1.039568, 0.766383, 1.210149, -0.398806, -0.036196, 0.115124},{1.148212, 1.208357, 1.125326, -0.403134, 0.099949, -1.268808, 0.265271, -1.007349, -0.085472, -0.646125, -2.550906, 0.202421, -0.265603, -1.165583, 1.354744, -0.153417, -2.421605, -0.739617, 0.528377, 0.366427, 0.661715, 0.752904, 0.145719, -1.195289, -0.374902, -1.462711, -0.160651, -0.397042, 0.181085, 1.025072, 0.204546, -0.231158, 0.199811, -0.965954, -0.164173, -0.153586, -1.559577, -0.015638, -0.618984, 0.469356, 1.110303, -0.515272, -0.173420, -0.742165, 0.463057, 0.697504, 1.045712, 0.098836, -0.303918, -0.885926, -0.885769, -0.343943, -0.858606, 1.240682, -0.988889, -1.074856, 1.102800, 1.896090, -1.052244, -1.022905, 0.959576, -0.427223, -0.552169, -0.799767, -0.049607, -0.519885, -1.514913, 0.185024, -0.544521, 0.620574, 2.324321, 0.367155, 2.181626, -1.510193, -0.593034, 0.154723, -0.389049, 0.834056},{-1.169802, 0.123954, -0.358752, -0.199779, 1.215362, 0.560698, 0.846013, 1.738860, -0.181310, -0.495596, -0.670581, -0.516026, -0.830364, 0.151028, -0.735510, -1.278058, -1.608466, -0.054015, -0.362361, 0.774954, 1.285473, 1.065209, -0.917102, 0.140042, 1.247998, -0.350517, -0.445882, -0.473453, 1.756530, -0.571878, 1.185340, 0.187058, -0.095837, -0.368804, -0.073255, -1.001571, -2.993219, -0.405125, 0.396499, 0.317627, 0.179446, 0.007787, -1.085423, 0.180754, -0.654412, -0.279544, -2.125260, 0.290910, 0.078282, 0.723064, -1.084065, 1.285645, 0.414899, -0.115691, -0.116239, -0.677752, 1.092825, 1.967529, 1.794350, 0.572701, 1.458955, 0.158193, 0.614328, -0.401045, 0.343270, 0.060994, 0.371596, -0.794117, -0.389743, 0.886670, -0.712762, -0.059392, 0.611242, 0.790641, 1.011936, -0.349346, 0.932622, 0.879119},{0.575241, 0.482994, 1.306653, -1.787081, 0.789972, -0.954061, -1.400547, 1.377089, -0.170628, 1.413385, -0.023247, 2.710382, -0.778218, -0.145345, -0.787352, -1.430951, 0.636164, 0.432652, -1.253772, -1.059936, -0.306041, 0.558807, 0.729705, -0.016590, 1.047592, 0.448574, 0.525160, -1.180935, -0.346234, 1.059139, 1.202943, -0.133949, -0.561562, 0.316681, 0.460800, -0.143147, 0.863322, -2.260607, -0.727353, 1.954237, -0.811495, -0.249977, 0.462113, -0.284680, -1.013026, 0.911846, 0.185680, -1.270205, -0.225612, 0.836038, 0.790277, 0.702782, 1.777938, 1.721817, -0.674754, -1.130739, 1.639930, 0.386578, 0.347085, 1.156814, 1.275682, 0.288385, 0.203501, -1.551878, -0.055341, -0.026426, -1.128350, -0.260764, 0.928845, -0.232158, 0.722492, -0.531051, 0.958230, -1.267835, 0.093079, -0.879352, -0.462818, -1.687293},{1.388658, -0.039167, -0.133980, 1.312977, 0.469603, 1.090935, -1.101840, 1.177370, -0.086780, -0.078130, -0.941392, -0.268378, -0.184488, -1.026555, -1.096480, -2.355593, -0.588868, -0.693335, -1.179229, 0.659415, -0.610875, -0.692545, -0.540367, 1.828372, -0.774708, 1.391763, -0.193622, -0.543905, 1.000404, 0.910962, -1.041553, -0.060020, -1.491683, -1.966514, -0.339409, -0.317945, -1.197989, -0.730751, 0.850631, 0.094330, 0.972962, 1.015945, -0.192799, 0.136853, 2.119800, -1.300460, 0.347057, 1.862029, 0.318270, 1.934880, -2.242496, 0.858274, 2.464064, 0.069518, -0.398845, -0.088150, 0.475350, -0.623764, -0.548627, -1.053312, 0.584092, 0.577197, -1.530596, -0.114810, -0.189557, -0.967918, 0.585931, 0.598347, 2.466233, -2.105834, 0.227186, 0.840656, 0.710563, 0.865759, -0.478673, 1.250072, -0.135530, 0.854704},{0.955618, -0.491855, 0.212531, -1.182093, 0.255881, -1.046782, -0.603876, 0.090565, 0.731114, 2.291002, 1.173103, -1.910377, 0.938635, 1.098465, 0.394932, -0.020771, 1.273238, -0.533049, 0.368235, -0.662380, 1.403550, -1.143371, 0.500497, 0.771308, 0.144683, 2.985431, 1.345831, 0.040496, 1.556323, -0.423465, -0.424535, -0.385739, 0.898985, 1.810730, -0.020868, 0.221836, -0.588325, 0.432495, -0.084818, 0.293152, -1.056414, -1.350708, 1.082460, 1.655664, -0.311226, 0.882643, 1.105735, 0.344981, -0.585507, 0.201984, -0.304113, 1.148616, 1.099664, 0.345088, 0.540960, 1.618305, 0.504802, -0.407293, 0.526392, -0.755114, -0.192284, 0.808259, 0.260131, -1.018742, 0.512614, 0.250254, -0.108152, -0.974200, -1.578913, -1.281020, -0.862121, -1.184996, 1.005286, -0.920512, 0.161581, 0.424891, 0.656772, 0.113606},{0.625981, 1.050447, -1.357973, 0.096533, -0.544352, 1.271449, -0.870436, 0.347107, -1.497072, 1.081426, 1.384826, 0.971201, 1.109922, -0.287688, 0.693698, -0.675654, 0.423287, -0.353255, -1.802858, -0.550382, 0.264397, 2.635339, -1.130229, -0.861133, -0.225127, -0.886979, 0.616944, 0.251918, -2.293516, 0.108145, -0.168230, 0.854750, 1.160829, 0.435409, 0.273336, -0.490043, 0.707358, 0.026795, -0.755229, 1.745335, 1.012208, -1.089613, -0.065111, -0.231006, -2.097186, -0.026419, -0.000253, -0.341371, 0.405547, 0.320826, -0.973345, -0.753630, 0.215935, -1.264042, 1.090537, 1.177421, 0.409595, -1.958562, -0.381768, 0.427244, -0.834316, -0.380112, -0.729662, -0.771756, 0.228055, 0.721282, 0.747357, -0.194958, -0.880869, -1.390404, -0.016582, -0.768242, -0.483758, -0.865320, -1.929568, 0.960379, 0.178087, 0.740796},{1.322537, 0.034068, 1.024714, 0.836590, -2.450264, 0.523959, 0.783987, 0.294630, 1.119650, 0.027255, 0.045186, 1.891990, 0.396021, -0.695660, -1.704567, -1.313269, 1.886688, -0.777863, 0.619693, 0.601746, -1.133204, -0.968528, -0.061151, -1.575777, -1.441528, 0.913200, -0.909203, -0.547670, -1.251994, -3.199334, 0.689333, -2.223763, -0.714984, 0.925721, 0.785251, 1.117151, 0.827086, 0.297046, -1.567277, -1.679628, 0.148801, 0.220784, -2.480744, -0.784962, -1.298915, -0.430478, -0.195965, -0.588739, 1.069565, 0.396812, -0.363640, 0.316264, 1.986414, 0.686379, -0.218833, 0.156098, -0.141161, -0.669113, 0.526521, 0.377418, 0.588794, 0.665951, 0.802661, -1.758855, 0.244957, 1.677652, 2.014962, 1.900813, -0.170541, 0.888423, -1.262739, -0.023997, -0.135687, -0.586976, 1.473626, 0.091240, -0.222289, 0.625626},{-1.206239, -1.158773, 1.450763, 1.895265, 0.759338, 1.379703, -1.414330, 1.160046, 0.733995, -2.216048, -0.418893, -0.132755, -0.181664, 0.321957, -1.282205, 0.898738, 2.114372, -1.185034, -0.228143, -1.368997, 0.318277, 2.087337, -0.631526, 0.641315, -0.734928, 0.367600, -0.287825, -1.238865, 0.310572, 1.499725, 0.948799, 0.411596, -1.042434, 0.054925, 0.036538, -1.196221, -1.582050, 1.357010, -0.297858, 1.792096, -0.316309, -0.399049, 0.814207, 0.885872, -1.192720, 2.045239, -0.650870, -1.114625, -0.831177, 3.670631, 0.050536, 0.633643, -1.920151, -1.389466, 1.857522, 0.396851, -1.325437, 0.262473, 0.677629, 2.059104, 0.669900, -0.760291, 0.513804, 0.879100, 0.410634, -1.169041, 0.603518, -0.972708, 0.321786, 0.209396, 2.471607, 0.608072, -0.759430, -1.280075, -0.796709, 1.390061, 0.193061, -0.080456},{1.953577, 0.870488, 0.608318, 1.487399, 0.326853, -2.110814, -1.667294, 0.432811, 0.766398, 0.257389, -2.570745, -0.101457, 0.671359, -1.449777, 0.092968, -0.711764, -0.421604, 1.665954, -0.618324, 0.087716, 1.427628, 0.342775, 0.395755, -0.149397, -0.271080, -0.220447, 0.535999, 0.695885, -0.981403, -0.230895, -0.353930, 0.338607, -0.799622, -0.255400, -0.461786, -1.220056, 2.884050, 0.436611, 0.052426, 2.189884, 1.068893, -0.787211, 0.089652, -0.950416, -0.597672, -0.535887, 0.128023, -0.874100, -1.013039, -0.129220, -2.301817, 0.951285, -0.390428, -1.085899, -0.055643, -2.319952, 0.165365, -0.810321, -0.044631, 0.380446, 0.987702, -1.968568, 0.891350, -0.787618, -1.071800, -0.299179, -0.216735, 0.345449, -0.704957, 1.052229, -0.542196, 1.864821, -1.203128, -0.186442, 2.384435, -1.205606, 0.840390, -1.101503},{0.231911, -1.967729, 0.339171, 0.636218, 0.353628, -1.316099, -0.791300, 1.557312, -0.216280, -0.652837, 0.984261, -0.317124, 1.716636, -0.303108, 0.417192, -0.239526, -0.095812, 0.657658, -0.733584, -0.353580, 1.032178, -0.647798, 0.014139, 1.878453, 1.068105, -0.520107, 0.064199, -0.322035, -1.645355, 1.184541, 0.281880, 1.152485, -1.065292, -0.620117, 1.142517, 0.752591, 0.304351, 0.660922, 0.167531, -0.931146, 0.453351, 0.611736, -0.902631, 0.598244, 0.062043, -1.023549, -0.447554, 0.225448, -0.192338, -0.674156, 0.060462, 0.065655, 0.745897, -0.700594, -1.491812, 0.187199, -1.087766, 1.594844, -3.090039, -1.313761, -0.306825, 0.696676, 0.597472, 1.733389, -0.724448, -0.125060, -0.190341, -0.309972, 0.816259, -1.480909, -0.117636, -0.725554, 0.954178, -1.164640, 1.014107, -0.133665, -0.752179, -0.129935},{-1.473857, -1.221983, -0.825480, 1.289811, -1.494501, 0.756832, -0.448189, -0.397790, -1.661778, -0.515721, -1.481883, -0.276851, 1.121389, -0.262835, -0.727209, 0.191735, -0.292793, -0.279370, 0.254334, -0.798754, 2.051344, -0.853004, -1.442887, -0.637461, -0.945085, 0.326764, -0.324100, 0.525982, 0.290566, 1.440301, -1.198085, 0.479196, -0.161737, 0.011601, 0.042773, -1.718023, 0.374380, -0.366024, 0.402419, -0.736928, -0.425795, -0.339250, 0.747322, -0.251882, 0.508596, -1.468340, -0.461795, -0.215939, -0.737229, -0.224022, 2.047367, 1.313677, -0.565543, -0.423010, 0.345443, 0.005370, -0.633994, -1.491530, 0.101953, 0.368280, -1.007200, 0.506223, 0.622198, 0.523304, -1.003351, -0.381440, 1.083907, -1.044851, 1.181138, -0.712372, -2.053771, 1.560660, 0.631688, -0.589972, 0.723249, -0.464097, -0.287152, 0.841619},{0.336515, 0.222179, 0.181732, 0.643522, 0.716314, 0.230014, -0.027396, 0.444114, 0.851238, -2.640153, 0.886555, -0.840910, 0.989355, 1.179913, 0.679239, -0.432027, 1.305997, 0.884237, -1.332729, -0.198029, -0.023367, 0.850317, -0.226922, 0.116859, -0.356000, -0.267182, 0.107358, 0.081614, -0.896791, -0.117714, 1.017700, -0.309579, 0.894685, -1.041214, -0.628801, -1.009879, -1.426647, 0.718149, 0.412764, 1.522120, 2.525786, -0.672133, -1.480903, 0.460591, -1.025932, -0.688097, 0.175140, -1.091402, -0.308075, 1.024530, -0.049067, 0.716775, -0.394867, -0.265446, -2.308205, 1.343126, -1.065740, 0.385518, -0.126691, -0.103656, -0.187731, 0.307510, 1.386534, 2.779080, 0.685651, -0.382035, 0.430393, -1.380306, -0.858149, -0.226881, -0.314121, 0.680707, -0.329189, -2.055356, -0.794479, 1.483027, 0.516377, -0.164265},{-0.723042, 1.821887, 0.516532, 0.153800, -0.327106, -0.048209, -0.503438, -0.028751, -0.445583, 1.302553, 2.042403, -1.126278, -0.604781, 0.576470, -0.310139, -0.296954, 0.108374, 1.233283, 1.300208, 0.502079, 0.379004, -1.068831, 1.012033, -0.821579, 0.711888, 0.477000, -0.350763, -0.454133, 0.236370, -0.456192, 0.337817, -0.336341, -1.569492, 1.198055, 0.635354, 1.454174, 0.009438, 0.578509, -0.009395, 0.034040, -0.656384, -0.976491, -1.529269, 0.358545, -0.336932, -1.256841, 1.594779, 0.704007, 1.683842, -0.661621, 1.101467, -0.674904, -1.038918, -2.316102, 0.710603, -0.137418, 1.232352, -1.228990, 1.006971, -1.553739, 1.429558, -1.636541, -0.691503, 1.214947, -0.473698, -1.711981, 0.597566, -0.912348, -0.211403, -0.819381, -0.239602, 1.432634, 0.589695, -0.062210, -0.285677, -1.148791, -0.662411, 1.663204},{-1.074489, -0.361615, -0.405400, 0.296542, -0.265336, -1.324807, -1.384140, 0.312251, 0.199788, 0.521182, -0.149156, 0.427842, -0.315554, 0.019167, 0.920518, 0.604731, 2.392770, -0.954736, 0.817589, 0.703317, -0.348494, -0.381433, 0.141841, -1.119997, 1.933836, 0.829426, -0.401756, -0.217998, -0.471501, -1.051948, -0.312930, 0.245083, 1.428086, -0.555286, -1.173458, -1.269762, 0.133851, -2.048338, 1.437820, -0.968389, 0.736653, -0.402894, 1.822961, 0.784267, 0.800810, -1.518370, 1.244188, 0.464979, -1.138492, -0.301596, -0.941548, -0.734510, 0.007673, 0.640676, -1.516899, -1.225572, 0.007324, 1.731522, -1.075267, -0.712208, -0.110073, -1.207070, -0.541296, -1.008167, -0.097684, 0.369601, -0.912024, 1.319437, 0.153144, 1.491016, 0.466005, -0.640074, 0.715186, -0.684949, -0.998434, 1.579632, 1.680696, 0.012941},{0.648899, -0.119622, 0.132639, 0.657930, -0.487007, -0.192684, 0.956340, 0.101852, 0.885092, 0.151791, -1.344785, -0.595677, -0.778694, -0.389974, 0.151678, -0.071384, -0.890041, 0.805125, -0.257425, -0.594752, 0.932989, -1.455300, -0.533827, -0.077693, -1.630006, -0.062172, -0.279533, 0.958304, 0.338955, 0.120034, -0.217600, -1.605696, -0.063559, -0.143721, -0.389853, -0.360308, 0.136869, -0.203341, 0.666192, -1.768489, 0.245555, 0.343161, -0.972568, -0.670094, 1.201816, 1.390349, 1.214411, 0.214982, -0.224587, -0.167998, 1.340948, -0.715075, -0.295085, 0.593501, -0.457459, 0.755206, -0.275220, 0.108404, 1.080672, 1.040415, -0.929338, -1.344272, -0.198023, 0.726646, 0.152901, -1.081062, -0.221494, -0.790068, 0.789265, -0.342434, -0.619805, 0.963582, 1.338207, -0.632126, -1.471775, 0.147544, 0.352893, 0.309401},{-0.821147, 1.474045, -2.742777, -0.388499, -0.508048, -0.949711, -0.031491, 0.428318, 0.778940, -1.369920, -1.655498, -1.336411, -0.432412, -0.599629, -1.639421, -0.199496, -0.226944, -1.197165, 1.357724, 1.158648, 0.906049, -0.262998, 0.767868, -0.423980, -0.007799, 0.863785, 1.419101, -0.151499, -0.583378, 0.690417, 0.445589, -1.134381, 0.388022, 0.468861, 0.801797, 0.299711, 0.911152, 0.235250, 0.466142, 0.070944, 0.234374, -0.107535, 0.500442, -0.852847, -2.634654, 1.270417, -1.527402, 0.766468, -0.191601, 0.203874, -0.148546, 0.875951, -0.127110, -1.251145, 0.292107, -1.512430, 0.180619, -0.089082, -0.654343, 0.168824, 0.903223, -1.445426, 1.422977, 0.138538, -1.422817, -0.516391, 0.955053, -1.403353, -1.595587, -1.241777, -0.415701, 0.251997, 0.851829, 1.824145, 0.946158, -0.737614, 0.317201, -0.769341},{1.489631, -0.213339, 0.292083, 0.016443, -0.394359, -1.316359, 0.758644, -0.545472, -0.473612, -0.524089, 0.733337, -0.031229, -1.004330, -0.178216, 1.128828, -1.275521, 2.445228, -1.631926, 0.005247, -0.574010, 0.110834, 1.170118, 0.137297, 0.607444, 0.495465, -0.111646, 0.790530, 1.448040, 0.340402, -0.038653, -0.748266, -0.219691, 1.432794, -1.585118, 0.284525, -0.516176, -2.334965, -0.035457, 0.464874, 0.523719, 0.278963, -0.906879, 1.687723, 0.780832, -1.502861, 0.974186, -0.814219, 1.484468, 0.171665, -0.061198, 0.194724, -2.437323, -0.235745, 0.446897, -1.808496, -1.506057, -0.890665, -0.149986, 0.624973, 0.098542, 1.022862, 0.871328, -1.162351, 0.722464, -0.225730, 1.730764, 0.419152, -0.180854, -0.921968, 0.475661, -0.365906, -0.582253, 1.753307, 0.534573, -0.146631, -0.076843, 0.498922, 0.412917},{-1.259000, 0.526411, -0.570735, -2.293432, 0.011266, 0.544934, -0.988240, -0.625461, -1.210998, -0.214964, -1.026355, -1.094184, 1.700815, 0.221138, 0.330577, 0.669677, -0.279825, 0.091965, -2.154220, -0.079537, -0.094621, 0.330796, 1.142917, 0.790682, 0.089084, -0.032055, 1.305329, -0.578597, 0.103315, -0.701182, 1.167326, -0.309111, 0.229976, 0.223288, -0.560494, 0.352592, -0.530049, 0.431991, 1.790537, 0.188285, 1.937364, 0.108831, -2.432022, -0.125243, 0.983070, -1.189949, 0.969863, 0.706704, 0.535926, -0.914722, -1.801928, 0.418682, 0.707837, -1.570511, -0.369315, -1.120162, -0.046869, 0.831821, -0.702469, 0.673331, 1.305049, -0.343680, -1.796155, 0.712090, 0.766070, -0.499140, 0.335271, 0.584566, 0.559736, -2.158991, -1.103802, 0.989007, 0.013886, -0.649441, 1.879770, 0.260267, 0.504989, -0.951712},{1.384527, 0.636422, 0.145578, 2.152293, 0.448626, -0.027010, -2.139611, -0.193165, 0.392789, 0.243704, -0.633587, 0.135180, -0.691429, 0.096301, -0.438276, 1.677092, 0.187432, 0.303907, 0.602737, -1.094026, 1.069701, 0.686072, -0.230518, 1.186895, -0.261708, 0.217161, 0.669662, 0.056538, -0.879006, 1.541267, -0.237198, 0.163011, 0.104377, -0.553960, -0.593278, -0.356383, 0.986690, -1.721720, 0.767141, 0.813723, -0.672427, -1.157599, -0.070441, 0.649271, -1.697923, 0.439829, -0.255468, 0.310087, 0.031952, 0.022646, -0.280338, -0.533251, -1.438074, -0.726802, -0.853901, -1.750285, 0.114343, 1.835909, -1.532722, -0.682835, -1.827449, 1.383816, -0.078091, 0.197501, -0.031439, 1.418682, -0.728629, -1.345438, 1.137742, -0.371231, -0.996552, -0.493080, 1.192790, 1.718877, -2.216579, -0.543564, -0.212950, -1.005393},{0.550346, 0.841384, -0.609939, 0.389770, -0.211168, 0.244649, -0.500652, 0.124537, 1.078683, 0.510030, 0.316830, 2.337373, -1.346295, 1.455152, 2.448156, 1.161708, -0.716464, -1.858288, -1.183469, -1.348429, 0.150304, 1.617310, 1.607315, 1.798634, -0.921727, -0.006043, -0.829598, -0.054101, -0.403866, 0.121654, 0.817531, 0.568789, -0.974618, 1.025751, 1.239780, 0.439045, 0.733991, -1.962242, 0.091597, -0.228322, -0.518344, -0.066590, -1.780456, -0.875255, -0.124896, 0.445319, -0.165715, -0.320443, 1.608646, 0.111724, -0.771438, -0.844789, 0.348780, -0.357201, 0.391142, -0.443717, 0.838201, 0.578263, 0.292841, 0.603126, -0.329257, 1.246852, -0.466574, -0.163465, -0.173659, 0.837140, 0.506066, 0.903286, 1.093134, 0.126612, -1.264435, 0.391602, -1.171624, 1.312989, -0.387553, 0.281731, 0.190119, 1.688591},{-1.256445, -0.917818, 0.532738, 0.338340, 0.123757, 1.153268, -0.921935, -1.058805, -0.703924, 1.602766, 0.008626, -0.434326, 0.169254, -0.362846, 0.615279, 0.590060, -3.363675, -0.722619, 1.636448, 1.459099, -0.035657, 1.050727, -2.419645, -0.897331, -0.271128, 1.698543, -0.540680, -1.282094, -0.216721, 0.882280, 0.566453, -0.601368, -1.396643, 0.663141, -1.157966, 0.398831, -0.363806, -1.221951, -0.174622, -0.286920, 1.462373, 1.136373, 0.703846, 0.541482, 0.095548, 1.175436, -0.375283, -0.052634, 1.206290, -0.900724, -0.377943, 0.325514, 0.010542, -0.057064, -0.193025, 0.614234, -0.295437, 1.120607, 0.618641, 0.325644, 0.807884, 1.422203, 0.874870, -0.954859, -2.783613, 0.842365, 0.594126, -0.415289, 1.519513, 0.138876, 0.069129, 0.825673, 0.118935, 0.908071, -2.032732, -0.072694, 1.248205, 0.801802},{0.570354, 1.911168, -1.826582, -1.608663, 0.361567, 1.025726, 0.619255, -0.021686, 0.531443, -1.037830, -0.901584, 1.246113, 1.491561, 0.091029, 0.176607, -2.620493, -0.607739, 0.295821, -0.639970, 0.536135, -1.293720, 0.835875, -0.035550, 0.134526, 0.860223, -1.424954, 0.661722, 1.459954, -0.850200, 1.450412, 1.971266, 0.368526, -0.982123, 0.308838, -0.278494, 0.912982, -0.703401, -1.265842, -0.938508, 0.673384, 0.521802, -0.878108, -0.573744, -0.249996, -1.466649, 2.646961, 1.574661, 0.976799, -0.824867, -0.619254, -0.998810, -0.004178, -0.992623, -0.745975, -1.228133, -2.249610, -0.891557, 0.170345, -0.901194, -0.620246, -0.906298, 0.385880, -1.327640, 1.563647, -1.120728, 0.785117, -0.672843, 0.289247, -0.849599, -0.913137, 0.832709, -1.777522, -0.337735, 0.195196, 0.029992, -0.488512, -0.701510, 0.537640},{0.348775, -0.404372, 0.509642, 0.597528, 1.420688, 0.389132, 1.178578, -0.173347, -1.474910, -1.128065, 1.439296, -0.411267, 0.301594, -0.888035, 0.576804, 0.056049, 0.692708, -2.392884, -0.581037, -0.708437, -0.018868, 1.158785, -1.478754, -0.287140, 1.624803, 0.705399, 0.702957, -0.674302, 0.227049, -0.949494, 0.573420, 0.339692, 0.484852, -0.537701, -0.084872, -0.794654, 1.037505, -0.021013, 1.165925, 0.234394, -0.432352, -1.714698, 0.026789, 1.528736, 2.204482, 0.314235, -0.471595, 1.399814, 0.491834, -0.963356, 0.214491, -1.484639, -1.559047, 1.176774, -0.664356, -0.299850, -1.289111, 1.401529, -0.536151, -0.857099, 1.200575, -0.677470, -1.096498, -0.711242, -0.626475, -0.416983, 1.784863, -0.621743, -0.377322, -0.958078, 0.669579, -0.495882, 0.781470, -0.673657, 1.367136, 1.412042, 2.230568, 0.682249},{-1.240978, 0.887540, -1.236534, 1.386019, -1.287188, -0.803838, -0.222006, -0.227527, 1.920491, -0.284767, -0.887282, 0.635369, 0.516160, 0.398072, -0.746848, 0.514078, -1.192308, 0.574821, 0.596414, 0.190821, -0.734514, 0.086445, 0.834469, 0.401582, 0.731293, -1.587850, -0.412463, 0.381528, 0.995696, -0.630289, 0.713841, -0.577228, 1.155977, 1.158106, -0.108088, -0.481969, 1.329333, -2.661249, -0.646274, 0.337779, 1.217021, 0.848217, 0.620278, -0.291181, -0.327338, 0.430913, -0.374017, -2.503841, 0.414019, 2.199201, -0.862977, 0.340717, 0.062293, 0.984229, -1.537237, 0.714019, -1.300175, -1.291814, 0.676614, 0.016206, 0.248990, -1.739794, -2.256508, -0.570416, -0.327639, 0.446134, -1.496048, -0.655488, -0.574194, -1.938858, 0.287843, 0.797530, 1.600306, -1.021062, 1.344429, 0.413466, 1.520019, -1.913795},{-0.345109, -2.129521, -0.175119, -0.070857, -0.162972, 0.395957, 0.655045, -0.389192, 0.654877, -0.909083, 0.107723, -0.734337, 0.035154, 0.646957, -1.221809, -0.839984, -0.306478, -1.421436, -0.522963, -0.942393, -0.870992, -0.119158, 0.842381, 0.189064, 0.571495, -0.786996, 2.717708, -1.303005, 0.340394, -1.671788, -1.415417, 0.190584, 0.502570, -0.731894, 0.267743, -0.034880, 1.522751, -0.201066, 0.945397, -0.014412, 1.720346, -0.457225, 0.328490, 1.193812, -0.348290, -0.226596, -0.276719, -1.035340, 0.897747, -0.097467, -0.673375, 0.004355, 1.962484, 0.417331, -1.586471, -2.529737, -1.070607, 2.006685, -0.681483, 0.191867, -0.314533, 0.063376, 1.388280, 1.738105, 1.234384, 0.397049, 0.155088, 2.239143, -1.351624, -1.142523, -0.530373, -0.124631, -0.462002, -2.778720, 1.217129, -1.259460, -1.128015, -1.501137},{0.587005, 0.088269, -0.335764, -0.124376, -0.676945, 0.829894, -1.265148, -0.198733, 0.557541, -0.108868, 0.411385, -1.496278, 0.751234, 0.938943, 0.538940, -1.262462, -0.502083, -1.161745, -0.453894, 0.354557, 0.238459, -0.146319, 1.097767, -1.712473, 0.755749, 0.719244, 0.655871, -0.985105, 0.759932, 0.125878, -1.058203, 1.323676, 0.148408, -0.066502, -0.044831, 0.300088, 0.600099, 0.563778, 0.114057, 0.904603, 0.074270, 0.227885, 1.007547, -2.272237, -0.344575, -0.350029, 0.236934, 0.474682, 1.314484, 0.984725, 0.323118, -1.186518, -1.345768, -0.228258, 0.383791, 0.527358, -1.325692, 0.094393, 1.499295, -1.030124, -0.096905, 0.280280, -0.677418, 1.361958, 1.112844, 0.112861, 0.062486, 1.386842, -2.172681, 0.098797, -0.261640, 1.729844, -1.213446, -0.832736, -0.820264, -1.416815, -0.901106, -1.175026},{-0.356388, -0.581055, -0.404239, -0.426550, -1.004551, 0.394558, -2.003404, -0.191844, 0.304101, -1.981258, 0.028092, -1.042805, -1.040997, -0.103804, -0.400918, 0.372227, -0.512877, 0.318194, -0.199662, -1.355370, -2.233835, 0.638654, 1.422221, 0.018020, 3.068506, 0.145179, 0.606804, 1.540668, -0.492473, 1.194030, -0.004966, 0.471562, -0.835570, 0.114668, -0.720522, -0.306087, 1.377708, -0.597747, -0.531131, 0.236104, -0.102064, -0.816361, -0.425093, -0.675446, -0.345580, 1.252542, -0.623571, 0.309139, 0.994860, -0.418031, 0.536827, -0.269209, 0.808167, -0.342793, 0.071751, 1.276316, -0.765185, 0.018917, 0.725791, 0.046699, -0.152857, 1.595702, 2.234386, -1.155880, 1.451187, -0.449272, -3.085617, -0.139391, 0.557591, -1.081589, 1.076504, 2.910503, 1.012134, 0.256392, -1.312565, -0.194911, 1.235138, 0.285321},{-0.393345, 0.448624, -1.977935, -1.869085, 1.589480, -1.053297, 0.364899, 0.704589, 0.477164, 0.407162, -0.008493, -0.797150, -0.694003, 0.284516, 0.236521, -0.040292, -0.127731, 0.654681, -1.745361, -0.236724, -0.270953, -0.189665, -1.151159, -0.473099, 1.773589, 0.047156, -0.033272, -0.225084, -0.107034, -0.019294, 1.609644, 1.531805, 0.374333, 0.958392, 1.506705, 0.443975, 2.129519, -1.715287, -0.590252, 0.369234, -0.199052, 1.450509, -0.512335, -0.591164, -2.160552, -0.531104, -1.019592, 0.003745, 1.369049, -2.583513, -0.506197, -0.924132, 1.006861, -0.682260, -2.222137, 0.994180, -0.342974, -1.679491, -0.385257, -0.263323, 0.356224, 1.649578, 0.720192, 0.801038, 0.194743, -1.518189, 0.207619, 0.349223, -0.926505, 1.370316, 1.836159, 0.436828, -0.164807, 1.763220, 1.105023, -1.619474, 1.308588, 0.474860},{2.030973, 0.505375, 1.243903, 0.742603, -0.132046, -0.579192, -1.551449, -0.970298, 1.793810, -0.553463, -1.690813, -0.018449, 2.738177, -0.095769, -1.924470, 0.949594, -1.753296, 0.271064, 0.074969, 0.130964, 0.169701, 0.045679, -0.112597, -1.031970, 0.594074, -0.736795, 1.180810, -0.611322, -1.444517, 1.041344, 0.292685, -1.446123, 0.426511, 1.125779, -0.816214, 1.994980, -1.041708, 0.818535, 0.047433, -1.321840, -0.229159, 0.002548, 0.210940, -1.069021, 1.828607, -1.696820, 0.653248, -0.778923, 0.791788, 0.613984, 0.496657, 0.783247, -0.403437, 0.421849, 0.958973, -0.842087, 1.290712, 0.752255, 1.405957, 0.221696, -1.344281, -0.528290, -0.289262, 0.361522, -0.369792, 1.758756, -0.991908, -0.127182, 0.570858, 0.307215, -0.217463, 1.698566, -1.112385, 1.753181, -0.221454, -0.298565, 1.265974, 0.447165},{0.633418, -2.074507, 0.355011, 0.270390, -1.578527, 0.861496, -0.231404, -1.803836, -0.371613, 0.221443, -0.057433, 1.211876, -0.886961, -1.123138, 1.584266, 0.082727, 0.674803, -0.629477, -1.117571, -0.384207, 0.645044, -1.201123, 0.301986, -0.052113, 0.379471, 0.250873, 2.151978, -1.583470, 0.715634, -0.132171, 0.598041, 0.469456, -0.726342, -0.577417, 0.448075, -1.656724, -0.466915, -0.004262, 2.021687, 0.276394, 1.119932, -0.218310, -0.837109, 0.868409, -0.202022, -0.955577, -0.362793, -0.857828, -0.115186, 0.841644, -0.131362, -0.085411, -1.285089, -0.442023, -0.602050, -0.410213, -0.625845, -1.851644, -0.917664, 0.690363, -1.372113, -0.729756, 0.882194, -0.844020, -0.348996, 0.573489, 1.399913, -1.294812, 2.939346, -0.253313, -0.788847, 0.236702, -0.360859, 0.155288, 0.793461, -0.389940, 1.482908, -0.697348},{-0.732799, -0.574264, -0.113943, -1.198738, 0.211847, -0.722749, -0.147637, 0.809916, 0.911622, -1.101524, 1.242078, -0.921908, 0.154755, -1.188671, 0.372980, -0.159658, 1.019245, -2.218988, -1.594197, -0.149017, -1.801629, 0.747823, 2.788979, -0.334670, -2.115480, -0.222256, -0.334686, 0.209409, 0.061731, 0.852120, 0.219671, 0.851257, 0.065853, 1.814170, -0.190619, -0.357302, -0.084541, -1.159098, -0.599496, 1.161769, 0.611145, 0.740870, -1.375770, -0.268656, 0.425058, 0.516174, 0.619906, -1.184260, 0.191858, 0.071959, 0.304730, 0.279802, -1.301217, -0.898794, 0.027280, 0.706747, -1.330800, 0.165168, 0.245971, 0.315341, 1.082611, -1.741520, -0.128976, 0.803693, -0.816744, 0.922687, -0.491269, 0.744792, 2.135937, 0.640596, -0.126202, -0.188688, -0.126179, 1.360715, 1.171516, -1.187045, -0.422261, -0.930546},{-0.754253, 0.005868, 1.725374, -0.768999, 1.396431, -0.548967, 0.033131, 0.240992, 0.419734, -0.115041, -0.676323, -0.063505, -0.923554, -0.676268, -1.725495, 1.077105, 1.916934, 0.936066, 1.348260, -0.553971, 0.361259, -0.757974, 0.364716, -0.107208, 1.119743, 0.067687, 0.286024, -2.665585, 1.403855, 1.009068, -0.738779, -0.807048, 0.675630, 0.983624, -0.655659, 0.154146, 1.049927, 0.176670, -0.039831, -0.445047, -0.473377, -0.929897, -0.136845, -1.229476, -1.472013, -0.016772, -0.521821, 0.177269, 0.478738, -0.964503, 0.849278, -0.798975, 2.092767, -1.389545, -0.007796, -0.768439, -1.587014, -0.462539, -1.325907, 0.648590, 1.228148, -0.005417, -1.225475, -0.670273, -0.717934, 0.267319, -2.773782, 0.390763, -0.466099, 0.582790, -1.292362, -1.449844, -1.172739, 0.051519, -1.139581, -1.123199, -0.692338, -0.415236},{-0.268170, -0.683310, 0.096278, -2.174183, -0.082664, 0.788831, -0.964857, -2.057935, -1.588843, -0.456892, -1.112679, 1.307759, -0.693550, 1.451111, 0.709213, -0.608123, 1.353556, 0.276849, 0.543205, 0.291619, 0.653513, 1.483959, -0.682049, 0.920579, 0.208220, 0.659896, -0.243949, 1.058667, 0.889237, 0.389754, -0.492160, 1.004567, -2.069118, 0.978943, -0.829294, 0.985651, 0.652785, -0.910345, -0.599684, 0.585527, -0.967744, -1.537265, -0.516956, 1.148530, 0.406491, -0.608568, 0.196164, -0.578651, 0.641657, -0.959918, -0.742122, 0.712954, -0.465065, 1.214723, 0.244846, 1.085714, 0.900055, 0.611329, 0.864370, -1.405303, -0.214525, -0.170624, -1.452918, -0.463677, 0.838173, -0.463096, 0.200818, 0.338963, -1.305332, 0.066391, 0.272901, -0.531961, -1.993923, 0.612360, 0.443686, -1.100850, -0.643811, 1.193131},{1.083110, 0.741315, 0.607160, 0.399249, -1.223962, 1.693242, 0.173693, 2.092256, -0.627612, -0.269874, 1.305670, -0.038505, -0.782861, -0.890849, -0.567807, 0.899665, -1.465509, 0.722999, 1.164278, 0.657993, -0.254148, -0.482168, 0.807729, -0.081320, 0.674078, -0.503661, 1.790514, -0.014891, -0.505417, 0.034912, 0.297819, -1.684096, -0.629606, 1.347394, 0.712693, 0.646645, 0.381208, -1.124679, 1.146806, -0.036445, -0.825404, 0.978204, -0.035869, 0.115380, -1.360013, 0.531307, -0.636983, 0.479939, -0.949687, -1.290943, -2.247886, 0.457918, 0.341813, -0.993057, 0.005262, -0.717169, 0.254664, -1.293516, -0.134251, -2.907002, -0.094573, -1.705844, -1.329746, 0.250586, -0.184743, 0.311674, -0.797677, 0.468149, 1.227226, 0.193190, 0.259738, 1.037777, 0.472580, 0.333170, 0.680849, 0.171109, 1.265749, 0.674203},{0.761990, -0.543077, 1.388873, -1.450733, 0.823415, 0.397585, -0.627295, -0.156746, -1.201932, -0.051000, -0.233062, -0.323391, 1.350916, 1.232915, 0.906099, -1.497365, 0.606865, -0.799301, -2.595280, -0.567831, 0.500679, 1.104127, 1.861857, -0.342369, 1.193676, -0.221733, 0.786486, -0.565909, 0.619112, 0.686185, -0.580748, 0.588663, 0.799956, 0.123909, 0.228937, 1.906350, 0.588913, -0.639518, 1.056510, 1.826273, 0.403646, -1.194768, 0.416860, -1.240334, 0.546697, -0.227159, 0.174398, 1.022959, -1.408084, 1.165361, -0.539006, 0.106513, -1.158660, -2.570185, 1.408190, -0.803102, 0.429939, -0.691877, 0.732648, -1.999509, 2.059417, -0.761195, -0.417110, 0.989903, -0.640547, 0.402369, -0.582007, -0.098804, -0.418397, -0.220393, 0.121299, -0.711592, 0.142533, 1.115023, 0.888230, 1.643998, 0.436412, 0.073658},{-0.255760, 0.011478, 0.052272, 0.005797, -1.080815, 0.240534, 0.416511, -0.242824, 1.232127, -1.336373, -0.813525, 0.010383, 2.134244, 0.262516, -2.137688, 0.593970, -1.333464, -0.591736, 0.712228, 1.906270, 0.434029, 0.146582, 1.961319, 1.325554, -0.248839, 0.518762, 0.902983, 0.206817, 0.969936, -0.046762, -0.360502, -0.597504, 0.427342, 1.858052, 0.201872, -1.847873, -1.209370, 0.459258, -0.496626, -0.504184, -0.237348, -1.299638, -1.487073, 1.891045, 1.373455, -0.332226, -2.620742, 0.170096, -0.108586, 0.396025, -1.366365, -1.476949, 0.534433, -1.126890, 0.040885, 1.949012, -1.009660, -1.088223, -0.509438, -0.829233, 0.466423, 0.851914, -0.591981, 0.000918, -0.513560, -0.153404, -1.134202, 0.117258, 0.756559, 0.284446, -0.921462, 1.442510, 1.447694, -0.096700, -0.362750, 0.919035, -0.025527, 1.258252},{-0.109050, 1.938348, -0.657848, 0.504802, -0.672741, -0.089043, 0.258005, 1.965504, 0.250108, 0.981429, 0.195638, -0.019995, 0.503242, -0.400262, 0.383406, -0.384482, -0.232367, 0.588552, -0.197658, 1.415161, 1.485072, -0.430796, -1.637941, 0.682747, -0.746829, -0.110944, -1.002512, 0.179445, -0.454797, 0.567930, 1.715043, 1.301987, -0.176386, -0.080130, -0.939093, -2.536473, -0.010549, -0.112720, 1.093265, -0.559401, 0.100391, 1.235275, 2.661838, -0.280229, 1.039767, 0.355360, -1.611775, -2.128664, 1.805467, -0.563264, 0.758777, 0.357148, 0.702648, -0.471081, -1.655229, 0.114028, -0.516617, -0.334685, -0.346543, 1.676669, -1.211023, -0.302663, 0.252633, 1.239742, 0.890764, 1.387584, -0.646376, 0.246742, 1.091589, 0.175966, 0.301833, -0.885780, -1.790780, -0.375413, -0.476393, -1.216687, -1.026443, 0.920215},{-0.447168, -1.518556, 1.280472, 0.421570, -1.691164, -0.047890, 0.724334, 0.487817, 0.823436, -2.155789, -1.040839, 0.646816, -0.726908, 0.926050, -1.152341, -1.374676, -0.099436, -0.133804, -0.052254, 0.893382, 0.264309, 1.303001, -0.663506, 1.434087, -0.886993, -0.533336, 0.889734, 0.874092, -0.111185, 1.046767, -1.031741, -0.258840, -0.511078, 1.504459, -0.551721, 1.932725, -1.763975, -0.784859, 1.015785, 1.153976, 1.217049, 0.214723, 0.223089, 0.512420, 3.043021, 0.400170, -0.152086, -0.213791, -0.308661, -1.235813, 0.143644, 0.494129, 0.223374, 0.901406, -0.707787, -1.498479, -0.764699, -0.539990, 0.931576, -0.314382, 0.964889, 1.374649, -0.321599, 0.146514, -1.484192, 0.306785, -2.170941, 0.076077, -1.379416, 0.152023, -0.432607, -0.280086, 1.614816, 1.221139, 1.047003, 2.140734, -0.512230, -1.533467},{-0.127601, -1.134035, -0.722974, 0.994002, 1.148545, 1.170689, -0.430313, -0.700049, 0.229229, -0.593369, -0.571954, -0.962420, -1.101738, -1.415757, 0.687048, 1.267085, -0.219932, 0.503954, 1.132380, -0.794581, -1.936304, -0.108686, 0.330609, 2.401318, 0.151221, 0.684970, 0.297711, 0.974990, 0.416035, 2.021439, -0.579971, 0.238844, -1.249824, 2.333574, -1.041551, -1.019030, -0.129162, 0.414458, -0.976343, -0.374113, -0.455509, 0.494641, -0.414933, 0.211601, -0.364690, -1.787513, 1.630657, -0.834544, -0.287473, 0.898755, 0.205023, 0.227684, 0.520567, -1.414342, 0.320768, 0.547729, 0.067188, 0.062147, 0.614763, -1.686645, -0.352573, -1.243333, -0.378232, -1.764911, 1.246913, -1.319531, -1.326628, 2.084296, 1.624954, 0.069881, -0.239976, 0.842944, -1.121524, -1.730786, -0.237744, -1.611058, -0.009542, -0.191339},{-0.940482, -1.421047, -0.755818, 0.465821, 0.499530, -0.439450, -1.216761, 0.310937, 0.386655, 0.616080, -1.952013, 0.319447, -0.834209, 2.096177, 0.421133, 0.605047, -0.540998, 0.708376, -0.657572, -0.588824, 0.343767, 1.186423, -0.190147, -0.256938, -1.184837, 0.832144, -0.231899, -0.829910, -0.626736, 0.782332, 0.484224, -0.112652, -0.487924, -1.437532, -1.097941, 0.260426, -2.221421, -1.675220, -1.322241, 0.109259, -0.354711, 0.814109, 0.687723, 0.672019, -0.417152, -0.991476, 0.582307, 1.183331, 0.232927, 1.467196, -0.965235, -2.305534, 0.462584, -0.347831, 0.561571, -0.286629, 0.734065, -1.425297, 0.743706, -1.780422, 0.598872, 1.439310, -0.316338, 0.677689, -0.381159, 2.619884, -0.769734, -0.319883, -2.772035, 2.124498, -0.755745, 1.702172, -0.127382, 1.279244, -0.098157, -0.572377, 1.006535, -0.613937},{0.725193, -0.835523, -0.315890, -0.077910, -0.778143, 0.143913, 0.829790, 0.217173, -0.703357, 0.941598, -0.192573, 0.191590, 0.022025, -1.510527, 0.078447, 0.631025, 1.607559, 0.190338, -0.069984, -0.575772, -0.155826, 1.401331, -1.360590, -0.838943, 0.760610, -0.688012, 1.346930, 1.482074, 0.314962, -0.970328, 0.640871, 0.354874, 1.229293, 0.939516, 1.683913, -0.485871, -1.051576, -0.356086, 0.115563, -0.259948, -0.437589, -0.899455, -0.220163, -0.108558, -1.368181, 0.160281, -0.928569, 1.530532, 0.145863, 1.043900, -1.231675, -1.218776, 0.867178, -0.362722, 1.067388, -1.198107, -0.704406, 1.620299, 0.568174, 0.177328, -1.245520, -0.954935, -0.313330, -0.672865, -0.779852, -0.031423, -1.205980, -0.412545, 1.330227, -1.093471, 0.799325, -0.957458, -1.710781, 0.302143, -0.557440, -0.850881, -0.639038, 0.675428},{1.346566, 0.506340, 0.735931, -0.454156, 0.586640, -1.481228, 1.515577, -0.423374, -0.420629, -0.929915, 0.098521, -0.356770, -2.003702, -0.189575, -0.338891, -0.585327, 0.690864, -0.222288, -0.693518, -0.952808, 0.814820, 0.451253, 0.658792, 0.028992, 0.836707, -0.313884, 0.360564, 0.006148, 0.513546, -0.378495, -0.968877, 0.207856, -0.839021, 0.084800, 0.448047, -0.456287, 0.620406, 0.114285, -0.441226, -1.062762, -0.079891, 0.118294, -0.631666, -0.310739, 2.175072, 0.087641, -0.827035, -0.829529, -0.337558, -1.121985, -2.203677, -0.167964, -0.452215, -0.260591, -1.829317, 1.184767, 0.621552, -0.060925, 1.062266, -0.540687, -1.989212, 0.715614, -0.079674, 0.538547, -0.732507, 0.409889, -0.442085, -2.025331, -0.357709, -0.166322, -0.203492, 0.636744, -0.888829, -0.917884, 0.557987, -1.800618, 0.369528, 0.159924},{-0.271439, -1.177497, 1.258475, 0.229565, 0.031356, -0.489368, 0.491775, -0.605701, -0.408573, -2.340011, -1.168144, -0.206975, 0.820928, -0.460326, -0.086389, -0.227354, -1.416498, -1.133090, -1.304221, 0.304628, 1.734739, -0.532030, -1.308355, 0.156002, -1.075511, -0.046775, 0.962755, 1.040834, -1.036194, -0.618129, -1.686578, 1.533166, -0.997218, -0.262343, -0.012110, 0.316583, -2.305009, -1.149625, -0.770896, 0.514613, -0.422411, -0.663850, -2.429819, 0.844950, -0.468146, -0.425911, -1.623791, -0.661524, -0.401977, -0.362584, 1.056710, 0.110448, -1.394161, 1.722991, -1.030829, 1.658128, 0.684504, -0.854804, -0.476318, -0.573135, 0.625853, -0.698696, -0.366006, -1.802744, 1.745772, -0.320640, 0.267590, -1.952097, 0.291629, -1.226236, 1.598937, 0.023128, 2.039528, 0.894081, -0.093514, -1.025468, -1.420437, 0.902114},{-0.738058, 0.542432, -2.677610, -0.144295, -0.596218, -1.140131, 0.169617, 0.734907, 2.236896, -0.655565, 1.359209, 0.172154, -0.418186, -0.035544, -0.574313, -0.619466, -0.900248, 0.795843, 2.080803, 1.807957, -0.440496, 1.209880, -1.473209, 0.053282, -0.251709, 1.027110, 0.670172, -0.751484, 0.347862, -1.545059, -2.218141, -1.287425, 0.651430, -2.546368, 1.058009, -0.916036, -0.888195, 0.649862, -0.284743, 0.780456, 0.324955, 0.625675, 0.180347, 1.167379, -1.319549, -1.531991, 0.791040, 0.736741, -0.038658, -0.659680, -0.375730, -1.297033, 1.821860, 0.047057, -0.520331, -0.308476, 0.185421, 0.078488, 1.599063, -0.128801, 0.477380, -0.252318, -1.486265, 0.286605, -1.295040, -0.269705, 2.311628, -1.183383, 0.424716, 0.514334, -1.371179, 0.351588, -0.619587, -0.367301, 0.497071, 0.367600, 1.310823, 0.397700},{-1.958520, -0.717709, -1.171817, 0.355416, -0.141966, 0.352455, 0.638506, -0.619606, -1.324289, -0.550029, -1.980426, 2.111652, -0.013769, 0.045823, -0.511262, -0.189947, 0.979636, 0.588419, 0.194671, 0.985015, 0.490543, -1.148837, 0.190985, -0.312616, -0.895753, -2.349217, 0.986655, -1.707032, 1.074579, 0.334594, -3.316829, 1.501779, 0.424039, -0.740652, -0.925067, 0.899302, 0.071318, 0.744669, -0.816219, -0.706832, -0.566865, 0.514438, -0.378083, 0.006550, -0.687112, 0.858503, -0.715379, -0.697571, 0.606824, -0.350115, -0.341787, 0.466277, 1.055243, 0.327485, -0.819248, 0.083459, -0.953867, 0.372687, -1.886777, 0.854909, -1.089322, 1.120044, 0.044509, -1.423461, -0.372982, 0.402456, 0.567438, 1.299382, -0.990886, -2.021819, 1.311177, 0.230173, -0.375778, 1.401462, 1.575990, 0.281884, -0.525987, -0.172917},{-0.999616, 1.064651, 0.843978, 0.936882, -1.055124, -1.315022, -0.196772, 0.327549, -0.040955, -1.108656, -2.514788, 0.727424, -0.501395, 0.358746, -0.158493, -0.096312, -1.209624, 1.867590, 0.761297, 0.596719, 0.277820, 0.267667, -0.062796, 0.761696, -0.498947, 1.888853, -1.397603, -1.544485, -3.306840, 0.338288, -1.571816, 0.200694, 0.071575, -2.193128, -1.534082, 1.762900, -0.146076, -0.825459, -0.272473, -0.699792, -0.745298, 0.055503, 0.352502, 2.596911, 0.146001, -0.743520, -1.453986, 1.070123, 1.965715, 0.227722, -0.158092, -0.368488, -0.035579, 1.365520, 0.601857, 0.670945, -1.888203, 0.703599, -0.299598, 0.318534, -1.523195, -0.941274, -0.554680, -0.576468, 1.881036, 0.140285, 0.298787, -0.961603, 0.354472, -0.466463, 0.119878, -0.330207, 0.074230, -2.119711, -0.032111, -1.299969, -0.904923, 0.443146},{0.552711, -1.077735, 1.358275, 1.403571, 0.288670, -0.827845, -0.790949, 0.701827, 0.376010, -0.275902, -0.222148, -0.058073, -1.073476, 1.487807, -0.614454, -0.902243, -0.154238, 2.142643, 1.025216, -0.409343, -1.753016, -0.701766, -0.409775, -0.381121, 1.298688, 1.080772, -0.046982, 0.543362, -0.335592, 1.479919, -0.490373, -0.176486, -0.163391, -0.434080, 0.264385, 0.073258, 0.688113, 0.763411, -0.638415, -0.979679, -0.086165, -0.488615, -1.310293, -0.483033, 0.499326, 2.081983, -0.962028, 0.298420, 0.467348, -0.001888, -1.171042, -0.584536, 1.175027, -0.381281, -0.839108, 0.940148, -1.354304, 0.331805, 0.332418, -0.224525, -0.109827, 0.612611, -1.117494, 1.176553, -0.130318, -0.293240, 0.104755, 0.692245, -0.461509, 0.744299, -1.258518, -1.097518, 0.583736, 0.632375, 0.058908, -0.003606, 0.362553, -1.494420},{0.262370, 1.898402, 1.368741, 0.250100, 1.633013, 0.168121, -0.252962, 0.462100, 0.723120, 1.047937, -1.122461, 0.872689, -1.016910, -0.144541, -0.835989, -0.021586, 0.603616, 0.260808, 1.843376, -0.897679, -0.993081, -1.072210, 1.023979, 1.826952, 0.017678, -0.831500, -0.417019, -0.488535, 0.254613, 0.365040, 1.279495, -1.016236, -0.034965, -0.932250, 0.438599, 0.739625, 0.579669, 0.353070, -0.636756, 0.208879, -0.034960, -0.946494, 0.366568, 1.341838, 0.612447, -1.511880, 0.724480, 0.994882, 0.180657, 1.214859, -1.680829, 0.480791, -0.676451, 0.162502, -0.645099, -1.129269, 1.304667, 1.482679, -0.985584, -0.669276, -0.558989, -0.937334, -1.096339, 0.051078, -0.540149, -0.308133, 0.491328, -0.383448, -0.070225, -0.837970, -0.457888, 0.179421, 1.017272, -1.948588, -0.196953, 0.748680, 0.143755, 0.201721},{-1.081479, 0.275967, 0.324848, 0.519656, 0.073649, 0.788647, 1.859026, -0.305194, -1.915553, -2.020347, 1.909658, 0.016461, -1.089362, -0.337371, 0.720261, -0.573178, 0.565670, -0.529182, 0.441374, 0.408305, 1.487696, 0.513299, -0.545869, 0.935812, -0.505486, -1.077021, 2.228465, 0.645203, 1.104452, -0.179766, 1.069637, -1.561278, 0.202866, -1.427349, -0.718834, 0.490023, 0.183462, 0.409034, 1.300285, -2.208790, -0.973841, -0.739022, -0.752884, 0.165161, -1.471877, 0.597276, -0.475406, 0.818405, -2.196967, -0.047580, 1.147601, -1.799839, -0.162808, 0.099865, -0.152405, -2.026078, 0.426200, -2.202248, -0.339399, -0.741895, 0.275893, -1.605448, 0.887002, -0.273318, 0.789663, 0.645952, -0.888444, -0.099564, -0.350763, -0.930159, 0.542152, 0.407349, -0.444782, 2.519573, 0.002872, -0.274331, -2.510642, -0.306660},{-0.839562, 0.630816, -0.470257, 1.261986, -2.487499, -0.676687, -0.165925, 0.134775, 1.208222, -1.379587, 0.455998, -0.157665, 0.487401, 1.684471, 0.509178, -1.396068, 1.311631, 0.813762, -0.316267, 1.166882, -1.090835, -0.592462, -0.823138, -0.595837, -0.102182, -0.267302, -0.313011, 0.829956, 0.874429, -0.360674, -1.585247, -0.656238, -0.485755, -0.916042, 0.601269, 0.358901, 0.060681, 1.060880, -0.286862, -0.405941, -1.055404, -0.651287, -0.140952, -0.043970, 0.660602, 0.644825, 0.262158, 1.204751, -0.354604, -0.618114, 0.494230, 0.714982, -0.423410, 1.764719, 0.381570, 0.095914, -1.587018, 0.939668, -0.194939, -0.775753, 1.042793, 0.293049, -0.144407, -0.765936, -0.661068, 0.286814, -0.588962, 1.142449, -0.201705, 1.328934, 0.426179, 0.205622, -0.017680, -2.448132, 0.894739, -0.188216, 0.536452, -0.575888},{1.573111, 1.103003, -0.239516, -0.558666, -1.115647, -1.531502, -0.340378, -1.664179, -2.574809, -0.197890, 0.530377, -0.516619, 0.722284, -1.829841, 0.305559, 0.910311, -2.567888, -0.348994, -1.096907, 0.615958, -1.210764, -0.199250, 0.625807, -1.416561, 0.561381, -0.031474, 1.306914, -1.098480, -0.604448, 0.288308, -1.205504, -0.550502, -0.614260, -0.028552, -0.334731, -1.155245, 0.458355, 1.386864, -0.652898, 0.195461, -0.833353, 0.585466, -0.702099, -0.685787, 0.718303, -0.790603, -0.365539, -0.768735, 0.546470, 0.499754, 0.728054, 0.709615, 0.877427, -0.209258, -1.356104, 1.551839, -0.766093, -1.506094, -0.702346, 0.512249, 0.532128, -0.276089, 0.861472, 0.948775, -0.788291, 0.846167, -2.507410, -0.673598, -0.583647, -0.763417, -0.039715, -0.239711, 0.898685, -0.704271, -0.025791, -0.247470, 2.778004, -0.457910},{-0.722144, 0.758192, -1.294854, 0.716921, -0.822247, -1.122573, -0.405511, -0.158588, 0.826207, 1.676083, -0.190258, -0.120187, -0.390736, 0.892335, 0.406197, -0.096903, 0.009100, -0.263564, 0.367019, -0.072933, -0.866145, 0.385991, 0.602993, 1.341717, -0.960933, -1.156628, -0.916280, -0.636711, 0.112800, -0.412460, -0.417378, -0.341818, -2.651617, 2.334149, -0.665067, 2.477956, 0.797676, -0.250883, -2.518536, -1.144587, -0.195227, 0.389237, -1.147881, -1.535489, -0.353655, -0.325096, -0.930404, 0.199708, -0.651036, -0.876854, 0.600219, -0.759397, -0.544158, 1.230118, -0.595047, 0.080837, -0.137066, -0.687791, -0.345986, 1.407840, 2.105005, 0.109285, -0.287001, 1.155796, -1.224472, 1.177706, -0.766494, -0.653371, 0.779683, 0.059888, -0.515975, 0.659334, 1.265849, 0.511995, -0.305340, -0.323221, 0.481479, 1.516431},{-0.314929, 0.255771, 1.236148, 1.732563, -0.230966, -3.228101, -0.632850, 0.559231, 0.078895, -0.258800, -1.849135, 0.102009, 0.111163, 0.406623, 0.545122, -0.988055, 0.543502, -0.841957, 0.409454, 1.837945, -0.619948, -2.109942, 0.060931, -1.365988, 0.989232, 0.629562, -0.200074, 1.988509, -1.485965, -0.334757, -0.266888, 0.002707, -0.480876, 1.070730, 1.605639, -1.193133, -1.023620, 1.371974, -2.061725, -0.719345, -0.619368, -0.528336, -2.378844, -0.442942, -0.737444, -0.393124, -1.599917, 1.594044, 1.023028, -0.424066, -0.477814, 0.144429, 1.027837, 0.689537, -0.347069, -1.539783, -0.349043, -0.126994, -0.791380, -0.632624, 0.303202, 0.412073, 0.049787, -0.976050, 0.301559, 0.223961, 0.537366, -1.169711, 1.313884, -0.453552, -0.552639, -0.062238, 0.106840, -0.226945, -0.227320, -0.192554, -1.862023, 0.242231},{1.266495, 0.768835, 1.230482, -0.238316, -0.031618, -2.048987, 0.376217, 0.082829, -0.107215, -1.246878, -0.358326, -0.446471, 0.019651, -0.125500, -2.456930, -0.472169, 1.053132, 0.595403, 0.388296, -1.217788, 1.457182, 0.414084, 1.102025, 0.922185, 0.696117, -0.346492, -0.504053, 0.337420, 1.526972, 0.608019, 0.141150, 0.906588, -0.613930, -0.117565, 0.223610, 0.621667, 0.615477, 1.236178, 0.654379, 0.657624, -2.048434, 0.246344, 0.531561, -0.631127, -0.452277, -0.013417, -0.522276, 0.238875, 2.149447, -0.310144, -1.051568, -0.829461, -1.270201, -0.317670, 0.359778, -0.158279, -0.742145, 1.185935, 1.352327, -0.786347, -0.244779, 0.414909, -0.537385, -0.124986, 1.453180, -0.091811, -1.826038, -0.522695, -0.162724, -0.161241, 0.857209, -0.236324, 2.762155, -0.844671, 1.246708, -0.234506, -0.308646, 1.652558},{-0.625964, 0.962176, -1.946824, 0.259219, -1.460579, -0.593788, 0.258055, -0.175046, -0.900794, -0.997321, 0.489052, -0.302165, -1.016227, -0.289142, -0.393596, 0.021506, 0.395896, -0.076147, -1.305665, -1.063460, -0.650030, -0.096538, 1.003304, -1.409987, -0.039501, 0.160228, -0.034847, 0.418348, 2.137920, 0.057802, -0.243134, -0.322338, 1.275111, -1.092052, -0.638543, 0.221411, -1.205282, -0.677935, 0.544389, 1.137973, -0.823691, 0.442079, -0.216811, -1.652714, 1.086747, 1.446257, -0.385044, -1.124456, -0.920592, -1.577564, 0.317506, 0.638788, 1.157309, 1.523036, -0.365945, 0.459634, -1.024942, -0.973368, 1.161691, -0.165391, 0.110933, 0.228276, 0.769161, 0.772405, -0.273517, -1.113316, 1.139897, 0.169909, -0.597986, -0.561142, 0.210349, -0.390917, 0.457547, -0.694451, -2.757201, -1.328620, 0.624002, -0.326416},{-0.120062, 0.535809, 0.556827, -0.769612, 0.211164, 0.013930, -1.192592, -0.502184, -0.992446, 0.225602, -0.281034, 1.458880, -0.158603, 1.006106, -0.145004, -1.137265, -0.224050, 0.914352, 0.025552, -0.940041, 0.949463, -0.762583, 1.177490, -1.156654, -1.313691, 0.507751, -0.909614, 0.210317, 0.979404, -0.774954, 0.158843, 0.945978, -1.336344, 0.989690, 0.533452, -0.792014, 2.369850, 1.062521, -1.100723, -0.371329, 0.672411, -0.045119, 0.948052, -0.002878, 1.548331, -0.233810, 1.310410, -0.791488, 1.157659, 1.372747, -0.143770, 1.285898, 0.149353, -1.820634, -0.116960, 1.419005, -0.385422, 1.833303, -1.243507, -1.273309, -0.422284, -1.130681, -0.125034, 0.028739, -0.331075, 1.459767, -0.549783, -0.894278, 0.052946, -1.949180, 1.942622, -0.550753, -0.485706, -1.343919, -1.546911, -0.903799, -0.233604, -0.305626},{1.360286, 0.445567, 0.144255, 1.132086, -1.146456, -0.073314, 0.454685, 0.173187, 0.410982, -0.741686, -1.199323, -0.826735, -0.490885, 1.225958, 0.381392, -1.235843, -0.110886, -0.922804, 0.288586, 0.135580, 0.554746, 0.460545, 0.955044, -3.253512, 0.043988, 0.762353, 1.183939, 0.266135, 2.066475, -0.377526, -0.610010, -1.242795, -2.102800, -0.720151, -0.982016, -0.007907, -0.601851, -0.627005, -0.216410, -0.198438, 1.540466, 0.939222, -0.642586, 0.241188, 0.095531, -0.986479, 1.062374, -0.379553, -0.458391, 1.543668, -1.379857, -1.160689, -1.262033, 0.127459, 0.826827, -0.701884, 0.608824, 0.955915, 0.587391, 0.896440, -0.429927, 0.266245, 0.785386, -1.416450, 0.417083, 0.617285, 0.515064, -0.343527, 0.571479, -0.522476, -0.190601, 1.800426, -0.694447, -1.196724, -1.271288, -0.889976, -1.033068, -1.891195},{-0.692729, -1.260755, 0.528932, -0.947354, -0.743254, -2.325050, 0.412332, 0.758725, 1.061363, -0.540945, -1.574109, 0.369595, 0.019815, 0.269684, -1.667687, -1.382761, -0.648659, -1.244733, 0.019154, 1.099422, 0.181740, -1.453816, 1.003136, -1.173705, -1.175417, 0.479611, 0.362281, -0.145359, 1.721048, -0.648697, 0.395815, -0.510808, 1.281148, 0.731878, -0.279839, -0.128384, 0.416432, -0.874578, -2.833537, -1.254016, -0.977079, 1.614286, 0.922440, 1.051221, 0.259594, 0.861627, 1.020003, -0.874784, -0.738773, 1.186475, 0.813695, -0.911018, -0.489627, -0.013063, 0.196334, -0.061598, -1.609193, -0.329622, -0.807819, 0.923564, -0.482569, -2.348413, 0.027364, -1.371113, -0.209289, 0.308652, 1.084025, -0.075557, -1.514531, 1.269420, -1.322801, -0.172341, -0.396622, 1.146586, 0.274398, 0.653687, -0.750363, 0.162350},{1.111887, 0.155293, -0.430214, -1.092320, 2.123627, 1.920514, -0.953455, 0.204961, 1.093812, -0.723095, 1.803361, 1.113026, 0.911201, 0.370271, -1.172094, -0.627512, -0.364669, 1.036273, -0.962321, 1.368211, 1.794379, 1.888893, 1.071310, 0.797472, -2.452569, -1.514693, -1.856935, -1.061418, 2.261357, 1.082808, 0.381693, -1.941533, 1.133052, -0.516072, 0.248652, -1.586938, -0.115618, 0.590629, -2.269992, 0.641614, 1.182786, -0.544168, 1.211606, 0.039637, 0.371699, -0.225987, 1.232860, -1.247683, 0.898212, 0.916964, -2.033088, -0.574160, -0.204482, 0.689184, 0.279705, -0.622467, -1.780325, -0.281056, 1.030918, -0.134983, 1.177255, -0.983001, -1.255638, -0.680469, 0.477446, -0.610151, 0.126809, -1.247808, 1.656000, -1.271459, -1.581652, 2.000761, 1.245245, 0.927004, 0.651661, 2.050053, 0.228630, 1.365335},{0.091922, 0.078730, 0.488965, -0.029268, 1.275536, -0.685524, 0.843817, 0.506854, -0.933862, -2.977556, -0.173956, 0.061078, -0.501411, -1.254773, -0.684833, -0.864127, 0.511783, 1.517169, 0.103365, 2.127656, 0.540870, -0.145558, 0.882199, 0.523474, 1.063273, -2.010309, 1.940816, -0.112906, 0.572486, 0.089592, 1.969280, 0.564364, 0.205225, -0.398707, -0.812100, 0.238145, 0.828999, -1.530349, 0.995856, 0.942996, -1.804240, -1.475839, -0.607001, 1.163178, -1.560925, -0.276317, 0.908656, 0.461987, 1.021770, -1.513913, 0.077877, 0.075176, 1.044678, 0.669782, -0.458328, -1.867334, -0.932743, 1.358092, 0.673275, 0.317662, -0.001415, -1.295187, 0.203155, -0.749374, 1.069556, -0.146572, -0.139220, 0.594439, 1.193096, -0.003954, -0.033623, -0.281516, -0.731268, 1.682827, -1.133955, 1.287879, -0.540605, 0.734071},{0.390526, 0.790738, 0.003295, -0.202258, 0.478541, 0.798976, 1.081239, -0.617409, 2.141517, -0.860964, 0.403443, -1.321005, -1.376278, 0.846828, -1.163870, -0.248480, 0.218574, -1.762635, 0.579077, 0.157097, -0.690275, 0.183683, 0.606253, -1.041801, 0.307932, 0.750359, 0.527419, -1.976520, 1.364602, -0.773725, 0.297846, 0.671464, 1.089280, -0.315745, -0.139831, 3.024848, -0.134035, -2.289064, 0.524144, -0.007971, 0.819861, 0.198858, -0.922689, 1.372606, -0.315920, 0.274562, 0.072662, -0.682994, 2.527316, 2.119661, -0.133690, -0.069768, -0.239585, -0.488452, 0.131340, -1.293001, -0.096095, -0.631479, 0.321421, -2.962704, 0.797506, -0.270647, 1.361760, -0.017658, 0.755502, -1.184377, 2.316498, -0.588890, -0.136273, 0.699520, 0.282306, -0.242041, -0.597606, 0.310319, -0.026909, 0.168658, 0.722027, 0.451731},{0.084126, 0.937386, -0.332806, -0.159877, -0.350530, -1.784844, 0.426087, -0.399315, 0.248015, -0.841837, 1.288941, 0.143147, 0.125607, -0.520336, 0.144669, -1.113548, 0.241962, -0.649354, 0.702299, -0.463498, 0.413567, -0.096757, -0.058270, 1.373144, 0.488462, -0.118112, -0.570992, -0.200532, 0.970852, 1.526904, -0.214404, -1.844390, 1.217871, 0.333456, -0.751780, -1.910559, -0.666378, 0.186204, -0.160012, 0.654879, -0.851623, 0.473980, 0.033201, 0.696014, 0.459809, -0.434267, -0.374263, 0.600382, 0.317550, -1.902552, -2.641805, -1.284407, 0.304074, 0.147587, 0.670803, -0.378556, -0.794499, -0.884671, -1.587423, -0.842484, 0.037846, -0.132531, -0.966024, -0.409786, 0.247683, 1.182517, 1.200151, -0.876457, -0.619105, 1.585679, -0.088875, 0.022784, 0.033140, 0.220438, -0.017308, 0.703818, -0.515563, -0.864167},{1.379770, 1.098225, 0.221987, -0.890841, 1.148493, -0.749966, -1.053921, -0.431305, 0.950681, 1.276645, -0.565944, -2.471470, 0.525917, 0.012792, 1.205985, -0.453177, 1.025133, 1.189814, 1.168913, 1.416796, 0.868488, -1.089904, -0.890029, 1.606749, -0.887328, -2.383524, -0.598111, -1.074929, -0.867601, 0.047503, 0.166655, -0.176473, 0.638055, -0.102195, 0.278019, -0.808401, -0.425913, 0.447426, 0.927030, -0.488428, -0.935128, -0.151012, 0.588912, -0.258166, -1.997311, 0.049992, 2.318220, -0.743922, 1.640872, 1.938689, 0.912639, 0.897849, 1.040597, -0.032818, -0.378097, 0.148932, -0.111308, -0.213333, 0.669774, 0.322910, -0.824079, 0.166787, -0.889196, 0.043054, 0.827224, -0.971385, -1.449036, 0.193054, 0.109596, -0.632279, -0.713360, -0.124352, 0.295680, 0.275885, -0.388681, 0.740946, 0.995816, -1.121468},{2.058353, 0.002045, -0.636154, -1.197857, 0.286086, -0.310568, -0.736859, -0.253418, 0.100200, 0.675067, 1.095473, -1.230387, -0.107417, -1.777189, 1.220627, 1.103343, -0.361926, 0.194511, 1.892141, 1.542696, -2.225966, 0.516195, 0.373235, 0.162188, 0.292760, -0.697566, 0.373175, 1.050154, 0.566309, 0.507264, 0.548602, -0.398634, 0.077176, -1.193340, 0.424968, -0.611801, 0.684634, -0.771602, 0.622512, 0.159949, -1.307299, 1.294084, 1.179910, -0.318669, -0.537663, 1.713507, -0.995127, 0.749364, -0.223528, 0.054773, 1.753947, 2.000252, 0.311338, -0.246732, -0.529585, -0.504797, 0.429589, -2.078162, -0.052671, 2.373525, 0.388194, -1.264064, -0.600088, 1.060508, 0.502799, 0.497258, 0.914705, 0.414322, 0.576069, -0.319521, -1.271945, -0.206440, 0.301341, 0.691279, -0.689533, -0.985591, 1.283765, 0.984119},{-1.165150, 0.668395, -1.330075, -2.947709, 0.262072, 0.518430, -0.277614, -0.189821, 0.948169, 1.396416, 0.836907, 0.976397, 0.999643, -0.254564, 0.729440, -1.086652, 0.200855, 1.817339, -0.548443, -1.629819, 0.545128, 0.835435, 2.440574, -0.907223, -0.416500, 1.989061, -1.685992, 0.405024, 0.710946, -0.024965, -0.881495, 0.547953, 0.116496, 1.302422, 0.349649, -0.359449, -0.126631, -1.287177, 0.386884, 0.271496, -1.352018, -1.554748, -0.439240, 1.466576, 0.016287, 1.366329, 1.145342, -1.999869, -2.596632, -0.854055, 0.210192, -1.408036, -0.101621, -0.738744, 0.464767, -0.834802, 0.400853, -1.033214, -1.695642, 0.021091, -0.264093, 0.324429, 0.153423, -0.035588, 0.599491, 0.669000, 0.389889, 0.422894, 1.467767, -0.176826, -1.311993, 0.965991, 0.486571, 0.087171, -0.746368, 0.441893, -0.051168, -1.224564},{-1.010163, 1.596896, 0.218637, -0.082176, 0.265049, 2.184532, -1.561135, -1.170109, -1.231340, -0.228740, -0.608802, -0.035062, -1.480556, -0.241735, -1.414281, 1.184299, 0.839796, 0.135981, 1.147040, 0.071587, 0.581703, 1.488825, 2.357134, -1.106063, 0.494339, 1.504215, -0.209968, -0.291674, 0.352801, 0.133854, -1.157967, -1.210873, 0.334531, 0.125620, -0.126309, 1.619229, -0.574186, 0.181927, -1.370043, 0.586406, -0.873165, -1.148581, -1.499527, -1.016668, -0.958807, 0.985534, -0.411477, -1.069007, -0.181624, 0.260140, 0.693821, 0.555981, 0.378959, -2.068631, 1.368004, 0.662849, -0.604086, -1.507839, -0.715599, -0.866178, 0.692195, 0.390872, 0.079836, 0.409640, -1.830869, 0.384253, 1.307535, -0.958164, -0.559591, 0.038267, -0.551613, -0.708324, -0.869489, 0.135877, 0.246592, -0.250252, 0.537295, 0.746614},{-0.588845, -0.993993, -0.784777, 0.204450, -0.466998, 0.057323, 0.202513, 0.744584, 0.452302, 0.105861, -0.581819, -0.100138, 0.866402, -0.365796, -0.705584, -1.340669, 0.023731, 0.235303, 0.520558, 0.081390, -0.291353, -0.547140, -2.692196, -0.651833, 0.471629, -0.020740, 0.569772, 1.463679, -1.341717, 0.124011, -0.303119, -0.391393, 1.931509, -0.005647, -0.219061, -0.185515, 0.408394, -1.428534, -0.215054, -0.050489, 0.460966, 0.953754, -0.755645, -1.317497, -0.436685, -1.939755, 1.533910, -0.426948, -0.737990, -1.505792, 0.155427, -1.687610, -0.761245, 0.031330, 0.237791, 0.134994, 0.676218, -0.299436, -1.478395, -0.825899, -1.560852, 0.270718, -0.121389, -1.010512, -0.400731, -1.549108, -1.164828, -0.502473, 0.843016, -0.502033, -0.786782, 1.673197, 1.716211, -0.562577, -0.313380, 0.484558, -0.007512, 0.151506},{-0.211330, 0.274081, -0.480389, -0.685734, 1.173896, 1.053036, 0.812091, -0.038316, -1.650330, 1.028684, -2.067605, 0.008763, 1.724324, -0.348004, 0.323840, 0.316277, 0.077452, 0.840368, -0.057644, 0.153310, -1.267614, -0.230094, -0.279914, 2.243419, -0.955025, 0.177332, 0.533362, -0.202351, -0.210517, 0.063597, 2.143019, 0.203820, -0.789834, -0.105613, 0.702351, -0.677250, -1.390863, 0.649729, 0.267296, 1.076263, 0.943221, -0.608565, -0.560583, 2.437684, 0.223649, -0.255896, -0.696095, -0.005389, 0.593274, 0.138094, 0.137066, 0.659367, -0.111777, -0.346963, -1.250916, -0.780682, -1.191221, -0.270259, -1.194997, 0.998405, -0.406872, -0.667160, -0.720862, -2.736131, -1.533850, 0.264312, 0.090983, 1.987401, -0.044040, 0.302427, -0.848926, -0.919445, -0.163758, -0.185043, 0.065144, 0.720269, 0.515682, -0.436974},{1.678035, 1.944995, 0.099834, -0.675010, 0.814863, 0.985479, -2.685395, 0.290625, -1.500154, -1.325766, 0.505972, 0.315658, 1.210893, -1.167593, -0.807330, 1.723073, -1.283693, -1.103459, -0.709469, 0.506182, 1.403526, 1.285711, 0.467035, 1.692889, -0.059732, -0.035817, 0.605659, -0.341521, -1.276601, 0.463917, -0.390097, -1.093092, 0.619204, 0.244128, -0.413710, 0.509947, 2.450338, -0.385298, 1.053688, 0.170588, 1.053051, 0.283689, -0.653461, -1.491133, -1.278483, -0.065763, -1.149426, -1.549802, 0.640778, -0.106895, -0.328355, -1.280302, -1.387189, -0.272391, -0.784788, -1.117794, 0.411214, -0.137077, -0.639126, 0.079050, -0.557425, -2.351156, 1.695245, -0.427497, -0.119723, -0.094491, 0.619569, 0.714385, 1.559672, 0.532543, 0.703889, -0.757244, 0.355058, 0.721426, -0.745742, 1.757114, 1.947683, 0.504677},{0.103356, 0.250986, -0.462191, -1.545079, 0.836031, 1.248855, 0.068962, 1.414231, -0.338191, -2.171448, 1.343934, 0.403979, 1.579563, -0.537125, 1.506762, -1.122718, -1.968999, -0.068675, 0.659893, -2.017806, -0.548875, 1.905979, -0.809364, 0.258298, 0.183954, -0.450043, -0.027458, -0.760412, 0.833990, 0.725608, 0.786784, -0.416870, 1.188095, -0.727990, -0.202746, -1.281698, 0.300776, 0.411315, 0.339234, 0.466669, 0.965966, 2.126491, -0.126121, 0.298104, 0.895276, 2.685228, -1.173901, -1.165343, 0.029790, -0.841621, -0.479185, -1.458308, -0.095647, -1.018944, -0.728009, 1.365187, -0.304572, -1.818196, 0.180286, 1.058872, 0.484421, 1.506472, 1.808995, 1.632735, -0.666611, -0.150869, -0.226475, -0.670790, -1.769920, 0.607862, 1.257444, -0.388292, 0.642150, -2.430091, 0.291452, -0.097899, -0.419658, 0.263325},{0.538312, -2.254816, -0.666226, 1.456937, -0.578543, -1.014143, 0.176091, -0.087864, 0.705424, -0.345782, -1.241842, -0.966889, -0.632488, -0.959258, 1.042891, 2.264944, 1.704111, 0.625756, -0.794863, -0.623433, 0.720841, 1.192014, 0.278952, -0.837809, -0.380973, 1.348509, 0.141767, -2.871732, -0.657542, -0.756635, 1.056667, -0.224620, 1.325113, -0.132884, 0.161064, -0.480004, -0.691418, -0.348104, 0.191675, -0.596890, 0.251701, 0.026797, 1.870686, -0.865382, 0.368639, 0.282984, -0.169927, -0.141409, -0.106800, -1.155502, 1.735127, -0.190473, 0.789665, 0.436521, -0.492175, 0.349310, 0.832439, -0.881328, -0.925396, -0.730516, -0.136580, -0.295903, 0.382923, -0.569868, -0.910425, -1.096051, -0.248316, 1.071685, -2.592246, -0.299593, -0.768257, 0.387646, -0.264635, -1.412337, -0.235778, 0.169444, -0.164573, 1.217537},{-1.390302, -0.623801, 0.733830, 0.591174, 0.555514, -1.099097, -1.437202, -1.119063, -0.584934, -0.908827, -0.269966, -0.982433, -0.290129, -2.486664, 0.265781, 0.590588, 1.337360, -0.272460, 0.882156, -0.237978, -1.332636, 0.410535, 1.795070, 0.822430, 0.434950, 0.386023, 1.407960, 1.273789, -1.340610, 0.803073, -0.676101, 0.482698, -0.314037, -0.925326, 0.861170, -0.589488, 0.659108, -0.564905, -1.060713, -0.405147, -0.053182, -0.516548, 0.238692, -0.240198, -1.393192, -3.129962, -0.604728, -0.314577, -0.154090, 1.473494, 0.288291, 1.058448, -0.045108, -0.460269, -0.142953, 1.875606, -0.160605, -0.437644, 0.316763, 0.439976, 1.197908, -1.020138, 0.351965, -0.134951, -0.789894, -0.680982, 0.267628, 1.459961, -0.256929, -1.096644, -0.228153, -0.643411, -0.240130, 1.810500, -0.394275, -0.068749, -0.491878, 0.924085},{-0.502341, 0.171963, 1.144175, -0.438921, 0.046024, -0.336767, 0.408465, -0.420141, -1.138631, 1.142901, 1.593172, -0.465449, -0.060885, 0.589949, 1.151035, 0.377015, -1.754301, -0.408340, 2.115331, -0.119819, 0.940555, 1.197302, 1.214014, -0.955287, 0.006553, 0.843281, -0.775741, 0.833216, 1.007355, 0.595047, 1.750483, 0.863050, 1.460584, 1.444377, 0.261158, 0.526251, 0.769751, 1.167703, -1.368880, -0.228173, 0.445539, -0.496363, -0.184984, 0.896062, -0.966307, -0.433824, 0.996643, -0.727143, -1.435599, 0.028845, -0.886545, -0.309731, 0.236678, 0.233911, -0.468079, 2.445384, -1.496716, 0.764405, 0.546025, -1.707396, 1.832166, 1.201517, 1.939624, -0.583731, 1.387933, 0.755804, 0.284959, -0.138841, -0.965578, 1.414609, -0.866013, -1.040418, 1.326025, 0.488948, 0.045942, 0.843122, -1.192533, -0.398194},{-0.124374, -0.236809, -1.267944, -0.766361, -0.163251, 1.834185, -2.544336, -2.028504, -1.113945, -0.172356, 0.652257, -0.091882, 0.751379, 1.130348, 0.639996, -1.057684, 1.969927, 0.206349, 0.399585, 1.694937, -0.438667, -1.206150, -0.823802, -0.437998, -0.197442, -0.906807, 0.104568, -0.255221, 0.118103, 1.746512, -1.188471, 2.588982, -0.408153, 1.689214, -0.253510, -0.257493, 2.364875, 0.242653, -0.425948, 0.455488, 0.184870, 0.563926, -1.194208, -1.041866, 1.206012, 0.288272, -0.354521, -1.846382, 0.197138, -1.033541, 1.054135, 0.219236, -0.392940, -0.434446, -1.744471, -0.463845, -0.440750, -1.108951, -0.200155, 0.601327, -2.359205, 0.179095, -0.112841, 0.418655, -0.372836, 0.991747, -1.209958, 0.342657, 1.484107, 0.657787, 0.360205, -0.446304, -0.247045, -0.549394, -2.293963, -1.183364, -1.023323, -1.035266},{1.124647, 1.140102, -0.224188, -1.832788, -1.057957, 0.614306, 1.259195, 1.126600, 0.570288, -0.143851, 0.310042, 2.205801, -0.264013, -0.004110, -2.098080, 0.673108, -1.684643, 1.504754, 1.350743, -1.027876, -0.280229, 0.143157, -0.812902, -1.286451, 0.456649, 1.648471, -1.052437, 1.859941, -1.887895, 0.948496, 0.087471, 0.099838, 0.363780, 3.311026, -1.085180, 0.577411, 0.476414, -1.290963, -0.912495, 1.676144, -1.698571, -0.305883, -0.278542, -1.012336, -0.496284, 1.456735, 0.165202, -0.448026, 1.163509, 0.554249, -1.154844, -1.694391, 1.528483, 0.849920, 0.114124, -0.350550, 1.349422, -1.462004, 0.067945, 0.294670, -0.548293, -0.103729, 1.344127, 0.048514, 0.115710, 0.960820, -0.676470, -1.411237, -0.034694, -1.231929, -1.823390, 1.201573, 0.710617, 0.274801, -0.518533, 0.460173, -2.148223, 0.977564},{-0.916147, -0.413193, -1.897249, -1.814386, 1.134101, 1.476204, 0.125766, -0.693986, 0.561750, 1.630658, 0.175484, 1.381859, 0.626598, -2.040813, 1.069944, -1.720343, 0.239363, 0.252649, -0.597700, 0.868079, 0.278462, -0.397429, 1.214716, -0.903994, -0.804887, -1.163986, 0.261286, 0.254459, -0.913122, -0.326089, -0.270844, 0.856343, 0.057366, -1.883865, 2.672897, 2.814798, -1.505744, -0.312933, 1.725997, 0.119226, 0.380484, 1.155660, 0.445301, -1.274264, -0.888673, -1.779769, -0.246711, -0.608266, 0.665000, 1.829311, -0.910298, -0.177577, -0.456278, -0.998574, -1.427028, 0.301312, 0.643529, -0.945095, -0.329446, -1.274001, -3.001239, -1.293348, -0.282349, 0.357909, -1.204790, -0.479854, -0.090476, 0.546355, 0.710847, -0.664090, -0.529013, 2.607946, -0.604772, -0.016828, -0.963981, -0.955162, 0.090848, -0.731392},{-0.145823, -1.027807, 0.574253, -0.612613, 1.435655, -2.594311, -0.921173, -0.942007, -0.793026, -0.244735, 0.568481, -0.469855, -0.724759, 0.721664, -0.565117, -0.638403, -0.688485, -0.249652, -0.429621, -1.459367, 0.187946, -0.960022, -0.529663, -0.493443, -0.354074, -0.508129, 1.322334, 0.617039, 0.565133, -0.153399, 1.628440, -0.833567, 0.379901, 0.413198, 1.029457, -2.237693, -1.444389, 0.824277, -0.206455, 0.807952, -2.337009, -0.317784, 1.510785, -1.421162, 0.897245, 0.493386, -0.274148, 0.532776, -0.905243, -0.515912, -0.590770, -0.679907, 1.518096, -0.112127, -0.775623, 1.832905, -0.532301, 0.854195, 0.195991, 1.248551, 0.717531, -0.122663, -0.244142, -0.921089, 1.270305, 0.804622, 0.545971, 1.708668, -1.265525, 1.114186, 1.471472, 1.052916, 0.571488, 2.491099, -0.921140, -0.111854, -1.660180, -0.294814},{0.307779, 0.867638, -0.436619, 0.066832, -0.571763, -0.141750, 0.536695, -0.532510, 1.033299, -0.141144, 0.729822, -1.040555, 0.424687, -0.551021, 0.179966, -0.004099, -0.738919, 0.616054, -0.254763, 0.571417, -0.542872, 0.710436, -0.029304, -0.024987, -0.792093, 1.208261, -0.928689, -1.746235, -1.531704, 0.102828, -0.937920, -0.484118, -1.423896, -0.843144, -0.806608, -0.743111, 0.022631, 0.326530, 1.081001, -0.527351, 0.758144, 1.785260, 0.125205, -1.457395, 0.888325, 0.271746, -0.980834, -0.887099, 0.878225, 0.077084, -0.667072, 0.182252, -0.542386, -0.460746, 0.372598, 1.704922, 0.347842, 0.499963, -1.502802, 0.260187, 0.564465, -1.511749, 1.275659, -0.378037, -0.245279, 0.001065, 0.328754, -1.151356, 1.102147, 0.111744, -0.129728, 0.618445, 0.014702, 0.322129, -0.688640, 0.719942, 1.603448, -0.581155},{1.248859, -0.249224, 1.127658, -1.399383, -0.720914, 1.046443, 1.338271, 0.087059, 0.397427, -0.260318, -1.095373, -0.362188, 0.125612, 0.049796, -0.339929, -0.840342, 1.277431, 0.738873, 1.015576, 0.110878, -0.953482, -0.276448, 1.860017, -0.731275, -0.798507, 0.222657, 0.946974, 0.111173, -1.198700, 1.061831, 0.557796, 0.979695, 1.217934, 0.506216, 0.766231, -0.573082, -0.300109, 0.653837, 0.206642, 1.512853, 0.011895, -0.928299, -0.362114, -0.103031, 0.286985, -0.030348, 1.026685, 1.229628, 2.038032, -0.664454, -1.624211, -1.206781, 0.981839, 2.136746, -0.609098, 1.500251, -0.000423, 0.683035, 0.087294, -0.959917, -0.388292, -1.387332, 0.951142, -0.488120, 0.133867, -1.726336, 0.370002, -1.842198, -0.025311, 0.243903, -0.759769, 0.938720, -0.906648, 0.307035, 1.273210, 0.749582, 0.811227, 0.175043},{0.304403, -0.351042, 1.044979, 1.093208, 0.381153, -0.532535, 0.217241, 1.608301, 0.233402, 0.385033, 0.148522, -0.988518, -1.153166, -0.437192, 1.410063, -1.262238, 2.097751, 0.185595, -0.563011, -2.227969, -0.208372, -1.359885, 0.208621, 1.060469, 0.762199, 0.038222, -0.757426, -0.625800, -0.267696, -0.933265, -1.199840, 1.565197, 0.880316, -0.025753, -1.299710, -0.129840, 1.800615, -0.366863, 0.460309, -0.778428, 0.150782, 0.691480, -1.614800, 0.444768, 0.792273, -0.416263, 0.128570, -1.067421, 1.426925, 0.132591, 1.245523, -0.329025, -0.628035, -0.135208, -0.471472, 0.867769, -0.309653, -0.055785, -0.117998, 2.215818, -0.517202, -0.295020, -1.675623, 0.502526, 0.939741, 0.929638, 0.437650, 1.012066, -0.654531, -1.153005, 1.033359, 0.721020, -0.537898, 2.925077, 1.285521, 0.501332, -0.610417, -1.473239},{0.874046, 1.073791, 0.274563, -0.155547, 0.783748, 1.819929, -0.270435, 0.313676, -0.172323, 2.070163, 0.547252, 0.250428, 1.100937, -1.029737, 1.419024, -0.787755, 1.668491, -0.121012, 1.522215, -0.783124, -0.438932, -1.636632, -0.287123, 0.389637, -1.226633, 1.343051, 0.455978, 0.336683, 1.393607, -0.010622, 0.678114, -0.349494, 0.143554, -0.963006, -2.140690, -1.203426, 0.788908, -0.339069, -0.266057, -2.710739, -0.305759, -1.017238, 0.503162, -0.470716, 1.177048, 0.369844, -1.133827, 1.357392, -2.485865, -1.158559, 0.937731, 0.505720, -0.057151, 1.981140, 0.450725, -2.055528, -0.539603, -0.128114, 0.097407, 0.126072, 0.363287, 1.492339, -0.549751, 0.160655, -1.492018, -0.838825, 0.573675, -0.337835, 1.545902, -1.949528, -1.306662, -0.206325, 0.164759, 0.832475, 0.515082, 0.880332, 0.959330, -1.324465},{0.825194, 0.187919, 0.500068, 0.462010, 0.486393, -0.167970, -0.503250, 0.661647, 0.482319, 0.746865, 1.150018, -1.500301, 0.613194, 0.268623, -0.210447, -2.711278, 1.791225, 0.970564, 0.798067, -0.847253, 0.229136, -0.085636, -0.649463, 0.659306, 0.184302, 1.161717, -1.001727, -1.374788, -0.428924, -1.307499, 0.374809, 1.540002, -0.914354, -0.766705, -0.874852, -1.825379, 0.964912, -1.264513, -0.076552, 0.036143, -1.564764, 0.232475, -1.639475, 0.990668, -0.282681, -0.323948, -0.992841, 1.015977, 2.405522, -1.115721, 0.597281, -0.714364, 0.057057, -1.098351, -0.883822, -0.722857, -1.446665, 0.100939, 0.923688, 0.783030, 1.698971, 0.528073, -0.205353, 0.238744, 0.290661, -0.611457, -0.117215, -1.601088, -1.019388, 0.708072, 0.273453, 0.779603, -1.136868, 0.507296, -0.765408, -1.061240, -0.402640, 0.842303},{0.347227, 1.303404, 1.193256, 0.638796, 1.086555, -2.587329, 0.121964, -0.076232, 1.013061, 0.323391, -0.704645, 0.107778, -1.249616, -0.871494, -0.114901, 1.552298, -0.349797, -2.509628, -2.703326, -0.610669, -1.413055, -0.104031, 0.622977, 0.664574, -1.035667, 1.154003, 0.655312, -0.254728, 0.660332, 0.369989, -1.560741, -1.161170, -0.097574, 0.042174, -0.401838, -1.036570, 0.360414, -1.331434, 1.134440, 0.852035, -1.669622, 0.075571, 0.901782, -0.950001, 1.900094, 0.203260, -1.609264, -0.297432, -0.630354, -0.776325, -0.218096, 0.892394, -2.136483, 1.490873, 0.038106, -0.137675, 1.388472, -0.316094, 1.671111, -1.495120, 2.213980, -0.256215, -0.073533, 1.619803, -0.708559, -0.494852, 1.127382, 0.170688, 0.809683, -0.364174, -0.367863, -0.232960, -0.267681, 0.430536, 1.533087, 2.032000, 0.756082, -0.956224},{-0.734962, 0.231593, 1.108036, 0.258060, -0.430878, -1.693292, -0.642750, -0.651199, 1.458021, -0.043429, -0.458015, -0.522384, -0.262886, -0.171268, 1.003940, -0.953704, -0.212186, -0.672456, -0.975928, -0.548901, 0.484233, -0.552977, -1.602505, 0.541198, -0.907176, 1.102413, -0.012395, 1.547898, -1.182794, -0.438286, 0.448625, -0.538807, -0.175796, 0.356950, 1.221719, -0.135143, -0.325682, 0.653595, -1.648021, -1.455858, -0.954477, -1.255178, -0.307056, 0.313852, 2.043458, 1.315622, 0.227681, -1.755864, -1.196630, -0.495222, -0.762575, 0.330500, 0.732552, 0.882444, 0.425688, -0.765213, -1.396182, 0.420386, 0.541743, 1.921299, -0.066480, -0.985431, -0.171852, 0.402264, -1.377345, 0.897015, 0.497751, -0.580472, 0.437756, 0.410489, -0.909916, 0.547199, -0.179862, -2.034878, 0.910900, 0.174578, 1.430605, 0.058160},{-0.068503, 0.330842, -0.569652, 0.924766, -2.735285, 0.035581, 2.026189, 0.111460, 1.563015, -0.299113, -1.002527, 0.544080, -0.455098, -1.007271, 1.090063, -2.101625, -0.308403, -0.653886, -0.220953, -0.317126, 0.384901, 0.195766, -1.658455, -2.642148, -1.037248, 0.245242, 0.896056, 0.330294, 1.289925, -0.192144, 0.228213, -1.231462, 0.096593, 0.694774, -0.307297, 0.165707, 0.439987, -0.682624, 0.954090, 0.020450, -0.410049, 0.409470, -0.451810, -0.455445, 0.272643, -0.177469, 0.272579, -0.529816, -0.194804, 1.872740, 0.000699, -0.310667, 0.225107, -0.791128, 0.288731, -1.777028, 2.758839, 0.923664, 0.822546, -1.488733, 0.269135, -0.301723, -1.000533, -1.652267, -0.056938, 0.260027, 0.605535, -0.582933, 0.051345, 2.503786, -0.013302, 0.049923, 0.451877, 1.351431, 0.031005, 2.220615, -0.815731, 0.382738},{-1.242838, -0.087482, 0.378243, 0.952423, 0.453381, -1.031719, -1.821887, -0.871246, 1.073752, -0.515742, -1.182869, -1.602856, -1.055912, -1.109215, -0.344112, 0.618313, -0.161159, -0.791483, 1.407894, -0.766248, 0.607524, -0.806352, 0.587876, 0.947440, -0.168345, -0.352362, 1.304853, 0.699830, -0.448698, -1.607320, 2.300506, -1.289468, 1.330471, -0.361735, 0.004024, 0.971436, 1.907079, 0.116721, -1.106652, 1.894788, -0.470204, 0.803172, 0.515928, 0.169523, -0.360870, -0.312783, 1.770159, -0.056594, 0.420402, -0.685207, 0.198835, 0.128265, -0.633023, 0.455406, -0.054675, -0.708088, -0.236457, -0.484529, 1.282253, 0.727338, -0.827326, 0.260116, -1.630578, 1.058972, 0.758224, -1.261478, -1.682811, 0.228002, -1.031095, 0.841892, -0.388451, 0.664787, 0.109556, 0.734203, -0.367650, 1.529547, 1.081707, 0.224931},{0.065434, 0.823680, -0.494907, -0.162366, 0.095921, 1.850838, -1.611907, -0.307000, 0.836464, -0.061142, -0.536106, 0.233706, 0.291463, -0.643583, -1.345606, -0.980802, 0.940993, -1.369609, 0.685968, 0.611796, 0.390900, -1.604295, -0.355826, -0.605989, 0.262946, -0.083820, -1.103237, -0.067092, 1.140070, -1.272907, 0.477890, 0.310296, 0.028026, 0.150962, 0.534738, 0.618977, -0.116301, -0.397904, -0.068333, 0.677527, -1.219154, -0.383582, 0.422101, -0.348244, 0.063802, 0.221707, -0.281828, 1.197024, 0.777570, -0.654148, 0.079381, 1.472699, -0.238923, 0.859972, -0.285976, -1.204633, -0.115093, 1.964634, 1.167520, -0.318433, -0.896580, 1.847997, -0.457989, -1.001784, 1.411314, 0.873296, -1.122879, -0.112580, 0.207666, 1.246974, 0.247541, -0.545374, -0.991609, 0.167786, -0.167455, -0.586088, 0.244690, 0.312577},{0.935923, -0.941167, -0.893746, -2.142988, -1.439431, -0.868217, 1.646662, -0.474151, 0.057701, 0.079518, 0.490433, 0.245671, 1.222843, -1.697053, -0.495686, 0.740891, 0.405418, -1.176929, 0.804722, -1.743510, 0.202097, 0.584059, -0.909980, 1.077615, -0.234263, -0.681918, -0.689005, -0.460604, 0.459459, 0.622250, -0.862162, -0.166753, 0.315624, 1.966950, 1.269523, 1.409194, 0.147398, 0.987648, 0.669902, 0.433454, -0.297154, 0.315621, 0.194471, 1.163740, -2.240741, 1.733170, 1.750550, 1.353018, 0.048828, -0.818973, 0.336258, 1.308359, 0.658298, 0.683187, 0.134698, -0.160137, 0.597908, 0.665033, 1.158970, 1.347930, 1.637212, -0.148646, 0.873376, -0.186951, -2.354882, -0.568830, -0.891577, -0.472271, -0.513272, 0.080875, -0.160306, -0.204745, -0.616101, -0.694834, -0.018590, -0.417179, -1.344792, -1.640694},{-1.846134, 0.596559, -0.593251, 1.136268, -0.596198, -0.280119, -1.695839, -0.658974, -0.341216, -0.309929, 0.157094, -0.386031, -0.288268, 0.949710, -0.945824, -1.343967, -1.185837, 1.919955, 0.939013, -0.234956, 0.409977, -1.420041, 1.657900, 1.502489, -0.832370, 0.488716, 0.575201, 1.149202, -0.155805, 0.234671, -0.971198, 0.297798, -0.110415, -0.128825, 0.534155, 0.673152, -0.940772, -0.002186, -0.520325, -1.089573, -0.990641, 1.831282, -1.896820, -1.716281, 0.848422, 0.768451, 0.768140, -0.052778, -1.465728, 0.386145, -0.349559, 0.184959, -1.004339, -0.537365, 0.686850, 0.306128, 0.574554, -1.367710, 0.591112, 1.376587, 1.220690, -0.425152, 0.016131, -1.559916, -0.452627, 0.391643, -0.903301, -1.044639, -0.281781, -0.600249, 1.187593, 0.692889, 0.704880, -0.819653, -0.916320, 1.154976, 1.945945, 0.149029},{-2.209047, -0.494862, -0.621695, 0.458927, -0.149818, -1.130627, -1.533991, -1.206903, -0.814375, 0.261017, 0.657991, 0.022620, 1.074357, -0.320445, 1.594799, -2.200635, -2.511015, -1.859027, -0.661557, -0.257752, -1.614083, -0.093470, -0.242557, 2.773590, 0.579684, -1.837898, -0.617714, 0.013490, 0.624755, -0.950673, -0.004234, -1.037992, -1.051669, 0.270099, 0.457236, -0.392724, 1.135538, 0.736580, -0.006044, -1.472493, -0.849256, 0.344343, 0.615280, -0.716642, 0.300830, -1.069919, -0.386368, -1.706023, -0.667991, 0.846164, -0.122417, -0.011989, 0.692237, 1.795999, -0.647781, -0.259465, 1.770040, -0.393914, -0.220770, 0.936978, 2.061991, -1.103118, -1.159779, 0.862960, -2.241405, 0.429345, -0.828357, -1.886756, 0.423590, -0.841546, -1.021754, -0.157628, -1.225727, -0.142953, 0.837408, 0.416443, 0.019316, 2.431753},{1.946696, 0.991095, 0.915774, -0.059946, 1.052462, -0.539174, 1.311120, -1.113779, -0.692263, 1.652341, -0.482538, -0.094557, -1.043974, 1.716989, 0.750341, 0.702751, -0.794042, -0.165723, -1.458800, -2.350381, -1.194390, 0.211157, -2.727314, 0.669783, 1.463071, 0.266424, 1.370140, 0.379951, 1.315723, 1.061350, 1.355309, -1.380153, -0.332142, -1.875737, -0.291051, -0.584078, -0.183499, 0.140236, -0.519647, 1.189929, 2.183240, -0.261490, 1.140203, 0.439049, -1.321923, 0.346368, -0.213581, 0.990944, 1.539280, 2.599534, -2.571655, -1.479896, 0.187433, 1.743848, -0.438653, 0.069842, -0.403113, 0.137668, -0.725202, 0.836918, 0.115372, -0.630890, 0.413748, 0.437234, -0.735775, 2.558413, 0.704065, -0.117558, -1.565316, 0.000620, 2.699909, 0.153700, 0.776406, -1.111804, -0.234599, 0.127270, 0.003854, 0.643656},{-0.191704, 0.159797, -1.276190, -0.430491, -0.537597, -0.529606, -1.632243, 1.234287, -0.272669, 0.354798, 1.264873, 0.235446, -0.442256, -0.549302, -0.277745, 0.970819, 0.481893, 1.000674, -0.606455, 0.834869, 0.936737, -1.040612, 0.069896, 0.474744, -0.724863, 0.529586, 0.208805, 0.159133, 0.802911, 0.590418, -0.588782, 0.854031, 0.682736, 1.508273, -0.515378, 0.581753, -1.124356, -0.623120, -3.339756, -0.247945, 1.322288, 0.606894, -0.321704, 0.562704, 0.740137, -0.444718, 0.027124, -0.633401, -0.448321, 0.087017, 0.681999, 0.525445, -1.103673, -1.610565, 1.949967, 0.044334, 1.427881, -0.994393, -1.350770, -1.218602, -0.582499, 0.288542, -0.946321, 1.601131, 0.147134, 1.835416, -0.136813, 0.968253, -0.773227, -0.323719, -1.071108, -0.370874, -0.249587, -1.047324, 0.003897, 0.132013, -0.166056, 0.946483},{0.961566, -0.318251, 0.075412, -0.507061, -0.455376, -1.903574, -0.724002, 0.914930, 0.369343, -0.150342, -1.859951, 2.304835, 0.269462, -0.000534, 0.077875, -0.365373, -1.032430, -0.961839, 0.951943, 0.274614, 1.008180, 0.701982, -0.536797, -1.067818, -1.229347, -0.233235, 0.210759, -0.474151, 0.417013, 1.120027, 0.440959, -0.645442, -2.787467, 0.711951, -0.118301, 0.495172, -1.056842, 0.445931, -0.073626, 0.236948, -0.098009, 0.199547, -0.661537, -0.157282, 0.105611, 1.112972, 0.274797, -0.411692, -1.167622, 1.698295, -1.696214, -0.388478, 1.099786, 0.640989, 0.465098, 0.248429, 1.698980, -0.147121, 1.056772, -0.292159, -0.075493, -1.218505, -1.973187, -0.550175, -0.469400, 0.503734, -1.776708, 0.627970, 1.281926, -1.097905, 1.455322, 0.096104, 0.086245, 1.949415, 0.095923, -0.591433, -0.110507, -0.359729},{1.278090, -0.639363, -0.701133, -1.022253, -0.994404, -0.589199, 0.336274, -0.696853, 0.538932, -0.017943, -0.146761, 0.926574, -0.687987, 0.662813, 0.166902, 0.045856, 2.255308, 0.623331, 0.975665, 0.571143, 0.194198, -0.778419, 0.906539, 1.316691, -0.116943, -0.836086, 1.425929, -0.366004, -0.792149, -0.181592, 1.119733, -0.165271, -1.859599, -0.559982, -0.668312, 0.302319, -0.167167, -0.461331, -0.248677, -1.387358, 0.308040, -2.357063, 0.844812, -0.361972, -1.706993, -0.677610, 0.008792, -0.702124, -0.232411, 2.237826, -0.266125, 0.143626, 0.250412, -0.098840, 0.132932, 1.830491, 0.611004, -0.563920, -0.281173, 0.837291, 1.701613, 0.897932, 0.655907, -0.211174, 0.141219, 1.660768, 0.849553, 0.658420, 1.373280, -0.620882, 0.918674, 1.003165, -0.926525, 1.557843, 0.424954, 1.557026, 0.270000, -0.908446},{0.433450, -0.245289, 1.825433, 0.671847, -0.728835, 0.063876, 0.777529, 0.178784, 0.441557, 0.367108, -0.681699, 0.846430, -0.038457, -1.166393, 0.215430, -0.899741, -0.596471, 0.849879, 1.563754, 0.181561, 1.590656, 0.732416, 0.824589, 0.261640, -0.334060, -0.094907, 0.764588, 0.291714, 0.683963, -1.690575, 0.173888, -0.156946, -0.205997, -2.206935, 0.183459, -0.636248, 0.126386, -0.362142, 0.357782, 0.815708, 2.334915, -0.154173, -0.648126, -0.823844, -1.144521, -2.737607, -1.727652, -0.476064, 0.817281, 0.029463, 1.504748, -0.712058, 0.502958, -1.718299, 1.056239, -0.202800, 0.077915, -0.258005, 0.303187, 2.355834, -1.278562, 1.138926, -0.369439, 0.790878, 2.166373, 0.406232, -0.702366, -0.305634, 2.332027, -0.640771, 0.902688, -0.801521, -0.103918, 0.171812, -0.830001, -0.908358, 2.180492, -0.677251},{0.622253, 0.393252, 0.750958, 1.646926, -0.435479, -1.288153, -1.255481, -1.236786, 0.950065, 1.223723, 0.575728, -0.327244, 1.137133, 2.065384, -1.067633, -0.211932, 0.158076, -0.446973, -1.103073, 0.730286, -1.935883, -0.148289, -1.520463, 1.178094, 0.139486, 0.060272, 0.406795, -0.307123, 0.455666, 0.487407, -1.384859, 1.515015, 0.799242, 0.902514, -1.094263, 1.513985, -0.542913, -0.042772, -0.882956, 0.286252, 0.252991, -1.167379, -0.273688, 0.379668, -0.119959, -0.721563, 0.560794, -0.107275, -0.758382, -0.442076, 1.995173, 0.587072, -0.132515, 0.940056, -0.564802, 0.812355, 1.056370, 2.118193, -1.553734, 2.870307, -0.993427, 1.111706, -0.479391, -0.921103, 0.165152, 1.256348, 1.170101, -0.388154, 2.690077, 1.999766, -1.924930, 0.257580, 1.652581, 0.428834, -0.118739, -0.671715, 0.779974, 1.426796},{-0.373232, 0.225388, 0.531432, 2.299237, 0.953297, 0.493168, 0.315742, -1.189952, -0.686902, -0.137046, 0.809100, 1.315931, 0.961870, 0.624383, 0.853569, -0.986298, 0.896109, -0.249493, 1.142946, 0.183007, 1.221267, -0.166791, 0.570891, -0.681030, -0.834313, 0.759411, -0.883839, -0.315371, -0.141450, 0.504133, -0.674323, 0.692269, -0.339663, 0.047262, 0.603253, -1.312835, -1.198077, -0.168491, 0.736730, 1.215778, -1.096014, -0.764557, -0.133033, -1.015636, 0.446351, -0.240124, 0.228222, 1.218251, -1.558193, -1.635670, 0.304876, -1.091027, 2.660441, 0.002941, 1.678417, -0.155755, -0.731084, 0.919081, -1.193525, 0.554410, 0.203543, -1.792941, -1.068701, 0.320998, -0.211628, 0.599017, 0.602887, -2.858242, -0.749536, 0.560355, 1.169812, 1.411790, -0.195279, 0.601787, -1.260387, -1.769519, -0.493806, -1.202828},{-0.713423, 0.677988, 0.509528, 0.732574, -0.115003, 0.101128, -0.859089, 1.778106, 0.847874, -0.473858, -0.924662, -1.275306, 0.060061, -0.440554, 0.769674, 1.840182, 1.457738, 0.208952, 0.663914, -1.054272, 0.039168, -0.151043, 1.812792, -0.164904, -0.285803, 0.511111, 0.150356, 0.485423, -0.777576, 0.997366, -0.093975, 0.867621, -0.571646, 1.112589, 1.269326, -0.043785, 0.065788, -0.744048, 0.420901, -0.466180, 0.267941, 0.535003, 1.741218, 0.422739, 0.510817, -0.662701, -0.004022, 1.654925, -1.337512, -0.757229, -1.254443, 1.537002, -2.760306, -0.997864, -0.259131, -2.353305, -1.680243, 0.350698, 0.226193, 1.361275, -0.116396, 2.334408, 0.703507, -0.357188, -0.138645, -0.563281, -0.265609, 0.955392, -0.668171, -0.138695, -0.068987, -0.211275, -1.140041, 0.021484, -0.582109, -1.353437, -1.823887, 0.247239},{-0.924495, 0.072157, 2.000208, -1.142990, 0.053687, 1.275565, -1.573927, -0.348877, 0.248851, -0.354651, 0.201344, 0.574946, -0.021164, 0.559380, -1.579730, -0.383998, -0.568720, -0.911527, -0.155266, -0.252730, 0.208267, -0.597616, -0.516322, 1.902777, -2.389011, 0.679755, -0.735910, -0.227522, 1.549924, -0.601264, 0.422992, 0.034711, 1.617997, 0.213884, -0.635083, -1.339115, -2.017078, 0.051900, -1.183639, 0.284909, -0.240650, -0.942250, 0.932634, -0.887621, 1.109831, -1.417052, 1.954846, -0.788343, 0.235581, 0.796004, 0.162157, 1.005158, -0.625973, -0.327374, -0.814335, -0.245826, 0.869554, 0.197567, 1.804799, 1.303949, 0.581952, -1.510418, 1.406434, -2.297978, 0.187016, 0.331977, 1.233199, -1.657053, 1.803668, 0.420847, -0.432031, 0.115906, 0.655757, 0.226596, -0.292510, 0.071873, -0.932504, -0.828995},{-0.376699, 2.320821, 0.269328, -0.968421, -1.952479, -2.430810, -0.435111, 0.819488, -1.323854, 0.795757, 0.744963, -0.696683, -0.493328, -0.094937, 0.977631, 0.029053, -0.113310, 0.720027, -0.152912, -0.021218, 1.526778, 0.348225, -0.451144, 1.164068, 0.271158, 1.418327, 0.406696, -2.342782, 0.707831, 1.808802, -1.190943, -0.855654, -1.420312, 0.899539, 1.267272, -0.285896, -1.360617, -0.554221, 0.505913, 0.859242, -1.085944, -0.388724, -0.577783, -1.307719, 0.571301, 0.022881, -1.151560, -0.481699, -1.195751, 1.512843, -2.951655, 0.128297, 0.502627, -0.607742, 0.801988, -2.434033, -1.340815, -0.760150, -1.488640, 0.776465, 1.502832, -0.020576, 1.584998, 0.850944, 2.090675, 2.263497, -1.341157, 0.811796, -0.289027, -0.061994, -0.146802, 1.431208, -0.359618, -1.420307, -0.655232, 1.236645, 0.315468, 0.417541},{0.157621, -0.401231, -0.869693, -0.444757, -0.089548, 1.961611, 1.042572, -0.428392, 1.037771, -0.055995, 0.831258, -0.670805, 0.864795, -1.850116, 0.640707, -0.308582, -1.146174, 1.347406, -0.193804, -1.182706, 1.068187, -1.264353, 2.082559, 1.121264, 0.986986, 0.091545, -2.002876, 0.247932, -1.411477, -2.076926, -0.548854, -0.507939, 0.381306, -0.430759, -0.273469, -0.416362, 0.383108, 0.817664, -2.331217, -0.167003, -0.906069, -1.874081, -0.276533, -1.362108, 0.256304, -1.654561, -0.944139, 0.866867, 0.007957, 0.498525, 0.168281, 0.382059, 0.054575, 0.616961, -0.446454, 0.902935, 1.703628, 0.370488, -0.470105, -0.868353, -0.381017, -1.880171, 0.568168, 0.281534, 0.696779, -0.221840, -1.671573, 0.314751, 0.562113, -0.397331, -1.175414, 2.119339, -1.132524, -1.897229, -0.852346, -0.070321, -0.926950, -0.067829},{-0.091979, 0.853973, 1.089394, 1.590963, 1.758375, 0.115633, 1.260948, 2.411238, 1.295415, -0.493561, 0.732745, 1.101276, -0.551183, 2.129310, 1.000776, -1.025018, 1.632825, 0.932333, -0.619352, -1.615224, 1.816437, -0.250852, 2.179366, -1.373496, -0.975269, 0.051163, 1.716456, 1.091869, -1.569467, -0.586515, -1.028117, 0.425231, 1.844890, -0.273909, 0.239133, -0.931429, 1.271302, -0.574579, -0.432558, -0.584957, 0.423002, -0.360851, -1.278819, 0.653146, 0.452132, -0.599214, -0.555430, 0.105502, 0.449807, 2.202483, -0.619206, 0.937110, -1.208254, 0.854975, -0.966953, 0.387480, 0.596204, -2.056776, -0.178522, 0.625051, 0.213281, 2.130709, 1.046155, 0.990580, 0.523604, 1.542905, -0.357069, -0.383383, 3.112994, 1.937193, 0.344411, -0.188273, 0.475857, -0.387356, 0.943681, 0.120523, 0.465461, 2.260632},{0.116629, 0.409428, -1.602727, 1.737875, -0.861391, 0.039012, 0.794232, 0.137399, -0.057803, -1.205353, 0.289521, -0.118329, 0.219933, -0.654027, -0.221279, 0.079421, -0.662557, 1.660576, 0.211080, -1.828642, 0.859010, -0.446768, 0.368759, 0.371329, -1.836783, -2.639578, -0.882990, 0.584414, 0.045325, -0.650025, 0.386357, -1.956419, 0.985744, -2.200631, -0.039705, -1.111955, -0.226832, -1.253523, 0.967629, 1.587991, 0.093906, 0.098560, 0.354606, -0.413757, 1.577813, 0.259060, 0.257197, 1.016981, 2.140101, -1.760393, -1.326725, -0.030763, -1.491511, 1.710303, -0.859457, -1.339866, 0.374712, 1.309424, 1.771350, 0.743529, 1.256616, -0.819297, 1.428414, 0.497149, 0.117446, -2.929146, -1.464311, 2.785049, 0.866149, 1.317300, -1.984076, 1.555187, 1.161310, 0.020007, -1.914358, 0.923642, -1.596720, 0.791623},{-0.249364, 0.024664, -1.340968, -0.715181, 0.599305, 1.898703, 1.541723, -0.393456, 0.190444, -0.437013, 0.815991, 0.093623, -0.630150, -0.481553, 1.592184, 1.034221, -0.847620, 0.517470, -0.138312, -0.603402, -0.340705, 0.281251, 0.963941, -1.356017, -0.288697, -1.431021, 1.303704, -0.484074, -0.454252, 1.063614, -0.208582, 0.182680, 1.792739, -0.626997, 0.010583, -1.891670, 0.748783, 2.032021, 0.031570, 1.003133, 0.234305, -0.022985, -0.691480, -0.791167, 0.108410, -0.675759, 0.857469, 0.963304, -0.972939, -0.846685, -0.714409, 0.231679, 0.103906, 1.144696, 0.709526, -0.751313, -0.497399, 1.739082, 0.510766, 0.116748, 1.474208, -1.210722, 0.909101, -0.140747, 1.181059, 0.165262, -0.774598, 0.280616, 0.332697, 1.329765, -0.672887, 1.722909, -2.543262, -0.174562, 1.347327, 1.422813, -0.426298, -0.687415},{0.298198, -0.974854, 0.175696, 1.279323, 0.540173, -2.419951, 2.320170, 0.014327, 0.534691, 0.049746, 0.378877, -0.021259, -0.018356, 0.281218, -1.517158, 0.509943, -0.535941, 0.886642, -0.913633, -1.827019, -1.117816, 0.532764, -1.100581, -0.339587, -0.552461, 1.362388, 0.637825, 1.783567, 1.076784, 0.325490, 0.595524, -0.308893, 0.320276, 1.478389, 1.485185, 0.871035, -1.409713, -0.099195, -0.251459, -0.915393, 1.521198, 0.307520, 1.074071, -1.381207, 0.671750, 0.896692, -3.018710, 0.184626, 0.385415, 2.041449, 0.515099, 1.132658, -1.696678, -0.558063, 0.338049, 1.007993, -0.192907, 0.548053, -0.577893, 0.417313, 0.356586, 1.723455, 0.291794, 0.302706, -0.123382, 0.528876, -0.082194, -0.236436, -0.580726, -0.193885, -0.696227, -0.462347, 0.403303, -0.332384, 0.996144, 0.729197, -0.680920, -1.966655},{0.534009, -0.373271, 0.165141, -0.886021, 1.033303, 0.226500, -0.406650, -0.999934, -0.471044, -0.085384, -1.478760, 0.428976, -0.416440, -0.454820, 0.086381, 0.861859, 0.905629, -1.087691, -1.231250, -0.513441, -1.456973, -1.102947, 1.400742, 0.429503, 0.636909, -1.237290, 1.560609, 0.265999, 0.340486, -0.416039, 0.441450, 0.034807, -1.342082, -0.296862, 0.549873, 2.318597, -1.134430, -1.026099, -1.648314, 0.458455, 1.391208, 1.450389, -0.816750, -0.802501, 1.519738, 1.050297, -0.486982, -0.367123, -1.479402, 0.476562, -0.257753, -0.891229, -1.179563, 0.163748, -0.095374, 1.663660, 2.031790, 0.700429, -0.094415, 0.795422, -0.993853, -0.184852, 0.112078, 0.507202, 2.573181, -0.294238, 1.449070, -0.770284, 0.903447, 1.199420, -1.716482, -0.274188, 0.902979, 1.195888, -1.778342, -0.629043, -2.161798, 0.234998},{2.050279, -0.742739, -0.211782, -0.481091, -1.041565, -1.632277, -0.705093, 0.060094, 0.208673, -0.542923, 0.018115, -0.979100, 1.350445, 0.938962, 2.233561, -0.746508, -0.442794, 0.540799, 0.106863, 0.313541, -0.879529, -0.110322, -0.664286, 0.760336, 0.056967, 0.019623, 0.437592, 0.005424, 1.219400, 0.143161, -0.149708, 2.699681, 1.194411, 0.112800, -0.415009, 0.583247, -0.263785, 0.324052, -0.647364, 1.395408, 0.950077, 0.483098, -1.292804, 1.563817, 0.523085, 0.486511, 0.065535, 0.719404, 0.070986, -0.960272, -0.467847, 0.489277, -0.099124, -0.144682, -0.243095, 1.826141, -2.021044, 0.792505, -1.092378, -0.100623, -1.188703, 1.545133, 1.339303, 0.102126, 0.958594, -0.508934, -0.271864, 0.391338, -0.084015, -0.575076, -1.047831, 1.162821, -0.662770, -0.015460, -0.022209, 0.106694, -1.253793, -1.191800},{-0.578102, -1.288552, -0.305989, -0.471014, 0.475344, -0.339811, 1.270545, -0.556285, -1.081255, -0.501908, -0.433673, 0.226631, -1.530848, -0.524058, 0.497600, 0.474897, 2.247358, -0.392168, 0.296761, 1.001793, -1.107951, 0.332846, -0.944963, -0.009230, 1.029651, -0.104057, -1.202412, -0.124222, -0.076610, 1.380906, -0.349940, 1.376080, 0.662855, -0.140285, -0.933274, -0.897541, 1.893876, -0.333757, 0.700178, -0.715916, -0.400041, 0.455620, -0.050648, -1.044431, 0.251547, -1.435126, 0.216059, -0.231664, -0.414922, -0.029859, 0.529793, -1.544060, -0.765078, -2.198811, 0.012043, 0.176110, 2.577023, -0.019545, -0.530049, 0.711597, -1.009961, -0.440057, -1.220375, 0.970076, -1.459059, -0.771698, -0.055504, -1.186919, 0.606050, -0.792996, 0.004171, 0.747094, 0.890937, -0.217671, -0.701464, 2.972044, -0.559852, -0.440411},{-0.320625, -1.087322, 1.043813, 0.710902, 0.109505, -0.731651, 2.691120, -1.786755, 0.961092, -1.523911, -0.303107, -0.159247, -1.393596, 0.123301, -0.784378, 0.227127, 1.291105, -0.147294, 0.367045, -1.722323, 0.837772, 1.000507, 0.466020, 0.141696, 0.664316, 1.088129, -0.655180, 0.393334, -0.804243, -0.300860, 0.538812, 0.706402, 2.168253, 1.882731, -1.323238, 0.736273, 1.935853, -0.037640, -0.374534, -1.003607, 0.021084, -0.951848, 1.507781, -0.232026, -0.861470, -0.038361, 2.179651, -0.616559, 1.099110, -1.968163, -0.000539, -0.889816, -1.176406, 0.628748, -0.359272, 1.405779, 0.445977, 0.069180, 0.489627, -0.449494, 1.457964, -1.332816, 0.823882, -1.551334, -0.094867, 1.025663, -0.087724, -0.786380, -1.256490, -0.073425, 1.465636, 0.086149, -1.218344, -0.499217, 0.405463, 0.340832, -0.499468, 0.286047},{0.506103, 0.747304, 0.953184, -0.595907, -0.263541, -0.000719, 1.890443, 0.597671, 0.374568, -1.610167, 0.760450, -0.119523, 1.096570, 0.083174, -0.606073, -0.881001, -0.532907, 3.191308, -0.669296, 1.804923, -0.828242, 0.080709, 1.488984, 1.216168, 1.150450, -0.422149, -1.308845, -1.790252, 0.179522, 0.291446, -0.155412, 1.020561, 0.572821, -0.831811, -1.476182, 0.773106, 1.028171, -0.577028, -1.359480, 2.127153, 0.540094, 0.118333, 0.320548, -0.037321, -1.486885, 1.202797, 0.826215, -0.856437, 0.676055, -0.683120, -0.473293, -0.037736, 0.876980, 0.200078, 0.055981, -1.565718, -0.257199, -0.718382, -0.108042, -1.285940, -0.374888, -0.293229, -0.746666, 1.182903, -1.463962, 0.495525, -0.230275, -0.154356, 1.264648, -1.466115, -0.874621, -0.714780, 0.544147, 1.299056, -1.301825, 1.745654, -0.613208, 1.063993},{0.703000, 0.780533, -0.213593, -0.291236, -0.205098, 1.437146, 1.054064, 0.534293, 1.155910, 0.197478, 1.140430, -0.408806, -1.823892, -0.102122, 0.272824, 1.162926, -1.786672, 0.850946, 1.511927, 1.780424, -0.401943, 1.768481, -0.990552, 0.850656, -0.440969, 0.091077, 0.105273, -0.005389, 1.468073, 0.548918, -0.628384, -1.386166, -0.471570, -0.427294, 0.652313, -0.389105, 0.618025, 0.562834, 0.124067, -1.311175, -1.562290, -0.183754, -0.252764, 1.205186, 0.786926, 0.018029, 0.250553, -0.776855, 0.852857, 0.294931, -0.414527, -1.320290, 0.274656, 0.846044, -1.121677, -0.109825, 2.714305, -0.447483, -0.281993, -1.454078, 0.824142, 0.259794, 1.287425, -1.990992, -0.692768, -0.793122, -0.233637, 0.103856, 2.332263, -0.246984, 1.043712, 0.043059, 0.139472, 1.597820, -0.331208, -0.892997, -0.307265, -1.290261},{-0.944165, -0.565688, -0.313016, 0.210611, -1.887754, 0.674209, 1.052693, -0.370714, -0.361459, 0.900654, -1.142968, -0.928359, -1.483362, -0.739846, 0.846062, 1.474447, 0.777474, 0.375184, -0.658940, 0.366458, 1.637536, -0.589138, -0.313369, 0.818522, -1.091403, -0.615916, 0.447298, -0.223432, 0.576594, 0.127738, -2.188739, -0.149386, -1.850276, 0.157558, -0.325052, -1.220743, -0.599702, 2.204914, -0.999756, 1.467099, 0.370938, -0.463499, -1.884320, -0.477642, -0.254137, -1.219570, -0.301088, -1.516228, 2.010357, -0.604219, 0.098294, 0.986871, -3.023935, 0.594973, 1.013023, -0.950668, 0.087052, -0.260444, -1.857336, 0.292288, -1.159644, 2.372322, 0.613631, -0.945734, -1.305553, 0.089796, -0.797210, 0.008938, 0.235797, 0.118131, 0.723123, 0.120366, -0.674472, 1.032763, -0.567216, -0.265100, -0.392444, -1.192368},{0.935652, 0.315882, -1.352470, 0.324027, -1.621180, -0.163871, -0.061875, 0.322967, -0.426485, 1.193707, 0.373893, -0.664138, 1.601752, 0.159177, 1.385289, 0.796206, 0.377089, 0.152730, 0.776674, 0.482479, 0.377496, 0.589136, -0.784849, -0.415826, 1.516509, 0.563484, -0.432096, -0.591555, -0.156898, -1.641359, 1.618161, 0.726520, 0.193735, -1.657526, 1.501416, 1.122874, 0.592466, -0.071070, 0.945032, 1.137317, -0.292742, -0.163053, 0.087108, -0.672773, -0.698451, -0.147964, -1.463675, -0.551070, -1.627082, 0.411806, 0.687457, 1.757603, -0.576553, -1.077769, 0.331635, 0.815471, -1.607954, -1.301160, 0.592666, -0.440400, 2.684352, -2.336065, -0.578562, 1.172063, 0.062703, 1.145260, 0.329463, -0.385288, 0.634140, 1.194591, 0.794436, -0.456115, 0.265709, -0.291204, -0.271634, -0.927461, -1.175827, -0.226358},{-2.046368, 0.750517, 0.761632, 0.529238, 0.328511, -0.668572, -0.978795, 0.191898, 0.424722, 0.789845, -0.699475, 0.438997, 1.986917, 1.216830, 0.547182, -0.444361, -0.243986, -0.357521, -0.029173, -0.249875, 2.047968, -1.665724, -1.918497, 0.080097, 0.675683, 0.483805, -0.282819, 1.483826, -1.222920, -0.905803, -2.129557, 0.300688, 0.510607, -2.093405, -1.126635, 0.659188, -1.549284, 0.706430, -0.615346, -0.572262, -0.446106, -0.465972, 1.083202, 0.065229, -1.522002, 1.036623, -0.487335, -0.429843, 1.725525, 1.221546, -1.604336, -0.160405, -0.672169, -2.172961, -0.865969, -0.338580, -1.547978, 0.138355, -0.066755, 2.347188, 0.130396, -1.858223, 0.903876, -0.327312, 0.027919, 0.332225, -2.197867, -0.663304, -0.833674, -0.297767, -0.021316, -0.582266, 1.452410, 1.511209, 0.118115, -0.763315, -1.157657, -1.410826},{0.204306, 1.570442, 0.647884, -1.223060, 0.988267, 0.062039, 0.221922, -1.557884, -0.968756, -0.194419, -0.924607, 1.064510, 0.077437, 1.555511, -0.929684, -0.648161, 0.229523, -0.704794, 0.022455, 0.209523, -0.602487, 0.979007, -0.360941, -0.023458, -0.237926, 2.435649, -0.409684, 0.476975, -0.461164, -0.922109, -0.265509, -0.814917, 1.535183, 0.839078, 0.038231, 1.433599, 0.032917, -0.467738, 1.117383, 1.087271, -0.002356, -0.508585, -0.479566, 0.367905, 0.464210, 0.103161, 0.199388, 0.739263, -2.613477, 0.251344, 0.003319, -0.637529, -0.634741, 0.708362, 0.516647, -0.040990, -0.153386, -1.306246, -0.130317, -0.200513, 0.610728, 0.017105, 1.870495, 1.901200, 0.069798, 0.364984, -0.071135, -0.786426, 0.068964, 1.010799, -0.220185, 0.114394, 2.075739, 0.576195, 0.542715, -0.471406, 0.194211, 0.053468},{-0.753711, -1.072562, 0.361116, 0.441999, -1.643642, -2.077473, 0.503986, 0.946926, 0.595710, -0.937661, 0.106331, 0.635929, 0.978247, -1.095392, 0.260473, 1.073138, -1.837849, -0.760207, 1.913726, 0.600846, 0.946748, -1.034502, 0.828054, -1.722623, 0.765497, -0.584020, 0.544012, 1.040959, 0.534153, 1.719260, -0.243852, 0.239571, 1.641090, -0.163517, 0.756319, 0.980174, 0.428967, 0.998340, -1.084357, 1.777300, 1.248163, -0.390523, -0.377874, -0.053757, -0.692486, 0.039568, 2.032170, -1.535776, 0.691472, -0.081602, 0.335273, 0.582478, -0.472591, 1.245131, -0.342429, 2.304247, 2.144504, -1.320819, -0.487168, 0.361877, 0.170629, 0.617801, -1.170859, 1.814789, -1.228076, -0.265482, -1.449673, 0.277055, -0.210993, -0.087871, 0.042360, -0.938937, 1.001643, 0.782085, -1.455742, 0.604390, 1.285762, 0.105974},{0.797479, 2.100374, 2.104120, 0.210857, -1.649853, 0.877593, -0.065154, 0.656747, 1.094240, -2.012221, -0.605612, -0.538211, 0.301035, -0.193443, 1.698610, 1.542256, -0.614609, -0.345707, 0.057906, -0.447693, -0.082975, -0.054856, 0.516863, 0.106527, -1.077586, 0.438187, -0.923553, -0.387445, 1.116161, -0.479183, 0.436977, 0.192772, -0.423602, -0.701632, 0.506558, -0.407355, -0.292902, -1.167504, -0.174222, 0.201325, -0.797089, 0.555707, -0.799076, -0.008155, 0.424170, 0.325096, -0.005685, 1.293090, 1.006879, 0.474922, 2.584745, -0.099297, 1.265711, 0.494596, -1.512392, -1.166725, -0.999864, -0.294054, 0.129129, 1.061318, 1.007648, -2.010650, 0.246173, -1.130930, 1.189103, 1.526244, 0.512361, 0.888609, 0.979583, -0.724374, -0.420298, 0.940462, -1.064939, -0.889147, 0.648771, -1.249568, -0.865795, -1.184111},{-0.134921, 1.225045, -1.758925, -1.713957, -0.147154, 0.795575, 0.767067, 0.464082, 0.213657, 1.806733, -0.580101, -0.986631, 0.872718, 0.465967, -0.508427, 0.040571, 1.850089, -0.491478, -0.621999, -1.129911, -0.251773, -0.201238, -0.798248, -0.130791, -0.476679, -1.490976, 0.589532, 0.980650, 0.256826, -0.380490, 0.272324, -0.355710, 1.602459, -1.154216, -0.307533, 1.264177, 0.882755, -1.349425, 0.283046, -2.112981, -0.001048, 0.780999, 0.800544, -1.831084, -0.721248, 1.056284, -0.705620, -0.667369, 0.450904, -0.694927, -0.466283, 1.797283, -0.714347, 1.620001, 0.336897, -0.016315, -0.736611, -1.754394, -2.014161, 0.825534, 1.205493, -0.493626, -0.112432, -2.637434, -0.432196, 0.822411, 1.564890, 0.223561, -0.107189, -1.634620, -1.696088, -0.501989, 0.523430, 2.312533, 0.336890, 0.467503, 0.342081, 0.019993},{0.848301, -1.425093, 0.250738, 0.301428, -1.028058, -0.725592, 0.247033, -1.219590, 0.086736, -1.177147, 0.046982, -1.665985, 1.014362, 0.669777, -0.459041, -0.316267, 2.021546, 1.850146, -0.549750, 0.152177, 0.989484, -0.355763, -0.734635, -1.134689, -0.399952, -0.313106, 0.073819, -1.742633, -1.453009, -1.082370, -0.505660, -0.773260, -0.327922, 0.280637, 0.857202, 0.943625, 0.417016, -0.422392, -0.424820, 1.025866, 1.180620, -0.296822, 1.446678, 2.004036, -0.781300, 2.035346, 0.424972, -0.888828, -0.356094, 1.090740, 0.093771, 1.027730, 0.714733, -1.311189, 1.326149, 1.149200, -2.551438, 0.075473, 1.510007, 0.172890, -1.257171, -0.017602, -0.558240, -0.726994, -0.518474, -0.348883, 0.870631, 0.783886, 0.663670, 0.644612, -0.422684, 1.517428, 0.224719, 0.241895, -1.327371, 0.583187, -1.959967, -0.705315},{0.908878, -0.986813, -0.836421, -1.420992, 0.487869, 1.257154, 1.283191, 0.271254, -0.135664, 1.275580, 0.682204, -0.632874, -0.852281, -1.144876, -0.348689, 0.965143, 0.453901, 1.819203, -0.439399, -1.508192, -1.561276, 0.741237, 0.321618, -0.455286, -0.774328, -0.621301, -0.426533, 0.884346, -1.063988, 0.050748, 0.228143, 2.225764, 0.168100, -0.766920, 1.639626, 1.326157, 0.515797, 0.157158, -0.878175, -0.587059, 0.101234, -0.155803, 0.379865, 0.588271, 0.183332, 1.294197, 0.176462, -0.072745, -0.530853, -0.826163, 1.008276, 1.278210, 0.874815, 0.249230, -0.006443, 0.062524, 0.751586, 0.477706, 0.392043, 0.518250, -0.230709, 1.489071, -0.528150, -1.332252, 0.889908, -0.191562, -0.840935, -0.413099, 0.878272, 1.678485, 0.497133, 0.423680, 0.195745, -0.384168, 1.035340, -0.929515, -0.808904, -1.347662},{-0.273918, 0.139876, -1.094762, -0.193844, -0.538951, 0.803776, 0.067040, 0.217594, -0.045262, 0.622864, -0.259603, 0.133965, 0.330358, 0.542143, 0.267544, -0.865310, -0.880588, 0.293466, 0.011377, -1.119569, -1.012623, 0.829218, 1.102660, -1.802477, -0.664280, 0.080856, -0.117006, -1.581659, 0.576813, -0.358743, -0.194199, 2.131667, -0.125044, -0.218889, -0.711153, 1.041592, 2.555360, -0.555221, -0.249225, 0.180528, 0.016068, 0.804846, -0.162987, 0.350255, -0.842194, 0.103640, 0.047412, 0.064805, -0.441038, -0.874648, -0.367600, 0.987963, 0.391862, 1.029123, -1.373987, -0.499012, 0.330637, 0.512866, -0.068754, -0.577575, 0.173123, -0.306727, -1.133484, -0.082811, 0.723288, -1.692633, 1.502324, 0.930626, -0.652628, -0.573569, 0.487176, 0.090673, -1.051624, 1.260061, -0.154481, 0.462902, -0.041442, 1.588319},{-0.352502, -0.103134, 0.289841, -0.249468, -0.127424, -1.012458, -1.143714, -1.258163, 0.235282, -0.350851, 0.437177, -0.777682, 0.059175, 2.846341, -0.147655, -1.430254, -2.301934, -0.891261, 1.411213, 1.138786, 0.899504, -0.062406, -0.444303, 0.594730, -0.544058, -0.408814, 0.211310, 0.816789, 0.249269, -1.258749, 0.409911, -0.554021, -0.407889, -3.297393, -0.278690, -1.764538, 0.708005, -0.314724, -0.919089, 0.592901, -0.354021, 1.118101, 0.006255, -0.323349, -0.405807, 0.153530, -0.145457, 0.578585, -0.872034, -0.699754, 1.830768, 0.368944, -0.430372, 1.752143, 0.250254, 1.206864, -0.387502, 0.699601, -0.506551, -0.781728, -0.388819, 0.031216, 0.473931, 0.645220, 1.583547, -1.053105, 0.697712, 0.505048, 1.362652, -0.500526, 0.053838, 0.213795, -0.048797, -2.641989, -0.493176, -0.169809, -0.593351, 0.511616},{-0.949303, -0.822081, 0.564075, -0.913135, -1.717991, -0.760847, 0.084523, -0.968957, -1.033925, 1.280197, 0.764028, 0.605885, -0.213318, -0.725844, -0.115823, 1.323373, 0.468771, 1.262043, 0.348813, 0.230454, -0.222308, -0.618154, -0.461484, 2.099317, 1.011333, 0.961324, -0.447156, 0.499277, -1.205277, 0.467799, -2.524146, 0.287449, 0.717676, 0.995434, -2.651054, -1.773511, -0.824970, -0.606265, 1.418253, 2.016207, -0.853954, 0.383364, -0.551657, -0.179398, -0.182985, -0.241215, 0.834769, 0.336657, 0.493376, -0.325300, 1.593071, -0.045695, -0.052613, -1.505444, -3.191725, 0.782899, 2.949049, 0.431051, -0.026256, -0.264015, 0.687325, -0.460515, -0.451479, 0.179197, -2.031120, 0.650272, 0.657024, 0.138646, 1.050122, -0.376977, -0.078884, 0.467430, 0.560347, 1.024338, -0.761347, -0.607479, -0.550663, -0.682827},{-0.037029, -1.052041, 1.825436, -0.038898, -1.119630, -0.408579, 0.512295, -1.015250, 0.828210, 0.052273, 0.137126, -1.785646, 1.581047, 0.361796, -0.838835, -0.631210, 1.858388, 0.115889, -0.083235, -1.020360, 1.357291, -2.406872, 0.544593, -1.174759, 0.340937, -0.022874, -0.863904, 0.550734, -1.088109, 1.250825, -0.783654, 0.114274, 0.296317, 0.760602, 0.113668, -0.685028, -0.031796, 1.749965, 1.124318, -1.074562, -0.088723, 0.916713, -1.238765, 2.163080, 1.873252, -0.093380, -0.688938, 0.378908, 0.433377, -0.138728, 0.270311, -0.326573, -1.332965, -0.811860, -0.429031, 0.825705, -1.326610, -0.697306, 0.430352, 1.007885, -0.191883, 0.772322, 1.314361, -0.495928, 0.160796, -0.275761, 1.247252, -1.616880, 0.000019, -0.211116, -1.778729, 1.535332, 0.480978, 0.554485, 1.507189, -0.336869, -1.020055, 0.713001},{-0.168309, 0.728817, -1.112312, -0.867993, -0.627325, -2.793737, 0.631422, -0.634898, -0.618287, 2.970442, -0.888989, -0.076710, 0.054773, -0.931233, 0.723121, 1.284958, 0.761635, -0.703413, 0.119959, 0.378037, -1.387126, -0.048878, 1.327648, -1.082490, 0.692521, 1.478009, -0.789702, -0.178400, -1.254322, 0.178560, 0.032143, 0.797685, -0.130925, -2.705675, -1.436367, 0.307722, 0.136733, -2.111168, 0.763191, -1.504850, -0.368348, 1.674434, -0.051410, 0.806462, 0.840993, -0.172052, 1.373481, -0.554223, -0.433343, -0.693859, -1.764513, -0.246945, 0.101815, -0.231176, 1.631494, -0.103957, -0.162907, -0.157858, 0.203569, 0.290144, -0.170912, -0.382782, -1.380129, -1.821892, 1.153721, 0.237605, 0.965684, 0.910875, 0.245691, 1.106196, 0.099033, -0.094925, -0.337897, 0.987913, 0.639406, 0.482080, 1.124987, -1.870676},{1.462448, 0.347001, 1.780021, 0.306346, -0.066097, 0.995691, -0.374336, 0.162640, 1.349954, 0.978302, 0.949279, -0.722331, 0.503967, -0.314429, 1.089449, -0.282290, 0.243926, -0.502218, 0.483403, 1.116141, 0.313103, 0.894181, 0.581276, 1.185632, 1.652080, 1.727929, -0.695778, 0.669653, -0.088431, -1.631511, -0.274623, 1.470276, -2.430154, 0.143211, -2.332098, -0.196757, -2.450317, 0.539651, -0.460060, -1.205188, -0.933611, -1.782467, -0.188652, 2.101921, -0.723949, -1.175911, 0.497355, 0.010609, 2.191852, 2.622349, 0.607028, -0.642121, -0.441501, -0.381173, 1.275442, 0.712379, 0.566587, 0.383033, 0.663794, 0.413770, -0.803613, -1.257034, -1.483176, -0.599732, -0.928977, 0.456862, -0.877404, 1.316389, -0.194072, -0.310946, -0.319301, 1.187102, 0.749626, 2.360975, 1.009086, 1.439780, 1.476296, 0.939194},{0.633351, -1.099095, 0.525037, 1.735702, -1.183905, -0.084835, 0.346045, -1.491087, -1.192588, -0.134183, -1.082566, 1.142727, -1.339355, -0.740960, -0.198415, -0.109485, 0.313413, -0.424466, 1.212484, -0.694167, -0.008426, 0.483779, -0.350675, -0.765567, 0.693608, 0.790445, -0.291822, 0.849093, -0.181340, 1.357631, -1.136226, 1.100693, -0.106962, -0.248985, 0.494999, -1.078816, -0.465369, 0.246134, 0.157459, -0.826903, -0.296091, 0.135162, 0.701050, 1.046635, -0.106478, -0.999768, -0.367912, -0.244729, -1.191425, 1.012473, -0.238084, -1.011324, -0.472198, -0.755514, 0.736576, 0.355630, 0.145023, 0.614308, -0.037598, 0.193872, -0.123557, -0.766231, -1.019987, 0.580852, -0.403011, 1.554099, 1.598795, 0.796783, -1.727696, -0.197212, -0.271965, -0.946856, 1.093153, -1.375981, -0.109776, 0.582646, 0.854090, -0.254522},{-2.434335, 0.441372, -0.436359, -0.437018, 1.059195, 1.468783, -1.573800, 0.003638, -0.645920, -1.036902, 0.209216, 0.422240, 1.354180, 0.558828, -1.890382, 0.977197, 0.108853, 1.344887, 0.610867, 0.335155, -0.727176, -0.035770, 1.172640, 0.011100, 0.686143, 0.123857, -0.966847, -0.400353, -0.742087, -1.553420, 1.114752, 0.456618, -0.221733, -0.620512, 0.057800, 0.796488, -1.206434, 1.428214, 0.354188, -0.375889, -0.392763, -0.663038, -0.593097, 1.724100, -1.946256, 0.146691, -0.258187, 0.836906, -0.369883, 2.811967, 2.367579, 0.824219, -2.406929, 0.203706, -1.129251, 0.986731, 0.999603, 1.194130, 0.674176, 1.618335, 1.644339, 0.762267, 2.001656, -1.921755, -1.335361, -0.611404, -1.198744, -1.071569, -0.885216, -0.272349, 0.443988, 0.415750, 0.873767, -0.047276, 0.290435, 0.025210, 0.168928, -0.322938},{-2.264725, -1.783711, 1.937635, 0.859415, 0.108351, -1.499736, -1.718704, -1.946448, 1.075873, -0.732325, 0.935719, -0.532846, 0.790539, -1.495086, 1.242083, 0.563925, 0.399122, -1.150420, -0.264837, -0.745960, 1.359814, -2.158507, 0.735699, 0.147403, 0.254613, 1.124775, 1.170186, 0.666283, -0.933437, -0.620957, 0.481397, 0.839412, -0.480199, -0.971681, 1.110869, 2.170355, -0.435740, 0.396517, -1.366540, -1.144425, 1.126175, -0.192248, 0.175577, -0.016147, 0.984282, 1.463957, -0.086752, -0.708771, -0.518670, -0.613601, 1.032067, 2.098297, -1.277598, 0.190915, -0.553245, 0.908728, 0.402152, -0.580091, 0.293032, 0.124926, -1.368784, -0.629574, 0.106972, 0.816065, 0.085001, 0.616875, 1.226447, -0.982859, -0.731637, -1.213662, -0.419459, 1.543588, -0.703153, 0.872250, -0.885230, -1.012264, 0.667662, 0.159562},{-2.138492, -0.582993, -0.202634, -1.629734, -1.219979, -1.141763, -0.084733, 0.614431, 1.045069, 0.745858, 0.155088, 1.067254, -0.393081, 0.543170, -1.536330, 0.601889, 0.820842, 0.272444, 0.907191, 1.843516, -0.575657, 0.833182, 0.070945, -0.486414, -0.269298, -1.230154, -0.961728, 0.130479, -0.797986, -0.664926, -0.313777, 1.066252, -0.563210, 2.006484, -1.456256, -0.508943, 0.917548, 0.435836, 1.397204, -0.406290, -0.221291, 0.605254, -1.803957, -0.408138, -1.063953, 1.298778, -1.740324, 2.277213, 0.176468, 0.150876, 0.708542, 0.535544, -0.696359, -0.732104, -0.969499, 0.603817, 0.645951, 0.514441, 0.523299, 0.975724, 0.277269, 0.502987, -0.654805, -0.216060, 2.297225, -0.172235, -0.518129, -1.640162, -0.126091, 0.263509, 0.469625, -0.572648, 0.632563, -0.600141, -0.421252, 1.432344, 0.454030, -0.341951},{1.256044, -0.356718, -0.393297, 1.260975, -0.618793, 1.984526, -0.128026, 0.303255, -0.471488, 0.488988, 0.106802, -0.950930, -0.523540, 0.217745, -0.440713, -0.658401, 0.978497, 1.143514, 1.210303, 0.985043, -1.226633, 0.529957, 0.099981, -0.068721, 0.603564, -0.195730, -0.780265, 1.085587, -0.609054, -0.577409, -1.013680, 0.417112, -0.007362, -0.915763, 0.825240, -0.202237, -0.189184, 0.100509, 0.032644, 0.642769, -1.166600, -1.868350, -0.026001, 0.243451, -0.778813, -1.596695, -1.439458, 1.544831, -0.843818, 2.338537, -1.199818, 1.393075, -0.194469, 0.120651, 0.562927, -1.471338, -0.381979, 0.593535, -0.908995, -0.145028, -0.171666, 0.809966, 0.256238, 0.376007, 1.346247, 0.989304, 0.490528, -0.438267, -0.674563, -0.550039, -2.320782, 1.810103, -0.230170, -0.808883, -1.153781, -0.880303, -1.242964, -0.493150},{0.414976, -0.210565, 0.991969, -0.827038, 0.413781, 1.151631, 0.160588, -1.299115, -2.828233, -0.443038, 1.210486, 0.696227, -0.016639, -2.652557, -1.077355, 2.105125, 0.291877, -0.209402, -0.663822, 1.086780, -3.096286, -0.165011, -0.045946, -1.253249, -0.126878, 1.332047, -1.067598, -0.932693, -0.120999, 0.473525, -0.675822, -0.330930, 0.263953, 1.896064, -0.519440, 1.559252, 1.262905, 0.317059, -0.608664, -0.590207, 0.015606, 0.125534, -1.315336, -0.284353, -0.522417, -0.271776, 0.960368, 1.499497, 0.272269, 0.667418, 0.369236, 1.113002, -1.168866, -2.548553, -0.032685, 0.837045, -0.056695, 1.854863, -0.462685, 0.306816, -0.844254, 1.494864, -0.436467, 0.265819, -1.671558, 1.120447, 1.108857, -1.154686, 1.469035, 0.209610, 1.018214, 0.665608, 1.092597, -0.179354, 0.138780, -0.631410, 0.078410, 0.705789},{0.647809, -0.588257, -1.402817, -0.776963, -0.968334, -0.487088, -0.879567, 1.368832, -0.459148, -1.861282, -0.420224, 0.420787, -0.302518, 0.661218, 1.505513, 0.778221, 0.234777, 0.733208, 0.399425, 0.011303, -0.011213, 0.867528, 0.479961, -0.548400, -0.049783, 1.224284, 1.381074, 1.848352, -0.398928, -1.032140, -0.061142, -0.305708, -0.324160, 2.215864, -0.856789, 0.408134, 0.462967, 0.512007, -1.976030, -0.054662, -1.507002, 0.547863, -0.828717, 0.888591, -1.858195, -0.996209, -0.517888, 2.166663, -0.242751, 2.129240, 0.267383, 0.143251, 0.628506, 0.999572, 0.290127, 1.604778, 1.013422, 0.399437, 0.858811, -0.737820, -0.157622, 0.597444, 0.390285, -0.314024, -0.024057, 1.782177, -1.349339, -0.987311, 0.194321, 1.298508, 0.901874, 0.482001, -1.523336, 1.248263, 0.324757, -1.628398, -0.285809, -0.478749},{-0.377737, -0.756328, -1.595870, 0.390272, 0.271167, -0.008235, 0.499870, -0.674456, -1.689897, -2.061901, -0.811821, -1.159729, -0.904202, -0.767026, 0.128174, 0.803203, 1.102914, 0.528491, 0.837976, 0.397724, -0.254230, 0.024578, 0.075558, 2.223938, -0.146099, -0.142776, 1.635130, -0.879381, -0.246954, 0.649074, 0.037982, -0.432897, -1.432192, 0.666757, -0.745400, -0.781384, 0.252588, -1.452446, -0.443504, -0.362509, 1.010613, 0.282973, -1.318304, -0.105385, -2.017797, -1.193019, 0.735275, 1.080162, 0.446281, 1.646885, -0.598736, 0.451698, -1.496676, -0.164984, -1.214806, 0.170460, 0.177840, -1.229625, -0.393497, -0.357236, 0.539927, -0.412971, 0.841212, -0.276896, 0.372192, -2.762941, -0.071363, -0.534041, -1.579683, 0.103165, 0.558229, 1.687418, -0.971590, -0.842405, 1.490013, -2.153980, -1.337731, -1.440365},{-1.633526, -1.919367, 0.111119, 0.308576, 0.876802, 0.843689, -0.698874, -0.124597, -1.146616, 0.617994, -0.453233, 0.475202, -0.890071, -1.204002, -0.442194, -1.777622, -0.050682, -0.317480, 0.044703, -2.638552, -0.517858, 1.373274, 0.679241, 1.542943, -1.052725, 0.223275, -1.895384, -0.351210, 0.415618, 0.574033, -0.301329, 2.124356, -0.213112, 0.532364, -0.108247, 0.715394, 0.440344, -0.703608, -0.249606, -0.257570, 0.181192, 2.027377, 1.784473, 0.563748, 0.908539, 1.945281, 0.852938, -1.353737, 2.019697, -1.123263, 1.254984, 0.376536, 1.187342, 0.872974, -0.251250, -0.230277, 0.441493, 1.043491, 0.601259, -0.880225, -0.353411, 0.484178, 0.642604, -1.867233, 1.561652, -0.009055, 0.312991, -0.946170, -1.191042, -0.265892, -0.076039, 0.399855, 2.416505, -0.226816, -0.064407, 0.167759, 1.124242, 1.637652},{0.755235, 1.801144, -0.342240, 0.003927, -1.266358, 1.143810, 0.279710, 1.805193, -0.673993, -1.159834, -1.414336, 0.759602, 0.512553, -0.785268, 0.940610, -0.751354, -1.621398, -0.605173, -0.074304, -0.166564, 1.072118, -1.652279, 0.503626, -1.771977, -0.767978, 0.587651, -0.232730, 0.422957, 1.298632, 0.463306, -1.441397, 0.508672, -0.110076, -1.423225, -1.289855, -0.425490, -0.278810, -0.317115, 1.423356, 2.840498, -1.193720, 0.584685, -0.355222, -0.291965, 1.188045, 0.627917, -0.059346, 0.221885, -0.550433, 0.050638, -0.664846, -0.450705, -1.579008, -1.406081, -0.185141, -1.552971, 0.480642, -0.383395, 1.962535, 0.467992, -1.164178, 0.238791, 2.155732, 0.492623, -0.700202, -0.686811, -2.061064, 0.321904, 0.024205, -0.629270, 0.156408, 1.652767, -0.072915, -0.336346, -1.260794, -0.833699, -0.889465, -0.147757},{-0.306177, 0.271849, 1.877547, 0.650714, -0.643484, 1.212658, 0.654943, 0.122603, 0.323948, 0.477905, -2.287622, -0.280224, 0.414778, -0.802562, 0.906246, 0.685932, -1.462217, -0.220326, 1.682728, -0.216195, 0.108231, -0.256497, 0.640668, 1.099795, 0.093687, 0.116419, 1.015183, 0.782016, -1.016741, 0.799096, 0.174431, 0.067613, 0.244090, -0.105395, -0.426218, 0.441283, 0.646144, -0.120205, 1.601986, -0.841303, 0.785476, 1.893366, 1.692717, 1.939757, -0.816666, -0.146466, 0.141047, 0.509940, 0.309102, 0.124039, 0.371926, 0.214084, -0.900639, -0.538085, 0.214489, 0.460174, -0.127757, -0.614065, -0.182765, -1.886344, 1.046352, 0.536174, 1.023579, 1.245248, -0.897741, 1.294312, 1.679718, -0.845431, -1.097340, 0.354883, 0.261700, 1.363464, 0.884867, -0.054084, 0.450780, -0.198025, -0.710624, -0.431659},{0.071972, -0.991026, 0.797323, -0.100653, -0.968163, 0.387849, 0.175307, 0.199414, -0.788862, -1.788640, -0.057847, 0.551551, -0.192837, 3.284519, -0.570392, -0.049074, -1.692433, 0.399686, 0.493987, -0.062475, 1.391165, -0.245560, 0.458868, 0.727436, -1.335090, 0.598340, 0.120799, 1.437903, 0.236990, 0.696863, -0.670009, -0.053890, -1.117709, 0.301631, -1.495107, 1.718483, -0.381179, -0.088806, 1.885621, -0.767734, 1.328298, 1.369586, 1.278922, -1.364777, -0.568132, -0.728238, 2.030568, 1.492449, -0.152545, 0.167188, 0.140158, -0.184202, -0.786010, 0.279648, 1.577892, 0.756600, 1.140902, -0.448955, -0.009703, 0.592027, -0.782566, -0.078091, 0.310218, 0.862618, 0.882558, -0.330205, 1.153471, -0.495149, -0.498383, -1.311802, -1.400587, 0.948587, -1.350865, -1.012108, 1.260628, 0.414157, 0.500043, 1.163272},{-0.140439, -0.798171, 0.468447, -0.084602, 0.453367, -1.467562, 0.117250, -1.056344, 0.513109, -1.640590, -0.299873, 0.012836, -0.394875, -1.258223, 0.489100, 0.390216, -0.827415, 0.959850, 0.338074, -0.534541, 2.256539, -0.910970, 0.220396, -0.670082, -0.928579, 0.865107, 0.388281, 0.201004, 0.656966, 1.648523, -0.599666, 0.561627, -0.079020, 0.544062, 0.355527, 0.382490, -1.479913, 1.035812, -2.032553, 0.617140, -0.879743, -2.141421, -0.758340, 1.420735, -0.264934, 0.782804, -1.178295, 1.396470, 0.119864, 1.584353, -0.533996, -1.258054, 0.213300, -0.855710, -1.361330, 0.917178, -2.245764, 0.334104, -1.104516, -0.616759, -1.401975, -0.555312, -0.417398, 1.158374, -0.393349, 0.292304, 1.434196, -0.488969, 0.059442, -1.497746, 1.043865, -0.430715, 0.288801, -0.643494, -0.882524, 0.452715, 1.754069, 0.278039},{-0.287003, 0.450442, 0.460472, -0.024542, 0.516941, 1.018216, -0.305067, 0.818469, 0.294979, -0.812129, 0.436440, -0.040686, -1.409562, 1.248765, -1.356118, -0.179043, -0.883749, 0.295285, -2.571411, -1.588591, 0.434975, -0.170459, -0.757646, -2.447433, 1.397195, 0.469228, 0.341026, -1.023450, 1.016378, 0.393043, -0.114916, 0.172638, 0.694516, 0.263561, 2.205423, 0.050186, -1.071245, 0.379480, -0.492695, -0.973263, 0.144834, 0.500343, 1.149662, 0.628437, -0.221532, -1.206153, 1.110750, -0.109176, -1.936686, 0.708037, 1.383021, 1.288617, 0.917562, 1.492523, -0.827008, 1.360496, 0.331107, -0.911610, 0.851769, 0.666430, 1.142901, 0.363566, 0.036247, -0.948170, -1.100364, 1.334596, 0.477098, -0.235414, 0.633469, -0.671710, 0.198120, -0.431003, 0.822433, 0.040442, -0.616010, 2.328394, 1.582570, -0.436777},{-0.444853, -1.611651, -0.360088, 0.901310, 1.563335, 1.356940, 1.328872, 0.504040, -0.664969, -0.032223, -0.044636, -1.798198, -0.820888, -1.178163, 0.929260, -1.658604, 1.786468, -2.191189, -1.575294, -0.301088, -1.533801, 1.420701, -1.615170, -0.936525, 0.345965, -0.759846, 0.547306, -0.000198, -0.799006, -0.423789, -0.289665, 0.971736, 0.343740, 0.197439, -1.062391, -2.256268, -0.207953, 0.028176, 0.468756, 0.942121, -0.472916, 1.162174, 1.624406, 0.021522, 1.131186, 0.065110, -0.083440, 0.882136, -0.792801, -0.743504, -0.169219, -0.846466, -1.528396, 0.772780, 1.057739, 0.329508, -0.355699, -1.396671, 1.135835, 1.236245, 0.717682, 0.495323, 0.481686, -0.053298, 0.588412, -0.711260, 0.746067, 0.157637, 0.624819, 0.049600, 0.482982, 0.701795, -0.188841, 1.775003, -1.938358, -0.372426, 1.477216, 1.310593},{-0.032978, 0.926239, 3.114714, 1.176881, 0.027547, -0.622121, -1.201548, -1.878994, 1.490461, 0.331162, -0.772986, 0.373792, -0.434617, -0.227246, -0.585325, 0.849867, 1.024123, 1.819340, -0.676727, -0.260400, -1.000832, -0.496468, -1.251759, -0.729459, -0.559508, 0.640571, 0.031879, -1.176535, 1.763597, -0.715458, -3.070272, -2.173423, 0.761762, -2.491358, -1.596763, 0.707817, -1.435499, -0.578195, 0.300870, 0.060674, 0.901910, 0.909899, -1.102195, -0.990288, -1.227445, -0.932084, 0.284506, -0.433993, 1.198338, 0.127714, 0.208685, 0.887200, 1.134927, 0.924610, -0.015256, 0.632027, -0.326469, -2.274174, -0.242207, -0.095855, -1.347852, 0.908629, 0.842778, -2.644035, 0.379204, 0.718849, -0.459910, -1.154447, 0.442624, -0.626356, -0.243843, 2.071395, -0.137463, 0.886284, -0.424034, 1.738732, 0.071123, -2.256211},{-0.614983, -0.517496, -0.787417, -0.502090, 1.110760, -1.159127, 1.248788, -1.896155, -0.133103, 0.232592, -0.727417, 0.299728, -0.544778, -0.410131, 0.368027, 1.344875, 0.514353, 1.393570, 0.482841, -3.394181, -0.471937, -0.502300, -0.034626, -0.273206, 0.511047, 1.224203, -0.596039, 0.528066, 1.148359, 1.143762, 0.176780, -0.059329, -0.329560, -0.792101, 0.182124, 0.444035, -0.402081, 1.216829, -0.389736, 1.091181, -2.187316, -1.629840, -0.571051, -1.138481, 0.375613, 1.882255, 0.292603, 1.871938, -0.016963, -0.910293, 1.739469, 0.526564, 0.740495, -0.763538, -0.519417, 0.853182, -0.400675, 1.418180, -0.360373, 1.261352, -2.046629, -1.244922, -1.566233, -0.873308, 0.482665, -0.109967, 0.694872, -0.010719, 0.540742, -0.336822, -0.510030, 1.878300, 0.463406, -1.072630, -0.522184, 0.344662, 0.772014, 1.238957},{0.765674, -1.471298, 1.838993, 0.099613, -0.204912, -2.166728, 0.860386, -1.355314, 0.986304, 0.988514, -0.191335, 0.816882, -0.240776, -0.121839, 0.698014, 0.125581, -0.627676, -0.604728, 0.140820, 0.317407, -0.444889, -1.266448, 0.944734, -0.510665, 0.623937, -0.872791, 0.745167, -0.587052, -1.932286, 0.496893, -0.174786, -2.219824, -0.630332, -1.055064, 1.527207, -0.167663, -0.549227, -2.025542, -0.990053, 0.137852, 2.349785, -1.334924, 0.375696, -0.129375, -2.025633, -1.098710, 1.114398, -2.206362, 1.096539, -1.524407, 0.752557, 0.777208, 2.934286, 0.038286, -1.854265, -0.338421, 0.374355, 0.874302, -1.940818, 0.697743, -0.512854, -0.176307, -1.429692, -0.053582, -0.719315, 0.218632, -0.222592, 0.250028, 1.286727, -0.965131, 1.179155, 1.561370, 0.887475, 0.459901, 0.597612, 1.179360, 0.080602, 0.550668},{1.322576, 0.512133, 1.387585, -0.931049, -2.140726, -1.472145, 1.452496, -1.379498, 0.619569, 0.992006, -1.364622, -1.506045, 0.851938, -0.203293, 0.336196, 0.263898, 0.560382, 0.332660, 0.378854, -0.105401, -1.969645, 1.979931, 0.813939, -0.403760, -0.358722, -0.646616, -2.036052, 0.382153, -0.347265, 0.390990, -0.137326, -0.175475, 0.714595, -1.302459, -1.478267, -0.915258, 1.421189, 0.806441, -0.498473, 1.696528, -1.136384, 0.312158, 0.175892, 0.191061, -0.216302, -0.295133, -1.373708, 0.822741, -0.136350, 0.045951, -0.178061, -1.729735, 0.346113, 2.973233, -1.793362, -0.916420, 1.306652, -0.041801, 2.839503, -0.710559, -1.440602, -0.621331, -0.862637, 1.563708, -0.598600, 0.289791, -0.887113, 1.911869, -1.262675, 0.937276, -0.288101, 0.225744, 1.108529, -0.316643, 0.328703, 1.342690, -0.483609, 1.473606},{1.863525, -0.015302, -0.295225, -0.728372, 1.234984, -0.593644, 1.289953, 0.660263, 0.343686, -0.392324, -1.744545, 1.727825, 0.193220, 1.223472, 0.343208, 0.756474, -0.425019, -0.745840, 0.068457, -0.709337, -1.303637, -0.264937, -1.192300, 1.190311, 0.127616, -0.334966, 0.674814, 1.213149, -0.333019, 1.724054, -1.685376, 1.488464, -1.093901, -1.027404, 0.471126, -0.520062, -0.595238, -2.610425, 0.133217, -0.314334, 0.325510, 0.278712, 1.326366, 0.895529, 1.621555, -1.752394, 0.917377, 0.114045, -1.137714, 0.208324, 0.775807, -0.456865, 0.109751, -0.724135, 0.999144, -0.201400, 0.168678, 0.706375, 0.034848, 0.101480, -2.104329, -0.502705, 0.672202, 0.733296, 0.141764, -0.861143, 0.094574, -1.303282, -0.369155, -0.484379, 0.636437, -0.622931, -1.268957, 1.920499, -0.911735, -0.525717, 1.561924, 0.430220},{3.234635, -0.012324, -0.272668, -0.372050, -0.568804, 1.142724, -0.619534, -1.107279, -0.277588, 2.277716, 0.420046, 0.259845, -0.190523, 0.391994, 0.668579, 0.002105, -0.200909, 0.255389, -0.651693, -0.212845, 0.428640, 1.370192, -0.133024, -1.248874, -0.394579, -0.225376, 0.120071, 0.886213, 0.089510, 0.808670, -0.365438, -0.269956, 0.604179, 1.302976, -0.629078, -0.330588, 1.660086, 1.065635, -0.251314, -2.222538, -1.377715, 0.055253, -0.473784, 0.337890, 0.107271, -0.315722, 0.337006, -0.068169, -0.543799, -0.970776, -0.620895, -0.976586, -1.569702, -0.229627, 0.370939, 0.206875, 0.779906, 0.008092, -0.282617, 0.076970, 2.045991, 0.332645, 0.491967, -1.195021, -0.520673, -0.066847, -2.559779, 0.920906, -0.139165, -0.604452, 0.077014, -0.095687, 1.513830, -0.820638, 0.366900, 0.591395, 0.364633, 1.895039},{-0.529116, -1.261102, 0.866323, -0.179370, 1.066506, -3.613671, -0.819154, 0.974568, 1.386672, -0.543757, -1.361570, 0.147042, -0.340240, 0.414485, -1.259728, 1.681601, 0.879043, -0.437367, 0.294549, -0.694506, -0.455968, 0.268024, -0.699344, 0.850233, 0.373980, 0.199154, 0.032268, -0.549151, 0.570711, -0.916532, -0.807009, -0.845838, -0.614997, 0.936483, 0.818014, 1.680230, 1.761259, -0.310685, 1.319555, -0.516004, 0.626317, 0.555516, 0.463281, -0.667257, -1.761634, 1.125160, 0.422113, 0.208232, -0.892483, -1.254971, -1.305292, 0.675620, -0.113816, 0.313476, 0.468666, 0.755908, -0.734826, 0.910307, 0.827228, 0.372576, 0.546781, -1.090377, 1.540913, 0.232693, 0.121192, -0.383398, 0.074722, -0.168539, -0.672897, 1.112505, 0.156670, -0.037680, 0.786677, -1.166729, -0.625424, -2.069217, 0.862614, -1.017536},{-0.567793, -1.029279, 1.523033, 1.299283, -0.431141, -0.698312, 1.691998, -0.433578, 1.045027, 0.381270, 0.718543, -0.283262, 0.374744, -0.341694, 0.509613, 0.382584, -0.645747, -2.377685, -0.780451, -0.225451, 1.067294, -0.508319, 0.359490, -0.262492, -0.684177, 0.971764, -1.228947, -0.331035, 0.635171, 0.365187, -0.544720, -0.241915, 0.041045, 0.310127, 0.507719, -2.411029, -1.850095, 1.098217, -0.062179, 1.121665, 1.783287, -1.689279, 2.286059, 1.507718, -1.406774, 1.526200, -0.444312, -0.442087, 0.953068, 0.571995, -0.872268, 0.680422, 0.726382, -0.609674, -2.363361, 0.205650, -0.382611, -1.193468, 0.512836, 1.112459, 0.812513, 0.413845, 0.432907, 0.269485, -0.100307, -0.073251, -0.476123, -1.420913, 0.149213, 0.271028, 0.152959, 0.057276, -1.690858, 0.221296, -0.462938, -1.701700, 1.961795, -0.516379},{0.218407, 1.181802, 0.181919, 0.848878, 0.341608, -0.642710, -0.448277, 0.687890, -0.876496, -0.459581, 0.653512, -2.028372, -1.378749, 1.011706, -1.278615, -0.136650, 1.695077, 0.066034, 2.058214, -0.861717, -1.386093, 0.688264, -0.128828, 1.031126, 1.451202, -0.450070, 0.873950, -0.827898, -0.029525, -0.991942, 1.334117, 1.607511, 0.714986, 0.476158, 1.961501, -0.070024, 0.114157, 2.063383, -1.406378, 0.221773, 0.005072, 0.200893, -1.993666, 0.001153, 0.058788, -0.935121, -0.667594, -1.319818, -1.701823, 1.803408, -0.017810, -0.857784, 0.853466, 1.065246, -0.142640, 1.732521, -0.500716, -0.117640, 0.591472, -0.582545, -1.955060, -0.838886, -1.372337, 0.395758, 1.142159, -0.527557, 2.272496, 0.190869, 0.235100, 0.515277, -1.162014, 0.527232, -0.612645, -0.502264, -0.445555, 0.711764, 0.804800, -0.496872},{-1.495119, 1.512544, -0.255766, 1.302364, 0.257540, 0.000324, -0.966229, -0.323123, -0.614925, -0.234242, 0.597486, 0.966469, -0.806529, -0.966807, -0.240967, -1.338919, 0.317012, 0.135306, -2.321506, -0.727775, -0.117094, -0.389260, 0.374027, 1.077581, 1.195623, -0.043590, 0.156842, -0.467380, 0.163326, 0.159436, 1.167011, -0.476705, 0.380092, -0.172257, 0.820911, -1.698462, 0.010723, -0.114703, -1.235758, -0.328556, -2.236223, 0.035241, 0.000727, -0.238981, -0.308721, 0.662033, -1.206479, -0.435696, 1.488128, 1.249597, -0.287337, 0.346152, 1.185708, -0.698874, 1.142639, -0.825363, -1.505967, -0.324379, -0.710852, -0.500942, -0.033157, -0.180936, -1.106351, -0.348973, -0.817271, -1.204353, 0.428943, -0.331239, -0.490635, 0.908053, 1.363681, 0.651320, -0.821309, -0.814610, 0.775032, -0.689610, 1.375019, -0.149324},{0.834010, -0.170535, 0.359864, -2.060640, 0.571266, 1.166672, 0.847486, 0.312560, 0.991073, 2.017406, 0.043047, 0.094895, -0.087498, -0.594943, -0.112479, -0.545869, -0.317971, -0.082122, 1.402803, 1.480353, -0.902650, -0.449577, 1.291040, 0.603515, -0.214116, -0.535642, 0.607466, 0.544095, -2.355791, 0.678055, 1.276930, -0.065261, 0.401781, -0.781649, 0.215514, -1.018114, -0.387951, 0.371149, 0.146253, 0.698137, 0.432378, 2.241854, -1.533266, -0.200637, -0.571096, 0.730792, 0.131673, 2.046521, -0.928081, -0.000387, 0.207334, -0.571455, 1.254430, -2.547381, -0.235246, -0.508467, -1.516177, -1.372239, -1.618997, -0.059076, -0.631756, -0.707804, 0.520395, 1.161468, 2.469577, -1.381828, -0.879954, -0.084113, -0.563149, -0.950947, 0.339622, 0.688892, 0.347567, -1.282655, 1.860428, 0.795875, 1.092604, 1.411208},{0.150576, 0.561989, 1.414435, -1.102731, -1.263632, -0.837274, -0.421445, -0.321446, 0.638045, -0.462159, -0.009828, 0.595453, -0.954261, 0.792618, -1.135407, -0.287053, -0.497659, -0.343395, -0.268438, -0.176749, -1.727763, 0.002518, -0.422258, 0.329114, 1.722252, -1.531295, -2.409313, -0.288746, 0.835955, 0.235888, -1.244709, 0.848303, 0.443694, 0.837227, -0.523591, 1.189405, -1.435482, 0.272744, -0.215524, -0.872140, -0.120413, 1.083881, -0.204008, 0.573328, 0.431566, 0.204649, 0.253776, 0.103888, -1.614312, -0.696778, 0.338073, 0.560599, 0.268525, 0.427351, -0.832557, 0.961510, -1.877862, -0.892016, -0.400055, -0.124686, -0.330152, 0.766541, 1.708445, -0.612400, -1.597340, -0.475309, -0.181839, -0.406017, -0.393127, -1.306148, 0.066185, 1.433498, 1.102757, 0.454114, -1.648032, 1.582394, -0.756804, 1.694048},{0.039441, 0.904857, -1.619829, -0.340033, 0.392811, -0.526018, 2.415454, -0.147070, -0.481958, 0.687364, 1.469707, -1.029464, 0.508989, 0.451694, 0.716436, 0.522696, -1.087311, -1.364212, -0.682905, 2.273684, 1.171454, 2.002204, 0.427941, 0.458805, 0.311996, -0.425911, 0.257438, -3.172763, 0.702460, -0.314160, -1.043833, 0.515086, 0.736139, -0.861110, -0.667857, 0.298765, 0.063144, 0.408071, 0.803020, -0.790644, 1.908617, -0.463852, 1.787635, 0.456099, -0.294843, 0.741446, 0.490601, -1.173441, 2.137996, 0.080092, -0.369963, -0.255941, -1.972031, -0.470472, 1.036052, 1.341822, 0.469680, -0.287008, 1.356426, -0.626299, 0.137159, -2.191986, 0.592699, 2.063872, -1.102245, -1.827147, 0.898365, 0.975464, -0.485572, 0.438021, 0.478645, 0.970578, -0.360919, -0.148367, -0.545859, -2.018223, -0.697892, -0.487072},{0.690407, 0.507208, -0.158673, 0.983072, -0.038370, 0.343223, 0.329753, -0.851926, -0.134866, -0.815083, -1.182985, 0.129443, -0.135894, -0.262786, -2.172613, 2.570275, 0.524637, -0.878288, -0.876530, 0.579752, -1.116915, 0.061912, 0.316278, -0.529233, 0.840921, -0.440765, -0.066688, 0.713267, 0.030396, 0.602101, -0.534761, -0.248854, -1.269003, -0.216857, 0.176801, -0.893966, -1.102746, 0.110978, -0.224706, 1.651422, -1.450521, 0.071620, 2.550453, 1.165462, -0.324403, 0.831334, 0.256665, 1.047417, -1.744237, -1.335906, -0.162432, 1.588354, -0.617603, -0.709083, -0.003673, 0.957292, 1.027677, 1.265043, 0.048469, -1.112334, 0.011628, 1.827265, 0.919440, -0.107592, 0.553799, 0.286428, -1.338001, 0.034888, -0.306943, -0.643053, 0.005156, -0.678169, -0.356807, 0.424696, 0.693886, 0.299342, 1.971104, 0.715925},{-0.589346, 1.550422, -1.227324, -0.774118, -0.582954, 0.520180, -0.769414, 1.327135, -0.400910, 0.598799, -0.638538, 0.025339, -0.042241, -1.188739, -2.505129, 0.263066, 0.367570, -0.375802, 0.311638, -1.903723, 0.852605, -0.090436, 0.223835, -1.903159, -1.897878, 0.595635, -0.227384, -0.692920, 0.417734, -0.480337, -0.371296, -1.165671, -0.218189, -0.467101, 0.461589, -0.965562, -1.069225, -0.576421, 0.881452, 0.091004, 0.277657, -0.327212, -0.842019, 0.388208, 0.561105, -0.898102, -0.489977, 1.910272, -0.099298, -1.065164, 0.453473, -0.537534, 0.479933, 0.499327, 0.421544, 0.750707, 0.868556, -0.733112, 0.736531, 1.370203, -0.381749, -0.545507, 1.879977, 0.327904, 0.475688, 0.107366, 0.617821, 0.399576, 0.118861, -1.081135, -0.376596, -0.203356, 0.451271, 0.197237, -0.904456, 0.466041, -0.412150, -1.331214},{-1.022156, 0.072652, 1.368676, 1.043260, -1.264928, 2.054355, -0.931629, -0.731374, -0.625396, -1.535623, -0.196977, -2.215225, -0.810515, -0.809544, 0.621106, -0.262182, 1.335201, -1.177040, -0.464073, 1.230384, -0.662867, -0.188475, 1.075219, -1.146926, -0.682883, 0.305231, 1.205352, 0.297578, 0.962816, -0.904864, -0.036475, 1.621611, 0.047382, -1.469211, -1.001206, 1.268574, -0.757875, -0.202563, -1.029322, 0.831963, -0.467787, 0.485679, -1.367486, -0.001003, 1.399268, -0.890535, -0.754931, 0.976856, -1.364124, 0.227341, -0.937806, -0.094460, -0.289527, -0.609746, -0.713362, 0.418922, -1.518741, -1.167063, -1.032481, -0.420353, -0.763358, -1.107838, 0.404817, 0.148159, 1.714983, -0.013329, 0.787134, 1.582585, -0.178117, -0.476165, 0.710962, -0.866173, 0.673996, -0.737084, -0.818496, 0.025424, 0.059977, -1.260955},{-0.336453, -0.039109, -0.482695, 1.122303, 1.132262, -0.355457, 0.422899, 0.183923, 0.356379, 2.400410, 1.309530, -1.768046, 0.882632, -0.873264, -0.091394, 0.765727, -0.063793, 0.454375, 0.334729, -1.889558, 1.517442, 0.993144, -0.500413, 0.538952, 0.044500, 1.425186, 0.468528, 1.219498, -1.182987, 0.853623, 0.245813, 1.043308, 0.001225, -0.654557, -0.505157, -0.357200, 0.181041, 1.006773, 0.584719, 1.634957, -0.586219, -0.709413, 0.168020, -0.780665, -0.469379, 0.581921, -0.381891, 1.275088, -0.285239, 0.500301, 0.166161, 0.734128, 0.756717, 1.152866, 0.366462, 1.272362, -0.307832, 0.453643, 1.058000, -0.045915, -0.858771, 2.832271, 1.451362, 0.452337, 2.064761, -1.310287, -0.331373, 0.938775, -0.624024, -0.419388, 0.328621, 1.231015, 1.476203, -1.603436, -1.299379, -1.350254, -1.007023, -0.264912},{0.290920, -0.047846, 1.754905, 1.082138, 0.106559, -0.443443, -1.125525, -0.497983, 1.410143, 0.309573, -0.768690, -0.793618, -0.008110, -1.328510, 1.721757, 0.466377, -0.013219, 0.288498, -1.197383, -1.879688, 0.808868, 0.259307, 0.950441, -0.653401, 0.649731, -0.034238, -0.064212, -0.041192, 0.028536, -0.786469, 0.047527, 1.916289, 1.188708, 0.081987, -1.183790, 0.569720, 0.856050, 0.393319, -0.198970, -0.738508, 0.583530, 1.452379, 0.474679, 0.498293, 0.440260, -0.073503, -0.069441, -0.424878, 0.158219, 0.147751, 0.320097, -0.793331, -0.467012, 0.105291, -1.382195, 0.759213, -0.412731, -1.495574, -0.769785, 0.271052, 2.412286, 0.488978, 0.826157, 0.112239, 0.479495, -0.741834, 0.324045, 1.515928, 0.194653, -0.716349, 1.405032, -0.850792, -0.198325, -0.242293, -1.086171, 0.414090, 0.940156, -1.072339},{-1.244472, 0.598762, -0.499893, 0.304610, -0.857995, -0.815006, 0.344444, 0.643038, 0.451915, -0.176618, 0.081860, -0.993875, -0.561321, -1.571792, -0.015564, 1.444777, -0.035849, -0.106753, 0.493830, 1.246764, 0.238798, 0.449282, -1.826827, 1.650674, 2.056165, -1.518787, 0.702494, 1.911000, -0.338569, -0.004251, -1.295322, -1.547274, -0.504992, -1.509119, 1.150781, -1.171936, -0.035851, 1.322846, 0.566192, 0.735900, 1.051870, -0.161280, -2.568883, -1.651358, -0.535735, 0.869497, 1.040095, 0.576147, -0.583671, -0.147846, -1.767373, 0.156323, 0.958574, 1.457819, 0.152706, -0.925404, -1.146132, 0.003926, 0.269998, -1.067421, -0.407572, 0.482754, -0.253071, -0.035136, -0.666194, 3.103048, 0.113843, -1.006533, 0.252335, 1.056349, -1.043074, 1.209865, -0.048703, -0.763917, 0.396893, 0.093916, -0.380905, 0.346198},{0.316347, 1.011670, -1.157726, 0.440992, 0.911454, -1.658174, -0.592503, -0.375146, 0.764825, -0.563756, 1.715678, 0.514630, 0.900863, -2.578215, -0.036487, -1.469548, -0.012584, 1.032870, 2.272986, -1.670951, 1.773349, -0.221108, -1.515062, -0.956550, -0.101911, 0.226111, -1.074964, -1.509323, 1.219527, 0.881987, 0.639035, 0.630632, -0.400999, 0.480166, -0.093463, 1.213582, -1.204577, -0.718805, 0.495796, -0.575227, 0.629254, 0.514921, 2.316512, 0.347029, 1.358266, -0.728511, 1.666334, -0.428846, 1.536652, -2.274156, -0.806944, 0.539080, -0.363683, 0.166317, -1.231850, -1.347905, 0.679334, 0.227801, 0.445800, 2.657903, -1.214638, 0.161537, 1.755998, 0.531522, 0.030049, -0.772518, 0.137051, 0.421461, -1.525825, 0.519215, 0.473259, 1.501782, 0.999283, -0.608852, -1.613718, 0.554715, 0.192659, 0.931253},{1.862446, 1.256734, -0.595510, 2.072338, -0.753815, -1.209000, 1.108174, 1.378758, 0.558116, -0.736574, -0.787949, -0.321297, -2.234877, -0.790356, 0.291903, 1.092051, -0.670775, 0.494325, -0.696895, -1.502121, -0.481088, -1.567953, -0.137886, -0.550064, 1.555735, 0.140434, -0.083106, 1.458082, 1.143155, -0.471037, 0.400991, 0.324377, 0.693575, -0.701523, -0.384508, 0.407869, 1.665361, 1.703844, 2.328223, 0.432721, 0.982944, -0.417088, 0.612290, 0.077696, 1.253857, -0.949802, 1.071051, 0.043167, 0.558133, -1.162585, 0.317012, -1.267887, -0.686337, 1.893438, 0.340454, 1.176855, -0.520826, 0.920354, -0.819750, 0.678910, -1.125715, -0.662035, 0.332590, -1.630807, 0.882160, -0.189640, -0.509401, -0.029692, -2.066550, 0.594170, 0.508533, 1.412848, 1.897698, -0.643986, 0.080288, 0.396899, 0.965289, 0.477611},{-1.546600, -0.384136, 1.488800, 1.165120, 1.446222, 0.715311, -2.133113, -0.168390, 0.254613, 0.491720, 0.822160, -0.821495, -0.732644, -0.158274, -0.287859, 0.585981, 0.023966, -0.362666, -0.674304, -0.041060, -1.635206, 0.067692, 0.665897, -1.776188, 0.560355, 0.321498, 1.173711, 0.149804, 1.424217, -0.833522, 0.141709, -0.723948, -1.103249, 0.526564, -0.664839, 0.288353, -1.295952, -0.169399, 0.658274, 0.635137, -0.960555, 1.059041, 0.880256, 0.108169, 0.429616, -1.988205, 0.601801, 0.387339, 0.328026, -0.518670, -1.469472, -0.070684, -0.964657, -1.010344, -2.013409, 0.993460, 0.389524, -1.308203, -0.263982, -0.421809, -1.161492, -0.696015, 0.998487, -0.968297, 0.730953, -0.150029, 0.080878, 0.704723, -0.075072, -0.573462, 1.582180, -0.745178, 0.655891, -0.971072, -0.452780, -0.607722, -0.880228, 0.435229},{0.561919, 0.724039, -0.562818, 0.345607, -0.044412, -0.490404, -0.622716, -0.202891, 1.057082, -0.611998, 0.461918, 0.984544, 0.720077, -0.149631, -0.044905, 0.534996, -1.198920, 0.512908, 1.245049, 0.234543, 0.088172, 1.510652, -0.544713, -1.069023, 1.112393, 1.940011, 0.261813, -1.384536, -0.758002, 0.046739, -1.014178, -0.788403, 0.291162, 0.184342, 0.449536, 0.022921, -1.477555, -0.808799, 0.726420, 0.362813, -0.302900, -0.434415, 0.689205, 2.479735, -0.285148, -1.174055, 0.121340, 2.307174, -1.061925, 1.113142, -0.955521, -0.258258, 0.358647, 1.425416, -0.228135, 0.250557, -1.112184, -0.720915, 0.296566, -1.117261, 0.320697, -0.486776, -0.615972, -0.432596, 0.091135, 0.499860, 0.184997, 0.918919, -1.377206, 0.330168, 0.028721, -1.200400, -0.222248, 0.805123, 0.823541, 0.835221, -0.224142, -0.130745},{-0.805990, -0.936226, -1.610248, -0.658576, -0.408059, 1.422124, -0.716442, 0.980121, -0.572717, -0.534607, -1.534909, 1.303228, -1.018555, 0.319661, -0.237518, 0.832079, -0.208428, -0.766845, 0.104370, -0.561811, -0.615886, -0.496368, -0.051697, -1.489330, 1.458185, 0.696073, -0.391004, 0.425928, -0.534338, -1.036119, -0.431814, -0.216505, -0.701425, -1.706179, 0.299513, 1.149139, -1.041158, 0.758983, 0.477459, -0.548817, -1.238601, 0.114978, 0.329344, -1.112520, 1.889565, 0.063745, -0.062549, 1.031458, 0.223497, 0.561879, 2.285934, 0.434944, 0.894044, 0.059348, 0.383537, 3.309391, 0.926687, -0.741606, -1.034208, -0.214751, 0.461670, 1.117606, -0.169450, -0.915186, 0.474031, 1.446602, 1.995524, 1.089225, -0.583776, -0.121146, -0.472483, 0.755603, -0.350314, 0.237326, 0.038960, 0.749690, -0.277887, 0.892426},{-0.889447, -0.596731, -0.893202, 0.548970, 0.243339, -1.101599, -0.618323, 0.459637, -1.028827, 0.688621, 1.200438, 0.940197, 1.221119, 1.778185, -0.711111, -0.001157, -0.350614, -1.198288, -1.041272, 1.610655, -2.009298, -0.427635, 2.201294, 0.830881, -0.280439, 0.276084, 0.555407, -0.285763, 0.603926, -1.295065, 0.703314, 1.083820, 0.067937, -1.269675, -0.092603, 2.268777, -0.138764, -0.835561, 0.056359, 0.885981, 0.837393, 0.543201, 1.475363, 0.401361, -0.741469, -0.948269, -0.644359, -0.108163, 1.014305, -0.776704, -1.251493, 0.605834, -0.700344, 0.990709, 1.670729, 0.052688, 0.661311, -1.309836, 0.430895, 0.507877, -1.463969, -1.095957, 0.305218, 0.088797, -0.949349, 0.664889, 0.650565, -1.087061, 0.046777, -0.043198, -0.057610, -0.072510, -0.373868, -0.809499, 0.181652, 0.078352, -0.662490, -0.879661},{0.357097, 0.847523, 0.345745, 0.253971, -0.004024, -1.418048, 0.350310, 0.686421, 1.223205, 1.041686, -1.294375, -0.073451, 0.192514, -2.165267, 0.529129, -0.855302, 0.008226, 1.788328, 1.613716, -0.476035, 0.935933, -0.440412, 0.490413, 1.794138, 0.017306, -1.509075, 0.256604, -0.606487, -0.460180, 2.533288, 1.929088, -1.214650, 1.093786, 0.148082, -0.724945, 0.667942, -0.159816, 1.159601, 1.667460, -0.479755, -2.032724, -2.275286, 0.662078, -1.474814, -0.047765, 1.087990, -1.366877, 1.104317, -0.414682, -0.043046, 0.392598, 0.890304, 0.199510, 2.675084, 0.642169, -0.433462, -0.961402, -1.242595, 1.879971, 0.061298, 0.004466, -1.547335, 0.327817, -0.592541, -1.671728, 1.422200, -0.888495, -2.153991, 0.770069, -0.071028, 0.160890, 0.369255, 1.094370, 1.793203, 0.615439, -0.932536, 0.633043, -0.676198},{2.178361, -1.323254, 0.070634, 1.355301, 0.759541, 1.701214, -1.459589, -0.639062, -0.258545, 0.262658, -1.230763, 0.602729, -0.022945, 0.408460, 1.288678, -0.943455, -0.530848, -0.666838, 0.066254, 1.833342, 0.316151, 0.461947, -1.541666, -0.343090, -1.444377, -1.342242, -0.406285, -1.716411, 2.930245, -1.031607, 0.168103, -0.561500, -1.097414, -1.558835, 0.153429, -0.161778, -0.605630, 0.793435, -0.700827, -0.783385, -0.550267, -0.889014, 1.865658, -3.206863, -0.228748, 0.140001, 0.612054, -0.127441, -0.475754, 1.875866, -2.070897, -0.757511, 0.604916, -1.017698, -0.005751, -0.118977, 1.001411, -0.353121, -2.495151, -0.247456, 2.637327, -0.495353, -0.401195, -0.132012, 0.920260, -1.020335, 0.369158, -0.004899, -0.639892, -0.703950, 0.623087, 0.106759, -0.965968, 0.629892, -0.497856, -2.034437, -0.895038, -0.188097},{-0.208322, -0.631990, 1.650096, -1.790446, 1.617580, -1.358766, 1.109713, 1.051225, 0.782953, -0.734979, -0.181787, -0.323635, 0.303113, -1.136428, -1.187408, 1.353936, 1.736673, -0.231999, 1.049610, 0.043887, -0.951422, 0.376354, -1.525259, -0.629901, -1.368276, -0.455933, 0.196887, 1.092942, 0.035851, -1.325989, -0.384949, -0.613373, 0.939904, 0.244853, 0.680028, 1.295904, 0.234654, 0.817626, -1.446113, -0.355438, 2.121412, 0.970160, 0.212111, -1.363416, -1.452104, 1.288221, -1.872721, -1.395410, 0.998784, 0.833497, -0.379496, 0.539130, -0.189901, 1.186623, 0.013871, -1.750561, -0.723852, 1.813288, -0.536353, -0.879444, -0.515858, 0.790254, -1.024478, -1.723177, 0.897797, 0.337214, 1.095938, 0.584142, 0.937194, 0.181253, 0.017206, 0.490240, 1.977221, 1.191182, 0.550565, -1.166157, -1.426726, 2.587784},{0.696500, -0.480996, 0.120106, 1.000568, -1.634218, 1.868439, -0.954166, -0.098411, 2.634869, 0.534409, 0.608921, -0.076928, 0.514582, -2.039062, 0.609463, -0.380805, -0.750287, -1.612104, -0.814293, 0.970266, 0.961828, -0.211852, -1.690935, -0.843529, 1.065954, 1.092460, -0.461870, 1.875306, -0.420601, -0.327513, 1.477674, -0.491900, 0.350936, -0.180327, -0.642248, -0.453909, -0.156776, -0.513071, -0.547101, -0.201561, 1.039449, 1.603185, 0.002168, -1.222649, 1.292494, 1.590091, 0.241690, -0.258168, 0.672743, 1.021389, 0.332831, 0.914335, 1.715573, 0.107005, 0.994115, 1.868135, 0.917588, 1.258928, -0.220806, 1.291186, 0.065596, -1.336233, 0.909154, 0.695443, -2.091896, 0.092945, -0.026601, 1.292548, 0.705210, 0.548160, 0.423105, 0.135495, 1.399082, -0.480571, 0.469840, -0.142281, 2.136724, -0.581575},{-0.150745, -0.730832, -0.600892, -0.034224, 1.635438, -1.245953, -1.000058, 1.626357, 0.162170, 1.382689, -1.594428, -0.437024, 1.038040, 0.291157, -0.158272, -0.123866, -1.257275, -0.022740, -0.909587, 0.092107, 1.292551, -0.771714, 0.162551, -0.501358, -0.295763, -1.369769, 0.270384, 0.265278, -1.225834, 2.306024, 1.334619, -0.509531, 0.262185, -0.761771, 0.834380, 1.217220, 0.247521, -0.896454, -0.372132, -0.180695, 0.335051, -0.876504, 1.110173, 0.977982, 0.391630, 1.376123, 0.853858, 0.383237, 0.744646, 2.206810, -0.164024, -0.923069, -0.118581, 1.006237, -0.874843, 1.185514, -0.075270, 1.121411, 0.317648, 0.258876, -1.553370, -0.616897, -0.968193, -1.035438, -0.226920, 1.116122, 0.674909, 0.287437, -0.981914, 0.930906, -1.956702, 0.735465, -0.332128, 0.025166, 0.790486, 0.995387, -0.863949, 1.077866},{-1.252328, -0.011072, 0.377562, 0.122991, -0.302105, 0.797784, -0.707721, 0.777719, -0.304353, -1.505917, -0.093120, -1.207873, 0.666917, -0.328891, 0.233951, 0.582337, -1.430457, -1.162119, 0.198918, -1.626391, 1.569292, -1.080626, 0.010143, 1.300799, -0.632320, -1.405298, 0.292735, -0.879023, -0.650671, 1.633747, 0.381643, -0.505902, 1.111345, -0.456824, -0.300760, 0.414755, -0.306727, 0.064284, 0.442018, 0.064045, -1.253082, -0.248792, -1.027518, 0.845155, 0.273157, 0.504080, 0.100602, -1.788384, 1.282817, -2.563643, -1.307425, 0.634295, -1.738103, -0.610384, -0.792371, -0.232082, 0.864370, -0.219901, -0.662691, 1.185158, 1.916379, 0.836269, 0.850886, 0.508149, -0.052659, -0.287612, 1.010479, 1.824568, 0.609669, -0.449372, -1.068030, -0.364214, 0.750918, -0.376617, 0.398099, 0.423278, 0.811677, 0.704345},{-0.984681, 1.801839, 1.539938, 0.298141, 0.210962, -0.231020, 0.350211, -0.186068, 1.478751, 0.166837, 0.136410, -0.125054, -0.299213, 0.623820, -0.708186, 0.609754, 0.758763, -0.132887, -1.060057, -1.477900, -1.339372, -0.630652, 0.441839, 2.506161, 0.511516, 0.376717, -0.992738, -0.113118, 1.441460, 1.575839, 1.288068, -0.354595, 0.049368, 1.923789, 0.269511, 0.934126, -1.221609, -0.558846, -1.379571, -0.433643, 0.307621, 0.726252, 1.923651, 0.475418, 0.868382, 0.874859, 0.181095, 0.779823, 0.762116, -0.136720, -0.492456, 0.074209, 0.576795, -2.561642, 0.655408, -1.826330, -0.066364, -1.576495, 0.182897, 0.212288, -1.465595, 1.124230, 0.584907, 0.538955, 0.343102, -0.415460, -0.900553, 1.210844, 1.123218, 1.241677, -0.584401, 0.718967, -0.394944, -0.412749, 0.424998, 0.400926, -1.379470, -0.056325},{0.998430, 0.917247, 1.033527, -0.196777, 0.617478, -2.198573, -0.998380, 0.132947, -0.337992, 0.532455, 0.750210, 0.054101, 0.809147, -0.937344, -1.616140, 0.811521, 1.346182, 0.103298, 0.939486, -1.103996, 2.476718, 0.723882, -0.178788, 0.312875, -0.295712, 0.991414, -1.103103, 0.259085, 0.940440, 0.362330, 0.300316, -0.037460, -1.000923, -0.823740, 2.201572, 0.921887, -0.097379, 1.610079, 0.876153, 0.599662, -0.680205, -0.974476, 0.001175, 0.956065, -0.720629, -1.393421, 1.622977, 0.794154, 1.219745, -0.553647, -1.157199, 0.988990, -0.109464, 0.638641, -0.619179, 0.194418, 0.136433, 1.693188, 0.160166, -0.325037, -0.569094, 1.787901, 1.269245, 0.836651, 0.030624, 0.901447, -0.563089, -1.357608, 0.140718, 0.396520, 0.096287, 1.396405, -0.543444, 0.788132, -0.797693, -0.498463, -0.318867, 0.406231},{-0.427950, -0.922530, 1.781641, 0.399316, -1.006859, 0.722337, 1.354104, 0.030071, -0.403916, -0.932886, 0.690304, -0.862344, 0.276558, 0.496824, 1.488630, 0.582276, -0.272380, 0.826549, 0.355263, 0.521556, 0.413635, -0.132933, -0.972243, -0.615257, 0.972137, 0.132494, -0.498025, 0.415882, 0.118313, -0.147020, -1.704220, -0.754800, -0.596986, -1.046272, 0.168916, -1.069134, 1.058735, 0.099651, 2.053967, 0.538508, 0.386126, -1.339216, 0.174017, 0.558719, -1.572942, -0.754359, 0.583103, -2.185277, -0.147083, 0.626659, -0.623898, -0.842223, -0.774428, -0.672222, -1.470079, 0.356885, -0.217517, 0.393340, -0.699200, -1.349539, 1.082511, -0.356104, -0.559422, -0.260328, 0.933541, 1.038508, -0.388036, -1.011722, 1.855398, 1.460812, 0.710802, 0.610079, 1.070119, -0.007699, -0.234682, 1.500093, -0.350665, 0.810930},{0.149064, -0.141529, 0.126238, -0.420929, -0.696074, -1.274260, -2.048099, -0.269338, 0.512485, 1.393755, -0.377932, -1.085124, 1.606931, 1.027058, -0.558641, -0.429558, -0.245547, 0.393948, 1.049247, -0.940295, 0.774104, 0.426015, -1.091697, 2.176562, 0.290919, -0.550017, -0.334002, 1.073430, 0.250315, -0.809212, 1.472149, 0.781895, 0.178385, 1.153656, 1.338474, 0.862242, -1.326166, 1.548915, 0.741127, -0.594925, -0.571776, 1.883010, 0.451056, -0.461671, -0.312420, -0.752093, 0.225743, -1.275490, -0.227162, -0.979383, 0.157209, -0.422121, 0.693642, -0.201163, 0.343477, 0.899123, 0.928887, 1.185428, 0.296741, 0.486217, 0.949348, 1.502486, 1.210950, 0.288683, 0.097527, -0.914328, -0.594893, 0.612253, -1.060249, 0.720785, -1.646726, 0.195292, 1.023139, 0.055421, -0.775255, 0.177608, 0.066923, 0.722469},{0.048619, 1.132518, 0.936091, 0.840618, 1.383525, 0.200749, 0.132591, -0.901394, 1.023701, -0.591079, 1.565457, 1.051882, 0.034490, -0.406312, 1.787189, 1.041440, 0.033661, -0.096334, -0.605675, 2.226391, -0.233249, 0.470506, 0.037553, 0.801304, -0.239188, -1.540577, 1.114833, -0.374326, 1.296156, -0.522957, -1.314834, 1.778890, -0.629841, 1.177225, -0.033908, -0.139139, 1.482451, 0.054615, -0.860557, -0.429820, 0.220844, 0.160092, 1.756616, 1.699861, -0.618340, -0.480965, -0.310101, 0.885464, 0.597529, 0.086229, 1.361108, -0.253332, -0.332728, 0.184501, 0.053398, 1.863860, 0.590044, -0.766260, 1.021596, -0.609916, 0.556850, 0.624822, -0.453105, 0.562222, 0.397474, -1.069689, 0.718318, 0.880134, 0.229822, 1.972834, -0.964869, -0.445241, 0.181031, 0.601366, 0.814834, -0.287990, 0.897918, -0.584748},{1.202340, 1.286284, -0.257015, 0.636215, 0.303411, 0.912047, -1.530185, 1.570173, 1.641347, 0.781143, -2.690284, -1.077484, -1.348459, 1.308786, -0.536917, 1.372113, -0.184440, -2.471829, -1.441647, -0.408291, -0.203379, 0.161308, -0.306888, -0.164279, -1.775835, 0.328591, -0.544810, 0.339404, 0.337760, 0.712220, -0.516687, -0.522427, 1.707500, -0.182937, 0.127720, -1.363438, -0.530668, 0.326344, -0.030240, 0.453086, -0.759110, 2.252020, -0.072860, 0.671348, 0.062531, -0.627780, 0.272903, -0.560513, -0.082090, 1.629820, 0.286836, -2.001661, -0.840168, 0.371238, 0.927516, -0.624081, -1.424857, -1.136474, -0.142470, -1.136491, 0.705751, -1.384781, -1.625376, 1.437353, -0.310744, 0.272861, -0.887363, 2.119830, -0.243979, -0.863247, 0.836684, 0.804965, 1.696706, -1.078393, -1.886223, 1.306334, 1.165196, 0.909203},{0.107185, -0.984772, 0.264283, -0.027590, 1.639968, -1.604234, 0.597283, -1.867326, 0.985746, -0.577731, 1.643802, -0.207201, -1.293788, -0.066806, 0.089643, 0.513626, -1.381825, 0.838022, -0.620056, 0.640043, -0.986717, 0.577630, 0.681142, 0.193179, -0.729955, -0.995329, 0.676796, 0.537071, -1.094873, -0.284023, -0.825294, -1.956523, 2.017695, 0.191069, 0.933604, 0.479760, -0.181529, 1.671816, 1.002476, 1.277343, 0.537028, 0.673360, 0.629322, 0.149957, 0.417825, -1.164246, -2.229786, 0.218222, 0.277907, -0.042837, 0.970586, 1.262294, 0.430398, 0.058582, -2.127424, 1.292992, 0.629130, -0.515168, -0.205336, -0.274264, 0.705736, 0.115544, -0.438405, 0.314225, 0.627517, -0.696517, -0.448990, 2.108567, 0.116166, 0.033083, -0.263453, -0.431948, -0.868696, 0.395245, 0.602546, 0.819728, 0.504197, -0.908731},{-1.507728, 0.964842, -1.502356, 0.280297, 1.139124, 0.304637, 0.570228, 0.687477, -0.141365, -1.387657, -0.377930, 0.637559, -0.447271, 1.023778, 0.008831, 1.186154, 0.576020, 0.304862, -0.504404, -0.410216, -0.726105, 0.780285, 0.078187, 0.232012, -0.571968, 1.019331, 1.595901, 0.974869, -0.727831, 1.681114, -0.120470, -0.255962, -0.995455, -0.589817, 1.043236, 0.639381, 1.588571, 0.703729, 0.158097, -0.292854, -1.247848, 0.363895, -1.083172, -0.227211, -1.352271, -0.828004, 0.180986, -1.115464, -0.225827, -0.109523, 1.103895, -0.569423, -1.699934, 1.102368, -0.725911, -0.934473, 2.133216, 0.504947, -0.736195, -0.389371, -1.871892, 0.644591, 1.160672, 1.143965, -0.739773, 0.309157, 2.405262, -0.066247, 1.105447, -1.400516, 0.398414, 0.278175, -0.322714, 1.670748, -1.156802, 0.726502, -0.115435, 0.104294},{-1.378182, 0.130399, -1.889558, -1.688067, 0.389905, 1.175183, 0.027597, -2.535238, 1.117160, 0.856249, 1.430101, -0.160360, 1.905076, 0.098126, -0.667988, -1.764298, -1.267499, -0.777804, 0.272944, 1.743227, -1.131174, -0.774271, -0.687423, -0.725206, 1.095124, 0.161013, -1.166958, 0.194308, 1.410575, -2.348265, -1.826646, -0.846868, -0.715506, 2.052784, -0.097575, -0.387838, -0.190494, 1.399469, -0.908163, -0.517051, -0.325846, -0.680762, -0.935394, -1.458243, 0.485659, -1.694168, 0.375795, -1.150309, 0.708856, 0.728231, 0.761321, -0.085416, -0.439600, -0.720109, -0.374110, -2.344404, -1.834829, 0.075136, -1.078452, -0.618495, -1.333984, -0.482466, -0.771842, -1.957392, -0.272954, 2.108462, -1.006680, 1.162814, -1.615421, -0.750335, -1.508256, 1.637312, -0.193545, -0.154323, -1.355222, 1.221883, -1.747085, -0.436097},{0.712854, -0.258717, -1.014375, -0.238655, 0.480662, 0.413496, -0.624173, 0.871518, -0.611625, 0.114765, 0.644202, 1.627428, -0.200023, -0.024306, 1.030638, 0.551161, -0.322378, 1.344355, -0.377925, 0.594885, -1.096620, -1.194368, 1.420581, 0.589923, -0.206121, 1.069318, 1.328818, -0.069958, 1.055527, 0.921009, 0.796732, -0.282720, 0.817370, -0.624161, -1.311546, 1.445076, -0.103576, 1.238259, 1.411617, 0.323250, 0.637538, -0.261853, -0.146776, 0.056479, -1.275064, -0.171985, 1.305737, 1.585046, 2.053367, 0.845523, -2.771449, 0.437320, 0.248996, 1.876375, -0.807968, 0.338114, -1.976755, -0.620231, 0.762640, 0.793164, -0.392063, 1.149060, 0.119396, 0.871948, -0.960198, -0.189188, 1.694243, -0.411489, 0.895361, 0.601005, -1.279029, -0.399070, 0.338329, -0.170770, 1.186115, 0.556032, 0.808832, 2.032530},{-0.711632, 1.046854, -0.314142, 0.150007, -0.491170, 0.391512, 0.365829, -1.821058, 0.338197, -1.036809, -0.003787, 1.260584, -0.560876, -0.448026, 1.469239, -0.724893, 0.287193, -0.197391, -1.762074, 0.098711, 0.260987, -1.392113, 0.886636, -0.070797, -0.631274, 1.483895, -1.220711, 0.038871, -0.105859, -0.113878, 0.804253, 0.896021, 0.881769, -0.496932, 1.048259, -0.288310, 0.795136, 0.698934, -0.424423, -1.459515, 0.993704, -0.393958, 0.246036, -1.370611, -0.329106, 1.744207, 0.480372, -0.741689, 0.017931, 0.066600, -0.789671, -1.267163, -1.193582, 0.613366, -0.592333, -0.665876, -0.504831, 0.377205, -0.338478, -0.209119, -0.739767, -0.886807, -1.092490, 0.730435, 0.440549, -0.646136, 0.195581, -0.268470, -2.227572, -0.605163, 0.560217, 1.838153, -1.895639, -0.361683, -0.255140, 0.709451, 0.440693, -0.019747},{0.660998, -0.028896, 2.486222, -0.277375, 0.014104, 0.855387, -0.597316, 1.008897, -1.622375, 1.220757, -0.023607, 0.835963, 0.016057, 1.738320, 0.891598, 0.607062, -0.311610, -1.175350, 0.334402, 0.420166, -2.059011, 0.433617, 0.493577, -3.068910, 0.697868, -0.865448, -0.036759, 0.201016, 1.030400, -0.163339, 1.164352, 0.403615, 0.890421, 0.601954, 0.040603, 0.102356, 0.016033, -0.182036, 1.882012, -0.977949, 0.109987, 1.581531, -0.013127, -0.906917, 0.149356, -0.844298, -1.631553, 0.059477, -0.707409, 0.815284, 0.001981, -0.195431, 0.635910, 0.442216, 0.203877, -1.294011, -1.089873, 1.041846, 0.048583, -1.583234, -1.800901, -0.424801, -0.537897, 1.764899, -0.844163, 1.083650, -1.228014, -0.416947, -0.741838, 0.806361, 0.420911, 0.099781, 0.031018, 0.517802, 2.026105, -0.361893, -0.952726, 0.080808},{-0.705986, -0.120926, 0.306720, -0.736672, 0.014006, 0.930672, -0.066497, -0.796662, 0.290978, 1.268981, -2.161122, 0.197946, -0.663906, 0.143343, -0.093644, 0.665643, 0.453131, 0.197302, -0.570309, 1.359611, 0.321852, -1.039800, -0.398550, 0.734867, -0.051971, -1.122889, -0.807569, -0.130347, 0.733007, 0.814514, 0.949523, 1.015062, 1.122683, 0.249771, -1.118700, 0.294014, -1.658841, -1.013747, 1.533228, 0.124044, -1.218450, -0.762050, 2.094055, 1.285235, -0.429331, 0.001681, 0.885581, 1.716189, 1.181035, -1.560383, 0.185172, -1.223062, -0.737261, -0.180783, -0.750215, 0.385462, -0.418803, 0.389966, -0.825573, 0.097593, 0.102742, -0.185477, 0.731969, -0.108964, -1.379951, 0.096061, -0.417986, 0.019524, -1.469251, -0.239976, -0.490191, -0.730185, 0.208430, -0.465724, -0.476494, 0.148059, -0.753925, 1.792209},{1.710440, -0.475511, 0.052743, 1.473497, 0.963586, -0.261191, 0.468295, 0.202222, 0.229833, 0.344717, 0.636366, 1.577097, 0.543790, 0.102940, -1.800697, 0.041536, 1.247287, 0.713597, 0.772504, 0.489790, 0.417044, -1.036023, -0.467564, -0.743680, 1.420266, 0.189171, 0.609459, 1.279047, -2.280531, -0.307574, 0.478978, -1.145817, -1.368308, 1.449788, -1.646100, 0.413926, 0.422774, -0.196739, 1.910529, -0.020116, -0.187288, -2.310928, 1.539770, -0.484140, 0.347448, 0.249482, 0.603948, -0.688226, -1.360998, -0.970692, 0.396186, -1.016986, -0.793307, 0.478196, 1.313292, 1.327273, 2.150801, 0.668110, 1.579517, 0.988712, 0.284571, 0.028483, -0.577579, -1.324911, -1.416398, -0.508333, -0.360541, -0.414103, -1.094445, -0.757813, -0.121558, -0.754859, 1.430104, -1.048844, -1.581221, -0.293468, -0.216777, -0.532070},{0.804318, -0.819845, 0.165614, -1.094131, 0.274121, -0.602385, -1.893294, 1.310046, -2.182639, 0.323645, 0.241935, 0.130847, 0.714030, 1.183853, -1.306277, -0.527653, -0.290935, -0.444323, -0.674850, -0.107538, -0.613505, -0.107853, -1.039033, 0.738113, -0.625132, 0.514608, -0.165391, 1.269214, -0.937461, 0.391017, -0.614560, 0.067709, -0.426488, -0.801506, -0.650899, 0.779727, -0.345703, -0.416336, -0.518740, 2.322605, 0.775035, 0.242548, -0.281636, 1.943896, 0.206608, -1.596200, 1.447555, -0.085374, -2.162942, -2.817671, -1.882932, 0.025913, 0.719729, -0.364114, 0.528872, -0.657920, 0.162122, 1.293340, -0.073006, -0.928202, -0.715428, 0.693919, 0.290069, 0.658490, 0.134266, -2.085740, -1.089660, -0.224670, 0.618480, -1.200073, 0.530390, 0.793993, 0.451574, 0.211920, -0.549891, 0.249421, 0.140718, 2.538680},{-1.958835, 0.846406, 1.854841, -1.404523, -1.660198, 0.449415, 0.301606, -0.886213, -1.100945, 0.329159, 1.810553, -0.661967, 0.707297, -0.927655, 0.726378, -1.426082, -1.001624, 0.593100, 0.607444, 1.441351, 1.898453, 0.045065, -0.290388, -1.804041, -0.265986, -0.307264, 0.429381, 0.428213, 0.792858, 1.613767, 1.682163, 0.159537, -1.593289, 0.290200, 1.249148, -0.906840, 0.423707, -0.730133, 0.262642, -0.257812, -0.253992, -0.212647, 0.255476, -0.243048, -1.370696, -0.841877, 0.123693, -0.916238, 1.519439, 1.465075, -0.060021, -0.288758, 1.441027, 1.080980, 0.329963, -0.055370, -0.012707, 0.398237, -0.958622, -0.138235, -0.748610, 0.454082, -0.702482, 0.590112, 1.158117, 0.116227, -0.075977, -1.348781, 0.664193, -1.367320, 0.654188, 1.034724, -0.105899, -1.827065, 0.255494, 0.909371, 0.095508, 0.974318},{0.202335, 0.510062, -1.792004, -0.885535, -0.134066, -0.637780, -0.122629, 2.192914, -0.153739, 1.018174, -0.132047, -0.992751, -0.974493, 0.239486, -1.138900, 0.425980, -0.918359, 0.022300, -2.777792, 0.273864, -0.948652, 0.275310, -0.952564, 1.102865, 0.494004, -0.590948, 1.012587, -1.131020, -0.142358, -1.154227, 0.590070, -1.363838, 1.003127, -0.540185, 0.278230, -1.216031, -0.818715, -1.826184, 0.248168, 0.753752, -0.154381, 0.386118, 0.527813, 0.664364, 0.402100, 0.205848, -3.173224, 1.385624, -1.266131, -1.162997, 0.897052, 0.536053, 0.175494, -0.383092, 0.461471, 0.012739, 0.582253, -0.896816, 0.524341, 0.940646, 1.730754, 0.381254, 1.571199, -0.558916, -0.425470, 0.124087, -0.002708, -0.067324, 0.258454, -1.965548, 0.343003, 0.547590, -1.255185, -1.460510, 0.557046, 0.336961, 0.820074, 0.122540},{1.207065, -1.395975, -0.340905, 1.321582, -1.224014, 0.153953, 0.000205, -0.533378, 1.859231, -0.350827, 1.173776, 0.611877, 1.630054, 0.474480, 1.539378, 1.665167, -0.395937, -0.065641, -0.929244, -0.978046, 1.300679, 1.109301, 0.692855, -1.429704, -0.023867, 0.228793, -0.934959, 0.458501, 0.254969, 0.379170, 1.293433, -2.770490, -0.322437, -0.307280, -0.525104, -0.526120, 0.411312, -2.123710, -1.534223, 0.165600, -0.778201, -1.021319, -1.675906, 0.029649, -1.301919, -0.555906, 0.813257, -0.072567, -0.400418, -0.214037, 1.039947, -0.660591, 0.108309, 0.293409, -1.354270, 0.341025, -0.808115, -0.470086, -1.271136, 0.287897, 0.146875, -0.112150, -0.105911, 0.892946, -0.490337, 0.278205, 1.983163, -0.199539, 0.316054, 0.836170, -0.182717, 0.212514, 1.100503, 1.746365, -0.707891, -1.164445, 0.842293, 0.921096},{0.463258, 0.116744, 0.036150, -1.245154, -0.755441, -0.489527, 1.407254, 0.565255, -0.499580, -0.451465, 0.160200, -0.495251, 0.469736, -0.439548, -0.152525, -1.415134, 0.624483, -1.001234, -0.770689, 0.686472, 0.325638, 0.394993, 1.459205, -0.654534, -0.306704, 0.450824, 0.947947, -0.018282, -0.069379, 0.539665, 0.225236, 0.713940, -2.362640, -0.927180, -0.620073, -2.220180, 1.752728, 0.373953, 0.207512, 0.383256, 1.734006, 0.168385, 0.835390, -0.635990, 0.348254, 0.141845, -1.278829, 1.170813, -2.391625, 0.651003, 2.093285, -0.941947, -1.326573, 0.061237, 1.493848, 2.239682, -0.964767, -0.220886, 0.291893, 1.357850, -0.009717, 1.311857, -0.751097, 1.951201, 1.421998, 1.071200, 0.767848, -1.695733, -0.121934, -1.667742, -0.025172, -0.205974, -1.229386, -0.863857, -0.132588, 0.187116, -2.501188, 0.982971},{2.700927, -0.763202, -1.086781, -0.366725, 0.000675, 1.613642, -1.457916, -1.137245, 0.150129, -1.915236, 0.094832, -0.889281, -1.271369, 0.061061, 0.603010, -1.665073, 1.554494, 0.139478, -0.233227, 0.707923, 0.395285, -0.556726, 0.738285, -1.387838, 1.524749, -0.633046, 0.009818, -1.456124, -2.312528, 0.913542, -1.437267, -0.955899, -0.467584, 0.872450, -0.943175, -1.822353, 0.703733, -0.300015, 0.061481, -1.024209, -0.116327, 0.233345, -0.697129, -0.137666, 0.330075, -0.515065, -0.761792, -0.339176, 0.349001, 0.938683, 1.046243, 1.127913, 0.032112, 2.000095, 1.448681, 1.739285, 1.105046, -1.559454, -0.093817, 1.405935, -1.091641, -0.684838, -1.385550, -0.627637, 0.257102, 1.136251, -0.014305, 0.536268, 0.862977, 0.971479, 1.253670, 1.155773, -1.058976, -2.901278, -0.111094, -0.306496, -0.143022, -0.585089},{-0.047050, -0.062931, -0.595170, -0.908133, 1.269441, 0.015358, 0.203686, -0.620729, -0.241180, -1.080596, -1.302972, -1.123051, 1.050581, -1.976244, 2.046059, 0.893901, 0.662471, -1.507848, 0.649017, 2.049596, 0.468854, 1.322673, -1.267645, -1.411211, -0.892082, -0.987286, -0.518245, 0.960450, 0.221566, -1.271691, -1.777729, -0.362307, -1.024786, 0.197275, -0.177605, -0.160853, 0.732648, -1.868642, -2.164805, 0.241057, 0.499985, 0.131389, 0.647958, -1.437059, -0.591223, 1.338235, -0.362691, -0.512533, 0.783735, -0.616641, 0.801084, -0.832536, 1.909318, 0.129251, -0.623988, -1.848660, 1.697772, -0.043273, -1.644013, 0.310566, 2.832026, -0.963794, 2.231351, 1.079144, -1.630856, 0.682262, -0.625492, -0.447636, -0.710901, -0.863103, -0.608905, 0.064489, -0.524810, 0.987167, -1.181771, 0.874757, 0.238209, 0.890517},{0.736763, 1.347826, 0.427658, 1.462649, 1.884718, 0.983991, 0.253910, -1.094625, 0.812379, -0.814908, -0.633168, -0.525773, 1.954345, -0.277006, 0.423889, -0.382285, 0.718494, -0.142837, 0.819954, -0.253454, -0.597452, -1.015461, -1.318940, 1.932460, -0.368957, -2.444912, 1.392133, 1.829155, 1.102852, -0.328245, -0.559117, 0.533958, -0.402852, -0.330753, -1.735188, 0.827989, -0.419438, -0.951094, 0.791629, 1.243810, 0.632124, 0.411734, -0.474105, -1.719088, 2.478383, 0.713273, -0.156124, 1.746330, 1.702907, -1.933412, -1.683390, 2.092027, 1.866826, 1.179000, -1.193421, -0.598966, -1.110783, -2.276882, 0.395497, -0.243237, 0.097260, -0.630551, 0.299543, -0.493027, 1.241957, -0.082381, -1.912355, -1.887964, -0.451511, 0.019132, 1.386718, -0.835509, 0.299935, -0.852767, -0.230543, -0.640169, 0.363687, 2.492970},{0.758004, 0.656775, 0.476628, 1.320882, -0.143548, -1.904101, -0.439514, -1.500251, -1.872455, 1.679301, -0.006382, 0.873386, -1.362419, 0.781611, 2.473244, -0.252771, -1.179628, 0.897600, -1.595448, 1.169267, 0.042554, 1.136323, -0.085484, 1.128681, -0.804623, -0.096541, 1.064281, 0.904875, 0.467713, -0.691267, -0.225009, -1.407159, 0.673489, -0.578983, 0.941690, 1.730453, 0.143477, 0.484655, 0.256151, 1.386863, -0.558535, -0.845595, 1.643679, -0.524305, -1.282759, 0.076644, 0.002470, -0.103672, 0.978103, 0.629463, 0.902346, -1.399237, -1.366335, 0.216360, -0.758044, 0.344949, -1.020266, 1.972905, 1.613201, -1.837607, -0.128942, 0.220281, -0.772674, 0.166304, 1.307803, 1.141785, -2.342202, 0.246573, 0.084206, -0.875934, 0.427312, -0.363859, -1.505242, -1.166209, 1.647412, -0.409783, 0.428788, -0.285900},{0.828656, -0.076261, 0.131228, -0.582004, -0.316476, 0.861609, -1.010286, -0.383270, -0.082015, 0.234899, -2.511488, -1.192718, 0.015046, 1.774221, 1.439120, 0.908529, 0.979880, 0.207194, -0.545371, 0.490859, 0.622710, 1.164901, 1.032111, -0.811781, -1.255567, -0.321309, 0.474609, -0.091171, 0.592370, -1.333925, -0.453004, 1.827254, 0.145528, 0.369042, -1.946737, -0.582845, -2.071719, -1.781473, -0.555225, 0.977846, -1.770675, 1.275887, -0.803041, 0.544987, -0.248947, 1.740729, -0.631165, -0.813975, -1.177920, -1.387334, 0.164632, 1.939899, 1.159177, -0.353161, 1.117554, -1.038664, -0.594998, -0.121844, -1.403441, -1.205523, -1.137121, 0.491511, -2.180502, 0.145199, 1.632856, 0.154141, -0.238836, -2.270670, -0.328815, -1.375459, 1.194417, -0.343324, 1.433255, -0.177625, -0.967981, -1.299671, -1.013777, -0.104449},{0.892222, 1.373797, -0.516961, -0.288856, -0.843751, 2.368372, 0.881486, -2.904902, -1.507137, -0.202651, 1.618346, -0.613902, -0.352299, 2.134657, -1.364242, -1.262991, -0.022795, -1.153153, -0.053173, 0.336048, 1.508871, 1.745651, -1.317534, -1.520300, -1.502056, -0.330709, -0.052510, -0.509597, -0.715510, 1.433752, 0.390778, 2.628594, -1.241441, -0.804986, -1.076561, -1.910976, 0.842597, -0.432114, 0.887519, 0.107983, -1.384793, 1.881943, -0.318434, -1.166346, 0.401840, -1.020287, -1.881085, -0.851838, -1.235735, -0.321130, -0.677084, 0.576874, -1.261827, 0.275449, 0.243209, -0.549033, 0.287531, -0.909662, -1.382152, 1.907545, -1.357962, 1.219704, 0.242082, -0.530329, 1.222291, 0.213468, -1.415939, 2.748728, 0.196160, -1.276614, 0.220612, -0.338803, -0.712474, 0.037926, -0.350707, -1.116450, -1.392252, 0.615231},{2.131622, -0.480812, 1.335791, 1.525232, 0.760422, 0.730582, -0.081357, -0.274996, 1.057019, 0.022331, -0.630987, -0.425598, 0.622150, 1.647645, 0.783484, 0.774826, 0.625007, -0.894233, -1.910277, -0.044046, -0.220104, 0.072541, -1.414539, -0.793840, -1.459352, 0.895430, 0.721882, -0.328063, 1.183346, 0.536080, -0.507773, -1.161696, -0.706057, 0.859123, 1.966867, -1.616636, 1.376525, -0.173907, 0.023591, 0.328552, -0.862429, 0.769999, 0.481689, -0.695226, -1.467939, 1.416150, -0.590606, -1.174618, 0.853853, -0.765788, -0.458559, -0.215222, -1.330820, 0.023448, 0.721447, -0.414526, -1.691467, -0.710752, -1.458183, 0.976639, -0.210867, 0.510031, -0.760908, 1.123271, -1.380910, 0.251244, 3.357594, 0.032924, -1.174180, 0.086767, -1.611174, 0.922714, 0.211336, 0.119209, -2.116633, -0.124466, 1.499655, 0.932085},{0.466041, -0.179316, -0.876393, 3.171607, 0.678824, -1.153775, 0.106238, -1.675368, -0.091385, -0.593122, -0.389058, 1.102408, 0.239723, 0.406130, -0.963716, -0.527623, -0.344666, 1.221472, 1.562134, -1.632832, 0.498189, 3.566316, -1.726907, -0.039165, 0.141698, 0.225054, -0.659929, 1.152231, 0.008492, 0.733107, 1.253173, 0.407216, 0.652081, 1.014507, 0.671253, 0.900145, 0.930497, -0.493110, -0.396380, -1.617009, 0.480639, 0.233412, 0.711746, 1.105478, 1.134206, 1.320687, 1.510763, 0.368286, -0.336189, -1.172779, -0.866798, 0.285648, -0.065742, 0.287145, -0.529936, 0.126873, 0.659801, -1.636931, 2.438754, 0.283906, 0.201261, 0.663429, 0.484164, 1.010420, -1.886755, 0.073382, -0.985252, -0.495697, 2.160096, 0.024761, -0.771244, -0.270039, 1.339671, 0.041359, 0.113123, -0.271503, 0.771348, 0.649163},{-1.476541, 1.107415, 0.696397, -0.465809, -0.329174, -0.717435, -0.916591, -1.952130, 0.006060, -0.910962, -1.595015, -1.615340, -0.485668, -0.591048, 0.843055, -0.119299, -0.465610, -0.789360, 1.857779, 0.970364, 1.688191, -0.187280, -1.106550, 1.213107, -0.361818, 0.009561, 1.235471, 1.868626, -0.186304, 1.175233, 0.453174, -0.956832, 0.682057, -0.217081, -0.181238, 1.289030, -0.444322, -0.448994, 0.121341, -1.479926, 0.577930, -2.125159, -0.732064, -1.113416, -0.764363, -1.028621, -0.142485, -0.252143, 0.550763, 0.042784, 3.313591, 0.956975, 0.389391, -1.474817, 0.236775, -1.044732, 0.158567, -0.720539, 0.882296, -0.027229, -1.944484, 0.389125, 0.041091, 0.408949, 0.275233, 0.108586, -1.047967, 0.266174, 0.606026, 0.744459, -1.100221, -0.149669, 0.574910, -0.937842, -0.056958, -0.505323, 1.530414, -0.174147},{-1.348325, -0.107772, -0.874017, 1.616106, 0.855066, -1.256504, 0.582611, -0.821493, 0.479211, 0.872207, -0.271262, 0.165298, 0.041837, 1.043969, 0.447947, -2.218279, -0.066076, -0.891586, -0.714144, -1.145497, -0.988010, -0.012688, 0.442525, 0.350019, 1.145890, -0.589901, 1.202818, 1.029202, -0.583989, -0.396168, 0.127846, -0.386707, -0.623717, 0.473447, -0.981708, 1.045215, -0.828807, 0.765270, 1.877597, 1.500825, -0.764861, -0.913235, -0.180647, 0.216977, -0.009707, 2.131064, -0.667846, -0.172217, 0.456260, -3.117008, -0.331040, -1.094108, 0.268323, 1.125656, -0.605592, -1.112829, -0.788139, 0.325996, -0.244559, -1.271545, -1.680115, -0.510041, -0.326760, -0.814568, 0.565281, -0.361556, -0.275049, -2.153128, 0.846620, 0.738802, 0.300841, 0.305442, 1.935034, 0.087499, 1.081925, 0.586175, -0.076046, -0.966708},{-1.162703, -0.822127, -0.434828, -0.935334, -1.357375, -1.421604, -0.776465, 1.571810, 2.523019, 0.630363, 1.485975, 0.508168, 0.513057, 0.826761, 0.987827, -0.480090, -0.392983, -1.000566, -0.595316, -2.490589, -0.509129, -0.692036, 0.285366, -2.113636, -2.412222, 3.136095, -0.960477, -0.340351, 0.653720, 0.328072, -1.190087, -1.320433, 1.697473, -0.226204, -0.421320, -0.853754, -0.664872, -1.298021, 0.545386, -0.921271, -0.043676, 0.438202, 0.277138, 0.873659, -0.005826, 0.766025, 0.447981, 2.472201, -1.400588, 1.581061, -0.750675, 1.082641, 1.099186, 2.053710, 2.497764, -1.907681, 0.392110, -1.001946, -0.181045, -0.595901, 1.732739, -0.967349, -0.925355, -1.572242, -0.432630, -0.256719, -0.100896, 0.843594, -1.255453, -0.684404, -0.091517, 1.093989, 0.358745, -0.040107, 1.158350, -0.234247, -0.572650, 0.975836},{1.568468, 0.245476, -0.331851, 0.599259, 0.454687, 0.513248, -0.798001, -0.957140, -0.857441, -0.481013, 0.393392, -0.662158, -0.881448, -1.216993, -0.349540, 1.759591, 0.371647, 0.357825, 1.556237, 1.068395, -0.315903, 0.216514, 0.505551, -0.576617, -0.548057, 0.325239, 0.263372, -0.271832, -1.370931, -0.225221, -1.250289, -0.086672, -1.467343, -1.385586, 1.052593, 1.291790, 0.073252, -0.124185, 0.148646, -1.140611, -0.067950, 0.792668, -0.562149, 0.621975, 0.987325, 0.043045, 0.737041, 0.726974, 1.545756, 0.525775, -1.290495, 1.246355, 1.474991, -0.707533, -1.328023, -1.762457, -1.983603, 1.474083, 0.383695, -0.602875, -0.674305, 0.806059, -1.398907, -0.950327, -0.997290, 1.852481, 1.438847, 0.148897, -1.593701, 0.300875, 0.231187, 0.449771, -1.919196, -1.037871, 1.151429, -0.418900, 1.512125, 3.058323},{-0.586548, 1.078935, 1.005482, -0.131363, 0.316458, 0.310173, 0.176939, -1.195144, 0.323783, -3.258154, 1.261348, -0.097070, 0.076625, 1.857208, -0.942489, -1.531568, 0.445126, -0.115109, -0.414437, -1.800980, 1.242706, -1.283709, 1.103824, 0.330660, 1.119850, 0.086128, 0.573645, -1.216875, -0.237584, 0.285403, -0.270086, -0.682714, -1.721935, -0.488671, -1.809097, -2.323576, 0.300617, -0.228538, 0.390499, 0.978455, -0.010433, -1.081720, -0.170225, -1.918556, -0.823374, -0.381877, -0.150531, -0.190558, -1.267426, -1.071530, 0.682336, -0.163159, -1.157825, 0.018587, -0.495053, -0.406912, -1.773015, -2.221407, 0.685495, -1.324558, -0.680446, -0.199079, -0.305439, 0.887281, -0.126784, -0.130961, -1.093867, 0.491301, 0.617263, -0.857895, 0.132369, 1.223602, 0.147401, 0.050104, -0.008302, 0.019746, -1.713108, -1.671039},{-1.620021, 0.007933, 0.891182, 1.501051, -1.055046, -2.237889, 0.199279, -0.158275, 1.083539, 1.059992, -0.260928, 1.529367, -0.524736, 1.651490, -0.818107, -0.795076, 1.490778, 0.575041, -0.982160, 0.355505, 1.210583, 1.239464, -0.639506, -0.212214, 0.487251, 0.156856, -0.465939, -1.102637, -1.835583, -0.988104, -0.714954, 1.754628, -0.946905, 0.166086, -0.847578, 0.414398, 0.386390, 0.205742, -1.068492, -2.031328, 0.293182, -0.324139, -2.676894, -0.252533, 1.274882, -0.210925, -0.512575, 0.678389, -3.213320, -1.512456, 0.887216, 0.440232, -0.471013, -1.314757, -0.112746, -1.367065, 0.115152, -0.908126, -1.160933, 0.798704, 0.469117, 0.701462, -0.762706, 0.131004, 0.198378, 0.888255, -0.058388, -0.851872, -0.413425, -0.758122, -0.231727, 0.356021, 0.180255, 0.449290, 0.849987, 0.169511, 0.141848, -0.797664},{0.185736, -0.877048, -0.599024, 0.364840, 0.601085, -0.604614, 1.967389, -1.174093, -0.159004, 0.315361, 0.411230, 0.802815, -1.468138, 0.223482, -1.068080, -0.323518, 0.432517, -0.063590, 0.174811, -0.380125, 0.841416, -0.702379, 1.413872, -1.023062, 1.816914, -0.304955, -0.462584, -0.219004, -0.399589, 1.001494, 1.206277, 0.754092, 0.994089, 0.965486, -0.236651, -0.394341, -0.528114, 2.307067, 2.047264, 0.462894, 0.402727, -1.542165, 0.419145, -0.017054, -0.598693, -2.422096, 1.314956, -0.045847, 1.456444, -1.732865, 1.376555, 2.036049, -1.238603, 0.275985, 0.540583, 1.520395, 0.269697, 0.841006, -0.180748, -0.341164, 0.835631, 1.075902, 0.713822, 1.867617, 0.102877, 0.905104, 0.952830, 1.401580, 0.000540, 1.902386, -0.502783, -0.210273, -0.154765, -1.103504, -0.583691, -1.883412, 1.202206, 0.294212},{-2.002821, 0.251334, -0.041680, 1.504531, -0.537839, 0.280853, 1.006542, -1.011568, 1.587394, -0.308041, 2.418930, 0.154098, -0.390219, -0.391247, -1.194344, -0.259051, -0.510569, 0.788128, -0.775801, 0.341085, -0.205450, -0.265293, 0.976800, -1.366131, -0.951037, -1.195047, -0.192167, -0.623330, -3.167560, 0.537171, 0.657304, -1.645622, 0.642779, -0.836539, 0.730243, 1.664675, 1.421000, 2.048058, -1.965122, 0.939391, 1.317722, 1.550991, -0.295595, 0.059415, -0.086382, -1.712903, 0.164651, 1.554827, -2.852803, -1.315220, -0.768400, -0.940828, 0.294721, 0.885119, 0.641429, -0.591760, -1.621274, 0.105725, 1.503236, 0.491154, 0.255156, -1.279668, 1.789868, -0.564793, 0.272184, -0.071266, -0.543082, 0.658672, -0.009103, -1.075137, -0.064447, 1.955197, -0.173543, -0.527722, 0.415067, 1.573716, -0.551371, 0.061205},{0.512567, 0.438897, 0.937654, 1.740463, -0.858621, -0.220459, 0.353429, 0.299959, 0.044631, 0.524806, 0.596654, 0.653276, 1.356777, -0.501132, 0.188389, 0.294138, 0.200064, -0.384080, 0.261561, -0.039556, 0.082252, -0.874826, -0.017779, 0.383148, 0.092027, -0.171715, -0.774222, 0.985138, 2.128544, 0.919315, 2.106347, -2.484647, -0.493943, 0.199880, 0.645135, 0.962382, 0.520078, 0.731344, 1.511453, -0.194945, 1.464916, -0.860272, 0.107299, 0.551178, -1.070378, -0.415880, 1.122373, 0.715737, -2.224622, -0.820591, 0.514257, -0.463683, 2.234120, 0.087727, 0.058486, -0.320816, -1.302186, 0.046227, -0.080270, -0.845291, 1.620522, -1.410422, -1.559285, -0.835184, -0.523783, -0.114715, 0.239462, -1.339582, -1.118855, 0.068203, 0.820866, -0.958465, 2.929709, -1.229716, 1.769394, -0.382065, 0.056792, -0.854522},{1.037729, -1.099238, -0.035660, 1.161786, -0.826940, -0.457166, -0.609850, 0.512307, -0.575849, -0.086791, 0.312903, 0.049663, 0.325387, -0.971619, -0.059308, 0.935445, 1.003307, -0.225549, 0.592460, 1.535644, -0.121780, 1.273182, -1.409950, 0.342338, 0.178671, 1.191193, 0.121807, 0.466292, 0.378567, -1.660876, 0.421538, 0.500505, 1.262155, -0.649436, 0.043944, 0.195323, -0.263161, -0.431642, 1.738590, 0.951079, -0.154895, -0.974655, 0.115253, 0.147944, 0.774032, -0.326422, 0.248892, -0.270836, -0.695679, 0.035438, -1.365294, -0.912970, 0.959785, -1.298103, -0.438867, -0.475948, 0.567614, -1.323455, 0.423992, 0.998671, 0.492421, 1.742096, 0.678154, -0.605085, -1.202251, -0.823500, -1.233171, -1.142424, -0.319440, -0.095829, -0.364216, -0.087616, 0.185023, -0.868934, 0.567535, 0.205525, -0.202047, 0.049525},{-0.698015, -2.133696, -0.606276, -0.430188, 0.554735, 0.460263, 0.808028, 1.522101, 1.117010, -0.741399, -0.723617, 0.895631, 1.094518, 0.181870, 1.783653, -0.019236, -0.358456, 0.614655, -0.618424, 1.074481, -0.260196, 1.383538, 0.117965, 0.424606, -0.486863, -1.401047, -0.702506, -0.870099, -1.252875, -0.747790, -0.386931, -0.809379, 1.212317, -0.566721, -0.685895, 1.704874, 1.029448, -0.032478, -1.014640, 1.177921, 0.357174, 0.418468, 0.736211, -0.000525, -0.270475, -0.557549, 0.553064, -0.715264, 0.674702, -1.806317, -0.998267, -0.806092, -0.468299, -2.457731, -0.465920, -1.300537, 1.927061, -1.023014, 0.301467, -1.988718, -0.675134, -0.235262, -0.519989, 0.131171, -0.981613, 0.205934, -0.424477, -0.408764, 0.989886, 0.879658, 0.069350, 0.646870, -0.787043, 1.857743, 0.816070, -0.783084, 1.620610, 0.005416},{1.370435, -0.944836, -0.696876, -0.282094, -0.837908, -0.377463, 1.217313, 0.735251, -0.981440, 1.211776, 0.805628, 0.938268, -0.726974, -1.459875, -1.227488, -1.383298, -0.780108, 2.388332, 1.281343, -0.853378, 1.074736, 0.840027, 1.307938, 0.570553, -0.586151, -1.134478, -0.356601, 0.979347, -0.792292, -0.925398, 0.298418, -1.363510, -0.880400, 1.344572, 1.378930, -2.944561, -0.432189, -1.295765, 1.471376, 0.054928, -0.393631, -3.162094, 0.069102, -1.112309, 2.668043, -2.030173, 0.020600, -0.179623, 0.712782, 1.450643, 1.189639, 1.660757, 0.390890, 0.265454, -0.536437, 1.347345, -2.158940, 0.525382, -2.217792, -0.180484, -0.776957, -0.101268, -1.505058, 0.629507, -0.229439, 0.770076, -1.346723, 1.441984, -0.278839, -1.132482, -1.251469, 0.801334, -1.086593, -1.111536, -0.047014, 0.176486, 1.644399, -1.378915},{1.360864, 0.955675, -0.441343, -0.078709, -0.122238, 1.086820, 1.525057, -0.204948, 1.254010, -0.285047, -0.797135, 1.260759, -0.672936, 1.072903, -0.999123, -1.613291, 0.396254, -0.491402, -0.566626, 0.423384, 0.445704, 0.764340, 0.085044, -2.832543, 1.922107, -1.319229, 0.320379, -0.851074, 0.278806, -0.549534, -1.014192, -0.414460, 0.090742, 1.074714, 0.814254, 0.939178, -0.835661, -1.325819, -1.853411, -0.190964, 0.437221, -0.918518, -0.004180, 0.435032, 0.483201, 0.129534, 1.519055, -2.207538, 1.108588, -0.147921, 0.639367, -1.063523, 0.235090, 0.909594, 1.222631, 1.077133, 0.639832, 1.448918, -1.297568, 0.286526, 0.434066, -1.019073, 0.652850, 0.469702, -0.494856, -1.138069, -0.639891, -1.225317, 1.236154, 0.370836, -1.231872, 0.916542, -1.026610, -0.700917, 0.085473, -0.010327, 1.030335, -0.590298},{0.266561, 0.282596, 0.342773, 0.486845, -1.255263, -1.908535, 0.423568, -0.187814, 0.211645, 0.223743, 0.084382, 1.074849, 0.698559, 0.250985, 0.079712, -0.127542, -2.337395, -1.089275, 0.650671, -0.871714, -1.207123, -0.261814, 0.930190, -0.863564, 0.046897, 1.173687, 0.694539, -1.948034, -0.787251, 1.571811, -0.674507, 0.009258, 2.876024, -1.004835, -0.075793, 0.518816, -0.523540, -0.555694, 1.073346, -0.658674, 0.883891, -1.493094, 2.008823, -0.230477, 1.577468, 1.100333, -0.665505, -0.567480, 2.053792, -0.018312, -1.317636, -0.834565, -0.548535, 0.187831, 1.057349, 0.779965, 0.090960, -0.781438, -1.036651, 1.512569, -0.160667, 0.710259, 0.618259, -1.323858, -0.326198, 0.639608, 0.279383, 0.524490, 0.502851, 0.079871, 1.587494, -0.558281, 0.190681, -0.663208, -0.726796, 0.956886, -1.302430, 0.199252},{-0.344783, -0.717175, -0.611074, 0.260563, -0.610123, 0.770578, 0.544228, -0.124139, -0.452142, -1.406362, -0.965510, 1.061925, -0.147890, 0.784081, 1.304171, 0.598303, 0.327648, 0.361274, -0.597091, 0.537376, -2.032180, -1.313533, -0.624451, 0.711026, -0.179622, -1.088107, 1.198753, 0.663527, 1.379136, -0.354507, 0.893255, 1.007813, -0.578685, 0.049618, -0.054155, 0.419719, 0.864162, -0.164606, 0.199650, 0.312863, -1.528943, -0.330987, -0.721897, -0.135159, 1.341535, 0.139309, 0.451627, 0.374704, -0.670460, -1.200036, -1.696689, 0.168114, -1.345063, -0.418298, 0.091415, -0.835647, 1.392285, -2.472760, 0.983924, -0.104302, -1.558614, 0.553980, 0.406007, 1.143841, 0.222021, 0.501728, -1.717293, 0.071212, -0.494166, -1.594124, 0.162996, -1.011719, 1.058152, -0.412691, 0.047146, 1.703470, -1.210002, 1.115410},{1.140404, -0.354591, -1.202127, 1.814303, -0.107270, -1.183079, -1.053318, 1.116049, -1.734096, -0.697380, 0.703729, 1.710443, -0.571990, -0.716748, 0.633877, 0.183590, -0.744324, -1.219622, 0.140477, 1.460281, 0.485045, -1.333251, 1.785896, -0.108840, 0.564232, -0.052536, 0.012517, -0.894760, -0.158768, -0.342060, -0.230427, -1.249756, -0.203901, -1.523438, 0.820442, -0.101164, -0.554263, -2.489522, -1.978333, 0.409720, 1.966574, -0.364952, -0.607178, 0.183362, -0.387861, 0.369621, 0.737071, -1.885171, -0.754685, 0.349231, -1.217029, 0.143547, -0.142770, 0.401653, 1.414366, -0.157160, 0.934060, -0.668116, -0.541852, -1.777509, 1.473644, -1.988947, 0.129222, 0.769266, 0.053479, 0.001084, 0.426286, 0.242533, 0.687570, -0.794593, 0.764736, -0.507202, 0.911487, 1.097497, -0.516436, 0.473549, 0.504679, -1.033236},{-0.825356, -1.045610, -0.021356, -1.638970, -1.513970, -0.989045, 0.085164, 0.118422, 1.254650, -1.009340, 1.499533, -1.413962, -0.575511, -0.076948, 0.829373, 1.633555, 0.886765, -0.554759, -0.207539, 0.866763, -1.867016, 3.200173, -0.492289, -1.550970, -2.290355, 0.581676, 1.101000, -0.578456, -0.083431, 0.117115, -0.274544, 0.934527, 0.976604, 1.015057, -1.394359, -0.821816, 0.005762, 0.869381, 0.654165, -0.631535, -3.381706, 0.126392, -0.088661, 0.563596, 0.703425, -2.330980, -0.460015, -0.557686, -0.823500, -0.067111, 0.998383, -0.217727, -0.243329, 0.346832, 0.564412, 0.013725, 1.309834, 0.525168, -0.674141, 0.537549, 0.051268, 0.645357, 0.218043, 0.128796, 1.140019, 0.650775, -1.265636, 0.972604, -1.390320, 0.432952, 0.693935, 0.392549, -0.279788, -0.112840, -0.411610, -0.229369, 1.923839, -0.986445},{0.857686, -0.083905, -0.184711, -0.074415, -0.429237, 0.402442, -1.148323, 0.690878, 0.317395, 0.538174, 2.020091, -1.796886, 1.778769, 0.069937, 0.593311, 0.283500, -0.125361, -1.639857, 0.281835, 0.320310, -0.080439, 0.877030, -0.140321, -0.005768, 0.451329, -1.057110, -0.592313, 0.829520, -1.039219, 0.783001, -0.514129, 0.512452, -0.038308, 0.555074, 1.258943, 0.959224, -1.687030, -0.469856, -0.892714, 1.349409, -0.889981, -0.472882, 1.461783, -0.424804, -0.562104, -0.686879, 0.576428, 1.612841, 0.388975, -0.370602, 0.252860, 0.976924, -1.156048, -0.638993, -0.016774, 1.198041, 1.464710, 0.441802, 1.441360, 0.418981, 0.636946, -0.757468, -0.067040, 1.212755, 0.253656, -0.790958, -1.099229, 0.015778, 0.865273, 1.111768, 0.233854, -0.755676, 1.000573, -1.112021, 0.006025, 1.216101, 0.179077, -0.928688},{0.552677, 0.977116, 1.468861, 1.319587, -0.200346, -0.992124, -0.795728, 0.863956, 0.377575, -0.397019, 0.106804, 3.381392, 0.225463, 0.308451, 1.294014, -1.474524, -0.337998, -1.041920, 1.520982, -0.627754, 0.531785, -1.218596, 0.821022, 2.493850, 1.017142, 0.322970, -0.221850, 1.080548, -0.131044, -0.883551, -0.450787, 1.548469, 0.713392, -0.591444, 0.204112, 1.626400, -0.838685, 0.252086, 1.875368, 1.273215, -0.248336, 0.124557, -0.135152, -0.312014, 0.470515, 1.028982, -2.576270, 1.267212, 0.639977, -0.850562, -1.290703, 2.583632, -1.627770, -0.168771, 1.749500, 2.787434, -0.344937, 0.190436, -0.579281, 0.347871, 0.095340, -1.066163, 0.119009, 0.193875, -0.449604, 0.563802, 0.233847, -0.602701, 0.148090, -1.459982, 0.198420, 1.297643, 0.706006, -1.172884, -0.070522, 0.736302, 0.090592, 0.077851},{-0.859458, -0.720062, 0.678561, 0.830752, 0.560602, -0.124964, 1.408707, 0.309580, -0.648566, 1.423989, -0.603390, 0.271580, 1.266152, 0.255370, -0.338661, 1.128123, -0.068062, -2.162421, -0.613612, -0.114695, 0.577261, -0.229727, -1.438442, -1.249240, 0.073191, -0.664342, 2.367558, 0.777644, -0.890293, -1.483205, -0.419052, 2.095685, 0.323372, -0.090781, -1.290453, 2.065810, -0.696193, -0.632172, 0.128734, -0.138960, -1.948791, -0.308845, 0.952883, 2.643179, 1.675307, -0.748609, 0.873399, 0.152136, 0.216417, -0.319524, 0.642655, 0.597964, 1.062407, 0.447065, 0.544973, 0.017747, 1.010317, 1.446162, 0.819118, 0.438985, 0.988837, 1.513944, -2.174949, 0.340114, -1.079547, -0.027235, -0.341846, -1.443315, 0.608072, -2.665720, -0.225163, -0.676064, -0.284709, 1.320632, 2.316829, -1.448217, 0.989698, 0.833296},{-0.537837, -0.060245, 0.826209, 0.443499, 0.410835, -0.578884, 0.642958, 0.201679, -0.147057, -0.716857, 0.548122, -0.020984, 0.683076, -0.802841, 0.797594, -0.084685, -1.949705, 0.847203, -0.284813, -0.277394, -1.452226, -0.397705, 0.430415, 0.434707, 0.606479, -0.685785, -0.574501, 1.874879, 1.970148, 1.240457, 0.666553, 0.119526, -0.799864, 0.244621, -0.632283, 0.705887, 0.096073, 0.380945, -1.040010, -0.489419, 2.533322, 1.485702, 1.101010, 0.539387, -0.283934, 2.419120, -1.249284, -0.809350, 0.860707, 1.229152, 0.484151, 0.227969, -0.658190, 0.239523, -0.619281, -2.235512, 1.012100, 0.168777, -0.814302, -0.448094, -0.514672, -3.114763, -1.222525, 0.793765, 0.271054, 0.163168, 1.885536, -1.576278, 1.080989, -2.145313, -0.304478, 0.030243, 0.853717, 0.351917, -1.383713, -0.838965, -2.361427, -1.176421},{-0.197906, 1.266233, -0.760181, 1.058895, 0.328863, -0.871236, -1.372187, 0.971570, -0.844918, -0.812523, 0.551968, -1.570604, 1.129927, -0.322145, -0.527650, -0.565018, 1.016601, -0.856435, -0.797786, -0.086414, 0.868345, -0.070326, 1.236049, -0.344073, -0.403644, 1.636188, 0.612408, -1.934936, 0.661988, -0.624273, 0.109618, -1.543060, 2.695301, 0.327353, 1.572896, -0.080838, -0.290429, 0.802285, -0.314162, 1.695854, -0.322251, 0.057998, 0.135790, -0.262133, -0.072020, -0.563321, -0.129461, -2.871672, 1.037643, -1.399415, -0.747641, -0.800907, 0.903143, 0.026847, -0.529646, -0.285156, -0.851106, -1.909727, -0.264315, -0.084713, -0.899324, -0.530982, -0.864157, 0.801214, -1.433195, 0.438844, 0.068528, -0.526696, 0.383060, -0.593759, 0.213399, 0.774556, 0.019293, -2.373129, -0.708466, 0.824504, -0.289252, -2.328544},{1.393627, 0.223380, 0.851266, -0.890282, -0.303956, 0.698504, 0.606961, -0.434786, 2.224519, 0.850092, -1.971223, 0.386309, -1.860293, -0.636971, -0.877075, 1.739511, 0.020745, -0.236212, 1.297360, 1.401964, -0.561722, 1.059895, -2.210060, 0.077588, 0.973777, -0.900863, -0.266498, -1.532192, -0.521517, -0.320155, -1.680284, -2.384339, -1.192456, -0.424535, -0.638260, -0.616231, -0.324072, -1.257577, 0.316757, -0.187691, 0.286170, 0.620693, -0.547269, -0.563388, -1.126609, 0.154962, 0.459964, -0.713014, -1.523535, -0.692468, -0.842139, 0.998107, -0.447846, 0.733442, -1.000792, -0.508959, -0.378608, -0.375582, -0.979476, 0.400832, -0.728841, -0.590090, -2.116175, -0.629514, -0.009182, -0.542561, -0.799089, 0.687259, -0.469272, -0.402428, 0.757015, 2.031780, -0.549400, 0.169940, -0.308680, 0.679662, 1.320150, -1.128823},{0.722196, -1.078287, -0.392357, 0.089428, -1.150140, 0.740679, -1.305946, 0.544172, 1.643386, -0.390052, 1.890610, -1.733365, 0.286755, -0.739831, 0.444937, -0.846233, -0.808505, 0.265475, 0.478063, 1.584293, 0.241626, 1.070274, -0.778301, 1.253233, 0.164632, 0.080146, -2.421919, -0.995938, 0.652063, -1.222599, -1.050301, 0.716733, 0.825554, 0.277447, -1.093925, 0.603929, -1.219404, -0.662534, -0.667314, 0.022610, -0.790913, 0.429027, -1.775174, 0.919365, -0.119054, -0.551601, 1.462406, 2.220200, -1.437407, 0.134242, 0.487798, 0.607478, 0.789529, -0.031380, 0.152819, 2.372512, -0.763060, -2.626030, -1.340714, 0.810181, -0.086801, 0.298735, 0.206172, 0.137084, -0.589036, -0.319520, 0.975236, 0.620206, 1.925835, -0.212504, 0.579715, 0.155075, 0.276961, -0.213983, -0.116460, -0.036435, 1.165476, -0.253109},{-0.987277, -0.364675, -0.264804, 1.480477, 0.483057, 0.931666, -1.404301, -0.968439, 0.402209, -0.023640, -0.194039, 0.689320, -1.206270, 0.539173, 1.196571, -2.232798, 1.196858, -0.043078, 1.664863, -1.719956, -0.370570, -1.494303, 0.304783, -0.123496, -1.639556, 2.245551, -0.586986, -0.292743, -0.143344, -0.404758, -2.045265, -1.298042, -0.081384, 0.417606, -0.500566, -0.910632, -1.960621, -1.144479, -0.141764, -0.150258, 1.463094, 0.812746, 0.576203, 0.511263, 1.842931, 0.013119, -1.418842, -0.928935, 0.195170, 1.006850, 0.959961, 0.207131, 0.048945, 0.224294, 0.300218, -1.500284, 0.773606, 0.470765, 1.093183, -0.286278, -0.805638, 0.135974, -0.133847, 0.961164, -1.893606, -1.011317, -0.454790, 0.070826, -0.594144, 0.111160, -0.519327, 0.430446, -0.846788, 0.954622, 0.027356, 0.518372, -1.117691, 0.268446},{-0.837382, -1.190867, -0.199842, -0.058238, -0.558369, 1.838926, -0.623417, -1.687103, 1.189517, 0.746925, 1.577787, -0.331856, -0.231039, -2.284487, 0.395178, 0.235073, -0.866898, -0.506183, -0.260884, 0.730599, 0.740890, -0.638476, 0.441789, -0.607368, -0.638576, -0.568371, -0.062758, 0.641059, 0.371380, 1.204122, -0.498793, 1.034272, -0.924838, 0.313101, 0.625503, 0.605496, -0.572926, 1.056978, 0.626468, 1.053286, 0.451961, 0.850009, -0.216211, -0.441272, -0.407804, -0.776287, 2.235483, 0.869422, 1.187279, -1.141647, -0.817614, -1.417815, -1.321505, -0.799487, 1.533547, -0.721327, 0.104401, 1.513191, 0.824847, -0.302298, 1.250089, 2.407109, -0.041936, -1.116362, 1.798214, -0.018333, -0.200456, -2.166893, 1.118874, 2.260085, -0.458259, -0.159351, -0.030807, -0.210622, -0.696375, 1.411936, -0.160560, 0.032827},{-0.157110, -0.434127, 1.774601, 0.474035, -1.125574, 0.194534, -0.426620, -0.625403, 0.071442, 0.209358, 0.203167, 0.725955, 0.365495, -1.154019, -0.767618, 1.595132, 0.489513, -1.128429, -0.485282, -0.342081, -0.806854, -1.124660, 1.449302, 0.985436, 0.397211, -0.254451, -0.288173, -0.574066, -0.716364, -0.330594, 0.979563, 0.839665, -1.472779, 1.196945, -0.203483, -1.904719, 0.525370, -0.715368, -0.426373, 1.594544, 0.671851, 0.833691, -0.449442, 0.970212, -0.942258, -1.491437, -0.524461, -0.572671, -0.623376, -0.541438, 0.598247, 0.514808, -0.912493, -0.929022, 1.717528, -0.625139, 0.063112, -1.868177, -1.423834, -0.068677, 1.053259, 1.159535, -1.978198, 0.922817, -0.666179, -1.304044, 0.970794, -2.429549, -1.078521, 0.248746, 1.419497, 0.478065, -0.882702, 1.452348, 0.434490, -0.109954, -0.263017, -0.126306},{-0.295961, 0.964100, -1.772540, -0.235947, 0.037753, 0.011325, -0.519466, 0.081796, -0.908898, -2.131307, 1.152544, 1.667211, -1.248119, 0.310054, 1.230223, -1.066183, 1.118724, -0.557467, 0.324618, -0.009564, -1.405261, 1.211257, 0.869523, -2.042475, -0.483115, 0.458541, 0.116654, 1.209106, -0.866662, 0.479384, 1.136656, 1.252730, -0.537797, -1.083149, -0.116735, 0.798481, 1.577063, 0.819093, -0.992910, 0.100207, -0.846492, 0.340638, -0.021338, 0.449168, -1.212269, 0.284580, 0.697193, -0.283711, 1.713202, -0.335924, 1.050453, -1.193400, 0.439461, -1.241947, -0.744634, 1.462365, 0.480783, -0.599881, 0.082802, 0.155997, -1.942238, 0.964525, 0.301883, 0.079103, -1.440251, -1.019173, 0.212409, 1.394643, -0.194812, -0.144507, -1.126647, 0.230655, -1.134875, 1.804235, 2.574968, -1.229677, -1.673985, 0.071243},{1.740561, 0.702157, -1.390983, -0.260918, -0.306784, 0.398285, -0.171284, -0.150131, 0.322751, -0.607210, -1.686223, -1.569930, -0.256110, -1.333118, -0.337471, -1.818753, -1.566601, 0.878796, 0.129908, 2.184061, -0.105815, 0.098234, 0.326057, -0.050224, 1.348468, 1.210673, -0.555844, -0.075673, -1.533033, 1.531852, -2.502542, -0.678446, -0.570288, -0.605584, -0.202091, -1.693228, 1.510136, 0.993889, -0.289728, 0.042270, 0.338284, -1.029043, -0.495199, 0.194275, 0.395860, 0.234629, 1.087737, 0.417674, 1.097922, 0.213776, 0.036195, -0.443682, 0.294546, 0.992427, 0.395214, 0.254403, -1.657987, -0.923122, -0.536006, -0.811236, -0.066120, 0.663948, 0.866652, -0.422143, 1.014993, 0.075213, 0.034003, 0.447010, 0.512562, -0.052902, -0.438524, 0.207298, -1.087468, -1.922158, 0.524910, 0.476224, 0.889347, 0.248292},{-0.057844, -0.340947, -0.156372, 1.299740, 0.262235, 0.095210, 0.670168, -1.434638, 1.003064, -1.017434, 0.932349, 1.265907, -0.170922, 0.127820, -1.286200, 0.687186, -0.976805, 0.350099, -0.817559, -0.246306, -0.791801, 1.200333, 1.143336, 1.667735, -0.381538, -0.495019, 0.387711, -1.175858, -0.876824, 1.379613, 0.495858, 1.011967, -2.231995, 0.396002, 0.839386, -0.131318, -2.053840, 0.271644, 1.036582, 0.877738, -0.481462, 0.202301, 0.315459, 0.921913, -0.104822, 0.735953, -0.273952, -0.680756, 0.273040, 0.208984, -0.754461, 2.041900, 0.689959, -1.653640, -0.468705, 0.539064, 0.038574, -0.239987, -0.291211, -0.633830, -0.535676, 0.292556, 0.988266, 0.337468, 1.074110, 1.766240, -0.008317, 1.432598, -0.760488, 1.268983, -0.619161, -0.436811, 1.512105, 0.249895, 1.690114, -1.412422, 0.878906, -0.720264},{0.391494, 1.923418, -1.377307, -0.714610, -0.536884, 0.203881, 1.292212, -0.167567, -2.118248, -1.437489, 1.226636, 0.157892, -1.096506, 0.790351, 0.747717, -0.485742, -0.135387, 1.055778, 0.024449, 0.743927, 0.426130, -0.576777, -0.542784, -0.677964, 0.344197, 0.485589, -0.391861, 0.220303, 0.884003, -1.511273, 0.363079, 0.329058, 0.877131, 0.761786, 1.792426, 0.743876, 0.459479, -0.249490, 1.468414, -0.446219, -1.006652, -0.820581, 1.224168, -0.305503, -0.249984, -0.367505, -0.027644, 0.777205, 0.241485, 0.042199, -0.646766, -1.180929, -1.255569, 1.321001, -1.095658, 2.241104, 0.240807, -0.710206, -0.630055, -0.092306, -0.111034, -0.151546, -1.193815, 0.412783, -0.430608, 1.155640, 1.138061, 0.916452, 1.149007, -1.381054, 0.525400, -0.140603, 2.119745, 1.262769, 0.529409, 1.391050, -0.059438, -0.684938},{-0.790714, 0.181855, 0.585655, -0.376685, 0.886809, 0.566839, -0.657239, -0.174109, 1.421346, -0.661520, 0.828221, -0.664644, 0.620459, -0.415888, 0.404102, 0.248676, 0.901185, 0.035424, -0.707180, 0.734981, 0.828590, 0.100660, -0.008167, 0.023567, -1.996052, 1.632199, 0.728950, -2.060919, 0.784708, -0.706709, 1.313793, -1.526283, -1.476811, 0.225746, 0.223686, -0.404969, 1.556742, 0.767514, 1.527539, -0.013889, -1.102594, 0.544840, -0.704854, 0.402598, 0.618359, -0.353618, 0.581184, -0.906127, 0.257738, -0.719674, 1.876918, 0.656838, 0.976086, -1.069569, 0.422687, 0.691699, -0.113311, 0.440938, -1.297157, 1.696371, 0.987111, 2.039731, 0.246526, -1.489078, 0.132810, 0.297652, -0.347774, 0.627955, -0.531529, -0.357258, -0.190694, 0.319382, 0.159843, -1.654022, -0.438172, 0.212447, 0.399088, 0.277309},{-1.183572, -0.636469, 0.419680, -0.385915, 2.053876, -1.168171, -0.470570, -1.949176, -1.275009, -0.104987, -0.647789, 0.591289, 1.407114, -0.808438, -0.445911, 0.370344, 1.010619, 1.561307, 0.758531, -1.121137, -0.472485, 0.307212, 1.309889, -0.020021, 1.175450, 1.315486, -0.603599, -0.269163, -1.312430, 1.049819, -0.396775, -0.459350, -0.656349, -1.409723, 1.455717, -0.444230, 0.849305, -0.413475, 1.768560, 0.888219, 0.438329, 0.365943, -0.164791, -0.488977, -0.170233, 1.183832, 0.643301, -0.541170, -0.500755, 1.746329, 0.630194, 1.241189, 0.354914, -0.105926, 1.399211, -0.792237, 1.098902, 1.254893, -1.527119, -0.885007, 1.175028, -0.436171, -0.442260, -0.250296, 0.138130, 0.207478, -0.161543, -1.976266, 1.010458, 1.719274, -1.706568, 1.495046, 0.537185, 0.540933, -0.844496, -0.118909, -0.078853, 1.989393},{0.272111, -0.718604, -0.424083, 1.081810, 0.423282, -0.377892, 0.290977, 0.961280, 0.465053, -1.267620, -1.793403, -1.058590, 0.795376, 0.020271, 0.927051, 0.949658, 0.087594, -0.716578, -0.339212, 0.245056, 0.418918, -0.752727, 0.978756, 0.114236, -0.210769, -0.285967, 0.859547, -0.478863, 2.001862, 1.605547, 0.358662, -1.510319, -0.040896, -0.482434, -1.924087, -2.451883, 1.991892, 0.183396, -1.077234, -0.244543, 0.460709, -0.787943, 0.723823, -0.418159, 0.603440, -1.671371, 0.000708, -1.691561, -0.618059, -1.109924, -0.202971, -0.302680, -0.804223, 0.460984, 0.927962, -0.461785, -2.132504, -1.014366, 0.135956, -1.748463, 0.030039, -0.927974, -0.022139, 1.087001, 0.400240, 0.231995, -1.887082, 1.315416, -0.194173, 1.172665, -0.791711, 0.405130, -0.463841, 0.451884, 1.249509, -0.575495, 0.205615, -0.609213},{0.512912, 0.759696, -0.081902, 0.440542, 0.328194, 1.745655, 1.150386, 0.962678, -1.639124, 0.825517, -0.722484, -0.266819, 0.210300, 0.992394, -1.260880, 0.119906, 1.524760, 0.941966, 0.921732, -0.677122, -0.394606, -2.241575, 0.435727, -0.238833, 0.025250, -0.338160, -2.395430, -1.025154, 0.432693, 0.578078, -0.621578, 0.745536, 0.538645, -0.364590, 2.617634, -0.019653, 1.062433, -1.242757, 1.032836, 1.448228, 0.201527, 1.201591, 0.711242, -1.506982, -1.269546, -0.132086, 0.587582, 0.764407, 0.672846, 0.154799, 0.102190, -0.385997, 0.644695, -1.290497, 0.473552, 0.487391, -0.910279, -0.859154, -0.654067, 0.597072, 0.925569, -0.385256, -1.704793, 0.329228, 0.684021, -0.918076, -0.671410, -0.122267, 0.623805, -0.145991, 0.499636, -0.460837, 1.000599, -0.217701, 1.940251, -0.613767, 1.902840, -0.717690},{-0.740004, 1.032252, -1.962366, 0.046903, -0.878521, -1.097669, 0.229515, -0.466512, -1.236704, -0.048284, -0.112712, 0.289263, 0.182068, 2.811005, -2.329282, 0.178288, -2.643279, 0.272414, 0.438237, -0.249734, -0.355672, 1.096345, -0.066248, 0.472297, 0.773195, -0.392641, -0.793280, 3.336091, 0.440017, 0.227111, 0.069896, -1.655785, 1.320156, 1.288523, 0.573938, -0.392020, 0.256187, -0.238575, 0.530825, 0.931504, 1.398353, 0.615401, 0.345629, 0.580364, 1.286785, -1.481415, -0.638394, -0.464483, -0.817344, 0.482326, 0.591233, 0.615021, -0.526293, 0.486694, -1.691249, 0.488345, 1.427633, 1.009037, 0.696342, -0.066078, -0.582540, 0.405969, -0.022883, 0.330165, 0.618386, 0.188560, -1.213047, -0.004724, 1.255268, 1.317742, 0.557537, 0.384260, 0.841261, 0.735157, -2.338310, 0.386643, 0.315458, 1.613995},{-1.919358, 0.728659, -2.532372, -0.517329, -0.505492, 0.074170, -0.653299, 0.292934, 0.539477, -0.324353, 0.372218, 0.952649, 0.852077, 0.533572, -1.535897, 1.850247, 0.450058, 0.783105, 1.453160, 0.373444, 0.818359, -1.397642, -0.148635, 1.379794, -0.347056, -0.698596, -0.031990, 0.325811, 0.475670, 0.865372, -0.238747, -0.101486, 0.569673, 0.030569, -1.075196, -0.564332, 0.485101, 0.897078, 0.650530, -0.056799, -0.431108, -1.341480, 0.326985, -0.122739, -0.157135, -0.803396, 1.017781, -0.865496, -1.494981, 1.511139, -0.915863, 0.269657, 0.506880, 0.015791, -1.468918, 0.067443, -0.696810, 0.563908, -0.989719, 0.441930, -0.258136, -0.199519, -0.496025, -1.706365, -0.528372, -0.283049, 1.317470, 0.646770, -0.437017, 0.311441, -0.958497, -0.605030, -1.923291, 0.633945, -0.412119, -0.914798, 0.600798, 0.146480},{-0.661405, 1.022878, 0.069946, -1.461876, -0.098113, -0.804730, -0.377031, 0.177574, -1.617074, 0.262792, 0.424622, -1.605709, 0.123073, 1.714304, 0.786256, 2.157766, -0.391455, 0.226569, -0.461302, 2.591007, -2.065666, 0.003710, -0.720694, -1.030881, 1.046244, 1.771737, -1.912286, 0.845056, -1.149170, -0.200927, 1.927110, -1.051826, 0.466081, 0.358757, 0.489777, 0.212333, -2.103762, 0.015504, -0.112813, -1.886757, 1.299848, 0.727014, 0.478424, -2.074682, -1.034420, 0.291177, 2.027488, 1.009061, 1.063019, -0.090043, 0.854532, 0.499773, 0.080852, -1.088546, 1.882370, -0.489778, -0.823128, -1.257163, -0.079645, -0.037675, 0.986031, -0.782064, -0.844859, 0.247282, 1.103613, -0.110296, -1.869771, 1.270739, 0.804433, 1.338970, -0.927761, -0.155659, 0.821856, -0.726684, -0.872093, 1.520730, -0.172454, -0.357152},{-0.072235, -0.550573, -1.783178, 0.136045, 0.030254, -0.986447, -0.311330, 0.175735, 0.499280, 0.106373, -1.184022, -0.013162, 0.624898, 1.077810, 0.179799, -0.219579, 0.346563, 0.858437, 0.697301, -0.444808, -1.267369, 0.336358, 0.892696, 0.399161, -0.297679, -0.764667, -0.794542, -0.021509, -1.835926, -1.033961, -0.936061, 0.171412, 0.789375, -0.082677, -1.203057, 0.668813, -0.919555, 0.714790, 1.322968, 0.436533, 2.101625, -0.162008, 1.112312, 2.741083, -0.678649, 1.585069, -0.583472, -1.122885, -1.275494, -1.138317, -0.977952, 1.331765, -0.517352, -0.232272, -0.450875, 0.256358, 0.486680, 0.117967, 0.728188, 1.601151, 1.374887, 0.126003, 0.359638, 0.814394, -1.210403, 1.305087, 1.163805, -0.134098, 0.899913, -0.976112, -0.193608, -1.933225, -0.382604, -1.319895, 0.112106, -1.574469, -0.212405, 0.814345},{-2.142858, 0.250394, -0.180872, 0.676055, -2.009686, -0.649542, -0.367432, 0.693431, 0.251647, -0.113548, 0.207452, -0.993166, -0.284646, -0.636373, -0.424378, -0.846611, -1.323650, 0.303408, 0.493168, 0.594370, -0.380824, -0.654497, 0.140973, -0.654801, -1.086264, 0.350837, 1.355648, -0.444742, 1.828845, -1.068344, 0.138402, 0.574102, -1.160359, -0.962425, -0.169920, 0.596007, -1.834912, -1.723995, -1.118137, 0.948606, -0.630281, 1.548840, -0.362615, 1.061358, -1.530111, -0.113911, -0.391080, -0.488359, -2.242538, 0.863406, 0.003839, 1.178555, 2.951045, -1.554732, -0.520561, -1.089172, -0.424120, -0.738391, -1.070322, 3.409103, 0.608565, -0.079070, 2.463844, 0.926346, -1.309806, 1.921643, 0.814932, 0.368525, -1.002622, -0.269729, -1.677028, -0.304278, 0.110581, 0.657997, -0.279514, -0.236828, 0.759143, -0.150679},{-1.971103, 1.031786, -1.705997, 0.103231, -0.069117, -0.397959, -1.374331, 1.531832, 1.397100, 0.622620, -1.841046, 0.894508, 0.289326, -0.905023, -1.395677, 0.309193, 0.321030, 0.925253, -0.621223, 0.105929, -0.039518, -0.218351, -0.851989, 1.592264, 0.526860, 0.465736, 1.215242, 1.530213, -0.412318, 2.140971, -0.242511, 0.152422, 0.694614, 0.199186, -0.006084, 1.576560, -0.313869, -0.799212, 1.276991, -0.822801, 0.806589, -0.586497, 1.302922, 1.215305, -0.984286, -0.491548, -0.003556, -0.193644, -1.599362, -1.419301, 0.727012, 0.664263, -0.739986, -0.389565, 0.187355, 0.055840, -0.250403, -0.314739, 0.618496, -0.501518, 0.487176, 1.549351, 0.145518, 0.454715, 0.458451, -1.415865, -0.308205, -0.994554, 0.075927, 0.706749, -0.701234, 2.742079, -0.984958, 1.187663, -1.311335, 0.856414, -0.327701, 0.167604},{1.804389, 0.721037, 0.810820, 0.205137, 0.352891, 0.179095, -0.480237, 0.140200, -0.618763, 0.545800, 0.163097, -0.527425, -2.291825, 1.289208, -0.230318, -3.015361, -1.233662, 0.277134, 1.110999, 0.600272, 1.576357, -1.867587, -0.176105, 0.558779, -0.825544, -0.375866, -1.985978, -0.547453, -0.329489, 1.440873, 0.633938, 0.791166, -1.413563, -0.130902, 1.043723, -0.013890, -0.156392, 0.566210, 0.099344, 0.190804, -0.233734, 0.756050, 0.663768, 0.099266, -0.609456, 0.326725, 0.329500, -1.465872, 0.918195, 1.438678, -0.122335, 0.510343, 0.851814, -0.053960, 0.384635, -0.020855, -0.836590, 1.467579, -0.739066, -1.464370, 2.212568, -1.169849, -0.171818, 0.262370, -2.237600, 1.556895, -1.474070, 0.106133, 0.461234, 2.343848, -0.675670, 2.218731, -1.550072, 0.186050, -0.017232, 0.568849, 0.966765, 1.043268},{0.416054, 0.766139, -1.493504, -0.863316, 0.443271, 1.824222, -1.831578, -0.398423, -1.005400, -0.568873, 2.138033, -0.679480, 2.147998, -1.726837, -0.538217, -0.296481, -0.092943, 1.156099, -0.502934, -0.020356, -0.188162, 1.502421, 0.544157, -0.437531, 1.041573, 0.476451, 0.265040, 1.303477, 0.133050, -0.992762, 0.103937, -1.012151, 0.578461, 0.702997, 0.177742, 0.555444, 1.937329, -0.045464, -0.538505, -0.307442, 0.908383, 1.320786, -0.562452, -0.280010, -1.189613, 1.022200, -0.431075, 0.196421, 0.115858, 1.278745, -0.530078, 1.544363, 0.674546, 0.186826, 1.425533, -0.881787, 0.022741, 0.479502, 1.143384, 0.230382, 1.472979, -0.448884, 1.417964, 0.138777, -1.843275, -2.085446, -1.673953, 0.296920, 1.058055, -0.521336, 0.649500, 1.382171, -1.408534, 1.333562, 0.114314, 0.007995, -0.504720, -0.858995},{-0.889539, -0.893065, -0.941332, 1.196635, -0.723729, -0.322180, 0.902812, 0.513585, 2.573038, -0.995585, -0.591780, 2.095949, 0.247223, -0.502066, 0.451085, -0.156180, 0.083320, -0.951657, 0.210346, 1.347724, 0.881154, -0.407112, 1.475853, 0.986602, -0.378928, -1.696769, -1.170084, -0.043479, -0.264942, 0.366318, 0.063248, 0.026889, 1.241233, 0.020993, -1.782214, 1.057955, 0.160964, 0.443801, 0.105762, -0.514347, -0.615090, -0.216463, -1.877422, -0.741614, -1.000097, -0.148820, -0.406277, 0.928589, 1.586824, -1.048994, -0.280798, 0.132296, -0.873233, 1.051369, -0.961427, -2.350614, 0.125599, 0.214349, 0.057164, -1.735916, 0.304278, 0.879227, 0.217257, -0.033566, -0.409754, -0.119722, 0.434488, 0.652320, 0.164130, -1.538829, -0.147366, 0.147253, 0.658620, 1.406574, 0.479154, -0.693886, -0.574451, -0.603700},{1.708085, -1.241710, 0.694378, 0.440616, -0.596438, -1.411736, 0.139076, 0.628581, 0.417304, -0.013099, 0.483838, 0.838632, -1.805198, -1.534052, -1.397977, 0.587001, -1.390105, -1.788343, -0.790372, -0.343723, 0.023655, 0.362714, -1.697230, -0.033590, -0.849742, 3.042114, 0.687783, -0.301529, 0.325635, -0.818891, -0.150163, 1.395332, 1.111177, -0.191634, -1.343271, -0.117232, -0.470137, 0.363643, -0.833174, 0.891151, 1.515649, -0.274722, 0.052332, 0.306304, -0.048460, 0.160195, 1.024801, 0.593649, 0.863087, 1.414164, 0.885138, 2.303438, -2.821050, -1.945555, 2.697072, 2.641664, 0.803925, -1.138404, 1.763373, -2.233611, 1.517150, 0.054717, -0.801668, -0.887363, -0.448528, -1.527424, -1.531171, 1.070713, -2.432184, -0.331745, -0.882553, 0.087632, -0.181291, 0.480292, 1.901050, 0.997020, 1.392997, 1.474424},{-0.421648, 0.414189, -0.760356, -0.208916, -0.300466, 0.337010, -0.441857, 0.546041, -1.018939, -0.642002, 0.049254, -0.059878, -0.072977, -0.211270, 1.138324, -0.798492, 0.795921, 0.550479, -1.670245, 1.090695, -1.141291, 1.942743, -0.028447, -0.343769, 0.871180, 1.400805, 0.378905, -0.602504, -0.636636, -2.474584, -0.266485, 0.831512, 0.527313, 0.060950, 1.755770, -0.007467, 0.125839, 2.113635, -1.252190, 0.423205, 0.013780, -2.391425, 0.901807, -0.598709, -1.562973, -0.058511, 0.654762, -0.388860, -0.026776, 0.523766, 0.731367, -0.065945, 0.522979, -0.661768, 1.900738, -1.168155, 0.786972, -0.319682, -0.721865, -0.912914, -0.178677, 0.157958, 1.034268, 1.315052, 0.220669, -1.173299, 0.701835, -0.258997, -1.476696, 0.784460, 0.159072, 1.002691, -1.156975, 1.705695, -0.694848, 0.913650, 0.914368, -1.388694},{2.520878, 0.535220, 1.003840, -0.504260, 1.220241, 0.361209, 0.099362, 1.531907, -2.104307, 0.450322, -1.279269, -0.546633, -1.245585, 0.667949, -0.244882, -1.507592, -2.411878, -0.634992, -1.238845, -1.358125, -0.766207, -0.705958, -1.279124, 1.140043, 0.553761, 0.261167, 1.916143, -0.175824, -0.891532, -1.563191, -1.786962, -0.257553, -0.349126, -1.254146, 0.069838, -0.431891, 0.337324, 0.811271, 1.103684, -1.373936, -0.824271, -0.120267, 1.211528, -1.022991, 1.563304, -1.315463, -0.569915, -0.884933, -0.120252, -0.011897, -1.472224, -0.769268, -1.155361, -1.428545, -0.804041, -0.765337, -0.949694, -0.527607, 1.345087, -0.572316, -0.673837, 1.227356, 0.223254, -0.263947, -0.216489, 0.104304, -2.754493, -0.680995, 1.563322, 0.999684, -0.632548, 0.323436, -1.154525, 1.536924, 0.112353, 0.902727, 0.557227, 1.527824},{-1.325974, 0.399750, -0.850217, -0.041560, -0.159838, -1.755562, -1.842535, 0.283887, -0.808541, 0.086137, -1.319482, 0.341857, 1.385444, -1.148550, 0.998384, -0.321271, 1.165839, 0.409612, -0.055023, -0.764953, 0.755376, 1.473535, -1.388253, -0.910256, -0.530486, 0.395250, 0.077058, 0.691494, -0.174249, 0.713210, 1.331230, 2.332165, 0.158666, 2.147178, 0.783091, 1.225522, 0.975020, 1.507895, 1.408379, -0.069284, -1.434906, -0.451436, -0.766655, 0.114601, 0.727701, -1.592031, -0.791850, -0.404800, -1.093671, 0.204921, 0.661453, -0.138438, 1.216578, -0.300733, 0.276591, -1.664361, -0.375809, 0.377646, 1.636310, -0.979509, -0.167096, 0.346395, -1.314639, -0.978982, 0.740755, 0.787128, 0.810259, 0.313562, -1.074583, 0.955060, -0.149832, 0.440398, -0.940961, -0.380941, -1.071728, -0.953240, -0.360304, 1.333316},{1.523484, -0.607224, 1.493772, 1.173835, 1.274792, 0.210319, 0.128749, 1.301437, -0.122479, 0.491935, -0.758044, -0.904764, -1.523221, -1.925029, -0.070556, 0.285201, 0.681165, -1.222564, -0.527269, 1.373746, 0.034014, -0.918179, 0.089456, -1.377539, -2.150852, -0.224235, -1.451834, 1.875229, 0.589821, 1.045354, 0.252343, 0.064553, 1.382705, 0.402661, -1.654893, 0.732239, 0.641671, -1.421111, -1.203480, -1.025362, -1.015253, 0.544687, 0.463448, 1.351638, -1.131348, -2.551308, 0.984224, 1.489576, 1.295481, -0.169847, 1.273240, 0.640486, 1.421976, 2.026277, -1.242786, 1.060380, -0.310270, -0.481865, -0.330198, -0.100426, 0.254511, 1.243968, 1.154760, -1.290530, -0.379092, 1.113848, 0.440083, 0.523375, -0.766121, 0.829461, 1.390884, -0.381881, 0.439276, 1.124859, 0.208452, -0.921101, -0.528203, -1.041086},{0.282046, -2.283581, -1.134033, -1.611252, -0.275891, -1.102151, -1.051398, -1.778164, -1.264535, -1.010784, -0.286896, 0.320054, -0.678704, 0.065199, -0.871764, -0.540167, 1.041714, -0.827419, -0.714445, 0.380646, -0.124048, -1.644787, 0.017774, -0.048221, 0.725154, 0.333479, 0.030370, -0.796417, -1.198852, -0.099632, 0.598574, 0.251542, 0.082570, -2.946200, -0.222096, 0.042328, -0.286438, -0.663245, 0.202168, 0.425168, 1.505527, 2.595217, 0.320928, 0.079985, -2.379883, -1.347057, 1.131298, 0.696528, 0.276346, 0.980908, -1.347514, 0.031262, -0.847708, 0.960277, 1.471608, -0.411402, 1.100989, -1.749543, -1.732879, 0.284450, -1.431314, -0.970393, 0.816106, -1.412662, -1.737181, -1.148187, 0.678529, -0.089061, -0.045099, -1.337595, 0.175649, -0.258130, -1.363120, -0.988069, -1.710763, 0.616103, -0.017698, 1.246780},{0.380151, 0.494589, -0.606321, -1.028426, 0.417695, 2.243173, -0.578907, 1.720501, 0.519697, 0.066531, -1.640612, 1.196593, -0.819246, -1.128871, -0.798829, 0.177814, 0.230233, 0.870910, 0.604150, 0.416784, 1.293946, 0.328544, 0.235685, -0.491400, -0.115707, 0.075204, 0.591757, 0.326255, 0.497608, -0.601247, -0.403229, 0.641620, -0.908177, 0.094078, 0.507916, -0.766024, 0.905341, -0.334974, 0.053175, 0.557546, -0.848722, -0.511087, 1.009714, 0.617735, 1.326744, -0.928455, 0.607719, 2.404678, -1.794566, -1.184407, 1.754840, -1.761953, -0.073749, -1.500152, 1.362143, -0.352411, -0.231915, -1.026791, -0.830162, -0.028314, 1.541632, -2.052790, 0.706445, -0.188993, -0.938867, -0.111060, -0.144070, -2.453534, -0.135045, 0.077437, -1.324339, -0.808638, 1.756482, -0.440634, 0.775704, -0.133885, 0.895293, 0.385682},{2.096407, 1.958397, -0.820129, -0.230993, 1.745427, 1.090541, 0.706097, -0.138326, -0.661246, -0.871467, 0.366115, 0.865940, 0.657546, 0.298961, 0.534752, 0.386276, -2.111923, 0.351606, -1.223024, 1.945684, -0.667529, 0.179647, 1.369206, 0.384253, 0.814243, 0.092225, 0.300692, -0.539828, -1.245156, 0.294460, 1.831881, 0.070510, -0.198635, -0.830336, 0.557288, 1.200527, 2.037159, -0.938538, 1.663253, -1.435398, -0.689164, 0.297117, -0.089661, -0.115886, -1.305580, 0.208606, -0.774380, -0.690064, 0.088758, 1.571931, -1.213162, 0.348167, -0.225093, -0.293733, 0.812725, -2.294289, 0.670875, 0.501885, 0.147677, -0.042365, -1.036610, -0.201290, 0.692377, -0.484129, 2.235916, -0.477449, -1.823417, 1.368169, 0.459820, -1.719567, -0.197928, -0.030292, -0.311467, -0.144499, -0.294715, -1.029565, 0.624511, 0.076532},{1.425312, 0.434863, -0.398955, 1.146583, 0.971938, 0.956785, -0.025284, 0.535148, -0.218017, 0.619853, 0.170876, -1.378079, 0.529387, -1.943840, -0.481096, 0.354447, 1.413044, 0.619635, 0.024238, 0.398771, 0.973691, 0.957777, -2.063710, -0.778143, 0.602653, -0.904451, -1.355666, -0.937352, 2.086345, -0.346722, 0.708633, 0.222858, -1.872886, -1.635948, -0.808687, -1.747043, -0.808743, -0.092143, -1.320410, -0.510289, 0.101517, 0.556084, 1.038910, -0.760390, 1.816766, -0.581062, -0.865296, -0.043205, 1.376519, -0.209529, -0.973317, 0.300747, 0.135638, -0.345619, -1.190165, -0.263300, 0.289769, 0.081539, -0.518330, 0.063350, 0.749514, 1.049800, -0.037298, -0.566920, 0.246742, -0.336608, -2.528460, -2.763811, -0.198740, 0.715114, 0.351651, 0.033081, 1.261171, 1.407314, 0.636817, -0.732290, 0.026944, 2.091702},{-0.577238, 1.164167, 0.545183, 0.164109, -0.550491, 0.125265, -0.226644, 1.943037, 1.000189, -0.430795, -0.648073, -1.884672, -1.000121, 0.281061, 0.479293, -0.608507, 0.806734, -1.100148, -0.760655, -0.392706, -0.696703, -0.013253, -0.614092, -0.526655, 2.047317, 1.820538, 0.920015, -1.942137, -1.067256, -1.867941, 0.636952, 0.427016, 0.081659, 0.343681, -0.710811, 0.883972, -1.533709, -0.922880, 0.788482, -1.224396, -0.219863, -1.479182, 2.170790, -1.082790, -0.219885, -1.278584, 1.252030, -0.233759, -0.767984, -0.268884, -0.753287, -0.965346, -0.776901, -1.024451, -0.682155, -0.450371, 0.163391, 0.128407, 1.565773, 0.274258, 0.799986, -0.233678, -0.259561, 0.132728, 1.084893, -0.369790, 0.958615, -0.334081, 0.864842, 0.205272, 0.617169, 0.155221, 0.056312, -0.408871, -0.999911, -0.790685, -0.071584, 0.792576},{-1.305608, 0.431395, 0.476563, 0.274546, 0.698034, 0.175717, 0.384934, -0.028014, 0.067227, 0.637121, 1.256505, -1.038226, 0.599545, -0.497549, -0.151838, -0.729002, 0.090534, 0.073780, 0.841249, -0.303408, -0.584356, -1.364214, 0.040765, 1.014561, -0.681003, -1.560964, 0.134204, -0.517633, 1.737794, -1.532826, -0.524030, 1.595754, -0.116618, 1.018859, 1.725038, 1.100907, -0.702276, -0.518774, -0.467603, -1.300554, 1.093686, 1.087939, 0.910308, -1.364133, 0.017936, -0.427467, 0.169935, -0.178523, -1.132591, -0.813050, 0.906940, 2.345746, 0.668635, -0.203310, -0.839786, -0.501324, -0.170572, -0.712966, -1.016311, -0.363673, 0.082417, 1.122448, -0.446509, 0.580097, -1.043279, 1.637270, 0.893542, -0.899821, -1.073701, 0.410169, 0.368478, 0.189991, -1.830506, 0.039312, 0.914006, -0.239966, -1.155824, 0.381595},{-0.386745, 1.214124, -0.963409, -0.398880, -0.396894, -0.894607, -0.876910, 0.796745, 0.286783, -0.921190, -0.235200, -0.163748, -1.175803, -1.107750, 1.175879, 1.553739, 0.076828, -0.087295, 0.212783, 0.631106, -0.120713, 0.590865, -0.141687, -0.255747, -0.440068, 0.264805, -2.167416, 0.376707, -1.071701, -0.510714, -1.220291, 1.753301, -0.327935, -0.909391, 0.080173, -0.577831, -0.598931, -0.696940, 0.503254, 0.393515, -1.393905, 1.601018, 0.413037, -0.466557, -0.266949, -0.601665, -0.307444, -1.108831, 0.669210, -0.802684, 0.796795, 0.484713, -1.052480, 0.959920, -1.618810, 0.187763, 2.622133, -0.266489, 1.069102, 0.744376, 1.214148, 0.830865, -1.156020, 1.315240, 0.095699, 0.469188, 1.587229, -1.558984, 0.212551, -0.444506, 1.131477, -0.981449, -0.901744, -1.324767, -0.066651, 0.604607, 1.209358, 1.143187},{-1.186731, -1.311912, 0.068350, 0.012918, 0.513175, -1.546046, 0.683299, -0.409508, -0.977990, 0.525502, -0.709898, 0.114534, -1.900876, 0.125091, 1.006917, 1.046002, 0.335669, 0.221052, 0.326558, -0.058333, 0.055743, 0.772096, 0.071136, -2.047398, 0.234796, 0.541686, 1.693377, 0.413345, 0.963170, 0.105383, -0.198945, 1.161383, -0.217310, -0.954990, 0.074406, 0.242186, 1.390257, -0.338966, 1.393515, 0.235492, -0.666519, 0.187645, -0.433480, 0.143343, 1.098757, -1.259240, -0.004551, -0.329340, 0.885901, -0.056681, 0.025762, 1.050655, -0.688457, 0.795677, -1.075945, -0.057190, -0.647084, -1.392161, -0.719390, -0.768992, 0.031843, 0.605550, 0.706615, -0.014174, -2.075799, 0.342288, -0.003564, -0.203814, -0.713196, -0.025876, 0.308918, 1.499683, -1.390271, -0.253702, 1.448319, -0.570769, -0.585500, 0.278948},{-0.128292, -1.309400, -0.276417, 0.228980, 0.496767, 0.244766, -0.533882, -1.860340, -0.704743, -0.280689, -0.116517, 0.385111, 1.374728, -0.107222, 0.428923, 0.955192, -2.168245, 0.302623, -1.040769, 0.131924, 0.253847, 1.588868, -0.492487, 0.009124, 0.068001, -1.172638, 0.403730, -0.374539, -0.862367, -1.902054, 1.481372, -0.004897, 0.406210, 0.420048, 0.759679, 0.837995, -0.393940, 0.345111, -0.926060, 0.368234, 0.335126, -0.172513, -0.857670, 0.138330, 0.385801, 0.079891, -0.259928, 2.254206, -0.947458, -1.434404, 0.642048, -0.554603, 1.078030, 1.413273, -0.127106, 1.123714, 1.080874, -0.020266, -0.499448, 1.086565, -0.509196, 1.253767, -0.318193, 0.639572, 0.779291, -0.208701, 0.452628, -2.081087, -0.180984, 0.537734, 0.307313, -0.390885, -0.079297, -0.604095, -0.902826, 0.215318, 1.315860, 0.674265},{-0.419474, -0.579257, 1.014855, -0.103925, -0.007178, -0.435875, 0.390348, -1.684371, 1.017399, -0.627064, 0.192733, -0.657782, -0.269219, 0.819415, 0.612872, -1.234869, -0.378300, -1.489999, -0.571734, -0.166806, 0.512631, -0.080979, -0.238376, -0.249186, -0.559439, -0.871751, -0.500510, 1.709684, -0.952040, -0.473564, -0.442168, 0.663687, 0.099089, -1.340962, 0.532133, -1.029166, -0.804896, 1.286666, -0.681736, -0.796487, 0.520986, -1.016275, 0.946479, -0.682945, -0.008203, 0.089971, -1.377214, -1.794940, -0.070279, 0.404994, -0.551695, 0.987410, 0.807017, 2.197288, 1.285735, -0.715490, -0.586323, 0.439830, -1.839442, 0.492080, 0.264660, 0.054278, 0.604782, -1.080166, -0.549826, -1.132538, 0.828081, -1.268541, -0.435890, -0.446417, -0.409914, -0.136135, -0.615730, -0.005621, -0.037470, -0.744725, -0.638713, -1.250880},{-0.098162, 0.761477, -0.181064, 0.950413, -0.863762, -0.286125, 0.369345, -0.149427, 0.616394, -0.633480, 0.007179, -0.762024, -0.571267, 0.645574, -2.477023, -0.172023, -1.718534, -0.864611, -1.462096, -2.372500, -0.385592, 1.217517, -0.439157, 0.243919, -1.983772, 0.902440, 0.065033, -1.685876, 0.466925, 0.464589, -2.725246, 0.426520, 0.624360, 0.338605, -0.114273, -2.053921, 1.040047, -1.472638, -0.980322, 0.855763, -0.179460, 0.127638, -0.005628, 0.086473, -1.056970, 0.899642, 0.869034, 0.552067, -1.220157, 0.209207, -1.133847, -0.706069, -0.110085, 0.450475, 0.079362, 1.086457, -0.602068, 2.370323, -0.002005, -0.707763, 1.113942, 0.046220, -0.199958, -0.374065, -2.357807, -0.498485, 1.025617, -0.398776, 1.412479, 1.606532, -0.644896, -0.327193, 0.576297, 1.037969, -1.491839, -0.427725, -0.920228, -0.878845},{0.044684, -0.582825, -0.654048, -0.706250, 0.750433, 1.640166, 0.534996, -0.016989, 0.447085, 1.079871, -0.734837, -2.368840, 0.243901, -1.732252, -0.804936, -0.127720, 1.366784, 1.001171, -0.747598, -0.000759, -2.424003, -1.595330, -0.058610, 0.491638, 0.435961, 0.046580, 0.460498, -0.708184, 0.327393, -0.219375, 0.657360, -0.315274, 0.737086, -0.067548, 0.902341, 1.136062, -1.041398, 0.002183, -0.552836, 0.350408, -1.256652, 0.194816, -0.851908, 0.821130, 0.742906, 0.830577, 0.522340, -1.299958, -1.582620, -0.828188, 1.002233, 2.386738, -1.591205, -0.472220, 0.550546, 0.348509, -0.731612, -1.961067, -0.470642, -1.509093, -0.236217, -1.718704, -0.636234, 0.134188, 0.472806, 0.333368, 0.916411, -0.096863, 0.186140, 0.296380, -0.795815, 0.981078, 0.098745, 0.169062, 0.120592, 1.153181, 0.802772, 0.808345},{0.340327, -1.279283, 0.117571, -0.331905, -1.632531, 0.307584, -1.367175, -0.104294, -0.045772, -0.862811, -1.923137, 0.001594, 0.345741, -0.593251, -0.380779, -1.917129, -0.535843, 0.096301, -1.232171, -0.077980, -0.694656, 1.847721, -0.621563, -0.372460, 0.322824, 0.727139, 1.454268, 0.886454, 0.944077, 0.758150, -2.347679, -0.770419, 0.710766, -0.509858, 2.621052, -0.472893, 0.079164, 0.290725, 0.977469, -0.947204, -0.678111, -0.165116, 1.019735, -0.651977, -0.433212, 0.841864, 0.291391, 1.861847, -1.244790, -0.613998, -0.069369, 0.647688, 0.012761, 0.662384, 0.576824, 0.022259, 0.717462, 0.519436, -0.141513, 0.761995, 0.766489, -0.287506, -1.014020, -0.969731, 0.236367, 2.235616, 1.540829, -0.370504, -0.197713, -0.152563, -0.183391, -0.340875, 1.452207, 1.845019, -0.040706, 0.413494, 0.224346, 0.124249},{-1.005610, -2.442886, 0.874215, 1.008265, -0.548058, 1.209410, -2.596604, 0.910374, -1.308409, -0.116295, 0.179222, 0.497157, 0.105419, 0.583299, 1.506761, -0.777093, -0.502819, 1.053185, 1.522721, 1.026179, 0.626173, 1.271031, -0.973351, 0.104161, 0.436518, 0.984606, 0.341556, 0.044844, -1.087329, 2.499933, -1.464255, -0.079060, -0.365943, -0.998452, -0.156090, -0.264412, 1.470430, -0.963052, -0.052069, 2.010334, -0.146543, 0.133519, -1.430971, -0.830553, 0.019934, -0.555867, -1.389038, -1.316776, -2.047866, -0.789209, 0.968296, -1.153054, -0.124485, 0.940297, -0.712763, -1.342136, -2.196649, 0.761900, 0.074428, 1.387352, -1.142325, -2.194163, -1.565772, 1.755882, -0.239853, -0.171043, 0.853122, -0.120216, -1.426403, -2.021722, -0.775363, 1.594438, 0.094334, -0.630857, 0.113509, -1.484294, 0.033288, -0.037367},{0.204886, 0.325429, 1.598892, -2.007428, 0.611973, -1.291431, 1.694937, -0.685324, -0.075808, -2.017637, -0.291756, -1.005488, 1.757195, -0.808681, -0.649232, -0.551148, -0.981146, 0.446490, -0.572963, 1.149688, -0.441195, -0.263604, -1.078751, 0.916193, 0.119596, -0.487243, 0.501208, 0.937417, 1.629166, 1.142922, 0.673531, -0.034153, 0.552253, -0.582530, 0.016354, -0.426553, -0.788570, 0.162804, -0.431256, 0.941130, -0.171764, 0.084394, -0.916091, -0.352829, 1.036669, 0.015778, 2.650734, -0.965606, 0.084788, 0.889822, -0.817019, -0.469547, 0.918300, 0.891797, 0.007721, -0.416325, -0.625813, -0.884261, 0.247644, 1.535908, 0.021250, 2.251065, -0.801592, 0.920750, -2.027568, -0.748016, -0.148567, -0.511925, -0.230843, -2.104566, -0.628366, -0.945617, -0.362546, -1.429493, 0.390611, 0.547049, 1.399573, -0.564533},{-0.368415, -0.349069, 0.679393, 0.782858, -0.686477, 1.089352, -1.179296, 1.231188, -0.980717, 1.472162, -0.228209, 0.137965, 1.461963, 1.139805, 0.472148, 0.475222, -0.210565, 0.213634, -0.727977, -2.053351, -0.582857, -0.186980, 0.396976, 2.291984, 1.500947, 0.461697, -0.287511, 1.051118, -0.178403, -1.843989, 0.240483, 0.070850, -0.341735, 1.402731, -1.705735, -1.354497, -0.385993, -0.072556, 1.763957, 0.528903, 0.135647, -1.117999, 0.135909, 0.127311, 0.094868, -0.762767, 1.086972, 0.881467, 0.876423, -0.636654, 0.031393, 0.312307, -0.389116, 0.010202, -0.519041, -0.435055, 0.321801, -1.293632, 1.064465, -0.245668, 0.566602, 0.775607, -0.952201, -0.331226, -0.556328, -0.484331, -0.842955, -1.918112, -0.043392, 0.748377, -1.139522, 0.812373, -1.534990, -2.640642, -0.547660, -3.156720, -0.826724, 0.834900},{-0.481594, -0.361905, 1.981487, 1.218184, -0.671514, -0.635892, 0.619333, -0.767227, -1.391811, 0.894673, -0.700464, 0.352849, -1.258131, -0.441520, -0.830693, -1.374236, -0.735891, -2.089423, 0.293800, -0.203483, 0.540528, 0.954022, -1.140219, -0.021877, -1.537856, -0.812942, 1.628997, 0.992778, -0.322215, -1.010688, 0.586286, -0.842847, 1.703004, 1.849276, 0.581492, 1.504553, 0.482182, 0.418353, -0.461409, -1.062733, -0.747551, 0.035493, -2.401748, -0.170675, -0.526032, 0.784957, -0.782086, 0.501071, 0.293479, -1.680314, 0.274715, 0.577382, 0.017333, -1.299975, -0.518810, 0.831768, 1.288852, 1.129888, 0.410247, -1.500305, 0.226820, 0.415290, -1.244185, -0.201782, 0.632291, 0.125012, 0.726272, -0.531904, 1.096310, 0.427947, -1.886574, 0.191821, -0.132907, -0.189818, -0.329741, 0.889314, 0.778376, -0.358272},{-0.875500, 1.416254, -0.706915, -0.691414, -1.351899, -0.190591, 0.724699, 0.036642, 0.600133, 1.143296, 1.211847, -0.580047, -1.133314, 0.109347, 0.604701, -0.528399, 1.578761, 0.321278, 1.994808, 0.831520, 0.363586, 0.901071, -0.565798, -0.813419, -0.611281, 0.792619, -0.484032, 0.289863, 0.229942, 1.451224, 0.685730, -1.359628, 0.760467, 1.783077, -0.758956, 0.831537, 0.968383, 1.267358, 1.037091, -0.339108, 0.949203, 1.237409, 0.605823, 0.307138, 0.287073, -0.151152, -0.373799, -0.468809, 0.901185, -1.009112, -1.096781, -0.558325, -0.030351, -0.063650, 2.138910, 1.458667, -0.905215, -0.804748, 1.666364, 1.008077, 1.124359, 0.993285, -0.356401, 0.849234, -1.171948, 0.069585, -0.179542, 0.938892, 0.094391, 0.649531, -1.809807, -0.331591, -1.216747, -1.450170, 0.711232, -1.139811, -1.017134, -0.024542},{-0.646132, -1.674651, -0.239938, -1.606160, -0.932522, -0.451738, -1.623385, 1.392919, -0.552740, 1.124509, -0.545617, 0.345042, -0.207179, 0.610022, -0.114450, -0.672625, 0.255539, 0.942603, -0.047647, 0.256485, -0.792039, -0.366454, -0.061191, -1.180056, -0.398023, 0.232005, 1.311444, 0.265638, -1.281337, 0.483485, 0.211611, 1.234227, 1.355329, -0.266221, -0.672751, 0.846893, -0.382682, -0.654203, 0.011760, 0.819333, -0.174884, -0.523461, -2.103052, 0.762761, 0.639975, 0.489945, 1.051750, -0.607647, 1.075444, -1.625572, 0.990487, -0.963562, -0.143602, -0.396975, -1.848387, -0.116532, 0.391913, -1.180132, -0.608923, -1.338951, 0.729884, 0.689732, -0.205836, 0.316538, 1.401512, 0.769330, -2.507132, 1.425887, -0.753993, -0.080845, 0.446752, -1.046204, -0.314226, 0.058775, -1.837177, -1.078474, 0.207249, -1.211769},{-0.236674, -1.707046, -2.185675, 1.152598, -0.557833, -0.455272, -0.009482, 0.962148, 0.081616, 2.480195, 0.502723, -0.522669, 1.679844, 0.485547, -1.236756, -1.268141, -1.434743, 0.310367, 0.777527, 0.945192, 0.652887, -1.403967, 0.696172, 0.348004, 1.277745, 0.016274, 0.351279, -0.676760, 0.102473, 0.705239, -0.755804, 0.926050, 2.207441, 2.775708, 0.617048, -0.673026, 1.554884, -0.954530, -0.113272, 0.223169, -0.570194, -0.668781, 0.569512, -1.709971, -1.060882, 2.126548, 0.145099, -0.763202, 0.381131, -0.911253, 1.246427, 0.473425, -0.418531, -0.659807, -0.610934, 2.533973, -1.027692, 1.355276, 0.993309, 2.825449, -0.410100, -0.798221, -0.427705, -1.330972, -1.280312, -0.576897, -1.090793, 0.572898, -1.309421, 1.528000, -1.186684, -0.656920, -1.804790, 0.837902, 0.679753, 0.261974, -0.453087, -0.117971},{0.421184, 0.375948, 1.439272, -0.193351, -0.149395, 2.250803, 2.417077, 0.346954, 0.639236, 0.574430, -0.180507, -0.553457, 2.726011, 0.088063, -0.246305, -1.252605, 0.222115, -0.006076, 0.660644, -0.879851, -0.452460, 0.490133, 1.813597, 1.611867, -0.301838, -0.918194, 0.310076, -0.625587, -0.160395, 1.929396, 0.663424, -0.333392, -0.274568, 0.037479, -0.310272, 0.319722, 0.589666, 0.251518, -0.754009, 0.031629, -0.784382, 0.568156, 1.168890, -0.962361, -1.933602, 1.012404, 0.274022, 0.212610, -0.359488, -1.922177, 0.853056, -0.280755, 0.504090, 1.452411, -0.065582, 1.201599, -0.788539, 0.952458, -0.672253, 0.067686, 0.031413, -0.326319, 1.062265, 1.555147, -2.078307, 0.220669, -0.555522, -0.930985, 1.233588, -0.065896, 0.864174, -0.493506, -0.400797, 1.083652, 0.327393, -0.002207, -0.091614, -1.662782},{0.739834, -0.270729, -0.102641, 0.310466, -1.164144, -1.992346, -1.384306, 0.348536, 0.863443, 0.676114, 0.299978, -0.313173, -2.100760, -0.089342, 1.915533, 1.472720, -0.781373, 1.904747, -0.084535, 0.204354, -0.676083, -0.203384, -0.988577, 0.588907, 0.111934, -0.274261, 0.270020, -0.877919, 0.858855, 0.124133, 0.853754, -0.709221, -0.418091, 1.009629, 1.257829, -0.301196, 0.434793, -1.188231, -0.820692, 0.371098, -0.327677, -0.283344, -0.726283, -0.518236, 1.094624, 0.594330, -0.897984, -0.562453, -1.207859, 0.430303, 0.451491, 1.778489, -0.971620, 0.005487, -0.265741, 0.427190, 0.037937, 0.291332, -0.529480, -0.104567, -0.553350, -0.902089, -0.544527, -0.479086, -1.329252, 2.711963, 0.311729, -0.416134, -0.036198, 2.228915, 0.207330, -0.058702, 2.099646, -0.583870, 0.240412, -0.099505, -1.710260, -0.215065},{-0.152258, 1.653323, 0.357111, 0.921017, -1.086005, -0.572083, -2.003407, 1.629991, 0.292758, -0.324997, -0.746938, -0.508988, 1.205487, 0.353199, 1.981157, -0.236184, -2.091619, 1.100497, -0.386810, 0.610701, 1.563836, -0.087427, -0.906035, 0.430899, -0.581963, 0.253216, -0.288683, -1.619661, -2.215706, 2.411005, -0.627925, -0.334705, -1.561889, 1.940225, 1.139051, 0.327588, 1.010513, -0.155119, 0.379706, -0.102029, -0.477425, 1.577299, 0.964419, 0.552387, -0.081169, 0.524398, 0.283490, 0.586336, -2.159835, -0.985851, -0.006100, 2.120856, 1.196722, 0.169056, -1.650184, -0.946182, -1.443542, 0.584378, 0.798388, 1.178157, -0.466019, -0.234330, 0.636740, 0.183774, 0.292084, 1.775439, 1.769573, -0.854839, 0.417626, 2.688839, 0.516320, 0.831183, 0.147450, -0.051574, 0.694111, -1.200368, 0.857382, -1.221867},{-0.593032, -2.253483, 0.233831, 0.393730, -0.647378, -0.055802, 1.268092, -0.138996, -0.034549, -0.111557, 0.132773, 1.103550, -0.509310, 0.763098, -2.795621, -0.416403, 1.259521, 0.024633, -0.987256, -1.054689, -0.055935, -0.589093, 0.514972, 0.046826, 0.696392, 1.394488, -0.541836, -0.258922, -1.546136, -0.258582, 0.644662, -0.577737, -1.776953, -1.587446, -0.472252, 0.664629, -2.930879, 2.210657, -0.731413, 0.501474, 1.808874, -1.016298, -0.635430, -0.000919, 0.134039, -1.487953, -0.134786, 0.057754, 0.555985, -0.349795, 0.541897, -0.332608, -0.474222, -1.516529, 0.184807, -0.362240, -0.091147, 0.898100, -0.697450, 3.035655, -0.376182, 0.655790, -1.557424, -0.325143, 0.482545, 0.634647, 0.666945, 0.067349, 0.495604, 1.335096, 1.114000, 1.372863, -0.318342, 1.095794, 1.227163, 0.186672, -0.230721, -1.318282},{-0.093887, 0.710387, -1.401107, 1.128063, -1.934354, 0.397173, -1.347007, -0.816130, 0.453452, -0.202018, -0.825484, -0.261856, 0.579374, 0.618865, 0.940338, -1.942666, -0.300614, 0.230742, -2.433861, 0.249441, 0.682150, 1.876457, -0.052002, 0.357592, 0.544000, -0.702219, 0.640882, -0.828967, 0.662642, -1.228895, -0.759607, -0.689650, 0.637153, 0.542462, 1.178344, 0.164012, 1.637238, 1.129046, 0.992684, 0.136303, -0.796937, -0.865232, 0.040062, -0.085738, -1.117468, 0.178679, 0.639666, 1.491483, 0.613175, -0.973624, -1.532560, -0.402378, -2.482789, 0.589862, 0.130420, 1.217004, -0.902123, 1.467389, -1.141090, -0.964627, 1.286055, -1.094115, -0.297194, -0.455086, 1.061099, 1.277108, -2.328316, 1.293587, 0.389160, 1.650129, 1.376610, 0.169288, 0.622624, -0.324551, 0.094202, -0.173041, -2.318428, 0.408406},{1.585063, -0.419471, 0.016011, -0.251618, 0.258385, 0.628313, 2.651974, -2.064520, -1.049496, -1.439457, -0.153193, -0.269432, -1.847124, 0.300794, 1.152780, 0.326970, -1.983412, 0.402995, 0.309017, -2.314881, 1.666999, 0.533074, 1.679452, -1.216027, 0.396891, 0.531353, -0.151383, 0.823879, 0.044652, 0.145753, 0.632518, -0.419145, -0.660907, 0.126688, 1.924203, -0.398558, -0.173536, 0.386860, -0.434461, 2.095341, 0.036002, -0.494711, -0.275243, -1.317008, 0.280209, -0.688418, 0.461891, -0.024571, -0.569219, 0.639915, 0.851341, -0.010431, 1.627304, 2.682026, 0.165218, 0.706892, -1.009542, -0.363101, -0.888387, -0.304435, 1.517718, -1.592202, -1.676920, -0.123514, 1.640750, -0.683206, 0.757966, -1.326919, 0.890915, 0.502989, -0.423247, 0.686593, 1.089723, -0.121970, 0.134696, 0.862879, -1.610833, -1.389269},{-0.120869, -0.526160, 1.027659, -1.334404, 0.028734, 1.954725, -0.379152, -1.481410, -1.632510, -0.104726, 1.103490, 0.804313, -0.371162, -1.344227, 0.732845, -1.373841, -1.865587, -1.217289, -0.782935, 0.494442, -1.326250, -0.790596, -1.518371, 2.897505, -0.503861, -0.908469, -0.269523, 0.748077, 0.624774, 0.575155, -0.277832, 0.568891, 0.596784, -0.382855, -1.120858, -0.316741, -0.451611, 0.131733, -1.451014, -0.069008, -0.513883, 0.552502, 0.445080, -0.025600, 0.699868, 0.944447, -0.175666, -0.116955, -0.393448, 0.459212, -0.203866, 0.049490, 0.125371, -0.612725, 1.825940, -2.209436, -0.667208, -0.987833, 0.354382, 0.693688, 0.088008, -0.376491, 0.441098, 1.075551, -0.670058, 1.263828, -1.090309, 0.010184, -1.831451, 0.992019, 0.143284, 1.259254, -0.453980, 0.482261, -1.265953, 1.137993, -2.101264, -0.416421},{-1.044861, 1.911525, 1.069728, 1.966417, 0.719812, -0.056499, -0.179520, -1.434610, -0.273340, -0.565891, -0.907894, 0.328870, 0.181011, 0.119019, -0.861317, 1.029841, 0.591803, 0.716932, 1.724153, 0.440404, 0.597066, -1.304310, 0.933495, 0.085646, 0.850601, -0.412855, -1.262943, -0.498261, 1.084276, 0.418053, 0.622979, -2.747199, -0.234953, 0.826334, -2.442651, 0.530667, 0.131484, 1.758855, -0.789001, -1.153359, -0.944929, -1.554604, -1.506136, -1.280789, 1.329563, 0.046922, -0.097061, 1.175516, -0.214451, 1.820926, -1.777627, 1.087828, 0.875621, 0.038641, -0.386489, -1.827360, 0.384326, 1.354318, 0.902430, 0.192962, -2.223680, -0.437541, -1.118350, 0.069186, -0.887997, 0.333759, 0.757711, 0.239404, 2.488057, 0.362586, 0.007189, -0.540743, 0.776505, -0.920831, 0.604164, -0.301412, 0.631043, 0.170624},{0.300002, -0.060474, -0.660337, -1.106537, 0.162835, -1.038820, -2.109917, 1.474189, 1.571304, -1.042370, -0.249416, 0.190681, -0.042106, -0.045013, 0.644664, -0.633153, 0.907473, 0.575535, -1.377201, 0.907575, -0.639095, 1.604248, 0.143532, -2.436924, 0.587385, -0.748188, -0.518916, -0.911657, -0.355299, -0.521251, -1.230309, 0.750574, -1.314375, -0.636438, -0.142320, 0.006807, -0.800719, -0.417501, -1.555847, -0.217108, -0.835339, 1.412730, 0.471412, -0.116793, 0.496643, -0.714272, -0.791886, 0.763438, 0.995827, 0.975526, 1.443584, 0.026316, 0.303534, -0.013467, 0.802971, 0.584569, 0.756752, -1.712374, 0.666744, -1.571887, 1.862841, -1.314108, 1.597796, 0.734475, 1.583296, 0.543054, -1.045326, 0.369906, 1.227494, 0.445880, 0.616392, 1.463580, 0.092224, -0.382072, -0.401310, 1.600371, 0.249412, -0.287465},{-0.662011, -1.251250, -0.882686, -0.505147, 2.504160, 0.346579, -0.139931, 0.016296, 0.807657, -1.552866, 0.115054, -0.542840, -0.531967, 0.121119, -0.647431, 0.294277, 0.735782, 0.941841, -1.151544, 0.611309, -0.052069, 1.036617, 1.117520, -0.649075, -0.641347, -1.462550, 0.119024, 0.153969, -1.581369, 0.225479, 0.656384, 1.028607, -1.148129, 0.609343, 1.457023, -0.937829, 2.426085, -0.645419, 0.865335, -0.211250, -0.666048, 0.312745, -0.083380, -1.949223, -0.661412, 1.832569, 1.180919, 0.808356, 1.221003, -0.177429, -0.084697, 0.331290, -0.281799, -0.383458, 1.422276, 0.361486, -1.044929, 0.079467, 0.323018, 0.601722, 1.246948, 2.052089, 0.680226, -1.534274, 1.289803, -0.612613, -0.322159, -1.776782, 0.025202, 1.374074, -0.765590, 0.228706, 0.552839, -0.456667, -0.952449, -0.371682, -1.426657, 1.284910},{0.260848, 0.479436, 0.650928, -3.020978, 0.669387, 0.176691, 1.493545, 1.872175, -0.714489, -0.066701, -1.848734, -0.228375, 2.066911, -0.678854, 0.193916, 1.721207, -0.185293, 0.003089, -0.330370, 0.210276, 0.581938, 2.513882, -0.300659, -2.317664, 0.354307, -0.033972, 0.107366, 0.408122, 0.417814, -2.077396, 0.164240, 0.511118, -1.123909, 0.553909, 0.553762, 0.774036, -0.857053, 0.204264, -0.665643, -0.838847, 0.469405, -1.531534, 1.992595, -1.136477, 0.486851, 0.891392, -1.071994, 1.016976, 1.085947, -0.686137, 1.237291, -1.467482, -0.378191, 0.271730, -0.112208, 1.364713, 0.029460, -1.008264, 1.093290, -1.183429, -0.645578, -0.515233, -0.400758, -1.224655, -1.894917, 0.747104, 1.342097, 0.375239, -1.020844, -0.542610, -0.398671, 0.245194, 0.044171, -0.528183, 2.560792, -1.061185, -1.767038, -0.140343},{-0.536316, -0.987403, 0.051395, -1.452256, 0.475866, -0.384688, -0.491257, -0.489575, -1.674422, -0.353099, -1.873147, -3.482438, 0.636856, 0.562340, -0.093121, -0.435433, 0.864162, 0.420384, 1.238500, -0.730086, 0.859825, 0.332911, -0.153453, 0.028917, -2.083744, 1.384769, 0.611011, -0.928210, -1.034889, -0.074617, -0.044909, -0.075850, 0.040950, 0.569180, 2.170012, 0.246810, -0.411862, 1.695528, -1.037683, 0.820698, 0.065362, 0.202305, -0.065237, -0.455177, 0.195495, 1.581963, -1.116942, -0.085610, 0.192980, -0.634615, -0.855589, 0.107855, -0.062380, -1.116837, 0.074008, -0.382157, -1.155363, -0.908115, 0.812390, 0.383643, 1.397659, -0.143366, -3.036449, -1.076202, 1.737879, -0.917680, -0.303503, 2.220139, -0.616231, 0.351625, 0.084903, -0.183455, 2.010216, -0.491729, -0.395111, 0.703115, 0.110363, -0.389992},{-0.939755, 1.949498, 0.004475, 0.249232, -0.263232, 1.183734, -0.410095, 0.618884, 1.088069, 1.258694, 0.085164, -0.562631, 0.918280, -0.748667, 0.172420, -0.783860, 0.132944, -0.029722, 0.270819, 0.718840, 1.399818, 1.504071, 0.159123, -0.838082, -0.359574, 1.336964, 0.407736, 1.600514, -0.798299, -0.778013, 0.418187, -1.888248, -0.687576, -0.651461, 0.315024, -0.878159, -0.403199, 0.751432, -0.030914, -1.305405, -1.442350, -0.412520, -0.333033, -0.351639, -0.501671, -1.014252, -1.050069, 0.373656, -0.959380, 0.570407, -0.449778, -0.157484, -0.128337, 0.734488, 0.602875, -0.183516, 0.252233, -0.040854, 0.469283, -1.640561, 0.605160, -0.752678, 0.120918, -0.752421, -1.643289, -0.060647, 0.821594, -0.410750, -1.547967, -0.131991, -1.161148, 0.195060, 0.285312, -0.195990, 0.303784, -0.260916, 0.134152, -0.289719},{1.957726, -0.588979, -0.620800, 1.492139, 0.080054, -1.470324, -0.053115, -0.384072, 0.924424, 0.170868, -0.969128, 0.344417, -0.008743, -0.113736, -0.698168, 0.195210, 0.087305, -1.069798, 1.199332, -1.479885, -0.467153, -0.520771, -0.417181, 0.290855, 0.192804, -0.957727, -1.957789, 0.121784, 0.399774, -0.416335, -0.726888, -0.815609, -0.003225, -0.361085, 0.212077, 2.004551, -1.884334, -0.100017, -1.621248, 0.776926, 0.545796, 0.525560, -0.302037, 1.206803, -1.245415, 0.826703, -0.597176, 0.326766, 0.696804, -0.417192, -0.119974, 0.873916, 1.207230, 3.083441, -0.786414, 0.427262, -0.152351, -0.424442, 0.656575, 0.973160, 1.327883, 0.930418, -1.465443, -0.346457, -1.321076, 0.032443, -0.323750, -0.134479, 1.456260, 0.294190, -1.464769, 0.118093, 0.239597, -0.085013, 0.488472, 0.421799, 0.745837, -0.881145},{0.526947, -0.301279, -0.397410, 0.035106, 0.738192, 0.201529, 0.622240, -1.027313, 0.909617, 1.691209, -0.907707, -1.056485, 0.239497, 0.379578, -0.317067, 1.101456, 0.075347, 0.534915, -0.863858, -0.006373, -0.266732, -0.830757, 0.141136, -0.686770, -2.207952, 0.637512, 0.142230, -0.885774, -1.459563, -1.526802, -0.100511, 0.829082, 3.999481, -2.182303, 0.553085, 2.084558, 2.214164, -0.195564, -1.276007, 0.462373, 0.856866, -0.585314, -1.189274, -0.645826, -3.024709, 0.526833, 1.287279, -0.509291, 1.348547, -1.423238, -0.628350, -0.055669, -0.206292, 1.098052, -0.537280, -1.361415, -1.250943, -0.997858, 0.819674, -2.012961, 0.535663, 0.812715, -2.034617, 0.014269, -1.148546, 0.861980, -0.324938, 0.823016, 2.173394, 0.392832, 0.831378, -0.405058, -1.047464, 0.765220, 0.479434, -1.607054, 0.546600, 0.019583},{1.856846, 0.134635, -0.694909, 1.121089, 0.981924, 0.074629, -0.141626, -0.318251, 0.071429, 0.484209, 0.132001, 0.887268, -0.224035, -1.599453, -0.512236, -0.496508, -0.451558, -1.004535, 1.555254, 0.429389, -1.343882, -0.043358, 0.525040, -0.289093, -0.494856, -0.303421, -0.552934, 0.083428, 0.471184, -0.548483, -0.353955, 0.415989, -0.146586, -0.284223, 0.797719, 0.105000, 2.318956, -0.690031, 0.727198, 0.931112, -0.199020, -2.360921, -1.528713, -1.197876, 0.147875, -1.159282, -0.285628, -0.069992, 0.148503, 1.029812, 2.602588, -0.010663, 0.904186, -0.223513, -0.787178, 0.936638, 0.537839, 1.911526, 1.604979, 0.957555, -1.101282, 1.898310, -0.615045, 0.113556, 0.464879, 0.372857, -1.344826, -0.883746, -1.340546, -1.474653, 0.375008, -0.355323, -0.499893, -0.142656, -0.640877, 0.902107, -0.105816, -0.648799},{0.766151, 0.730295, 1.924028, -0.322432, 2.318861, 0.792188, 0.226318, -1.121889, -1.182904, 1.484005, 0.103051, -0.092289, 0.484238, -1.035555, 0.701254, -0.283115, 0.563789, 1.242930, -0.593679, 0.667933, 1.146079, 0.177577, -1.263541, -0.524572, 1.027259, -1.316812, 0.049144, -0.291516, 0.736520, 0.463867, 0.102613, -1.134512, 0.998251, 0.078579, -0.168369, 1.705540, 0.495443, -0.354277, 0.493860, 0.433503, -0.603559, -1.427936, -0.421430, -0.271589, 1.210267, -0.680127, 1.833657, -0.303212, 0.925875, -0.009668, 0.244513, 0.399041, 0.707385, 0.508661, -1.877501, -1.075599, -0.949589, -0.173875, 1.124160, 0.670190, -0.051774, 0.612342, -0.684671, 3.151435, 0.420147, -0.976325, 0.230265, 0.680738, 0.198642, -0.749303, 0.779587, -0.507461, 0.638212, 1.700646, -0.282113, -0.647712, 0.424146, 2.040997},{0.102942, 0.517051, -1.479570, -0.291886, -0.751346, 1.018423, -0.826295, 0.179475, 0.337108, 0.146847, 0.197356, 1.000573, -1.088748, -0.354192, -1.026543, 0.102897, -0.779533, 0.181070, -0.759616, 1.092161, 1.436819, 0.403884, 0.283918, 1.027682, 0.572167, -0.760188, -0.784794, 1.117175, -0.182519, -0.923753, 0.056745, 2.376016, -0.704974, 1.608658, 1.917438, -0.075869, 0.289219, -0.255563, 0.340480, 0.025505, 0.465405, -1.273479, -0.179890, 0.660742, 0.692112, 0.200599, -0.722124, 1.898853, -1.144482, -0.284434, -1.018514, -0.131734, -0.179151, 0.071415, -0.079319, 0.481937, 1.633389, 0.861939, 0.070647, -1.120586, 1.966398, -1.833424, 0.805761, 1.143010, -0.286580, 0.351318, -0.715379, 0.283722, 0.469378, 1.738452, -0.645458, -0.947861, -0.680968, -0.710440, 1.443993, -1.287045, 1.237553, -0.251786},{0.097698, -0.148306, -0.258927, -0.359101, 0.583008, -1.742582, 0.083191, 1.917065, 1.837889, -0.057945, -0.686834, 0.540577, 0.340917, -0.103728, 1.863049, 0.783743, 1.463011, -0.644930, 0.318131, -1.826504, 0.023353, 0.950030, -1.004407, 0.005554, -0.882225, -0.022531, -0.357495, 0.826526, -0.576450, -0.013762, -0.097918, 0.054758, 1.365595, 1.048619, -0.368312, -0.232648, 0.696073, -1.530676, 1.574176, 1.568569, -0.431807, -1.306850, -2.561960, -0.033138, 0.919445, -0.531145, -2.357995, 0.550647, 2.242666, -0.830212, 0.390120, 0.105580, -1.260367, -0.111752, 0.668607, -0.349015, 0.280271, 0.210987, 0.994634, -1.114493, -0.345966, 1.346915, -0.251347, 1.845404, -0.344887, 1.407532, -2.586959, 0.271371, -1.216424, 0.529519, -0.571881, -0.565880, 0.060559, -0.209297, -3.057953, -0.621730, 1.533168, 0.895896},{-1.239868, -0.796296, 0.087918, -0.897948, 0.136186, 0.663392, -0.106500, 1.482025, -0.815139, -0.451493, -1.439552, 1.090837, 0.832601, -0.611841, 1.625793, -0.350675, 1.825931, -0.536226, 1.345261, 0.508901, 0.166573, -0.372211, -0.133345, -0.708429, -1.613943, 0.596475, 0.243839, -0.538317, 0.258531, -0.670014, 0.556542, -1.796544, 0.713065, -1.340644, -0.658394, -0.263499, -0.897000, -0.247690, -0.545829, 0.394454, 1.287893, -2.409043, -0.126737, -1.048696, -1.333167, 0.062925, -1.076092, 1.307827, -0.744327, -1.181316, 0.848372, -0.308444, 1.105295, 0.698379, 0.999532, -1.536998, 0.147631, -1.218628, -0.168968, 0.821768, -0.326378, -0.025078, 1.212529, -1.350640, -0.204604, -0.000942, -0.155867, 1.489402, 1.174834, 0.355192, -0.838671, -0.490084, 0.179512, 0.400637, -0.348930, 1.256075, -1.006007, -0.098444},{2.394793, 0.623828, 1.016197, -1.360180, 1.811148, -1.558025, 0.387708, -1.007494, 1.045779, -0.234867, 0.865148, 0.183681, 0.241225, 0.427541, 1.145053, -1.434154, 0.097062, -0.462069, -1.520201, 1.900607, 0.689763, -0.286783, -1.653557, -0.848218, -2.150993, 0.724493, -0.090971, 1.305810, 0.283696, 0.981299, 1.095195, -0.021879, -0.040896, 3.744075, 1.243461, -0.226040, -0.256403, -0.234269, 0.162227, 1.284814, -0.155023, -0.562337, -0.948121, -0.509138, -0.045662, -0.341452, -1.443802, 1.626298, -0.613278, -0.112878, -1.208623, 1.448665, -0.172487, 0.128997, -1.181584, -1.319798, 1.096720, -1.392436, -0.432175, -0.011830, 1.079872, 0.894498, 1.446022, -0.968000, -0.273638, -0.915241, 0.670839, -0.385509, 1.013542, 1.020950, -0.840212, 1.862372, -0.309676, 0.142339, 1.500045, -0.392524, -1.370772, -0.756755},{0.800649, 0.189800, 0.644053, -0.186595, -2.841779, 0.031177, 0.610708, 0.593793, -1.092007, 0.465292, 0.657054, -0.333134, -0.348268, 3.375579, -0.156525, -0.938352, 1.203655, -1.578921, -1.395460, -1.477253, 0.149525, 0.304548, -1.089615, -0.239873, -1.334908, -1.240072, 0.744927, -0.908900, -2.761402, -0.714644, -1.140126, 0.838266, -0.444782, -0.899815, 1.378381, -1.205058, -0.663936, -0.458215, -0.388985, 2.227699, 0.282162, -0.564602, -1.051717, -1.164201, 1.193256, 0.194475, -0.148279, 0.206141, -0.865543, -0.210687, -0.257081, -1.371943, 0.621278, 1.185972, -0.160372, -0.382752, -2.304232, 0.836810, -0.649530, 1.576208, -0.502505, -1.045215, 0.875930, -0.047752, -0.020163, -0.083223, 0.279660, 0.625242, 0.320500, -0.130535, -0.907653, 0.136735, 1.467152, 2.235112, -1.542950, 1.476000, 0.992472, -0.691695},{0.176848, -1.860670, 0.588794, 0.520599, -1.364253, 0.184696, -0.517688, 0.699735, -1.036943, 1.604269, 0.863672, -0.347595, 0.233725, -1.557253, -0.517708, 0.828630, -0.774732, -0.097376, 0.279228, -0.305566, 0.131781, -0.248675, -0.667535, -0.132477, 0.513397, 0.154967, -0.505300, -0.560045, 1.059127, 0.841783, -0.234868, 0.414994, 1.660614, -1.448225, -1.507527, 0.668665, -2.009484, -0.464798, 0.647097, 0.594611, 0.838469, 0.707948, -0.154771, -1.923087, -0.533673, -0.375337, -0.123303, 0.819208, 0.431033, -1.316413, -0.321335, 1.004297, 1.523037, 0.666233, -0.344854, 1.935690, -0.019740, -1.653644, -0.317445, 0.296337, -0.745359, 1.412154, 0.500605, -0.111885, -0.951733, -0.186791, -0.053986, -1.711587, -1.601681, 1.864381, -0.546382, 1.089161, 1.009317, 0.363147, 2.822274, 0.300991, 0.630114, -0.151684},{0.148595, 0.169423, 1.387087, 1.164704, -0.305720, -2.173385, 0.465497, 1.672018, 0.008249, -1.100317, -1.246081, -0.248055, -0.166964, -0.922142, 1.166449, -0.304228, 0.710869, -1.159697, 0.394798, -0.133262, -0.551316, -0.301962, -0.749806, 0.002665, 0.223652, -0.485480, -0.430520, -2.087294, 0.882548, -1.508561, 1.210271, 0.843034, 0.037630, 1.038751, 0.941628, -1.888327, 0.383514, -0.214756, 0.232660, 0.341864, 1.334708, -0.393398, -1.734969, 0.330954, 0.989405, -0.086456, 0.596199, -0.643719, -0.945073, -0.759581, -1.504975, -0.034159, 0.138605, -1.165720, -0.138017, -0.881410, -0.167149, 1.228546, -1.777002, -0.592603, 0.271608, 1.498430, -1.613666, -0.457157, -1.864805, 0.598951, -0.960976, 0.287890, 0.597716, 1.227609, -1.620865, -0.253790, -1.589791, 0.015956, 0.883451, -0.001538, 0.985970, 0.158451},{-0.542410, 0.312036, 0.455649, -1.434389, -0.294019, 0.289563, 0.981165, -0.038512, -0.171557, -0.585219, -0.173835, -0.721939, 0.732917, 0.110112, 1.416507, 0.593849, -0.127755, -0.182679, 0.028357, 0.229998, -1.561339, -2.004166, 1.349840, 1.254300, -1.073650, 1.067516, -0.357969, -0.609767, 0.152451, 2.574293, 0.106259, -1.831864, -1.133011, 0.978242, -0.216116, 1.252169, 0.861408, 0.974128, 0.866491, 1.601343, 0.595666, 0.951741, 1.385639, -0.406768, -0.174305, 0.170070, 0.694078, 0.876826, 0.139263, -1.682142, 0.587987, -1.232227, -1.003015, -0.154173, -1.352476, 0.025567, 0.046709, 1.496391, 1.202071, 0.866655, -2.155662, 0.024597, -0.030142, -0.867713, -0.292583, -0.439316, -1.212762, 0.307897, 1.131748, -1.161284, -0.262051, 1.272672, 0.468389, -1.040975, 0.182139, 0.108263, 0.441762, 0.083225},{0.842316, -0.803920, 1.068600, 2.087739, -0.041537, -0.081581, 0.893112, 0.087213, -0.654528, 1.203444, 0.333448, -0.793611, 0.155505, 1.168995, 0.035204, -0.430504, -0.569021, 0.449066, -0.469262, -0.143324, 0.154363, 0.609212, -1.446775, -0.349528, -2.885036, 1.192562, -1.614359, -0.323557, 0.458911, -1.352172, -0.470897, -0.825302, 0.200148, -0.825993, 0.277476, 0.698201, -1.396866, -0.690624, 2.043650, -0.022941, -0.690968, -0.530967, -0.828760, -0.334149, 1.504910, 0.561676, -0.984908, -0.998494, 0.661315, -1.134759, -0.244072, -0.322668, 2.182607, 0.383694, -0.466908, 0.137040, -0.685182, -0.873191, -0.635233, 0.491038, -0.024070, 0.513659, 0.287048, 0.845669, -0.813074, -0.085434, -0.711899, -0.300191, -0.146196, -0.970633, 0.834702, 1.602835, 0.727199, 1.497163, 0.975998, -0.118398, 0.977649, -0.576088},{-0.700100, 0.176233, 0.397948, 0.648545, 1.129990, 0.110887, 0.843520, -0.886303, 1.440859, -0.064221, 0.297203, 1.634365, 1.087516, 1.411476, -0.125111, -0.333417, -0.392472, 1.165362, -0.332223, -0.024517, -0.023061, -0.211476, -0.779174, 0.415395, 0.860274, 0.195994, 0.739929, -0.510522, -0.451181, 0.257610, -0.127337, -1.123853, 0.742925, 1.199659, 1.070844, -0.941496, -1.738278, -1.713675, -0.013582, 1.261743, 0.686471, 1.415818, -1.692642, -1.857820, 0.215706, 0.412131, -0.198831, -0.689364, 2.015327, -1.288915, -0.978172, 0.917301, 0.138170, -0.657632, -1.677765, 0.025756, 0.727254, -0.524041, -1.665395, 0.244495, 0.257778, -1.529928, 0.110675, 0.191136, 1.544888, 0.503214, 0.523705, 0.488908, 0.289490, -1.104993, -1.022054, 0.778520, -0.926316, 0.929292, -0.312546, -1.694188, -0.199801, 0.050573},{1.727309, 0.323208, 0.229736, -0.077660, -0.880982, 0.082965, -0.062413, 0.479017, -0.735554, -0.719550, -1.017878, 1.581716, 1.532733, 2.388168, 0.933653, -0.359949, 0.141139, 0.524546, 0.131201, 1.985599, -0.901042, -0.659825, 0.640588, 1.489978, 1.450361, -0.383106, -1.261415, 0.177974, -0.890007, -0.019736, -0.533885, 0.499761, 0.052371, -0.591382, -1.184485, -0.871323, 0.146162, 0.931696, -0.226734, -1.640623, -0.232176, -0.146711, -0.851005, -1.030597, -0.589688, -0.499998, 0.351859, -0.193027, -0.801227, 3.326910, 1.510455, 1.189682, 0.237714, 0.946687, 2.046628, 0.549431, -1.119244, 0.989130, 0.077503, 1.724069, -1.118495, 0.749268, -1.455534, 0.654514, 0.523680, -0.348875, 0.226088, 0.370626, -1.013163, -0.292554, -0.786540, -1.203901, 0.789487, 0.518652, -1.370637, -1.488916, 0.400481, -0.719176},{-0.062824, -1.089225, -0.013565, 0.469316, 0.845391, -0.128078, 1.193677, -0.399231, -1.205232, -0.011724, 1.118942, -1.831947, -0.134483, -0.299653, -0.781013, 0.865286, 0.455671, -0.771904, 0.722524, 0.625889, 0.966746, -0.680334, -1.039725, 0.887416, -0.127046, -0.461729, -0.325240, -0.916134, 0.120725, 1.281707, 1.401229, -0.018462, -0.758194, 0.801884, 0.768344, 0.215229, 0.785162, 0.363988, 0.062831, 0.726579, -0.235361, -0.809450, -0.171582, 1.132676, 0.077636, 0.487996, 1.605120, -1.281898, 0.350676, -1.433718, -0.552537, 0.750686, 1.007854, 0.608877, 0.339499, -1.053419, -1.126899, -0.162064, 0.702372, 1.013570, -1.237070, -0.781945, -0.273884, 1.393438, -0.054252, 0.572451, 0.451719, 0.789493, 0.663326, -3.314268, -1.012780, -1.120762, -0.529701, 1.639518, -0.029966, 0.799259, -0.681879, -0.087391},{-0.973342, -0.077967, -0.238301, 0.911127, -0.283887, 0.704598, -0.304211, -1.517624, -0.059702, 0.332969, -0.621319, -0.042782, 1.070269, 1.213702, 0.482204, -1.384753, -0.140320, 0.281981, 1.064694, -1.094205, -0.123914, 1.730955, -1.311665, -1.481394, -1.014298, -0.319828, -0.151471, -1.656708, 0.902742, 0.713246, 0.788542, -1.094325, -0.129174, 0.067466, -0.188762, 0.596471, -1.811206, 1.502459, -0.950592, 0.180980, -0.834849, 1.427647, -0.218321, 0.096936, 0.399586, 1.219222, 0.144344, -2.607422, 1.606575, -1.487031, 1.529447, 1.237949, -0.442580, 0.019478, 1.378039, 0.355003, 0.855029, -0.155066, -0.901961, 1.257761, 0.044065, -1.676158, -0.937940, 0.817568, 1.383324, 0.330869, 1.217398, -0.361614, 1.316655, 0.756613, 1.654138, 1.103885, 0.251014, -0.344074, 0.448583, -1.006410, -0.171473, -0.375580},{-1.651065, -0.454532, -0.713247, 0.178382, -1.241952, -1.214686, 1.572447, 0.956172, -0.357292, 1.457577, -0.277273, -1.559535, -0.349665, -0.183771, 0.239724, 0.706131, -0.387295, -1.642244, -0.741710, 0.240870, 1.644399, -2.600956, 1.318758, 0.787986, -1.304872, -0.598121, 0.821620, 0.830197, 0.307600, 0.735787, 0.916759, 1.308401, -0.575672, 0.110928, 0.985797, 0.279276, -0.103821, 1.716697, -1.279508, 0.462992, 0.942012, -0.979537, 0.768766, -0.186201, -0.102910, -1.167282, 0.834757, -0.433917, 0.723078, -0.441334, -0.957893, -0.755791, 0.238614, 0.757032, 1.572678, -0.496824, 0.421437, -0.691445, 1.560370, -0.651515, -0.150485, 1.196836, -0.491295, -1.708685, -1.052686, 0.010290, -0.678541, -0.216498, -1.096399, -0.603742, 0.703100, 0.991080, -2.067131, -1.126316, 0.585117, -1.185806, -0.603757, 0.739666},{-0.149677, -0.111388, 0.422489, -0.921086, 0.214382, 0.467037, 1.152058, -1.341312, 0.366385, 0.286092, -0.415765, 1.887529, 0.283547, -1.392830, -0.240326, 0.412371, -0.333746, 1.609218, 0.018874, -0.000261, 0.546050, 0.395687, -0.550188, 0.521437, -0.550474, 0.308300, 2.913857, -1.034457, -1.291633, 2.219368, -0.301758, 0.706570, -0.868657, -0.528685, 0.644989, -0.306996, 0.410541, -0.138139, 0.234958, 0.252781, 0.666055, 0.370346, 0.706267, 0.253588, 1.492522, -0.254122, -1.673681, -0.282763, -0.219579, -0.323134, 0.650136, 0.245836, -1.567885, 0.021785, -0.029586, 0.309630, 0.695847, 0.239887, -0.268411, 0.909369, -0.215154, 0.284182, 2.770774, 1.006941, -0.348890, 1.428128, 0.795622, -0.100063, 0.519598, -1.926958, -0.629066, 1.478527, 0.627040, 1.451572, -0.024015, 2.206476, 0.076215, -0.085710},{-2.288803, -0.705731, 0.509720, 1.987710, 0.205499, -1.459713, 0.178796, 0.221214, -1.354073, 0.851948, 1.051101, -1.307235, -2.709913, -3.134519, -1.473446, 1.085865, 0.791947, 0.006695, 0.234715, -0.513411, -0.375710, 0.280850, -0.059232, 0.079978, 0.928780, -1.752663, -0.818690, -1.239431, -0.559694, 0.218781, 0.030605, 0.009159, 0.006255, 0.100105, -0.987369, 0.499469, -0.836115, 0.352717, 0.537679, -0.499245, -0.007027, 1.055840, -0.559465, -0.110622, 1.392914, 0.142966, 1.414659, -0.838746, 0.521744, -1.033102, 0.210690, 0.043249, -1.663610, 2.340324, 1.102092, 1.926769, -0.260588, -1.040407, 2.226459, -1.178994, 0.099123, -1.068405, 0.267609, -1.070608, 0.129060, -0.275431, 0.689270, 0.523975, -1.610078, -1.123436, 0.248733, 2.355222, -0.699567, 1.134912, -0.064031, -0.564052, -0.152053, 0.350327},{0.568522, -0.531482, -0.723825, 0.388651, 1.025146, 1.757296, 0.331688, 0.712342, -1.274969, -0.893294, -1.253167, 1.003944, 1.243302, 0.704398, -0.652738, -1.138299, -0.268040, -0.833735, 0.324555, -0.663875, -2.158798, -0.701935, -0.002097, 0.198190, 0.398877, 1.799199, -0.888006, 1.625681, 0.336840, -0.924724, -1.566627, 0.123862, 0.500233, -0.826716, 0.841712, 1.786154, -0.783867, 0.827291, 0.733179, 0.138650, -2.087399, 0.805594, 0.617409, -1.747926, -0.033988, 0.620065, -1.239854, -0.334901, 0.024008, -0.787606, 1.074059, -0.613778, 1.231430, -0.740170, 0.906750, -1.084684, 1.239292, 1.867168, 1.420117, 1.003675, -0.437400, 0.152488, 0.301851, 0.756101, -0.068485, -0.142961, 1.491892, -1.408569, 0.819387, 1.643577, 1.075723, -1.607056, -0.923784, -0.108724, 0.642704, 0.455016, -1.395501, -0.962620},{-0.527883, -0.256857, 0.347922, 0.350536, 0.301017, 0.507440, 0.115428, 1.805844, 1.835971, 0.143107, 0.121254, -0.019376, 0.883018, 0.180084, -0.838277, 0.320220, 2.035985, -1.300526, 0.612995, -0.274410, 0.947719, 0.184391, 1.132010, 0.861451, -2.240703, -2.570462, 1.116985, -1.324823, 0.789555, -1.216986, -0.546844, -0.217510, 1.493250, -0.391072, 0.368428, 0.731786, -0.057440, -0.975866, -0.694077, -0.188874, -2.560496, -1.131425, -1.660657, -1.238886, 0.324198, 0.914170, 0.356862, 0.210505, -1.010445, -0.304607, 1.247187, 0.888201, 0.780743, -0.978093, 0.368263, 1.028258, -1.221155, -1.229082, -1.142297, -0.558293, 0.263171, -1.963603, 0.493139, 1.269294, 0.776401, 0.830775, 0.279478, -2.109570, 0.199264, 0.729017, -1.222934, -0.987887, -0.773792, 0.376637, -1.436401, -0.023355, -1.235580, -0.163174},{-0.465367, 1.198852, 1.180280, 1.161909, 0.903462, 1.193221, 0.433000, 1.152374, 0.348401, -0.154652, 0.425415, 1.082681, -0.223135, 0.169179, -0.172403, 0.441272, -0.293090, 1.240516, -1.510859, 0.205634, -0.405267, 0.775630, 0.456714, -0.454706, -0.830506, 0.106833, -0.150838, 0.279919, 0.244321, 0.088378, 0.945764, 2.334094, 0.269136, 0.500370, -0.331326, 1.130670, -1.607980, 0.977532, 1.670207, -0.799574, 0.754865, -0.924579, 0.194465, -0.281512, 2.714792, 0.498431, -0.132045, 0.312172, -0.392337, 0.343301, -1.034527, -0.456689, -0.582263, -0.302178, 0.178639, 0.093466, -0.995019, -0.347192, 0.644822, 0.198102, 1.590964, 0.757497, -0.370889, -0.296025, 0.973225, 0.025606, -0.692330, 1.302407, 0.548328, 0.503915, -0.691555, 0.011372, -0.293569, 0.810947, -0.215866, 0.254749, -0.110504, 0.290306},{0.448711, -1.669259, 1.176501, -2.059034, -0.213882, -0.135653, 1.586059, 1.337308, -1.598445, 1.009467, 1.742380, 1.011587, -1.232132, -0.226277, -1.160830, -0.338293, 1.448476, -0.419428, 0.336393, 0.612624, 0.631907, 1.348373, -1.449559, 0.525818, -1.792697, 0.639405, -0.848258, 0.518807, -0.457449, -0.691029, -1.250229, -0.371239, 1.495290, -0.579539, 0.025536, -0.078851, -1.798116, -1.721966, -0.792708, -0.664140, -0.182608, -0.562602, -1.397810, -1.268811, 1.049133, -0.535307, -0.550376, -0.643512, 0.787736, -1.489386, 0.266503, -0.402935, 0.698813, 1.088740, -1.473201, -0.137272, -0.334119, -0.761101, 0.478927, 0.731422, -2.000814, 2.321947, 2.080563, -0.144458, -0.238605, 1.031842, -0.345056, 0.083321, -0.110038, 0.656339, 0.053614, 0.726381, 0.077141, 0.362409, 0.163068, -1.767732, 0.093152, 1.027215},{1.834450, -0.062197, 1.130369, 0.006047, 0.371889, -1.595883, -1.526920, -0.433817, -1.031987, -1.181690, -1.021939, -0.894842, -0.498799, 0.027069, 0.461960, -0.653362, 0.872614, 0.308503, -1.764669, -1.262537, -0.148315, 1.050154, -0.047434, 0.100970, 0.126337, 0.466571, 1.040278, -1.194612, 0.984152, -1.060875, 0.093162, -0.062325, 0.221891, -1.811898, 2.095982, 0.485084, 0.095578, 1.311746, -0.347672, 0.028484, -0.351844, 0.290158, 1.016543, 1.461724, 0.203497, -2.319538, -0.278216, 1.691686, -0.709975, -0.463115, -2.031015, -0.040573, 0.574463, 0.952281, 0.116559, 0.706189, -0.904950, -0.532350, -0.454361, 0.471712, -0.029662, 0.606608, -0.384717, -0.897486, -0.863412, -1.824882, -1.559158, 0.203892, 0.728770, -0.718008, 0.052547, 0.762671, -1.115231, 0.743813, 0.719556, -0.656431, -0.832916, -1.680627},{-0.985901, -0.624102, -0.801817, -1.761617, 1.089604, -0.825159, -1.394355, -1.232599, -1.030174, -0.057085, -0.577342, -0.452802, -0.303342, -0.867580, -0.702439, 0.584201, 0.943940, 0.067113, 1.052635, 0.444811, 2.004902, 1.925233, 0.238311, 0.337739, 0.684756, -1.467159, -0.012835, -2.642469, -0.952544, 0.863388, -1.816154, -1.612229, 0.420913, 0.148273, -1.351760, -1.264611, -0.832505, -1.898905, 2.677955, 0.953040, 1.028044, 1.398584, -0.201142, -0.562899, 1.532697, -1.326048, 0.822152, 0.040751, 1.007349, -1.746829, -0.980333, 0.662153, 0.472197, 1.273243, 0.006627, 0.497993, 1.850666, -0.552316, 0.114384, -1.193856, -1.102213, -0.574334, -0.387281, 1.221296, -0.491452, 0.321816, 0.430199, 0.868040, 0.758850, -0.907532, 0.327337, -0.150282, -0.288359, -0.878657, 0.929553, 0.296394, 0.468961, 2.045575},{0.700674, 0.436102, -0.040496, -0.028613, 0.484475, -2.482195, -0.236317, -0.746719, -0.915272, 0.240576, -0.208988, -0.654454, -1.664423, -0.707469, 1.590012, 0.273949, -0.293852, -1.717995, 0.291226, -1.272712, -1.846309, 0.351928, -0.710803, 0.721700, 0.109492, 0.625742, -0.063624, -0.704806, 0.694220, 1.527339, 1.242451, -0.596028, 1.191906, -1.200429, 1.168108, -0.434382, -1.139967, -2.094606, -0.133248, 1.373938, 0.868321, 0.740534, -2.009940, -0.521548, 1.019466, -0.213906, 0.582309, 0.319308, -2.207396, 0.074094, -1.957354, -0.415229, 1.268782, 1.784767, -3.013162, -0.292049, 0.274447, 1.837215, -1.121574, 1.016980, 1.480163, -0.620872, 1.049206, 0.044020, 0.380023, -0.472078, -0.846455, -0.395163, 0.263630, 0.374036, 0.215267, -0.344873, -0.234476, 0.451575, 0.474491, -0.152151, -0.534337, -0.822208},{1.906039, -0.808885, 0.764578, -0.517925, 0.874986, -0.024488, -0.638296, 0.473365, 0.962027, -0.103387, 0.181383, 1.362442, 0.086458, 0.766811, -1.543172, -0.442464, 0.655023, 0.125207, -0.741639, -0.835384, -0.012903, -0.379040, 0.015438, 2.029412, 1.250739, -0.834583, 1.032216, 0.744374, 0.118625, -0.337820, 1.231936, 0.782887, 0.648988, 0.182450, -0.119235, 0.866605, -0.146515, 0.316934, 1.491839, 1.170841, 1.362236, -0.857753, -0.379936, 0.888795, -0.247111, -0.477389, -0.456177, 0.512253, 0.157907, -0.578234, -1.507823, 0.565072, 2.426598, 1.557474, 0.200141, 1.399907, -0.869356, 0.841788, 0.468952, -0.550635, -0.372789, -0.195410, -0.813587, 0.007409, 0.764951, 0.132829, 0.464691, -1.479758, 0.606107, 1.036239, 0.132398, 0.242819, -1.141554, -0.539232, 0.598725, -0.125122, -0.462699, -0.960433},{0.722202, 0.391474, 1.171172, -0.140799, -0.294600, -1.675043, 0.546347, 0.660532, -2.038309, 1.517781, 0.394858, -0.725110, -1.979591, -0.316749, -1.314783, -0.101022, 0.441915, 1.612225, 0.000449, -0.116685, -2.699457, 0.403875, -0.855374, -1.124763, 0.089225, 0.649495, -1.024841, -0.706067, 0.038239, -0.317279, -0.352848, -0.897440, 0.715682, -0.365896, -0.659433, 1.872016, 1.413429, -1.030639, -0.741162, 1.533016, -0.321781, -0.116953, 1.320873, 1.585271, 0.901327, -0.288914, 0.491003, -0.867478, -0.334551, 0.763029, -1.380630, -1.172247, 1.198351, 0.486318, -0.059129, 0.758257, -2.115966, -0.241810, -0.104660, 1.751901, -1.402642, -0.558885, -0.664907, -0.611730, -0.163991, 1.079546, -0.312580, 1.308334, 0.992508, -0.405722, 0.566049, -1.615977, -0.609632, -1.394819, 0.315542, 0.019426, 1.387854, 0.402175},{2.540679, 1.684087, -0.201053, 1.435910, -0.953762, -2.000056, 0.298059, -0.301138, 1.203890, -0.711487, -1.192795, -0.050777, 0.824640, -0.454334, -0.322275, -0.202020, -0.736811, 1.933686, 0.288213, -1.556391, -1.899317, 0.591427, -0.029896, -0.062817, -1.386697, -1.917662, 1.165830, 0.393499, -0.484306, 0.739234, -0.465630, -1.426195, -0.839730, 0.443789, -0.034721, -0.062894, 0.189363, 1.207444, 0.004114, 0.818590, 2.718635, 0.801146, -0.609280, -1.225267, -0.466688, 1.174490, -1.988116, -0.010655, -0.844196, -0.432889, 1.009653, -1.497212, -0.563651, -0.169607, 0.381587, -1.401021, -0.145359, -0.860903, 2.683651, 0.732237, 0.449948, 0.252241, 0.885770, -0.016057, 0.910948, 1.212616, -0.557676, 2.483513, -0.783527, -0.012999, 0.008371, 0.037248, 0.562407, -0.105300, 0.251830, 1.998826, -0.894204, -0.384835},{-1.188345, 0.931485, 0.070481, -0.847196, -0.312811, -0.526860, 0.391011, -0.002729, 0.580766, 0.338645, 0.091653, -1.256477, 0.079087, 1.051609, -1.510707, 0.897549, 0.271956, 1.462118, 0.760684, 0.154129, -0.215380, -0.298195, -3.476591, -1.480209, 0.787366, 1.291499, 0.792080, 0.960381, 1.438179, -0.407617, -0.288522, -1.874581, -2.159733, -2.203182, -1.282813, -0.703188, 0.279800, 1.159754, -0.912699, -0.228391, 1.163482, -0.112020, 1.184440, 1.923006, -1.235385, -0.523211, 0.007218, 1.581517, -0.912072, -0.756134, 0.730706, 0.152108, 0.580274, 0.378663, -1.479380, -0.263511, 1.322384, 0.702914, 0.236709, -0.670857, 0.321692, -0.816681, -1.171428, 0.285797, -1.107879, -0.157714, -0.526106, -0.500608, -1.770968, -1.842366, -0.270180, -0.074368, -0.631325, 0.594273, 0.624505, -1.597878, 0.275712, -0.104413},{-1.760679, 0.883394, -1.123756, -0.068391, -0.895505, 0.805237, 1.095888, -0.709883, 0.312467, 0.072190, -1.025446, 0.636331, 0.317082, 1.233532, -0.403986, -0.343610, 0.715558, -0.684813, -2.817741, 0.439884, 0.733294, 0.092253, 0.127369, -0.448600, -1.758748, 0.054339, -1.746008, 0.392494, 0.901650, -0.564831, 0.144771, 1.346826, 0.997650, -0.626620, -0.074837, 0.983664, 0.194302, -0.399165, 0.675275, -0.100349, 0.537990, 0.821830, -0.211688, -0.474044, 0.988566, -0.314544, 0.390844, 0.209573, -0.573284, 1.474880, -2.932371, 0.039364, 2.837974, 0.524808, -0.804498, -1.022310, -2.055237, -1.393098, -1.429432, -0.772229, -1.412580, 0.203719, 1.094936, 0.455664, 0.607477, -0.712902, -0.194432, -0.117476, -0.731267, -0.353404, -0.514055, -0.832907, -0.250360, -0.374714, -0.096854, 1.591905, 1.016112, 0.619134},{0.757468, -0.493955, 2.249424, -2.080327, -1.822667, -1.354506, 0.696984, 0.275139, 1.088666, -0.556749, 1.713462, 0.094322, 0.072045, -0.122859, 0.512984, 0.499655, 0.995794, -0.166031, 0.793114, 1.022230, -1.263178, -0.357193, -1.628946, 0.894788, 0.040216, 0.242307, 0.439258, -1.177836, -1.089735, 0.255178, -0.297533, -0.101690, 0.562214, 0.185820, -1.526221, 1.505203, -2.080765, -0.307092, -0.942968, -0.260440, -1.050433, 0.783335, -0.700226, 0.138222, -1.126153, 0.115088, -0.326344, -0.866674, 2.287741, 1.318710, -0.075302, 2.234913, 0.224986, -0.542296, 1.224354, -1.192833, -0.004578, 1.349525, 0.367754, -0.027526, -1.818693, 1.601844, -1.468781, 0.183987, -2.106174, 1.248238, -0.064813, 0.350854, -0.081985, -1.287228, 0.067072, 0.803684, -1.066499, -0.380326, -0.357485, -0.833692, -0.980093, 0.486826},{-0.258268, -1.583922, -0.895688, -1.743386, 0.373772, -0.756022, -0.103584, -0.191331, 0.610182, 0.165924, 0.847702, 0.581758, -0.953043, 0.401101, -0.010396, 1.022018, -0.909689, 1.037571, -1.732350, -0.674485, -0.338651, -1.335695, 0.374535, -0.895758, 1.610056, -0.072855, 0.355217, -0.902066, -0.153094, 0.634729, 1.698475, 0.228256, 1.496639, -1.804268, 0.457905, 1.074784, 0.789548, -0.211583, 1.234034, 0.384204, -0.493169, -0.058874, 1.276815, -1.799310, 0.039108, -0.985193, -0.214569, 0.707645, -0.099472, 0.842942, -2.102566, -0.278414, 0.988108, 0.631237, 0.773178, -1.368213, -0.894080, 0.922269, 1.253889, -0.997104, 0.192569, -1.172648, 0.339812, 0.935889, -0.119472, -0.730113, -0.589426, -0.181029, -0.201813, 1.433539, -1.087312, 1.296963, -1.182691, 0.812923, -0.392704, -0.848981, -0.533996, -0.414430},{0.463597, 0.044188, 1.797466, -1.277202, 0.814829, -2.107167, 1.087220, 0.284716, -0.590784, -0.416853, 0.505634, 0.144319, -0.360356, 0.740816, 0.526367, 0.015442, 0.400372, -0.414638, -1.940154, 1.304772, -0.958693, -0.540807, -1.119918, 0.301504, 0.061732, 0.517651, 0.810048, 1.349101, -0.941247, -1.802547, -0.791187, -1.131728, -0.460396, -0.797268, -1.130996, -0.857371, -1.098936, -0.056696, -0.789897, -1.464856, -1.320612, -0.105154, 0.849115, 2.245319, -0.133857, -0.814270, 0.038370, -0.597712, 0.395137, -0.812985, -0.746470, -0.416881, -0.728327, 1.134601, -1.725183, 1.434278, 0.060701, 1.079949, -0.045156, 0.054905, 0.301893, -1.001054, 0.558451, 0.598112, 0.732785, -0.155387, -0.363655, -0.484544, -0.785243, -0.741359, 0.488935, 1.898245, -0.335294, 1.311116, -0.042626, -0.821323, 1.104237, 0.733183},{-1.987847, 0.683512, 0.246893, 0.826311, -0.279779, 0.044552, 0.345617, 0.472769, -0.061734, -0.157191, -0.329443, -0.728478, -0.931940, 1.098523, -3.373317, -1.512402, -0.264706, 0.829873, 0.344374, -0.732772, 0.442728, 0.383598, -2.636312, -0.745733, -1.065985, -0.169700, -2.459104, -0.912575, 1.121541, 1.270105, -1.165215, 0.129417, 0.540955, -0.139889, 1.047060, 0.303929, -0.906335, 0.947435, -0.094327, -1.085377, 0.744953, -0.452191, -0.511883, 0.799328, -0.356443, 0.685119, -0.818129, 1.297170, -0.795893, -1.481924, -0.282289, 0.273706, -0.323737, -1.019407, -0.608516, -0.726671, -0.425070, -1.529003, -1.476265, 0.312068, 1.115775, -0.450457, 0.085674, -0.541679, 0.528090, 0.550373, -0.062507, 0.675366, 0.108412, -0.684409, 0.380215, 0.881230, -0.516090, 0.212606, -0.542701, -0.398418, 1.531888, 0.576216},{0.073933, -0.086243, -0.994640, 0.346138, 1.512136, 0.486516, -0.723110, 1.418824, -1.180655, -0.458359, -0.696182, 0.014961, 1.010919, -0.952360, -0.052919, -1.163284, 0.716999, 0.378652, -1.082957, -0.469288, -0.132567, 1.469580, -1.727614, 1.538901, -0.298695, 0.516013, 1.147963, -1.473564, 1.030591, -0.388190, -0.101066, 0.741715, -0.481809, 0.275909, -0.424693, 0.634702, -1.258739, 1.067144, -1.665681, -0.489684, -1.850918, -0.887299, 0.163253, 1.512528, -0.687061, -0.488356, 0.997440, 0.222228, -0.661982, 0.738375, -0.815571, -0.355038, 0.891620, -0.257984, -1.025854, 2.286342, 1.063228, -0.839478, -0.152043, -1.760501, 0.636917, -0.463164, -1.240662, -0.797214, 0.442158, 0.397441, 0.074430, 0.994532, -1.529873, 0.198704, 1.133843, -0.791382, -0.246723, 0.530575, 0.428470, 0.997565, -0.643647, 1.331796},{-2.353844, 1.152695, -1.094799, -1.059595, 0.549216, -1.406580, 0.391952, -0.178871, -0.783840, -0.777789, -0.593972, 2.127054, 0.583057, -0.593699, -0.546370, 0.559184, 0.717403, -0.800610, -1.213045, -1.654930, 1.316543, 0.796006, -0.797000, -0.369536, 2.428290, 1.566901, -0.275958, -1.176432, -1.390123, -0.453137, 0.288629, 0.176541, -1.778830, -0.993595, 1.086236, 0.234593, -0.084251, -0.858667, -0.517810, -0.563519, -1.951294, -0.017173, -0.214104, 0.637201, -1.181383, -0.879225, 0.491949, -0.620266, 1.610696, 0.982810, 0.580757, -0.588909, 0.717760, 0.418645, -0.125344, -2.402151, -0.229389, -0.542371, -0.748331, 0.531175, 1.911098, 0.112503, -0.351573, -0.256122, -0.004821, 0.805631, -0.015951, -0.272668, -0.274830, 1.469668, -2.271059, -0.740279, 1.136417, -0.533085, -0.230980, -0.042529, -0.297803, -0.962109},{0.079045, 0.543868, 0.354331, -0.735997, 0.526652, -0.762780, 0.599128, 0.350947, -2.738517, -0.057148, 1.970552, 0.964241, 0.349394, -0.078693, -0.747797, -0.441622, 0.314969, -0.433383, -1.823331, 1.877410, -1.216888, -0.534172, -1.327179, 1.747483, 0.294776, -1.119815, -0.281271, -0.856603, 0.032490, 0.896329, 0.830110, -0.773129, 1.328305, 0.237293, -0.591421, 0.876345, 0.574107, 2.702398, 1.132224, 1.063723, 1.270007, 1.508004, 0.465202, 1.793842, -0.261179, 1.740866, 0.007812, 1.245977, 0.121925, 0.251562, -0.800982, 0.868651, -1.822766, 1.961506, -0.859516, 0.987227, -1.097458, -0.018304, -0.119813, -0.473126, -0.936888, -0.700947, 0.630902, -1.167547, 1.525495, 2.023282, 0.273241, 0.399658, -0.566814, -0.815684, -0.554438, -1.639057, -2.662487, 0.755525, -0.096896, 0.698711, -1.311415, -2.009668},{-1.115549, 0.446809, 0.445103, -0.238838, -0.846738, -0.547049, -0.986580, 0.079636, -0.929626, -2.626254, 0.238519, 0.330802, 1.271177, -0.352262, -1.133254, -0.447007, -0.045175, 0.425769, -0.303551, -1.872512, 0.015787, -0.686312, -0.157661, -0.534772, 0.404425, 1.606222, 1.004035, 2.099118, -0.392421, -1.078075, -1.001533, -1.201557, -0.889367, -1.525393, 1.006063, -1.435662, 0.088335, 0.230665, 0.091561, -1.533775, -1.064437, -0.851763, 2.707222, 0.840010, -0.430050, 1.052186, -2.025909, 1.415063, -0.066267, -0.552192, -0.411880, -0.780829, 0.733405, -0.857789, -0.564865, 0.340015, 0.137228, -0.844991, -0.431091, -0.543793, 0.518410, -0.755876, 0.218882, -0.690307, -0.111266, 0.843232, 0.860032, 1.163931, 0.192771, -0.831561, -1.209268, -0.240803, -1.012869, 0.826091, -0.926733, -0.019279, 1.155344, -1.139901},{2.079458, -0.297055, 1.549471, -0.196983, 1.019810, 1.423495, -0.887253, 2.316243, 2.628678, 1.443813, 0.449555, -1.075276, 0.462368, -0.931590, 1.037437, -1.095930, 1.410725, 0.797820, 0.864163, -0.630516, 1.302150, 1.011368, 0.780313, -0.129752, -2.987721, -0.138275, -0.015390, 0.213416, -1.611472, 1.709975, -0.764109, -2.568138, -0.123239, -1.035062, 0.706430, 0.964952, 0.414212, -0.802499, 0.045631, 2.060276, -2.211184, 0.080321, -0.688516, -0.529687, 1.055537, 0.709236, 2.085828, -1.067221, -0.915090, 0.413246, -0.055615, 0.674817, -0.813994, -0.899618, 0.562156, -1.153349, -1.528187, -0.324475, -0.580364, -0.649421, 1.523615, -1.316299, -0.320929, -0.738272, -1.013555, 0.577758, -0.177031, 0.430206, 2.548383, -1.459210, -1.084326, -0.329822, -1.298302, -0.741563, -2.947710, 0.095146, 1.920758, -0.266810},{-0.198116, -0.242541, 0.994466, -0.208125, 0.582352, 0.756210, 1.850938, 0.427651, 0.255657, -0.395655, 0.429158, -0.744002, 0.519287, 1.000795, 0.811981, -1.066811, -0.595257, 1.541870, -0.020487, 1.243642, -0.577084, -0.798885, -1.571835, -1.615311, 0.354391, 0.170670, -1.113785, -0.065130, 0.056976, -1.074609, 0.920013, -1.200728, 1.815023, 0.376872, -0.142960, -0.780989, -1.408130, -1.352868, -1.147921, 0.553457, 0.385230, 0.945442, -0.103261, 0.197078, 0.017139, 0.596597, 1.325099, -0.765168, -2.189019, -0.214063, 1.372158, 0.424522, -0.133216, -0.669626, 0.545020, 1.157092, 0.848184, 1.131455, 1.960516, 0.174650, 1.127584, 1.702355, 0.976336, -1.178789, -0.562688, 0.734703, -0.023258, 1.755645, -0.078800, -0.265983, -0.346677, 0.522886, -0.863423, 0.878457, 0.530073, 2.095196, 0.174855, 1.441263},{-0.797077, 0.196301, 0.208507, -0.101883, 0.328674, -0.458023, 0.485461, -1.162428, -1.350981, 1.246288, -0.377541, 0.271344, 0.951630, 0.919642, 0.297951, 1.539630, 0.479071, 1.527804, -0.539127, -1.413253, 1.091037, -0.986511, -0.248660, 0.680965, 0.122713, -0.087724, 1.264106, -2.321095, -0.647421, -0.232128, 0.305513, 0.995028, 0.501611, 1.333804, 1.173972, -0.372101, -1.115193, -0.290418, -1.289316, 0.490157, 0.559459, -0.160164, -1.096184, 0.404664, 0.901486, -0.548280, 0.386060, -1.282187, 0.647668, 0.121998, 0.446704, -0.678134, 0.994088, -1.441530, 0.891071, 0.207254, -0.931368, -1.219280, 0.841423, 1.840985, -0.214640, 0.093403, 0.643517, -0.363767, -0.365185, 0.948436, 1.083442, -1.735990, 2.230997, 1.702086, -0.164142, 0.003129, -0.425938, -0.015872, 0.118281, 1.174391, 0.026951, 0.639288},{-0.503106, -0.198359, 0.063803, 0.007610, -2.094629, 0.484474, 1.766611, 0.846239, 0.137113, -0.082810, 3.068287, 0.705012, -0.065140, 1.112687, 1.662135, 0.766383, 0.283604, -0.617210, 1.508392, 0.794711, 0.462510, -0.395745, -1.791178, -0.884631, -0.254006, 0.222970, 1.284250, -0.646351, -0.953664, -0.473019, 0.947020, -0.060717, 1.406887, -1.250238, -0.550228, -0.103723, -0.617746, 0.568104, -1.928510, 0.487118, -2.406273, 1.596343, -0.700121, 0.095609, 0.978956, -1.605004, -0.253629, 1.064134, -0.665448, 0.222961, -1.779088, -2.740554, -0.649616, 0.216411, -0.729466, 0.647442, 0.621001, -0.391477, -0.460622, 0.199264, 1.157474, 0.295354, -0.213820, 0.430644, -0.251326, 1.977923, -0.397983, -0.251840, 1.407378, -0.921409, 1.164894, -1.739486, -0.289598, 2.191874, 0.447298, -1.113293, 0.102704, -1.291291},{-0.925229, 0.605455, -0.868902, -0.153604, 1.373970, -0.990722, -0.610027, -0.823493, -0.989275, 0.413707, -0.079185, -0.457793, 0.921814, 0.747386, 0.775413, -0.761783, -1.020073, 0.098645, 0.984205, 1.743861, -1.488325, -0.000252, 1.845048, -0.437111, -1.080309, -1.494028, 0.005888, -1.143309, -2.695265, 0.894431, 0.425012, -2.006074, -0.436960, 0.075687, 1.727522, -0.255485, -0.298483, 0.165891, 2.041622, -1.252663, 0.108257, -0.920002, 0.585456, -0.790687, 1.003304, 0.966865, -0.964455, 0.547320, -0.675615, 0.695229, -1.239231, -1.639631, -0.884475, -0.368049, -0.484539, 1.602160, 1.025759, -1.938577, -0.031151, -0.373609, 0.900410, 0.810097, -0.462920, -0.560221, 0.170638, 0.093037, -1.492403, -2.546471, -0.358174, 0.023255, 0.570283, -0.496671, -0.137327, -0.356685, 1.085629, 0.237522, -0.866668, 0.018416},{0.024473, -0.424523, 1.268002, -0.460024, -0.141153, 2.935903, 0.775879, -0.025996, -1.819530, -0.104684, -0.035171, -1.301075, -0.333299, -1.414314, 0.803711, -0.911389, 0.446330, 0.444552, -0.886438, -0.952952, -0.861270, 1.718910, -0.143570, -0.727919, 0.380831, -1.243039, -0.745059, 1.658164, -1.319119, -0.598427, -0.679129, -0.888919, 0.700591, -0.295300, -0.104262, -0.428246, -0.788501, 1.841284, 2.247248, 0.759413, 1.099051, -0.424398, -0.079261, -0.309851, 0.107524, 1.848805, -0.791560, 0.069583, 1.725379, -0.043918, 1.643485, -1.799817, 1.494975, -1.442779, 0.270056, -0.654303, 0.146476, -1.438303, -0.524280, -0.059146, -0.044416, -1.645839, -1.006939, -1.627855, 2.634865, -0.118330, 1.470782, 0.653635, 0.689811, 1.332168, 0.511734, -0.775482, 1.192926, -0.391644, -0.819239, -0.628932, 1.554308, 0.126473},{0.470875, 0.428796, -0.423796, -1.053368, 0.811452, -0.481661, -1.002122, -2.027901, -1.230071, 0.207659, -1.832593, -0.764145, -0.592267, 0.722314, 1.719498, 0.713414, -1.531916, 0.166879, -1.675179, 0.192068, 0.474188, -0.597892, -1.866321, -1.163732, -2.409139, 0.218957, -1.323574, -0.370532, 0.685998, 0.885144, -0.391263, -1.089608, 0.496658, 0.727122, -0.399724, 3.010324, -0.656613, 1.448693, -1.221609, -0.067999, 2.185264, 0.563775, -1.798112, 0.550954, -1.059307, -1.348237, -0.949573, 0.505565, -0.557131, 0.874778, 0.288601, 0.480617, 0.863355, 0.173963, -0.123705, 0.558004, -0.362907, 0.282738, -1.530818, 0.789139, -1.321546, -0.773032, 0.714484, 1.039734, 0.046734, 0.169455, 0.989492, 0.452231, -0.488256, 2.303858, 1.166860, 1.155132, -0.634941, -0.196707, 1.918879, 0.983180, 0.182490, -0.839946},{0.136516, 1.763518, 1.042724, 1.281485, 1.255934, 0.239387, 1.014292, -0.548395, -1.569151, -1.054272, 2.433510, 0.944968, 1.337477, -1.066174, 1.131279, -0.348103, 1.341814, -1.323475, 0.317332, 0.758907, -1.119452, 0.397695, -0.588931, 0.626017, -0.311663, -0.552112, 0.549831, -0.152081, 0.392896, 0.547771, -0.569109, 0.206032, -0.661025, 1.725306, -2.276862, -0.586227, 0.162007, -1.107915, -1.731015, 0.627607, 0.966407, 0.550923, -0.599270, -0.100391, 0.535965, -0.372368, -2.125477, 1.244910, 0.779625, 0.160137, 1.943099, -0.242628, 0.118837, -0.581652, 1.305308, 0.828161, -0.628649, -0.248130, 1.212492, 0.784955, 0.298227, -2.074783, -0.099421, 1.114513, 0.280730, 0.320828, 1.066828, 0.072835, -1.133144, -0.237126, -1.003396, -0.232562, -0.851357, -0.236398, 0.283853, 0.150517, 3.401039, -1.887927},{-1.166284, -2.053726, -1.174037, -0.747538, 0.280979, -0.869484, 0.005423, -0.955724, 1.619866, -1.213714, -1.946651, 1.931738, 0.972383, -0.436645, -0.515203, 0.001105, 0.852507, -0.665704, 0.373259, -1.044971, -0.624986, -0.210634, 0.398277, -0.447856, -1.419995, 0.325201, 0.707756, 1.072389, -0.429168, 1.085082, 0.862428, 0.034973, 1.005319, -2.217786, 0.761605, -0.132335, -0.066337, 1.641516, -0.094677, 0.393353, 0.065415, -0.779967, 0.846681, -0.591786, -1.221501, 0.265346, 0.063720, 1.252248, 0.128018, -1.470119, 1.935904, -1.500673, 0.698314, 0.615559, -0.349687, -0.549762, -0.633184, -0.417976, -1.246777, -0.621851, 0.271543, -0.106678, 1.311938, -1.644927, 1.788604, 0.079518, 1.149566, 0.109522, -2.834307, -0.861263, -1.268125, -0.094692, 2.267583, -0.671823, -0.229845, 2.061277, -0.197076, -1.360417},{1.711880, -0.761808, 1.136524, -0.225236, 0.653622, -0.157527, -0.420677, 1.055384, 0.018049, 1.113626, 1.044176, 0.016428, -0.519749, -0.666963, -0.242525, 0.630952, -0.819840, 0.025603, 0.373781, 1.165292, 1.327761, 0.240403, -0.328122, 1.483348, 1.046471, 0.370028, -0.103576, -0.790161, -0.583194, -0.816640, 0.830129, 0.224029, 1.561599, 1.734225, -0.263032, 0.800277, -0.180641, -0.224827, -0.136044, -1.067630, 0.229476, -0.496345, -2.019717, 0.511696, 0.046653, -1.028321, -0.266629, -0.235672, 1.786586, 1.124011, -0.141991, -0.087426, -0.631409, -1.326182, -1.744493, 0.874602, -0.861089, 1.023467, -0.189062, 0.120629, 0.294829, -0.814090, 1.018148, 0.840813, -0.030426, 0.919776, -0.857317, 0.252892, 0.635466, 0.621444, 0.512471, 2.811755, 1.107812, -0.932418, -0.215618, 0.342755, 2.029854, 1.640545},{-0.400705, -1.332061, -1.187015, -0.901786, -0.160773, -1.331182, -0.061321, -0.507955, 0.081373, 1.322078, -0.787595, -0.686221, -0.533360, -0.918355, -1.265706, 1.051696, 0.118838, 3.075303, -0.145775, -2.554332, 2.407644, 1.272998, -0.605974, 0.629091, -1.291762, 0.328152, -1.308108, -0.117752, 2.428119, -1.209754, 1.152963, 1.072283, 0.250391, 0.268243, 1.058869, 1.155058, 1.253821, 0.197314, -1.085696, -2.305840, 0.504630, 0.690215, 0.244331, -0.105383, 1.461025, 0.894976, 0.799338, 0.227988, 0.078978, -0.171460, 0.166634, 2.114021, 1.097786, -1.422501, -0.575077, -1.329703, -0.592543, 0.463019, 0.027746, -1.240012, 0.403193, -0.497949, 0.347588, -0.351344, -1.023568, -0.660516, 1.225828, 0.973796, 0.772722, 0.955259, 0.459777, -0.304140, -0.736363, 1.525070, 0.636695, -0.215330, 0.086099, 3.946016},{-0.024764, -1.320775, 0.358638, 0.333028, -0.014370, -0.200300, -0.395550, -0.123112, -2.158561, 0.359961, -0.956492, -1.148155, 0.292980, 1.280911, -0.517524, 0.398203, 0.704758, -0.694648, -1.619113, -0.252061, -0.478522, 0.396166, 0.152614, -0.953592, -0.475787, -0.402779, 0.660837, 1.364169, 0.754234, -0.885940, 0.158545, 2.037381, -0.307728, 2.302028, -1.051956, -0.853992, -0.421435, -0.301928, 2.365197, -0.581837, -0.177265, -0.288694, 1.119281, 0.671866, 0.088961, 0.581152, 0.850658, 1.557308, 1.245381, -0.592606, -1.085308, -0.190399, 1.383392, -0.475320, 1.267891, -0.261338, -0.484819, 1.398636, 0.308186, 0.608407, -2.101702, -0.511522, 0.872702, -0.102802, 0.058631, -0.314810, -1.333702, 1.587090, 0.122092, 1.364943, 0.165548, -0.584902, 1.478675, 0.358118, 0.451559, -1.133336, 0.464587, 1.110893},{0.342628, -0.255500, 1.602947, -0.195860, 0.222190, -0.215117, 0.596275, 1.809960, -0.026191, -0.693462, -1.370478, 0.000588, -0.653868, 0.829020, -0.087670, 1.351442, -0.196791, 1.328876, 0.954769, 0.086714, -0.382453, 1.806059, 0.206899, -0.215547, 0.009571, 0.144051, -1.239734, -0.852078, 0.227296, -1.012013, -0.513947, -0.361203, -0.436334, 0.686115, 0.998402, -0.170871, -1.099455, -1.115788, -0.337033, 0.239711, -0.118051, -0.523735, -1.112568, -0.053653, -0.870615, 0.189654, -0.059495, -0.458429, 0.001674, 1.931525, -0.221917, -0.173500, -0.414461, -1.551408, -1.877956, -0.191642, 0.598062, 0.650427, 0.439390, 0.948234, -0.049042, 0.381711, 0.313079, 0.577684, 0.288957, -0.646398, 1.964458, -0.491208, -0.754920, -0.035731, 0.325758, 0.207101, 1.441566, -0.789264, -0.371648, -1.141212, -1.218025, -1.022567},{-0.849956, -1.317301, -0.195529, 0.119197, -1.445324, -0.219675, -1.432843, 2.996098, 0.498935, 0.625244, -0.928006, 0.024144, 0.471002, -0.941711, -0.758245, 0.389613, -0.108467, 0.248398, -1.394312, -1.241975, 0.485913, -0.742332, -1.294971, 0.510043, -1.247771, -0.403067, -0.161141, 0.934080, -1.317766, -0.143914, 0.643334, 0.395191, 0.263791, -1.762302, -2.691025, 0.305380, -0.899540, 0.757876, 1.299995, 0.785413, 0.432423, 0.242987, 1.292805, 0.783207, -0.247531, 1.832906, -2.341434, -1.018122, 0.476874, -2.158733, -0.734413, 0.855714, 1.401250, 0.680789, 0.641275, -0.659574, -0.554651, -0.238118, 1.294206, -0.328114, 1.672573, 1.409252, 0.999324, -0.570096, 0.522279, 0.338088, -0.417747, 1.432501, -2.476443, -0.361200, -0.304849, 0.059488, -1.045279, -0.276273, -1.253499, 0.456399, -1.647451, -0.063916},{-1.590544, -0.175028, -0.645301, -1.758388, 1.075089, 1.098834, 1.694797, 0.238167, -0.396195, 0.415617, -0.752928, -1.210199, 0.923099, -2.421752, -1.732869, -0.805533, -1.602965, 1.473997, 0.537274, 0.079291, 2.404075, 0.166875, 0.825017, 1.265122, -0.390964, 0.178400, 0.159760, 0.115902, -0.099848, 1.339310, -0.702135, -1.588013, -0.901091, 1.401052, 0.887618, -0.473388, -0.048593, -0.036230, -0.534908, -0.895760, -0.168542, 0.703891, 0.048194, -1.321438, -0.748557, -0.754263, 0.231125, -0.273501, 0.000044, -1.459718, 0.224839, 0.142866, -0.094155, -0.201164, -0.614662, 1.037365, 0.403091, -0.176196, -0.947316, 0.672563, -0.210824, -1.660438, -0.842677, 0.860567, -0.022947, -1.497544, 0.618880, -1.801295, -0.549460, 1.464776, -0.323976, 0.018998, -0.891556, 0.635392, 0.314759, 1.995750, -0.718089, 0.518194},{-0.613428, 1.307883, 0.109719, -0.496215, 1.081059, -0.490962, 0.831465, -0.898842, -0.584739, 1.503693, 1.726085, 0.097722, 0.735724, -0.213644, 0.018651, -0.007946, 1.237199, -1.428339, -0.659935, 0.492919, 0.604349, 0.128043, 0.580439, -0.367822, -0.386257, 1.261939, 1.142920, 0.780764, 1.710309, -0.677667, -2.088326, 0.504438, -0.140314, 0.194787, 0.826035, -0.838211, -0.843430, 0.494767, 0.416216, -1.066345, 1.584470, -0.552002, -1.162249, -0.934858, -1.201305, -0.430859, 0.361757, 0.795568, 0.528161, 0.103728, -0.644694, 0.873589, -0.856046, 0.230958, 0.303968, -1.185580, 0.117724, -1.059598, 0.164741, -0.664010, -2.743144, -0.542423, -1.081832, 0.567771, 0.144517, 0.692064, 0.854294, 1.001810, -1.913300, 0.396570, 0.277599, -0.073553, 0.294000, -0.620046, 1.422532, 0.614309, 0.079704, 0.137786},{0.647795, 1.124352, 2.298506, 1.333033, 0.206670, 0.587474, 0.108546, 0.795231, -0.163482, 1.861499, -1.762024, 0.474049, 0.298272, -1.230967, 0.172520, -1.413693, -0.616205, -0.866995, -0.435503, -0.262782, 0.082902, -0.770060, -1.290997, -1.249416, 0.476614, -1.276565, 0.433320, -0.409040, -0.367899, 0.983355, -0.545373, 1.252461, -0.772376, -2.511130, 0.107250, -0.557007, 0.863207, 0.887340, 0.766351, -0.316575, -2.866457, 0.519020, -0.740906, -0.088939, 0.187482, -1.127010, 0.337666, 0.158496, 0.604062, -0.578083, 0.376194, 1.129514, 0.251485, -0.691634, -1.256998, -0.061997, 0.095867, 0.367453, 0.898268, -0.683092, 0.335971, 0.921003, 0.409763, -0.722204, -1.505182, 0.982832, -0.209683, 0.702511, 0.840052, -2.092376, -1.389229, 1.211280, -0.122082, 0.054563, 0.977836, -0.213011, 0.666088, 1.739724},{-0.300688, -1.826635, 0.242054, -0.212698, 1.457903, -0.188440, 1.360645, -1.101811, -1.203209, 0.640041, -2.419762, -0.225251, 0.316095, -0.589550, 1.067130, 0.063260, -0.033082, -1.174060, 0.740049, 1.692703, -0.323434, -0.753697, 0.248588, -0.990290, -0.267375, 0.544862, -0.305643, 2.136055, -0.064729, 0.650750, -0.328226, -0.215575, -0.214598, 0.908595, -0.611246, -0.759742, 0.436735, -0.520830, -0.197299, 0.740625, -0.231379, -1.480409, 0.328525, 1.142584, -1.747140, -0.821683, -0.492302, 0.807868, -0.221220, -1.746330, 0.399033, 1.287156, -1.063293, -0.116417, 0.017892, 0.238006, 1.520790, -0.075007, 1.851287, 0.198122, 1.464093, -0.550845, 0.709640, 0.211061, -1.455705, -1.045722, -1.195062, -0.784818, -0.216277, 0.355199, -0.021097, 2.803187, 1.264340, -0.334935, -0.165991, 0.378979, 1.820815, 0.980454},{-0.682168, 1.237702, -0.119411, -0.156692, -0.251860, -0.878588, -0.970808, 0.340879, -1.020191, -1.277434, 0.730150, -1.036095, -1.501702, 0.289116, 0.322867, 0.142292, -1.201374, 1.104718, 1.025984, 1.260592, -0.438121, -0.842299, -1.756113, 1.509207, 0.152776, -1.250607, -0.605930, -1.896842, 0.720205, 0.824719, -1.680459, 0.297344, 1.361205, 1.691543, -1.747124, 1.345468, -1.288233, 0.046859, 0.779798, -0.458410, -0.857268, -0.118199, 0.518462, -0.827354, 1.013989, -0.658729, 1.632971, -0.996238, 0.791935, -0.282192, -1.094390, 0.300771, 2.040493, -0.953955, -1.456557, -1.696378, -1.994740, -1.933130, 0.077624, -1.241060, -0.824037, 0.538144, -2.116047, 0.539053, -0.818989, 0.059920, -0.741978, -0.056471, 0.889186, 1.991754, 1.184028, -0.856893, -0.640365, -0.567797, -1.773777, 1.316733, -1.213798, -0.593454},{1.652584, 0.423523, 0.200070, 0.061317, 1.729331, 0.956388, 0.665288, 0.074415, 0.202195, -0.103534, 2.056022, -0.848137, 2.085408, 0.945330, -0.312097, -1.110217, 0.905414, 0.160429, -0.301217, -0.586255, -0.810224, -1.587160, 2.375172, 1.281911, -1.019050, 0.231888, -0.048665, 0.975558, -1.680349, 0.225847, -0.916775, -0.897906, -0.355200, 0.743862, -1.052228, -0.025325, -0.038128, -0.018375, 0.618990, 1.333717, 0.269647, -0.095829, 0.358436, 1.438257, -0.335393, -0.433483, 0.911148, -0.399700, 1.404656, -0.627071, 0.630945, 0.091088, 0.420827, -0.440724, -1.058387, 1.464539, 0.641563, -0.593441, 0.349911, 1.467353, -0.416542, 0.181919, 1.109405, 0.572303, -1.303945, -0.646967, 0.209462, -0.842975, -0.257966, -2.417012, -0.791536, -0.290090, 0.292801, 1.390567, -1.009574, -0.279493, -0.327589, 2.143242},{-1.388449, -0.649815, 0.139786, -0.024399, 0.729268, 2.179479, -0.587929, -1.542645, 3.180772, 0.117897, -0.583397, -1.980827, 0.243890, 1.546464, 1.575882, 0.024548, 1.440208, -0.702648, -0.443406, -1.627693, 1.537072, 0.164954, -0.947010, 0.355942, 2.184960, -1.154696, 1.416552, 0.249699, 1.460604, 1.512949, -0.418513, 1.129351, -1.042588, -0.223334, 0.068521, -0.439098, 0.742730, -1.106984, 0.886004, 0.531294, 0.087667, 0.182180, 0.510536, -0.960042, -1.614848, 0.956725, 0.097642, -1.858288, 1.577098, -0.073655, -0.385170, 1.010619, 1.457272, -2.089652, -1.095982, -0.524885, -0.078929, -1.174794, -1.216214, 1.205479, -0.213917, 1.109484, 0.371574, -0.914168, -0.643835, 0.498351, -0.455435, -1.134421, 0.375483, 0.736476, -0.958185, -0.081481, 0.951574, 0.246102, -0.617804, -0.829805, 0.381813, 0.747829},{-1.086726, 1.278323, 0.525174, -1.675430, -0.051674, 0.007239, -2.185642, 0.873080, -1.149149, -1.693231, 1.927673, -0.388584, 1.797001, 1.236032, 0.386885, -1.109162, 0.086201, 0.984596, 0.046264, 1.413607, -0.127026, 1.499471, 0.762976, -1.812532, 0.265743, -0.106178, 1.832608, -0.916475, 0.359515, -0.370173, 1.976600, -0.378221, -0.973667, 1.518229, 0.510058, -1.973089, 2.216220, -0.414499, -0.244659, 0.123091, 0.959940, 1.344165, 0.767499, -1.037348, -0.556309, -0.455139, 1.031721, 1.866639, 0.034169, 0.181751, -1.224896, 0.998829, -0.368873, -1.408499, -0.162332, 0.171930, -0.176550, 0.989841, -2.727262, 0.862638, -0.952812, -0.121409, 1.225698, -1.055081, 2.320868, 1.386439, -0.664537, 0.302437, -0.076775, 0.449534, 1.187533, -0.582780, 0.933639, -1.143059, -2.449321, -0.707783, -1.542982, 2.027819},{-2.336223, -1.095431, -0.783105, -1.477693, 0.465903, -0.659703, -0.933606, -0.376190, -0.678645, 0.016205, -1.107969, 1.338711, -0.546223, 0.293745, 0.476064, 1.096702, 1.177171, 0.107118, 1.625208, 1.781959, 0.081373, 0.358977, 1.974622, -0.828493, 1.065920, -0.487181, 1.664798, 0.775558, 1.117007, 0.790145, 0.461146, 0.046804, -0.582351, 0.963087, -0.359241, -1.086241, 1.053139, -2.214384, -0.029294, -0.291854, -0.033703, -1.070766, 0.020927, 1.851185, 1.446897, -2.312512, -0.713001, -1.254751, -0.341461, 1.410653, -0.973899, -0.378428, 0.160960, -1.583412, -1.955058, 0.606220, 0.590938, -1.937754, -1.759228, 0.000019, 0.215393, -0.883637, -1.315889, -0.504333, 1.250856, 0.993763, -1.164432, 0.560826, -0.125043, 0.252307, -0.073218, -0.224182, 1.523182, -1.013346, 0.104320, 0.219632, 0.719546, -1.796013},{-0.651465, 1.000054, -0.162971, -0.557133, -1.615416, -1.252049, 0.181618, 0.363454, -0.735279, -1.210388, -0.679780, -0.165707, -1.652344, 2.496621, -0.786544, -0.946560, -0.224197, -0.651903, -1.702248, -0.686902, -1.133980, 1.508809, 1.050901, -1.051454, -0.680069, 0.526334, 1.793749, 0.564248, 1.119841, -0.282153, -0.682114, -0.914102, -0.619760, -1.814550, -1.433025, -0.410021, -1.635476, 1.479830, -1.416691, 0.953967, 2.359665, -0.533111, 1.524634, -1.130035, 0.810766, -1.596978, 1.031794, 1.454408, 0.692606, -0.828903, -0.165181, -1.491978, -0.499294, 0.951399, -0.255480, 1.517824, 1.102338, 1.458860, -0.762927, -0.709540, 0.876774, 1.220157, -1.034457, -0.170785, 0.023000, -0.407956, -1.921558, 1.161514, -1.796527, 0.062989, -0.117515, 0.640013, -1.747457, -0.102035, 0.463604, -0.013384, -0.689872, -0.162040},{-0.380087, -0.119220, 0.626030, 0.731368, 0.088448, -0.605937, -0.337584, 0.135994, 1.149421, 0.642804, -0.024815, -0.636063, 0.260532, -0.312946, 0.255034, 0.074147, 1.944222, 0.411564, 2.193599, -0.864121, 2.693686, -1.561557, 1.011006, 0.081106, 1.355312, 1.475870, 1.441504, 0.845026, 0.529658, -0.589314, 0.075960, -0.215859, -1.423712, -0.030029, -2.264137, -0.468699, 2.027293, -0.065269, -0.480452, 1.666871, 0.366992, 1.539922, -0.490930, 0.718613, 0.362617, 0.575977, -1.096321, -1.020363, -0.351053, -0.191560, -0.309268, 0.409688, 0.530762, 0.298952, 1.222456, 0.323763, -1.453262, -0.166526, 2.703785, 2.537109, 0.893921, 0.239410, -1.635483, 0.698761, -0.314865, -1.303769, 0.303234, -0.627721, -1.726426, 1.605291, -0.206985, -0.366129, -0.200529, 0.590384, 1.107011, 1.204066, 1.681798, 0.013273},{0.562534, -0.113191, 0.268751, -1.586150, -0.633701, -0.549797, -0.771864, 1.057825, 0.482430, 1.125212, -1.683674, -0.248811, 0.389645, 0.261787, -0.465228, 0.483324, 1.688219, -0.238058, 0.814662, 0.033029, -0.008098, 1.320143, -2.495665, 1.785411, 2.065431, 0.667112, -0.314256, 1.835153, -0.944365, 0.610847, -0.714608, 0.997081, 0.976035, 0.924444, -0.598708, 0.021826, 1.102210, 1.062940, 0.082304, 0.818288, 0.097190, 0.499911, 1.310558, -0.674167, -0.622150, -0.919215, 0.226065, -0.151454, 0.984759, -0.178880, -0.665575, 0.661515, -1.025327, -1.268450, 1.072918, 0.179196, -0.669803, -0.212341, -0.844627, 0.518194, -1.358420, 1.314538, -0.339190, 0.240055, 0.648364, 1.202963, -0.444717, -2.044660, 0.857573, -0.509420, 0.424088, -0.633130, 0.457002, 0.791181, 0.362690, -1.898440, 0.848856, 0.717218},{-0.530931, 1.487501, 1.977539, 1.354651, 0.669228, -1.507415, -0.108678, -1.084407, -0.328307, -0.274454, 1.819578, -0.396692, -1.173397, -0.337595, -1.373665, -0.050892, 0.450861, 1.106976, -0.993931, -0.274093, -0.721799, -0.431993, 0.824727, 1.134933, 2.153073, -1.533377, -0.036355, 1.806713, 0.669233, -0.344286, -0.882197, 0.563835, 1.170194, -1.016818, 0.068381, -1.651447, -0.859858, 0.030437, 0.446272, 0.627519, 1.193263, -2.472180, 1.139099, -1.206327, -0.158266, 0.119542, 1.473175, -0.715890, -0.521447, 0.969408, 0.009932, 1.063258, -0.317398, -0.696300, 0.234515, -1.814494, -0.662051, -0.613043, -1.014067, 1.197809, -0.154223, 2.320152, -0.528850, 0.935489, 0.260612, 1.204248, 1.862738, 0.066747, 0.751562, 0.520986, -1.603117, 0.655347, 0.473315, -0.139880, -0.645233, 0.674219, -0.578110, 0.966640},{1.878705, 0.021428, -1.118703, 0.131432, 0.136835, -1.529620, 2.369238, 0.873255, 1.074603, -0.965431, 0.030541, -0.960835, 0.678074, 0.193588, 0.825057, -1.696674, 0.118311, 0.198621, 0.641980, 1.158970, 0.413092, -0.728298, -0.339069, -0.326128, 0.664674, -1.611065, 2.246924, -0.700514, -0.449834, 0.197641, -0.209801, -0.627984, 1.333814, -0.469627, 0.796824, 0.495433, 0.526076, -1.196602, 1.625786, 0.976921, 0.336420, -1.098342, -0.583606, -1.975886, -1.244373, -0.172282, 0.695662, 1.139519, -1.157931, -0.085181, 1.030122, 0.241600, -1.165557, 0.653504, -0.799522, 0.094380, -0.652569, 0.294986, -0.483254, -0.325737, 0.022317, -0.025522, -1.106698, 1.883102, -0.192676, -0.111002, -1.081006, -0.242486, 0.129529, 0.473163, 0.843677, 1.331702, 1.337662, -0.503859, -1.100853, -0.379255, -0.081658, 0.025889},{0.560090, 0.753726, -0.127908, 0.029698, 0.709030, -0.658212, -0.309401, -0.385258, 0.035248, -1.109628, -0.312232, -0.106266, -0.964359, 0.766192, 0.427141, -0.408107, -0.771498, -0.253951, 0.333478, -1.008890, 1.553975, -0.846173, -0.354143, 1.027945, 0.733257, 0.183596, 0.445503, 0.947560, -0.511315, -0.581207, 0.409651, 0.695824, -0.444668, -0.743503, -0.204292, 0.438002, 0.442053, -0.233103, 1.809144, -1.363603, 1.626121, -0.490124, 0.442368, -0.624420, -0.227342, -0.336274, -1.346778, 0.739896, -0.331371, 0.911027, -0.853827, -0.723549, -0.308367, 2.068305, -0.071428, -0.167095, -0.044295, 0.923128, 0.599410, -0.786653, 0.364344, 0.280333, -0.391608, -1.097523, -0.055933, -0.085578, -0.809561, 0.774926, -0.580647, -1.572024, -0.091358, -0.620810, 0.546163, -2.401483, 0.095035, -0.322281, 0.845328, -0.306385},{0.742117, 0.044288, 0.469010, 0.791329, -0.542366, -0.804165, 0.227915, 1.618699, -1.120546, -2.183094, -0.276619, -0.892810, -2.067345, -0.060786, -0.809083, -0.629085, -2.959774, -0.858364, 0.287964, 0.549230, -0.018204, -2.257949, 0.041712, 0.996886, -0.808240, 0.676644, 0.921105, -0.992227, -0.480750, -1.028289, 1.584474, 0.105185, -1.055134, -0.542587, 0.092985, -0.039283, 0.980700, -0.383085, 0.926645, 0.178688, 1.433954, 0.137927, -0.392467, 1.899258, -0.876090, 0.344425, 0.020018, 1.086408, -0.541975, -0.179971, -0.486551, -0.838436, 0.351812, 1.173615, -0.435972, -1.434740, 0.426168, 0.762130, 0.429818, 2.667169, 1.447899, 0.041802, -0.168708, -0.046081, 0.086109, -0.232173, -0.522561, 0.948996, -0.526753, 0.149503, 0.610924, 0.171095, -0.183644, -0.025402, 0.808155, -0.318243, 1.845096, 1.206455},{1.460177, 0.266966, -0.031989, 0.332614, -1.100259, 0.141084, 1.113701, 0.208035, -1.095021, -1.003892, 0.328105, -0.940873, 0.249138, 1.289468, 1.231625, 0.405842, -0.221431, -0.662338, 1.289769, 0.309710, -0.043472, -0.666588, -0.165366, -0.450123, 0.890515, -1.319840, 0.268957, -0.833053, -1.101990, -0.390347, -0.921910, -1.475776, 0.374779, -0.148489, 1.941152, 0.762437, -0.040618, -0.161769, -1.424356, -0.394325, -0.240644, 0.462601, -0.477685, -0.556276, 0.888253, 0.168359, 0.007514, -0.230447, 1.354310, 2.112239, 0.302640, -2.129396, 0.713139, -1.047414, 0.189184, 1.155178, 0.688297, -0.373739, 0.012156, 0.711814, 1.812578, 1.397712, -0.217159, -0.567903, 0.595824, 0.138697, 0.781736, 1.158586, 0.563449, -0.914218, -1.308871, -0.851770, -1.642992, 0.017286, 0.607258, 0.245194, 1.445021, -0.812937},{1.697304, -0.474793, -0.689629, 0.483467, -0.985772, -1.221722, 0.528626, -0.636246, -1.397451, 0.332961, -0.664424, -0.968286, 0.182642, -0.006590, 1.508088, 1.500027, 1.930154, -0.201072, -1.191270, 0.929080, 0.876225, 1.093940, -0.364184, 0.606735, -1.255310, -0.295932, -0.926056, -0.207106, -0.487134, 2.414706, 0.261590, -0.022845, -0.644865, 0.429558, 0.297709, 0.028683, 0.596056, 0.432476, -0.253000, 0.095893, 0.239024, -0.894712, -0.043937, 0.722192, -0.772489, -0.863317, 0.248350, 0.294298, 0.647634, 0.149864, -0.206652, -0.595008, 1.806334, -0.132177, -0.107083, -2.043530, 0.583840, -0.099846, 0.714495, 0.019393, 0.403262, -0.931063, 0.169026, 0.049977, -0.034266, -0.529015, -0.326307, 1.490068, 0.865283, -0.763381, 0.315269, 1.034792, 1.396784, 0.043375, -0.341349, 0.320435, 0.594176, -0.859729},{1.468161, 0.987143, 0.543026, 0.979976, 1.274330, 1.499822, -0.218865, 1.012030, -0.075879, -0.092718, 1.321145, -0.707341, -0.475874, -1.574529, -1.696398, 0.152546, -0.738253, 0.692553, 0.156155, 0.358410, 2.360801, -0.033401, 0.407897, 2.957183, 0.922980, 1.188327, 0.147920, -0.629228, 0.181642, 0.632613, 0.702000, -0.537536, -0.036105, -0.529250, -1.723928, -0.076053, -1.903099, 2.206602, -0.557129, -1.439872, 1.257998, -0.379160, 1.331230, 0.672905, 0.883378, 1.022387, 0.067446, -1.678450, -0.439941, -1.164464, 0.062196, 0.139631, -0.518858, -0.825414, -1.525812, -0.873638, -0.017753, -0.395825, -0.231334, -1.018888, -0.008422, -0.156571, 1.227143, -0.011651, -0.206232, -0.508023, 0.229060, -1.116939, 1.469232, 0.424740, 0.005199, 0.299859, 0.149099, -1.345777, -0.264193, 0.105547, 0.966603, 1.008697},{0.802966, -0.156540, -1.520927, 1.507572, 0.485722, -0.622210, -0.065622, -0.218491, -1.157166, -0.575784, -0.562475, 0.820130, -0.170250, -0.829066, 0.245311, -1.544056, -0.596599, -0.623356, 0.313560, -0.987935, 0.728388, 0.521084, 0.876047, -1.150387, 0.112615, -1.083338, -1.528704, 1.487664, 0.166530, 1.056779, -2.159382, -0.085207, 0.208826, -1.045041, -0.654426, -0.368288, 0.644279, 0.699974, 0.529470, 1.141511, 1.231642, -0.781984, -0.425477, -0.074948, -0.112216, -0.151612, -1.006309, 1.216424, 1.049597, -1.311527, 0.057442, 0.396790, -0.737344, 1.377572, -0.217347, -0.765348, -0.925792, 1.275847, -0.579960, -2.212918, -0.633793, -1.533560, 0.609071, -0.098419, -1.893668, -1.339297, -2.129130, 0.577650, 1.475615, -0.442035, 0.082654, -0.061340, -0.994142, -0.297640, 1.004510, 1.551189, -2.473145, 2.027274},{0.612983, -0.362293, 0.685175, -0.204867, -0.435518, 1.583893, -0.504077, -0.003846, 0.912580, 0.192831, -1.667491, -0.658288, -1.879907, 0.092793, -1.548470, -0.149652, -0.411123, 1.579788, -1.114146, -0.175098, -0.012935, -0.445440, 1.163241, 1.739319, -0.568773, -0.638051, 1.126375, 2.655287, 1.128082, 0.326971, 0.021788, -1.269269, -0.323844, 0.567383, 1.109066, -0.779289, 0.478620, 0.672421, 1.280079, -0.375234, 0.924073, -0.802714, -0.338064, 0.046407, -0.161264, 0.393268, 0.716361, -1.127240, -0.242637, 0.151018, -1.902391, -1.222731, 0.069374, -0.563952, 0.931096, 0.976476, -0.551493, 0.079836, -0.520590, -0.837570, 1.127874, 1.287040, -1.520048, 1.434924, -0.844943, -0.490615, -0.405142, 0.001959, 1.353834, -0.522660, -0.863875, 0.608260, -0.874132, 0.241919, -1.361974, 1.418122, 0.087220, 0.915397},{0.598408, 0.081761, -0.375708, 1.229054, 0.467169, -0.963210, -1.759974, 0.477259, -0.017232, -0.833241, -0.641005, 0.049376, -1.288963, 0.703960, 0.928769, 1.003386, -0.529602, 0.085083, -0.881345, -1.101382, 0.327754, -1.137538, -1.417475, -0.696384, -0.481625, 0.264611, 2.022722, 0.143593, 1.711784, -1.340775, -0.487478, 0.672692, -0.117157, 0.446202, -0.722230, -0.385058, -1.426463, 0.024007, -0.285211, -1.350298, -0.953169, -0.228461, -1.750620, -0.117977, -0.813063, -0.503455, 0.618250, 0.670440, -0.057424, 0.113560, 0.343843, 0.053638, -0.158571, -1.286322, -0.327064, 1.480559, -1.058980, -0.596384, -0.754496, 0.032483, -3.315621, 0.881229, 0.584587, 0.083511, -0.921826, 1.020759, 0.619574, -1.088923, -0.642469, 1.661809, -0.647689, -0.464633, 0.953683, -1.586113, -0.028937, -1.369382, 1.294950, 0.561305},{0.802757, -1.761840, 0.097676, 0.644870, 0.531544, 1.350331, 0.511636, -0.747550, -0.554046, 0.230407, 0.567193, 3.351428, 1.778528, -1.349818, -0.326719, 0.689882, -0.853800, -0.484965, 1.222829, 0.037342, -0.167388, 0.836398, -0.357326, 0.478730, -0.028759, -2.214870, -0.116091, -0.803657, -0.478229, 1.253977, -0.690444, 0.090478, 0.448554, -0.501408, 1.891313, -0.812037, -1.721133, 0.172455, 0.552506, -0.907170, -0.466593, -0.555206, 0.284250, -0.625390, 0.399701, 0.777744, -0.149895, 0.650203, 0.094668, -0.528305, -0.718363, 0.639533, 0.157982, -0.483284, -1.342524, -0.436230, -0.022765, -1.826964, -0.501836, -1.962042, -0.048587, -0.498664, 1.209649, 0.803625, 0.522823, -0.247898, 0.061201, -0.239828, 0.484992, 0.002694, 1.276111, -0.717354, 0.459794, 0.107825, 0.489195, -0.851858, 1.209729, 0.841132},{-1.036494, 1.471582, -1.419399, 0.607703, -0.512299, 2.288408, 0.765589, 0.715314, -0.414233, -0.229124, -1.311441, 0.532958, -1.321730, 0.585370, -0.208847, -0.815975, -2.206497, -1.757904, -0.727766, -0.092559, -1.594771, -0.482577, -0.439149, 1.063080, 1.095131, 0.864240, 1.522500, 0.988437, -0.821072, -0.406002, -1.254981, 0.627079, 1.005455, 0.585354, -0.318751, -0.384419, -0.304769, -1.039975, 0.454919, 0.988599, 0.145173, -1.430639, -3.102873, -0.341607, -0.378817, -1.733476, 0.104722, 1.420707, -0.057335, 1.026917, -1.371172, 0.129267, 0.598849, -1.357819, -0.448245, 0.345939, -0.348466, 0.042078, 0.625062, 2.819724, 0.994940, 0.852171, 0.157258, -1.686915, -0.447189, 0.978521, -2.731044, 0.251796, -0.410184, 2.633145, -0.178276, 0.112434, -0.444268, 1.975796, -1.419948, 2.956310, 0.641943, -0.179895},{-1.564825, -1.491868, 2.407343, 1.309092, 0.202358, 0.722608, -0.672977, -0.812150, 0.602498, -1.584308, -1.223232, 1.256397, 0.876945, 0.192826, -0.264766, 1.162788, 0.440426, 0.593139, -0.178707, 0.285526, -0.739843, 1.503205, 0.224928, 0.250074, -0.616659, 0.259491, -0.464402, 1.769304, -0.756271, -0.083149, 1.643697, 1.051511, 0.133198, -0.508798, -0.899578, 0.233829, -0.210875, -1.821173, -0.762498, 1.369069, 0.886168, -0.209245, 1.028837, 0.616485, -0.464320, -0.593548, 0.865851, -0.376041, -0.501202, 0.255869, 0.955366, 1.417868, -0.205895, -0.796900, 1.278409, 0.159134, 0.358095, 0.117448, -1.221259, -0.753593, -1.038123, -0.100344, -0.080854, 0.493591, 0.928285, -1.422524, 0.294375, 1.259106, -1.246004, -1.973779, 0.577172, 1.092146, -1.381303, 0.079538, -0.136173, 0.737387, -0.374147, 0.350140},{0.229929, 0.355669, 0.799341, -0.365543, 0.574005, 0.482542, -0.638037, -2.196945, -0.715178, 1.393705, -0.157336, -0.028707, 1.417371, -0.136352, 0.692375, 0.040315, 0.406824, -0.704194, -0.087662, 0.900444, 1.094483, -0.810606, 0.651309, 1.276368, -0.604324, -1.809465, 0.322727, -0.561905, -1.147319, -1.454523, -1.140557, 1.011377, -0.244762, -0.528611, -0.016165, -0.167891, -1.762847, 0.325091, 0.844904, 0.674975, -0.661821, -0.101997, 1.386276, 1.101139, -0.407658, -0.185698, 0.228935, 0.753469, -0.883317, -0.044088, -1.907526, -1.117125, -0.905495, -0.793564, -0.993203, 0.642839, -0.868200, 1.088096, 0.770310, 0.082318, -0.826214, 1.154149, 0.622245, 0.185361, -1.215343, -1.531918, -0.544377, 0.622564, 0.298212, 0.387421, -0.232415, 0.982622, 0.174233, -0.794600, 0.134973, -0.419004, 2.203532, -1.359937},{-1.236308, 1.068216, 0.206184, 0.020340, 1.143406, 0.172264, 1.069438, -0.098891, 0.428037, -0.089990, -1.155658, 0.793249, -0.486786, -1.546327, -0.287856, 0.346119, 0.486253, 0.583334, 0.578415, -0.589161, 0.102277, 1.474931, -0.220232, 0.880843, -0.366356, -0.637268, -1.354219, 1.457648, 0.156985, -1.793699, 0.381605, 0.668636, 0.822097, -1.544334, -1.158761, -1.299827, -0.340779, 0.526868, -1.053561, -0.969410, -0.038305, 0.080592, -1.590455, 0.885370, -0.688770, -1.639775, -0.473865, 1.221552, -1.602216, 1.113423, -0.177671, -1.236542, -0.819913, -0.276551, -0.672162, -0.109806, 1.504845, 0.566346, 0.080096, -1.162518, -1.029259, -0.423493, -0.029851, -0.341131, 1.027626, -1.101353, 0.224763, -1.613362, 0.143255, 1.505751, 1.661357, 0.223599, 0.163925, 1.894215, -0.816765, -1.891611, -1.141144, 0.461890},{-3.079247, 1.322234, 0.875092, -1.059464, -0.147799, 1.414090, 0.201354, -0.176847, -0.291818, 0.316921, -0.362421, -0.072607, 1.235749, -0.939513, -0.577986, -1.397358, 0.289325, 0.391423, -1.204957, -0.645057, -0.683636, -0.456941, -0.288603, -0.267033, -0.533298, 2.157072, 1.486590, 0.707017, -1.383387, -0.622489, 0.492122, -0.665904, -0.864494, 0.421203, -0.290778, -0.491043, 0.715628, -1.304419, -0.595898, 0.423269, 2.096546, -0.961690, 0.421278, -0.159906, -0.684402, -0.178305, 0.121562, -0.273049, -1.280184, 0.552823, 0.658929, -0.509465, 0.263201, -0.033463, -0.304601, -0.231756, 1.750443, -0.583517, 0.821855, 1.929324, 0.483732, 0.156402, -0.049324, 0.085695, 0.210917, -1.533971, -0.837750, -0.364746, -0.817376, -0.850520, -1.078728, 0.860650, -0.712673, -0.812606, 1.704990, -0.623903, -0.094058, 0.252786},{-0.077000, 1.566860, 0.671462, 1.603477, 0.436665, 0.340073, -0.340994, -0.006153, 1.973477, -0.348484, 0.331403, -1.422966, -0.429110, 0.668780, -0.052909, 0.307515, -1.270468, -0.179842, -0.334494, -1.117638, 0.140029, 0.482086, -0.313642, 0.802863, 0.380703, 0.918985, 1.421371, -0.352661, 0.078957, 2.479308, 0.678175, 0.840569, -0.181128, -1.364531, -0.673532, 0.825883, 0.345159, 0.400575, -0.431696, 1.396823, 0.884800, -1.048241, -0.636818, 0.683276, 2.120195, 0.154850, 1.531137, 1.974005, 0.930517, 1.629437, -0.237754, -2.132235, 0.113258, 0.003689, -0.562215, 2.256354, -1.423666, 2.625378, -0.777875, 0.578061, 1.903779, -1.870601, 0.281140, 1.128917, -1.100901, 0.514550, -0.948594, -0.557754, -1.939680, 1.045392, -0.616927, -0.523348, -0.594238, -0.860639, 0.812573, 0.389623, -0.630034, 1.580064},{1.419733, 2.559364, 0.991391, -0.326614, -1.425697, -0.150315, -1.762851, -0.743962, 0.457238, 0.037959, 0.363113, -0.585791, 0.723742, -0.659791, -0.460781, -0.694607, -0.767632, -0.235405, -0.372335, -0.564874, -0.928109, 0.030132, 1.266251, -0.903283, 0.501118, 0.827676, 0.596586, -1.585668, 0.448388, 0.559148, 0.617064, -0.710017, -0.470719, -1.807984, 0.303370, 0.011363, 1.007002, 0.666973, 0.493145, 0.582719, 0.117085, -1.248098, -0.335931, 1.312149, -0.597530, -0.160626, 0.494804, 0.244198, 1.233184, 1.216237, 0.757030, 0.362462, -0.448902, 0.490668, 0.580945, 1.029880, 0.798441, -0.568825, 0.265296, 1.165807, 1.471326, 1.390423, -0.525670, -1.166594, 0.222834, -0.321439, -0.704858, 0.929579, 0.640294, -1.315580, -1.287278, -0.026279, -0.818759, 1.099115, 0.268978, 0.664573, 0.949154, 2.289402},{-0.172279, 1.615744, -0.625089, -0.101621, 0.919132, -0.295449, 0.067899, 1.720737, -1.167279, -0.228451, 0.634748, -1.493925, -0.358772, 0.318758, 0.337567, -0.887744, -0.732265, 0.799767, -0.106037, -0.788645, 1.045720, -1.061068, -0.112606, -2.096582, 0.027947, -0.638468, -0.339600, 0.423160, 1.151892, -1.815587, -1.468502, -1.147673, -0.555226, -3.007455, 0.647137, -0.359914, -1.727920, -0.232510, -0.058736, -0.540449, -0.971923, -2.146097, -1.645414, -0.616258, -2.084405, -0.972089, -1.228633, -1.139185, 1.019529, -1.138351, 1.308534, 0.213397, 0.393043, -0.287846, 0.539291, -1.202294, 1.051889, -0.987626, 0.434220, 0.440629, 0.583908, 0.648910, 0.257388, -0.450609, -1.060536, -0.440100, -1.072344, 0.416976, -1.876073, 1.208601, -1.008430, 0.464669, 0.032471, 0.321564, -0.202761, 0.491180, 0.703162, -1.184145},{-0.401228, -0.012197, -0.226118, 0.975946, 1.424911, -0.146267, 0.969978, -1.058743, -1.304745, 0.607704, -0.107989, 0.619894, 0.048193, -0.979194, -0.094553, 1.542139, 1.196525, 1.638880, 1.762154, -0.158823, -1.844131, 0.766841, -1.949473, -0.901258, 0.593899, 1.665699, -0.625891, 1.063375, -0.857551, 0.625290, 0.060446, -0.402681, -0.487044, 1.209638, 0.689327, 0.763370, 0.275537, -0.375146, 0.299570, -0.295887, -0.186231, -0.463044, -1.419608, 1.349478, -0.701055, -0.009059, -0.748376, 0.185691, -0.117867, -0.854989, -0.762364, -1.263677, 0.564779, -1.346942, 0.928535, 0.430629, 1.665257, -0.547969, 0.065742, -0.031661, 0.221521, -0.651521, 0.437444, 0.465676, -0.675081, -0.267751, 1.108004, 0.284762, 1.823219, -1.152886, 2.478243, 0.040795, 0.067937, -0.706190, 0.739744, -2.346213, -0.241842, 1.836558},{-0.733389, -1.120508, -0.367148, 0.627801, -0.952825, 0.766739, -0.476345, 0.261735, -1.009423, -0.080065, -0.238682, 0.060913, 0.525377, -0.500409, 0.065401, 0.052497, -0.785451, 0.280433, -0.461635, -0.601548, 0.037011, -0.375058, -0.148416, 2.574213, 0.547825, 1.915236, -0.719301, -0.451950, 0.214270, -0.317193, 2.419090, 0.677681, -0.112813, -0.378102, -2.150839, 0.158353, -0.176460, -0.391684, -0.673316, 0.786145, 0.209323, -1.036105, 1.046926, -0.088996, -0.292310, 0.244870, 0.402176, -0.463785, -0.009013, 0.013106, 0.542657, -2.139457, -0.929422, -0.072831, 1.113198, 0.884526, 0.414035, 2.273099, 0.802475, 0.729441, -0.686219, 1.470560, 0.169766, 0.441350, 0.119445, 1.552475, 1.824298, 0.213251, 2.023132, 0.748294, -0.464876, -0.866097, -0.360199, 0.665685, -1.598996, -0.106673, -1.478418, -0.703468},{-0.768478, -0.167850, -0.927047, 0.897606, 1.356967, 1.726519, 0.610032, 0.722886, -0.650343, -0.698569, 0.406090, 0.701889, 0.190222, -0.720698, 0.745339, -0.300359, -0.064805, 0.445035, 0.673243, -1.565871, -1.658108, -0.333467, 1.074066, 1.337577, 0.498953, 0.107563, -0.386598, 1.739915, 0.546314, -0.584660, 0.989475, 0.097307, -2.176813, 0.149349, 0.973567, -0.071651, -0.524436, -1.580913, 1.695143, 0.783425, 1.700949, 0.932981, -1.207551, 1.268567, -1.825830, 1.605699, 0.857867, -1.998546, -0.733365, 0.602107, -0.562691, -0.145486, 0.429639, -0.584980, 2.003545, 0.266677, 0.242025, 0.593219, 1.918375, -2.284202, -0.302560, -0.984835, 0.444083, 1.506720, 0.721874, 1.540537, -0.461445, -0.563549, 0.383965, 2.209047, -0.223314, 3.494403, -0.207545, -0.436617, -1.338144, -0.258145, 0.911521, -0.289460},{-0.119010, 0.412665, 0.487914, 0.262464, -0.069710, 1.601859, 0.589146, -1.645633, 1.420221, -0.097388, 0.016855, 1.115183, -0.217637, 0.255755, -0.423265, 1.539189, 1.260776, 1.901860, -1.569771, 1.521610, 0.202600, -1.036671, 0.343159, 0.110945, 1.525194, -0.275152, 1.953863, 0.629868, -1.143837, 1.325683, 0.114433, 0.543981, 1.366695, 0.406725, -0.492635, 1.864171, 0.358407, -0.886687, 2.249894, 1.000859, -0.226891, 0.934668, 1.236510, -2.053174, -0.433598, 0.635581, 0.680735, -1.358209, 1.064831, -0.250398, 0.628745, -0.518505, 0.416517, 0.074531, -1.368495, -0.411221, 0.526091, -0.911633, 0.640057, 1.036204, 2.035635, 1.104520, -0.515564, 1.209240, -1.307337, -0.947297, -1.122375, -1.067807, 0.495385, 1.579865, 0.214599, 0.521223, -0.174222, -2.016532, 0.957699, 1.119302, 0.405113, 0.068040},{-0.461703, 1.106553, 1.471300, -0.363640, -1.104908, 1.560982, 0.102352, 1.896277, 0.659205, -1.579346, 0.247880, 0.177234, -2.265111, -0.147715, -1.502517, 0.324393, 0.700979, 0.113605, -2.127146, 0.114626, -0.157542, -0.939752, -0.238833, -0.483525, -0.214802, -1.597337, -0.330654, -1.872742, -0.821237, 1.145899, 0.154414, 0.839778, 1.494808, 1.548908, 1.320830, 0.097656, 0.002124, -0.650261, 0.365343, 0.546919, -0.628387, -1.307876, -2.797100, 0.593329, 0.185505, -1.150681, 0.417914, -0.817402, 1.375477, -1.479501, 1.157801, 0.786664, -0.512203, 0.476582, 1.756083, 0.805863, -0.519696, 0.321667, -0.065397, 0.678229, 0.518226, 0.546078, -1.318087, -2.842883, 1.591030, -0.887482, -0.715443, 1.045695, 0.630355, -0.753590, 0.693788, 1.472685, -0.004034, -1.620763, 0.178015, 0.317395, -1.387596, 1.218247},{0.629350, -2.288898, 0.874204, 1.190219, 0.379746, -0.983707, -1.223850, -0.613593, -0.184313, 0.056648, -0.998814, -0.649009, 0.300604, 1.718567, 0.261646, -0.855090, -0.282377, -2.395235, -0.122162, -0.616011, 0.573304, -0.163113, 0.641171, 0.097663, 0.118604, -0.103984, 0.757072, 1.108047, -1.307164, 0.131540, -0.394061, 1.394844, 0.382754, -1.537438, 0.993753, -0.118328, 0.145281, -0.447765, 1.413604, -2.841432, -0.226497, 0.171779, -0.157910, 1.874409, -1.425035, -2.264284, 0.400675, 1.076222, 1.231075, 0.305860, 0.703908, -1.873410, 2.210049, -0.314967, 2.954761, -0.577087, -0.590299, -1.198352, -0.895647, 0.885273, -0.890971, -0.095126, 0.788168, 0.842324, -0.463875, -0.769516, 2.592687, -1.064635, 0.445886, 1.312954, 0.477182, 0.925971, -0.817662, 1.045199, 0.521626, 0.855318, -1.894650, -0.525036},{1.420367, 0.797208, -0.040773, -0.183522, 1.078639, 0.648292, -0.127691, -0.042340, 0.952969, -0.970178, 0.965384, 1.421101, -0.216126, -0.576299, 0.233306, 1.114279, 1.563343, -0.415891, 1.067467, -1.666367, 0.474310, -1.872059, -0.574479, -0.291919, 0.357092, -0.333471, 0.085834, 0.263806, -0.126923, -0.530521, -1.000048, 1.226138, -1.567978, -1.510554, -0.923392, 0.280704, 0.335427, -0.171475, 1.061077, -2.154268, -0.193223, 0.364806, 1.848661, -0.132460, 1.008968, -0.351227, -0.212577, -0.027767, -0.639024, -2.125488, -1.940857, -1.720340, 1.187624, -0.661358, 0.047616, 1.689924, -0.769296, 0.030325, 0.601078, 0.644731, -0.738164, 0.372693, -0.364575, -0.418550, -0.530334, 0.450484, -0.438018, -1.052906, 1.170137, -1.800290, 0.885876, -0.724140, 1.168037, 0.707972, 1.350498, 0.746410, -0.364002, -2.015983},{0.576853, -0.740597, -1.395803, -0.682810, -1.387244, -0.920130, -0.532645, 0.465282, 1.208274, 1.596827, 0.835850, -1.092513, -1.190253, 0.913060, 0.592364, -0.339028, -0.252481, -0.635361, 0.471487, -0.137230, 0.226333, -0.591113, 0.093737, 1.802883, -0.064510, 1.103691, -0.124813, 0.755519, -0.974369, -2.229091, 0.191076, -0.320503, -0.961966, 0.180281, -1.399963, 0.555810, -0.131233, -0.878879, 0.211745, 1.641476, -0.360007, 0.967006, -0.184280, -2.843629, -0.501940, -1.596613, 0.071077, 0.461322, 0.944802, -1.002722, 0.263025, 0.664790, 0.192004, 0.650180, 0.119138, -0.874320, 0.689513, 0.685924, -0.595889, 0.936360, -1.647913, -0.371723, -0.765879, -0.212494, -1.463527, 1.654586, 0.235638, -0.787399, -1.521096, 0.137223, -0.120820, -1.175439, 1.189362, -0.641019, -0.969509, 1.613221, 0.398210, 0.087715},{-0.398316, 1.783350, -0.705300, 0.026049, -0.739275, -0.250999, 0.994094, 0.970875, 1.163913, -0.214951, -0.558074, -1.151117, -1.140371, 0.312157, 0.954551, -1.157469, -0.949177, -0.486859, 2.064636, -0.036284, -1.185627, 1.074592, -0.145383, -0.310821, -1.378773, -0.909686, 0.163442, 0.945161, 0.598005, -0.907886, 0.459615, -1.354731, -0.894257, -0.486331, -0.139191, -0.040643, 0.698752, 0.204206, 0.801217, -2.217935, 0.708938, 0.353188, -0.379991, 1.488595, -1.262344, -0.409396, 1.357709, -0.203343, -0.273210, 3.571077, -0.988936, -1.119185, -0.257182, 0.124484, 1.970180, -0.236198, -0.522922, 1.789178, 1.866771, -1.553303, -1.268648, -0.085343, -0.493102, 1.474023, -1.612300, -0.694754, -0.681076, -1.386237, -0.729618, -0.133644, -0.732520, 0.378148, -1.396848, -0.283871, 0.087090, 1.532198, -0.432468, -0.098870},{1.050936, 0.464139, -1.057098, -0.814941, 1.748725, -0.450005, -0.224795, 1.367319, 0.501642, -0.223060, 0.197313, -0.902769, -1.092255, 0.082399, 1.486455, -0.167728, 0.846161, 2.094441, -2.165912, -0.652893, -0.175574, 0.679889, -0.003609, 1.442631, -0.337018, 1.266438, 0.333264, 0.490875, 0.425331, 0.590684, -1.423153, -1.539249, 1.531845, 0.934236, 0.816903, -1.370578, -0.736492, 0.826665, 0.427702, 2.348495, -0.597424, 1.648961, 0.843914, 0.746285, -0.071451, -0.760411, 0.760081, 1.127643, -2.256376, 0.314838, -0.542852, 0.178048, -1.806584, 0.279579, 0.909069, -0.508123, -1.125661, 0.855384, 0.763273, 1.037089, -1.273860, -0.879409, -0.880845, -0.264735, -1.676216, -1.036302, 0.434890, 1.827864, 0.265450, 0.094307, 1.684061, 2.458773, 0.112636, 0.211547, 0.641617, -0.664471, 1.535505, 1.310213},{-1.590811, -2.086049, 2.022325, -1.561106, -0.855760, 0.650676, -0.793047, 0.635680, 0.981903, 0.569716, 0.810534, -1.163917, -0.781119, 0.030365, -3.604771, 1.134899, 0.311823, -0.712075, 0.085152, 0.445703, -0.077952, 0.270270, 0.110295, 0.289489, -0.163692, -0.409651, -0.305335, -0.308451, 0.893253, -0.630143, 0.261206, 0.457832, -0.889794, 1.274084, -0.060796, -0.148941, -0.389668, 1.538949, 0.590460, -1.889038, -0.085475, -0.426798, -0.209976, 0.511890, -0.061244, 1.379100, -0.207272, 0.236629, -0.867738, -0.310811, 1.142001, 0.125109, -1.013792, -0.178183, 0.579174, -0.578663, 0.458414, 0.983947, 1.035761, 1.148166, -0.404663, -2.056823, -0.057927, -1.200345, 0.555449, 0.423630, -1.336274, -0.628244, 0.076198, -1.347778, 0.615205, 0.264017, -0.154977, 1.302679, -0.530088, 1.157639, 1.608847, -0.800940},{0.518327, 0.865738, -1.799632, 1.360230, -0.634909, -0.230398, -0.118641, 0.194372, 1.636856, -1.498551, 2.410008, 0.677776, 0.338279, -2.495456, 0.147655, -1.359721, 1.205907, -2.315812, 0.607750, 2.532425, -1.782978, 0.703153, 0.270111, -0.965636, -0.244579, -0.550378, -0.793588, -1.405600, 0.862276, -1.796511, -1.510016, 1.100241, 0.301621, 0.800322, 2.240543, 1.380424, 0.054572, 0.631599, -0.536179, 0.312936, -2.692465, -0.492293, -0.064218, -1.444554, -0.593635, 0.599293, 1.226234, -0.611592, -1.330026, -0.003709, 1.381240, 0.852800, 0.273661, 1.518361, 0.289047, -0.246453, -0.327040, -1.046847, -0.428986, -0.517821, -1.448707, 1.257862, 0.067645, 1.093870, 0.671662, 0.232389, -0.492787, -1.421025, -0.324858, -0.282311, 0.138143, -0.735559, -1.200804, 1.586368, -0.059631, -0.568343, -0.689303, -0.043090},{0.540431, 0.506867, 1.635820, 0.030542, -0.391034, -0.854048, 0.370436, 0.238107, 0.766386, 0.060807, 0.533620, 0.726087, -0.267894, -0.022689, 1.136232, 0.791444, -1.197573, -1.596139, 0.453203, 0.158583, -0.244599, 0.373000, -0.194723, 2.214879, -0.379280, 0.940092, -0.785936, 0.782419, 0.278884, -0.107065, 1.362121, -0.076154, 1.063547, -0.547133, -0.903757, -0.255643, -1.373437, 0.399797, -0.639683, 0.923051, -2.309619, -1.182976, 3.102244, -0.171663, -0.300253, 1.136943, -2.102564, -0.041187, -0.996397, 1.500116, 0.422290, 0.762740, -0.799948, 0.725715, 0.871538, 1.026088, -0.963450, -0.243067, 0.429311, 1.421199, 0.746644, -1.346173, 1.179798, -0.697311, 1.581731, 0.321080, -0.494275, -0.499376, -0.506420, -0.419101, -0.844500, 0.930598, 0.807373, -0.654919, 1.284318, 0.950767, 0.721668, -1.556073},{-0.825920, 0.229248, 0.853817, 0.851860, 0.992005, 0.657160, 0.683119, 0.986037, -0.607381, -0.513115, 1.512265, 1.467999, -0.948263, 0.402575, -0.696999, -0.505828, -1.490498, 1.541133, 0.615747, -0.216362, 0.808046, 0.889706, -1.975712, 0.636142, -1.453420, -0.955735, 0.521845, -0.224569, 2.543887, -1.443287, 0.460929, -1.746609, -1.658416, 1.114912, 1.737228, 0.011521, 0.780379, -0.692630, -1.958709, -0.702574, 2.372437, -2.045605, -1.691567, -0.210793, 2.192462, -0.451878, -0.694140, 2.011850, 0.160959, 0.286235, 0.362780, -1.926772, 2.076404, -0.947575, 1.029003, 1.225742, 0.302811, 0.037491, 0.622449, 1.409151, 0.467245, -0.050329, 1.219895, -0.142682, 0.917890, 0.266032, -0.107234, 0.364743, 0.794097, 0.156517, 0.837850, 0.126019, -0.226482, -0.328487, 0.920397, -0.269492, 0.114822, 1.990543},{-0.508053, -0.678576, 0.574469, 0.843724, 0.055367, 3.384059, -0.232406, -0.879656, -0.373195, -0.226264, 0.079177, -1.545771, -0.526754, 0.663148, 1.178499, 0.817509, 0.888977, -1.140273, -0.843346, -1.642738, -0.796001, -1.128298, 1.256731, -2.562520, -0.907000, -0.524778, -0.317955, -0.189212, 0.747877, 0.523156, 1.152164, -0.481489, 0.601401, -0.853051, 0.516441, -0.158915, -0.923559, 0.888719, 0.809200, 0.774075, 0.502132, 0.850569, 2.132159, 0.333946, 0.095208, -0.483034, 0.679018, 1.053916, -1.688118, 1.650211, -1.045412, -1.188456, 1.230701, 1.132028, 1.411636, 0.233210, 0.479721, 0.477916, 2.800933, 0.010363, -0.200707, -1.382675, -0.288728, 0.181793, -0.217769, -0.761943, -0.801088, -1.530851, -1.308183, 0.107366, -1.293972, -0.315000, 0.445903, -0.232399, -0.116284, -0.093871, 0.813215, 0.835340},{-0.056167, -0.741765, 0.474436, -1.480246, -0.961727, -1.029186, -0.437768, -1.047437, -0.759159, -0.806698, 1.621187, -0.246829, 1.286508, 1.623641, 0.723344, -0.003003, 0.368181, -0.917666, -0.863281, -1.011722, 3.513348, 0.101868, -0.385217, 1.306024, 0.420282, 0.820881, -0.447768, 0.561012, -1.243939, 0.844723, -0.162302, -1.525696, 1.306793, 0.924815, 0.366446, -0.841677, -0.289905, 0.827171, -1.019232, 0.892517, -0.951347, 2.360611, 0.345230, -0.791400, 0.173926, -0.808716, 0.223173, 0.377277, -0.708077, 1.028327, 0.581951, -0.531250, 1.414991, -1.501827, 0.450946, -0.807891, -0.418932, 1.575956, 0.468744, -0.935510, 0.838724, -0.111530, -0.518098, -0.656441, 0.452204, -0.602239, -1.208986, 0.347860, -0.170115, -0.119708, -0.160164, 0.611973, -0.256638, 0.526555, 0.097284, -1.496857, 1.690984, -0.794711},{-2.234519, -1.614419, 0.321897, -1.314725, 0.531946, 0.558788, -0.892016, -0.013274, -0.547539, -0.576969, 0.139920, -0.651161, -0.255321, -1.615819, -0.236174, 0.208572, -0.152859, 0.616097, 0.219429, 1.468276, 0.512892, -0.277878, 0.287495, -1.321318, 0.785539, -1.972561, -0.187106, 0.553693, 0.488464, -0.282674, -0.563066, -0.507476, 0.944862, 0.806615, 0.637112, -0.366118, 0.081529, -0.505436, 1.022712, -1.825009, -0.619752, -0.205404, -2.107674, -0.596809, -0.998761, 0.128873, 1.027292, 1.591928, 1.427887, 0.378891, 0.256337, -0.104220, -1.769013, 0.234604, 1.351102, 1.052149, -1.344317, 0.711358, -0.088151, -1.325000, 0.668816, 1.237458, -0.064204, -0.583202, 1.839819, 2.029767, -1.034636, 0.292225, 2.508453, -1.213856, 0.262888, 0.924511, -0.275202, -0.019216, -1.185760, 0.731912, 1.609722, -1.025474},{-1.148252, -0.604502, 0.293414, -1.587396, 2.004241, 0.276592, -0.984772, 0.567703, 0.770737, 0.655192, 1.283909, 0.187413, -0.651754, 1.593228, 0.777764, 0.396980, -0.967787, 0.264753, 1.071231, 1.270864, -0.426702, -0.754190, -0.079566, -0.197263, 0.627513, -1.704062, -1.483124, 1.506672, -1.221923, -0.610863, 0.658876, -0.707120, -0.562954, 0.367504, 1.091969, 1.893045, -0.318581, -0.193444, 0.546706, -0.031840, -1.789905, 0.371704, -0.096840, 0.927898, 0.711164, 0.778555, -0.633189, -0.986186, -0.807408, -0.600100, 1.075460, 0.720629, -1.266887, 0.375210, 1.017880, -0.416979, 0.812043, -2.401490, -1.527097, 0.297047, -1.198876, -1.005524, 0.221516, 0.339920, 0.211132, 0.844894, -0.978309, 0.527053, 1.457716, 0.917095, 0.554116, 0.547002, 0.095201, 0.620301, 0.704965, 1.334951, -2.128293, 0.429052},{0.892577, 0.267456, 0.114700, 0.964016, -0.700330, 0.638682, 0.879696, 0.005283, -0.344412, 0.398878, 1.394722, 0.703307, -0.742754, 0.738454, 1.846313, 1.461107, -0.393658, 1.403435, 1.634311, -0.600149, -0.430618, -1.151033, -0.201814, -0.517488, 1.047245, 0.170204, -1.420997, -3.276457, -0.216410, -0.242470, 0.427401, -0.016406, -2.647296, -1.185967, 0.375044, -1.816280, -1.741018, 0.528446, -0.220596, -1.403128, -0.213238, -1.364113, -0.001457, 1.711590, -0.230624, -1.108180, -0.503361, -0.218765, 0.540686, 0.316883, -1.293972, -0.180155, -1.223482, 2.265802, 1.388482, -0.067304, -0.312499, -1.129539, -0.287854, -0.201548, 0.368832, -0.721261, -3.122008, 1.011824, -0.760187, 0.172152, -1.268948, 1.333549, 0.137003, 0.159417, 0.284295, -0.584248, 0.357609, -0.178364, -0.721228, 0.374142, 0.349390, 0.649773},{1.288758, -0.736775, 0.458669, -0.209747, 0.164134, 0.589838, 0.098049, -0.594039, -0.778506, 0.050949, -0.710564, 0.431822, -0.192446, -2.404336, -3.733970, -1.389040, 2.175924, -1.202511, -2.318622, 0.071586, 0.275295, 0.650488, -0.078962, -0.443827, -0.293498, 0.509617, -0.168533, 0.382395, 0.548865, -0.712471, -1.290657, -2.231806, 1.652223, -1.699342, -0.279670, -0.331109, 0.431537, -2.336437, -0.328156, -1.329616, -0.918294, 0.609590, 0.227640, 0.765076, 1.237451, -0.427971, 0.274374, -0.071824, -0.795550, 0.254525, -0.224523, -0.450860, 0.193185, 0.097178, -0.591754, -1.004019, -0.040897, -0.091400, 0.702856, 0.529260, 2.270248, 1.452587, -0.225067, 1.090945, 0.036018, 1.154460, -0.952630, -0.469283, 0.523395, 1.390872, 1.499427, 1.811157, -0.948343, -0.829710, -0.873411, -2.614155, -1.218086, -0.280396},{0.108325, -0.129878, -1.307276, 0.923794, 0.159223, 1.565545, -1.209172, 0.064590, -0.275830, -0.879287, 0.579408, -0.562914, -0.170274, -3.146101, 0.514197, 1.655485, -1.307798, 0.568301, -0.335896, -0.060686, 1.842079, 0.491621, -0.681738, -0.151393, -0.101544, -0.400764, 2.064399, -1.550979, 1.302121, 1.210042, -1.683347, 0.604879, -0.384203, -0.434818, 0.108323, -0.971970, 2.520114, 0.399872, 0.278867, -0.394757, 0.104836, 0.110890, -0.576593, -0.273330, 0.953145, -0.785442, -1.715336, 1.115119, 0.931999, 0.173434, 0.644382, -0.395796, -0.476361, 1.456639, 0.363577, 1.686427, 2.012766, -0.099094, 0.969284, -1.792196, -2.390140, 0.599771, -0.722820, 0.166857, 1.111235, 1.174489, -2.251334, 2.499971, -0.185323, -0.526401, -1.139900, 0.347767, -1.105121, 0.006863, 1.005477, 0.316598, 1.526873, -0.870387},{-1.469699, 0.698518, 1.264436, -0.139678, -1.415667, -2.836756, 0.870998, 0.710289, 0.144673, 0.122259, -0.443290, 1.218066, -0.658112, 0.139835, -2.716873, 1.314677, 0.048349, -0.485571, -0.222299, -1.326853, 0.670550, -0.319493, 0.550635, 0.872899, 0.360185, 0.396209, 0.121985, -0.721057, -0.621805, 1.340755, -0.261748, -0.801818, -0.286354, 1.627511, -0.433866, 0.278460, -0.423304, 0.378305, -1.060514, -1.032462, 0.077272, 0.744844, -0.821531, 2.414462, -0.048384, -0.851671, -1.158683, 1.562313, 0.109957, 2.590013, 1.164533, -0.851338, -0.558062, 1.041799, 0.437424, 0.619285, 0.682572, -0.018233, -1.195737, 1.221533, 0.780963, 0.958229, 0.551505, 0.097330, 0.614031, 0.485127, 1.930238, -0.801100, -0.652715, 0.920791, -0.198376, -0.734767, 1.197000, -2.012585, 0.188346, 0.466670, 1.234632, -0.246790},{0.990981, 1.047331, 1.192111, -0.363005, -0.480349, 2.101386, -0.145217, -0.135584, -0.440413, -0.722951, -1.326488, -0.242378, 0.420447, -0.604524, 1.321878, 0.816776, 1.580488, 1.827923, 2.080380, 1.081145, 0.761475, -1.730789, 2.288253, -0.544415, 0.435357, -1.694017, -0.530582, 1.215028, -0.132566, -1.975245, 0.952353, -0.755957, 0.964113, 0.081370, 0.234625, 1.896223, 0.808430, 2.284173, -1.323891, -0.299428, -0.140964, 0.499493, -0.385165, 1.249825, -0.785437, 1.676618, -0.737543, -1.136469, 0.074712, -0.137980, -0.742055, 0.780922, -0.560198, -0.098646, 1.233810, 0.998758, -1.257252, -0.217799, -0.385264, -0.829051, -0.009812, -1.391677, 0.512824, 0.591816, -1.269115, -0.194563, -0.015451, 1.910510, -0.510226, 2.180821, -1.400271, 0.678414, -0.446949, 1.046224, -0.652827, -1.412867, -0.553489, -0.177981},{-0.585838, -0.084215, -0.406816, 1.029838, -0.129055, 0.437792, 0.172876, 0.579103, 0.415457, -2.130626, -0.027483, 1.327954, -0.747820, -0.130665, 2.412771, -0.015582, -0.399927, 1.470577, -0.168697, 1.224260, 1.154416, -0.835225, 1.142693, 0.231932, -0.783483, 0.065600, -0.352233, 1.021067, -0.257796, -0.847913, 0.286940, 0.431891, 0.810581, -1.615084, 0.832619, 1.550840, -0.853045, -0.200991, -0.062271, -0.134421, -0.324969, 0.926193, -0.693710, 1.312295, 0.257875, -0.176047, -0.141954, -2.300955, -0.660602, 0.504604, -0.244150, -0.002432, 1.279263, -0.415399, -1.092232, -0.397309, -2.401830, 0.947173, 0.708175, -0.746796, 1.242921, 1.407318, 0.856622, -0.625902, -0.658848, 0.429186, -0.016368, -0.021643, 0.604365, -0.865632, -1.754718, 0.177028, -1.002693, -1.245247, -1.013096, -0.820480, 0.941987, -1.209730},{-0.488340, 2.091802, 0.938166, 0.781103, -0.364418, -0.362977, 0.364204, -0.328398, -0.582096, -0.245233, 0.890632, -0.091221, -0.175002, -1.299260, -0.665232, -1.096631, 0.832635, -0.482294, 1.011326, 0.665138, -0.275080, 1.379051, -0.986942, -0.573828, -0.114982, 0.288127, 0.189687, -0.723616, -0.377549, 1.402852, 0.830222, -0.188154, 0.374924, -0.878795, 0.871264, 0.497723, 0.380425, 1.882374, -0.649897, 1.284372, -0.190799, -0.184405, 0.139673, 1.602805, -0.148807, -1.338399, 0.868006, 1.253878, 0.997539, -0.343448, -0.046448, 1.206305, -0.432720, -0.562370, -1.408000, -0.898498, -0.496914, -0.257872, -1.214256, -0.285519, 0.782683, -0.511837, 0.183080, 0.657225, 1.980427, -0.732416, 0.864695, 1.052098, -1.600723, -2.157146, 0.873304, 1.124416, 0.242349, -1.087529, 1.751212, -0.859802, 0.010220, 1.716166},{-0.530286, -1.631642, -0.619072, -1.637513, -0.939648, 0.432235, 0.761375, 0.273965, 1.035787, 0.334592, 0.415881, 1.265171, -0.844969, 1.076065, -1.135855, 0.163735, -0.811622, 1.104192, 0.437810, 0.545729, 0.385101, -0.690559, -0.021176, 1.807889, -0.241600, -1.532984, 0.218435, -0.137285, 1.628977, 0.898065, 0.117232, 0.073904, -0.437760, 2.326572, -0.550720, -0.893580, -0.832461, 1.315050, -0.184489, 0.414629, 0.509418, -0.505139, -0.419551, 2.102766, 0.209868, 1.268558, 0.456977, -1.035550, -0.381906, -1.068638, 0.270227, -1.904026, 0.582133, 2.113138, -0.875756, 0.715560, 1.794676, -0.412029, 1.290295, 0.158802, -1.111638, 0.435484, 0.490289, -0.374108, 1.446944, 0.578175, 0.879139, 1.520105, -0.750860, 0.470085, 0.793183, -0.330205, -0.122907, -1.528432, -1.063425, 0.747807, 0.766351, 0.465323},{-0.439105, 0.341546, -0.092971, 0.088414, 0.581969, -1.736292, 1.427604, -0.912273, 1.013765, -1.544821, 0.712116, -2.192603, 0.410208, 1.993798, 0.722796, 0.146053, -0.610590, -0.010335, 0.570272, -1.939367, 0.186954, -1.510498, -0.960220, -0.611947, 0.353843, 0.454767, 0.525961, -1.306213, -0.447317, 0.019183, -0.562154, 1.078792, -0.929488, 1.754477, 0.400383, 0.851205, -0.984689, 1.519525, -0.756360, -0.949164, 0.486246, 0.344508, 0.347560, -0.249993, 0.023535, 1.760369, 0.602218, -0.352860, 1.612078, 0.422455, -0.020626, -0.614714, -0.770058, 0.910494, 1.550776, -0.780530, -0.562325, 0.249705, -0.355742, 1.365002, -1.056182, 0.125571, 0.104012, 0.830802, -0.838184, 0.193139, -0.351522, 0.260609, 1.064426, -1.227402, 0.025335, 0.853371, -0.989504, 0.213085, 0.069455, 0.805206, 0.083945, -0.032855},{-0.409294, 0.315385, 1.179445, -1.179404, -0.644239, -0.788445, 0.683593, -0.341538, -1.083211, 0.775320, 1.817174, 0.015928, -0.196073, -0.476772, 0.377101, -0.669976, 1.018298, -0.393479, 0.000443, 0.963584, 0.828507, 1.013412, -0.297107, -2.202551, 0.483266, 2.006454, 0.955078, -2.162915, 0.885433, -0.294745, 1.464880, 0.682755, -1.527620, -1.272040, 0.213157, 0.178353, -0.599168, -0.854656, -0.063205, 0.969782, 0.295905, -0.616613, -0.596544, 0.271170, 0.819547, -1.106742, 0.411949, -0.236251, -0.335116, -0.408317, 1.333886, 0.265284, -1.984391, -0.275040, -0.538864, -1.246598, -2.206763, 1.033879, 1.180531, 0.596346, -1.100524, 0.044956, -0.191740, -1.063976, 1.031689, -0.667199, 0.139366, -0.790166, -0.326031, 0.520371, -0.452054, 0.910747, 1.321029, -0.544650, -0.624260, 0.486673, -1.396757, 0.468615},{1.603705, -0.939617, 0.288165, 1.132989, -0.087409, -0.112612, -0.891718, -1.443033, -2.444012, -0.194955, -0.663694, 0.760028, 0.154269, -1.054432, -2.005107, -0.782611, -0.937265, -0.768096, -0.949302, -0.237208, 1.411955, -0.644624, 1.753564, 1.181340, 0.761957, 0.722506, -0.801384, 0.292996, -0.719566, -0.708769, 0.174969, -0.261303, -1.740200, -1.940321, 0.276409, -0.006105, 0.676158, 0.968852, -1.881243, 0.493698, 0.015224, 0.113820, 0.480643, 0.307634, -0.978420, 0.563723, -0.320390, -0.640533, -1.107946, -0.829785, -0.382883, 2.204278, -1.679586, 1.442681, -1.069839, -1.324852, 0.759726, -0.709433, -0.171573, -1.155657, -2.060400, -1.623515, -2.010308, 1.439354, 1.135339, 1.099178, -1.115246, -1.149233, -2.063170, 0.354791, -0.691289, -0.575248, -1.314962, -0.228141, -0.395779, 1.317914, -0.464722, 1.424006},{0.180128, 2.021518, 0.752126, -0.595665, -0.777545, 0.614578, 0.620940, -1.155452, -0.343134, 0.813657, -0.887482, 0.932326, 0.164441, -1.197553, 1.830018, 0.540522, -0.844014, 1.137136, -1.733283, -2.773775, -1.252216, 0.344551, 0.495623, -0.831166, -0.554635, -1.321649, -1.304434, -1.647217, 1.116520, -0.783448, 0.849139, 0.800720, -0.493637, -0.026571, -0.582214, -0.444748, 0.497710, -0.661308, -0.791938, 0.269156, -0.758686, -0.429796, -1.480906, 0.026538, 0.480317, -0.629286, -1.424768, -0.103333, -0.381089, -0.109375, -1.595137, -0.042531, -0.044584, -0.325601, 0.572572, 1.918105, 0.688521, -1.003669, -1.694701, -0.005010, 0.678488, -0.339143, 0.701521, 1.181883, 1.376392, 0.329364, -1.717361, 0.745370, -0.396010, -2.133154, -0.880313, 1.701800, -0.848099, -0.180856, -1.198224, 0.433581, -0.791236, -0.309511},{-1.357947, 1.411168, 0.499372, 0.795117, -0.260378, 0.243839, 0.961965, -0.424606, 0.549234, -1.369743, 0.219447, -1.027702, -0.478718, 0.589475, 0.288354, 0.762936, -1.375704, 2.299904, -0.467321, 0.253728, 1.079361, 0.112280, -0.011385, 0.533309, 0.462389, -1.530430, -0.566594, 1.806593, -1.399365, 1.331640, -0.939550, 0.405235, -0.297758, 0.532261, -1.141247, -0.718951, 0.059457, 1.436574, -0.406622, 0.661323, -1.062710, 0.370929, 1.311586, -1.841771, -0.299483, 0.153759, 0.193670, 1.728668, 1.671163, 0.031655, 1.741223, 0.688220, 0.420668, -0.392426, 1.070911, -0.088707, -0.368488, 0.256865, 1.081307, 0.449887, -0.658321, 1.846566, 0.127124, -0.465219, -0.340917, 1.003565, 0.299680, 0.579954, 0.447402, -1.110420, -0.744926, -1.617382, 0.376811, 0.779300, -1.377599, 0.668526, 0.344093, -0.613523},{-0.413872, -1.205787, -0.023182, -0.176842, -0.405418, -1.373244, 0.194646, 0.907134, -1.135543, 0.380779, 1.785896, -0.584746, -0.192108, 1.222829, -1.165221, 0.844760, 0.057290, -0.297916, 2.019704, 0.751065, 0.106935, 0.795411, -1.249146, 1.054616, 0.361607, -1.279612, -0.094221, -1.446929, -2.141815, -0.394147, 1.272524, 1.080269, -0.340406, -1.485816, -1.424086, -0.139799, -0.476663, 1.925699, -1.530758, -0.256583, -1.158317, -2.457649, -1.629587, -0.539835, -0.982242, -0.792606, 0.249810, -0.065374, -0.621675, -0.548477, 0.287945, -0.120091, -0.605034, 0.358237, 1.058316, -0.615810, 0.063546, 0.208307, -1.061011, 0.295075, 1.685213, -0.745021, -0.089906, -0.149496, -0.702689, 0.260359, 1.279557, -0.843667, 0.401022, 0.663487, 0.454191, 0.618426, -1.102386, 0.067844, 0.828297, -0.734618, -0.324389, -0.199676},{1.435323, 0.646648, -0.071266, -0.823721, -0.893489, -1.001980, -0.674518, 0.529876, -1.062185, 0.124665, -0.372138, 1.904992, 0.047535, -0.580874, 0.027064, 0.700389, -1.150559, 0.182253, -2.353968, 1.236951, -0.811090, 0.546391, 0.801801, -0.130366, 0.006145, -1.500666, 1.881547, 0.676650, 0.259576, -0.971989, 0.815620, 0.089456, -1.100833, -1.877464, -1.459218, 0.742218, -0.942965, 0.928902, 1.374765, 0.026750, 1.563719, -1.205154, 0.616566, 0.863916, -0.022016, -0.684168, 0.347449, 0.068111, -0.289898, 0.924389, 1.424277, -1.492552, -0.974506, -0.451148, -1.837634, 0.513810, -0.513428, -0.291781, 0.097644, 0.689675, 1.402399, 0.839103, 0.919916, 0.197044, -1.836042, -1.207199, 1.159452, 1.375240, -0.762438, -1.497382, 0.565184, 0.603382, -0.761998, -0.142125, 1.861794, -0.372700, -0.525096, -0.290747},{0.102363, 0.303448, -1.234783, 0.116975, 2.073528, 1.182399, -1.872834, -0.041176, -0.677158, -0.763701, 0.639708, 1.550945, 0.653763, -0.285522, -0.772595, -0.109617, -0.180304, 0.736248, -0.130436, -0.395586, -0.929735, 0.772094, -0.346402, 0.108424, 0.554415, -0.424025, -0.234366, 0.587950, -0.184726, -1.269554, 1.533328, -0.847502, -0.983476, 1.120459, -0.612272, 0.778337, -0.201498, -0.729919, 0.267627, 0.291098, -0.791241, 1.557716, -0.372485, 1.111004, -0.513513, -0.049598, 0.147910, 0.818410, 0.148878, -0.232168, 1.588887, 1.075533, 0.421870, 1.476448, -0.409250, 0.010397, -0.323491, -0.209284, 0.565893, 1.520700, 0.962662, -1.072183, -1.775644, 0.170912, 0.101219, 1.740312, -0.700434, -0.096018, 1.468903, 1.221962, -0.153544, -0.640208, -0.972119, -0.918492, 1.345617, -1.307790, -1.016471, 0.314709},{-0.175172, 0.125496, -0.543936, 0.732264, 1.382233, -1.218967, -0.076626, 0.154196, 1.168884, -0.083800, 0.222888, 0.512783, 0.761499, 0.476342, 2.049853, -0.985323, -0.027410, -1.409355, -2.004261, 1.613041, 0.174103, 1.817689, 0.711152, -0.976341, 1.374046, 0.747785, -1.073315, 0.101080, 1.209897, -0.293379, 1.134590, 2.040311, 0.107127, -2.256953, -0.569385, -0.098336, 0.553831, 0.171363, 0.092507, -0.588369, -0.833760, 0.008592, -1.211810, 0.188829, 1.050519, -0.590308, -0.117407, -2.136019, 0.577093, 1.773308, 1.359720, 0.574976, -0.877411, -0.139406, -1.253379, 0.513317, -0.955858, 0.112140, -0.882116, 0.937716, -2.142229, -0.787688, -1.602680, 0.724583, -1.447874, 0.773060, 1.254044, 2.169448, 1.389658, 0.337173, -1.100956, 2.330298, -0.010061, -0.456942, 1.519976, 0.985291, -0.178617, -0.123807},{0.430105, -1.319246, -0.010347, 0.647149, 2.150406, -1.057418, -0.517988, -0.673865, -1.374534, 0.869258, -1.049871, -0.348516, -0.067615, 0.130156, -1.187359, 0.690948, 0.130824, 0.028096, 0.878366, 1.209061, -0.453697, 0.263917, -1.687503, 0.516826, 1.316921, -0.939946, 0.811715, -0.693879, 0.279689, 1.766118, 0.932181, -1.415037, -3.153040, 1.048723, 0.343396, 1.159655, 0.685411, -1.871647, 1.456443, -0.084258, -0.195965, 1.325621, -0.649809, -1.710921, -0.005168, -1.408210, -1.062529, 0.501586, 0.900934, 0.943085, -0.504882, 0.946886, 0.477699, -1.256800, -2.503693, 0.061703, -1.382639, -1.648717, -0.988308, -0.969883, 0.260981, -1.785131, 0.630923, 0.514041, 0.569099, 1.244864, -0.436703, -0.314617, 0.717966, -0.757181, -0.189114, -0.142554, -1.969269, -1.973183, -0.049674, -1.094337, -0.234130, -0.847051},{0.357317, -0.609214, 1.332198, -2.042791, 0.952835, -0.416360, 0.276695, 1.833624, 0.670295, 1.865981, -0.123952, -0.730757, 0.045906, -1.301349, -1.200669, 1.514120, 1.278670, 2.559047, -0.685443, -1.283367, 0.356201, -0.929475, -1.564055, 0.633827, 0.047010, -1.100700, -0.142971, 0.334151, -0.317559, -0.219731, -0.660843, -1.074251, 0.270938, 1.354432, -0.860263, 1.151805, 0.567807, -0.865674, -0.519414, -0.177642, -0.338398, -0.410799, 1.016774, 0.124822, -0.531798, -0.222160, -0.345044, -1.112381, -0.120503, -1.539198, -1.193387, -0.318654, 0.493289, -1.279952, 0.073685, -1.353380, -0.687406, -0.414591, -1.399316, 0.857468, -1.020081, -1.799599, 1.921265, 0.978247, -0.211866, 0.608090, -0.843043, 0.342818, 1.080410, 0.852982, 0.143071, -0.506683, -1.849807, 0.012920, 0.201023, 0.144724, -0.272838, -1.999335},{0.046842, 1.385849, 0.698331, -0.780734, -0.952138, 0.718746, 0.546967, -1.369072, 0.804681, 0.440187, -0.506435, 0.524569, -0.379671, 0.325512, -1.996239, -1.155283, 0.755697, -0.356868, 0.607793, -0.736431, 1.279245, 0.509370, 2.026648, 0.310314, -0.220510, -0.061182, -1.006328, -0.530928, -0.526577, -0.861913, 0.845347, -0.183865, -0.435005, 0.634578, 0.317238, -1.273389, -0.806068, -1.090848, 0.188679, 0.832225, -0.203409, -0.280365, -1.012004, 0.473261, 0.017350, -0.062733, 0.435950, 0.030952, 0.290000, 0.162531, 0.951122, -1.282720, 0.576843, -0.987053, -1.849643, 1.150759, 0.848097, -0.503598, -0.029792, 1.222532, -0.018289, -2.350339, 2.698647, -0.392551, -0.345572, 1.315940, 0.701651, 1.000895, 0.498235, -0.031574, -2.240468, 1.219238, 0.233184, 0.759197, -0.183599, 2.919077, -1.826025, 0.422808},{-0.572108, 0.512709, 0.914179, -0.400373, 0.042285, -0.024606, -1.490280, 1.179247, -0.483661, 0.855409, -0.442617, -0.367697, -1.200210, -0.966806, 0.892918, -0.111211, 1.781506, 1.098821, 0.089243, -0.171057, 0.237046, 1.290265, -0.977927, -0.464890, -1.559754, -2.170369, 1.720786, -1.330806, -0.757652, -0.598429, -0.164712, -0.244889, -1.048878, 1.465255, -0.865544, -0.518356, -0.483285, -1.052110, -0.894621, 0.278544, 1.586180, 1.334229, -0.606736, -0.809547, -1.444111, -0.107966, 0.367180, -1.163058, -0.023739, -0.095710, -0.454951, 2.397322, 0.055656, -0.517539, 0.881339, -0.174774, 0.898447, -0.390445, 1.367778, -0.135530, 0.497005, 1.474497, 0.079571, -0.485502, 0.777018, 0.134464, -0.264917, 0.391290, -0.641221, -0.500492, -0.678449, 0.242026, 2.333752, 0.022242, -0.877969, 1.348711, -3.198368, -0.227709},{-1.498837, -2.112968, -1.468389, -1.518523, -0.319796, 0.312451, 1.548325, 0.560070, -1.972368, -1.082473, 1.181281, -1.601266, -0.183209, 0.544321, -0.422206, 0.816170, -0.476420, 0.122093, -1.394184, -0.246358, -0.731394, 1.008699, 0.396949, -0.099489, 0.828181, 1.902876, -1.969137, -0.111988, 0.624319, 0.261463, 0.478099, 0.209701, -0.861255, 0.068704, 0.989999, -0.807904, 0.270853, -0.282202, 0.520939, 0.307805, -0.138610, -0.436800, -0.612795, 1.143032, 2.233583, 0.542607, 0.091381, 0.046758, -0.736193, 0.871452, 1.425599, -0.739089, -0.066076, -0.447911, -0.324937, -1.381993, 0.703907, -0.931593, 0.419892, 0.564441, 0.984856, 1.281550, 1.456873, -1.287952, -2.379035, 2.112760, -0.145529, -0.501487, 2.054040, 0.369733, -0.780761, 0.108961, -0.530542, -0.026995, -0.467316, 0.683022, -1.651935, -0.423118},{-0.577695, -1.839288, -0.306134, -0.172295, -1.917019, -0.005248, -0.138758, 1.041404, 0.054353, -0.828688, 1.843906, -1.892444, 0.869100, 0.936202, 1.004182, 1.131799, -0.810037, -0.695225, 0.393975, 0.839589, 0.828278, -0.197422, 0.093073, -0.387439, -0.523515, -0.102310, -0.457891, -0.943240, 0.227094, 0.456374, -0.382655, 0.810510, 1.649588, 0.238089, -0.323394, -0.837846, -1.309705, 0.414067, 0.001898, 1.800964, -0.914169, -0.246405, -0.236123, 0.160179, 0.055221, 0.663205, -0.432997, 1.812522, -0.261444, -1.345199, -1.098678, 0.939709, 0.747778, -1.362424, 1.203425, -1.059934, 1.470264, 3.033722, -0.962141, 0.835713, -0.144554, 1.169123, 0.247078, 0.903033, 0.470634, 0.936551, -0.286177, -1.171413, 1.508232, -1.246741, -0.334690, 0.407829, -0.883781, 0.953622, 0.090750, -0.897362, 0.526340, -1.223100},{-1.316284, -0.765667, -0.221770, -0.257181, 0.613038, -0.748482, 1.770344, -1.363233, -0.939763, -0.055940, 1.116079, 0.054970, -0.700119, -0.150463, -1.794499, 0.313948, 0.414478, -0.231675, -0.927236, -0.132581, -0.163134, 1.269291, -0.348227, -0.239065, -0.283978, -1.338060, 0.264976, -0.041631, 0.662607, 0.665147, 2.105738, -0.873922, -0.193753, -0.749995, 0.012745, 0.434890, -0.047432, -0.885121, -1.042544, -1.726602, 1.621089, -0.678591, 1.118596, -0.167370, -0.001547, 0.355930, -0.049428, -0.652891, 0.118100, -1.289442, 0.082871, -1.571719, -1.307889, -1.564656, -0.691914, 1.196060, 0.647496, 1.187874, 0.558322, -1.055463, 0.084511, -1.885323, -1.390189, -1.385992, 0.818444, 0.436411, 0.775592, -0.975130, -0.269911, -0.060139, -0.104927, 0.886628, -1.090813, -1.296231, 0.775973, -0.667025, 0.407737, 0.812068},{-1.282681, -2.016865, -0.960907, 0.364229, 2.475264, 1.164574, -0.744867, -0.097890, -0.537813, -0.079119, -1.108666, -0.055599, 1.038804, 0.751951, 0.382653, 0.176508, -0.954594, -1.553347, 0.839730, -0.932986, 0.539950, 0.246597, -1.758501, 0.211032, 0.893758, -1.359277, -0.775328, -0.206097, 0.655825, -0.439632, 1.133753, 0.017797, -0.016720, -0.215007, -0.994442, -0.116007, 0.075261, 0.392195, 0.540041, 0.581614, -1.451021, -0.359303, 0.912513, 0.984779, -0.800921, -0.002893, -0.680367, 0.643462, -0.225685, 2.432636, 1.030256, -0.401327, -0.360382, 1.547737, 1.534509, 0.825954, -1.674216, 0.022361, -0.137224, -0.433855, 0.409585, 0.541435, 0.418799, -0.474855, 1.335149, 1.326051, 0.023642, 1.144158, 0.798652, -0.492514, -1.476833, 2.798865, 0.685935, 0.966747, 0.366944, -1.069775, -0.109724, -0.574798},{0.962504, 0.105184, 0.305298, -2.036238, 1.444028, -0.081440, 0.231078, -0.013526, -1.782119, 0.646519, -1.050687, 1.440825, -1.810307, -0.806825, -1.500092, 1.564197, -0.222493, 1.066791, -1.107051, 0.119185, -0.163089, -1.414626, -0.903409, -0.595694, 0.896165, 0.618765, 0.071891, 0.607766, 1.705058, 1.166322, 1.235413, -0.186413, 1.871849, -0.384732, -0.041515, 0.380526, -0.076075, -1.499125, -1.193660, 1.607124, -0.392969, -0.686462, 1.524357, 0.952372, -0.706530, 0.014349, 0.088996, 0.769155, -0.496664, 0.345933, -1.104150, 1.000389, -0.387515, 0.538736, -0.964441, -0.826973, -2.161029, -1.601663, -0.741022, -1.024705, -0.700848, -1.091121, -0.291100, 0.145728, 0.494382, 1.281073, -1.479774, -1.088780, 1.960656, 0.224540, 0.992380, 0.233111, 0.741342, -0.809829, 0.380977, -1.481088, -1.017575, -0.753749},{-0.145785, 1.224214, 0.329947, 0.021139, 0.889716, 1.157573, -0.078774, -1.523703, 0.206463, -1.837233, 0.108425, 0.564664, 0.488210, -0.140787, -0.900327, 1.752830, 1.717850, -1.916088, -2.533592, 1.676779, -0.376773, -0.552765, 0.758493, 1.669169, 0.623193, -0.285959, -1.146852, -1.354758, -1.661241, 0.048632, -0.413572, -0.075974, 1.524990, 0.618944, 0.283274, -1.141399, 0.063121, -1.440181, 2.206625, -1.669142, -1.482532, 2.096342, -0.058576, 0.157094, 0.203235, 1.624283, -0.279410, 0.686986, 1.058579, -0.245304, 1.356801, 0.615973, -0.630620, -0.575870, -0.558397, 0.240390, -1.046582, 0.150505, 2.154203, -0.342951, 1.439938, -0.401078, 0.709991, 1.137398, 0.208670, -0.298536, -0.606423, 0.265838, -0.616004, -0.919141, -1.406324, 1.007734, -0.157824, 1.203512, 0.146390, 0.537627, -0.927031, -0.217011},{0.138066, 0.759193, 1.382286, -0.066922, -2.296667, 1.268965, -1.617288, -1.282591, 1.272883, -0.372087, 0.699952, 0.614815, 0.583701, 0.967195, 0.056955, -1.457688, 0.832684, 0.270075, 0.854098, 0.465105, -0.225194, 0.686601, 0.602404, -0.168966, -0.652696, -0.460203, 0.076127, -0.493570, -0.311073, -1.990782, 0.772935, -0.096611, 0.979698, 0.202732, 0.187717, -0.985449, 0.652731, -0.033074, -1.624211, -1.540261, -2.216374, 0.533072, -0.652561, -0.135438, -0.482885, 1.113350, 0.192338, -1.522606, 0.116742, -0.258717, -0.149744, -0.294949, 0.888588, -0.165757, -1.624564, 1.214118, -1.678852, -0.998318, 0.217014, -2.084067, -0.763619, -0.954360, 0.061435, 0.792308, -0.824704, -0.615190, -1.697496, 0.990577, -0.421084, 0.868527, -1.188775, 1.477994, 2.252108, -1.568735, 0.452649, 0.134150, -1.314590, 0.716846},{0.262700, 1.166702, -0.799823, -0.811366, -0.135444, -0.809461, 1.393681, -0.131017, 0.040355, -0.139640, -0.372563, 0.154916, -0.620430, 1.608580, 1.489295, 0.319119, -0.105592, -0.530464, 0.661258, -1.749314, 0.750733, 0.267014, 1.066839, -0.611858, 2.164018, -1.853669, 0.480135, 1.046825, 0.108085, 0.050939, 0.787071, -1.457792, 1.700190, -0.219649, 0.503296, -0.669424, -0.722245, -0.040210, -0.625959, 0.051451, 0.685465, 0.331302, -0.797060, 0.952468, -1.328932, -0.238436, -0.410948, 0.356217, 0.082202, 2.204853, -0.157118, 0.245249, -0.302244, -0.997386, 0.223609, -0.156475, 0.032668, 0.201696, -1.843198, 0.859015, 0.554416, -1.485397, -0.747166, 1.103517, 0.320689, 1.117564, 0.554230, 1.194483, 0.446813, -2.071088, 0.276648, 0.107045, 0.627285, 1.021409, -0.039043, -0.485656, 0.465870, 0.964179},{0.537175, 1.234273, -0.198401, -0.079040, 0.273923, -0.330345, 1.456860, -0.209712, 0.527766, 0.437740, 1.269209, 1.056227, -1.183787, 0.030262, 0.224694, -0.813315, -0.455426, -2.166576, -0.438409, -0.828441, 0.025763, 0.206950, 0.444836, -1.173807, 2.204503, 1.276316, 1.115478, -1.150837, 1.490998, -0.034844, 0.392457, -0.419167, -0.578645, 0.110548, 2.252193, -0.025509, 1.393533, -0.812363, 0.994237, -0.407301, 0.138118, 0.089279, 0.833929, -0.058064, -1.373654, -0.519666, 0.116518, -0.344309, -0.031140, -1.694732, -1.974120, -0.744414, -0.278401, 0.973262, 0.229812, 1.037062, -1.577135, 0.871729, 0.773898, -1.678561, -0.884261, 0.452290, 0.880225, 0.804704, 0.089664, 1.088301, 0.676935, -2.006441, -0.807113, 1.840460, -0.196690, -1.076191, -1.565710, -0.000391, -1.119388, -0.628752, 0.018815, 2.233174},{1.250635, 0.449371, -0.824456, 0.115915, 0.669837, 0.210217, -1.053029, 0.093394, 2.942886, -1.111784, -0.988940, 0.346414, 0.599572, 0.435806, -0.598090, 1.098492, 0.957484, -1.498850, -0.548629, -1.173866, 0.865487, 1.673078, 0.177525, -1.000831, 0.259276, 1.987211, -1.000997, 0.414260, 2.162414, -0.031570, -2.022666, 1.730797, 0.603030, 0.080064, 1.214109, 0.379413, 1.052513, 0.404458, 0.598082, -0.333729, -0.569194, 1.511518, 1.737301, 0.749539, 2.174215, -0.743554, 0.631690, -1.200200, 1.074477, -0.129519, -1.741890, -0.419998, 2.416049, 0.349748, 1.636482, 1.330621, -1.895948, 1.535961, 1.039462, -1.164978, -0.169644, 0.676292, -1.722163, -0.310740, -1.036431, -1.015685, -2.189608, 0.026476, -0.248003, 0.276064, -0.309724, 2.295830, -0.807818, -0.374487, -0.476235, 0.045737, -0.857109, -0.187808},{-0.166051, 0.401885, -0.577738, 1.171017, -0.865180, -1.727844, 2.014007, -0.750370, -1.195028, 1.671372, -0.149928, 0.481431, 1.957748, -1.533721, -0.583576, -0.278717, 0.388859, 0.747661, -1.320461, -0.272809, 0.181899, -1.436629, -0.827356, -0.441458, 0.752325, 0.255442, -1.841300, 0.195495, -1.567166, -0.767538, -0.781032, -0.175931, -0.820087, -0.677183, -0.335771, -0.204834, 1.333361, -1.225940, -0.828925, -0.147202, 0.164025, -0.594650, 0.049929, -1.101505, 1.280411, -1.641956, -1.491360, 0.994407, 0.878316, -0.082161, -0.520108, -1.328789, -0.435611, -0.180934, 0.914255, 0.508009, -1.982680, -1.403927, -1.104967, -0.351935, -0.264197, 0.661162, 0.929791, 0.838322, -0.717559, -0.609220, -1.581593, -0.728147, -1.353536, -0.770350, 0.050190, 0.094369, 0.747397, -1.866626, -1.069039, -0.740771, 1.113426, -0.158679},{1.827797, -0.348675, -0.713554, 0.822190, -0.912277, 0.156994, -0.570428, 0.735120, 0.101297, 0.293973, -0.481516, -0.758679, 1.834477, 0.364588, 0.816963, 0.960498, -2.154277, 0.673877, -0.473852, -0.463476, -0.074125, -0.967016, -0.407690, -0.874865, 2.371113, -0.388319, 0.282277, -0.852494, 0.018967, 0.067279, 0.217829, -0.984046, 1.482888, -1.195122, -0.304189, -0.564262, 0.822340, 0.945998, 0.281058, 0.178960, 0.469474, -0.547960, 1.219448, -0.158928, -0.126951, -0.806827, -1.141800, -0.351214, 0.826077, 0.299232, -0.018334, 0.086412, 0.079574, 0.845281, 0.226861, -0.307928, 0.960455, -0.291480, 1.185228, -1.329563, -0.473120, 0.359575, 0.292442, -0.901561, 0.871768, 1.143756, 0.997031, -1.625452, 0.654568, 1.270043, -0.596519, 0.699918, -0.337522, 1.785115, 0.838365, 1.057362, 0.233281, 1.741162},{-0.761845, -0.168628, -0.761732, 2.181331, 0.444276, 1.533843, 1.643557, -0.193675, 1.893404, 2.830130, -1.737089, 2.053363, -0.009223, -0.080742, 1.170287, 0.347866, -0.809372, -1.667578, 0.057779, -1.796664, -0.387507, -0.350507, -0.444102, -1.046736, -0.242844, 1.509820, -0.473255, -0.997966, -1.504285, -2.835721, -0.180026, 1.528574, -0.128765, -0.589249, -1.150133, -0.459548, -0.139662, -0.184689, 1.152244, 0.116342, 0.335788, -0.099479, -0.531309, 0.464503, -0.645622, -0.903025, 0.077886, -0.270101, -1.092886, 0.193881, -0.303572, -0.749838, 0.093431, 3.155711, 0.647345, -0.282376, -0.876155, -0.495911, -0.567954, -0.207325, 0.513773, -0.169980, -1.270612, 0.347666, 1.799430, 0.325353, 1.685406, -0.198685, -0.321738, -1.631333, 1.152649, -0.237204, 0.240268, 1.507695, -0.098065, 1.569446, -0.204879, 0.712120},{0.543635, 1.041583, 1.017164, -1.002075, 1.326334, -0.308154, 0.500482, 1.236785, 0.695917, 0.629831, -2.432107, 0.658047, -1.687021, -0.962413, 0.981443, 0.353146, -0.114125, 0.982125, 0.452280, -0.036677, -1.833936, -1.970784, -0.596744, 1.655168, -1.198594, 0.292357, -1.040103, -1.027999, 0.682059, 0.964402, 0.421050, 0.074594, -0.533908, 1.538133, -2.249078, -0.172837, -0.061493, -0.618321, 0.246884, 0.099754, 0.032940, -1.599510, -0.657344, -0.604113, -0.130675, -0.685972, 0.527736, 0.681465, -0.699601, 1.745731, 0.213686, -0.285650, 0.124009, 0.468062, 0.750594, -1.447773, -0.123791, 0.832555, 0.158332, 0.079927, -1.176750, -0.286659, -1.527957, -0.214484, 0.903000, 0.474720, -1.136834, -0.045626, -0.411589, -1.265100, 0.497466, 0.192017, -0.672137, 0.008771, 0.071734, -1.278627, 0.355603, 0.995678},{-0.638842, 0.699702, -0.697219, -0.059798, 0.488408, 1.450536, -0.657385, 0.892467, 1.102183, -0.437595, -1.375796, 0.914313, -0.569388, -1.110522, -1.264284, 1.287688, 2.271027, 3.288102, -0.283719, 1.588634, -0.630863, 1.132614, -0.206317, -0.065637, 1.780201, 0.294903, 0.691936, 1.844754, 1.013630, 0.219401, 0.347019, 0.983117, 0.217584, -0.190948, 0.235693, -0.424265, -1.127991, -0.095233, -1.086536, -0.527030, -0.762299, -1.314843, -0.423275, -0.551190, -0.066559, 0.341698, -0.039916, -2.638274, -0.663977, 1.340964, -0.123229, 0.352012, 0.054158, -1.979436, -0.176720, -0.414530, -0.620370, 0.316043, -0.747634, 0.245759, 1.539412, 1.007466, 0.921678, -1.527906, -0.835643, 0.300339, 1.373535, -0.611869, 0.032953, 1.415719, 2.302378, -0.104913, -0.763635, -0.157152, 0.390396, -0.979764, 0.154901, -0.611422},{1.076913, 0.341235, -0.953832, -0.680556, -0.367735, 0.788819, 0.210511, -0.493583, -1.595783, 0.642844, -2.604781, 0.933692, 1.801101, -0.991194, 1.470321, 0.044112, 2.207038, 1.395851, 1.698892, 0.476745, 0.281768, 0.795857, 0.551189, 2.001385, 0.922759, -0.320614, 0.083615, 0.662993, 0.729194, 0.631989, -0.538527, -0.508308, -1.108588, -1.629001, 0.026741, 1.138806, -0.076011, 1.552935, 0.523622, -0.527089, 0.103882, 1.137172, -1.382315, -0.119199, -0.405008, -1.200122, 0.494479, 0.661318, -0.223354, -0.263373, -0.761695, 0.135740, -0.793905, 1.364455, 0.867092, 0.177397, 0.332281, -0.154697, 0.879724, 1.080438, -0.133160, 0.093215, -1.082119, -0.080967, 0.487103, 0.003954, 1.098513, -1.174554, 0.808863, -2.354670, 0.012412, -0.149314, 0.122235, 1.670863, -0.689409, 1.524203, -0.846835, -1.561114},{0.335348, -0.954199, -0.040027, 1.712446, 0.199808, 1.335336, 0.298073, -0.516391, -0.262968, 0.794722, 0.761829, 0.008536, 0.076707, 0.037281, 1.151274, -0.861781, 0.380119, 0.078939, -3.209893, -1.171192, 0.779054, 2.349658, 1.714491, -0.639114, 1.523647, 1.653559, -0.130339, -0.319276, -1.159878, -0.609141, 2.001850, 0.205239, -0.934459, 1.490595, 0.456103, 0.452403, 0.069317, -0.382846, -1.097590, 1.059392, 0.790637, 1.625397, -1.629353, 1.160955, 0.671719, 0.397063, 0.007384, 1.867853, -1.191469, 0.357517, -1.984828, 0.939987, -0.665277, 1.151006, -0.761987, 0.759454, 0.246569, 0.005592, 0.814520, -2.781866, 2.351136, 1.240613, 0.201756, -1.063766, -0.838859, 2.037162, -1.353769, 0.755593, 0.564946, -0.032369, 0.717266, -0.622106, 0.919678, -0.631592, -0.817617, 0.091322, 0.589831, 0.300903},{-1.751109, 0.490643, -1.110622, -0.646114, 0.222320, 0.772794, 0.540715, 0.749601, 1.332019, -0.426499, -0.135940, 0.874777, 0.687711, -1.782783, -1.325101, -0.643278, 1.062837, -1.133425, 1.007361, 0.992054, -1.281561, -0.638433, -0.178851, 0.059718, 0.365611, -2.084388, 0.417447, 0.899187, 0.478950, -0.981653, 0.545046, -0.841744, 0.486426, -1.322160, -0.193034, 0.217477, -0.115857, 0.791174, -0.653165, 0.670397, -1.598639, -0.229464, 0.986295, -0.913760, -0.034469, -2.518580, 0.432038, 0.081226, 2.023854, 0.291669, 1.083153, -1.892463, -0.810453, 0.606394, 0.481117, 1.703636, -0.347773, -0.364194, 0.065638, -0.550486, -0.351528, 1.139470, 0.236512, -0.251479, 0.105582, -0.118788, -0.350383, -1.117047, -0.655885, 0.289497, -0.914939, 0.636273, -1.252334, -0.487815, 0.246479, -1.385505, -0.540004, 0.469969},{-0.470942, 0.784906, -0.063962, 1.053689, -1.134834, 0.515330, -0.217724, 1.184349, 0.669531, 0.419772, -0.537242, 0.832353, -2.712418, 1.534976, -0.880013, -1.693188, -1.354656, 1.021036, -0.783497, -0.375433, 0.664155, 0.618816, -0.192153, -0.987675, -0.869623, 0.928380, 1.419211, 0.643774, -0.712456, 0.990232, -1.212232, 0.702349, -0.367924, -1.071383, 0.247605, -2.311673, -0.234234, -0.228405, 0.275653, -2.376266, 0.605750, 1.210990, 0.797135, 0.272325, -0.710021, 2.515249, 0.642365, -0.592691, 1.836621, 1.535590, 0.526655, 1.428036, 0.260606, -2.539398, -0.722012, -1.609148, -0.874338, -0.027592, -0.557982, 0.689133, -0.670734, 0.809223, 2.022291, -0.154947, -1.222586, -0.302182, -0.747400, 1.043273, 0.410178, 0.205938, -1.225680, 0.313525, -0.802265, 0.669018, 0.993892, -0.073321, 1.015340, 1.269768},{0.190851, 0.218976, 0.064589, -0.779831, 1.521967, 0.730977, -0.596579, 0.418621, 0.704221, 0.448857, -0.628757, 0.567179, 2.050343, -0.535982, 0.247424, -0.712336, 1.561409, -2.265996, 0.833275, -0.545297, -0.101399, 1.066870, -1.323914, -2.058414, 0.051345, 0.955929, -0.303216, 0.006846, -0.642837, -1.180850, -0.065952, -0.084859, 2.499692, -0.060628, -0.374554, 1.277744, 0.694257, -2.055883, 0.104933, 1.664006, 1.192066, 1.195457, 1.451523, -1.295540, 0.059544, 0.251758, -0.979611, -0.464945, -0.522485, -1.695353, -1.653432, 0.007179, 0.803630, 0.590193, 0.641057, -0.102026, -0.276551, 0.281728, -1.174382, 0.445546, 0.230572, -0.211125, 0.267771, 0.442534, -1.456536, -0.423838, 0.662210, -0.177400, 0.663692, 0.593523, 0.845660, -1.416181, -0.997008, -0.768582, 0.362304, -1.322607, 0.127508, 0.102542},{0.724711, -0.195258, 1.188995, -1.796136, -0.236839, 0.001875, 0.162545, -1.355155, -0.996556, -1.768176, -0.147709, -0.107982, 1.472868, 0.811439, 0.468367, -1.248721, -0.226257, -0.339383, -0.807870, -0.373864, -1.605374, -0.468666, -1.068360, -1.284161, 0.413089, -0.995887, -0.638498, -0.964426, -0.752403, -0.160776, -0.071286, -1.126709, 0.587224, 0.517177, -0.554739, 1.044793, 0.643629, 0.128759, 0.398523, 0.340393, 1.566855, 0.334372, 0.919044, -0.573871, -0.503984, 1.339506, -0.556329, 0.833745, 1.926219, -0.801598, -0.120643, -0.773456, 1.646688, -3.440828, 0.925939, 0.682399, 0.348686, -0.680170, -0.077336, 0.438958, 0.682494, -0.853787, 0.146291, 3.420981, -0.296316, -0.033176, -0.564613, -1.422273, -0.116454, 0.583307, -0.145094, 1.204371, 0.718472, -0.234168, -0.152394, -0.255655, 1.115102, 0.238779},{-0.303517, -0.213566, -1.067407, -0.186014, -1.714713, 0.360969, 0.147945, -0.096831, -0.783350, -0.388350, -1.810297, 0.329941, 2.542279, -0.267488, -1.010965, -0.013432, -0.486663, 0.285422, 0.222180, 1.486053, 0.186837, 0.117908, 1.133987, 0.755581, 1.454823, 0.734058, -0.376483, -0.104658, 2.125198, 0.593731, 1.264140, 1.313945, 0.925774, -0.725607, 1.156380, -0.830264, 0.792709, -0.042140, -1.441192, 1.733173, -1.073579, -0.441012, -1.275356, 0.100994, -0.349017, -1.441408, 0.240323, 0.831417, 0.797583, 0.532040, -0.815712, -0.805121, -0.484107, -0.244071, -0.327299, 0.898819, -1.002129, 0.470416, 0.450725, 0.035077, -0.815686, -0.553811, 0.425441, 0.264474, 0.478110, 1.622467, -1.635304, -0.637358, 0.916174, 0.165073, 1.207169, -0.210202, -0.330804, 0.727878, -1.143857, 0.787693, 0.933327, -0.521220},{-0.957092, 0.672407, -0.148784, -0.308939, -0.645202, -1.218011, 0.807785, -1.443880, 0.366511, 0.129501, -1.910982, -0.745203, -0.185162, -0.681906, -0.187500, -0.334768, 0.713117, 0.981483, 0.441710, -0.648616, -0.926274, -0.276710, 1.029612, -0.772778, 0.746726, -0.323102, 0.540869, 0.316432, 0.825347, -1.267451, -0.902931, 1.112813, 0.008950, -0.550458, 0.963655, -2.377270, 1.107552, -0.537312, -0.394524, 0.963010, -0.558379, 0.037247, 0.271378, -0.950799, -0.158309, 0.312363, -0.917927, -0.651914, 0.488887, 1.710880, -0.491616, 0.301438, 0.581809, -0.441367, 0.655393, 1.939141, -0.743795, -1.805788, 0.074925, 1.086113, -0.491336, 1.950116, 0.675152, -0.076486, -0.012691, 0.682958, 1.815462, -0.675353, 0.126406, 0.666670, 0.723380, 0.355150, 0.415683, 0.647127, -0.139196, 0.803709, 0.409068, -0.348084},{0.892496, -1.078676, -0.053424, 0.776034, 0.061573, -0.266351, 1.632474, -0.058906, -0.079235, -0.237703, 0.641167, 0.503858, 0.925072, 0.124225, 2.178301, 0.674665, 0.766643, -0.625071, -1.304965, 2.002518, -1.366115, -0.384556, 1.801031, -2.132460, 1.706510, -1.015394, -0.631973, -0.153948, -1.042331, -2.550815, -0.898521, 0.647538, 1.895635, -0.617742, -0.255789, -0.319904, -0.102441, 0.037215, -0.735570, -1.448942, -2.282763, 0.831822, 1.117211, 0.749256, -0.964086, -0.354225, 1.772091, -1.686597, 0.810047, -0.011335, 0.546399, 0.313156, 0.369734, -1.462825, -0.102282, 1.247597, 0.066818, -1.974991, 0.061094, 0.917725, 1.189905, 3.137558, -0.004642, 0.743617, -0.268244, 1.889533, 0.148829, 0.384709, -0.080451, -0.185353, -0.490420, 0.993040, 0.161945, -0.356057, -1.011407, 0.848630, 0.581571, -0.010967},{-0.708247, 0.508912, -2.218434, -0.209013, 0.848980, -1.009158, -1.381972, -0.223854, 0.647227, 1.606837, 0.754381, 0.024358, -1.207302, -1.002697, 0.178409, -0.755740, 0.573283, 0.959806, 1.500933, 0.093375, 2.001063, 1.731116, 0.501515, -0.468486, -0.021027, -0.562755, 0.566655, -0.871257, -0.109348, -0.129570, 1.182250, -0.480904, -1.937641, -0.862023, 0.465009, 3.167489, 2.349428, -0.951304, 0.120663, 1.053303, -2.892003, 2.933113, -1.360585, -0.086922, -2.197876, -0.378099, -0.223489, 0.570505, 1.533860, 0.842932, 0.983167, 0.462049, 0.732166, -0.269665, -0.281571, 0.936369, 1.114549, -0.043543, -0.617503, -0.550378, -0.366704, -2.272036, 0.148455, -0.721159, 0.531113, 0.888245, -0.384891, 0.066372, 1.906650, 0.181258, -0.562052, 1.504457, -1.129342, -1.213326, 0.184552, 0.346710, 0.376701, 0.095965},{0.409961, 0.671824, -0.200599, -1.687282, -0.200134, 1.223087, 0.323698, 1.056553, -0.182458, -0.725416, -1.955471, 0.395694, -0.352789, 1.360710, -0.828506, 1.352723, -0.440138, -1.310805, -1.000302, 0.068898, 0.524897, 0.368990, 1.992319, -0.543608, 0.506715, -0.408909, -0.375232, 0.678150, -0.306353, 0.164226, 0.736235, -1.534202, 0.072123, 1.101328, -0.841450, 2.561081, 0.737860, -1.983671, -0.024867, 0.252615, 0.777024, 0.709155, 0.014577, -0.429381, -1.667971, -0.609711, 1.089635, 0.171048, 0.295034, 0.001253, 0.520581, 0.153295, -1.738471, 0.375850, 0.406359, -1.247447, -0.062332, 0.508047, 0.757950, 1.090068, 1.983863, -1.064310, -0.556641, -0.254274, 0.563532, 1.411980, 0.125112, -0.024366, 0.912034, 0.644570, 1.821930, -1.284885, 0.721774, -0.273913, -0.565729, -2.572690, 0.578095, -0.901491},{0.209996, -1.108724, -0.211281, 0.453412, -1.611677, -1.230443, -0.536140, -1.080269, 0.997497, -0.787911, -0.009482, -1.228796, 2.500237, 0.516002, 1.335172, -0.779251, -1.620602, -0.597084, 2.583064, 1.540073, 2.070949, 0.592958, -0.106627, -0.039918, 2.067276, 1.061351, -1.132434, -0.035262, -1.953243, -0.341404, -1.115177, 0.093475, -0.741762, -0.981970, 0.500274, -0.927595, 0.039080, 0.800508, -1.440560, -1.409546, 1.177583, 0.087985, -0.589601, 0.926847, 0.040947, 0.392405, 0.649415, 0.894644, 0.008205, 0.977841, 1.007814, -1.261309, 0.705262, 0.051456, 0.247763, 0.710904, 0.090247, -0.091359, -0.230141, 1.646684, 0.430104, -0.701376, -1.183539, -0.431762, -0.172024, 0.169976, 0.131992, -0.424984, -0.097364, 0.925476, 1.601587, 0.759181, -0.992422, -1.413012, -1.327973, 1.230426, 0.073066, -1.340645},{0.433820, -0.807143, -0.155523, -0.552650, 0.221765, -0.051881, 0.089387, -0.648085, -0.069970, -0.321147, -0.374987, -1.395556, -1.198249, -2.310273, 0.006713, 1.473824, -0.322233, -1.682998, 0.590198, 0.873304, 0.925333, -1.217732, 0.277637, -1.003203, -0.754292, 0.695388, 0.569965, -1.250471, -0.162409, 0.204968, 1.772095, 0.302265, 0.406149, 0.302066, 1.150902, -0.951888, -0.352101, 0.578265, 0.384637, -0.932789, 0.630488, -0.662750, -0.922568, 0.331958, -0.861619, 0.588135, -0.754805, 0.290998, 1.499398, -0.697992, -0.429991, -2.266082, -0.122517, 0.742343, -1.205079, -0.361505, -0.021731, 0.636806, 0.598532, 0.478911, -0.675074, -0.571024, -1.501945, 1.303893, 0.782415, 0.594743, -0.385074, 1.004268, 1.671987, 1.385269, 0.453275, -1.315487, 0.480973, 0.323854, -0.099378, -0.393487, 0.037428, -0.497837},{-0.304973, 1.506796, 0.423544, 0.278268, 0.829677, 0.703123, -1.663563, -0.669862, -0.949738, -0.664025, 1.481264, 0.803828, 1.434252, 1.660890, 0.975147, -0.500667, 0.979654, -0.612495, -0.813580, 0.863564, -0.529159, -0.370348, 0.121561, 0.695856, -0.504985, -1.824569, -1.453941, -1.642599, -1.889810, -0.103704, -0.106210, -1.749458, -0.180077, 0.309329, -0.389654, 1.283966, 0.322524, 1.778801, -1.385758, 1.442236, -0.056296, -0.755385, 0.796649, -1.416239, -1.022301, -0.082872, 0.502073, -1.736884, 1.094575, -0.415994, 1.167708, 0.007357, -1.048085, 0.113282, -1.643639, -0.640620, -0.287239, 0.717848, 1.789324, 0.932156, 1.280617, 0.614617, -0.772108, 1.317834, 0.210622, 0.356268, 0.912480, -0.677660, 0.136328, 1.007791, 0.101178, 1.129083, 0.008233, -0.018384, 0.689772, 0.496337, -0.426117, 0.032326},{-0.832456, 0.673303, 0.007354, -0.402189, -0.777803, -0.349053, 0.687475, 0.607683, 0.028998, -1.770104, -1.614581, 1.071912, 0.498647, 0.317444, -0.028513, -1.285373, 1.462605, 0.119035, -0.956517, -0.744902, 1.964502, 0.610554, -1.389451, 1.813979, -1.293168, -0.844664, 0.417662, 1.155671, 0.173443, 0.904968, 0.184231, 0.733487, -0.918618, 0.323315, -0.883369, -0.085066, -0.524166, 0.464281, -0.674469, -0.352580, 0.135048, 1.719250, -0.428276, -1.480246, 1.639428, 0.258187, -2.433903, -0.771714, 0.021822, -0.723025, -1.637191, -1.720378, 0.997939, 0.527841, 0.266255, -0.067642, -2.152850, 0.617998, 0.430101, 0.056258, 0.087711, -0.386848, 1.107902, -0.171240, -0.055964, -0.001913, -0.580202, 0.677504, 0.727199, 0.737474, 0.360225, 0.088663, 0.911380, -0.886000, 0.221093, -0.446116, -0.140254, -0.665119},{-0.674920, -1.461894, 0.251851, 0.800272, -0.973214, -1.790743, -0.798233, 0.651003, 2.901460, -1.261445, 1.571600, -0.074216, -0.797416, -0.297986, 0.445689, -1.470080, -0.462274, 0.450319, 1.045229, 0.294883, 1.736270, -2.839717, 0.945035, -1.666299, -0.674941, -1.400239, 0.077973, 0.074625, -0.473735, -0.523133, 1.150662, 0.243782, 1.544392, -0.175079, 0.037771, -0.530881, 1.291330, 0.568087, -0.242744, 1.715991, 0.676713, 1.472358, -0.302885, 0.657242, 0.903533, -1.111600, -0.811876, -0.479337, 0.299404, -1.581453, 0.692882, 0.317706, 0.400670, -0.377863, 0.902299, 1.299247, -0.195436, -1.485414, -0.280439, 0.567474, 0.854957, 2.639009, 0.268632, 1.544209, 1.805970, 2.831416, -1.177654, -0.348069, 0.182074, -1.045076, 0.045668, -1.294435, -0.443202, -0.544942, -1.486031, -1.516681, -2.667961, 0.708876},{-0.391569, -1.300395, 0.137048, -0.212127, -0.387860, 0.405180, 0.119392, 0.179059, 1.015217, 1.425687, 0.286650, -1.493790, -1.211253, 0.448483, 0.321779, -0.162489, 0.128857, -1.141635, 1.477331, 0.060750, 0.008240, 0.579338, 1.791984, 0.438764, -2.569216, 1.393878, 0.715370, -0.528558, -0.146777, 1.249878, 1.065527, 0.342983, -1.084685, -1.510736, -1.063070, -0.569781, -0.432242, 0.123527, -0.375430, -0.104933, 0.808130, -1.028558, -0.178294, -0.432860, -0.344052, -0.004000, -1.441837, -1.046362, -1.489278, -0.631865, 0.379270, -2.332715, -0.395808, 1.205913, 0.940771, 0.865596, 1.082108, 0.288715, 0.282267, 0.727679, -0.795690, 0.021359, -0.963926, 0.270339, 1.542709, -0.707992, -1.432112, 0.477252, -1.225866, -1.757527, 1.295362, 0.285738, 1.715659, 1.333010, 1.745832, 2.339344, -1.937571, -1.093047},{-0.333631, -0.099394, -0.387980, -0.986949, -0.445652, 0.556182, -0.201631, -1.434157, 0.195767, 1.453956, 1.401565, 1.508563, -1.645803, -0.544183, -1.979852, 0.363452, 0.566369, -0.171767, -0.317423, 0.887712, -1.738808, 0.523120, -1.425836, 0.533602, -1.364271, -0.107672, -0.657613, 3.219426, 0.784449, -1.487140, -1.507639, -0.375289, 0.232838, 2.006195, 1.000838, -1.090699, -1.398949, 0.032542, 0.492534, -0.230301, -0.746788, -0.506856, 0.113840, 0.564722, 1.828090, -1.602911, -0.816806, -0.081415, -1.315966, 1.079337, 0.876872, 0.541829, 0.477928, -0.017043, 0.099601, 1.075683, 1.781002, -0.947957, -1.673875, 1.331447, 1.492077, 0.218612, -1.037005, 0.604094, -1.435568, 0.076496, -0.202597, 1.996227, -1.309928, -0.191192, 1.151042, -0.395206, -1.615919, 0.348369, -0.887745, 0.889076, -0.548082, 1.804072},{-1.909311, 0.256751, -0.878922, -0.999516, 0.847835, -0.458694, -0.234183, 0.252260, -0.469333, 1.278249, -0.657011, 0.899957, -1.377803, -0.501294, 0.984903, -1.865052, 0.337839, 0.827653, -2.185416, -0.275249, 0.635957, -0.510032, 0.066600, -0.055925, -0.004419, -0.306368, 0.292120, -0.792659, 1.982174, 1.026836, 2.886519, -0.356171, -0.932221, -1.207133, -1.038400, 1.618961, 0.693618, -1.037559, 0.098691, 0.975222, -0.174972, 0.413044, -0.334095, -0.236825, 1.219005, 0.530925, 0.185702, -0.091784, -1.660759, 1.112897, 0.478267, -0.196989, -0.422976, 0.038558, 0.046331, 0.846929, 0.739102, -1.498341, -0.756373, 0.046704, 0.865226, 0.889904, 0.049399, -0.120171, -0.174506, 1.507328, 0.229679, -1.414101, -0.033864, 1.253042, -1.036721, -0.241006, 0.550313, -0.362760, 0.688173, 0.371484, 1.598928, 0.564817},{-1.145520, -0.341924, -0.221128, 0.674897, -0.420604, -0.652096, -0.892852, 0.817408, -0.983624, 0.517054, 0.232840, -0.273556, 0.083711, -0.691859, 1.655879, -0.028939, 1.595994, 0.316786, -1.384818, -0.116097, 0.467111, 0.593981, 0.572126, -0.675544, -0.764170, -0.287232, -0.287941, -0.275336, -0.251025, 0.745179, 2.858179, -0.683043, -0.599649, 1.384251, 1.596077, 0.174292, -0.401643, -0.410526, 0.134660, 1.038393, 0.168562, -1.613002, -0.615491, -0.140676, 0.919020, -1.024851, 0.855416, 0.830963, -2.429887, 1.035648, -0.199610, 0.469261, 0.708721, -0.113055, -1.643380, -1.693444, 0.483870, 1.330742, 0.142386, -0.591335, -0.325216, 0.969409, -0.860295, -0.194598, -0.218520, 1.377125, -0.446851, 0.639143, -0.164401, 1.706531, 0.128825, 1.042791, -0.435623, 1.656417, 2.543393, -0.437391, -1.507812, 0.237477},{-0.856480, 0.155918, 0.322681, 1.112500, 0.652968, 2.377812, 0.202935, -2.035964, 0.897724, 0.413028, 0.562113, -0.033829, 0.394300, -1.007519, 2.013854, -1.310022, -0.108301, 1.354251, 0.148619, -0.116028, -1.413678, 0.406538, -2.438252, 0.570119, -0.038574, 0.808257, 0.450804, 0.128310, -1.282097, 0.308442, 0.459456, -0.251301, -0.838452, -0.041147, -1.375552, 0.073808, 1.213293, 0.114019, 1.901862, -0.360576, 1.033604, 1.802748, -0.760179, 1.478117, 0.222337, 0.377707, -0.043196, 0.478486, 0.659873, -0.881474, 0.390524, 1.641883, -1.152596, -0.615322, 0.625179, -0.982402, 0.176898, 1.703536, 0.343688, -0.624323, 0.699581, -0.052614, 0.287106, 0.365110, 0.296158, 0.417116, -0.991507, -0.806294, -0.143680, 0.651239, -0.608836, 0.434513, 0.278117, -0.289288, 0.999843, 2.542320, 2.058616, -2.251131},{0.476516, -0.919886, -0.081602, -0.531369, 0.078375, -1.877103, -0.055758, -0.908610, 0.016050, 0.696436, 1.485603, -3.087224, 0.370704, -0.928257, 0.800225, -0.542186, -0.145134, -1.733676, -0.098058, 0.843927, 0.215286, -1.742390, 0.105511, 1.032902, -2.815959, -1.149040, 0.309964, 0.777766, 0.403029, -0.075875, 1.178847, -0.276084, 0.443758, -0.772693, 0.055983, -2.117726, 1.566959, 0.380701, 0.030462, 1.045886, 0.171812, 0.690470, 0.306490, -0.265983, 0.355452, 2.164336, 1.317607, -0.463005, 1.029508, 0.903740, 0.273963, 0.759041, 0.934836, 0.865285, -1.443869, 0.939559, -0.228922, 0.452852, 1.518180, -1.144919, -0.433941, -0.069409, -0.547931, -0.115873, -0.164200, 0.526340, -0.448240, 0.210635, -0.559733, 0.004924, -0.442587, -0.839917, -0.632743, 0.342389, -0.476777, 0.415221, -1.198304, -0.551692},{-1.195047, -1.751885, 0.742406, 0.382855, 1.292677, 0.178035, 1.516534, 0.835782, 0.440544, -1.054573, -0.337383, -2.233207, 1.808035, -0.396677, 1.196508, 0.237531, -0.207992, -0.344666, 0.555482, -0.458581, 2.067345, -0.438696, -0.297628, -0.667381, 0.786066, -1.886708, -1.002521, -0.022667, -1.805021, -0.652585, 0.784401, -0.579343, -0.095290, 0.994675, -0.133951, 0.342145, -0.437968, 0.023120, 0.513035, -0.192450, 0.952334, 0.464122, -0.490040, 0.548132, 0.357144, -0.950098, 0.220902, 1.364627, -1.623208, -1.441769, 0.052465, -1.224228, 2.654399, 0.321448, -0.457468, 1.197776, 0.858171, 1.280652, -0.453271, -0.075843, 0.133394, -0.716437, -0.181580, -2.054091, 0.993108, -0.870183, -0.263180, -0.441658, 1.345467, 0.792951, -2.182398, 2.211025, -0.369492, 1.376481, 1.250747, -0.426762, 0.627249, -1.081611},{0.522097, 0.405834, -0.689983, 0.317119, -1.204282, 0.101843, -0.711832, -0.465339, 0.021892, -0.072798, -0.326081, -0.179159, -0.144971, 0.624956, 1.293864, 2.346740, -0.360696, 0.020919, 0.680666, -0.108168, -0.832917, -1.836039, 0.544183, 0.477481, 0.443051, 1.596533, -1.494057, -0.831251, 0.123650, 0.595234, -0.145249, -0.483799, -1.088368, -1.662356, 2.947354, -0.088733, 0.031220, -0.635466, -1.431764, -0.730562, 0.238175, 0.265713, 0.126319, -1.064158, -0.178572, 0.069356, 0.665797, -0.411208, -0.052320, 1.025912, 0.475909, -0.177381, -0.270553, 0.160648, -0.399136, 0.832734, 0.399432, 0.549407, -1.128311, -1.183034, 0.338444, 1.351751, 0.411004, -0.733631, -0.615474, 0.105823, 0.066157, 0.249706, -0.508282, -0.630281, -0.291881, -0.088582, -2.174592, 0.243614, -1.978576, 1.828265, 1.216150, 1.396046},{-0.056871, 1.108141, -1.488493, 0.307751, 0.425097, -1.550845, -0.482240, -1.173238, 0.256476, -1.358859, 0.432239, -0.255442, 0.142223, 0.138684, -0.389881, -1.564614, 1.284971, -0.216759, -0.774951, -0.312429, -1.738747, 0.478630, -1.175223, 0.219039, 0.931257, -1.985487, 0.041317, 0.600549, 0.893220, -0.978707, -0.393039, -1.326408, 0.853757, 0.183063, 0.686403, -0.330046, -0.967316, -0.310915, -0.021624, 0.835208, -0.560879, 1.527843, 0.007213, -0.020945, 0.625870, 0.165676, -0.054830, -0.188798, -0.831008, 0.152168, 1.015674, 0.358257, -0.577928, 0.125113, 1.843105, 0.285106, 1.682348, 0.873286, -1.324355, 1.042234, 0.426161, -0.397082, -1.282214, -0.496901, -0.181276, -0.211436, 0.396162, 1.423175, -0.742988, -0.871904, 1.317077, 0.828357, -0.897915, -0.485511, 0.269753, 1.429572, -0.545081, 0.647387},{-0.207107, -1.530989, 0.577163, -1.591435, -1.364336, 2.511683, -1.181432, -1.178414, 1.893147, 0.211432, 0.336010, 2.711250, -0.731751, -0.191901, -1.252201, 0.000074, 0.319467, -0.453254, -0.363064, 0.875825, 1.308184, 0.804285, -1.735057, 0.185087, 2.321522, -1.721172, -0.377513, -0.515005, -0.189042, 1.895233, -0.169146, 0.527526, -0.107516, -0.869575, 0.624810, 0.528839, -0.402282, -0.080946, -0.449643, -0.573850, 0.779380, -1.132189, 0.934420, 1.434072, 0.103582, 1.094611, 0.712968, -2.720160, 0.344619, -0.410112, 0.276971, 1.431757, 0.303214, -0.327742, -1.181279, -0.498647, -0.441170, -0.199643, -0.443025, -0.783763, -0.670536, -1.014705, -0.836429, 0.837185, 0.242524, -0.638335, -0.013484, 0.452285, 0.595679, -0.855687, -0.348464, 1.409278, 1.523826, 0.379937, -0.105199, -0.802594, 0.634175, -0.507047},{-0.732178, 0.320009, 1.279300, -0.397497, -0.107913, -1.362262, 0.877773, 2.039920, -0.829703, -0.127845, 0.056993, 0.345366, 0.128789, 0.131599, 0.408878, -1.208310, 0.500222, 0.382173, -1.366209, 2.335696, 0.511852, 0.168662, -0.454130, 2.048929, -0.912380, 0.186989, 0.809508, -0.525613, 0.006935, 1.430287, -0.907160, -0.511776, 0.452910, -0.410354, 0.037818, -0.653543, -0.992338, 1.559077, -0.245213, -0.010752, 0.784179, -0.126786, 1.241537, -0.499783, 0.385894, 1.056692, 0.287885, -0.452851, 0.165088, -0.258328, -0.854211, -0.743449, 0.115827, -0.455905, 2.489750, 1.239205, -0.468855, 0.456827, -0.111196, -1.129135, 1.072395, 0.189597, 1.438951, -1.226790, 0.792385, -0.028831, -0.149240, -1.257055, -0.070151, -1.906316, -0.025290, 0.791695, -0.467021, -0.475638, 1.463027, 0.127286, 0.087547, 0.302954},{-0.408401, -1.309044, 0.444987, 1.885729, -0.061520, -0.122875, 0.272670, -0.422015, -0.242535, -0.405813, -0.303048, 0.623475, 0.796403, 0.976529, 1.195093, -0.005187, -0.206070, 0.694476, 0.383422, 0.586944, 0.525265, 0.416570, 1.515138, -0.126206, 0.593924, -0.439769, 0.686727, 0.341964, -1.177407, -1.708306, 0.363678, -0.376355, -1.292590, -0.189021, 0.082062, 0.885822, -0.618508, 0.679406, -1.593472, -1.494283, 0.759659, -0.564234, -1.060146, 0.480675, 1.199354, 1.040265, 2.137615, -0.693016, -0.507670, -0.925171, 1.385452, -0.989698, -1.181277, 1.802200, -0.032291, 0.894179, 0.825316, -0.521428, -1.606425, -1.505013, 1.034467, 2.052423, 2.117664, -1.146527, 0.931377, 0.872191, 1.647540, -0.900870, -0.714579, -1.023023, 1.985423, -0.552584, -1.671942, -0.927057, -0.734964, 0.576976, 1.518741, -0.261856},{0.548267, 1.121674, -0.073423, 0.924443, 1.321724, 0.512161, -0.452183, 0.529258, 0.519390, -0.397218, -1.063663, -0.717595, 0.145787, -0.004228, -1.051291, 0.534186, -0.441327, -0.978332, 1.777113, -0.901195, 1.812696, -0.362306, 0.309242, -0.577725, -0.501268, 0.607469, 1.395256, 0.489170, -1.376058, 0.267942, -0.155347, -0.807984, 0.195577, 0.864856, 0.175214, -1.666423, 0.659720, -0.729610, -1.179075, -0.516015, 0.922529, 0.862449, -0.846353, -0.688655, -0.183009, -0.818062, 0.095267, -0.301140, 1.419986, -0.118981, 0.690639, 1.098624, 2.382064, 0.089651, -1.209808, -0.874712, -0.086226, -0.403327, 0.419774, 1.192962, 0.203694, 0.738938, 1.175699, -1.086929, -0.776477, 0.024181, -0.138161, 1.342056, -1.414793, -0.727387, -0.792478, 1.267840, -0.070886, -1.187642, -0.807770, 0.554980, 1.598443, 0.198155},{0.302549, -0.029950, 0.480528, -0.178897, -0.277440, -1.107416, 0.152933, 1.435951, -0.110486, -1.262649, -1.041931, 1.447419, 0.531887, -0.668678, 0.227306, 0.306498, -1.785963, 0.207545, 0.428192, 0.011331, -0.186677, 1.634910, -1.359665, -0.078369, 0.712147, -0.726978, 1.146374, 0.641766, 2.714779, 1.416654, -1.036946, 0.441375, 1.516624, -0.519510, -0.323449, -0.850097, 0.758273, -0.262894, -0.265242, 1.176802, 0.886861, 0.306648, 0.163226, 0.544080, -1.261228, 0.153850, -2.335450, 0.960944, -0.476114, 1.471280, 1.215637, -0.116191, 1.729850, -0.083019, 1.657275, -1.614395, 1.222911, 0.590065, -0.794385, -0.277407, 0.181350, -0.474636, -0.858259, -0.134735, 0.109699, 1.073097, -0.868032, 0.186681, -0.914113, 0.061813, -1.303627, 0.147293, 2.075909, -0.229975, 0.529642, 0.780577, -0.269082, 0.707522},{0.779832, 0.059095, -1.105844, 0.148607, -0.188988, 0.341700, 1.594957, 0.216587, 0.294588, -0.983012, -0.512005, -1.180614, -0.268552, 1.627715, 0.741392, -0.259450, 0.317892, -0.519909, 0.382329, -0.196752, -1.187661, -0.255610, 0.806118, -0.760971, 0.507997, -0.103040, 0.443061, 1.005072, -0.469463, -0.469471, -0.588652, 0.510567, 1.524095, -0.532410, -0.197379, -0.591434, -0.358359, 0.590086, -1.107799, -1.864768, -0.796599, -0.893871, 1.189658, 1.050619, 0.544884, -0.174165, 0.389509, 2.919211, -0.817160, -0.729503, 1.907014, 0.222574, -2.192920, 1.324768, 0.143550, -0.605526, -0.924522, -0.483239, 0.479738, -0.753178, 1.391695, -0.381960, -0.462852, -0.957919, 0.426000, -1.158685, 0.672826, -0.166493, 0.503395, -0.927499, -0.763335, -0.658518, 0.017020, 0.429617, 1.462865, 0.200725, -1.107864, 0.661019},{0.432047, 0.895750, -0.827415, 1.755205, -0.855019, 0.418827, 0.869101, 0.463705, -1.018184, 0.851474, 0.771903, -1.254502, 0.179973, 0.163464, 0.222553, -0.465711, -1.647020, 0.030902, -1.186291, -0.802615, 0.235014, 0.224277, 0.182232, -0.564180, 1.557852, -0.494798, 1.477021, 0.273297, -0.230274, 0.305140, 0.213422, 0.242273, -0.568185, 2.222052, -1.876315, -1.539616, 1.501543, 0.411039, 0.043348, 3.061569, 0.781687, 0.338055, 0.865021, 0.220536, -0.918620, -0.259470, -0.975681, -0.257198, -0.549430, 0.379066, -0.268568, 0.273688, 0.342287, -0.481687, -1.302073, 2.081645, 1.007044, 0.445836, 0.062241, -0.926756, -0.328722, 0.881303, -1.540532, -0.118796, 0.286746, -0.303875, -1.100619, -0.758238, 1.252733, -1.628397, 1.558288, -0.283081, 0.498565, -0.274465, 0.560576, -0.502449, -1.472752, 0.198875},{-0.684850, -0.870906, 0.725008, 0.723685, 0.354812, -1.709247, -0.984374, 1.019279, -0.899533, -0.926328, -0.053536, 0.048079, -1.306434, 0.478227, 0.051209, -1.100748, -1.126348, 0.727270, -0.563291, -0.077934, -0.705657, -1.498233, 0.559448, -0.661725, -1.001327, 0.563756, -0.839903, 0.517067, -0.485890, -0.183689, 0.212951, 0.620957, 2.077535, 1.678884, 0.535555, 2.071225, 0.208852, 0.842424, -1.006323, 1.203766, -1.179959, 0.201730, 0.320708, 0.109753, 0.120762, -0.607356, 1.013688, -1.461354, -0.836828, 0.581255, 0.323272, -0.155753, 0.447015, -0.749306, 1.742374, -0.975620, 0.421902, -0.624759, 0.995300, -1.236236, 0.165305, 0.251489, 0.201603, -0.381049, 1.665245, 1.856458, 0.448882, 0.599383, -1.148804, 0.427412, 0.951758, -2.199366, 0.871908, -1.868477, -1.960107, 0.192676, -0.271888, 0.202302},{-0.922972, 0.984533, -0.572659, -0.114169, 1.540137, -0.586917, -0.553874, -0.606984, -2.374557, 1.273688, -0.309839, -0.732496, 0.267673, -0.010769, 0.598263, -1.768622, -0.062270, 0.933365, 1.887966, 1.895374, 0.562093, 1.479507, 0.422502, 0.110482, 0.186204, -1.582513, -0.236099, 0.393688, -0.369913, -0.113290, 0.766610, -0.316462, 0.821522, -0.479367, 0.685343, 0.991616, 0.064497, -0.079905, 0.627110, 0.698080, 3.052130, -0.483921, 0.392877, -0.936833, 0.829740, -1.356362, -0.230918, 1.610012, 0.085208, -0.355228, 0.221929, 0.502088, 0.738754, 1.050601, 1.122206, 0.984325, -0.485332, 1.530442, -0.340911, 1.912163, -0.750908, 0.601510, -0.483000, -1.610865, -0.438776, -0.457147, -0.501571, 0.050494, 0.514156, -1.999121, -0.164690, -1.243006, 1.867022, -0.667332, 0.903031, 0.304879, -0.744818, -0.599063},{1.747321, 1.882183, -0.163223, 0.485646, 1.036425, -1.388326, -0.429741, -1.100210, 0.628626, -0.797486, -0.529664, 0.623754, -0.291339, 0.505184, -0.855019, -0.191210, -0.341172, -1.224459, -0.125735, -0.020949, 1.178402, -0.213218, -1.138203, 0.159715, -0.163306, -0.237541, 0.021602, -0.198800, -0.804764, -0.031041, -0.967652, -0.519421, 1.523180, 1.433609, 0.968203, 1.024276, -0.626670, -1.424535, -0.141803, -1.262095, -0.552842, -0.901595, 0.729597, -0.987202, 0.767964, -0.315832, -0.601737, -0.791065, -0.037888, -0.435986, 1.163827, 1.279024, -0.223010, 0.148667, 0.906911, 0.271591, 0.048218, 1.147208, -1.417248, 0.114291, 0.018805, -0.804620, -1.606770, 0.792990, -0.133602, -0.797003, 0.024094, 1.603764, 1.390078, 0.490670, -0.005914, 1.574288, -0.724865, 1.799318, 1.550922, 2.504335, -1.201625, -2.464565},{-0.512035, 0.001547, 1.548704, -0.929612, -0.391871, 1.299908, 0.108145, 2.229754, 0.268177, -1.910056, -0.353067, -1.193376, -1.251883, 0.904484, 1.906232, -0.310044, 0.209649, 0.029253, -2.028744, 0.062085, 1.704620, 1.846219, 0.382475, 0.356542, -0.513582, 0.515485, 0.983546, -0.429016, -0.227163, 1.264762, 0.537878, 0.837980, -0.770535, 1.234670, -0.362885, 1.027188, 0.571786, -0.804311, 1.064992, 1.808458, 0.279176, 0.786041, -0.875258, -0.106560, -0.441976, -0.705174, -0.058529, -0.203801, -0.092600, 1.856127, 0.439321, -0.330185, -0.370705, -0.616022, 0.774190, 0.922842, -0.105363, -0.531536, 0.594023, 1.272006, 0.799555, 0.152080, 0.684083, 0.336895, -1.102947, -0.887499, 1.425466, -0.187189, 1.361167, -0.377746, 0.760321, 0.250923, 0.073215, -0.247010, 0.093338, 1.402210, -0.389386, -0.343671},{-0.808887, 1.863835, -1.196712, -0.020142, 0.882994, 0.418630, -0.507480, 0.358443, 0.316099, -1.041611, 0.636353, -0.258043, -0.151060, -1.616344, 0.749286, 0.358511, 0.381088, -0.157464, -2.765106, -0.595181, 0.860362, -0.476204, 0.791640, -0.673396, 0.509517, -0.020320, 0.329221, 0.291920, 0.197639, 1.282340, 1.821028, -0.247608, -0.566633, -0.306569, 0.296387, 0.356131, -1.157276, 1.358976, -1.461270, -1.163289, 0.710486, -1.300723, -0.697058, 1.892112, -0.170106, -0.409516, 0.053985, -1.673245, 2.475991, 1.062439, -0.284189, -0.818018, 0.074940, -0.258750, -1.816610, 0.994734, -1.317304, 0.664450, -0.152354, -0.961766, -1.346987, -0.168270, -0.765877, 2.515760, -1.433525, -0.267453, -2.668757, 0.307179, 0.426976, 1.214815, -0.366579, 0.854922, -0.459463, -0.434242, -0.906222, 0.389164, -1.530646, -0.980105},{-0.560972, -0.188553, -0.916924, 1.029485, -0.396511, 1.768164, -0.430902, 1.501650, -0.412015, -0.203439, -0.055207, -1.004411, -1.469800, -0.306286, 1.130719, -0.288710, -1.093723, -0.017047, -0.155989, 1.305084, -2.244333, -0.337763, -0.583308, 0.656213, 0.494966, -1.190260, -0.888572, 1.350993, 0.188395, -0.546856, 2.383114, -1.606898, -0.615164, 0.856951, 0.035298, 1.307829, 2.023604, -1.330547, 1.570452, -0.749864, 0.665388, -0.310380, -1.187194, -0.530819, -0.357728, -1.404002, -0.564934, -1.377298, -0.291269, 0.046064, -0.362291, -0.487097, 1.216563, -0.816366, -1.488379, -0.027541, 1.114372, -0.575748, -0.090069, 1.412862, -1.027761, -1.095480, 0.802125, 0.184316, 2.312897, 0.518810, -0.838718, 0.191831, 1.955875, 1.414588, -1.017104, -0.778064, -0.095229, 0.233204, 0.942696, -0.180324, 1.005151, 0.820272},{-0.178578, -0.923423, -0.925289, 0.380314, 1.466871, -0.842013, 0.380202, 0.759980, 1.074641, 0.602922, -0.999977, -1.951885, 0.383013, 0.804673, 0.528454, -1.785198, -2.303143, -0.755418, 0.188094, 0.176172, 0.793176, 0.035347, -0.236496, -0.529142, 1.980115, 0.156774, 0.096322, 0.801243, 1.823510, -1.695501, 0.404638, -0.776599, -1.965889, 0.231616, -0.985666, 0.880679, 1.526268, 0.570141, -1.729555, 0.611723, 1.178176, 1.724980, -1.589314, -1.382730, -0.113269, 0.593095, 0.910452, 0.733193, -0.218833, -0.484848, 2.660027, 0.815928, 0.732550, 1.356188, 1.015331, 1.221072, 0.849828, -0.742274, -0.050521, -1.929861, -0.237383, -1.325432, 0.512449, 1.086421, 1.445139, 1.026216, 0.964309, 1.035591, -1.759688, 0.561089, -0.226128, 0.017615, 1.353283, -0.876539, 1.382729, 0.126553, 1.436989, 1.249899},{1.376498, 0.117318, 1.325356, -0.069760, 0.288720, -1.847172, 0.864770, 1.160953, 1.463266, 1.229714, 0.438910, 0.249342, -0.127284, 0.520105, -2.070499, -0.290806, 0.369345, -0.766095, 0.625895, -1.357830, -0.719866, -2.335952, 0.271938, 0.823986, 0.174929, 0.071087, -0.055294, -1.340662, -0.330181, 1.637161, 0.209098, 1.063554, -0.170622, 0.515116, -1.417851, 1.246112, -1.546821, 0.156778, -1.344933, -1.075926, -0.586133, -0.776296, -0.129470, 0.619169, -0.460014, -0.103363, 0.128871, -1.672689, -0.256005, -0.804588, -0.009591, 0.124957, 0.425915, -0.278213, -0.973410, 1.524135, -0.676927, 0.217427, -0.461868, -1.236014, -0.930619, -0.220349, 0.603346, 1.090213, -2.583587, -0.059162, -0.096959, -0.219464, -0.629848, 1.905348, 0.052968, 0.385604, -0.211130, 0.394370, -0.489049, 1.194867, 0.473944, -0.159948},{-1.761018, 0.000299, 0.180117, 1.232829, -0.305635, -0.592497, 0.269839, 1.372273, 0.901550, 1.274565, -0.706014, 0.075116, 0.278845, -0.779482, 1.723669, 0.649000, -1.026319, -0.891921, -0.770815, -1.826534, -0.920175, -0.923220, 0.411042, 0.939833, -0.932317, -1.876217, 1.465428, 2.282834, -1.499510, -1.533266, 0.612982, 0.141928, -1.330738, -1.617424, -0.947513, -1.957246, -1.603828, -1.088753, 0.656778, -0.265332, 0.659452, 0.439678, -0.097647, 0.310901, 1.269727, -0.145432, -1.887977, 1.598877, -0.396247, -0.321235, 0.301301, -0.556237, 1.128903, -1.521631, -1.052897, -0.775611, 0.881011, 0.965151, -1.489432, -1.402697, -1.071823, 0.216992, -1.094857, -1.742637, 0.958530, -0.414090, 0.747506, 1.924176, 0.453496, -0.506872, 0.761563, 0.229160, -0.457624, -0.347789, 0.147986, 0.338435, -1.365705, 2.100489},{0.693765, -0.158403, -0.001237, -0.790555, -1.249737, 2.724367, 0.061519, 0.321481, -0.014504, -0.391628, -1.319074, 1.103137, 0.514600, -1.148066, 0.406894, -0.395326, 0.119147, 0.363928, 0.342393, 2.003862, 0.094031, -0.206941, -0.629133, -0.538252, 2.822415, -0.311536, -0.000464, -1.294296, -1.166544, 0.408682, -0.285136, -1.244199, -1.007359, -1.255750, 0.113052, -0.687745, 0.576534, 0.613301, -1.663826, -0.604541, 0.357110, -0.628121, -0.296433, -0.788778, -0.548893, -0.243828, -0.353998, -1.350578, 0.984320, 0.668858, -2.202821, -0.275280, 0.428187, -1.965792, -0.570061, 1.345932, 2.180122, -0.018802, -1.251625, -0.147865, -0.858231, 1.222833, 1.176338, 0.460661, -2.496679, -0.355803, -0.394140, -2.338780, 0.806623, 0.847151, -0.414959, 0.382378, -0.210071, 1.865946, 0.534049, -0.342411, -1.017897, -0.877936},{0.782954, 2.255769, -0.347201, -1.217542, 0.129429, 0.350782, 0.954966, 2.575108, -0.300032, 1.242731, 0.193795, 0.594747, -0.334197, 0.335606, -0.471200, -1.429433, -1.303756, -0.523578, -1.772016, -0.319669, 1.456796, -1.477092, -0.408138, -0.142213, -1.034372, 0.724196, -0.952797, -0.548710, -0.115696, 2.331300, -1.271588, 1.143996, -1.145269, -0.590769, 0.214977, -0.488257, 0.996909, 0.021004, 1.290112, -0.235530, -1.462812, -0.150416, -1.341210, -0.304800, 0.493541, -2.073643, 1.245151, 0.111423, 0.799049, -0.403929, -0.687636, -0.507172, -0.950270, 0.815876, -0.192919, -0.233803, 0.183664, -1.377393, 0.570758, -0.193795, 1.254950, 0.115457, 0.821297, 0.174096, 0.912322, -0.848490, -1.279044, 0.970329, 0.135997, -0.301784, 1.237342, 0.915805, 1.066565, -0.177879, -0.433926, 0.990353, 0.390867, 0.834354},{0.821687, 0.488946, -0.384955, 0.493563, -0.139254, -0.301286, -2.140271, -0.130234, -0.909544, -0.959303, -0.268163, -0.246383, -1.551740, 1.840851, -1.034316, 0.854570, 0.294290, 0.399188, -2.067619, -1.136071, -0.675098, 0.451668, 0.473475, -0.310588, 0.201164, 0.849011, -1.050000, -1.150755, -1.122887, -1.822905, -0.138502, -0.969393, -0.441929, -0.443277, -2.891556, 0.627328, 0.465668, 1.378438, -1.388188, -2.234387, 0.472888, 2.407999, -0.889241, 0.372981, 0.302670, 1.795828, -0.419566, 1.109879, -0.946613, -1.316517, 1.038652, 0.220653, -0.282505, -0.163109, 0.746678, 0.609125, -1.099721, 0.124904, -1.301862, -1.582324, 0.836443, 0.251569, 0.552529, 0.791096, 0.176664, -0.751565, -0.982669, 0.210600, -1.872161, 0.726907, 0.045503, -0.500131, -0.647016, 0.613078, 0.513505, -0.242012, 0.800048, -1.255730},{1.545392, 0.292322, -0.535896, 0.240016, 1.314263, -0.636052, 0.508196, 0.210302, 1.380934, 0.000417, 0.131803, 0.382589, 1.571596, -0.298914, -1.037406, 1.112983, -0.610265, 0.022751, -1.000450, 0.111735, 0.332050, -1.709974, -0.541121, -0.673786, -0.355486, 1.287360, 1.141762, 1.630599, 0.283138, -0.150448, -1.011431, -1.943000, -0.408190, 1.830944, 1.316371, 1.295583, 0.965575, -0.798477, -2.214569, 0.218113, -2.730390, -0.631306, 1.479663, -1.180738, -0.004421, 2.148420, 0.769664, 1.184664, -2.480962, 0.422083, 1.174371, 0.352990, -1.663019, -0.134602, -0.710519, 0.718887, 0.041333, 1.270635, 0.209750, -1.147717, 0.830876, 0.501508, 0.191622, -0.120754, 2.042568, -0.210775, -0.341230, -1.798201, 0.792968, -0.073800, -0.786772, 0.523110, 1.127243, 1.864145, -0.370459, -0.099134, -1.157479, 0.867817},{1.131016, -0.860727, -0.995118, -0.358749, -0.007448, -0.073123, 1.623360, -1.700585, -0.950277, -0.182541, 1.057357, 1.487224, 1.724075, -0.087223, -0.698209, -1.683530, 0.460320, 1.873162, -0.248167, -0.190293, -0.582524, 0.275485, 1.863249, -0.529252, -0.936532, -1.364852, -0.447956, 0.434795, -0.538614, -0.918729, -0.229519, 1.124868, -1.199487, -0.818228, 0.500379, 1.395880, 0.087719, -1.930908, 0.765393, 0.607559, -0.423751, -1.557793, -0.362265, 1.076723, 2.167471, 0.197752, 0.725404, -1.108248, -1.189652, 0.449924, 1.550245, 0.062665, -0.064697, -0.101532, -1.246659, -1.294179, 0.760158, 1.442326, 0.926825, 0.951647, 1.072251, -0.505848, -0.500933, 1.021674, 1.779137, 0.053557, 0.011634, 0.897070, 1.131293, -0.179806, -0.775864, -0.181585, 1.008195, 0.427412, 0.014467, -0.015568, 0.852978, -0.718902},{1.103103, 0.993554, -2.077634, 0.347947, 0.095316, 0.972648, -0.679232, -1.297440, 0.468001, -0.237252, 0.251224, -0.598477, 0.002000, -1.124995, 2.014684, -1.455246, 1.494347, -0.274449, -0.617443, -0.285913, 0.730931, 0.495959, -0.882203, 0.142663, -0.508494, -0.085233, -0.831674, -1.627256, 0.103032, 1.212028, 1.039008, 0.928793, 0.814673, 2.691411, 0.044563, -0.305985, -0.941187, -0.195425, -0.447451, 0.008403, 0.596807, -0.807418, -1.143537, 0.480104, 0.849393, -1.006804, 0.966693, 1.083784, 1.125273, 1.650031, 1.429062, -0.701624, -0.243606, -0.598778, 1.046055, -2.044941, -1.485280, -2.597156, 1.066147, -0.264382, 0.778477, -0.070879, 0.182300, -0.913025, 0.554990, -0.402876, 0.718522, -0.655126, -0.838868, -0.702317, 0.290818, -1.112908, 0.391190, -2.348211, 0.280729, -0.784828, -1.093070, 0.742839},{-0.014831, 1.748901, 0.152845, -2.403297, 0.561168, -1.446404, -1.186993, 0.306914, 0.524107, -0.285245, 0.707480, 0.448176, 0.702236, 0.016420, 0.287789, 0.051819, -1.151910, -0.755187, -1.920720, 1.960033, 0.501828, -0.173118, 1.250249, -2.137029, 0.002543, -0.358117, 0.324157, -0.268126, -1.643223, 0.395734, -1.170941, 0.611818, 0.647129, -1.375999, 0.826623, 0.160812, 0.282352, -0.409885, 1.803930, 0.005874, 0.555459, 1.385143, -0.751126, 0.762139, -0.838056, 0.016281, -0.088573, 0.369840, -1.079548, 0.940957, 1.258269, 0.696023, 1.913426, 0.191395, 1.393309, -0.912237, -0.185749, -1.235290, 1.079023, -0.087322, -0.496215, 0.455637, -0.467581, 1.070850, -1.674037, 0.282578, 2.045490, 2.432102, -1.530616, 0.210232, 0.256082, -0.883393, 1.924927, 0.635006, 1.605072, -0.373577, 0.079605, -1.101828},{1.245986, -0.827924, 0.936451, -2.262310, 0.315788, 0.460888, 1.043853, -0.216981, -2.258498, -0.956804, -1.389122, 0.861987, -1.411439, 0.390374, 0.086542, 0.265811, 0.865553, -0.620280, -1.629028, 1.828419, -0.953889, -0.085693, 1.180909, 0.895642, 0.528520, 1.167979, 1.667965, -0.141732, 1.368293, 1.316888, 1.311176, 0.715935, 0.113244, -0.332234, -1.147216, -0.645211, -0.385889, 0.122769, 1.937562, -0.182098, -1.069857, -1.367619, -1.331136, 0.315601, -0.886398, 0.164139, 1.323778, 0.850036, -0.641617, 1.690393, -0.262157, 2.587822, -0.143572, 2.292947, 0.024786, -0.279432, -0.838291, -0.306691, -0.618193, 0.983175, 0.312077, -0.132831, -1.749446, -0.718321, -0.842996, -0.318495, 1.578802, 1.919868, 0.850778, -1.630826, 1.784841, 2.637478, -0.215291, -1.815264, 1.504027, 0.662499, 1.733858, -0.627625},{-0.665558, -0.711855, 0.246060, 0.283987, -2.624208, -1.382689, -0.912380, 0.174684, -0.668197, -0.730113, -1.125492, 1.070577, 1.494777, 0.295511, -0.836062, -1.433046, 0.689545, 1.055044, 2.603152, 0.617436, 0.029022, -0.597231, -0.293348, 0.306139, 0.608345, 1.106045, 1.068081, -0.923478, -0.528179, 1.150568, -1.540438, 1.016885, -0.950785, -0.039436, 0.406231, 0.060839, 1.151417, -0.173209, -0.987295, 0.398817, 0.820638, -0.228976, -0.954195, 1.156636, 0.401587, -0.842299, 0.668042, -0.212238, -1.027107, -1.029360, 0.891478, -1.461182, -0.476540, 0.431802, 0.503563, -1.592213, 1.826740, -1.211702, 0.130141, 1.246280, 0.200907, 0.629133, 0.241774, -1.130156, 0.987822, 0.657279, 2.433442, -0.345661, 0.452807, -1.028098, 0.799383, -0.863834, 0.509163, 1.583389, 0.670910, -1.202701, -0.034324, 1.888662},{0.566362, 0.729422, 2.086843, 0.182729, 0.653242, -0.222037, 1.309775, -0.149118, -0.896804, 0.057593, -0.101186, 0.749192, -0.561928, -1.451614, -0.330506, 0.732752, -0.259047, 1.253631, -1.118365, 0.401510, -0.784326, 0.718477, 0.950726, -0.709066, 0.500841, 0.507206, -0.994338, -0.694071, 0.734227, -0.424158, -0.758345, -1.581643, -1.336247, 0.465704, -0.696470, -0.832847, 1.163558, -0.222032, 1.124799, 0.077238, 1.364134, 0.707624, 0.463941, 1.108330, 0.100010, -0.664331, 0.448403, -0.373382, -2.380509, 0.875158, 0.585024, -0.638929, 1.215779, 1.822487, 0.375904, 0.913276, -0.023737, -1.509240, -0.839764, -0.323304, -0.226035, -0.120068, 1.732448, 0.606660, -1.011428, -0.456762, -0.410854, -0.157056, 0.151944, -0.885594, -0.799940, -0.055080, -1.123783, 0.975429, 0.174031, 1.497649, -0.116503, -0.938349},{-0.194859, 0.771800, 1.025704, 1.251063, -0.708206, -0.871400, 0.897480, -0.962763, 1.329723, 0.295738, 1.010061, -0.070239, -0.251355, 0.942375, -0.712824, -1.641876, -1.459464, -2.159908, 0.470511, -0.865685, 0.069313, 1.216087, 1.215476, 0.132992, 0.883975, -1.146276, 2.621047, -0.596836, -0.579406, -1.208559, -0.507117, 2.127355, 0.869550, -1.467865, 0.592818, -0.265001, 0.236683, -0.959230, -0.363906, 0.203101, 0.158686, 0.267029, 0.088705, -0.882790, 0.649972, 0.121983, 0.647944, -0.368530, 1.573843, -0.654757, 0.728000, 0.247551, 0.885179, -1.223254, -0.298221, 0.486784, -0.098983, 2.374551, -1.534714, 0.585222, -0.548906, 0.023118, 1.706310, 1.602497, -1.448924, -1.112360, 1.094728, 0.261754, -0.865379, 0.262104, 0.493671, 0.228154, -0.097242, 0.040818, 0.332466, 1.582315, 1.682551, 1.042315},{0.302049, -1.340695, -3.075905, 0.380821, 0.185592, 1.314708, -0.627845, -0.202272, 0.298364, -0.113848, 0.004511, 1.104572, 0.795038, 0.404890, 0.350096, -0.904139, -1.047697, -0.459078, -1.451047, -1.131076, -0.797377, -0.582647, -2.403083, -0.658256, -1.053421, -1.901606, -1.729715, 0.598530, -0.413439, -0.739375, -0.318218, 0.208036, 0.601703, 0.062345, 0.214210, -0.169200, 2.253343, -0.015724, -0.592163, -0.552874, 0.193952, -1.501548, 1.034217, -0.350043, -0.307432, -0.433612, 0.363828, -0.215179, 0.057623, 0.327609, 0.170047, -1.607694, 0.276923, 0.455207, 0.713796, 0.326545, -0.961400, -0.351263, -0.416681, -0.839344, -1.146812, -0.502174, 2.334575, -0.129328, -0.684916, 0.671800, -0.866661, -0.162731, 1.094504, 0.565979, -0.895071, 0.383259, -1.480025, 0.912008, 1.218781, 1.806585, -0.349402, -0.825171},{-0.376063, 0.488863, -0.232834, 0.077252, -0.318453, 0.539167, 0.831241, -0.021881, 1.262962, -0.099745, -0.775321, -1.460563, -0.105988, 1.232737, 1.625917, 0.385645, -0.016130, -0.700590, 1.384684, 0.328548, 0.527896, -1.392225, -0.720592, -0.712368, 1.211626, 0.004584, 1.094263, 0.409158, -0.349682, -0.967205, 1.290934, 0.494683, 0.399645, -0.241944, -0.751643, -0.370586, 1.209535, -0.052310, 1.834678, 0.736686, 0.295126, 0.226811, 0.528027, 1.560508, 0.578068, 1.041342, -0.114481, 1.286297, 0.510903, -0.478598, 0.688541, -1.586433, 0.899509, -1.656018, -0.787953, -0.229305, -0.947756, 1.843980, 1.058838, 0.463394, 1.613378, 0.938962, 0.753453, -1.222766, -1.303165, -1.184669, -1.041203, 0.570121, 1.435213, 1.952487, 0.288962, 1.654633, 0.438717, -1.913364, -1.523806, 0.371014, 1.094661, -1.078692},{-0.439191, 1.064512, 0.723836, -1.558427, 0.122681, 0.427952, -1.054402, 0.632273, 0.810331, 0.535811, -0.846973, 0.180365, 1.962297, 0.323204, 1.217570, -0.725175, -0.683001, 2.120715, 1.202330, -0.399156, -0.007181, 1.317142, 0.323231, -0.179812, 0.658117, -0.874558, 0.562271, 0.900098, 0.722691, 0.182167, 0.393310, 0.830917, 0.815863, -0.522509, 2.033072, 0.430277, -2.152539, -0.231905, 0.360947, 0.047904, 0.835090, 2.404288, -0.835962, -2.298092, 0.887298, -0.090498, 0.933873, -1.732655, -0.035107, 1.325319, 1.060914, -1.355767, 0.351713, -0.029437, 0.362379, -0.481475, 0.873225, 1.743227, 1.562034, 0.891764, 0.011815, 0.315599, -0.196376, 0.487547, 0.430973, 0.539670, 1.035308, -0.324200, 0.574101, -0.296858, -0.518514, 1.281315, 0.177567, 0.223105, 0.412803, -0.403116, -1.539477, -1.432425},{1.727179, -0.571168, -0.201931, 0.477542, -0.320757, 0.750403, 0.575376, 0.030020, 0.521233, 0.752038, -0.903808, 0.367918, -0.708883, 0.865717, -0.088988, -0.426283, 0.117092, -0.973927, -0.999275, -2.141449, -0.526815, -0.585351, 0.676823, 0.114879, -0.730127, -0.637525, -0.478076, 0.434321, -0.421462, 0.932752, 1.361567, -0.055924, -0.830224, 0.131914, -1.546931, -1.076635, -1.740578, -1.220815, -0.240222, -2.313844, -0.657734, -0.685655, 0.342963, -0.293047, 1.490613, -0.426541, -0.336064, -0.335989, 0.773199, 0.006953, -1.100501, -1.393280, 0.844794, -1.181467, -0.546744, -0.195379, -1.253090, -0.323371, 0.540306, -0.549800, -1.113576, 0.189958, -0.210357, -0.594051, 0.254950, -0.265423, -0.777045, -1.088969, -0.928643, 0.315729, -1.497810, 0.822960, -1.065655, -1.513340, -1.944590, -1.294420, 1.082889, -1.852242},{0.283381, -0.169007, 0.364855, 0.961408, 1.192749, 2.308808, 0.886478, -0.314500, 1.619111, -0.606058, 0.038281, -1.188813, 2.341379, -0.496453, 1.701211, -0.274609, 0.516187, 0.645139, 0.481172, -1.401073, -0.131801, 1.573555, 0.257926, -2.640678, 1.742645, 0.741028, -0.182831, -1.448507, 1.145775, 1.363336, 0.715959, 1.470886, 1.452354, -0.748481, -1.000145, 0.622084, 0.523740, 0.196222, 0.598479, -2.016125, -1.381907, -4.036925, 2.116927, -0.381760, -1.346780, 0.603443, -0.178522, -0.649838, -2.043209, -0.175398, 0.324978, -0.939249, 0.363791, 0.879010, -0.340126, 1.760235, -0.794501, -0.434346, 1.752350, -0.798197, 0.613238, 1.197539, -1.182922, -0.082517, -0.890256, 0.634351, 1.196550, 1.102693, -0.686199, 0.243724, -1.097880, 0.380239, -1.682652, -0.023916, -0.470344, -0.232986, -1.197580, -0.548729},{0.226272, 1.781736, 0.529782, -1.025748, -1.278879, -0.243266, -1.247134, -1.328438, -0.865450, -0.347479, 0.173748, -0.838681, -0.580734, 0.681140, 0.812982, -0.379343, 1.481757, 0.183799, -0.959991, 0.828031, 0.250895, 0.978788, 2.429743, 1.867255, -0.764379, 0.131654, -2.137318, 1.005564, -0.058951, -0.285476, 1.990978, -1.562273, -0.082363, -0.317814, 1.551334, 0.942287, 0.018553, 0.733948, -0.039747, 1.212962, 0.956824, -1.324080, -0.644455, -0.239849, 1.238600, -0.040802, -0.318086, -1.704570, 0.675657, 0.489758, 0.144678, 0.364657, -0.704770, 0.378404, -0.095352, -0.788889, 0.368192, -0.598753, -1.557220, -0.205928, 0.484653, 0.466328, 2.171555, -0.150843, 1.895810, 1.308898, -0.392542, 0.507648, 0.625831, -0.978162, 0.628570, 1.453760, -0.500998, 1.007270, -1.624569, -0.809724, 0.400598, -0.600267},{-1.246478, 0.566076, 0.854787, 0.103172, 0.274746, 0.825426, 1.482793, 0.053404, -1.468219, -1.380790, -0.417214, 0.306697, -0.471072, 0.777629, 1.120886, -0.407647, -0.089226, 2.743185, -0.530118, -0.626921, -0.904790, -0.037084, 0.109279, -0.014061, -0.544617, -0.062811, 0.934292, -2.251238, 0.751737, -1.310837, 0.431349, -0.678429, 1.258064, -0.260209, -1.477970, 0.770571, 1.447654, 0.974468, 0.096869, 1.582658, -0.266819, -1.112690, -0.386066, -0.805803, 0.041513, -1.613746, -0.701203, -0.717774, -0.402155, 0.720351, 1.150089, 0.477565, 1.401794, -0.760007, 0.179287, 0.361311, -0.667495, -0.118065, 0.042076, -3.041979, 0.376912, -0.195140, -0.037829, 0.501301, -0.004621, -0.914703, -1.313027, 0.978486, -1.808976, 0.622645, 0.702234, 2.328907, -0.508431, -0.980316, -0.510541, 0.421628, 1.495220, 1.108376},{0.391423, -0.595498, 0.811642, 1.688620, 2.693810, -0.551264, 0.757702, -0.573202, 1.010710, 1.442141, -1.647238, 0.975098, -0.465089, 0.189169, 0.133244, 0.109080, -1.109308, 0.276706, 0.692738, -0.502977, 0.635351, 0.134015, 1.067605, -1.157033, 0.944786, -0.174450, -0.484072, -1.355286, -1.687200, -0.008309, -0.855827, -0.054940, 1.855329, -0.869327, -0.515369, -0.024135, 0.255798, 1.711910, 0.857613, 0.065055, -0.179451, 0.127631, 1.659671, -0.107946, 0.486734, -0.222233, -0.368831, -1.033248, 0.746718, -1.701033, 1.842582, -0.742778, 1.297574, 0.766199, 1.491214, 0.004433, 0.397158, -0.875429, -0.002901, 0.867056, 1.248832, 0.090058, -0.488615, 0.035828, -1.273569, 0.831188, -1.931104, -0.614764, -1.452085, -0.774812, -0.230417, -1.053958, 0.543426, 0.022416, 0.097750, 1.317797, 0.632840, 0.679742},{0.943285, 0.091593, 0.607000, -1.494252, 0.476937, -0.130731, 0.815024, 0.456229, -0.583490, 0.313657, 0.026850, 0.690892, -1.576502, -0.309446, 0.410838, 0.146686, -1.167795, -0.316138, 0.603206, 0.559363, 0.669134, -1.184128, -0.457798, -1.118219, -1.062601, -0.335294, -0.388017, -0.725211, -0.598918, -0.714366, -0.593662, -0.533814, 0.803366, 0.269449, 0.227105, -0.668012, -0.058835, 0.881672, 0.666076, 0.387799, 1.065037, 2.053689, -0.127049, 1.782427, -0.080480, -0.343430, 0.189639, 1.816674, 1.978945, -1.376628, -0.185083, 0.418161, -0.597986, -2.463723, 0.087134, 1.362553, -0.170917, 0.242641, 0.619936, 0.832045, -0.147170, -1.254082, -0.002685, 0.210555, -0.606749, 1.453235, -0.986164, 0.852564, 0.844535, -2.875067, -1.602073, 2.216542, -0.298496, -0.433971, 1.514928, 2.980917, 0.200844, 0.508242},{0.492402, -0.067003, 2.965576, 0.757006, 0.472016, -1.000364, 0.688148, 0.860497, -0.318783, 0.421319, 1.042047, 0.581711, 0.691435, -0.061865, 1.239108, -0.434678, 1.564631, -0.787105, -0.698816, 0.967919, -1.025052, -1.305047, -0.602350, 0.681890, 0.153614, -0.402999, 1.397904, -0.826499, -1.320419, -1.447122, 0.630540, 0.426620, -0.959306, -0.544540, 0.060724, 0.861528, -0.003486, 0.123558, -0.016125, -0.739927, 0.688716, -0.261189, -0.457589, 0.280384, 2.041016, 1.636161, -0.102918, -1.659824, 0.218525, -0.469339, -0.751114, -0.881051, -0.716598, 0.849901, 0.232911, -0.096044, -0.770220, 1.266461, -0.632847, -0.738116, -0.524328, 0.609412, -0.333586, 0.205119, -1.210955, 2.636650, -0.098561, 0.987781, 0.910435, -0.101384, 1.616258, -1.721549, 1.288512, -1.170860, -0.141915, -1.218524, 0.344677, -0.540426},{-1.147939, 0.657441, 0.274523, 0.528892, 0.411390, -1.071015, 1.840383, -2.279965, 1.528435, 1.484140, -1.352242, -0.007534, 0.397619, -0.395727, 0.453085, -0.939820, -1.214195, -0.727696, 0.626806, 0.239010, 0.547333, -1.239812, -0.332157, 1.220811, -0.612488, 2.038756, 0.580968, -0.284702, 1.111197, -0.793567, -1.404952, 0.442196, 0.483000, 1.281129, 1.300870, 1.915378, 0.033838, -1.129562, 0.046526, 0.738765, -0.483761, 0.551293, -0.472312, 0.783378, 0.210168, -0.429504, 0.351446, -0.198611, 0.200341, 1.398249, 0.314557, -0.722455, 0.048075, -0.271550, 0.060935, -0.322159, -0.597803, 1.183604, -1.091232, -1.087783, -0.324470, 1.145676, 0.740739, -0.656049, 0.677886, 1.650383, -0.672902, -0.448176, -0.349852, -0.738339, 1.098403, -0.798327, 1.129804, -1.455374, 0.020663, -1.022605, -0.281792, 1.381411},{-1.404326, -2.987756, -1.304632, 0.619296, -0.926549, -1.754939, 0.593939, -0.113120, -0.655985, -0.137503, -0.125442, -0.823362, 1.814568, -0.346613, -1.465964, -2.096056, 0.109242, -1.168638, 0.528763, -0.086810, 1.458031, 0.545973, -0.931433, -0.501198, 0.890815, 0.136050, -0.086520, 1.183684, -0.414594, 1.368429, 0.329950, -0.120051, 0.285298, 0.627248, 1.011822, -1.191797, 0.715012, -0.495413, 1.988224, 0.514212, 0.619924, -0.327478, -1.546628, 0.658451, 0.664081, -0.592265, 0.513230, 0.337053, 0.400883, 1.603585, -0.580499, -2.413077, -0.847600, 2.550160, -1.508803, 0.564320, -1.275671, 0.857866, 2.183324, 0.925132, 1.040389, -0.090701, -0.595954, 0.442519, 0.337761, -0.141145, -0.113292, -2.141855, 0.134775, -2.036128, 0.122351, 1.060117, 1.314649, 0.568296, -0.677162, 0.661592, -0.231679, -1.552166},{0.937589, 0.892402, -1.589839, 0.326018, -0.231045, 0.306897, -0.414714, -0.751660, 0.079034, 1.655206, -0.753357, 1.921339, 0.593525, 0.357152, -0.163059, -1.080126, -0.279713, 0.317971, 2.063543, -0.675821, -0.702234, 1.216125, -1.365349, 3.077467, -0.795580, 1.016043, 0.293318, 0.721352, -0.604101, -1.162730, -1.241770, -0.693864, 0.623105, -0.162349, -0.021342, -0.203854, -0.283869, 0.044037, 0.855687, 0.015348, 2.882250, -0.645152, 0.587124, 1.745074, 0.598722, -0.288942, 0.675055, 1.656007, 1.548385, -0.923520, 0.906931, -1.954542, -0.098412, -0.280248, -0.708736, 0.903426, 0.913538, 1.157960, -1.016967, -1.002804, 0.603517, 0.366591, 0.776783, -0.509779, -0.288367, 0.941918, 0.371472, -0.379450, 0.658129, -0.078276, -0.411562, 0.548586, -0.380914, -1.374258, -0.656134, 0.975951, 0.162554, 0.477459},{-2.110096, -0.026277, 0.188727, -0.575148, 1.727914, -0.120590, 1.056813, 0.874623, -0.271253, 0.420630, -0.796168, -0.335296, -0.984085, 0.962883, 1.002770, -1.296271, -1.291453, -0.699834, 0.758685, -0.486318, 1.282587, -0.193607, 0.110775, 1.097014, -2.798200, -0.819140, -0.301010, 0.397754, -1.418374, 0.764054, 0.861892, -0.018331, -1.906853, 1.330341, 0.595593, -0.261057, 0.041096, -0.032246, -0.062229, 0.266207, -0.887128, -1.798092, -0.024891, 1.951736, 1.189991, -0.996592, -0.117988, 1.521350, 0.653991, -0.379692, 0.532476, -0.274116, 0.821591, 0.354031, -1.791291, 0.586675, 0.500306, -0.136684, -0.327160, -0.872816, 1.599588, -0.405178, 0.016359, -1.259946, -0.747196, -1.049648, -1.018150, 1.678887, -0.596932, 2.654982, 0.134913, 0.523098, -0.608816, -1.299536, 1.290118, 0.325852, -0.071671, 0.031065},{0.206168, -0.861842, -0.602118, -0.056182, -0.854133, -0.961220, 0.415517, -0.538933, 0.346449, 0.586639, -1.871466, 1.426760, -0.280853, 0.017277, 1.808493, 1.143795, -0.270817, 0.023660, -0.413930, 1.080299, 0.445040, 0.143793, 1.217408, -1.260877, 0.103273, 0.403420, 0.090075, 0.754350, 1.452122, 1.930470, -0.129050, -0.453059, -1.117337, -1.542997, 0.298911, 1.696553, -1.299920, 0.573915, 0.821230, -0.626020, -1.024164, 0.049828, 1.161847, -1.174055, 0.542513, -1.279924, 0.882955, -0.723064, 1.510541, 1.559032, 0.420761, 0.250065, -0.920924, -0.835332, 0.187243, 0.680039, 1.079057, -1.653799, -0.422280, 2.357547, 0.114677, -0.048118, 1.945843, 0.347194, 0.475171, -1.091634, 0.773548, -0.338974, 0.190708, 1.717971, 0.511168, 1.638044, 2.312382, -2.085231, -0.006171, -0.023157, 1.702426, -0.285942},{0.808133, -0.551131, -0.940602, 1.665033, 0.016249, 0.142268, -0.537800, -0.205840, -0.936481, 0.684538, -0.290591, 0.787467, -0.129532, 1.099744, 0.934031, -0.644086, 0.216799, 0.565583, -1.719045, 0.379542, 1.443561, 0.463100, 0.028129, 2.240576, -1.386349, 1.126497, -0.229512, -0.928603, -1.048073, 1.514976, -0.107346, -0.966249, -0.554483, -0.425452, 0.049923, 1.491964, -0.947668, 1.241626, -1.339909, 1.260271, -0.045514, 0.870057, 0.110353, -0.705305, -0.193385, -0.351258, 1.834490, 0.021918, -1.336916, -1.170156, 1.294579, -0.422995, -1.306596, 0.318417, -0.225972, -0.578180, -0.104552, 1.247024, -2.204719, -2.000616, -0.022895, -0.008965, -1.607618, -0.962003, 1.104390, -0.418554, -0.866268, 1.286705, -2.541661, 0.265914, -1.301955, 0.331291, 1.193590, 0.842141, -1.491958, -0.929221, -0.746341, 0.402406},{1.886091, 0.631682, 1.979710, -1.024180, -0.376724, 1.263895, 0.855150, 1.496024, -0.184429, 2.038798, -0.159234, -0.576979, -0.622279, -0.726223, 0.517346, 0.363456, -0.934127, -2.286875, -0.536815, -0.753107, 0.943737, 0.594187, -0.023916, -1.366616, 1.970344, 1.373720, 0.015648, -0.107755, -0.149229, -0.104548, -0.473338, -2.075648, 0.750798, 0.856961, 0.061131, -0.591622, 0.229498, 0.272006, -0.088211, -0.281498, 0.612582, -1.371105, 0.019123, 0.689102, -0.710880, 0.940636, 0.720941, 0.330228, -0.588133, 1.291371, -0.518547, -0.949714, 2.163773, -0.127536, 2.169784, 0.272704, -0.462616, 1.029680, -1.506939, 0.949286, 2.684664, -0.766460, 0.394215, -0.244766, 1.531349, -0.833661, -0.403545, -0.395550, -0.849874, 1.169536, 1.173593, 1.268946, -0.814018, -0.011986, -0.825702, -0.227604, 0.760754, -1.306718},{1.772520, -2.200075, -1.564155, 0.801077, -0.257233, 1.613264, 0.030693, 0.124438, -0.083232, -0.053614, 1.687666, 2.080994, 0.577186, -0.416226, -0.546899, -0.402750, -0.046292, 0.291391, 1.132654, -0.950927, 0.560793, -2.861871, -0.534851, -0.063625, 0.436620, 0.499112, -2.364742, 0.462127, -2.047132, 1.060072, -1.346342, -1.703055, 2.077440, -0.270601, -0.376233, 0.403474, 0.723771, -0.759709, -0.107932, -2.000506, -1.068134, 1.213966, 0.450247, 0.342432, 0.653898, -0.457370, 0.104954, 1.765821, -0.967243, -0.881765, -0.028558, 1.029961, 0.014185, -0.158053, -0.118432, 0.007524, -0.509612, -1.650113, -0.873794, -1.421836, -0.264174, -0.754714, -0.701981, 0.008727, -0.436634, -0.554610, -1.427708, 0.841663, 0.070819, 0.874598, 0.645089, -0.797333, -0.065336, -0.829441, 0.849764, -1.341290, -1.030704, -1.010705},{-1.202797, 2.005566, -0.392294, -0.699527, 0.230305, 1.897578, -0.533504, -1.153223, 0.407401, -1.280386, -0.576004, -0.468397, 0.942289, -0.912877, 0.715401, -2.088131, 0.814567, -0.666087, 0.030800, -1.800406, -1.554577, -0.912384, -0.901491, 0.372599, -1.540770, 0.328037, -0.427932, 1.045349, 0.967672, -0.815782, 0.622275, -1.524459, 0.720155, -1.216184, 2.203498, -0.314435, -0.763719, 0.223426, 0.856677, -0.677863, -0.109293, -0.136165, 0.596029, -0.422160, -0.695662, 0.911355, 1.082066, 2.997692, 1.446091, 0.564534, -0.658956, 0.856308, -0.167003, 1.134761, -0.014899, -0.264806, 0.640174, -0.900447, -1.527656, 1.407315, 1.416063, 0.393198, 0.395586, 0.018896, 0.227015, 1.147158, 1.313846, -1.418256, 1.356341, 0.127247, -1.009493, 0.869641, -0.034663, 0.849021, 2.499725, 1.114610, 0.234695, 1.006512},{1.294299, -0.122976, -0.254988, -0.132100, 1.937563, -0.249486, -0.281973, -0.264775, -0.453028, 0.126384, 0.412872, -0.956206, 1.025633, -0.106264, 0.870092, -0.421151, -0.643254, 0.904098, -0.517559, -0.472950, -0.859824, 1.509498, 1.985770, -0.438542, -1.036851, -0.356775, 1.813905, 0.918064, 1.419863, 1.463836, 0.294589, 1.853389, 0.413018, -0.315257, -0.430924, -0.514362, -1.707371, 2.146438, 0.013760, 0.472004, 0.536624, -1.070861, -1.188331, -2.019715, 0.749692, 0.482812, -1.636766, 1.955041, -1.226421, 0.039485, -0.383344, 0.070002, 0.509137, -0.444360, 0.080289, -1.200098, -0.265045, 0.209430, 1.399290, 1.659373, -0.340931, 0.843614, -1.979904, 0.909582, 1.751703, 0.300479, 0.268040, -0.840527, 0.629187, -0.044268, -1.245090, 0.639876, -0.040764, -0.838651, -0.560511, -0.305022, 0.621261, 0.440482},{-0.821461, 1.076708, -0.022415, 1.156135, -0.920637, -0.361385, -0.892419, 0.198423, 1.020435, 0.539042, -0.687365, -0.965094, 0.349541, 0.467448, -0.336548, -2.051296, 0.057487, -0.643838, -1.598613, -0.634809, -0.312045, -1.017927, -0.881615, 0.355304, 0.760499, 2.860144, 0.632380, 0.224776, 1.690151, -0.028505, 1.008039, 0.573831, -1.282888, 0.588371, -1.174242, -1.331874, -0.021263, -1.090630, 0.214125, 1.105626, -0.077869, -0.756408, -0.903643, -0.004797, 0.503880, 0.233477, -0.505233, -1.269145, -1.345292, 0.537574, 0.277683, -0.591944, 1.738645, 0.382913, 0.121611, -0.503460, -0.494016, -1.597014, 0.262147, 0.832920, -0.534858, -0.549789, -0.514410, 1.195191, 1.122826, -0.804583, -0.277288, 1.040008, 0.575899, -1.558758, 0.350133, -0.300531, -0.723695, -1.226118, 1.603286, 0.745504, 1.493113, 0.066733},{0.618462, 0.782589, 0.301164, -1.641146, 0.226887, -1.172547, -1.748448, -0.161524, 0.053225, -0.764659, 0.525500, 1.528893, -0.183831, 0.613847, 0.333681, -1.738395, 1.375286, -0.993523, 1.042774, -0.194793, -0.999365, -0.329717, 1.106591, 0.290837, 1.134289, -0.755677, -1.029383, -1.220051, 0.768586, -0.607894, 1.535144, -0.143897, -0.724618, 0.751487, -1.794269, 0.070140, -0.637215, -1.067412, 0.226547, 0.964691, -0.588528, 0.262108, -0.405146, -1.082800, 0.604212, -1.046260, 1.627314, 0.612852, -0.831737, -0.919097, 0.734648, 0.448257, 0.848409, -0.459075, 0.728739, -0.931521, 0.878968, -0.133912, 0.910215, -1.110248, 0.508972, -1.893777, -0.386664, -0.812719, -0.918251, -1.436284, 1.680998, 0.723151, 1.564621, -1.163100, -0.163293, 0.005829, 0.402671, 0.490983, -0.376207, 1.459684, -0.031337, 0.701572},{2.045401, -0.862298, 0.732434, 0.037877, -2.214528, 0.667661, -1.509170, 0.060197, 1.184255, 1.656573, -1.488574, 2.145065, 1.022729, 0.689393, -1.170986, 0.463562, 0.612490, 0.075430, 0.947455, -2.933897, -0.394435, -2.208637, -0.572280, -0.407071, -0.959423, 0.934706, -0.013701, 0.501466, -0.582084, -0.450700, 0.651789, 1.518096, -0.603547, 0.516600, -0.503777, -1.577069, -1.214670, -0.944835, -0.628356, -0.243566, -0.212064, -1.185756, -0.403441, 0.320963, -0.196117, 0.374615, 0.074247, 0.842639, -0.451752, 2.097963, 0.189339, 0.117708, -1.028057, -0.057341, -2.666334, 1.065818, -1.547709, 0.430091, -0.789700, -0.797030, -1.939089, -0.263262, -0.148563, 0.751371, -0.528146, 0.730731, -0.801467, 0.762606, -0.957025, 0.517950, -0.724117, 1.512834, -0.726274, -0.518757, 1.186759, 1.251252, 1.380399, -0.391611},{0.732401, -1.189439, 0.595855, -1.676466, 1.110837, 0.860643, 0.412188, 0.067082, 0.293331, 0.537668, 0.267903, -0.837493, -2.193472, 0.300036, -1.308358, -0.200379, 0.486755, -0.444073, 0.586698, 1.108613, -1.464786, -0.416433, 0.700569, 1.265416, -0.176816, 0.683276, -0.531644, 0.559724, 0.725451, 0.770576, 0.274382, -0.751419, 0.646978, 0.724687, 0.454889, -1.688783, -0.130329, 0.582109, -1.461550, 0.547152, -0.081735, -1.097355, -1.094369, 0.423166, 1.049178, 1.298372, 0.710686, -0.293662, -0.325886, -0.683943, 0.435616, 1.413898, -0.817346, 1.424268, -1.055128, -0.290457, 0.310609, -1.046462, 0.212494, 0.046943, 0.037145, -2.239092, -0.389004, -0.051992, -2.982925, 0.015018, -1.466173, 1.739206, -0.256675, 2.126473, 0.196756, 1.100872, -0.056899, -1.294481, -1.267647, -0.749255, -0.538520, 0.341693},{-0.478351, -1.264818, 0.285424, 0.046657, 0.115270, 0.236797, -0.742254, -0.177566, -1.267977, 1.817541, 1.425731, -0.892171, 0.238067, 0.982501, -0.025339, 1.441955, 0.155223, -0.470936, 1.388847, -2.259281, 1.390443, -1.611796, -1.587400, 0.396716, 0.903666, 0.287901, -1.950391, 0.464189, -0.733168, 0.756835, 0.236099, -0.272376, -2.031494, -0.895491, -0.169473, 0.743727, -0.468040, 0.863396, -0.383154, -1.132375, -2.045541, 1.708112, -0.303181, 0.486219, 0.853436, 0.170722, 0.022232, 0.819644, -1.189270, 0.147402, -0.683150, -0.615314, 0.023709, 0.888488, -0.069267, -1.431582, -1.333632, -1.836727, 0.415529, -0.345482, -1.363264, 0.103806, -1.244380, 0.058283, 1.341738, -1.356471, -0.148043, 0.768647, -1.188361, 0.043365, 0.433553, -0.377882, -1.239766, -0.975511, 1.191461, -0.870338, -0.327339, 1.530200},{1.648764, 0.322320, 0.392696, 0.942146, 0.882250, 0.748429, 0.771618, 1.582062, -1.035442, 2.378748, 0.906952, -0.483822, 0.127297, -0.212641, -0.208386, -0.793026, 3.254512, -1.100632, -0.059678, -0.778030, 1.144802, -0.902820, 1.466499, -0.222133, 0.534781, -1.390798, -0.637674, -1.954924, 0.182434, 0.927818, 1.448406, -0.472744, -1.057748, -0.433767, -0.999961, -0.667623, -1.137062, 0.013413, 0.466522, -0.078758, 0.152944, 0.684879, 1.216052, -0.570189, 0.277416, 0.994579, 0.086860, -1.290533, -1.539108, -2.260298, 0.035183, -0.956228, 0.284231, -0.099047, 0.555801, 0.044613, -0.683420, -0.364026, -0.581526, 0.209245, -1.297891, -1.098055, -0.326609, -1.064138, 0.829632, 0.034656, 0.229203, -0.895058, 0.832639, -1.814044, 0.478649, -0.048903, -0.300495, 0.806949, 1.906262, -0.183769, 1.816854, 1.905919},{1.407168, -0.414209, -0.429838, 0.794273, -0.250723, 1.366353, -1.494181, -0.852480, 0.060808, 2.562565, 0.554686, 2.079298, 0.921151, 0.234519, 1.430206, 0.604646, -0.292620, -0.357174, 0.776576, 1.256300, -0.090132, 2.062898, -0.191668, -0.686151, 0.270954, -1.180438, -0.829410, 0.079910, -0.626028, -0.766707, 0.344324, 1.474782, 0.511855, -0.645510, -0.608563, 0.913897, 0.220854, 0.064941, 0.382698, 1.281528, 0.054534, 0.256905, 0.186236, -0.248440, 2.270653, 0.229922, -0.792891, -0.691416, 0.804993, 0.626477, 0.971261, -1.282302, -0.031440, -0.618230, 0.492507, 0.046633, 0.858180, 0.185171, 2.089061, -0.533760, 0.568730, 0.998191, -2.642586, 2.153985, 1.397505, -0.896373, -0.323528, 0.156076, -2.693535, 1.715454, -1.149915, 0.006556, 0.807556, -0.341201, 1.803652, 0.324783, -0.615853, 0.876711},{-0.696775, 0.276294, -0.121222, -0.771863, 0.234730, -1.026766, 1.643144, 0.523103, -1.787395, 0.123507, 0.492669, 0.316944, -0.682835, -0.049677, -0.393491, 1.321563, 0.306166, -1.096494, -0.524434, -0.091090, -0.591989, 0.467926, 1.384016, -0.598676, -0.032597, -0.105289, 0.221339, -0.834185, 0.105719, -0.908258, 0.548519, -0.087396, -2.498218, 1.031206, 1.442372, -0.124862, 0.292518, -0.100818, 1.032519, -0.121282, 1.066952, 1.100459, 1.843806, -0.040286, -0.173252, 0.867894, 1.483945, -1.473058, 0.010794, 0.640338, 0.470469, -1.654297, 1.460288, -0.746491, -1.146928, 0.449999, 1.234771, 1.767659, 1.562576, -1.163964, 0.279522, 0.388638, 0.486481, -0.809188, -0.079902, 0.197774, 1.129557, 0.426553, 0.304664, -0.023809, 0.407704, -0.653389, 1.450965, 0.665896, 0.182104, 1.850196, -0.814991, 0.237094},{0.396503, -0.924474, 0.098660, -0.035277, -0.513415, -0.592191, -0.630173, -0.547627, -0.871440, -0.971433, -0.393405, 0.580697, -0.121538, -1.847061, -0.091106, 0.004237, -0.160114, 0.081352, 1.853329, -0.128766, 0.237576, -1.061034, -1.072343, -0.357321, 0.752432, 1.297911, 1.093642, -0.418041, -0.105392, 0.116704, 0.728092, -0.284326, 0.070522, 1.147593, -0.690970, -1.087140, -0.659912, 1.457666, -0.359532, -0.812303, 0.285006, -1.025823, 0.987491, 1.112893, -0.869577, -0.375632, 0.039496, 0.398807, -0.235505, -1.721623, 0.777359, 1.994043, 0.630155, 1.305621, -0.194036, 0.711482, -0.080506, 0.050683, -0.155987, 0.363123, -0.111668, -0.464129, 0.339397, 2.007554, 0.510037, 0.089233, 0.096051, 0.989366, 0.062391, 1.439446, -1.468193, 0.479841, -0.137740, 1.474770, 0.834214, -0.625899, -0.162830, -1.123500},{1.379030, 0.396207, 0.102874, -0.447124, -1.497953, -0.544856, 0.061085, -2.674811, -0.536097, 1.068647, 0.675305, -0.349067, -0.041378, 0.802459, 0.162419, 0.175742, -0.771806, -2.367103, -1.608540, 0.942399, 0.618724, -1.147996, -0.078930, -0.598561, -0.780181, -1.394327, 1.458848, 0.685548, -1.384950, 0.468481, 0.997735, -0.820127, 0.630216, 2.170308, 0.715809, 0.960558, -0.987748, 0.780018, -1.221453, -1.429510, 1.574780, 1.478306, 0.365213, -1.796483, 0.690670, 1.130627, 0.154134, 0.305119, -0.588280, -0.337004, 1.678946, 0.846152, -0.108616, 2.000069, -0.680629, 3.231226, 1.279247, -1.448372, -1.069877, -0.015603, -0.155100, -0.421007, 0.658036, -0.972741, -0.139214, 1.066783, -0.245704, -0.299233, 1.175838, 0.366929, -0.977335, 0.566142, -0.367558, -0.003949, -1.103840, -1.077547, 0.316890, 0.105598},{0.882652, -0.502656, -0.013507, 1.887575, 1.187126, 0.365466, 0.975262, 1.261876, 2.026253, -0.740644, 0.173715, -0.853254, -0.495673, 0.186746, -0.412467, -0.242186, 0.508602, -0.019134, -0.646480, 0.008435, -0.336277, 2.355489, -0.996199, 0.647462, -1.011258, 0.489262, 0.602932, 0.378480, -0.132467, -0.307213, -0.182266, 1.629125, -1.389933, -1.223924, -1.850392, 1.233572, -1.822762, -1.701260, 0.132530, 1.824939, -0.596578, 2.051768, 0.834072, 0.666250, 0.159290, 1.752916, 0.114577, 0.940860, -1.617733, -1.655306, 0.194652, 0.657671, 0.440796, -2.291303, -1.694375, 0.770817, -1.543278, -0.419771, -1.867102, 0.588448, -0.340601, 0.123417, 1.439494, -1.634102, 0.613715, -0.229553, -0.313120, 0.866134, 0.538669, -0.198551, -0.650858, -1.017590, 0.373352, -1.043217, 0.101256, 1.665537, -1.055579, 1.343551},{-1.119843, 0.603392, 2.120909, -1.210140, 0.476924, 0.937636, -0.022715, -2.902664, 0.454824, -1.115825, 0.515125, -0.740120, -0.728333, 1.689555, 0.561461, 1.123487, 0.539377, -1.894098, -1.589148, -2.341811, 0.886095, -0.231673, -0.423397, 0.314094, -0.532345, 0.120051, 1.594398, 0.624348, 1.344448, 0.061832, 1.874675, 0.466196, -1.367327, -1.071949, 1.101110, -1.004615, -1.410331, -1.492892, 1.659740, 0.823219, 0.940095, -1.085857, 1.419429, -1.575675, -0.062517, 1.071192, 0.056212, -1.188355, 1.930993, -0.561709, -1.557630, -0.548768, -0.465031, 0.512631, 0.546230, -0.954080, -0.273880, -1.000291, -0.044029, 0.000217, 0.188315, -0.166237, -1.860506, -0.991628, -0.717616, -0.498930, 0.830319, -0.490041, 0.035546, -0.875775, -0.559589, 0.265527, -0.699164, -0.858206, -0.116518, 0.680050, 0.373586, -0.355468},{-1.392808, 0.250236, -0.302043, -0.491044, -0.470044, 0.102669, 1.243996, -0.616693, 0.954361, 1.426229, 0.117969, -0.642486, 0.317448, 0.541435, -1.280965, -0.515822, 1.165148, -0.232629, 0.392961, 0.199567, -0.005856, -0.333842, 0.702442, 0.311440, 0.152828, 1.801578, -0.500996, 2.405181, 0.923476, 0.818497, 0.436434, -0.737890, -0.861739, 0.030030, 0.721843, -0.874037, -0.451905, 0.150052, 0.279720, 0.343730, 0.418174, -0.170837, 0.468036, -1.141073, -0.429990, -0.891749, 0.340909, -0.260317, -0.965793, -0.154199, 0.751733, -0.025806, -1.417852, -1.138645, 1.909596, -0.195682, 0.702883, -0.748553, 0.920296, -0.462849, 1.332794, -0.588303, 0.451837, 0.171167, -0.450711, -0.385967, 1.229363, -0.402669, -1.046200, 0.725481, 0.674836, 0.767848, 0.060177, -0.689410, -1.517108, -0.876734, 0.609241, -0.053603},{1.175021, 1.564676, 0.268220, -1.265329, 1.100385, -0.763754, -0.378957, -0.001050, 1.175682, -1.390329, 0.132111, 1.615242, -0.554981, -0.830950, -1.553052, 0.569481, -0.355935, 0.283777, 0.059201, 0.484066, 0.828297, -0.243339, 0.877904, -0.421693, -0.338901, 1.023538, 0.247774, 1.022590, -0.257419, -0.601154, 0.063512, -1.292302, -0.618399, 0.991596, 0.239936, -1.191893, -1.158997, -1.420460, -1.331089, 1.502762, -0.408185, 0.776185, 0.574984, -1.388042, 0.950968, -0.952170, -0.606300, 1.032026, 0.576567, -0.024066, 0.281643, -0.179571, 1.870372, -1.022541, 1.002905, 0.367588, -0.856739, -0.930914, -1.755364, -0.769651, 0.608131, 0.272936, -0.590173, 1.070046, -1.301062, -0.065400, -0.350217, -0.836340, 1.831765, 1.141405, 0.297839, 0.278053, -0.242155, 1.079332, -0.518107, 0.421019, 0.326470, 0.646413},{1.223706, -1.876849, 1.005514, -1.394771, -0.590252, 1.115636, 0.373838, 0.519975, 2.103507, -1.690377, 0.463697, 0.010884, -1.299507, 2.377188, 1.301037, 1.043869, -0.520127, -0.538531, 1.237168, 0.491616, -0.923376, 0.739507, -0.668696, -0.166303, -0.789816, 0.706977, -1.929165, -0.320091, 1.362750, 0.215418, 0.904226, -0.144299, 0.436314, -0.497025, 0.296567, -0.152728, -2.091911, 0.443927, -1.211243, -0.591893, 1.098317, -1.536949, 1.367908, 0.203957, -0.820205, 2.370301, -1.363533, 2.098032, -0.981998, 0.048261, 0.846169, 0.916053, 0.996567, -0.282979, 0.263690, 0.463194, 0.084818, -0.180960, -0.358800, -0.616029, 0.280349, 1.196183, -0.686472, -0.365314, 0.926563, 1.075902, -1.217173, 1.145437, -0.782486, -1.253506, 0.269068, -0.632774, 1.058529, -1.403160, 0.960117, 1.093073, 1.123518, 0.059563},{-1.480935, -0.456253, 0.585481, -0.550595, -0.090382, -0.959392, -0.757009, -0.280452, -0.382213, -0.279346, -2.313217, -1.297758, 0.024602, -0.721796, -1.068080, -0.146902, -1.626966, 1.033819, 1.295286, -0.675149, 0.341774, 0.980649, 1.524998, -1.498802, -1.623436, -1.246680, 0.124095, -1.576250, 2.325213, 0.085814, -0.755928, -0.502656, 0.606789, 1.744792, 1.919719, -0.202505, -0.272537, -0.890880, 0.060907, 1.014875, -1.001014, 1.294766, -1.478017, -0.226259, 0.215252, 0.173326, -0.199593, 0.464324, -2.538308, -0.404760, 0.212610, 1.374004, 0.157954, -0.025797, -0.372743, 0.728589, 0.559846, 0.234591, 0.544641, 1.517589, -0.769879, -1.083707, -0.182839, 1.085515, 1.108700, 0.232853, -0.087501, 0.719710, -1.539594, -2.022402, 0.704380, -0.033928, -0.436108, 0.014096, 0.938559, -1.461267, 0.571387, -2.037985},{-1.126499, 0.050463, 0.169131, 1.056306, 1.545992, 0.709694, 0.520311, 1.626918, -1.590264, 0.677691, 0.921086, 1.286434, 2.376834, -0.731261, -0.002872, -0.719550, 2.354545, -0.623859, -0.760258, 0.325833, -0.066154, -1.237948, -0.183036, -0.080049, 0.728593, 0.061110, 0.605501, -0.687800, -1.490516, -0.874247, -0.709251, 0.586121, 0.593565, -0.505939, 1.780152, -0.024427, -0.605799, 0.622875, 1.255179, 1.716650, 0.137713, -0.443784, -0.035118, 1.046323, 0.094978, 0.757271, -0.580364, 0.437148, 0.007717, -0.324587, 0.372334, -0.824651, 0.200221, -0.328252, -0.868827, -1.322966, 0.082360, 1.640973, 2.742703, 1.018103, -0.224341, 0.960576, -0.364630, 1.931942, -0.959362, 0.546705, 0.132908, -1.419762, -2.253623, -1.539396, 0.566189, -0.018881, 0.727412, 0.125297, 0.455985, -0.386669, 0.656445, -0.260712},{0.664613, 0.595968, -0.514675, 0.716308, 0.850123, 0.174855, 0.608345, -0.466622, -1.663178, -1.256433, -0.491266, 1.035462, 0.130785, 0.216379, -1.718527, 0.016220, -1.247880, -0.868793, -0.329852, 0.303312, -0.169996, -0.443588, 0.285353, -0.432688, -0.389296, -0.980866, -0.467847, -1.219991, -3.408804, -1.118100, 0.172674, 0.016516, -0.350079, 0.697824, -1.030093, 2.082869, 0.006325, -1.718451, -0.681957, -0.155988, -0.193562, 0.681948, -0.548670, -0.337077, -1.668702, -0.376474, -0.074611, 1.049932, 0.262768, 0.750767, 1.009362, 0.874726, -0.455917, -0.239959, -1.786886, 0.185653, 0.682662, 0.961434, 0.413805, -0.856769, 0.831852, -0.020625, 0.130311, 0.775488, -0.709767, -0.596785, -0.703689, 0.353966, -0.337217, 1.175919, 0.467648, 0.683456, 0.480767, 1.506305, -0.992464, -1.753425, 1.261325, -1.056508},{-1.109316, 0.296100, 0.303072, -0.035841, 0.060353, -0.231700, -0.608321, -0.046559, -0.061305, -0.278207, -1.750311, 0.290046, -0.432264, 0.641501, 0.476336, 0.889661, 0.872140, 0.796353, 1.480059, 0.097184, -0.887555, -0.372856, -0.344506, -1.508759, -1.007428, 1.144068, 0.421861, -1.103747, 0.540494, 0.036669, 2.554140, -0.129798, 0.643180, 2.222819, 0.541077, -1.091289, 0.886174, -1.151236, 0.156660, -0.481420, -2.079778, -1.072138, -0.761087, 1.634439, -1.394059, 0.726997, 1.076526, 0.781386, -0.690201, 0.660582, -0.471680, -0.769361, 2.492824, 0.676453, 0.281280, -0.259356, 2.455872, 0.328358, -0.496913, 0.729934, -0.812555, -2.049630, 0.395194, -1.030710, -0.262916, 0.558621, -0.504747, -0.033406, -1.305522, 0.388169, 0.383107, -0.938482, 0.010859, -0.356734, 1.226894, -0.718107, 0.805430, 1.537783},{-1.446676, 0.066729, 0.952143, -0.558385, 0.733851, 0.868362, 1.702242, -1.459024, 1.039317, -0.310424, -1.007489, 0.678464, 0.922645, -0.471704, 0.193795, 0.254807, -0.662994, 0.116499, 0.529694, -0.293345, -0.072382, 1.689498, -1.029349, 0.720473, -0.779580, 0.164314, -0.262848, -1.358454, 0.965028, -0.330031, -0.215336, 0.321951, 0.231529, 0.215874, 1.092896, 1.148657, -0.097671, -0.271308, 0.100689, 1.512145, -1.375549, 0.523295, -0.446504, 0.327049, -0.045870, -0.574762, 0.043771, 1.788221, 0.290613, -0.751971, 1.788995, 0.153685, -0.892753, -0.473811, -0.662344, 1.605406, -1.246438, 0.458395, 1.032731, -0.427309, 1.015079, -0.606641, -1.859338, -1.206012, 0.623784, -0.041081, 0.189280, -0.644956, 0.747579, -0.887056, -0.956818, 0.127431, 0.795524, 0.386482, -0.716746, -0.927389, 0.241777, -0.677851},{-1.533689, -0.148254, 0.652829, -1.116428, -0.420902, -0.219465, -0.279354, -0.998235, -1.911879, -0.475457, 1.033859, -0.231761, -0.157847, 0.507615, 1.017701, -1.517891, 1.045440, -3.212337, -0.439030, 0.846274, -0.021967, 0.446969, -0.195160, 1.034875, -1.540745, 0.542831, 0.897658, -1.739816, 0.417909, 0.630000, 0.332195, -0.548046, 0.501141, 0.353069, 1.097520, -0.387037, -1.753137, 1.033701, -0.204207, -0.877462, 0.332388, -1.527334, -0.303692, 0.902646, -0.521518, 2.097113, -1.938365, -1.349928, 2.417181, 1.766150, 0.656788, -0.293779, -0.383641, -1.692113, -0.590424, -1.536701, 1.694235, 0.272623, 1.378920, 2.987912, 0.319051, -0.483573, -1.222581, -1.758770, 0.416330, 0.313038, -0.179250, 2.804852, -1.435978, 1.144095, 1.529729, -0.837902, 0.969474, 0.613402, -0.279064, 0.654971, -1.137820, -0.032298},{-0.303869, 1.002447, 2.296969, -0.856769, -0.445993, -2.059789, -0.270339, -0.253426, -1.307326, 0.215978, -0.941704, 0.090408, -0.924652, 0.267454, -0.971287, 1.265484, 0.840435, 0.127401, 2.365042, -0.317982, -0.823130, -0.598282, -0.133131, 0.696351, -1.485231, 0.536547, 0.398439, -2.049003, 0.024969, -1.480672, -0.686611, 0.481346, 0.058497, 0.909588, 0.031384, -0.282278, 2.658809, -1.126136, -0.922659, -0.375517, -0.637911, -0.947298, 0.055161, -1.027811, -2.581999, -0.325024, 0.583661, -0.246386, -2.393276, -0.141919, -0.043782, 0.531321, -0.052619, 0.655742, -0.072849, -0.715594, 0.457009, 1.177718, -0.003472, 0.275376, 1.171173, -1.881172, -1.276120, -1.763153, 0.824800, 0.359447, -0.058912, 0.976139, -0.297294, 0.458601, -0.707519, -0.503539, -2.116668, -0.464563, -0.615618, -1.374379, 2.085331, -0.116560},{0.225650, 0.221342, 0.798674, -0.944319, 0.125837, -2.175275, 2.877349, 0.825513, -1.225484, -0.211774, 0.044345, -0.362879, -0.602665, -2.179471, 0.322129, 0.220191, -0.099140, -1.709836, 0.861444, 1.743164, 1.704870, -0.639051, -0.555357, -0.382747, 0.371769, 1.140935, 0.561188, -1.068272, -1.210448, 0.987421, -2.066391, 1.663875, 0.126117, -1.563056, -1.229850, 1.727010, -1.647271, -1.490652, 1.303402, -0.722170, -1.151517, 0.353455, 0.041554, 0.934281, -0.940519, 0.354449, 1.178546, -0.242072, -0.054240, -1.520671, 0.510620, -1.371668, -0.454614, -0.088218, -0.333167, -0.630864, -1.130710, 0.506196, -1.130367, 0.370522, -0.441322, 0.928116, 0.182230, -0.495387, 0.570342, -0.379482, -0.387441, -0.775829, 0.057322, 0.350443, -0.324771, 0.627847, 1.578881, -0.894505, 0.821432, 2.187649, -0.270688, 0.260630},{-1.119508, -1.445745, 0.450734, 0.232115, 0.470467, -0.267264, 0.356631, 3.123563, 0.986090, -0.810049, -0.404085, 1.141016, -1.488707, 0.213308, -0.280659, 0.812333, 1.801973, -0.582496, -0.137108, 0.910612, -0.193514, -0.231840, -0.531095, -0.169678, -0.420928, 0.433184, 0.981543, -0.884774, 0.293112, -1.241834, 0.014210, -1.665089, -0.126036, -0.535984, -1.899279, 0.544832, 0.706166, -1.201016, -1.791543, -1.688012, 0.734742, 0.998274, 0.321153, 0.140036, -0.003064, -0.424363, 0.319388, 0.080807, 0.874900, -1.166512, 0.680426, 0.066017, -0.225250, -0.533728, 1.141348, 0.184832, 0.723208, 0.057137, 1.527483, 1.567381, 1.115321, 1.234137, 0.661323, -0.425282, -1.573123, 0.325874, -1.267946, -2.211993, -2.205945, 0.526374, -0.102374, -0.359130, 0.267304, -1.315972, -0.131810, -0.099423, 1.231628, -1.227390},{0.348504, 0.522478, 1.061673, 0.143687, 0.194423, -0.505792, -1.024693, -1.001823, 0.947138, -1.869479, -1.303094, 2.052660, -0.059531, -0.567866, 0.462115, -0.356367, -0.354358, -0.842309, 0.964952, 1.262443, -0.009609, -1.491327, -0.535789, 1.243834, -1.369976, -0.877986, 0.619300, 0.523352, -0.941796, -0.534368, 0.720334, -0.310073, -1.057231, 0.528911, -0.669068, 0.364432, 1.599203, 0.418151, -0.895120, 0.340261, -0.324821, 1.233165, 0.227621, 0.795696, 0.315229, 0.917124, 1.690852, -1.741314, 1.292447, -1.144565, -1.415881, -1.372674, -0.165804, -1.449373, -0.133808, -0.747599, 0.700586, 0.024174, 1.933310, -0.155479, -1.357765, 0.717289, 1.045479, -2.101274, 1.680757, 0.349675, 0.400330, -0.829347, 1.323106, 1.575638, 1.089652, -0.229865, 1.614870, -0.161147, -0.342406, 0.462840, -2.154137, 0.323581},{-1.054729, 0.138973, 0.035267, -0.078664, -1.917042, 0.972753, 0.232132, -1.744659, -0.840120, -0.838510, -0.629530, -0.982775, 0.807117, -0.274989, 0.606682, -0.376111, -0.258602, -0.463633, 0.745540, -0.370528, 0.360902, 0.573039, -0.845970, -1.008786, -0.993347, 0.002128, -0.588510, 0.511957, 0.315544, 0.865023, 1.648146, 0.069795, 1.211238, 0.782670, 0.437906, 0.869755, 1.289219, 0.327179, 0.982264, -1.159780, -1.537123, -0.209537, -0.343260, -2.232505, -1.432277, -0.513287, -0.395067, 0.292287, 0.098270, 1.047382, 0.024851, 1.326922, 0.774433, -1.430950, 0.410624, -0.639453, 0.812766, 0.455208, -1.084995, -0.372564, -0.292848, 0.796684, -1.270424, -0.247675, -1.112529, 0.577164, -1.685089, -1.024243, -0.866915, -0.448931, 0.825153, 0.828249, 1.137444, -0.817826, 1.013033, 0.999638, -1.056423, -1.441634},{-0.711564, 0.377201, 1.035518, -0.246745, -0.755108, -0.175516, 1.292738, -0.301827, 2.473845, -1.196370, -0.677284, 2.061157, 0.904341, 0.219369, -0.582297, -0.868888, 0.263756, -2.374869, 0.098145, -1.804556, -0.574886, -0.370967, -1.578430, 1.036831, -0.388696, 0.921117, -0.060480, 0.301388, 1.734154, 1.074242, 1.137097, -1.810346, 0.884683, 0.257589, 2.225486, 0.171078, 0.294318, 0.061597, -1.112309, -0.219912, -0.443577, 0.115695, 0.466819, -1.780510, -0.574179, -1.186859, -0.435710, 0.226072, -0.572452, -1.061665, 0.132953, -0.165571, -1.015487, 0.798015, 0.631904, 1.449646, -1.030132, -0.187251, -0.320790, 0.582371, 1.768423, 0.220326, -2.031126, 0.882304, -1.458384, -0.528628, 0.463714, 0.593891, 0.531618, -0.570922, -0.959093, 0.033701, -0.246219, -0.622263, 0.239149, -0.066388, -1.009843, 0.298251},{0.199279, 0.794096, -1.122839, -0.412972, -1.351264, -1.425512, 0.258934, 0.025355, -0.688575, -1.988681, 1.629724, 0.702074, 0.459048, 0.723518, 0.351356, 0.201624, -1.328148, 0.968729, 1.013811, 0.048946, 0.188625, 0.566539, 1.116880, 0.663486, 2.055250, -1.074363, 0.851871, 0.422555, -0.214556, 0.091594, 0.232173, -0.233039, -0.470918, 0.351282, -0.655113, 0.125039, -1.108843, 1.980018, 1.006911, 0.486698, -0.431550, -0.990919, 0.536330, 0.045882, 0.645682, 0.314793, 3.212781, 1.648321, -0.598935, -0.816033, 0.183038, -1.259472, -0.338312, 1.697638, 0.688052, -0.517188, 1.484234, -0.613636, -0.093461, 0.254347, 1.068881, 2.351421, 0.677571, 0.952973, -0.016249, 0.686958, -0.526862, -0.864033, 0.216469, -0.057094, -0.834911, 1.555895, 0.979780, 1.239926, -1.798148, -0.183433, -0.179192, 1.985859},{-0.691862, -0.064893, -1.563567, -0.690178, -1.852039, 0.763647, 0.125285, 2.596558, 0.432880, -0.355373, 0.373186, 1.209194, 0.040147, -1.158788, 1.604351, -1.520185, 1.253779, 0.407576, -1.958230, -0.779861, -0.121908, -0.638901, -0.539930, 0.580641, -0.885022, -0.067494, -1.445082, -0.533279, 0.722844, 0.819341, 0.836692, -0.963064, -1.254905, 1.029660, 2.097136, 1.361888, 0.197627, -0.350379, 0.426406, -0.607916, -0.902804, 1.190338, 1.873801, 1.356817, 0.710615, -0.399235, 0.238106, -0.529071, -0.799443, -0.480557, 0.125234, 1.586047, -0.517941, 0.358729, -0.728436, -0.722228, 0.568310, -0.221366, -0.863018, -0.272419, 0.857977, -1.731580, 0.184949, -1.655734, 1.138549, 1.741725, -1.518840, 0.988789, -0.473051, 0.514173, 0.719539, -0.203147, 0.729027, -0.260472, -0.654812, -1.233654, -2.241668, -0.805148},{-0.884155, 1.642982, -2.162625, 0.293308, -0.803680, -0.565514, 0.064697, -1.665517, -0.063310, 0.075535, 0.651570, -0.033466, 0.489583, 1.385738, 1.209872, 1.837682, -0.811481, -0.041370, -0.166578, -0.500585, -0.756145, -0.460629, 0.554953, 1.805840, 0.489161, 0.138103, 0.174636, -1.854673, 0.384578, -1.241497, 0.507993, -0.539671, 0.328714, -0.833435, -1.855096, 1.387425, -0.216879, -1.418997, 1.326035, -1.944001, 0.464112, 1.099157, -0.254339, -0.893201, 0.920073, -0.324108, 0.407122, -1.571933, -0.946572, -2.063404, 0.395955, -0.672245, -1.602019, 0.170746, -0.651879, 0.584047, -1.976091, 0.956201, 1.894321, -0.353183, 0.667811, -1.123798, 0.785275, 1.410798, -0.430449, 1.372502, -1.318164, -0.592128, 1.375439, -0.052880, 0.786301, -0.737169, -1.064543, -0.570737, -0.192154, 0.798311, -0.762749, -2.259788},{0.050211, 0.446360, 0.099472, 0.059807, -0.331616, 1.392362, 0.463433, -0.662069, 0.622662, 0.069350, 0.307144, 0.128418, -1.589755, -0.467320, 0.219177, -1.254149, -0.329699, 0.150812, 0.006055, -0.543647, -0.959880, -1.676349, -0.976393, -1.113199, -0.857173, 0.417507, -0.354510, 0.878728, 1.133629, -2.081681, 0.434340, 1.084256, -0.782122, -1.774076, -0.422508, 0.317061, -0.711695, 0.668104, 0.694416, -0.156333, 2.039374, 1.108482, -0.230807, -0.310673, -2.200795, 0.242615, 0.301232, 1.964095, 0.468752, -0.018068, 0.010645, -0.322084, 1.241330, 1.154723, 0.710521, -0.036210, 1.238850, -1.362399, -0.268798, 1.722360, -0.378173, -1.950322, 0.241386, 1.999061, 0.181177, -0.041759, 0.160431, -0.906863, 2.432923, -1.659990, -0.680967, -0.748525, 0.304941, 1.214014, 1.183135, -0.444772, 1.493210, 0.528783},{-1.567101, 0.293220, -2.660456, 0.823578, 1.007140, 0.982008, -0.403328, 1.509304, -1.303499, -1.985102, -0.011263, -0.561688, 0.397186, 0.544970, 2.884936, -1.457575, 1.267224, -0.419800, -0.322270, 0.892571, -1.517919, -0.220903, -1.557328, -0.871568, 0.381323, -0.215369, 1.226413, -0.054992, -1.272976, 2.492713, 0.092370, -0.261933, 1.182275, 0.659343, 0.538086, 0.838985, 1.100615, -1.671443, 0.491409, 1.578457, 0.470955, -1.021134, -2.389182, 2.199497, 0.126334, 0.312351, -1.942130, 0.253225, 0.161779, -1.575656, 0.086793, -0.407955, 1.297166, 0.763434, 3.622095, 0.143657, -0.614025, 0.842445, 0.811603, 0.472808, 0.245438, 0.616192, 0.626480, 0.631515, 1.412015, -0.635319, 0.165565, 0.762430, 0.192522, 0.421101, -0.941826, 0.480212, -0.150706, -1.008469, -0.642494, 0.411901, -0.881291, -0.347564},{-0.924530, 0.604417, 0.434949, -0.829802, -1.115053, -0.870653, -2.457943, 2.311796, -0.240992, 0.957734, -0.362737, -0.188860, 0.516113, 0.032616, -0.968516, 2.017158, 0.530429, -0.976851, -1.378241, -0.126091, -0.138082, 0.664566, -0.341463, 0.150039, 0.054544, 0.435607, 0.174354, 0.356093, -0.563500, 0.581463, 0.127742, 1.815333, 1.346552, 0.933383, 1.138167, -1.180713, -1.164947, 1.885079, -0.133229, 0.869921, 0.132041, -0.256569, -0.616482, 0.860305, 0.620459, 0.781939, -0.891734, -0.381750, 0.365533, -0.969376, -0.836624, -0.360170, 0.268858, 0.001876, -0.837290, 0.846664, 1.195718, 0.857569, -0.418273, 0.091742, 1.309250, -1.233098, 0.958304, 0.414491, 1.660094, 0.207156, 1.623175, 0.997076, 0.445486, 1.029891, 0.432553, -0.866975, -0.805299, 0.018453, 0.119211, 0.214408, 1.585104, -0.498368},{-1.627336, 0.256559, -0.476562, -0.089363, 0.395447, -1.295861, 0.143754, 0.539338, 1.304917, -1.286873, 0.912695, 0.389002, 0.063698, 0.578411, -1.079875, -0.879527, -0.425050, 0.267420, -0.420387, -1.265996, 0.463751, -1.308932, 1.293148, -0.306891, 0.384826, -0.739213, -0.802496, -1.660104, 2.987261, -0.484005, -1.279425, -0.790558, 1.273579, 1.331116, -0.137008, 0.835713, -0.882964, 0.696662, 1.091877, -0.479495, 0.645261, -2.631923, 0.030795, -1.262975, 1.826108, -0.018312, -0.878338, 0.860469, 0.210983, 1.022766, 2.557299, 0.570286, 0.852986, -0.336198, 0.177577, -0.021445, 0.135660, 1.414905, -2.719277, 0.461184, 0.769211, 0.205645, -0.166120, 0.529018, -2.309391, 0.243489, 0.758440, 0.942266, 0.312629, -0.423586, 0.155955, -1.503942, 1.200478, -0.739954, -0.661061, -1.213034, -0.750171, 0.098762},{-2.009836, 1.386285, -0.209075, -0.911597, 0.444987, -0.830166, 0.127918, -1.273504, -1.037730, 1.044705, -1.077928, -1.450931, 0.323389, 1.323909, -0.019160, 1.247864, -0.771075, 0.474737, 1.247842, -0.857348, -0.882312, -0.578086, 1.086836, -0.628869, 0.637251, -0.500733, -1.250060, -1.029501, -1.273576, -0.736602, 0.291536, -1.722996, -2.298103, -0.823724, -0.120964, -0.579971, 0.990851, -0.432407, -0.686377, -0.191058, 0.516940, -1.244396, 0.478723, 0.033408, -1.094976, 0.559533, -0.275468, 0.554783, -0.179044, 1.266047, -0.395393, -1.002327, -1.315257, -0.153365, -0.132161, 1.081144, -1.097474, -1.599788, -0.581942, 1.132399, 0.524643, 0.219212, -0.280975, -1.075909, -0.119668, 1.059351, 0.064393, -0.833639, -1.163359, -0.493964, 0.226577, -0.260963, -0.671169, -0.422725, 0.336802, -0.656419, -2.168560, -0.603462},{-0.063401, -0.094481, -0.894565, 0.813484, 0.351507, 0.975078, -0.799060, 2.074054, 0.071908, 0.042373, -0.210628, 0.525626, 1.274206, 0.066879, 1.115007, 1.195359, -0.269969, -0.470445, -1.332711, -0.570344, -2.729299, 0.845584, -0.864378, 0.696467, 0.881839, 2.677908, 0.402432, -1.559938, 1.300720, -1.144274, -1.483320, 0.906618, -0.491656, 1.136896, 2.111129, -0.743612, -0.157781, -0.922535, -1.482952, -1.420995, -0.841755, 0.715420, 0.469203, 0.406396, 0.428256, 0.422615, -1.623003, 0.341303, -2.265564, 1.962986, 0.652992, -0.220578, 0.056506, 0.327646, -0.814408, 0.406826, 1.074861, -0.418398, 1.433986, 0.697595, 0.759600, 0.923638, -0.748526, -1.803021, 0.603661, 0.318408, 0.107340, 0.981015, 0.512564, -0.801200, -0.487693, 0.238448, 0.562028, -0.459537, 0.185631, 1.722323, 0.994010, -0.283941},{0.304244, 0.648990, -1.145922, 0.075522, -1.086627, 0.868630, 1.097813, 1.120824, -1.315400, -0.715195, -0.790804, 0.785662, 0.884372, -0.785890, -0.746778, -0.864248, 0.783154, -0.025340, -1.024233, 0.348507, -0.805074, 1.013821, -1.279308, -0.616174, -2.218429, 1.115581, 0.497971, -0.869856, -0.423246, 0.303044, -0.034762, 0.391286, -0.197030, -0.353250, 0.594615, -0.536182, -1.565715, -0.986087, 1.304925, -0.218906, 1.776103, 0.254115, -2.022709, -0.404625, 0.045781, -0.350407, -0.071728, 0.707416, 0.862164, -0.268534, -0.210179, -0.591942, 0.268390, -1.959807, 2.332624, 0.840376, -0.709361, 0.460194, 0.089149, -0.189120, -2.001113, -0.399500, 2.493119, 0.333764, -0.825579, -1.002492, 0.411014, 1.926192, 0.586658, 0.739441, -1.714127, 1.758402, -0.126404, 0.734657, -0.146047, -0.714143, -0.288585, -0.120431},{0.990190, -0.313052, -0.949717, 0.376156, 1.297366, -2.737927, 1.671082, -0.593186, 0.512685, -0.067908, -0.522964, 1.256926, 1.840949, -0.195147, 0.599246, -1.264441, -1.797019, -2.617985, 1.036517, 0.935207, -1.102696, -1.692074, -1.213334, 1.053633, -0.518611, -1.005956, 1.710637, -0.496899, -2.545683, 0.665317, -0.880883, 0.309800, -0.483300, 0.580589, 1.134547, 0.271419, -0.664700, -0.223614, -0.218787, 1.211447, -0.432086, 0.089835, 0.387160, 1.158489, 0.087553, -0.270891, 0.829622, 0.714532, -0.067837, 0.202355, 1.522020, 0.564964, -0.346041, -1.004057, 0.669538, -0.377448, -1.267809, -0.736158, 0.904875, 0.851581, 0.332875, -0.691517, 0.840859, 1.229829, -1.568045, 0.818150, 0.279353, -0.809873, -1.307372, 0.608644, 0.320681, -0.250332, 0.264260, -0.969977, 0.968917, 0.261730, -0.184736, -0.055669},{-1.715276, -0.495529, -1.766758, 0.386637, -0.459590, 0.607831, -0.675837, -0.362178, -0.256272, -0.459196, -0.057187, -0.819227, 1.611377, -0.705321, 0.395367, -0.689175, -0.414138, -0.891508, -0.129045, -0.356059, 0.381987, -1.092083, -0.588899, 0.098173, 0.045932, -0.499951, -1.923720, -0.021073, -0.678093, -0.187554, 0.178021, -0.563807, 0.689276, -0.540977, -0.184264, -1.637375, 0.917820, -0.080961, 1.437240, 0.070903, -0.246494, 0.888974, 0.248781, 1.252116, -0.537859, -0.382860, -0.081380, 1.664644, -0.050138, 0.978425, 1.170452, 0.914963, -0.338852, -0.676757, 0.159971, -0.415442, -0.364958, 0.082141, 1.630492, 0.201060, -0.624610, 0.369518, 0.742478, 1.147057, 0.254422, 1.090120, 1.279373, -1.690071, 0.503753, 0.151198, 0.655970, 0.482766, 0.421747, 0.353204, 1.735837, -0.235608, -2.079730, -0.492216},{0.688052, -0.678590, -0.550365, -0.644438, 1.417908, 0.617257, 0.277555, -1.624303, -0.311299, -0.175391, 2.679981, -0.724834, -1.177561, -0.370002, -0.187907, -0.529147, 0.290016, 0.196521, 0.718946, -0.175486, -1.830806, 0.603016, 0.658623, 1.030071, -1.509186, -0.543239, -0.735766, -1.660086, -0.390941, -1.180283, -0.404032, 0.585316, -0.776988, -0.740953, -0.164464, -1.060701, 0.040021, 1.264636, 0.079713, -1.283872, 0.567453, -0.064063, 0.236404, 1.067370, -1.965197, -3.081926, 1.379631, -0.972166, -1.254558, 0.531497, -0.110203, 0.690557, -2.013152, -1.299782, -0.550364, -0.807057, 0.973295, -0.974893, 0.653105, -1.355444, -0.417679, -0.691278, 1.027881, 0.399862, -0.902461, 0.360224, 1.230709, 2.186373, 0.934299, -0.494962, 0.537444, -0.228263, -0.625362, 0.793042, -0.226196, 0.563103, 0.157006, 0.090246},{0.141127, -0.814033, 0.645104, 1.046415, 1.546581, 1.214991, -0.107568, -1.840597, 0.141000, -0.074496, -0.756946, -0.980495, -0.232939, -0.903405, 0.019215, -1.163510, 0.152552, -1.111932, 1.912456, 0.343361, 0.578856, -1.913249, -0.369476, -1.218550, -0.642824, -1.746241, 0.742343, -0.751444, 0.675014, 0.153210, 0.691675, -1.000912, -2.839509, 0.016616, -0.351636, 1.343041, -0.466704, -0.895369, -0.292235, -0.054613, -1.667716, -0.517763, 0.857891, -0.412162, 0.737553, -2.303034, 0.546806, 0.889426, -0.028446, -1.576848, 1.800916, -0.820474, -0.828494, 0.384080, -0.789100, -0.412665, -1.170952, -0.344853, 1.245834, -1.093889, -0.537275, -0.266523, -0.179112, 1.675624, -0.249324, -0.587262, -0.212437, -0.127274, -0.480031, -0.061047, -1.045263, -0.890131, -0.846140, -0.782764, 0.454100, 0.185669, 0.646559, -0.608148},{-0.692200, 0.454754, -0.602050, -1.807389, -0.434309, 0.737993, 1.763956, -0.237204, 0.663416, -0.352533, 1.292428, 1.353221, -0.420652, 1.061845, -1.480595, 1.472879, 0.387209, -0.058265, -0.690525, 0.386019, -0.424712, -0.826969, 0.870719, 2.779703, -0.532147, 0.240150, -2.160398, -0.542770, -0.374170, -0.943588, 1.355401, 0.001681, -0.685026, 1.874094, 0.521708, 0.889714, -0.948194, 0.774067, 0.358282, 0.774431, -0.642433, 0.090863, 0.920506, 0.903967, 1.626305, -1.771526, -0.012062, -0.985185, 0.965655, 0.950100, 1.914449, -0.973517, -2.797962, -0.951271, 1.817064, -0.912786, 0.922236, 0.141133, -0.559960, -0.431791, -0.795159, 0.379642, -1.188080, 0.347033, 0.000314, -1.738411, 1.905804, -0.026928, 2.212987, 0.846804, 0.111744, 1.078465, 1.602420, -0.475870, 1.132794, -0.070563, -0.183990, -0.665412},{-0.424872, 0.249161, -1.292071, 1.867738, -0.768741, -0.855326, 0.370828, 2.875337, 1.613080, 1.295477, 1.494082, 1.040019, 1.026197, 0.473155, 0.363290, 0.614955, 1.180423, 1.894330, -1.323800, 0.551978, -0.622968, -0.864044, -1.000573, -1.745166, 0.617163, -0.594470, 1.047452, 2.375226, 0.418189, -0.276136, -0.677620, -0.566945, 0.627288, 0.380963, -0.766536, 0.090902, -0.367434, 0.665759, 0.658113, -0.065704, 1.068473, -0.332476, 0.863583, 1.042191, 0.434937, 0.493917, -1.765916, 0.080610, 1.567211, 0.287957, -0.667040, -1.619614, 0.637669, -1.007897, 0.336358, 1.049226, 1.011023, 0.698414, -1.831858, -2.331305, 0.778206, 0.848785, 0.142955, -1.193299, -0.405373, 0.433222, -1.714917, 0.678521, 0.570640, -1.253710, 0.489986, -0.698877, 0.090349, 0.466381, 2.192538, -1.506169, 0.218520, -1.530918},{0.819245, -0.783391, 1.608687, 1.720893, 0.223227, -0.821704, 0.375543, -0.127260, -0.069467, 1.293859, 0.073363, 0.869685, 0.750073, -0.695664, 0.249962, -0.393154, -0.348325, -0.576939, -1.116438, 1.210546, -2.246543, 1.273677, 0.110882, -1.202105, -0.002424, -0.764671, -0.555551, 0.484152, -0.877362, -0.838036, -0.678463, 1.182301, 1.561438, 0.047829, -0.383763, 0.261999, -0.905571, 0.167790, 1.252175, 0.081267, 1.828846, -0.529818, -0.165437, 2.255794, -0.140746, -0.664779, 0.237500, -0.625343, -0.395384, 0.075791, -0.978821, -0.084137, -0.780864, -1.540007, -0.774883, 2.468979, -1.417050, -0.096810, 1.255396, 0.584391, 0.083572, 0.907447, -0.196585, 0.436629, 0.621020, 0.998370, -0.948427, -0.372421, -2.696106, 1.014236, 0.445149, 0.219445, 0.912190, -0.145392, 2.374080, -2.173167, 1.445491, -0.165990},{0.194909, 0.481945, 0.063635, 0.998442, -0.392546, 0.028129, -0.753177, -0.261488, 0.966564, -2.648123, 0.710965, 0.295607, -0.718383, -0.929442, -2.052226, 1.265311, -0.163347, -0.928931, 0.057212, 1.136982, -0.402179, 0.221946, 0.179944, -1.297317, 0.638095, -0.097285, 1.016131, -1.817574, -1.783938, 0.064680, -0.720842, -1.168475, -0.309392, -0.399472, 0.199228, 0.442252, 0.163833, -0.520178, 0.251788, -0.471886, 0.687080, -0.402843, 1.426143, -0.023183, 0.674995, 1.463007, 0.829053, 1.041115, 0.206476, -0.953489, 0.867079, 1.425562, -0.939457, 0.511129, 0.377570, -2.188773, 0.905136, -0.775643, -0.104175, 0.426255, 1.270665, 2.461678, 0.930117, 1.284354, 1.559779, 0.148685, -0.253979, 0.250673, 0.176069, 0.972161, -0.191449, 0.116869, 0.490243, -0.058672, 2.051862, -0.624291, -0.575989, 0.145456},{-0.873201, -0.242366, -0.161986, -0.385293, -0.573140, 1.592076, 0.093114, -1.060521, 0.013065, 0.262195, 1.090711, -0.257985, 1.484514, -0.959198, -0.719321, 2.015253, 1.402338, -0.646575, 0.189154, -0.918868, -0.200253, 1.376326, 0.756219, -0.280930, -1.164298, 0.861871, -0.911801, 0.697500, 0.305450, 1.229599, -0.284993, 0.215270, -0.343771, 0.322292, -0.208298, -0.400638, -0.601971, 1.130018, -1.590713, 0.934714, 0.136264, -1.047899, 0.459562, -0.308109, 1.405684, -0.355417, 0.358722, 1.017889, 1.386087, 0.645688, 1.032504, -1.703518, 1.199960, -0.244519, -0.470844, 0.859605, -0.952872, -1.803258, 0.611145, 0.951550, -0.312916, -0.218690, 1.154793, 0.205868, 1.487135, 0.340251, -0.101030, -0.524874, 1.673515, -1.769815, -0.298351, 0.326174, -0.193078, -0.015587, 2.146288, -0.855720, -0.109076, 0.268797},{1.522112, -0.554757, 0.543631, -0.647036, 0.681230, -0.178704, 0.173263, 0.194886, -0.453186, 1.580369, -2.377560, 0.117417, 0.760083, 0.513568, 0.480001, 1.056732, 0.002616, 0.180547, 0.160004, 0.600145, 0.643505, 1.984174, -0.325507, 1.155249, 1.710823, 0.651109, -1.859217, 1.823801, 0.781775, 0.300882, 0.164645, 1.400237, -0.834835, 0.286992, -0.570217, 0.084968, -0.409607, 0.405105, -1.399379, -1.756746, 0.329736, 1.039603, 0.324738, -0.168235, -1.747163, -0.483727, 0.525898, -0.919585, 1.126786, 0.842222, -1.283379, -0.188002, 0.294628, -0.477486, 0.948069, 0.212109, -0.263123, 0.406570, 0.092468, 1.795205, 0.089190, -2.187657, 1.378719, 0.075160, -0.456030, 1.351141, 1.367106, 0.930814, 1.198950, -0.159497, -1.399409, -0.854693, -1.092290, 0.913892, -1.486535, -2.104182, 0.615488, 0.772059},{-0.098196, -0.951182, -0.070414, -2.409366, -0.950025, -2.320578, 1.158539, 0.411563, -0.767055, -1.516317, 0.885142, 0.170350, 1.262672, -0.894627, -2.488990, 0.467324, -1.948788, -1.306273, -0.532729, -0.103277, 0.350198, -0.585150, 1.608667, 0.917633, -0.438693, -0.700073, -0.870660, -0.426217, -2.111569, 0.582282, -0.846418, 1.231653, -2.279396, 0.929010, -0.937105, 0.509261, -0.589390, -0.758197, -1.214326, 1.361224, 0.499919, -0.749526, 0.926412, -0.441852, 1.242015, 1.262168, -0.046183, 0.378199, 1.554033, -0.238462, 1.843279, 0.845372, 0.745020, 0.151124, 0.787461, -0.934517, -0.497785, 0.378307, 1.711994, 0.138700, 0.169810, -1.417075, -0.602094, -1.493234, 0.245602, -0.364291, 0.456536, 0.444601, 0.993268, -0.738376, -1.411016, -0.971816, 0.839954, -0.109293, -0.122366, -0.912312, -1.961600, 0.576208},{-1.295052, -0.440710, -0.781159, -0.334694, -0.708227, 0.521601, -0.641475, 0.333921, 0.045411, 0.063004, -0.155800, -2.369863, 1.209663, 1.025947, 0.417744, 0.420205, 0.357800, -1.030630, -0.737552, -0.377744, 2.187775, -0.434576, 1.342660, 0.158867, 0.010079, 0.060082, -0.148405, -0.302014, 2.805818, 0.174550, -0.434936, -0.434341, 0.116512, -0.652584, -0.889236, -0.064145, 0.276515, -0.372973, 0.843520, -0.044930, 0.067426, -0.911843, 0.284769, -0.037838, 2.890859, -2.273438, 0.940086, -0.240961, -1.146637, 0.391389, 1.061713, -0.894259, -0.545493, 1.055789, -0.203398, -0.215115, -0.109987, -0.116167, 0.543425, 0.905977, -1.412040, -0.020914, -0.332074, 0.299265, 0.433056, -0.816727, -2.658008, -0.835774, -0.330090, 0.564940, -0.935246, 1.297334, 0.853624, -0.869041, -0.631460, 1.730177, 0.435758, 0.529687},{-1.357966, 0.304674, 0.810077, -0.014502, 0.233371, -0.562118, 0.964892, -1.056613, 0.023487, -0.752270, 0.662290, 0.305960, 0.928991, -1.906051, 1.002833, 0.263529, 0.189873, 0.529458, 1.902740, 0.483650, 1.283085, -0.544505, -2.018432, 1.556634, -0.243716, 0.124021, -0.273232, 0.508492, -0.062731, 1.223386, 0.402591, -0.097349, -0.921811, 0.721435, 0.144619, 1.276341, 0.571609, 0.645800, -0.400782, -0.202593, -0.794633, 0.141295, 0.194062, 1.824579, 0.374993, -0.017174, -0.500234, -0.631533, -0.846538, 0.459302, 0.344259, -0.671089, 0.065425, 0.785866, -1.370669, 0.352541, 0.552598, -0.064037, 2.256415, 0.468047, 0.038246, 0.563176, -0.884540, 2.335051, 0.581802, -0.613532, 1.045363, -1.179737, 1.835181, -2.022406, -0.665186, 1.001323, 0.292343, 0.744884, 0.494626, 1.076603, -0.233366, 0.997857},{1.003159, -0.728070, 0.900938, -1.416745, 1.224599, -0.333097, -0.611141, 0.888057, -1.017428, -0.079564, 0.701672, -2.893494, -0.470836, 0.710407, 0.611246, 1.060288, -0.995411, -0.100683, -1.729307, 1.160269, 1.485552, 0.025897, 0.314576, -0.141227, 0.700790, -1.382841, -0.449156, 2.261625, -0.139529, -0.179876, -1.027594, -0.820573, -2.279763, 0.038573, -0.234792, -0.209471, -0.524511, 0.112180, 0.079694, 1.191706, 1.441111, 0.401776, 1.338330, -0.125161, 0.074277, 0.263850, 0.221567, -0.236514, -0.800265, -0.249462, 2.225315, 0.901881, -1.434481, 1.597998, -0.276051, 1.112642, 1.035147, -0.687190, 0.245032, 0.447067, 1.318213, 0.404190, 1.934345, -0.180192, -0.421347, 0.626595, -0.748078, 0.918140, -1.640023, -0.724381, -0.812061, 0.287982, 0.126886, -2.080269, -1.538835, -1.678225, -0.313528, -0.153525},{-0.508192, -0.135620, -1.797999, -0.647548, 1.003747, 0.178359, 0.083042, 0.566896, -0.658329, -0.457592, -1.455272, 0.531358, -0.310749, 0.506173, 0.106116, 1.264649, -1.366160, 0.202801, 2.126061, 0.311244, 0.493469, -2.055073, -0.350486, 0.750406, 1.676839, -0.419723, 0.580813, -1.054911, 1.451023, -0.703320, -1.477667, 1.746915, 0.433207, 0.833077, 0.369743, 1.070668, 0.088371, -0.377892, 0.039870, -1.751465, -1.366589, 1.138581, -0.419144, 0.541612, 0.726206, -0.714748, -2.319082, 0.865585, 1.024770, 0.512238, 1.384255, -0.935734, -0.785270, 0.902289, -0.239599, -1.943165, 2.667500, 0.605368, -0.410427, 0.331228, -0.181716, 1.279263, -1.841851, 1.610183, 2.011326, 0.446721, -0.806328, 0.726728, -1.126437, -1.638511, 1.112337, -0.051916, -0.840220, -0.468929, -0.679572, -0.966396, 1.367535, -1.134149},{0.770520, -0.420953, -1.346587, 0.941657, -0.977383, -1.460854, 0.271893, 0.231447, -1.880882, -0.886266, 0.581572, -1.558775, 0.756844, -1.515255, 1.038134, -0.964533, -0.836700, 0.730451, 0.627083, 0.721289, -0.516530, -0.901013, 0.390264, 1.491478, -0.628614, -2.140702, 0.004641, -1.303194, -0.859624, 1.483496, 0.288025, 2.039630, 0.041247, 0.120487, 1.248362, -0.837673, -0.435850, 0.034449, -0.152885, -0.290654, 0.421217, 1.608878, -0.447576, 1.684639, -0.245482, 1.412743, 0.397771, 0.428602, 0.181071, -1.615916, -0.456887, -0.226799, -0.149065, 0.401386, 1.489425, -1.092758, -0.497806, 1.492609, 1.172911, 0.022207, -0.837395, -0.185171, 0.457980, -0.175833, 0.829282, -1.256807, -0.339090, -0.799720, -0.336029, -1.611212, 0.355039, 0.885361, 0.448117, -0.387020, 1.202371, -0.193446, 0.067446, 0.881905},{-1.064430, -0.087828, -1.031140, 0.559680, 1.176289, -1.081963, -1.658293, 0.577235, 0.246783, -1.737580, 0.340424, 0.001275, 0.273786, -1.117620, -0.252103, -1.594174, 1.123902, 0.513361, 0.064078, 2.176206, -0.680054, -0.294326, -0.145959, -1.041838, 1.076025, -0.270193, -0.217872, -0.817067, 0.789398, -0.011747, -0.176724, 0.395252, -0.905947, 0.269966, -1.884208, -0.377611, -0.720100, 0.616662, 0.634771, 0.831526, -0.328389, -0.009364, -0.805507, 1.213006, -0.323367, 0.295723, 0.172741, -0.116782, 0.829438, -0.769499, -0.360345, -0.310219, -1.388145, -1.170946, -0.463540, 0.885774, 0.247631, 2.205068, -0.321815, -0.017100, 0.444016, 1.159450, 0.236582, -0.160283, 0.654974, 0.878974, 0.334684, -0.693771, 0.615698, 0.716549, 2.462000, 1.516442, -0.555717, -0.229144, 1.029701, 0.052235, 0.273517, -0.216887},{0.329707, 0.367400, 0.780039, 0.928597, 0.580802, -0.844885, 1.823114, -0.670043, -0.684984, 1.250686, 0.670670, 0.707786, 0.489835, -1.415802, -0.564501, -0.464169, -0.767875, -0.871189, -0.245440, -0.128299, 0.048550, -1.666146, 0.202775, 0.782008, 1.582929, 1.761573, -2.262312, 0.995494, 0.060014, -0.427322, 0.560360, 0.107916, 3.280004, -0.338270, 0.892023, -0.629803, -0.364755, 0.558072, 1.443754, 0.124812, 0.137876, 0.532445, -0.068216, 1.041412, -0.016703, -0.319365, -1.927935, -0.860319, 0.943375, -1.770404, -0.342200, -0.455244, -1.236302, -1.038837, 0.980522, 0.910561, -0.454330, -1.526982, 0.221238, 1.028035, -0.449081, -0.929361, -0.845065, -1.319330, 1.130101, 1.403905, 0.754788, 0.317721, -1.166495, 1.111567, -0.433913, -2.063575, -1.083568, -0.643774, -1.017632, -0.473429, -0.443898, 0.034968},{-0.462906, 0.259756, 0.585535, 0.320492, -0.561614, -1.072892, 1.360916, 0.795805, -0.186095, 0.948412, -1.320932, 0.936733, 0.718811, 0.833176, -0.382260, -0.888613, -0.423856, 1.643594, 0.639788, 0.252645, -1.153712, -0.517033, 0.685202, 1.437843, 0.984719, -1.019595, 0.353132, 1.910409, 1.002423, 3.104665, 0.541144, 0.001390, -0.446163, -0.824236, 0.277110, 0.840692, -0.892639, 2.052623, 0.011984, 0.013052, 0.418081, 0.539496, 0.261460, -0.152929, 2.185179, 0.535026, 1.788783, -0.111235, 1.310008, 2.183361, 1.101859, 2.024420, -0.813730, -0.216712, -0.268850, 0.664679, 0.408582, -0.331507, -2.154660, 0.299026, 2.066091, -0.133014, -0.065808, -0.039622, 1.511299, 1.317693, -0.766738, 0.388819, -0.717624, -1.294403, 0.532745, -0.848319, -1.208866, 0.328981, -0.081912, -1.296884, 0.247564, -1.003766},{-0.575213, 1.287279, 1.196347, 0.420885, 0.334881, 0.806913, 1.680019, 0.682573, 0.239636, -1.757203, 0.065132, 2.261748, -0.679658, -1.986812, -0.774692, -0.333063, 0.437382, 1.635522, 0.775621, -0.546663, 0.466703, -1.927608, 1.040959, 0.287383, -1.912982, -1.986414, -0.463187, -0.065513, -1.085896, 0.422356, 0.063151, -0.431549, -0.466185, 0.505155, 1.005449, -0.324778, -0.250298, 2.001134, -0.100497, 1.528260, -0.101551, 0.920202, 0.091299, 1.408945, 0.497571, -0.598819, 1.212727, 0.276467, 0.615901, 0.750598, 0.586252, 0.179214, -0.311489, -0.573903, -1.821112, 0.005004, -0.413919, -0.437541, -0.301904, 0.252716, -0.433680, -2.101657, -0.214156, 2.686988, -0.776858, 1.831644, -0.065243, 1.433319, -0.559761, -0.616964, 0.276706, -1.546630, 1.105014, -0.309351, -1.370588, 1.234902, -0.008943, -1.242079},{1.406779, 1.055122, 0.422531, 1.317605, -0.426628, -0.203629, -2.638998, 0.289945, -0.532331, 0.408321, -0.245053, -0.362518, 0.173504, 0.313716, 0.261084, -1.378751, -0.955900, 2.627573, 1.381526, -0.107164, 0.362801, -3.323941, -0.044014, -0.833906, -0.458876, -0.940482, -0.396844, -0.040165, -0.286130, 0.039462, 0.359890, -0.182782, -0.121529, 0.144931, -0.402933, -0.537980, -1.515171, 0.652380, 0.552857, 0.097723, -0.401971, -1.342909, -1.061448, 1.644385, 1.218540, 1.412847, -1.838215, 0.734251, -2.312287, -0.111020, -0.979364, -1.645597, 2.175902, 1.082809, -0.348248, 0.963802, -0.618782, 0.465542, 0.068507, 0.481739, -0.207956, 0.517098, 1.398819, 0.820162, -1.061695, -0.681342, -0.500766, 1.270432, 0.152331, 1.820819, -0.346625, 0.038172, 0.241516, -0.999040, 0.161845, 0.632217, 1.549880, -0.389191},{-0.701058, 0.060596, -0.446582, -0.573172, 0.671041, 0.120659, 0.499305, 0.342005, 2.261204, -0.478528, 0.640723, 0.673167, -1.542400, -1.648789, 1.446059, 0.012716, 1.354968, -1.662299, -0.331723, -0.436831, -0.603312, -1.533721, 0.750779, -0.599146, -1.121854, 0.211131, 0.781092, -1.247048, 0.618289, 0.762182, -0.819970, -0.622359, -0.781338, -1.307001, 1.148028, 1.758671, 0.449410, 0.607956, 0.527136, -0.666590, -0.522010, -0.111030, -0.824405, -0.492414, -0.116020, -1.032810, -0.141548, 0.172551, 0.865194, -0.753028, 0.308243, -0.565750, -2.156764, -0.237365, -0.755132, -0.874166, -0.769460, -0.408697, -0.527429, 0.349297, 0.281155, 0.331400, -1.435130, 0.138958, 1.055755, -0.930124, 0.537626, -1.312973, 0.721861, -0.722193, 0.481360, 0.921579, -1.476130, -0.738058, -1.335684, 0.747237, 0.004636, -0.408282},{2.131874, 0.355693, -1.026542, -1.963945, -0.204670, -0.048706, 1.962163, 0.051951, -0.987999, 0.521879, -1.122721, 1.482751, -0.874905, 1.633886, -1.710708, 0.817928, 0.941635, 1.618801, -0.364603, -0.051161, -1.311305, -0.855656, -0.765864, -0.220036, 1.269024, 0.834519, 0.916045, 1.033629, -1.329088, 0.732556, 0.601739, 0.671966, -0.922432, 0.089859, 0.778533, -0.208652, -2.499785, 0.638525, 1.266303, -0.909892, -1.919921, 0.401598, 0.392296, -0.739387, -0.039449, 0.084164, 0.115508, 0.906507, 0.295044, -0.766582, -1.499316, 0.158602, 0.350740, 0.340412, 2.118366, -0.065880, 0.702587, -0.193729, 0.405666, 0.669873, -0.570530, -0.225879, 0.089090, 0.905779, -1.044685, 1.254049, 0.226426, -0.864897, -1.828255, 0.895023, 0.476588, -0.355571, 0.594116, 0.957923, -0.693933, -0.381012, -0.868395, -1.811496},{-1.667803, -0.581529, -0.120078, 0.501045, -0.826744, 0.197613, -0.210706, -0.132883, -1.832935, -1.140112, 0.400233, 0.153160, 1.592114, -0.341105, 1.623568, 0.207637, -0.835588, -0.237888, -0.194013, -0.427422, -0.360326, -2.023952, 0.729256, -0.698380, -0.896453, 0.760874, -0.229901, -0.939184, -1.532774, 1.857872, -1.800451, 0.034482, 1.430815, 1.023981, 0.719369, -0.490213, -0.641541, 0.353184, 0.837251, -0.452527, 0.241298, 0.809069, 0.412585, 0.225785, -0.633960, -0.501362, 0.701041, 0.671307, 0.020839, -0.727563, -0.383660, -0.285168, 0.106433, 0.597406, 0.879484, -0.360154, -3.801574, 0.111183, 0.140738, -0.077858, 0.388592, 0.078865, -0.396160, 0.365172, 0.986352, -1.159731, 1.506080, 1.564710, 0.789319, 0.756027, 1.234820, -0.316289, -0.020024, -0.993648, 1.381294, -1.420956, 1.228651, -0.855904},{0.037803, -0.456995, -0.732199, -0.106995, -0.832090, 1.537443, -1.397218, 0.337015, 1.463635, -0.562921, 0.044490, 0.193325, 0.168770, 1.190624, -0.912492, -0.075525, 0.166290, -0.002337, 0.597419, 0.865376, 0.831677, 0.286938, 0.952540, 1.403770, -0.097185, -0.116189, 0.139913, -1.261681, -0.035489, -0.305011, 0.497578, 1.331496, 0.358319, 0.126225, -3.185301, 0.033550, 0.106827, -0.089813, -1.828285, 0.844156, -0.652142, 0.707832, 0.422017, -0.141002, 0.077213, -1.507843, 0.072626, 1.026718, -0.288543, -1.692599, -0.958874, 1.200446, 1.079903, -0.578656, 0.312636, 0.128351, 0.261937, 0.156950, 0.964488, 0.722429, -1.469321, 1.235000, -0.461032, -0.363344, -1.961073, -0.721199, 0.320135, -0.167298, 0.137774, -0.154499, 0.546150, 1.591811, 1.195592, 2.158122, -0.215460, -0.860339, -0.277242, -1.085198},{-0.831120, 0.790229, 0.790503, -1.766949, -1.542321, 0.410314, -2.610716, 0.070650, 0.415688, 1.446754, -1.155291, -1.231704, -0.126003, 0.037232, -0.741758, -0.790501, -1.667510, -1.760587, -2.059875, 0.508949, 1.025948, 0.779020, -2.294051, -0.120467, 0.039759, 0.097020, 0.961008, 1.912555, -1.399120, -1.677991, 0.158795, 0.143137, -0.443302, -0.401661, -0.521947, 0.420059, -1.871376, -1.255183, 0.290799, -1.631129, -1.159482, -0.122883, -0.203989, -0.245066, 0.314041, -0.458133, -0.256095, -0.100914, 1.067358, -0.031967, -0.253876, -0.990399, 0.705091, 1.612476, -0.217907, -0.761819, 0.800469, -1.224503, -0.134471, -1.704759, 0.365461, 1.148183, -0.648032, 0.001804, 1.086684, 0.997859, 0.555793, 0.118990, -0.584880, -0.890947, -0.845962, -1.251761, -1.080039, -1.445320, 0.387004, 1.928651, 0.137894, 0.580048},{-0.849383, 1.523623, -0.618584, -0.620991, 0.718890, 0.720919, 2.093552, -2.620111, -0.208528, -0.676455, 0.692375, 0.074953, -1.295163, -1.549336, 0.307805, -2.145490, -1.193070, 0.230536, 1.859690, 0.359207, 1.570445, 1.279683, 0.002878, 0.402969, 0.147742, 0.577469, 1.313156, 0.936473, 0.746075, 0.082932, 0.985824, -0.273860, 1.260831, 0.716142, -1.030030, -0.780845, -0.035689, -1.050440, -0.076226, 1.208357, 0.051897, -1.170910, 1.128697, 0.997598, 0.639988, 0.593359, 0.583701, 0.535922, -0.343844, -1.013152, -0.438153, 0.339213, 0.259957, -0.068667, -2.437286, 1.616208, -0.606544, -0.424260, -0.718946, -0.774777, 0.982407, -0.232630, 0.756771, -0.204043, -1.697771, 0.290758, 2.390516, -2.152436, -0.200575, -0.677584, -0.902552, 0.003676, -0.135557, 0.198658, 0.566375, 0.432479, -0.251216, -0.829832},{-2.410872, -1.264171, -0.738510, 0.201238, -1.324019, 1.276956, 0.255784, 0.062674, -0.759503, 0.639135, -0.725840, -1.538296, 1.613393, -2.199086, -0.737167, -0.482233, 1.344294, -0.634854, -1.439875, -0.364548, 0.655082, -0.741865, 0.868312, 0.800602, -1.245994, -0.331967, -0.081864, -0.796628, 2.639253, 0.558970, -0.459772, -0.116864, 2.025918, -0.111107, -0.810085, 0.603614, -0.632800, -0.178155, -0.587546, -1.383452, 0.013339, -0.558234, -0.819991, 1.012308, -0.563997, 0.261703, -0.345152, -0.606432, -1.341437, -1.430029, 0.550112, 0.501346, -0.853155, 1.173861, 0.502738, -0.511271, -0.336785, -1.139982, 1.517746, 0.103864, -1.550148, 0.624486, -1.362471, 0.878439, 0.283555, -0.170247, 0.434360, 1.024267, -0.216441, 0.074577, -0.895588, -0.319504, 1.497558, -0.780845, -0.457824, 0.321427, 1.019441, 0.254304},{-0.879836, -0.184851, 0.228112, -1.960792, 0.694139, 0.098584, 0.689099, -0.915429, 0.337260, 1.129365, -0.423635, -1.564683, -0.948425, -0.780080, 0.969791, -1.127808, -0.539365, -0.388727, 0.333507, 0.122837, 0.995211, -0.056480, 0.225750, 1.792288, 0.304301, 0.235624, 0.131749, 0.086705, -0.964335, -1.299405, 0.516510, 0.165039, -1.899323, 0.655843, 0.408722, 0.275676, 0.298884, -0.439987, 0.180767, -0.456425, -2.495865, 1.803430, 1.868127, -0.151062, 1.452431, -0.649148, -0.643052, -1.810022, 0.277690, 0.526460, 1.351980, 0.147617, -1.546102, 0.597278, 0.429835, 0.210229, -0.981217, -0.019471, 0.138474, 0.611308, 1.431454, 0.097362, -0.141853, 1.266668, -1.282638, 1.000878, 1.738141, -0.015187, -0.749818, -0.587161, 0.373076, -0.451645, 1.243635, -0.250170, 1.311602, 0.789408, -1.122192, 2.124770},{1.622167, 0.533443, 0.062459, -1.032689, 1.125832, -0.613236, -1.122264, -0.772131, -2.363557, 0.210551, 1.818814, -0.523991, 0.071201, -2.110294, -0.321706, 0.461690, 0.204882, 1.160826, 0.191236, 0.276222, 0.212154, 0.649688, -0.232913, -1.183779, -0.295988, -0.843053, -1.404105, 1.463581, -1.707187, 0.299707, -0.399915, 2.677441, 0.104441, 2.242530, -0.601752, -1.138239, 0.678280, -0.597419, 0.395818, -0.167337, 0.822602, -1.096208, -0.239647, 0.235287, 0.282368, -0.861028, 0.939877, 1.464596, 0.781927, 1.125048, 0.208824, 1.376697, 0.630124, -0.310353, 0.101343, -1.451597, 0.559474, -0.773597, 1.485116, 0.711839, 0.759128, 0.800246, 1.713137, 2.506139, -0.043136, 0.145472, -0.291071, 0.968419, -0.271658, -0.008737, 0.123751, -0.310863, -1.830119, 0.720645, 0.163721, 0.978243, 0.436439, 0.965064},{-0.959560, 1.227651, -0.216475, 0.269669, 0.314499, -0.253023, 0.013210, -1.208214, 0.075644, 1.102687, -0.412526, -0.583345, -0.256738, 1.051508, -1.642515, -0.019817, 0.581418, 0.582839, -0.768547, 1.059895, 1.717877, -0.350010, -1.356269, -0.285482, -0.796436, 0.535581, 1.244487, -1.044887, 0.435507, 0.422021, -0.407855, -0.042782, -0.111038, -0.624249, 0.418741, 0.578124, -0.923059, -0.491214, -0.264082, 0.258559, -0.154542, -0.206531, -1.491579, 1.424660, 1.460014, 1.137043, -1.533306, 1.122716, -1.251468, -1.240633, 2.043486, 1.131720, 0.042371, 1.062467, 0.258750, -0.661156, 0.612010, 0.683736, -1.367513, 0.338165, -0.227782, 0.584513, -0.240571, 0.281991, -1.352530, -0.659574, 0.567870, 1.071250, 2.976526, 0.028645, -1.126580, -0.167146, 1.060056, 0.068309, -0.111261, -1.274917, 0.328502, -0.823919},{-0.592991, -0.792189, 0.280966, -0.311120, -1.694438, 2.156940, -0.548724, 0.806885, 1.990093, -0.112599, -0.646136, 1.087367, 1.084156, -0.347739, -1.645724, 0.181948, -0.923461, -0.717979, -0.567368, -0.723381, 1.211243, -0.232644, -0.445059, -0.326270, -0.945455, -0.991953, 0.327277, 0.474477, 0.291772, -0.626155, 0.769343, 0.616587, -0.117944, -0.749529, -1.324496, 1.281743, 0.232681, 0.468564, 1.354522, 0.023785, -0.713827, 0.745881, -0.660762, -0.075665, -1.622541, 0.962015, -0.323692, -0.600262, -0.015218, -0.090268, -1.331703, -0.105409, 1.372231, 0.884649, -1.528668, 0.944257, 1.657866, 0.578656, -0.578251, -0.214299, -0.347454, -1.559692, -0.438718, 1.387881, 0.843827, -0.168835, -0.537167, -0.677114, 1.212477, -0.726008, 0.440244, 0.691073, 1.480090, -0.878907, -1.243969, -0.983872, -0.958929, 1.078223},{-0.148820, 1.335674, -0.878142, -0.638929, 0.218962, 0.543117, -0.260953, 0.624888, -1.341230, 0.552178, -0.327110, -0.136337, -1.063390, 1.328344, 2.466250, 0.136393, -1.278990, -0.328832, 1.056869, 1.089500, 0.584213, 1.712093, 1.087212, 0.108885, -1.501895, -0.378168, 0.531245, 0.231400, 0.091512, 0.538198, 0.916633, 1.007453, -1.734063, 0.950220, -0.163224, 0.346805, 1.008733, -0.216757, -1.131188, 0.368127, 1.046118, -0.189916, 0.777794, -1.112860, -0.280628, 2.131239, 0.374269, -1.262956, 2.046785, 1.343500, 1.547519, 0.328675, 0.356601, 1.075243, -1.061584, 0.010697, -0.466547, 0.575935, -0.260455, -0.787434, 0.332064, -2.299033, 0.324077, 0.592131, 0.521473, 1.399397, 0.886035, 0.560743, -0.525934, 0.619973, -0.003476, 0.743007, 1.344140, 0.498055, -0.667893, 1.986422, 0.293655, 0.709435},{-0.976649, 0.854733, -1.613985, -0.164051, 0.198914, -0.349442, 1.238068, -0.081439, 1.465836, 0.276031, -0.912910, -0.075192, 1.047021, -0.429406, -0.764413, 0.701631, -0.288814, -0.173922, 0.909595, -0.733370, -1.821403, 0.227223, 1.290685, 0.730416, -2.398862, 0.223825, 1.005900, 0.590843, 0.678548, -0.021679, 0.711900, -0.687440, -0.822879, -2.382527, -1.245692, -0.679214, -0.463996, 0.640039, -0.198630, 1.607338, -0.042681, -0.339347, -0.118378, -1.242370, 0.294835, 0.240136, 0.590527, -0.032599, 1.562332, -0.746556, 2.024237, 0.898060, -0.882574, -0.781632, 0.745580, 0.363560, 0.573624, 0.667770, -1.372407, -0.368480, -0.943751, -0.621780, 0.737986, 0.076694, 0.990589, -0.216747, -0.691170, -1.328640, 0.631874, 1.634202, 0.945153, -0.087321, -0.288397, 0.264542, -0.096854, 0.229547, -2.351505, -0.085850},{0.341214, -1.306376, -1.060360, 1.487273, -0.312140, -1.243207, -0.166007, -0.132846, -1.850839, -0.324151, -0.604356, -0.371342, 1.339694, 1.785685, -0.759107, -0.094871, -0.143592, -1.265250, 0.075649, 1.603217, 0.455517, -0.480025, 0.160483, 0.415383, -2.253802, -0.104727, -0.772248, -0.351369, 0.991623, -1.121373, -0.467245, -0.964208, 0.593487, 1.535583, -0.405720, 1.060422, 1.293783, -1.510263, 1.132581, -0.701390, 0.366942, -0.134444, 1.374466, -0.040959, -0.761119, -1.128342, -0.249629, -0.938005, -0.776953, -1.198405, -0.885937, -1.592189, 1.661340, -1.837175, -0.054054, -1.991756, -1.996570, 0.352109, -0.209184, -0.421768, 0.563730, -0.868019, 0.344815, 0.189592, -0.592378, 0.178825, 1.788546, 1.425485, -0.602929, 0.405749, -0.639797, -0.558139, 0.356823, -0.581037, 0.449377, 0.758963, -0.098090, 0.285538},{0.725127, -0.449906, 0.485415, 0.765476, 1.240018, -2.171417, -0.343043, -0.570927, -1.238105, 1.386409, -0.104582, 0.852350, -1.408536, -0.053385, -0.629444, -0.722154, -2.187022, 0.080256, -1.248381, 1.739900, -0.823329, 1.550933, 1.521635, -0.596241, 0.440043, 1.502105, -1.124075, -1.083127, 0.130893, -0.135736, -1.246394, 0.239589, -1.176061, 0.344524, 0.357652, 1.287884, -1.610480, 0.168254, -0.322524, -1.876827, -0.622483, 0.867967, 0.702260, 0.074335, -0.427487, -0.734148, -0.130010, 0.130693, -2.600644, -0.436127, -0.005343, -0.262335, -0.471471, -0.673641, -0.066051, 0.282192, -0.629815, 0.379580, -1.063995, -1.711647, -0.659273, 0.240847, -1.493853, -0.839465, 0.282796, -0.168997, -1.339339, 0.977861, 0.195977, 1.036687, 0.838132, -0.093093, -1.053020, 1.415272, -0.337440, 0.268360, -1.063814, 0.106584},{0.829278, 1.024692, -0.848921, -0.641773, 1.145716, 1.131910, -0.464716, -0.438882, -1.188746, -0.188205, -1.056115, -2.464079, -0.636091, 1.293998, -1.411473, 0.663023, -0.012587, -0.658744, -3.155060, -0.398429, 0.248761, 0.415714, 2.583112, 0.961064, -0.765162, 0.956035, 1.799797, -0.390527, 1.404513, 0.048286, -0.091013, -2.082654, -2.508014, -1.003587, 0.577806, 0.320939, 0.647167, -0.680785, 1.079779, 1.087468, 0.590305, 0.732065, 0.133681, -0.424447, -1.575538, 2.462236, -0.579939, -1.360274, -0.610865, -1.499555, -1.387361, 0.335924, -1.196453, 0.577356, 0.640118, -1.354496, -0.852928, 0.997658, -0.286676, -1.193168, -0.198205, -1.136704, -0.534089, -1.954405, 0.135888, 0.250006, -0.675590, 1.551179, 1.147799, -0.331470, -0.677975, 2.659000, -0.074620, -0.563588, -0.872215, 0.051155, -2.381966, 0.702158},{-1.576281, 1.065229, -0.680338, -0.279283, 2.209193, -0.853933, 0.419795, 0.577311, -0.749226, 2.436402, 1.496781, 0.267227, -1.357888, -1.529281, 0.095868, -0.202052, -2.253128, 0.643470, 0.378593, -1.576426, -0.146443, -1.906176, -0.697455, -1.166111, 0.848443, -0.701530, -0.573928, 0.768560, 1.026608, 1.165692, 0.783723, -0.568824, -0.667686, 1.543598, -0.065106, -0.529648, -0.541335, -1.357927, 1.473364, -1.690562, 2.772734, -0.462845, 1.387468, 0.279387, -0.710842, -0.544478, 1.238051, -0.409093, -0.016520, 0.096538, -0.225857, -1.843877, -0.016619, -0.434838, 0.163730, -0.192566, 2.192121, 0.148435, -0.817566, 1.800324, -0.026951, 0.792215, 0.274206, -0.871072, 0.907761, -0.256481, -0.474238, -0.440087, -0.370510, 0.541439, 0.944584, 0.987246, -0.573709, -0.134744, 0.220311, -0.848367, 0.016642, -0.067370},{-1.793390, -0.913109, -0.484246, -0.191911, 0.077162, -0.919014, -1.135538, 0.049257, -0.295770, 0.634671, -0.788354, -1.533538, -0.107819, -0.806748, 2.143855, -0.899558, -0.731026, -0.811485, 0.353363, -1.049305, 0.070388, -0.127062, -1.533692, -0.888621, -0.717793, -0.438904, 1.877816, 2.765506, -0.429662, 1.235314, 0.178120, -0.889535, -0.841154, -1.326723, 0.065824, 0.941365, -1.397070, -2.775621, -0.184081, -0.457427, 1.178226, 0.956509, 0.342593, 1.578162, 0.567845, -0.200542, -0.448541, 0.392936, 0.334108, -0.079066, 2.123209, 1.228961, 0.423887, -0.018487, 0.377134, 0.124890, -0.733862, -0.096238, -1.271038, 1.457634, 0.379167, 0.820919, -1.404292, 0.163786, -2.444942, 0.001332, 0.034864, -0.244167, -0.464387, 0.381294, -0.861105, 0.653285, -0.001788, 0.105338, 0.521465, 0.182044, -1.646649, -0.259330},{-0.856530, -0.394918, 0.106097, 0.176511, 0.847545, 1.155592, 1.328127, 0.101121, 0.428989, -0.454984, 0.193551, 1.193105, 1.121730, -0.007419, -0.895734, -0.496939, 0.716914, 0.941538, -0.224143, 1.633299, 0.836514, -0.315049, 0.462892, -1.238655, -0.320609, -1.763823, -0.619427, 0.245168, 1.333800, -0.964708, -0.665890, 0.461765, 0.534200, 0.934089, 0.012168, 0.868560, -1.516212, 1.763693, 0.478457, 0.531330, 0.877791, 1.933907, -0.059530, 0.115172, 0.942894, 0.138407, -1.690520, 0.876381, 0.194145, 0.558883, 0.663980, -0.112911, -0.451466, -1.415336, 0.794549, 0.650410, 1.325417, 0.285789, 1.209923, 1.131651, 0.544154, 0.696629, -0.233005, -0.471007, 0.836573, 0.132962, 1.178376, -0.631555, 0.361979, -0.918219, -0.587647, 0.794936, 0.556699, -0.322038, 1.583318, -1.053345, -0.109412, -1.719259},{-0.894079, 0.047900, -0.082826, -0.447547, 0.693713, 1.688774, 0.596852, 1.144623, 0.347141, 1.838256, 1.472339, -0.142672, -0.960230, -0.562897, 1.209576, 0.567138, 1.383824, 0.433657, 0.417356, -1.262222, -0.021295, 0.894762, -0.030693, -0.631489, 1.283876, -1.046873, 0.270140, -1.488791, -0.281711, 0.749460, -2.598966, 0.063354, -1.347227, 0.617952, -0.768633, -1.050254, 0.558935, 0.178100, -1.036351, 0.191567, -0.375737, -0.484653, -0.170591, -0.553210, -0.442501, 0.427781, -0.269733, -0.781970, -1.577563, -0.762832, 1.035226, 1.117797, -1.019606, 0.007161, -1.966951, 0.389833, 2.107259, 1.188248, 0.074670, 0.771193, -1.301248, 0.938745, 0.882182, 0.421671, -0.137223, -0.165651, -1.080058, -1.373130, 0.048403, -0.458493, -0.508156, -0.636150, -1.278823, 0.046374, 0.895807, 0.073763, 0.928056, 1.078463},{-1.597766, -0.177969, -0.064830, -0.589248, -0.414217, -1.669459, -2.481844, 0.021635, -0.557305, 1.143336, -0.449795, 0.468073, -0.987333, -0.164921, -0.808997, -0.201946, 1.490253, -0.988443, 0.971494, -0.418948, -0.096931, 0.513187, -0.543846, -1.259801, -0.805475, -0.693412, -0.969258, -0.174631, -0.797823, -0.355650, -0.628375, 0.522787, -0.432947, 0.638384, -0.535284, -1.610940, 0.320978, -1.526470, -0.407705, 0.259438, -0.895174, -0.405195, -1.703024, 0.878797, -0.320869, 0.318881, -1.557128, -0.495578, -1.111014, 0.439279, -1.949189, -0.932689, -0.177207, 1.349307, -0.951648, 0.875372, -1.460482, -2.378689, 0.560276, 1.015868, -0.864036, -2.555652, -0.940978, -0.391270, -1.055445, 1.718771, -0.445251, -0.588458, -0.805682, -0.832871, -0.451043, 0.064780, 1.123974, 0.204199, 1.718569, 1.221323, 1.788209, -0.233976},{-1.256448, 2.305117, -2.547304, -0.527049, -0.152750, 0.809181, 0.413675, -0.483450, -0.290338, -1.688170, -0.584102, -1.310079, -1.313955, 0.776115, 0.420195, 0.193086, -0.422182, -1.949312, -0.537017, -1.398452, 0.469461, 0.965639, 1.384923, 1.642369, -0.056996, -0.204819, 0.374280, -0.491096, 0.817729, 0.111664, 1.634412, 0.793786, -2.249192, 0.361926, -0.015485, -0.989546, -1.112637, 1.984710, 0.825981, -0.957874, -0.072052, 1.146431, 0.335617, 0.412784, 0.037751, -1.673557, 0.952747, 0.086750, -0.267282, 0.145302, -0.210132, -0.612718, -1.021087, -0.159915, 0.058247, -0.012150, -0.648996, 0.151565, 2.318758, 0.511439, 1.238157, -1.507120, 1.099794, 0.615532, -0.870028, -0.133903, 0.708937, 0.613537, 0.310103, 0.732485, 1.250690, -0.567293, 0.669689, -1.477845, -0.151835, 0.439647, 0.434422, -1.350297},{-0.749043, 1.118715, -0.267964, 1.234513, 0.470356, -0.235801, -0.148016, 0.061467, 1.660799, 1.701889, -0.106952, -1.964155, 0.950999, 0.570566, 0.881075, -0.916825, -0.124934, -2.020557, 0.241106, -0.827253, 0.083542, 0.031573, -1.319043, 0.359637, 0.675974, -0.890211, 1.223712, 1.429856, -0.664739, -0.745697, 1.138265, 0.271832, -1.552605, 1.561345, -0.270749, -1.453576, -1.476088, -0.370462, 1.212074, 0.097283, 1.835736, -1.279566, 2.501309, -0.350838, 0.270270, 2.066054, 0.600569, 0.841978, 1.728719, -1.270850, -1.440120, -1.739725, 0.061174, 0.182087, -0.321909, 0.515309, 1.022653, -1.111899, -0.180923, -0.081710, 0.792227, -2.109343, -0.543445, 0.187993, 1.157500, -0.389914, -0.455522, 1.392098, -0.014273, -0.374202, -1.932321, 2.470274, 0.944796, 0.830193, 0.048520, -1.453012, -1.278126, 0.190274},{2.846075, 0.129442, -2.127172, -0.237559, -0.187201, 0.244154, 0.686780, 1.304965, 1.682731, -0.610893, -0.512041, -0.358161, 1.198621, 1.587718, 2.896231, -1.117339, -0.552280, -0.294973, 0.259234, -0.583896, 0.736361, 1.839326, 0.134406, 2.533103, -1.857585, -0.858177, 0.156234, -0.428594, 0.480393, 1.547972, -0.653361, -0.250444, 0.767395, -0.770190, 0.020117, 1.129170, -2.215789, -0.205139, 0.926208, -0.275711, 1.127209, -0.667748, -0.359209, -0.118928, -0.254754, 0.161579, -0.810747, 0.733748, 0.032020, -0.362462, 0.984005, 2.020303, -0.433411, 0.090233, 1.851418, -0.609970, -0.401417, 0.087610, 0.564733, 0.975953, 1.380956, 1.260642, -1.232579, -2.289711, 0.400378, -0.301161, -1.456233, -0.772888, -0.791259, 0.310332, -0.005859, -1.418311, -0.826999, 0.619969, 0.482894, 3.868977, -0.255894, -0.306793},{-0.090828, -0.380383, 0.017477, -0.070491, -0.403121, -0.536633, 1.061822, -1.231085, -0.864954, -2.002047, -0.156668, -1.894520, -1.359539, -0.951623, -0.485962, -0.103719, -1.832891, -0.022388, -0.382540, -2.933065, 0.661892, 0.081268, -1.701665, 1.704454, 0.605117, -0.543629, -0.422492, -0.010678, -0.097359, -0.780361, 0.010383, 1.184066, -0.054494, -1.330531, 0.781881, -0.298706, 2.348066, -0.485232, 1.690914, -1.504718, 0.982779, 1.383344, 0.125067, 0.281208, 0.893207, -0.051554, 0.102272, -0.183969, 0.716309, -1.365938, -1.000247, -0.235792, 0.387225, -1.133715, -0.993310, -1.329395, -0.233396, 0.587169, -0.183233, 0.996121, -0.840779, 0.153629, 0.626882, 0.794556, -0.788800, 0.600056, 1.465888, 0.042657, -0.740290, 1.578951, -0.792032, -1.649747, 1.424173, -0.499758, 4.357475, -0.769279, -0.304047, 1.437260},{-0.470074, -0.134986, -0.346913, 0.080320, -1.193238, 0.054008, -0.494830, 0.057092, 0.339457, 1.308125, -0.160547, 0.537069, 0.136482, -0.994869, -0.314822, 0.564611, 0.478889, -1.542596, 0.477594, 1.518303, 0.201742, 0.218315, 0.547116, -1.266662, 0.225536, -1.603361, -0.531887, 0.424268, -0.686863, 0.983208, -1.167150, -0.146879, 0.090964, 1.127725, 2.131139, 0.137762, 0.872632, 0.542801, -0.123001, 0.487010, -0.916018, -0.770274, -0.931372, -1.361024, 1.920487, -1.330623, 1.250265, -0.006692, 0.939251, -0.583533, -0.817803, -0.055404, 0.425015, -0.030820, 0.019600, 0.002536, 1.149496, 1.211604, -0.055855, 0.508753, -0.278793, 0.244032, 0.417093, -0.905837, -0.431602, -0.613122, 0.541554, -0.209036, -0.779801, -1.926835, 0.536628, 0.171442, -1.975172, 0.030978, 0.885320, 0.933045, 0.081390, -0.496314},{-0.448741, 0.273664, -0.279845, 0.077992, 0.090028, -1.265300, 0.343486, 0.017125, 1.131444, -0.475404, -0.713481, 0.803218, 0.183155, -1.040252, -0.130689, -0.008711, 0.540368, 0.755450, 0.449150, 1.624277, -1.114548, 2.867226, -0.534562, 0.597700, 0.744694, 1.876611, 0.578598, 0.554916, -1.627823, 1.636229, 0.625842, -0.853472, 1.065350, -0.051571, 0.041221, 0.081924, -0.841299, 1.607743, -0.340388, 1.135579, 0.102566, 1.029170, 1.052764, 0.684575, 0.192857, 1.050566, -1.158499, 1.453873, 0.935478, -0.361928, 1.423748, 0.087462, 0.126161, -0.068417, -1.188649, 1.690641, 0.262919, 0.043866, 0.306466, 2.004817, 1.409446, -0.098970, 0.872406, 0.138190, 2.302299, 0.122153, -0.750939, 0.012709, 0.945946, 0.076600, 0.430010, 0.000079, -1.684840, 1.089175, 0.136038, -0.921004, -1.640231, 0.005325},{-0.977134, 0.565063, 1.952077, -1.041228, -1.170709, -1.108185, -0.554772, 0.915247, -1.065114, 0.448458, -0.585656, -0.350563, 0.003809, -0.011977, 0.469774, 0.104510, -0.710167, 1.677107, -1.437158, -0.151361, 1.221506, -1.276961, -1.023643, -0.824056, 2.125331, 0.834394, 0.179017, -1.262121, -1.405878, -0.016146, 0.426003, -0.351413, 0.347885, 2.664144, 1.017235, 1.017127, -0.564445, 0.483763, -0.058792, 0.524764, 0.788938, 0.658484, -2.125070, 0.410399, -0.489244, -1.377907, 0.240284, 0.280447, 1.749748, 1.205210, 1.305962, 2.701367, 0.746866, -0.369516, -0.055003, -0.784805, -0.049743, 0.669303, -0.503346, -2.181326, -1.092737, -1.040379, -0.511745, -0.542943, -0.407495, 1.919914, 0.458964, -0.397714, 0.090578, 0.413066, -3.018965, -1.939765, 0.226963, 0.160913, -1.098822, -0.268927, 0.031889, -0.620956},{0.352352, -0.284445, 0.204970, 0.162823, 0.000607, 1.129600, -0.562665, 1.682284, -1.092547, -1.032205, -1.789612, 1.246908, -0.085718, -0.583274, 0.813127, -0.122588, 2.317856, 2.073121, -0.314014, -0.161597, -1.298074, 1.497029, 0.937737, 0.045877, -3.160944, 0.984842, 0.942274, -0.022404, -0.450796, -1.626795, 0.395619, 0.761745, 0.579689, 1.226088, 0.833670, 0.610520, 0.459586, -0.556153, -1.107918, 0.701704, 1.253584, 0.277494, 1.212212, 0.283284, -0.441288, -1.001059, 1.708422, 0.434980, -1.261640, 1.962624, 1.166977, 0.627170, 0.598382, -1.332365, 0.499150, 0.063295, -0.524100, -0.257028, -0.851680, 0.842121, -0.443928, -0.901814, -0.934568, 1.165043, -0.591617, 1.218785, 0.996447, -0.914093, -0.188247, -1.253483, -0.761266, 1.299286, -1.866086, -0.825329, 1.518930, -0.820371, -0.708862, -1.086239},{-0.320307, 1.786318, 2.727584, -0.725889, -0.202783, -1.231723, -1.869061, 0.143519, 0.187862, -0.740393, -0.776278, -0.493116, 0.387660, -0.381173, 0.759645, -0.694988, -0.248757, 0.163925, 0.843829, -0.468071, -0.667415, 2.438156, 0.439425, -0.449405, 0.150436, 0.409280, 0.888950, -0.498088, -1.026080, 0.776994, 0.249420, 0.940986, -0.228739, 1.826562, -0.802676, 0.311042, -0.034122, -0.149389, 0.743555, 1.117825, 0.947025, 0.164214, 0.065405, 0.192377, -0.773767, -2.296330, -0.255806, -0.050754, -0.791353, 0.184723, 1.419649, -0.946383, -0.081368, 0.914007, 1.324004, -1.622363, 0.512591, -0.852046, -0.308096, -1.061058, -0.312108, -0.809662, -1.649680, -2.158890, 0.886346, 1.010208, 0.617659, 0.441751, 1.093975, 1.134969, -0.754351, -1.329210, 0.022760, 0.393145, -0.347759, -0.712120, 1.537675, -0.594011},{0.942486, 2.284098, -0.380782, 0.057638, 0.599297, 0.198426, -0.055769, 1.184142, -0.144094, 1.517752, 0.361070, -1.500250, -0.074647, 0.538397, -1.920004, -0.481958, -0.321185, 1.802757, -1.653518, -0.697640, 0.279409, -0.442062, 0.728913, -2.070041, 0.460765, 0.918908, 1.704518, -1.093856, -1.328628, -0.238364, 0.510363, 2.519397, -0.813527, -0.411839, -0.358671, 2.959888, -0.565416, -0.043005, -0.341465, 0.375692, -1.060512, 1.189003, 0.116550, 0.731000, -0.324541, 1.073973, 0.891064, 1.750537, -0.592640, -1.146675, -0.698492, 0.842580, 0.147237, -0.263162, 0.572155, -0.033737, 0.570311, -0.324937, 0.708144, -0.809475, -0.573474, -0.253620, 0.333655, 0.072583, -1.587366, -2.237636, 2.220171, -0.646960, -0.359303, 0.947428, 1.341664, 0.333601, 0.852077, 0.958659, -0.505586, 0.117358, 0.006005, 3.517252},{-1.060278, 0.043156, -1.143653, -0.438686, 0.345285, 0.276914, -0.619761, 2.023032, 0.525887, -1.232121, 0.307775, -0.487178, -0.933270, -0.111489, -1.800449, 0.799317, -0.096839, 0.829196, -0.562480, -1.791130, 0.288897, 1.482574, -0.899270, 2.160679, -0.968765, -1.853474, 0.572190, -1.055890, -1.481350, -0.495219, -0.368486, 0.233121, 0.422790, 0.246432, 1.612080, -0.144379, 2.733563, 0.079132, 0.108036, 0.985717, -0.958638, -0.009731, 1.120834, 0.403812, -2.147786, 0.334174, -1.629382, 1.261308, 0.946797, 1.038325, -1.570829, -0.929338, 0.830442, -0.372981, -0.690114, -0.852691, -0.535505, -2.514924, 0.006977, 1.389598, -0.273776, -1.080916, -0.663117, 0.252429, 0.155497, -1.930400, 0.192200, 1.508346, 0.944240, 0.496585, 1.206556, 1.222260, 0.272059, -0.265000, -0.299141, -0.793375, 0.619092, -1.155411},{-0.487708, -0.201500, -0.275502, -0.581368, 0.735894, -0.201831, 1.356070, -1.829314, -1.398605, -1.281300, 1.735456, -0.241106, -0.736675, -2.276768, 0.528621, 0.393226, 0.091360, -0.623713, 1.166384, -1.234218, -1.149265, -0.002821, -2.687695, 1.140847, -1.753017, 0.859754, -0.447177, -0.550149, 0.653531, 2.086555, 0.191494, 0.169868, 0.121062, -1.520390, -0.719954, -0.105347, -0.236352, -0.008598, 1.708847, -1.459152, -1.249565, 0.623315, 0.956561, 0.212140, 0.328382, -0.184764, -0.551480, 0.860514, 1.176422, -0.511515, -0.532715, -0.061992, -0.867836, 0.975637, 0.710620, -0.081702, 0.664577, -0.067932, 0.642263, 0.919335, 1.516617, -0.358114, -0.779253, -0.252938, -0.003112, 0.427328, -0.181484, -1.213647, -0.862252, 0.617520, -0.259669, 1.563358, -0.008725, -0.560508, -0.351539, -0.815219, 3.254368, -0.306353},{2.455167, -0.148733, -0.025376, -0.176204, -0.666140, 0.039917, -0.264007, -1.443669, 0.130475, -0.468130, 0.012620, -0.573748, -0.469931, 1.111340, 0.355769, -1.127670, 0.602355, -0.231060, 0.701446, -1.325239, -0.833566, 1.080484, 0.812371, 0.523983, -0.688361, 0.686704, 0.790793, 1.971862, 0.269130, -0.872812, -2.300692, 1.275366, 0.749430, -0.783032, -1.017969, 0.356181, -0.797206, -0.162411, -1.408716, 0.918451, 0.861023, 0.070674, 0.036304, 0.098545, 0.783378, -0.383186, 0.917328, -0.828809, 2.446447, -0.522090, 0.065160, 1.219107, -1.359254, -1.274415, -0.181690, -0.507531, 2.015517, -2.025285, 0.337680, -0.874442, 0.102181, -0.667478, -1.153105, -0.533057, -0.991014, -1.576869, 0.024209, -0.174893, 0.012447, 0.655737, 1.062017, 0.133379, 0.145202, 0.454249, 0.863050, 0.252297, -0.035988, 2.213628},{0.106117, 1.833644, -0.052489, -0.556856, -0.052832, -2.571640, 1.577419, 1.705223, 0.269593, -0.766249, -0.833048, -0.132574, 0.440774, 1.458893, -0.632177, -2.468513, -1.902638, 0.872360, -0.003809, 1.712571, -1.029731, -0.541930, 1.026269, -0.346565, 0.135349, 0.319605, -0.706640, 1.169969, 0.296422, 1.257257, 0.051038, 0.403120, -1.003790, -0.812778, 1.145237, -1.440361, 0.560935, 0.122191, 1.244892, 0.353006, -1.133722, 0.540264, -1.186334, 1.650621, 0.873411, -0.023587, 0.184046, -0.363590, 1.239342, -0.291088, 0.682523, 1.289472, -0.246586, 0.501697, -0.215752, -0.334110, -0.091706, -1.385864, -0.521151, 3.013249, 0.894918, -0.287966, 0.439227, -0.667229, 0.159960, 0.373955, -0.658571, 2.526382, -1.044709, 0.361376, 0.764845, -0.460904, 0.446851, 0.777893, -0.808750, -1.459542, -0.952783, 0.047854},{-0.920895, 0.356302, 0.416326, -2.307744, 0.583347, -1.294274, 0.809645, 0.994025, -1.540586, -0.116091, 1.154754, -1.243117, -1.136806, 0.786784, -1.036479, 1.765810, -0.491322, 0.289348, 2.327372, 0.476871, 0.073592, 0.324871, 1.283686, -1.072284, 0.096462, 1.044766, -1.892378, -1.363847, 2.798330, 0.260882, 0.392118, 0.056769, -1.620830, -0.312782, -0.654268, 0.118961, 1.084483, -1.187864, -0.524715, 0.421093, -0.111901, -0.644572, 0.154581, 1.036961, 0.318873, 0.507065, 1.670564, 0.359104, 0.318509, -0.516015, -0.671021, 0.232170, -1.558529, 0.391797, 0.524209, -0.245757, -0.630713, 0.083507, 1.220604, -0.453266, -0.550306, 0.049398, -1.407726, -1.244692, 2.070432, -0.826213, -0.547575, -1.876227, -0.633336, 1.636211, -0.521094, -0.034997, 0.535552, 0.371584, -0.534954, -0.110909, -0.116918, 1.462069},{1.986563, -2.704162, -1.231393, 0.442097, 1.714878, 0.792953, 0.989819, 0.008339, 1.848201, -0.061361, -0.309218, -1.891667, -0.085984, 0.585077, -0.645425, -0.884392, 0.035848, -1.992437, -0.144957, -0.209783, -0.951705, 1.883142, -0.398429, 1.335216, 1.364497, 0.870034, -1.621582, -0.266958, -1.343432, -0.412504, -0.031723, -0.715023, 0.025333, -0.539891, -0.484395, 0.665202, 0.008741, 0.876919, -0.188611, 0.120442, 1.849946, -0.958919, 0.235190, -0.791059, 0.383826, 1.540097, 2.378468, 0.674409, 0.725505, 0.440610, 1.152556, 0.343960, 0.530022, 0.078023, -0.070391, -0.305945, -2.743298, 0.169892, -0.480773, -0.781201, -0.615270, -0.758982, 0.012558, -0.153335, 0.339108, 0.263843, -0.456785, 2.265964, 0.081323, 1.157070, 0.996720, 1.257745, 0.492202, -0.403878, -0.688860, 1.332531, -0.508518, -0.538566},{0.133174, -1.027318, -1.032171, -0.295804, -0.821271, 0.345748, 1.232788, -0.159300, 0.778666, -0.274656, -0.306315, -0.231134, 0.325724, 1.349379, 1.321038, -0.523374, -0.907542, 0.377354, 0.875675, 0.244673, 1.881501, 1.465958, -0.061373, 0.844341, -1.329984, -0.107813, -1.052499, 0.145795, -1.143033, 0.444993, 0.404289, -1.334420, 1.384346, -1.037590, 0.541171, 0.113973, -0.547821, 0.701539, 1.763777, -0.346825, -0.988855, -0.160479, 0.393316, -0.097930, 0.793450, -0.908972, 1.319723, -0.017633, -0.548507, -1.065829, 1.981411, 1.127585, -0.118015, -0.663811, -0.628338, -0.658003, -0.049250, -0.824232, -0.041853, -0.326386, 0.989799, -0.821520, 0.902946, 0.621917, -1.501266, 0.159851, 1.080057, 0.210122, -1.793042, -2.144892, -2.519333, 0.425861, -0.121412, -0.237205, -0.735175, 0.117299, 0.606739, -0.080272},{-0.137461, -1.456802, 1.247939, -1.061621, 0.944977, -1.728350, 0.294290, -2.634593, 0.621233, 0.140710, -0.369691, -1.250277, 1.479753, -0.732840, -0.705224, -0.800349, -1.415774, 1.556588, 1.136779, -0.346244, 0.897110, -1.568259, -1.514926, 0.079933, 1.316896, 1.583131, -0.990105, 0.790525, 1.981253, 0.457399, -0.314491, -0.897236, 1.688210, 0.389367, 0.093703, -1.251684, -0.797795, -0.490318, -1.455546, 0.230197, 1.289196, 0.100733, -1.365425, 0.687898, -0.229961, -0.903691, 0.372552, -0.886819, 0.383764, -0.307721, 0.947190, 2.059146, -0.298710, -0.111559, -1.232470, -0.649338, 1.055029, 1.139362, -1.304701, 0.933063, -0.157457, -1.540349, -0.310129, 1.191319, 0.013681, 0.689640, 0.131461, -0.857156, 1.271256, 0.767151, 3.117429, 2.080648, -0.441656, 2.119308, 0.253734, 0.649921, -0.303804, -0.318056},{-1.076012, -0.589566, -1.239494, 0.783343, -0.360632, 0.736162, 2.046013, -0.919698, 1.048262, 1.086275, 0.432578, 0.374403, 0.418361, -0.479700, -0.132524, 0.135200, -0.007755, 0.278360, -0.171937, -0.024110, -0.533456, 0.868374, 0.082768, 0.584043, 0.507607, 0.725303, 0.499177, 0.311502, -0.327144, -1.392250, -1.576417, -0.093082, -0.560825, -0.882896, -2.472230, 0.804959, -2.501578, -0.756899, -0.829085, 0.663824, 0.404010, 0.020317, -0.217636, 0.223445, 0.232193, 1.725238, 0.899565, 0.660922, 0.801869, -0.443608, 0.529869, 0.705620, -0.175464, -0.301544, -0.245682, -0.862349, -0.443317, -0.685362, -1.403208, -0.904676, 0.715723, 0.413053, -1.896344, -1.790439, -0.625276, -0.102003, -2.103044, 1.229279, -0.128447, -0.403628, 0.441965, -0.304830, 0.273601, 0.838946, 1.302516, -1.077007, -1.537552, 0.430289},{-1.968202, -0.180470, 0.629173, 0.925636, -0.331867, -0.476242, -0.154739, 1.554900, -0.905859, -0.940097, -1.056603, -0.768786, -0.499467, 0.801630, 0.143077, 0.367715, -1.756420, -0.761879, 0.939195, 0.397513, 0.365859, 0.539448, -0.099147, -0.546036, 1.500344, -0.729060, 0.087791, -0.754973, -0.742607, 1.554158, -0.525771, -0.058504, -0.884787, -0.174285, -0.267917, 0.093103, 0.501531, -0.664739, 0.772035, 0.766936, 1.225191, 1.592729, -3.145757, 0.532396, -0.540767, 0.580408, -2.095836, 0.656667, 0.141843, 0.273438, 0.603797, -0.543809, 0.355789, -0.497540, 1.107417, -1.729231, -0.984331, -1.531059, 0.277245, 0.142893, -1.057845, 2.208259, -0.484994, 0.145166, 0.598811, 1.126043, 0.158405, -0.977215, 0.272892, -0.236124, -2.778555, -0.606406, -0.404792, 0.313502, 1.035494, 1.154492, 0.975565, 0.169993},{-1.771664, 0.517437, -1.579537, -1.254286, 0.900467, 0.028376, 0.827492, -1.585660, -0.149392, 0.582527, 1.615341, -0.383392, 0.672406, 0.229117, 0.094268, -1.583365, -0.634725, 0.671881, -0.701455, 1.318850, 0.940430, -0.204707, -0.100172, 1.145331, -0.439819, 1.830847, 1.191923, -0.421275, -0.261513, -0.353595, -0.449498, 0.122975, -0.063303, 0.897519, 1.216249, -1.244688, 0.159144, -0.943737, 0.527183, -0.748593, 0.552892, -0.111088, -1.451082, -1.031649, 0.207586, 1.443649, -1.518042, 0.720841, 1.065445, -0.554055, -1.431416, -1.193323, 0.797162, 0.105777, -2.372404, -1.510516, 1.715277, -0.755865, -0.229293, -0.593184, -3.269374, 0.169724, -1.677851, 0.215005, -2.100843, -0.840377, -0.401843, 0.402931, -1.260047, 1.699732, -1.397405, -0.536641, -0.616728, -1.713282, -1.240816, -0.325946, -0.747898, -0.130618},{-0.071652, 0.298056, 0.572314, 0.933863, 0.145035, 0.062427, 0.200816, 0.443577, 1.015791, -0.332124, -0.113822, 1.614424, 0.264849, 0.329796, -1.681364, 1.099573, 0.182846, -0.235416, -0.305848, -0.356872, -0.781862, 0.269408, 0.793760, 0.055653, -0.080186, 0.871444, 1.733178, 0.211514, -0.384203, 0.084265, 0.377570, -0.730616, 0.400037, -1.818761, 0.467092, -0.796735, 0.388796, -0.679520, -1.266757, 0.982786, -0.621070, -0.865573, 0.474384, -0.557001, 0.976164, -0.181382, -0.261857, 0.727362, 1.851434, 0.464314, -1.548560, 0.195932, -0.004756, -0.336316, -0.269958, 1.890797, 0.087497, -0.149276, 1.687897, -1.072769, 0.526815, -0.419008, -1.108464, -0.762841, 0.509483, 2.414680, 0.835193, -0.505686, -0.565280, -0.438220, -0.247928, 1.377482, 0.456818, -0.447722, 1.681779, 2.215853, -0.313311, -0.150895},{-0.573114, 0.187491, -0.711262, 0.010310, 0.304169, -0.936608, 0.518041, -0.723910, -0.079248, -0.266204, -0.811583, -1.105957, -1.591270, -1.213678, -1.084610, 1.888000, -0.173043, 0.740764, 0.381561, -0.621113, -1.496223, -0.332267, -0.442837, -0.105494, 1.021753, 1.156266, 0.524933, 0.895982, -0.613578, -0.950585, -0.968121, -1.569136, 0.993020, -0.059038, 0.345270, -0.494597, -2.084340, 0.217804, -0.645369, -0.832284, -0.825139, 0.516566, 0.079877, 0.063376, 0.013981, 1.417488, -0.568802, -0.554851, 0.278205, 0.575549, 0.527068, 0.847482, -1.799102, -1.274409, -0.444331, -0.609574, 0.201281, -1.085635, 0.406675, 1.794533, -2.051546, 1.732220, -2.057092, 0.433826, 0.629059, 2.523771, 1.322261, 1.442085, 1.531169, -0.702991, -0.796277, 0.236822, 1.773774, 0.235122, 0.531371, 0.103429, 1.010350, -0.032669},{0.827456, -0.525269, 0.674956, 0.709975, -0.815038, 0.601838, 0.468581, -0.379551, -0.399605, -1.308124, 0.929007, 1.427908, -0.384714, -2.346768, -2.873364, -0.125765, -1.154095, -0.257032, 2.065692, -0.617156, -1.215750, 1.560218, -0.044990, -0.454553, -1.797016, 0.718261, 0.438325, 1.009602, 1.061671, -0.381875, 1.707572, -2.101868, -0.398329, -0.841653, 1.494675, -0.675192, -0.560001, 0.304063, -0.367846, -1.106258, -0.567029, 0.186096, -0.214342, -0.989620, -1.541185, -1.072645, 0.451862, -0.245217, -0.770840, -0.274099, 1.030898, 1.876929, -0.435720, -0.355741, 0.125814, -0.810784, 0.356234, 0.237597, -0.357729, -0.916244, 0.570078, 0.058268, -0.490640, 0.623336, -0.514762, 0.334310, 0.068414, -0.798787, 2.731401, -0.047112, 0.688589, 0.779422, -0.898956, -0.237054, -1.223198, 0.242880, 0.127154, 0.441642},{0.741506, -0.699278, -0.462421, 0.571113, 0.183860, -1.149973, 0.317095, 2.169098, 0.159627, 1.447107, -0.963176, -1.423452, 0.532720, -2.646729, 0.255230, 0.081175, -1.357750, -1.183486, 0.592244, -0.150169, 1.589529, 0.430942, -0.062877, 0.312037, -0.259255, -0.394503, -0.570559, -0.237587, -1.592668, -0.720183, 0.016805, 1.985360, -0.678075, 0.015875, 0.886990, 2.548701, 0.113663, -0.358443, -0.414968, -0.104133, 0.161984, 0.559167, 0.225664, 0.065342, 1.541301, -0.562959, 0.668184, 0.962017, -0.758735, 0.049747, 0.667905, 0.273928, -0.326089, 1.072239, 1.318971, -0.474526, -0.382004, -0.883219, 0.093983, 1.003915, -0.771951, 0.687605, 0.221684, 0.519092, 1.396845, 0.588932, 1.459136, -0.410400, 0.333160, -0.852579, -0.535757, 1.410434, -1.382527, 0.203980, -2.406198, -0.480026, -0.634383, 1.560021},{0.125504, 0.283842, -0.671459, 0.141688, -0.674194, 0.633541, -0.651843, -0.141002, 0.208390, -0.435452, -1.055102, 0.153317, 0.606804, -0.558780, -1.031147, 2.224665, 1.035063, -0.035748, -1.069876, 0.813544, -0.371834, -0.604132, 1.155589, -0.518625, -0.847430, 0.567694, -0.265449, 0.966497, 2.045081, 1.707604, -0.562551, 0.015715, -1.348505, -0.044139, -2.364199, 0.026444, -1.555705, -2.333568, -0.127265, -0.049450, -0.334935, 0.097391, 0.901600, -1.358634, 1.279608, -0.070952, -1.338781, -0.221239, -0.606086, 0.408436, 0.190213, 0.504491, -0.014427, -0.412917, 1.818943, 1.097876, -1.945404, -2.262745, -0.672765, -0.871613, -1.451986, -1.134469, 0.658827, -1.066286, -0.674507, -0.710008, 0.513458, -0.348226, -1.054553, 1.288126, 0.346180, 0.676410, 0.665611, 0.667679, 0.183142, -0.343915, -0.293395, -0.892026},{1.282492, -0.355959, 0.763173, -0.964260, -0.407172, -0.507013, 1.180771, 0.226893, 0.807654, 0.510175, 1.107141, 0.182625, -1.632923, 0.626432, 0.110936, -0.406041, -1.344251, -1.162075, 0.954889, -0.113260, -0.291045, -0.045791, 0.080436, -0.578580, -0.868292, -1.930144, 0.158461, 1.734231, -1.410232, 0.143151, -0.483962, 1.621054, 0.191224, -1.271956, -1.515048, -0.601906, -0.291784, -1.393767, 1.785523, -0.194277, 1.000815, 1.128065, -0.189420, -0.236296, 0.088692, -0.865930, -0.825046, -0.364142, -0.712922, 2.309920, 0.050814, 2.751845, -0.710970, 0.245369, 0.307029, 1.204468, 1.524023, -0.564725, -0.116638, 0.401987, -0.106652, -0.579506, 1.237081, -0.706807, -1.447903, 0.504082, -0.860128, 1.623106, -1.169166, 1.663041, -0.701454, 0.267725, -1.541870, -0.289750, 0.363513, 0.734830, 0.966819, 1.010496},{1.448840, 1.208789, -0.629952, 0.623956, -0.624570, -0.125545, 1.434466, -1.258806, 0.928961, -0.307920, 0.468601, 0.142922, 0.015211, 1.884232, 0.560296, -0.858106, -0.408514, -1.207332, -0.575117, -1.000032, 0.796977, -1.128199, 1.363093, 0.953475, -1.215680, 1.790362, -0.322652, -0.628087, 1.760667, -2.088660, -0.540200, 0.275593, -1.329268, -0.738382, 0.647553, -0.511993, -0.685633, 0.252519, 2.062809, -1.687116, -1.182757, 1.035472, -1.600818, 0.052230, 0.891946, 0.299764, -1.286336, 1.783643, -0.386877, -0.215978, -0.715176, 0.086170, -0.370601, -0.686966, -0.977306, -1.136771, -1.677801, -0.860410, -0.769075, -0.568266, 0.997735, -1.144002, -0.685577, -1.047850, 0.586130, -0.043588, -1.610117, -0.651115, -0.631189, 0.501024, -0.368930, -0.136675, -1.375435, 0.452148, 0.775264, 0.100176, -0.510037, -0.709855},{0.508233, 0.527059, -0.235557, -2.449000, 2.053073, -1.567383, 0.796391, 0.451659, 0.603324, -0.833506, 0.801885, -0.951887, 1.153050, -0.210641, 1.385573, -0.812972, -1.523552, -0.749331, 0.544274, 0.840490, -0.127001, 0.925796, 0.903830, -0.285127, 0.745578, 1.369046, -0.931667, 0.134745, 0.097687, -0.303030, -0.290588, 0.720197, 0.962012, 0.293803, -0.254014, 1.212169, -0.830297, 1.428971, 0.547190, 0.034759, 1.111109, 0.757161, 0.323764, -1.225795, 0.396561, 1.984400, -0.540483, 1.860538, -1.474203, 0.171657, -0.850236, -1.535034, -0.260703, -0.120660, 0.302508, -0.709291, 1.081885, 0.280928, -0.076529, -0.087351, -0.454818, -0.734583, 0.003241, -0.916887, 1.144878, -0.397251, 1.094917, -0.503577, -1.171354, 1.647927, 0.020165, -0.694190, 1.487172, 0.045827, -0.119888, 0.599595, 1.293642, -0.653414},{-0.443061, -2.857682, 0.959759, -0.001366, -1.597197, -0.175606, -0.018807, 1.499990, 0.518994, -1.050607, 0.793450, -1.992139, 0.447198, 0.264566, 0.834098, -1.129994, -1.485345, -0.066950, 0.060236, -0.771794, 1.588231, 0.071164, 2.081387, -1.079039, 0.483962, 0.102449, -3.124783, 3.164270, -0.185436, 0.285576, -1.645414, 1.272946, 0.666302, 0.693137, -0.112098, -0.917630, -1.662859, -1.743810, 0.866257, -0.185896, 1.845501, -1.474042, -0.602132, -1.502787, 0.131355, -0.290500, -0.611656, -1.216854, -0.969656, 0.074271, -1.938444, 0.625553, 0.629042, 1.980694, -1.266199, -1.443553, -0.035848, 0.018776, 1.560385, 2.088941, -0.341534, -0.791674, 0.367993, -0.307912, 0.112722, 1.256872, -1.072780, 2.315053, 0.597275, -1.165168, 0.135896, -1.765818, 1.044693, 1.461739, 0.009865, 1.540619, 0.479852, -0.633450},{1.771785, -0.323833, 0.358073, 1.114558, -1.689252, 0.215385, 0.608254, -0.339778, -0.564445, 0.200910, 2.182609, 0.217227, 1.927975, 0.721714, -1.043412, -1.178440, 1.070332, 1.056724, 0.684795, -0.658151, -0.561713, -0.534672, -0.018313, -1.093233, -0.213340, 0.589899, -0.934334, 1.024699, -1.354760, 0.379553, -0.369062, 0.290601, -0.722113, 1.303584, 0.175063, 0.020801, 1.200598, -1.380156, 1.510935, 0.578746, -0.939802, 0.596136, -2.413746, 0.260951, 1.658114, 0.852090, -0.329313, 1.087225, -0.305978, 2.253638, 1.029434, -0.384101, -1.191587, -1.945289, -0.497852, -0.569465, -2.231724, 0.175307, 0.248941, -0.754888, 0.969294, -1.449908, 0.437928, 0.143283, 2.382187, 0.625424, 0.559444, -0.142946, 1.828229, -1.240646, 0.221058, 0.858876, 1.499159, -1.510867, 1.783658, -0.091822, -0.637877, -0.250874},{-0.608641, 0.168100, 0.940526, 0.626169, -1.452310, 1.851951, -0.874162, 0.557636, 1.276375, 1.334042, -1.339591, 1.081075, 0.556107, -1.076309, 0.664769, -0.524531, -2.420942, 0.355229, -0.574213, 0.416494, 0.162831, 0.057430, 1.427138, 0.038715, -0.608515, 0.516256, 0.172850, -0.913517, -0.600180, 1.213230, -0.628680, -0.421415, 2.253724, -0.592392, 0.240683, -1.132347, 1.958704, -1.801778, -0.085909, -0.792874, 1.273238, 1.228654, -0.285016, -2.249525, -0.549196, 0.141855, 0.220228, -0.602828, 1.119110, 0.226091, -0.012075, 1.495736, -1.281365, -1.416609, 0.389513, -0.268108, 0.702078, -1.372295, -0.339542, 1.179864, 0.290059, 0.066721, -0.122026, 0.291316, 1.065468, 0.547116, -1.108414, 0.827685, 1.879549, 0.122212, 0.220221, 1.433238, 0.771384, -0.249782, 1.040250, -0.189955, -0.619037, 1.231681},{0.930303, 1.298824, 1.059568, 0.305306, 2.535825, -0.154911, -1.758648, 0.326528, -1.297869, -0.791948, -2.513791, 0.353729, 1.137912, 1.415857, -0.560127, -0.481439, 0.148968, 1.235975, -0.453978, 1.371530, -0.744923, 1.067545, 0.233738, 0.003601, 0.463222, -1.554506, 0.256818, 1.270175, 0.683971, 2.043440, 0.020386, -0.477357, -1.172333, 0.459752, 0.147000, 0.244662, -0.319689, 1.625135, -0.991754, 1.244551, 0.261996, -0.745819, 1.826712, -0.122110, -0.993046, -0.458816, 0.427094, 1.186620, 0.728672, -0.280871, -0.115207, -1.138948, -0.126492, 0.216375, -0.563993, -0.422380, 1.409726, 0.991092, -0.497650, 0.381948, 0.583198, -0.671919, 0.000323, -1.145682, -0.710317, 1.567274, -1.549843, -0.946244, 2.355954, 1.017993, -0.496186, -1.155587, -1.934166, -0.106046, -0.270547, -1.115662, -0.442442, -0.983342},{-0.621471, 0.513005, -1.275600, -0.982183, 1.006455, -0.908902, 0.820296, 1.362085, 1.414153, -0.161677, 0.996603, -1.564500, -1.635391, 1.842217, 0.847149, 0.501428, 0.654869, -0.714577, 0.792111, -0.381905, -0.369963, -0.039649, 1.148298, 0.053431, -1.711409, -0.528226, -1.450313, 1.432047, -0.307187, -0.511180, -0.620890, -0.382011, -0.830410, -0.288577, -0.853549, -0.550061, 1.933186, -0.311448, -0.168652, 0.333508, -1.069734, 0.720610, -0.403527, 0.189612, -0.385362, 0.340163, 0.212201, 0.916175, 0.801516, -0.165640, 0.031431, 0.373824, 1.057318, 0.825186, -0.596486, -1.065764, -1.408857, 1.167845, 0.448743, -1.333549, -3.088275, 1.146430, -0.921510, 1.312204, 0.836063, -0.281190, 0.936501, 0.388332, -3.058845, 0.159049, 0.737465, -0.689895, 0.749220, -0.209328, -2.284078, -0.073068, -1.150718, 0.023441},{0.116813, -0.282902, 0.857869, 0.465103, 1.651673, -1.455696, -0.157345, -1.299676, -1.258912, 0.372593, -0.379198, -0.212394, 0.564010, 1.159093, 1.535162, 0.779533, -0.821965, 0.580099, -0.113540, -0.068362, 0.247339, 0.062788, -1.467285, -0.060695, 0.588388, -1.125944, -1.755470, 0.641168, -1.456091, 1.346581, 0.622884, -0.349547, 0.932610, 0.845458, -0.756856, -0.519864, -2.161960, 0.782773, 0.134516, 1.032859, -0.634672, -0.600560, 1.196701, -1.417278, -0.487467, 2.071630, 0.491591, 2.118452, -0.745612, 0.555333, 1.245154, -1.039232, -1.270823, 1.021635, -0.430432, 1.307377, 1.278677, 1.361280, 0.256521, -1.071124, 0.368950, -0.527076, -0.284959, -1.299730, -0.289077, 2.070435, -3.331199, 0.605389, 0.302624, 0.254198, -0.319209, 0.474652, -0.587144, 1.428908, -0.434328, 1.245437, -0.601269, -0.816371},{0.295995, 0.799696, -0.379039, -1.553222, 1.335878, -0.420661, -0.137473, -0.849531, -1.569005, -0.514165, -0.446481, -1.212104, 0.103684, -1.428134, 0.379840, -0.917710, -0.254424, 1.520264, 0.532832, 0.267873, 0.228248, 1.485157, 0.777171, -0.161393, 0.073939, -2.029435, -0.957026, 0.861144, 0.553879, -0.326998, -0.766024, 0.173282, -0.133426, 0.249773, 1.094714, 0.565437, -1.384435, 0.532806, -0.499015, 2.446635, 0.187783, 1.121813, 1.556534, -0.347153, -0.857585, -1.730922, 0.664440, -0.030709, -1.885906, -1.212932, -0.834352, -0.643612, 0.441813, 1.011665, -1.290881, -0.864207, 1.341328, 0.413777, -0.270400, -0.804086, 0.082228, 1.789022, -0.549807, 0.075153, -0.209782, -1.635264, -0.030546, 0.616180, 0.306612, -0.601536, 0.141075, 0.526938, -0.282281, -0.366918, -0.202910, -0.634102, -1.173802, 0.889219},{-0.808136, 0.528034, -0.231328, -0.074498, 1.098603, -0.504871, 1.199072, -0.272910, 0.058429, 0.512508, -0.934849, 0.902047, 1.063772, 0.770070, -1.031499, -1.445801, -0.478738, -1.492776, 0.200953, 0.750224, 0.020507, -0.516928, 0.082318, -0.281512, 0.820566, 0.451158, -0.994209, -0.639797, -0.132929, 1.194642, -0.521017, 2.061626, 1.368386, -0.164909, 0.307006, 1.185217, 0.893766, -1.386106, 0.158298, 1.012004, 0.585842, 1.016710, -0.250797, 1.605625, 0.157217, 1.783108, -0.680760, 0.081525, -0.470522, 2.119165, -0.018970, -0.146232, 0.084312, 1.814122, -0.736637, -0.516587, -1.322356, -1.396603, -1.083083, 1.360519, 0.411706, 0.553797, -1.070287, -0.540566, -1.533139, 0.115786, -1.396972, -0.730435, 0.649688, -1.308401, -0.507571, 0.986138, 0.007747, 0.455266, 1.405271, 2.875540, 0.848451, -0.663746},{0.268519, -1.652143, 0.518535, -0.452196, 2.021174, 2.823288, -1.646150, -1.137066, -0.730630, 0.578326, 0.004575, -0.240312, -0.554077, 1.209208, 0.124900, 0.402143, -1.240446, -1.163121, -1.044356, 1.318010, -0.793460, -0.289156, -2.038024, -2.344244, 0.564448, 1.113271, 1.203627, 1.286614, 0.216882, -0.867932, 0.144460, 1.040273, 1.113123, -0.774799, 1.000670, 0.385127, 0.810505, -1.574793, 0.044770, -1.279054, 0.630919, -0.978593, 0.672114, 0.112184, -0.899262, -1.079551, 1.271637, -0.540066, 0.219299, -0.511186, 0.870956, 0.458752, -0.426017, -1.423823, 0.671364, -1.046914, 0.125693, 0.447495, -0.331912, 1.038158, -1.034709, 0.340034, 0.411088, 0.124297, 1.588640, -1.411882, 0.111056, 0.606888, 0.839273, -0.095097, 0.856901, 1.602994, -0.223973, 0.587248, -0.068681, 0.306012, 0.159322, -0.862312},{0.925696, -1.580769, 0.121110, 0.222050, 0.153245, -1.202654, -0.963123, 2.064395, -0.202268, -2.209253, 1.138564, -0.478907, 0.309886, -0.148252, 1.816283, 0.410962, -0.690367, -0.004277, -0.250412, 1.174663, 0.002899, -0.005395, 0.656167, -0.879626, 0.748784, 0.644896, 1.063881, 0.417902, 1.262777, -0.057067, 0.839632, -0.827524, -0.277789, 0.275713, -0.705437, 1.321590, -0.456891, 0.500477, 0.321154, -1.031682, 0.657254, 1.194796, -0.781201, -0.675206, -0.595461, 2.082255, -1.482826, 2.042918, 0.606992, -1.098899, 0.044336, 0.340139, 1.483972, 0.785917, -0.691233, 0.599230, 0.134874, -1.812085, 0.234264, -0.975708, 0.413250, 0.395977, -0.649688, -1.540413, 0.017591, 0.723988, -1.828070, -0.215649, 1.894239, -0.177094, -0.130021, -0.388044, 1.273436, -0.992672, 0.473518, -2.296338, -1.059953, -0.990151},{0.921477, 0.900401, 0.066870, -0.102222, -0.120043, 0.676465, 1.135218, -0.090563, -1.669530, 0.163470, 0.337447, -0.091081, -0.108666, -2.539302, 0.455191, 0.518944, -0.460828, 0.927601, -2.281937, -2.118797, -0.723470, -0.253232, -0.519255, 1.435992, -1.579499, 0.172762, 0.586496, 0.645727, -0.704153, 0.579213, 1.554068, 0.385911, -1.327926, -2.280598, 1.153865, -0.526549, -0.337161, 0.786707, -1.666863, -1.220258, -0.901218, -1.158978, -1.250784, -0.433864, 1.145942, -0.460056, 1.837633, 0.004235, 0.763030, -1.200897, -0.264001, 0.214956, 0.264630, -0.581882, -0.259157, -1.602061, -0.260056, -0.228757, 0.612291, -1.333133, 1.899176, -1.082610, 0.615965, -0.207228, -0.181797, 1.663743, -0.881224, 0.057440, 0.451984, 0.490173, -0.600684, 0.825664, 0.682313, 0.666643, 0.209755, 0.984744, -1.222674, -0.781017},{-0.060275, 0.911725, 0.265680, -0.824895, 0.834919, 0.306421, 1.714265, -0.120114, -0.447006, 1.760482, 0.529831, 0.214922, -0.867931, 0.695510, 1.344619, -1.057893, 0.841245, -1.079453, 0.340682, 0.692361, 0.356874, -0.394659, 0.405173, -3.056485, -0.013797, -1.027670, -0.383117, -0.199060, 1.190606, 0.261778, 1.004534, -0.243978, 2.422873, 2.072601, -0.609096, 0.620154, -0.838867, 0.123041, 0.307208, -0.620287, -0.350350, 0.634605, 1.508793, 0.454788, 0.816607, 0.633976, 0.482442, -0.574137, -0.748851, 0.998376, 0.166284, 0.708011, 0.087605, 0.747787, -0.367830, 1.088531, -0.780249, 0.399756, -0.837752, -0.162450, 0.716443, 0.260323, 1.387828, -0.473366, -1.027007, -1.185609, 2.090104, 0.524531, -0.088331, 0.215552, 0.128994, 0.263349, -0.168783, 0.568722, 0.754051, 0.417314, 0.577795, -0.417088},{1.064704, 1.267945, 1.227945, 0.939153, 0.247351, -1.355427, -0.440838, -0.540530, 0.505325, -0.494446, -0.142465, 1.887447, 0.536559, 0.197823, 1.523817, -0.067055, 1.277970, -0.833605, 1.793652, 0.345150, -1.314712, 0.501988, 2.146557, -0.965993, 0.053288, 0.638256, -0.898380, -0.892120, 1.003024, -1.448365, 0.974967, -0.514815, 1.192501, 0.892522, -0.055050, 0.534140, -1.251857, 1.151628, 0.185711, -2.118615, -0.467747, 0.386209, 2.099702, 0.140220, -0.768802, 0.508346, 0.903936, 0.356580, 0.705048, -1.274275, -0.323045, -1.007415, -0.453382, 0.655727, 0.089186, -0.049259, 0.746722, 0.527326, -0.148523, 0.258081, -1.705551, -0.778528, 1.263006, -0.277387, 0.581659, 1.326007, 2.441356, -0.568161, 1.545692, -1.205000, 0.550621, 0.146622, -0.733756, 0.148452, 1.860604, 0.532256, 0.884283, -0.775635},{-1.194800, -0.570028, -0.954902, 1.134252, -0.244477, 2.295781, -0.080164, 0.932310, 1.062063, 0.432342, 1.433093, -0.409212, 1.748997, -0.243188, 1.842272, 0.390160, -0.266530, 0.725538, -0.669585, 0.485303, -1.189664, 0.030978, 1.679144, 0.667269, 0.635415, -0.690510, -1.124121, 0.991194, -2.698233, -1.068848, 0.016705, 0.476047, 0.479127, -1.178256, -0.242912, -0.340764, -0.685188, -0.200798, 0.375558, 0.945311, 1.902380, -1.065695, 1.619699, 0.946404, 0.542139, 0.105265, -0.786896, -1.742692, -1.528296, -0.590139, -1.276064, 1.885522, -0.017534, -1.915116, 0.543298, 0.271390, 0.201357, -1.260551, 1.309054, -0.238979, 2.108892, 0.735644, 0.241658, 2.867180, -2.067394, 0.480300, -2.009500, -0.656791, 1.084882, -1.135580, 1.262692, 0.434044, -1.127465, 0.769640, -0.927028, -0.417373, -0.690450, 1.042467},{1.099548, 0.123528, -0.386866, 0.387007, 0.690636, 0.155804, 0.285383, -0.493504, 1.148732, 0.320387, 1.420869, 0.014068, 0.890913, -1.056819, 0.437483, -1.359444, 1.588364, 0.631175, 0.159084, 1.025773, 1.322751, 0.377842, 0.438362, 0.290428, 0.482028, -0.099990, 0.551326, -0.468778, 0.710534, 0.125864, -0.063985, 1.041451, -0.360651, -0.760499, 1.301336, 1.515293, 1.753017, 0.734094, 1.992390, -0.193863, -0.399503, 0.207155, -0.458122, 1.022015, 0.452083, 1.099470, -0.126453, 0.513443, 0.162394, -0.323022, 0.508791, 0.621429, -0.512835, 1.322877, -0.336337, -2.371952, 0.016119, 0.763659, -1.028759, 1.667652, 0.840630, 0.850614, 0.068631, -1.050468, 0.348941, 0.385145, -0.695446, 0.019024, -0.354362, 0.252386, -1.045564, -0.347904, -0.126006, 0.456343, 0.137010, 0.471421, -1.998261, -0.152830},{-0.182069, -0.867966, -0.368394, -1.829748, -0.603628, 0.667957, -0.971471, -2.231528, 2.526896, 0.542663, -0.752210, -0.121732, -0.229353, -0.325294, -0.186596, 0.158171, -1.141509, 0.055434, 1.348602, 0.766489, 0.318273, -2.369642, -0.389412, 0.093423, -0.360832, -0.065211, -0.561768, 1.560627, 0.026551, -1.415313, -0.658953, 0.239619, -0.215301, 0.015870, -0.507614, 1.380288, 0.242444, -0.103979, -0.130791, 0.794002, -1.567083, 1.005660, -0.737158, -0.325363, -1.054337, 1.007867, -0.010813, 1.783986, 1.628245, 0.089229, 0.444277, -0.629276, 0.189048, -0.419101, 0.114460, 0.423205, 0.629636, 0.287154, 2.275778, 0.753135, 0.261369, 1.568946, 2.094142, -0.949235, 1.786213, 0.153653, 0.160607, -1.035830, -0.422286, -1.045982, -0.210524, -1.550853, 2.182231, 1.299185, 1.874974, 0.039602, -0.370099, 0.397446},{1.374925, -1.030065, 0.024061, -0.584350, -0.874560, 0.049317, 0.513160, 2.257613, 1.858493, 1.515037, -0.511141, -0.116721, 0.032616, -0.236389, 0.380636, -0.984566, -0.974413, -1.691050, -0.031797, -0.982568, -0.971537, 0.244150, -0.933365, 1.189220, 0.094966, -0.210738, -0.365033, -0.567413, -2.113727, -0.764663, 1.369118, 0.017832, -0.251960, -0.607413, -0.026098, -0.052229, -0.728292, 1.995538, 2.081809, -1.235159, 0.058212, 0.363901, 0.347158, -0.336541, 0.224369, -0.270860, 1.590957, -0.391700, 0.150572, -0.760625, 0.102773, -1.349370, 0.433585, -0.637426, 0.728261, 1.718032, 0.098432, 0.170673, 0.331483, -0.811231, 0.036540, 1.328108, -0.801035, 0.391509, 1.670323, 1.302507, 0.081298, -2.008598, -0.280119, 0.046323, 0.740737, -0.866985, -2.059738, -1.014596, -0.372568, -1.280798, 1.364668, 0.958043},{0.237402, -0.818430, 0.348997, -1.098294, -0.061177, 0.449462, -0.563324, 0.800173, -1.307423, -0.041554, 0.431692, 0.175936, 2.422549, -0.973548, -0.799509, -0.109455, -0.288129, 1.396713, -0.050366, -2.121996, -0.371179, -0.602747, 0.108507, 0.445811, -0.916181, -0.115023, 1.447306, -0.335564, 0.512118, -0.439959, -0.180489, -0.228869, -1.122897, 0.001069, -0.299071, 0.889080, -0.935290, -1.603519, 2.484544, 0.324223, -0.473009, 0.790086, 1.257582, 0.572851, -1.697459, -1.503232, 0.113471, 0.950840, 0.298549, -0.992958, -2.624164, -0.226652, 0.223016, 0.272895, 0.286477, -1.114466, 0.312690, -1.402121, -0.595032, 0.760944, -0.443299, 0.426610, 0.793079, 2.016433, 0.115749, 0.216887, 0.284637, 0.007079, -0.081422, -0.937001, -0.360611, -0.118471, -0.746019, -0.373538, -0.878348, -0.122663, 0.333689, 0.071081},{-0.661307, -2.023111, -0.481285, 0.423583, -0.415647, -2.063357, 0.429289, 0.889635, -0.464627, 0.634504, 0.711986, -2.041130, -2.551254, 0.866385, 0.394023, 0.103537, 0.466099, 0.643800, 0.316474, -1.114004, 0.947353, 0.935301, -1.329778, 0.851835, 0.920237, 0.457060, 1.418435, -0.313888, 1.320915, -1.065480, -0.465951, 0.884979, 0.657102, -0.663662, 0.887441, -0.201363, 1.208161, -0.869441, -1.441677, 0.240032, -0.768296, -1.041196, 0.132280, -0.971176, 0.662149, 0.345418, -0.376661, 0.329526, 1.451246, -1.660823, -0.430534, -0.298945, -0.839820, 0.495659, 1.107962, -0.178966, 0.275459, 1.711969, 0.585248, -1.476440, -0.122698, 0.307471, 0.478339, -0.615536, -1.708310, 0.397708, -1.694798, 0.638352, 0.499521, 0.282046, 0.519023, -0.620436, -0.872290, -0.852696, 0.914788, -0.430970, -0.616891, 0.014404},{0.105532, 0.488627, -0.824729, -1.218096, -0.219203, -0.401779, 0.874236, 0.226533, 0.732042, 1.474906, 0.034547, 0.752980, -1.020954, 0.294943, 0.932084, 2.609939, 0.573841, -0.954919, -1.959019, 1.796157, -2.409494, 0.681210, -0.733668, -0.184006, 1.034615, -0.467959, -0.673349, 0.332724, 2.194935, -1.815171, 0.638656, -0.235577, -0.402407, 0.164203, -0.368298, 0.409316, 1.796778, -0.897296, 0.281582, -0.550113, -0.438196, -1.210133, 0.100782, -0.145248, -0.682981, -0.626073, 0.147556, 0.850400, 0.899669, 0.270254, 0.360896, 1.518921, -0.376861, 0.627082, -3.247909, 2.876701, -0.264195, -0.920037, 0.217532, -0.313950, 0.178848, -0.849470, -0.685776, -0.842550, -0.704801, 1.041571, -1.497976, -0.394299, 1.627141, -0.198659, 0.855597, -0.542336, -0.792837, -1.585041, -0.534176, 0.333222, -0.225383, -0.412962},{-1.769576, -0.805446, 1.238615, 0.955593, -1.062213, 0.458382, -0.150307, -1.170241, -0.250632, 1.258985, -0.159592, 0.191067, -0.437176, 0.090382, -1.510296, -0.272736, 0.439082, 0.201941, -0.527317, 0.951184, -0.061383, 1.230180, -0.887593, 1.299464, 0.581891, -0.383955, 0.043538, -0.093672, -0.936570, 0.907358, -0.600902, 0.144185, 0.198955, -0.011325, -0.326476, -0.437231, -0.106922, -1.148104, 0.952515, 1.523848, -0.063103, -0.380125, 1.734179, -0.787367, 0.998828, -0.640433, -0.235537, -0.413756, -0.723963, 0.792026, -0.865392, 0.385600, 0.880314, -1.196492, -1.299276, -0.177100, -0.223884, 1.478435, -0.641082, 0.774290, 0.270932, 1.370339, -0.108089, -1.459283, 0.102116, -0.430936, -0.224888, -0.163705, 1.389138, 0.045556, -0.789954, 1.172201, -1.799787, -0.781358, -0.127670, -0.738019, 1.857215, -1.187162},{0.274063, -1.959023, 0.365893, -0.852719, 0.870519, -0.289666, -0.656522, -0.221301, -0.277522, 1.356508, 0.778019, 2.099951, -0.583689, -0.914523, 1.170447, 0.142287, -0.912425, 1.314652, 0.816777, -1.338950, 1.209482, 0.558597, -1.549423, 0.656694, 2.126622, 1.305733, -0.330209, -0.721020, -0.853592, 0.441966, -1.118349, 0.389582, 0.456795, 0.949140, -0.149090, 0.059844, -0.563970, -1.038760, -0.641175, -1.309980, -1.265006, 0.999695, 0.231073, -0.692685, 0.756493, -0.755878, -1.869572, -0.695104, -0.797730, 1.614698, 0.393654, 1.355838, -1.638355, -0.032685, 2.396317, -0.756509, -0.927357, 0.749476, -0.893739, 0.055484, 0.893753, 0.622820, 0.121868, 1.373196, 0.408720, 2.260817, -1.088009, 1.147761, 0.847274, 0.840653, -0.534914, 0.060812, 0.965423, -0.934310, -0.758537, -1.156276, -1.724036, -0.977478},{-1.087873, 0.276988, 0.924760, 2.151991, 0.023932, -0.008296, -0.982641, 1.313691, -1.768265, 3.114930, -0.701036, 0.332594, 0.646543, -0.758599, 1.027964, -2.458871, 2.312337, 0.698046, -0.856793, -0.421708, -1.115664, -0.067457, 0.106864, 1.683989, 0.944561, -0.294706, -1.623733, 0.138827, 1.757520, 0.216496, -0.144224, -0.178804, -0.114752, -0.625126, -0.376474, 0.188924, -0.927172, 0.708884, -0.923497, -0.387956, 1.275919, 1.341081, -0.266292, -0.261587, 0.071471, -0.649638, 1.731181, 0.664054, -0.594163, -0.475404, 0.481147, 0.578050, -1.139624, -0.310093, 0.214624, -1.609158, -2.819581, 0.612600, -1.745760, 0.951655, -0.277275, -0.752859, 0.285261, -1.374545, 0.902928, -0.016793, -0.305215, 0.485079, -1.933432, -0.540696, -0.529339, 0.513742, 1.195184, -0.420798, -0.217768, 0.122213, 0.501855, -0.534182},{-1.214114, 0.921185, -0.504267, 0.441487, 0.954225, -0.470222, -1.637230, 1.909909, 0.087015, -0.362575, -2.055235, -0.769572, -0.729334, -0.459389, 0.422245, -0.620346, 0.259388, -0.267428, 0.959874, -0.659025, -0.676541, -0.399460, -0.759464, -1.757325, 1.255871, -0.206840, -0.232270, -1.842925, 0.009237, -0.286281, 0.207810, 0.231303, -1.198025, 1.260867, -1.188930, 1.011769, 0.742835, 1.018897, -0.217481, 0.911641, 0.623611, 0.008157, -1.191164, -0.641748, -1.076765, -0.944525, -0.721767, -0.901189, 1.426345, 1.037724, 0.298892, 0.556499, -0.088297, -0.554027, -0.338886, 0.167367, 0.048438, 0.125710, 1.222296, -0.090042, -0.745100, -1.036181, -0.377898, -0.354315, 1.654648, -1.595200, -0.050631, -1.261497, -1.583351, 0.111383, -0.059739, 0.320480, -0.449444, -0.026126, -0.754576, 0.583265, -2.513877, 1.083590},{0.280098, -0.246656, 0.433326, 0.451370, 1.828636, -0.062422, 2.229215, 0.097395, -1.160124, 0.844394, -0.303646, -1.291945, 0.856450, 0.123659, -0.281001, -0.687954, -1.382544, -0.450702, -0.068433, 1.757733, -0.725314, -0.556371, 1.704515, 0.858056, -2.463384, 1.115221, -2.133692, -2.815350, 0.792729, -1.958597, 1.494175, 0.496708, 0.516135, 0.400923, -2.205319, 1.354067, 0.601838, 1.652636, 0.392983, 0.084151, 0.584010, -1.338869, -0.317918, 1.497765, 0.253477, 1.086892, -0.468007, 0.023577, -0.088843, 0.389390, -0.832188, 0.725521, -1.412316, -0.029437, -1.422957, -0.014321, 1.815276, 0.637450, 0.607753, -0.510668, 2.223356, 0.426955, -0.211142, 1.717658, 0.255681, 1.192342, 0.666268, -1.183441, 0.756036, 2.943061, -0.153847, -1.174673, -0.661458, 0.293596, 0.456400, -1.989123, 0.384093, -1.778949},{1.279392, -0.811568, -0.510138, -0.491348, -0.275449, 0.783103, -0.499230, 0.837692, -1.150636, 0.815209, -1.258776, 0.777740, -0.095723, 1.077454, 0.437305, 0.031047, 1.020843, -0.043318, 0.674347, 1.640810, -0.132775, -0.088354, 0.301860, -1.499424, -1.409572, -0.571896, 0.353576, 0.171682, -0.646855, 0.274714, 0.255595, 0.646319, -1.254190, -0.828445, -1.447255, -0.483404, -1.394001, -0.796565, 1.583342, 0.623917, 0.476008, -0.892950, -0.753835, 0.434448, 0.676490, 0.787920, -1.127126, 0.561953, -1.404351, -1.197807, -0.450970, -0.527666, -0.869565, 0.072872, 1.528418, -1.137938, 0.462500, 1.046514, -1.423048, -2.217561, 1.179049, 1.130509, -0.123961, 2.787168, 0.403550, -0.402607, -0.732133, -0.582084, -0.840751, -0.609434, -0.501907, -0.318570, -0.084582, -0.240716, -0.444629, -1.474244, -0.249954, -0.642619},{-1.305296, 0.969863, -0.514450, 0.412096, -0.265973, -1.944754, 1.816582, 0.477726, 1.377414, 0.485623, 1.010348, 0.018663, -2.306376, -0.986722, -0.634933, -2.162458, -1.421049, 1.498020, 0.174833, -0.609307, 0.980920, -0.313473, -0.742678, -1.193113, -0.358105, -1.068990, -0.371196, 0.737532, -1.099909, -0.217225, -2.005228, -0.758875, -1.023608, -0.256013, 1.577104, 0.008837, 0.140665, 0.836558, -1.293869, 1.624279, 0.601508, -0.583141, 1.647107, -0.269163, 0.231736, 0.087590, 0.537172, 0.543872, 1.179845, 0.184728, 0.583192, 0.460290, -1.091678, 1.372610, 0.711314, 0.234756, -0.177662, 1.702059, 0.605351, 0.002806, -0.658633, 0.442047, -0.811441, 1.242839, 0.513353, 0.730119, -1.927294, -1.007368, 0.192705, 0.129897, -0.460930, -0.464745, -0.638043, 0.213615, 0.214786, -1.100780, -0.099998, -0.381538},{1.999914, 0.076291, 0.738395, -1.262447, 1.300279, 0.213497, 0.556844, -0.387180, -0.273661, 0.392097, 2.692240, 1.233839, -0.658730, 1.254089, 0.135443, -0.232875, 2.060375, 0.037749, -0.424017, -1.171079, 0.544775, 0.959439, -1.139056, 0.128059, 1.971713, 0.548806, 2.047476, -0.111509, -0.224927, -0.624806, -1.399867, -1.333363, -1.283552, 1.704500, -1.352378, 1.419569, 0.996159, 0.903389, 1.288845, -0.976342, 0.732409, 0.570255, -0.528627, 0.394393, 0.435626, -1.625999, -0.988114, -1.886716, 1.565248, 0.445432, -1.134922, 0.009222, 1.133014, 2.173221, -0.445206, 0.773796, 2.127919, 1.984523, 1.148325, -2.222305, 0.126472, 0.810089, 0.068444, 0.406255, -0.866074, 1.516708, -1.864124, -0.178725, -0.536752, 1.591713, -0.779733, 0.823777, 1.561878, 0.238170, -1.951276, -0.870528, 0.385670, -0.794418},{1.083996, 0.029836, 0.413129, 0.615609, 0.698919, -0.675547, 0.812706, 2.713830, 0.021439, 0.871499, -0.885641, -1.014323, 1.301927, 0.067162, 0.676465, -0.955957, -2.277139, -0.916253, 2.096660, -0.600780, 0.427629, 1.053214, 0.831747, 0.851689, -0.989436, -0.820277, 0.022861, 1.084922, -0.559302, 0.859339, -0.445095, 0.173457, -0.437097, -0.208552, -1.504402, 0.495664, 0.475636, -1.523640, -0.578641, 0.580362, 0.574911, -3.007848, 0.596135, -0.387135, 0.535588, 2.070900, 1.146071, 0.385916, -0.509877, -1.721706, -0.160072, 0.764161, 1.269742, 0.819062, -0.794497, 0.119879, -0.715230, -0.103162, 0.737217, 1.411498, -0.770877, 0.151870, -0.353501, -1.196864, -0.460471, -0.946849, -1.882041, 0.817693, -0.138226, -0.649840, -2.448384, -0.051155, -0.419462, 0.506305, -0.542003, -0.685376, 0.507733, 0.243725},{-1.085828, -0.512947, -1.071104, 0.570962, 0.596479, 1.021019, 0.494657, 1.110621, 1.672694, 0.242093, -0.836071, 1.309321, -0.388505, 0.339756, -0.513706, -1.817258, 1.296719, 0.390635, 0.249790, 0.178742, -0.625762, -0.155251, 0.464869, 1.420258, -1.280253, 0.678735, -1.936402, 0.213891, -0.902479, 0.441783, -0.900369, 0.041039, -0.167237, 0.115962, -1.728288, 0.405570, 0.852612, -0.628125, 0.083423, -0.806714, -1.399269, 0.214111, -1.600661, 0.096566, -0.016219, -2.016333, 0.237008, -0.049915, 0.018504, 0.326181, -0.547961, 2.381629, 0.795897, -0.257423, 0.081128, 0.028937, 0.026184, 0.494086, 1.151127, 0.928508, 0.102487, -0.576215, 1.665981, 0.512160, -0.749133, 1.102131, 0.328567, 1.596045, -0.920810, 0.656807, -0.332936, -1.353739, 1.793238, 0.827355, 0.772354, -0.490168, -0.835795, 0.770895},{-1.868117, -0.415531, 0.444531, 3.408001, 0.453098, 0.078004, 0.334161, -1.251980, -0.198742, -1.449795, -0.882542, -0.589552, 0.059044, 1.758670, -0.592595, -0.730235, -1.589355, 0.177764, -0.634297, -0.564365, -0.660030, 1.339847, 0.078427, 1.274276, 0.080425, 0.309464, 0.768813, -0.384933, -2.009049, -1.826366, -1.489393, 0.244766, 0.022208, -0.384300, 0.961621, 1.504275, -1.316658, 0.264147, -1.782885, 1.606045, -0.980690, 1.386559, -0.390305, -0.643874, 0.889694, -1.934603, -0.887490, 2.180390, 0.965120, -2.576908, 1.878780, 1.092414, 0.931495, 1.515911, -0.584739, 0.748836, 1.979777, 1.711113, 1.615675, 1.035591, 1.062031, -0.125136, -1.848381, 0.581480, -1.005965, 2.241101, 0.800746, -0.078153, 1.433615, 0.517625, -0.142921, 0.350881, 0.398735, -0.772220, -0.213904, 0.902254, -0.975543, 0.515548},{0.588012, 1.228664, 0.203556, 0.278917, -0.105836, -1.964337, 0.347594, -0.815910, -0.118105, 0.250857, -0.233093, 0.486952, 1.239971, -0.838505, 0.413119, 0.001418, 0.456746, -0.105156, 0.010593, 1.791082, -1.003559, 0.415994, 0.359139, -0.586178, -1.876943, 0.796264, -1.551583, -1.010764, -1.107404, -0.424370, 1.601862, -0.498853, 1.098548, 0.564256, -1.138424, 0.942529, 0.166749, 0.391873, 0.639237, -1.011419, 2.201134, 0.025280, -1.089487, 0.069139, -0.023780, -2.098342, -0.905141, -0.155756, -0.170828, 1.354739, 0.453072, -0.781298, 0.274429, 0.069953, -1.509891, 1.194277, 0.793592, 0.281630, -1.808690, 0.085886, -0.043514, 0.103437, 0.080270, -1.109128, -1.222043, 0.940022, 1.005816, -0.233467, -1.400911, 0.176804, -0.563245, 2.526516, 1.494042, -0.494032, -0.119207, 0.575495, 0.825302, 0.386187},{0.130683, -1.061265, -0.692459, -1.349131, 1.456809, 0.276230, -0.044326, -0.369021, -0.591128, -0.353479, 2.622334, 0.133385, 0.185712, 1.379271, -0.854137, 1.581292, 0.254628, 1.004853, 1.422752, -1.017720, -0.450019, -0.309428, 0.726753, -1.357400, -0.921778, 1.005531, 0.110269, -0.226089, -0.599418, 0.104201, 1.331793, 0.316721, -2.031252, 0.123187, 0.115379, 1.878951, -0.429994, -1.758615, -1.308861, -2.577553, 1.806733, 0.519137, -0.469292, 1.631586, -0.555682, 1.198705, -1.863508, -1.017995, 0.540412, 0.306785, -0.221590, 1.179263, -0.308763, 0.684763, -0.094837, -0.183584, 0.858925, 1.827556, -0.310623, 0.225133, -0.085006, -0.674574, 0.154564, -1.301448, -0.004036, 0.388766, -0.480094, -0.251109, 0.232703, -0.742248, 0.062619, -0.908948, 1.205603, -1.069690, 0.133505, 1.228086, 0.272068, -1.875052},{-0.913037, -0.616150, 0.217081, -0.615035, 2.697668, 0.500935, -0.150474, -0.400679, -0.013615, 1.156451, 0.012181, 0.574787, 1.530521, -0.738755, 2.581494, -0.681897, 0.456841, -0.557311, 0.309220, -0.059735, 0.462738, 0.224372, 0.657308, 0.291794, -1.657358, -1.075168, 0.707603, 0.030462, 0.574025, -0.452784, 1.666548, -0.893305, 0.298098, 0.875225, 0.348931, -0.471336, 1.391354, -0.619304, 0.181377, -0.096895, 0.830252, 1.136281, 1.737970, -0.240651, -1.085876, -1.477442, 0.422390, 0.336513, -0.156454, 0.492333, 0.989489, -1.247909, -2.442540, 1.207759, 1.553108, -0.231800, 0.964225, 0.068536, -0.097467, 1.240257, 1.329933, -0.602637, -1.914468, -0.381171, -0.473908, 0.071798, -0.263456, -0.215931, 0.354343, -0.281499, -0.810433, -0.361254, -0.662390, 1.880269, -0.486836, 1.262253, 1.258021, 0.383483},{-0.934176, -0.839723, -2.378686, 3.080999, 0.711226, -0.821815, -0.429785, -0.412032, 0.201014, 0.108207, 0.582529, 0.896714, 1.212951, -0.906979, 1.858699, -1.116448, 1.260763, -0.414023, 0.972397, -0.434068, -0.465327, -0.213559, 1.459067, 0.768530, -0.566040, 0.230278, -1.084217, 0.026772, -0.144649, 1.738552, 0.075114, -0.089629, -1.472225, -0.320791, -0.596711, 0.525134, -1.959424, 0.589968, 2.110875, 0.775301, 0.121419, -0.244078, 1.188754, 2.712485, -0.974925, 0.461849, 1.550823, -0.417386, 0.375124, 0.108669, 1.668787, -1.777976, 0.862278, 0.537163, 0.690157, 1.177216, 0.190236, -0.086987, -1.437078, -1.882971, 0.754536, -1.565228, -0.058319, -1.470871, -1.106772, 0.979168, -1.107632, -2.028710, 0.369102, 1.775324, 1.331304, 0.258828, -0.779487, 0.479260, 0.845147, 0.059565, 0.027336, -0.311029},{-0.913224, -0.478849, -1.590925, -0.790681, 1.488273, -0.466624, -0.623874, 0.896240, 1.894229, -1.713131, 0.270357, 1.119611, 0.474626, -0.207327, 0.105505, 1.952436, 2.080321, -0.240801, 0.127728, -0.581873, 1.075603, -1.666284, -0.675769, -1.542821, -1.228119, -0.973884, 0.523370, 0.091980, 0.229510, 0.158681, 0.905782, 0.786288, 0.111723, -0.236736, -0.266824, 0.982793, -0.392495, -1.388569, 0.564453, -0.970110, 1.472546, -0.668348, -1.725673, -1.284247, 0.580774, -0.040282, 0.386909, 0.433383, 0.536118, -2.159799, 0.038850, 0.520521, -1.085856, -0.426248, -1.938882, 2.374785, 0.488335, -0.810928, 2.086972, -0.073806, 0.688750, -0.830832, -0.758742, 0.780337, 1.250932, -0.727123, 0.811619, 0.035701, -0.975814, 0.113790, -0.041373, 1.350883, -1.037109, -0.335412, -1.020274, -0.180174, -0.504393, -0.365625},{0.367219, 1.092615, 0.896856, -0.561062, 0.408622, -0.366719, 0.627034, 0.441182, 1.185100, -0.571511, -0.005496, 0.121054, -0.129883, 0.044000, 2.322456, -1.473839, 0.571424, -0.306539, -0.279694, 0.518108, 0.877406, 0.484844, -0.263908, -0.475521, 1.211844, -0.053419, 1.329427, -0.020135, -1.997012, -0.088984, 0.362249, -0.560983, 1.694721, 1.509353, -1.202715, 0.556882, 0.591018, 0.785552, -0.690015, 0.141398, 1.225321, 0.690854, -0.395092, 0.012961, -0.663848, 0.284503, 0.759092, 0.499216, 0.069251, -1.254854, -1.185910, -1.423715, -0.367998, 0.054001, -0.199369, 0.273450, -0.334451, -1.327391, -0.563742, 1.289747, -0.934161, -0.632844, 0.223322, 1.026584, -2.360254, -1.107000, -0.282258, -0.703417, -0.397832, 0.659449, 0.553312, 0.240409, -1.161445, 0.109992, 0.200310, -0.860982, 1.875383, -1.286394},{2.144798, 0.107212, -0.233774, 0.967782, 1.296795, 0.195903, 0.610476, -0.510658, 0.827299, -0.606883, 0.674456, 1.381317, 0.393097, 0.805396, 0.267008, 0.395507, -0.654210, 0.994951, -0.875220, -0.113095, -0.815998, -0.596760, -0.343048, 0.552612, -0.094339, -0.218662, -0.766663, 2.505386, 1.749710, -0.355282, -0.357537, 0.219142, 0.809714, -0.427045, 1.214848, 0.487767, 0.258115, 0.813996, -0.363340, 0.784852, -0.174531, -0.199820, 1.785781, -1.383900, 0.614393, -1.750658, -1.609346, 0.527375, -0.316112, 0.432923, -1.227917, 0.171652, 2.080360, 0.789520, -0.844195, -1.415020, -1.933735, -1.631522, 0.568575, 0.469311, 0.412715, -0.969776, 0.155774, 2.312475, 0.103167, -0.458838, -1.216493, 0.430937, 1.240320, 0.100139, -0.973010, 1.068768, 0.929257, 0.986553, 1.378150, 0.033081, -0.122933, 0.934745},{-0.824002, 1.785403, 0.569761, -0.810574, 0.703987, -1.775678, -1.176843, -0.276728, -0.234398, -0.198957, 0.113880, -0.147096, 0.383705, -0.485947, 1.613246, -0.688614, 1.107324, 0.639018, 0.764870, 0.172511, 1.845245, -0.952477, 0.465890, -0.913937, -0.771297, 0.686897, 0.869241, 1.372406, 0.262523, -0.916622, -1.496178, -1.018277, 1.400542, 1.500810, 0.505684, 0.049568, -0.556065, -1.533486, -0.588459, -0.697122, 0.343354, -0.759986, 3.333842, 1.642106, 2.141378, 1.336391, 0.467420, -0.644941, 1.914208, -0.568904, -1.249501, 0.095898, -1.147936, 0.406986, -0.402989, 1.175606, -0.538332, 0.369823, 1.738311, -1.257837, -1.911626, -1.327542, -0.758828, 0.949974, -1.202852, 0.235206, 1.614823, 0.692335, 1.826381, -1.968274, 0.918593, 0.393329, -0.941734, -0.634615, 0.057628, -1.014093, 1.071510, 1.116752},{0.028392, -1.300485, 1.368804, -1.852280, -1.072023, 1.210555, 0.982895, -0.658136, -0.473054, 0.253940, -0.308796, 0.672421, 1.012835, -1.300612, 1.269637, 0.790683, -0.167780, -1.305753, 0.635048, 0.805209, -0.587449, -0.574193, 1.810642, 1.749509, 1.052305, 0.176335, -0.492700, 0.886512, 0.482430, 1.308584, 1.354599, -1.028918, -0.304775, 0.084136, -2.154659, 0.105249, 0.146410, 1.408196, -0.618902, -1.124470, 0.237092, 1.558266, 1.038796, -0.743958, 1.277497, -0.195917, 2.579279, -0.204604, 0.025680, -1.387499, 0.901382, -1.118767, 1.919485, 0.503146, -1.254586, -0.790699, 1.161250, -0.914789, 1.746041, 0.007877, -0.196315, 1.223740, -1.320156, 2.199472, 2.576975, 0.565478, 0.726431, 0.770137, -2.684511, 2.218457, 0.344176, 0.069952, 0.769153, -0.715431, -0.320127, 0.808651, -0.029191, -0.499796},{-0.925609, -0.700266, -0.627870, 0.381336, 1.196748, 0.925167, 1.556885, -1.267247, 0.720647, -0.363469, 0.304106, 0.925311, 0.051693, 0.969409, -0.282028, -0.640141, -0.472743, 0.678878, 0.362723, 0.639639, 0.589121, -0.058280, -0.354296, -0.601069, 0.021463, 0.066400, 0.776055, -0.739284, 0.005448, 0.935851, 1.463771, 0.931538, -0.027569, 0.906494, 0.324823, -0.223106, 0.233346, 0.355299, -0.894108, -0.707332, -0.248331, 0.979458, 0.388060, -0.779818, -0.253538, -0.467974, -0.888054, 1.921513, -0.060070, 0.523072, 2.636045, -0.777728, 0.601814, -2.208872, 1.761013, -1.471806, 0.311299, 0.341315, -0.004689, -0.554954, -0.180787, 0.249034, -0.206332, -0.607204, -0.409297, 2.423530, -0.077257, -0.532104, 0.923867, 0.226641, -0.434670, 0.331060, 0.195047, 0.676299, -0.613170, -0.732254, -1.210374, -0.452996},{-1.569945, -0.744728, 0.144633, -0.066022, 2.549844, 1.554375, -2.191340, 1.485369, -0.078383, 1.476464, -0.753977, 1.176672, -1.993939, 0.623904, -0.566068, -0.026410, 0.896606, -0.444447, -1.004623, -1.225813, 0.265581, 0.405593, -0.274035, 1.034936, 1.900396, -0.659667, -0.200676, 0.705171, 0.281712, 0.080502, -1.675430, -2.090018, 0.401759, -0.600396, 1.204083, -0.869556, 0.745861, -1.173255, -0.021157, 0.128772, -1.779854, -0.106006, -1.143950, 0.638016, -1.128215, 0.866855, 0.881944, -1.154910, 0.998572, 0.432184, -0.359650, -0.446252, -0.707270, 1.063758, -0.624989, -1.934895, -0.355418, -0.907905, 1.049283, 0.999426, -0.088247, 1.910761, 1.509996, -0.430910, -1.136913, -0.412536, -0.059740, 2.646269, -0.697465, -0.833656, 0.829332, 0.840920, -0.128473, 0.897629, -0.512995, 0.153959, 1.376208, -1.141878},{0.922282, 0.372746, -0.624224, 2.287309, -0.384273, 2.056377, 0.921572, 0.241079, -0.318370, 0.409337, -0.067055, -0.216780, -0.389317, -0.336433, 0.131863, -0.271254, 0.443435, 1.182651, 1.978880, -1.174939, 0.483645, -1.803523, -1.096316, 0.594250, -0.069282, -0.845461, 0.368772, 1.801749, -0.265746, 1.304269, -2.121979, 0.899340, -2.556687, -1.314987, -0.641730, -0.053079, 0.535101, 1.047523, -1.169006, -0.995753, -0.655444, -1.134588, 0.739612, 0.953120, -0.592012, -0.337059, 1.362802, 0.355476, -0.680539, -1.749629, -0.359055, 0.870182, -1.021189, 0.395010, 0.044898, 0.472679, 0.310983, 0.064470, -0.366013, -1.076720, -0.948121, 1.141234, 0.455959, 0.225026, -0.813275, -0.652176, -0.302493, -0.778808, 0.635472, -0.750044, 1.720289, -0.023676, -0.441561, -0.931404, 1.146918, -0.148460, -0.995539, 0.010118},{-1.041557, -1.255855, -0.538867, -0.438901, 0.080900, -1.964264, 0.079162, -0.669248, 0.797385, 1.085374, -1.502055, 0.324898, -0.016186, 0.801519, -0.897605, -0.543869, 0.307546, -0.018438, 0.750105, -0.988332, 1.126801, -0.176734, -0.727970, 1.445319, 0.785164, -0.095164, -0.060926, 0.440775, -0.679734, -0.066615, -1.628358, 0.437378, 1.305116, 0.759451, -0.198422, -1.703498, 0.195462, 1.632205, -1.189035, 0.952071, 0.496917, 1.640524, -0.109604, 0.881573, -0.115185, -2.205793, 1.757912, -0.438863, -1.505696, -1.236306, 0.961420, -0.056774, -1.097708, 1.193192, 0.426490, -0.837561, 2.578071, 0.294218, 0.666511, -0.081837, -3.035554, 0.040680, 1.714746, 1.121930, -2.123342, 0.192167, -0.173377, 1.795015, -0.856238, 0.163334, -0.313032, 1.555762, 0.892669, 0.909832, -1.110934, 0.806406, -0.815985, 0.231492},{-1.527420, -0.416533, -1.900758, 1.112960, 0.585401, 0.388904, 0.434197, -0.725841, 0.844588, -0.111249, 1.425950, -0.149121, 1.009151, 0.357109, 0.236730, 0.263134, -0.504972, -0.521273, 0.237395, -2.265142, -0.332727, 0.541246, 0.555623, 0.687034, 0.374299, -0.566216, -0.721639, 2.340945, -0.217149, 0.859303, 0.053330, -0.308905, 0.493309, -0.426931, -0.357418, 0.076814, 0.568145, -0.412161, -0.827414, 1.468461, 0.936139, 0.347243, 1.240733, -0.672445, 1.466382, 0.813311, -0.943153, 0.068653, -1.549166, -2.220085, 0.261468, -0.134956, 0.438591, 0.245042, -0.984658, 0.379854, 0.761384, -1.579345, -1.830802, 2.457031, 0.714505, -0.544856, -0.732312, -0.342227, 0.556398, -0.191111, 2.369012, -0.109780, 0.883481, 2.254321, -0.394734, 0.247928, 0.982586, -0.414397, -1.538827, -1.444304, -1.218862, -0.360453},{-0.049767, 0.543839, 0.289740, -0.074312, -0.637951, 1.431239, 0.299050, 0.093405, 0.146055, 0.017738, -0.238769, 0.059480, 1.121061, -0.022472, 0.837066, -2.184714, -0.230972, -0.498041, -0.285088, -0.134506, 1.111063, -1.114977, 0.841389, -1.580711, 1.821135, -1.353720, -0.043126, 0.189355, 0.744345, 0.511215, -0.501060, 1.934369, 0.505237, 0.372648, 0.784196, -0.488473, -0.882091, -0.263955, 0.696410, -0.225754, -1.854907, 1.701226, 1.457654, -1.941587, 0.406590, -0.841425, 0.706808, 0.339081, 0.749078, -1.055230, 0.759196, 0.538263, 0.204634, -0.347498, -0.268290, -1.438660, -0.061015, 0.212967, -1.855963, -0.930358, -1.347924, -0.876092, 2.016841, -0.608782, 1.113456, -0.734417, 1.002220, 0.198635, 0.347267, 0.404039, 1.227344, 0.458497, -1.145385, 0.397991, -1.236780, 0.691971, -0.610930, -0.297797},{1.842079, -2.911156, -1.997885, -0.504984, -1.365722, 1.643943, -0.283272, 0.357693, 0.561661, -1.997378, 0.225375, -0.946581, 0.765703, 2.522857, -0.184804, -0.526181, 0.178204, -2.012666, 1.414588, 0.745690, -0.216697, -0.863266, 1.367997, -0.503759, -0.457566, 1.204098, -0.546572, -0.254152, 0.086595, 0.172628, -0.738400, -2.056364, 1.550370, -0.097243, -0.075779, 0.622601, 1.044051, -0.182606, -1.144054, -1.351922, 0.798874, 0.769735, 0.409624, 0.618644, 0.479912, 0.412380, -1.112258, -0.643074, -0.796687, 2.666702, 0.535524, 1.854604, -1.853241, -1.737871, 0.806232, -0.708258, 1.511278, -0.806860, 0.595794, -0.577209, -1.792635, 1.961925, -1.007945, -0.030300, 1.483243, 1.826420, -0.939258, 1.334962, -0.417890, 0.653662, -0.484123, -0.729065, 0.437285, -2.028448, -1.780653, 0.912366, 0.022175, 0.299232},{0.405312, 0.457001, 0.446703, -0.097269, 0.318586, -1.126921, 0.016728, -1.224846, 0.233361, -0.299736, -0.319838, 0.285156, -0.527271, 0.443809, 0.805382, 0.976270, 1.068715, -0.332535, 0.113234, 0.571058, -1.820288, -1.170631, -0.636365, -0.585633, 1.679841, 2.004891, -0.831991, -0.164730, -0.315255, 0.941524, -0.113804, 0.566187, 1.310943, 0.445828, -1.418975, -0.870628, 0.808026, 1.050073, 0.575730, 1.502216, -0.129040, -1.077250, 1.208059, -0.295309, 0.360269, 0.753865, 1.921481, 1.010734, 0.654878, 0.241354, 0.944471, -0.408105, -0.055694, -0.428839, 0.086968, 1.556342, -1.102839, 1.976206, -0.055400, -0.215336, 0.341921, 0.212954, 0.620955, -0.277024, 0.249913, -0.591570, 0.020734, 0.832052, 0.916254, -0.293652, 0.192312, 1.268548, -1.478629, 0.452302, 0.303926, -1.322054, -0.564622, 0.495020},{0.300375, -0.974366, 0.944884, 0.619385, -0.185112, -0.265507, -0.728582, 0.502780, 2.250170, 2.461614, 0.387612, -0.733066, -0.999481, -0.899569, -0.747609, -0.134069, 0.282618, -0.643296, -2.040780, 1.832643, 0.984766, -2.391593, 0.333806, 0.663108, -1.833098, -0.558850, -0.577683, 0.467941, -0.365881, -0.299550, 0.391291, -1.203188, 1.208411, 0.056842, 1.641148, 0.201591, 0.062034, -1.208912, 0.798660, -0.159722, 0.502321, -1.629378, 0.479660, -0.418872, 0.205207, 1.981681, 0.974183, 0.994847, 0.867854, -1.238721, -0.671162, 1.199833, -1.039942, -0.458591, -1.390532, -0.002976, -0.411923, 1.602575, -0.150408, -0.924330, 0.495477, -0.395966, 2.455465, -0.690233, -0.339188, 1.061504, 0.696203, -0.774202, 1.027392, -0.257455, 1.263224, 0.712716, 0.632614, -0.376820, -1.324081, -2.819313, -1.450703, -0.572632},{-0.604025, 0.494182, -0.129794, -0.431410, -0.122921, -0.958367, -2.218310, -0.774490, -1.388366, 0.441094, 0.351264, 0.415367, -0.956654, -0.613527, 0.087289, -0.938236, -1.089483, 0.391865, 1.155317, -0.060471, 0.238956, 0.358995, -0.923072, 0.600584, -0.016257, 0.884535, 0.907058, 1.581512, 0.799541, 0.249914, -0.695593, -2.380849, -0.224784, -0.981830, -0.311025, -0.698094, 0.233579, 0.265581, -0.377702, 0.467998, 0.248475, 1.126552, -0.152810, 1.424634, 0.036886, -1.467271, -0.817284, -0.669801, 0.259501, -0.846646, 0.829152, -0.953034, -0.338693, -0.839773, -0.235144, -0.006563, -0.298623, -1.895219, -0.092826, 1.294760, 0.239684, 1.674685, 0.211119, -0.892755, 1.232973, 0.692059, 1.816301, 1.098519, -0.391722, 3.106606, 0.446738, 0.122978, -1.290715, 0.422128, -0.266991, 0.159644, 0.401139, 0.686962},{-0.886210, 0.469495, -0.465586, -0.367451, -1.176238, 2.715706, -0.780129, 1.335527, 0.891462, 0.569605, -0.940904, 1.177009, -0.881319, 0.772183, 0.758363, 0.155564, -0.219949, 0.799070, 0.859546, -0.877910, 0.045095, 1.155829, -0.200590, 0.192526, -1.741728, -0.231368, -0.169326, 0.201707, 0.031479, 0.975496, -0.159998, -0.260548, 0.344676, -1.644462, 0.140446, -2.006412, -1.879056, -0.802878, -0.066361, 0.493824, -0.185723, 0.070177, 1.471180, 0.513409, 0.955335, 0.334197, -0.488232, -0.007828, -0.352861, 2.413311, -1.903795, -1.734697, 1.344608, -1.463318, 0.010359, -0.141257, -0.620851, -0.295199, 0.763128, 0.887922, 0.802003, -1.549946, -0.819785, 0.104958, -2.166690, -1.338224, -0.153819, -0.110402, 0.499106, -0.182149, -0.755407, 0.497615, -1.530323, -0.343680, 1.192490, 1.149757, 2.359980, -0.145221},{0.399369, 0.237768, -1.465964, 1.252872, 0.849226, 0.082125, 0.941948, 0.346695, 0.384814, -0.046853, -1.189567, -0.179807, 1.288636, -0.855146, -0.913128, -0.203178, 0.478242, -0.461519, 1.395305, -2.453624, 0.920604, 0.197720, -0.185247, -1.096394, -0.811706, 0.181147, 2.035496, 0.101920, -1.721579, -1.183315, 1.452259, -0.363147, 0.344606, -0.293742, 1.038491, -0.485357, 0.059635, 0.741605, -0.788354, 0.357734, 1.068687, 1.052114, 0.332955, -0.590156, 1.425844, -1.570751, -0.830786, 0.390418, -0.237808, -0.834732, -0.909959, 1.641674, -0.746448, -0.392109, 1.015511, -0.812880, 0.098713, -1.001990, -0.682152, -0.572019, -0.905336, 0.132094, 1.377175, 0.481063, -0.101706, -1.794516, -0.319971, -0.471071, 0.385150, -1.531980, -0.126544, 1.187513, -0.441219, 0.163552, -1.287538, 0.189124, -0.196534, 0.734492},{-2.048834, -1.654676, 0.778444, 0.514510, 0.512918, -0.945530, 0.495066, -2.116865, 0.281071, 1.473206, 1.083443, 0.345683, -0.949653, -0.473245, -0.622192, -0.076629, -0.973996, 0.178167, -1.285401, 0.972969, 2.294234, -0.958661, -0.713801, -1.923012, -0.487148, -0.018267, -2.292487, -1.684506, -0.460657, -0.790930, -0.358921, 1.992141, -0.318704, 1.439571, 1.024981, -0.299499, -1.158234, -0.445453, 0.012308, -0.366600, 0.637072, 0.172711, 0.167149, 1.960144, -0.627312, 1.133533, -0.004304, 0.528590, 1.132483, 1.524694, 0.027362, -0.153744, -0.380437, -1.068445, -1.479294, -0.115795, 0.125790, 0.377440, 0.672961, 0.868030, 1.368001, -0.228832, -3.115921, -0.088133, 0.733853, -0.732575, -0.563590, 0.512883, 1.046734, -1.356119, -0.923644, 0.241630, -0.299074, 0.680300, 0.179656, 0.139939, 0.900505, 0.283704},{-0.817531, -0.480565, 0.403535, -0.060639, 0.063633, -0.294778, 0.976092, 2.290814, -0.334616, 0.909507, -0.796061, -0.950731, -0.667903, 1.428896, 0.696892, 0.954396, 0.186805, -0.322124, 0.001418, -0.111138, 1.359817, -1.323246, 1.292452, 1.207369, 0.833877, 0.634794, -0.458184, -0.533716, -0.899430, -1.801643, 1.860042, 0.127119, -0.353580, 0.312465, -0.352006, -0.737998, 0.462284, 0.253770, -0.531159, 1.983816, -1.962757, -0.399449, -0.910345, -0.122343, 0.055679, 0.372497, -0.356897, 0.518458, 0.419749, -1.547119, -0.682804, 1.097835, 0.899806, -1.501650, -0.683734, -0.215292, 0.140572, -0.108033, 1.849428, -0.238333, -0.672925, -0.253661, 0.341475, -0.718504, -2.043673, 0.193666, 0.679323, 2.318129, -1.648760, 0.389894, 0.851459, 0.832460, 0.917388, -0.286038, 0.354440, -0.185944, -0.719905, 0.331525},{-0.334538, -1.455854, 1.451185, 1.851517, -1.574716, -1.677487, 0.469054, 1.174051, -0.985792, -0.901174, 0.686279, 1.984485, 1.007362, -0.130544, 0.452078, -0.567018, -0.562556, 0.904764, -0.958561, 0.520718, 1.487126, -0.420079, -1.312533, 0.404130, 1.942981, 0.338083, 0.017746, -0.541851, -0.046116, -0.540093, -0.344743, -1.225068, 0.775072, 0.316629, -0.010607, 0.552597, -2.366443, -1.430887, -0.211932, 0.433151, 1.339983, -1.195294, -0.031289, 1.836121, 2.490566, 0.907037, -0.870335, -0.563709, 1.110875, 0.345312, -0.586386, 1.569201, 0.792596, 1.301658, 1.265175, -0.182421, -1.845956, -0.760553, -2.198938, -1.042275, 0.363284, 0.971623, 0.351461, -1.731407, 0.618816, -0.869359, -0.877612, 0.283261, -0.432038, -0.805778, -0.135334, -0.405247, 2.199604, 0.759138, -0.496914, -0.171940, -1.410710, 0.636012},{0.326284, -1.659069, -0.901666, -1.725702, 1.091635, 2.089400, 0.225485, -0.550879, 0.719390, -1.459707, -0.221985, 0.023419, 1.397582, 1.717396, -0.398108, -0.759603, -1.149141, 1.232439, 1.039504, -0.976928, 0.854340, 1.273539, 0.125435, 0.788189, -0.938558, -0.791240, 0.093252, -1.022390, -1.037785, 0.364415, 0.438748, 0.105445, -0.171218, 0.661580, 1.110577, -0.933894, -1.939363, 0.859111, 1.963228, 0.437152, 0.523697, 1.116469, -1.559035, -1.249574, -1.261430, -0.121934, 1.835637, 0.656275, 0.666162, -0.043996, -0.578321, -0.646233, 1.095309, -0.054921, -0.703401, 0.316322, -0.031400, 0.497891, -0.786708, -1.460061, 0.993516, 0.252777, -1.242908, 0.699292, 1.350111, 1.243616, 0.809407, 1.038853, 0.466174, 0.879377, -1.840553, -0.262349, -2.662256, 0.356512, -0.891863, -1.859825, 2.190553, -0.497095},{-0.826212, 0.385653, -0.648378, 0.834282, -0.406110, 0.004930, -0.060157, 0.702719, 0.143484, -0.782728, 0.254585, 0.709087, 1.823377, 0.236810, 0.536168, 0.683146, -0.050163, -0.716810, 2.360104, -1.326592, 1.545238, -1.888342, 1.327411, -1.737810, 2.725191, -0.236392, -1.084520, -0.223905, 0.070196, 0.550477, 1.061833, 1.010214, -0.420429, 1.221299, -1.344574, 1.423508, 0.427228, 0.694306, -1.125657, 1.773762, 0.917284, -1.713525, -0.807537, 1.036693, -0.684983, 1.737676, -0.109715, -0.885730, -0.980352, 0.606563, 0.609322, 0.644188, -0.225450, -1.014180, 0.883443, 0.052446, 0.046800, 0.153767, 0.115755, 0.091737, 0.673434, 0.382502, -1.133260, 2.163013, -0.231397, -0.789115, 0.096338, 0.657721, 0.553095, -0.192004, -0.370307, -0.095109, -0.106345, -0.603394, -0.338504, -1.806504, 0.930761, -0.036444},{-1.653583, -0.214738, -1.222353, 0.835439, 1.107340, -1.147244, -0.576968, -1.704536, -0.046548, 0.016871, 0.230011, 0.430723, 0.203339, -0.596713, -0.747477, 0.559758, -0.625902, -1.238664, -1.262541, -0.366297, 0.887960, -0.820962, 1.567653, 1.192522, -0.346017, -0.756029, 1.503302, -0.047458, -0.756053, 0.586619, -0.204607, -0.765922, 2.015882, -0.356819, 1.209459, 0.615134, -0.353119, 0.775235, -0.616914, 0.071414, 0.565311, 1.667920, -0.502541, 2.631573, -0.234308, -0.641971, -0.210171, -0.445361, 1.637256, 1.166964, -0.463611, 0.577407, 1.045545, 3.269557, -1.669459, -1.056391, -1.071935, -0.221763, 1.823323, -1.536123, 0.351948, -2.398684, -0.543229, -0.197161, 1.992872, -0.124263, 0.722425, 0.401471, -0.876195, 0.265332, -1.156432, 0.377108, -2.042982, 0.575876, -0.892356, 0.479113, -0.090219, -0.550633},{-0.714010, -0.392635, 0.314162, 0.021760, 0.315689, -0.138981, 2.219904, 0.588111, 0.200014, 1.085493, -3.050854, 0.779140, -0.102136, -0.105592, 1.543313, -1.251225, -0.585117, 0.153189, 1.932724, -0.035297, -1.076457, 0.492525, 0.309348, -0.792738, -0.218296, 2.442098, -0.502298, 1.299202, -1.324238, -0.288889, -0.299837, 0.247434, -0.529121, -0.145166, 0.617581, -2.146778, 1.035700, 1.068927, 0.451862, 0.013720, -0.714485, -1.251709, -0.675297, 0.854975, 1.284241, -0.317298, -1.344311, 0.428491, 0.373267, -0.611801, 1.008771, -1.856224, -2.196337, 0.700308, -0.632551, -1.060280, -1.712124, 0.727618, -0.282035, 0.843175, -0.366178, 0.628436, 0.747321, -0.441986, -0.072175, 1.220138, -0.524303, 0.949380, 0.818554, -0.949103, 0.461543, -0.686948, -1.148191, 1.547280, -0.670087, -2.774095, -0.624283, -0.412193},{0.408178, -1.316728, 1.004633, 0.230370, -0.934479, -0.554412, 0.881425, -0.904484, 2.026111, 0.295317, 1.342684, -1.214672, 0.903939, 0.682142, -0.069854, -0.508563, 0.203723, -0.012601, -0.756644, 0.099138, 0.310712, -0.082927, 0.364053, -0.695014, 1.998814, -2.082633, -1.483345, 2.139415, -1.300973, 0.482903, -0.945146, 0.526672, 1.786734, 0.591799, -0.440708, 1.426226, -0.027309, 0.942935, -0.033433, 0.051887, -0.717481, -0.204119, -1.334280, -0.409858, 0.379911, -0.335316, -0.104607, 0.246146, -0.411639, 0.382233, 0.882076, -2.286190, 0.895698, 1.265855, -0.650988, -2.034064, 0.385277, -0.007042, 0.291785, -0.154346, 0.737990, -0.863032, 1.352964, -0.647697, -0.547481, 1.304823, 0.164510, -1.448062, -1.086456, -0.992867, 0.484246, -0.073773, 0.229376, 0.941604, -1.278917, -2.010780, 1.337119, -0.869198},{0.478366, -0.980057, 0.206322, -1.163766, -0.137196, 0.107765, -0.708648, -0.361224, -0.542336, 0.007364, -0.016329, -1.498922, 1.448253, 1.979683, -1.131499, -0.820697, 0.068450, 0.731058, 0.368747, -0.573582, -1.675451, 0.117746, -0.836821, -0.241693, -0.581475, 0.556859, 0.278521, 1.076789, -0.774624, -0.070336, -2.186296, 1.089624, 1.023951, 0.607663, 0.002412, -0.083274, -0.485626, 1.000282, -0.646832, -1.577450, -1.703693, 0.153929, 1.501588, -1.381236, 0.560432, -0.452680, -0.090618, -0.277260, 1.147922, 0.912962, -1.113460, -1.355795, -0.975969, -1.642125, 0.984364, -2.218804, -0.013126, -1.119198, -0.870190, 0.438604, 0.447602, 0.818453, 0.480158, -0.334497, 1.044934, 1.325705, -1.151911, -0.890541, -0.855507, 0.579670, -1.966028, 2.361700, -0.445992, -0.178268, 0.219859, 1.124995, 0.404214, 0.351512},{0.110405, 0.322667, 0.540436, -0.511383, -0.546704, 3.388518, -0.053709, -0.202270, -0.867292, -0.999789, 0.438546, 0.949356, -0.420634, -0.227617, -0.337692, -1.059656, 0.459026, -0.034355, -0.211251, 0.492645, -0.374050, -0.481690, 0.764348, -1.041700, -0.695512, -0.374450, 0.374498, 1.956945, -0.680507, 1.005433, -1.192062, 0.586766, 1.071385, 0.152357, -0.249547, 0.257215, 0.013142, -1.250262, 1.497803, 1.174125, -0.422895, 0.729311, -0.247905, -0.663452, 0.974925, -1.155959, -2.058773, 0.470596, 0.695508, 0.307035, 0.455097, -0.620653, 0.794369, 0.281043, -1.724493, 0.470462, 1.063237, 0.162528, 1.132307, -0.090887, 0.279788, 0.124170, 2.284801, -0.160552, 0.465719, 0.546968, -1.396272, 0.689852, -0.593539, 1.587567, -1.742286, -0.558358, -0.384144, 1.238284, -0.588191, -0.916606, 0.299593, -0.666232},{-1.736114, -0.052075, 0.371602, 1.028151, -1.571423, -1.532329, 0.073035, -0.694532, 1.633909, 0.197375, -2.591441, -0.600296, -0.594561, 1.587860, -0.456850, 0.892753, 0.468701, 0.577206, 0.757969, -0.295026, -1.648132, 0.906335, -0.475236, 0.144717, -0.663309, 0.482857, -0.727645, 1.066527, -0.574755, -0.150766, -0.588326, 0.089660, 0.361355, -0.516600, -0.857845, -0.809237, 0.878110, 1.119279, -0.282663, 2.143513, -0.673625, 0.810919, -0.804657, 0.079838, -0.568751, 0.665533, 0.223407, -0.630135, -0.310943, -1.954148, 1.232984, -1.754310, -0.035271, 2.696736, 0.285262, 0.564593, 1.117239, -0.505478, -1.022993, -1.471506, -0.819830, 0.414357, 1.254819, -0.060715, 2.684294, 1.248814, 0.332673, -0.184932, -0.009053, 0.887339, 0.159064, 0.226986, -0.459313, -1.893279, -0.357271, -0.040813, -0.145916, 0.545534},{1.181008, 0.485454, 0.756435, -0.355922, -0.799094, -0.786331, -1.047232, -0.113225, -0.146274, 0.450691, -0.381256, 0.006362, -0.219910, -1.364022, 0.372379, -0.526706, -0.893331, -0.214691, 1.253323, -1.305414, 0.030993, -1.604210, 0.103133, -1.941354, 0.007460, -1.278430, 1.221874, -0.172288, -0.316609, 0.139532, 1.290109, -0.395117, 0.744810, 0.224577, -1.815629, -0.762441, -0.727534, 0.151151, -0.369074, -0.893020, 1.085032, -0.179173, 0.085860, -0.856683, -0.467273, -1.705484, -0.412224, 0.382914, 0.484295, 0.948309, -0.419185, 0.624675, 1.440975, 0.972616, 0.292741, -1.107518, 0.842342, 0.746353, 0.955686, -0.596775, 0.193066, 0.932488, 1.746094, 1.247779, -1.375352, 1.218944, -0.139966, -1.266206, -0.808362, -1.210629, -0.581754, 1.637160, 1.071241, -0.180553, 2.203092, -0.637661, -1.006757, -1.045522},{-1.520012, 1.527415, 2.670244, 0.366540, -1.581806, 0.216106, 0.524326, 2.827454, 0.484702, -0.660702, 0.308926, -0.711251, 0.137296, -0.581395, 1.132451, -0.210386, 0.609382, -0.412268, -1.140646, 0.233961, 0.815965, 0.023730, 0.709184, -0.817575, -0.322603, -0.655014, 0.744244, -1.771609, -0.002300, -1.766068, 0.679515, 0.809589, 0.192565, 0.229606, 0.715779, -0.713451, 0.619155, 0.994095, 0.964149, 0.433483, -0.475584, -1.350128, 0.412593, 0.553786, -0.541457, -0.709687, 0.376269, -0.567212, 0.421179, -0.835933, 0.094622, -0.410394, 0.324830, -0.417818, 0.133566, 0.578795, -0.499374, 0.680160, -0.218022, 0.287670, -1.628416, -1.976534, 1.122962, -0.461024, -1.186220, 1.476666, 1.112348, -0.285417, -0.254967, -0.245203, 1.243924, -0.204901, -0.922849, 0.413766, -0.914904, -1.167343, 1.805278, -0.737705},{2.118146, -0.502284, 0.908514, -1.608495, 1.542755, 1.319040, 0.813089, -0.415306, 0.038229, -0.515813, 1.730638, -1.567658, -1.051203, 0.298662, 1.035293, -0.552267, 1.336124, 0.333663, -0.538119, -1.105363, -1.079123, -1.909808, 1.058224, 0.148368, -0.974278, -1.266645, 0.367094, 1.397810, -1.469831, -0.557522, 0.576425, -0.332268, 0.950162, -0.835437, -0.196921, -0.665587, 1.454291, -1.269434, 1.689042, -1.039044, 0.570104, 1.739548, -0.769283, -0.820727, 1.001514, -0.266355, 0.684251, -1.005231, 1.036271, -0.806745, -0.215995, -0.324411, -1.197310, 0.216598, -0.088045, -0.050703, -1.819502, -0.026589, 0.767668, -0.838454, 0.196214, 0.120778, 0.065363, 1.979010, 0.324804, 1.493503, 0.032200, 0.286219, -0.476986, 0.303540, 0.723420, 1.467653, -1.670167, -0.673658, 0.842442, 1.982167, -0.476185, 0.809477},{-0.624135, 1.453438, -0.166284, 0.709336, 0.027999, 2.040358, -0.544117, -0.240611, 0.965107, -0.528679, 0.475365, -0.414623, 1.534796, 0.044747, 0.694611, -0.841959, 1.632557, -1.498892, -1.485323, -0.531658, 0.744435, 1.213014, 0.256476, -3.016250, -0.736656, 0.808062, -0.981904, -1.122355, 1.697038, 0.867869, -0.425232, -0.804556, -1.082761, -0.626214, 0.943711, -1.152857, 0.470992, 2.165302, -1.815336, 0.149926, -1.560034, -0.203136, 2.325923, -0.434116, -1.845351, -0.010129, 0.829776, 1.945755, -0.718174, 0.776555, 0.458684, 0.552705, 0.562301, 0.377762, -0.229603, 0.428245, -0.898572, 0.740710, 0.037748, -0.924830, -0.751710, 0.453659, 0.018229, 0.206344, -0.973301, 0.701121, 1.219655, -1.645424, 0.119470, -0.063987, -1.546826, 0.221301, 1.255246, 2.471498, -1.161100, -0.078664, 0.977349, -0.517525},{-0.331043, -0.657531, 0.379366, 0.835119, 0.769897, -1.229788, 1.259718, 0.560636, 1.123322, -1.399509, -0.367237, 0.524041, -3.917388, 1.776157, 0.558667, 0.141481, 2.348713, -0.708956, -0.571201, 0.035005, 0.353061, 1.127845, 0.040807, -0.313562, 0.084901, 0.814107, -1.438933, 0.026790, 1.570622, 0.536274, -2.295918, 0.368605, -0.678262, 0.006446, 0.711315, 1.268095, -0.141089, -0.919947, -1.293660, 0.961310, 0.476339, 1.564483, 1.535713, 0.873654, -0.947547, -0.918683, -0.912011, -0.701620, -0.613575, -0.662228, -1.646769, -1.073632, 1.086039, 0.152833, 0.688027, 0.294778, -1.311458, 0.878713, 2.014420, 1.069833, -0.387676, 0.767517, -0.523647, 0.345800, 0.951020, -0.003412, -1.226751, 1.973749, 0.362420, 0.836015, -0.619303, -0.384676, -0.273280, 1.023285, -0.813092, -0.439268, 1.003561, 0.250016},{-1.232594, -0.320302, 0.127185, 1.352224, -1.634432, 1.199514, -0.995246, 1.359650, -1.121636, 2.563944, -0.800738, -0.635842, -0.813151, 1.329620, 0.526528, 0.017085, -1.572978, -0.934710, -0.132374, 1.005658, 0.813823, -0.442667, -0.431271, 0.542286, -0.662047, -0.676339, 0.504437, -0.689731, -1.184647, -1.140651, 0.042134, 0.684833, 0.312740, 1.059498, -1.616873, 0.564246, -0.110095, 1.226885, 0.538755, -0.282303, -1.270732, 0.790303, -2.705889, 0.940941, -1.387041, -0.459344, 1.607736, 0.278812, 1.232025, 0.956667, 1.066766, -1.000714, -0.348062, -1.881248, -0.159927, -1.898394, -0.290056, 2.198207, 0.759133, 0.528120, -1.058620, -0.393895, -0.838635, 0.903106, 0.047835, 0.474433, 0.920548, 0.597957, 0.144032, -0.359439, -0.750266, -0.429531, -0.117558, 0.983478, -1.583262, -0.465604, 0.851591, 0.504377},{0.071152, -0.273601, -0.891912, 0.260954, -0.484627, 1.229364, 0.150460, 0.711702, -1.085656, 0.565654, 0.323237, -0.137675, -1.551269, -0.493670, 1.764565, -0.226604, -0.280020, 0.277273, 0.987217, 0.026762, -0.364730, 0.315761, 1.163827, 1.498488, 0.055715, 0.267528, 1.099028, -0.316913, 0.900667, 0.738048, -0.581837, -0.816373, -0.764909, 1.289864, 1.037916, 0.913881, -1.515036, 0.906006, -1.729380, -1.489943, -0.645529, -1.699699, -1.068236, 1.657630, 0.607313, 1.691184, -0.519807, -0.417128, 1.588852, -0.165014, -0.245743, 1.006100, 1.694266, -0.944428, -0.783258, -2.191185, 0.580103, 0.546510, -0.039257, 0.312021, -0.760571, -1.784512, 0.815402, 0.112457, -1.400594, -0.030489, -0.303022, -0.206409, -1.377408, 1.725677, -1.556139, 0.698486, -0.561883, 0.423993, 2.136932, -0.069284, 0.163043, -0.126716},{-0.523464, -0.592715, -0.714923, -0.334096, -0.856367, 0.351268, 0.202015, 1.582938, -0.371224, -1.547207, -0.171830, 0.576657, 0.744157, -1.466534, 0.396532, 0.696021, 0.715433, -0.836946, -0.584876, -1.086384, -0.791377, 0.037508, 1.639955, -0.042234, 0.602421, 0.735092, 0.543562, -0.250361, 0.854840, -0.419981, -1.375928, 0.942752, -0.150205, 0.097001, 0.940863, 0.253919, -0.042168, -0.254131, -1.614517, -0.527213, -0.633544, 0.373831, 0.296413, -0.726445, -1.032331, -0.453225, -0.599114, -0.564980, 0.200278, -0.174838, -0.385854, -1.368703, 0.993418, 0.504424, -0.002155, 0.702621, 0.359278, -0.117951, -1.387347, 0.058462, 1.732725, -0.757372, -0.513391, 1.061841, -0.733983, 1.978522, -0.753415, -1.125459, 2.138955, 0.802641, 0.458316, -0.280528, 1.076362, -0.690062, -2.597946, -1.711915, -1.267185, 0.475496},{0.913672, -1.981561, 0.761092, 0.239895, 0.844515, 0.963655, -0.009737, -0.387600, -1.816782, 0.844892, -1.486758, -0.609554, -0.204991, 0.866798, -0.611044, 1.835428, 0.121462, 0.857872, 0.183723, 1.041874, -0.348822, -0.876430, -0.585836, 1.176682, 0.960855, 0.615225, -0.693342, -0.240418, -0.573755, -1.530172, 0.079080, 1.260734, 0.746388, -1.191862, -0.008450, 0.065058, 0.022468, -0.914215, 0.989026, 1.781662, -1.707342, 1.176675, -1.052421, -1.031941, 0.385538, 0.599524, -0.918973, -0.216933, 0.137821, 3.330509, -0.674062, -0.224746, -0.839200, 0.730359, -1.216865, -0.974452, -0.464536, 0.141808, 0.777718, -0.569106, 0.432379, -1.857875, -1.827903, -0.772016, 0.695715, 1.164042, -0.969168, -0.639109, -0.532929, 0.635039, -1.598283, 0.784049, -0.458191, -0.582726, -0.549801, -0.340052, -1.330654, 0.572639},{1.150049, 2.240555, 0.447043, 0.200025, -0.498980, -1.831960, 0.274425, -0.544267, -0.885782, 1.245205, 0.956557, 0.557262, -1.723884, 0.328489, 0.192879, 1.054273, 1.778683, -0.669194, 0.050512, 0.935385, 0.837264, 1.962349, -0.065122, -0.473678, 0.183671, -1.760817, 1.017066, -0.014863, -0.692008, 0.736752, 0.638948, -0.585741, -0.624570, -0.406291, 0.166522, -0.584005, 1.722366, 2.769243, -0.016057, -1.086433, 1.032144, -0.134231, -0.035938, 0.013903, -0.926856, -0.849589, 0.348120, 2.000779, 1.610245, -1.324865, 0.973020, -0.040631, 0.233636, 0.110056, 0.208551, 1.299373, 0.228466, -0.340585, 1.132210, -1.153501, -0.250550, -1.075335, 0.872656, -0.459536, 0.499197, 1.032247, -1.211015, 1.446891, 0.256386, -0.595873, 1.305790, -0.102157, 1.162066, -0.577172, 0.071035, 2.272710, -1.853256, -0.581529},{1.020487, -0.906777, -0.619451, 0.615218, 2.168201, 0.909623, -0.814513, 0.310697, 0.915875, 0.291239, -1.499041, -0.748182, 1.044456, 0.096370, 0.604445, -0.620642, -0.352407, 0.155925, -1.757884, 0.424643, -0.611056, -0.313943, -0.108356, -0.885493, -0.645606, -1.668339, -1.257858, 0.010990, 0.385062, 0.173326, -1.256500, 1.148907, 0.668778, 1.809537, 0.258512, -0.287262, 1.391871, 1.117992, -0.430256, -0.081383, -1.097971, 1.507290, 0.399882, 0.517486, -0.981200, 0.529204, -0.533777, -0.382023, 0.293512, 0.258167, 0.479035, -0.474580, 0.219486, 0.829242, 1.933991, 0.050834, 0.820854, -0.371587, -1.287120, -1.169964, -0.227427, 1.298068, -0.103126, 1.181435, 0.108182, 0.642115, -1.090262, -0.638358, -0.241482, -1.165709, 1.789971, 0.791198, 0.169538, -0.487841, 1.565061, 1.468405, -0.868747, 1.424435},{0.546002, 0.868114, 1.966820, -0.848487, 1.266913, 0.644660, -1.488171, -0.182142, 1.128994, -0.432953, -0.222312, 1.351218, -0.461112, 2.268942, 0.981108, -0.067564, -1.357608, 0.494586, -0.435214, -1.196692, 1.091557, -0.181994, 0.587484, 1.594758, -2.210584, 0.802829, -1.775414, -0.701781, -1.550552, 1.216675, -1.729242, 1.383223, -1.441630, -0.376492, 0.294990, 0.362627, -0.191883, -0.285256, 0.293488, -0.481866, -0.953569, 0.416820, 0.893544, -0.412170, -0.864495, 0.629673, -0.384149, 1.449846, -0.104117, 1.968791, 0.316287, 0.060254, -0.629855, -0.403027, 0.935350, -0.592745, -0.226726, 1.431707, 0.181389, -0.819891, 0.180981, -1.636864, -0.460142, 0.024043, 0.240305, -0.385426, -2.133950, 0.050280, 1.167453, 0.217961, 0.808663, 0.543058, -0.307911, 1.812690, 1.768161, 0.594342, -0.117370, 0.244609},{0.004154, 0.035141, -0.259377, -0.107832, 0.901689, -0.497275, -1.299973, 0.712935, -0.112018, -0.318933, -0.318246, 0.424268, 0.721949, -1.366459, -0.450505, 0.771404, -0.158012, 0.099822, -0.082164, -0.766312, 1.024290, 0.273569, -0.053362, -0.717895, 2.494203, 0.755553, -0.357822, -0.355361, 1.325774, -0.066314, 0.432273, -1.443295, -0.695024, 0.142916, 1.658543, 1.810309, -0.188504, -0.403729, 0.351683, 0.212830, 0.060955, -0.464611, -0.009312, -0.437581, -0.277452, 0.231422, -0.746346, -0.172520, 1.545595, -0.844299, 0.604350, -0.324841, -1.844901, -1.569512, 0.375256, -0.514169, -0.055723, 1.150033, -0.407933, 0.468228, 1.175346, -0.129188, -1.344741, -1.024970, -1.118580, 0.316954, -0.806631, -0.555664, 1.279919, 0.635356, -0.308448, -0.722453, 0.972277, -0.691853, -0.915917, 0.387553, -0.932138, -0.277289},{0.138593, 0.911498, -0.638035, -0.556860, 0.152154, 0.055991, -1.467184, -1.413714, -0.423747, -1.466566, 0.983472, -2.766499, 0.641669, 0.521586, 0.457400, 0.802702, 0.051685, 0.006253, 0.097818, 1.824339, 1.342346, 2.263471, -0.878863, 2.148538, 0.185406, -0.918974, -1.607478, 0.249482, -1.659931, -0.292427, 0.504502, 1.033282, 0.427577, -0.354868, 0.869335, -0.490259, -0.207466, 0.106586, -0.464434, -0.672873, 1.062151, 0.085158, 1.827287, -0.269747, 1.719494, 0.319069, 1.277944, 0.331687, 1.350957, 2.307306, -1.126287, -0.275726, -0.846188, 1.971339, -1.050726, -0.931284, -1.093337, 1.307240, -0.376182, -0.475160, -1.083663, -2.471817, 0.107860, -1.705539, -0.783243, -0.395842, -0.103199, -0.177748, 0.080754, -1.460363, 0.168592, -0.809234, -0.413335, -1.497461, 2.201343, -0.387412, -1.565119, -0.630960},{0.639922, -0.899333, 0.840700, -0.030896, 0.976103, -0.579725, -0.659554, 1.720061, 1.562002, -0.453973, 1.866335, -1.633351, 1.472552, 1.138352, 1.524031, -0.057504, -1.404162, 0.764156, -0.545700, -0.601547, 0.352695, -1.428300, -1.413725, -0.422334, 0.277276, -0.823371, 0.072449, 0.679725, -0.438144, -1.207191, 1.319744, 1.746338, -0.802032, -0.917684, -1.080523, -0.755665, 2.094714, 0.453257, 0.665772, -0.315932, 0.608988, 0.810096, 1.105048, -0.316522, -1.499297, -1.503103, 0.371035, -0.110510, 0.677314, 0.474036, 1.343468, -0.119011, -2.209503, 1.735813, -0.068768, -1.481496, -1.722183, 2.151500, -0.423397, -0.035164, -0.357195, -0.285029, -0.362034, 0.674270, 0.916595, -0.283971, -0.687957, 0.657075, 1.218527, 0.619596, -0.807814, 1.331904, -1.455496, 2.723079, 0.349960, -0.028985, -1.220582, -0.032736},{1.335104, -0.364440, -1.545384, -0.141707, -0.748815, 1.767786, 0.055074, -0.740908, -0.403622, 0.276229, -0.130857, -0.534803, -0.482557, 0.050151, 0.013775, -0.300286, 0.280640, -1.451393, -0.224480, -1.754998, -0.154108, 1.134363, -0.146857, -1.580821, 0.458912, 0.145177, -0.163359, -1.208962, -1.982816, -1.270130, 0.972823, -0.415831, 0.557846, 1.475624, 0.085813, 1.033445, -1.227314, 0.042972, -1.073692, 0.664707, -1.092056, -0.666981, 0.063789, 0.968739, -1.183870, -0.865455, 0.065694, -0.138766, -0.917196, -0.980745, 1.205861, 1.275248, -0.219081, 0.796048, -0.173973, -0.504905, -0.777419, -0.991533, 0.123065, 0.829518, 0.960166, -1.138865, 0.912346, 0.713031, -0.656978, 0.054961, 0.154797, -0.161998, -0.865196, 0.177386, 0.696930, 0.024514, -0.421652, -0.788219, 1.584549, 0.190558, -0.610504, 0.508787},{-0.602117, 0.028268, 1.134979, 0.163776, 0.740189, -0.752611, -0.692269, 1.242053, -0.476585, 0.181074, 1.926423, 0.520055, -0.397187, 2.043748, -0.100869, 0.950656, 0.421155, -1.808522, -0.020086, -0.449577, 0.058617, -0.767691, 0.193764, -2.247738, 1.114785, -0.235197, -0.831449, 0.647958, -1.917978, -1.026358, -0.929775, 0.028869, 0.078782, 0.969178, 1.202993, -0.591248, 1.305199, -0.891223, 0.177725, -1.781399, 0.277857, 0.215670, 2.677441, -2.990913, -0.221005, -0.090521, -0.082286, 1.378349, 0.730735, -0.620663, -0.281768, -0.026399, -1.408525, -0.906150, 0.137791, -0.733237, -0.999407, 1.457009, 1.324662, -0.886317, 0.192147, -0.547517, -2.916028, 0.227540, 0.028425, -0.364534, -1.714263, -3.237822, -1.247097, 0.001763, 1.644054, 2.145973, 0.832289, 0.641309, -0.131113, -1.748546, 0.579820, -0.365617},{-0.006514, -1.699023, 1.072652, 0.624050, -2.852906, 0.758361, 0.749561, 0.523937, 1.150685, 0.724119, 0.094881, -1.765419, 1.125984, -1.309808, -0.142829, -0.095065, 0.158923, 0.167637, -0.005708, -0.042141, -0.784010, 0.291129, 0.679184, 1.238924, -1.391739, -0.988695, 2.010118, -1.941722, -1.350038, 1.476910, 1.099453, 0.449603, 0.323395, 0.498478, 0.114500, 0.085418, -1.052036, 0.687386, 1.056385, -0.757347, 0.619965, -0.326668, 0.868233, 0.177450, -0.541305, -1.331351, -1.066001, -1.159054, -2.192207, -0.686311, 0.094310, 0.220206, 1.655605, 0.189843, -1.599966, 0.793765, -0.638163, -1.320422, 0.388895, 0.097187, 0.350121, -0.569340, -1.041371, 1.396217, 0.298965, -0.625080, -0.751844, -0.383737, 0.947675, -0.209501, -0.853345, -0.309699, 0.364024, 0.763509, -0.435108, -1.724222, 1.071717, -0.081923},{-0.405212, -1.126467, 1.997064, -2.122293, 0.476104, 1.967888, -1.186228, -1.147230, 0.283249, -0.929309, 0.358074, 0.762220, -1.205565, 0.916240, 1.176307, 1.631154, 0.341225, -0.271199, 2.170889, 1.744502, -0.946272, -0.314769, -0.276265, -0.173177, 0.366816, 0.207916, 1.577421, -2.500998, -0.242522, -0.119886, -0.953221, -0.257890, -0.766562, 0.041956, 1.661291, -2.454960, 0.197806, -0.552376, 0.429432, -1.786128, -0.351112, -0.157890, 0.156031, 0.450304, 1.262487, -0.496680, -2.224540, 1.058550, -0.469931, -0.311509, -1.395437, 0.387945, -0.152049, 0.987311, 0.173556, 0.113497, 0.572716, 0.204085, -0.779715, -0.905435, 1.241841, 0.115091, 0.335915, 0.868121, -0.478783, 0.167353, 0.120322, 0.718738, 0.258395, 0.268573, 0.967929, -0.440688, -1.142572, -2.223310, 0.556334, 0.077751, 2.276434, -0.386790},{1.172550, 0.843624, 1.128073, 0.813906, -0.135626, 0.555495, 1.152705, 0.052382, 0.173482, 2.285161, -0.315951, 0.241228, -2.226379, -0.599115, 1.121064, 0.701716, 0.442782, 1.929325, 0.759062, 0.223156, -0.084948, 0.440324, -0.731577, -1.534211, 0.221983, -0.676926, 2.198015, 1.306878, -0.380798, -0.405820, 0.424336, -0.596909, -2.765270, -0.204020, 0.404990, 0.189734, -0.099539, 2.071660, 0.541785, 0.433609, -1.162758, 0.049547, -1.372166, -0.765051, -2.289110, -1.186066, -0.533481, 1.646838, 0.428218, 0.899546, -2.828454, 1.428029, 0.180879, -0.481275, 1.787619, 1.056302, 1.019994, -1.801666, 1.195356, 0.773246, -0.708636, 0.268476, -1.674111, -0.263726, -0.140066, -0.405860, -0.043597, 0.286812, -0.434864, 1.802745, 0.143031, 1.920832, -0.393755, -0.411490, -0.026510, -0.353921, 0.309228, 1.075817},{-0.198606, -1.318934, -0.239635, -1.392046, 1.785319, 0.574822, 1.171977, -2.274508, 0.162060, 1.762673, 0.493743, 1.226678, -1.547728, 0.584356, 0.062793, 0.041899, -1.568978, 1.171720, 1.165549, 0.305068, 0.455297, -0.199773, 0.052779, -0.826482, -1.263311, -0.234224, -0.459215, -0.088850, -0.064268, -0.890016, -1.360745, 1.032383, -0.630126, 0.058100, -0.573549, 0.658084, -0.263737, 0.321528, -1.348287, 2.181210, -0.770414, -0.972403, -0.988244, 0.187396, -0.119718, -1.802318, -0.014316, 0.220580, 1.220557, 0.206099, -1.318509, 1.010110, 0.771667, 0.081905, 0.117691, -0.343853, -0.163584, 0.621075, -0.487396, 0.440334, 0.153336, -0.211954, 0.109345, -0.905826, -0.797764, 1.406846, -0.652321, -0.974700, 1.149991, -1.212614, 1.341222, -1.602722, -0.217513, -1.897935, -1.761500, 1.490810, 0.926233, -0.278384},{-0.505242, 0.384156, -1.226467, 0.329066, -0.269127, 0.507854, -1.535516, 1.403775, 0.825851, -0.337458, -1.776344, -0.519208, -0.399104, 0.306289, -1.271657, 0.486847, 2.794742, -0.310444, 0.411369, 0.093679, -0.542017, -0.042105, -0.450789, -0.805661, 1.358940, 1.730718, -1.400684, 0.718629, 0.857490, -1.548419, -0.510102, -1.006024, 0.149436, -0.626173, 1.273357, -1.133125, -1.063060, -0.700686, 0.747226, 1.055981, -1.704962, 0.751124, 1.411324, -0.273857, -0.429009, 1.061933, -0.763919, 2.602259, 0.901260, -0.795075, -1.342106, -0.483915, 0.777067, -1.066901, 0.789809, -0.810991, 0.817298, -0.261471, -0.474595, -0.421654, -0.182249, 1.085921, 0.881798, 1.506646, 1.026173, -0.942345, -0.512815, 0.190260, 1.933581, -2.614702, 0.151625, -0.569960, -0.566640, -0.347761, -0.117445, 0.634465, -0.150883, 1.795954},{-1.411375, 1.180697, 0.588927, -1.085598, -0.867321, -0.811543, 2.037179, 0.283013, 1.293439, 1.275343, -1.447305, -0.495739, -1.016844, 1.109067, -0.333901, -0.112038, 0.051586, 0.235154, 0.416385, -0.073589, 1.651530, -1.304922, 0.784371, 1.569585, 1.055785, -0.150117, -0.734195, 1.017172, -0.465431, 0.794512, 0.853969, -0.492044, 0.863006, -1.242427, -0.237089, 1.481120, 1.370428, -0.102171, -2.481496, 0.553590, -0.759978, -1.307939, -0.713988, 1.217298, 0.192265, 1.528013, 2.131576, 0.054876, 0.755395, 0.378710, -1.427009, 0.568559, -1.063609, 0.205684, -1.657182, 0.430861, 1.834438, -0.358029, -1.460675, 0.133512, 1.104622, -0.203087, 0.198376, 1.510380, 1.100716, 1.367698, 1.002355, -0.126844, 2.529967, -0.087441, -0.875196, -0.302970, -0.117799, -0.126086, -1.704336, -0.698586, -0.109683, -0.778795},{0.893504, -0.989080, -0.099376, 0.324045, -1.655477, -0.997458, -1.338123, 0.103611, -1.244543, -1.498169, -1.832069, -0.635051, -0.252165, -0.065542, -1.494744, 0.556878, 1.419594, -0.273030, 1.171218, 1.437127, 0.286887, -0.227682, 1.073348, 0.005581, 0.982377, 1.319219, -0.215475, -0.351978, 0.268229, -0.070091, 0.468516, 0.465298, 0.699033, 0.183599, -0.156491, -0.677538, -0.571862, -0.467073, -0.156253, -0.701311, 0.152214, -0.098964, 2.122586, -2.015157, -1.204983, 0.945864, -0.365100, -1.225397, 2.410470, 0.543501, 0.634504, 0.046307, 0.053531, -1.412541, 0.327848, 0.103302, 0.214636, 0.772948, -0.033608, 1.095627, 0.618617, -0.164000, -0.541543, 0.931250, -1.405215, -0.156677, -1.813682, 1.166473, -0.763437, -0.637076, -0.667460, -0.313226, 0.067709, -0.122751, -2.323658, 1.127162, -0.052615, -0.246139},{-0.224904, -0.673312, 0.363870, -0.364728, 0.800849, -0.856882, -0.501066, -0.526916, 0.621799, -1.037708, -0.727569, -0.153445, 1.816698, 2.319931, -0.603053, -1.460390, 0.136987, 0.738488, -0.877534, 1.672508, 1.084462, -0.513538, -1.149117, -2.151568, 1.141553, 0.896488, 1.545928, -0.247176, 0.177432, 0.926233, -0.472431, -0.469460, 0.432211, 1.298825, 1.845969, -0.034961, 1.276798, -0.122087, 0.853226, 1.192846, 2.226785, 0.115035, 1.458464, 0.104909, -0.380600, -0.845971, -0.206303, -0.064029, 0.469331, 0.646801, -1.355123, -2.849041, -1.730074, -0.326142, 0.335282, 0.786205, 0.813201, 1.034728, -0.292546, 0.679354, -0.892504, -2.702908, 1.228957, 0.220251, 0.755560, 0.727346, 1.192565, 0.774430, 0.554078, 0.175436, -1.500443, 0.035435, -2.396910, -1.167439, 0.983888, 1.179323, -0.632763, 0.819814},{-0.965784, -1.859233, -0.484106, 0.373039, -0.581706, -0.662447, 0.293770, -0.880628, 0.506382, -0.383730, -1.130712, -0.330019, 0.683644, -0.903360, -0.011173, -1.403883, -1.253084, 0.775728, 1.049454, 0.486494, -1.082366, -0.049390, -0.078287, -1.542570, 0.443881, -0.466687, 0.544694, 2.479541, 0.928626, 0.965603, -2.039384, 1.036837, -1.296697, -0.598094, -1.434797, 0.387782, 2.152482, -1.095784, -0.398694, -0.782385, -1.794648, -0.020454, 2.038350, 0.130501, 0.816650, 0.408850, 0.739463, 1.031941, -1.448325, 0.026336, -1.085878, 0.566059, -1.307521, 0.857068, -0.726939, 0.386755, -0.756272, -0.170922, -1.118944, -0.850406, -0.948200, 0.767493, -0.417910, -0.081493, -0.821275, 0.177093, 0.136968, 0.690193, -0.881512, -0.906425, -1.267723, 0.825454, -1.173640, -0.317083, 1.788511, -0.672546, 0.467253, -0.229469},{0.506204, -0.249694, 1.352229, -0.123771, -1.047647, 0.399544, 0.000988, -0.783321, -0.740690, -0.153113, 0.447424, -0.737898, -0.220774, 0.732548, 2.020539, 0.642832, -0.985149, 0.171267, -0.728927, 1.110594, 0.567500, -0.225400, -1.200049, 0.618797, 0.529685, -0.008224, -1.273362, -0.398429, 1.091019, -0.046319, -0.608342, -1.163634, 0.638631, 2.015707, -0.750895, -0.763915, 0.431313, -0.491342, 0.313024, -0.316464, 0.797120, -0.402003, 1.322049, -1.225825, -0.623566, -0.566667, -1.116918, 0.810850, -1.658551, -0.705267, 1.005775, 0.085429, 0.088022, 0.709369, 0.581357, -0.730256, 0.602215, 2.403996, -1.125366, -0.786418, 0.505888, 0.397067, -0.119921, 0.887402, -2.907367, -0.254322, 0.592906, -0.365883, -0.658824, -0.585256, -0.091225, 0.312201, 0.565256, -0.016526, -0.995458, 0.375803, -1.030127, 1.294666},{0.129531, 0.078883, -0.666624, 2.262770, -0.111495, -0.865345, -1.013857, -0.994827, -0.037346, 0.176359, -0.760333, 0.046234, -2.700093, 0.017613, -0.265704, 0.338530, -1.206377, -1.359883, 0.784555, 0.361090, 0.990675, 0.547134, -2.046768, -0.331526, 0.357027, 1.043686, 1.161575, 0.744451, 0.110723, -0.415021, 0.960365, -2.014179, -1.307655, -0.536471, 1.802031, 0.850840, 0.416849, -0.039304, 1.112383, -1.067840, -1.042498, -0.337133, -1.386907, -1.011427, 0.041556, 0.719926, 0.145954, -0.563340, -2.094486, 0.183190, -1.665728, 0.642276, -1.894745, 0.704629, -2.002750, -0.982436, -0.902485, 1.097597, 0.340866, -1.568303, -0.163494, 0.028079, 0.087321, 3.155283, 0.674323, -0.896233, 0.336974, 0.967440, -1.486894, -0.080333, 0.432557, 0.188463, 0.784990, -0.654952, -1.408344, -1.329334, 1.517055, -0.541250},{-0.419962, -1.528336, 0.889238, -0.965733, -0.187020, -0.669697, 0.042847, 0.332766, 0.001126, -0.684654, 0.285514, 0.527609, -0.153837, 1.061453, 1.290796, -0.904871, -0.345033, 1.998873, 1.074672, -0.246085, 1.180799, 0.054963, 0.223183, 0.310405, -0.290944, 1.419013, 1.296243, -1.077359, 0.381296, 0.158646, 0.925647, 0.791407, 1.836506, 0.198258, -0.248583, -0.122086, -1.414778, 1.958145, 0.132101, 0.947304, -0.751327, -1.184881, -0.338709, 1.342178, 1.352840, 0.592152, -0.009098, -0.355939, 0.545562, 0.436000, 1.634678, -1.793489, 0.320894, -0.845379, -0.684885, 0.263603, 0.012149, -0.265153, 0.431725, 0.145510, 0.975093, 0.596967, -0.457897, 0.098349, -2.579716, 0.345721, 0.515088, -0.474021, 0.775000, 2.500803, -0.105574, 1.158066, 2.372528, -1.826315, 0.309915, 0.131630, 0.406407, 0.838970},{1.418337, 2.044315, -1.414590, -0.563094, -0.530374, 1.360129, -0.974888, 0.532302, 0.295091, -0.273358, 1.502582, 0.745340, 0.075057, -0.519926, 1.149597, -1.623610, 1.130089, -0.476816, -0.496010, -0.178373, 0.358354, 0.231577, -1.059107, -0.201045, -1.279503, -1.857778, 0.767872, 0.694211, -0.193944, 0.260788, -1.384458, -0.431078, -0.396313, -0.251735, -0.581435, -0.261619, -0.359730, 0.512708, 0.186511, 1.117174, 1.481489, -0.108924, -2.216605, 0.153366, -1.235186, -0.457729, -0.427598, -1.952196, 0.872259, -0.228646, -0.477195, 1.657710, -0.775660, -0.831079, 0.210323, 1.054012, -1.110299, -0.314441, 0.753488, -0.862359, -0.706135, 3.380528, 0.308587, -1.077678, -0.674621, 0.984735, -0.583513, 0.827745, 1.285692, 0.152227, -0.608529, -0.547055, 1.276045, 0.219274, 0.435515, 1.586124, 1.659939, -1.466726},{-1.427210, 0.358746, 1.439102, -0.179526, -0.775909, -0.550349, -0.302154, 0.604316, 0.745106, 1.583077, 1.430230, -0.470227, 2.057712, -0.750807, 1.527271, -1.461952, 1.583452, 2.303490, -1.138469, 0.404610, 0.139486, -1.461427, -0.893798, -0.099397, 0.012574, -0.564607, 0.536136, -0.626115, -0.016151, -0.907216, -1.072204, -1.198247, -0.712966, 0.408569, -1.848992, 0.425192, -1.165326, -1.369864, -0.328635, 0.336644, 0.516824, -0.470443, 0.549734, -0.334694, -0.329184, 2.262688, -0.226382, 0.688311, 1.056118, -1.072605, -1.644585, -1.964914, -1.582137, 0.358443, 1.233520, -1.010704, 1.632768, -2.247619, -0.710399, -1.945596, 0.621880, 1.068229, -0.278340, -2.567718, -2.249074, -0.015267, -0.869037, 1.399485, -0.488496, 0.496672, -0.411207, -0.756508, -0.085004, 0.741955, 1.132691, -1.439021, -0.533655, 1.672501},{-1.111763, -1.063531, 0.288433, -0.877929, -0.473622, -1.643914, 0.712501, 0.240044, -0.533606, -0.199924, -0.007699, -0.147511, -1.671985, -1.444819, 0.033262, -1.627170, 0.362526, -1.011305, -1.588463, 0.819078, -0.655283, -0.149177, -2.329167, 1.200231, 2.796586, -0.191437, -0.219679, -1.327930, -1.110452, 0.013750, -0.993620, -0.469836, -0.104159, -0.118273, -1.272797, 0.694316, 0.851889, 0.836134, -0.002447, 1.837898, -1.112973, -0.923406, 0.211725, 0.573439, 1.137252, -0.266188, -0.659982, -0.890646, 1.623392, -0.557687, 0.374059, 0.437055, 0.903331, -1.219082, 2.588973, -0.759860, 0.088016, -0.915552, 1.201806, 0.929621, 0.551746, 0.679430, 0.218439, -0.464255, -1.376356, 0.606930, 0.579954, -0.453916, -0.297447, -0.386748, 0.414957, 0.916439, 1.512810, -0.119291, -1.081911, -1.012120, -0.545640, 0.364567},{1.608445, -0.251819, -0.042740, -2.810702, -0.952596, 0.505542, -0.326240, -1.900524, -1.000077, -1.122848, -1.048340, -2.128827, 2.337319, -0.387406, 0.998669, 0.946320, -1.486848, -0.672446, 0.186071, 0.415355, 1.217314, -0.676316, 0.158346, 1.102377, -0.796827, 0.347191, 1.327965, 0.930120, -0.256331, 1.916404, -1.379638, -0.860901, 0.471384, -0.605244, 0.873473, 0.552637, -0.142000, 0.347183, -0.661269, 0.361903, -0.000030, -0.741654, -0.189448, 0.376282, 0.158173, 1.532568, -0.767797, 0.225949, 0.138551, 0.531196, -1.838420, -0.490477, 1.155421, 0.955600, 0.170938, 1.217969, 0.091090, 0.260973, 1.076285, 0.059230, 0.942888, -2.289493, -0.096291, -0.264020, -1.186481, 0.394102, 1.000837, 0.184932, 0.482583, 0.010352, -0.680069, -0.924442, 0.106414, -1.786223, 1.099710, 0.557602, 1.005483, 0.451929},{0.711733, -0.632575, -1.123389, -0.442116, -0.814716, -0.753508, 2.093029, 0.482687, 1.075876, -0.609820, -1.169573, 0.453742, -0.483406, -2.031268, 1.177184, -1.682881, 0.488039, 0.689825, -1.081401, 2.027946, 0.960973, 0.062657, -0.640799, 0.169448, -1.038867, 0.698242, -0.883943, 2.234341, 1.671302, -0.514020, -1.764271, -0.167139, 2.423580, 1.301726, 0.490066, 0.484226, -0.343616, -1.263175, 0.077393, 0.652288, 0.083476, 0.945755, -0.729546, 0.888828, -0.117927, 0.765302, 1.599859, -0.380343, 1.040931, 1.291106, -0.654884, -0.180050, -0.543147, 1.474465, 1.314798, 0.460039, -0.658548, 0.229604, -0.167465, 0.182570, 0.933963, -0.582205, -0.903285, 0.436503, -0.180118, 1.673528, -0.270056, 1.488384, 0.203151, 0.234757, 0.712669, -1.312249, -0.146856, -1.475996, 0.471296, 0.293563, 1.245443, 0.308082},{-1.037798, 1.428510, -0.645865, 1.634945, 0.809963, 0.569799, 0.044742, -2.439845, 1.375722, 1.916498, -1.884394, 0.200348, -0.962275, 1.275788, -0.123887, 0.226196, 2.059375, 1.669427, 0.597926, -1.157630, 0.468957, -0.809663, 0.383656, 0.763748, -1.033349, 0.677599, 1.690920, 0.278654, -0.514332, -0.656819, -1.266968, 0.328832, -0.634742, 0.622501, 0.501719, 0.297151, 0.972136, -1.043288, -0.637035, 1.441429, -0.326528, -0.250692, -1.350072, 1.503898, -1.060501, 1.429536, -0.217505, 0.524023, 1.144194, 0.597628, -0.393895, 1.513705, -0.008430, 0.438123, -1.028654, 0.769571, -1.992189, 0.774726, -0.245025, -0.008533, 0.458071, 1.293196, -0.015123, 0.141943, 0.851887, -0.397219, 1.077667, 0.403437, -0.355150, 1.679908, 1.364200, -0.956018, 1.101424, 0.429805, -0.028819, -0.403310, 0.509899, -0.288260},{0.612845, -0.844913, 1.934817, 0.356531, -0.657013, -0.455876, 0.616551, 0.627106, -1.180724, 0.221455, 1.649723, 0.047098, -0.043083, -1.246745, -0.399233, 0.496665, -0.749230, 0.472931, 0.523418, -0.562026, 1.191211, 0.080859, 0.771914, 0.896683, -1.249756, 1.459650, -1.038922, -1.051183, -2.621901, -0.508585, -1.087839, -0.611142, -0.319368, 1.255246, 0.742905, 0.399982, -1.240169, -0.343434, 2.239831, -0.550041, 0.136451, -1.202939, -0.295818, -1.004388, 0.362513, 0.604993, -0.320051, 1.163778, -0.407942, 1.124255, 0.268113, -0.532011, 1.163397, -0.251041, -0.845174, 0.355134, 0.116883, 0.037561, -0.199542, 0.993198, 0.800769, -1.244576, 0.840364, -0.315660, 0.540655, 0.514202, -1.162919, -0.454659, 0.998431, 0.696109, 0.815989, -1.409458, 0.748694, -1.834270, -0.332894, 1.043785, 0.436644, -0.129408},{-1.292783, -0.935154, -0.182260, 0.851937, 0.605223, 0.814934, 0.669757, -1.061818, -1.567136, -0.114161, -1.420819, -1.420050, -0.275706, -0.671701, 1.228100, 2.217811, -1.556153, 0.302372, 0.812515, -0.144643, -0.236906, -0.864243, -1.508380, -0.093194, 0.155206, 1.122748, 0.557740, 0.030581, 1.229409, -0.617596, 0.276441, 0.713475, -0.497766, -0.508898, 1.400392, -1.485809, 0.162875, -1.419119, -0.699694, 1.245607, -0.185040, 1.341091, -0.281002, -0.065534, 0.501471, 0.744861, -1.122771, -2.700167, -0.620752, -0.431599, 0.149450, 0.847539, -1.096290, -1.676775, 1.828238, -0.792459, -0.364761, 1.386907, -0.117763, 0.790639, 0.860627, -1.209404, -0.083895, 1.331740, -0.141526, -0.349511, 0.103684, -1.274650, -0.899751, 1.557526, 0.959370, 1.631548, 1.098242, -0.221326, 1.263415, 0.900573, 0.244269, 0.294558},{-0.595963, 0.073271, -0.328266, 2.129700, 0.550212, 0.735660, -1.114464, 0.566098, 0.557075, 0.194687, 0.752673, -1.371818, 1.653964, -0.055305, 0.255611, -0.263967, -0.129013, 0.415553, 0.090153, 0.740904, -0.846500, 0.928710, 0.579586, 0.009638, -0.597086, 0.556603, -0.626688, -0.586265, -0.564894, 0.848489, -0.383683, -0.692522, 1.985009, 1.047818, -0.469950, 0.849021, 1.179732, -0.525583, -0.670681, -0.404270, -0.998547, -0.709248, -0.201665, 0.755706, 1.034093, -0.505083, 1.245395, -0.935517, 1.497485, -0.464198, 0.553851, 1.135513, 0.675108, -1.348598, 0.504350, 0.986192, 0.237777, -0.457318, 0.218839, 0.611569, 0.388628, 1.426298, 1.003115, 0.859945, -0.442952, 0.440351, 0.352583, 0.579278, -0.342568, 1.211217, 0.138965, -0.544827, 2.047301, -0.477808, -1.669719, -0.733609, 0.023214, -0.348944},{1.090525, -0.052992, -0.186663, 0.537570, 1.052164, 2.077123, 0.728424, 0.401738, 0.947004, 1.673879, -2.229424, 1.115328, -0.741727, 0.096979, 0.658373, 1.326173, 1.099630, -0.344179, -1.246997, 0.443876, 0.549990, -1.188717, 0.725010, -1.342256, 0.192408, -0.484849, 0.652376, -0.665052, 0.902945, -1.104705, 0.828512, 0.293725, 0.240326, -0.001227, -0.087364, 0.250874, -1.782852, 0.212760, -1.541519, 0.406320, -1.126239, -1.706683, 0.055340, -2.198730, 0.293874, -1.185995, -0.604900, -1.314205, -1.016899, 1.012758, 0.466790, 0.414858, 0.579738, 0.003893, -0.645078, -0.367957, 0.312656, -1.194000, -1.184046, 1.709509, -0.153409, -0.807611, 0.669674, 1.247188, -0.302667, 0.286438, 1.077028, -1.461872, -2.214592, -1.595547, -0.372515, -0.078667, 0.117996, 0.745955, 0.022023, 1.776600, 2.075691, -1.239329},{-0.621966, 0.343852, -0.613778, 0.443887, 0.103856, 1.662374, 0.115008, -0.015834, 0.317011, 1.598386, 0.576740, 0.428567, 1.142429, -1.067475, 1.789126, 2.034928, 1.143972, 0.141133, 0.326208, 1.177553, 0.084093, -0.490566, 0.637622, 0.550053, 1.050894, 0.788501, 1.362380, -0.759438, 0.204513, -1.227773, 0.144954, -1.045869, 0.467201, 0.226066, 0.168051, -1.171991, -1.033272, -0.699190, -1.836673, -0.050537, 0.278838, -1.031842, 2.235409, 0.085981, -2.043921, -0.015525, -1.297214, -0.179580, -0.944641, -1.588637, -0.273171, -1.391136, 0.538604, 0.191871, 1.138906, -0.293477, 0.164868, 1.539231, -0.817394, -0.455557, -1.170808, 2.206409, 0.905840, -2.275134, -0.904655, 2.306774, 0.800195, 0.910171, -0.679656, -1.042547, -0.549632, -0.130137, 1.091432, -0.809431, -1.955793, -0.029006, -1.295047, -0.361909},{-1.072144, 0.051709, -0.868281, 1.118636, -1.631893, -0.538810, -0.304589, 0.642650, 1.664188, 0.942551, 0.167856, -0.608336, 0.369206, 1.338267, 0.172820, 0.145512, -0.694539, -0.312800, 0.249353, 0.128936, 0.675002, 0.622154, -0.861948, 0.391353, -1.607161, -2.086334, -0.671064, 0.316619, -0.763339, 0.397345, 1.450042, -0.406825, -0.368938, -0.988208, 0.075034, 0.716442, 0.805300, -0.169212, -0.940775, 1.414508, -0.541988, 0.949740, 0.419674, -0.252862, 0.718474, -0.447713, 0.072572, -0.274092, -2.067705, 0.299678, 1.542434, -0.911319, 0.398287, 0.712020, 0.552996, 0.719880, 0.883262, -0.031586, 0.409321, 0.698951, -1.050533, 0.957543, 0.544902, -1.724258, 0.814953, 0.698204, -0.792632, -0.920732, 0.976431, 0.853004, -0.524457, -1.404260, -1.807481, -1.033076, -0.232487, 0.958572, 0.833770, -0.625968},{-1.111995, -0.909690, 0.197797, 0.133707, 1.035584, -1.494923, -0.162511, 0.744323, 0.542260, -1.608261, 0.166713, -0.599888, -1.045729, -0.130430, -1.066046, 0.788083, 1.211280, -2.431139, -0.553281, -0.665983, -0.220823, 1.900058, 0.349707, -0.731987, 0.729466, 0.723842, 1.102921, -0.566942, -1.077773, -0.215007, -1.860914, 2.399885, 0.551481, -1.219343, -1.699602, -1.827960, 0.062493, -0.144331, -0.779744, 0.670164, -0.966507, -1.772247, 0.324654, -1.143126, -0.746268, 0.544051, -0.388540, 0.607973, -1.868506, 1.390090, 1.231062, -2.486953, -1.019338, 0.118926, -0.471670, -1.542625, -1.185408, 0.266240, -1.374076, 0.291312, -0.315949, -1.024459, 0.248048, 0.137566, -1.640419, 1.598936, 0.735377, 0.516861, -0.736484, -0.126964, 0.436632, -0.073555, -0.270188, -0.438936, 1.255427, -0.530032, -1.066386, -0.144284},{1.456663, 1.218109, -0.685008, -0.697971, -0.428639, -1.436329, -2.071180, -1.075263, 0.865502, 1.408601, 0.629202, -0.409350, -1.873287, -0.354202, -0.461288, 0.978521, 0.001616, -0.293767, 0.216417, -0.126329, -2.141425, 1.274208, 0.499786, 1.658792, 0.864617, -0.320072, -1.028475, 0.490367, -1.024085, -1.674341, 0.527230, 1.858474, 0.799081, -0.317240, 0.844206, -1.053290, -0.650637, -0.280307, 0.249327, 0.569596, 1.373659, 0.604053, -0.125373, 0.822868, -0.890797, -0.744473, -1.093536, 0.492560, 2.222987, 1.675479, -1.048337, 0.428595, 0.316329, -0.037248, -0.619566, 1.136759, 1.181486, 0.368161, 0.152667, 0.838959, -0.696446, 2.934292, 0.165969, 0.520454, 2.567893, -0.377537, -0.386909, 0.588254, -1.101573, 1.205669, 0.830668, -0.434901, -0.973351, 1.821114, 1.577409, -0.638726, -1.952931, 0.454609},{1.858340, 0.919751, 1.332957, 0.827134, 0.559214, 1.166720, -0.184201, -0.397882, 0.532105, 1.708509, -1.073109, -0.054655, 0.075091, -2.059457, -0.462205, 0.124637, -0.658965, 0.078014, 1.552180, 0.210164, 0.835394, -0.464741, -0.015958, 0.700919, 0.983664, -0.042050, -0.024632, 1.720302, 0.411558, -1.805377, -0.637456, -0.308516, 0.965721, 0.903518, 2.043952, -0.469600, -0.122415, 0.668008, 0.570309, -1.467020, -0.031728, 0.054179, 0.802204, -0.210260, -2.124946, 1.378189, -1.211586, 0.990794, 0.936683, -0.678160, -0.380460, -0.041405, 0.721138, 0.943602, -0.735085, -1.167191, 0.329084, 1.958676, -0.628150, -1.771166, -0.872672, -1.635771, 1.077322, -0.850741, -0.490072, -0.731142, 0.180100, -1.458761, -1.209948, -0.398078, 1.959289, -0.323718, 1.653963, 0.502313, 0.877554, 0.418593, -0.700021, 0.151783},{1.951739, 0.275983, -0.725564, 0.960885, 0.428223, -2.468101, 0.644404, 2.503083, -0.354479, 0.702045, 1.285687, -0.426395, -1.216459, 0.317796, 1.420485, -0.274220, 0.071547, -1.615969, -0.845778, -1.124947, -0.996220, -0.683473, -1.333302, 1.581089, -0.072758, -0.177583, 0.675508, -0.597915, -2.690151, -0.600725, -0.882030, 1.835042, 0.114402, 1.095117, 0.065888, -0.022872, 1.275701, -0.557201, 1.297782, -1.478051, -0.759232, -0.637389, 0.201151, -1.517608, -0.834251, 0.513178, 1.202135, -1.762550, -0.306129, 0.805291, 0.494919, -0.934808, 0.307546, -1.043558, 0.130025, 0.944956, 0.466706, -0.686053, -1.492299, -0.086511, 1.156327, -0.113063, -0.696126, 0.146081, -0.102093, -0.088535, 0.419421, 0.266369, -1.030436, 0.918396, 0.860708, -0.375511, 0.637686, -0.103744, -0.002036, -0.498098, -1.633847, 0.776012},{1.143114, 0.890981, 0.422178, -0.091356, 2.254549, 0.467017, 0.009374, -0.291243, -0.613481, -0.763981, -0.594182, 1.309198, -0.721415, 0.695761, 1.321531, -2.014548, 0.601561, 0.351642, -0.596512, 0.275435, -0.638018, -0.905467, -0.404036, 0.640629, 0.155797, -0.729306, 1.630888, -2.461981, -0.181171, -0.380454, 0.495214, -1.482911, 0.621927, 1.016973, 0.598851, -0.776830, 1.154800, 0.689867, 2.494056, 0.087997, -1.234327, -1.009471, -0.217843, -0.438795, 1.722433, 0.898773, 0.099559, -2.148551, -1.307626, 0.608111, 0.540423, -0.729014, -0.260700, 0.134219, -0.395977, 1.189286, 2.022649, -0.907111, 1.059892, -1.343143, -0.908637, -0.082873, 0.484076, 1.201861, 0.670280, 0.676211, -0.674294, -0.871163, 0.113361, -1.261027, 0.485464, 0.764546, -1.354476, 2.034636, 0.377366, 0.098590, -0.387881, 0.362522},{0.256909, 0.814148, 1.955180, 0.910129, -1.211439, 0.519376, 0.598771, -0.737010, -2.037024, 0.892270, 0.723416, 0.670829, -0.039231, -0.312054, -0.165849, 0.568744, -0.654527, -0.539097, -0.651681, 0.627968, -1.749353, 0.382337, 0.860806, -1.935483, -1.068905, -0.634011, 2.841521, -0.176386, -0.721505, 0.349968, 0.026153, 1.033452, 0.235948, 0.408732, -0.955403, -0.556356, -0.531932, 0.615101, -0.378663, 0.361909, 0.778603, -1.324209, -1.383835, -0.484117, -2.337136, -0.373497, 0.066254, 1.097984, 0.299330, 0.113104, 0.200799, -0.121216, -0.584234, -0.166951, 0.963560, -1.325046, 2.504130, 0.804686, -0.358018, 0.588768, 0.381190, -0.405466, -0.319094, 1.922012, -0.823175, 0.494864, 0.928795, -0.562693, -1.386652, 1.053185, 0.221911, 0.762111, 1.606759, -0.136834, -0.521410, -0.850639, 0.323793, 0.157466},{-1.668618, -0.005538, 0.017170, -0.408831, 0.167673, 0.413202, -0.480238, -0.932592, -0.693725, -0.794786, -0.051587, 0.347002, 0.905464, 1.228118, 2.536478, 1.059591, -1.120122, 0.346630, -0.428097, 1.887087, 0.241264, -1.602467, -2.356618, -1.353105, -0.176465, -1.120235, -0.878043, 0.969068, -2.013535, -1.069798, -0.146361, 0.569222, -1.086883, -0.840260, 0.347018, -1.266845, -0.243028, 0.111701, 0.872397, 0.360347, 0.883357, -0.332980, -1.150843, 0.749319, -0.837746, -0.120754, -0.043928, 0.316512, 1.172124, -0.386067, -0.149032, -0.923807, -1.667536, -1.011731, 1.016091, 0.806141, -1.831270, 0.936501, 0.340344, -0.424181, -0.808820, 0.443785, 0.386519, 1.140685, -0.251606, 1.157217, -0.171083, -0.735443, -0.463338, -0.033336, -0.274092, 0.324955, -0.942366, 0.024729, 0.440180, -0.515225, -0.668296, -0.841617},{1.918459, 1.518658, -0.578393, 0.498609, -0.132280, -0.097237, -0.562605, 0.060273, -1.518989, 1.518089, 0.121089, 0.096328, 0.276587, -1.047388, -1.418768, 0.268430, -0.905508, -0.910932, 0.346743, -0.795624, 1.608102, 0.697534, -0.379055, -0.629848, 0.908787, 1.215132, 1.823354, 0.449836, -0.457268, 0.361734, -2.880162, 1.431617, -0.434485, 0.298110, 0.681700, 2.199925, -0.434502, -0.074865, -0.474319, 0.371033, 0.254412, 0.246359, 2.100063, 1.301185, 1.225233, -0.108645, 0.834635, -0.413096, 0.700295, -0.473490, 0.801444, 0.462648, -1.587132, 0.870563, 1.368590, -2.419179, 0.186966, -0.800579, 0.147477, -1.381440, -0.177490, 0.700336, -1.583153, 0.312661, 0.759588, 1.396144, 0.071640, -0.006267, -0.527873, -2.620107, -2.061377, -0.432301, 1.583887, -1.821125, -0.981315, 1.406560, 0.512070, 1.859050},{0.293645, -0.112738, -0.604236, 1.386999, 0.085042, -0.656252, -1.186600, 0.578155, 0.695558, 1.402539, -0.533820, 0.869614, -1.947714, -0.753307, -2.133509, 0.340871, 0.472142, -0.693833, 0.589966, -1.404343, -0.255034, 0.146587, 0.641693, -2.614999, -0.626060, 1.191292, -0.144155, 1.232250, -1.173147, 1.597712, -0.158677, 1.692181, 0.740664, -0.389971, 1.332954, 1.501541, 0.617534, 0.661819, -0.255363, 0.834535, -0.630785, 0.420447, -1.569840, 0.624887, 0.536918, 2.085716, -2.538342, -1.140835, -0.202542, 0.098469, 0.714156, 0.406334, -1.275270, 1.114089, 0.858587, -0.110064, 0.357022, -0.835350, -0.913152, 0.203786, 1.210344, 0.817877, -0.032106, -0.094185, 0.513889, -0.499728, -0.349970, 1.111000, 0.226688, 0.625333, -0.044221, 2.027345, 0.671291, 0.438688, -0.949105, 2.757244, -1.631427, 0.520210},{-1.993357, -0.059427, 0.286832, 1.826406, 0.044419, 0.569032, -1.826524, 0.285825, 0.196861, -0.474084, -0.181858, 1.144959, -0.000992, 0.620662, -0.561808, 0.167305, 0.189405, -0.771710, 0.159257, -3.480247, -0.212203, -2.255263, 0.564675, 0.822626, 0.242263, -0.300698, 0.291024, -0.513502, -0.847012, 0.018240, 0.567849, 0.168613, -0.098784, 0.295908, -1.381053, 0.643083, -0.407640, -1.048008, -1.187876, 1.688347, 0.779305, -0.979082, 0.204555, 0.175796, -0.618486, -0.772411, -0.912162, 0.467786, 0.464279, 1.314642, 0.001687, -0.195879, 1.609525, -0.401964, -0.608080, -0.318751, 0.274229, 0.451241, -0.779478, 1.116278, 0.479447, -2.471637, -0.266616, -0.491622, -1.096288, 0.675303, 0.339098, 0.193115, -0.519387, -0.258058, -1.341588, -1.660078, -1.451092, -0.677381, -0.371788, -1.023897, 0.858584, 0.580608},{-0.515055, 0.716094, 0.227795, 0.060324, 0.099613, -1.498441, 1.721670, 0.647787, -0.031931, -0.699444, 1.745283, 0.750560, -0.923394, 0.338633, -1.285730, 0.453215, -0.654734, 0.848618, 0.527258, 0.586372, 2.655387, -0.579335, -0.427854, 0.236137, 0.887392, -0.696792, 0.272879, 0.041070, -0.487127, -0.931957, 1.622962, 0.391172, -0.627744, 0.596194, -1.624375, -1.621825, -0.333203, -0.583732, -0.241813, 0.309277, 1.149642, 0.678470, -0.599589, 3.174994, 0.827104, -0.976373, -0.347157, 0.297253, 0.050798, -0.308538, -0.482035, 0.771500, -0.188353, -0.395008, -0.335607, 0.621075, 0.918930, 0.823844, 2.067628, -1.809713, 0.822817, -0.376431, -1.006150, 0.641802, 0.218995, -0.815500, -0.473651, -0.160987, 0.395325, -0.468628, -2.411596, 0.046757, -1.178173, 0.529573, -0.428791, 0.397874, -0.601444, -0.920663},{0.157201, -1.299160, 0.451010, -0.752523, 0.367441, -1.503965, -0.340974, 0.087453, 0.405603, 0.221448, 3.260089, 1.127894, -1.041592, 0.417648, 1.203183, 0.327788, -0.968806, -1.061093, -0.164195, 0.309602, 0.268021, 1.543147, 1.008222, 0.280443, 0.558066, 0.679920, -0.190525, -0.210004, -0.691604, 0.596872, 0.675652, 0.279175, 0.348354, 0.391055, 0.266749, 1.987186, 0.766760, -1.573845, 0.706285, -1.814788, -0.852966, -0.913514, -0.639726, 0.747974, -1.305911, -0.243504, -1.577723, -0.800255, 1.480935, -1.137134, 0.665559, -0.343766, 0.724318, 1.473491, 0.850989, 1.247986, 0.628870, 1.224386, -0.431515, 0.462191, 2.186532, -0.277254, -1.220456, 1.270127, 0.550131, -0.546862, -0.519001, -0.651793, -0.737015, -0.696176, -0.453045, -0.604589, 0.930323, 0.550047, -0.329064, -0.667464, -1.390882, 0.442512},{-0.842422, 1.164833, 0.318006, -1.572837, -0.371934, 0.476104, -1.226178, -0.477055, 0.914722, -0.327526, -1.607851, 0.687372, -0.093475, 0.335857, 0.030243, 0.404861, -0.926729, -1.279775, -0.142603, -0.306730, 0.732390, 0.254046, -0.687411, -0.667720, -0.496105, 0.889519, -0.416458, 0.674040, -0.766011, -0.253059, -0.629833, -0.476514, -0.506669, 0.215122, 0.241300, 1.586336, -0.094268, 1.605412, -0.548283, 0.352557, 1.963339, 0.611991, -0.073432, 0.272298, -1.057492, -0.690311, 1.329454, -0.207486, -0.278364, 3.384429, 2.481205, -1.058145, 1.080715, 0.238209, 0.341360, 1.199594, 0.838394, 0.073731, -0.098451, 0.514324, -0.123899, -0.528143, 0.587290, -0.446334, -0.214693, 2.053724, 1.785255, 0.079469, -1.301123, -1.283621, -2.224416, 2.229799, -0.816236, 0.122080, -1.876908, 1.656893, -2.441421, 1.059591},{-1.096330, 0.956136, -0.380915, -0.416602, -0.401707, 0.166838, -0.738759, -0.668471, 0.910398, 0.047003, -0.502190, -0.143358, 1.442042, 1.173124, 0.474100, -1.041202, -0.772026, 0.154791, -0.124455, 0.224464, 1.011816, 0.476954, 0.339248, -1.498695, -0.185301, -1.836026, -0.797021, -0.044854, 0.446128, -0.504219, -0.423211, 0.323109, 1.580398, -0.458915, -0.516292, -2.153188, -1.041331, -1.252344, -0.012578, -1.053235, 1.035190, -0.631860, 1.413007, -0.420083, -2.139161, -0.316045, -0.999508, 0.629118, -0.006343, 0.485953, -1.521213, 0.250027, 0.515195, 0.643866, 1.074533, 1.225672, 0.109877, -0.341067, -1.107567, 0.169544, -1.830365, 0.168399, 0.561803, -0.518699, -0.045806, 1.065471, 0.773745, -0.036111, -0.080582, 0.817036, 0.309745, 0.393683, -0.783748, 1.382864, -0.812378, -0.743813, -0.229529, -0.416591},{0.081246, 1.677179, 0.173500, -1.172163, 0.529798, 1.673815, -0.194437, -0.271777, -0.438591, -0.822364, -0.509363, -0.397242, -0.087788, 0.015934, -0.488020, -0.607586, -0.348812, -1.679472, -1.666410, -0.247996, -0.917113, -2.052190, -1.027187, -0.105531, 1.519640, -0.161280, -0.258684, 0.096296, -0.457243, 0.203194, 0.153861, -0.244711, -1.205002, -0.696824, 0.206377, 0.035040, -0.810179, -0.971998, -0.612476, 0.383041, 0.126815, 0.204568, 1.538547, -1.283477, 1.072155, 0.592029, -0.198152, 1.110984, -0.140243, 0.982709, 1.188742, -0.538993, -0.415568, 0.249867, 0.678143, 0.008935, -0.203518, -0.888907, 1.220183, 1.083296, -0.227092, -1.015566, -0.463204, 0.332999, 1.185722, 0.382263, 0.253206, 0.756628, 1.878454, -1.079354, 0.003761, -0.682326, 0.563729, 1.348062, 0.378706, 0.028622, -1.150987, 0.738038},{-2.012628, 2.692379, 0.077714, -1.511741, 0.674317, -0.931569, 1.181293, -1.088910, -1.550621, -0.057807, 0.628388, 1.051963, -0.343837, -0.298404, -0.618633, -0.773183, -0.852326, 0.138963, -0.558911, 1.753196, 1.193700, -0.425821, 1.018136, -0.058184, -0.066893, -0.774721, 1.292547, -0.258378, -1.367512, -0.762644, 0.434433, -0.123527, -0.594732, -0.143392, -1.433184, 1.486577, 0.588033, -0.242962, -2.274801, -1.095538, -1.662907, -0.265503, 1.997650, -0.061586, -0.939463, 0.707225, 0.870321, 0.365524, -1.167394, 0.156977, -0.750078, -1.717725, -1.353150, -0.595710, 0.382924, 0.398647, 0.942556, -0.033480, -1.590605, 0.682033, 0.285302, 0.230181, -0.146448, 1.032212, 0.043459, -0.161419, -2.048786, -0.748095, -0.208504, -1.604226, 2.065361, -0.943793, -1.387770, 0.933508, -1.831857, -0.043054, 1.225885, 0.347585},{-0.647994, -1.384173, -1.824510, 1.337602, 1.868276, 1.290771, -0.680997, 1.884456, 1.045860, 0.281564, -0.524443, 1.267989, 0.610955, 0.839328, 1.085431, 1.020314, 0.191179, 0.886142, -0.518218, 0.334733, -0.047408, -0.663281, -0.968858, 0.538115, 0.977861, -1.681358, -0.159607, -1.393361, 2.013648, -0.516308, 1.494341, 0.082059, 0.325314, 0.748045, -0.502340, 0.068121, 1.363782, -0.057635, 0.900917, -0.516442, 0.696156, -0.089847, 0.017613, 0.356909, -0.531335, -0.218482, -0.652461, -0.784248, 1.863903, -0.395855, -1.779374, 0.366788, 0.347921, 1.446565, 1.018127, 0.092187, 0.236688, -1.278673, 0.488921, -0.532482, -0.233557, 0.148045, -1.684997, -1.633481, -0.193525, 0.894841, 1.338685, 1.685759, -0.435607, 0.613143, -1.531026, -0.746764, 0.822286, 1.178443, 1.789972, -0.786148, -0.451448, 0.180723},{-0.184610, 0.333962, -0.826651, -0.209538, -1.839624, 0.398704, -0.460329, 0.822934, -0.766876, 1.156212, -0.194419, 1.663680, 1.710971, -0.381127, 0.185280, -0.813804, 0.018462, -0.457006, -0.910313, -0.479113, -0.393318, 0.400667, 1.328594, 1.057205, 0.117361, -0.584788, 0.184215, -0.036766, 0.804908, -1.882059, -0.348044, 0.125348, 0.517862, -0.571466, -0.180940, 1.778042, -2.292267, -0.367007, 1.810345, -0.440096, -1.452662, 0.826825, -0.412183, -0.375718, 0.397336, -0.423606, 0.671599, -0.134528, -0.208237, -0.998458, -1.432967, 0.892011, -0.268141, 0.379168, -1.528572, 1.703711, 0.344398, 0.700668, 0.459206, -2.621828, 0.040696, 0.624553, -1.496348, -0.870177, -2.002800, -0.264376, -0.422557, 2.498763, 0.331576, -0.013821, 0.380302, 0.033375, -1.009162, 0.310583, -0.853410, -0.772561, -1.245630, -0.562396},{0.118149, 0.218007, -0.594894, -0.061796, -0.005760, 0.403423, 1.641795, 1.734589, 0.566179, -0.979267, -1.823425, 0.678949, -0.516901, -0.116659, -1.623626, -0.334226, 0.945396, -1.388478, 0.450731, -0.036265, 0.572522, -0.499505, -0.223043, 0.663288, 0.553662, 0.706470, -1.323530, -0.084701, -0.251167, -0.415096, 0.798265, 1.615444, 0.204431, -0.598280, -0.780680, -1.707953, 0.129592, -0.892350, -0.675107, 0.615655, 1.003709, 0.976813, 0.210046, 0.373047, 0.575754, 0.591005, 0.285549, -1.193765, -1.488956, 0.536425, 1.595146, 0.074386, -1.623362, -0.027340, -1.955475, 2.142146, 0.935727, -0.762143, -0.066346, 1.217355, 0.960209, -0.188577, 0.608980, -0.558767, 1.756839, 1.063542, -0.642347, -1.159990, 0.260179, -1.625820, -0.033459, 1.252133, 0.858569, 0.924530, -2.548651, 1.061767, -0.036475, 0.258437},{-0.427116, 0.409589, 1.207891, -0.816869, -1.283343, 0.245523, -0.680696, -1.428097, 0.057078, 0.514153, -0.668374, 0.807495, -1.119149, -1.281969, 0.102850, 0.705053, 0.548236, -0.336853, -0.744345, -1.705202, -0.619693, 0.924411, -0.672191, 0.173265, -0.290628, -0.848256, 0.141409, 1.502637, -1.560286, -1.054065, -1.224234, -0.411040, -1.091325, -0.299023, -0.212911, -0.175668, -1.218926, 1.033194, 0.660468, -0.479545, 0.649607, -0.651173, 1.034805, 0.362277, -0.930719, -0.663650, 0.269960, 1.086356, 1.314339, 0.466196, -2.586544, 0.344633, 1.593567, 0.413448, -0.758073, -1.201099, 0.607679, -0.268359, -1.633081, 0.159746, -0.331373, -1.138898, -0.234998, -0.169248, -1.610671, 0.589449, 1.719372, 0.832542, -0.980684, -0.829901, 0.683520, -1.908695, 1.334582, -0.037509, 0.531304, 0.045441, -0.103172, -1.657894},{-0.780590, -0.567871, -0.815088, -0.934798, -0.915869, 1.794328, 1.994012, 2.774467, -1.377043, 1.794077, -0.938475, 0.055802, 0.267441, -0.750445, 0.435878, 1.770736, 0.429580, 0.149485, 0.633418, -0.343186, -0.622445, 1.290958, -1.386032, -0.143596, -0.411004, -0.250046, 0.510458, -0.988271, 0.825963, 0.280803, 1.001005, -0.497578, -1.340920, 0.046499, 0.184766, -0.064115, 1.406197, 0.693558, -0.343894, 0.059328, -0.421016, 0.216040, -1.685327, -0.262912, -1.252725, -0.272990, 0.057813, 0.865521, 1.046172, -0.861886, -0.638568, -0.734343, 0.079653, 0.000423, -0.592564, -0.577146, 1.584877, 2.949658, 0.071307, 0.307844, -0.823225, 0.102432, 0.909724, -0.118285, 0.137165, 1.212609, -0.066585, -0.614881, -1.352505, 0.891308, 1.550379, 0.191044, 0.244784, 0.868576, -1.629115, 0.111780, 0.516675, -0.316671},{2.674337, 1.118533, 2.198195, 1.666275, 0.029589, 1.748086, -2.614532, -1.746509, -1.387370, -1.229016, 0.714656, 0.095151, 1.132989, 0.454190, 0.941176, -0.104368, -1.228039, -0.847230, 1.131907, 0.489692, 0.875945, -0.910327, -1.587311, -0.301732, -1.038382, 0.215258, -0.171556, 0.355576, 0.013898, -0.308450, -0.084445, 0.968193, 1.319121, 0.519991, 1.346633, -0.430599, -1.367519, 0.312246, 0.991863, 0.296482, -1.216091, 2.159523, -1.403423, -0.081229, -2.174160, -2.441926, -1.689712, -1.828641, -0.198564, -0.970853, -0.007040, -0.795729, 0.235621, -0.120018, 0.546682, -0.189581, 1.699602, 0.516900, -0.744622, 1.197496, 1.480682, -0.694405, 0.380523, -0.424733, 0.436143, 2.166490, 0.555719, -0.337283, -0.697853, 0.017266, 0.054730, 1.270509, 2.064713, -0.869329, 0.792839, -0.021407, -0.312481, 0.189146},{-2.737085, 0.927786, -0.889377, -0.037668, 0.279715, 0.290618, -0.864419, 0.556998, -2.200706, 1.157820, -0.643410, 1.557100, 0.956914, 2.645213, -0.694983, -0.242237, -0.121950, 1.398289, -1.369528, -2.210748, 0.819412, 1.264291, 0.411449, 0.683979, -1.016744, 0.189510, 0.928797, -0.366373, -1.450503, -0.778416, -2.683348, -2.244198, 0.301350, -0.940377, -0.016301, -0.395421, -0.963043, -0.551303, 0.268570, 0.021573, 1.457220, -0.930259, 1.524640, 0.742812, -0.312258, -0.802855, 0.244730, -1.789245, 0.268966, 0.741933, -1.558979, -1.240026, 0.664454, -0.147410, -0.803294, 1.538828, -0.186052, -0.228327, 2.505657, 1.117292, -1.135953, 0.504591, -0.371155, -0.986103, 0.319264, -0.344493, -1.314338, 0.500613, -2.167210, -1.123894, 1.226859, 0.098323, 0.768502, -1.222535, 0.674690, 0.525941, -1.283531, -1.047430},{-1.088975, -1.011045, -0.496605, 0.601273, 0.774784, -0.259027, 0.016243, -0.039466, -0.200388, 1.483254, 0.367085, -1.118668, -0.336318, 0.712974, 0.471690, 0.684992, -0.384062, 0.624371, 1.221300, -1.846897, 0.350119, -1.084253, -1.519954, 1.207282, -0.460123, 0.792508, -1.354342, 1.036083, 1.763896, 1.792594, -0.269266, 0.634590, 0.766136, 0.848354, 0.729329, -1.220630, 1.469716, 0.793974, 0.056169, 0.878346, 0.824418, 0.563695, -2.645332, -0.999282, 0.521441, 1.005913, -0.140331, 0.594816, 0.850520, -0.433817, 0.289456, 1.054289, 0.494759, 0.774813, -0.210488, 0.355422, -0.602572, 0.822208, 0.405476, 1.420541, 1.512119, -0.835606, -1.382700, -2.093113, 0.720379, -0.620114, -0.714527, -1.454517, -0.921845, -1.661158, 0.347579, -0.413518, -0.814923, -1.373563, 0.061028, 0.581321, -1.237788, 0.172634},{-0.050332, 0.827555, 0.116868, 1.152277, -0.300119, 0.585483, 0.692087, 0.208487, 1.199725, 1.029996, -0.626003, -0.939490, -1.384470, 0.078853, 1.076391, -1.936981, -0.045074, 0.376854, -1.710135, 2.139882, -1.078898, 0.225222, 0.116430, -0.077723, -0.920289, -2.969239, -1.028758, 0.123681, -0.338804, 0.499262, 2.056094, -0.589147, -0.717109, 0.517996, -0.913208, 0.168114, 0.594076, 1.685977, 0.729341, -0.245713, 0.813593, -0.481323, -1.428027, -0.717565, 0.896801, -0.305214, 0.406297, -1.296328, 0.057301, 0.081809, 0.931622, 0.043375, 0.668974, 0.395483, -0.499803, -0.958516, -1.211571, 0.514740, 0.778470, 0.281162, -1.782161, 0.257162, 1.124513, -0.165104, -0.202809, 1.613452, 0.426388, -0.333121, -1.069898, -0.676947, 1.270684, -0.544322, -1.302827, 0.600056, 1.038479, 0.463148, -1.407098, 0.051409},{0.037045, -1.247959, -1.739879, -1.225199, 0.910052, 1.483009, -0.529202, 0.051984, 0.810471, -0.685715, -0.562438, -0.451118, -0.214022, -0.347717, 0.008845, -0.311464, -0.201378, 0.742822, -0.079533, -0.195555, 0.637110, 0.856253, -1.080042, -1.012189, 0.250695, 1.935781, -0.162805, 0.984530, 0.622389, -0.549110, 0.306090, -0.997544, 0.367399, 0.636518, -0.365050, -0.173224, 0.548486, 0.757841, 0.302859, -0.631394, 0.837418, 0.844998, -1.306764, -0.706762, 0.351298, -0.366145, -0.524737, 1.196538, -1.513119, 0.079988, 2.570621, -0.258017, -1.360839, 0.822635, -0.169620, -1.333174, -0.311655, -0.177956, 0.717057, 0.324793, 0.164577, 0.208132, 0.047742, 1.587526, -0.268850, 0.669530, 0.798837, 0.072677, -1.286419, -0.395956, 0.950056, -1.125932, -1.032468, -0.123239, 2.214608, 1.942772, -1.854009, -0.011410},{1.362027, 0.167253, -1.234695, -0.475793, -0.603625, -0.744905, 0.640232, 0.514691, -0.285015, 2.226092, -1.726504, -0.821627, 1.708524, 1.384609, 0.889709, 0.421812, -0.887989, 1.850169, 0.092007, -2.010412, -1.283444, 0.230860, -0.735558, 0.204794, 0.630587, -0.458763, -0.161616, -0.916157, 0.253039, 0.085047, -0.642888, -0.261066, 2.195163, 0.791743, -0.417525, -0.748101, 1.827705, 1.162358, -0.383196, 0.721346, -0.586274, -0.542434, 0.978298, -0.732253, 0.142590, 1.884388, -0.762722, 1.646181, -0.876251, 3.338030, -1.105136, -0.471528, -1.121426, 0.392603, -2.031954, -1.631690, 0.465141, 1.785473, 0.087411, -0.788504, 0.187629, -1.243191, -1.362635, -1.670830, -0.069653, -1.128017, -0.885289, -0.422513, 0.273876, -1.233085, 0.446093, 0.497927, -0.162738, 0.707336, -0.418727, -0.998246, 0.040872, -0.030568},{-0.183922, -0.214437, -0.380931, 0.736018, -0.455389, 1.839137, -0.891762, 0.024500, -1.416484, 0.566503, -0.397860, -1.214269, -1.506695, 1.210039, -1.385408, 1.550759, 0.986120, 0.186644, -0.714115, -2.094389, -1.521121, 0.515643, 0.444096, 0.052105, -1.587937, -0.833343, 1.331213, 0.495646, 1.412761, -1.116964, 0.138461, 0.387054, 0.313957, 0.184647, 1.515934, 1.824502, 0.788242, 0.388329, 1.706108, 0.563631, -1.350392, 0.763576, -0.073742, 0.458441, 0.890991, 1.972723, -0.739791, 0.868524, 0.373193, 1.177504, -0.735790, -1.875860, 0.281281, 0.649043, -0.589531, -1.260971, 1.683594, -0.204608, 0.970770, -2.492841, -1.884661, 0.993229, -1.910771, 1.567840, -0.190709, -0.288859, -0.813955, -0.177113, -0.579046, 0.787359, -1.803397, 0.812240, 1.877977, -2.526760, -0.045379, 0.476944, -0.598433, -2.324880},{-2.315003, -1.443620, -0.790544, -0.023115, -0.945669, -0.637479, -0.512495, -0.118634, -0.641111, -0.160937, -0.910971, 0.604719, 1.650107, -0.052472, -0.563831, -0.581798, -0.009684, -1.317088, 1.394739, -2.463219, -0.378260, 0.632275, 0.554626, 1.692344, -0.146960, 0.550363, 0.378192, 0.697057, -0.137756, 0.441022, -0.694390, 0.434347, 2.197914, -0.908223, -0.749672, 0.440223, -1.746074, -0.009109, -0.209967, -0.668822, -0.724029, -0.728000, 0.241270, -1.005450, -0.945325, 0.475063, -0.495816, -0.943324, 0.461413, -0.278756, 0.315814, -0.805831, -2.047561, 1.231365, -1.502515, -0.578225, 1.038213, -0.284874, -1.648974, -1.922305, -1.315483, -1.799864, 0.477974, 0.942957, 2.327549, -0.563038, 0.145411, 0.482517, -0.430544, -1.393508, 0.581990, -0.375206, -1.693760, -0.026261, -1.661412, 1.933493, -1.211772, 0.148658},{-0.524828, 0.744950, 0.756511, -0.219382, 0.111209, 0.730373, -1.184134, -0.338109, 0.056229, -0.951665, 0.685043, 0.048675, -0.846335, 0.816516, -0.726644, 1.180193, 0.077549, 0.649134, 1.474637, -1.038251, 0.056313, -1.047747, -0.746784, 1.239680, 0.700880, 1.781409, 0.642043, 1.007998, -0.931928, -0.275115, 0.538846, -0.010004, 1.152070, -0.805838, 1.146262, -0.523347, -1.417098, -0.390095, -0.638902, -0.095858, -1.441689, -1.389490, -1.077280, 0.697022, 0.229173, 1.836506, 0.568171, -0.587620, -0.679882, -1.444897, -0.898045, 0.935542, 1.816785, 0.783710, -0.174182, -0.429137, 0.639260, 0.452450, 0.193294, -0.412748, -0.995833, 0.676312, 1.616766, 0.815384, 0.647399, -0.665611, -1.033666, 0.506791, -1.435862, 0.484346, -0.821708, -0.061324, -0.931507, -0.243114, -1.529979, 0.594175, 0.518715, 0.075656},{0.220871, 0.909944, 0.227743, -0.578568, 0.897696, -0.739226, -1.315140, -0.564932, 0.653835, 2.305558, -1.464103, -0.563038, 0.730322, -0.866798, -0.467545, -0.519701, -0.823176, 1.013098, 1.408435, 0.444907, 0.574584, -1.827240, 1.742799, -0.225262, 0.627544, -0.950496, 1.154688, 0.130359, 1.125427, 0.202984, -0.712873, 0.291212, -0.415008, -0.260238, -0.282332, -1.661202, -0.905414, 0.124212, -1.116624, 2.281932, -1.619451, -0.102792, -0.829127, -0.246432, -0.120468, 0.348592, -1.620272, 0.125402, 0.284305, -0.749042, -0.183798, -1.212788, -0.101447, 0.416475, -1.192220, -2.486891, -0.831790, -1.320510, -0.080153, 0.352895, -0.072472, 1.700914, -1.410965, -0.829539, 0.941191, 1.081916, 1.429483, 0.090944, 0.322460, -1.527200, 0.919709, 1.670147, -1.261960, 0.653615, 0.240329, 0.729081, -0.498590, 1.250372},{-0.744764, 0.521079, 1.863613, -0.030726, 0.053538, 0.355711, -0.206045, -0.811615, 2.027106, -1.012526, 0.523364, -1.700069, 0.321357, -1.293282, 0.125755, -0.422910, -0.992411, -1.639471, 0.268208, -0.518557, 1.901047, 0.169785, 1.966954, -1.523839, 1.238357, -2.431184, -0.829504, 1.103738, -0.574972, -0.014298, 1.234041, -1.114457, -0.437592, 1.463040, 2.162562, 0.989291, 0.483274, 0.933091, 0.115592, -0.136898, 1.579787, -2.362884, 0.378220, -0.496660, -1.146795, -0.666622, 0.116705, 0.022293, -1.105890, -1.364449, -0.115580, -0.378983, 1.359306, -0.481560, 0.186789, 0.547333, -1.741939, -0.154939, 0.568193, -0.638890, 1.158115, -2.593153, -0.687771, 0.348617, 0.396211, 1.021437, -0.936317, 2.331085, -0.780051, 1.532974, 0.490948, 1.525740, -0.521028, -1.128944, -0.696486, 1.258327, -1.557245, -0.464586},{0.432893, 0.349034, -0.087033, 0.764320, -0.931057, 1.320054, 0.313936, -0.484390, 1.959120, 0.606846, -0.133844, -0.385299, -0.497067, 0.474741, 1.164816, -0.243565, -1.261212, -1.116733, -0.201738, -1.021923, -0.856559, 0.874188, 0.968027, 0.040305, -1.203866, -1.213052, 0.070360, 0.078159, -0.925096, -0.262067, -0.524454, 0.137863, 0.339487, -0.589980, 0.929990, -0.523726, -2.418362, -0.239503, 2.945355, -1.305132, 1.200062, -0.413145, -0.751647, -1.334811, 0.840350, -0.716906, -0.972604, -0.965220, 2.163945, 0.520048, 0.203668, 1.151640, -1.155480, 0.849241, -0.068962, 1.126293, 0.419815, 1.002868, 0.517375, 0.159696, 0.489237, 0.250405, -0.460899, 0.292805, -1.212919, 1.646093, -2.423642, 0.354390, -1.078911, 0.362131, 0.137935, -0.418721, 0.560524, 0.510370, 0.923781, -1.343437, -0.785086, 0.790961},{-0.289581, 0.692027, -0.974500, -0.884711, -0.567719, 1.141412, -1.292680, -0.100046, 1.467802, 1.226633, -1.251990, 0.465703, 0.993442, 1.193931, 1.922923, 0.419771, -1.839813, -2.378613, -1.543782, 0.134793, 0.526105, -1.007192, -0.130727, -0.110045, 1.304733, -1.441899, -1.199705, 0.421866, 1.258526, -0.996164, 0.914525, -0.041284, -1.043901, 0.347497, -2.274936, 1.121230, -0.328408, -0.559104, -0.038932, -1.204482, -0.026521, 0.561115, -0.693851, 2.268648, -1.285467, 1.446013, -0.322129, 0.547783, -1.060860, 1.419567, 0.804479, -0.064608, -0.757992, -1.805728, 1.744399, -0.983774, -0.627604, -1.888373, -0.826334, -0.765597, 0.667916, -1.536810, 0.469910, 0.556716, -1.331435, 0.348185, -1.612409, -0.268703, -0.425691, 2.002933, 0.048001, -0.883242, 0.934376, 0.290821, -1.050618, 0.054483, 0.212838, 0.671599},{-1.027713, -1.874010, -1.997493, 1.184759, 0.293465, 0.770264, -1.824755, -2.098405, -0.812621, 1.031349, -0.743467, 0.881268, -1.943445, 0.146885, 0.572199, 0.837902, -1.553265, -1.720352, -0.908294, -0.134197, -0.192528, -1.159448, 0.178983, -1.402266, 0.008684, -0.071151, 0.313185, 0.854679, -1.434983, -0.350886, -2.044984, 0.031709, -1.036169, 0.710706, -1.193827, -0.187486, -1.269492, 0.453191, 0.924194, 0.517784, -0.141910, 0.093487, -0.900229, 0.983402, 0.540010, 0.119637, 0.152210, 0.220130, -1.991472, -0.588839, 1.154282, 1.612511, -1.294491, -0.101471, 0.271435, -0.283736, 0.727204, 1.354859, 0.916766, 0.522354, 0.184156, -0.120169, -0.716094, -0.434065, 0.402913, 0.132111, -0.873746, 1.501688, -0.438255, -0.085253, -0.935737, -0.724314, 1.129109, 1.364007, 1.175788, -0.501410, -0.012333, -0.788540},{1.161587, -1.226146, 1.623454, 0.261658, -0.316579, -0.105433, -0.327601, 1.638583, 0.204123, -0.026623, 0.059669, -0.957371, 0.571843, 0.024214, 1.155037, 0.440497, 0.498606, 2.365395, 1.205479, -0.554663, 0.944691, 0.334874, -0.306103, -0.948738, 1.212300, -2.421088, 0.599530, 0.099861, 0.032103, 0.162828, 1.020222, -0.534459, -0.404186, 0.011713, 0.543293, 0.209065, 0.572467, 0.690062, 0.364014, 1.132101, 2.394846, -0.838049, -0.087330, 0.449285, 1.376636, 2.056954, -0.417462, -1.764580, 1.408363, 0.336107, 1.574671, 0.515610, -0.543202, 0.251117, 0.386591, -1.586356, -1.781983, -0.759447, 0.230564, 0.216904, 0.418455, -2.698866, 0.060318, -0.699313, -0.513330, 0.470119, -0.470605, 0.870909, 0.983216, 0.951919, -0.768404, -0.362835, -0.884061, -1.746094, -0.442304, 1.294920, -0.885779, 0.897978},{0.097524, 2.376037, -1.742865, 1.706109, -0.195618, -1.788939, 0.514234, -1.356725, 0.425386, 0.160999, 0.726644, -0.652535, 0.320071, 0.284250, -1.316922, -0.647904, -1.323614, -0.512232, 2.297206, -0.908262, -0.237345, 1.128497, -1.888318, 0.083217, 0.159480, -2.045380, -0.272307, -0.324210, -0.196771, -0.763483, -0.864350, 0.658251, 0.396740, -0.357837, 0.998903, -0.872674, 1.037157, 0.345532, 0.723774, -0.099398, -0.252789, -0.709029, -1.389768, -1.543700, -1.229925, -0.733462, 0.519125, 0.187727, -0.237452, 0.748578, 2.206936, -0.660608, 1.903959, 1.092974, 0.220317, -1.723719, 0.122155, 1.396689, 0.679090, -2.049590, -0.204440, -1.159360, -0.901395, -0.775903, -0.567582, 0.795469, 0.642155, 1.469694, -0.020312, -1.240050, 0.734971, -0.203523, 0.099917, -1.471396, -0.295928, 0.170596, 0.279805, 1.018254},{-1.015301, 1.133881, -0.106133, -2.111360, 0.689904, 0.744373, -0.547269, -1.826386, -1.013968, -0.161581, 0.034062, 1.382118, 1.137416, -0.776119, -0.736496, -0.834204, 0.917838, -0.709680, -1.294852, 0.887697, 0.815060, -0.216544, -0.371408, -0.073441, 1.094233, 0.040174, -0.130963, -0.668190, -0.822928, -0.999820, -0.632552, 1.212345, -0.311593, 0.625314, -0.717103, -0.983377, 0.066014, 0.094546, 0.440923, -0.039741, 0.451023, -0.113373, 0.086467, 1.754413, 2.078755, 0.314567, 1.573309, -0.816586, -0.700492, 1.732736, 0.208345, 0.896718, 0.709204, -0.327146, 0.498058, -2.153168, 1.439587, 0.024070, -0.098487, 0.504976, -0.080710, -0.848910, 0.706971, 0.125069, -1.236650, 0.256998, 0.713625, 1.578389, -0.144633, 1.277156, 0.470828, 1.241025, -0.043226, -0.082853, 1.203143, 0.646914, -0.508443, -0.633580},{-0.998090, -0.553063, -2.144139, -1.028027, 2.687314, 0.062655, 1.854759, 0.314133, -0.068997, -1.823505, -0.631899, -0.156228, 0.022782, -0.285849, 0.828020, 1.197102, -0.791856, 0.076889, -0.718106, -0.265087, -0.776722, 1.318726, -1.356714, -1.113399, -0.408480, 0.255372, 0.404979, -1.297569, -0.909795, -0.119603, -1.731921, 0.676897, 0.675384, -0.416440, -0.294256, 0.967449, 0.823438, 1.383988, -0.475253, 0.503461, 1.470096, -0.634577, -0.076465, 0.424789, -1.451063, -1.194559, -1.991276, -0.223257, -0.069278, -0.780252, -0.580841, -0.590764, -0.260769, -1.184068, 0.864433, -1.275713, -0.087008, 1.179682, 1.452721, -1.319799, -1.921197, 0.770265, 0.437511, -1.048842, 0.118832, 0.448544, 0.572459, 0.103250, 1.341512, -0.443975, 0.553179, 0.474879, -0.627846, 1.234660, -0.064649, -1.282296, 0.874530, 1.378899},{-1.742359, 1.535038, -0.442259, -1.504656, 0.655489, -0.520440, -0.693746, 0.079085, 1.122673, 0.919966, -0.674264, -0.236804, 0.558504, 1.963808, 0.435050, -0.670175, -0.538110, -0.830063, -0.060761, 0.760978, 0.390946, 1.219161, -0.424528, -0.084010, 0.523521, -0.418452, -0.798873, -0.794274, -0.189640, 0.258539, 0.300741, -0.142826, 0.750868, 1.971829, -1.003226, 1.686840, -1.232322, -0.575944, -0.025286, 0.542657, 0.911567, 1.054280, 0.807677, -0.328353, -1.034757, -0.688406, -0.075955, 1.598699, 1.222839, -0.335725, -0.110187, -0.163652, -0.059324, 0.528002, -0.694863, -0.869956, -1.729333, 0.634928, 1.113410, -0.537958, -1.542658, -0.018414, 0.266630, 1.242546, -0.388389, -0.144048, -0.080922, 1.563501, 1.181562, 0.136126, 0.993578, 0.626368, 0.644634, -0.126821, 0.470669, -0.393187, -1.174573, -0.690749},{0.966142, -0.229580, -0.136002, 0.157147, -0.620641, -2.544183, 1.670398, 2.477331, 0.420872, 0.271783, -0.356407, -1.008436, -0.577046, -0.318733, 0.706348, -0.202370, -0.140534, -2.667588, -1.119107, -0.266599, -0.705086, 1.258719, 0.372355, 0.197863, -1.062725, -0.660341, -0.683028, -1.852658, -1.423212, -1.155321, -1.586801, -1.120801, -0.291886, -0.258628, -0.292103, 0.714249, -0.095933, -1.031770, 0.528949, -1.284266, 0.760281, 0.435211, -1.535443, -1.362975, 1.113858, 0.790537, 1.259384, 1.050990, -0.001756, 0.231028, -0.012537, 1.519236, -0.688601, 0.792620, -1.007000, 1.788419, -0.351951, -1.183319, -0.485172, -0.569955, 1.436915, 1.203472, -0.633119, 0.187710, -0.060910, -0.325714, -0.264509, 0.474631, -0.494976, 0.335654, -1.225661, -0.267990, -0.722022, -0.137092, 0.066914, -1.440141, 1.006624, -1.181928},{-0.208327, -0.772000, -0.330733, 0.149244, 0.931474, 0.335230, -0.596561, -0.477247, 1.527868, -0.778500, 1.315642, 0.871466, 0.241488, 0.517277, -1.315685, -0.624452, -0.775124, 0.195574, -0.275068, 2.147057, -1.656126, 1.158872, -0.494776, -0.943675, 0.781597, 0.474503, 1.118255, 1.854186, -1.589409, -1.458924, 0.073017, 0.911258, 0.564947, -0.626835, -0.780735, 0.311773, -0.258310, -0.344637, 0.528926, 0.222211, -1.492694, -0.526763, -2.059938, -1.893831, 0.240768, -1.653444, 0.110330, 1.749620, 0.100446, -1.346400, 0.191099, -1.607154, -0.422965, 1.312048, -0.667781, -1.457460, 1.331954, -0.001451, 0.001107, -1.919400, 0.270440, -0.764839, -0.122461, -0.446964, 0.498057, 0.170108, -1.705646, 0.054348, 1.540101, -1.261288, -0.113566, -0.094541, 0.647025, -0.578387, -0.512946, -0.920101, -0.046266, -0.742068},{-0.058696, 1.465456, 0.572314, -0.520873, 2.232288, 0.264910, -1.332248, 0.598988, 0.693252, 0.465470, 0.250499, -1.098350, -0.578918, -0.106624, -1.057793, 0.404898, -0.491037, -0.552317, -0.708496, -0.599892, 1.186182, 0.247138, 1.558095, 0.262119, -0.704498, 0.263135, -0.605720, -0.414025, -1.695074, -0.801142, -2.273886, -0.068679, 1.652733, 1.939065, 0.069131, 1.509446, -0.931148, -1.271765, 0.585795, -1.064553, -0.566338, -0.202051, 1.456572, -0.739515, 1.209261, -0.489356, 0.565503, -0.861991, -0.838095, 1.115310, 0.890823, -0.066377, -0.563702, 1.482508, 0.373641, 0.439215, -0.598271, 0.354685, -0.647570, 0.333692, -1.225946, 0.281819, 1.306215, -1.127567, -0.004565, -0.478196, 0.669807, -0.184957, -0.573881, -0.390426, -0.701014, 0.895564, -1.232036, 0.927419, 0.264998, 0.075390, -0.569025, 0.682824},{-0.761985, 1.112975, -1.438960, 1.508480, -1.272515, -0.378194, 0.903279, -0.825942, -0.452187, -1.926759, -0.210095, 0.616127, -0.264129, -1.068549, -0.361444, -0.355443, -0.014225, -0.348004, -0.102680, 0.497541, 0.687605, 0.306572, 0.412274, 1.240788, -0.659334, -0.494390, -0.234320, 0.246139, 0.310769, 0.265827, -0.879074, 0.425939, 1.590982, -1.288502, 2.012415, -0.408447, -0.098931, 0.193146, 1.053758, 0.732371, -0.344729, 0.027895, -0.702347, -0.511798, -0.194693, 1.054924, 0.819257, -1.003004, -2.112696, -1.114302, -0.530508, 1.429844, -0.439856, 0.231835, -0.385309, -0.569329, -1.273987, 0.069358, 1.144161, -0.449095, -0.926004, -0.172277, 0.793114, -0.581609, 0.947504, -0.863417, 1.108823, 3.797630, 0.571727, 1.166440, -0.140244, 0.611693, -1.026283, -1.418190, -2.166291, 0.162628, -1.177688, 0.996084},{-1.081033, 0.713943, 0.256485, -0.987346, -0.566068, 0.490690, -0.750644, 0.616770, -0.006839, -0.131336, 0.344124, -0.192209, -1.703080, -0.128792, 0.376899, 1.589000, -0.574965, 1.667246, -0.302265, 0.014034, -0.177276, 0.201724, 0.497215, -0.223714, -1.216358, 0.941642, 0.790385, 0.696222, 0.017878, 1.101988, 0.520440, -0.245962, 0.435307, -0.379967, -0.917874, -0.076832, -0.952330, 0.109978, 0.263203, 0.623982, -0.579687, -0.518802, 0.689246, 1.404008, -0.248685, -0.275784, -0.387358, 1.125330, 0.155992, -1.600864, 0.405375, -2.468584, 0.829942, -0.277980, 1.377661, 0.700867, -0.433481, -0.197725, 0.020440, 0.074019, -0.159543, -0.339494, 0.762901, 0.701961, 0.461915, -0.635812, 0.112958, 0.675714, -1.633373, 1.520211, -2.180816, -0.217565, -1.384169, -0.532257, -0.959100, 1.140837, 0.528107, -0.123665},{0.300176, -1.684167, -0.628211, 0.325545, 1.182064, 0.389370, 0.240437, 1.648255, -1.783219, 0.333448, -0.293465, -0.679599, -1.757119, 1.144898, -0.726285, -0.175659, 0.967337, 0.796782, -1.123082, 0.794671, 1.006505, 1.400878, 0.881512, 0.799725, -0.524989, 0.677002, 1.884460, 0.247987, -0.379332, -0.063513, 1.018402, -0.866160, 1.850398, 0.083135, 0.647432, 0.828813, -0.038783, 0.186557, 0.194796, -2.013440, -0.564200, 0.411406, 0.414846, 0.483148, 0.326582, 1.684672, 0.416743, -1.009754, -0.665926, 0.062902, -0.363006, 1.968360, 0.380271, 1.508973, -0.194860, 1.150105, -1.219570, 1.765225, 1.883453, 1.893849, 1.932514, -0.489765, 0.701711, -0.770640, 1.104182, -0.072854, -0.185572, 0.358057, 1.225018, 1.391009, -0.338468, 1.520411, 3.014806, 1.493979, 1.846776, 1.918957, -0.735512, 0.132238},{0.724699, -1.764047, 0.257403, 0.259022, -1.709260, -0.760144, 0.426486, 0.577797, 1.387810, -1.639227, 0.275119, -0.324860, 0.892196, 0.974753, -0.140674, -1.184824, -2.376254, -0.048815, 0.575763, 1.892833, 0.898995, -0.818906, -0.670023, 1.691716, 0.659507, -1.467857, -0.312408, -1.723089, -0.812457, 0.599220, 0.026962, -0.939497, 0.069649, 0.043080, -0.818598, 0.235459, 1.206869, -0.070340, -0.421251, 0.954297, -0.093254, 0.882123, -0.353829, 0.800803, -0.426331, -1.563110, 0.692943, -0.171123, 0.367255, 1.917967, -0.956386, -1.319388, -0.619908, -0.372517, 0.361510, -0.304122, -1.526728, -1.206156, -0.138787, -0.307613, -0.236840, -0.539575, -0.043922, -0.134073, -0.142039, 0.359930, 1.917192, -0.031614, 0.101732, 1.332402, 0.766148, -1.418138, -0.512292, 0.822057, 0.090074, 0.770028, -0.089602, 0.234318},{0.793861, -0.170302, 1.412693, 0.593794, -0.653408, -0.436246, 0.970782, -0.792518, 0.689475, -0.806897, 1.226681, 1.224011, -0.346802, -0.394405, -0.412805, 1.602498, 0.456751, -2.513475, -0.911990, 0.468973, -0.017673, 1.704948, 0.677518, 0.168318, -0.651762, -2.388460, 0.903640, -0.840724, 0.405553, -0.023886, -0.886081, -0.626698, -0.253099, -0.348849, -0.252531, 0.930171, -0.752179, -1.445234, 0.305393, -0.795371, 2.372237, -0.860273, 0.608570, -0.229427, 0.310686, -0.210614, -0.033622, -0.531033, -0.717292, 1.912526, -1.694089, -0.278105, -1.355909, -0.109083, 1.201307, -0.718992, 0.133965, -0.676941, -2.222989, 0.851814, 0.377048, 0.584780, -0.287489, 1.819337, 0.937260, 0.240770, -0.749665, 1.025117, -0.458349, -0.229459, -0.273392, -1.192573, -0.649304, 1.223022, -0.147363, 0.460432, -0.785342, 0.049850},{0.478543, 0.420678, -1.602864, 0.353526, -0.780397, -1.101861, 1.723586, -0.661115, 1.605035, 0.504808, 0.412076, -0.760947, -1.922165, -0.504588, -0.286234, -1.416829, 0.290069, 1.524786, -0.018179, 1.651756, -0.623312, -0.335778, -0.031612, -0.934135, 1.097892, 0.436659, -0.422473, -0.441115, 0.943858, -0.841541, 2.382590, 1.066998, 0.177265, 1.391873, -0.068105, -0.804773, -0.339700, -2.182756, -1.443065, 0.521239, 0.145120, 0.534474, -2.544418, -0.822905, -0.265709, -0.429635, -1.091988, -0.999276, -0.047513, -1.352275, 0.015856, 1.584040, -0.058693, 1.710926, -1.742288, -1.184328, 0.622640, 0.020308, 1.467279, 0.181917, -0.709508, -3.306637, -0.110441, -0.061769, -1.023711, 0.423445, 1.676337, -0.147909, -0.090875, -1.350396, -0.693753, -0.512665, 0.830483, 0.199449, -2.098192, 0.785214, -0.074683, -0.074253},{-1.763815, -0.900283, 0.511364, -0.913277, -0.608845, -0.001396, -0.202427, 1.050652, -0.906114, -1.431965, -0.394876, 1.426170, 0.319217, -0.767654, -0.141367, 0.580236, 0.760077, -0.555799, -0.980919, -0.215919, 1.111907, 1.062744, 0.138061, -0.713315, 0.691544, 1.058208, 0.794176, 0.481127, -0.863202, 0.355168, -0.381264, -0.316989, 0.862415, -1.036838, 0.434565, 0.679891, 0.660093, 0.740806, -0.479820, 0.984256, 0.584938, 0.722292, 0.282725, -0.733020, 0.288778, -0.837489, -1.053465, 1.366865, -0.706160, -0.022710, 0.527818, -0.757392, 1.833785, -0.634519, -0.120398, -1.142599, -0.465188, -0.337906, 2.616819, -0.114143, 0.207161, 0.471171, -1.131807, -0.024596, 0.146360, 0.620114, -2.393801, 1.247588, -1.689375, -1.107542, -0.069688, -0.567691, 1.445411, 0.223480, -0.407280, 0.538470, -1.173294, -0.592709},{0.652142, 0.293632, 0.662598, -0.898506, -0.702750, -1.201874, 0.749516, 1.045107, 0.495433, 1.092854, 1.099528, 0.425662, -1.706257, -0.390420, -0.819788, 0.140413, -0.560509, 1.027377, -0.553012, 0.194231, 0.103439, -0.268177, -1.008279, 0.363830, 0.959318, 1.211270, 0.183734, 0.255038, 0.023869, 0.515955, 0.238993, 1.720511, -1.320542, -1.671059, 0.760096, 1.709445, 0.784123, 1.318591, 1.200732, -1.371413, 0.986739, -1.205945, 0.013308, -1.217844, 1.099792, -1.350612, 0.857044, 0.395957, 0.454392, 2.241640, -0.620922, 0.782489, -0.489136, 0.044228, 0.558684, 0.595752, 0.201764, 0.702034, 0.435078, -0.073111, 0.922450, -0.237149, 2.163251, 0.525967, 0.300901, -1.595558, 1.551060, 0.975302, 0.231924, -0.979827, -1.320093, 1.786182, -1.500894, 0.679526, -0.633453, 0.641790, -0.150948, 0.943409},{-1.102917, -1.597130, -0.161821, 0.565568, 0.570652, 0.071726, 0.422954, -0.584332, -0.440032, -2.449249, 1.323676, 0.587172, -0.850823, -0.839806, 0.251631, 1.058829, 1.390243, -1.361855, -0.622221, 0.972196, -0.553282, 0.138464, 0.125879, 1.764014, 0.184904, -2.500574, 1.841893, 1.520705, 0.211614, 0.490986, -0.367031, 0.357796, 0.037195, -1.294474, -2.090620, 0.985694, -1.200346, 0.176663, -0.771346, 0.813921, -1.117738, -1.210581, 1.952165, -1.609843, 1.479706, -1.247596, -0.677477, 0.631049, 1.081973, 0.075962, -0.019640, -2.440531, -0.059121, 0.251427, 1.396385, -0.997205, 0.695372, -0.007415, 0.327627, -1.237389, 0.019544, -0.903588, -1.223512, -0.334759, -1.576240, 0.059707, -1.315201, -0.458073, 0.370121, 0.037478, -0.179513, 1.915709, -1.545015, -0.404284, 0.250098, 0.552064, 0.106469, -0.796254},{0.630486, -0.384739, 0.918471, -0.256949, -0.643018, -1.232218, -0.088148, -0.510151, -0.317814, -1.361574, -0.006313, -0.221925, -1.598727, 0.685592, -0.880755, -0.640019, -0.739584, -0.562733, -1.799177, -2.121539, 1.404091, 0.078191, 0.102904, -0.857298, 0.438174, 0.121997, 0.426545, -1.602075, 0.703364, -0.066247, 0.688810, -0.124066, -0.287952, -0.242657, 0.968861, -1.245896, 1.301750, 0.252778, -1.773173, 0.074735, -0.758349, -0.546184, 0.188222, -0.478594, -0.780201, 0.780499, -2.309712, 0.170602, 0.058133, 0.558656, -0.469309, 1.014021, -0.169845, 0.987565, 0.248980, 1.287882, 0.461879, 2.049304, -0.837665, -2.084827, 1.713647, 0.905981, -0.663202, -0.512022, -0.362167, -0.072758, -2.030860, -0.169294, -1.598520, -1.031175, 0.084487, 0.282050, -1.346413, 2.340269, -0.551492, -1.824710, -1.158091, 1.257975},{-0.762252, 1.177910, 0.486774, 0.180779, -2.318443, 1.034001, 0.114219, 0.831316, -2.202777, 1.850460, 0.068404, 1.064531, -1.129928, -0.742223, -0.130711, 0.972557, -0.168660, 0.284311, 1.708465, 0.372717, -0.349286, 0.626803, 0.103062, 2.989658, 1.590806, -0.363005, 1.158472, 0.990924, 1.712852, 1.792173, -2.661909, -1.598852, -1.148429, -1.028610, -2.151853, 1.365208, -2.149880, 0.288246, -1.691699, -0.524116, 0.499997, -0.944436, -0.384435, -1.559222, -0.681727, -0.610635, 0.472968, -2.052727, -0.732572, -1.495164, -0.295168, -0.262108, 1.111038, 1.207479, 0.053663, -0.255048, 0.009866, 0.626400, -0.512269, -1.144190, 0.035595, -1.695420, -0.393414, -0.337128, -0.878945, -0.623176, -0.412317, 0.041414, 1.511553, -0.655993, 1.151583, 0.101274, 0.188318, -1.532685, 0.522200, 1.997806, -0.281004, 1.119234},{-0.442671, 1.759496, -1.581115, 0.401503, -0.374856, -0.413671, 1.725135, -0.778347, -1.526014, -1.152270, -0.697111, -0.466836, 0.713129, -0.866566, 0.887062, -0.118614, 1.013481, -0.295204, 1.044261, 0.969647, 1.814813, 0.806585, 0.061754, -0.188773, -1.045876, -0.012723, 0.358647, -1.231645, -2.608810, -0.353233, -0.013600, -0.934461, 1.120433, -0.109166, 0.171721, -0.850923, -1.029791, 0.431091, 1.983852, 1.394445, 1.339163, 0.887192, -0.401596, -0.275496, 1.287424, -1.783684, -0.146670, 0.804148, -0.714256, -0.587699, -0.653437, -1.054705, -0.606478, 0.044114, 0.418575, 0.215715, 0.520655, -0.875221, 1.841604, 1.357003, 0.824930, 0.480317, 0.834480, -1.260839, -0.158400, 0.225147, -2.214532, -2.080806, 1.850248, -2.412279, -0.588595, 0.296379, 1.033541, 0.046907, -2.113540, 0.835281, -0.008356, -0.641821},{-1.297134, -0.039251, -0.189881, -0.206566, 0.110656, -0.537896, 0.265355, 0.164970, 1.583793, 0.439037, -0.337714, 0.414660, -0.628758, -0.309360, -0.058427, -0.998921, 0.314439, 0.114197, 0.800673, 0.242380, -0.240498, -0.785584, 0.769699, 2.030388, 0.040536, 0.289371, -1.333212, -3.048604, -0.533008, 0.012747, 0.544850, 1.190672, -0.182334, 0.622612, -1.522392, -0.695751, 1.500881, -0.362798, -1.914865, 1.245239, -1.251621, -0.969999, -0.823397, 1.263923, -0.266415, -1.123182, -1.649930, -0.846169, 0.622887, -0.183568, 0.420988, -1.561726, 1.352750, -1.690773, -1.033695, 0.311679, 0.578923, 0.089616, -0.334977, -2.033983, -1.129512, -2.169387, 0.341854, 0.975008, -1.498222, 0.146227, -0.343926, 0.164494, -1.495024, 0.280934, 0.943642, 1.197031, 1.039465, 0.054783, 0.401842, -0.094395, 1.222630, 2.313557},{-1.797157, -1.605837, -0.716085, 0.310286, 0.251347, 1.191538, -0.431265, 1.989482, -0.297330, -0.314556, -0.803185, 0.303019, 0.394487, 1.766729, 0.607887, -2.697651, -0.478949, -0.882299, 0.185415, -1.179183, 0.966437, 0.341105, -0.708341, -1.354597, 0.119104, -0.307410, 0.631968, -0.709708, -0.594295, -0.229480, -0.050327, -1.179126, 0.779450, 0.807889, 0.220043, -1.509856, -1.233557, -0.718718, 0.036929, 1.856626, -0.498660, -0.392528, -1.371319, -1.215082, -0.203363, 1.041661, -1.185018, 0.751526, 0.705389, 1.142990, 0.780051, 0.165311, -0.237375, -0.622014, -2.379821, -0.782108, 0.438805, 0.257644, -0.064372, 0.738855, 0.634076, 0.646515, 0.146702, -0.669769, -0.013815, 0.645942, 1.330898, 0.067850, -0.765292, -0.948438, -0.976945, 0.259160, 1.011108, 1.325440, 0.373963, 0.175671, -1.596774, 2.449448},{-2.490656, -0.238718, -0.743761, 1.271621, 0.736171, -0.381155, 0.131596, 0.971896, 0.954485, 1.962015, 1.318219, 0.269997, -0.645347, 0.723150, -0.061207, 1.140235, -1.471496, -0.586615, -1.649366, 0.057724, -2.103088, 1.388479, -0.722973, -0.908143, 1.230522, 0.957862, 0.589363, 0.355607, 0.696328, -1.323147, -0.603626, -1.626527, -0.036208, -1.861564, 0.165092, 0.066270, -0.798279, 2.360813, -1.924702, 0.215946, 0.984086, -0.089771, -0.731320, -1.549592, -2.049068, 0.092493, 0.188130, -0.158233, 0.582488, -1.118752, 0.530403, 0.161637, 0.768948, 0.210953, 1.106818, -0.655572, -0.950137, -1.276447, 1.018458, 0.797179, 0.807678, -0.136631, 0.076664, 0.994173, -0.266046, -0.926100, -0.049028, -0.816612, 0.915281, 0.187846, -0.023068, 0.211812, 0.574904, -0.005701, -1.609976, -0.214551, -1.295502, -0.797638},{0.511683, -1.041439, -0.112587, 0.692213, -0.209240, -0.839266, 0.508313, -0.064894, 1.217581, -1.649910, 2.167522, -0.314336, 0.093197, -1.742797, 0.486980, -0.628097, -1.171715, 0.790042, 0.077233, -2.992113, 2.594834, -0.867132, 0.662295, 1.371927, -0.018957, 0.113247, -1.747190, -0.267346, 0.742450, 0.153524, 0.069898, 1.882172, 2.063602, -1.259065, 0.016499, 0.037466, -0.587045, -0.618110, -0.201232, -0.190801, 0.090072, 0.168766, -0.259741, -1.194532, 1.871279, 0.968356, 0.090281, -0.066078, 0.569483, 1.066085, 0.281547, -1.662385, 1.350936, -0.201891, 0.184234, 0.239280, -1.880111, -1.608908, -0.661578, -0.400056, 0.283048, -0.335099, 1.244227, 0.213353, -1.349343, 1.262554, 0.404243, 1.258481, 0.395863, 0.933636, -3.451376, 1.091280, 2.196188, 0.725356, -0.853634, 1.032590, -0.252524, -0.232906},{0.953617, -1.713262, 0.904754, 0.991238, -1.636586, -1.173923, -0.607558, 0.017205, 0.814943, 0.868007, 0.423620, 0.164650, -0.673936, 0.360295, -0.219031, 0.949204, -0.440346, -0.213712, 0.475268, -0.256420, 0.940301, -0.883727, 1.297784, 0.373887, -0.202227, 0.287929, -0.228854, 0.476102, 1.217838, 0.285818, -0.167363, -0.582570, -1.861013, -1.944522, 0.415345, -0.146078, -0.714622, -0.204299, -0.093526, 1.837011, 0.449698, 0.139767, -0.624200, -0.159762, 1.588272, 1.020593, 1.924409, -1.849271, 0.415815, 0.328930, 0.730816, 0.658555, -2.128650, 0.414304, -0.036042, 0.358761, -0.749661, -0.904359, -2.044137, -1.642487, -1.240883, 1.341588, -0.705541, -1.543020, -0.476641, 0.751311, 0.711024, 0.476131, 1.914468, 1.001230, -0.887663, -0.513114, -0.178914, 0.311344, 0.437752, -0.619673, -0.889195, 1.641735},{1.016067, 0.582379, 1.382602, 1.367408, -0.631094, 2.094626, -0.755266, 1.429192, -0.017719, -0.473084, 0.732539, 1.568756, 2.889609, -1.039288, 1.004963, 0.213560, 0.229726, -1.309598, -0.554891, -0.591193, -0.243864, -0.834183, -1.500963, 0.019329, -0.450460, -1.016149, -1.146322, 1.867301, -0.124576, 0.260283, -2.020005, -0.950719, 0.144877, 0.150475, -0.360533, -0.843521, 0.655025, -0.908438, 1.174654, 0.080451, -2.037738, -0.826087, -0.723266, 0.708449, 0.681019, 0.758767, 0.977927, 0.693495, 0.769111, -0.773278, 0.988688, -1.061765, -0.451678, -0.224737, 0.103752, 0.466556, -0.836444, -0.110667, -1.550583, 0.174877, 0.442484, 1.114852, -1.283620, -1.487553, -0.928299, 2.557518, -0.201772, -0.175232, 0.695203, -0.339373, 0.941219, -0.983825, 0.958037, -0.289668, 0.141578, -0.076186, 0.386321, 0.077689},{0.497424, 0.051297, 1.386888, 0.554797, -0.961767, -1.303207, -0.606485, -1.161454, -0.184809, -1.645274, 1.253336, 0.682145, 0.962329, 0.166071, -0.728800, -1.894800, 1.248464, 0.399991, 0.115835, -0.704696, 0.567859, 0.200096, -0.426206, 1.186800, 0.063982, 1.336578, 1.024601, 0.593144, -0.274839, 0.487389, 0.022164, 0.307952, 0.280280, -0.065874, -0.460302, 0.426248, -0.145214, 0.927738, -0.655408, 0.617842, 0.185264, -0.409401, 1.434435, 0.572105, 0.853725, 1.916025, -0.771443, -0.326508, -0.374878, -0.841955, 0.618629, 1.526332, 0.240024, -0.839110, -0.167700, 1.132247, -2.632199, 0.904257, 0.378582, -0.639632, -1.493468, 0.205911, 0.552683, -0.128463, -1.812483, -0.117900, 0.915461, -0.047011, -0.467132, -0.314714, 0.879677, -1.355753, -0.074056, -0.997469, -0.792768, -2.467539, -2.280995, 0.148233},{-0.463596, 0.384782, -0.919328, -2.060867, -0.416740, 1.068968, 0.004930, 1.563046, 0.452849, 0.753354, 1.723338, -0.087285, -1.073681, -1.784404, -0.016201, -0.568815, 1.522688, 0.662085, 0.650985, 2.077590, -0.582617, -1.047987, 2.418269, -0.657641, -2.060534, -1.092795, 0.277712, -0.573053, -1.277682, -0.669780, 0.784757, -0.818086, -0.302670, -1.823319, -1.359960, 0.065248, -0.830876, 0.077911, -0.076033, 0.210967, 0.723503, -0.545964, 0.768179, -0.229075, -0.325308, 0.459331, 1.371374, 0.230093, 0.891869, -0.501408, -0.015361, 0.668154, 2.020408, -0.200565, -0.542942, 0.639497, 0.523580, 0.386184, -0.475606, -2.061518, 0.054514, 0.398336, -0.724807, 2.209165, 0.446560, 0.641794, -0.635121, 0.259475, 0.672220, -1.086781, -1.004858, -0.606894, 1.516906, 1.300116, 0.659164, -1.288161, -1.475094, 0.308570},{-0.401315, -0.646392, -0.026864, -1.499359, 0.955489, -0.272265, -1.549447, 0.119482, -0.590582, -0.420602, 0.382962, -0.327687, -1.285329, 0.141870, 0.499420, -0.408439, 0.025137, -0.166219, 0.643836, 0.216796, 0.239619, 2.214240, -0.148487, 0.614677, -0.154831, -1.740570, -0.478056, -0.558527, 0.379195, 1.795399, -1.300548, 0.374383, 0.527486, 0.384258, -0.631082, 1.559570, -0.357712, 1.937537, -0.134055, 0.338543, -1.068344, 0.059083, 0.243246, 0.961215, -0.163285, -0.609311, -1.425006, 0.887094, 0.967057, 0.767784, -0.095960, 1.133406, -1.762566, 0.946378, 0.028167, -0.615093, 0.780886, -0.211815, 0.253147, 1.198573, 0.444906, -0.123966, -0.032311, -1.220662, 1.809949, 1.636588, 0.082029, -1.908841, -0.042471, 0.123551, -0.428937, -0.934129, -0.301037, -1.320076, 1.505956, 0.596383, 0.220931, 0.550827},{-0.249171, -0.720078, -0.038482, 1.866083, -0.790932, 0.410430, -0.375148, 1.190713, -0.588480, -2.153755, 1.534988, -0.246738, -1.218898, 0.796434, 1.304363, 1.438903, 1.476748, 1.212076, 0.636867, 0.168240, 0.415871, 0.386131, -0.629133, -0.165185, 1.954794, -0.163303, 0.841332, -0.311571, -0.673849, 1.551468, 0.685131, -0.249556, 1.293996, 0.328818, -1.238965, 1.802887, -0.788582, -0.272784, 1.115612, 1.275983, -0.135984, -0.026356, 1.232782, 0.283940, -0.870965, 1.082711, 0.593766, 1.383826, 1.067959, -1.110595, -1.722050, 1.004609, 0.687503, -0.942892, 1.135182, -2.330099, 0.944670, -1.881575, -0.532033, -1.056191, 0.854973, 0.008441, -0.552320, -1.357211, 0.134401, 1.100956, -1.418122, 0.059696, -1.919403, 0.330900, -0.314576, -0.271600, 0.796586, 0.399242, -0.115439, -0.093355, 1.124360, 0.670476},{0.556224, 0.198299, -0.623851, 0.560198, -0.456848, 0.438204, -1.031249, -0.650583, 0.213426, -1.189937, -1.750458, 0.159829, 1.152344, 0.934538, 1.736886, 0.054524, 0.114406, -0.263475, -1.269119, -0.702242, -1.231588, 0.678911, -1.038072, -1.771896, -1.641012, -0.375551, -0.221134, 0.155234, -0.798477, 0.195469, 0.036859, -1.026890, 0.192957, -2.563145, -0.440489, -0.494906, 0.626750, 0.920915, 2.124664, 1.476308, -1.198292, -0.755198, 0.161480, -1.079415, -0.968083, 2.195547, -0.711644, -0.891184, 0.452327, 0.363541, -0.105326, -0.149318, 0.894071, 1.731338, 2.131708, -1.573979, 0.867441, 0.593974, 0.324213, -0.203059, 0.629526, -0.955502, -0.551012, 0.589163, -0.617802, -0.730247, 1.016483, 2.192260, 0.619579, 0.340681, -0.157512, 0.553988, 0.688104, -0.999156, -0.774228, -0.435028, -1.256821, -0.700904},{-1.821440, -0.320499, 0.086497, -0.513211, 0.635079, -2.208740, -0.475401, -0.295972, 1.748266, 0.234614, -0.095597, 0.258247, -0.342117, -0.114377, 0.519129, 1.605085, -1.093688, -1.998204, 0.006404, 0.336433, -0.637526, -2.481783, -0.272168, 0.872249, -0.346175, 1.013330, -0.153589, -0.437633, -0.290353, 0.482834, 1.781439, 1.222152, 0.469439, 0.764347, 0.378440, 0.042284, -0.016532, -0.873593, 0.528554, -0.497185, -1.168108, -0.772552, 0.659016, 0.414224, 0.220957, -1.457850, 0.246676, -0.311015, 2.239161, -1.590415, -0.414899, 1.095441, 0.392854, -0.901977, -1.269559, 0.452111, 0.376966, -1.377427, -1.851917, -0.244974, 0.933853, -0.142859, -0.374280, -0.464171, 1.233736, 0.055327, -0.879191, 0.621205, -0.051993, -1.876886, -0.074716, 0.246301, 0.330636, 0.486058, -0.402972, 0.177961, 0.083527, 1.052488},{0.929535, 2.121467, 0.532343, 0.044025, -2.302675, 1.349094, 1.974664, -1.396877, 0.494407, 0.457259, -0.912501, -0.457916, -0.931886, 2.512606, 0.889821, 0.088657, -0.272010, -2.112737, 0.720887, -0.003691, 0.279980, 0.777801, 1.410992, 0.503996, -0.030608, 1.356430, -1.330723, 1.430809, -0.559814, 0.811542, -1.199477, -1.380833, 1.484595, -0.598714, 0.204508, 0.153270, 0.591331, 1.906135, 0.388803, -0.273482, 1.413709, 0.331208, 1.556098, 0.515913, -0.899343, 1.382215, 0.977397, -0.138884, -0.680375, -0.270110, -0.552059, -0.327311, -0.013782, 1.518579, -0.366309, -0.035057, -0.417544, 0.133730, -1.983245, 0.583653, 0.147750, 0.238256, 0.881068, 0.248464, -0.129587, -0.105690, 0.490398, -1.318240, -1.268648, -0.009072, -0.180913, -0.744771, -0.551360, 1.771293, 2.044334, 1.046617, -0.633677, 1.338479},{-0.519314, 1.789725, 0.221037, 0.198696, 0.144362, 0.549620, -2.024800, -1.593566, -0.785292, 0.754368, -0.513181, 0.102072, 0.558953, 0.624510, -0.944436, 0.880008, 0.053633, 1.123578, -0.406395, -0.703669, -0.290799, -0.122463, -0.088350, -1.149534, -0.516146, 0.437732, 0.854328, -1.000262, -0.008309, -0.683104, -1.767993, 0.728439, 0.611207, 0.095789, -1.124713, -1.024499, 0.702770, 2.081842, -0.704217, 0.224456, 1.481973, -0.770620, 1.204437, -0.336346, -0.760565, 1.239254, -0.492107, -0.460383, 1.526573, 2.840834, -0.426775, 1.210105, 0.127161, 0.905216, -0.443867, -0.711614, -0.926739, -1.090708, 0.276048, -0.844470, 2.214915, 0.002157, 0.192116, -1.808786, -0.368762, 0.426904, -0.277236, 0.048961, 0.590658, 0.013814, -0.353718, -0.140659, -0.681435, -1.174703, 0.341123, 0.276724, -2.143992, 1.860969},{-1.979143, -1.278387, -0.841738, 0.608497, -0.432053, -0.566003, -2.236937, -0.068504, 0.141678, 2.944789, -0.274850, -0.485860, 0.702542, -0.259866, 0.325268, -0.152545, -0.495403, 0.385113, 0.025986, 1.459293, -0.274879, -0.065318, 1.244156, -1.936983, -1.189655, -0.834180, 1.370315, 0.658612, 1.130001, -0.970821, 0.739201, -0.719811, 0.413134, -0.533867, 0.299000, -1.011017, -1.515395, 1.694552, -0.368307, 0.948371, -2.025620, -1.291967, 0.293280, -0.512110, 0.589875, 0.953228, -1.171139, -0.926373, 1.360566, 0.184546, 0.349242, 1.170625, -0.292566, 0.412893, 0.329184, -0.591202, -0.459819, -0.402643, -0.830648, 0.203247, 1.043945, 0.224827, -0.069517, -0.293135, 1.781510, -0.779270, 1.492350, -1.600666, -0.052604, -0.528246, -1.685727, 0.880952, 0.022545, 0.372225, 1.893874, -0.854006, -1.525333, -0.222116},{-0.567194, 0.528134, -1.293423, -0.383188, 0.305924, -0.572733, 1.005356, 0.993622, -0.770566, 1.174392, 0.837978, -0.496510, -1.398262, 1.424824, -0.233450, -0.612298, 0.355870, -1.590042, 2.235322, -0.154128, -1.403946, -0.146348, 0.963887, -0.777357, 1.676976, 2.394187, 1.087660, -1.867211, 0.381314, 1.099496, -1.559224, -1.301609, -0.329169, -0.610509, -1.103171, -1.924366, 0.097550, 0.855749, -0.589331, -0.078310, 1.959979, -0.008736, 1.190247, -1.691751, 1.531043, 2.809034, 0.619821, 0.431628, 2.056415, 1.930228, -0.108280, 0.230874, 0.271933, 0.185853, 1.312064, -0.214245, -0.430190, -1.111844, -1.401858, -0.792249, -0.731369, 0.988057, 0.448256, -0.029700, 0.790024, -0.856418, 2.412014, -1.273734, 0.365321, -1.147242, 0.168215, 0.465199, 1.109614, 1.081483, -0.356497, 0.024316, -0.185771, -0.453610},{0.653093, 2.354411, -0.114986, 1.698166, -0.187007, 0.123738, -0.126864, -0.078243, 0.160277, 1.183406, 1.409994, -0.298827, 0.639973, -0.511698, -0.791261, -1.016566, -0.558432, 0.363081, -0.104392, 0.885226, -0.854381, 0.208884, -0.128847, 0.143262, 0.891572, 0.167759, -1.366660, -0.434128, 0.888036, -1.641109, 1.097589, 0.760973, -0.573020, -0.287905, 0.055512, 0.192402, 0.583621, -1.472517, -0.623956, -0.623771, 0.201942, 0.689409, -1.228808, 1.144580, -0.522842, 2.158054, 0.587816, -0.098726, -2.115353, -0.365500, -1.309205, 0.370557, 0.379359, 1.387694, 0.801603, -0.075201, 1.744851, -0.974209, 0.415142, 1.211329, -0.123537, -1.006216, 1.371843, -2.104071, 0.102267, -1.137153, -0.043474, -0.561774, -1.028677, -1.163560, -1.541050, 0.963293, 0.047423, -0.309633, -0.289234, -0.338029, 0.788832, -0.468183},{-0.219553, 0.964451, -0.632407, -1.066975, 0.455010, -0.752267, -0.453484, -1.233477, 0.940707, 1.337488, 0.436062, -1.847597, -2.408745, -0.404811, 1.013158, 0.632294, 0.078330, 0.111976, 1.070527, 1.145080, 0.513378, -0.069456, -1.500713, -0.796171, 1.229062, 1.210487, 0.603367, 1.318429, -0.607273, 0.490003, -0.890972, 2.045780, 0.141562, -0.416129, -0.287310, -0.440561, 0.281567, 1.269715, -1.097974, -0.212315, -0.385238, -1.979175, 2.170321, -1.136322, -0.033672, -0.579863, 1.080234, -0.757300, 0.144727, -1.195434, -1.356952, -0.356493, -0.537094, -1.275621, 0.598274, 0.681684, -1.136823, -0.342640, -0.678895, 0.806671, -0.543272, 0.092626, -1.025209, -0.996116, 0.470515, 0.419102, 1.390744, 1.260175, -0.269176, -0.355303, 1.414737, 0.621820, 1.185817, -1.051727, 0.806485, 0.094110, -1.724511, 1.053628},{-0.080947, 0.566422, 0.336373, 0.294977, -0.325178, 0.042052, -1.594006, 0.013757, 1.203302, -0.343442, -1.016262, 0.018750, 2.119518, 1.571112, 1.578466, 1.352804, -1.204201, 1.572157, -0.341318, -1.589937, -0.585538, 0.045068, 0.825897, -1.160975, -0.492896, -1.278843, -0.658374, 0.706999, -0.231274, -1.414031, 1.559342, -0.304012, -0.488772, 1.232686, -1.078335, -2.264316, 0.881759, 0.678840, -1.988477, 0.595031, 0.254929, 0.191762, -0.228405, 0.741597, -0.224081, 0.398908, -1.140689, 0.374724, 0.572948, 2.647717, -0.374293, -0.207972, -0.579135, -1.127559, 1.583028, 0.950973, 2.314459, -0.689661, -0.428530, -0.870909, -0.156135, 1.176197, 0.618693, -1.100569, 0.621726, 0.885930, 0.468965, 0.123385, -0.728174, 0.855087, 0.611048, 0.484618, 0.254218, -0.803156, 2.272432, 0.921042, 0.552375, 0.521094},{1.383975, 0.512012, -0.483633, 1.661976, 1.022204, -1.117786, 0.486363, 0.598179, 0.367237, -0.497561, 0.081886, 1.199549, -0.280788, -0.753416, -0.064034, 0.569030, 0.686515, 0.192949, -0.328708, -1.725187, 0.057071, -1.594372, -1.757454, -3.030642, -0.481534, -1.269346, 0.667035, 2.319814, 0.825312, 1.636988, -0.374671, -0.178456, 0.844274, -0.824944, 1.950880, -0.290863, 0.055360, -0.855600, -0.246975, 0.254065, 0.917138, -0.836100, -1.571629, 0.420557, 0.172358, 0.349006, 0.913698, 1.917745, -1.445433, 0.929893, -1.804224, 0.379366, -1.474590, -0.276189, 0.931770, 1.374129, 0.432788, -2.421989, -0.536083, 0.886538, 0.385263, -0.155305, 1.124102, 1.260957, 0.172976, -0.193585, -2.350599, 0.799399, -0.043283, 0.888745, 0.355383, 2.151691, 1.555542, -1.715190, 0.210276, 0.134959, -0.939359, 2.935498},{-0.887149, 0.397844, -0.363080, -0.266028, 0.820690, -1.059916, -0.646891, -0.799169, 1.723782, -0.313391, 0.629914, 0.175102, 1.406701, -0.088751, 2.448795, -0.386723, -1.368374, -1.012184, 0.282685, 0.204182, 1.222307, 1.337009, 0.888061, -1.399157, -1.149239, 1.729011, 0.571424, 0.332592, -0.902203, -0.286892, 0.311546, -0.077950, 0.142279, -0.337911, -1.675808, 0.945517, 0.107666, 0.088739, 0.189436, 1.828781, 1.955508, 1.333738, -1.950646, 0.849982, 0.728881, 0.310380, -1.559507, 0.992839, -0.402770, 0.782945, 1.051275, 1.618838, -1.507986, -1.712314, 1.033790, -0.452515, 1.007036, 0.625626, 1.842922, -2.226279, 0.312411, 0.698228, -1.000078, -0.145301, 0.458056, 0.349610, 0.678723, 1.299415, 1.151875, -0.098091, 1.074220, 1.115692, 0.152577, -0.714176, 1.288004, 0.889222, -0.097732, 0.683085},{1.642220, 1.734736, 0.537846, 0.973101, 0.199438, -1.057629, -0.376988, -0.342364, 0.076319, -0.747508, -0.742737, 1.361545, -0.717751, 0.219167, 0.761997, -0.764172, 0.159571, -0.236201, 1.031826, -0.035243, -0.272397, 1.548356, 0.676284, -1.182603, 0.694911, 0.783091, 0.486793, -1.098359, 0.406626, -0.675382, 0.441307, 0.176032, 0.665902, -0.726231, -1.430844, -1.320877, 0.482524, 0.825159, 0.613553, -0.141942, -1.341702, 1.738927, 0.167413, 1.222187, -1.812991, 1.456148, -1.010513, -1.670668, -0.109558, 0.539118, 0.523858, 1.181335, 0.068335, 1.441133, -1.218479, -0.861696, -1.444565, -1.844750, 0.815929, -1.592472, -1.590726, 1.325108, -0.899256, -0.876241, -0.850433, 1.455847, -0.710477, 2.071235, -0.123411, 1.055619, 0.404471, 0.241020, 0.573070, -0.325429, 0.535067, -0.901772, -1.190964, -0.613648},{-0.301503, 2.209847, 0.464914, 0.725877, -0.530459, 0.887894, -1.196525, -0.373098, -0.174483, 1.601822, 0.655418, -0.021598, 0.008996, 0.667825, -0.491360, 0.545728, 0.493860, 0.387538, 0.396404, -0.698577, -0.398238, -0.329837, 0.220340, -0.576529, 0.157018, 1.954514, 0.650606, -0.066233, -0.140243, 0.772272, -1.885559, -0.301482, -2.019645, -1.448754, 0.216735, -1.064797, 1.568529, -0.944514, -0.081892, 0.843383, -0.416989, 0.227737, -1.185808, 0.514054, -0.896018, -2.618052, -0.437215, 0.883607, -0.190989, -2.825197, -0.041239, -0.911079, 0.280747, 0.953653, 1.863509, 1.100642, -0.366053, -0.538459, 0.610620, -1.177965, -0.233528, 0.707648, 0.067226, 1.589845, 1.000633, 0.898012, -2.171667, 0.264894, 0.230026, -1.202295, 0.252887, 0.191983, -0.395309, -0.340797, 0.515280, -0.435592, 0.310662, -0.252656},{0.337481, -0.420800, -2.064605, 0.990148, 1.403821, 0.394290, 0.108119, -0.529167, -0.605358, 1.514181, -0.803186, -0.343414, -1.168802, -1.849579, 1.313775, -0.618550, -0.069003, 0.884913, -1.175825, 0.110938, -0.416571, -1.393410, 0.634362, -1.210228, -0.294996, 0.682214, -1.134341, 0.269973, 1.864841, 2.166806, -1.349636, -0.406345, -0.565414, 0.083847, -0.887295, 1.653367, -0.453644, -0.929189, 0.609202, 0.640974, -0.010470, -0.226249, -0.051552, 1.249390, -1.304367, -1.010434, -0.204875, -0.615393, 0.577451, -0.677613, 0.445539, -0.578510, 0.006798, -0.684245, -1.091977, 0.112854, 0.875591, -0.515896, 0.292196, 0.306218, -1.363162, -0.571720, 0.770310, -1.412996, -0.591351, -2.246537, 2.151672, 1.708719, 0.281617, -0.773246, 0.244741, 1.846092, 1.288797, -0.824857, -0.114129, -0.354381, 0.475327, 0.277597},{-1.810264, 1.187774, -0.636373, -1.415394, 0.965421, -0.155251, -0.617686, 0.409767, -0.004198, 0.296954, 0.157039, 1.530686, -2.129308, -1.607164, 1.158630, 0.062430, -0.617193, -1.201761, -0.251698, -0.847000, 1.044942, 0.336800, 1.263827, -0.964564, -2.173301, -0.635170, 1.161010, -0.193628, -0.480733, -1.916449, -0.018067, -1.701402, 0.730270, 0.660550, -0.273048, 0.270273, 1.669134, 1.874303, -1.108212, 1.355625, 1.128493, 0.203398, -0.913395, -0.132303, 0.508464, 0.299920, 0.250456, 1.669185, -0.820881, 0.413811, 0.145074, -0.185120, -0.789851, 2.161447, 1.624083, -0.263463, -0.304224, -0.008265, 0.929278, 2.586401, -0.669760, 0.772326, -0.843759, 1.158547, -0.918223, 1.178360, 1.171345, -0.864953, -0.385412, -1.667997, 0.608054, -0.394248, 0.448734, -1.469151, -2.171477, 0.050822, -0.167560, -0.994346},{0.438062, 0.468268, 0.039282, 0.307551, -0.822549, 1.256477, -0.393318, 0.383893, 0.619204, -0.828072, -0.319395, -2.402428, -0.090159, -0.871999, -0.592594, 1.215050, -1.238516, 0.301277, 0.335482, -1.347335, -0.815700, 0.940436, -0.824346, -0.192356, -0.034769, 1.148103, -1.840782, -1.363721, -0.234632, -0.554110, -0.170920, 1.373972, -0.844429, -0.045388, -1.678290, 1.734755, 1.527740, -0.520042, -0.187786, 0.002555, 1.105331, 0.275010, -0.762973, 1.681184, 0.451931, 0.859902, -0.512296, 0.160165, 0.930903, -1.045609, -0.376099, 1.424118, -1.156725, 0.053102, -1.828581, -0.360674, -0.357159, 0.291413, 0.938211, -0.871935, -0.102614, 1.954626, 1.702610, -0.605663, -0.171140, 0.704646, -0.054282, 0.833090, -2.399055, 0.455926, 0.151241, 0.037921, 0.617559, 0.459233, -1.301476, 0.062227, -0.823845, 0.578413},{0.559351, -0.301620, 0.211408, 0.744806, -0.263588, 1.495340, 0.353493, -1.884304, -0.964753, 2.325844, 0.796806, -1.276559, 0.349569, -0.880303, 1.610937, 1.977522, 1.332708, -2.191849, 0.362476, -2.629761, 1.100649, 1.136023, 1.267390, -0.115470, -0.901448, -0.626252, 0.386738, 0.329296, -0.376816, 1.798877, -2.307595, -0.957267, 1.401309, -0.715659, -0.206975, -2.956558, -0.767811, -0.052558, 0.248835, -0.454029, 0.408169, 1.415178, 1.801665, -0.569080, -0.389252, 0.470888, -0.181876, -0.296940, -0.592068, 0.345405, -0.125221, -0.102670, -0.979464, 1.512072, 0.545377, 0.706694, 0.941977, -0.974102, -0.818991, -0.631120, 1.576839, 1.225446, -0.707064, -0.051633, -0.281495, 0.192074, -0.527933, -0.999876, 0.952241, 0.341656, -0.147687, 0.621282, -2.104959, -0.758763, -0.555269, 0.792117, -1.224940, 0.509035},{-1.978355, -0.001060, -0.556497, 0.887493, 0.778809, 2.062877, 0.854420, -0.140343, -1.183377, 2.528825, -1.788133, 0.139427, -0.365837, -0.952227, -0.208402, 0.398537, 0.157923, -0.918739, -0.291384, -0.294403, -0.239784, 0.404109, -0.068875, 0.437211, -0.380636, 0.580982, 0.533875, 1.021906, -0.632342, 1.033089, -0.372075, 0.237859, -0.167172, 1.376226, 0.610850, 1.366387, 1.644040, 1.428720, -0.846790, 0.415947, -1.511037, -1.576395, -0.438807, -1.167034, -0.741437, 1.412580, 1.057213, 0.334590, 0.149193, -2.353665, -0.878975, -1.704585, -0.162629, 0.175019, -0.398327, 2.035791, -0.288990, -2.237472, 0.234496, 0.547527, 1.138536, 1.018239, 1.128515, 0.171648, 1.128820, 0.146276, 0.261178, -0.610845, -0.330892, 0.135951, -0.619183, 1.180005, 0.784830, 0.267829, -0.507454, -0.934576, -0.900166, -0.146173},{0.808215, -1.168213, -0.372772, 1.975561, 0.846305, -0.648290, -0.644757, -1.823345, 1.251493, 0.284433, 2.094789, 1.164325, 0.176047, 0.512976, 0.778553, -0.817768, -0.595413, 1.171639, -0.538145, 0.511651, -0.032636, 0.039391, 0.500509, 0.565202, -0.441957, -0.574906, -0.569568, 0.677307, -0.684864, 0.502871, -1.064161, 0.665147, -0.534738, 0.892955, -0.509176, 0.470014, 0.576431, -0.640051, -0.150125, 0.864025, 1.147664, -1.042649, -1.110848, 0.558971, 2.430661, 0.189017, 0.282008, -0.329471, 1.976204, 1.094327, 0.174413, -0.277931, -1.582211, 0.409094, -1.340061, -1.384811, -2.362426, -0.203987, 2.098165, -0.521209, -2.283879, -0.049913, 0.280661, -0.823911, -1.012100, 0.699442, -1.341235, -1.583114, 1.530123, 1.106924, 0.772844, 1.021864, -1.308267, -0.899791, -0.474848, 0.833886, 0.672169, -0.369930},{1.042818, -1.215715, -1.288209, 0.224623, -0.402737, 0.395781, -1.249694, -2.237922, -1.638885, 0.077249, -1.122658, -1.182648, -1.659138, -0.155058, -0.801305, 0.630127, -0.420838, 2.079164, -1.938565, 0.029290, -0.521654, 0.001254, 0.697556, -0.629542, -0.777696, -0.382128, -0.927911, 0.256558, -0.732364, 0.702350, 0.074020, 0.419144, 0.717027, -1.771729, 0.064589, -0.932535, 1.332158, -1.311589, 0.047617, -0.618845, 1.369850, 0.240172, -1.021437, 1.269225, 1.342054, 1.041313, 0.833232, -0.850036, 1.031880, 2.488530, -0.632791, -1.942149, -0.366302, 0.509318, 1.481822, 0.406896, -0.386661, -0.605906, -0.216106, -0.088150, -0.577721, -1.485254, 0.077640, 0.082575, -0.527175, -0.887400, 0.256322, -0.229207, -0.252246, 0.380082, -0.424208, 0.571449, 0.238270, -0.852588, -0.987409, 1.172024, -0.754334, 0.500567},{0.613162, -2.160532, -2.248370, 0.728659, -1.379981, 1.102155, 0.254492, -0.864618, -0.402598, 0.522733, 0.448941, -0.780352, -2.750790, -0.511015, 1.740795, -0.850852, 1.702785, -0.684401, 0.355804, -1.256796, 0.392062, 0.234744, 0.102770, -0.123472, -0.917604, -0.317157, 0.873457, 2.799906, -0.259934, -0.262024, 0.895362, -0.565525, 0.087633, 0.431015, 1.331345, 1.217606, 0.667837, 0.629007, -0.886501, 0.023618, -0.573382, -0.635930, -0.834085, -0.205844, -1.605302, 0.075197, -0.270869, 1.440636, -0.045636, -0.406528, 0.841777, 0.758807, -0.275874, 1.732809, 0.764364, 0.242023, 1.202918, -0.131644, 1.380278, -0.297261, -0.051594, 0.215955, 0.019133, -0.519014, -0.982199, 0.425827, -0.535597, -0.824872, -1.067038, 2.520172, -0.233822, -1.237274, -0.030301, -0.952557, -0.379742, -1.063795, -0.915999, 1.233569},{-0.538945, 0.263007, 0.529264, -1.163078, -0.384757, -0.541924, -0.015767, -0.704173, 0.730852, -1.217812, -0.453551, -0.703506, 1.811032, 0.231330, -0.416372, -0.713287, 0.003183, 0.409449, -0.412764, 0.106481, -0.086378, -2.880254, 0.583261, -0.002443, 1.029933, -0.454281, -0.549275, 0.066248, -1.395648, 1.046251, 0.031070, 0.987733, 1.119734, 1.170082, -0.171609, -0.040862, -2.240724, 0.098392, -1.051754, -0.243222, -1.857095, 0.528078, 0.359823, 0.532903, 0.619107, 1.663934, 0.424385, 0.305626, -0.567394, -1.860554, 0.402485, -0.064312, -0.431457, 0.614971, 0.061360, -1.091225, -1.173472, 1.141207, -0.506730, 0.369417, -1.206212, 0.488769, 0.459910, 1.000427, -0.255543, -0.643245, -0.140046, 2.322747, -1.517465, -0.443236, 0.977249, 0.631600, -0.101513, 0.198699, 0.457115, -1.172203, -1.333748, 0.285620},{0.378554, 2.104970, 0.368123, 0.642459, -0.855964, -0.474155, -0.423484, 0.262879, -0.534044, 0.487472, 0.711723, 0.238511, -1.008233, -1.965904, -1.544374, -0.075498, -1.415960, 0.617465, 0.863492, -0.927724, 0.670562, 0.577499, -0.148392, 0.977056, 0.094414, 0.583227, -0.315942, 0.673815, 0.264646, 1.046989, -0.050682, -1.033813, -0.952202, 0.769910, -0.318979, -1.703011, -1.018914, -0.156276, 0.318671, -0.593617, -0.674389, -3.268210, -1.402187, -0.565216, -1.598002, 0.850105, 1.360898, -2.203304, 1.230590, 0.127393, 0.857851, -0.671869, -1.295368, 0.615956, -0.569384, -0.514769, -0.230376, 1.253893, -0.860097, 0.265765, -1.135057, -1.793516, 0.823756, -0.141600, 0.511218, 0.636544, -0.717027, 1.582717, -0.846106, -0.242711, 0.055025, -0.893537, -0.335861, 0.190239, -0.438993, -1.091815, -0.243868, 0.623416},{0.447014, 0.425162, 2.010734, 0.966350, -0.734452, -0.259710, -1.855450, -0.984952, 0.507511, 0.469829, 0.349899, 0.924630, 1.261228, 0.581944, 0.490046, -0.684960, -1.153451, 0.114607, -0.562598, -0.144754, -0.245971, 0.077728, -0.065804, -0.492108, -0.349330, 0.873292, -0.358034, 0.938914, 2.283073, -1.341580, 2.588907, 2.290680, -0.387505, -0.663927, -0.779557, -0.009867, 0.510650, 0.454424, 1.970361, -0.500924, -1.280428, 0.596154, 0.776695, -0.518118, -0.513160, 0.201956, 0.630690, -0.608977, 0.315375, 1.523535, -0.348405, 0.446324, -0.349706, -2.059832, 0.396235, 0.264312, -0.396565, 0.301983, -0.601042, 0.393240, -0.316961, -0.127847, -0.491490, 1.651048, -2.112453, -0.151823, 0.215830, -0.181804, -0.236242, 0.777588, -1.290019, 0.262138, 2.536084, 1.750705, -0.041801, -0.461081, 0.042727, -0.362446},{-0.463190, -1.158274, 1.040596, -0.074995, -0.443347, -1.442585, -0.087322, 1.354741, -0.190991, -0.529386, -1.678641, 1.079256, 0.124961, -0.352123, -0.992702, -2.082681, -1.936718, 0.810324, 0.520515, -0.189692, 0.939306, 0.091131, -1.527046, -0.133883, -0.249072, -1.539123, -0.202534, -0.620787, 1.244071, 0.530264, -1.066862, -0.502833, 0.325347, -0.367628, 0.708780, 0.762135, 0.462513, 0.013994, 0.542033, -2.092944, 0.228838, -0.267648, -0.822404, -0.125519, -0.753321, 1.037663, 0.839578, -1.761266, -0.097724, 0.695391, 0.292359, 1.209701, -0.837924, 1.815144, -0.721138, -0.019213, -1.166455, 1.899942, 1.414285, 1.237449, 1.546454, -0.301798, 0.841316, -0.696621, -0.112451, 0.925125, 0.777585, 1.296561, -0.045659, 1.255875, -0.980261, -1.027490, -1.345942, 0.515513, 0.053287, 0.110424, 1.333340, -0.843558},{-1.191580, 0.980561, -0.834826, 1.030847, 1.669448, -0.783521, 0.273418, 1.430827, 0.111434, 0.671092, -0.882445, -1.335065, -0.233961, -1.650978, 0.271099, 0.701460, 0.783599, -0.377894, -1.049743, 1.571046, -1.008231, -0.727303, 0.473048, -1.162518, -1.009071, -1.103692, 0.694866, 1.674028, 0.066630, -0.130934, -0.609561, 0.954607, -0.123251, 0.366963, -1.626485, -0.381876, -0.581630, 0.650973, 0.833224, 0.043588, -0.505110, -1.169378, 0.435746, 1.828345, -0.388107, 0.366221, 0.095557, 0.454156, -1.026125, 0.866677, 0.135758, 0.516658, -0.979159, -0.038180, 0.291374, -0.540048, 1.071042, -0.864071, 0.729787, 0.399199, 1.122540, 1.472275, -0.225084, 1.557933, -0.214497, -1.414146, 1.470000, 1.325079, 0.501955, 0.928303, 1.346380, 1.234302, 1.039639, 1.298699, -0.752233, 0.271139, 0.613587, -0.247481},{-1.499556, -1.516897, 2.052359, 0.047139, 0.369445, 1.306076, -1.079306, -0.360158, -0.142407, -1.235345, -0.290910, -0.027808, 1.302973, -0.811510, 0.065363, 1.669027, -0.863818, -1.232852, -0.246813, -0.207915, -0.253570, 1.165979, -0.446842, -1.151494, 1.259684, 1.555444, -0.309484, -1.537073, 0.426579, -0.518839, 1.903138, -0.487700, 0.555203, 0.202050, -0.738252, -0.392422, -0.967313, 0.183877, 1.927156, -0.351202, 0.949693, -1.228296, -1.330880, -0.489292, -1.776092, 1.187589, -0.036569, -0.195750, 1.006021, 0.306226, -1.067473, 0.682548, 1.086619, 0.962746, 0.460395, 1.174377, -0.357293, -0.834578, 0.318692, -1.083530, 1.842275, -0.046407, 1.546661, 0.053794, 2.700134, -0.399376, 0.245569, -0.656100, -1.126701, -0.077385, 0.841436, 0.582332, 0.525060, 1.336392, -0.852177, -0.189060, 1.360102, -0.962915},{-0.953943, 0.083268, 1.639038, 1.713037, 0.651241, 0.356831, -1.694170, 0.136362, 0.996552, -1.320035, -0.846484, -0.339251, -0.484214, -0.648613, 0.954438, 0.425005, -0.865875, -0.648976, -0.750348, -0.131048, -0.921387, 0.251839, -0.530804, 1.648663, -0.691367, -0.894226, -1.083324, 0.243599, -1.007933, -1.591802, -1.313096, -1.008803, -0.017023, -0.396290, -0.457299, 0.014342, -0.307306, 0.131577, -0.602816, 0.739534, -0.446981, 0.726786, 0.057541, 0.024233, 0.210225, -1.634110, -1.458515, -0.204123, 0.524050, -1.219014, -0.622724, 0.850683, 1.940414, -0.102387, -0.752950, 1.229625, -0.044067, -1.306759, 0.774133, 1.419870, -1.810223, -0.684662, -0.311764, 0.011013, -2.744300, -0.754660, -1.110817, -1.704554, -0.030781, -0.543098, 0.768441, -0.808700, 0.302862, 0.664865, -0.187472, 0.233755, -0.486273, -0.581872},{0.129897, 0.522225, 1.066863, 0.114899, -0.506545, 1.061059, -1.737007, 0.202718, 1.084567, 0.877800, -0.715105, -1.383179, 1.040002, 0.824417, 0.323672, -0.520220, -1.529817, 0.694086, 0.760631, 1.088145, 0.267059, 0.163078, -0.583640, -0.912612, -0.611205, 0.206304, -0.078870, -0.450118, 0.235731, -0.691595, 0.243767, -1.315815, -0.491579, 1.532576, -0.222800, -0.447141, -0.516288, 0.950060, 0.214100, -0.239874, -0.543659, -0.106143, -0.169726, -0.356249, 0.383291, -0.932825, -0.156243, -0.074372, -0.381299, 0.139268, -0.629442, -0.531985, -1.428998, -0.720962, 1.088916, -0.910430, 0.874642, -1.304776, 0.159778, -0.748690, -0.210635, -0.095024, -0.036512, 0.799515, 0.263004, 0.565202, 1.664314, 1.240305, 0.405633, -0.585384, 1.987673, 0.003929, 0.264744, -0.428930, -0.475471, -1.429798, -1.528047, -0.670145},{-0.385486, 0.470526, -0.236531, -2.085973, 0.883630, 1.135535, 1.298343, 0.711462, -0.755286, -0.114619, 0.883309, 0.635522, 0.039459, 0.384809, -0.702825, -0.364426, 0.387101, -0.441048, 0.202525, -0.336913, 0.653061, 0.707890, -0.355986, 1.288800, -0.530347, -0.556516, -0.127415, 1.177209, -1.662501, 0.078095, 1.317206, 0.747431, 0.827540, 0.523833, -1.322495, -0.735770, -0.540433, -0.182394, 0.961851, -0.324745, 0.632808, 0.910345, 0.948495, 0.207762, -0.080049, -1.648748, 0.785681, -0.666161, 0.175788, -0.959158, 0.269909, 0.880982, 0.027417, 0.437706, -0.421123, -0.756798, -0.568810, -0.620436, -0.452305, 1.450509, 0.948223, 1.066364, -1.343449, 0.630204, -0.099006, 0.304157, -0.225563, -0.527316, -1.313141, 0.718589, -0.354965, -0.743256, -1.485293, -0.517441, -0.677774, -0.300919, -0.347417, -0.021754},{0.278016, 0.635269, 0.482798, 0.595223, 1.464445, 1.948195, -0.358589, 1.863238, -0.948928, -1.242699, 1.704553, 0.414593, -1.365246, -0.704196, 0.856535, 0.069528, 0.097336, 0.718250, 0.824149, 0.788403, -0.750854, 0.171983, 1.331084, -0.633786, 0.816103, 0.118321, -0.740617, -0.078400, -1.025068, -0.176218, -0.369219, 1.005019, -0.069857, -0.810083, -1.810469, 0.464243, 0.560274, -0.770743, -0.458802, -0.210199, 0.210399, 0.117619, 1.326674, 0.262585, 0.794697, -0.873336, -0.154694, 0.478634, 1.448241, -0.214774, -0.488676, -0.436437, 2.494007, -0.707908, 0.807671, -1.602763, 0.130501, -0.339405, 0.037366, -2.084563, 0.624810, -0.743659, -0.549261, -1.313778, 0.038694, 0.049768, 0.139013, -0.594398, -0.624493, 0.106564, 0.841445, 0.022262, 1.078584, -0.643543, -0.194179, -0.098748, -0.548137, -2.071421},{-2.279711, 0.559144, 2.763570, -0.595456, 0.454062, 0.888014, -0.435055, -1.659867, 0.244933, 0.507690, -0.042044, -0.159054, -1.284265, -2.524609, -2.604433, 0.459118, 0.743807, -0.391204, 0.063242, 0.069875, 0.048207, 0.583094, -0.806215, 1.718664, -1.311783, -1.080578, -0.628933, 0.583349, -1.622878, 0.729662, 1.193526, 0.762035, 0.657311, 0.622159, 0.931500, 0.676410, -0.465009, 0.482878, -0.897168, -0.473732, -0.081878, -0.260040, -1.436511, -1.162063, 0.026127, -0.085593, 1.098931, 0.040609, 1.015278, -1.154629, -1.580602, -1.158578, 1.463169, 0.711009, -0.385127, 0.236698, -1.817799, 1.289511, -0.181144, 0.611344, 0.098931, -1.693269, -0.380593, -0.428094, -0.166468, -1.096649, -1.043148, 0.030608, -1.369887, 0.453225, 1.797975, 1.162265, 0.614869, 0.443481, -0.743886, 0.491282, 0.663846, 1.696028},{-1.456171, 0.039849, -0.313693, -1.018212, -0.155138, 0.801978, -1.961189, 1.412654, -0.487231, -0.358313, -0.384655, 0.639751, -0.639496, 0.273723, -1.324783, 0.444047, -0.132972, -0.442362, 0.034883, -1.629900, 1.341920, -1.405821, -0.994950, -0.277647, 1.241046, -0.041613, -0.825599, -1.562292, 1.099163, 1.068171, 0.562681, -1.222980, 1.435298, 0.089302, 0.294907, -0.597083, 1.471855, 1.547509, -1.610152, 1.648277, 0.495104, -0.868201, -0.686493, -1.629948, -0.621826, -0.929607, 0.958486, -1.026448, 1.244312, -0.751136, 1.700756, -1.582648, -0.420050, 0.358292, -0.411998, -0.890667, -0.391416, 0.167500, 0.236978, 2.315147, -0.371419, -0.472724, 0.023197, 2.888586, -2.281548, -0.639470, -0.623428, 1.892365, 0.687718, -0.385343, -1.672796, -0.540097, -0.923483, 0.048745, -0.055650, 1.252331, 0.727310, 0.694006},{0.798972, -0.363991, -0.841056, 0.020450, 0.403158, -0.009533, 0.703255, 0.497667, -0.003921, -1.910689, 2.247352, 1.048590, -0.637638, -1.586853, -0.608808, 1.710234, -0.835878, -0.258903, -0.081695, 2.210913, 0.334254, -0.759086, -0.620984, -0.650652, -1.490278, -0.136806, 0.314257, 0.585898, -0.628260, -0.187497, 1.252831, -0.341096, 1.803296, -0.717751, 0.001205, -2.050914, -2.317821, -1.196414, -0.947303, 2.396635, 1.014712, 0.215332, 1.449168, 0.097372, -0.448868, 0.203039, -1.465761, -0.558053, -2.046368, -0.115367, 0.565813, 0.084406, -0.028079, 1.081423, -1.100906, 0.576419, -0.753403, 0.298587, -0.905528, 0.856269, -0.478355, 0.780084, 0.537494, 0.511261, 0.411521, 1.008907, -0.968768, 0.356052, 1.071171, -2.418071, -0.850980, 1.980849, -1.900374, -0.724900, -0.950318, 0.497370, 0.381628, -1.589672},{0.734353, 0.155727, -1.145170, -2.170507, -0.440213, 1.050234, 0.854087, -1.487317, 0.766882, 1.782872, -1.735679, 1.440306, 0.494469, -0.814718, 0.147751, -1.064200, -1.456464, -0.122665, -0.288968, -0.274504, -0.733791, 1.337821, -0.105784, -0.303900, -0.311407, 1.406653, -0.650820, -0.744069, 2.753484, 0.294671, -0.647089, -0.459090, 0.298114, -0.258324, -0.812564, -0.315133, 1.659531, -0.527875, 0.339219, -0.991080, 0.990421, 0.041609, -0.049327, 0.191563, 0.645235, -0.912297, 0.569008, 0.428584, 0.525201, 1.414517, -0.436245, 0.814073, 0.073000, 0.016499, -0.624834, 0.835554, 0.719040, 0.843114, 0.493807, 0.801704, 0.500719, -0.645455, -1.882557, 0.719378, 0.161435, -0.026348, -0.302577, -1.273347, -0.381916, 0.385323, -0.342714, 1.270461, 1.227799, -0.801438, 0.209059, -0.539430, -1.231424, -0.178843},{1.237328, 0.442219, -0.354469, -0.519669, 1.939790, 0.716685, -0.040438, 0.531670, -0.063363, 0.819035, 0.273982, -0.128902, -1.626811, 0.107410, 0.300302, -0.326213, -1.710447, -0.088467, 0.104785, -0.228906, 0.792139, 0.669154, -0.339360, 0.333021, 0.490374, -0.441310, 1.037174, -0.286498, -0.745423, 0.612568, 1.191688, -1.433024, 0.123815, -0.720849, -0.138599, -0.558803, 0.650288, 1.921607, -1.706880, -0.205769, -0.530976, 0.805130, 0.778679, -0.130543, 1.303423, 0.016588, 1.278151, 0.815082, -1.246852, -0.282107, 1.120217, 0.162571, 1.294954, 0.945835, -0.025140, -0.191865, 0.726707, 0.332649, -1.207616, 0.350764, -0.247853, 1.212701, -0.405279, -1.722123, -0.124440, 0.611722, 1.100323, -1.684169, 0.639249, -0.012810, 0.381327, -0.983998, 0.195894, -0.574820, -1.741900, 0.069688, 0.329230, -0.655232},{-0.076826, 2.101183, -0.774307, 0.714529, -0.565780, 0.353001, 0.793318, 0.385506, -0.539950, 0.867497, -0.613367, 1.410921, 1.119181, -1.055414, 1.008844, -0.273942, -1.257525, 1.148984, -0.471204, 1.450500, -2.068598, 0.633082, 0.413706, 1.082925, 0.273360, 0.325114, -1.955849, -2.632710, 0.707903, 0.209062, 1.612556, 1.495362, -0.689499, -0.839463, 1.094123, -0.122968, -0.823257, 0.575293, 1.782561, -0.565830, 0.732596, -0.316286, -0.349683, -0.303336, -0.339903, 0.548334, 0.401014, -0.194535, -1.367549, 0.258882, -0.017916, 1.220404, 0.740114, -0.530958, 0.928521, -0.356640, -0.588555, 0.493611, 0.484854, -0.207394, 0.574355, 1.064718, -0.663802, -0.402286, 0.276501, 0.768441, 1.184191, -0.873027, 1.129914, -0.498007, -1.669274, 0.056577, -1.909333, -2.474038, -0.205955, -0.690143, 1.027552, 0.496990},{0.220736, 2.035749, 0.485070, 0.387768, -0.949477, 0.682089, 0.074289, -0.544199, -0.908973, -0.465721, -1.214232, 0.480947, 0.998827, 1.712514, 0.180022, -0.340311, 0.825498, 0.758102, 0.033334, -0.033982, -0.893934, -0.125209, 0.372717, -1.693335, -0.822154, 0.573078, 1.438029, 0.927529, 0.148839, 1.418291, -0.174709, 0.066677, 0.592809, 0.781419, -1.716205, 2.046453, 1.256678, -0.478798, 0.343727, -0.978030, -0.008886, -0.566781, 0.888719, -0.959266, 0.502482, -1.084812, -0.665601, -1.094768, 1.145544, 0.659958, -0.519392, 1.375161, -0.362409, -0.983940, -0.079048, 0.936469, -0.604232, 0.458142, -1.236316, -0.508747, 0.720094, -1.569296, -0.481191, -1.439074, 1.579905, 1.310701, 0.632351, 0.021122, -0.652655, -1.190873, -0.055787, -0.579231, 0.343398, -0.880602, -0.227836, 0.722530, 0.874523, -0.221769},{-0.503422, 0.786677, -1.507521, 0.582801, 0.016944, 1.595678, -0.387839, -0.641675, 1.445799, -1.700697, 0.602233, 0.095055, -0.428225, 0.311486, -0.835762, 1.614498, -1.682840, -0.242455, -2.092213, -1.571287, -0.617053, -0.002969, 1.089070, -1.851221, 0.771192, 0.021619, -0.230211, -0.083979, 0.010192, -0.267431, 1.515977, -1.735082, 0.549191, -1.190446, -1.252482, 0.845173, -0.629981, 1.017679, -1.452829, -1.769626, 1.331113, 0.616239, -0.882565, -0.551587, -0.453349, -2.366530, 0.416138, -0.828111, -0.186623, 0.022843, -0.972900, -0.278512, -1.334535, 0.943958, 0.438973, -0.934119, 0.801198, 0.215706, 0.730649, -2.694976, 0.110661, 1.956925, -1.291186, 0.959139, 1.085958, -0.378685, 1.512388, -1.523321, -0.139454, -0.846705, 0.341009, 0.358389, -0.740506, 1.041083, 0.784934, 2.906177, -2.614465, -0.157975},{-0.578777, 0.335455, 0.069346, 1.291555, -0.476698, 0.292759, 0.068559, -1.538806, 0.147781, -0.408179, -0.749545, -0.373395, -0.767765, -1.724753, -0.182269, 1.323035, 0.744854, -0.407527, -1.083435, 0.778741, 0.689629, -0.070336, -1.052613, 0.457913, 0.860812, -0.446411, -1.026280, 0.540300, 0.611674, -0.475018, 0.510931, -0.464830, 0.956634, -0.269145, -1.170009, 0.245573, -0.064999, 1.589215, -0.190207, -1.202263, 0.482919, 0.914705, -1.690733, 0.148160, -1.501599, -0.029010, -0.150702, -1.478569, 0.877956, 1.447280, 0.717154, 1.141217, -0.957709, -0.143149, -1.622086, -0.820784, 1.201362, 1.554414, 0.274075, -0.312616, 0.890145, -1.152088, 1.264518, -0.179120, -0.621203, 0.501398, 0.455952, -1.382032, 1.599600, 0.857979, -0.861248, -0.782086, -0.145926, -0.623095, -0.872666, 0.468458, 0.025343, 0.676740},{0.630869, 1.361622, 0.551291, 0.217364, 0.258372, -0.962110, -0.022286, -0.261300, -0.598886, -1.330832, -0.894702, -0.652642, -0.099808, -1.833369, -0.059764, -1.174862, 0.758519, -2.989707, -2.044434, -0.220977, 0.989683, -0.546654, -0.673001, 0.386387, 1.890718, 0.970213, -0.636520, -0.184031, -0.278589, 0.093394, 0.501293, -0.943898, 0.692222, -1.089700, 0.468792, 0.819225, -0.881480, 2.107422, -1.698656, 0.580416, -0.896883, 2.038809, -1.283714, -2.589068, -1.510426, 1.915163, 1.251527, -0.804762, -0.070336, 2.489976, -0.480629, -0.856608, 2.321511, 0.168973, -0.937636, 2.147454, -0.245685, 0.071986, -0.750559, -1.443997, -0.576127, 0.213640, -0.146753, 0.402949, 0.515759, -1.019864, 1.620954, 1.281906, -0.035540, 1.843459, 0.419734, 1.877984, 0.491204, 0.019840, -0.366525, -0.461757, 0.853036, 1.076478},{0.880503, 1.118570, -0.973003, -0.140090, 0.572820, 0.235366, -0.826071, 1.128426, 0.226783, -0.901150, 0.680800, 0.950553, -0.825801, 0.614470, -0.719709, -0.155438, -0.500777, 0.013213, 0.537529, 0.424580, -1.337690, 0.501443, -0.141112, -0.205390, 1.086797, 0.925826, 0.022356, -0.131586, 0.245773, -0.761831, 0.424656, 0.509330, -0.803145, -0.629440, -0.934487, -1.374545, -0.310835, 1.004194, 0.553959, 0.139536, 0.994725, -0.578964, -0.584849, -1.183949, 1.514623, -1.158255, -1.019897, 0.452364, -0.046893, 0.199709, -2.202540, 0.242659, -0.901972, 1.162286, 0.110419, 0.037152, 1.248708, -0.327737, -0.238130, -0.202651, 1.025429, -0.906369, -0.545580, 1.738122, 0.223076, -0.016390, -0.421710, 0.486121, -0.370049, -0.048237, 0.472878, -0.547666, -0.022949, 0.383079, -1.137960, 1.143410, 1.247409, -0.902289},{0.445107, 0.904523, -2.063876, -0.266745, 1.435716, -1.842561, 0.050693, -0.987615, 2.094664, 0.805058, -0.117784, 1.060757, -1.282456, 2.035109, -0.189771, -1.026767, -0.894124, 0.903200, -1.929660, 0.778197, -0.981170, 1.158349, 0.034062, 0.225151, 0.980185, -1.490921, -0.920485, 1.546378, -1.099554, 0.862813, -0.619338, 0.195248, -0.194275, -0.761748, 0.425736, 1.229420, 0.087565, 1.494840, -0.545941, 0.301762, 0.484387, -0.796379, 1.315292, 0.868127, -0.670268, 0.038801, -1.631387, 0.349895, -1.022633, 1.869284, -1.097474, -0.723226, 0.381166, 1.242048, 0.931462, 2.017329, 0.418147, -0.060274, 0.185900, -0.786929, 0.993491, 2.668876, 1.404199, 1.113004, -1.829271, -0.391303, -0.517596, 0.400608, -1.031619, 1.929277, 0.279169, 1.293739, -0.946321, 1.184536, 1.216106, 1.664302, 1.208317, 0.359290},{0.210792, 0.019387, 0.874901, 0.934729, 2.022212, 1.697233, -0.230759, -0.123840, 0.101883, 1.561894, -1.557099, 0.051336, 2.105036, -0.990840, 0.710724, 0.397408, -0.632204, -0.501858, 1.601861, -0.187982, 0.067708, -1.636841, -0.852801, 0.559366, -1.241914, 0.045080, -0.196946, 0.694086, 0.232430, 2.362604, 0.302771, -0.231930, 0.083580, -0.211251, -1.189186, 1.079952, 1.883131, 0.725560, 1.965799, 1.627857, 0.459686, -1.822687, 1.198380, -0.520900, -0.058361, 0.783270, -1.987664, -0.805884, 2.991848, -1.134293, -1.364693, -0.965548, 0.727799, 0.284808, 0.873554, -0.615295, 0.477410, -1.011898, -0.428968, -2.048506, 0.570177, -0.492482, -1.148250, 0.107811, 1.340064, 0.443979, 0.405453, 0.218154, -0.154585, -0.145221, 0.230671, 0.624436, -0.256195, -1.041303, 2.846250, -2.022366, 1.145979, -0.829595},{-0.379466, 0.814787, 0.361721, -0.247429, 1.483541, -1.469791, 0.857694, 0.225156, -0.108526, -0.087399, 0.669037, 1.013206, 0.129970, -0.695610, 0.396144, 0.296721, 0.645561, 0.080729, -0.241488, 0.909947, -0.204622, -1.014648, -0.178049, -1.128562, -0.288601, 0.480170, -0.302001, 0.238529, -0.030641, 0.869988, 0.576702, -1.704545, 0.360658, -0.286660, -1.198615, 0.050847, 0.788752, -0.113413, -0.831197, 0.648812, -1.189588, 0.680168, -0.829071, 2.570808, -1.362277, 0.043732, -2.941915, -0.613762, -1.245725, -0.559419, -0.576607, 0.000569, 1.998838, -0.215724, 0.766240, 0.082043, -0.606243, 0.912831, -0.146728, 1.911245, 1.323091, 1.555678, 0.492695, -2.270365, 0.670912, 1.066266, 1.242650, 2.746320, -1.789452, -0.687636, 0.625504, 0.306455, 0.820588, -0.435461, 0.559050, 0.323780, -0.864998, 0.330052},{0.975122, -0.403475, -0.375287, -1.011722, 0.379001, -0.474282, 0.187533, 0.609411, -0.902823, 0.859663, 0.469617, 1.056008, 1.732904, 0.226569, 1.414662, -1.010345, 1.290033, -0.353352, 0.327700, -0.510259, 0.104484, -0.076207, -0.142420, -0.716259, 0.018585, 0.490649, 0.825604, -1.785343, 0.083275, -0.298538, 0.631974, 0.506919, -0.682243, 0.033992, 1.206915, -0.884927, 1.221419, -1.586492, 0.999116, 0.985117, 0.495032, -1.243849, -0.211895, -1.735193, -0.160498, 1.605862, 1.514397, -0.454523, 0.351691, -0.676189, -1.581426, -0.529694, -0.649477, -0.938153, 0.613976, 0.080400, 0.284079, 0.753731, 0.860301, 1.050168, 0.875245, 1.213437, -0.940762, -1.171113, 1.250992, -0.484158, -0.467489, 1.805179, -0.661961, 0.475468, -0.553469, 1.231907, -0.455197, 0.641444, 0.441321, 0.468559, -0.186528, 1.018898},{-1.182354, 0.043528, 0.284658, 0.240840, 0.295771, 0.853119, 1.343119, 0.592231, 0.914611, -0.726719, -1.332725, -0.185716, -0.572342, 1.121920, 0.003026, -1.699135, 0.355682, -1.157104, -0.122193, -0.789804, 1.099955, -2.184692, 0.084590, 0.352557, 2.008606, 0.091213, -1.864995, 0.329564, -0.784284, -1.208653, 1.130964, -0.987840, -1.191723, 0.133939, -0.729299, 0.693727, 0.650962, -0.210224, 0.733989, 0.301483, -1.886991, 2.437260, 1.031168, -0.680371, -0.176617, 0.157600, 1.035659, -0.129175, -0.532239, 0.372126, -0.276653, 0.902267, -1.016674, -0.589196, -0.480704, -0.319919, -0.356337, -0.216393, 0.753441, 2.355780, 2.003791, -0.575927, -0.083193, 0.802104, -0.544796, -0.538805, -0.360381, 0.561897, -0.041235, 0.149734, -0.151952, -1.109240, 0.151013, -0.417620, -0.067848, -0.697148, -0.469023, 1.242806},{0.776345, 0.309900, -0.628489, -2.088015, 2.086120, 0.182454, 1.360773, -1.275107, 0.451050, 0.423524, -0.472080, -0.133054, -0.002711, -1.559287, -0.055662, 1.102350, -0.473901, -0.080495, -0.934942, 1.891764, -0.138069, -2.054375, 0.226463, 1.847722, 1.308412, -0.988545, 0.186662, -0.967465, 1.521419, -0.188536, -0.268809, 0.890513, 0.088851, -1.181909, 0.520046, -0.882776, 0.130546, -0.004246, 0.356835, 0.290211, 2.603014, -1.907549, -0.355824, 0.904417, -1.500336, 0.374263, -1.342175, 0.547469, 0.123565, -0.406000, -0.681886, 0.575759, -1.128656, 1.449183, -3.087950, -0.119665, -1.770956, -1.271232, 0.149877, 0.240276, -0.408288, 0.211819, -0.695825, -0.167700, -0.099988, 0.047430, -0.756795, 0.029964, -1.633706, 0.189575, 0.395568, 2.417529, -1.386399, -1.839621, -0.370641, -0.347290, 0.332396, 0.453145},{-0.591381, -1.723801, 0.070540, 0.564864, -0.338965, 0.240046, 1.396570, 0.023786, -0.809936, 0.782446, -0.321654, 0.059029, -1.275387, -2.054874, 1.325250, -0.834338, 0.453042, 0.522373, -1.111705, -2.887964, -0.386989, 0.903219, -1.595338, -0.157470, 0.163037, -0.696528, 1.068521, 0.584761, -0.373473, -0.533508, 2.388319, 0.214622, -0.419841, -0.625280, -0.028616, -0.953855, -0.351758, 0.960595, -2.163608, 0.318559, 0.583457, 0.354415, -0.320539, -0.472057, 0.467412, -0.968705, 0.714846, -0.429721, -0.326800, 0.108046, 0.218800, -0.232657, 0.420850, 1.302616, -0.418066, 0.144698, -0.698706, -1.351762, 1.848520, 0.338700, 0.546027, -1.313188, 0.485486, 0.779810, 2.534519, -0.924336, 0.067289, -0.871993, 1.023191, 0.806528, -0.444217, 0.467601, -0.185450, 0.074098, -1.009177, -1.074082, -0.204292, 0.063667},{-0.772620, 1.195951, 0.325785, 1.873059, 0.216226, -0.446404, 0.470743, -0.619840, -1.050038, 1.481652, 0.052564, -0.524077, -0.891675, 1.183125, -1.331380, -1.539743, 0.540022, 1.522215, -0.505346, -0.324378, 0.008400, 0.003152, 0.968918, -0.481465, -0.458291, -1.020286, -0.923809, -0.811927, 0.672986, 0.499443, -0.567898, 0.538265, -0.372215, -0.959050, -0.410995, 0.376689, -0.674843, -0.597126, 0.430740, 0.139216, -0.178407, 1.114714, -1.641691, 0.241645, 1.291611, -1.136954, -1.155213, -0.124863, 1.374605, -2.189498, 0.040301, 1.417807, -0.720409, -0.464347, 0.465496, 0.486095, 0.891586, -1.488479, 0.306497, 0.243759, -0.210267, -0.519599, -0.327516, -1.141743, 0.994226, -0.132314, -0.019642, -1.729267, -0.371179, -1.956470, 1.456836, 1.226711, -0.758301, -0.821492, 1.238972, -1.745955, 0.039217, 0.861809},{-1.191347, 0.380755, -0.546730, 0.182943, 0.143366, -0.108433, 1.152139, 0.712013, 0.032343, 0.052727, 0.114192, -0.076423, 1.100059, -0.018238, 0.843226, 0.104133, 0.526392, 0.428608, 0.899766, -0.175347, -0.488399, -1.041957, -0.968528, 0.459914, 0.513716, 1.743765, 0.159925, 0.953640, 0.224377, -1.166334, -1.927885, -2.144683, -0.327609, 0.663378, -0.284157, -0.259382, -1.105913, 0.192346, 1.661437, 0.069658, 0.103307, -0.410139, -0.371862, -1.997671, -0.079257, -0.126043, -1.128464, -0.160047, -0.947653, 2.115855, 0.724373, -0.024487, 0.352835, -0.387454, 1.691336, 1.524753, -1.962590, 1.348967, 0.573093, -0.837723, -1.243220, -0.773450, 1.261508, -0.766701, 0.530791, 0.604253, 0.654671, 2.359637, -0.432609, 0.702320, 1.011183, 0.415871, 0.805811, 1.198364, 2.167190, 1.471740, -1.310436, -0.018470},{-0.858046, -0.210489, -0.850454, 0.017412, 0.763175, -0.249210, -0.938134, 1.962730, 1.570657, 0.478572, -1.148433, 1.249856, 2.066736, 1.078275, -0.987372, 1.229732, 0.631897, 0.710800, -1.008112, -0.670473, -0.582460, 0.923511, 0.660626, -0.003947, -0.633648, 0.212205, 0.815529, -0.796530, -0.529983, -0.575050, 0.865111, 1.083316, -2.218682, 1.438208, 0.204853, -1.115459, 1.493204, 0.922322, -0.055208, -0.212955, -0.071073, 0.915020, -0.523778, 0.600565, -0.238487, 2.120426, -1.442458, 0.203323, 0.205165, 0.904297, -0.745728, -0.888842, -0.803109, -1.042178, -0.333456, 1.985307, -1.936081, 1.006722, 1.074264, -1.018644, -0.663288, 0.326097, 1.030615, -0.095577, 1.489089, 1.616073, -0.541701, -0.890580, -2.313391, 0.607341, 1.206171, 0.570835, 0.464431, -0.711717, -0.658710, 0.941492, -0.341014, -0.216637},{0.423437, 1.772101, 0.353902, -0.087536, -0.111918, -0.258213, -0.005598, 0.233125, -0.983122, -0.003048, 0.569809, 1.241722, 0.401575, -0.208447, -0.071689, 0.756680, -0.201122, 0.064043, 0.464749, 0.031228, -1.258575, -0.209970, -2.017025, -1.283914, -0.406325, 0.215327, 0.139984, -1.056282, 0.808856, 0.235793, 1.183224, 0.993065, -0.941250, 0.155129, -1.079411, 0.234338, 0.354388, -0.219330, -0.233033, -2.639669, -1.092192, -0.266399, 0.027676, -0.860589, 0.174806, -0.422401, -0.412683, 2.203152, -1.485297, -1.725466, -0.143766, 0.607641, -1.262745, 1.960202, -0.462398, -0.562197, -0.168248, 0.314617, 0.050623, -2.762232, 0.746062, 0.196199, -0.153114, -0.092075, 0.309888, 0.808767, 0.739867, -1.945850, -0.622625, -0.868536, 1.406430, 0.910652, 0.679587, 1.290995, 0.503931, -0.915739, -1.493959, -1.785473},{1.434076, -1.155161, 2.053330, -1.329836, 0.374430, -0.506483, -0.085128, -1.453466, -0.526602, -2.056782, -0.699883, 0.169066, -2.181543, 0.445284, -3.131989, 0.371916, -1.129352, -0.616126, -0.198774, 0.147025, -0.254824, -1.385302, 0.700785, -0.161125, 1.124841, -0.265009, 0.778468, 0.029326, 0.941385, -1.007316, 0.364800, -0.167619, -0.633803, 1.051333, -0.398585, -1.215864, -0.177079, -0.357182, 0.518057, 1.764478, 0.853427, -0.811672, -0.257426, -0.478651, -1.391984, -0.142073, 0.190715, -2.360342, 0.794392, 1.075272, -0.657750, -0.666323, -0.886394, -3.497521, 1.233659, -0.806355, 0.402034, 0.451338, -1.356801, -1.047510, -1.085021, -0.489720, -0.702307, 1.193811, -0.220836, -0.810124, -1.397459, -0.920466, 1.389947, -0.502998, 0.002105, -0.060628, 0.593356, 1.539485, -0.540866, 1.586104, -0.319970, 0.691227},{0.867181, -0.514229, -0.478949, 0.830346, -0.264724, -0.226133, -0.524905, 0.752796, 1.051551, 0.238396, 0.460194, 0.270001, -0.571642, -1.705715, -0.538245, -0.592139, -0.093965, -0.957227, 1.130451, 0.036699, 2.786130, -0.502819, -0.454339, 0.223091, -0.024846, 0.949665, 0.565526, 1.957858, -1.083974, -1.030736, -1.455664, -0.110398, -0.418832, 0.189543, 0.834597, -0.202355, -0.493636, 0.873602, -1.202587, 1.282107, -0.748779, 0.156925, 1.061163, 0.737863, -1.509148, -0.453983, -0.945892, 0.481245, -1.960034, 0.566344, -0.386848, -0.105557, 0.348690, 0.524191, 0.578211, 1.389907, 0.519662, -0.985137, -1.090282, 0.098810, -0.222000, -0.253632, -1.144486, -1.120804, 1.009527, 0.033706, 1.468290, 1.044908, -0.243671, 0.625704, -0.450761, 0.848470, 0.722420, 0.820667, 0.926827, 0.833493, -0.631604, -0.003706},{-0.014628, 0.898974, 1.627495, -0.663821, -1.585292, 1.239842, 0.250659, 0.962480, -0.713591, -1.013000, 0.449939, -0.043958, -0.086815, 0.192936, 0.871494, 0.724496, 1.634218, 0.187964, -0.381943, -0.447374, -0.903242, -0.143090, -1.403320, -0.233493, -0.163171, -0.651942, -1.041157, -0.618101, -0.119712, 1.499343, 0.914824, -1.016764, 0.921555, -0.402509, 1.918241, 0.457198, 1.490349, 2.129340, 0.311790, -0.166701, -0.126381, 0.336230, -0.933939, -0.577463, 0.316021, 0.209133, 0.899294, 1.140451, 1.847731, 0.709457, 1.533575, 1.130373, -0.616186, 0.550978, 0.063265, -0.464824, -1.436566, 1.260388, 0.012672, 0.386086, -0.377777, 0.795964, -1.617597, 0.561639, -2.213909, 0.676307, -0.989601, 1.749222, 0.459950, 0.486048, 0.416955, 0.268324, -0.708715, -0.399650, 0.614284, 1.295991, 0.332509, 0.902321},{0.989159, -0.397065, 1.190931, 0.991345, 0.994644, -0.373745, -0.041881, -1.028907, 0.613803, -0.924512, 0.701619, 0.608576, -0.388503, 0.277274, 1.175010, -0.340871, 1.022648, -2.238746, 0.473784, -0.091464, -0.526066, -0.272286, -0.638113, 0.528609, -0.850492, -1.397429, -1.103941, -0.996823, 1.512725, 0.322932, -0.008444, 1.188775, 0.729195, -0.136789, -1.423201, -0.840724, -2.483371, -0.195142, 0.859227, 0.580763, 0.282185, -1.034235, -0.216774, -0.621513, -0.362041, -1.071106, -1.846369, 0.315475, 0.192589, 0.378145, 0.399707, 1.787190, -0.348751, -0.428766, 0.370631, -0.229214, 0.008882, 0.398527, -0.233006, -1.822067, 0.354324, 0.263728, 0.098502, 0.968631, 1.567161, -0.477883, 0.649977, 0.599241, 0.733654, 0.392107, 0.059109, -0.775886, -0.906805, -1.495412, 0.598674, 3.944621, -0.983965, 0.296145},{1.684057, -0.307844, 0.391844, 0.492568, -0.098373, -1.816175, -0.630839, 0.227349, -0.019902, 0.324241, -0.046857, 0.403052, 0.761749, 0.820773, -0.305980, -0.690187, -0.060884, -0.961080, 0.156009, 0.449925, -0.549179, -0.088849, -0.347822, 0.469645, 1.140589, 1.672396, -2.219617, 1.379911, -0.477655, -0.301291, -0.345270, -1.236956, 0.221243, -1.181436, 1.306029, -1.665637, 0.123625, -0.775606, 1.244047, -2.162536, -0.288873, 0.381285, 0.594033, -0.892456, 1.848087, -2.088528, -0.713142, 0.544676, 1.383695, -0.299216, -1.693797, -0.135344, 1.343662, 0.997929, 0.875426, 0.924523, -0.281097, 0.129425, 1.525920, -1.455992, 0.885315, 0.817019, -0.524839, -1.104063, -0.086296, 1.143103, -1.571583, 1.849601, -0.574820, -1.515296, 0.029144, -0.485324, 2.024616, -1.574271, -1.627284, -0.616871, 0.329636, 2.986667},{-1.128049, 1.144406, -1.975471, -0.579089, -0.693790, 1.708831, -0.235461, -0.678701, -0.362137, -2.687723, -0.694183, -0.690456, 0.350049, -0.954475, 1.088899, -0.565350, -0.542865, -0.226170, -0.631237, 0.614432, -1.730364, 0.930721, -1.825952, 1.357822, 1.861388, -0.597955, 0.324118, 0.968396, -1.463482, -0.206626, 0.045852, 1.107194, -1.070880, 1.306588, -1.927081, 0.174838, 0.497703, 0.261415, 0.686261, 0.107474, 1.093182, 0.079490, 0.723455, 0.258828, 0.039537, -1.307323, 0.066048, 0.064958, -1.479710, -1.175453, -0.021354, -0.217734, -0.310514, 0.651580, 0.143725, 0.011268, 1.034600, 0.310827, 0.096461, 0.311600, 0.570684, -0.125653, 1.949940, 0.793091, -0.188404, -0.763152, -2.808376, 1.013295, -0.639999, 1.553380, 0.963019, 0.809964, 1.084333, -0.094218, 0.350878, -0.320370, 0.194612, 0.642167},{1.486502, 0.608281, 2.312022, 0.836206, -0.792548, 1.031854, 0.055461, -0.848927, 0.310347, -0.730836, -0.077647, -1.713428, 0.345964, -1.191804, 0.967254, -0.327032, -0.499091, 0.586965, 1.205653, 2.118273, 2.208092, 0.571431, 2.151517, 1.292709, -0.423484, -0.871735, 0.839882, -1.734413, 0.845020, 0.321264, 0.633166, -1.113042, -1.334233, 0.062540, -0.124516, 0.108115, 1.672574, 0.194608, 0.053644, 2.258695, 0.099117, 0.669932, -1.774575, -1.188994, 0.252158, 0.764137, -0.236198, -0.213101, -1.149129, -1.363198, -0.434583, -1.355503, 1.651420, -0.019677, 0.415735, 1.248461, -0.139416, 0.569835, -0.046812, 0.881783, 0.901654, -1.000730, -0.411442, -0.820287, 0.442179, 0.403595, 0.373823, -0.317865, 1.451401, 0.363834, 1.618267, -0.238147, 1.153136, 0.223654, -0.214702, -0.073886, -2.917557, -0.681874},{-0.570539, 1.859103, -0.447125, 0.024470, 0.845450, -0.831003, 0.265627, -0.670819, -1.125770, -0.636777, -0.117363, 1.136762, -1.206691, -0.014896, 0.865278, -0.147493, -1.130158, 0.091628, -0.394157, -1.774417, -1.043629, 0.775153, 0.019888, 0.332094, -0.700428, -2.007447, 0.493744, 1.145399, -0.721618, -1.166711, -2.018121, 0.979039, -0.240484, 0.287500, 2.223531, 0.047924, 0.367498, 1.158687, -1.018313, 0.278059, 0.810903, 0.320220, -0.067492, 0.735233, 0.808835, -0.768729, 1.786624, -1.262917, -1.312465, -0.236620, -0.317382, 0.021727, 0.035478, -1.612134, -0.919350, -0.228888, 0.589098, 1.188244, -0.588637, -1.877624, -0.346104, -0.366276, 1.259363, 0.401519, 0.476972, 0.310055, 1.870331, 0.370772, 0.547462, 0.254106, -0.914389, 0.476661, 0.838850, -0.186169, 1.432382, -0.013030, 0.187846, -0.354662},{1.571913, -0.699921, -1.198466, 1.474967, 0.206494, -0.370685, 0.054887, -0.998439, 0.914253, 0.616251, -0.760703, 0.529866, -0.585261, 0.126028, 0.445938, 1.025604, -1.064618, -0.325709, 0.936031, 0.914637, -0.837575, 0.639758, -2.153820, 1.274070, -0.894514, 0.296408, 1.080097, -0.400476, -1.608662, 0.672483, -1.709075, 0.269255, -0.747872, 2.062528, -0.821912, -0.084881, 1.131074, -0.425486, 0.596921, 0.303592, -0.083946, 0.282675, -0.618674, -1.061708, 0.981967, 0.872836, 0.762448, 0.917546, -1.346654, -0.088807, -0.333608, -0.215036, 0.649919, 0.067699, 0.997240, -0.240748, 0.733453, 0.880302, -1.302451, 0.643330, 0.617380, 1.820862, 1.569770, 1.326247, -0.756682, 0.691076, -0.787981, 0.322263, 0.410381, 1.058441, -1.961616, 0.184371, 0.419863, -0.810355, 0.526590, 1.479542, -0.279526, 2.229871},{-0.348576, 0.195294, 2.151180, -1.199138, -1.751091, 0.622836, 0.725597, -1.558304, -0.616290, 1.070312, 0.206042, 0.531012, 1.394249, -0.805146, -0.788692, -0.155496, -1.500793, 0.411407, -1.765941, -1.228922, -1.192093, -0.967608, 0.841943, -0.717394, 0.354969, 0.385537, -1.626024, -0.262633, 0.480359, 0.391445, 0.667288, -0.089310, -0.204973, -0.957933, 0.404987, -1.060212, 1.165045, -1.832433, -0.510226, 0.354464, 0.067130, -1.031231, -1.910551, 0.917305, 0.548521, -0.788813, 0.289392, -0.213483, 0.373829, 0.412610, 0.160829, -1.619625, -1.323000, -0.714167, -0.585259, 0.674367, -0.437527, -0.045565, 0.360781, -0.644037, -0.971279, 0.344031, -0.077602, -0.396383, -0.203576, 0.232285, -1.223718, 0.079247, 1.045258, 1.070983, 0.433679, -0.520020, -0.375696, 0.725186, 0.995028, -0.160845, -1.147600, -1.607567},{-1.462669, 0.933219, 0.283915, -0.754110, -0.174514, 1.063161, 1.422075, 0.353689, 0.439723, 1.373372, 1.487228, -2.014307, -0.045990, 1.139463, -0.539631, 1.145980, 0.451177, 1.615138, 1.009990, -0.367827, -0.110511, -1.486586, -0.785604, 0.191848, 1.000266, -0.550491, -0.762946, 0.585516, 1.167258, 0.026199, -0.906726, -0.660338, 0.829590, -0.772015, 0.951680, -0.915490, -1.056391, -1.278543, -0.325441, 0.795969, -0.310077, 2.022812, -0.631895, -0.446993, -2.332608, -0.416918, 0.367978, -0.146786, -0.420480, 1.344020, 1.390585, -1.154583, 1.538371, 0.191979, -2.432127, -0.583281, -0.748674, -0.214687, 0.514363, 0.787828, -0.141573, 0.810334, -0.497639, -1.161045, -1.591809, 1.442048, 0.551518, -1.102335, -0.339587, -0.360312, 0.454755, 0.134590, 0.474601, 2.449597, 0.808825, -1.071493, -1.610441, -0.583776},{-1.268035, 0.872909, -1.900590, -2.215249, 0.043389, -2.110371, -0.135181, -0.374833, 1.482455, -2.164042, 1.834209, 0.559055, 0.917589, -1.206475, 0.276386, -0.319039, -0.795762, -0.749732, 2.157323, -0.115076, -0.370712, -0.776414, 0.858857, -0.264081, -0.137182, -0.384532, 0.658867, 1.724203, -1.092240, -0.215786, 0.293937, -0.859871, 0.906471, 2.468228, -0.011868, -0.126338, -0.205754, -0.545732, -1.375294, -0.733375, 0.567236, 2.419546, -0.569566, 3.320449, -1.688753, 0.703441, -1.110011, 1.768195, 0.319263, -0.116414, -0.280710, -1.510105, 0.665367, 0.342949, 0.271894, -0.541348, -0.498002, -0.457650, 1.250892, -1.397982, 1.444613, -0.446458, 0.915779, 1.260466, -0.252033, 0.388977, 1.541952, -1.318962, -0.859628, -0.859123, 0.422976, -0.419276, -1.155066, 0.762155, -0.319144, -1.458818, 0.803509, -1.485738},{-1.445401, 2.753147, 0.390295, 2.158052, -0.266906, -0.851626, 1.657161, -2.018899, -1.472551, 0.016695, -0.779971, 0.794040, 0.953045, -0.211515, -0.375386, -0.505663, -0.556825, 0.457157, -0.556964, -1.131812, -1.779226, 1.483491, -0.655603, -0.818663, 0.704441, -0.449464, 0.850998, 0.953479, -1.733655, -1.213239, -1.559401, -1.016191, -0.279201, -1.263591, -0.009334, 0.697464, -0.689409, 0.227690, -1.571676, 1.056545, -0.090856, 0.237080, 0.103587, -0.778284, -0.638277, 0.942524, -0.251502, 0.294814, 1.322339, 1.624848, 0.776676, -1.423922, 0.072385, 1.151163, -0.803840, -0.248967, -0.560337, -1.432332, -0.187838, -0.664796, -0.036128, 1.494484, -1.031050, 1.009390, -0.238288, 0.828026, 0.381879, -1.823962, -0.099254, 1.422616, 0.640990, -0.779881, -1.292453, -0.702617, 0.253452, -0.458961, 0.585061, -1.269311},{0.592685, 1.257264, -1.163291, 0.363354, -0.028025, -1.650807, 1.356889, -0.540199, 0.289538, 0.358096, -1.000149, -0.671610, -0.624637, 0.052970, 0.451850, -0.351782, 0.781579, 0.296663, 2.131403, -0.999305, 0.198330, -0.611133, 0.597913, -1.906659, 0.242120, 1.476787, 0.791166, -0.784278, 1.654789, -0.948495, 0.758164, 0.545745, -2.675354, 0.849469, -0.567548, 0.709630, -0.033124, -0.347422, 0.334925, -0.049703, 0.263652, 0.447323, 0.662952, 1.206313, 0.033513, -0.927357, 0.428809, -0.019172, 1.423392, -0.617120, -2.348004, 0.315959, 1.411921, -1.990565, 0.373556, 0.625635, -1.187218, 0.293534, -1.072080, -0.196976, 1.025778, -0.418616, 0.316386, -0.063855, -1.081687, 0.149680, 1.483993, -2.319444, -0.669705, 1.500809, -2.072020, 0.801497, 0.346664, -1.252133, 0.951619, 0.140921, 1.641627, -0.854445},{-0.847135, 0.811482, -0.638187, 0.541822, -0.256911, -0.805397, -0.297458, 0.746416, -2.734893, 0.693750, 1.261694, 0.973147, -0.121250, -0.992770, 2.543199, -1.388786, 0.805448, 0.268633, -1.668568, -0.365845, 0.526752, -1.271700, -1.244410, -0.761224, -0.446023, 0.808586, 0.717975, -0.010674, -1.059707, -1.222289, -0.737021, -1.057410, -0.992836, -0.141501, -1.221997, 0.272050, -1.172571, 0.012994, 1.214019, 0.033444, 1.137078, 0.548929, 0.998292, 0.193908, -3.150311, -0.645565, -0.655846, 1.022436, -1.479536, -0.730447, -0.837935, -0.192022, 0.882660, 1.830319, 0.721263, 0.210503, -0.425661, 0.375609, -0.066160, -0.392602, 1.500444, 0.229029, 0.143432, -0.538789, -1.287835, -0.837166, -0.696291, -0.173438, 0.340517, -0.441466, 0.806406, 1.432618, -1.463922, 0.206763, -0.207811, -0.542350, 0.863595, 1.616390},{2.296589, -0.232655, 0.428047, 0.540589, -0.038271, -0.706629, -0.234714, -0.687445, 1.446272, -0.023066, -0.072410, 0.112657, 3.211666, -0.226626, -0.263490, 0.869973, 1.135507, 0.422362, 2.522375, -0.661814, 0.262619, -0.911146, 0.679464, 1.030251, -1.750711, 0.135332, -0.588087, -0.604785, 0.911956, -0.091061, -0.053950, -0.038409, -0.961490, 0.243048, -0.262908, 0.392571, 0.909000, -1.203255, -0.730222, 0.914864, 1.157443, -1.804836, -1.951729, 0.653231, 0.796593, 0.398829, 0.620307, 0.078774, 0.075443, 1.133625, -0.424753, -1.694643, 1.099276, 1.409298, 0.101857, -1.087760, 0.946959, -0.155453, 0.375569, -0.828011, -1.748585, -0.625420, -0.411790, -1.188691, -0.501569, 1.451972, -0.127939, 0.151604, 0.728836, 0.150729, 0.159112, 0.821123, -0.327478, 1.144750, 0.131141, 1.034614, 1.651182, -0.593334},{-0.416595, -0.815932, -0.088858, 0.829427, 0.427715, 1.539443, 0.658596, 1.586463, 0.213589, -0.566613, -0.139856, 0.013293, -1.507220, 0.258788, -0.422709, 0.683542, 1.392667, -1.634890, 0.286229, -1.262881, -1.048266, 1.746647, 0.586736, 0.796583, 1.068416, -0.311670, 1.075858, 0.811277, 0.690942, 0.204385, 1.687477, -0.030165, 0.577408, 0.444152, 1.295205, 0.028406, -1.260999, -1.176259, -1.198940, -1.898483, -0.266188, -0.398690, -0.362074, 0.659312, 0.466116, -2.730512, -0.651358, 0.871826, -0.335117, -0.106346, -0.772596, 0.787721, 0.816931, 0.530239, 0.250207, 1.517990, 0.435731, -0.610925, 1.041349, 1.202985, -0.726756, 0.475388, -1.092080, 0.938056, -0.753347, 0.239133, -0.117906, 0.729819, -0.500518, 0.137273, -1.114974, 0.644895, 0.187643, 0.229591, -1.701342, -0.868641, 1.754133, -0.644209},{1.643560, 0.113992, 0.990151, 0.185589, -0.551511, -0.850777, -0.224186, 0.050495, -0.387724, 1.614498, -0.031505, -0.147290, 0.832445, 0.043661, -1.435335, 1.741146, -2.532398, -1.044450, -2.781686, 0.674575, 0.371183, 0.463920, 0.527708, 0.469488, 0.922782, -0.444329, -1.095733, -0.309510, 0.658842, -0.652435, 0.910719, 0.911893, 0.178089, 0.627718, -0.824083, -0.600030, -0.283319, 1.413638, 0.467599, 0.820370, -0.189952, -0.643897, -0.127382, -1.681134, -0.832365, -1.894690, 1.665868, -0.103744, 0.109715, -0.469433, 0.325322, 0.250661, 1.251460, 0.347348, -0.987100, -0.591002, 0.217547, -1.156849, -0.089056, -0.753817, 0.729130, -0.594419, -0.185809, 0.215196, 1.265103, 1.135083, 0.618670, 2.737352, 1.826922, -0.356013, -0.247018, -0.630110, -0.849018, 0.472633, -0.569283, -0.445955, -0.614918, 1.112959},{0.617452, 0.940644, -1.027104, 2.079533, 0.530562, -0.472602, -0.108188, 0.726510, 0.314061, 0.017496, 0.040038, 0.453271, -1.024143, 0.094994, 1.832128, 0.385556, 1.226954, 2.015981, 0.724556, -1.503729, -0.819505, 0.862690, 2.005968, 0.351914, -0.029432, -1.281865, 2.693514, 0.125656, -0.804003, 0.989003, -0.654492, 0.133333, -0.440170, -0.896229, -1.089419, 1.133876, -0.754079, 1.326875, -1.205628, 1.040965, -2.043822, 0.977666, -0.611716, -0.110603, -0.238342, -0.588685, 1.166345, 1.937604, -0.103627, -0.617472, -1.397043, -1.247772, 0.164494, -1.148515, -0.574031, 0.686697, 0.146263, 0.593679, -1.655179, 0.749818, -1.035771, -2.698289, -0.599956, -0.557618, 1.164202, 1.145146, -2.704873, -0.294509, -0.389586, -0.070468, -0.734687, 0.586467, -1.354752, -0.338065, -0.758584, -0.452694, -0.859420, -1.506216},{0.167668, -0.720612, 1.701294, 1.010687, -0.269433, -0.008995, -0.297460, 0.487705, 1.805989, 1.019618, 1.521424, 0.309826, 1.974606, 0.478186, -1.092935, 0.478865, -0.960661, 0.493517, 0.532548, -0.049801, -0.594134, 1.370159, 1.096162, 1.257859, 0.290485, -0.797927, 0.182125, 0.223099, -0.258411, 0.310590, 0.714623, 0.573570, -0.279397, -0.193720, 0.097048, 0.395541, 1.022016, 0.828922, -0.883545, -1.281424, -0.812466, 2.159965, -1.332307, -1.179505, 1.295549, -0.210072, 0.943571, 2.439837, 0.073246, -0.332668, 0.977663, 0.483723, -1.421272, -1.400491, -1.734580, 1.771429, 0.305688, 0.460263, 0.489765, -0.408156, -1.106434, 0.059556, 0.259375, -0.314146, -0.398904, 0.169849, 1.372172, 0.794108, 0.547180, -0.883790, -0.795595, -0.090700, 1.944000, -0.089833, -0.653992, -1.204490, -0.793882, -1.415959},{0.665649, -0.243425, -0.104831, 0.110623, 1.007042, -1.267614, -0.026957, -0.175685, 0.538826, -1.242778, 0.909513, 0.862862, -0.623465, -0.297105, 1.170336, 0.510873, 1.479468, 0.166355, 0.152822, 0.837679, -0.269413, 2.126364, -1.849049, -0.201144, -0.145266, 1.207651, 1.239675, -3.178010, 1.626481, 0.070327, 1.104458, 0.731177, -0.615266, 0.004340, 1.082908, -0.124870, -0.453285, -0.388894, -0.652087, 0.508236, 1.072848, 0.243760, 1.565831, -1.176794, 1.114644, 0.606713, -0.361032, -0.782135, -1.285069, -2.640120, -0.014914, -0.316420, 0.867736, 0.320252, 0.718569, -0.739692, 2.249219, 0.295256, -0.345247, -0.780478, -1.905518, -0.616171, 0.614969, -0.797604, -0.411064, 0.119905, 0.722587, -1.013659, 0.114817, -0.622179, -0.949547, -1.159589, -0.200828, -0.296060, 0.003698, 0.958149, 0.572162, 0.799012},{0.142937, 0.046164, 1.913782, -0.437405, -0.260878, -0.250062, 0.086132, -1.846596, -0.136825, -0.365601, 1.115564, -0.232446, 0.765596, 0.883962, 0.398818, -0.385987, -1.728977, 0.379136, -0.602426, 0.333531, 1.084109, 1.328833, -0.483143, 0.861314, 1.326448, -0.278194, 0.850220, -1.202668, 0.028106, -1.581047, -0.045289, -1.661748, -1.084572, -0.296996, -0.952238, 0.332990, 0.164325, -0.384930, 0.363891, 1.479638, 0.312335, 0.117652, -0.596593, 0.575861, 1.472779, -0.113379, -0.042242, 1.891640, -0.588673, 1.411694, 0.958179, -0.374389, 0.109390, 2.213268, -0.638168, 0.155601, -0.789233, 0.792162, 0.465812, 0.668781, -0.670691, -0.037438, -1.381326, -1.351985, 0.393686, -0.628862, -0.463498, 0.467548, -0.535834, 0.512380, -2.225602, 1.165476, -0.331757, 0.329954, 1.426494, -0.320651, -0.048948, -0.865518},{-2.359821, -0.275659, -0.159050, 1.409336, 0.128106, 0.613730, -0.423656, 0.542589, 0.377200, -0.915690, -0.947002, 1.657303, -2.298333, -0.135173, -0.180984, -0.636578, 1.736782, -0.882576, -1.104188, -0.271956, 0.415120, -0.545525, -2.104178, -0.162905, -0.863597, -1.208923, 0.145465, -0.513326, -0.403183, -1.459538, -0.925732, -0.147985, -2.339366, 1.552124, 2.314989, -0.565236, -0.741182, 0.049919, -1.417893, 0.030708, -0.759436, 2.506826, -1.208159, 0.278875, -0.095819, 0.279476, -1.439439, 0.317172, 1.005043, -0.638486, 0.879637, -0.096520, 1.703714, 0.744356, 0.016501, -0.316953, 0.157235, 1.848551, 0.589244, 1.163924, -0.598148, 1.375871, -0.475252, 1.504903, -0.173289, -0.538485, 0.051901, 0.277453, 0.291315, 2.565681, -1.628928, 1.117381, -0.339182, -0.548542, 0.390176, 1.277677, 0.300794, 0.593709},{0.302758, -0.555702, -0.449413, -0.769380, -0.805025, -1.123601, -1.106174, -0.939069, 0.094469, -2.463345, 0.316598, 1.819435, -0.404853, 0.275708, -0.019458, 0.085577, -0.184489, 0.445547, 1.386632, 0.572663, 0.431821, 0.338018, 0.681077, 0.487054, -0.440609, -0.924082, -0.305806, -0.110712, 1.459340, 0.837717, 0.369335, 0.323890, 1.524631, -0.969291, 0.205018, -0.446271, 0.956709, -1.050061, 0.594464, 0.855298, -0.483503, 0.929135, -1.181348, 1.798926, 0.272018, -0.456172, -1.401883, 1.138308, 0.203525, 1.538841, 1.336019, -0.047619, -0.026122, 0.270943, 0.782536, -1.520318, 0.391726, -0.650015, 1.731043, 0.602306, -0.550590, 0.158004, -0.982252, 0.191286, -0.506956, -1.390587, 0.191813, -1.333242, 0.658616, 0.534268, -0.539474, -0.108722, -0.210851, -2.410970, -1.193318, 0.085556, 0.388745, -0.075539},{0.762046, 1.378344, -1.554233, -0.542176, -0.657796, -0.774346, 0.017556, 0.375967, 1.042215, 1.117052, 1.361496, 1.318228, 0.219555, 0.687370, 1.647828, 0.626290, 0.201283, -1.076101, -0.110420, 1.514143, -1.244039, -0.396053, 0.716386, -2.369796, 2.154731, -0.029069, -0.555342, -0.412588, 0.705493, -0.423397, 0.238552, -0.389077, 1.854273, -1.007785, -0.361078, 0.270589, 1.296059, -0.913069, -0.193053, -1.115320, 0.902677, 0.889374, -0.467794, 1.039435, -0.437353, -0.083727, 0.183956, 0.804580, 0.878164, 1.048385, -0.730753, 0.749730, 0.111451, 0.236111, -0.003455, -0.980284, -0.316911, -0.124963, -0.300961, -1.178450, 0.428757, -0.294958, 0.037702, -0.563473, -2.072316, 0.649342, 0.080406, -0.548867, -0.721875, 1.247900, 1.922951, -1.642023, -1.306450, -1.862579, -0.644888, 0.309908, -1.161236, -0.012729},{1.465102, -0.114645, -0.771226, 0.255526, 0.140199, -0.131608, -0.483084, -1.205557, -0.213420, -1.729668, -0.033180, -0.951986, -0.938353, -0.841673, -0.898782, 0.716078, -0.930535, 1.278922, -1.674791, -0.257644, 0.289348, -0.383438, 0.555052, 0.655406, 0.011602, -0.809759, 0.406882, -0.019274, 1.187839, -0.330653, -2.378730, 0.206658, -1.141350, -1.082774, -1.111028, 1.573605, -0.494123, 0.671806, -0.156760, -1.901387, -2.699467, -0.235108, 1.449966, -2.283322, 1.369375, -1.633606, -1.702833, 1.145477, -0.295826, 1.012202, -0.411326, 0.016166, 0.815082, 0.721447, 1.306975, -1.604877, -0.495235, 0.441703, 0.623311, 0.736628, -0.233231, 0.847697, -0.714407, 0.036974, 0.905266, 0.001672, -1.330003, -0.845750, 1.397909, -0.226829, -0.414367, 1.520179, 0.053972, -0.147892, -1.142722, 1.501120, 2.014919, 0.338378},{-2.888296, -0.668904, 0.340919, -0.954359, 2.130985, -0.474525, 0.481687, -1.013230, 1.231836, -1.532773, -0.604206, -0.594649, -0.338576, -1.493359, -0.347757, -1.925630, 0.943018, -0.195639, -0.718480, 0.222944, 0.017020, 0.793450, -0.810115, 0.962511, -0.292028, -1.695497, 0.137337, -0.323331, -1.275340, -0.872615, 1.892074, -2.200779, 0.055642, -0.331285, 1.357427, -1.477406, -0.997733, 0.106359, -0.778232, 0.929960, -0.943532, 0.831268, -0.551624, 1.162826, -1.225415, 0.459341, -0.393286, 0.423276, -0.703982, -0.822569, 2.661440, 1.287613, 0.589184, -0.988955, -0.156854, 0.424523, 1.087749, 0.708364, -1.551839, -0.219966, 0.187824, 0.306791, 0.260016, -1.755259, 0.724774, 0.082802, 0.785679, 1.179779, -0.470585, 1.913366, 0.838566, -0.553723, 1.027071, -1.129946, 0.461510, 0.088946, 0.866919, 1.267362},{-0.937995, -1.370822, 0.582915, 0.820328, -0.124846, -0.088412, -0.125408, 0.310288, 1.777956, -0.785322, -0.843831, -3.104510, -1.796797, -1.145256, -0.693197, 0.783792, 0.753173, 0.400629, 0.079960, 0.440163, -0.668168, 2.167672, 1.609177, 0.605163, 0.369278, -0.482051, 1.777995, 0.558833, 1.651038, 0.177594, -0.732197, -1.848732, 0.081815, 1.379087, -0.545116, -1.332757, 0.930458, 1.204351, -0.400395, -0.872851, 0.778699, -0.009577, 0.906780, -0.251215, 0.391293, 0.322140, 0.643540, -0.064232, 1.152452, -1.047521, -1.363885, -0.187100, -1.651479, 0.406513, 0.287740, 1.715070, 1.208244, 0.152810, -0.545462, 0.362582, 1.848099, 0.728325, -0.393735, -2.104172, 1.295372, 1.574292, 1.266928, -0.173672, -0.574545, 0.212043, 1.595527, -1.265645, -0.603821, -0.429175, -0.045823, 0.794377, 1.257653, -0.653176},{0.541002, 0.605495, -1.068737, 0.235912, -2.622653, 0.003142, -0.344369, 0.725207, -0.831497, 0.645849, 1.113803, 0.555126, -0.564016, 0.191145, 0.495657, 0.401583, 2.167122, -1.208763, -1.877034, 1.789843, 1.168318, 0.663094, 1.031977, -1.358288, -0.404347, 1.473252, 1.278527, 0.877681, 0.619419, 1.388451, 0.097723, 0.432340, 0.137838, -1.003002, -0.866488, -0.253629, 2.861583, 1.109160, 0.893579, 0.450070, -0.352587, -2.077338, -2.363037, 0.765549, 0.596881, -0.476204, -0.414519, -1.064174, 0.557586, 2.436956, 0.681672, 1.932253, -0.658221, -1.141004, -0.065421, -0.058479, 1.017301, 0.765767, 0.542987, -1.602014, 0.967664, 1.179794, -1.364526, 0.194889, -0.876306, 1.200732, -0.194887, 0.942503, 0.073298, -1.049129, 0.922329, 0.905506, -1.883862, -0.729158, 0.064453, -0.655265, 0.720279, -0.795282},{-0.654273, 0.520225, -1.139593, -1.042497, 0.156158, -0.815119, -0.581358, -1.327758, 2.270286, -0.612506, -1.232753, -1.194338, -0.661771, -1.575137, -0.496726, -0.406860, -1.310543, -0.707448, -0.403698, -0.340225, -1.180987, 0.123573, 0.639694, -1.152271, -0.934611, -0.560884, 1.103894, -1.822031, -1.440635, 1.741708, 1.140473, 0.202557, 1.012126, -1.197399, 0.068217, 0.766699, -1.592381, -0.390454, -0.233946, 0.476764, -0.136888, -0.234804, 0.927402, 0.991860, -0.474601, -0.953031, -0.041549, -0.058920, -1.217676, 0.573705, -0.960025, 0.165959, -0.039009, 1.925434, -0.990159, 0.844917, -1.076544, 0.023517, 0.341162, -0.437108, 0.411637, 1.216290, -0.465421, -0.017888, 2.289653, 1.658113, -1.080140, -1.857057, -0.245856, -0.010441, 0.196801, 0.777497, 0.329240, 1.549003, 1.089496, 1.094657, -0.360953, 0.830560},{-0.270748, -1.402220, 2.040990, 0.198911, 1.964529, -1.380272, -1.132869, 0.161459, -1.529376, 2.441155, -0.101789, -1.047366, 0.108847, -0.737570, 0.080229, 0.092340, 0.137724, 0.434518, -1.554710, -0.923743, 0.616426, -0.438760, 2.204739, -0.205334, -0.306207, -1.678627, 0.456993, 1.265734, -0.383184, -1.764714, 1.152421, 1.436966, -0.217159, -1.093758, 1.413079, -0.964199, 0.727101, 0.741214, 0.316521, -0.084596, 0.609026, 1.310455, 1.781099, 1.138196, 0.380439, 1.235964, -1.579374, 0.877896, 1.488457, 0.703476, -0.608782, 0.230960, 0.432133, -0.963611, 1.445514, 2.061981, -1.391671, -0.038665, 0.473008, 0.427579, 0.741834, -0.974319, -1.566287, -1.055591, 0.319637, 0.384904, -0.267789, 2.071061, 0.291143, 1.624297, 1.580882, -1.366978, -0.334931, 1.257506, 0.475527, -0.327327, -0.234570, 0.421991},{0.522750, 0.010570, 0.792399, -1.538427, -0.839820, 0.983385, -0.409132, -2.289445, -0.364014, 1.046432, 0.144677, -0.130817, -1.595736, 2.031544, -2.270901, 0.700405, -0.200952, 0.634915, -1.418301, -0.271735, -0.928113, 0.101340, -0.848752, 2.106421, -1.917283, -0.176396, -1.496147, 0.655610, -0.705027, 1.318613, -0.720486, 0.106685, -0.114330, -0.388892, -0.157386, 1.665447, -0.432042, -0.905633, -0.992907, -0.446022, -0.145163, 0.765424, -0.050696, 0.891779, -1.621116, 0.333181, -0.654799, -0.034877, -0.970555, -2.605138, 0.449899, 0.594233, -1.105756, -0.442054, 1.039025, 0.953790, -0.276474, 1.891230, 1.099226, -0.787044, 0.046087, -1.345084, 0.429389, -0.296717, 1.198533, -0.603637, 2.150187, 1.762118, -1.030818, 1.075345, 1.050225, -0.794093, 1.058796, -0.176623, -0.942323, 0.083385, 0.791942, 0.998833},{0.883432, -2.009283, -0.113681, -2.392778, 0.668274, 0.835136, 1.558792, 1.050085, 0.309539, -0.666052, 0.415777, 1.029730, 0.235929, -1.608716, 0.287695, 0.396635, 0.889196, 0.460848, -0.215000, -1.702796, 0.362972, 1.172979, -0.285725, -0.995683, 0.130777, -0.682577, 0.306053, -1.701615, 0.402796, -0.349692, -1.738578, 0.234060, 0.767407, 1.716461, 0.692043, 1.349234, 1.467755, -0.105356, -0.374526, 2.165519, -0.639645, -0.216636, 0.759727, -0.514235, 0.738230, -1.048504, 1.747705, -1.194749, -0.157090, -0.140095, 1.288533, 0.565907, -0.680282, -1.055515, -1.942570, 1.424105, 0.410093, -1.283207, -0.852104, 0.585014, 0.176186, 1.894864, 1.475175, -0.019726, -1.758596, 0.181395, 1.367418, 1.442394, -0.761974, 1.602653, 0.410111, -0.719532, -1.853875, -0.030534, -0.100634, 0.833302, -1.458113, 0.905201},{-0.430008, 0.109656, -0.273758, -0.717366, -0.758167, -1.448441, -0.370433, -1.190116, -0.457043, -0.173634, 1.105976, -0.083885, 0.669668, 1.477570, 1.017788, 0.495258, -1.122843, -0.046297, 0.776176, 0.723323, 0.129451, 0.803497, -1.228446, -2.636050, -0.832076, 0.276360, 0.632397, 0.969326, 1.161441, -1.091075, -0.366137, -0.451120, -0.284198, -0.279018, 1.511221, 0.934034, 0.973934, 0.228940, 0.126052, -0.527443, 0.591439, -0.334509, 1.563543, 0.356015, 0.334876, -0.965238, -0.900041, 0.334170, -0.275276, -1.565118, -1.609394, 0.300644, -0.382424, 1.012260, 0.537448, -1.305836, 2.170769, -0.747622, -1.531043, -1.094966, -1.230960, 0.431354, -1.069096, 0.253120, -0.650504, 0.582696, -1.441359, 0.046324, -0.499917, 0.523101, 0.539857, -1.220951, -0.193021, 0.492400, 0.456367, -0.462333, 0.382676, -0.251815},{0.494882, -1.410419, 0.825513, 0.405694, 0.986934, -0.126551, 0.789373, 0.507430, -0.166004, 0.163362, -0.508597, 1.777452, 1.096019, -0.537941, -0.508416, 1.303384, -0.006593, 0.707071, 0.351130, 0.803336, -0.160842, 1.362465, 0.251818, 1.153914, 1.284071, -0.812424, -0.514631, -1.987141, 1.312380, -0.309159, 0.437828, -0.823323, -0.013081, 1.569829, -1.382787, 1.076921, 1.474975, -0.664807, 0.443533, 1.115965, -0.140982, -3.091938, -1.277276, -0.318296, -0.160727, -0.118831, 0.502776, -0.696527, -0.470627, 0.495893, -1.787813, -1.631053, 0.433344, 0.258088, -1.590148, 2.766865, -1.943659, -0.671157, 1.359677, 2.062126, -2.395662, 1.593145, 1.074211, -0.722388, -1.043439, 1.855159, 1.067788, -0.320014, -0.915366, 0.609494, 2.439504, 1.688562, -0.080927, -1.029607, 1.163254, 0.824736, -1.590713, 0.268377},{1.568984, 0.702034, -0.090387, 0.775639, -0.107036, 0.973227, 1.007022, 0.144859, -1.143333, 0.232761, 1.354267, -0.331797, -0.144389, 0.468000, 0.697436, -0.198178, -1.782660, 0.463335, 0.146435, 0.082704, -2.976063, 0.138306, 1.531662, -1.483072, -0.043199, 0.664399, -0.261512, -0.826609, -0.443280, 1.955873, -1.224316, -0.435066, -1.089395, -1.394404, 1.582034, 0.831995, -0.693202, 0.732029, -0.755801, -0.029977, 0.794690, 0.721420, -0.944517, 1.165774, 0.455962, -0.292773, -0.874967, 0.531324, -0.246020, -0.186062, 0.230477, -0.774950, -0.517895, -0.742689, -0.281335, -0.431045, -0.062522, -2.122131, -0.255429, 0.359898, -1.343167, -0.941098, -0.561131, -1.093207, 0.048683, -0.284387, 0.096467, -1.276403, -0.367537, 0.829760, -0.018055, -0.764568, -0.471001, -0.798714, 0.422644, -0.056788, 0.458492, -0.495481},{0.958137, -0.528093, 1.192876, 0.787777, 0.382048, -0.748969, 1.655826, -1.833755, 0.809820, -0.141527, -1.461405, -0.153615, -1.015462, 1.121826, -0.016356, -0.066824, -1.041413, 1.352685, -0.138649, -0.533299, -1.116174, -0.110767, 0.589730, -0.198132, 0.398973, 1.441877, 0.170953, 0.243687, -0.058626, 1.112727, 0.290837, 0.258021, 0.043237, -0.271838, 0.818860, -0.476207, -0.201346, 1.484918, -0.226226, -0.978406, -0.617795, 0.658923, 0.285784, -2.317864, 1.743968, -0.047412, -1.531166, 0.249977, -0.759315, 0.297370, 0.561447, 0.556040, 0.278930, 0.794835, -0.462231, 0.441123, -0.839141, 3.000674, -1.211990, -1.858229, 0.040512, 0.250359, 0.781342, 1.714655, 0.684534, 0.767685, -0.283747, -0.589671, 0.165485, -0.587745, -0.153271, -0.661476, -1.359434, -0.827891, 1.712339, -0.255170, 0.072256, -0.450818},{0.940676, 1.157409, 0.361195, 0.403166, 0.017529, -0.302672, -0.731142, 1.733371, -1.602536, 2.240434, 1.582233, 0.818038, -2.105233, 2.275883, -1.749681, 1.419473, 0.636463, -1.606071, -0.417410, 0.705793, 0.206354, -1.034269, 0.208879, 0.505272, -0.100680, -1.671515, -0.800351, 0.694164, -2.065622, -0.411346, 0.190874, -0.412982, 0.476028, -0.784363, -0.816144, -0.288404, 0.098121, 1.257098, 2.533017, 0.502128, -0.214459, -0.055249, -0.159787, 1.030561, 0.453184, -0.585338, -0.007131, -0.832416, 0.240984, 0.769691, -0.952189, -1.180679, 0.439430, 0.235456, 2.759773, 0.029369, 0.369547, -0.917955, 0.451464, -0.029362, 1.704395, -1.433501, 0.740020, 1.085427, -0.291329, -0.419534, 0.138506, 1.102113, 0.199953, 0.004339, -2.252286, -0.746176, 0.617033, 1.148893, 0.667729, 0.083239, 0.770331, -1.663203},{0.783112, 0.791951, 0.756669, -1.725615, 0.749417, -1.085562, -1.970676, 2.211972, 1.555790, 1.550286, -0.486526, -0.763808, -0.756161, 0.147421, -0.057505, 1.353027, 0.321465, -0.606232, -0.970931, -1.178145, -1.092896, 0.340662, 0.742202, -0.132787, 1.092101, -0.200550, -2.150656, 0.407050, -0.189268, -0.602692, 0.553870, -0.847173, 0.162701, -0.493086, -0.036668, -2.585127, -0.740537, -0.773172, -0.419493, -0.587314, -0.903730, -0.413573, -1.743848, 0.121606, 0.041117, -2.295577, 1.336633, -1.265131, -1.999017, -0.200430, 0.725693, 0.578057, -0.212986, -0.460145, -0.302104, -0.305244, 1.176795, -1.619107, -1.155494, 1.462078, -0.950213, 0.306919, -1.096105, 0.202361, -0.763110, 0.695358, -1.866712, 0.949811, -0.910078, -0.196512, 0.534200, -0.500948, 0.286941, -0.714914, -0.685430, -0.323491, 0.887730, 1.507412},{-1.627177, 1.444575, 0.751918, -0.725844, 0.113894, 2.479375, -0.763666, 0.722576, -1.902689, -0.858886, 0.933686, 0.309457, -0.535045, -0.387346, 0.089972, 0.674825, 0.185634, 1.726129, 0.726460, 0.856102, 1.842136, 0.216765, 0.445920, -1.606914, -0.436497, 0.790897, -0.364898, -1.092834, 0.214452, -0.215105, -0.463527, 2.344273, -2.177791, -0.188781, 0.100087, -0.435451, 0.945968, -0.021438, 0.099256, 0.310675, -1.721441, -0.175635, 0.052655, 1.140368, -0.746984, 1.159996, 0.847416, -0.317608, 1.618654, -0.807115, -0.509304, 1.200269, -0.141391, -0.091593, 1.400599, 0.544695, 0.555767, 0.969482, -1.466666, 1.157400, -1.310779, -0.990851, -0.216255, 2.082423, -1.138995, 0.126526, 0.799856, -0.641527, -1.082261, 0.431543, -1.103188, -0.045851, -0.241962, 2.279278, -2.947514, -1.595829, -1.693335, 0.219511},{1.413583, -1.809135, -1.312937, 0.778545, 0.222198, -0.124560, -0.807223, 1.449175, -0.293860, -0.780098, -0.995856, 0.354472, 0.417868, -0.017384, -0.430978, 0.286120, 0.075215, -0.483840, 2.087227, 0.788273, 1.089857, -0.369962, 0.878400, 0.897781, -0.537272, -1.574215, -0.382309, -0.539029, -0.420257, 1.470918, -1.183068, 0.755441, 0.772098, -0.007310, 0.566532, 0.335475, -0.018479, 0.450192, -0.312658, -1.193144, 0.697144, -0.552364, -0.587774, 0.663919, 0.177980, 1.781308, -0.678391, 0.260214, -0.339238, -1.123160, -2.082350, 0.024789, -0.984058, 1.297990, 1.196008, -0.504760, -0.723454, -0.477049, -1.167780, -0.098264, 0.752344, 1.220423, 0.918453, -0.402709, -0.093788, 0.970724, -0.371018, 0.672507, -0.156096, -1.943560, -0.565836, -1.810507, 0.595127, 0.158696, -1.066791, -1.765025, 0.807449, 0.948569},{-0.337987, -0.986090, -0.381412, 1.276999, -1.194644, -0.577815, 2.105494, -0.144148, 1.042029, -0.295355, -0.401053, 1.511432, -0.793920, -0.279042, -0.424443, 0.890890, 0.410781, 0.411046, -0.128925, -1.313611, 0.243435, 0.590878, -1.121349, 0.674601, -0.225752, -0.499277, 0.368742, 0.477542, 1.388120, 0.059246, -0.069639, -0.107281, -0.469166, -1.657654, 0.095329, -0.967912, -0.331134, -0.594313, 0.662171, 0.353270, 0.286708, 0.377281, 1.871292, -0.291254, 2.000222, -0.680100, 1.181369, -1.126786, 0.389590, -1.964111, -0.748171, -0.203602, 1.489974, -1.397459, -0.332020, -0.695691, 0.158304, 1.372133, 0.408645, -0.521948, -0.880600, -1.193331, 1.303454, 1.635880, 0.677869, 0.432728, 1.218999, 0.696888, -1.114501, 0.252507, 0.286780, 0.494915, 0.228938, -0.481993, 1.188396, -0.625346, -0.948533, 1.368202},{0.071806, 0.357744, 0.317996, 0.559390, -0.728443, 0.051352, -0.824292, 0.408627, 0.323365, -0.784714, 0.172450, -1.773197, -1.194919, -1.492327, 1.736455, 0.211250, -0.369110, -0.811208, -1.403471, -2.126677, 0.540306, -0.911982, 0.534306, 0.009241, 1.721452, -0.694624, -1.108370, -0.468053, -1.341908, -1.670354, -0.560186, 0.194049, 0.398164, 0.877316, -0.527899, 1.733601, -1.127363, 0.392965, 1.142244, 1.261419, 1.219860, 1.101512, -1.547714, 0.581131, 0.104243, 0.710472, -0.739595, 0.143686, -1.109187, 0.004044, -0.687891, -0.497361, 1.871505, -0.205981, -0.083539, 0.399726, -0.411220, -0.776432, 0.278754, 0.670922, -0.674862, -0.486711, 0.898154, -0.805855, -1.186124, -0.927262, 0.979220, -0.863663, -0.650637, 1.026307, 0.973439, -0.375122, -0.643458, 0.290570, -1.564640, -1.151247, 0.479406, 0.477210},{0.009938, -0.888345, -0.338784, 0.086097, 1.293878, 0.149860, 1.903723, 1.087510, -0.083738, 1.041638, 0.096584, -1.988700, 1.037182, -0.582276, -0.767694, -0.453140, 1.021721, -1.001182, -0.598153, 1.976288, -0.220339, 0.682131, 0.591791, -1.170805, 0.500369, -1.477750, -0.670866, 0.442309, 2.260025, -1.091128, 0.042238, -1.395895, 1.759127, -0.635287, 1.126251, -0.487074, 0.273228, 1.956367, -0.433747, -0.161456, -1.188408, -1.013045, 2.406648, -0.057728, 0.352135, -0.006978, 0.515349, -0.828752, -0.790081, 0.157926, -0.160730, 1.247376, -1.579574, -1.496236, -0.671159, 0.367220, 1.059835, -2.485705, -0.273024, -1.978974, 1.697638, -2.190871, -1.475061, -0.980224, -0.797012, -0.225522, 0.044135, -0.932913, 1.407063, -1.291559, 0.711079, 1.705184, -0.835658, 0.056557, 0.638200, 0.380530, -0.034578, -0.564281},{-0.549183, 2.109934, -1.099453, -0.347871, -1.027282, -0.985885, -1.935056, 0.338412, -0.260950, -0.439906, 0.288166, 0.133610, 0.389040, -0.060383, 0.868807, -1.389031, -1.545300, 1.140392, -0.188507, -0.023156, -0.650263, 0.411974, 0.474514, -0.912557, -2.026324, 0.917479, -0.066148, -0.412481, 0.098018, -0.537167, -0.757604, 1.458722, 0.712231, 0.431979, 0.614121, -2.240145, 0.079621, -0.429233, 2.375166, -0.978005, 0.367094, 1.301282, -0.101724, 1.873446, -0.692790, 0.319806, 0.064991, -0.339267, -1.366146, -0.161212, -1.061454, -0.021992, -0.004095, -0.808471, -0.262462, -1.194102, 1.270854, -0.871827, 0.005720, 0.243918, -0.705087, 1.317379, 0.647943, -1.696011, 1.622224, -0.189652, -0.311050, 0.651249, 2.640142, 0.747189, -0.346049, 0.298032, 1.424552, -1.676939, 0.283441, -1.217431, 2.545496, 0.666772},{-0.982307, -0.709544, 2.577285, 0.867353, 0.219827, -1.507391, 0.712888, -0.416456, 1.079904, 0.084149, -0.224403, -1.640692, 0.036832, -0.100897, 0.107975, 0.139801, -3.047690, 0.823253, -0.550199, 0.084927, -1.086574, -0.245098, 0.381817, -0.347862, 0.799896, 0.299608, 0.377787, 1.071670, -0.073263, -0.842108, 0.614308, 0.723038, -0.246167, -1.346424, -0.737612, 0.448701, -0.607899, -0.170363, 1.805288, 0.510300, -1.162071, 0.416437, -0.122643, 0.598351, -1.710380, -1.727411, 0.385348, -0.226829, 0.769320, 0.915694, -1.864233, 0.607408, -0.008382, -1.459584, 0.837239, -1.119118, 0.922896, -0.645145, 0.218654, -0.144616, 2.533258, 0.127675, -1.778028, -0.988108, -0.316746, -0.338946, 0.524955, -0.949528, 1.320466, 0.065309, 0.640353, -0.090672, 0.942423, -0.498620, 0.765139, 1.075995, -0.291159, -0.004401},{0.567975, -0.945138, -1.077922, 0.154204, -1.262905, -0.917991, -1.130112, 0.617827, 0.791359, -0.702480, 0.614200, -1.576038, 1.096346, -1.017913, -0.357780, -0.846663, -0.365217, -0.391675, -0.959767, 1.906245, 0.953765, -0.764898, -0.526104, -0.755630, 0.739495, -0.062088, -0.152794, 0.733796, 1.950318, -0.583216, -0.483508, -0.541770, -0.724407, 0.537325, 1.028495, 0.525087, -0.344617, -0.311243, -0.721698, 0.647555, 0.939424, 1.101946, -0.123257, -1.697405, -0.130596, 0.530137, 1.566435, -0.476270, 1.056698, -0.257891, -1.004456, 0.005237, 1.811927, -0.768202, 0.437304, 2.692778, -1.583646, 0.702590, 0.382096, 0.120973, -1.028859, -0.646318, 0.910607, -0.606599, -0.961027, -0.875823, -2.122630, 0.964876, 1.912652, -1.563485, 0.732396, -0.546772, -0.415739, -2.661530, -0.803093, -0.676058, -1.496523, 0.334227},{-1.998231, -0.724538, 0.758385, -1.593493, -0.116983, -0.416525, 0.668172, 0.155716, 0.443082, -0.823274, -0.713926, 0.597729, 0.059540, -0.027222, -2.427222, 1.708977, -0.054189, 1.506721, 0.634982, -1.162988, 0.669040, 0.702054, -0.263556, -0.062761, 0.482279, 1.774397, -0.753217, -0.235440, -0.178937, -0.759080, -0.241935, 0.079071, -1.118598, 0.197308, -0.175285, 1.386797, -0.558962, -0.731870, 0.674563, 0.627402, -0.724912, 0.371948, 0.474560, -1.909744, -0.983712, 1.499104, -0.539843, 1.722131, 0.760466, 0.912731, -2.699951, 0.601787, 0.745460, -1.274938, -0.998437, -1.195910, 0.281635, 0.105288, -1.363134, 0.705655, -0.720249, -0.044643, 0.596740, 0.444259, 1.542527, 2.332677, -0.292738, -0.017894, 0.154135, 0.952516, -0.960914, -1.335321, -0.930615, 0.407248, -0.456861, -1.958880, 1.316179, -0.277297},{0.727450, -0.395326, 1.499100, 0.618669, 0.183745, 1.403030, 0.276514, -1.986692, -0.999504, -1.023498, -0.076789, 0.907580, -0.298070, 0.215234, 1.140755, 2.440431, -0.185312, -1.318489, 0.863262, -0.665774, 0.675669, 1.380175, -0.095997, -1.578548, -1.556212, -0.872805, -0.749999, 1.059008, -0.010610, -0.135060, -0.298344, 0.099614, 1.210834, -0.949664, -1.843497, -0.069901, 0.695129, -0.442040, -0.865892, -0.097154, 1.204423, 1.446331, 1.134660, -0.446642, 1.003978, 0.903365, 1.792113, -0.482356, -0.459907, -0.435339, -0.130076, 0.592324, 1.382079, 0.262210, -0.262851, -0.993179, -0.877597, 0.088902, 0.108937, 1.097247, -0.252480, 0.355402, -1.391832, 0.899050, -1.255339, -1.334043, -0.894099, -0.461979, -1.171041, -0.390275, 0.412919, -0.556193, -0.409434, 0.219063, -0.234770, -0.284272, 1.762241, 1.657903},{0.205156, -0.258284, 0.537057, 1.790639, 0.393811, -0.268370, 0.209961, 0.074585, 0.602601, -1.125874, 1.529259, -0.932447, 0.143498, -0.616607, -0.906456, 1.097133, -0.020583, -0.502241, 0.678155, -0.143502, 0.223663, 0.898197, 1.543233, -1.400336, -0.688983, 0.474619, -0.480816, 1.018623, -0.390862, -0.759208, 1.458201, -0.602848, 0.826408, 0.205350, 0.673878, 1.106505, 0.011643, -1.974569, -1.585380, -1.069933, 0.051059, -0.615373, -0.897540, -0.521298, -1.235086, -0.297585, -2.522228, -1.079195, -0.430738, 1.320458, 0.136137, 0.500439, -0.341848, 0.427007, 0.838869, -0.386503, 1.295858, -0.772919, -1.397263, 1.426605, -1.597775, 1.464765, -0.842437, 0.777786, -2.165098, 0.403095, 0.377117, -0.256925, 0.440598, -1.012246, 0.079158, 0.476869, 0.760474, 1.981374, 0.128751, -2.539079, -1.487349, -0.120372},{-0.398927, -0.788997, -0.220486, 0.698535, 0.561706, 1.682470, -1.804322, 1.060274, 0.266567, -0.123866, 0.706712, -0.544562, 0.101503, 0.396115, -1.574825, 0.006482, 1.893763, -0.563023, 1.069615, 0.346020, -0.210357, 1.281971, 0.863133, -2.121154, 1.069203, 0.402004, -0.153168, -1.709720, 0.228401, -1.133724, 0.384311, -1.451918, -0.593485, -1.081405, 0.929507, 1.494567, 0.375108, 1.195941, -0.493453, -0.657989, 0.461772, 0.774746, -0.657940, -0.111070, -0.566512, 0.953041, 0.277390, -0.290868, -0.395314, 0.386588, 0.159223, -0.250557, -1.724295, 2.201768, -0.205303, -0.156689, 0.284605, 1.224447, 1.659683, -0.916990, -2.359442, -0.717727, -1.515905, -0.844918, -1.173172, -0.345878, 0.573626, -0.058882, 1.617377, 1.955982, -0.071415, 0.278333, -0.267839, -0.201638, 0.957431, -1.214704, 1.021887, -1.502040},{0.597803, 0.853119, 1.251034, 0.855465, 0.082549, -0.185393, 2.108435, -1.963808, 0.171558, 0.590875, 0.201921, -1.714895, 2.686172, 0.160980, -1.307174, -0.740482, 1.144531, -0.945541, -0.638019, 1.613533, 0.378588, 0.263647, 1.633795, 0.952579, -0.792418, -0.479572, 0.357133, -0.764717, 0.550124, 0.353859, -0.340022, -0.378612, 0.629354, 0.009337, 1.067045, 1.421951, -0.537449, -0.992768, 0.584547, 1.225534, 0.353492, 0.354741, 0.798710, 0.863053, -1.017748, -1.094463, 0.375511, -1.255748, 0.585287, -0.338672, -1.130694, -0.490442, -0.473497, 2.046536, -2.050399, 0.372947, 0.490897, -0.721971, -1.154711, 0.805856, -0.690782, 0.162453, -0.788618, -0.592628, -0.170261, -1.239337, 0.928113, -0.274797, 0.050141, -0.005531, 0.559360, -0.876000, 1.379578, -0.800069, 0.483347, -1.239013, -0.741352, -0.822965},{0.433938, 0.078785, -1.150879, -2.233255, -0.998537, -1.688719, -1.258914, 0.489604, 0.200354, -0.484746, 1.142678, 0.025741, -0.841812, 0.506483, 1.685397, 1.094466, -0.063984, 0.959111, 1.551743, -1.070871, 1.091050, -0.540101, -0.228946, -0.099529, 0.723492, 0.907170, 0.064375, -1.119803, 0.150628, -0.250918, -2.363088, -0.171926, 0.919401, -1.544267, -1.374928, 0.344580, -1.135272, -0.804758, 1.100943, 1.643798, 0.739045, -0.068838, -1.790556, 1.536845, 0.775046, -0.899293, -0.079407, -0.374176, -0.107983, -0.358349, 1.131208, 0.496988, 2.002332, -0.411215, 1.237105, 0.053585, -1.116175, 0.044387, 1.415285, 1.848692, 0.117310, -0.624637, 0.220607, -1.597135, 0.111265, 1.043978, 1.385478, 2.205754, 0.445275, 0.031483, -0.158819, -0.253342, -0.101915, -1.056438, 0.172543, -2.050661, 0.309115, 0.114294},{-0.623434, -0.204871, 1.586049, -1.011917, -1.842913, 0.525272, 0.220343, 1.091604, -2.753356, -0.651116, -0.490726, -0.577185, 1.325081, 1.447172, -0.995346, -0.078726, -1.938836, 0.082397, 0.639531, -0.400299, 0.728739, 2.951077, 0.017332, -0.066705, -0.737531, 0.446266, -0.042136, 0.489989, 0.199102, -0.906656, 0.949221, 0.191590, 0.262241, -1.876396, 1.885696, 0.373393, -0.083146, 0.116106, -0.588315, -1.375282, -1.237221, 0.651557, 0.040940, -0.679367, -0.225053, -0.087688, 0.004604, 0.887719, -0.496995, 0.415979, 2.086485, -0.220171, 0.082063, 0.162578, -0.192577, -0.432469, 1.343467, -0.492639, 0.858496, -0.799407, 2.594565, 0.514319, -1.246784, 1.471877, -0.852594, -0.018940, -1.219774, 1.908569, 0.496425, 0.754540, -0.842523, 0.467032, 0.049340, 0.320793, 1.320275, 0.934874, -0.625596, -2.503803},{-1.981401, -0.688722, 0.773111, -0.680010, 0.109227, -0.660196, -1.070131, 0.301150, 0.291633, 1.258569, -0.595865, 0.986350, 0.082599, -0.845272, -1.616377, 0.706262, -0.198594, -0.937356, 0.328057, -0.018376, 0.776742, 0.036485, -0.188203, 0.735900, -1.134593, -0.667169, 0.816294, 0.061428, -2.303220, 2.374872, 0.692945, -0.814172, -0.781067, -0.020973, -0.715823, 0.763644, 0.561221, -1.024039, -0.653976, -0.392881, -0.747372, -0.769829, 0.091114, 0.869719, 0.353115, 0.882683, 0.518345, -0.599828, -0.042436, 0.301722, 0.549845, 0.310587, -0.337711, -0.405727, 0.311165, 1.312513, 0.882764, 1.553058, -1.059459, 0.048906, -1.441122, 0.349268, -2.177562, 1.725207, 0.928575, -0.818577, -1.161762, 2.486054, -0.146038, 1.287430, -0.258424, 1.457542, 1.366939, -1.836899, 0.706947, 0.738748, -1.383659, 0.080711},{-0.133238, -0.233665, -1.265935, -0.896414, 0.223929, -0.508513, -3.458683, -0.183829, -0.327132, -1.173880, 1.675311, -0.455075, -0.285301, -0.726607, 0.071183, 1.338112, -0.104744, -0.955676, 0.812084, -0.504638, -0.244096, 0.524325, 0.666789, -0.121271, 0.888994, 0.806385, 1.690526, 0.163082, -0.154506, 0.377031, 0.945160, -0.306070, -1.700824, 0.947109, -1.515843, 0.738681, -0.683982, -0.235452, -1.444453, -0.797369, 1.419466, -0.141106, -0.708323, -2.395045, -0.626549, 0.207827, -1.134469, 1.219611, 0.446811, -2.142661, -0.210889, -0.595781, 0.873478, -0.440092, 0.030814, 2.230244, 0.136905, 0.529938, -1.128505, 0.099906, 0.021378, -1.403795, -0.087073, 0.209304, 0.019817, 2.607816, 0.167713, -1.441220, -0.584452, -2.181164, 0.149375, 1.196838, -0.478983, 0.302841, -1.108225, -1.207330, -0.139646, -0.027343},{-0.664336, -0.010757, -1.221030, -0.804858, 0.627348, 0.628919, 0.029845, 1.248504, 0.467574, 0.070933, 0.441702, -0.163938, -0.190855, 0.601362, -0.024705, -1.263676, -0.464962, -0.799813, 0.418494, -2.324510, 1.149639, 0.827703, 0.918836, 0.833502, 1.423900, 1.374558, -0.147151, 0.823092, 1.340252, 0.102510, -0.115154, 1.239392, 0.621056, -2.055992, 0.684527, 0.177582, 1.006723, -1.886700, -0.333248, 1.328854, 0.519389, 1.073401, 1.203464, -0.158967, -0.363892, -0.978581, 0.620595, 0.171740, -0.681775, -0.298512, 2.214006, 1.140039, -1.036761, 0.185634, -0.497026, 0.212475, 0.336877, -0.526244, -1.313878, 0.136283, -0.388787, 1.260611, 0.826634, 1.472692, 0.348629, -1.531358, 0.613992, -0.162844, 1.553480, 1.463747, -1.239578, 2.049282, -0.346558, 0.894856, -0.085133, 0.397471, 1.283834, -1.119960},{-0.337819, -0.455597, -0.981162, 0.205901, 0.002220, -0.064316, -0.682864, 0.023982, 0.322906, 1.913239, -1.095376, -0.603234, 0.225734, 0.511208, 0.791717, -0.399511, 0.365953, 0.690764, -1.012071, 0.029093, 0.374072, 0.338177, -0.192815, 1.085788, 1.390006, -0.253213, -1.578759, -0.769923, 0.806289, -0.331643, -0.815070, -1.354456, -0.930064, -1.563095, 2.067694, 0.852223, -1.952239, 1.410677, 1.326064, -0.310103, 0.268710, 0.599220, 0.318899, 0.136933, 0.014255, 1.133161, 0.802426, -0.364095, -0.974054, 1.161375, -0.023700, 0.396491, -0.033003, 0.870395, -0.505820, 0.381667, 1.092728, -0.982699, 0.525016, -1.740623, -2.394644, 1.095678, 0.482340, -0.717486, 0.974388, 0.955400, 0.510091, 0.052930, 0.266458, -1.862700, 0.019723, 0.955028, -0.295878, -0.327456, -0.580875, -0.832269, -0.350828, 1.749016},{0.813253, -1.419834, 0.098258, 0.506125, 0.937232, -1.168726, 0.435139, -0.656446, -0.029277, -0.727358, -0.242631, 0.330063, 0.807689, 0.870100, -1.154384, -0.570585, 1.293522, -0.211761, 1.333635, 1.198071, -0.329759, -1.756262, 0.614597, 0.071252, -0.026366, -1.362003, 1.450151, -1.026461, 0.477166, -0.237371, 0.319684, -0.094116, 0.339705, 0.986082, -0.555357, -0.018690, -1.043833, -0.147509, 0.781340, 1.985881, -1.670775, 0.758060, 1.240807, -1.031204, -0.081234, 1.491552, -0.072234, 2.125904, -1.122372, 0.247953, 1.131754, 0.513991, 1.315206, -0.367504, -0.676386, 0.102301, 2.241863, 0.736023, 0.230742, 0.849596, -0.522147, -0.195437, 0.003741, 2.761632, -1.396381, 0.444587, 0.119832, 0.400040, -3.022516, 0.136606, -0.585475, -1.189382, -0.420170, -1.770768, -1.168242, 0.943948, -0.465397, -0.685949},{1.738432, -1.303878, 1.473967, -0.055154, -0.288609, 1.113125, -0.013765, -0.766852, 0.230418, 1.177927, -1.110411, -1.592881, 1.230434, 1.869876, -1.910937, -0.213023, 0.653945, -1.096882, 0.494845, 0.252270, 0.811940, -0.549774, 0.500474, 0.119648, -0.049991, -0.047762, -0.985824, 1.130758, -0.841350, 0.497562, 0.858810, 0.379895, 0.499959, 0.467521, 0.682764, -1.013464, 0.305888, 2.727838, 0.446782, -0.315724, 0.215699, -1.027713, 1.360474, -0.216796, 0.000727, 1.109042, 1.409326, -0.464257, -0.236960, -0.124755, 1.106287, 1.132470, 0.755199, -0.687870, 2.097729, -1.190491, -0.851963, -0.377069, -1.702514, 0.159618, 0.613278, 1.018239, 0.842436, -0.998001, -0.374472, -0.359625, 2.260261, -0.609335, 1.931908, -1.113555, 0.283815, 1.401404, -0.786875, -1.364342, -0.111001, 2.711531, 0.198573, -0.478715},{-1.242050, 0.754331, 1.629809, -0.027165, -0.691819, -0.631289, 1.969849, -0.337258, -0.663347, 0.336117, -0.859850, 0.841741, 0.682912, 0.657405, 0.541927, 0.065370, -1.344335, 0.083224, -0.210379, -0.218729, 1.179238, -0.613847, -0.189474, 1.418380, 0.904966, -2.034167, -1.536933, -1.180913, 0.208796, 0.124237, 0.379061, -1.249171, 0.222626, -1.706681, 0.078839, 0.891616, 1.475545, -0.787839, 1.897913, 0.730088, 0.761175, 2.397828, 1.812890, -1.807152, -0.466117, -0.622432, -1.369037, -0.596289, 0.799266, 0.605832, -0.256909, 1.103194, 0.647868, -0.231738, 0.403939, 0.872355, -0.173284, -0.093382, -0.803038, 0.491488, -2.517607, -0.881055, 1.269624, 1.396520, 1.590524, -0.348443, 1.570281, -0.393154, -0.365916, -0.179957, -0.537017, 0.294991, -1.216498, 0.353844, 0.682652, -0.013501, 0.577052, 1.158441},{-1.949433, 0.830664, 1.575387, 1.074411, 0.387344, -0.301714, -0.709490, -0.324988, -0.134680, 0.328025, 1.244051, 1.758074, -1.088920, 1.933767, 0.837207, 0.882324, 0.006030, -2.826828, 0.436732, 1.186909, 0.038914, -1.091160, -1.513388, 0.237275, -0.360774, -0.702782, 0.176803, 0.262553, -0.042109, -0.336155, -0.860537, -0.067885, 0.565574, -0.369578, -0.534333, -0.394146, -0.266444, 0.209501, -0.375727, 1.242370, -0.340058, 1.156848, -1.735110, -1.873014, 0.154650, -0.447469, 1.900382, -0.132941, -1.399639, 1.449695, -0.131392, 1.016642, -1.167458, -0.382229, -0.870502, -1.474568, -1.228703, -0.786517, 0.374810, -0.691060, 0.277560, 0.309569, -0.308493, -1.158722, -0.338528, 0.209328, 0.183133, -0.117022, 0.291250, -0.106646, 0.640240, -0.007065, 1.543711, 0.359634, -0.390191, -3.016589, -2.040717, -0.561929},{1.587982, -0.465448, -0.487821, -0.581778, -0.996698, -1.872454, -0.982622, 0.292731, 0.840426, -0.046890, 0.525344, 0.224838, -1.159091, 0.740322, 0.557051, 1.131612, -0.104964, 0.262890, -0.814637, -0.832601, 1.221108, -1.001852, -0.687357, -0.374877, 0.648884, 0.168359, 0.962586, -0.335278, -0.171288, -0.200645, 2.541051, -1.030999, 1.442221, -0.135267, -0.962570, -0.935906, -0.979157, 0.155152, 1.377781, 0.718468, -0.034140, 0.395772, 0.932488, 0.577449, 0.335913, -1.441123, 0.039610, -0.366518, -0.351261, -1.651196, -0.149732, 0.072163, -1.521930, 0.007730, -0.430643, -1.089923, -0.824762, -0.345209, 0.331170, 0.350125, -1.046446, 0.561649, -0.322317, -0.035488, -1.864007, 1.106945, 1.213732, -1.095194, -0.716956, -2.062448, -0.622051, -0.780145, 0.047434, 0.504434, -0.103647, -1.115689, 0.869592, -0.233007},{0.736743, 0.411577, -0.496497, -0.483372, 0.216023, 0.155795, 1.869510, 0.968484, -1.953591, -0.528717, 0.041310, 2.388239, 1.703909, 0.806634, 1.816057, 0.740095, -0.613288, -0.632051, -0.202350, -0.177305, 0.153743, 0.479199, -0.264111, 0.651489, -0.106927, 0.872442, -0.082767, 0.810786, -0.311833, -0.008476, 0.677711, 1.301185, 0.601003, -0.002196, -0.337731, -0.024751, 1.028166, 0.498380, 0.691472, 0.074501, -0.475147, -0.138903, -1.414930, -0.527062, 1.912287, -0.921328, -2.232166, 1.048787, 1.337275, -0.631478, -0.314507, -0.593336, 1.724228, 0.206841, 0.223818, -0.413281, 0.516787, -0.704792, 0.290216, -0.421682, -1.816599, 0.758405, -1.236948, 2.114430, 0.211554, -1.032419, -1.542909, 0.234176, 1.096959, 0.657704, 0.672169, 0.123436, 1.105903, 1.878801, 0.344563, -0.298910, 0.480203, 0.489744},{0.465997, 1.262854, 0.184268, 0.785555, 1.233031, -0.018764, -0.095440, -0.562488, 1.546119, -0.440119, 1.058056, -0.155698, -0.826838, 0.146567, -0.255345, -0.386135, -1.316751, -1.513776, -0.099909, -1.180049, 0.881426, 0.359732, -1.872743, -0.949409, 0.134202, 0.326153, 1.103871, 0.990729, -0.777145, -0.319219, -0.870164, -2.201159, 0.175969, 0.238406, -0.314211, -0.704469, -1.475441, 0.407426, 0.044117, -1.227386, -1.066913, 1.465584, 1.494341, -0.225393, 0.562182, -2.128621, -0.049825, -1.248745, 0.655165, 0.236591, 0.757226, -1.063438, 0.332287, 0.806329, 0.436290, -0.458931, -0.420210, -1.422798, -0.426261, 0.609604, 0.575154, 3.202308, -0.558121, -0.411229, 0.698983, 0.252956, -1.922084, 0.292679, -1.277152, 0.342225, -0.740931, -0.098424, 0.527667, 1.896061, 0.967021, -0.824185, -0.611399, -0.705727},{1.168675, -0.367166, 1.124351, 0.481866, -0.919972, 0.316791, -0.878941, 0.344672, -1.025152, 0.342775, -0.442073, -0.361987, 0.094552, 1.783103, -0.352462, 0.015851, -1.245193, -0.713560, 0.244142, 0.748944, -0.205624, -1.375517, 0.735830, 1.447501, 0.234242, 1.081006, -0.704060, -0.168107, 0.751179, 0.584257, 0.934337, -1.736333, -0.632337, -0.615252, -0.117483, 0.507891, 1.219753, 2.231011, 0.822029, -0.276730, -0.540159, 0.162977, -1.185538, 1.524413, 0.258772, 1.632136, -0.346145, 0.343553, 0.349434, 0.453449, -1.333751, -1.870534, 0.676881, 0.363198, 1.105548, -0.200610, -0.874143, -1.036112, -1.004793, 1.404685, 0.365202, 1.951918, 0.990683, 0.322171, 0.296181, 1.716635, 1.216093, -0.221702, -0.411344, -1.030554, 0.719605, 0.500098, -1.349296, -0.992057, -0.930317, -1.014948, -1.726153, 1.697128},{-1.153283, 0.908980, -0.196110, 2.156470, 0.561544, 1.390267, 1.074394, -1.236509, 0.419948, 0.235866, 0.411894, 1.064143, 0.872064, -1.088460, -0.174066, -0.200207, -0.131320, 0.632036, -0.929406, -1.050662, -1.980313, 0.020083, 0.494575, 1.092440, -1.145727, 1.944436, -0.442050, -2.067354, 1.524164, -1.677835, 0.395373, -0.045281, -1.207995, -0.480631, -0.926982, 0.705389, -0.480543, 0.614667, -0.429977, 1.112087, -1.266994, -0.882564, 1.371295, -0.340890, -0.103126, 0.702179, -0.678659, 1.496585, 0.296682, 0.847613, 0.265295, -0.777038, 0.005742, 1.592726, -0.121643, -1.489658, 0.695537, 0.198107, 1.180037, 1.040337, 0.315720, 0.763552, 1.834020, -0.811790, 0.430981, 0.355888, 1.482144, 1.128557, 1.581554, -0.138349, 0.753130, -0.904244, -1.716610, 0.561052, -1.509675, -1.732555, -1.601313, -0.469126},{-1.285154, -1.078976, 0.821194, -1.968634, 2.042533, 0.291170, -0.106789, 1.271537, -0.179667, 1.179213, -1.293575, 1.342651, 0.477858, 0.321107, -1.117427, 0.028740, 2.513707, 0.577454, -0.884125, 0.087141, 1.849592, 0.013982, -1.124471, 1.714968, -0.031398, -0.124609, 0.176923, 0.872895, -1.297709, -0.096620, -1.039100, 0.658068, -0.646889, -0.797683, -1.106589, -1.667480, 0.900876, 0.322890, 0.774252, -1.677933, -0.451835, -1.046750, -1.269117, 0.434209, -0.142059, 0.147788, 0.121511, -1.640794, 1.150011, 0.454410, 0.845839, 1.572098, 0.821966, -1.126221, 0.600080, -0.214522, 2.039400, -0.486650, -0.534620, -1.515141, -0.865604, 0.161439, 1.153026, 0.445971, 0.034564, 1.095447, -1.644278, 0.041932, 1.849040, -1.156531, -0.108572, 0.278153, -1.584914, 0.502315, 0.199892, -0.065223, 1.615415, 0.888545},{0.563165, -1.595915, 0.193487, -1.513127, 0.941314, 0.723925, 0.872292, 0.086513, 1.052422, -0.138179, -0.214033, 0.324879, -1.320533, 0.989240, 0.192927, 0.928947, -0.881735, 0.248227, 0.682449, 0.800641, 0.868162, 0.890262, 0.346095, -1.517566, -1.212785, -0.545442, 0.336645, 1.098385, 0.314835, 0.074106, 0.971071, 1.854036, 0.356314, -0.677225, 0.972253, 0.447221, 0.187340, -0.275943, -1.328133, 1.736473, -0.127563, 0.684306, 0.104786, -0.052753, 0.637614, -0.194312, -0.144839, 0.612244, 0.929602, -1.504112, -0.442604, 0.621356, -0.190832, -0.974433, 0.805886, -1.396685, -0.638285, -2.029166, 0.081367, 1.637847, 0.379116, 0.580742, 0.797053, 1.273887, 0.274240, 0.017333, -0.390580, -0.140623, -1.673099, -1.076112, -1.431249, 0.546385, 0.427663, -1.348357, -0.507945, 0.922491, 1.276811, -1.266550},{0.378357, -2.785192, -0.687665, -0.007407, 0.446394, -1.682604, -0.906402, -0.816915, -1.725333, -0.247510, -0.361355, -0.489327, -0.962867, 0.289659, 1.152441, 0.567187, -1.247136, 0.751508, -0.944093, -0.580756, 0.440581, 0.930791, 0.548947, -0.417510, 0.409110, 0.095734, 0.973979, 1.321885, 0.710336, -0.752178, -0.695364, 1.022675, -1.056757, -2.802916, -0.849057, -0.854573, 1.879195, 0.377812, 0.218488, -0.240405, -1.418581, -2.480174, -1.052574, -1.002154, -0.539586, 1.577565, -0.568360, -0.985869, -1.419916, -0.566494, 0.679449, -0.315559, -0.754711, -0.057517, 0.707872, 0.154257, 0.967325, 1.457185, -0.993325, -0.162881, 0.780532, 0.994538, 1.951946, -0.334964, -0.268915, 1.189628, -0.066483, -0.700899, 0.340212, 0.872668, -1.906496, -0.768655, 0.443222, 1.305422, -0.012896, -1.055681, 0.895067, 0.584909},{0.117710, 0.602140, -1.545587, 0.407686, -0.085446, -0.372273, 0.176339, 0.679329, -1.379411, 1.280109, 0.464269, -0.312717, -1.262381, 0.488162, -0.688856, -1.128769, 0.929461, 0.212890, -0.565393, -2.190831, -0.107011, -0.580342, -0.363867, -0.226628, 0.234141, -0.073562, 0.673979, -0.470623, -2.038830, 0.404824, 0.578954, -0.897341, 1.029802, 2.577911, -1.151083, -2.254898, 0.335339, 0.503919, 0.430984, 0.774516, 0.740140, -0.156505, -0.197713, 0.155550, -0.152241, 1.349847, -0.058839, -0.929633, 2.022207, -0.573968, -0.283314, -1.048064, -0.564158, 0.972539, -0.556000, -1.405907, -0.072843, -0.305125, 1.671075, 0.404227, 0.212275, 0.433895, -1.535141, 1.135311, 0.141599, -0.441032, 1.060335, 0.634518, 0.136348, -0.426238, -0.286899, 0.818321, 0.328582, 0.441639, 0.661389, 0.184872, 0.830199, -1.552297},{-1.221771, 0.895285, 0.599122, -0.101147, 1.082870, 0.795558, 1.288079, -0.796058, 0.238631, 0.671265, -0.823902, 0.688983, 1.129882, 1.033487, 1.528547, -0.605592, 1.321157, 0.317271, -0.250441, 0.182526, -0.338534, 0.173438, -1.008245, 1.525028, 0.186405, 0.736284, -1.982236, 1.461678, -0.062350, -0.473631, -1.561324, 0.428515, 0.932683, 0.217330, -1.616389, 1.736479, -3.085152, -0.078588, -1.885955, -0.199423, -0.366246, 1.649745, -1.596850, 0.947714, 0.016317, -1.921757, -1.355168, 0.708993, 0.061000, 1.764356, 1.933129, -0.319315, 1.464956, 1.159080, 0.254657, -0.118955, 0.093311, -0.385755, -0.004996, -0.584159, 1.442268, 0.735094, 0.269374, 0.638496, 1.287283, -0.011144, -0.654809, 1.666270, -0.473379, -0.737220, -2.100922, -0.520230, 0.336008, -0.598977, 0.167171, -1.155751, 0.397304, 0.300270},{-0.582852, -0.158673, 0.498666, 1.100044, -1.051785, -1.495945, -1.423036, 1.803862, 2.514824, 1.084124, -1.657346, -1.235139, -1.368180, -0.902444, 1.121470, -1.838776, 0.303467, -0.165725, -0.154800, 0.297444, 1.017270, 0.709237, 1.399710, -0.082964, 1.787542, 1.197192, -0.197298, 0.196241, 0.343566, 0.179288, 0.324933, 0.706714, 1.153565, 0.487464, 0.786259, 0.131321, 0.225475, 0.191649, -0.901775, 1.347554, -0.276583, -1.226028, -0.957407, 0.137475, 1.560685, 0.473230, -1.488900, -0.341760, 0.287422, 0.013425, -0.694782, 1.788348, 0.126442, -0.119032, -0.588140, 0.680319, -1.069457, -0.295039, -0.422848, -2.203535, 0.744128, 0.682156, 0.759458, -1.083381, -1.321051, -0.257141, 0.071408, -0.784472, -1.365732, -1.186258, -0.114209, 0.778534, -0.723758, -1.025695, 0.722545, -1.570484, 0.437536, 1.700075},{1.913941, 0.010069, 0.516268, 0.930875, -1.746440, 0.551887, 1.624853, -0.653968, -1.181861, -1.335615, 1.060658, 0.194307, 1.002448, -1.130315, 0.049578, 0.182612, -0.499166, 0.191905, 0.960792, -0.625312, -0.233707, -0.377512, -1.428911, -0.897694, 0.150503, -1.115401, 0.855492, 0.716707, -0.660315, 1.171977, -0.124805, -0.164129, 0.138975, -2.056349, 0.512066, -0.699645, -1.486320, 0.057052, -0.311209, -0.192514, 0.897885, 1.524263, 1.803108, -0.781968, 1.028010, 0.467078, 0.455016, 0.589710, 1.508919, -0.235770, 0.606008, -1.686088, 0.203519, 0.111427, 0.111649, 0.160629, 2.292526, -0.280547, 0.279422, -1.549711, -1.637084, -0.320199, -2.596371, -0.909985, 1.714064, 0.162448, 0.560899, -1.100575, -0.119176, 0.302940, -0.686892, -1.351693, 2.082932, -0.268011, -0.278104, -0.233352, 0.788307, 1.064493},{0.418073, 0.672781, 1.780583, -0.229225, 0.964219, 0.260261, -1.064923, -0.147229, -0.628198, 1.136524, 0.134788, -0.170520, -0.331675, -1.476629, 0.125565, -0.855956, 0.298827, -0.784795, -0.647872, 1.433283, -1.228246, 0.937109, -0.881566, 1.223096, 1.524740, -1.107905, 1.034645, -0.158242, 0.620502, -0.957616, -0.413810, -0.632931, -1.023107, 1.389897, 0.972966, -0.773444, 1.843624, 0.235348, -1.077735, -0.176927, -1.739756, -0.565295, 1.457148, -0.947640, -0.042247, 1.300176, 0.333087, -1.067579, 0.175129, -0.496360, 0.815045, -0.378053, -1.745281, -1.292722, -0.070455, -1.500368, -0.445328, 0.578230, 0.338151, 0.711116, -0.582393, -1.942765, -0.820859, -0.798949, -1.280905, 1.834780, 0.300745, 0.075945, 0.463452, -0.378748, -1.465485, 0.728226, 0.648775, 1.243778, 0.102504, -0.022173, -0.764076, -0.292674},{-1.657325, 0.655690, -0.893765, 1.175894, -0.894701, 1.418885, 0.551868, 0.826803, 0.171951, 0.992291, 1.269117, -0.462642, 0.267059, 0.509457, 0.974954, 0.009410, -0.623216, -1.225202, -0.092022, -0.412655, -0.660300, 0.788145, 2.243960, -0.616164, -0.296682, -0.207065, 0.197662, -0.129104, -0.264214, 0.624965, -0.385307, -0.803212, -1.570658, 2.261579, 0.249074, -1.040591, -1.692618, -1.762189, -0.029004, 0.414491, 0.844139, -0.362963, -0.864539, 0.597398, 1.787256, -0.201886, -0.139618, 0.007326, -1.752850, -0.877614, -1.871115, 0.952138, -1.085912, -0.047731, 0.105271, -0.872954, 0.814171, 0.744816, 1.570659, -1.769538, -1.101726, 0.474541, -1.135697, -0.515792, -1.128633, -0.072948, -0.343187, 0.930341, 0.150182, 0.480544, -0.672175, -0.585654, 0.194820, -1.646063, -0.637818, -1.310922, 0.467250, 0.105510},{1.161958, -1.647173, -0.052533, -1.227897, 0.033337, -0.222045, 1.292012, -0.350694, 0.171463, -0.495808, 0.688467, 0.821908, -0.230619, -1.538987, 1.409873, 0.754277, -0.544678, 1.900575, 0.950314, -0.520199, 0.755023, -1.370571, 0.214682, -0.180428, -0.097618, -0.727500, 0.840534, 1.008100, 0.651879, 1.231474, -0.685996, 0.593769, 1.776141, 0.674187, 1.682023, -0.056904, -2.840061, -0.270732, -0.344654, -0.090360, 0.292082, 1.837689, 2.162561, -0.355984, -0.576534, 0.425937, -0.669080, 0.914019, 0.385190, 0.544264, 0.465122, 1.188531, -1.060459, -0.126413, -1.498111, -0.992918, -1.471120, 0.606814, 0.662264, 0.351422, 0.280085, -0.323917, -1.731515, -0.291520, 0.855046, -2.211694, 0.613996, -2.362946, -1.508849, -0.315669, 1.022508, 0.564961, 1.078998, 0.487967, 1.065404, -1.038400, -0.324365, 0.192566},{2.093861, 0.654232, 0.758255, 0.086097, -1.724202, 0.589783, -2.008512, -1.340657, 0.017849, -0.777002, 0.479857, -0.891231, -0.494518, 0.229183, -0.486573, 0.058118, -0.071907, -0.546510, -2.307576, 1.026107, 0.016895, -0.947755, 1.931338, 0.920078, 1.584925, 0.963728, -0.203227, -0.883938, -0.140772, 0.614540, -0.069479, -2.846402, 0.413958, 0.011461, -0.309337, 0.322248, -0.924214, 2.060939, 1.010618, 1.018893, 0.715414, 0.698705, 2.084359, 0.798163, 0.551141, -1.498142, 0.661532, 0.093785, 1.538628, -0.808459, -1.000900, -0.464196, 0.772118, -0.085826, 0.038455, 0.159294, -0.944716, -1.252421, 0.008813, 0.679243, 1.052315, 1.288334, 0.723955, -0.379876, -0.081351, 2.783318, -0.220103, 1.228034, 1.060902, 1.072013, 0.758124, -1.380732, -0.578166, 0.620418, 0.611520, 0.279306, -0.364803, 0.677172},{-0.006376, 0.832463, -1.205942, -0.892278, -1.358776, -0.495798, -0.491189, -0.770466, 0.747624, 0.443075, -0.796604, 1.349724, 1.193683, -0.199396, 0.639593, -1.091040, 0.040384, 2.150459, 0.586838, -0.868969, 0.248093, 0.139543, 0.512460, -0.914450, -0.032426, 0.527231, 0.542864, 0.910552, -0.432413, -0.546431, -0.868006, 0.228598, 1.030136, -0.067833, 0.963792, 0.413877, 1.172911, -0.179066, 1.026825, -0.013795, -0.390338, -0.922856, -0.250812, 0.595524, -0.775983, -1.226642, -1.011068, 0.461676, 0.199928, 0.277507, 0.448215, -1.137196, -0.342902, 0.135000, 1.410691, -0.210556, -0.269474, 1.683880, 1.301508, 0.620648, 0.923787, -0.045809, 1.155810, 1.045354, -0.352661, -1.168588, 0.240795, 0.728819, 1.317570, -2.001837, -0.576378, 1.356665, -1.399301, 1.682780, -0.797997, 0.364705, -0.987425, 0.485256},{0.196215, 0.541874, -0.929535, 1.658840, -1.424910, -0.428045, 0.404951, 1.414436, -0.139654, 0.331621, -0.694402, 0.753993, 0.282148, -1.010067, -2.065294, 0.121898, 1.712763, -0.038355, 0.201922, 0.223756, -1.589174, -1.409667, 0.560617, -0.138292, -2.208508, -0.544548, 1.602754, 1.537859, -0.978874, 0.711850, -0.947024, 1.857605, -0.213016, 1.806035, 0.399521, -0.509229, -0.972530, -0.653201, 0.530993, -0.412796, 0.683307, 0.028602, 0.556948, 1.234984, 2.362314, -0.498621, 1.150431, -0.276662, 0.402656, 0.008290, 0.846690, 0.358053, 1.670925, 0.404584, 1.271122, -1.631650, -1.881726, -0.130996, -0.026420, 1.873757, -0.157888, 0.866476, 0.375466, -0.945646, -1.055926, 0.011219, 0.200710, 0.148639, -0.660194, -0.201611, -0.323862, 0.306146, 0.537467, -1.457446, -0.189956, -0.429668, -0.205439, 0.440344},{-0.387482, -0.486600, -0.003017, -0.747672, -1.366090, 0.207191, 1.295020, 0.312285, 0.648006, -0.788645, -0.290621, 0.627542, 1.142345, 0.577196, -0.990963, -0.188660, -2.108452, -1.077979, 1.296304, -1.651579, 1.793638, -1.449661, 0.688504, 0.808826, 2.303301, -0.084618, -1.579338, 0.689219, -1.169761, 2.104135, -0.845669, -0.823420, -0.339584, 1.643077, -0.570058, -1.211660, -1.846284, -0.513800, 1.581337, 1.251864, -0.959955, 0.992842, 0.397651, 0.520998, -0.883020, 0.817785, -0.995716, 0.522060, -0.219994, 0.690439, -0.361187, -0.944478, 0.062672, 0.138214, 1.503438, -0.133406, -0.609752, 0.062649, 1.858463, 0.673632, 0.325861, 0.091326, 1.240691, -0.526739, 0.498597, 0.535401, -0.813761, 0.704053, 0.138396, 1.754620, 1.705716, -0.715490, 0.866281, -1.159473, -0.318132, -0.544870, -2.490068, 0.669339},{-0.174001, -1.151371, 0.737266, -0.514878, -0.804261, 1.764669, -0.143045, -0.680233, -1.300948, -0.608915, -0.576932, 1.854929, -0.537883, -1.650885, -0.019024, -1.882750, 0.719413, -0.357442, -1.237802, -0.476939, -0.943490, -0.791152, -0.166069, 0.937091, 0.299308, -0.629985, 1.446124, -2.086602, 0.624240, 1.000800, 1.330769, -0.018566, -0.747309, -0.693700, -1.225998, -1.337241, 0.277766, -0.685379, -0.185788, 0.519939, 1.159354, 0.788029, -0.351951, -0.731268, 0.928087, 0.232660, -0.592094, -0.486631, 0.732892, -0.741523, -0.294761, -0.158832, -0.602646, 0.454798, -0.068052, 0.278893, 0.047879, 0.333591, -0.109642, -0.534624, 0.217197, 0.041180, -0.428687, -0.490974, 1.079544, 0.197311, 1.242713, 0.549900, -0.403973, -2.248666, -2.677828, -0.366435, -0.612023, -1.763990, 0.409111, 0.530485, 0.781564, 0.341999},{0.468415, -0.480865, 0.372236, -0.327635, 0.819565, 0.960292, 1.216101, 0.424092, -1.321579, 1.782080, 1.502364, -0.611645, 0.180642, -0.417259, 1.142215, 1.298557, 1.504090, -0.075399, -1.705794, 0.290135, 2.467637, -0.214533, -0.290191, -0.345038, -1.118033, -1.373140, 0.009945, -1.035629, -1.267596, -0.030561, 0.249775, 0.458174, 0.074797, -0.664268, -0.044408, 0.529991, 0.563270, 0.534550, -1.424507, -0.296641, -1.399811, 0.753514, -0.493726, 0.174609, -0.956890, 0.760435, -0.955732, -0.081926, -0.264543, -1.776703, -2.094588, -0.043065, 0.179054, -0.515741, 0.267352, -0.299844, 1.552060, 1.196054, -0.399365, 0.713687, 0.995368, 0.086123, -0.029119, -0.925583, -0.388101, 0.568327, 0.568022, -1.404442, -0.892940, -0.756602, 1.548143, -1.281987, 0.364398, -0.775932, 0.786918, 1.416262, -0.877659, 0.951764},{-1.803958, -0.424952, -0.288988, -0.173885, -0.021209, -0.775752, 0.281423, -0.842363, -0.535675, -0.519436, 2.224005, -0.449835, -0.151089, 0.566968, -2.192694, -0.121032, -1.739368, 0.471112, -1.455511, 1.070693, 0.305378, 2.485793, -0.110787, 1.585681, -0.792922, 0.945132, -0.976393, 0.303700, -1.624140, 1.526783, -1.202038, -0.363149, -0.144846, -0.923067, 0.442825, 1.761051, 0.276979, 1.178214, -2.111354, -0.105919, -0.883129, -1.452900, 0.333954, 0.145803, 0.738175, -1.246947, -0.243131, -2.168874, -0.494038, -1.442897, 0.916610, 0.223703, -0.060202, 0.699906, 0.105815, -0.009899, -0.064324, 0.114940, 0.030953, -1.565301, 1.050055, 2.713083, -0.137661, 0.920992, 0.599251, 0.377032, 0.731680, 0.235621, 0.780887, -0.419733, -0.363044, 0.730487, 0.775170, 0.874892, 0.310931, 1.812913, 1.782123, 0.590478},{-0.130396, -0.628169, 0.316226, 0.358004, -1.981429, 1.117653, -1.188434, -1.304367, 0.043784, 0.628184, -1.439075, 1.121569, -1.980084, 0.350309, -0.640007, 0.296119, -0.655024, 0.061698, -0.746481, 0.886645, -0.162076, 0.672233, 1.425622, -0.265172, -0.338011, 0.234399, -0.237300, -0.014512, -0.615508, -1.209133, -0.085085, 1.222608, 1.528929, -0.325386, 2.474056, 1.022125, 0.159292, 1.461725, 0.306972, -2.329525, -1.422169, -1.012479, 0.983768, 0.912148, 0.539784, 0.275315, 0.517671, 0.517451, 0.036069, -0.542714, -0.345513, 0.399573, -0.533209, -0.615130, -0.242591, 0.528572, 1.018886, 0.304414, -1.039641, -1.472554, -1.171264, -1.522357, -2.137315, 1.646133, -1.569277, -0.430096, -0.369113, -1.502734, 0.240857, -0.418335, -0.014925, 0.202072, 0.824502, -1.038371, 1.653378, 0.344849, -1.228142, -1.731131},{-0.510843, -1.706534, 1.885658, -0.831124, 0.861613, 0.274240, 0.369751, -0.247764, 1.413107, -0.133977, 0.547386, 0.585327, -0.577685, -1.504673, 0.060411, 0.448060, -0.486916, 0.869580, -0.581498, 0.402377, -0.724923, -1.200961, -0.277350, -0.890143, -0.489971, 0.097094, -0.594610, 0.852914, -0.000810, -3.037069, 0.677711, 1.868536, -0.572700, 0.493511, -0.562240, 0.028970, 1.537194, -0.900767, -0.012508, 1.589697, 0.130799, 2.341537, 0.133518, 0.854374, 1.692934, -0.681569, -0.152681, 1.602008, 1.120476, -0.363866, -0.259715, 1.055167, 0.858313, -0.493453, 0.662286, 0.788936, -1.172105, -0.640647, 0.498769, -0.377058, 0.707426, 0.619903, -1.638601, -0.339608, -0.116140, 0.024329, 1.193186, 0.074276, -0.201011, -0.071350, -0.374727, -1.007946, 0.783254, 0.397245, -0.410134, -1.148527, -0.695087, -0.264616},{-0.309958, -0.127567, 0.360634, 0.121468, 0.668717, 1.597999, 0.159107, 0.113028, 0.641328, 0.375938, 1.184456, 1.172225, 0.967287, 0.597938, 0.689796, 0.166358, 0.201359, -1.817709, 0.199443, -0.147752, 0.235155, -0.239147, 0.564844, -0.668801, -0.882806, -1.583652, -1.094259, 0.608691, 0.903458, 0.327398, -0.065759, -0.111956, -0.279618, 1.170972, 2.059786, 0.754801, -0.992900, 1.605575, -0.046160, 0.527597, -1.116058, 0.705520, 0.234939, 1.177993, -0.255440, 0.896811, 0.550260, 0.250037, 0.004556, 0.887139, 0.373171, -0.333203, -0.896695, 1.469515, 0.145587, -0.701248, 1.502482, 0.189546, -0.801933, 1.209197, -0.549520, -0.141000, -0.451698, 0.082501, -0.148065, -1.852271, -1.304835, -1.088866, 1.219954, -1.043699, -0.311641, 0.143936, -0.578895, -0.427148, 0.478088, -0.423616, 0.995741, 0.110182},{-0.648855, 0.523012, 2.007572, 1.044726, 0.032936, -1.167891, 0.918424, -1.670153, -0.492904, -1.229060, -0.957122, -0.555952, 0.724038, 0.008287, 2.028124, -0.994508, 0.757178, -0.255839, -0.217749, -1.308045, -0.844536, 0.019979, -1.037231, 1.557324, 0.439195, -0.058464, 1.873917, 0.118161, 0.824782, 1.878465, -0.995900, 2.531672, -2.424695, 1.032588, -1.427260, -1.809085, -0.539678, 0.029738, -0.205435, 1.087130, 2.634985, -0.644247, -0.659898, -0.942712, 0.517787, -0.874991, -0.234733, 1.095343, -0.815635, -0.221856, -0.257905, -0.365517, -0.047153, -0.412215, -0.545946, 0.367469, 0.043344, 0.659438, 0.154771, -0.756143, 0.570229, -0.260434, -0.014602, 1.886595, -1.103645, 0.419467, 1.466995, -0.106119, -0.477713, 1.350242, 0.612651, 1.884418, 1.099434, -1.521656, -0.736031, -0.313602, 0.994101, -0.477979},{0.683004, -0.768849, 0.233627, -0.219879, -0.354496, 0.439132, 0.125493, 0.183233, -0.823088, 0.057971, -1.193543, -0.174931, 0.626941, -2.373893, -2.278402, -0.433287, 0.164654, -0.229175, 0.093673, -0.708060, -1.204625, 0.318299, -0.270116, 1.268504, 0.238433, -0.425026, 0.245810, -0.825309, -0.315748, 0.782546, 2.203868, -0.631304, 1.596603, -1.399712, 0.605342, 0.569862, -1.230014, 0.487855, 0.933472, 0.287486, 0.615595, 0.158972, -0.900191, 0.004989, 1.401098, -0.205041, -0.447988, -1.244612, -0.310221, -0.552022, 0.526212, -0.648388, -0.349117, 0.694589, -0.052878, 0.388369, -0.079807, 0.125320, -0.437260, 0.576278, -0.821446, -1.031162, -0.102252, -1.890552, 0.631009, 3.170210, -0.105081, 0.517148, 0.087005, 0.105658, 0.291103, -0.557124, 0.834106, 0.647914, 1.727407, -0.597592, -0.706239, -0.744837},{-0.142863, -0.190687, -0.677802, 0.541928, -0.528350, -0.086251, -0.541061, -1.465353, -0.779398, 0.243972, 0.889706, 0.555673, -0.155074, 0.122988, 1.643163, 0.501338, 1.010653, -1.464879, 0.407671, -0.557713, 0.374782, -1.987558, 0.312985, -0.119365, -1.038046, 0.361601, -2.203876, 1.202820, 0.546213, 1.198247, -1.013558, -0.784017, 1.469409, 0.014891, 0.621243, 1.188732, -1.744372, 1.112306, -0.107002, -0.030896, 2.004882, 0.784063, 0.094462, 0.889418, -1.202030, 0.015549, 0.965122, 0.595463, -0.114772, -1.641691, 1.252105, 0.171299, -0.887676, 1.290318, 1.631975, 0.753095, 1.865888, 1.957356, -0.365470, -0.756155, 0.473213, -1.733213, -0.169810, 1.214663, -1.324402, -0.235976, -0.975301, -0.079904, -0.974595, -1.793948, -1.040145, -0.339380, 0.566489, 2.108562, 0.536009, -0.943189, -0.591965, -1.310724},{-0.267734, 0.733389, -0.207361, 0.920365, -0.824099, -0.264518, 2.209799, -2.215483, -1.677794, -0.165419, -2.038902, 0.641803, 0.128960, 0.318033, 0.535143, -2.236907, -1.507000, 0.264504, -1.515159, -2.712717, 0.931541, 0.288292, -0.979555, 0.460291, -0.197681, 0.205745, -0.055009, -0.870403, 0.526138, -0.441143, -0.027580, -1.313584, -0.080180, -0.276264, -0.053315, 0.436185, -1.224242, -1.051437, -0.411047, 0.418574, -1.047181, -0.309450, 0.925271, 0.484478, -0.426105, 0.872605, 2.360284, 2.316416, 1.528572, -0.612140, 1.185332, -0.412340, -0.501624, 0.833317, 1.179694, 0.834088, 0.026720, -0.231518, -0.749734, -0.446390, -1.521167, -0.244811, -0.275274, 1.291191, 1.214845, 0.482296, 1.160955, -0.595950, -1.025571, 0.499376, 1.634628, 2.194716, -0.577890, -0.477640, 0.811492, -1.256759, 0.299773, -0.944199},{-1.206295, 0.246115, -0.060188, -1.222964, 1.224429, -0.263254, -1.795640, 0.598747, 0.956433, -0.718401, -1.416244, -0.060418, 2.384520, -2.660911, -0.619069, 0.256993, -1.221679, -0.184686, 0.022268, 0.598271, -1.169631, 0.797558, -0.163624, 0.339723, 1.888329, 0.680604, 2.069029, -0.594587, 1.053054, -2.494215, -1.814901, -1.248295, -0.886255, -1.899531, -0.297783, 0.181499, 1.917798, 0.520569, 1.310232, -0.146241, -0.495754, 0.772147, 1.456715, 0.412983, -0.035819, 0.312913, -0.727644, 0.360882, -1.783769, 0.856056, -1.285535, -1.074697, -0.447824, 0.044584, 2.251877, -1.543802, -0.116071, 0.223978, 0.578537, -0.182480, -0.302596, -0.137659, -2.022104, 0.429648, 1.805056, -0.107302, -0.111220, 0.756540, 0.169686, 1.150339, -1.931295, -0.190681, 1.329445, -0.269141, 1.629698, -1.455760, 0.625711, -1.229518},{-0.851008, -0.636024, 0.462526, 0.979550, -0.994911, -0.333219, -0.104331, -0.384656, 0.081727, -1.125633, 0.967852, -1.733762, -0.909368, 0.556380, -0.833990, 0.140227, 1.132169, -0.693703, -0.155859, 0.617583, 0.241920, -0.058501, -0.387551, 1.128470, 0.218007, -0.534354, 0.768261, -0.517934, 0.834150, 0.525596, 2.262703, -3.154880, -0.206904, -0.041780, 0.762556, 0.341749, 1.563098, 1.521854, -0.895386, 0.397092, -0.309828, 0.486571, -0.724250, -0.186717, -2.963530, -0.507607, -0.206215, 1.469232, -1.016699, 0.154739, -0.427824, -0.717655, -1.013010, -0.157206, -0.526444, -1.512255, -0.519020, 0.501946, -0.887444, -1.273439, -0.555250, 0.861181, 0.248475, 1.784143, 0.053307, -0.979233, 1.303880, 1.080608, -1.588869, -0.109861, -0.956825, -0.629084, -0.078659, -0.433425, 0.241477, -0.113841, -0.197823, 1.395718},{0.476033, -1.009619, -0.721935, 0.798826, 0.441473, 0.226326, -0.897795, -0.925193, -1.679902, -0.351700, 0.040851, -0.099315, -0.381963, 1.079571, 0.200320, -1.242889, 0.891734, -1.175091, -0.241028, -1.583610, -0.526870, 0.689872, -2.134821, -0.207993, -0.750640, 0.113236, 2.244138, 0.173515, -0.180880, 1.080229, -0.388039, -0.240378, -0.261793, -0.135600, 0.551713, -0.345641, 0.636675, 1.060635, 1.311736, -0.088116, -0.218184, -0.244053, 0.258182, 0.144813, -3.345943, 0.081189, -0.415121, 0.209904, 0.007740, 1.362328, -0.510977, 0.508995, 0.282863, -1.213765, -0.625425, -0.001629, -1.254496, 0.529254, 0.871632, 0.964324, 0.521693, 0.958560, 0.616770, 1.843329, 0.660315, -0.445535, -0.930374, -1.302784, -0.410160, 1.996556, 0.408960, 0.860363, -1.056438, 0.605459, -0.032286, 0.321060, 2.723595, -0.863396},{-0.700138, 0.066611, -0.774179, -0.798456, 0.384013, 0.179239, 0.285824, 0.116296, 0.307186, -0.872686, 1.985589, 1.213113, 0.909091, -1.149293, 1.080755, 0.136044, 0.054826, -0.795982, 1.616245, 0.279993, 1.531700, 0.014757, 1.618340, -1.008801, -0.569593, 0.880015, -0.320489, 1.646014, -0.753439, 1.519376, -0.776864, -0.750922, -0.451440, -1.534397, -0.365596, 1.127907, 1.050664, 0.681251, -1.610273, -1.640810, 0.648671, -0.883960, 0.716491, 0.544750, -0.072777, -0.998140, 0.202657, -1.492792, 1.023753, 0.999901, -0.858224, 1.650700, -0.200368, -0.753593, -2.851123, -1.376096, -0.213908, 1.079478, 1.186513, 0.841188, 2.106498, -0.198842, -1.114765, 1.062665, -0.134148, 0.776591, -0.060930, 0.590895, 1.125044, 0.218089, 0.162081, 0.248379, 1.527457, 0.529427, 0.878407, 1.270157, -0.013654, -0.108410},{-1.588153, -0.421466, 0.605644, -1.060856, 0.838189, 0.594776, -0.878044, 0.440953, 0.004224, 1.290913, 1.771426, -0.780668, 0.066318, -0.395810, -0.555335, 0.347399, -0.131673, 0.481034, 0.994205, 1.332058, 1.202172, -0.800471, -0.424733, 0.999982, 0.041743, 0.380229, -0.322865, 0.477330, -1.689340, 0.977037, -2.252662, 0.988834, -1.790763, -0.218670, 0.259598, -0.501719, -0.413971, -1.354939, -0.513405, 0.029260, -0.803197, 1.381602, -1.246884, 0.376666, -1.107084, 0.020522, -0.362909, -0.996966, -0.661848, 1.038547, 1.878820, 2.118067, 0.596795, 0.391692, 1.441459, -1.499767, 2.412202, -1.655803, -0.801255, 1.109182, -0.099172, -0.907470, 0.371175, 1.702145, 0.726440, -0.870293, 0.398332, 0.542905, -0.387659, -0.590364, -1.070426, 0.550871, 0.768675, -0.330425, -0.120935, 1.183161, -0.976378, 0.918887},{2.228434, -1.528227, -0.849397, -0.694923, 0.079115, -0.336255, -0.295179, -0.206728, 0.751156, -2.110033, -1.627435, 0.436255, 1.638358, 0.831379, 1.696089, 0.052747, 0.702156, -0.060824, 0.612243, 0.532200, -0.450702, -1.379250, 0.165490, -0.208264, 0.810103, 0.328582, -0.057971, -0.983661, 0.142667, 0.098780, -0.935165, 1.912544, 1.002547, -0.414514, -0.426996, 0.374504, 1.050152, -0.192725, -2.701013, -1.742646, -0.366244, -0.438989, -0.131761, -1.069694, 0.240529, 1.108272, 0.715119, -0.400200, 0.809668, 0.840018, 0.674084, -0.614722, -1.482987, -0.340168, 0.905308, 1.593230, 0.060528, 2.038234, 0.089053, 0.436026, 0.873785, -0.942839, -1.103112, 0.256839, -1.453179, -0.089928, -0.359595, -0.528707, 0.081493, 0.484348, -1.425590, 0.776773, -0.080274, -0.631850, 0.782331, -3.727566, -2.052440, 0.364854},{-1.335714, -0.482016, -0.752755, 0.547553, -1.165855, 0.044346, 0.746998, -1.898308, 0.380827, 1.394765, -0.716658, -1.870556, 0.423328, -0.734768, 0.928901, 1.368002, -1.108339, 1.528893, -1.160797, 0.762015, 0.221249, -1.610745, -2.262324, 0.145479, -0.690088, 0.451431, -0.116803, 0.069014, -0.064888, -0.196528, 2.307142, 0.905623, -0.241105, -0.517897, 0.595578, -0.299620, 0.258262, -1.604824, 0.352101, 0.826207, -0.139957, 0.115325, 0.110420, 0.348534, 0.385586, 0.863836, -0.752375, -0.742398, 0.593699, 0.969832, 1.207920, -0.492792, 0.918786, 0.963369, 1.012142, -2.107645, 0.163450, 0.299781, 0.740577, -0.479756, -0.146813, 2.053976, -0.675735, 0.340459, 0.729430, 0.018248, -1.018669, 0.413034, 0.597287, 1.304820, 1.191479, -0.084604, -1.211023, -0.621937, -2.209849, -0.045870, -0.497383, -0.031710},{-0.594989, -0.336443, -0.126910, -0.186528, -0.111337, 0.740537, -0.665457, -0.737788, 0.153414, -0.257460, 1.177071, 1.355893, 2.317161, -1.002087, -0.807857, -0.825271, -1.184168, -1.774867, 1.135116, 0.942927, -0.833868, 0.242509, 1.128247, 2.262981, -1.846215, -0.529825, 0.227055, -0.158405, 1.651928, 0.924358, -0.901959, -0.133478, 1.091063, -1.178793, 0.775860, 0.574031, -1.367770, 0.604636, -0.284111, 0.186425, -0.233974, -0.053385, 0.273673, -0.426634, -0.248205, -0.400365, 0.547591, 0.941225, 0.431926, 0.357007, 0.118138, -1.787831, -0.645065, 0.198514, 0.156171, -1.497465, 0.372656, -0.415442, -0.060924, 0.215709, -1.427342, 0.069688, 0.920818, -0.379009, 0.211650, -0.129655, 0.561208, 0.508141, -1.312868, 0.916806, -0.588678, 0.381284, -0.841687, 0.128492, 0.384595, 0.592637, 0.291945, 3.153694},{0.903559, -1.517404, 1.628130, -1.082985, 1.965069, 2.422973, 0.016402, -0.362319, -0.347065, -1.126085, 0.527248, 0.325295, -0.670971, 0.585930, 0.832786, 0.696466, -1.230005, -0.753318, -1.005320, -1.112428, -1.633348, 1.766511, 0.245358, -0.790826, -0.701541, -1.306814, -0.527674, -0.377703, 0.302730, -1.196995, -1.580525, 1.743997, -0.383698, -0.968103, 0.124302, -0.343551, 1.568691, -1.298990, -0.782093, 1.318485, 0.249548, -0.352855, 0.646746, -0.214108, -1.299218, -1.346035, 1.760402, -1.743783, -0.830452, -0.588720, -0.220110, -0.030668, -0.131085, 0.359130, 0.315190, 0.606150, 0.088569, -0.972970, 0.079549, 0.424555, -0.815048, -1.293175, 1.144907, 1.325616, 0.602961, -0.795350, -0.313160, 0.659436, -0.558702, 0.959823, 1.097190, 1.963691, 1.078878, -0.960092, -1.073002, 0.967926, 0.196728, -0.215126},{0.031865, -0.483448, 1.103623, -1.567401, -0.187923, 1.461036, 1.599438, 0.530294, -1.716598, -0.837459, 0.685970, 1.259481, -0.902560, -0.066288, -1.482046, 1.346024, -1.354612, 0.346600, -0.120411, 1.012764, 0.598952, 0.014415, 1.030380, -0.050549, 0.599356, 0.637964, -0.209006, -1.114369, -0.942798, 0.118786, -0.613396, 1.027125, 0.527938, -0.815659, -1.522921, 1.083711, 0.646364, 0.782980, -0.410478, 1.631173, 0.076557, 0.238875, -0.739031, -0.786522, 0.928857, 0.041973, -0.912388, -0.785056, 0.381600, 1.238479, -1.644315, -0.193170, -0.290307, 0.939362, 0.137029, -1.073983, -0.247027, -0.540271, -1.075449, -0.496832, -0.585705, -0.585664, -1.278417, 1.550173, 1.112851, -0.053415, -0.002788, -0.841807, -0.038545, 0.430084, -0.032530, 0.008228, 0.965201, 1.002355, -0.701344, 0.799484, -1.071060, 0.650942},{0.032132, 2.198429, -0.222899, 0.799606, -0.548316, -1.719928, -1.321554, -1.176140, -1.452384, 1.259704, 0.276077, -1.348058, -0.964405, 0.599079, 0.459933, -1.274490, -1.664017, -0.873268, -1.567642, -0.001383, -1.692010, -0.598725, 1.908933, 0.418036, 0.789237, 1.578277, -0.927123, -1.354874, -1.161622, 0.250322, -0.617651, 0.142531, -0.182842, -0.205349, -1.203654, 0.914639, -1.109092, -1.186782, -1.172788, 0.531561, 1.002963, -1.434993, -2.843421, 0.363784, 0.943076, -0.634501, -0.626600, -1.875978, -0.837864, -0.092376, -0.265720, 0.095579, 1.419373, 0.535004, 1.100214, -0.113858, -0.618655, -1.026940, -0.744154, -0.594237, -0.725300, -0.427056, 0.772660, -1.214909, 0.302890, 0.279482, 0.973766, -1.804720, 0.059858, 1.306160, -0.649076, 0.062056, 1.654788, -0.897606, -0.034418, 1.688697, -0.437536, -0.423248},{0.314488, 1.193254, -0.166173, 0.260885, -1.414464, -0.824496, 0.032277, 1.925294, -0.944866, 0.108379, -0.191577, 0.417808, -2.286343, -1.525273, -0.072904, -1.480363, 1.011190, 1.210164, 0.174930, -0.438526, -0.919454, 0.894515, 0.188031, -2.359729, -1.638977, -1.029233, -0.437276, -1.734726, 2.965376, 0.386261, 0.169602, -1.454210, 1.808609, 1.059604, -0.462906, 1.789364, 1.469563, -0.901236, -0.266702, 0.803832, 1.515232, 0.606445, 1.172794, -0.233183, -0.732770, 2.187882, 1.209345, -1.345170, 0.853290, 1.478981, 1.945995, -0.482739, -0.286668, -0.984943, 0.903528, -0.464864, 0.554906, 2.430953, -0.722737, -0.925394, 2.127716, 1.592402, 0.055532, 0.916619, -0.678885, 0.850625, 0.062058, 1.236757, -0.328315, -0.119132, -0.145977, 1.807368, -1.977152, -1.584025, 0.512447, -1.878879, -1.388870, 1.973303},{1.468946, 0.205776, -0.048651, -1.114933, 1.468991, -2.464538, 0.469804, -0.594461, 0.079471, -1.945434, 1.275049, 1.487507, 0.830929, 0.368122, 1.246604, -2.227906, -1.306044, 0.622903, 0.195917, -0.525030, 0.518316, -1.669244, 0.609639, 0.391406, 0.134723, -0.624359, -1.927354, 0.886156, -1.553503, -0.778129, -1.401680, 0.913102, -0.311489, 2.522339, 1.286108, -0.690505, -0.501252, -1.588239, -0.486704, -1.516470, 0.158626, -2.345729, -0.089805, -0.477187, 1.233060, -1.692596, 0.405637, -1.483418, 0.860104, 1.048531, -0.427145, -0.979272, 1.122299, 0.206988, 0.130231, -1.144450, -0.087012, -1.102339, -1.315372, 0.849950, -1.837356, -1.188119, -0.413141, -0.098552, -1.190795, -1.207585, -0.404703, 1.267034, 0.999622, 0.904429, -0.159667, -0.268345, -1.313549, 1.939271, -1.474685, 1.568101, 0.372939, -0.421785},{1.229758, 0.200827, -0.061423, -1.892222, 0.058342, 1.011763, 0.544888, 0.083092, -0.865575, -0.003479, 1.387338, 2.475002, -0.097467, 0.853397, 0.201883, -0.679892, -1.539576, 0.916504, -0.965895, 0.062688, -0.225552, -0.194010, 0.559596, -0.316714, -0.007284, -1.494586, 1.587757, 0.988889, -1.180409, -0.353079, 0.532296, -1.381330, -0.548438, 1.293064, 0.537833, -1.520305, -1.471750, -0.310568, 0.286368, 1.755702, 0.398061, -0.177464, -1.232927, -0.362768, 1.075149, 0.940022, -0.102705, 1.200626, -0.079852, 0.048823, 1.014653, 0.204476, -0.395499, 1.063427, -0.099324, 0.177893, -0.591094, 0.049180, 1.045753, -0.523569, -0.237298, 0.756410, 0.112970, -0.524842, -1.410921, 0.030948, -0.801416, -0.436094, 0.345355, 0.855858, -0.153909, 0.684032, -1.301564, 1.318298, 0.773925, -0.833309, 0.552166, 0.088264},{-1.363392, 1.211600, -0.489368, 0.972049, -0.047682, -0.949704, -0.596748, -1.332954, 0.115690, -1.185670, 0.455400, -0.540886, -2.279615, -0.093430, 0.442764, 1.175685, -0.501330, -1.216144, 0.800239, -1.639767, 0.763941, 0.250539, -0.132532, 1.410604, 0.671308, 0.253954, 1.016961, 0.210665, 0.637539, 0.240025, -0.465364, 1.326497, -1.915731, -0.403848, 0.537379, 1.664413, -0.862367, -0.040887, 0.611955, -0.366645, -0.182355, -1.814915, -0.218803, 0.524333, 1.102297, 1.059135, 0.249988, -0.873688, -0.714285, 0.719275, -1.482008, -0.483832, 0.236916, -0.109905, 1.035110, 0.752547, 0.149113, 0.267994, -0.226537, 0.887043, -1.751284, 0.882188, -1.195644, -0.015178, 1.098721, -0.424986, -0.484118, 1.003302, 0.449546, 0.483597, -0.328321, -0.653789, -0.138565, 1.050625, 1.898460, -1.002182, -1.425112, 0.403818},{0.959638, -0.750351, 0.712093, 0.880186, -1.886882, 0.548720, 1.461237, -0.536054, 0.188495, -0.792863, -0.553672, 0.833350, -0.158201, -0.217825, -0.179395, -0.347227, 0.495473, -0.713665, 1.190354, 1.656700, -0.936910, -0.352038, -0.241867, -2.612673, -0.172559, 0.484093, 2.555736, 1.684779, 0.333454, -0.731458, -0.576656, 1.096703, -1.517442, 0.648607, 0.693152, -1.172142, -0.251944, 0.067529, 0.219167, 0.352806, -1.398740, 0.881636, -1.854338, -0.697920, -1.287362, 1.477442, 0.563645, -0.947312, 0.369729, -1.156230, -0.046676, 0.309436, 0.467969, 0.137059, -0.785482, -0.621247, 1.876609, 2.026780, 0.462955, -0.410086, 1.284065, 1.531808, -1.057184, -1.843818, 0.449412, 0.193601, -0.315475, -0.974837, 1.446631, 0.111388, -0.101112, 1.085343, -0.630661, 0.307368, -0.353931, -0.364534, -0.951972, 0.993272},{-0.218590, 0.415020, -1.336333, 0.174542, -1.609044, 1.167489, 0.150929, 1.773506, 0.836223, 1.108376, 0.947564, 1.380958, -0.714919, -0.057328, 0.386645, 0.551176, 1.825358, 0.743084, -1.071641, 0.117675, 0.185421, -1.282665, -1.004734, 0.076215, 0.176836, 0.091805, 0.246205, -0.259839, -1.777182, -2.143513, -1.864963, 0.447279, -0.039136, -2.022352, 0.004204, 0.157371, 0.726512, -0.726503, -1.036176, -2.054439, -1.381973, 0.690906, 1.091289, -0.254556, 1.182356, -0.225190, -0.340722, -0.620706, -1.656702, -1.071648, -0.081227, 0.853145, 0.478266, -1.878385, -0.374242, -1.307910, -1.078688, -0.443453, 0.675261, -0.255084, -0.962531, -0.406052, 1.591331, -1.033466, 0.430898, 1.119917, -0.803116, 0.102695, -0.967901, -0.480593, -0.821401, 1.445583, -0.752634, 1.532722, 1.375580, -0.063267, -0.224015, -1.010520},{0.563630, 0.698352, 1.655066, 0.930560, 0.358130, -2.600687, -0.331811, 0.556786, -2.160057, 0.640919, -0.700508, 0.410469, -0.664145, 0.792340, -1.446547, -1.229146, -1.519459, -1.088831, -1.267460, 0.879497, 0.024306, 0.272405, 0.050700, 0.368552, 1.048758, -0.152013, -0.437354, 0.037890, 1.413719, 1.122456, 2.027371, -0.849690, -1.311004, 0.762888, 1.225760, -0.694820, -0.541314, 0.088208, -0.612456, -0.806560, 0.420513, -0.970796, 1.441218, -0.184960, 0.223295, -0.886951, -0.947456, -0.304377, -0.694290, -1.312015, -0.049597, 1.159259, 0.542950, -1.377214, 0.164808, -1.484300, 1.348379, 1.194054, 0.171322, -0.302090, 0.362688, -1.853755, -0.976516, -0.532870, 0.417847, 1.416528, -0.150558, 0.056691, 0.230042, 0.556444, 0.469575, 1.199921, -0.478478, -1.300589, 1.576676, -0.495897, 0.008029, -0.402053},{0.450114, 1.873244, -1.800558, 1.147428, 0.955914, -1.545758, -1.368239, -0.183724, -0.832036, 0.572161, 0.856407, 1.103212, 0.790918, -0.578510, -0.430446, -0.565827, -0.834244, 0.505762, -0.428392, -0.650230, -0.180137, -0.142475, -2.255085, 0.577574, 1.891399, -0.189144, -1.484373, 0.649034, -2.315541, -1.906094, -0.949576, -0.166973, -1.753462, 1.079769, 1.443401, 1.303179, 0.715028, -1.039608, -1.437515, -0.588704, -1.117844, -0.936180, -0.758758, -0.981856, -0.376246, -0.042838, 1.852040, -1.782394, 1.259120, -1.373753, -0.385758, -0.919441, -1.296816, 0.640830, 0.214092, 1.437561, -0.555327, 0.614783, -0.359094, 1.375763, 1.072275, 0.744273, 1.447830, -0.717081, 0.169539, 0.947743, 1.899992, -0.753801, -1.358851, -0.095882, 1.424051, -1.704102, 1.915693, 0.401761, 2.280415, -0.717752, 0.081183, -0.568036},{0.691374, -2.105562, -0.338004, 0.630477, 1.432226, -1.751157, 0.330413, 2.193668, 0.602818, 0.224898, 0.636847, -0.485806, 1.544669, 1.099519, -0.335826, -1.064701, 1.283175, -0.783685, -1.412806, -1.246124, 0.841903, 0.157535, -0.093159, 0.075899, 0.441879, -0.192072, -0.110143, -0.255988, -0.580239, -0.786227, -1.130678, 0.726615, -1.526730, -0.899016, 1.046947, 0.936721, 0.169265, -0.134627, -1.215874, 0.675398, 0.636564, -0.500249, 0.046246, 0.515713, 1.183964, -0.608415, 0.947502, -0.481247, 1.734230, 1.256046, -0.829061, -0.040957, 1.450780, 1.387289, -0.803757, -1.578688, -1.133498, 1.361166, -1.334961, -0.013086, -0.887724, 1.860592, -0.146788, 0.037652, 0.840725, 0.651761, 0.344669, -0.750986, -2.197398, -0.689865, 1.507173, -0.762899, -0.270020, -1.553340, 1.225375, 0.375602, -0.577963, -0.415229},{-1.335091, 0.414939, 0.120274, -0.875972, -1.059791, 0.182435, 0.139674, -1.022939, -0.009729, 0.214148, -0.934048, 0.905113, 0.170045, 0.217431, 0.174057, 1.553167, 1.124867, 0.204594, -0.105318, -0.067278, -0.978060, -0.478679, -0.380042, -1.645813, 0.804375, 0.215784, -0.347173, 1.035379, -0.906560, 0.342691, -0.825881, -0.820042, -2.372825, 0.251021, -0.065701, 1.370336, 0.783486, 0.073949, 0.893969, 0.363377, 0.160130, -1.634097, 0.352365, -0.377206, -0.740164, 0.306366, 0.495450, 1.183893, 1.282092, -1.482326, 0.466785, -0.093085, -0.772763, -1.853761, -0.616067, -0.169517, -1.153946, 0.456316, -0.540897, 0.450005, -0.269871, 0.963563, 1.318050, -1.328638, -1.237731, 0.135590, 1.084800, 0.013097, -0.270807, 0.987402, 1.072446, -0.959856, 0.372825, -0.202003, -0.781296, 0.156092, 0.621582, 0.904631},{1.631550, 0.140260, -0.630303, 0.448119, -0.506330, 1.640566, 1.192373, 0.641894, -1.780559, -1.715105, 1.658473, -1.571096, 0.180432, 0.230235, -0.100262, 0.007662, 1.061819, -1.599337, 0.600736, 0.616471, 1.651000, -1.015840, -0.473634, 1.447175, -1.168922, -0.799010, 0.307120, -0.323751, -0.650056, -2.662814, 1.267948, 0.986798, -0.122472, 0.582722, -0.144655, 0.710294, -0.809096, 0.534816, 0.892958, 0.159389, -1.031711, -0.626421, -0.904081, 1.028254, -0.366225, -0.039511, 0.682227, -1.454977, 2.127183, 0.189998, -0.652036, 0.757976, -1.260872, 1.404328, 0.262066, -1.076003, -2.094957, -1.649132, 0.009712, 0.424415, -1.601104, 0.617837, -0.401908, -1.694190, 1.107294, -0.610286, -0.777729, -0.966525, 1.988396, -1.715682, -1.343881, 0.493795, -0.504467, 1.114396, -0.296006, 0.809484, 0.802793, -0.296544},{-0.410056, -1.520714, -0.702321, 0.263317, 0.072894, -0.075457, -0.246967, 1.454950, -1.339580, -1.027056, -0.461721, 0.053037, 0.156801, 0.056495, 1.536552, -1.102850, 0.547137, -0.507327, 1.287642, -0.420891, 0.883055, -0.847065, -0.378231, 0.668727, 0.954024, 0.997325, -1.351893, 0.258613, 0.004370, -0.001543, -0.237355, -0.514868, 0.128280, 0.573153, 1.547302, 0.203990, 1.300307, -1.029449, -0.866945, -1.924908, 0.854107, -1.480641, -0.770646, 1.062980, -0.849055, 0.927578, 1.773136, 0.367383, -0.180951, 0.665012, -0.374763, 2.367730, -0.073781, 0.466919, 1.204699, -0.218970, 0.899180, 1.177133, -0.947156, -0.612698, 1.117557, 0.402475, 0.238229, -1.427640, -1.244451, -0.329005, 0.617061, -0.309998, 1.280155, 0.125699, 1.506418, 0.172419, 1.933983, 0.850458, 2.419332, 0.153970, 0.017666, 1.542590},{-1.014851, -0.897315, -0.632823, -1.842679, -0.093989, -0.585680, -0.239971, -0.652873, 0.922446, -0.378591, 0.215997, 0.137765, 1.011977, -0.880682, 0.127477, 0.798986, -1.644234, -0.029838, 0.177388, -0.188676, 0.262171, 1.371172, 0.217470, 0.746661, 0.203146, -0.718815, -0.422170, -1.197330, 0.221717, -0.307011, -0.079788, -0.409875, 0.396267, 0.445527, 0.351264, -0.297396, -0.692850, -0.538188, -0.282413, 0.684386, -0.007088, 1.632013, 0.884186, 0.259589, -1.242744, -0.619740, -0.583269, 1.541604, 0.385534, 0.641836, -2.542657, -1.210664, 1.187132, -0.805657, -1.254078, 1.476216, 1.187540, 0.296042, 0.062769, -1.386942, -0.043033, -2.243891, 1.232670, -0.638158, -1.444451, 0.029870, 0.543256, -0.718481, -0.234076, -0.340991, -0.122265, -0.380971, -1.965548, 0.172847, 0.619018, -1.104841, -0.347299, -0.269475},{0.489590, 0.320464, -0.491696, -1.612521, 0.234536, 0.623435, -0.360350, 0.396667, -0.242034, -0.910583, -1.025498, -0.543775, 0.472410, 1.982962, -0.797615, 1.335382, -1.327187, 0.043715, 0.272480, 2.104299, -0.489569, 0.842946, 1.355976, 0.722832, 2.317532, -0.176714, -1.896448, -1.366867, -0.680967, 1.313815, -0.830166, 2.201501, 0.410386, -2.297712, -0.234309, 0.748030, -0.022660, 1.488608, 0.757213, 1.252601, 0.518089, -0.389866, -0.469822, 0.913817, 1.190712, -0.273165, -0.033005, -0.263522, 0.468781, 0.186735, 1.600255, 1.502456, -1.531359, 0.060597, 1.040604, 1.310934, -0.286533, 0.829723, -0.323212, -1.157836, -0.550166, -0.362587, 0.999197, -0.097608, 0.311940, -0.704688, 0.385154, -0.508881, 0.500481, -0.257543, 0.268984, 0.594493, -0.607843, -0.931215, -0.556773, -0.516683, 1.305094, -1.020667},{0.063886, -1.069204, -0.071680, 1.125582, 2.073904, 0.279222, 0.538300, 2.047395, -0.387370, 0.953824, 1.599146, -0.166843, 1.503631, 0.211735, -0.329890, 0.056150, 1.565406, 0.061965, -0.385415, -0.358870, -2.237869, -1.058882, 0.383726, -0.405771, -0.575692, -0.589962, -0.269583, 1.121961, -0.366907, 0.677855, 0.195972, 0.278751, -2.761982, -0.367476, -0.124397, -1.595963, 0.572108, 0.874683, -0.835395, -0.005444, 0.754455, 0.607180, 1.477767, -1.811046, 0.824995, -0.298280, -0.218228, 0.185661, -0.509863, -0.457950, 0.716498, -0.313139, -0.073347, 0.376487, -0.414808, 0.812816, 0.247079, -0.095718, -0.795053, 2.760766, 0.150886, -0.316517, 0.033765, -2.823794, -0.217763, -0.663484, -1.015242, -0.372737, 0.773482, 1.462557, 0.277506, 0.369615, 0.143711, -0.250095, -0.263355, -0.190214, -0.331472, -0.423782},{0.734181, 0.195729, 0.297261, -0.875599, -0.888205, -0.861456, 0.661573, 0.259029, -0.722430, 1.339450, 0.731612, -0.487773, -1.185022, 0.520004, 0.247028, -0.635292, 0.429494, 0.776728, 1.885614, -1.426404, 0.086682, -0.654757, 0.595463, 1.434791, -0.296138, -1.140647, 0.471269, -0.363691, -0.576443, -0.033010, -0.820491, -0.403179, 1.383825, -0.627015, 1.407188, -1.109394, 0.578667, 1.600035, 0.250867, -0.655871, 1.606314, -0.520251, 0.502984, 0.723003, 1.537313, 0.215720, 0.117950, 0.443128, 2.323730, 1.617600, -1.773326, -0.271083, 0.475939, -0.649588, 0.980504, 0.256234, -0.079586, 1.181617, -2.008850, -0.367020, -0.683848, -0.155488, 1.014387, 0.453326, -0.164216, 0.446262, -0.500624, -0.303656, -1.103706, 0.213765, 0.947444, 0.104487, -0.081734, 0.724383, -0.776140, -1.299124, 0.154568, 0.451041},{-0.675608, -0.332612, -1.150571, -0.170217, -0.485510, -0.653436, -0.255074, -0.833046, 0.609805, 0.304806, 1.152352, -0.049783, -0.519317, -0.379608, 0.369704, -0.089302, 0.810030, 0.777218, 0.246671, -0.702889, 0.198109, 0.062560, 1.086216, 1.283279, -0.029388, -0.974390, 0.410449, 0.694389, 1.560269, 0.343533, 0.184221, 1.143693, -0.797068, 0.796657, 2.290685, 0.305673, 0.045762, -0.228172, 0.235915, 0.906106, 0.151781, -0.119054, -1.403694, -0.794069, 0.378298, -0.036112, -1.167148, 1.072537, -0.222551, 1.340137, -1.242772, 0.851907, -1.235551, 2.173001, 0.737346, 1.254648, -1.178580, -1.321434, 0.326096, -0.926192, 0.100943, 0.367082, -1.172773, 0.382394, 0.756246, 0.029265, 1.300090, 0.436868, -0.559270, -0.634880, -0.890298, 1.729750, -0.021888, 0.911720, -1.228079, -1.297428, 1.804699, 0.616081},{-0.282248, -0.097211, 1.239041, 0.649842, 1.535752, 0.299419, 0.916934, -0.324986, -0.767887, -0.828009, 1.526272, 1.553900, 1.728285, -0.582681, 0.282935, -0.696401, 1.111460, -0.631572, -1.475934, -0.121123, -1.379852, -1.301099, -0.138688, 1.015751, 0.396959, 1.010792, -0.820320, -0.721659, 0.725659, -0.198090, 0.372141, -1.437677, -0.876359, 0.280804, 0.942295, -1.474997, 1.322601, 0.043657, -0.464212, -0.049005, -0.608928, -0.487235, -0.683043, 2.207237, 0.912242, 0.175239, 0.166344, -1.019725, 0.624140, -0.976313, 1.160714, 0.429836, 1.724023, -0.525935, 0.078078, 0.565444, 0.247683, 2.146359, -0.315030, -0.532675, 0.108550, -0.849114, -0.009014, -0.096812, -1.141700, 0.228802, 2.039348, -0.399589, 0.715624, 0.002989, -0.515198, 1.102514, 0.446398, -0.028554, -0.379106, 0.029541, 1.981033, 1.105502},{-0.509344, -0.282913, 1.017556, 0.431193, 0.675116, -0.401723, -0.000734, 1.045676, -0.136667, 0.319280, -1.379405, 0.530134, -0.648107, -0.496030, 0.797145, 1.202397, 0.273912, -0.476936, 1.381036, 0.578175, -0.454359, -0.483358, -0.910108, -0.658181, 0.201024, -1.129826, 0.683487, 0.098488, -1.126548, -0.123694, 0.272281, -1.144920, -0.905827, 0.467321, 0.096746, -0.794664, -1.317062, 0.593045, -1.119874, 0.782660, 0.649234, 0.089873, 0.141319, -0.966570, 1.659325, 0.356163, -1.280981, 0.979788, 1.132313, -2.355753, -0.396018, 0.402868, -0.030917, -1.102188, 0.678901, 0.977189, 1.349103, 1.636123, 0.623130, 1.836413, -0.603234, 0.666727, 0.211759, -0.499997, 0.522304, -1.870206, -0.393031, -0.543480, -2.462680, -0.602603, -1.160446, 0.582653, -1.187043, -0.230957, -0.136000, -0.133399, 0.091103, 1.686849},{1.347740, -1.550953, -0.344870, -0.604030, -0.378649, 0.907116, 0.613902, 0.532246, -0.624703, 0.989156, 0.033402, -1.563957, 0.316851, 0.973054, -0.764868, 1.684296, 0.055724, 1.177379, -1.905337, 2.183383, -0.376929, -1.626617, -0.046993, -0.121022, -2.057528, 0.508840, 0.504750, 0.406572, 1.334563, -0.770641, 2.055397, -1.009161, -0.103449, -0.186160, -1.707965, 1.184423, -1.460780, 0.382436, -0.605924, 0.041024, 1.103910, 0.523631, -0.673590, 1.176915, -1.714070, -0.992360, -0.450924, 1.052964, -0.651341, 1.422899, 0.591118, -0.233446, 0.291797, -0.895886, -2.014398, -1.208728, -0.919784, 0.985215, 0.329574, -0.304587, -0.365486, 0.974916, 0.306754, -1.855232, 0.561157, -0.056683, -0.844443, 0.574991, 2.164025, -1.234992, -0.576600, 0.212095, -0.726463, -2.321480, -0.306109, -0.263817, -0.662923, 0.952182},{0.385360, -1.021354, 0.245198, 0.604870, 0.054826, 1.445271, 0.347872, 0.673263, -1.172087, -0.193542, -0.116051, -0.595482, -0.496315, -0.620641, 0.053017, 0.104561, -0.935729, -0.301506, -1.175666, -1.380387, -1.416016, -0.631730, -0.528696, -0.899831, 1.839599, 0.889386, -0.660127, 2.152388, 0.032361, -0.136328, 1.045746, -1.124075, -0.455168, 0.575796, -1.009840, 0.772084, 1.508267, -0.859296, -0.973145, -1.234482, 0.781946, 1.719738, -1.003536, -1.551075, -0.461491, -0.013072, -0.934008, 0.795741, -0.233391, -1.733067, 0.447193, -0.756939, -1.055192, 0.304295, -0.781116, 0.688486, 0.010902, 0.127946, -0.617075, 2.685107, -1.322844, -0.456169, -0.940775, -0.108247, -2.686069, -0.264016, 0.307927, 0.215291, -0.924731, 0.613912, 0.613585, 0.271496, 1.475657, -0.635889, 0.625085, -0.527734, 0.913534, 0.118784},{0.875749, -0.031268, 1.032377, 0.776795, -0.775139, -0.313630, 1.510128, -0.134559, -1.079489, -0.882340, 0.530933, -0.230272, -0.449712, -0.700343, -2.484543, 0.578196, -1.482517, -1.260625, 0.258966, 0.461040, 0.047975, -1.035656, 0.172883, 0.345617, -0.278290, 1.609599, -0.495351, 0.954420, 0.471983, 1.034199, -0.768473, -0.311624, -2.317568, -0.443146, -0.245553, -0.111020, -0.934351, 1.636011, 0.768573, 0.682988, -1.588436, 1.321610, 0.254038, -0.105772, 0.811823, 2.875495, 0.187273, 0.371092, -0.549297, 1.549770, -1.491339, -0.171668, -0.518171, 2.236433, 0.783552, -0.053168, 0.158647, 0.649231, 1.460191, -0.077762, -0.385602, -0.637726, 1.445983, -0.103681, -0.577004, 0.861720, -0.200706, -0.087243, -0.232372, -0.416909, 0.935558, 0.985686, -0.731116, -0.645527, 0.885007, 0.161396, 0.203085, -0.256649},{0.228074, 2.543255, -0.468297, -0.661448, -0.457601, 0.621100, 0.443110, 1.040660, -0.076347, -1.752678, 1.107957, -0.461986, 0.381518, -1.100898, 0.309258, 1.088718, -0.472730, -0.686661, -0.088362, 0.219243, 1.270113, 0.410284, 0.284141, -0.586144, -0.457766, -0.977297, 0.819154, -0.292507, 1.516009, -2.524202, -2.405726, -0.559208, 0.475767, -0.087865, -0.170571, 0.335588, -0.120507, 1.272226, -0.265361, -1.183921, -0.722804, -0.013040, 0.340146, -1.032814, -0.301644, 0.979688, 1.338930, 2.388514, 0.013292, 0.800374, -0.099567, -2.211156, 1.021384, -0.048554, 0.620105, 1.134327, -0.377353, -0.773540, 0.632503, -0.182331, -1.112268, 1.306894, -0.460578, -0.267860, 0.604398, -1.160055, -0.875771, 0.794781, 1.006813, 0.285195, 0.185713, 0.537144, -0.365067, 0.508283, 0.494064, 0.166478, -0.935002, 1.976422},{1.363222, -0.958915, -1.157167, 0.254575, 0.263580, 0.733295, -0.379018, 0.944858, -1.461318, 1.307547, -1.039307, 1.360838, -0.445528, 0.047441, 0.009107, -0.438579, -1.890118, -0.137739, 0.182428, 0.550347, 0.803856, -0.184512, 0.664843, -0.272056, -0.587949, -1.564829, 0.359395, -0.960435, 0.465523, -1.674098, -0.132710, -0.100221, -0.908928, -0.527496, 1.371106, -0.606040, 1.462539, 0.639553, 0.592993, -0.054209, 2.273849, 0.400956, -1.415874, -1.333579, -0.276168, 2.617396, -2.049576, -1.081071, 0.086937, 0.262464, 0.596878, -0.255815, -1.481891, -0.313476, -0.141873, 1.125096, 0.908114, 0.473774, -0.589922, -0.108258, -0.015383, 0.386909, -0.863442, -0.566437, -1.687912, 0.963020, -0.034029, 0.086307, 0.359721, 0.801495, -1.997831, -1.264953, -1.040457, 0.837208, -0.694968, 1.616942, -0.078933, 0.335510},{0.432354, 0.418001, 0.082852, 0.648146, 0.719057, 1.023731, -0.205582, -0.865197, 0.675715, -0.633736, -0.009973, -0.274644, 0.581612, 1.001467, 2.000310, -0.590670, 0.118689, -0.352531, 0.195508, 0.670889, -0.717208, 0.945371, 1.361197, -0.326183, 0.789158, 0.316975, -1.134349, -0.924006, 0.072085, 1.593672, 0.148662, 1.869037, 0.361461, -0.401303, 1.462689, -0.044192, 1.180971, -0.384769, 1.284327, 0.371624, 0.831447, 0.412132, -1.974627, -0.959473, -0.902074, -0.572650, -0.440279, 2.100981, -0.397394, 0.617557, 0.829266, -0.538977, -0.325316, 1.346604, 0.826367, 0.672057, -0.781393, -0.797684, -0.971669, 0.179077, -0.998901, 0.053153, 1.060765, 0.634154, -1.547862, -0.115150, -0.449588, 0.324876, 0.157881, 1.458399, 1.026472, -1.828327, 0.051610, 1.843239, -0.006247, -0.621677, 0.294815, -0.477508},{1.731394, -1.432960, -0.297448, 1.266854, 0.685387, -1.221226, 0.122766, 0.695061, -0.709157, -0.599873, 0.895109, -1.155888, 1.166764, -0.356836, -0.303848, 1.608311, 1.505753, -0.063994, 1.769317, 0.343612, -0.151113, -0.655328, 0.620339, -0.375404, -2.030066, 0.091930, 0.392726, -1.871812, 0.230739, -1.547497, -1.467897, 0.136815, -1.684916, 0.040382, -0.325940, -0.340864, 1.079147, -0.210670, -2.003445, 0.043715, 0.891348, 0.857435, 0.774968, 2.914853, -0.150394, 0.320233, 3.216483, 0.080061, 0.369820, 0.005830, -0.734460, -0.252006, 2.080884, -0.967762, -0.350694, -1.331637, 1.112199, 0.914143, 0.379990, -1.216934, 0.366516, 0.430863, 1.027318, 1.467426, 0.356622, 1.171530, 0.030343, 1.401006, 1.183005, -0.181555, -0.810345, 0.817671, 1.356802, -0.568799, -0.093789, 1.105537, 0.621142, -1.158608},{0.992976, -0.536532, -0.423868, -1.015361, 1.693675, 0.163576, 1.974154, 0.009099, 1.790734, 0.354651, 0.121696, -0.452226, 0.017320, -1.284313, 1.675259, -0.305019, -0.240185, 0.800344, 0.916263, -0.426401, 2.267398, 0.722403, 0.659606, 0.135020, 0.320511, -1.318399, -0.756403, -0.051484, 1.726008, 0.014167, -0.159603, -1.517709, -1.157309, -0.148625, -0.387178, -1.443741, 1.380708, -0.505819, 0.024457, 0.185744, 1.020393, -0.988470, 1.583580, 0.466284, 0.842697, 0.431195, 0.095376, -1.255418, -0.775380, 0.826518, -1.602453, -1.059644, 0.320056, -1.725567, 0.873830, -0.673802, -1.128839, 0.701105, 0.577807, 0.768485, 0.431619, -0.543662, -1.421446, 1.051901, 0.362551, 0.645495, 1.346168, -1.023121, 0.563128, -0.739037, 0.292860, 2.330013, 2.229919, 1.616717, 0.345603, 0.221445, -2.368499, 0.290588},{0.168933, -0.138252, -1.591772, -0.448438, -0.410957, 0.262582, 0.307848, -0.548674, 0.578106, -1.063280, 0.249074, -0.353216, 0.862258, -0.222883, -0.172738, -0.263113, -0.419461, -0.473513, 0.589179, 1.759956, -1.890373, -0.722756, -0.409530, 0.533513, 0.816688, 0.110879, 0.094989, -0.578292, -0.595759, -0.156180, -2.387195, 0.459590, 0.361139, -0.270984, 0.788924, 0.943377, 0.119539, -0.134631, -0.010243, 0.557518, -1.186473, 0.387847, 0.044318, 0.346412, -0.472603, -0.816864, -0.266372, 0.972513, -2.666901, 0.387974, -1.480345, -1.044073, 0.059437, -1.890384, 0.200947, 0.853230, -0.670584, 1.124687, 0.006642, -0.603935, 0.085047, -0.809051, 0.276606, 0.428397, 0.481298, -0.380020, -1.053703, 0.965590, 0.299415, 0.290356, 0.194599, -0.020323, -1.019165, 0.230373, -1.023036, 0.628210, -1.571389, -0.648683},{-1.134766, -0.322957, 1.335809, 0.663012, -0.384196, -0.523007, -1.223540, 0.853804, 0.972147, 2.182696, 0.142374, -0.843305, 0.429923, 0.479373, -0.282400, -0.065125, -2.272869, 0.391532, 0.134344, 0.577712, 0.087520, -1.610765, 0.708399, -0.130109, 0.731262, -0.036256, 0.231937, 0.692306, 0.936394, 0.052967, 0.148189, -1.130662, -1.666126, 1.139962, 0.504689, -0.111749, -0.330905, -1.057894, -1.039344, -0.487264, 1.037427, -0.109845, 0.156493, -0.391722, 0.466912, 0.006490, 0.448480, 0.233294, -1.690980, -0.213945, 0.849492, 1.992244, 0.670181, 0.880763, 0.016694, -0.281486, -0.226785, 0.328010, -0.417491, -1.519248, -1.624609, 1.683321, -0.037423, 1.231048, -0.866500, -0.445492, 1.698708, -0.739375, -2.129812, 0.090588, 1.749488, 0.315314, -1.055426, 2.392438, 1.119244, 0.039701, -0.598901, -0.487653},{0.660287, 1.110130, 0.142109, 0.331649, 0.072401, 0.720009, -2.145049, 1.907996, 0.570473, -1.533132, 0.341742, 0.347606, 1.077663, 2.158283, 0.850198, 0.017907, 1.667289, 0.954349, -0.908976, 0.860666, -0.403902, 1.200506, 1.484833, 0.071767, -0.591879, 0.598758, -0.494318, -0.418649, -0.194351, 0.010149, 0.043966, 0.357019, -0.271395, 0.152740, -0.502622, 1.300009, -0.112579, -0.040906, 0.528270, 0.676197, -0.461748, 0.409964, -1.624739, -0.764907, 1.145239, -0.504144, -0.379634, -1.505093, 0.233666, -0.167479, 0.092638, -1.463185, 1.333724, 0.325444, -1.386145, -1.564424, 0.429734, 1.226936, 0.788916, -0.636344, -0.321578, 2.056267, -0.678714, 0.605907, 1.469952, 0.901767, 1.307873, -0.822144, -0.703458, 0.546574, 1.171719, 0.360915, -0.080724, -1.416829, 0.404329, 0.291646, -1.024517, 0.230101},{0.762951, 1.788789, -0.498489, 1.164157, -0.424972, -0.931049, 0.259670, -0.063569, 0.083207, 0.575702, 0.444861, -0.634680, -0.755498, -0.225755, -0.087300, -0.790916, -0.364050, 0.063748, -0.899056, 0.033074, 1.211014, -1.193703, -0.232095, -0.587912, -0.173295, 0.859976, -0.695424, -0.196195, -0.342199, -0.337737, 0.404680, 0.796009, 0.298527, -0.613383, -0.369520, -0.383449, 2.198098, -0.551340, 0.142933, 1.695604, -0.127369, -0.235616, -0.134422, 0.421855, -1.507558, -1.474540, 0.060466, 1.387495, 0.809023, 1.465250, 0.805811, -1.454647, -0.447507, 0.337745, -0.822617, 2.553028, 0.213324, 1.031338, -0.253808, 0.830974, 0.337159, 0.594237, -0.313671, 0.781610, -0.255894, 0.592325, -0.996568, 0.275328, -1.188245, -0.562170, 1.587197, -0.190401, -0.997471, -1.871457, -0.337216, -0.424778, -1.475241, 1.372328},{2.643594, -0.006109, 0.307475, -1.552530, -0.319056, 0.571120, 1.793045, -0.143487, -1.126593, 1.206959, -0.812815, 0.917518, -0.478596, 0.140734, 1.383055, 0.737347, -1.634381, -2.065592, -0.667024, -0.975252, -0.677133, -0.844229, 0.116120, 0.255876, -0.019060, 0.538397, -1.827830, -0.616147, -0.085429, 0.006452, 0.301072, 0.115803, -1.249722, -0.378682, 0.536487, 0.924883, -0.077610, -0.969620, -2.491856, 0.330723, -1.413026, -0.939454, -1.816283, -0.748209, 0.509990, 2.063774, -0.103882, 0.906512, -1.213012, -1.065527, -0.486743, -0.792036, -1.344238, 0.065045, -1.059101, 1.595781, -0.376512, 0.997970, 3.556356, -0.410782, 1.092948, 1.192755, -0.413857, 0.622390, 0.194840, -0.058504, 0.502060, -0.837730, -1.115740, -0.295954, 0.603445, 0.318121, -0.069672, -0.401658, 1.024648, 1.333116, 0.179406, 2.174232},{-0.709291, 1.069768, -1.019294, 0.513644, -3.375488, -0.617071, -0.443338, -0.023530, 0.124346, 0.534544, 0.810944, -0.482358, -0.746002, 0.394406, 0.067401, 0.328098, 0.181726, -0.120826, 0.331621, 0.954679, 0.688446, -0.407060, 1.517826, -1.183704, 2.825401, -2.033752, -1.126430, 0.497775, -1.133022, -0.311726, 0.224060, 0.279169, -0.467244, -0.618936, 0.942191, -1.347638, -0.871757, -0.586552, -0.731487, 0.378511, 0.785603, -0.823731, 0.058287, -0.405288, 1.674863, 2.017200, -0.814766, -0.501003, -0.880845, -0.212925, 0.002393, -1.754687, -0.667965, 0.515494, 1.646485, 0.187975, 0.088931, -0.064778, 0.437542, 0.058614, -0.686790, 0.495558, -1.594769, 0.306651, 0.077981, 1.089305, 0.639497, 0.549995, 0.314802, -1.138836, 0.890586, 0.930267, 0.374853, 1.580326, 2.362383, 0.426477, -0.584107, 1.057197},{-0.725401, -0.161244, -0.100850, 0.635996, -0.845466, 0.329968, 0.086248, -0.383480, -2.080950, 0.602458, 0.210130, 0.444969, 0.831156, -0.783481, 0.717019, 0.108609, -2.199755, -0.416581, 0.472757, -3.166715, 0.606020, -0.430337, -0.318996, -0.757115, 1.015539, 0.013197, -0.549206, 1.759154, 3.137348, 0.245482, -0.000425, -1.869362, -0.078530, 0.552650, 1.786508, -0.350487, -1.602018, 0.593448, 0.285880, -1.407156, 0.171084, 0.338026, -0.950037, -0.439153, 1.041137, -0.741373, 0.899759, 0.065542, -0.141619, -0.631475, -2.569604, 2.339200, 0.873478, 0.504627, 0.882183, -0.894945, 0.893906, -0.059658, 1.410606, -1.062377, -0.358143, 0.315124, -0.323083, -0.821385, -1.054152, -0.273393, 1.052195, 1.424937, -1.685720, -0.198355, -1.593350, -0.982974, -1.536881, 0.750845, -0.570742, -0.332339, -1.577387, -1.312300},{-0.778319, -0.289853, -0.534332, -0.952515, 0.300087, 1.652725, -2.397071, 1.436740, 0.354622, -0.053310, 0.192906, -1.562122, 0.179580, -3.381541, 0.002137, -0.400742, -1.093074, 0.441417, -1.788132, -0.435965, -2.032391, -0.357335, 0.307676, -1.326333, 1.057773, -0.773703, -0.704963, -0.317923, -0.125569, 0.039906, -0.045928, -0.387423, 0.174265, -0.008731, -0.558063, 0.339273, -2.012356, 0.145275, 1.951477, 0.461786, 1.797038, -0.294848, 0.400720, 2.031648, -0.983011, -0.423168, -0.176912, -1.303354, -1.473064, -0.753472, 0.576259, 1.334517, -1.401870, -1.060482, 1.095208, 0.848515, 1.650867, -0.033262, -0.430775, -0.930914, 2.380796, -1.123334, -0.749676, -0.421871, 1.531921, -1.313924, -0.508350, -1.361168, 0.280003, 0.499637, -1.070184, 1.107007, -1.036712, 2.337859, -0.674436, 0.047290, -0.115884, 0.402434},{-0.227257, 0.021841, -0.330127, -1.255752, 0.081314, 0.199532, -0.471312, -0.427221, -0.348835, 2.654990, 0.576008, 1.097563, 0.314227, -0.170618, 0.837412, 0.942292, -2.187392, -1.463352, -1.804460, -0.825589, 0.158699, 0.926379, 2.008562, 1.095676, -1.089675, 0.817003, -0.298155, 0.138649, -1.355718, -0.295596, 0.636647, -1.489038, 0.544270, -0.743589, 1.436673, 0.222774, 0.508930, 0.932904, 0.470135, 0.896989, -1.040321, 0.037033, 0.319265, 0.911456, -0.037212, -1.960460, 0.184227, -1.124267, 0.438920, -0.025990, 1.100171, 0.190826, -0.870183, -0.003018, -0.927554, -1.056595, 1.848419, -0.490307, 1.011070, -1.156598, -1.639003, 0.680830, -0.137345, 0.850835, 1.160698, 0.648935, 0.401396, 0.326090, 0.303049, -0.862716, 0.491964, -0.591796, -1.754958, -0.357485, 0.821524, -0.118750, -0.342433, 0.119299},{-1.640854, -1.394735, -0.782489, -0.251207, -1.428422, -0.238120, 0.410328, 1.563354, 0.972715, -0.538380, 0.046522, 1.657897, -1.165405, -0.003831, 0.976995, -0.616035, -1.039896, -0.577960, 0.052218, 0.061835, 1.275034, 1.014775, 0.729169, 0.974784, -0.988690, 1.287194, 0.199886, -1.782182, -0.689256, 0.727623, -0.950884, 1.026290, 0.235597, -0.317937, 0.580763, 0.354108, 0.320025, 0.920650, 1.237485, -1.923887, 0.126643, -0.007166, 0.697376, 0.126158, 0.584031, 0.656022, 0.608496, -0.757574, 0.592899, -0.811068, 1.190011, -0.725638, -0.018486, 0.521611, 1.054694, -0.082081, -0.841852, 0.007010, 0.541551, 1.124515, -0.031539, -1.011170, -0.151107, -0.778899, -2.182041, -0.342462, 0.738274, 0.926323, -0.863219, -1.478628, 0.824142, -0.063361, 0.192341, -1.284148, 0.628192, -0.324129, 0.498030, 1.735421},{0.148062, 0.226563, 2.227684, 1.932883, 1.008104, 0.001478, 0.882495, 1.420573, 0.983096, 2.294364, 0.816552, 1.374709, 0.917622, 0.917288, -0.705744, 0.368675, -0.024284, 0.164274, -1.540974, -0.125134, 0.009269, -0.264863, 0.034930, 0.092631, 0.055676, 1.471589, 0.076168, -0.853694, 0.503567, 0.204184, 0.592691, 0.853728, 0.356453, -1.217707, 0.848486, 0.361957, 0.448762, 0.016488, 0.074689, 1.041252, -0.399627, -0.898227, 0.735667, 1.088131, -0.834884, -0.354484, -1.769415, 1.552833, 0.413910, 1.285713, 0.526547, -0.926305, 1.960351, 0.495352, -0.239305, 2.152616, -0.516094, 0.304856, -1.504464, -1.068996, 0.076706, 0.313081, -0.727009, 2.568430, -1.652364, 1.426304, 0.994471, -0.211524, -0.084805, 0.901054, 1.162280, 0.883851, 0.681579, 0.269411, 1.430980, -0.184514, 0.911148, 0.321590},{-0.131957, -0.697271, 0.147309, 0.999587, -0.710643, -0.645630, 0.265473, 0.824399, 3.355001, -0.975512, 0.799064, 1.427217, 1.806810, -1.340426, -0.401878, -0.722435, 0.101077, 1.487493, -0.785723, -1.424406, -0.878753, -0.297706, -0.353711, -0.075032, 0.528918, -0.302238, -1.306442, -0.159893, -1.616994, -0.675837, 0.105663, -2.334908, -0.807922, -0.017958, 1.241284, -1.425486, -1.082113, 0.254752, 1.796380, -0.605595, 2.213174, -1.735789, 0.987132, 0.402000, -1.127947, 0.544940, -0.073033, 0.878314, 2.703205, -1.236934, -0.800926, 1.342768, 0.439708, -1.824372, 0.574467, 1.237502, -0.593900, -2.118414, -1.351180, -1.075555, -1.054718, 0.759320, 1.425950, -0.432169, -1.257386, 0.783693, 0.902238, 1.943602, 0.482239, -2.419976, -1.178569, -0.943105, -0.729282, 0.760710, 0.614896, 0.969175, -1.117949, 0.585424},{-0.860734, 0.292277, -0.998063, -2.045955, -0.309366, -0.919664, 0.319642, -0.153394, -2.095132, 1.262594, -0.105451, -2.587211, 0.033038, -0.500610, 1.421676, 0.497700, -0.353789, -0.520180, -0.154664, 0.146298, 1.049799, -0.743163, 2.588269, -0.372184, -0.044382, -2.892500, -1.085604, -1.490036, 0.489360, 2.257166, 2.171010, 1.584506, -1.548493, 1.234759, -0.620377, -1.215304, 1.294072, -0.996164, -2.763155, -0.093062, -2.168127, 0.471569, 0.334850, 1.584215, 1.545077, -0.036509, 1.340238, -0.022630, 0.598828, 0.083686, -0.442364, -0.723955, -0.748181, -1.927072, -1.055053, -0.378216, 0.248156, 1.458252, -0.117210, -0.695935, 0.625886, -0.565720, 0.723409, 0.196922, 0.342877, -0.107192, 1.490757, -0.627337, -1.160812, 0.361398, 1.165522, -0.811642, -2.305166, 1.857580, -0.626524, -0.512641, 0.989710, 1.118362},{-2.418534, 0.011598, -1.336602, -1.128685, 2.781837, -0.873621, -0.368470, -0.031792, -0.817087, 0.114020, 1.072816, 0.200003, -0.972753, 0.375600, 0.261785, -1.908329, 0.098695, 1.230191, -0.381372, 0.880962, -0.113841, -0.112570, -0.783231, -0.266714, -0.048642, -0.543773, -0.178087, 1.381046, -0.546697, 0.906507, 0.430843, 0.349659, 0.814326, 0.498412, 0.234942, 0.372531, -1.681319, -0.072655, 0.540450, -0.516877, -0.678395, -0.994370, 0.711904, 1.616408, -0.698273, -0.080118, -2.852770, -0.466945, 0.920531, -0.295779, -2.062525, 1.037823, 1.175419, 0.175420, 1.383797, 0.050615, -0.616313, -0.239917, 1.071362, -0.346312, -1.756247, 0.383727, -1.270429, 0.175497, 0.011101, 1.177846, 0.131652, -1.193857, -0.622617, -2.241930, 1.516178, -1.865589, 0.663740, 0.199638, 0.612064, -0.374811, -0.179013, 0.954179},{1.783775, 0.049377, -2.861315, -1.266782, 0.152867, -0.111496, 0.610904, 0.695577, 0.098755, 0.148566, 0.093452, -1.287570, 0.494258, -1.114806, -0.135401, -0.633220, 0.514160, -0.716985, 1.268062, -1.317116, 0.312762, 1.752270, 0.385132, 0.871821, -0.101566, 0.932832, 0.322429, 1.150265, -1.280174, 1.908979, 0.344844, -1.023438, -0.190759, -0.211963, 0.958453, 1.035703, -0.394110, 1.447190, -0.769512, 0.831090, -1.385659, 1.054106, 1.349620, 1.953579, 0.356249, 0.150211, 0.444589, 0.673342, 1.211264, -0.285551, -0.800019, 1.146809, 0.746182, -1.062897, 0.237369, -0.183710, -0.329516, 0.462009, -1.321246, 0.389071, -0.269428, 0.183833, 0.162983, 3.116718, 0.888995, -0.290417, 0.274476, -0.681088, 0.103875, -1.100413, 1.214490, 0.221777, 1.005683, -0.306518, -1.301382, 0.329745, -1.553066, -0.028248},{-0.720572, 0.207533, 0.055282, -1.298626, -0.692288, 0.990955, 0.673709, -0.019389, -0.381848, -1.919030, -0.406277, 0.459357, -0.303607, -0.370242, 0.664761, 2.530907, 0.825008, -0.526763, 0.287852, 0.914683, -1.353008, 2.087193, -1.016918, -1.134907, -0.127300, 1.163274, 1.622087, -0.459317, 0.223172, 1.017787, -0.395101, 0.019591, 0.430868, -0.139660, 0.868062, 0.399347, 0.058146, 1.995703, -0.019223, 0.741975, 0.565629, -0.231230, 0.347579, -0.662856, -0.585146, 0.029324, 0.097352, -0.827373, 0.921188, 0.809013, 1.367398, -1.741538, -0.800994, 0.747731, 1.160640, -0.802791, 0.887387, -0.028868, 2.077157, 0.881726, 0.425414, -2.260725, 1.415051, 1.053241, -0.354571, 0.161290, 0.428117, -1.240366, -0.231566, -0.145891, -0.688320, -0.234818, 0.717725, -0.266455, 1.596945, -0.355611, -0.569850, 0.900441},{-0.951902, 1.939963, 0.299456, -0.934464, 0.126135, -1.332341, 0.128337, -1.202190, 0.571479, -0.312233, 0.948812, -0.629833, -1.132409, -0.616641, -0.052224, -0.472502, -0.468141, 0.646497, 1.153909, 0.076824, -0.552673, -1.164124, 1.151671, 0.211302, -0.348079, -0.866002, -0.638907, 1.882385, 1.694201, 0.165210, -1.305907, -2.261328, 0.710044, 0.338295, -0.464631, 0.285839, -1.148273, -1.569444, -0.027853, 0.834212, -2.349985, -0.217965, 0.271149, -1.053317, 0.131186, 0.342907, 0.925570, 0.214893, -1.675846, 1.517030, -0.152400, 1.030083, -1.257240, -0.411422, -0.549317, 0.882819, -1.040925, 1.566983, 0.414900, 0.869213, 0.125922, -0.314344, -1.208089, -0.064007, 2.054922, 0.338945, -0.092329, 0.404221, -0.021656, 0.950910, -1.774286, 2.004040, 1.461269, -0.047535, -0.775729, -0.126045, 0.003794, 0.360799},{0.145638, 1.250619, -0.929423, -1.235311, -0.779535, 1.378058, -0.960226, 0.278307, -1.056811, -0.752549, 0.392609, -0.835640, 1.151755, -1.990793, -0.058234, 0.464779, -1.344882, -0.631388, -0.251169, 1.036250, -1.143330, -0.947677, -1.083578, 0.976441, 0.668270, 1.151124, -0.500023, -0.257328, 0.272650, 0.343950, -0.340852, 1.260131, -0.623432, 1.107222, -2.546425, 1.028465, 1.417052, 0.097225, 0.410462, 0.929696, 1.516653, 0.201781, -0.386931, -0.292904, 0.343385, -0.245008, -1.607226, 0.230756, -0.769195, -0.032904, 0.193604, -0.763261, 0.317479, 0.231004, -0.519741, -0.198084, 0.436928, 0.608122, 0.703052, -0.584887, -0.183185, -2.322164, -0.157582, -1.023274, -0.499062, 0.748150, 0.544159, -1.976940, -1.098886, -0.026874, 0.342351, -0.832748, -1.255771, -0.305718, 0.790250, 1.389118, -0.892049, 0.608687},{-0.369951, 1.236766, 1.830576, 0.537243, -0.115173, 0.471122, -2.106066, 1.135002, -0.340703, 0.729486, 1.917295, -2.771682, -0.029683, 1.407413, -0.499582, -0.278713, -0.163901, -1.259029, -0.801727, 0.424647, 0.589383, 0.758195, -2.030843, -1.615655, -0.839592, -0.145661, 0.388618, 0.159712, 0.038732, -0.529435, -0.255814, 0.323977, -0.556348, -0.155085, -0.713957, -0.316568, 0.461890, 0.389151, 0.144311, 0.371871, -1.284116, 1.158062, 1.626088, -0.359755, -0.931958, 0.785485, -1.216478, -2.534799, -0.420685, -0.504939, 0.540316, -0.007257, 0.497675, -0.683662, -0.513720, 0.159279, 0.062960, -1.476891, 0.194917, 0.038721, -1.241205, -0.419424, -0.467041, -0.528115, 0.178288, 0.633130, 0.045586, -1.491090, 0.615636, -1.503606, 0.249900, 1.284787, -0.898001, 0.074311, -0.340214, 3.116782, 0.064390, 0.409707},{0.026730, 0.543787, 0.430521, 1.076304, -0.359490, -0.486542, -1.812431, 0.519518, 0.079401, 1.964393, 0.139006, -0.304694, 0.612755, -0.261146, -0.529625, 2.063740, 0.501622, 0.308054, 0.426986, -0.337720, 0.170815, -0.418839, -3.044058, -0.088897, -0.490614, 0.839301, 0.170052, 1.531386, -0.038224, 0.104836, -0.564488, -0.120335, 1.654996, 0.994537, 0.851040, -0.029286, -0.315633, 0.262299, 1.249552, 0.415643, 1.275242, -0.805549, 0.742227, 0.426378, 0.187235, 0.010138, -1.499254, -1.076057, 0.212103, 0.992592, 0.194897, -1.678121, 0.568103, -0.070065, 0.447298, -0.767869, 0.484307, -1.468370, -0.326987, 0.099473, -0.905822, 0.182718, -0.957923, -1.061318, -1.323458, -0.310509, -2.213531, -0.994610, -0.980292, -0.553291, 0.586322, -0.484913, 0.262020, 0.327716, 1.348780, -0.973618, -0.375176, -0.105157},{-1.267416, 1.636437, -1.301734, 0.395508, 0.651274, -1.276319, -0.218437, 1.337689, 0.762193, -2.171887, 0.033296, 0.350835, -0.044220, 1.263983, 0.052888, -0.049525, -0.645501, 0.417430, -0.971664, -0.072470, -0.649421, 0.119452, -1.847740, -0.001506, -0.570797, 1.458348, 0.225836, 1.586422, 0.022993, -1.034263, -0.465473, 0.273217, -2.105138, -1.828087, 0.475390, -0.144790, 0.342495, 0.525309, 0.483247, 0.438119, 1.131961, -0.147953, 0.164763, 0.295021, -1.951689, 0.544715, 0.222753, 0.190591, -1.048644, 1.375281, 1.304664, -1.908844, -0.655745, 0.388968, 0.572933, -0.751228, 0.617867, -0.322288, 0.104823, -0.306798, -1.000554, 0.768526, -0.772598, -0.008642, 0.436818, 1.098672, 1.392444, -0.906777, 0.024298, -0.866532, -0.742768, 1.514422, -0.711160, 1.023022, -1.321308, 0.532451, -0.086663, 0.175132},{0.448455, 1.678371, 2.020170, -1.664509, -0.248383, -0.656137, 1.511249, -0.254445, -0.080033, 0.706450, -0.297645, -1.264812, -0.714675, -1.192046, 0.368750, 0.957690, -1.772361, 1.047762, -0.715896, -0.605408, -0.764262, 0.658762, -1.137040, 0.725393, -0.613125, 0.575073, -0.249537, 0.410510, -0.924819, -0.839059, 1.943632, 0.753824, 1.289733, -0.657789, 1.505667, 0.202246, -0.849736, -1.165939, 0.189364, -0.192005, 0.458923, 0.448706, -1.018702, -1.765029, -1.163523, -1.901618, 2.407434, 0.005734, 2.629610, 0.588634, -0.932003, -0.028409, 0.240135, -0.156035, 2.351227, -1.804927, 0.335952, -1.332679, 0.197340, 0.253922, -1.661958, 0.317659, -0.378511, 0.594699, -0.026356, -0.161366, -0.690173, -1.223167, -0.046329, 1.330308, 0.583497, -0.410292, -0.587964, -0.450483, -0.508655, -0.506787, 0.344933, -0.543856},{0.916829, 1.058465, -1.394803, -2.205250, -1.676426, 0.384129, 0.235238, -0.354159, 0.205576, -1.950349, -1.213420, 0.066245, 0.243078, -0.413641, 0.387194, 3.220294, -0.995052, -0.093075, 0.211335, 0.867414, -3.217594, 0.105447, 0.417462, 0.083196, -0.872196, -0.039579, -0.980284, 0.580287, 1.185141, 0.640212, -0.065945, 0.700132, 0.918341, -0.133238, -0.902130, -0.687669, -1.426168, 1.594114, 1.212668, -1.839406, 0.331431, 0.601364, 1.144108, -0.783742, 0.635594, 0.348388, 0.266030, -0.295978, -0.991447, -0.054089, -1.417275, 1.430076, -0.687563, 0.840747, 0.557359, 0.509822, 0.817518, 1.311960, 1.336059, 1.062883, 0.893950, -1.298761, -0.866314, 1.909798, -1.427275, -0.079422, 1.161800, -0.191978, -0.014668, -1.343419, 0.144219, 0.067705, -0.954921, -0.142567, -1.454793, 0.448490, -1.283394, -0.414339},{-0.031698, -0.554861, 0.907872, 1.406478, -2.550631, -1.373020, 2.809658, 0.405669, 0.065656, 1.180367, -1.813056, 0.196832, 1.102434, -1.118852, 1.216097, -0.220112, 0.921421, -2.234711, 2.507092, 2.214343, 0.556662, -1.976272, -1.134495, 0.450728, -1.264039, -0.455571, 0.306444, -1.485002, 1.187993, 1.039157, -0.363165, 0.787902, -1.474083, 0.734009, 0.160191, 0.888792, -1.250029, -0.058817, -0.366897, -0.892213, 0.444441, 0.363146, 0.529792, -2.120260, 1.163637, 0.419945, -0.425940, -0.599771, -0.844463, 0.216664, -3.434697, -1.497583, 1.308081, 0.381253, -0.522982, 0.462569, 0.940922, 0.667746, -0.367746, -0.224049, 0.930940, 1.754126, 0.032005, -0.715258, 0.435845, 0.046921, -1.073448, -0.352173, -0.669167, 0.148562, -0.839503, 0.028318, 0.521168, 1.350102, 0.116541, -0.338373, -0.355858, -0.647525},{0.287834, -1.323725, 0.410554, -1.252039, 0.626933, 0.333162, 0.307835, 0.362202, 0.493161, -1.256443, 0.920796, -0.679303, -0.112318, 1.510641, -0.437625, -0.068879, -0.194357, 1.705686, 0.156561, -0.365353, -1.391787, -1.086604, 0.298128, 0.501180, 1.078571, -0.149969, -0.066779, -1.690722, 1.307951, -0.196383, 0.774853, -1.723263, 0.425005, 1.284823, 0.742830, 0.893891, -0.996447, 0.286559, 1.236795, 1.487188, -0.806081, -0.193546, 0.307369, 0.749514, 0.244269, 0.197695, 0.071620, -1.599149, 1.364142, -0.121536, 1.342321, 0.822900, -2.350901, 0.127347, 1.144114, -0.186686, -0.081423, 0.665907, 1.421499, -0.527054, -1.522788, -1.293779, 0.523281, -2.208591, 0.000910, -0.938353, 0.833987, -1.973578, -0.237336, -0.358795, 1.718456, -0.022204, -0.050154, 0.255952, -0.253922, -1.625932, -0.628710, -0.620268},{0.262809, -0.235837, 0.360616, 0.914945, 0.391027, 0.914366, 1.230017, -0.404003, -0.629391, -2.149037, -0.572241, 0.028748, -0.180642, -0.923215, -1.289199, -1.020535, -1.136584, -0.418428, 0.788055, -0.796640, -0.141185, 0.824536, -0.735305, 0.353551, -0.460540, 1.265607, -0.424357, -1.368536, 0.398967, -0.510105, 1.136575, 0.584388, -1.947176, 0.408168, 1.883777, 1.348889, 1.071398, -1.643474, -0.883126, -0.080773, 0.035848, 1.017550, -0.906522, -0.526341, -1.391762, 0.298918, 1.428032, -0.695454, -1.847017, -1.478826, 0.195546, -0.123616, -0.213812, 0.113956, 0.942162, -0.154847, -0.347048, -0.066058, -1.112432, 1.550433, 0.837430, -0.443117, -0.332514, -1.810538, 0.460576, 0.428062, 0.266970, -0.425910, 2.100900, 0.461801, 0.674152, 0.194585, -0.462138, -1.184093, -1.506892, 0.685731, 0.846117, -1.782961},{-1.671864, -1.956110, -1.541999, -0.384824, 0.937088, -1.594164, -1.651829, -2.110588, -0.901655, -0.902653, 1.230393, -0.481295, -1.114966, -3.073336, 1.056347, -1.984004, -0.529847, -0.052475, -0.495286, 0.294931, -1.131448, 0.525664, -0.425760, -1.528689, -0.862041, 0.602659, 2.072102, 0.034546, -1.194297, -1.164341, 0.291844, 0.832576, 0.183489, -0.151181, 0.138783, -1.370306, 0.209253, 2.521479, 0.267128, 1.054982, -1.220645, 1.009567, 0.070139, 0.211392, -1.306202, 0.824826, -0.545544, 0.086661, 0.338616, 0.778787, 1.300461, 0.641677, -0.998115, -1.148289, -0.090609, -1.635900, -0.434402, -0.610561, 0.222076, 0.406371, -0.186337, 0.057017, 0.982235, 0.048933, -1.016170, 1.087283, 0.526306, 0.379905, -0.742425, -0.342476, -1.290186, -1.578052, 0.508061, 1.130842, 1.512466, -2.071141, 0.059663, 0.732265},{0.205421, -1.554593, 0.028027, 1.252937, 1.092751, 0.322465, 1.008714, 0.215073, -0.108087, 0.707865, 0.123485, -0.987093, -1.587329, 0.970679, 0.891074, 1.454803, -0.975196, -0.325732, -1.421621, -0.282622, 0.891389, 0.524280, -0.584589, 0.123892, 0.655127, 0.550028, -0.970762, 0.782411, 0.416765, 0.576394, 0.519562, -0.377671, 0.450565, 0.779762, -0.665346, -0.123441, 0.403275, 0.509419, -0.227294, 1.119073, 0.017622, 0.057533, -0.976319, -1.222805, 0.280630, -0.238323, -0.424942, 0.538378, 1.442826, -2.103100, -1.487853, 0.161045, -0.722356, -0.764033, 0.031752, 1.131860, -0.763434, -1.297404, 0.074763, 0.757318, 1.636162, -0.665746, -0.221211, 0.261093, -1.102523, -0.079831, -0.280793, 0.919207, -0.509822, 1.379594, 0.947542, -0.140496, -2.758438, -0.830495, -0.481635, 1.797615, -2.027104, 0.334999},{1.697079, -1.732068, -1.691643, -2.009334, 1.996656, 0.085207, 0.825566, -0.176230, 1.095678, -0.579266, 0.312274, -0.086317, 0.306498, 0.027987, -3.261889, -1.714305, -0.544537, -2.133399, -0.884381, -1.864192, -1.099468, -0.396932, -0.212171, -0.242912, -1.008027, -0.400536, -1.115314, -1.519564, -1.687689, 1.611203, 0.456348, -0.030446, -1.000307, 0.487077, 0.907943, -0.541995, 1.067173, 1.303068, -0.323189, 0.057140, 0.676381, 0.304594, -0.990553, 0.243084, 0.873070, -0.594745, 1.578513, 0.968386, 0.978671, -0.169924, -0.120502, 0.826851, -1.059985, -0.903948, -0.224419, 0.751652, 0.147433, 0.236081, -1.133886, 0.945130, 1.786918, -0.232342, -0.044923, 1.676535, -0.910821, -0.199906, -0.408386, -0.100916, -1.226847, 0.691622, 0.151039, -1.671856, 1.315552, -0.151377, 1.427673, -0.211311, 0.442357, 0.535330}};
    }
    else{
      throw art::Exception(art::errors::StdException)
	<< "SW+Splines MiniBooNE random numbers are only configured for Charged Pions";	       
    }
    
  }



  REGISTER_WEIGHTCALC(PrimaryHadronSWCentralSplineVariationWeightCalc)
}

